%line 8+1 ./travis/test/weirdpaste.asm
 dw 25
%line 12+1 ./travis/test/weirdpaste.asm

%line 18+1 ./travis/test/weirdpaste.asm

%line 15+1 ./travis/test/weirdpaste.asm
 dd 2067830734
 dd 1e+36
%line 15+1 ./travis/test/weirdpaste.asm
 dd 1664279731
 dd 33e+20
%line 15+1 ./travis/test/weirdpaste.asm
 dd 0
 dd 0e+2
%line 15+1 ./travis/test/weirdpaste.asm
 dd 1206542336
 dd 1.2e+5
%line 23+1 ./travis/test/weirdpaste.asm



 dd 1e+5, 1e+5








%line 2+1 '`weirdpaste.asm'
mov eax, eax
