/* automatically generated from ./x86/regs.dat - do not edit */

#include "tables.h"
#include "nasm.h"

const opflags_t nasm_reg_flags[] = {
    0,
    /* ah    */    RN_FLAGS( 4) | REG_HIGH,
    /* al    */    RN_FLAGS( 0) | REG_AL,
    /* ax    */    RN_FLAGS( 0) | REG_AX,
    /* bh    */    RN_FLAGS( 7) | REG_HIGH,
    /* bl    */    RN_FLAGS( 3) | REG_BL,
    /* bnd0  */    RN_FLAGS( 0) | BNDREG,
    /* bnd1  */    RN_FLAGS( 1) | BNDREG,
    /* bnd2  */    RN_FLAGS( 2) | BNDREG,
    /* bnd3  */    RN_FLAGS( 3) | BNDREG,
    /* bp    */    RN_FLAGS( 5) | REG16,
    /* bpl   */    RN_FLAGS( 5) | REG8,
    /* bx    */    RN_FLAGS( 3) | REG_BX,
    /* ch    */    RN_FLAGS( 5) | REG_HIGH,
    /* cl    */    RN_FLAGS( 1) | REG_CL,
    /* cr0   */    RN_FLAGS( 0) | REG_CREG,
    /* cr1   */    RN_FLAGS( 1) | REG_CREG,
    /* cr10  */    RN_FLAGS(10) | REG_CREG,
    /* cr11  */    RN_FLAGS(11) | REG_CREG,
    /* cr12  */    RN_FLAGS(12) | REG_CREG,
    /* cr13  */    RN_FLAGS(13) | REG_CREG,
    /* cr14  */    RN_FLAGS(14) | REG_CREG,
    /* cr15  */    RN_FLAGS(15) | REG_CREG,
    /* cr16  */    RN_FLAGS(16) | REG_CREG,
    /* cr17  */    RN_FLAGS(17) | REG_CREG,
    /* cr18  */    RN_FLAGS(18) | REG_CREG,
    /* cr19  */    RN_FLAGS(19) | REG_CREG,
    /* cr2   */    RN_FLAGS( 2) | REG_CREG,
    /* cr20  */    RN_FLAGS(20) | REG_CREG,
    /* cr21  */    RN_FLAGS(21) | REG_CREG,
    /* cr22  */    RN_FLAGS(22) | REG_CREG,
    /* cr23  */    RN_FLAGS(23) | REG_CREG,
    /* cr24  */    RN_FLAGS(24) | REG_CREG,
    /* cr25  */    RN_FLAGS(25) | REG_CREG,
    /* cr26  */    RN_FLAGS(26) | REG_CREG,
    /* cr27  */    RN_FLAGS(27) | REG_CREG,
    /* cr28  */    RN_FLAGS(28) | REG_CREG,
    /* cr29  */    RN_FLAGS(29) | REG_CREG,
    /* cr3   */    RN_FLAGS( 3) | REG_CREG,
    /* cr30  */    RN_FLAGS(30) | REG_CREG,
    /* cr31  */    RN_FLAGS(31) | REG_CREG,
    /* cr4   */    RN_FLAGS( 4) | REG_CREG,
    /* cr5   */    RN_FLAGS( 5) | REG_CREG,
    /* cr6   */    RN_FLAGS( 6) | REG_CREG,
    /* cr7   */    RN_FLAGS( 7) | REG_CREG,
    /* cr8   */    RN_FLAGS( 8) | REG_CREG,
    /* cr9   */    RN_FLAGS( 9) | REG_CREG,
    /* cs    */    RN_FLAGS( 1) | REG_CS,
    /* cx    */    RN_FLAGS( 1) | REG_CX,
    /* dh    */    RN_FLAGS( 6) | REG_HIGH,
    /* di    */    RN_FLAGS( 7) | REG16,
    /* dil   */    RN_FLAGS( 7) | REG8,
    /* dl    */    RN_FLAGS( 2) | REG_DL,
    /* dr0   */    RN_FLAGS( 0) | REG_DREG,
    /* dr1   */    RN_FLAGS( 1) | REG_DREG,
    /* dr10  */    RN_FLAGS(10) | REG_DREG,
    /* dr11  */    RN_FLAGS(11) | REG_DREG,
    /* dr12  */    RN_FLAGS(12) | REG_DREG,
    /* dr13  */    RN_FLAGS(13) | REG_DREG,
    /* dr14  */    RN_FLAGS(14) | REG_DREG,
    /* dr15  */    RN_FLAGS(15) | REG_DREG,
    /* dr16  */    RN_FLAGS(16) | REG_DREG,
    /* dr17  */    RN_FLAGS(17) | REG_DREG,
    /* dr18  */    RN_FLAGS(18) | REG_DREG,
    /* dr19  */    RN_FLAGS(19) | REG_DREG,
    /* dr2   */    RN_FLAGS( 2) | REG_DREG,
    /* dr20  */    RN_FLAGS(20) | REG_DREG,
    /* dr21  */    RN_FLAGS(21) | REG_DREG,
    /* dr22  */    RN_FLAGS(22) | REG_DREG,
    /* dr23  */    RN_FLAGS(23) | REG_DREG,
    /* dr24  */    RN_FLAGS(24) | REG_DREG,
    /* dr25  */    RN_FLAGS(25) | REG_DREG,
    /* dr26  */    RN_FLAGS(26) | REG_DREG,
    /* dr27  */    RN_FLAGS(27) | REG_DREG,
    /* dr28  */    RN_FLAGS(28) | REG_DREG,
    /* dr29  */    RN_FLAGS(29) | REG_DREG,
    /* dr3   */    RN_FLAGS( 3) | REG_DREG,
    /* dr30  */    RN_FLAGS(30) | REG_DREG,
    /* dr31  */    RN_FLAGS(31) | REG_DREG,
    /* dr4   */    RN_FLAGS( 4) | REG_DREG,
    /* dr5   */    RN_FLAGS( 5) | REG_DREG,
    /* dr6   */    RN_FLAGS( 6) | REG_DREG,
    /* dr7   */    RN_FLAGS( 7) | REG_DREG,
    /* dr8   */    RN_FLAGS( 8) | REG_DREG,
    /* dr9   */    RN_FLAGS( 9) | REG_DREG,
    /* ds    */    RN_FLAGS( 3) | REG_DS,
    /* dx    */    RN_FLAGS( 2) | REG_DX,
    /* eax   */    RN_FLAGS( 0) | REG_EAX,
    /* ebp   */    RN_FLAGS( 5) | REG32,
    /* ebx   */    RN_FLAGS( 3) | REG_EBX,
    /* ecx   */    RN_FLAGS( 1) | REG_ECX,
    /* edi   */    RN_FLAGS( 7) | REG32,
    /* edx   */    RN_FLAGS( 2) | REG_EDX,
    /* es    */    RN_FLAGS( 0) | REG_ES,
    /* esi   */    RN_FLAGS( 6) | REG32,
    /* esp   */    RN_FLAGS( 4) | REG32,
    /* fs    */    RN_FLAGS( 4) | REG_FS,
    /* gs    */    RN_FLAGS( 5) | REG_GS,
    /* k0    */    RN_FLAGS( 0) | OPMASKREG,
    /* k1    */    RN_FLAGS( 1) | OPMASKREG,
    /* k2    */    RN_FLAGS( 2) | OPMASKREG,
    /* k3    */    RN_FLAGS( 3) | OPMASKREG,
    /* k4    */    RN_FLAGS( 4) | OPMASKREG,
    /* k5    */    RN_FLAGS( 5) | OPMASKREG,
    /* k6    */    RN_FLAGS( 6) | OPMASKREG,
    /* k7    */    RN_FLAGS( 7) | OPMASKREG,
    /* mm0   */    RN_FLAGS( 0) | MMXREG,
    /* mm1   */    RN_FLAGS( 1) | MMXREG,
    /* mm2   */    RN_FLAGS( 2) | MMXREG,
    /* mm3   */    RN_FLAGS( 3) | MMXREG,
    /* mm4   */    RN_FLAGS( 4) | MMXREG,
    /* mm5   */    RN_FLAGS( 5) | MMXREG,
    /* mm6   */    RN_FLAGS( 6) | MMXREG,
    /* mm7   */    RN_FLAGS( 7) | MMXREG,
    /* r10   */    RN_FLAGS(10) | REG64,
    /* r10b  */    RN_FLAGS(10) | REG8,
    /* r10d  */    RN_FLAGS(10) | REG32,
    /* r10w  */    RN_FLAGS(10) | REG16,
    /* r11   */    RN_FLAGS(11) | REG64,
    /* r11b  */    RN_FLAGS(11) | REG8,
    /* r11d  */    RN_FLAGS(11) | REG32,
    /* r11w  */    RN_FLAGS(11) | REG16,
    /* r12   */    RN_FLAGS(12) | REG64,
    /* r12b  */    RN_FLAGS(12) | REG8,
    /* r12d  */    RN_FLAGS(12) | REG32,
    /* r12w  */    RN_FLAGS(12) | REG16,
    /* r13   */    RN_FLAGS(13) | REG64,
    /* r13b  */    RN_FLAGS(13) | REG8,
    /* r13d  */    RN_FLAGS(13) | REG32,
    /* r13w  */    RN_FLAGS(13) | REG16,
    /* r14   */    RN_FLAGS(14) | REG64,
    /* r14b  */    RN_FLAGS(14) | REG8,
    /* r14d  */    RN_FLAGS(14) | REG32,
    /* r14w  */    RN_FLAGS(14) | REG16,
    /* r15   */    RN_FLAGS(15) | REG64,
    /* r15b  */    RN_FLAGS(15) | REG8,
    /* r15d  */    RN_FLAGS(15) | REG32,
    /* r15w  */    RN_FLAGS(15) | REG16,
    /* r16   */    RN_FLAGS(16) | REG64,
    /* r16b  */    RN_FLAGS(16) | REG8,
    /* r16d  */    RN_FLAGS(16) | REG32,
    /* r16w  */    RN_FLAGS(16) | REG16,
    /* r17   */    RN_FLAGS(17) | REG64,
    /* r17b  */    RN_FLAGS(17) | REG8,
    /* r17d  */    RN_FLAGS(17) | REG32,
    /* r17w  */    RN_FLAGS(17) | REG16,
    /* r18   */    RN_FLAGS(18) | REG64,
    /* r18b  */    RN_FLAGS(18) | REG8,
    /* r18d  */    RN_FLAGS(18) | REG32,
    /* r18w  */    RN_FLAGS(18) | REG16,
    /* r19   */    RN_FLAGS(19) | REG64,
    /* r19b  */    RN_FLAGS(19) | REG8,
    /* r19d  */    RN_FLAGS(19) | REG32,
    /* r19w  */    RN_FLAGS(19) | REG16,
    /* r20   */    RN_FLAGS(20) | REG64,
    /* r20b  */    RN_FLAGS(20) | REG8,
    /* r20d  */    RN_FLAGS(20) | REG32,
    /* r20w  */    RN_FLAGS(20) | REG16,
    /* r21   */    RN_FLAGS(21) | REG64,
    /* r21b  */    RN_FLAGS(21) | REG8,
    /* r21d  */    RN_FLAGS(21) | REG32,
    /* r21w  */    RN_FLAGS(21) | REG16,
    /* r22   */    RN_FLAGS(22) | REG64,
    /* r22b  */    RN_FLAGS(22) | REG8,
    /* r22d  */    RN_FLAGS(22) | REG32,
    /* r22w  */    RN_FLAGS(22) | REG16,
    /* r23   */    RN_FLAGS(23) | REG64,
    /* r23b  */    RN_FLAGS(23) | REG8,
    /* r23d  */    RN_FLAGS(23) | REG32,
    /* r23w  */    RN_FLAGS(23) | REG16,
    /* r24   */    RN_FLAGS(24) | REG64,
    /* r24b  */    RN_FLAGS(24) | REG8,
    /* r24d  */    RN_FLAGS(24) | REG32,
    /* r24w  */    RN_FLAGS(24) | REG16,
    /* r25   */    RN_FLAGS(25) | REG64,
    /* r25b  */    RN_FLAGS(25) | REG8,
    /* r25d  */    RN_FLAGS(25) | REG32,
    /* r25w  */    RN_FLAGS(25) | REG16,
    /* r26   */    RN_FLAGS(26) | REG64,
    /* r26b  */    RN_FLAGS(26) | REG8,
    /* r26d  */    RN_FLAGS(26) | REG32,
    /* r26w  */    RN_FLAGS(26) | REG16,
    /* r27   */    RN_FLAGS(27) | REG64,
    /* r27b  */    RN_FLAGS(27) | REG8,
    /* r27d  */    RN_FLAGS(27) | REG32,
    /* r27w  */    RN_FLAGS(27) | REG16,
    /* r28   */    RN_FLAGS(28) | REG64,
    /* r28b  */    RN_FLAGS(28) | REG8,
    /* r28d  */    RN_FLAGS(28) | REG32,
    /* r28w  */    RN_FLAGS(28) | REG16,
    /* r29   */    RN_FLAGS(29) | REG64,
    /* r29b  */    RN_FLAGS(29) | REG8,
    /* r29d  */    RN_FLAGS(29) | REG32,
    /* r29w  */    RN_FLAGS(29) | REG16,
    /* r30   */    RN_FLAGS(30) | REG64,
    /* r30b  */    RN_FLAGS(30) | REG8,
    /* r30d  */    RN_FLAGS(30) | REG32,
    /* r30w  */    RN_FLAGS(30) | REG16,
    /* r31   */    RN_FLAGS(31) | REG64,
    /* r31b  */    RN_FLAGS(31) | REG8,
    /* r31d  */    RN_FLAGS(31) | REG32,
    /* r31w  */    RN_FLAGS(31) | REG16,
    /* r8    */    RN_FLAGS( 8) | REG64,
    /* r8b   */    RN_FLAGS( 8) | REG8,
    /* r8d   */    RN_FLAGS( 8) | REG32,
    /* r8w   */    RN_FLAGS( 8) | REG16,
    /* r9    */    RN_FLAGS( 9) | REG64,
    /* r9b   */    RN_FLAGS( 9) | REG8,
    /* r9d   */    RN_FLAGS( 9) | REG32,
    /* r9w   */    RN_FLAGS( 9) | REG16,
    /* rax   */    RN_FLAGS( 0) | REG_RAX,
    /* rbp   */    RN_FLAGS( 5) | REG64,
    /* rbx   */    RN_FLAGS( 3) | REG_RBX,
    /* rcx   */    RN_FLAGS( 1) | REG_RCX,
    /* rdi   */    RN_FLAGS( 7) | REG64,
    /* rdx   */    RN_FLAGS( 2) | REG_RDX,
    /* rsi   */    RN_FLAGS( 6) | REG64,
    /* rsp   */    RN_FLAGS( 4) | REG64,
    /* segr6 */    RN_FLAGS( 6) | REG_SEG67,
    /* segr7 */    RN_FLAGS( 7) | REG_SEG67,
    /* si    */    RN_FLAGS( 6) | REG16,
    /* sil   */    RN_FLAGS( 6) | REG8,
    /* sp    */    RN_FLAGS( 4) | REG16,
    /* spl   */    RN_FLAGS( 4) | REG8,
    /* ss    */    RN_FLAGS( 2) | REG_SS,
    /* st0   */    RN_FLAGS( 0) | FPUREG,
    /* st1   */    RN_FLAGS( 1) | FPUREG,
    /* st2   */    RN_FLAGS( 2) | FPUREG,
    /* st3   */    RN_FLAGS( 3) | FPUREG,
    /* st4   */    RN_FLAGS( 4) | FPUREG,
    /* st5   */    RN_FLAGS( 5) | FPUREG,
    /* st6   */    RN_FLAGS( 6) | FPUREG,
    /* st7   */    RN_FLAGS( 7) | FPUREG,
    /* tmm0  */    RN_FLAGS( 0) | TMMREG,
    /* tmm1  */    RN_FLAGS( 1) | TMMREG,
    /* tmm2  */    RN_FLAGS( 2) | TMMREG,
    /* tmm3  */    RN_FLAGS( 3) | TMMREG,
    /* tmm4  */    RN_FLAGS( 4) | TMMREG,
    /* tmm5  */    RN_FLAGS( 5) | TMMREG,
    /* tmm6  */    RN_FLAGS( 6) | TMMREG,
    /* tmm7  */    RN_FLAGS( 7) | TMMREG,
    /* tr0   */    RN_FLAGS( 0) | REG_TREG,
    /* tr1   */    RN_FLAGS( 1) | REG_TREG,
    /* tr2   */    RN_FLAGS( 2) | REG_TREG,
    /* tr3   */    RN_FLAGS( 3) | REG_TREG,
    /* tr4   */    RN_FLAGS( 4) | REG_TREG,
    /* tr5   */    RN_FLAGS( 5) | REG_TREG,
    /* tr6   */    RN_FLAGS( 6) | REG_TREG,
    /* tr7   */    RN_FLAGS( 7) | REG_TREG,
    /* xmm0  */    RN_FLAGS( 0) | XMMREG,
    /* xmm1  */    RN_FLAGS( 1) | XMMREG,
    /* xmm10 */    RN_FLAGS(10) | XMMREG,
    /* xmm11 */    RN_FLAGS(11) | XMMREG,
    /* xmm12 */    RN_FLAGS(12) | XMMREG,
    /* xmm13 */    RN_FLAGS(13) | XMMREG,
    /* xmm14 */    RN_FLAGS(14) | XMMREG,
    /* xmm15 */    RN_FLAGS(15) | XMMREG,
    /* xmm16 */    RN_FLAGS(16) | XMMREG,
    /* xmm17 */    RN_FLAGS(17) | XMMREG,
    /* xmm18 */    RN_FLAGS(18) | XMMREG,
    /* xmm19 */    RN_FLAGS(19) | XMMREG,
    /* xmm2  */    RN_FLAGS( 2) | XMMREG,
    /* xmm20 */    RN_FLAGS(20) | XMMREG,
    /* xmm21 */    RN_FLAGS(21) | XMMREG,
    /* xmm22 */    RN_FLAGS(22) | XMMREG,
    /* xmm23 */    RN_FLAGS(23) | XMMREG,
    /* xmm24 */    RN_FLAGS(24) | XMMREG,
    /* xmm25 */    RN_FLAGS(25) | XMMREG,
    /* xmm26 */    RN_FLAGS(26) | XMMREG,
    /* xmm27 */    RN_FLAGS(27) | XMMREG,
    /* xmm28 */    RN_FLAGS(28) | XMMREG,
    /* xmm29 */    RN_FLAGS(29) | XMMREG,
    /* xmm3  */    RN_FLAGS( 3) | XMMREG,
    /* xmm30 */    RN_FLAGS(30) | XMMREG,
    /* xmm31 */    RN_FLAGS(31) | XMMREG,
    /* xmm4  */    RN_FLAGS( 4) | XMMREG,
    /* xmm5  */    RN_FLAGS( 5) | XMMREG,
    /* xmm6  */    RN_FLAGS( 6) | XMMREG,
    /* xmm7  */    RN_FLAGS( 7) | XMMREG,
    /* xmm8  */    RN_FLAGS( 8) | XMMREG,
    /* xmm9  */    RN_FLAGS( 9) | XMMREG,
    /* ymm0  */    RN_FLAGS( 0) | YMMREG,
    /* ymm1  */    RN_FLAGS( 1) | YMMREG,
    /* ymm10 */    RN_FLAGS(10) | YMMREG,
    /* ymm11 */    RN_FLAGS(11) | YMMREG,
    /* ymm12 */    RN_FLAGS(12) | YMMREG,
    /* ymm13 */    RN_FLAGS(13) | YMMREG,
    /* ymm14 */    RN_FLAGS(14) | YMMREG,
    /* ymm15 */    RN_FLAGS(15) | YMMREG,
    /* ymm16 */    RN_FLAGS(16) | YMMREG,
    /* ymm17 */    RN_FLAGS(17) | YMMREG,
    /* ymm18 */    RN_FLAGS(18) | YMMREG,
    /* ymm19 */    RN_FLAGS(19) | YMMREG,
    /* ymm2  */    RN_FLAGS( 2) | YMMREG,
    /* ymm20 */    RN_FLAGS(20) | YMMREG,
    /* ymm21 */    RN_FLAGS(21) | YMMREG,
    /* ymm22 */    RN_FLAGS(22) | YMMREG,
    /* ymm23 */    RN_FLAGS(23) | YMMREG,
    /* ymm24 */    RN_FLAGS(24) | YMMREG,
    /* ymm25 */    RN_FLAGS(25) | YMMREG,
    /* ymm26 */    RN_FLAGS(26) | YMMREG,
    /* ymm27 */    RN_FLAGS(27) | YMMREG,
    /* ymm28 */    RN_FLAGS(28) | YMMREG,
    /* ymm29 */    RN_FLAGS(29) | YMMREG,
    /* ymm3  */    RN_FLAGS( 3) | YMMREG,
    /* ymm30 */    RN_FLAGS(30) | YMMREG,
    /* ymm31 */    RN_FLAGS(31) | YMMREG,
    /* ymm4  */    RN_FLAGS( 4) | YMMREG,
    /* ymm5  */    RN_FLAGS( 5) | YMMREG,
    /* ymm6  */    RN_FLAGS( 6) | YMMREG,
    /* ymm7  */    RN_FLAGS( 7) | YMMREG,
    /* ymm8  */    RN_FLAGS( 8) | YMMREG,
    /* ymm9  */    RN_FLAGS( 9) | YMMREG,
    /* zmm0  */    RN_FLAGS( 0) | ZMMREG,
    /* zmm1  */    RN_FLAGS( 1) | ZMMREG,
    /* zmm10 */    RN_FLAGS(10) | ZMMREG,
    /* zmm11 */    RN_FLAGS(11) | ZMMREG,
    /* zmm12 */    RN_FLAGS(12) | ZMMREG,
    /* zmm13 */    RN_FLAGS(13) | ZMMREG,
    /* zmm14 */    RN_FLAGS(14) | ZMMREG,
    /* zmm15 */    RN_FLAGS(15) | ZMMREG,
    /* zmm16 */    RN_FLAGS(16) | ZMMREG,
    /* zmm17 */    RN_FLAGS(17) | ZMMREG,
    /* zmm18 */    RN_FLAGS(18) | ZMMREG,
    /* zmm19 */    RN_FLAGS(19) | ZMMREG,
    /* zmm2  */    RN_FLAGS( 2) | ZMMREG,
    /* zmm20 */    RN_FLAGS(20) | ZMMREG,
    /* zmm21 */    RN_FLAGS(21) | ZMMREG,
    /* zmm22 */    RN_FLAGS(22) | ZMMREG,
    /* zmm23 */    RN_FLAGS(23) | ZMMREG,
    /* zmm24 */    RN_FLAGS(24) | ZMMREG,
    /* zmm25 */    RN_FLAGS(25) | ZMMREG,
    /* zmm26 */    RN_FLAGS(26) | ZMMREG,
    /* zmm27 */    RN_FLAGS(27) | ZMMREG,
    /* zmm28 */    RN_FLAGS(28) | ZMMREG,
    /* zmm29 */    RN_FLAGS(29) | ZMMREG,
    /* zmm3  */    RN_FLAGS( 3) | ZMMREG,
    /* zmm30 */    RN_FLAGS(30) | ZMMREG,
    /* zmm31 */    RN_FLAGS(31) | ZMMREG,
    /* zmm4  */    RN_FLAGS( 4) | ZMMREG,
    /* zmm5  */    RN_FLAGS( 5) | ZMMREG,
    /* zmm6  */    RN_FLAGS( 6) | ZMMREG,
    /* zmm7  */    RN_FLAGS( 7) | ZMMREG,
    /* zmm8  */    RN_FLAGS( 8) | ZMMREG,
    /* zmm9  */    RN_FLAGS( 9) | ZMMREG,
};
