s/@@NASM_MAJOR@@/3/g
s/@@NASM_MINOR@@/1/g
s/@@NASM_SUBMINOR@@/0/g
s/@@NASM_PATCHLEVEL@@/0/g
s/@@NASM_SNAPSHOT@@/0/g
s/@@NASM_VERSION_ID@@/50397184/g
s/@@NASM_VERSION_XID@@/03010000/g
s/@@NASM_VER@@/3.01/g
s/@@NASM_MANGLED_VER@@/3.01/g
