BITS 16
cmp ax, 1
o16 cmp ax, 1
o32 cmp ax, 1

cmp eax, 1
o16 cmp eax, 1
o32 cmp eax, 1

BITS 32
cmp ax, 1
o16 cmp ax, 1
o32 cmp ax, 1

cmp eax, 1
o16 cmp eax, 1
o32 cmp eax, 1

BITS 64
cmp ax, 1
o16 cmp ax, 1
o32 cmp ax, 1

cmp eax, 1
o16 cmp eax, 1
o32 cmp eax, 1
