/*
 * This file is generated from ./asm/directiv.dat
 * by perfhash.pl; do not edit.
 */

#include "directiv.h"

const char * const directive_tbl[65] = {
    "absolute",
    "bits",
    "common",
    "cpu",
    "debug",
    "default",
    "dollarhex",
    "extern",
    "float",
    "global",
    "list",
    "pragma",
    "required",
    "sectalign",
    "section",
    "segment",
    "static",
    "warning",
    "prefix",
    "suffix",
    "postfix",
    "gprefix",
    "gsuffix",
    "gpostfix",
    "lprefix",
    "lsuffix",
    "lpostfix",
    NULL,
    "db",
    "dw",
    "dd",
    "dq",
    "dt",
    "do",
    "dy",
    "dz",
    "resb",
    "resw",
    "resd",
    "resq",
    "rest",
    "reso",
    "resy",
    "resz",
    "incbin",
    "equ",
    NULL,
    "export",
    "group",
    "import",
    "library",
    "map",
    "module",
    "org",
    "osabi",
    "safeseh",
    "uppercase",
    NULL,
    "limit",
    "options",
    "subsections_via_symbols",
    "no_dead_strip",
    "maxdump",
    "nodepend",
    "noseclabels"
};

#define INVALID_HASH_ENTRY (65536/3)

static const int16_t directive_hashvals[256] = {
    0,
    INVALID_HASH_ENTRY,
    0,
    INVALID_HASH_ENTRY,
    0,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    0,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    0,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    0,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    0,
    0,
    0,
    INVALID_HASH_ENTRY,
    19,
    0,
    0,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    -34,
    15,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    0,
    INVALID_HASH_ENTRY,
    0,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    0,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    58,
    INVALID_HASH_ENTRY,
    0,
    INVALID_HASH_ENTRY,
    0,
    0,
    0,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    16,
    INVALID_HASH_ENTRY,
    24,
    INVALID_HASH_ENTRY,
    0,
    INVALID_HASH_ENTRY,
    75,
    INVALID_HASH_ENTRY,
    0,
    0,
    0,
    59,
    0,
    32,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    47,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    32,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    -11,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    9,
    52,
    0,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    0,
    INVALID_HASH_ENTRY,
    0,
    INVALID_HASH_ENTRY,
    0,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    14,
    INVALID_HASH_ENTRY,
    0,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    0,
    INVALID_HASH_ENTRY,
    27,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    51,
    INVALID_HASH_ENTRY,
    0,
    39,
    -24,
    INVALID_HASH_ENTRY,
    13,
    INVALID_HASH_ENTRY,
    31,
    INVALID_HASH_ENTRY,
    19,
    26,
    INVALID_HASH_ENTRY,
    10,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    0,
    7,
    INVALID_HASH_ENTRY,
    49,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    3,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    62,
    -37,
    33,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    0,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    11,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    14,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    15,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    0,
    INVALID_HASH_ENTRY,
    -9,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    55,
    63,
    1,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    45,
    INVALID_HASH_ENTRY,
    17,
    INVALID_HASH_ENTRY,
    0,
    INVALID_HASH_ENTRY,
    17,
    42,
    0,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    4,
    2,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    40,
    0,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    49,
    INVALID_HASH_ENTRY,
    8,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    5,
    INVALID_HASH_ENTRY,
    6,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    12,
    11,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    53,
    -4,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    61,
    39,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    8,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    56,
    13,
    INVALID_HASH_ENTRY,
    21,
    27,
    27,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    INVALID_HASH_ENTRY,
    4,
    INVALID_HASH_ENTRY
};

const struct perfect_hash directive_hash = {
    UINT64_C(0x076259c3e291c26c),
    UINT32_C(0xfe),
    UINT32_C(65),
    3,
    (D_unknown),
    directive_hashvals,
    directive_tbl
};
