/* automatically generated from ./x86/regs.dat - do not edit */

#ifndef NASM_REGDIS_H
#define NASM_REGDIS_H

#include "regs.h"

#define DISREGTBLSZ 32

extern const enum reg_enum nasm_rd_bndreg[DISREGTBLSZ];
extern const enum reg_enum nasm_rd_creg[DISREGTBLSZ];
extern const enum reg_enum nasm_rd_dreg[DISREGTBLSZ];
extern const enum reg_enum nasm_rd_fpureg[DISREGTBLSZ];
extern const enum reg_enum nasm_rd_mmxreg[DISREGTBLSZ];
extern const enum reg_enum nasm_rd_opmaskreg[DISREGTBLSZ];
extern const enum reg_enum nasm_rd_reg16[DISREGTBLSZ];
extern const enum reg_enum nasm_rd_reg32[DISREGTBLSZ];
extern const enum reg_enum nasm_rd_reg64[DISREGTBLSZ];
extern const enum reg_enum nasm_rd_reg8[DISREGTBLSZ];
extern const enum reg_enum nasm_rd_reg8_rex[DISREGTBLSZ];
extern const enum reg_enum nasm_rd_sreg[DISREGTBLSZ];
extern const enum reg_enum nasm_rd_tmmreg[DISREGTBLSZ];
extern const enum reg_enum nasm_rd_treg[DISREGTBLSZ];
extern const enum reg_enum nasm_rd_xmmreg[DISREGTBLSZ];
extern const enum reg_enum nasm_rd_ymmreg[DISREGTBLSZ];
extern const enum reg_enum nasm_rd_zmmreg[DISREGTBLSZ];

#endif /* NASM_REGDIS_H */
