/* This file is auto-generated from insns.dat by insns.pl - don't edit it */

#include "tables.h"

const char * const nasm_insn_names[] = {
	"db",
	"dw",
	"dd",
	"dq",
	"dt",
	"do",
	"dy",
	"dz",
	"resb",
	"resw",
	"resd",
	"resq",
	"rest",
	"reso",
	"resy",
	"resz",
	"incbin",
	"equ",
	"nop",
	"nop2",
	"mov",
	"movabs",
	"movrs",
	"lea",
	"add",
	"or",
	"adc",
	"sbb",
	"and",
	"sub",
	"xor",
	"cmp",
	"test",
	"rol",
	"ror",
	"rcl",
	"rcr",
	"shl",
	"sal",
	"shr",
	"sar",
	"rorx",
	"rolx",
	"shlx",
	"salx",
	"sarx",
	"shrx",
	"inc",
	"dec",
	"imul",
	"mul",
	"mulx",
	"idiv",
	"div",
	"neg",
	"not",
	"adcx",
	"adox",
	"shld",
	"shrd",
	"bt",
	"btc",
	"btr",
	"bts",
	"bsf",
	"bsr",
	"ibts",
	"xbts",
	"lzcnt",
	"tzcnt",
	"andn",
	"bextr",
	"blsmsk",
	"blsr",
	"blsi",
	"bzhi",
	"pdep",
	"pext",
	"blci",
	"blcic",
	"blsic",
	"blcfill",
	"blsfill",
	"blcmsk",
	"blcs",
	"tzmsk",
	"t1mskc",
	"aaa",
	"aad",
	"aam",
	"aas",
	"daa",
	"das",
	"bswap",
	"movbe",
	"cbw",
	"cdq",
	"cdqe",
	"cqo",
	"cwd",
	"cwde",
	"movsx",
	"movsxb",
	"movsxw",
	"movsxd",
	"movzx",
	"movzxb",
	"movzxw",
	"movzxd",
	"cmpxchg",
	"cmpxchg8b",
	"cmpxchg16b",
	"xadd",
	"xchg",
	"cmpxchg486",
	"jmpabs",
	"jmp",
	"ja",
	"jae",
	"jb",
	"jbe",
	"jc",
	"je",
	"jg",
	"jge",
	"jl",
	"jle",
	"jna",
	"jnae",
	"jnb",
	"jnbe",
	"jnc",
	"jne",
	"jng",
	"jnge",
	"jnl",
	"jnle",
	"jno",
	"jnp",
	"jns",
	"jnz",
	"jo",
	"jp",
	"jpe",
	"jpo",
	"js",
	"jz",
	"jcxz",
	"jecxz",
	"jrcxz",
	"loop",
	"loopw",
	"loopd",
	"loopq",
	"loope",
	"loopew",
	"looped",
	"loopeq",
	"loopne",
	"loopnew",
	"loopned",
	"loopneq",
	"loopz",
	"loopzw",
	"loopzd",
	"loopzq",
	"loopnz",
	"loopnzw",
	"loopnzd",
	"loopnzq",
	"jmpe",
	"call",
	"ret",
	"retw",
	"retd",
	"retq",
	"retf",
	"retfw",
	"retfd",
	"retfq",
	"retn",
	"retnw",
	"retnd",
	"retnq",
	"int",
	"int1",
	"int01",
	"icebp",
	"int3",
	"int03",
	"brkpt",
	"into",
	"syscall",
	"sysenter",
	"sysexit",
	"sysret",
	"iret",
	"iretw",
	"iretd",
	"iretq",
	"erets",
	"eretu",
	"clc",
	"cld",
	"cli",
	"clac",
	"stc",
	"std",
	"sti",
	"stac",
	"cmc",
	"lahf",
	"sahf",
	"salc",
	"pushf",
	"pushfw",
	"pushfd",
	"pushfq",
	"popf",
	"popfw",
	"popfd",
	"popfq",
	"cmpsb",
	"cmpsw",
	"cmpsd",
	"cmpsq",
	"lodsb",
	"lodsw",
	"lodsd",
	"lodsq",
	"movsb",
	"movsw",
	"movsd",
	"movsq",
	"stosb",
	"stosw",
	"stosd",
	"stosq",
	"scasb",
	"scasw",
	"scasd",
	"scasq",
	"insb",
	"insw",
	"insd",
	"outsb",
	"outsw",
	"outsd",
	"lfence",
	"mfence",
	"sfence",
	"serialize",
	"clflush",
	"clflushopt",
	"clwb",
	"pcommit",
	"clzero",
	"invd",
	"wbinvd",
	"wbnoinvd",
	"invpcid",
	"invlpg",
	"invlpga",
	"rdpmc",
	"rdtsc",
	"rdtscp",
	"rdrand",
	"rdseed",
	"rdpid",
	"clts",
	"cpuid",
	"lmsw",
	"smsw",
	"wrmsr",
	"rdmsr",
	"wrmsrns",
	"rdmsrlist",
	"wrmsrlist",
	"urdmsr",
	"uwrmsr",
	"umov",
	"bb0_reset",
	"bb1_reset",
	"cpu_read",
	"cpu_write",
	"dmint",
	"rdm",
	"smint",
	"smintold",
	"rsm",
	"wrshr",
	"rdshr",
	"rsdc",
	"rsldt",
	"rsts",
	"svdc",
	"svldt",
	"svts",
	"hlt",
	"pause",
	"monitor",
	"monitorw",
	"monitord",
	"monitorq",
	"mwait",
	"monitorx",
	"mwaitx",
	"tpause",
	"umonitor",
	"umwait",
	"in",
	"out",
	"lds",
	"les",
	"lfs",
	"lgs",
	"lss",
	"push",
	"pop",
	"rdfsbase",
	"rdgsbase",
	"wrfsbase",
	"wrgsbase",
	"arpl",
	"lar",
	"lsl",
	"verr",
	"verw",
	"swapgs",
	"lkgs",
	"lgdt",
	"lidt",
	"lldt",
	"ltr",
	"sgdt",
	"sidt",
	"sldt",
	"str",
	"loadall",
	"loadall286",
	"f2xm1",
	"fabs",
	"fadd",
	"faddp",
	"fbld",
	"fbstp",
	"fchs",
	"fclex",
	"fcmovb",
	"fcmovbe",
	"fcmove",
	"fcmovnb",
	"fcmovnbe",
	"fcmovne",
	"fcmovnu",
	"fcmovu",
	"fcom",
	"fcomi",
	"fcomip",
	"fcomp",
	"fcompp",
	"fcos",
	"fdecstp",
	"fdisi",
	"fdiv",
	"fdivp",
	"fdivr",
	"fdivrp",
	"femms",
	"feni",
	"ffree",
	"ffreep",
	"fiadd",
	"ficom",
	"ficomp",
	"fidiv",
	"fidivr",
	"fild",
	"fimul",
	"fincstp",
	"finit",
	"fist",
	"fistp",
	"fisttp",
	"fisub",
	"fisubr",
	"fld",
	"fld1",
	"fldcw",
	"fldenv",
	"fldl2e",
	"fldl2t",
	"fldlg2",
	"fldln2",
	"fldpi",
	"fldz",
	"fmul",
	"fmulp",
	"fnclex",
	"fndisi",
	"fneni",
	"fninit",
	"fnop",
	"fnsave",
	"fnstcw",
	"fnstenv",
	"fnstsw",
	"fpatan",
	"fprem",
	"fprem1",
	"fptan",
	"frndint",
	"frstor",
	"fsave",
	"fscale",
	"fsetpm",
	"fsin",
	"fsincos",
	"fsqrt",
	"fst",
	"fstcw",
	"fstenv",
	"fstp",
	"fstsw",
	"fsub",
	"fsubp",
	"fsubr",
	"fsubrp",
	"ftst",
	"fucom",
	"fucomi",
	"fucomip",
	"fucomp",
	"fucompp",
	"fxam",
	"fxch",
	"fxtract",
	"fyl2x",
	"fyl2xp1",
	"emms",
	"movd",
	"movq",
	"packssdw",
	"packsswb",
	"packuswb",
	"paddb",
	"paddd",
	"paddsb",
	"paddsiw",
	"paddsw",
	"paddusb",
	"paddusw",
	"paddw",
	"pand",
	"pandn",
	"paveb",
	"pavgusb",
	"pcmpeqb",
	"pcmpeqd",
	"pcmpeqw",
	"pcmpgtb",
	"pcmpgtd",
	"pcmpgtw",
	"pdistib",
	"pf2id",
	"pfacc",
	"pfadd",
	"pfcmpeq",
	"pfcmpge",
	"pfcmpgt",
	"pfmax",
	"pfmin",
	"pfmul",
	"pfrcp",
	"pfrcpit1",
	"pfrcpit2",
	"pfrsqit1",
	"pfrsqrt",
	"pfsub",
	"pfsubr",
	"pi2fd",
	"pmachriw",
	"pmaddwd",
	"pmagw",
	"pmulhriw",
	"pmulhrwa",
	"pmulhrwc",
	"pmulhw",
	"pmullw",
	"pmvgezb",
	"pmvlzb",
	"pmvnzb",
	"pmvzb",
	"por",
	"prefetch",
	"prefetchw",
	"pslld",
	"psllq",
	"psllw",
	"psrad",
	"psraw",
	"psrld",
	"psrlq",
	"psrlw",
	"psubb",
	"psubd",
	"psubsb",
	"psubsiw",
	"psubsw",
	"psubusb",
	"psubusw",
	"psubw",
	"punpckhbw",
	"punpckhdq",
	"punpckhwd",
	"punpcklbw",
	"punpckldq",
	"punpcklwd",
	"pusha",
	"pushaw",
	"pushad",
	"popa",
	"popaw",
	"popad",
	"enter",
	"enterw",
	"enterd",
	"enterq",
	"leave",
	"leavew",
	"leaved",
	"leaveq",
	"bound",
	"pushp",
	"popp",
	"push2",
	"push2p",
	"pop2",
	"pop2p",
	"pxor",
	"skinit",
	"ud0",
	"ud1",
	"ud2b",
	"ud2",
	"ud2a",
	"udb",
	"fwait",
	"xlatb",
	"xlat",
	"ccmp",
	"ccmpa",
	"ccmpae",
	"ccmpb",
	"ccmpbe",
	"ccmpc",
	"ccmpe",
	"ccmpf",
	"ccmpg",
	"ccmpge",
	"ccmpl",
	"ccmple",
	"ccmpna",
	"ccmpnae",
	"ccmpnb",
	"ccmpnbe",
	"ccmpnc",
	"ccmpne",
	"ccmpng",
	"ccmpnge",
	"ccmpnl",
	"ccmpnle",
	"ccmpno",
	"ccmpns",
	"ccmpnz",
	"ccmpo",
	"ccmps",
	"ccmpt",
	"ccmpz",
	"ctest",
	"ctesta",
	"ctestae",
	"ctestb",
	"ctestbe",
	"ctestc",
	"cteste",
	"ctestf",
	"ctestg",
	"ctestge",
	"ctestl",
	"ctestle",
	"ctestna",
	"ctestnae",
	"ctestnb",
	"ctestnbe",
	"ctestnc",
	"ctestne",
	"ctestng",
	"ctestnge",
	"ctestnl",
	"ctestnle",
	"ctestno",
	"ctestns",
	"ctestnz",
	"ctesto",
	"ctests",
	"ctestt",
	"ctestz",
	"cmova",
	"cmovae",
	"cmovb",
	"cmovbe",
	"cmovc",
	"cmove",
	"cmovg",
	"cmovge",
	"cmovl",
	"cmovle",
	"cmovna",
	"cmovnae",
	"cmovnb",
	"cmovnbe",
	"cmovnc",
	"cmovne",
	"cmovng",
	"cmovnge",
	"cmovnl",
	"cmovnle",
	"cmovno",
	"cmovnp",
	"cmovns",
	"cmovnz",
	"cmovo",
	"cmovp",
	"cmovpe",
	"cmovpo",
	"cmovs",
	"cmovz",
	"cfcmova",
	"cfcmovae",
	"cfcmovb",
	"cfcmovbe",
	"cfcmovc",
	"cfcmove",
	"cfcmovg",
	"cfcmovge",
	"cfcmovl",
	"cfcmovle",
	"cfcmovna",
	"cfcmovnae",
	"cfcmovnb",
	"cfcmovnbe",
	"cfcmovnc",
	"cfcmovne",
	"cfcmovng",
	"cfcmovnge",
	"cfcmovnl",
	"cfcmovnle",
	"cfcmovno",
	"cfcmovnp",
	"cfcmovns",
	"cfcmovnz",
	"cfcmovo",
	"cfcmovp",
	"cfcmovpe",
	"cfcmovpo",
	"cfcmovs",
	"cfcmovz",
	"seta",
	"setae",
	"setb",
	"setbe",
	"setc",
	"sete",
	"setg",
	"setge",
	"setl",
	"setle",
	"setna",
	"setnae",
	"setnb",
	"setnbe",
	"setnc",
	"setne",
	"setng",
	"setnge",
	"setnl",
	"setnle",
	"setno",
	"setnp",
	"setns",
	"setnz",
	"seto",
	"setp",
	"setpe",
	"setpo",
	"sets",
	"setz",
	"setazu",
	"setaezu",
	"setbzu",
	"setbezu",
	"setczu",
	"setezu",
	"setgzu",
	"setgezu",
	"setlzu",
	"setlezu",
	"setnazu",
	"setnaezu",
	"setnbzu",
	"setnbezu",
	"setnczu",
	"setnezu",
	"setngzu",
	"setngezu",
	"setnlzu",
	"setnlezu",
	"setnozu",
	"setnpzu",
	"setnszu",
	"setnzzu",
	"setozu",
	"setpzu",
	"setpezu",
	"setpozu",
	"setszu",
	"setzzu",
	"cmpaxadd",
	"cmpaexadd",
	"cmpbxadd",
	"cmpbexadd",
	"cmpcxadd",
	"cmpexadd",
	"cmpgxadd",
	"cmpgexadd",
	"cmplxadd",
	"cmplexadd",
	"cmpnaxadd",
	"cmpnaexadd",
	"cmpnbxadd",
	"cmpnbexadd",
	"cmpncxadd",
	"cmpnexadd",
	"cmpngxadd",
	"cmpngexadd",
	"cmpnlxadd",
	"cmpnlexadd",
	"cmpnoxadd",
	"cmpnpxadd",
	"cmpnsxadd",
	"cmpnzxadd",
	"cmpoxadd",
	"cmppxadd",
	"cmppexadd",
	"cmppoxadd",
	"cmpsxadd",
	"cmpzxadd",
	"addps",
	"addss",
	"andnps",
	"andps",
	"cmpeqps",
	"cmpeqss",
	"cmpleps",
	"cmpless",
	"cmpltps",
	"cmpltss",
	"cmpneqps",
	"cmpneqss",
	"cmpnleps",
	"cmpnless",
	"cmpnltps",
	"cmpnltss",
	"cmpordps",
	"cmpordss",
	"cmpunordps",
	"cmpunordss",
	"cmpps",
	"cmpss",
	"comiss",
	"cvtpi2ps",
	"cvtps2pi",
	"cvtsi2ss",
	"cvtss2si",
	"cvttps2pi",
	"cvttss2si",
	"divps",
	"divss",
	"ldmxcsr",
	"maxps",
	"maxss",
	"minps",
	"minss",
	"movaps",
	"movhps",
	"movlhps",
	"movlps",
	"movhlps",
	"movmskps",
	"movntps",
	"movss",
	"movups",
	"mulps",
	"mulss",
	"orps",
	"rcpps",
	"rcpss",
	"rsqrtps",
	"rsqrtss",
	"shufps",
	"sqrtps",
	"sqrtss",
	"stmxcsr",
	"subps",
	"subss",
	"ucomiss",
	"unpckhps",
	"unpcklps",
	"xorps",
	"fxrstor",
	"fxrstor64",
	"fxsave",
	"fxsave64",
	"xgetbv",
	"xsetbv",
	"xsave",
	"xsave64",
	"xsavec",
	"xsavec64",
	"xsaveopt",
	"xsaveopt64",
	"xsaves",
	"xsaves64",
	"xrstor",
	"xrstor64",
	"xrstors",
	"xrstors64",
	"prefetchnta",
	"prefetcht0",
	"prefetcht1",
	"prefetcht2",
	"prefetchit0",
	"prefetchit1",
	"maskmovq",
	"movntq",
	"pavgb",
	"pavgw",
	"pextrw",
	"pinsrw",
	"pmaxsw",
	"pmaxub",
	"pminsw",
	"pminub",
	"pmovmskb",
	"pmulhuw",
	"psadbw",
	"pshufw",
	"pf2iw",
	"pfnacc",
	"pfpnacc",
	"pi2fw",
	"pswapd",
	"maskmovdqu",
	"movntdq",
	"movnti",
	"movntpd",
	"movdqa",
	"movdqu",
	"movdq2q",
	"movq2dq",
	"paddq",
	"pmuludq",
	"pshufd",
	"pshufhw",
	"pshuflw",
	"pslldq",
	"psrldq",
	"psubq",
	"punpckhqdq",
	"punpcklqdq",
	"addpd",
	"addsd",
	"andnpd",
	"andpd",
	"cmpeqpd",
	"cmpeqsd",
	"cmplepd",
	"cmplesd",
	"cmpltpd",
	"cmpltsd",
	"cmpneqpd",
	"cmpneqsd",
	"cmpnlepd",
	"cmpnlesd",
	"cmpnltpd",
	"cmpnltsd",
	"cmpordpd",
	"cmpordsd",
	"cmpunordpd",
	"cmpunordsd",
	"cmppd",
	"comisd",
	"cvtdq2pd",
	"cvtdq2ps",
	"cvtpd2dq",
	"cvtpd2pi",
	"cvtpd2ps",
	"cvtpi2pd",
	"cvtps2dq",
	"cvtps2pd",
	"cvtsd2si",
	"cvtsd2ss",
	"cvtsi2sd",
	"cvtss2sd",
	"cvttpd2pi",
	"cvttpd2dq",
	"cvttps2dq",
	"cvttsd2si",
	"divpd",
	"divsd",
	"maxpd",
	"maxsd",
	"minpd",
	"minsd",
	"movapd",
	"movhpd",
	"movlpd",
	"movmskpd",
	"movupd",
	"mulpd",
	"mulsd",
	"orpd",
	"shufpd",
	"sqrtpd",
	"sqrtsd",
	"subpd",
	"subsd",
	"ucomisd",
	"unpckhpd",
	"unpcklpd",
	"xorpd",
	"addsubpd",
	"addsubps",
	"haddpd",
	"haddps",
	"hsubpd",
	"hsubps",
	"lddqu",
	"movddup",
	"movshdup",
	"movsldup",
	"clgi",
	"stgi",
	"vmcall",
	"vmclear",
	"vmfunc",
	"vmlaunch",
	"vmload",
	"vmmcall",
	"vmptrld",
	"vmptrst",
	"vmread",
	"vmresume",
	"vmrun",
	"vmsave",
	"vmwrite",
	"vmxoff",
	"vmxon",
	"invept",
	"invvpid",
	"pvalidate",
	"rmpadjust",
	"vmgexit",
	"pabsb",
	"pabsw",
	"pabsd",
	"palignr",
	"phaddw",
	"phaddd",
	"phaddsw",
	"phsubw",
	"phsubd",
	"phsubsw",
	"pmaddubsw",
	"pmulhrsw",
	"pshufb",
	"psignb",
	"psignw",
	"psignd",
	"extrq",
	"insertq",
	"movntsd",
	"movntss",
	"blendpd",
	"blendps",
	"blendvpd",
	"blendvps",
	"dppd",
	"dpps",
	"extractps",
	"insertps",
	"movntdqa",
	"mpsadbw",
	"packusdw",
	"pblendvb",
	"pblendw",
	"pcmpeqq",
	"pextrb",
	"pextrd",
	"pextrq",
	"phminposuw",
	"pinsrb",
	"pinsrd",
	"pinsrq",
	"pmaxsb",
	"pmaxsd",
	"pmaxud",
	"pmaxuw",
	"pminsb",
	"pminsd",
	"pminud",
	"pminuw",
	"pmovsxbw",
	"pmovsxbd",
	"pmovsxbq",
	"pmovsxwd",
	"pmovsxwq",
	"pmovsxdq",
	"pmovzxbw",
	"pmovzxbd",
	"pmovzxbq",
	"pmovzxwd",
	"pmovzxwq",
	"pmovzxdq",
	"pmuldq",
	"pmulld",
	"ptest",
	"roundpd",
	"roundps",
	"roundsd",
	"roundss",
	"crc32",
	"pcmpestri",
	"pcmpestrm",
	"pcmpistri",
	"pcmpistrm",
	"pcmpgtq",
	"popcnt",
	"getsec",
	"pfrcpv",
	"pfrsqrtv",
	"aesenc",
	"aesenclast",
	"aesdec",
	"aesdeclast",
	"aesimc",
	"aeskeygenassist",
	"vaesenc",
	"vaesenclast",
	"vaesdec",
	"vaesdeclast",
	"vaesimc",
	"vaeskeygenassist",
	"aesdec128kl",
	"aesdec256kl",
	"aesenc128kl",
	"aesenc256kl",
	"encodekey128",
	"encodekey256",
	"loadiwkey",
	"aesdecwide128kl",
	"aesdecwide256kl",
	"aesencwide128kl",
	"aesencwide256kl",
	"vaddpd",
	"vaddps",
	"vaddsd",
	"vaddss",
	"vaddsubpd",
	"vaddsubps",
	"vandpd",
	"vandps",
	"vandnpd",
	"vandnps",
	"vblendpd",
	"vblendps",
	"vblendvpd",
	"vblendvps",
	"vbroadcastss",
	"vbroadcastsd",
	"vbroadcastf128",
	"vcmpeq_ospd",
	"vcmpeqpd",
	"vcmplt_ospd",
	"vcmpltpd",
	"vcmple_ospd",
	"vcmplepd",
	"vcmpunord_qpd",
	"vcmpunordpd",
	"vcmpneq_uqpd",
	"vcmpneqpd",
	"vcmpnlt_uspd",
	"vcmpnltpd",
	"vcmpnle_uspd",
	"vcmpnlepd",
	"vcmpord_qpd",
	"vcmpordpd",
	"vcmpeq_uqpd",
	"vcmpnge_uspd",
	"vcmpngepd",
	"vcmpngt_uspd",
	"vcmpngtpd",
	"vcmpfalse_oqpd",
	"vcmpfalsepd",
	"vcmpneq_oqpd",
	"vcmpge_ospd",
	"vcmpgepd",
	"vcmpgt_ospd",
	"vcmpgtpd",
	"vcmptrue_uqpd",
	"vcmptruepd",
	"vcmplt_oqpd",
	"vcmple_oqpd",
	"vcmpunord_spd",
	"vcmpneq_uspd",
	"vcmpnlt_uqpd",
	"vcmpnle_uqpd",
	"vcmpord_spd",
	"vcmpeq_uspd",
	"vcmpnge_uqpd",
	"vcmpngt_uqpd",
	"vcmpfalse_ospd",
	"vcmpneq_ospd",
	"vcmpge_oqpd",
	"vcmpgt_oqpd",
	"vcmptrue_uspd",
	"vcmppd",
	"vcmpeq_osps",
	"vcmpeqps",
	"vcmplt_osps",
	"vcmpltps",
	"vcmple_osps",
	"vcmpleps",
	"vcmpunord_qps",
	"vcmpunordps",
	"vcmpneq_uqps",
	"vcmpneqps",
	"vcmpnlt_usps",
	"vcmpnltps",
	"vcmpnle_usps",
	"vcmpnleps",
	"vcmpord_qps",
	"vcmpordps",
	"vcmpeq_uqps",
	"vcmpnge_usps",
	"vcmpngeps",
	"vcmpngt_usps",
	"vcmpngtps",
	"vcmpfalse_oqps",
	"vcmpfalseps",
	"vcmpneq_oqps",
	"vcmpge_osps",
	"vcmpgeps",
	"vcmpgt_osps",
	"vcmpgtps",
	"vcmptrue_uqps",
	"vcmptrueps",
	"vcmplt_oqps",
	"vcmple_oqps",
	"vcmpunord_sps",
	"vcmpneq_usps",
	"vcmpnlt_uqps",
	"vcmpnle_uqps",
	"vcmpord_sps",
	"vcmpeq_usps",
	"vcmpnge_uqps",
	"vcmpngt_uqps",
	"vcmpfalse_osps",
	"vcmpneq_osps",
	"vcmpge_oqps",
	"vcmpgt_oqps",
	"vcmptrue_usps",
	"vcmpps",
	"vcmpeq_ossd",
	"vcmpeqsd",
	"vcmplt_ossd",
	"vcmpltsd",
	"vcmple_ossd",
	"vcmplesd",
	"vcmpunord_qsd",
	"vcmpunordsd",
	"vcmpneq_uqsd",
	"vcmpneqsd",
	"vcmpnlt_ussd",
	"vcmpnltsd",
	"vcmpnle_ussd",
	"vcmpnlesd",
	"vcmpord_qsd",
	"vcmpordsd",
	"vcmpeq_uqsd",
	"vcmpnge_ussd",
	"vcmpngesd",
	"vcmpngt_ussd",
	"vcmpngtsd",
	"vcmpfalse_oqsd",
	"vcmpfalsesd",
	"vcmpneq_oqsd",
	"vcmpge_ossd",
	"vcmpgesd",
	"vcmpgt_ossd",
	"vcmpgtsd",
	"vcmptrue_uqsd",
	"vcmptruesd",
	"vcmplt_oqsd",
	"vcmple_oqsd",
	"vcmpunord_ssd",
	"vcmpneq_ussd",
	"vcmpnlt_uqsd",
	"vcmpnle_uqsd",
	"vcmpord_ssd",
	"vcmpeq_ussd",
	"vcmpnge_uqsd",
	"vcmpngt_uqsd",
	"vcmpfalse_ossd",
	"vcmpneq_ossd",
	"vcmpge_oqsd",
	"vcmpgt_oqsd",
	"vcmptrue_ussd",
	"vcmpsd",
	"vcmpeq_osss",
	"vcmpeqss",
	"vcmplt_osss",
	"vcmpltss",
	"vcmple_osss",
	"vcmpless",
	"vcmpunord_qss",
	"vcmpunordss",
	"vcmpneq_uqss",
	"vcmpneqss",
	"vcmpnlt_usss",
	"vcmpnltss",
	"vcmpnle_usss",
	"vcmpnless",
	"vcmpord_qss",
	"vcmpordss",
	"vcmpeq_uqss",
	"vcmpnge_usss",
	"vcmpngess",
	"vcmpngt_usss",
	"vcmpngtss",
	"vcmpfalse_oqss",
	"vcmpfalsess",
	"vcmpneq_oqss",
	"vcmpge_osss",
	"vcmpgess",
	"vcmpgt_osss",
	"vcmpgtss",
	"vcmptrue_uqss",
	"vcmptruess",
	"vcmplt_oqss",
	"vcmple_oqss",
	"vcmpunord_sss",
	"vcmpneq_usss",
	"vcmpnlt_uqss",
	"vcmpnle_uqss",
	"vcmpord_sss",
	"vcmpeq_usss",
	"vcmpnge_uqss",
	"vcmpngt_uqss",
	"vcmpfalse_osss",
	"vcmpneq_osss",
	"vcmpge_oqss",
	"vcmpgt_oqss",
	"vcmptrue_usss",
	"vcmpss",
	"vcomisd",
	"vcomiss",
	"vcvtdq2pd",
	"vcvtdq2ps",
	"vcvtpd2dq",
	"vcvtpd2ps",
	"vcvtps2dq",
	"vcvtps2pd",
	"vcvtsd2si",
	"vcvtsd2ss",
	"vcvtsi2sd",
	"vcvtsi2ss",
	"vcvtss2sd",
	"vcvtss2si",
	"vcvttpd2dq",
	"vcvttps2dq",
	"vcvttsd2si",
	"vcvttss2si",
	"vdivpd",
	"vdivps",
	"vdivsd",
	"vdivss",
	"vdppd",
	"vdpps",
	"vextractf128",
	"vextractps",
	"vhaddpd",
	"vhaddps",
	"vhsubpd",
	"vhsubps",
	"vinsertf128",
	"vinsertps",
	"vlddqu",
	"vldqqu",
	"vldmxcsr",
	"vmaskmovdqu",
	"vmaskmovps",
	"vmaskmovpd",
	"vmaxpd",
	"vmaxps",
	"vmaxsd",
	"vmaxss",
	"vminpd",
	"vminps",
	"vminsd",
	"vminss",
	"vmovapd",
	"vmovaps",
	"vmovd",
	"vmovq",
	"vmovddup",
	"vmovdqa",
	"vmovqqa",
	"vmovdqu",
	"vmovqqu",
	"vmovhlps",
	"vmovhpd",
	"vmovhps",
	"vmovlhps",
	"vmovlpd",
	"vmovlps",
	"vmovmskpd",
	"vmovmskps",
	"vmovntdq",
	"vmovntqq",
	"vmovntdqa",
	"vmovntpd",
	"vmovntps",
	"vmovsd",
	"vmovshdup",
	"vmovsldup",
	"vmovss",
	"vmovupd",
	"vmovups",
	"vmpsadbw",
	"vmulpd",
	"vmulps",
	"vmulsd",
	"vmulss",
	"vorpd",
	"vorps",
	"vpabsb",
	"vpabsw",
	"vpabsd",
	"vpacksswb",
	"vpackssdw",
	"vpackuswb",
	"vpackusdw",
	"vpaddb",
	"vpaddw",
	"vpaddd",
	"vpaddq",
	"vpaddsb",
	"vpaddsw",
	"vpaddusb",
	"vpaddusw",
	"vpalignr",
	"vpand",
	"vpandn",
	"vpavgb",
	"vpavgw",
	"vpblendvb",
	"vpblendw",
	"vpcmpestri",
	"vpcmpestrm",
	"vpcmpistri",
	"vpcmpistrm",
	"vpcmpeqb",
	"vpcmpeqw",
	"vpcmpeqd",
	"vpcmpeqq",
	"vpcmpgtb",
	"vpcmpgtw",
	"vpcmpgtd",
	"vpcmpgtq",
	"vpermilpd",
	"vpermilps",
	"vperm2f128",
	"vpextrb",
	"vpextrw",
	"vpextrd",
	"vpextrq",
	"vphaddw",
	"vphaddd",
	"vphaddsw",
	"vphminposuw",
	"vphsubw",
	"vphsubd",
	"vphsubsw",
	"vpinsrb",
	"vpinsrw",
	"vpinsrd",
	"vpinsrq",
	"vpmaddwd",
	"vpmaddubsw",
	"vpmaxsb",
	"vpmaxsw",
	"vpmaxsd",
	"vpmaxub",
	"vpmaxuw",
	"vpmaxud",
	"vpminsb",
	"vpminsw",
	"vpminsd",
	"vpminub",
	"vpminuw",
	"vpminud",
	"vpmovmskb",
	"vpmovsxbw",
	"vpmovsxbd",
	"vpmovsxbq",
	"vpmovsxwd",
	"vpmovsxwq",
	"vpmovsxdq",
	"vpmovzxbw",
	"vpmovzxbd",
	"vpmovzxbq",
	"vpmovzxwd",
	"vpmovzxwq",
	"vpmovzxdq",
	"vpmulhuw",
	"vpmulhrsw",
	"vpmulhw",
	"vpmullw",
	"vpmulld",
	"vpmuludq",
	"vpmuldq",
	"vpor",
	"vpsadbw",
	"vpshufb",
	"vpshufd",
	"vpshufhw",
	"vpshuflw",
	"vpsignb",
	"vpsignw",
	"vpsignd",
	"vpslldq",
	"vpsrldq",
	"vpsllw",
	"vpslld",
	"vpsllq",
	"vpsraw",
	"vpsrad",
	"vpsrlw",
	"vpsrld",
	"vpsrlq",
	"vptest",
	"vpsubb",
	"vpsubw",
	"vpsubd",
	"vpsubq",
	"vpsubsb",
	"vpsubsw",
	"vpsubusb",
	"vpsubusw",
	"vpunpckhbw",
	"vpunpckhwd",
	"vpunpckhdq",
	"vpunpckhqdq",
	"vpunpcklbw",
	"vpunpcklwd",
	"vpunpckldq",
	"vpunpcklqdq",
	"vpxor",
	"vrcpps",
	"vrcpss",
	"vrsqrtps",
	"vrsqrtss",
	"vroundpd",
	"vroundps",
	"vroundsd",
	"vroundss",
	"vshufpd",
	"vshufps",
	"vsqrtpd",
	"vsqrtps",
	"vsqrtsd",
	"vsqrtss",
	"vstmxcsr",
	"vsubpd",
	"vsubps",
	"vsubsd",
	"vsubss",
	"vtestps",
	"vtestpd",
	"vucomisd",
	"vucomiss",
	"vunpckhpd",
	"vunpckhps",
	"vunpcklpd",
	"vunpcklps",
	"vxorpd",
	"vxorps",
	"vzeroall",
	"vzeroupper",
	"pclmullqlqdq",
	"pclmulhqlqdq",
	"pclmullqhqdq",
	"pclmulhqhqdq",
	"pclmulqdq",
	"vpclmullqlqdq",
	"vpclmulhqlqdq",
	"vpclmullqhqdq",
	"vpclmulhqhqdq",
	"vpclmulqdq",
	"vfmadd132ps",
	"vfmadd132pd",
	"vfmadd312ps",
	"vfmadd312pd",
	"vfmadd213ps",
	"vfmadd213pd",
	"vfmadd123ps",
	"vfmadd123pd",
	"vfmadd231ps",
	"vfmadd231pd",
	"vfmadd321ps",
	"vfmadd321pd",
	"vfmaddsub132ps",
	"vfmaddsub132pd",
	"vfmaddsub312ps",
	"vfmaddsub312pd",
	"vfmaddsub213ps",
	"vfmaddsub213pd",
	"vfmaddsub123ps",
	"vfmaddsub123pd",
	"vfmaddsub231ps",
	"vfmaddsub231pd",
	"vfmaddsub321ps",
	"vfmaddsub321pd",
	"vfmsub132ps",
	"vfmsub132pd",
	"vfmsub312ps",
	"vfmsub312pd",
	"vfmsub213ps",
	"vfmsub213pd",
	"vfmsub123ps",
	"vfmsub123pd",
	"vfmsub231ps",
	"vfmsub231pd",
	"vfmsub321ps",
	"vfmsub321pd",
	"vfmsubadd132ps",
	"vfmsubadd132pd",
	"vfmsubadd312ps",
	"vfmsubadd312pd",
	"vfmsubadd213ps",
	"vfmsubadd213pd",
	"vfmsubadd123ps",
	"vfmsubadd123pd",
	"vfmsubadd231ps",
	"vfmsubadd231pd",
	"vfmsubadd321ps",
	"vfmsubadd321pd",
	"vfnmadd132ps",
	"vfnmadd132pd",
	"vfnmadd312ps",
	"vfnmadd312pd",
	"vfnmadd213ps",
	"vfnmadd213pd",
	"vfnmadd123ps",
	"vfnmadd123pd",
	"vfnmadd231ps",
	"vfnmadd231pd",
	"vfnmadd321ps",
	"vfnmadd321pd",
	"vfnmsub132ps",
	"vfnmsub132pd",
	"vfnmsub312ps",
	"vfnmsub312pd",
	"vfnmsub213ps",
	"vfnmsub213pd",
	"vfnmsub123ps",
	"vfnmsub123pd",
	"vfnmsub231ps",
	"vfnmsub231pd",
	"vfnmsub321ps",
	"vfnmsub321pd",
	"vfmadd132ss",
	"vfmadd132sd",
	"vfmadd312ss",
	"vfmadd312sd",
	"vfmadd213ss",
	"vfmadd213sd",
	"vfmadd123ss",
	"vfmadd123sd",
	"vfmadd231ss",
	"vfmadd231sd",
	"vfmadd321ss",
	"vfmadd321sd",
	"vfmsub132ss",
	"vfmsub132sd",
	"vfmsub312ss",
	"vfmsub312sd",
	"vfmsub213ss",
	"vfmsub213sd",
	"vfmsub123ss",
	"vfmsub123sd",
	"vfmsub231ss",
	"vfmsub231sd",
	"vfmsub321ss",
	"vfmsub321sd",
	"vfnmadd132ss",
	"vfnmadd132sd",
	"vfnmadd312ss",
	"vfnmadd312sd",
	"vfnmadd213ss",
	"vfnmadd213sd",
	"vfnmadd123ss",
	"vfnmadd123sd",
	"vfnmadd231ss",
	"vfnmadd231sd",
	"vfnmadd321ss",
	"vfnmadd321sd",
	"vfnmsub132ss",
	"vfnmsub132sd",
	"vfnmsub312ss",
	"vfnmsub312sd",
	"vfnmsub213ss",
	"vfnmsub213sd",
	"vfnmsub123ss",
	"vfnmsub123sd",
	"vfnmsub231ss",
	"vfnmsub231sd",
	"vfnmsub321ss",
	"vfnmsub321sd",
	"vcvtph2ps",
	"vcvtps2ph",
	"xstore",
	"xcryptecb",
	"xcryptcbc",
	"xcryptctr",
	"xcryptcfb",
	"xcryptofb",
	"montmul",
	"xsha1",
	"xsha256",
	"llwpcb",
	"slwpcb",
	"lwpval",
	"lwpins",
	"vfmaddpd",
	"vfmaddps",
	"vfmaddsd",
	"vfmaddss",
	"vfmaddsubpd",
	"vfmaddsubps",
	"vfmsubaddpd",
	"vfmsubaddps",
	"vfmsubpd",
	"vfmsubps",
	"vfmsubsd",
	"vfmsubss",
	"vfnmaddpd",
	"vfnmaddps",
	"vfnmaddsd",
	"vfnmaddss",
	"vfnmsubpd",
	"vfnmsubps",
	"vfnmsubsd",
	"vfnmsubss",
	"vfrczpd",
	"vfrczps",
	"vfrczsd",
	"vfrczss",
	"vpcmov",
	"vpcomb",
	"vpcomd",
	"vpcomq",
	"vpcomub",
	"vpcomud",
	"vpcomuq",
	"vpcomuw",
	"vpcomw",
	"vphaddbd",
	"vphaddbq",
	"vphaddbw",
	"vphadddq",
	"vphaddubd",
	"vphaddubq",
	"vphaddubw",
	"vphaddudq",
	"vphadduwd",
	"vphadduwq",
	"vphaddwd",
	"vphaddwq",
	"vphsubbw",
	"vphsubdq",
	"vphsubwd",
	"vpmacsdd",
	"vpmacsdqh",
	"vpmacsdql",
	"vpmacssdd",
	"vpmacssdqh",
	"vpmacssdql",
	"vpmacsswd",
	"vpmacssww",
	"vpmacswd",
	"vpmacsww",
	"vpmadcsswd",
	"vpmadcswd",
	"vpperm",
	"vprotb",
	"vprotd",
	"vprotq",
	"vprotw",
	"vpshab",
	"vpshad",
	"vpshaq",
	"vpshaw",
	"vpshlb",
	"vpshld",
	"vpshlq",
	"vpshlw",
	"vbroadcasti128",
	"vpblendd",
	"vpbroadcastb",
	"vpbroadcastw",
	"vpbroadcastd",
	"vpbroadcastq",
	"vpermd",
	"vpermpd",
	"vpermps",
	"vpermq",
	"vperm2i128",
	"vextracti128",
	"vinserti128",
	"vpmaskmovd",
	"vpmaskmovq",
	"vpsllvd",
	"vpsllvq",
	"vpsravd",
	"vpsrlvd",
	"vpsrlvq",
	"vgatherdpd",
	"vgatherqpd",
	"vgatherdps",
	"vgatherqps",
	"vpgatherdd",
	"vpgatherqd",
	"vpgatherdq",
	"vpgatherqq",
	"xabort",
	"xbegin",
	"xend",
	"xtest",
	"prefetchwt1",
	"bndmk",
	"bndcl",
	"bndcu",
	"bndcn",
	"bndmov",
	"bndldx",
	"bndstx",
	"sha1msg1",
	"sha1msg2",
	"sha1nexte",
	"sha1rnds4",
	"sha256msg1",
	"sha256msg2",
	"sha256rnds2",
	"vsha512msg1",
	"vsha512msg2",
	"vsha512rnds2",
	"vsm3msg1",
	"vsm3msg2",
	"vsm3rnds2",
	"vsm4key4",
	"vsm4rnds4",
	"vbcstnebf16ps",
	"vbcstnebf162ps",
	"vbcstnesh2ps",
	"vcvtneebf162ps",
	"vcvtneeph2ps",
	"vcvtneobf162ps",
	"vcvtneoph2ps",
	"vcvtneps2bf16",
	"vpdpbusd",
	"vpdpbusds",
	"vpdpwssd",
	"vpdpwssds",
	"vpdpbssd",
	"vpdpbssds",
	"vpdpbsud",
	"vpdpbsuds",
	"vpdpbuud",
	"vpdpbuuds",
	"vpdpwsud",
	"vpdpwsuds",
	"vpdpwusd",
	"vpdpwusds",
	"vpdpwuud",
	"vpdpwuuds",
	"vpmadd52huq",
	"vpmadd52luq",
	"kmovb",
	"kmovw",
	"kmovd",
	"kmovq",
	"kmov",
	"movb",
	"movw",
	"kaddb",
	"kaddw",
	"kaddd",
	"kaddq",
	"kadd",
	"addb",
	"addw",
	"addd",
	"addq",
	"kandb",
	"kandw",
	"kandd",
	"kandq",
	"kand",
	"andb",
	"andw",
	"andd",
	"andq",
	"kandnb",
	"kandnw",
	"kandnd",
	"kandnq",
	"kandn",
	"andnb",
	"andnw",
	"andnd",
	"andnq",
	"knotb",
	"knotw",
	"knotd",
	"knotq",
	"knot",
	"notb",
	"notw",
	"notd",
	"notq",
	"korb",
	"korw",
	"kord",
	"korq",
	"kor",
	"orb",
	"orw",
	"ord",
	"orq",
	"kortestb",
	"kortestw",
	"kortestd",
	"kortestq",
	"kortest",
	"ortestb",
	"ortestw",
	"ortestd",
	"ortestq",
	"ortest",
	"kshiftlb",
	"kshiftlw",
	"kshiftld",
	"kshiftlq",
	"kshiftl",
	"shiftlb",
	"shiftlw",
	"shiftld",
	"shiftlq",
	"shiftl",
	"kshlb",
	"kshlw",
	"kshld",
	"kshlq",
	"kshl",
	"shlb",
	"shlw",
	"shlq",
	"kshiftrb",
	"kshiftrw",
	"kshiftrd",
	"kshiftrq",
	"kshiftr",
	"shiftrb",
	"shiftrw",
	"shiftrd",
	"shiftrq",
	"shiftr",
	"kshrb",
	"kshrw",
	"kshrd",
	"kshrq",
	"kshr",
	"shrb",
	"shrw",
	"shrq",
	"ktestb",
	"ktestw",
	"ktestd",
	"ktestq",
	"ktest",
	"testb",
	"testw",
	"testd",
	"testq",
	"kunpckbw",
	"kunpckw",
	"kunpck",
	"unpckbw",
	"unpckw",
	"unpck",
	"kunpckwd",
	"kunpckd",
	"unpckwd",
	"unpckd",
	"kunpckdq",
	"kunpckq",
	"unpckdq",
	"unpckq",
	"kxnorb",
	"kxnorw",
	"kxnord",
	"kxnorq",
	"kxnor",
	"xnorb",
	"xnorw",
	"xnord",
	"xnorq",
	"xnor",
	"kxorb",
	"kxorw",
	"kxord",
	"kxorq",
	"kxor",
	"xorb",
	"xorw",
	"xord",
	"xorq",
	"valignd",
	"valignq",
	"vblendmpd",
	"vblendmps",
	"vbroadcastf32x2",
	"vbroadcastf32x4",
	"vbroadcastf32x8",
	"vbroadcastf64x2",
	"vbroadcastf64x4",
	"vbroadcasti32x2",
	"vbroadcasti32x4",
	"vbroadcasti32x8",
	"vbroadcasti64x2",
	"vbroadcasti64x4",
	"vcmpeq_oqpd",
	"vcmpeq_oqps",
	"vcmpeq_oqsd",
	"vcmpeq_oqss",
	"vcompresspd",
	"vcompressps",
	"vcvtpd2qq",
	"vcvtpd2udq",
	"vcvtpd2uqq",
	"vcvtps2qq",
	"vcvtps2udq",
	"vcvtps2uqq",
	"vcvtqq2pd",
	"vcvtqq2ps",
	"vcvtsd2usi",
	"vcvtss2usi",
	"vcvttpd2qq",
	"vcvttpd2udq",
	"vcvttpd2uqq",
	"vcvttps2qq",
	"vcvttps2udq",
	"vcvttps2uqq",
	"vcvttsd2usi",
	"vcvttss2usi",
	"vcvtudq2pd",
	"vcvtudq2ps",
	"vcvtuqq2pd",
	"vcvtuqq2ps",
	"vcvtusi2sd",
	"vcvtusi2ss",
	"vdbpsadbw",
	"vexp2pd",
	"vexp2ps",
	"vexpandpd",
	"vexpandps",
	"vextractf32x4",
	"vextractf32x8",
	"vextractf64x2",
	"vextractf64x4",
	"vextracti32x4",
	"vextracti32x8",
	"vextracti64x2",
	"vextracti64x4",
	"vfixupimmpd",
	"vfixupimmps",
	"vfixupimmsd",
	"vfixupimmss",
	"vfpclasspd",
	"vfpclassps",
	"vfpclasssd",
	"vfpclassss",
	"vgatherpf0dpd",
	"vgatherpf0dps",
	"vgatherpf0qpd",
	"vgatherpf0qps",
	"vgatherpf1dpd",
	"vgatherpf1dps",
	"vgatherpf1qpd",
	"vgatherpf1qps",
	"vgetexppd",
	"vgetexpps",
	"vgetexpsd",
	"vgetexpss",
	"vgetmantpd",
	"vgetmantps",
	"vgetmantsd",
	"vgetmantss",
	"vinsertf32x4",
	"vinsertf32x8",
	"vinsertf64x2",
	"vinsertf64x4",
	"vinserti32x4",
	"vinserti32x8",
	"vinserti64x2",
	"vinserti64x4",
	"vmovdqa32",
	"vmovdqa64",
	"vmovdqu16",
	"vmovdqu32",
	"vmovdqu64",
	"vmovdqu8",
	"vpabsq",
	"vpandd",
	"vpandnd",
	"vpandnq",
	"vpandq",
	"vpblendmb",
	"vpblendmd",
	"vpblendmq",
	"vpblendmw",
	"vpbroadcastmb2q",
	"vpbroadcastmw2d",
	"vpcmpequb",
	"vpcmpequd",
	"vpcmpequq",
	"vpcmpequw",
	"vpcmpgeb",
	"vpcmpged",
	"vpcmpgeq",
	"vpcmpgeub",
	"vpcmpgeud",
	"vpcmpgeuq",
	"vpcmpgeuw",
	"vpcmpgew",
	"vpcmpgtub",
	"vpcmpgtud",
	"vpcmpgtuq",
	"vpcmpgtuw",
	"vpcmpleb",
	"vpcmpled",
	"vpcmpleq",
	"vpcmpleub",
	"vpcmpleud",
	"vpcmpleuq",
	"vpcmpleuw",
	"vpcmplew",
	"vpcmpltb",
	"vpcmpltd",
	"vpcmpltq",
	"vpcmpltub",
	"vpcmpltud",
	"vpcmpltuq",
	"vpcmpltuw",
	"vpcmpltw",
	"vpcmpneqb",
	"vpcmpneqd",
	"vpcmpneqq",
	"vpcmpnequb",
	"vpcmpnequd",
	"vpcmpnequq",
	"vpcmpnequw",
	"vpcmpneqw",
	"vpcmpngtb",
	"vpcmpngtd",
	"vpcmpngtq",
	"vpcmpngtub",
	"vpcmpngtud",
	"vpcmpngtuq",
	"vpcmpngtuw",
	"vpcmpngtw",
	"vpcmpnleb",
	"vpcmpnled",
	"vpcmpnleq",
	"vpcmpnleub",
	"vpcmpnleud",
	"vpcmpnleuq",
	"vpcmpnleuw",
	"vpcmpnlew",
	"vpcmpnltb",
	"vpcmpnltd",
	"vpcmpnltq",
	"vpcmpnltub",
	"vpcmpnltud",
	"vpcmpnltuq",
	"vpcmpnltuw",
	"vpcmpnltw",
	"vpcmpb",
	"vpcmpd",
	"vpcmpq",
	"vpcmpub",
	"vpcmpud",
	"vpcmpuq",
	"vpcmpuw",
	"vpcmpw",
	"vpcompressd",
	"vpcompressq",
	"vpconflictd",
	"vpconflictq",
	"vpermb",
	"vpermi2b",
	"vpermi2d",
	"vpermi2pd",
	"vpermi2ps",
	"vpermi2q",
	"vpermi2w",
	"vpermt2b",
	"vpermt2d",
	"vpermt2pd",
	"vpermt2ps",
	"vpermt2q",
	"vpermt2w",
	"vpermw",
	"vpexpandd",
	"vpexpandq",
	"vplzcntd",
	"vplzcntq",
	"vpmaxsq",
	"vpmaxuq",
	"vpminsq",
	"vpminuq",
	"vpmovb2m",
	"vpmovd2m",
	"vpmovdb",
	"vpmovdw",
	"vpmovm2b",
	"vpmovm2d",
	"vpmovm2q",
	"vpmovm2w",
	"vpmovq2m",
	"vpmovqb",
	"vpmovqd",
	"vpmovqw",
	"vpmovsdb",
	"vpmovsdw",
	"vpmovsqb",
	"vpmovsqd",
	"vpmovsqw",
	"vpmovswb",
	"vpmovusdb",
	"vpmovusdw",
	"vpmovusqb",
	"vpmovusqd",
	"vpmovusqw",
	"vpmovuswb",
	"vpmovw2m",
	"vpmovwb",
	"vpmullq",
	"vpmultishiftqb",
	"vpord",
	"vporq",
	"vprold",
	"vprolq",
	"vprolvd",
	"vprolvq",
	"vprord",
	"vprorq",
	"vprorvd",
	"vprorvq",
	"vpscatterdd",
	"vpscatterdq",
	"vpscatterqd",
	"vpscatterqq",
	"vpsllvw",
	"vpsraq",
	"vpsravq",
	"vpsravw",
	"vpsrlvw",
	"vpternlogd",
	"vpternlogq",
	"vptestmb",
	"vptestmd",
	"vptestmq",
	"vptestmw",
	"vptestnmb",
	"vptestnmd",
	"vptestnmq",
	"vptestnmw",
	"vpxord",
	"vpxorq",
	"vrangepd",
	"vrangeps",
	"vrangesd",
	"vrangess",
	"vrcp14pd",
	"vrcp14ps",
	"vrcp14sd",
	"vrcp14ss",
	"vrcp28pd",
	"vrcp28ps",
	"vrcp28sd",
	"vrcp28ss",
	"vreducepd",
	"vreduceps",
	"vreducesd",
	"vreducess",
	"vrndscalepd",
	"vrndscaleph",
	"vrndscaleps",
	"vrndscalesd",
	"vrndscalesh",
	"vrndscaless",
	"vrsqrt14pd",
	"vrsqrt14ps",
	"vrsqrt14sd",
	"vrsqrt14ss",
	"vrsqrt28pd",
	"vrsqrt28ps",
	"vrsqrt28sd",
	"vrsqrt28ss",
	"vscalefpd",
	"vscalefps",
	"vscalefsd",
	"vscalefss",
	"vscatterdpd",
	"vscatterdps",
	"vscatterpf0dpd",
	"vscatterpf0dps",
	"vscatterpf0qpd",
	"vscatterpf0qps",
	"vscatterpf1dpd",
	"vscatterpf1dps",
	"vscatterpf1qpd",
	"vscatterpf1qps",
	"vscatterqpd",
	"vscatterqps",
	"vshuff32x4",
	"vshuff64x2",
	"vshufi32x4",
	"vshufi64x2",
	"rdpkru",
	"wrpkru",
	"ptwrite",
	"cldemote",
	"movdiri",
	"movdir64b",
	"pconfig",
	"pbndkb",
	"prefetchrst2",
	"gf2p8affineinvqb",
	"vgf2p8affineinvqb",
	"gf2p8affineqb",
	"vgf2p8affineqb",
	"gf2p8mulb",
	"vgf2p8mulb",
	"vpcompressb",
	"vpcompressw",
	"vpexpandb",
	"vpexpandw",
	"vpshldw",
	"vpshldd",
	"vpshldq",
	"vpshldvw",
	"vpshldvd",
	"vpshldvq",
	"vpshrdw",
	"vpshrdd",
	"vpshrdq",
	"vpshrdvw",
	"vpshrdvd",
	"vpshrdvq",
	"vpopcntb",
	"vpopcntw",
	"vpopcntd",
	"vpopcntq",
	"vpshufbitqmb",
	"v4fmaddps",
	"v4fnmaddps",
	"v4fmaddss",
	"v4fnmaddss",
	"v4dpwssds",
	"v4dpwssd",
	"encls",
	"enclu",
	"enclv",
	"clrssbsy",
	"endbr32",
	"endbr64",
	"incsspd",
	"incsspq",
	"rdsspd",
	"rdsspq",
	"rstorssp",
	"saveprevssp",
	"setssbsy",
	"wrussd",
	"wrussq",
	"wrssd",
	"wrssq",
	"enqcmd",
	"enqcmds",
	"xresldtrk",
	"xsusldtrk",
	"vcvtne2ps2bf16",
	"vdpbf16ps",
	"vp2intersectd",
	"vp2intersectq",
	"ldtilecfg",
	"sttilecfg",
	"tdpbf16ps",
	"tdpfp16ps",
	"tcmmimfp16ps",
	"tcmmrlfp16ps",
	"tdpbssd",
	"tdpbsud",
	"tdpbusd",
	"tdpbuud",
	"tileloadd",
	"tileloaddt1",
	"tilerelease",
	"tilestored",
	"tilezero",
	"tileloaddrs",
	"tileloaddrst1",
	"t2rpntlvwz0",
	"t2rpntlvwz0t1",
	"t2rpntlvwz1",
	"t2rpntlvwz1t1",
	"t2rpntlvwz0rs",
	"t2rpntlvwz0rst1",
	"t2rpntlvwz1rs",
	"t2rpntlvwz1rst1",
	"tconjtcmmimfp16ps",
	"tconjtfp16",
	"tcvtrowd2ps",
	"tcvtrowps2bf16h",
	"tcvtrowps2bf16l",
	"tcvtrowps2phh",
	"tcvtrowps2phl",
	"tdpbf8ps",
	"tdpbhf8ps",
	"tdphbf8ps",
	"tdphf8ps",
	"tilemovrow",
	"tmmultf32ps",
	"ttcmmimfp16ps",
	"ttcmmrlfp16ps",
	"ttdpbf16ps",
	"ttdpfp16ps",
	"ttmmultf32ps",
	"ttransposed",
	"vaddph",
	"vaddsh",
	"vcmpph",
	"vcmpsh",
	"vcomish",
	"vcvtdq2ph",
	"vcvtpd2ph",
	"vcvtph2dq",
	"vcvtph2pd",
	"vcvtph2psx",
	"vcvtph2qq",
	"vcvtph2udq",
	"vcvtph2uqq",
	"vcvtph2uw",
	"vcvtph2w",
	"vcvtps2phx",
	"vcvtqq2ph",
	"vcvtsd2sh",
	"vcvtsh2sd",
	"vcvtsh2si",
	"vcvtsh2ss",
	"vcvtsh2usi",
	"vcvtsi2sh",
	"vcvtss2sh",
	"vcvttph2dq",
	"vcvttph2qq",
	"vcvttph2udq",
	"vcvttph2uqq",
	"vcvttph2uw",
	"vcvttph2w",
	"vcvttsh2si",
	"vcvttsh2usi",
	"vcvtudq2ph",
	"vcvtuqq2ph",
	"vcvtusi2sh",
	"vcvtuw2ph",
	"vcvtw2ph",
	"vdivph",
	"vdivsh",
	"vfcmaddcph",
	"vfmaddcph",
	"vfcmaddcsh",
	"vfmaddcsh",
	"vfcmulcph",
	"vfmulcph",
	"vfcmulcsh",
	"vfmulcsh",
	"vfmaddsub132ph",
	"vfmaddsub213ph",
	"vfmaddsub231ph",
	"vfmsubadd132ph",
	"vfmsubadd213ph",
	"vfmsubadd231ph",
	"vfmadd132ph",
	"vfmadd213ph",
	"vfmadd231ph",
	"vfnmadd132ph",
	"vfnmadd213ph",
	"vfnmadd231ph",
	"vfmadd132sh",
	"vfmadd213sh",
	"vfmadd231sh",
	"vfnmadd132sh",
	"vfnmadd213sh",
	"vfnmadd231sh",
	"vfmsub132ph",
	"vfmsub213ph",
	"vfmsub231ph",
	"vfnmsub132ph",
	"vfnmsub213ph",
	"vfnmsub231ph",
	"vfmsub132sh",
	"vfmsub213sh",
	"vfmsub231sh",
	"vfnmsub132sh",
	"vfnmsub213sh",
	"vfnmsub231sh",
	"vfpclassph",
	"vfpclasssh",
	"vgetexpph",
	"vgetexpsh",
	"vgetmantph",
	"vgetmantsh",
	"vmaxph",
	"vmaxsh",
	"vminph",
	"vminsh",
	"vmovsh",
	"vmovw",
	"vmulph",
	"vmulsh",
	"vrcpph",
	"vrcpsh",
	"vreduceph",
	"vreducesh",
	"vendscaleph",
	"vendscalesh",
	"vrsqrtph",
	"vrsqrtsh",
	"vscalefph",
	"vscalefsh",
	"vsqrtph",
	"vsqrtsh",
	"vsubph",
	"vsubsh",
	"vucomish",
	"aadd",
	"aand",
	"aor",
	"axor",
	"clui",
	"senduipi",
	"stui",
	"testui",
	"uiret",
	"hreset",
	"vaddbf16",
	"vcmpbf16",
	"vcomisbf16",
	"vdivbf16",
	"vfmadd132bf16",
	"vfmadd213bf16",
	"vfmadd231bf16",
	"vfmsub132bf16",
	"vfmsub213bf16",
	"vfmsub231bf16",
	"vfnmadd132bf16",
	"vfnmadd213bf16",
	"vfnmadd231bf16",
	"vfnmsub132bf16",
	"vfnmsub213bf16",
	"vfnmsub231bf16",
	"vfpclassbf16",
	"vgetexpbf16",
	"vgetmantbf16",
	"vmaxbf16",
	"vminbf16",
	"vmulbf16",
	"vrcpbf16",
	"vreducebf16",
	"vrndscalebf16",
	"vrsqrtbf16",
	"vscalefbf16",
	"vsqrtbf16",
	"vsubbf16",
	"vcomxsd",
	"vcomxsh",
	"vcomxss",
	"vucomxsd",
	"vucomxsh",
	"vucomxss",
	"vcvt2ph2bf8",
	"vcvt2ph2bf8s",
	"vcvt2ph2hf8",
	"vcvt2ph2hf8s",
	"vcvtph2bf8",
	"vcvtph2bf8s",
	"vcvtph2hf8",
	"vcvtph2hf8s",
	"vcvt2ps2phx",
	"vcvtbiasph2bf8",
	"vcvtbiasph2bf8s",
	"vcvtbiasph2hf8",
	"vcvtbiasph2hf8s",
	"vcvthf82ph",
	"vdpphps",
	"vminmaxbf16",
	"vminmaxpd",
	"vminmaxph",
	"vminmaxps",
	"vminmaxsd",
	"vminmaxsh",
	"vminmaxss",
	"vcvtbf162ibs",
	"vcvtbf162iubs",
	"vcvttbf162ibs",
	"vcvttbf162iubs",
	"vcvttpd2dqs",
	"vcvttpd2qqs",
	"vcvttpd2udqs",
	"vcvttpd2uqqs",
	"vcvtph2ibs",
	"vcvtph2iubs",
	"vcvttph2ibs",
	"vcvttph2iubs",
	"vcvttps2dqs",
	"vcvtps2ibs",
	"vcvtps2iubs",
	"vcvttps2ibs",
	"vcvttps2iubs",
	"vcvttps2qqs",
	"vcvttps2udqs",
	"vcvttps2uqqs",
	"vcvttsd2sis",
	"vcvttsd2usis",
	"vcvttss2sis",
	"vcvttss2usis",
	"hint_nop",
	"hint_nop0",
	"hint_nop1",
	"hint_nop2",
	"hint_nop3",
	"hint_nop4",
	"hint_nop5",
	"hint_nop6",
	"hint_nop7",
	"hint_nop8",
	"hint_nop9",
	"hint_nop10",
	"hint_nop11",
	"hint_nop12",
	"hint_nop13",
	"hint_nop14",
	"hint_nop15",
	"hint_nop16",
	"hint_nop17",
	"hint_nop18",
	"hint_nop19",
	"hint_nop20",
	"hint_nop21",
	"hint_nop22",
	"hint_nop23",
	"hint_nop24",
	"hint_nop25",
	"hint_nop26",
	"hint_nop27",
	"hint_nop28",
	"hint_nop29",
	"hint_nop30",
	"hint_nop31",
	"hint_nop32",
	"hint_nop33",
	"hint_nop34",
	"hint_nop35",
	"hint_nop36",
	"hint_nop37",
	"hint_nop38",
	"hint_nop39",
	"hint_nop40",
	"hint_nop41",
	"hint_nop42",
	"hint_nop43",
	"hint_nop44",
	"hint_nop45",
	"hint_nop46",
	"hint_nop47",
	"hint_nop48",
	"hint_nop49",
	"hint_nop50",
	"hint_nop51",
	"hint_nop52",
	"hint_nop53",
	"hint_nop54",
	"hint_nop55",
	"hint_nop56",
	"hint_nop57",
	"hint_nop58",
	"hint_nop59",
	"hint_nop60",
	"hint_nop61",
	"hint_nop62",
	"hint_nop63",
};
