%pragma
%pragma bluttan
%pragma bluttan blej
%pragma "Hej tomtegubbar"
%define PR asm foobar
%pragma PR
%pragma preproc
%pragma preproc tjo fidelittan preproc
%pragma dbg tjo fidelittan output
%pragma dbgdbg tjo fidelittan debug format
%pragma Dbg Tjo Fidelittan Output
%pragma Dbgdbg Tjo Fidelittan Debug Format
