;Testname=unoptimized; Arguments=-fbin -oriprel.bin -O0; Files=stdout stderr riprel.bin
;Testname=optimized;   Arguments=-fbin -oriprel.bin -Ox; Files=stdout stderr riprel.bin
	bits 64

	;; dotest [seg:]
%macro dotest 0-1
	mov al,[%1 foo]
	mov bl,[%1 foo]
	mov ax,[%1 foo]
	mov bx,[%1 foo]
	mov eax,[%1 foo]
	mov ebx,[%1 foo]
	mov rax,[%1 foo]
	mov rbx,[%1 foo]
	mov al,[%1 0xaaaaaaaaaaaaaaaa]
	mov bl,[%1 0xaaaaaaaaaaaaaaaa]
	mov ax,[%1 0xaaaaaaaaaaaaaaaa]
	mov bx,[%1 0xaaaaaaaaaaaaaaaa]
	mov eax,[%1 0xaaaaaaaaaaaaaaaa]
	mov ebx,[%1 0xaaaaaaaaaaaaaaaa]
	mov rax,[%1 0xaaaaaaaaaaaaaaaa]
	mov rbx,[%1 0xaaaaaaaaaaaaaaaa]
	mov al,[%1 0xbbbbbbbb]
	mov bl,[%1 0xbbbbbbbb]
	mov ax,[%1 0xbbbbbbbb]
	mov bx,[%1 0xbbbbbbbb]
	mov eax,[%1 0xbbbbbbbb]
	mov ebx,[%1 0xbbbbbbbb]
	mov rax,[%1 0xbbbbbbbb]
	mov rbx,[%1 0xbbbbbbbb]
	mov al,[%1 0xffffffffcccccccc]
	mov bl,[%1 0xffffffffcccccccc]
	mov ax,[%1 0xffffffffcccccccc]
	mov bx,[%1 0xffffffffcccccccc]
	mov eax,[%1 0xffffffffcccccccc]
	mov ebx,[%1 0xffffffffcccccccc]
	mov rax,[%1 0xffffffffcccccccc]
	mov rbx,[%1 0xffffffffcccccccc]

	mov al,[%1 dword foo]
	mov bl,[%1 dword foo]
	mov ax,[%1 dword foo]
	mov bx,[%1 dword foo]
	mov eax,[%1 dword foo]
	mov ebx,[%1 dword foo]
	mov rax,[%1 dword foo]
	mov rbx,[%1 dword foo]
	mov al,[%1 dword 0xaaaaaaaaaaaaaaaa]
	mov bl,[%1 dword 0xaaaaaaaaaaaaaaaa]
	mov ax,[%1 dword 0xaaaaaaaaaaaaaaaa]
	mov bx,[%1 dword 0xaaaaaaaaaaaaaaaa]
	mov eax,[%1 dword 0xaaaaaaaaaaaaaaaa]
	mov ebx,[%1 dword 0xaaaaaaaaaaaaaaaa]
	mov rax,[%1 dword 0xaaaaaaaaaaaaaaaa]
	mov rbx,[%1 dword 0xaaaaaaaaaaaaaaaa]
	mov al,[%1 dword 0xbbbbbbbb]
	mov bl,[%1 dword 0xbbbbbbbb]
	mov ax,[%1 dword 0xbbbbbbbb]
	mov bx,[%1 dword 0xbbbbbbbb]
	mov eax,[%1 dword 0xbbbbbbbb]
	mov ebx,[%1 dword 0xbbbbbbbb]
	mov rax,[%1 dword 0xbbbbbbbb]
	mov rbx,[%1 dword 0xbbbbbbbb]
	mov al,[%1 dword 0xffffffffcccccccc]
	mov bl,[%1 dword 0xffffffffcccccccc]
	mov ax,[%1 dword 0xffffffffcccccccc]
	mov bx,[%1 dword 0xffffffffcccccccc]
	mov eax,[%1 dword 0xffffffffcccccccc]
	mov ebx,[%1 dword 0xffffffffcccccccc]
	mov rax,[%1 dword 0xffffffffcccccccc]
	mov rbx,[%1 dword 0xffffffffcccccccc]

	mov al,[%1 qword foo]
	mov bl,[%1 qword foo]
	mov ax,[%1 qword foo]
	mov bx,[%1 qword foo]
	mov eax,[%1 qword foo]
	mov ebx,[%1 qword foo]
	mov rax,[%1 qword foo]
	mov rbx,[%1 qword foo]
	mov al,[%1 qword 0xaaaaaaaaaaaaaaaa]
	mov bl,[%1 qword 0xaaaaaaaaaaaaaaaa]
	mov ax,[%1 qword 0xaaaaaaaaaaaaaaaa]
	mov bx,[%1 qword 0xaaaaaaaaaaaaaaaa]
	mov eax,[%1 qword 0xaaaaaaaaaaaaaaaa]
	mov ebx,[%1 qword 0xaaaaaaaaaaaaaaaa]
	mov rax,[%1 qword 0xaaaaaaaaaaaaaaaa]
	mov rbx,[%1 qword 0xaaaaaaaaaaaaaaaa]
	mov al,[%1 qword 0xbbbbbbbb]
	mov bl,[%1 qword 0xbbbbbbbb]
	mov ax,[%1 qword 0xbbbbbbbb]
	mov bx,[%1 qword 0xbbbbbbbb]
	mov eax,[%1 qword 0xbbbbbbbb]
	mov ebx,[%1 qword 0xbbbbbbbb]
	mov rax,[%1 qword 0xbbbbbbbb]
	mov rbx,[%1 qword 0xbbbbbbbb]
	mov al,[%1 qword 0xffffffffcccccccc]
	mov bl,[%1 qword 0xffffffffcccccccc]
	mov ax,[%1 qword 0xffffffffcccccccc]
	mov bx,[%1 qword 0xffffffffcccccccc]
	mov eax,[%1 qword 0xffffffffcccccccc]
	mov ebx,[%1 qword 0xffffffffcccccccc]
	mov rax,[%1 qword 0xffffffffcccccccc]
	mov rbx,[%1 qword 0xffffffffcccccccc]

	mov al,[%1 a64 foo]
	mov bl,[%1 a64 foo]
	mov ax,[%1 a64 foo]
	mov bx,[%1 a64 foo]
	mov eax,[%1 a64 foo]
	mov ebx,[%1 a64 foo]
	mov rax,[%1 a64 foo]
	mov rbx,[%1 a64 foo]
	mov al,[%1 a64 0xaaaaaaaaaaaaaaaa]
	mov bl,[%1 a64 0xaaaaaaaaaaaaaaaa]
	mov ax,[%1 a64 0xaaaaaaaaaaaaaaaa]
	mov bx,[%1 a64 0xaaaaaaaaaaaaaaaa]
	mov eax,[%1 a64 0xaaaaaaaaaaaaaaaa]
	mov ebx,[%1 a64 0xaaaaaaaaaaaaaaaa]
	mov rax,[%1 a64 0xaaaaaaaaaaaaaaaa]
	mov rbx,[%1 a64 0xaaaaaaaaaaaaaaaa]
	mov al,[%1 a64 0xbbbbbbbb]
	mov bl,[%1 a64 0xbbbbbbbb]
	mov ax,[%1 a64 0xbbbbbbbb]
	mov bx,[%1 a64 0xbbbbbbbb]
	mov eax,[%1 a64 0xbbbbbbbb]
	mov ebx,[%1 a64 0xbbbbbbbb]
	mov rax,[%1 a64 0xbbbbbbbb]
	mov rbx,[%1 a64 0xbbbbbbbb]
	mov al,[%1 a64 0xffffffffcccccccc]
	mov bl,[%1 a64 0xffffffffcccccccc]
	mov ax,[%1 a64 0xffffffffcccccccc]
	mov bx,[%1 a64 0xffffffffcccccccc]
	mov eax,[%1 a64 0xffffffffcccccccc]
	mov ebx,[%1 a64 0xffffffffcccccccc]
	mov rax,[%1 a64 0xffffffffcccccccc]
	mov rbx,[%1 a64 0xffffffffcccccccc]

	mov al,[%1 a64 dword foo]
	mov bl,[%1 a64 dword foo]
	mov ax,[%1 a64 dword foo]
	mov bx,[%1 a64 dword foo]
	mov eax,[%1 a64 dword foo]
	mov ebx,[%1 a64 dword foo]
	mov rax,[%1 a64 dword foo]
	mov rbx,[%1 a64 dword foo]
	mov al,[%1 a64 dword 0xaaaaaaaaaaaaaaaa]
	mov bl,[%1 a64 dword 0xaaaaaaaaaaaaaaaa]
	mov ax,[%1 a64 dword 0xaaaaaaaaaaaaaaaa]
	mov bx,[%1 a64 dword 0xaaaaaaaaaaaaaaaa]
	mov eax,[%1 a64 dword 0xaaaaaaaaaaaaaaaa]
	mov ebx,[%1 a64 dword 0xaaaaaaaaaaaaaaaa]
	mov rax,[%1 a64 dword 0xaaaaaaaaaaaaaaaa]
	mov rbx,[%1 a64 dword 0xaaaaaaaaaaaaaaaa]
	mov al,[%1 a64 dword 0xbbbbbbbb]
	mov bl,[%1 a64 dword 0xbbbbbbbb]
	mov ax,[%1 a64 dword 0xbbbbbbbb]
	mov bx,[%1 a64 dword 0xbbbbbbbb]
	mov eax,[%1 a64 dword 0xbbbbbbbb]
	mov ebx,[%1 a64 dword 0xbbbbbbbb]
	mov rax,[%1 a64 dword 0xbbbbbbbb]
	mov rbx,[%1 a64 dword 0xbbbbbbbb]
	mov al,[%1 a64 dword 0xffffffffcccccccc]
	mov bl,[%1 a64 dword 0xffffffffcccccccc]
	mov ax,[%1 a64 dword 0xffffffffcccccccc]
	mov bx,[%1 a64 dword 0xffffffffcccccccc]
	mov eax,[%1 a64 dword 0xffffffffcccccccc]
	mov ebx,[%1 a64 dword 0xffffffffcccccccc]
	mov rax,[%1 a64 dword 0xffffffffcccccccc]
	mov rbx,[%1 a64 dword 0xffffffffcccccccc]

	mov al,[%1 a64 qword foo]
	mov bl,[%1 a64 qword foo]
	mov ax,[%1 a64 qword foo]
	mov bx,[%1 a64 qword foo]
	mov eax,[%1 a64 qword foo]
	mov ebx,[%1 a64 qword foo]
	mov rax,[%1 a64 qword foo]
	mov rbx,[%1 a64 qword foo]
	mov al,[%1 a64 qword 0xaaaaaaaaaaaaaaaa]
	mov bl,[%1 a64 qword 0xaaaaaaaaaaaaaaaa]
	mov ax,[%1 a64 qword 0xaaaaaaaaaaaaaaaa]
	mov bx,[%1 a64 qword 0xaaaaaaaaaaaaaaaa]
	mov eax,[%1 a64 qword 0xaaaaaaaaaaaaaaaa]
	mov ebx,[%1 a64 qword 0xaaaaaaaaaaaaaaaa]
	mov rax,[%1 a64 qword 0xaaaaaaaaaaaaaaaa]
	mov rbx,[%1 a64 qword 0xaaaaaaaaaaaaaaaa]
	mov al,[%1 a64 qword 0xbbbbbbbb]
	mov bl,[%1 a64 qword 0xbbbbbbbb]
	mov ax,[%1 a64 qword 0xbbbbbbbb]
	mov bx,[%1 a64 qword 0xbbbbbbbb]
	mov eax,[%1 a64 qword 0xbbbbbbbb]
	mov ebx,[%1 a64 qword 0xbbbbbbbb]
	mov rax,[%1 a64 qword 0xbbbbbbbb]
	mov rbx,[%1 a64 qword 0xbbbbbbbb]
	mov al,[%1 a64 qword 0xffffffffcccccccc]
	mov bl,[%1 a64 qword 0xffffffffcccccccc]
	mov ax,[%1 a64 qword 0xffffffffcccccccc]
	mov bx,[%1 a64 qword 0xffffffffcccccccc]
	mov eax,[%1 a64 qword 0xffffffffcccccccc]
	mov ebx,[%1 a64 qword 0xffffffffcccccccc]
	mov rax,[%1 a64 qword 0xffffffffcccccccc]
	mov rbx,[%1 a64 qword 0xffffffffcccccccc]

	mov al,[%1 a32 foo]
	mov bl,[%1 a32 foo]
	mov ax,[%1 a32 foo]
	mov bx,[%1 a32 foo]
	mov eax,[%1 a32 foo]
	mov ebx,[%1 a32 foo]
	mov rax,[%1 a32 foo]
	mov rbx,[%1 a32 foo]
	mov al,[%1 a32 0xaaaaaaaaaaaaaaaa]
	mov bl,[%1 a32 0xaaaaaaaaaaaaaaaa]
	mov ax,[%1 a32 0xaaaaaaaaaaaaaaaa]
	mov bx,[%1 a32 0xaaaaaaaaaaaaaaaa]
	mov eax,[%1 a32 0xaaaaaaaaaaaaaaaa]
	mov ebx,[%1 a32 0xaaaaaaaaaaaaaaaa]
	mov rax,[%1 a32 0xaaaaaaaaaaaaaaaa]
	mov rbx,[%1 a32 0xaaaaaaaaaaaaaaaa]
	mov al,[%1 a32 0xbbbbbbbb]
	mov bl,[%1 a32 0xbbbbbbbb]
	mov ax,[%1 a32 0xbbbbbbbb]
	mov bx,[%1 a32 0xbbbbbbbb]
	mov eax,[%1 a32 0xbbbbbbbb]
	mov ebx,[%1 a32 0xbbbbbbbb]
	mov rax,[%1 a32 0xbbbbbbbb]
	mov rbx,[%1 a32 0xbbbbbbbb]
	mov al,[%1 a32 0xffffffffcccccccc]
	mov bl,[%1 a32 0xffffffffcccccccc]
	mov ax,[%1 a32 0xffffffffcccccccc]
	mov bx,[%1 a32 0xffffffffcccccccc]
	mov eax,[%1 a32 0xffffffffcccccccc]
	mov ebx,[%1 a32 0xffffffffcccccccc]
	mov rax,[%1 a32 0xffffffffcccccccc]
	mov rbx,[%1 a32 0xffffffffcccccccc]

	mov al,[%1 a32 dword foo]
	mov bl,[%1 a32 dword foo]
	mov ax,[%1 a32 dword foo]
	mov bx,[%1 a32 dword foo]
	mov eax,[%1 a32 dword foo]
	mov ebx,[%1 a32 dword foo]
	mov rax,[%1 a32 dword foo]
	mov rbx,[%1 a32 dword foo]
	mov al,[%1 a32 dword 0xaaaaaaaaaaaaaaaa]
	mov bl,[%1 a32 dword 0xaaaaaaaaaaaaaaaa]
	mov ax,[%1 a32 dword 0xaaaaaaaaaaaaaaaa]
	mov bx,[%1 a32 dword 0xaaaaaaaaaaaaaaaa]
	mov eax,[%1 a32 dword 0xaaaaaaaaaaaaaaaa]
	mov ebx,[%1 a32 dword 0xaaaaaaaaaaaaaaaa]
	mov rax,[%1 a32 dword 0xaaaaaaaaaaaaaaaa]
	mov rbx,[%1 a32 dword 0xaaaaaaaaaaaaaaaa]
	mov al,[%1 a32 dword 0xbbbbbbbb]
	mov bl,[%1 a32 dword 0xbbbbbbbb]
	mov ax,[%1 a32 dword 0xbbbbbbbb]
	mov bx,[%1 a32 dword 0xbbbbbbbb]
	mov eax,[%1 a32 dword 0xbbbbbbbb]
	mov ebx,[%1 a32 dword 0xbbbbbbbb]
	mov rax,[%1 a32 dword 0xbbbbbbbb]
	mov rbx,[%1 a32 dword 0xbbbbbbbb]
	mov al,[%1 a32 dword 0xffffffffcccccccc]
	mov bl,[%1 a32 dword 0xffffffffcccccccc]
	mov ax,[%1 a32 dword 0xffffffffcccccccc]
	mov bx,[%1 a32 dword 0xffffffffcccccccc]
	mov eax,[%1 a32 dword 0xffffffffcccccccc]
	mov ebx,[%1 a32 dword 0xffffffffcccccccc]
	mov rax,[%1 a32 dword 0xffffffffcccccccc]
	mov rbx,[%1 a32 dword 0xffffffffcccccccc]

	mov al,[%1 a32 qword foo]
	mov bl,[%1 a32 qword foo]
	mov ax,[%1 a32 qword foo]
	mov bx,[%1 a32 qword foo]
	mov eax,[%1 a32 qword foo]
	mov ebx,[%1 a32 qword foo]
	mov rax,[%1 a32 qword foo]
	mov rbx,[%1 a32 qword foo]
	mov al,[%1 a32 qword 0xaaaaaaaaaaaaaaaa]
	mov bl,[%1 a32 qword 0xaaaaaaaaaaaaaaaa]
	mov ax,[%1 a32 qword 0xaaaaaaaaaaaaaaaa]
	mov bx,[%1 a32 qword 0xaaaaaaaaaaaaaaaa]
	mov eax,[%1 a32 qword 0xaaaaaaaaaaaaaaaa]
	mov ebx,[%1 a32 qword 0xaaaaaaaaaaaaaaaa]
	mov rax,[%1 a32 qword 0xaaaaaaaaaaaaaaaa]
	mov rbx,[%1 a32 qword 0xaaaaaaaaaaaaaaaa]
	mov al,[%1 a32 qword 0xbbbbbbbb]
	mov bl,[%1 a32 qword 0xbbbbbbbb]
	mov ax,[%1 a32 qword 0xbbbbbbbb]
	mov bx,[%1 a32 qword 0xbbbbbbbb]
	mov eax,[%1 a32 qword 0xbbbbbbbb]
	mov ebx,[%1 a32 qword 0xbbbbbbbb]
	mov rax,[%1 a32 qword 0xbbbbbbbb]
	mov rbx,[%1 a32 qword 0xbbbbbbbb]
	mov al,[%1 a32 qword 0xffffffffcccccccc]
	mov bl,[%1 a32 qword 0xffffffffcccccccc]
	mov ax,[%1 a32 qword 0xffffffffcccccccc]
	mov bx,[%1 a32 qword 0xffffffffcccccccc]
	mov eax,[%1 a32 qword 0xffffffffcccccccc]
	mov ebx,[%1 a32 qword 0xffffffffcccccccc]
	mov rax,[%1 a32 qword 0xffffffffcccccccc]
	mov rbx,[%1 a32 qword 0xffffffffcccccccc]

	mov al,[%1 abs foo]
	mov bl,[%1 abs foo]
	mov ax,[%1 abs foo]
	mov bx,[%1 abs foo]
	mov eax,[%1 abs foo]
	mov ebx,[%1 abs foo]
	mov rax,[%1 abs foo]
	mov rbx,[%1 abs foo]
	mov al,[%1 abs 0xaaaaaaaaaaaaaaaa]
	mov bl,[%1 abs 0xaaaaaaaaaaaaaaaa]
	mov ax,[%1 abs 0xaaaaaaaaaaaaaaaa]
	mov bx,[%1 abs 0xaaaaaaaaaaaaaaaa]
	mov eax,[%1 abs 0xaaaaaaaaaaaaaaaa]
	mov ebx,[%1 abs 0xaaaaaaaaaaaaaaaa]
	mov rax,[%1 abs 0xaaaaaaaaaaaaaaaa]
	mov rbx,[%1 abs 0xaaaaaaaaaaaaaaaa]
	mov al,[%1 abs 0xbbbbbbbb]
	mov bl,[%1 abs 0xbbbbbbbb]
	mov ax,[%1 abs 0xbbbbbbbb]
	mov bx,[%1 abs 0xbbbbbbbb]
	mov eax,[%1 abs 0xbbbbbbbb]
	mov ebx,[%1 abs 0xbbbbbbbb]
	mov rax,[%1 abs 0xbbbbbbbb]
	mov rbx,[%1 abs 0xbbbbbbbb]
	mov al,[%1 abs 0xffffffffcccccccc]
	mov bl,[%1 abs 0xffffffffcccccccc]
	mov ax,[%1 abs 0xffffffffcccccccc]
	mov bx,[%1 abs 0xffffffffcccccccc]
	mov eax,[%1 abs 0xffffffffcccccccc]
	mov ebx,[%1 abs 0xffffffffcccccccc]
	mov rax,[%1 abs 0xffffffffcccccccc]
	mov rbx,[%1 abs 0xffffffffcccccccc]

	mov al,[%1 abs dword foo]
	mov bl,[%1 abs dword foo]
	mov ax,[%1 abs dword foo]
	mov bx,[%1 abs dword foo]
	mov eax,[%1 abs dword foo]
	mov ebx,[%1 abs dword foo]
	mov rax,[%1 abs dword foo]
	mov rbx,[%1 abs dword foo]
	mov al,[%1 abs dword 0xaaaaaaaaaaaaaaaa]
	mov bl,[%1 abs dword 0xaaaaaaaaaaaaaaaa]
	mov ax,[%1 abs dword 0xaaaaaaaaaaaaaaaa]
	mov bx,[%1 abs dword 0xaaaaaaaaaaaaaaaa]
	mov eax,[%1 abs dword 0xaaaaaaaaaaaaaaaa]
	mov ebx,[%1 abs dword 0xaaaaaaaaaaaaaaaa]
	mov rax,[%1 abs dword 0xaaaaaaaaaaaaaaaa]
	mov rbx,[%1 abs dword 0xaaaaaaaaaaaaaaaa]
	mov al,[%1 abs dword 0xbbbbbbbb]
	mov bl,[%1 abs dword 0xbbbbbbbb]
	mov ax,[%1 abs dword 0xbbbbbbbb]
	mov bx,[%1 abs dword 0xbbbbbbbb]
	mov eax,[%1 abs dword 0xbbbbbbbb]
	mov ebx,[%1 abs dword 0xbbbbbbbb]
	mov rax,[%1 abs dword 0xbbbbbbbb]
	mov rbx,[%1 abs dword 0xbbbbbbbb]
	mov al,[%1 abs dword 0xffffffffcccccccc]
	mov bl,[%1 abs dword 0xffffffffcccccccc]
	mov ax,[%1 abs dword 0xffffffffcccccccc]
	mov bx,[%1 abs dword 0xffffffffcccccccc]
	mov eax,[%1 abs dword 0xffffffffcccccccc]
	mov ebx,[%1 abs dword 0xffffffffcccccccc]
	mov rax,[%1 abs dword 0xffffffffcccccccc]
	mov rbx,[%1 abs dword 0xffffffffcccccccc]

	mov al,[%1 abs qword foo]
	mov bl,[%1 abs qword foo]
	mov ax,[%1 abs qword foo]
	mov bx,[%1 abs qword foo]
	mov eax,[%1 abs qword foo]
	mov ebx,[%1 abs qword foo]
	mov rax,[%1 abs qword foo]
	mov rbx,[%1 abs qword foo]
	mov al,[%1 abs qword 0xaaaaaaaaaaaaaaaa]
	mov bl,[%1 abs qword 0xaaaaaaaaaaaaaaaa]
	mov ax,[%1 abs qword 0xaaaaaaaaaaaaaaaa]
	mov bx,[%1 abs qword 0xaaaaaaaaaaaaaaaa]
	mov eax,[%1 abs qword 0xaaaaaaaaaaaaaaaa]
	mov ebx,[%1 abs qword 0xaaaaaaaaaaaaaaaa]
	mov rax,[%1 abs qword 0xaaaaaaaaaaaaaaaa]
	mov rbx,[%1 abs qword 0xaaaaaaaaaaaaaaaa]
	mov al,[%1 abs qword 0xbbbbbbbb]
	mov bl,[%1 abs qword 0xbbbbbbbb]
	mov ax,[%1 abs qword 0xbbbbbbbb]
	mov bx,[%1 abs qword 0xbbbbbbbb]
	mov eax,[%1 abs qword 0xbbbbbbbb]
	mov ebx,[%1 abs qword 0xbbbbbbbb]
	mov rax,[%1 abs qword 0xbbbbbbbb]
	mov rbx,[%1 abs qword 0xbbbbbbbb]
	mov al,[%1 abs qword 0xffffffffcccccccc]
	mov bl,[%1 abs qword 0xffffffffcccccccc]
	mov ax,[%1 abs qword 0xffffffffcccccccc]
	mov bx,[%1 abs qword 0xffffffffcccccccc]
	mov eax,[%1 abs qword 0xffffffffcccccccc]
	mov ebx,[%1 abs qword 0xffffffffcccccccc]
	mov rax,[%1 abs qword 0xffffffffcccccccc]
	mov rbx,[%1 abs qword 0xffffffffcccccccc]

	mov al,[%1 a64 abs foo]
	mov bl,[%1 a64 abs foo]
	mov ax,[%1 a64 abs foo]
	mov bx,[%1 a64 abs foo]
	mov eax,[%1 a64 abs foo]
	mov ebx,[%1 a64 abs foo]
	mov rax,[%1 a64 abs foo]
	mov rbx,[%1 a64 abs foo]
	mov al,[%1 a64 abs 0xaaaaaaaaaaaaaaaa]
	mov bl,[%1 a64 abs 0xaaaaaaaaaaaaaaaa]
	mov ax,[%1 a64 abs 0xaaaaaaaaaaaaaaaa]
	mov bx,[%1 a64 abs 0xaaaaaaaaaaaaaaaa]
	mov eax,[%1 a64 abs 0xaaaaaaaaaaaaaaaa]
	mov ebx,[%1 a64 abs 0xaaaaaaaaaaaaaaaa]
	mov rax,[%1 a64 abs 0xaaaaaaaaaaaaaaaa]
	mov rbx,[%1 a64 abs 0xaaaaaaaaaaaaaaaa]
	mov al,[%1 a64 abs 0xbbbbbbbb]
	mov bl,[%1 a64 abs 0xbbbbbbbb]
	mov ax,[%1 a64 abs 0xbbbbbbbb]
	mov bx,[%1 a64 abs 0xbbbbbbbb]
	mov eax,[%1 a64 abs 0xbbbbbbbb]
	mov ebx,[%1 a64 abs 0xbbbbbbbb]
	mov rax,[%1 a64 abs 0xbbbbbbbb]
	mov rbx,[%1 a64 abs 0xbbbbbbbb]
	mov al,[%1 a64 abs 0xffffffffcccccccc]
	mov bl,[%1 a64 abs 0xffffffffcccccccc]
	mov ax,[%1 a64 abs 0xffffffffcccccccc]
	mov bx,[%1 a64 abs 0xffffffffcccccccc]
	mov eax,[%1 a64 abs 0xffffffffcccccccc]
	mov ebx,[%1 a64 abs 0xffffffffcccccccc]
	mov rax,[%1 a64 abs 0xffffffffcccccccc]
	mov rbx,[%1 a64 abs 0xffffffffcccccccc]

	mov al,[%1 a64 abs dword foo]
	mov bl,[%1 a64 abs dword foo]
	mov ax,[%1 a64 abs dword foo]
	mov bx,[%1 a64 abs dword foo]
	mov eax,[%1 a64 abs dword foo]
	mov ebx,[%1 a64 abs dword foo]
	mov rax,[%1 a64 abs dword foo]
	mov rbx,[%1 a64 abs dword foo]
	mov al,[%1 a64 abs dword 0xaaaaaaaaaaaaaaaa]
	mov bl,[%1 a64 abs dword 0xaaaaaaaaaaaaaaaa]
	mov ax,[%1 a64 abs dword 0xaaaaaaaaaaaaaaaa]
	mov bx,[%1 a64 abs dword 0xaaaaaaaaaaaaaaaa]
	mov eax,[%1 a64 abs dword 0xaaaaaaaaaaaaaaaa]
	mov ebx,[%1 a64 abs dword 0xaaaaaaaaaaaaaaaa]
	mov rax,[%1 a64 abs dword 0xaaaaaaaaaaaaaaaa]
	mov rbx,[%1 a64 abs dword 0xaaaaaaaaaaaaaaaa]
	mov al,[%1 a64 abs dword 0xbbbbbbbb]
	mov bl,[%1 a64 abs dword 0xbbbbbbbb]
	mov ax,[%1 a64 abs dword 0xbbbbbbbb]
	mov bx,[%1 a64 abs dword 0xbbbbbbbb]
	mov eax,[%1 a64 abs dword 0xbbbbbbbb]
	mov ebx,[%1 a64 abs dword 0xbbbbbbbb]
	mov rax,[%1 a64 abs dword 0xbbbbbbbb]
	mov rbx,[%1 a64 abs dword 0xbbbbbbbb]
	mov al,[%1 a64 abs dword 0xffffffffcccccccc]
	mov bl,[%1 a64 abs dword 0xffffffffcccccccc]
	mov ax,[%1 a64 abs dword 0xffffffffcccccccc]
	mov bx,[%1 a64 abs dword 0xffffffffcccccccc]
	mov eax,[%1 a64 abs dword 0xffffffffcccccccc]
	mov ebx,[%1 a64 abs dword 0xffffffffcccccccc]
	mov rax,[%1 a64 abs dword 0xffffffffcccccccc]
	mov rbx,[%1 a64 abs dword 0xffffffffcccccccc]

	mov al,[%1 a64 abs qword foo]
	mov bl,[%1 a64 abs qword foo]
	mov ax,[%1 a64 abs qword foo]
	mov bx,[%1 a64 abs qword foo]
	mov eax,[%1 a64 abs qword foo]
	mov ebx,[%1 a64 abs qword foo]
	mov rax,[%1 a64 abs qword foo]
	mov rbx,[%1 a64 abs qword foo]
	mov al,[%1 a64 abs qword 0xaaaaaaaaaaaaaaaa]
	mov bl,[%1 a64 abs qword 0xaaaaaaaaaaaaaaaa]
	mov ax,[%1 a64 abs qword 0xaaaaaaaaaaaaaaaa]
	mov bx,[%1 a64 abs qword 0xaaaaaaaaaaaaaaaa]
	mov eax,[%1 a64 abs qword 0xaaaaaaaaaaaaaaaa]
	mov ebx,[%1 a64 abs qword 0xaaaaaaaaaaaaaaaa]
	mov rax,[%1 a64 abs qword 0xaaaaaaaaaaaaaaaa]
	mov rbx,[%1 a64 abs qword 0xaaaaaaaaaaaaaaaa]
	mov al,[%1 a64 abs qword 0xbbbbbbbb]
	mov bl,[%1 a64 abs qword 0xbbbbbbbb]
	mov ax,[%1 a64 abs qword 0xbbbbbbbb]
	mov bx,[%1 a64 abs qword 0xbbbbbbbb]
	mov eax,[%1 a64 abs qword 0xbbbbbbbb]
	mov ebx,[%1 a64 abs qword 0xbbbbbbbb]
	mov rax,[%1 a64 abs qword 0xbbbbbbbb]
	mov rbx,[%1 a64 abs qword 0xbbbbbbbb]
	mov al,[%1 a64 abs qword 0xffffffffcccccccc]
	mov bl,[%1 a64 abs qword 0xffffffffcccccccc]
	mov ax,[%1 a64 abs qword 0xffffffffcccccccc]
	mov bx,[%1 a64 abs qword 0xffffffffcccccccc]
	mov eax,[%1 a64 abs qword 0xffffffffcccccccc]
	mov ebx,[%1 a64 abs qword 0xffffffffcccccccc]
	mov rax,[%1 a64 abs qword 0xffffffffcccccccc]
	mov rbx,[%1 a64 abs qword 0xffffffffcccccccc]

	mov al,[%1 a32 abs foo]
	mov bl,[%1 a32 abs foo]
	mov ax,[%1 a32 abs foo]
	mov bx,[%1 a32 abs foo]
	mov eax,[%1 a32 abs foo]
	mov ebx,[%1 a32 abs foo]
	mov rax,[%1 a32 abs foo]
	mov rbx,[%1 a32 abs foo]
	mov al,[%1 a32 abs 0xaaaaaaaaaaaaaaaa]
	mov bl,[%1 a32 abs 0xaaaaaaaaaaaaaaaa]
	mov ax,[%1 a32 abs 0xaaaaaaaaaaaaaaaa]
	mov bx,[%1 a32 abs 0xaaaaaaaaaaaaaaaa]
	mov eax,[%1 a32 abs 0xaaaaaaaaaaaaaaaa]
	mov ebx,[%1 a32 abs 0xaaaaaaaaaaaaaaaa]
	mov rax,[%1 a32 abs 0xaaaaaaaaaaaaaaaa]
	mov rbx,[%1 a32 abs 0xaaaaaaaaaaaaaaaa]
	mov al,[%1 a32 abs 0xbbbbbbbb]
	mov bl,[%1 a32 abs 0xbbbbbbbb]
	mov ax,[%1 a32 abs 0xbbbbbbbb]
	mov bx,[%1 a32 abs 0xbbbbbbbb]
	mov eax,[%1 a32 abs 0xbbbbbbbb]
	mov ebx,[%1 a32 abs 0xbbbbbbbb]
	mov rax,[%1 a32 abs 0xbbbbbbbb]
	mov rbx,[%1 a32 abs 0xbbbbbbbb]
	mov al,[%1 a32 abs 0xffffffffcccccccc]
	mov bl,[%1 a32 abs 0xffffffffcccccccc]
	mov ax,[%1 a32 abs 0xffffffffcccccccc]
	mov bx,[%1 a32 abs 0xffffffffcccccccc]
	mov eax,[%1 a32 abs 0xffffffffcccccccc]
	mov ebx,[%1 a32 abs 0xffffffffcccccccc]
	mov rax,[%1 a32 abs 0xffffffffcccccccc]
	mov rbx,[%1 a32 abs 0xffffffffcccccccc]

	mov al,[%1 a32 abs dword foo]
	mov bl,[%1 a32 abs dword foo]
	mov ax,[%1 a32 abs dword foo]
	mov bx,[%1 a32 abs dword foo]
	mov eax,[%1 a32 abs dword foo]
	mov ebx,[%1 a32 abs dword foo]
	mov rax,[%1 a32 abs dword foo]
	mov rbx,[%1 a32 abs dword foo]
	mov al,[%1 a32 abs dword 0xaaaaaaaaaaaaaaaa]
	mov bl,[%1 a32 abs dword 0xaaaaaaaaaaaaaaaa]
	mov ax,[%1 a32 abs dword 0xaaaaaaaaaaaaaaaa]
	mov bx,[%1 a32 abs dword 0xaaaaaaaaaaaaaaaa]
	mov eax,[%1 a32 abs dword 0xaaaaaaaaaaaaaaaa]
	mov ebx,[%1 a32 abs dword 0xaaaaaaaaaaaaaaaa]
	mov rax,[%1 a32 abs dword 0xaaaaaaaaaaaaaaaa]
	mov rbx,[%1 a32 abs dword 0xaaaaaaaaaaaaaaaa]
	mov al,[%1 a32 abs dword 0xbbbbbbbb]
	mov bl,[%1 a32 abs dword 0xbbbbbbbb]
	mov ax,[%1 a32 abs dword 0xbbbbbbbb]
	mov bx,[%1 a32 abs dword 0xbbbbbbbb]
	mov eax,[%1 a32 abs dword 0xbbbbbbbb]
	mov ebx,[%1 a32 abs dword 0xbbbbbbbb]
	mov rax,[%1 a32 abs dword 0xbbbbbbbb]
	mov rbx,[%1 a32 abs dword 0xbbbbbbbb]
	mov al,[%1 a32 abs dword 0xffffffffcccccccc]
	mov bl,[%1 a32 abs dword 0xffffffffcccccccc]
	mov ax,[%1 a32 abs dword 0xffffffffcccccccc]
	mov bx,[%1 a32 abs dword 0xffffffffcccccccc]
	mov eax,[%1 a32 abs dword 0xffffffffcccccccc]
	mov ebx,[%1 a32 abs dword 0xffffffffcccccccc]
	mov rax,[%1 a32 abs dword 0xffffffffcccccccc]
	mov rbx,[%1 a32 abs dword 0xffffffffcccccccc]

	mov al,[%1 a32 abs qword foo]
	mov bl,[%1 a32 abs qword foo]
	mov ax,[%1 a32 abs qword foo]
	mov bx,[%1 a32 abs qword foo]
	mov eax,[%1 a32 abs qword foo]
	mov ebx,[%1 a32 abs qword foo]
	mov rax,[%1 a32 abs qword foo]
	mov rbx,[%1 a32 abs qword foo]
	mov al,[%1 a32 abs qword 0xaaaaaaaaaaaaaaaa]
	mov bl,[%1 a32 abs qword 0xaaaaaaaaaaaaaaaa]
	mov ax,[%1 a32 abs qword 0xaaaaaaaaaaaaaaaa]
	mov bx,[%1 a32 abs qword 0xaaaaaaaaaaaaaaaa]
	mov eax,[%1 a32 abs qword 0xaaaaaaaaaaaaaaaa]
	mov ebx,[%1 a32 abs qword 0xaaaaaaaaaaaaaaaa]
	mov rax,[%1 a32 abs qword 0xaaaaaaaaaaaaaaaa]
	mov rbx,[%1 a32 abs qword 0xaaaaaaaaaaaaaaaa]
	mov al,[%1 a32 abs qword 0xbbbbbbbb]
	mov bl,[%1 a32 abs qword 0xbbbbbbbb]
	mov ax,[%1 a32 abs qword 0xbbbbbbbb]
	mov bx,[%1 a32 abs qword 0xbbbbbbbb]
	mov eax,[%1 a32 abs qword 0xbbbbbbbb]
	mov ebx,[%1 a32 abs qword 0xbbbbbbbb]
	mov rax,[%1 a32 abs qword 0xbbbbbbbb]
	mov rbx,[%1 a32 abs qword 0xbbbbbbbb]
	mov al,[%1 a32 abs qword 0xffffffffcccccccc]
	mov bl,[%1 a32 abs qword 0xffffffffcccccccc]
	mov ax,[%1 a32 abs qword 0xffffffffcccccccc]
	mov bx,[%1 a32 abs qword 0xffffffffcccccccc]
	mov eax,[%1 a32 abs qword 0xffffffffcccccccc]
	mov ebx,[%1 a32 abs qword 0xffffffffcccccccc]
	mov rax,[%1 a32 abs qword 0xffffffffcccccccc]
	mov rbx,[%1 a32 abs qword 0xffffffffcccccccc]

	mov al,[%1 rel foo]
	mov bl,[%1 rel foo]
	mov ax,[%1 rel foo]
	mov bx,[%1 rel foo]
	mov eax,[%1 rel foo]
	mov ebx,[%1 rel foo]
	mov rax,[%1 rel foo]
	mov rbx,[%1 rel foo]
	mov al,[%1 rel 0xaaaaaaaaaaaaaaaa]
	mov bl,[%1 rel 0xaaaaaaaaaaaaaaaa]
	mov ax,[%1 rel 0xaaaaaaaaaaaaaaaa]
	mov bx,[%1 rel 0xaaaaaaaaaaaaaaaa]
	mov eax,[%1 rel 0xaaaaaaaaaaaaaaaa]
	mov ebx,[%1 rel 0xaaaaaaaaaaaaaaaa]
	mov rax,[%1 rel 0xaaaaaaaaaaaaaaaa]
	mov rbx,[%1 rel 0xaaaaaaaaaaaaaaaa]
	mov al,[%1 rel 0xbbbbbbbb]
	mov bl,[%1 rel 0xbbbbbbbb]
	mov ax,[%1 rel 0xbbbbbbbb]
	mov bx,[%1 rel 0xbbbbbbbb]
	mov eax,[%1 rel 0xbbbbbbbb]
	mov ebx,[%1 rel 0xbbbbbbbb]
	mov rax,[%1 rel 0xbbbbbbbb]
	mov rbx,[%1 rel 0xbbbbbbbb]
	mov al,[%1 rel 0xffffffffcccccccc]
	mov bl,[%1 rel 0xffffffffcccccccc]
	mov ax,[%1 rel 0xffffffffcccccccc]
	mov bx,[%1 rel 0xffffffffcccccccc]
	mov eax,[%1 rel 0xffffffffcccccccc]
	mov ebx,[%1 rel 0xffffffffcccccccc]
	mov rax,[%1 rel 0xffffffffcccccccc]
	mov rbx,[%1 rel 0xffffffffcccccccc]

	mov al,[%1 rel dword foo]
	mov bl,[%1 rel dword foo]
	mov ax,[%1 rel dword foo]
	mov bx,[%1 rel dword foo]
	mov eax,[%1 rel dword foo]
	mov ebx,[%1 rel dword foo]
	mov rax,[%1 rel dword foo]
	mov rbx,[%1 rel dword foo]
	mov al,[%1 rel dword 0xaaaaaaaaaaaaaaaa]
	mov bl,[%1 rel dword 0xaaaaaaaaaaaaaaaa]
	mov ax,[%1 rel dword 0xaaaaaaaaaaaaaaaa]
	mov bx,[%1 rel dword 0xaaaaaaaaaaaaaaaa]
	mov eax,[%1 rel dword 0xaaaaaaaaaaaaaaaa]
	mov ebx,[%1 rel dword 0xaaaaaaaaaaaaaaaa]
	mov rax,[%1 rel dword 0xaaaaaaaaaaaaaaaa]
	mov rbx,[%1 rel dword 0xaaaaaaaaaaaaaaaa]
	mov al,[%1 rel dword 0xbbbbbbbb]
	mov bl,[%1 rel dword 0xbbbbbbbb]
	mov ax,[%1 rel dword 0xbbbbbbbb]
	mov bx,[%1 rel dword 0xbbbbbbbb]
	mov eax,[%1 rel dword 0xbbbbbbbb]
	mov ebx,[%1 rel dword 0xbbbbbbbb]
	mov rax,[%1 rel dword 0xbbbbbbbb]
	mov rbx,[%1 rel dword 0xbbbbbbbb]
	mov al,[%1 rel dword 0xffffffffcccccccc]
	mov bl,[%1 rel dword 0xffffffffcccccccc]
	mov ax,[%1 rel dword 0xffffffffcccccccc]
	mov bx,[%1 rel dword 0xffffffffcccccccc]
	mov eax,[%1 rel dword 0xffffffffcccccccc]
	mov ebx,[%1 rel dword 0xffffffffcccccccc]
	mov rax,[%1 rel dword 0xffffffffcccccccc]
	mov rbx,[%1 rel dword 0xffffffffcccccccc]

	mov al,[%1 rel qword foo]
	mov bl,[%1 rel qword foo]
	mov ax,[%1 rel qword foo]
	mov bx,[%1 rel qword foo]
	mov eax,[%1 rel qword foo]
	mov ebx,[%1 rel qword foo]
	mov rax,[%1 rel qword foo]
	mov rbx,[%1 rel qword foo]
	mov al,[%1 rel qword 0xaaaaaaaaaaaaaaaa]
	mov bl,[%1 rel qword 0xaaaaaaaaaaaaaaaa]
	mov ax,[%1 rel qword 0xaaaaaaaaaaaaaaaa]
	mov bx,[%1 rel qword 0xaaaaaaaaaaaaaaaa]
	mov eax,[%1 rel qword 0xaaaaaaaaaaaaaaaa]
	mov ebx,[%1 rel qword 0xaaaaaaaaaaaaaaaa]
	mov rax,[%1 rel qword 0xaaaaaaaaaaaaaaaa]
	mov rbx,[%1 rel qword 0xaaaaaaaaaaaaaaaa]
	mov al,[%1 rel qword 0xbbbbbbbb]
	mov bl,[%1 rel qword 0xbbbbbbbb]
	mov ax,[%1 rel qword 0xbbbbbbbb]
	mov bx,[%1 rel qword 0xbbbbbbbb]
	mov eax,[%1 rel qword 0xbbbbbbbb]
	mov ebx,[%1 rel qword 0xbbbbbbbb]
	mov rax,[%1 rel qword 0xbbbbbbbb]
	mov rbx,[%1 rel qword 0xbbbbbbbb]
	mov al,[%1 rel qword 0xffffffffcccccccc]
	mov bl,[%1 rel qword 0xffffffffcccccccc]
	mov ax,[%1 rel qword 0xffffffffcccccccc]
	mov bx,[%1 rel qword 0xffffffffcccccccc]
	mov eax,[%1 rel qword 0xffffffffcccccccc]
	mov ebx,[%1 rel qword 0xffffffffcccccccc]
	mov rax,[%1 rel qword 0xffffffffcccccccc]
	mov rbx,[%1 rel qword 0xffffffffcccccccc]

	mov al,[%1 a64 rel foo]
	mov bl,[%1 a64 rel foo]
	mov ax,[%1 a64 rel foo]
	mov bx,[%1 a64 rel foo]
	mov eax,[%1 a64 rel foo]
	mov ebx,[%1 a64 rel foo]
	mov rax,[%1 a64 rel foo]
	mov rbx,[%1 a64 rel foo]
	mov al,[%1 a64 rel 0xaaaaaaaaaaaaaaaa]
	mov bl,[%1 a64 rel 0xaaaaaaaaaaaaaaaa]
	mov ax,[%1 a64 rel 0xaaaaaaaaaaaaaaaa]
	mov bx,[%1 a64 rel 0xaaaaaaaaaaaaaaaa]
	mov eax,[%1 a64 rel 0xaaaaaaaaaaaaaaaa]
	mov ebx,[%1 a64 rel 0xaaaaaaaaaaaaaaaa]
	mov rax,[%1 a64 rel 0xaaaaaaaaaaaaaaaa]
	mov rbx,[%1 a64 rel 0xaaaaaaaaaaaaaaaa]
	mov al,[%1 a64 rel 0xbbbbbbbb]
	mov bl,[%1 a64 rel 0xbbbbbbbb]
	mov ax,[%1 a64 rel 0xbbbbbbbb]
	mov bx,[%1 a64 rel 0xbbbbbbbb]
	mov eax,[%1 a64 rel 0xbbbbbbbb]
	mov ebx,[%1 a64 rel 0xbbbbbbbb]
	mov rax,[%1 a64 rel 0xbbbbbbbb]
	mov rbx,[%1 a64 rel 0xbbbbbbbb]
	mov al,[%1 a64 rel 0xffffffffcccccccc]
	mov bl,[%1 a64 rel 0xffffffffcccccccc]
	mov ax,[%1 a64 rel 0xffffffffcccccccc]
	mov bx,[%1 a64 rel 0xffffffffcccccccc]
	mov eax,[%1 a64 rel 0xffffffffcccccccc]
	mov ebx,[%1 a64 rel 0xffffffffcccccccc]
	mov rax,[%1 a64 rel 0xffffffffcccccccc]
	mov rbx,[%1 a64 rel 0xffffffffcccccccc]

	mov al,[%1 a64 rel dword foo]
	mov bl,[%1 a64 rel dword foo]
	mov ax,[%1 a64 rel dword foo]
	mov bx,[%1 a64 rel dword foo]
	mov eax,[%1 a64 rel dword foo]
	mov ebx,[%1 a64 rel dword foo]
	mov rax,[%1 a64 rel dword foo]
	mov rbx,[%1 a64 rel dword foo]
	mov al,[%1 a64 rel dword 0xaaaaaaaaaaaaaaaa]
	mov bl,[%1 a64 rel dword 0xaaaaaaaaaaaaaaaa]
	mov ax,[%1 a64 rel dword 0xaaaaaaaaaaaaaaaa]
	mov bx,[%1 a64 rel dword 0xaaaaaaaaaaaaaaaa]
	mov eax,[%1 a64 rel dword 0xaaaaaaaaaaaaaaaa]
	mov ebx,[%1 a64 rel dword 0xaaaaaaaaaaaaaaaa]
	mov rax,[%1 a64 rel dword 0xaaaaaaaaaaaaaaaa]
	mov rbx,[%1 a64 rel dword 0xaaaaaaaaaaaaaaaa]
	mov al,[%1 a64 rel dword 0xbbbbbbbb]
	mov bl,[%1 a64 rel dword 0xbbbbbbbb]
	mov ax,[%1 a64 rel dword 0xbbbbbbbb]
	mov bx,[%1 a64 rel dword 0xbbbbbbbb]
	mov eax,[%1 a64 rel dword 0xbbbbbbbb]
	mov ebx,[%1 a64 rel dword 0xbbbbbbbb]
	mov rax,[%1 a64 rel dword 0xbbbbbbbb]
	mov rbx,[%1 a64 rel dword 0xbbbbbbbb]
	mov al,[%1 a64 rel dword 0xffffffffcccccccc]
	mov bl,[%1 a64 rel dword 0xffffffffcccccccc]
	mov ax,[%1 a64 rel dword 0xffffffffcccccccc]
	mov bx,[%1 a64 rel dword 0xffffffffcccccccc]
	mov eax,[%1 a64 rel dword 0xffffffffcccccccc]
	mov ebx,[%1 a64 rel dword 0xffffffffcccccccc]
	mov rax,[%1 a64 rel dword 0xffffffffcccccccc]
	mov rbx,[%1 a64 rel dword 0xffffffffcccccccc]

	mov al,[%1 a64 rel qword foo]
	mov bl,[%1 a64 rel qword foo]
	mov ax,[%1 a64 rel qword foo]
	mov bx,[%1 a64 rel qword foo]
	mov eax,[%1 a64 rel qword foo]
	mov ebx,[%1 a64 rel qword foo]
	mov rax,[%1 a64 rel qword foo]
	mov rbx,[%1 a64 rel qword foo]
	mov al,[%1 a64 rel qword 0xaaaaaaaaaaaaaaaa]
	mov bl,[%1 a64 rel qword 0xaaaaaaaaaaaaaaaa]
	mov ax,[%1 a64 rel qword 0xaaaaaaaaaaaaaaaa]
	mov bx,[%1 a64 rel qword 0xaaaaaaaaaaaaaaaa]
	mov eax,[%1 a64 rel qword 0xaaaaaaaaaaaaaaaa]
	mov ebx,[%1 a64 rel qword 0xaaaaaaaaaaaaaaaa]
	mov rax,[%1 a64 rel qword 0xaaaaaaaaaaaaaaaa]
	mov rbx,[%1 a64 rel qword 0xaaaaaaaaaaaaaaaa]
	mov al,[%1 a64 rel qword 0xbbbbbbbb]
	mov bl,[%1 a64 rel qword 0xbbbbbbbb]
	mov ax,[%1 a64 rel qword 0xbbbbbbbb]
	mov bx,[%1 a64 rel qword 0xbbbbbbbb]
	mov eax,[%1 a64 rel qword 0xbbbbbbbb]
	mov ebx,[%1 a64 rel qword 0xbbbbbbbb]
	mov rax,[%1 a64 rel qword 0xbbbbbbbb]
	mov rbx,[%1 a64 rel qword 0xbbbbbbbb]
	mov al,[%1 a64 rel qword 0xffffffffcccccccc]
	mov bl,[%1 a64 rel qword 0xffffffffcccccccc]
	mov ax,[%1 a64 rel qword 0xffffffffcccccccc]
	mov bx,[%1 a64 rel qword 0xffffffffcccccccc]
	mov eax,[%1 a64 rel qword 0xffffffffcccccccc]
	mov ebx,[%1 a64 rel qword 0xffffffffcccccccc]
	mov rax,[%1 a64 rel qword 0xffffffffcccccccc]
	mov rbx,[%1 a64 rel qword 0xffffffffcccccccc]

	mov al,[%1 a32 rel foo]
	mov bl,[%1 a32 rel foo]
	mov ax,[%1 a32 rel foo]
	mov bx,[%1 a32 rel foo]
	mov eax,[%1 a32 rel foo]
	mov ebx,[%1 a32 rel foo]
	mov rax,[%1 a32 rel foo]
	mov rbx,[%1 a32 rel foo]
	mov al,[%1 a32 rel 0xaaaaaaaaaaaaaaaa]
	mov bl,[%1 a32 rel 0xaaaaaaaaaaaaaaaa]
	mov ax,[%1 a32 rel 0xaaaaaaaaaaaaaaaa]
	mov bx,[%1 a32 rel 0xaaaaaaaaaaaaaaaa]
	mov eax,[%1 a32 rel 0xaaaaaaaaaaaaaaaa]
	mov ebx,[%1 a32 rel 0xaaaaaaaaaaaaaaaa]
	mov rax,[%1 a32 rel 0xaaaaaaaaaaaaaaaa]
	mov rbx,[%1 a32 rel 0xaaaaaaaaaaaaaaaa]
	mov al,[%1 a32 rel 0xbbbbbbbb]
	mov bl,[%1 a32 rel 0xbbbbbbbb]
	mov ax,[%1 a32 rel 0xbbbbbbbb]
	mov bx,[%1 a32 rel 0xbbbbbbbb]
	mov eax,[%1 a32 rel 0xbbbbbbbb]
	mov ebx,[%1 a32 rel 0xbbbbbbbb]
	mov rax,[%1 a32 rel 0xbbbbbbbb]
	mov rbx,[%1 a32 rel 0xbbbbbbbb]
	mov al,[%1 a32 rel 0xffffffffcccccccc]
	mov bl,[%1 a32 rel 0xffffffffcccccccc]
	mov ax,[%1 a32 rel 0xffffffffcccccccc]
	mov bx,[%1 a32 rel 0xffffffffcccccccc]
	mov eax,[%1 a32 rel 0xffffffffcccccccc]
	mov ebx,[%1 a32 rel 0xffffffffcccccccc]
	mov rax,[%1 a32 rel 0xffffffffcccccccc]
	mov rbx,[%1 a32 rel 0xffffffffcccccccc]

	mov al,[%1 a32 rel dword foo]
	mov bl,[%1 a32 rel dword foo]
	mov ax,[%1 a32 rel dword foo]
	mov bx,[%1 a32 rel dword foo]
	mov eax,[%1 a32 rel dword foo]
	mov ebx,[%1 a32 rel dword foo]
	mov rax,[%1 a32 rel dword foo]
	mov rbx,[%1 a32 rel dword foo]
	mov al,[%1 a32 rel dword 0xaaaaaaaaaaaaaaaa]
	mov bl,[%1 a32 rel dword 0xaaaaaaaaaaaaaaaa]
	mov ax,[%1 a32 rel dword 0xaaaaaaaaaaaaaaaa]
	mov bx,[%1 a32 rel dword 0xaaaaaaaaaaaaaaaa]
	mov eax,[%1 a32 rel dword 0xaaaaaaaaaaaaaaaa]
	mov ebx,[%1 a32 rel dword 0xaaaaaaaaaaaaaaaa]
	mov rax,[%1 a32 rel dword 0xaaaaaaaaaaaaaaaa]
	mov rbx,[%1 a32 rel dword 0xaaaaaaaaaaaaaaaa]
	mov al,[%1 a32 rel dword 0xbbbbbbbb]
	mov bl,[%1 a32 rel dword 0xbbbbbbbb]
	mov ax,[%1 a32 rel dword 0xbbbbbbbb]
	mov bx,[%1 a32 rel dword 0xbbbbbbbb]
	mov eax,[%1 a32 rel dword 0xbbbbbbbb]
	mov ebx,[%1 a32 rel dword 0xbbbbbbbb]
	mov rax,[%1 a32 rel dword 0xbbbbbbbb]
	mov rbx,[%1 a32 rel dword 0xbbbbbbbb]
	mov al,[%1 a32 rel dword 0xffffffffcccccccc]
	mov bl,[%1 a32 rel dword 0xffffffffcccccccc]
	mov ax,[%1 a32 rel dword 0xffffffffcccccccc]
	mov bx,[%1 a32 rel dword 0xffffffffcccccccc]
	mov eax,[%1 a32 rel dword 0xffffffffcccccccc]
	mov ebx,[%1 a32 rel dword 0xffffffffcccccccc]
	mov rax,[%1 a32 rel dword 0xffffffffcccccccc]
	mov rbx,[%1 a32 rel dword 0xffffffffcccccccc]

	mov al,[%1 a32 rel qword foo]
	mov bl,[%1 a32 rel qword foo]
	mov ax,[%1 a32 rel qword foo]
	mov bx,[%1 a32 rel qword foo]
	mov eax,[%1 a32 rel qword foo]
	mov ebx,[%1 a32 rel qword foo]
	mov rax,[%1 a32 rel qword foo]
	mov rbx,[%1 a32 rel qword foo]
	mov al,[%1 a32 rel qword 0xaaaaaaaaaaaaaaaa]
	mov bl,[%1 a32 rel qword 0xaaaaaaaaaaaaaaaa]
	mov ax,[%1 a32 rel qword 0xaaaaaaaaaaaaaaaa]
	mov bx,[%1 a32 rel qword 0xaaaaaaaaaaaaaaaa]
	mov eax,[%1 a32 rel qword 0xaaaaaaaaaaaaaaaa]
	mov ebx,[%1 a32 rel qword 0xaaaaaaaaaaaaaaaa]
	mov rax,[%1 a32 rel qword 0xaaaaaaaaaaaaaaaa]
	mov rbx,[%1 a32 rel qword 0xaaaaaaaaaaaaaaaa]
	mov al,[%1 a32 rel qword 0xbbbbbbbb]
	mov bl,[%1 a32 rel qword 0xbbbbbbbb]
	mov ax,[%1 a32 rel qword 0xbbbbbbbb]
	mov bx,[%1 a32 rel qword 0xbbbbbbbb]
	mov eax,[%1 a32 rel qword 0xbbbbbbbb]
	mov ebx,[%1 a32 rel qword 0xbbbbbbbb]
	mov rax,[%1 a32 rel qword 0xbbbbbbbb]
	mov rbx,[%1 a32 rel qword 0xbbbbbbbb]
	mov al,[%1 a32 rel qword 0xffffffffcccccccc]
	mov bl,[%1 a32 rel qword 0xffffffffcccccccc]
	mov ax,[%1 a32 rel qword 0xffffffffcccccccc]
	mov bx,[%1 a32 rel qword 0xffffffffcccccccc]
	mov eax,[%1 a32 rel qword 0xffffffffcccccccc]
	mov ebx,[%1 a32 rel qword 0xffffffffcccccccc]
	mov rax,[%1 a32 rel qword 0xffffffffcccccccc]
	mov rbx,[%1 a32 rel qword 0xffffffffcccccccc]
%endmacro

	default abs
	dotest
	dotest es:
	dotest fs:
	dotest gs:

	default rel
	dotest
	dotest es:
	dotest fs:
	dotest gs:

	default rel fs:abs gs:rel
	dotest
	dotest es:
	dotest fs:
	dotest gs:

foo:
