N "": ntoken empty
C "": ntoken empty
N "ZMACRO": ntoken empty
C "ZMACRO": ntoken empty
N "NMACRO": token nempty
C "NMACRO": token nempty
N "TMACRO": ntoken nempty
C "TMACRO": ntoken nempty
N "1": token nempty
C "1": token nempty
N "+1": ntoken nempty
C "+1": ntoken nempty
N "1 2": ntoken nempty
C "1 2": ntoken nempty
N "1,2": ntoken nempty
C "1,2": ntoken nempty
N "foo": token nempty
C "foo": token nempty
N "foo bar": ntoken nempty
C "foo bar": ntoken nempty
N "%": token nempty
C "%": token nempty
N "+foo": ntoken nempty
C "+foo": ntoken nempty
N "<<": token nempty
C "<<": token nempty
