/* This file auto-generated from insns.dat by insns.pl - don't edit it */

#include "nasm.h"
#include "insns.h"

static const struct itemplate instrux[] = {
    /*   0 : NOP void [          osz norexb nof3 90                ] BESTDIS,8086 */
        /* 330 314 326 001(90) : BESTDIS,8086 */
        {I_NOP, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66224, 2},
    /*   1 : NOP rm16 [m:        o16 0f 1f /0                      ] BESTDIS,P6 */
        /* 320 355 001(1f) 200 : BESTDIS,P6 */
        {I_NOP, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66236, 4},
    /*   2 : NOP rm32 [m:        o32 0f 1f /0                      ] BESTDIS,P6 */
        /* 321 355 001(1f) 200 : BESTDIS,P6 */
        {I_NOP, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66242, 4},
    /*   3 : NOP rm64 [m:        o64 0f 1f /0                      ] LONG,BESTDIS,PROT,X86_64 */
        /* 324 355 001(1f) 200 : LONG,BESTDIS,PROT,X86_64 */
        {I_NOP, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66248, 5},
    /*   4 : MOV reg_al,mem_offs [-i:       o8 a0 iwdq                        ] SM0-1,NOAPX,8086 */
        /* 001(a0) 045 : SM0,SM1,NOAPX,8086 */
        {I_MOV, 2, {REG_AL|RN_L16,MEM_OFFS,0,0,0}, NO_DECORATOR, nasm_bytecodes+72433, 6},
    /*   5 : MOV reg_ax,mem_offs [-i:       o16 a1 iwdq                       ] SM0-1,NOAPX,8086 */
        /* 320 001(a1) 045 : SM0,SM1,NOAPX,8086 */
        {I_MOV, 2, {REG_AX|RN_L16,MEM_OFFS,0,0,0}, NO_DECORATOR, nasm_bytecodes+71168, 6},
    /*   6 : MOV reg_eax,mem_offs [-i:       o32 a1 iwdq                       ] SM0-1,NOAPX,ZU,386 */
        /* 321 001(a1) 045 : SM0,SM1,NOAPX,ZU,386 */
        {I_MOV, 2, {REG_EAX|RN_L16,MEM_OFFS,0,0,0}, NO_DECORATOR, nasm_bytecodes+71173, 7},
    /*   7 : MOV reg_rax,mem_offs [-i:       o64 a1 iwdq                       ] SM0-1,NOAPX,ZU,LONG,PROT,X86_64 */
        /* 324 001(a1) 045 : SM0,SM1,NOAPX,ZU,LONG,PROT,X86_64 */
        {I_MOV, 2, {REG_RAX|RN_L16,MEM_OFFS,0,0,0}, NO_DECORATOR, nasm_bytecodes+71178, 8},
    /*   8 : MOV mem_offs,reg_al [i-:       o8 a2 iwdq                        ] SM0-1,NOAPX,NOHLE,8086 */
        /* 001(a2) 044 : SM0,SM1,NOAPX,NOHLE,8086 */
        {I_MOV, 2, {MEM_OFFS,REG_AL|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72437, 9},
    /*   9 : MOV mem_offs,reg_ax [i-:       o16 a3 iwdq                       ] SM0-1,NOAPX,NOHLE,8086 */
        /* 320 001(a3) 044 : SM0,SM1,NOAPX,NOHLE,8086 */
        {I_MOV, 2, {MEM_OFFS,REG_AX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71183, 9},
    /*  10 : MOV mem_offs,reg_eax [i-:       o32 a3 iwdq                       ] SM0-1,NOAPX,NOHLE,386 */
        /* 321 001(a3) 044 : SM0,SM1,NOAPX,NOHLE,386 */
        {I_MOV, 2, {MEM_OFFS,REG_EAX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71188, 10},
    /*  11 : MOV mem_offs,reg_rax [i-:       o64 a3 iwdq                       ] SM0-1,NOAPX,LONG,NOHLE,PROT,X86_64 */
        /* 324 001(a3) 044 : SM0,SM1,NOAPX,LONG,NOHLE,PROT,X86_64 */
        {I_MOV, 2, {MEM_OFFS,REG_RAX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71193, 11},
    /*  12 : MOV rm8,reg8 [mr:       hlexr o8 88 /r                    ] SM0-1,8086 */
        /* 271 001(88) 101 : SM0,SM1,8086 */
        {I_MOV, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+71198, 12},
    /*  13 : MOV rm16,reg16 [mr:       hlexr o16 89 /r                   ] SM0-1,8086 */
        /* 271 320 001(89) 101 : SM0,SM1,8086 */
        {I_MOV, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66254, 12},
    /*  14 : MOV rm32,reg32 [mr:       hlexr o32 89 /r                   ] SM0-1,386 */
        /* 271 321 001(89) 101 : SM0,SM1,386 */
        {I_MOV, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66260, 13},
    /*  15 : MOV rm64,reg64 [mr:       hlexr o64 89 /r                   ] SM0-1,LONG,PROT,X86_64 */
        /* 271 324 001(89) 101 : SM0,SM1,LONG,PROT,X86_64 */
        {I_MOV, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66266, 14},
    /*  16 : MOV reg8,rm8 [rm:       o8 8a /r                          ] SM0-1,8086 */
        /* 001(8a) 110 : SM0,SM1,8086 */
        {I_MOV, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66280, 12},
    /*  17 : MOV reg16,rm16 [rm:       o16 8b /r                         ] SM0-1,8086 */
        /* 320 001(8b) 110 : SM0,SM1,8086 */
        {I_MOV, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66825, 12},
    /*  18 : MOV reg32,rm32 [rm:       o32 8b /r                         ] SM0-1,386 */
        /* 321 001(8b) 110 : SM0,SM1,386 */
        {I_MOV, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66831, 13},
    /*  19 : MOV reg64,rm64 [rm:       o64 8b /r                         ] SM0-1,LONG,PROT,X86_64 */
        /* 324 001(8b) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_MOV, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66837, 14},
    /*  20 : MOV reg8,imm8 [ri:       o8 b0+r ib                        ] SM0-1,8086 */
        /* 010 260(021  : SM0,SM1,8086 */
        {I_MOV, 2, {REG_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+72441, 12},
    /*  21 : MOV reg16,imm16 [ri:       o16 b8+r iw                       ] SM0-1,8086 */
        /* 320 010 270(031  : SM0,SM1,8086 */
        {I_MOV, 2, {REG_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71208, 12},
    /*  22 : MOV reg32,imm32 [ri:       o32 b8+r id                       ] SM0-1,386 */
        /* 321 010 270(041  : SM0,SM1,386 */
        {I_MOV, 2, {REG_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71213, 13},
    /*  23 : MOV reg64,imm64 [ri:       o64 b8+r iq                       ] SM0-1,LONG,PROT,X86_64 */
        /* 324 010 270(055  : SM0,SM1,LONG,PROT,X86_64 */
        {I_MOV, 2, {REG_GPR|BITS64,IMM_NORMAL|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+71218, 14},
    /*  24 : MOV rm8,imm8 [mi:       hlexr o8 c6 /0 ib                 ] SM0-1,8086 */
        /* 271 001(c6) 200 021 : SM0,SM1,8086 */
        {I_MOV, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66272, 12},
    /*  25 : MOV rm16,imm16 [mi:       hlexr o16 c7 /0 iw                ] SM0-1,8086 */
        /* 271 320 001(c7) 200 031 : SM0,SM1,8086 */
        {I_MOV, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+54709, 12},
    /*  26 : MOV rm32,imm32 [mi:       hlexr o32 c7 /0 id                ] SM0-1,386 */
        /* 271 321 001(c7) 200 041 : SM0,SM1,386 */
        {I_MOV, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+54716, 13},
    /*  27 : MOV rm64,sdword64 [mi:       hlexr o64 c7 /0 id,s              ] SM0-1,LONG,PROT,X86_64 */
        /* 271 324 001(c7) 200 255 : SM0,SM1,LONG,PROT,X86_64 */
        {I_MOV, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54723, 14},
    /*  28 : MOVRS reg8,mem8 [rm:       o8 norep 0f38 8a /r               ] SM0-1,FUTURE */
        /* 331 356 001(8a) 110 : SM0,SM1,NOAPX,FUTURE */
        {I_MOVRS, 2, {REG_GPR|BITS8|RN_L16,MEMORY|BITS8|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66278, 17},
    /*  29 : MOVRS reg16,mem16 [rm:       o16 norep 0f38 8b /r              ] SM0-1,FUTURE */
        /* 320 331 356 001(8b) 110 : SM0,SM1,NOAPX,FUTURE */
        {I_MOVRS, 2, {REG_GPR|BITS16|RN_L16,MEMORY|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+54730, 17},
    /*  30 : MOVRS reg32,mem32 [rm:       o32 norep 0f38 8b /r              ] SM0-1,FUTURE */
        /* 321 331 356 001(8b) 110 : SM0,SM1,NOAPX,FUTURE */
        {I_MOVRS, 2, {REG_GPR|BITS32|RN_L16,MEMORY|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+54737, 17},
    /*  31 : MOVRS reg64,mem64 [rm:       o64 norep 0f38 8b /r              ] SM0-1,LONG,PROT,X86_64,FUTURE */
        /* 324 331 356 001(8b) 110 : SM0,SM1,NOAPX,LONG,PROT,FUTURE */
        {I_MOVRS, 2, {REG_GPR|BITS64|RN_L16,MEMORY|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+54744, 18},
    /*  32 : MOVRS reg8,mem8 [rm:       evex.nf0.nd0.l0.m4.o8 8a /r       ] SM0-1,FUTURE */
        /* 250(f4 7c 08) 300 001(8a) 110 : SM0,SM1,WIG,EVEX,PROT,FUTURE */
        {I_MOVRS, 2, {REG_GPR|BITS8,MEMORY|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23373, 19},
    /*  33 : MOVRS reg16,mem16 [rm:       evex.nf0.nd0.l0.m4.o16 8b /r      ] SM0-1,FUTURE */
        /* 250(f4 7d 08) 300 320 001(8b) 110 : SM0,SM1,EVEX,PROT,FUTURE */
        {I_MOVRS, 2, {REG_GPR|BITS16,MEMORY|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+9143, 20},
    /*  34 : MOVRS reg32,mem32 [rm:       evex.nf0.nd0.l0.m4.o32 8b /r      ] SM0-1,FUTURE */
        /* 250(f4 7c 08) 300 321 001(8b) 110 : SM0,SM1,WW,EVEX,PROT,FUTURE */
        {I_MOVRS, 2, {REG_GPR|BITS32,MEMORY|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+9153, 21},
    /*  35 : MOVRS reg64,mem64 [rm:       evex.nf0.nd0.l0.m4.o64 8b /r      ] SM0-1,LONG,PROT,X86_64,FUTURE */
        /* 250(f4 fc 08) 300 324 001(8b) 110 : SM0,SM1,WW,LONG,EVEX,PROT,FUTURE */
        {I_MOVRS, 2, {REG_GPR|BITS64,MEMORY|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+9163, 22},
    /*  36 : LEA reg16,mem [rm:       o16 8d /r                         ] 8086 */
        /* 320 001(8d) 110 : 8086 */
        {I_LEA, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+71223, 23},
    /*  37 : LEA reg32,mem [rm:       o32 8d /r                         ] 386 */
        /* 321 001(8d) 110 : 386 */
        {I_LEA, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+71228, 3},
    /*  38 : LEA reg64,mem [rm:       o64 8d /r                         ] LONG,PROT,X86_64 */
        /* 324 001(8d) 110 : LONG,PROT,X86_64 */
        {I_LEA, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+71233, 24},
    /*  39 : ADD rm8,reg8 [mr:       hle o8 00 /r                      ] SM0-1,LOCK,FL,8086 */
        /* 273 001(00) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_ADD, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+71238, 25},
    /*  40 : ADD rm16,reg16 [mr:       hle o16 01 /r                     ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(01) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_ADD, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66284, 25},
    /*  41 : ADD rm32,reg32 [mr:       hle o32 01 /r                     ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(01) 101 : SM0,SM1,LOCK,FL,386 */
        {I_ADD, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66290, 26},
    /*  42 : ADD rm64,reg64 [mr:       hle o64 01 /r                     ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(01) 101 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66296, 27},
    /*  43 : ADD reg8,rm8 [rm:       o8 02 /r                          ] SM0-1,FL,8086 */
        /* 001(02) 110 : SM0,SM1,FL,8086 */
        {I_ADD, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+69388, 28},
    /*  44 : ADD reg16,rm16 [rm:       o16 03 /r                         ] SM0-1,FL,8086 */
        /* 320 001(03) 110 : SM0,SM1,FL,8086 */
        {I_ADD, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+9198, 28},
    /*  45 : ADD reg32,rm32 [rm:       o32 03 /r                         ] SM0-1,FL,386 */
        /* 321 001(03) 110 : SM0,SM1,FL,386 */
        {I_ADD, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+9228, 29},
    /*  46 : ADD reg64,rm64 [rm:       o64 03 /r                         ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 001(03) 110 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_ADD, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+9258, 30},
    /*  47 : ADD rm16,sbyteword16 [mi:       hle o16 83 /0 ib,s                ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(83) 200 275 : SM0,SM1,LOCK,FL,8086 */
        {I_ADD, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54751, 25},
    /*  48 : ADD rm32,sbytedword32 [mi:       hle o32 83 /0 ib,s                ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(83) 200 275 : SM0,SM1,LOCK,FL,386 */
        {I_ADD, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54758, 26},
    /*  49 : ADD rm64,sbytedword64 [mi:       hle o64 83 /0 ib,s                ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(83) 200 275 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54765, 27},
    /*  50 : ADD reg_al,imm8 [-i:       o8 04 ib                          ] SM0-1,FL,8086 */
        /* 001(04) 021 : SM0,SM1,FL,8086 */
        {I_ADD, 2, {REG_AL,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+72445, 28},
    /*  51 : ADD reg_ax,imm16 [-i:       o16 05 iw                         ] SM0-1,FL,8086 */
        /* 320 001(05) 031 : SM0,SM1,FL,8086 */
        {I_ADD, 2, {REG_AX,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71243, 28},
    /*  52 : ADD reg_eax,imm32 [-i:       o32 05 id                         ] SM0-1,ZU,FL,386 */
        /* 321 001(05) 041 : SM0,SM1,ZU,FL,386 */
        {I_ADD, 2, {REG_EAX,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71248, 31},
    /*  53 : ADD reg_rax,sdword64 [-i:       o64 05 id,s                       ] SM0-1,ZU,LONG,FL,PROT,X86_64 */
        /* 324 001(05) 255 : SM0,SM1,ZU,LONG,FL,PROT,X86_64 */
        {I_ADD, 2, {REG_RAX,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+71253, 32},
    /*  54 : ADD rm8,imm8 [mi:       hle o8 80 /0 ib                   ] SM0-1,LOCK,FL,8086 */
        /* 273 001(80) 200 021 : SM0,SM1,LOCK,FL,8086 */
        {I_ADD, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66302, 25},
    /*  55 : ADD rm16,imm16 [mi:       hle o16 81 /0 iw                  ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(81) 200 031 : SM0,SM1,LOCK,FL,8086 */
        {I_ADD, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+54772, 25},
    /*  56 : ADD rm32,imm32 [mi:       hle o32 81 /0 id                  ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(81) 200 041 : SM0,SM1,LOCK,FL,386 */
        {I_ADD, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+54779, 26},
    /*  57 : ADD rm64,sdword64 [mi:       hle o64 81 /0 id,s                ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(81) 200 255 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54786, 27},
    /*  58 : ADD reg8?,reg8,rm8 [vrm:      evex.nd1.nf.l0.m4.o8 02 /r        ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(02) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS8,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23382, 33},
    /*  59 : ADD reg8,rm8 [rm:      evex.nd0.nf.l0.m4.o8 02 /r        ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(02) 110 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23391, 34},
    /*  60 : ADD reg16?,reg16,rm16 [vrm:      evex.nd1.nf.l0.m4.o16 03 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(03) 121 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+9173, 36},
    /*  61 : ADD reg16,rm16 [rm:      evex.nd0.nf.l0.m4.o16 03 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(03) 110 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+9183, 37},
    /*  62 : ADD reg32?,reg32,rm32 [vrm:      evex.nd1.nf.l0.m4.o32 03 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(03) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+9203, 39},
    /*  63 : ADD reg32,rm32 [rm:      evex.nd0.nf.l0.m4.o32 03 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(03) 110 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+9213, 40},
    /*  64 : ADD reg64?,reg64,rm64 [vrm:      evex.nd1.nf.l0.m4.o64 03 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(03) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+9233, 39},
    /*  65 : ADD reg64,rm64 [rm:      evex.nd0.nf.l0.m4.o64 03 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(03) 110 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+9243, 40},
    /*  66 : ADD reg8?,rm8,reg8 [vmr:      evex.nd1.nf.l0.m4.o8 00 /r        ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(00) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23409, 33},
    /*  67 : ADD rm8,reg8 [mr:      evex.nd0.nf.l0.m4.o8 00 /r        ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(00) 101 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23418, 34},
    /*  68 : ADD reg16?,rm16,reg16 [vmr:      evex.nd1.nf.l0.m4.o16 01 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(01) 112 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+9263, 36},
    /*  69 : ADD rm16,reg16 [mr:      evex.nd0.nf.l0.m4.o16 01 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(01) 101 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+9273, 37},
    /*  70 : ADD reg32?,rm32,reg32 [vmr:      evex.nd1.nf.l0.m4.o32 01 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(01) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+9293, 39},
    /*  71 : ADD rm32,reg32 [mr:      evex.nd0.nf.l0.m4.o32 01 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(01) 101 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+9303, 40},
    /*  72 : ADD reg64?,rm64,reg64 [vmr:      evex.nd1.nf.l0.m4.o64 01 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(01) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+9323, 39},
    /*  73 : ADD rm64,reg64 [mr:      evex.nd0.nf.l0.m4.o64 01 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(01) 101 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+9333, 40},
    /*  74 : ADD reg16?,rm16,sbyteword16 [vmi:      evex.nd1.nf.l0.m4.o16 83 /0 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(83) 210 276 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS16,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+24, 36},
    /*  75 : ADD rm16,sbyteword16 [mi:      evex.nd0.nf.l0.m4.o16 83 /0 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(83) 200 275 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+35, 37},
    /*  76 : ADD reg32?,rm32,sbytedword32 [vmi:      evex.nd1.nf.l0.m4.o32 83 /0 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(83) 210 276 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS32,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+57, 39},
    /*  77 : ADD rm32,sbytedword32 [mi:      evex.nd0.nf.l0.m4.o32 83 /0 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(83) 200 275 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+68, 40},
    /*  78 : ADD reg64?,rm64,sbytedword64 [vmi:      evex.nd1.nf.l0.m4.o64 83 /0 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(83) 210 276 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+90, 39},
    /*  79 : ADD rm64,sbytedword64 [mi:      evex.nd0.nf.l0.m4.o64 83 /0 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(83) 200 275 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+101, 40},
    /*  80 : ADD reg8?,rm8,imm8 [vmi:      evex.nd1.nf.l0.m4.o8 80 /0 ib     ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(80) 210 022 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+9353, 33},
    /*  81 : ADD rm8,imm8 [mi:      evex.nd0.nf.l0.m4.o8 80 /0 ib     ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(80) 200 021 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+9363, 34},
    /*  82 : ADD reg16?,rm16,imm16 [vmi:      evex.nd1.nf.l0.m4.o16 81 /0 iw    ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(81) 210 032 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+123, 36},
    /*  83 : ADD rm16,imm16 [mi:      evex.nd0.nf.l0.m4.o16 81 /0 iw    ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(81) 200 031 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+134, 37},
    /*  84 : ADD reg32?,rm32,imm32 [vmi:      evex.nd1.nf.l0.m4.o32 81 /0 id    ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(81) 210 042 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+156, 39},
    /*  85 : ADD rm32,imm32 [mi:      evex.nd0.nf.l0.m4.o32 81 /0 id    ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(81) 200 041 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+167, 40},
    /*  86 : ADD reg64?,rm64,sdword64 [vmi:      evex.nd1.nf.l0.m4.o64 81 /0 id,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(81) 210 256 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+189, 39},
    /*  87 : ADD rm64,sdword64 [mi:      evex.nd0.nf.l0.m4.o64 81 /0 id,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(81) 200 255 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+200, 40},
    /*  88 : OR rm8,reg8 [mr:       hle o8 08 /r                      ] SM0-1,LOCK,FL,8086 */
        /* 273 001(08) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_OR, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+71258, 25},
    /*  89 : OR rm16,reg16 [mr:       hle o16 09 /r                     ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(09) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_OR, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66308, 25},
    /*  90 : OR rm32,reg32 [mr:       hle o32 09 /r                     ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(09) 101 : SM0,SM1,LOCK,FL,386 */
        {I_OR, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66314, 26},
    /*  91 : OR rm64,reg64 [mr:       hle o64 09 /r                     ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(09) 101 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_OR, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66320, 27},
    /*  92 : OR reg8,rm8 [rm:       o8 0a /r                          ] SM0-1,FL,8086 */
        /* 001(0a) 110 : SM0,SM1,FL,8086 */
        {I_OR, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+69508, 28},
    /*  93 : OR reg16,rm16 [rm:       o16 0b /r                         ] SM0-1,FL,8086 */
        /* 320 001(0b) 110 : SM0,SM1,FL,8086 */
        {I_OR, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+9408, 28},
    /*  94 : OR reg32,rm32 [rm:       o32 0b /r                         ] SM0-1,FL,386 */
        /* 321 001(0b) 110 : SM0,SM1,FL,386 */
        {I_OR, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+9438, 29},
    /*  95 : OR reg64,rm64 [rm:       o64 0b /r                         ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 001(0b) 110 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_OR, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+9468, 30},
    /*  96 : OR rm16,sbyteword16 [mi:       hle o16 83 /1 ib,s                ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(83) 201 275 : SM0,SM1,LOCK,FL,8086 */
        {I_OR, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54793, 25},
    /*  97 : OR rm32,sbytedword32 [mi:       hle o32 83 /1 ib,s                ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(83) 201 275 : SM0,SM1,LOCK,FL,386 */
        {I_OR, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54800, 26},
    /*  98 : OR rm64,sbytedword64 [mi:       hle o64 83 /1 ib,s                ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(83) 201 275 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_OR, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54807, 27},
    /*  99 : OR reg_al,imm8 [-i:       o8 0c ib                          ] SM0-1,FL,8086 */
        /* 001(0c) 021 : SM0,SM1,FL,8086 */
        {I_OR, 2, {REG_AL,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+72449, 28},
    /* 100 : OR reg_ax,imm16 [-i:       o16 0d iw                         ] SM0-1,FL,8086 */
        /* 320 001(0d) 031 : SM0,SM1,FL,8086 */
        {I_OR, 2, {REG_AX,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71263, 28},
    /* 101 : OR reg_eax,imm32 [-i:       o32 0d id                         ] SM0-1,ZU,FL,386 */
        /* 321 001(0d) 041 : SM0,SM1,ZU,FL,386 */
        {I_OR, 2, {REG_EAX,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71268, 31},
    /* 102 : OR reg_rax,sdword64 [-i:       o64 0d id,s                       ] SM0-1,ZU,LONG,FL,PROT,X86_64 */
        /* 324 001(0d) 255 : SM0,SM1,ZU,LONG,FL,PROT,X86_64 */
        {I_OR, 2, {REG_RAX,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+71273, 32},
    /* 103 : OR rm8,imm8 [mi:       hle o8 80 /1 ib                   ] SM0-1,LOCK,FL,8086 */
        /* 273 001(80) 201 021 : SM0,SM1,LOCK,FL,8086 */
        {I_OR, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66326, 25},
    /* 104 : OR rm16,imm16 [mi:       hle o16 81 /1 iw                  ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(81) 201 031 : SM0,SM1,LOCK,FL,8086 */
        {I_OR, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+54814, 25},
    /* 105 : OR rm32,imm32 [mi:       hle o32 81 /1 id                  ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(81) 201 041 : SM0,SM1,LOCK,FL,386 */
        {I_OR, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+54821, 26},
    /* 106 : OR rm64,sdword64 [mi:       hle o64 81 /1 id,s                ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(81) 201 255 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_OR, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54828, 27},
    /* 107 : OR reg8?,reg8,rm8 [vrm:      evex.nd1.nf.l0.m4.o8 0a /r        ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(0a) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS8,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23436, 33},
    /* 108 : OR reg8,rm8 [rm:      evex.nd0.nf.l0.m4.o8 0a /r        ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(0a) 110 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23445, 34},
    /* 109 : OR reg16?,reg16,rm16 [vrm:      evex.nd1.nf.l0.m4.o16 0b /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(0b) 121 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+9383, 36},
    /* 110 : OR reg16,rm16 [rm:      evex.nd0.nf.l0.m4.o16 0b /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(0b) 110 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+9393, 37},
    /* 111 : OR reg32?,reg32,rm32 [vrm:      evex.nd1.nf.l0.m4.o32 0b /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(0b) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+9413, 39},
    /* 112 : OR reg32,rm32 [rm:      evex.nd0.nf.l0.m4.o32 0b /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(0b) 110 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+9423, 40},
    /* 113 : OR reg64?,reg64,rm64 [vrm:      evex.nd1.nf.l0.m4.o64 0b /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(0b) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+9443, 39},
    /* 114 : OR reg64,rm64 [rm:      evex.nd0.nf.l0.m4.o64 0b /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(0b) 110 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+9453, 40},
    /* 115 : OR reg8?,rm8,reg8 [vmr:      evex.nd1.nf.l0.m4.o8 08 /r        ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(08) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23463, 33},
    /* 116 : OR rm8,reg8 [mr:      evex.nd0.nf.l0.m4.o8 08 /r        ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(08) 101 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23472, 34},
    /* 117 : OR reg16?,rm16,reg16 [vmr:      evex.nd1.nf.l0.m4.o16 09 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(09) 112 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+9473, 36},
    /* 118 : OR rm16,reg16 [mr:      evex.nd0.nf.l0.m4.o16 09 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(09) 101 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+9483, 37},
    /* 119 : OR reg32?,rm32,reg32 [vmr:      evex.nd1.nf.l0.m4.o32 09 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(09) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+9503, 39},
    /* 120 : OR rm32,reg32 [mr:      evex.nd0.nf.l0.m4.o32 09 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(09) 101 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+9513, 40},
    /* 121 : OR reg64?,rm64,reg64 [vmr:      evex.nd1.nf.l0.m4.o64 09 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(09) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+9533, 39},
    /* 122 : OR rm64,reg64 [mr:      evex.nd0.nf.l0.m4.o64 09 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(09) 101 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+9543, 40},
    /* 123 : OR reg16?,rm16,sbyteword16 [vmi:      evex.nd1.nf.l0.m4.o16 83 /1 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(83) 211 276 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+222, 36},
    /* 124 : OR rm16,sbyteword16 [mi:      evex.nd0.nf.l0.m4.o16 83 /1 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(83) 201 275 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+233, 37},
    /* 125 : OR reg32?,rm32,sbytedword32 [vmi:      evex.nd1.nf.l0.m4.o32 83 /1 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(83) 211 276 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+255, 39},
    /* 126 : OR rm32,sbytedword32 [mi:      evex.nd0.nf.l0.m4.o32 83 /1 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(83) 201 275 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+266, 40},
    /* 127 : OR reg64?,rm64,sbytedword64 [vmi:      evex.nd1.nf.l0.m4.o64 83 /1 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(83) 211 276 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+288, 39},
    /* 128 : OR rm64,sbytedword64 [mi:      evex.nd0.nf.l0.m4.o64 83 /1 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(83) 201 275 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+299, 40},
    /* 129 : OR reg8?,rm8,imm8 [vmi:      evex.nd1.nf.l0.m4.o8 80 /1 ib     ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(80) 211 022 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+9563, 33},
    /* 130 : OR rm8,imm8 [mi:      evex.nd0.nf.l0.m4.o8 80 /1 ib     ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(80) 201 021 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+9573, 34},
    /* 131 : OR reg16?,rm16,imm16 [vmi:      evex.nd1.nf.l0.m4.o16 81 /1 iw    ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(81) 211 032 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+321, 36},
    /* 132 : OR rm16,imm16 [mi:      evex.nd0.nf.l0.m4.o16 81 /1 iw    ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(81) 201 031 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+332, 37},
    /* 133 : OR reg32?,rm32,imm32 [vmi:      evex.nd1.nf.l0.m4.o32 81 /1 id    ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(81) 211 042 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+354, 39},
    /* 134 : OR rm32,imm32 [mi:      evex.nd0.nf.l0.m4.o32 81 /1 id    ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(81) 201 041 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+365, 40},
    /* 135 : OR reg64?,rm64,sdword64 [vmi:      evex.nd1.nf.l0.m4.o64 81 /1 id,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(81) 211 256 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+387, 39},
    /* 136 : OR rm64,sdword64 [mi:      evex.nd0.nf.l0.m4.o64 81 /1 id,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(81) 201 255 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+398, 40},
    /* 137 : ADC rm8,reg8 [mr:       hle o8 10 /r                      ] SM0-1,LOCK,FL,8086 */
        /* 273 001(10) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_ADC, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+71278, 25},
    /* 138 : ADC rm16,reg16 [mr:       hle o16 11 /r                     ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(11) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_ADC, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66332, 25},
    /* 139 : ADC rm32,reg32 [mr:       hle o32 11 /r                     ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(11) 101 : SM0,SM1,LOCK,FL,386 */
        {I_ADC, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66338, 26},
    /* 140 : ADC rm64,reg64 [mr:       hle o64 11 /r                     ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(11) 101 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66344, 27},
    /* 141 : ADC reg8,rm8 [rm:       o8 12 /r                          ] SM0-1,FL,8086 */
        /* 001(12) 110 : SM0,SM1,FL,8086 */
        {I_ADC, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+69286, 28},
    /* 142 : ADC reg16,rm16 [rm:       o16 13 /r                         ] SM0-1,FL,8086 */
        /* 320 001(13) 110 : SM0,SM1,FL,8086 */
        {I_ADC, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+9618, 28},
    /* 143 : ADC reg32,rm32 [rm:       o32 13 /r                         ] SM0-1,FL,386 */
        /* 321 001(13) 110 : SM0,SM1,FL,386 */
        {I_ADC, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+9648, 29},
    /* 144 : ADC reg64,rm64 [rm:       o64 13 /r                         ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 001(13) 110 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_ADC, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+9678, 30},
    /* 145 : ADC rm16,sbyteword16 [mi:       hle o16 83 /2 ib,s                ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(83) 202 275 : SM0,SM1,LOCK,FL,8086 */
        {I_ADC, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54835, 25},
    /* 146 : ADC rm32,sbytedword32 [mi:       hle o32 83 /2 ib,s                ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(83) 202 275 : SM0,SM1,LOCK,FL,386 */
        {I_ADC, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54842, 26},
    /* 147 : ADC rm64,sbytedword64 [mi:       hle o64 83 /2 ib,s                ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(83) 202 275 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54849, 27},
    /* 148 : ADC reg_al,imm8 [-i:       o8 14 ib                          ] SM0-1,FL,8086 */
        /* 001(14) 021 : SM0,SM1,FL,8086 */
        {I_ADC, 2, {REG_AL,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+72453, 28},
    /* 149 : ADC reg_ax,imm16 [-i:       o16 15 iw                         ] SM0-1,FL,8086 */
        /* 320 001(15) 031 : SM0,SM1,FL,8086 */
        {I_ADC, 2, {REG_AX,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71283, 28},
    /* 150 : ADC reg_eax,imm32 [-i:       o32 15 id                         ] SM0-1,ZU,FL,386 */
        /* 321 001(15) 041 : SM0,SM1,ZU,FL,386 */
        {I_ADC, 2, {REG_EAX,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71288, 31},
    /* 151 : ADC reg_rax,sdword64 [-i:       o64 15 id,s                       ] SM0-1,ZU,LONG,FL,PROT,X86_64 */
        /* 324 001(15) 255 : SM0,SM1,ZU,LONG,FL,PROT,X86_64 */
        {I_ADC, 2, {REG_RAX,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+71293, 32},
    /* 152 : ADC rm8,imm8 [mi:       hle o8 80 /2 ib                   ] SM0-1,LOCK,FL,8086 */
        /* 273 001(80) 202 021 : SM0,SM1,LOCK,FL,8086 */
        {I_ADC, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66350, 25},
    /* 153 : ADC rm16,imm16 [mi:       hle o16 81 /2 iw                  ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(81) 202 031 : SM0,SM1,LOCK,FL,8086 */
        {I_ADC, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+54856, 25},
    /* 154 : ADC rm32,imm32 [mi:       hle o32 81 /2 id                  ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(81) 202 041 : SM0,SM1,LOCK,FL,386 */
        {I_ADC, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+54863, 26},
    /* 155 : ADC rm64,sdword64 [mi:       hle o64 81 /2 id,s                ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(81) 202 255 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54870, 27},
    /* 156 : ADC reg8?,reg8,rm8 [vrm:      evex.nd1..l0.m4.o8 12 /r          ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(12) 121 : SM0,SM1,SM2,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS8,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23490, 42},
    /* 157 : ADC reg8,rm8 [rm:      evex.nd0..l0.m4.o8 12 /r          ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(12) 110 : SM0,SM1,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23499, 43},
    /* 158 : ADC reg16?,reg16,rm16 [vrm:      evex.nd1..l0.m4.o16 13 /r         ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(13) 121 : SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+9593, 45},
    /* 159 : ADC reg16,rm16 [rm:      evex.nd0..l0.m4.o16 13 /r         ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(13) 110 : SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+9603, 46},
    /* 160 : ADC reg32?,reg32,rm32 [vrm:      evex.nd1..l0.m4.o32 13 /r         ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(13) 121 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+9623, 48},
    /* 161 : ADC reg32,rm32 [rm:      evex.nd0..l0.m4.o32 13 /r         ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(13) 110 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+9633, 49},
    /* 162 : ADC reg64?,reg64,rm64 [vrm:      evex.nd1..l0.m4.o64 13 /r         ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(13) 121 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+9653, 48},
    /* 163 : ADC reg64,rm64 [rm:      evex.nd0..l0.m4.o64 13 /r         ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(13) 110 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+9663, 49},
    /* 164 : ADC reg8?,rm8,reg8 [vmr:      evex.nd1..l0.m4.o8 10 /r          ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(10) 112 : SM0,SM1,SM2,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23517, 42},
    /* 165 : ADC rm8,reg8 [mr:      evex.nd0..l0.m4.o8 10 /r          ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(10) 101 : SM0,SM1,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23526, 43},
    /* 166 : ADC reg16?,rm16,reg16 [vmr:      evex.nd1..l0.m4.o16 11 /r         ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(11) 112 : SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+9683, 45},
    /* 167 : ADC rm16,reg16 [mr:      evex.nd0..l0.m4.o16 11 /r         ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(11) 101 : SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+9693, 46},
    /* 168 : ADC reg32?,rm32,reg32 [vmr:      evex.nd1..l0.m4.o32 11 /r         ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(11) 112 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+9713, 48},
    /* 169 : ADC rm32,reg32 [mr:      evex.nd0..l0.m4.o32 11 /r         ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(11) 101 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+9723, 49},
    /* 170 : ADC reg64?,rm64,reg64 [vmr:      evex.nd1..l0.m4.o64 11 /r         ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(11) 112 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+9743, 48},
    /* 171 : ADC rm64,reg64 [mr:      evex.nd0..l0.m4.o64 11 /r         ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(11) 101 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+9753, 49},
    /* 172 : ADC reg16?,rm16,sbyteword16 [vmi:      evex.nd1..l0.m4.o16 83 /2 ib,s    ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(83) 212 276 : SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS16,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+420, 45},
    /* 173 : ADC rm16,sbyteword16 [mi:      evex.nd0..l0.m4.o16 83 /2 ib,s    ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(83) 202 275 : SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+431, 46},
    /* 174 : ADC reg32?,rm32,sbytedword32 [vmi:      evex.nd1..l0.m4.o32 83 /2 ib,s    ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(83) 212 276 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS32,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+453, 48},
    /* 175 : ADC rm32,sbytedword32 [mi:      evex.nd0..l0.m4.o32 83 /2 ib,s    ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(83) 202 275 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+464, 49},
    /* 176 : ADC reg64?,rm64,sbytedword64 [vmi:      evex.nd1..l0.m4.o64 83 /2 ib,s    ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(83) 212 276 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+486, 48},
    /* 177 : ADC rm64,sbytedword64 [mi:      evex.nd0..l0.m4.o64 83 /2 ib,s    ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(83) 202 275 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+497, 49},
    /* 178 : ADC reg8?,rm8,imm8 [vmi:      evex.nd1..l0.m4.o8 80 /2 ib       ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(80) 212 022 : SM0,SM1,SM2,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+9773, 42},
    /* 179 : ADC rm8,imm8 [mi:      evex.nd0..l0.m4.o8 80 /2 ib       ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(80) 202 021 : SM0,SM1,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+9783, 43},
    /* 180 : ADC reg16?,rm16,imm16 [vmi:      evex.nd1..l0.m4.o16 81 /2 iw      ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(81) 212 032 : SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+519, 45},
    /* 181 : ADC rm16,imm16 [mi:      evex.nd0..l0.m4.o16 81 /2 iw      ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(81) 202 031 : SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+530, 46},
    /* 182 : ADC reg32?,rm32,imm32 [vmi:      evex.nd1..l0.m4.o32 81 /2 id      ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(81) 212 042 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+552, 48},
    /* 183 : ADC rm32,imm32 [mi:      evex.nd0..l0.m4.o32 81 /2 id      ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(81) 202 041 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+563, 49},
    /* 184 : ADC reg64?,rm64,sdword64 [vmi:      evex.nd1..l0.m4.o64 81 /2 id,s    ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(81) 212 256 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+585, 48},
    /* 185 : ADC rm64,sdword64 [mi:      evex.nd0..l0.m4.o64 81 /2 id,s    ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(81) 202 255 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+596, 49},
    /* 186 : SBB rm8,reg8 [mr:       hle o8 18 /r                      ] SM0-1,LOCK,FL,8086 */
        /* 273 001(18) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_SBB, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+71298, 25},
    /* 187 : SBB rm16,reg16 [mr:       hle o16 19 /r                     ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(19) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_SBB, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66356, 25},
    /* 188 : SBB rm32,reg32 [mr:       hle o32 19 /r                     ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(19) 101 : SM0,SM1,LOCK,FL,386 */
        {I_SBB, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66362, 26},
    /* 189 : SBB rm64,reg64 [mr:       hle o64 19 /r                     ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(19) 101 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66368, 27},
    /* 190 : SBB reg8,rm8 [rm:       o8 1a /r                          ] SM0-1,FL,8086 */
        /* 001(1a) 110 : SM0,SM1,FL,8086 */
        {I_SBB, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+69880, 28},
    /* 191 : SBB reg16,rm16 [rm:       o16 1b /r                         ] SM0-1,FL,8086 */
        /* 320 001(1b) 110 : SM0,SM1,FL,8086 */
        {I_SBB, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+9828, 28},
    /* 192 : SBB reg32,rm32 [rm:       o32 1b /r                         ] SM0-1,FL,386 */
        /* 321 001(1b) 110 : SM0,SM1,FL,386 */
        {I_SBB, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+9858, 29},
    /* 193 : SBB reg64,rm64 [rm:       o64 1b /r                         ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 001(1b) 110 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_SBB, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+9888, 30},
    /* 194 : SBB rm16,sbyteword16 [mi:       hle o16 83 /3 ib,s                ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(83) 203 275 : SM0,SM1,LOCK,FL,8086 */
        {I_SBB, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54877, 25},
    /* 195 : SBB rm32,sbytedword32 [mi:       hle o32 83 /3 ib,s                ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(83) 203 275 : SM0,SM1,LOCK,FL,386 */
        {I_SBB, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54884, 26},
    /* 196 : SBB rm64,sbytedword64 [mi:       hle o64 83 /3 ib,s                ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(83) 203 275 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54891, 27},
    /* 197 : SBB reg_al,imm8 [-i:       o8 1c ib                          ] SM0-1,FL,8086 */
        /* 001(1c) 021 : SM0,SM1,FL,8086 */
        {I_SBB, 2, {REG_AL,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+72457, 28},
    /* 198 : SBB reg_ax,imm16 [-i:       o16 1d iw                         ] SM0-1,FL,8086 */
        /* 320 001(1d) 031 : SM0,SM1,FL,8086 */
        {I_SBB, 2, {REG_AX,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71303, 28},
    /* 199 : SBB reg_eax,imm32 [-i:       o32 1d id                         ] SM0-1,ZU,FL,386 */
        /* 321 001(1d) 041 : SM0,SM1,ZU,FL,386 */
        {I_SBB, 2, {REG_EAX,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71308, 31},
    /* 200 : SBB reg_rax,sdword64 [-i:       o64 1d id,s                       ] SM0-1,ZU,LONG,FL,PROT,X86_64 */
        /* 324 001(1d) 255 : SM0,SM1,ZU,LONG,FL,PROT,X86_64 */
        {I_SBB, 2, {REG_RAX,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+71313, 32},
    /* 201 : SBB rm8,imm8 [mi:       hle o8 80 /3 ib                   ] SM0-1,LOCK,FL,8086 */
        /* 273 001(80) 203 021 : SM0,SM1,LOCK,FL,8086 */
        {I_SBB, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66374, 25},
    /* 202 : SBB rm16,imm16 [mi:       hle o16 81 /3 iw                  ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(81) 203 031 : SM0,SM1,LOCK,FL,8086 */
        {I_SBB, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+54898, 25},
    /* 203 : SBB rm32,imm32 [mi:       hle o32 81 /3 id                  ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(81) 203 041 : SM0,SM1,LOCK,FL,386 */
        {I_SBB, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+54905, 26},
    /* 204 : SBB rm64,sdword64 [mi:       hle o64 81 /3 id,s                ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(81) 203 255 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54912, 27},
    /* 205 : SBB reg8?,reg8,rm8 [vrm:      evex.nd1..l0.m4.o8 1a /r          ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(1a) 121 : SM0,SM1,SM2,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS8,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23544, 42},
    /* 206 : SBB reg8,rm8 [rm:      evex.nd0..l0.m4.o8 1a /r          ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(1a) 110 : SM0,SM1,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23553, 43},
    /* 207 : SBB reg16?,reg16,rm16 [vrm:      evex.nd1..l0.m4.o16 1b /r         ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(1b) 121 : SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+9803, 45},
    /* 208 : SBB reg16,rm16 [rm:      evex.nd0..l0.m4.o16 1b /r         ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(1b) 110 : SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+9813, 46},
    /* 209 : SBB reg32?,reg32,rm32 [vrm:      evex.nd1..l0.m4.o32 1b /r         ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(1b) 121 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+9833, 48},
    /* 210 : SBB reg32,rm32 [rm:      evex.nd0..l0.m4.o32 1b /r         ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(1b) 110 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+9843, 49},
    /* 211 : SBB reg64?,reg64,rm64 [vrm:      evex.nd1..l0.m4.o64 1b /r         ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(1b) 121 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+9863, 48},
    /* 212 : SBB reg64,rm64 [rm:      evex.nd0..l0.m4.o64 1b /r         ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(1b) 110 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+9873, 49},
    /* 213 : SBB reg8?,rm8,reg8 [vmr:      evex.nd1..l0.m4.o8 18 /r          ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(18) 112 : SM0,SM1,SM2,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23571, 42},
    /* 214 : SBB rm8,reg8 [mr:      evex.nd0..l0.m4.o8 18 /r          ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(18) 101 : SM0,SM1,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23580, 43},
    /* 215 : SBB reg16?,rm16,reg16 [vmr:      evex.nd1..l0.m4.o16 19 /r         ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(19) 112 : SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+9893, 45},
    /* 216 : SBB rm16,reg16 [mr:      evex.nd0..l0.m4.o16 19 /r         ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(19) 101 : SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+9903, 46},
    /* 217 : SBB reg32?,rm32,reg32 [vmr:      evex.nd1..l0.m4.o32 19 /r         ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(19) 112 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+9923, 48},
    /* 218 : SBB rm32,reg32 [mr:      evex.nd0..l0.m4.o32 19 /r         ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(19) 101 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+9933, 49},
    /* 219 : SBB reg64?,rm64,reg64 [vmr:      evex.nd1..l0.m4.o64 19 /r         ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(19) 112 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+9953, 48},
    /* 220 : SBB rm64,reg64 [mr:      evex.nd0..l0.m4.o64 19 /r         ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(19) 101 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+9963, 49},
    /* 221 : SBB reg16?,rm16,sbyteword16 [vmi:      evex.nd1..l0.m4.o16 83 /3 ib,s    ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(83) 213 276 : SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS16,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+618, 45},
    /* 222 : SBB rm16,sbyteword16 [mi:      evex.nd0..l0.m4.o16 83 /3 ib,s    ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(83) 203 275 : SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+629, 46},
    /* 223 : SBB reg32?,rm32,sbytedword32 [vmi:      evex.nd1..l0.m4.o32 83 /3 ib,s    ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(83) 213 276 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS32,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+651, 48},
    /* 224 : SBB rm32,sbytedword32 [mi:      evex.nd0..l0.m4.o32 83 /3 ib,s    ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(83) 203 275 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+662, 49},
    /* 225 : SBB reg64?,rm64,sbytedword64 [vmi:      evex.nd1..l0.m4.o64 83 /3 ib,s    ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(83) 213 276 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+684, 48},
    /* 226 : SBB rm64,sbytedword64 [mi:      evex.nd0..l0.m4.o64 83 /3 ib,s    ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(83) 203 275 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+695, 49},
    /* 227 : SBB reg8?,rm8,imm8 [vmi:      evex.nd1..l0.m4.o8 80 /3 ib       ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(80) 213 022 : SM0,SM1,SM2,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+9983, 42},
    /* 228 : SBB rm8,imm8 [mi:      evex.nd0..l0.m4.o8 80 /3 ib       ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(80) 203 021 : SM0,SM1,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+9993, 43},
    /* 229 : SBB reg16?,rm16,imm16 [vmi:      evex.nd1..l0.m4.o16 81 /3 iw      ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(81) 213 032 : SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+717, 45},
    /* 230 : SBB rm16,imm16 [mi:      evex.nd0..l0.m4.o16 81 /3 iw      ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(81) 203 031 : SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+728, 46},
    /* 231 : SBB reg32?,rm32,imm32 [vmi:      evex.nd1..l0.m4.o32 81 /3 id      ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(81) 213 042 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+750, 48},
    /* 232 : SBB rm32,imm32 [mi:      evex.nd0..l0.m4.o32 81 /3 id      ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(81) 203 041 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+761, 49},
    /* 233 : SBB reg64?,rm64,sdword64 [vmi:      evex.nd1..l0.m4.o64 81 /3 id,s    ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(81) 213 256 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+783, 48},
    /* 234 : SBB rm64,sdword64 [mi:      evex.nd0..l0.m4.o64 81 /3 id,s    ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(81) 203 255 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+794, 49},
    /* 235 : AND rm8,reg8 [mr:       hle o8 20 /r                      ] SM0-1,LOCK,FL,8086 */
        /* 273 001(20) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_AND, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+71318, 25},
    /* 236 : AND rm16,reg16 [mr:       hle o16 21 /r                     ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(21) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_AND, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66380, 25},
    /* 237 : AND rm32,reg32 [mr:       hle o32 21 /r                     ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(21) 101 : SM0,SM1,LOCK,FL,386 */
        {I_AND, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66386, 26},
    /* 238 : AND rm64,reg64 [mr:       hle o64 21 /r                     ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(21) 101 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_AND, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66392, 27},
    /* 239 : AND reg8,rm8 [rm:       o8 22 /r                          ] SM0-1,FL,8086 */
        /* 001(22) 110 : SM0,SM1,FL,8086 */
        {I_AND, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+69640, 28},
    /* 240 : AND reg16,rm16 [rm:       o16 23 /r                         ] SM0-1,FL,8086 */
        /* 320 001(23) 110 : SM0,SM1,FL,8086 */
        {I_AND, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+10038, 28},
    /* 241 : AND reg32,rm32 [rm:       o32 23 /r                         ] SM0-1,FL,386 */
        /* 321 001(23) 110 : SM0,SM1,FL,386 */
        {I_AND, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+10068, 29},
    /* 242 : AND reg64,rm64 [rm:       o64 23 /r                         ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 001(23) 110 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_AND, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+10098, 30},
    /* 243 : AND rm16,sbyteword16 [mi:       hle o16 83 /4 ib,s                ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(83) 204 275 : SM0,SM1,LOCK,FL,8086 */
        {I_AND, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54919, 25},
    /* 244 : AND rm32,sbytedword32 [mi:       hle o32 83 /4 ib,s                ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(83) 204 275 : SM0,SM1,LOCK,FL,386 */
        {I_AND, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54926, 26},
    /* 245 : AND rm64,sbytedword64 [mi:       hle o64 83 /4 ib,s                ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(83) 204 275 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_AND, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54933, 27},
    /* 246 : AND reg_al,imm8 [-i:       o8 24 ib                          ] SM0-1,FL,8086 */
        /* 001(24) 021 : SM0,SM1,FL,8086 */
        {I_AND, 2, {REG_AL,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+72461, 28},
    /* 247 : AND reg_ax,imm16 [-i:       o16 25 iw                         ] SM0-1,FL,8086 */
        /* 320 001(25) 031 : SM0,SM1,FL,8086 */
        {I_AND, 2, {REG_AX,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71323, 28},
    /* 248 : AND reg_eax,imm32 [-i:       o32 25 id                         ] SM0-1,ZU,FL,386 */
        /* 321 001(25) 041 : SM0,SM1,ZU,FL,386 */
        {I_AND, 2, {REG_EAX,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71328, 31},
    /* 249 : AND reg_rax,sdword64 [-i:       o64 25 id,s                       ] SM0-1,ZU,LONG,FL,PROT,X86_64 */
        /* 324 001(25) 255 : SM0,SM1,ZU,LONG,FL,PROT,X86_64 */
        {I_AND, 2, {REG_RAX,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+71333, 32},
    /* 250 : AND rm8,imm8 [mi:       hle o8 80 /4 ib                   ] SM0-1,LOCK,FL,8086 */
        /* 273 001(80) 204 021 : SM0,SM1,LOCK,FL,8086 */
        {I_AND, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66398, 25},
    /* 251 : AND rm16,imm16 [mi:       hle o16 81 /4 iw                  ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(81) 204 031 : SM0,SM1,LOCK,FL,8086 */
        {I_AND, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+54940, 25},
    /* 252 : AND rm32,imm32 [mi:       hle o32 81 /4 id                  ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(81) 204 041 : SM0,SM1,LOCK,FL,386 */
        {I_AND, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+54947, 26},
    /* 253 : AND rm64,sdword64 [mi:       hle o64 81 /4 id,s                ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(81) 204 255 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_AND, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54954, 27},
    /* 254 : AND reg8?,reg8,rm8 [vrm:      evex.nd1.nf.l0.m4.o8 22 /r        ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(22) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS8,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23598, 33},
    /* 255 : AND reg8,rm8 [rm:      evex.nd0.nf.l0.m4.o8 22 /r        ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(22) 110 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23607, 34},
    /* 256 : AND reg16?,reg16,rm16 [vrm:      evex.nd1.nf.l0.m4.o16 23 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(23) 121 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+10013, 36},
    /* 257 : AND reg16,rm16 [rm:      evex.nd0.nf.l0.m4.o16 23 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(23) 110 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+10023, 37},
    /* 258 : AND reg32?,reg32,rm32 [vrm:      evex.nd1.nf.l0.m4.o32 23 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(23) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+10043, 39},
    /* 259 : AND reg32,rm32 [rm:      evex.nd0.nf.l0.m4.o32 23 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(23) 110 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+10053, 40},
    /* 260 : AND reg64?,reg64,rm64 [vrm:      evex.nd1.nf.l0.m4.o64 23 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(23) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+10073, 39},
    /* 261 : AND reg64,rm64 [rm:      evex.nd0.nf.l0.m4.o64 23 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(23) 110 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+10083, 40},
    /* 262 : AND reg8?,rm8,reg8 [vmr:      evex.nd1.nf.l0.m4.o8 20 /r        ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(20) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23625, 33},
    /* 263 : AND rm8,reg8 [mr:      evex.nd0.nf.l0.m4.o8 20 /r        ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(20) 101 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23634, 34},
    /* 264 : AND reg16?,rm16,reg16 [vmr:      evex.nd1.nf.l0.m4.o16 21 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(21) 112 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+10103, 36},
    /* 265 : AND rm16,reg16 [mr:      evex.nd0.nf.l0.m4.o16 21 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(21) 101 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+10113, 37},
    /* 266 : AND reg32?,rm32,reg32 [vmr:      evex.nd1.nf.l0.m4.o32 21 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(21) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+10133, 39},
    /* 267 : AND rm32,reg32 [mr:      evex.nd0.nf.l0.m4.o32 21 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(21) 101 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+10143, 40},
    /* 268 : AND reg64?,rm64,reg64 [vmr:      evex.nd1.nf.l0.m4.o64 21 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(21) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+10163, 39},
    /* 269 : AND rm64,reg64 [mr:      evex.nd0.nf.l0.m4.o64 21 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(21) 101 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+10173, 40},
    /* 270 : AND reg16?,rm16,sbyteword16 [vmi:      evex.nd1.nf.l0.m4.o16 83 /4 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(83) 214 276 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS16,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+816, 36},
    /* 271 : AND rm16,sbyteword16 [mi:      evex.nd0.nf.l0.m4.o16 83 /4 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(83) 204 275 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+827, 37},
    /* 272 : AND reg32?,rm32,sbytedword32 [vmi:      evex.nd1.nf.l0.m4.o32 83 /4 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(83) 214 276 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS32,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+849, 39},
    /* 273 : AND rm32,sbytedword32 [mi:      evex.nd0.nf.l0.m4.o32 83 /4 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(83) 204 275 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+860, 40},
    /* 274 : AND reg64?,rm64,sbytedword64 [vmi:      evex.nd1.nf.l0.m4.o64 83 /4 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(83) 214 276 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+882, 39},
    /* 275 : AND rm64,sbytedword64 [mi:      evex.nd0.nf.l0.m4.o64 83 /4 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(83) 204 275 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+893, 40},
    /* 276 : AND reg8?,rm8,imm8 [vmi:      evex.nd1.nf.l0.m4.o8 80 /4 ib     ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(80) 214 022 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+10193, 33},
    /* 277 : AND rm8,imm8 [mi:      evex.nd0.nf.l0.m4.o8 80 /4 ib     ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(80) 204 021 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+10203, 34},
    /* 278 : AND reg16?,rm16,imm16 [vmi:      evex.nd1.nf.l0.m4.o16 81 /4 iw    ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(81) 214 032 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+915, 36},
    /* 279 : AND rm16,imm16 [mi:      evex.nd0.nf.l0.m4.o16 81 /4 iw    ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(81) 204 031 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+926, 37},
    /* 280 : AND reg32?,rm32,imm32 [vmi:      evex.nd1.nf.l0.m4.o32 81 /4 id    ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(81) 214 042 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+948, 39},
    /* 281 : AND rm32,imm32 [mi:      evex.nd0.nf.l0.m4.o32 81 /4 id    ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(81) 204 041 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+959, 40},
    /* 282 : AND reg64?,rm64,sdword64 [vmi:      evex.nd1.nf.l0.m4.o64 81 /4 id,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(81) 214 256 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+981, 39},
    /* 283 : AND rm64,sdword64 [mi:      evex.nd0.nf.l0.m4.o64 81 /4 id,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(81) 204 255 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+992, 40},
    /* 284 : SUB rm8,reg8 [mr:       hle o8 28 /r                      ] SM0-1,LOCK,FL,8086 */
        /* 273 001(28) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_SUB, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+71338, 25},
    /* 285 : SUB rm16,reg16 [mr:       hle o16 29 /r                     ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(29) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_SUB, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66404, 25},
    /* 286 : SUB rm32,reg32 [mr:       hle o32 29 /r                     ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(29) 101 : SM0,SM1,LOCK,FL,386 */
        {I_SUB, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66410, 26},
    /* 287 : SUB rm64,reg64 [mr:       hle o64 29 /r                     ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(29) 101 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66416, 27},
    /* 288 : SUB reg8,rm8 [rm:       o8 2a /r                          ] SM0-1,FL,8086 */
        /* 001(2a) 110 : SM0,SM1,FL,8086 */
        {I_SUB, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+69550, 28},
    /* 289 : SUB reg16,rm16 [rm:       o16 2b /r                         ] SM0-1,FL,8086 */
        /* 320 001(2b) 110 : SM0,SM1,FL,8086 */
        {I_SUB, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+10248, 28},
    /* 290 : SUB reg32,rm32 [rm:       o32 2b /r                         ] SM0-1,FL,386 */
        /* 321 001(2b) 110 : SM0,SM1,FL,386 */
        {I_SUB, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+10278, 29},
    /* 291 : SUB reg64,rm64 [rm:       o64 2b /r                         ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 001(2b) 110 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_SUB, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+10308, 30},
    /* 292 : SUB rm16,sbyteword16 [mi:       hle o16 83 /5 ib,s                ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(83) 205 275 : SM0,SM1,LOCK,FL,8086 */
        {I_SUB, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54961, 25},
    /* 293 : SUB rm32,sbytedword32 [mi:       hle o32 83 /5 ib,s                ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(83) 205 275 : SM0,SM1,LOCK,FL,386 */
        {I_SUB, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54968, 26},
    /* 294 : SUB rm64,sbytedword64 [mi:       hle o64 83 /5 ib,s                ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(83) 205 275 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54975, 27},
    /* 295 : SUB reg_al,imm8 [-i:       o8 2c ib                          ] SM0-1,FL,8086 */
        /* 001(2c) 021 : SM0,SM1,FL,8086 */
        {I_SUB, 2, {REG_AL,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+72465, 28},
    /* 296 : SUB reg_ax,imm16 [-i:       o16 2d iw                         ] SM0-1,FL,8086 */
        /* 320 001(2d) 031 : SM0,SM1,FL,8086 */
        {I_SUB, 2, {REG_AX,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71343, 28},
    /* 297 : SUB reg_eax,imm32 [-i:       o32 2d id                         ] SM0-1,ZU,FL,386 */
        /* 321 001(2d) 041 : SM0,SM1,ZU,FL,386 */
        {I_SUB, 2, {REG_EAX,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71348, 31},
    /* 298 : SUB reg_rax,sdword64 [-i:       o64 2d id,s                       ] SM0-1,ZU,LONG,FL,PROT,X86_64 */
        /* 324 001(2d) 255 : SM0,SM1,ZU,LONG,FL,PROT,X86_64 */
        {I_SUB, 2, {REG_RAX,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+71353, 32},
    /* 299 : SUB rm8,imm8 [mi:       hle o8 80 /5 ib                   ] SM0-1,LOCK,FL,8086 */
        /* 273 001(80) 205 021 : SM0,SM1,LOCK,FL,8086 */
        {I_SUB, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66422, 25},
    /* 300 : SUB rm16,imm16 [mi:       hle o16 81 /5 iw                  ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(81) 205 031 : SM0,SM1,LOCK,FL,8086 */
        {I_SUB, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+54982, 25},
    /* 301 : SUB rm32,imm32 [mi:       hle o32 81 /5 id                  ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(81) 205 041 : SM0,SM1,LOCK,FL,386 */
        {I_SUB, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+54989, 26},
    /* 302 : SUB rm64,sdword64 [mi:       hle o64 81 /5 id,s                ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(81) 205 255 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54996, 27},
    /* 303 : SUB reg8?,reg8,rm8 [vrm:      evex.nd1.nf.l0.m4.o8 2a /r        ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(2a) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS8,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23652, 33},
    /* 304 : SUB reg8,rm8 [rm:      evex.nd0.nf.l0.m4.o8 2a /r        ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(2a) 110 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23661, 34},
    /* 305 : SUB reg16?,reg16,rm16 [vrm:      evex.nd1.nf.l0.m4.o16 2b /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(2b) 121 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+10223, 36},
    /* 306 : SUB reg16,rm16 [rm:      evex.nd0.nf.l0.m4.o16 2b /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(2b) 110 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+10233, 37},
    /* 307 : SUB reg32?,reg32,rm32 [vrm:      evex.nd1.nf.l0.m4.o32 2b /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(2b) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+10253, 39},
    /* 308 : SUB reg32,rm32 [rm:      evex.nd0.nf.l0.m4.o32 2b /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(2b) 110 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+10263, 40},
    /* 309 : SUB reg64?,reg64,rm64 [vrm:      evex.nd1.nf.l0.m4.o64 2b /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(2b) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+10283, 39},
    /* 310 : SUB reg64,rm64 [rm:      evex.nd0.nf.l0.m4.o64 2b /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(2b) 110 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+10293, 40},
    /* 311 : SUB reg8?,rm8,reg8 [vmr:      evex.nd1.nf.l0.m4.o8 28 /r        ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(28) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23679, 33},
    /* 312 : SUB rm8,reg8 [mr:      evex.nd0.nf.l0.m4.o8 28 /r        ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(28) 101 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23688, 34},
    /* 313 : SUB reg16?,rm16,reg16 [vmr:      evex.nd1.nf.l0.m4.o16 29 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(29) 112 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+10313, 36},
    /* 314 : SUB rm16,reg16 [mr:      evex.nd0.nf.l0.m4.o16 29 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(29) 101 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+10323, 37},
    /* 315 : SUB reg32?,rm32,reg32 [vmr:      evex.nd1.nf.l0.m4.o32 29 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(29) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+10343, 39},
    /* 316 : SUB rm32,reg32 [mr:      evex.nd0.nf.l0.m4.o32 29 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(29) 101 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+10353, 40},
    /* 317 : SUB reg64?,rm64,reg64 [vmr:      evex.nd1.nf.l0.m4.o64 29 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(29) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+10373, 39},
    /* 318 : SUB rm64,reg64 [mr:      evex.nd0.nf.l0.m4.o64 29 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(29) 101 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+10383, 40},
    /* 319 : SUB reg16?,rm16,sbyteword16 [vmi:      evex.nd1.nf.l0.m4.o16 83 /5 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(83) 215 276 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS16,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+1014, 36},
    /* 320 : SUB rm16,sbyteword16 [mi:      evex.nd0.nf.l0.m4.o16 83 /5 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(83) 205 275 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+1025, 37},
    /* 321 : SUB reg32?,rm32,sbytedword32 [vmi:      evex.nd1.nf.l0.m4.o32 83 /5 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(83) 215 276 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS32,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+1047, 39},
    /* 322 : SUB rm32,sbytedword32 [mi:      evex.nd0.nf.l0.m4.o32 83 /5 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(83) 205 275 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+1058, 40},
    /* 323 : SUB reg64?,rm64,sbytedword64 [vmi:      evex.nd1.nf.l0.m4.o64 83 /5 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(83) 215 276 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+1080, 39},
    /* 324 : SUB rm64,sbytedword64 [mi:      evex.nd0.nf.l0.m4.o64 83 /5 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(83) 205 275 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+1091, 40},
    /* 325 : SUB reg8?,rm8,imm8 [vmi:      evex.nd1.nf.l0.m4.o8 80 /5 ib     ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(80) 215 022 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+10403, 33},
    /* 326 : SUB rm8,imm8 [mi:      evex.nd0.nf.l0.m4.o8 80 /5 ib     ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(80) 205 021 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+10413, 34},
    /* 327 : SUB reg16?,rm16,imm16 [vmi:      evex.nd1.nf.l0.m4.o16 81 /5 iw    ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(81) 215 032 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+1113, 36},
    /* 328 : SUB rm16,imm16 [mi:      evex.nd0.nf.l0.m4.o16 81 /5 iw    ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(81) 205 031 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+1124, 37},
    /* 329 : SUB reg32?,rm32,imm32 [vmi:      evex.nd1.nf.l0.m4.o32 81 /5 id    ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(81) 215 042 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+1146, 39},
    /* 330 : SUB rm32,imm32 [mi:      evex.nd0.nf.l0.m4.o32 81 /5 id    ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(81) 205 041 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+1157, 40},
    /* 331 : SUB reg64?,rm64,sdword64 [vmi:      evex.nd1.nf.l0.m4.o64 81 /5 id,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(81) 215 256 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+1179, 39},
    /* 332 : SUB rm64,sdword64 [mi:      evex.nd0.nf.l0.m4.o64 81 /5 id,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(81) 205 255 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+1190, 40},
    /* 333 : XOR rm8,reg8 [mr:       hle o8 30 /r                      ] SM0-1,LOCK,FL,8086 */
        /* 273 001(30) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_XOR, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+71358, 25},
    /* 334 : XOR rm16,reg16 [mr:       hle o16 31 /r                     ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(31) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_XOR, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66428, 25},
    /* 335 : XOR rm32,reg32 [mr:       hle o32 31 /r                     ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(31) 101 : SM0,SM1,LOCK,FL,386 */
        {I_XOR, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66434, 26},
    /* 336 : XOR rm64,reg64 [mr:       hle o64 31 /r                     ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(31) 101 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66440, 27},
    /* 337 : XOR reg8,rm8 [rm:       o8 32 /r                          ] SM0-1,FL,8086 */
        /* 001(32) 110 : SM0,SM1,FL,8086 */
        {I_XOR, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+69676, 28},
    /* 338 : XOR reg16,rm16 [rm:       o16 33 /r                         ] SM0-1,FL,8086 */
        /* 320 001(33) 110 : SM0,SM1,FL,8086 */
        {I_XOR, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+10458, 28},
    /* 339 : XOR reg32,rm32 [rm:       o32 33 /r                         ] SM0-1,FL,386 */
        /* 321 001(33) 110 : SM0,SM1,FL,386 */
        {I_XOR, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+10488, 29},
    /* 340 : XOR reg64,rm64 [rm:       o64 33 /r                         ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 001(33) 110 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_XOR, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+10518, 30},
    /* 341 : XOR rm16,sbyteword16 [mi:       hle o16 83 /6 ib,s                ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(83) 206 275 : SM0,SM1,LOCK,FL,8086 */
        {I_XOR, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55003, 25},
    /* 342 : XOR rm32,sbytedword32 [mi:       hle o32 83 /6 ib,s                ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(83) 206 275 : SM0,SM1,LOCK,FL,386 */
        {I_XOR, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55010, 26},
    /* 343 : XOR rm64,sbytedword64 [mi:       hle o64 83 /6 ib,s                ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(83) 206 275 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55017, 27},
    /* 344 : XOR reg_al,imm8 [-i:       o8 34 ib                          ] SM0-1,FL,8086 */
        /* 001(34) 021 : SM0,SM1,FL,8086 */
        {I_XOR, 2, {REG_AL,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+72469, 28},
    /* 345 : XOR reg_ax,imm16 [-i:       o16 35 iw                         ] SM0-1,FL,8086 */
        /* 320 001(35) 031 : SM0,SM1,FL,8086 */
        {I_XOR, 2, {REG_AX,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71363, 28},
    /* 346 : XOR reg_eax,imm32 [-i:       o32 35 id                         ] SM0-1,ZU,FL,386 */
        /* 321 001(35) 041 : SM0,SM1,ZU,FL,386 */
        {I_XOR, 2, {REG_EAX,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71368, 31},
    /* 347 : XOR reg_rax,sdword64 [-i:       o64 35 id,s                       ] SM0-1,ZU,LONG,FL,PROT,X86_64 */
        /* 324 001(35) 255 : SM0,SM1,ZU,LONG,FL,PROT,X86_64 */
        {I_XOR, 2, {REG_RAX,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+71373, 32},
    /* 348 : XOR rm8,imm8 [mi:       hle o8 80 /6 ib                   ] SM0-1,LOCK,FL,8086 */
        /* 273 001(80) 206 021 : SM0,SM1,LOCK,FL,8086 */
        {I_XOR, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66446, 25},
    /* 349 : XOR rm16,imm16 [mi:       hle o16 81 /6 iw                  ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(81) 206 031 : SM0,SM1,LOCK,FL,8086 */
        {I_XOR, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55024, 25},
    /* 350 : XOR rm32,imm32 [mi:       hle o32 81 /6 id                  ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(81) 206 041 : SM0,SM1,LOCK,FL,386 */
        {I_XOR, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+55031, 26},
    /* 351 : XOR rm64,sdword64 [mi:       hle o64 81 /6 id,s                ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(81) 206 255 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55038, 27},
    /* 352 : XOR reg8?,reg8,rm8 [vrm:      evex.nd1.nf.l0.m4.o8 32 /r        ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(32) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS8,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23706, 33},
    /* 353 : XOR reg8,rm8 [rm:      evex.nd0.nf.l0.m4.o8 32 /r        ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(32) 110 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23715, 34},
    /* 354 : XOR reg16?,reg16,rm16 [vrm:      evex.nd1.nf.l0.m4.o16 33 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(33) 121 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+10433, 36},
    /* 355 : XOR reg16,rm16 [rm:      evex.nd0.nf.l0.m4.o16 33 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(33) 110 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+10443, 37},
    /* 356 : XOR reg32?,reg32,rm32 [vrm:      evex.nd1.nf.l0.m4.o32 33 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(33) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+10463, 39},
    /* 357 : XOR reg32,rm32 [rm:      evex.nd0.nf.l0.m4.o32 33 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(33) 110 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+10473, 40},
    /* 358 : XOR reg64?,reg64,rm64 [vrm:      evex.nd1.nf.l0.m4.o64 33 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(33) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+10493, 39},
    /* 359 : XOR reg64,rm64 [rm:      evex.nd0.nf.l0.m4.o64 33 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(33) 110 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+10503, 40},
    /* 360 : XOR reg8?,rm8,reg8 [vmr:      evex.nd1.nf.l0.m4.o8 30 /r        ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(30) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23733, 33},
    /* 361 : XOR rm8,reg8 [mr:      evex.nd0.nf.l0.m4.o8 30 /r        ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(30) 101 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23742, 34},
    /* 362 : XOR reg16?,rm16,reg16 [vmr:      evex.nd1.nf.l0.m4.o16 31 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(31) 112 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+10523, 36},
    /* 363 : XOR rm16,reg16 [mr:      evex.nd0.nf.l0.m4.o16 31 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(31) 101 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+10533, 37},
    /* 364 : XOR reg32?,rm32,reg32 [vmr:      evex.nd1.nf.l0.m4.o32 31 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(31) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+10553, 39},
    /* 365 : XOR rm32,reg32 [mr:      evex.nd0.nf.l0.m4.o32 31 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(31) 101 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+10563, 40},
    /* 366 : XOR reg64?,rm64,reg64 [vmr:      evex.nd1.nf.l0.m4.o64 31 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(31) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+10583, 39},
    /* 367 : XOR rm64,reg64 [mr:      evex.nd0.nf.l0.m4.o64 31 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(31) 101 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+10593, 40},
    /* 368 : XOR reg16?,rm16,sbyteword16 [vmi:      evex.nd1.nf.l0.m4.o16 83 /6 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(83) 216 276 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+1212, 36},
    /* 369 : XOR rm16,sbyteword16 [mi:      evex.nd0.nf.l0.m4.o16 83 /6 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(83) 206 275 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+1223, 37},
    /* 370 : XOR reg32?,rm32,sbytedword32 [vmi:      evex.nd1.nf.l0.m4.o32 83 /6 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(83) 216 276 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+1245, 39},
    /* 371 : XOR rm32,sbytedword32 [mi:      evex.nd0.nf.l0.m4.o32 83 /6 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(83) 206 275 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+1256, 40},
    /* 372 : XOR reg64?,rm64,sbytedword64 [vmi:      evex.nd1.nf.l0.m4.o64 83 /6 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(83) 216 276 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+1278, 39},
    /* 373 : XOR rm64,sbytedword64 [mi:      evex.nd0.nf.l0.m4.o64 83 /6 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(83) 206 275 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+1289, 40},
    /* 374 : XOR reg8?,rm8,imm8 [vmi:      evex.nd1.nf.l0.m4.o8 80 /6 ib     ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(80) 216 022 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+10613, 33},
    /* 375 : XOR rm8,imm8 [mi:      evex.nd0.nf.l0.m4.o8 80 /6 ib     ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(80) 206 021 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+10623, 34},
    /* 376 : XOR reg16?,rm16,imm16 [vmi:      evex.nd1.nf.l0.m4.o16 81 /6 iw    ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(81) 216 032 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+1311, 36},
    /* 377 : XOR rm16,imm16 [mi:      evex.nd0.nf.l0.m4.o16 81 /6 iw    ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(81) 206 031 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+1322, 37},
    /* 378 : XOR reg32?,rm32,imm32 [vmi:      evex.nd1.nf.l0.m4.o32 81 /6 id    ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(81) 216 042 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+1344, 39},
    /* 379 : XOR rm32,imm32 [mi:      evex.nd0.nf.l0.m4.o32 81 /6 id    ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(81) 206 041 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+1355, 40},
    /* 380 : XOR reg64?,rm64,sdword64 [vmi:      evex.nd1.nf.l0.m4.o64 81 /6 id,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(81) 216 256 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+1377, 39},
    /* 381 : XOR rm64,sdword64 [mi:      evex.nd0.nf.l0.m4.o64 81 /6 id,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(81) 206 255 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+1388, 40},
    /* 382 : CMP rm8,reg8 [mr:       hle o8 38 /r                      ] SM0-1,LOCK,FL,8086 */
        /* 273 001(38) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_CMP, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+71378, 25},
    /* 383 : CMP rm16,reg16 [mr:       hle o16 39 /r                     ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(39) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_CMP, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66452, 25},
    /* 384 : CMP rm32,reg32 [mr:       hle o32 39 /r                     ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(39) 101 : SM0,SM1,LOCK,FL,386 */
        {I_CMP, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66458, 26},
    /* 385 : CMP rm64,reg64 [mr:       hle o64 39 /r                     ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(39) 101 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_CMP, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66464, 27},
    /* 386 : CMP reg8,rm8 [rm:       o8 3a /r                          ] SM0-1,FL,8086 */
        /* 001(3a) 110 : SM0,SM1,FL,8086 */
        {I_CMP, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+69622, 28},
    /* 387 : CMP reg16,rm16 [rm:       o16 3b /r                         ] SM0-1,FL,8086 */
        /* 320 001(3b) 110 : SM0,SM1,FL,8086 */
        {I_CMP, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71383, 28},
    /* 388 : CMP reg32,rm32 [rm:       o32 3b /r                         ] SM0-1,FL,386 */
        /* 321 001(3b) 110 : SM0,SM1,FL,386 */
        {I_CMP, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71388, 29},
    /* 389 : CMP reg64,rm64 [rm:       o64 3b /r                         ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 001(3b) 110 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_CMP, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+71393, 30},
    /* 390 : CMP rm16,sbyteword16 [mi:       hle o16 83 /7 ib,s                ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(83) 207 275 : SM0,SM1,LOCK,FL,8086 */
        {I_CMP, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55045, 25},
    /* 391 : CMP rm32,sbytedword32 [mi:       hle o32 83 /7 ib,s                ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(83) 207 275 : SM0,SM1,LOCK,FL,386 */
        {I_CMP, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55052, 26},
    /* 392 : CMP rm64,sbytedword64 [mi:       hle o64 83 /7 ib,s                ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(83) 207 275 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_CMP, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55059, 27},
    /* 393 : CMP reg_al,imm8 [-i:       o8 3c ib                          ] SM0-1,FL,8086 */
        /* 001(3c) 021 : SM0,SM1,FL,8086 */
        {I_CMP, 2, {REG_AL,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+72473, 28},
    /* 394 : CMP reg_ax,imm16 [-i:       o16 3d iw                         ] SM0-1,FL,8086 */
        /* 320 001(3d) 031 : SM0,SM1,FL,8086 */
        {I_CMP, 2, {REG_AX,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71398, 28},
    /* 395 : CMP reg_eax,imm32 [-i:       o32 3d id                         ] SM0-1,FL,386 */
        /* 321 001(3d) 041 : SM0,SM1,FL,386 */
        {I_CMP, 2, {REG_EAX,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71403, 29},
    /* 396 : CMP reg_rax,sdword64 [-i:       o64 3d id,s                       ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 001(3d) 255 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_CMP, 2, {REG_RAX,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+71408, 30},
    /* 397 : CMP rm8,imm8 [mi:       hle o8 80 /7 ib                   ] SM0-1,LOCK,FL,8086 */
        /* 273 001(80) 207 021 : SM0,SM1,LOCK,FL,8086 */
        {I_CMP, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66470, 25},
    /* 398 : CMP rm16,imm16 [mi:       hle o16 81 /7 iw                  ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(81) 207 031 : SM0,SM1,LOCK,FL,8086 */
        {I_CMP, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55066, 25},
    /* 399 : CMP rm32,imm32 [mi:       hle o32 81 /7 id                  ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(81) 207 041 : SM0,SM1,LOCK,FL,386 */
        {I_CMP, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+55073, 26},
    /* 400 : CMP rm64,sdword64 [mi:       hle o64 81 /7 id,s                ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(81) 207 255 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_CMP, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55080, 27},
    /* 401 : TEST rm8,reg8 [mr:       o8 84 /r                          ] SM0-1,FL,8086 */
        /* 001(84) 101 : SM0,SM1,FL,8086 */
        {I_TEST, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+72477, 28},
    /* 402 : TEST rm16,reg16 [mr:       o16 85 /r                         ] SM0-1,FL,8086 */
        /* 320 001(85) 101 : SM0,SM1,FL,8086 */
        {I_TEST, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71413, 28},
    /* 403 : TEST rm32,reg32 [mr:       o32 85 /r                         ] SM0-1,FL,386 */
        /* 321 001(85) 101 : SM0,SM1,FL,386 */
        {I_TEST, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71418, 29},
    /* 404 : TEST rm64,reg64 [mr:       o64 85 /r                         ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 001(85) 101 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_TEST, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+71423, 30},
    /* 405 : TEST reg_al,imm8 [-i:       o8 a8 ib                          ] SM0-1,NOAPX,FL,8086 */
        /* 001(a8) 021 : SM0,SM1,NOAPX,FL,8086 */
        {I_TEST, 2, {REG_AL|RN_L16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+72485, 51},
    /* 406 : TEST reg_ax,imm16 [-i:       o16 a9 iw                         ] SM0-1,NOAPX,FL,8086 */
        /* 320 001(a9) 031 : SM0,SM1,NOAPX,FL,8086 */
        {I_TEST, 2, {REG_AX|RN_L16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71443, 51},
    /* 407 : TEST reg_eax,imm32 [-i:       o32 a9 id                         ] SM0-1,NOAPX,FL,386 */
        /* 321 001(a9) 041 : SM0,SM1,NOAPX,FL,386 */
        {I_TEST, 2, {REG_EAX|RN_L16,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71448, 52},
    /* 408 : TEST reg_rax,sdword64 [-i:       o64 a9 id,s                       ] SM0-1,NOAPX,LONG,FL,PROT,X86_64 */
        /* 324 001(a9) 255 : SM0,SM1,NOAPX,LONG,FL,PROT,X86_64 */
        {I_TEST, 2, {REG_RAX|RN_L16,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+71453, 53},
    /* 409 : TEST rm8,imm8 [mi:       o8 f6 /0 ib                       ] SM0-1,FL,8086 */
        /* 001(f6) 200 021 : SM0,SM1,FL,8086 */
        {I_TEST, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+71458, 28},
    /* 410 : TEST rm16,imm16 [mi:       o16 f7 /0 iw                      ] SM0-1,FL,8086 */
        /* 320 001(f7) 200 031 : SM0,SM1,FL,8086 */
        {I_TEST, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66476, 28},
    /* 411 : TEST rm32,imm32 [mi:       o32 f7 /0 id                      ] SM0-1,FL,386 */
        /* 321 001(f7) 200 041 : SM0,SM1,FL,386 */
        {I_TEST, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66482, 29},
    /* 412 : TEST rm64,sdword64 [mi:       o64 f7 /0 id,s                    ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 001(f7) 200 255 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_TEST, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+66488, 30},
    /* 413 : ROL rm8,unity [m-:       o8 d0 /0                          ] FL,8086 */
        /* 001(d0) 200 : FL,8086 */
        {I_ROL, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23783, 54},
    /* 414 : ROL rm16,unity [m-:       o16 d1 /0                         ] FL,8086 */
        /* 320 001(d1) 200 : FL,8086 */
        {I_ROL, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10688, 54},
    /* 415 : ROL rm32,unity [m-:       o32 d1 /0                         ] FL,386 */
        /* 321 001(d1) 200 : FL,386 */
        {I_ROL, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10718, 55},
    /* 416 : ROL rm64,unity [m-:       o64 d1 /0                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d1) 200 : LONG,FL,PROT,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10748, 56},
    /* 417 : ROL rm8,reg_cl [m-:       o8 d2 /0                          ] FL,8086 */
        /* 001(d2) 200 : FL,8086 */
        {I_ROL, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23810, 54},
    /* 418 : ROL rm16,reg_cl [m-:       o16 d3 /0                         ] FL,8086 */
        /* 320 001(d3) 200 : FL,8086 */
        {I_ROL, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10778, 54},
    /* 419 : ROL rm32,reg_cl [m-:       o32 d3 /0                         ] FL,386 */
        /* 321 001(d3) 200 : FL,386 */
        {I_ROL, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10808, 55},
    /* 420 : ROL rm64,reg_cl [m-:       o64 d3 /0                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d3) 200 : LONG,FL,PROT,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10838, 56},
    /* 421 : ROL rm8,imm8 [mi:       o8 c0 /0 ib,u                     ] FL,186 */
        /* 001(c0) 200 025 : FL,186 */
        {I_ROL, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+10868, 57},
    /* 422 : ROL rm16,imm8 [mi:       o16 c1 /0 ib,u                    ] FL,186 */
        /* 320 001(c1) 200 025 : FL,186 */
        {I_ROL, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1437, 57},
    /* 423 : ROL rm32,imm8 [mi:       o32 c1 /0 ib,u                    ] FL,386 */
        /* 321 001(c1) 200 025 : FL,386 */
        {I_ROL, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1470, 55},
    /* 424 : ROL rm64,imm8 [mi:       o64 c1 /0 ib,u                    ] LONG,FL,PROT,X86_64 */
        /* 324 001(c1) 200 025 : LONG,FL,PROT,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1503, 56},
    /* 425 : ROR rm8,unity [m-:       o8 d0 /1                          ] FL,8086 */
        /* 001(d0) 201 : FL,8086 */
        {I_ROR, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23837, 54},
    /* 426 : ROR rm16,unity [m-:       o16 d1 /1                         ] FL,8086 */
        /* 320 001(d1) 201 : FL,8086 */
        {I_ROR, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10898, 54},
    /* 427 : ROR rm32,unity [m-:       o32 d1 /1                         ] FL,386 */
        /* 321 001(d1) 201 : FL,386 */
        {I_ROR, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10928, 55},
    /* 428 : ROR rm64,unity [m-:       o64 d1 /1                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d1) 201 : LONG,FL,PROT,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10958, 56},
    /* 429 : ROR rm8,reg_cl [m-:       o8 d2 /1                          ] FL,8086 */
        /* 001(d2) 201 : FL,8086 */
        {I_ROR, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23864, 54},
    /* 430 : ROR rm16,reg_cl [m-:       o16 d3 /1                         ] FL,8086 */
        /* 320 001(d3) 201 : FL,8086 */
        {I_ROR, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10988, 54},
    /* 431 : ROR rm32,reg_cl [m-:       o32 d3 /1                         ] FL,386 */
        /* 321 001(d3) 201 : FL,386 */
        {I_ROR, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11018, 55},
    /* 432 : ROR rm64,reg_cl [m-:       o64 d3 /1                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d3) 201 : LONG,FL,PROT,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11048, 56},
    /* 433 : ROR rm8,imm8 [mi:       o8 c0 /1 ib,u                     ] FL,186 */
        /* 001(c0) 201 025 : FL,186 */
        {I_ROR, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+11078, 57},
    /* 434 : ROR rm16,imm8 [mi:       o16 c1 /1 ib,u                    ] FL,186 */
        /* 320 001(c1) 201 025 : FL,186 */
        {I_ROR, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1536, 57},
    /* 435 : ROR rm32,imm8 [mi:       o32 c1 /1 ib,u                    ] FL,386 */
        /* 321 001(c1) 201 025 : FL,386 */
        {I_ROR, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1569, 55},
    /* 436 : ROR rm64,imm8 [mi:       o64 c1 /1 ib,u                    ] LONG,FL,PROT,X86_64 */
        /* 324 001(c1) 201 025 : LONG,FL,PROT,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1602, 56},
    /* 437 : RCL rm8,unity [m-:       o8 d0 /2                          ] FL,8086 */
        /* 001(d0) 202 : FL,8086 */
        {I_RCL, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23891, 54},
    /* 438 : RCL rm16,unity [m-:       o16 d1 /2                         ] FL,8086 */
        /* 320 001(d1) 202 : FL,8086 */
        {I_RCL, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11108, 54},
    /* 439 : RCL rm32,unity [m-:       o32 d1 /2                         ] FL,386 */
        /* 321 001(d1) 202 : FL,386 */
        {I_RCL, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11138, 55},
    /* 440 : RCL rm64,unity [m-:       o64 d1 /2                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d1) 202 : LONG,FL,PROT,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11168, 56},
    /* 441 : RCL rm8,reg_cl [m-:       o8 d2 /2                          ] FL,8086 */
        /* 001(d2) 202 : FL,8086 */
        {I_RCL, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23918, 54},
    /* 442 : RCL rm16,reg_cl [m-:       o16 d3 /2                         ] FL,8086 */
        /* 320 001(d3) 202 : FL,8086 */
        {I_RCL, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11198, 54},
    /* 443 : RCL rm32,reg_cl [m-:       o32 d3 /2                         ] FL,386 */
        /* 321 001(d3) 202 : FL,386 */
        {I_RCL, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11228, 55},
    /* 444 : RCL rm64,reg_cl [m-:       o64 d3 /2                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d3) 202 : LONG,FL,PROT,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11258, 56},
    /* 445 : RCL rm8,imm8 [mi:       o8 c0 /2 ib,u                     ] FL,186 */
        /* 001(c0) 202 025 : FL,186 */
        {I_RCL, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+11288, 57},
    /* 446 : RCL rm16,imm8 [mi:       o16 c1 /2 ib,u                    ] FL,186 */
        /* 320 001(c1) 202 025 : FL,186 */
        {I_RCL, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1635, 57},
    /* 447 : RCL rm32,imm8 [mi:       o32 c1 /2 ib,u                    ] FL,386 */
        /* 321 001(c1) 202 025 : FL,386 */
        {I_RCL, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1668, 55},
    /* 448 : RCL rm64,imm8 [mi:       o64 c1 /2 ib,u                    ] LONG,FL,PROT,X86_64 */
        /* 324 001(c1) 202 025 : LONG,FL,PROT,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1701, 56},
    /* 449 : RCR rm8,unity [m-:       o8 d0 /3                          ] FL,8086 */
        /* 001(d0) 203 : FL,8086 */
        {I_RCR, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23945, 54},
    /* 450 : RCR rm16,unity [m-:       o16 d1 /3                         ] FL,8086 */
        /* 320 001(d1) 203 : FL,8086 */
        {I_RCR, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11318, 54},
    /* 451 : RCR rm32,unity [m-:       o32 d1 /3                         ] FL,386 */
        /* 321 001(d1) 203 : FL,386 */
        {I_RCR, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11348, 55},
    /* 452 : RCR rm64,unity [m-:       o64 d1 /3                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d1) 203 : LONG,FL,PROT,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11378, 56},
    /* 453 : RCR rm8,reg_cl [m-:       o8 d2 /3                          ] FL,8086 */
        /* 001(d2) 203 : FL,8086 */
        {I_RCR, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23972, 54},
    /* 454 : RCR rm16,reg_cl [m-:       o16 d3 /3                         ] FL,8086 */
        /* 320 001(d3) 203 : FL,8086 */
        {I_RCR, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11408, 54},
    /* 455 : RCR rm32,reg_cl [m-:       o32 d3 /3                         ] FL,386 */
        /* 321 001(d3) 203 : FL,386 */
        {I_RCR, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11438, 55},
    /* 456 : RCR rm64,reg_cl [m-:       o64 d3 /3                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d3) 203 : LONG,FL,PROT,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11468, 56},
    /* 457 : RCR rm8,imm8 [mi:       o8 c0 /3 ib,u                     ] FL,186 */
        /* 001(c0) 203 025 : FL,186 */
        {I_RCR, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+11498, 57},
    /* 458 : RCR rm16,imm8 [mi:       o16 c1 /3 ib,u                    ] FL,186 */
        /* 320 001(c1) 203 025 : FL,186 */
        {I_RCR, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1734, 57},
    /* 459 : RCR rm32,imm8 [mi:       o32 c1 /3 ib,u                    ] FL,386 */
        /* 321 001(c1) 203 025 : FL,386 */
        {I_RCR, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1767, 55},
    /* 460 : RCR rm64,imm8 [mi:       o64 c1 /3 ib,u                    ] LONG,FL,PROT,X86_64 */
        /* 324 001(c1) 203 025 : LONG,FL,PROT,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1800, 56},
    /* 461 : SHL rm8,unity [m-:       o8 d0 /4                          ] FL,8086 */
        /* 001(d0) 204 : FL,8086 */
        {I_SHL, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23999, 54},
    /* 462 : SHL rm16,unity [m-:       o16 d1 /4                         ] FL,8086 */
        /* 320 001(d1) 204 : FL,8086 */
        {I_SHL, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11528, 54},
    /* 463 : SHL rm32,unity [m-:       o32 d1 /4                         ] FL,386 */
        /* 321 001(d1) 204 : FL,386 */
        {I_SHL, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11558, 55},
    /* 464 : SHL rm64,unity [m-:       o64 d1 /4                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d1) 204 : LONG,FL,PROT,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11588, 56},
    /* 465 : SHL rm8,reg_cl [m-:       o8 d2 /4                          ] FL,8086 */
        /* 001(d2) 204 : FL,8086 */
        {I_SHL, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+24026, 54},
    /* 466 : SHL rm16,reg_cl [m-:       o16 d3 /4                         ] FL,8086 */
        /* 320 001(d3) 204 : FL,8086 */
        {I_SHL, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11618, 54},
    /* 467 : SHL rm32,reg_cl [m-:       o32 d3 /4                         ] FL,386 */
        /* 321 001(d3) 204 : FL,386 */
        {I_SHL, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11648, 55},
    /* 468 : SHL rm64,reg_cl [m-:       o64 d3 /4                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d3) 204 : LONG,FL,PROT,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11678, 56},
    /* 469 : SHL rm8,imm8 [mi:       o8 c0 /4 ib,u                     ] FL,186 */
        /* 001(c0) 204 025 : FL,186 */
        {I_SHL, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+11708, 57},
    /* 470 : SHL rm16,imm8 [mi:       o16 c1 /4 ib,u                    ] FL,186 */
        /* 320 001(c1) 204 025 : FL,186 */
        {I_SHL, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1833, 57},
    /* 471 : SHL rm32,imm8 [mi:       o32 c1 /4 ib,u                    ] FL,386 */
        /* 321 001(c1) 204 025 : FL,386 */
        {I_SHL, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1866, 55},
    /* 472 : SHL rm64,imm8 [mi:       o64 c1 /4 ib,u                    ] LONG,FL,PROT,X86_64 */
        /* 324 001(c1) 204 025 : LONG,FL,PROT,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1899, 56},
    /* 473 : SAL rm8,unity [m-:       o8 d0 /4                          ] FL,8086 */
        /* 001(d0) 204 : FL,8086 */
        {I_SAL, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23999, 54},
    /* 474 : SAL rm16,unity [m-:       o16 d1 /4                         ] FL,8086 */
        /* 320 001(d1) 204 : FL,8086 */
        {I_SAL, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11528, 54},
    /* 475 : SAL rm32,unity [m-:       o32 d1 /4                         ] FL,386 */
        /* 321 001(d1) 204 : FL,386 */
        {I_SAL, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11558, 55},
    /* 476 : SAL rm64,unity [m-:       o64 d1 /4                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d1) 204 : LONG,FL,PROT,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11588, 56},
    /* 477 : SAL rm8,reg_cl [m-:       o8 d2 /4                          ] FL,8086 */
        /* 001(d2) 204 : FL,8086 */
        {I_SAL, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+24026, 54},
    /* 478 : SAL rm16,reg_cl [m-:       o16 d3 /4                         ] FL,8086 */
        /* 320 001(d3) 204 : FL,8086 */
        {I_SAL, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11618, 54},
    /* 479 : SAL rm32,reg_cl [m-:       o32 d3 /4                         ] FL,386 */
        /* 321 001(d3) 204 : FL,386 */
        {I_SAL, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11648, 55},
    /* 480 : SAL rm64,reg_cl [m-:       o64 d3 /4                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d3) 204 : LONG,FL,PROT,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11678, 56},
    /* 481 : SAL rm8,imm8 [mi:       o8 c0 /4 ib,u                     ] FL,186 */
        /* 001(c0) 204 025 : FL,186 */
        {I_SAL, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+11708, 57},
    /* 482 : SAL rm16,imm8 [mi:       o16 c1 /4 ib,u                    ] FL,186 */
        /* 320 001(c1) 204 025 : FL,186 */
        {I_SAL, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1833, 57},
    /* 483 : SAL rm32,imm8 [mi:       o32 c1 /4 ib,u                    ] FL,386 */
        /* 321 001(c1) 204 025 : FL,386 */
        {I_SAL, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1866, 55},
    /* 484 : SAL rm64,imm8 [mi:       o64 c1 /4 ib,u                    ] LONG,FL,PROT,X86_64 */
        /* 324 001(c1) 204 025 : LONG,FL,PROT,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1899, 56},
    /* 485 : SHR rm8,unity [m-:       o8 d0 /5                          ] FL,8086 */
        /* 001(d0) 205 : FL,8086 */
        {I_SHR, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+24053, 54},
    /* 486 : SHR rm16,unity [m-:       o16 d1 /5                         ] FL,8086 */
        /* 320 001(d1) 205 : FL,8086 */
        {I_SHR, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11738, 54},
    /* 487 : SHR rm32,unity [m-:       o32 d1 /5                         ] FL,386 */
        /* 321 001(d1) 205 : FL,386 */
        {I_SHR, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11768, 55},
    /* 488 : SHR rm64,unity [m-:       o64 d1 /5                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d1) 205 : LONG,FL,PROT,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11798, 56},
    /* 489 : SHR rm8,reg_cl [m-:       o8 d2 /5                          ] FL,8086 */
        /* 001(d2) 205 : FL,8086 */
        {I_SHR, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+24080, 54},
    /* 490 : SHR rm16,reg_cl [m-:       o16 d3 /5                         ] FL,8086 */
        /* 320 001(d3) 205 : FL,8086 */
        {I_SHR, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11828, 54},
    /* 491 : SHR rm32,reg_cl [m-:       o32 d3 /5                         ] FL,386 */
        /* 321 001(d3) 205 : FL,386 */
        {I_SHR, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11858, 55},
    /* 492 : SHR rm64,reg_cl [m-:       o64 d3 /5                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d3) 205 : LONG,FL,PROT,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11888, 56},
    /* 493 : SHR rm8,imm8 [mi:       o8 c0 /5 ib,u                     ] FL,186 */
        /* 001(c0) 205 025 : FL,186 */
        {I_SHR, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+11918, 57},
    /* 494 : SHR rm16,imm8 [mi:       o16 c1 /5 ib,u                    ] FL,186 */
        /* 320 001(c1) 205 025 : FL,186 */
        {I_SHR, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1932, 57},
    /* 495 : SHR rm32,imm8 [mi:       o32 c1 /5 ib,u                    ] FL,386 */
        /* 321 001(c1) 205 025 : FL,386 */
        {I_SHR, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1965, 55},
    /* 496 : SHR rm64,imm8 [mi:       o64 c1 /5 ib,u                    ] LONG,FL,PROT,X86_64 */
        /* 324 001(c1) 205 025 : LONG,FL,PROT,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1998, 56},
    /* 497 : SAR rm8,unity [m-:       o8 d0 /7                          ] FL,8086 */
        /* 001(d0) 207 : FL,8086 */
        {I_SAR, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+24107, 54},
    /* 498 : SAR rm16,unity [m-:       o16 d1 /7                         ] FL,8086 */
        /* 320 001(d1) 207 : FL,8086 */
        {I_SAR, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11948, 54},
    /* 499 : SAR rm32,unity [m-:       o32 d1 /7                         ] FL,386 */
        /* 321 001(d1) 207 : FL,386 */
        {I_SAR, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11978, 55},
    /* 500 : SAR rm64,unity [m-:       o64 d1 /7                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d1) 207 : LONG,FL,PROT,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+12008, 56},
    /* 501 : SAR rm8,reg_cl [m-:       o8 d2 /7                          ] FL,8086 */
        /* 001(d2) 207 : FL,8086 */
        {I_SAR, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+24134, 54},
    /* 502 : SAR rm16,reg_cl [m-:       o16 d3 /7                         ] FL,8086 */
        /* 320 001(d3) 207 : FL,8086 */
        {I_SAR, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+12038, 54},
    /* 503 : SAR rm32,reg_cl [m-:       o32 d3 /7                         ] FL,386 */
        /* 321 001(d3) 207 : FL,386 */
        {I_SAR, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+12068, 55},
    /* 504 : SAR rm64,reg_cl [m-:       o64 d3 /7                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d3) 207 : LONG,FL,PROT,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+12098, 56},
    /* 505 : SAR rm8,imm8 [mi:       o8 c0 /7 ib,u                     ] FL,186 */
        /* 001(c0) 207 025 : FL,186 */
        {I_SAR, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+12128, 57},
    /* 506 : SAR rm16,imm8 [mi:       o16 c1 /7 ib,u                    ] FL,186 */
        /* 320 001(c1) 207 025 : FL,186 */
        {I_SAR, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+2031, 57},
    /* 507 : SAR rm32,imm8 [mi:       o32 c1 /7 ib,u                    ] FL,386 */
        /* 321 001(c1) 207 025 : FL,386 */
        {I_SAR, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+2064, 55},
    /* 508 : SAR rm64,imm8 [mi:       o64 c1 /7 ib,u                    ] LONG,FL,PROT,X86_64 */
        /* 324 001(c1) 207 025 : LONG,FL,PROT,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+2097, 56},
    /* 509 : RORX reg32,rm32,imm8 [rmi:      vex.lz.f2.0f3a.w0 f0 /r ib        ] SM0-1,BMI2,FUTURE */
        /* 270(003 003) 001(f0) 110 022 : SM0,SM1,NOAPX,VEX,BMI2,FUTURE */
        {I_RORX, 3, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51093, 58},
    /* 510 : SHLX reg32,rm32*,reg32 [rmv:      vex.lz.66.0f38.w0 f7 /r           ] SM0,SM1,BMI2,FUTURE */
        /* 262(002 001) 001(f7) 110 : SM0,SM1,NOAPX,VEX,BMI2,FUTURE */
        {I_SHLX, 3, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55087, 58},
    /* 511 : SARX reg32,rm32*,reg32 [rmv:      vex.lz.f3.0f38.w0 f7 /r           ] SM0,SM1,BMI2,FUTURE */
        /* 262(002 002) 001(f7) 110 : SM0,SM1,NOAPX,VEX,BMI2,FUTURE */
        {I_SARX, 3, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55101, 58},
    /* 512 : SHRX reg32,rm32*,reg32 [rmv:      vex.lz.f2.0f38.w0 f7 /r           ] SM0,SM1,BMI2,FUTURE */
        /* 262(002 003) 001(f7) 110 : SM0,SM1,NOAPX,VEX,BMI2,FUTURE */
        {I_SHRX, 3, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55115, 58},
    /* 513 : RORX reg64,rm64,imm8 [rmi:      vex.lz.f2.0f3a.w1 f0 /r ib        ] SM0-1,BMI2,FUTURE */
        /* 270(003 203) 001(f0) 110 022 : SM0,SM1,NOAPX,VEX,BMI2,FUTURE */
        {I_RORX, 3, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51101, 58},
    /* 514 : SHLX reg64,rm64*,reg64 [rmv:      vex.lz.66.0f38.w1 f7 /r           ] SM0,SM1,BMI2,FUTURE */
        /* 262(002 201) 001(f7) 110 : SM0,SM1,NOAPX,VEX,BMI2,FUTURE */
        {I_SHLX, 3, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55129, 58},
    /* 515 : SARX reg64,rm64*,reg64 [rmv:      vex.lz.f3.0f38.w1 f7 /r           ] SM0,SM1,BMI2,FUTURE */
        /* 262(002 202) 001(f7) 110 : SM0,SM1,NOAPX,VEX,BMI2,FUTURE */
        {I_SARX, 3, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55143, 58},
    /* 516 : SHRX reg64,rm64*,reg64 [rmv:      vex.lz.f2.0f38.w1 f7 /r           ] SM0,SM1,BMI2,FUTURE */
        /* 262(002 203) 001(f7) 110 : SM0,SM1,NOAPX,VEX,BMI2,FUTURE */
        {I_SHRX, 3, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55157, 58},
    /* 517 : ROL reg8?,rm8,unity [vm-:      evex.nd1.nf.l0.m4.o8 d0 /0        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d0) 210 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 3, {REG_GPR|BITS8,RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+23760, 35},
    /* 518 : ROL rm8,unity [m-:      evex.nd0.nf.l0.m4.o8 d0 /0        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d0) 200 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23769, 62},
    /* 519 : ROL reg16?,rm16,unity [vm-:      evex.nd1.nf.l0.m4.o16 d1 /0       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d1) 210 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+10663, 38},
    /* 520 : ROL rm16,unity [m-:      evex.nd0.nf.l0.m4.o16 d1 /0       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d1) 200 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10673, 64},
    /* 521 : ROL reg32?,rm32,unity [vm-:      evex.nd1.nf.l0.m4.o32 d1 /0       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d1) 210 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+10693, 41},
    /* 522 : ROL rm32,unity [m-:      evex.nd0.nf.l0.m4.o32 d1 /0       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d1) 200 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10703, 66},
    /* 523 : ROL reg64?,rm64,unity [vm-:      evex.nd1.nf.l0.m4.o64 d1 /0       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d1) 210 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+10723, 41},
    /* 524 : ROL rm64,unity [m-:      evex.nd0.nf.l0.m4.o64 d1 /0       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d1) 200 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10733, 66},
    /* 525 : ROL reg8?,rm8,reg_cl [vm-:      evex.nd1.nf.l0.m4.o8 d2 /0        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d2) 210 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+23787, 35},
    /* 526 : ROL rm8,reg_cl [m-:      evex.nd0.nf.l0.m4.o8 d2 /0        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d2) 200 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23796, 62},
    /* 527 : ROL reg16?,rm16,reg_cl [vm-:      evex.nd1.nf.l0.m4.o16 d3 /0       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d3) 210 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+10753, 38},
    /* 528 : ROL rm16,reg_cl [m-:      evex.nd0.nf.l0.m4.o16 d3 /0       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d3) 200 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10763, 64},
    /* 529 : ROL reg32?,rm32,reg_cl [vm-:      evex.nd1.nf.l0.m4.o32 d3 /0       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d3) 210 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+10783, 41},
    /* 530 : ROL rm32,reg_cl [m-:      evex.nd0.nf.l0.m4.o32 d3 /0       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d3) 200 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10793, 66},
    /* 531 : ROL reg64?,rm64,reg_cl [vm-:      evex.nd1.nf.l0.m4.o64 d3 /0       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d3) 210 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+10813, 41},
    /* 532 : ROL rm64,reg_cl [m-:      evex.nd0.nf.l0.m4.o64 d3 /0       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d3) 200 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10823, 66},
    /* 533 : ROL reg8?,rm8,imm8 [vmi:      evex.nd1.nf.l0.m4.o8 c0 /0 ib,u   ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(c0) 210 026 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+10843, 35},
    /* 534 : ROL rm8,imm8 [mi:      evex.nd0.nf.l0.m4.o8 c0 /0 ib,u   ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(c0) 200 025 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+10853, 62},
    /* 535 : ROL reg16?,rm16,imm8 [vmi:      evex.nd1.nf.l0.m4.o16 c1 /0 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(c1) 210 026 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1410, 38},
    /* 536 : ROL rm16,imm8 [mi:      evex.nd0.nf.l0.m4.o16 c1 /0 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(c1) 200 025 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1421, 64},
    /* 537 : ROL reg32?,rm32,imm8 [vmi:      evex.nd1.nf.l0.m4.o32 c1 /0 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(c1) 210 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1443, 41},
    /* 538 : ROL rm32,imm8 [mi:      evex.nd0.nf.l0.m4.o32 c1 /0 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(c1) 200 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1454, 66},
    /* 539 : ROL reg64?,rm64,imm8 [vmi:      evex.nd1.nf.l0.m4.o64 c1 /0 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(c1) 210 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1476, 41},
    /* 540 : ROL rm64,imm8 [mi:      evex.nd0.nf.l0.m4.o64 c1 /0 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(c1) 200 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1487, 66},
    /* 541 : ROR reg8?,rm8,unity [vm-:      evex.nd1.nf.l0.m4.o8 d0 /1        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d0) 211 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+23814, 35},
    /* 542 : ROR rm8,unity [m-:      evex.nd0.nf.l0.m4.o8 d0 /1        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d0) 201 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23823, 62},
    /* 543 : ROR reg16?,rm16,unity [vm-:      evex.nd1.nf.l0.m4.o16 d1 /1       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d1) 211 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+10873, 38},
    /* 544 : ROR rm16,unity [m-:      evex.nd0.nf.l0.m4.o16 d1 /1       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d1) 201 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10883, 64},
    /* 545 : ROR reg32?,rm32,unity [vm-:      evex.nd1.nf.l0.m4.o32 d1 /1       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d1) 211 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+10903, 41},
    /* 546 : ROR rm32,unity [m-:      evex.nd0.nf.l0.m4.o32 d1 /1       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d1) 201 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10913, 66},
    /* 547 : ROR reg64?,rm64,unity [vm-:      evex.nd1.nf.l0.m4.o64 d1 /1       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d1) 211 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+10933, 41},
    /* 548 : ROR rm64,unity [m-:      evex.nd0.nf.l0.m4.o64 d1 /1       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d1) 201 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10943, 66},
    /* 549 : ROR reg8?,rm8,reg_cl [vm-:      evex.nd1.nf.l0.m4.o8 d2 /1        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d2) 211 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+23841, 35},
    /* 550 : ROR rm8,reg_cl [m-:      evex.nd0.nf.l0.m4.o8 d2 /1        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d2) 201 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23850, 62},
    /* 551 : ROR reg16?,rm16,reg_cl [vm-:      evex.nd1.nf.l0.m4.o16 d3 /1       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d3) 211 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+10963, 38},
    /* 552 : ROR rm16,reg_cl [m-:      evex.nd0.nf.l0.m4.o16 d3 /1       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d3) 201 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10973, 64},
    /* 553 : ROR reg32?,rm32,reg_cl [vm-:      evex.nd1.nf.l0.m4.o32 d3 /1       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d3) 211 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+10993, 41},
    /* 554 : ROR rm32,reg_cl [m-:      evex.nd0.nf.l0.m4.o32 d3 /1       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d3) 201 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11003, 66},
    /* 555 : ROR reg64?,rm64,reg_cl [vm-:      evex.nd1.nf.l0.m4.o64 d3 /1       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d3) 211 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11023, 41},
    /* 556 : ROR rm64,reg_cl [m-:      evex.nd0.nf.l0.m4.o64 d3 /1       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d3) 201 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11033, 66},
    /* 557 : ROR reg8?,rm8,imm8 [vmi:      evex.nd1.nf.l0.m4.o8 c0 /1 ib,u   ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(c0) 211 026 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+11053, 35},
    /* 558 : ROR rm8,imm8 [mi:      evex.nd0.nf.l0.m4.o8 c0 /1 ib,u   ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(c0) 201 025 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+11063, 62},
    /* 559 : ROR reg16?,rm16,imm8 [vmi:      evex.nd1.nf.l0.m4.o16 c1 /1 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(c1) 211 026 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1509, 38},
    /* 560 : ROR rm16,imm8 [mi:      evex.nd0.nf.l0.m4.o16 c1 /1 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(c1) 201 025 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1520, 64},
    /* 561 : ROR reg32?,rm32,imm8 [vmi:      evex.nd1.nf.l0.m4.o32 c1 /1 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(c1) 211 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1542, 41},
    /* 562 : ROR rm32,imm8 [mi:      evex.nd0.nf.l0.m4.o32 c1 /1 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(c1) 201 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1553, 66},
    /* 563 : ROR reg64?,rm64,imm8 [vmi:      evex.nd1.nf.l0.m4.o64 c1 /1 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(c1) 211 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1575, 41},
    /* 564 : ROR rm64,imm8 [mi:      evex.nd0.nf.l0.m4.o64 c1 /1 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(c1) 201 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1586, 66},
    /* 565 : RCL reg8?,rm8,unity [vm-:      evex.nd1.nf.l0.m4.o8 d0 /2        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d0) 212 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 3, {REG_GPR|BITS8,RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+23868, 35},
    /* 566 : RCL rm8,unity [m-:      evex.nd0.nf.l0.m4.o8 d0 /2        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d0) 202 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23877, 62},
    /* 567 : RCL reg16?,rm16,unity [vm-:      evex.nd1.nf.l0.m4.o16 d1 /2       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d1) 212 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11083, 38},
    /* 568 : RCL rm16,unity [m-:      evex.nd0.nf.l0.m4.o16 d1 /2       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d1) 202 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11093, 64},
    /* 569 : RCL reg32?,rm32,unity [vm-:      evex.nd1.nf.l0.m4.o32 d1 /2       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d1) 212 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11113, 41},
    /* 570 : RCL rm32,unity [m-:      evex.nd0.nf.l0.m4.o32 d1 /2       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d1) 202 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11123, 66},
    /* 571 : RCL reg64?,rm64,unity [vm-:      evex.nd1.nf.l0.m4.o64 d1 /2       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d1) 212 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11143, 41},
    /* 572 : RCL rm64,unity [m-:      evex.nd0.nf.l0.m4.o64 d1 /2       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d1) 202 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11153, 66},
    /* 573 : RCL reg8?,rm8,reg_cl [vm-:      evex.nd1.nf.l0.m4.o8 d2 /2        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d2) 212 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+23895, 35},
    /* 574 : RCL rm8,reg_cl [m-:      evex.nd0.nf.l0.m4.o8 d2 /2        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d2) 202 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23904, 62},
    /* 575 : RCL reg16?,rm16,reg_cl [vm-:      evex.nd1.nf.l0.m4.o16 d3 /2       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d3) 212 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11173, 38},
    /* 576 : RCL rm16,reg_cl [m-:      evex.nd0.nf.l0.m4.o16 d3 /2       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d3) 202 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11183, 64},
    /* 577 : RCL reg32?,rm32,reg_cl [vm-:      evex.nd1.nf.l0.m4.o32 d3 /2       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d3) 212 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11203, 41},
    /* 578 : RCL rm32,reg_cl [m-:      evex.nd0.nf.l0.m4.o32 d3 /2       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d3) 202 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11213, 66},
    /* 579 : RCL reg64?,rm64,reg_cl [vm-:      evex.nd1.nf.l0.m4.o64 d3 /2       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d3) 212 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11233, 41},
    /* 580 : RCL rm64,reg_cl [m-:      evex.nd0.nf.l0.m4.o64 d3 /2       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d3) 202 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11243, 66},
    /* 581 : RCL reg8?,rm8,imm8 [vmi:      evex.nd1.nf.l0.m4.o8 c0 /2 ib,u   ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(c0) 212 026 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+11263, 35},
    /* 582 : RCL rm8,imm8 [mi:      evex.nd0.nf.l0.m4.o8 c0 /2 ib,u   ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(c0) 202 025 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+11273, 62},
    /* 583 : RCL reg16?,rm16,imm8 [vmi:      evex.nd1.nf.l0.m4.o16 c1 /2 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(c1) 212 026 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1608, 38},
    /* 584 : RCL rm16,imm8 [mi:      evex.nd0.nf.l0.m4.o16 c1 /2 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(c1) 202 025 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1619, 64},
    /* 585 : RCL reg32?,rm32,imm8 [vmi:      evex.nd1.nf.l0.m4.o32 c1 /2 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(c1) 212 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1641, 41},
    /* 586 : RCL rm32,imm8 [mi:      evex.nd0.nf.l0.m4.o32 c1 /2 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(c1) 202 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1652, 66},
    /* 587 : RCL reg64?,rm64,imm8 [vmi:      evex.nd1.nf.l0.m4.o64 c1 /2 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(c1) 212 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1674, 41},
    /* 588 : RCL rm64,imm8 [mi:      evex.nd0.nf.l0.m4.o64 c1 /2 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(c1) 202 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1685, 66},
    /* 589 : RCR reg8?,rm8,unity [vm-:      evex.nd1.nf.l0.m4.o8 d0 /3        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d0) 213 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+23922, 35},
    /* 590 : RCR rm8,unity [m-:      evex.nd0.nf.l0.m4.o8 d0 /3        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d0) 203 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23931, 62},
    /* 591 : RCR reg16?,rm16,unity [vm-:      evex.nd1.nf.l0.m4.o16 d1 /3       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d1) 213 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11293, 38},
    /* 592 : RCR rm16,unity [m-:      evex.nd0.nf.l0.m4.o16 d1 /3       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d1) 203 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11303, 64},
    /* 593 : RCR reg32?,rm32,unity [vm-:      evex.nd1.nf.l0.m4.o32 d1 /3       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d1) 213 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11323, 41},
    /* 594 : RCR rm32,unity [m-:      evex.nd0.nf.l0.m4.o32 d1 /3       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d1) 203 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11333, 66},
    /* 595 : RCR reg64?,rm64,unity [vm-:      evex.nd1.nf.l0.m4.o64 d1 /3       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d1) 213 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11353, 41},
    /* 596 : RCR rm64,unity [m-:      evex.nd0.nf.l0.m4.o64 d1 /3       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d1) 203 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11363, 66},
    /* 597 : RCR reg8?,rm8,reg_cl [vm-:      evex.nd1.nf.l0.m4.o8 d2 /3        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d2) 213 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+23949, 35},
    /* 598 : RCR rm8,reg_cl [m-:      evex.nd0.nf.l0.m4.o8 d2 /3        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d2) 203 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23958, 62},
    /* 599 : RCR reg16?,rm16,reg_cl [vm-:      evex.nd1.nf.l0.m4.o16 d3 /3       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d3) 213 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11383, 38},
    /* 600 : RCR rm16,reg_cl [m-:      evex.nd0.nf.l0.m4.o16 d3 /3       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d3) 203 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11393, 64},
    /* 601 : RCR reg32?,rm32,reg_cl [vm-:      evex.nd1.nf.l0.m4.o32 d3 /3       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d3) 213 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11413, 41},
    /* 602 : RCR rm32,reg_cl [m-:      evex.nd0.nf.l0.m4.o32 d3 /3       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d3) 203 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11423, 66},
    /* 603 : RCR reg64?,rm64,reg_cl [vm-:      evex.nd1.nf.l0.m4.o64 d3 /3       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d3) 213 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11443, 41},
    /* 604 : RCR rm64,reg_cl [m-:      evex.nd0.nf.l0.m4.o64 d3 /3       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d3) 203 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11453, 66},
    /* 605 : RCR reg8?,rm8,imm8 [vmi:      evex.nd1.nf.l0.m4.o8 c0 /3 ib,u   ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(c0) 213 026 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+11473, 35},
    /* 606 : RCR rm8,imm8 [mi:      evex.nd0.nf.l0.m4.o8 c0 /3 ib,u   ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(c0) 203 025 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+11483, 62},
    /* 607 : RCR reg16?,rm16,imm8 [vmi:      evex.nd1.nf.l0.m4.o16 c1 /3 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(c1) 213 026 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1707, 38},
    /* 608 : RCR rm16,imm8 [mi:      evex.nd0.nf.l0.m4.o16 c1 /3 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(c1) 203 025 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1718, 64},
    /* 609 : RCR reg32?,rm32,imm8 [vmi:      evex.nd1.nf.l0.m4.o32 c1 /3 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(c1) 213 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1740, 41},
    /* 610 : RCR rm32,imm8 [mi:      evex.nd0.nf.l0.m4.o32 c1 /3 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(c1) 203 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1751, 66},
    /* 611 : RCR reg64?,rm64,imm8 [vmi:      evex.nd1.nf.l0.m4.o64 c1 /3 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(c1) 213 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1773, 41},
    /* 612 : RCR rm64,imm8 [mi:      evex.nd0.nf.l0.m4.o64 c1 /3 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(c1) 203 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1784, 66},
    /* 613 : SHL reg8?,rm8,unity [vm-:      evex.nd1.nf.l0.m4.o8 d0 /4        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d0) 214 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 3, {REG_GPR|BITS8,RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+23976, 35},
    /* 614 : SHL rm8,unity [m-:      evex.nd0.nf.l0.m4.o8 d0 /4        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d0) 204 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23985, 62},
    /* 615 : SHL reg16?,rm16,unity [vm-:      evex.nd1.nf.l0.m4.o16 d1 /4       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d1) 214 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11503, 38},
    /* 616 : SHL rm16,unity [m-:      evex.nd0.nf.l0.m4.o16 d1 /4       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d1) 204 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11513, 64},
    /* 617 : SHL reg32?,rm32,unity [vm-:      evex.nd1.nf.l0.m4.o32 d1 /4       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d1) 214 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11533, 41},
    /* 618 : SHL rm32,unity [m-:      evex.nd0.nf.l0.m4.o32 d1 /4       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d1) 204 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11543, 66},
    /* 619 : SHL reg64?,rm64,unity [vm-:      evex.nd1.nf.l0.m4.o64 d1 /4       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d1) 214 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11563, 41},
    /* 620 : SHL rm64,unity [m-:      evex.nd0.nf.l0.m4.o64 d1 /4       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d1) 204 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11573, 66},
    /* 621 : SHL reg8?,rm8,reg_cl [vm-:      evex.nd1.nf.l0.m4.o8 d2 /4        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d2) 214 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+24003, 35},
    /* 622 : SHL rm8,reg_cl [m-:      evex.nd0.nf.l0.m4.o8 d2 /4        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d2) 204 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+24012, 62},
    /* 623 : SHL reg16?,rm16,reg_cl [vm-:      evex.nd1.nf.l0.m4.o16 d3 /4       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d3) 214 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11593, 38},
    /* 624 : SHL rm16,reg_cl [m-:      evex.nd0.nf.l0.m4.o16 d3 /4       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d3) 204 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11603, 64},
    /* 625 : SHL reg32?,rm32,reg_cl [vm-:      evex.nd1.nf.l0.m4.o32 d3 /4       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d3) 214 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11623, 41},
    /* 626 : SHL rm32,reg_cl [m-:      evex.nd0.nf.l0.m4.o32 d3 /4       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d3) 204 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11633, 66},
    /* 627 : SHL reg64?,rm64,reg_cl [vm-:      evex.nd1.nf.l0.m4.o64 d3 /4       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d3) 214 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11653, 41},
    /* 628 : SHL rm64,reg_cl [m-:      evex.nd0.nf.l0.m4.o64 d3 /4       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d3) 204 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11663, 66},
    /* 629 : SHL reg8?,rm8,imm8 [vmi:      evex.nd1.nf.l0.m4.o8 c0 /4 ib,u   ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(c0) 214 026 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+11683, 35},
    /* 630 : SHL rm8,imm8 [mi:      evex.nd0.nf.l0.m4.o8 c0 /4 ib,u   ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(c0) 204 025 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+11693, 62},
    /* 631 : SHL reg16?,rm16,imm8 [vmi:      evex.nd1.nf.l0.m4.o16 c1 /4 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(c1) 214 026 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1806, 38},
    /* 632 : SHL rm16,imm8 [mi:      evex.nd0.nf.l0.m4.o16 c1 /4 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(c1) 204 025 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1817, 64},
    /* 633 : SHL reg32?,rm32,imm8 [vmi:      evex.nd1.nf.l0.m4.o32 c1 /4 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(c1) 214 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1839, 41},
    /* 634 : SHL rm32,imm8 [mi:      evex.nd0.nf.l0.m4.o32 c1 /4 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(c1) 204 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1850, 66},
    /* 635 : SHL reg64?,rm64,imm8 [vmi:      evex.nd1.nf.l0.m4.o64 c1 /4 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(c1) 214 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1872, 41},
    /* 636 : SHL rm64,imm8 [mi:      evex.nd0.nf.l0.m4.o64 c1 /4 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(c1) 204 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1883, 66},
    /* 637 : SAL reg8?,rm8,unity [vm-:      evex.nd1.nf.l0.m4.o8 d0 /4        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d0) 214 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 3, {REG_GPR|BITS8,RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+23976, 35},
    /* 638 : SAL rm8,unity [m-:      evex.nd0.nf.l0.m4.o8 d0 /4        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d0) 204 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23985, 62},
    /* 639 : SAL reg16?,rm16,unity [vm-:      evex.nd1.nf.l0.m4.o16 d1 /4       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d1) 214 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11503, 38},
    /* 640 : SAL rm16,unity [m-:      evex.nd0.nf.l0.m4.o16 d1 /4       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d1) 204 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11513, 64},
    /* 641 : SAL reg32?,rm32,unity [vm-:      evex.nd1.nf.l0.m4.o32 d1 /4       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d1) 214 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11533, 41},
    /* 642 : SAL rm32,unity [m-:      evex.nd0.nf.l0.m4.o32 d1 /4       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d1) 204 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11543, 66},
    /* 643 : SAL reg64?,rm64,unity [vm-:      evex.nd1.nf.l0.m4.o64 d1 /4       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d1) 214 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11563, 41},
    /* 644 : SAL rm64,unity [m-:      evex.nd0.nf.l0.m4.o64 d1 /4       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d1) 204 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11573, 66},
    /* 645 : SAL reg8?,rm8,reg_cl [vm-:      evex.nd1.nf.l0.m4.o8 d2 /4        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d2) 214 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+24003, 35},
    /* 646 : SAL rm8,reg_cl [m-:      evex.nd0.nf.l0.m4.o8 d2 /4        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d2) 204 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+24012, 62},
    /* 647 : SAL reg16?,rm16,reg_cl [vm-:      evex.nd1.nf.l0.m4.o16 d3 /4       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d3) 214 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11593, 38},
    /* 648 : SAL rm16,reg_cl [m-:      evex.nd0.nf.l0.m4.o16 d3 /4       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d3) 204 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11603, 64},
    /* 649 : SAL reg32?,rm32,reg_cl [vm-:      evex.nd1.nf.l0.m4.o32 d3 /4       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d3) 214 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11623, 41},
    /* 650 : SAL rm32,reg_cl [m-:      evex.nd0.nf.l0.m4.o32 d3 /4       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d3) 204 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11633, 66},
    /* 651 : SAL reg64?,rm64,reg_cl [vm-:      evex.nd1.nf.l0.m4.o64 d3 /4       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d3) 214 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11653, 41},
    /* 652 : SAL rm64,reg_cl [m-:      evex.nd0.nf.l0.m4.o64 d3 /4       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d3) 204 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11663, 66},
    /* 653 : SAL reg8?,rm8,imm8 [vmi:      evex.nd1.nf.l0.m4.o8 c0 /4 ib,u   ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(c0) 214 026 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+11683, 35},
    /* 654 : SAL rm8,imm8 [mi:      evex.nd0.nf.l0.m4.o8 c0 /4 ib,u   ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(c0) 204 025 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+11693, 62},
    /* 655 : SAL reg16?,rm16,imm8 [vmi:      evex.nd1.nf.l0.m4.o16 c1 /4 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(c1) 214 026 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1806, 38},
    /* 656 : SAL rm16,imm8 [mi:      evex.nd0.nf.l0.m4.o16 c1 /4 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(c1) 204 025 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1817, 64},
    /* 657 : SAL reg32?,rm32,imm8 [vmi:      evex.nd1.nf.l0.m4.o32 c1 /4 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(c1) 214 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1839, 41},
    /* 658 : SAL rm32,imm8 [mi:      evex.nd0.nf.l0.m4.o32 c1 /4 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(c1) 204 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1850, 66},
    /* 659 : SAL reg64?,rm64,imm8 [vmi:      evex.nd1.nf.l0.m4.o64 c1 /4 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(c1) 214 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1872, 41},
    /* 660 : SAL rm64,imm8 [mi:      evex.nd0.nf.l0.m4.o64 c1 /4 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(c1) 204 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1883, 66},
    /* 661 : SHR reg8?,rm8,unity [vm-:      evex.nd1.nf.l0.m4.o8 d0 /5        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d0) 215 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+24030, 35},
    /* 662 : SHR rm8,unity [m-:      evex.nd0.nf.l0.m4.o8 d0 /5        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d0) 205 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+24039, 62},
    /* 663 : SHR reg16?,rm16,unity [vm-:      evex.nd1.nf.l0.m4.o16 d1 /5       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d1) 215 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11713, 38},
    /* 664 : SHR rm16,unity [m-:      evex.nd0.nf.l0.m4.o16 d1 /5       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d1) 205 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11723, 64},
    /* 665 : SHR reg32?,rm32,unity [vm-:      evex.nd1.nf.l0.m4.o32 d1 /5       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d1) 215 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11743, 41},
    /* 666 : SHR rm32,unity [m-:      evex.nd0.nf.l0.m4.o32 d1 /5       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d1) 205 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11753, 66},
    /* 667 : SHR reg64?,rm64,unity [vm-:      evex.nd1.nf.l0.m4.o64 d1 /5       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d1) 215 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11773, 41},
    /* 668 : SHR rm64,unity [m-:      evex.nd0.nf.l0.m4.o64 d1 /5       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d1) 205 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11783, 66},
    /* 669 : SHR reg8?,rm8,reg_cl [vm-:      evex.nd1.nf.l0.m4.o8 d2 /5        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d2) 215 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+24057, 35},
    /* 670 : SHR rm8,reg_cl [m-:      evex.nd0.nf.l0.m4.o8 d2 /5        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d2) 205 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+24066, 62},
    /* 671 : SHR reg16?,rm16,reg_cl [vm-:      evex.nd1.nf.l0.m4.o16 d3 /5       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d3) 215 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11803, 38},
    /* 672 : SHR rm16,reg_cl [m-:      evex.nd0.nf.l0.m4.o16 d3 /5       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d3) 205 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11813, 64},
    /* 673 : SHR reg32?,rm32,reg_cl [vm-:      evex.nd1.nf.l0.m4.o32 d3 /5       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d3) 215 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11833, 41},
    /* 674 : SHR rm32,reg_cl [m-:      evex.nd0.nf.l0.m4.o32 d3 /5       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d3) 205 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11843, 66},
    /* 675 : SHR reg64?,rm64,reg_cl [vm-:      evex.nd1.nf.l0.m4.o64 d3 /5       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d3) 215 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11863, 41},
    /* 676 : SHR rm64,reg_cl [m-:      evex.nd0.nf.l0.m4.o64 d3 /5       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d3) 205 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11873, 66},
    /* 677 : SHR reg8?,rm8,imm8 [vmi:      evex.nd1.nf.l0.m4.o8 c0 /5 ib,u   ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(c0) 215 026 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+11893, 35},
    /* 678 : SHR rm8,imm8 [mi:      evex.nd0.nf.l0.m4.o8 c0 /5 ib,u   ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(c0) 205 025 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+11903, 62},
    /* 679 : SHR reg16?,rm16,imm8 [vmi:      evex.nd1.nf.l0.m4.o16 c1 /5 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(c1) 215 026 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1905, 38},
    /* 680 : SHR rm16,imm8 [mi:      evex.nd0.nf.l0.m4.o16 c1 /5 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(c1) 205 025 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1916, 64},
    /* 681 : SHR reg32?,rm32,imm8 [vmi:      evex.nd1.nf.l0.m4.o32 c1 /5 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(c1) 215 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1938, 41},
    /* 682 : SHR rm32,imm8 [mi:      evex.nd0.nf.l0.m4.o32 c1 /5 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(c1) 205 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1949, 66},
    /* 683 : SHR reg64?,rm64,imm8 [vmi:      evex.nd1.nf.l0.m4.o64 c1 /5 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(c1) 215 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1971, 41},
    /* 684 : SHR rm64,imm8 [mi:      evex.nd0.nf.l0.m4.o64 c1 /5 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(c1) 205 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1982, 66},
    /* 685 : SAR reg8?,rm8,unity [vm-:      evex.nd1.nf.l0.m4.o8 d0 /7        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d0) 217 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+24084, 35},
    /* 686 : SAR rm8,unity [m-:      evex.nd0.nf.l0.m4.o8 d0 /7        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d0) 207 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+24093, 62},
    /* 687 : SAR reg16?,rm16,unity [vm-:      evex.nd1.nf.l0.m4.o16 d1 /7       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d1) 217 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11923, 38},
    /* 688 : SAR rm16,unity [m-:      evex.nd0.nf.l0.m4.o16 d1 /7       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d1) 207 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11933, 64},
    /* 689 : SAR reg32?,rm32,unity [vm-:      evex.nd1.nf.l0.m4.o32 d1 /7       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d1) 217 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11953, 41},
    /* 690 : SAR rm32,unity [m-:      evex.nd0.nf.l0.m4.o32 d1 /7       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d1) 207 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11963, 66},
    /* 691 : SAR reg64?,rm64,unity [vm-:      evex.nd1.nf.l0.m4.o64 d1 /7       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d1) 217 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11983, 41},
    /* 692 : SAR rm64,unity [m-:      evex.nd0.nf.l0.m4.o64 d1 /7       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d1) 207 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11993, 66},
    /* 693 : SAR reg8?,rm8,reg_cl [vm-:      evex.nd1.nf.l0.m4.o8 d2 /7        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d2) 217 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+24111, 35},
    /* 694 : SAR rm8,reg_cl [m-:      evex.nd0.nf.l0.m4.o8 d2 /7        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d2) 207 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+24120, 62},
    /* 695 : SAR reg16?,rm16,reg_cl [vm-:      evex.nd1.nf.l0.m4.o16 d3 /7       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d3) 217 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+12013, 38},
    /* 696 : SAR rm16,reg_cl [m-:      evex.nd0.nf.l0.m4.o16 d3 /7       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d3) 207 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+12023, 64},
    /* 697 : SAR reg32?,rm32,reg_cl [vm-:      evex.nd1.nf.l0.m4.o32 d3 /7       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d3) 217 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+12043, 41},
    /* 698 : SAR rm32,reg_cl [m-:      evex.nd0.nf.l0.m4.o32 d3 /7       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d3) 207 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+12053, 66},
    /* 699 : SAR reg64?,rm64,reg_cl [vm-:      evex.nd1.nf.l0.m4.o64 d3 /7       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d3) 217 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+12073, 41},
    /* 700 : SAR rm64,reg_cl [m-:      evex.nd0.nf.l0.m4.o64 d3 /7       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d3) 207 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+12083, 66},
    /* 701 : SAR reg8?,rm8,imm8 [vmi:      evex.nd1.nf.l0.m4.o8 c0 /7 ib,u   ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(c0) 217 026 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+12103, 35},
    /* 702 : SAR rm8,imm8 [mi:      evex.nd0.nf.l0.m4.o8 c0 /7 ib,u   ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(c0) 207 025 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+12113, 62},
    /* 703 : SAR reg16?,rm16,imm8 [vmi:      evex.nd1.nf.l0.m4.o16 c1 /7 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(c1) 217 026 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+2004, 38},
    /* 704 : SAR rm16,imm8 [mi:      evex.nd0.nf.l0.m4.o16 c1 /7 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(c1) 207 025 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+2015, 64},
    /* 705 : SAR reg32?,rm32,imm8 [vmi:      evex.nd1.nf.l0.m4.o32 c1 /7 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(c1) 217 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+2037, 41},
    /* 706 : SAR rm32,imm8 [mi:      evex.nd0.nf.l0.m4.o32 c1 /7 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(c1) 207 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+2048, 66},
    /* 707 : SAR reg64?,rm64,imm8 [vmi:      evex.nd1.nf.l0.m4.o64 c1 /7 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(c1) 217 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+2070, 41},
    /* 708 : SAR rm64,imm8 [mi:      evex.nd0.nf.l0.m4.o64 c1 /7 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(c1) 207 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+2081, 66},
    /* 709 : RORX reg32,rm32,imm8 [rmi:      evex.lz.f2.0f3a.w0 f0 /r ib       ] SM0-1,LONG,PROT,BMI2,APX,X86_64 */
        /* 250(f3 7f 08) 300 001(f0) 110 022 : SM0,SM1,LONG,EVEX,PROT,BMI2,APX,X86_64 */
        {I_RORX, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+12133, 68},
    /* 710 : SHLX reg32,rm32*,reg32 [rmv:      evex.lz.66.0f38.w0 f7 /r          ] SM0,SM1,LONG,PROT,BMI2,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(f7) 110 : SM0,SM1,LONG,EVEX,PROT,BMI2,APX,X86_64 */
        {I_SHLX, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+24138, 68},
    /* 711 : SARX reg32,rm32*,reg32 [rmv:      evex.lz.f3.0f38.w0 f7 /r          ] SM0,SM1,LONG,PROT,BMI2,APX,X86_64 */
        /* 242(f2 7e 08) 300 001(f7) 110 : SM0,SM1,LONG,EVEX,PROT,BMI2,APX,X86_64 */
        {I_SARX, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+24156, 68},
    /* 712 : SHRX reg32,rm32*,reg32 [rmv:      evex.lz.f2.0f38.w0 f7 /r          ] SM0,SM1,LONG,PROT,BMI2,APX,X86_64 */
        /* 242(f2 7f 08) 300 001(f7) 110 : SM0,SM1,LONG,EVEX,PROT,BMI2,APX,X86_64 */
        {I_SHRX, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+24174, 68},
    /* 713 : RORX reg64,rm64,imm8 [rmi:      evex.lz.f2.0f3a.w1 f0 /r ib       ] SM0-1,LONG,PROT,BMI2,APX,X86_64 */
        /* 250(f3 ff 08) 300 001(f0) 110 022 : SM0,SM1,LONG,EVEX,PROT,BMI2,APX,X86_64 */
        {I_RORX, 3, {REG_GPR|BITS64,RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+12143, 68},
    /* 714 : SHLX reg64,rm64*,reg64 [rmv:      evex.lz.66.0f38.w1 f7 /r          ] SM0,SM1,LONG,PROT,BMI2,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(f7) 110 : SM0,SM1,LONG,EVEX,PROT,BMI2,APX,X86_64 */
        {I_SHLX, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+24192, 68},
    /* 715 : SARX reg64,rm64*,reg64 [rmv:      evex.lz.f3.0f38.w1 f7 /r          ] SM0,SM1,LONG,PROT,BMI2,APX,X86_64 */
        /* 242(f2 fe 08) 300 001(f7) 110 : SM0,SM1,LONG,EVEX,PROT,BMI2,APX,X86_64 */
        {I_SARX, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+24210, 68},
    /* 716 : SHRX reg64,rm64*,reg64 [rmv:      evex.lz.f2.0f38.w1 f7 /r          ] SM0,SM1,LONG,PROT,BMI2,APX,X86_64 */
        /* 242(f2 ff 08) 300 001(f7) 110 : SM0,SM1,LONG,EVEX,PROT,BMI2,APX,X86_64 */
        {I_SHRX, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+24228, 68},
    /* 717 : INC reg16 [r:        o16 40+r                          ] NOREX,NOAPX,NOLONG,FL,8086 */
        /* 320 010 100 : NOREX,NOAPX,NOLONG,FL,8086 */
        {I_INC, 1, {REG_GPR|BITS16|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72489, 72},
    /* 718 : INC reg32 [r:        o32 40+r                          ] NOREX,NOAPX,NOLONG,FL,386 */
        /* 321 010 100 : NOREX,NOAPX,NOLONG,FL,386 */
        {I_INC, 1, {REG_GPR|BITS32|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72493, 73},
    /* 719 : INC rm8 [m:        hle o8 fe /0                      ] LOCK,FL,8086 */
        /* 273 001(fe) 200 : LOCK,FL,8086 */
        {I_INC, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71463, 74},
    /* 720 : INC rm16 [m:        hle o16 ff /0                     ] LOCK,FL,8086 */
        /* 273 320 001(ff) 200 : LOCK,FL,8086 */
        {I_INC, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66494, 74},
    /* 721 : INC rm32 [m:        hle o32 ff /0                     ] LOCK,FL,386 */
        /* 273 321 001(ff) 200 : LOCK,FL,386 */
        {I_INC, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66500, 75},
    /* 722 : INC rm64 [m:        hle o64 ff /0                     ] LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(ff) 200 : LOCK,LONG,FL,PROT,X86_64 */
        {I_INC, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66506, 76},
    /* 723 : INC reg8?,rm8 [vm:       evex.nf.nd1.l0.m4.o8 fe /0        ] NF,ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(fe) 210 : NF,NF_E,ZU,WIG,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_INC, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+24246, 77},
    /* 724 : INC rm8 [m:       evex.nf.nd0.l0.m4.o8 fe /0        ] NF,LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(fe) 200 : NF,NF_E,WIG,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_INC, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+24255, 78},
    /* 725 : INC reg16?,rm16 [vm:       evex.nf.nd1.l0.m4.o16 ff /0       ] NF,ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(ff) 210 : NF,NF_E,ZU,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_INC, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+12153, 79},
    /* 726 : INC rm16 [m:       evex.nf.nd0.l0.m4.o16 ff /0       ] NF,LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(ff) 200 : NF,NF_E,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_INC, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12163, 80},
    /* 727 : INC reg32?,rm32 [vm:       evex.nf.nd1.l0.m4.o32 ff /0       ] NF,ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(ff) 210 : NF,NF_E,ZU,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_INC, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+12183, 81},
    /* 728 : INC rm32 [m:       evex.nf.nd0.l0.m4.o32 ff /0       ] NF,LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(ff) 200 : NF,NF_E,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_INC, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12193, 82},
    /* 729 : INC reg64?,rm64 [vm:       evex.nf.nd1.l0.m4.o64 ff /0       ] NF,ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(ff) 210 : NF,NF_E,ZU,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_INC, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+12213, 81},
    /* 730 : INC rm64 [m:       evex.nf.nd0.l0.m4.o64 ff /0       ] NF,LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(ff) 200 : NF,NF_E,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_INC, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12223, 82},
    /* 731 : DEC reg16 [r:        o16 48+r                          ] NOREX,NOAPX,NOLONG,FL,8086 */
        /* 320 010 110 : NOREX,NOAPX,NOLONG,FL,8086 */
        {I_DEC, 1, {REG_GPR|BITS16|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72497, 72},
    /* 732 : DEC reg32 [r:        o32 48+r                          ] NOREX,NOAPX,NOLONG,FL,386 */
        /* 321 010 110 : NOREX,NOAPX,NOLONG,FL,386 */
        {I_DEC, 1, {REG_GPR|BITS32|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72501, 73},
    /* 733 : DEC rm8 [m:        hle o8 fe /1                      ] LOCK,FL,8086 */
        /* 273 001(fe) 201 : LOCK,FL,8086 */
        {I_DEC, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71468, 74},
    /* 734 : DEC rm16 [m:        hle o16 ff /1                     ] LOCK,FL,8086 */
        /* 273 320 001(ff) 201 : LOCK,FL,8086 */
        {I_DEC, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66512, 74},
    /* 735 : DEC rm32 [m:        hle o32 ff /1                     ] LOCK,FL,386 */
        /* 273 321 001(ff) 201 : LOCK,FL,386 */
        {I_DEC, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66518, 75},
    /* 736 : DEC rm64 [m:        hle o64 ff /1                     ] LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(ff) 201 : LOCK,LONG,FL,PROT,X86_64 */
        {I_DEC, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66524, 76},
    /* 737 : DEC reg8?,rm8 [vm:       evex.nf.nd1.l0.m4.o8 fe /1        ] NF,ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(fe) 211 : NF,NF_E,ZU,WIG,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_DEC, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+24273, 77},
    /* 738 : DEC rm8 [m:       evex.nf.nd0.l0.m4.o8 fe /1        ] NF,LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(fe) 201 : NF,NF_E,WIG,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_DEC, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+24282, 78},
    /* 739 : DEC reg16?,rm16 [vm:       evex.nf.nd1.l0.m4.o16 ff /1       ] NF,ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(ff) 211 : NF,NF_E,ZU,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_DEC, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+12243, 79},
    /* 740 : DEC rm16 [m:       evex.nf.nd0.l0.m4.o16 ff /1       ] NF,LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(ff) 201 : NF,NF_E,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_DEC, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12253, 80},
    /* 741 : DEC reg32?,rm32 [vm:       evex.nf.nd1.l0.m4.o32 ff /1       ] NF,ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(ff) 211 : NF,NF_E,ZU,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_DEC, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+12273, 81},
    /* 742 : DEC rm32 [m:       evex.nf.nd0.l0.m4.o32 ff /1       ] NF,LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(ff) 201 : NF,NF_E,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_DEC, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12283, 82},
    /* 743 : DEC reg64?,rm64 [vm:       evex.nf.nd1.l0.m4.o64 ff /1       ] NF,ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(ff) 211 : NF,NF_E,ZU,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_DEC, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+12303, 81},
    /* 744 : DEC rm64 [m:       evex.nf.nd0.l0.m4.o64 ff /1       ] NF,LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(ff) 201 : NF,NF_E,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_DEC, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12313, 82},
    /* 745 : IMUL rm8 [m:        o8 f6 /5                          ] FL,8086 */
        /* 001(f6) 205 : FL,8086 */
        {I_IMUL, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+24305, 54},
    /* 746 : IMUL rm16 [m:        o16 f7 /5                         ] FL,8086 */
        /* 320 001(f7) 205 : FL,8086 */
        {I_IMUL, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12338, 54},
    /* 747 : IMUL rm32 [m:        o32 f7 /5                         ] FL,386 */
        /* 321 001(f7) 205 : FL,386 */
        {I_IMUL, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12348, 55},
    /* 748 : IMUL rm64 [m:        o64 f7 /5                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(f7) 205 : LONG,FL,PROT,X86_64 */
        {I_IMUL, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12358, 56},
    /* 749 : IMUL rm8 [m:        evex.nf.nd0.l0.m4.o8 f6 /5        ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(f6) 205 : SM0,NF,NF_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+24300, 83},
    /* 750 : IMUL rm16 [m:        evex.nf.nd0.l0.m4.o16 f7 /5       ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(f7) 205 : SM0,NF,NF_E,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12333, 84},
    /* 751 : IMUL rm32 [m:        evex.nf.nd0.l0.m4.o32 f7 /5       ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(f7) 205 : SM0,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12343, 85},
    /* 752 : IMUL rm64 [m:        evex.nf.nd0.l0.m4.o64 f7 /5       ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(f7) 205 : SM0,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12353, 85},
    /* 753 : IMUL reg16,rm16 [rm:       o16 0f af /r                      ] SM0-1,FL,386 */
        /* 320 355 001(af) 110 : SM0,SM1,FL,386 */
        {I_IMUL, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66530, 29},
    /* 754 : IMUL reg32,rm32 [rm:       o32 0f af /r                      ] SM0-1,FL,386 */
        /* 321 355 001(af) 110 : SM0,SM1,FL,386 */
        {I_IMUL, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66536, 29},
    /* 755 : IMUL reg64,rm64 [rm:       o64 0f af /r                      ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 355 001(af) 110 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_IMUL, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66542, 30},
    /* 756 : IMUL reg16,rm16*,sbyteword16 [rmi:      o16 6b /r ib,s                    ] SM0,SM1,SM2,FL,186 */
        /* 320 001(6b) 110 276 : SM0,SM1,SM2,FL,186 */
        {I_IMUL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2108, 86},
    /* 757 : IMUL reg32,rm32*,sbytedword32 [rmi:      o32 6b /r ib,s                    ] SM0,SM1,SM2,FL,386 */
        /* 321 001(6b) 110 276 : SM0,SM1,SM2,FL,386 */
        {I_IMUL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2119, 88},
    /* 758 : IMUL reg64,rm64*,sbytedword64 [rmi:      o64 6b /r ib,s                    ] SM0,SM1,SM2,LONG,FL,PROT,X86_64 */
        /* 324 001(6b) 110 276 : SM0,SM1,SM2,LONG,FL,PROT,X86_64 */
        {I_IMUL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2130, 89},
    /* 759 : IMUL reg16,rm16*,imm16 [rmi:      o16 69 /r iw                      ] SM0,SM1,SM2,FL,186 */
        /* 320 001(69) 110 032 : SM0,SM1,SM2,FL,186 */
        {I_IMUL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+2141, 86},
    /* 760 : IMUL reg32,rm32*,imm32 [rmi:      o32 69 /r id                      ] SM0,SM1,SM2,FL,386 */
        /* 321 001(69) 110 042 : SM0,SM1,SM2,FL,386 */
        {I_IMUL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+2152, 88},
    /* 761 : IMUL reg64,rm64*,sdword64 [rmi:      o64 69 /r id,s                    ] SM0,SM1,SM2,LONG,FL,PROT,X86_64 */
        /* 324 001(69) 110 256 : SM0,SM1,SM2,LONG,FL,PROT,X86_64 */
        {I_IMUL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2163, 89},
    /* 762 : IMUL reg16?,reg16,rm16 [vrm:      evex.nf.nd1.l0.m4.o16 af /r       ] SM0,SM1,SM2,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(af) 121 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+12363, 90},
    /* 763 : IMUL reg16,rm16 [rm:      evex.nf.nd0.l0.m4.o16 af /r       ] SM0,SM1,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(af) 110 : SM0,SM1,NF,NF_E,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+12373, 91},
    /* 764 : IMUL reg32?,reg32,rm32 [vrm:      evex.nf.nd1.l0.m4.o32 af /r       ] SM0,SM1,SM2,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(af) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+12393, 93},
    /* 765 : IMUL reg32,rm32 [rm:      evex.nf.nd0.l0.m4.o32 af /r       ] SM0,SM1,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(af) 110 : SM0,SM1,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+12403, 94},
    /* 766 : IMUL reg64?,reg64,rm64 [vrm:      evex.nf.nd1.l0.m4.o64 af /r       ] SM0,SM1,SM2,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(af) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+12423, 93},
    /* 767 : IMUL reg64,rm64 [rm:      evex.nf.nd0.l0.m4.o64 af /r       ] SM0,SM1,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(af) 110 : SM0,SM1,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+12433, 94},
    /* 768 : IMUL reg16,rm16,sbyteword16 [rmi:      evex.nf.zu.l0.m4.o16 6b /r ib,s   ] SM0-2,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(6b) 110 276 : SM0,SM1,SM2,NF,NF_E,ZU,ZU_E,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2103, 96},
    /* 769 : IMUL reg32,rm32,sbytedword32 [rmi:      evex.nf.zu.l0.m4.o32 6b /r ib,s   ] SM0-2,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(6b) 110 276 : SM0,SM1,SM2,NF,NF_E,ZU,ZU_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2114, 97},
    /* 770 : IMUL reg64,rm64,sbytedword64 [rmi:      evex.nf.zu.l0.m4.o64 6b /r ib,s   ] SM0-2,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(6b) 110 276 : SM0,SM1,SM2,NF,NF_E,ZU,ZU_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2125, 97},
    /* 771 : IMUL reg16,rm16,imm16 [rmi:      evex.nf.zu.l0.m4.o16 69 /r iw     ] SM0-2,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(69) 110 032 : SM0,SM1,SM2,NF,NF_E,ZU,ZU_E,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+2136, 96},
    /* 772 : IMUL reg32,rm32,imm32 [rmi:      evex.nf.zu.l0.m4.o32 69 /r id     ] SM0-2,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(69) 110 042 : SM0,SM1,SM2,NF,NF_E,ZU,ZU_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+2147, 97},
    /* 773 : IMUL reg64,rm64,sdword64 [rmi:      evex.nf.zu.l0.m4.o64 69 /r id,s   ] SM0-2,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(69) 110 256 : SM0,SM1,SM2,NF,NF_E,ZU,ZU_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2158, 97},
    /* 774 : MUL rm8 [m:        o8 f6 /4                          ] FL,8086 */
        /* 001(f6) 204 : FL,8086 */
        {I_MUL, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+24350, 54},
    /* 775 : MUL rm16 [m:        o16 f7 /4                         ] FL,8086 */
        /* 320 001(f7) 204 : FL,8086 */
        {I_MUL, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12458, 54},
    /* 776 : MUL rm32 [m:        o32 f7 /4                         ] FL,386 */
        /* 321 001(f7) 204 : FL,386 */
        {I_MUL, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12468, 55},
    /* 777 : MUL rm64 [m:        o64 f7 /4                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(f7) 204 : LONG,FL,PROT,X86_64 */
        {I_MUL, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12478, 56},
    /* 778 : MULX reg32,reg32*,rm32 [rvm:      vex.ndd.lz.f2.0f38.w0 f6 /r      ] BMI2,FUTURE */
        /* 261(002 003) 001(f6) 120 : NOAPX,VEX,BMI2,FUTURE */
        {I_MULX, 3, {REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55171, 98},
    /* 779 : MULX reg32,reg32*,rm32 [rvm:      evex.ndd.lz.f2.0f38.w0 f6 /r      ] BMI2,FUTURE,APX */
        /* 241(f2 7f 08) 300 001(f6) 120 : LONG,EVEX,PROT,BMI2,APX,FUTURE */
        {I_MULX, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+24309, 99},
    /* 780 : MULX reg64,reg64*,rm64 [rvm:      vex.ndd.lz.f2.0f38.w1 f6 /r      ] LONG,PROT,BMI2,X86_64 */
        /* 261(002 203) 001(f6) 120 : NOAPX,LONG,VEX,PROT,BMI2,X86_64 */
        {I_MULX, 3, {REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55185, 100},
    /* 781 : MULX reg64,reg64*,rm64 [rvm:      evex.ndd.lz.f2.0f38.w1 f6 /r      ] LONG,PROT,BMI2,X86_64,APX */
        /* 241(f2 ff 08) 300 001(f6) 120 : LONG,EVEX,PROT,BMI2,APX,X86_64 */
        {I_MULX, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+24327, 101},
    /* 782 : MUL rm8 [m:        evex.nf.nd0.l0.m4.o8 f6 /4        ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(f6) 204 : SM0,NF,NF_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_MUL, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+24345, 83},
    /* 783 : MUL rm16 [m:        evex.nf.nd0.l0.m4.o16 f7 /4       ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(f7) 204 : SM0,NF,NF_E,LONG,EVEX,PROT,APX,X86_64 */
        {I_MUL, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12453, 84},
    /* 784 : MUL rm32 [m:        evex.nf.nd0.l0.m4.o32 f7 /4       ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(f7) 204 : SM0,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_MUL, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12463, 85},
    /* 785 : MUL rm64 [m:        evex.nf.nd0.l0.m4.o64 f7 /4       ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(f7) 204 : SM0,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_MUL, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12473, 85},
    /* 786 : IDIV rm8 [m:        o8 f6 /7                          ] FL,8086 */
        /* 001(f6) 207 : FL,8086 */
        {I_IDIV, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+24359, 54},
    /* 787 : IDIV rm16 [m:        o16 f7 /7                         ] FL,8086 */
        /* 320 001(f7) 207 : FL,8086 */
        {I_IDIV, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12488, 54},
    /* 788 : IDIV rm32 [m:        o32 f7 /7                         ] FL,386 */
        /* 321 001(f7) 207 : FL,386 */
        {I_IDIV, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12498, 55},
    /* 789 : IDIV rm64 [m:        o64 f7 /7                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(f7) 207 : LONG,FL,PROT,X86_64 */
        {I_IDIV, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12508, 56},
    /* 790 : IDIV rm8 [m:        evex.nf.nd0.l0.m4.o8 f6 /7        ] NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(f6) 207 : NF,NF_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_IDIV, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+24354, 104},
    /* 791 : IDIV rm16 [m:        evex.nf.nd0.l0.m4.o16 f7 /7       ] NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(f7) 207 : NF,NF_E,LONG,EVEX,PROT,APX,X86_64 */
        {I_IDIV, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12483, 105},
    /* 792 : IDIV rm32 [m:        evex.nf.nd0.l0.m4.o32 f7 /7       ] NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(f7) 207 : NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_IDIV, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12493, 106},
    /* 793 : IDIV rm64 [m:        evex.nf.nd0.l0.m4.o64 f7 /7       ] NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(f7) 207 : NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_IDIV, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12503, 106},
    /* 794 : DIV rm8 [m:        o8 f6 /6                          ] FL,8086 */
        /* 001(f6) 206 : FL,8086 */
        {I_DIV, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+24368, 54},
    /* 795 : DIV rm16 [m:        o16 f7 /6                         ] FL,8086 */
        /* 320 001(f7) 206 : FL,8086 */
        {I_DIV, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12518, 54},
    /* 796 : DIV rm32 [m:        o32 f7 /6                         ] FL,386 */
        /* 321 001(f7) 206 : FL,386 */
        {I_DIV, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12528, 55},
    /* 797 : DIV rm64 [m:        o64 f7 /6                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(f7) 206 : LONG,FL,PROT,X86_64 */
        {I_DIV, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12538, 56},
    /* 798 : DIV rm8 [m:        evex.nf.nd0.l0.m4.o8 f6 /6        ] NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(f6) 206 : NF,NF_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_DIV, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+24363, 104},
    /* 799 : DIV rm16 [m:        evex.nf.nd0.l0.m4.o16 f7 /6       ] NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(f7) 206 : NF,NF_E,LONG,EVEX,PROT,APX,X86_64 */
        {I_DIV, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12513, 105},
    /* 800 : DIV rm32 [m:        evex.nf.nd0.l0.m4.o32 f7 /6       ] NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(f7) 206 : NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_DIV, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12523, 106},
    /* 801 : DIV rm64 [m:        evex.nf.nd0.l0.m4.o64 f7 /6       ] NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(f7) 206 : NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_DIV, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12533, 106},
    /* 802 : NEG rm8 [m:        hle o8 f6 /3                      ] LOCK,FL,8086 */
        /* 273 001(f6) 203 : LOCK,FL,8086 */
        {I_NEG, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71473, 74},
    /* 803 : NEG rm16 [m:        hle o16 f7 /3                     ] LOCK,FL,8086 */
        /* 273 320 001(f7) 203 : LOCK,FL,8086 */
        {I_NEG, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66584, 74},
    /* 804 : NEG rm32 [m:        hle o32 f7 /3                     ] LOCK,FL,386 */
        /* 273 321 001(f7) 203 : LOCK,FL,386 */
        {I_NEG, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66590, 75},
    /* 805 : NEG rm64 [m:        hle o64 f7 /3                     ] LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(f7) 203 : LOCK,LONG,FL,PROT,X86_64 */
        {I_NEG, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66596, 76},
    /* 806 : NEG reg8?,rm8 [vm:       evex.nf.nd1.l0.m4.o8 f6 /3        ] NF,ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(f6) 213 : NF,NF_E,ZU,WIG,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NEG, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+24372, 77},
    /* 807 : NEG rm8 [m:       evex.nf.nd0.l0.m4.o8 f6 /3        ] NF,LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(f6) 203 : NF,NF_E,WIG,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NEG, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+24381, 78},
    /* 808 : NEG reg16?,rm16 [vm:       evex.nf.nd1.l0.m4.o16 f7 /3       ] NF,ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(f7) 213 : NF,NF_E,ZU,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NEG, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+12543, 79},
    /* 809 : NEG rm16 [m:       evex.nf.nd0.l0.m4.o16 f7 /3       ] NF,LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(f7) 203 : NF,NF_E,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NEG, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12553, 80},
    /* 810 : NEG reg32?,rm32 [vm:       evex.nf.nd1.l0.m4.o32 f7 /3       ] NF,ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(f7) 213 : NF,NF_E,ZU,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NEG, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+12573, 81},
    /* 811 : NEG rm32 [m:       evex.nf.nd0.l0.m4.o32 f7 /3       ] NF,LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(f7) 203 : NF,NF_E,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NEG, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12583, 82},
    /* 812 : NEG reg64?,rm64 [vm:       evex.nf.nd1.l0.m4.o64 f7 /3       ] NF,ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(f7) 213 : NF,NF_E,ZU,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NEG, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+12603, 81},
    /* 813 : NEG rm64 [m:       evex.nf.nd0.l0.m4.o64 f7 /3       ] NF,LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(f7) 203 : NF,NF_E,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NEG, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12613, 82},
    /* 814 : NOT rm8 [m:        hle o8 f6 /2                      ] LOCK,8086 */
        /* 273 001(f6) 202 : LOCK,8086 */
        {I_NOT, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71478, 107},
    /* 815 : NOT rm16 [m:        hle o16 f7 /2                     ] LOCK,8086 */
        /* 273 320 001(f7) 202 : LOCK,8086 */
        {I_NOT, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66602, 107},
    /* 816 : NOT rm32 [m:        hle o32 f7 /2                     ] LOCK,386 */
        /* 273 321 001(f7) 202 : LOCK,386 */
        {I_NOT, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66608, 108},
    /* 817 : NOT rm64 [m:        hle o64 f7 /2                     ] LOCK,LONG,PROT,X86_64 */
        /* 273 324 001(f7) 202 : LOCK,LONG,PROT,X86_64 */
        {I_NOT, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66614, 109},
    /* 818 : NOT reg8?,rm8 [vm:       evex.nf0.nd1.l0.m4.o8 f6 /2       ] ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(f6) 212 : ZU,WIG,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NOT, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+24399, 110},
    /* 819 : NOT rm8 [m:       evex.nf0.nd0.l0.m4.o8 f6 /2       ] LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(f6) 202 : WIG,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NOT, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+24408, 111},
    /* 820 : NOT reg16?,rm16 [vm:       evex.nf0.nd1.l0.m4.o16 f7 /2      ] ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(f7) 212 : ZU,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NOT, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+12633, 112},
    /* 821 : NOT rm16 [m:       evex.nf0.nd0.l0.m4.o16 f7 /2      ] LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(f7) 202 : LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NOT, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12643, 113},
    /* 822 : NOT reg32?,rm32 [vm:       evex.nf0.nd1.l0.m4.o32 f7 /2      ] ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(f7) 212 : ZU,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NOT, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+12663, 114},
    /* 823 : NOT rm32 [m:       evex.nf0.nd0.l0.m4.o32 f7 /2      ] LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(f7) 202 : WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NOT, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12673, 115},
    /* 824 : NOT reg64?,rm64 [vm:       evex.nf0.nd1.l0.m4.o64 f7 /2      ] ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(f7) 212 : ZU,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NOT, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+12693, 114},
    /* 825 : NOT rm64 [m:       evex.nf0.nd0.l0.m4.o64 f7 /2      ] LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(f7) 202 : WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NOT, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12703, 115},
    /* 826 : ADCX reg32,rm32 [rm:       66 w0 0f38 f6 /r                  ] ZU,FL,ADX,FUTURE */
        /* 361 317 356 001(f6) 110 : NOAPX,ZU,FL,ADX,FUTURE */
        {I_ADCX, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55199, 116},
    /* 827 : ADCX reg64,rm64 [rm:       66 w1 0f38 f6 /r                  ] ZU,LONG,FL,PROT,ADX,X86_64 */
        /* 361 324 356 001(f6) 110 : NOAPX,ZU,LONG,FL,PROT,ADX,X86_64 */
        {I_ADCX, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55206, 117},
    /* 828 : ADCX reg32?,reg32,rm32 [vrm:      evex.nd1.l0.66.m4.w0 66 /r        ] ZU,LONG,FL,PROT,APX,ADX,X86_64 */
        /* 240(f4 7d 18) 300 001(66) 121 : ZU,LONG,FL,EVEX,PROT,APX,ADX,X86_64 */
        {I_ADCX, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+24426, 118},
    /* 829 : ADCX reg32,rm32 [rm:      evex.nd0.l0.66.m4.w0 66 /r        ] ZU,LONG,FL,PROT,APX,ADX,X86_64 */
        /* 250(f4 7d 08) 300 001(66) 110 : ZU,LONG,FL,EVEX,PROT,APX,ADX,X86_64 */
        {I_ADCX, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+24435, 118},
    /* 830 : ADCX reg64?,reg64,rm64 [vrm:      evex.nd1.l0.66.m4.w1 66 /r        ] ZU,LONG,FL,PROT,APX,ADX,X86_64 */
        /* 240(f4 fd 18) 300 001(66) 121 : ZU,LONG,FL,EVEX,PROT,APX,ADX,X86_64 */
        {I_ADCX, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+24444, 118},
    /* 831 : ADCX reg64,rm64 [rm:      evex.nd0.l0.66.m4.w1 66 /r        ] ZU,LONG,FL,PROT,APX,ADX,X86_64 */
        /* 250(f4 fd 08) 300 001(66) 110 : ZU,LONG,FL,EVEX,PROT,APX,ADX,X86_64 */
        {I_ADCX, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+24453, 118},
    /* 832 : ADOX reg32,rm32 [rm:       f3 !osp w0 0f38 f6 /r             ] ZU,FL,ADX,FUTURE */
        /* 333 364 317 356 001(f6) 110 : NOAPX,ZU,FL,ADX,FUTURE */
        {I_ADOX, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+51109, 116},
    /* 833 : ADOX reg64,rm64 [rm:       f3 !osp w1 0f38 f6 /r             ] ZU,LONG,FL,PROT,ADX,X86_64 */
        /* 333 364 324 356 001(f6) 110 : NOAPX,ZU,LONG,FL,PROT,ADX,X86_64 */
        {I_ADOX, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+51117, 117},
    /* 834 : ADOX reg32?,reg32,rm32 [vrm:      evex.nd1.l0.f3.m4.w0 66 /r        ] ZU,LONG,FL,PROT,APX,ADX,X86_64 */
        /* 240(f4 7e 18) 300 001(66) 121 : ZU,LONG,FL,EVEX,PROT,APX,ADX,X86_64 */
        {I_ADOX, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+24462, 118},
    /* 835 : ADOX reg32,rm32 [rm:      evex.nd0.l0.f3.m4.w0 66 /r        ] ZU,LONG,FL,PROT,APX,ADX,X86_64 */
        /* 250(f4 7e 08) 300 001(66) 110 : ZU,LONG,FL,EVEX,PROT,APX,ADX,X86_64 */
        {I_ADOX, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+24471, 118},
    /* 836 : ADOX reg64?,reg64,rm64 [vrm:      evex.nd1.l0.f3.m4.w1 66 /r        ] ZU,LONG,FL,PROT,APX,ADX,X86_64 */
        /* 240(f4 fe 18) 300 001(66) 121 : ZU,LONG,FL,EVEX,PROT,APX,ADX,X86_64 */
        {I_ADOX, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+24480, 118},
    /* 837 : ADOX reg64,rm64 [rm:      evex.nd0.l0.f3.m4.w1 66 /r        ] ZU,LONG,FL,PROT,APX,ADX,X86_64 */
        /* 250(f4 fe 08) 300 001(66) 110 : ZU,LONG,FL,EVEX,PROT,APX,ADX,X86_64 */
        {I_ADOX, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+24489, 118},
    /* 838 : SHLD rm16,reg16,imm8 [mri:      o16 0f a4 /r ib,u                 ] SM0-1,FL,386 */
        /* 320 355 001(a4) 101 026 : SM0,SM1,FL,386 */
        {I_SHLD, 3, {RM_GPR|BITS16,REG_GPR|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+55213, 29},
    /* 839 : SHLD rm32,reg32,imm8 [mri:      o32 0f a4 /r ib,u                 ] SM0-1,FL,386 */
        /* 321 355 001(a4) 101 026 : SM0,SM1,FL,386 */
        {I_SHLD, 3, {RM_GPR|BITS32,REG_GPR|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+55220, 29},
    /* 840 : SHLD rm64,reg64,imm8 [mri:      o64 0f a4 /r ib,u                 ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 355 001(a4) 101 026 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_SHLD, 3, {RM_GPR|BITS64,REG_GPR|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+55227, 30},
    /* 841 : SHLD rm16,reg16,reg_cl [mr-:      o16 0f a5 /r                      ] SM0-1,FL,386 */
        /* 320 355 001(a5) 101 : SM0,SM1,FL,386 */
        {I_SHLD, 3, {RM_GPR|BITS16,REG_GPR|BITS16,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+66620, 29},
    /* 842 : SHLD rm32,reg32,reg_cl [mr-:      o32 0f a5 /r                      ] SM0-1,FL,386 */
        /* 321 355 001(a5) 101 : SM0,SM1,FL,386 */
        {I_SHLD, 3, {RM_GPR|BITS32,REG_GPR|BITS32,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+66626, 29},
    /* 843 : SHLD rm64,reg64,reg_cl [mr-:      o64 0f a5 /r                      ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 355 001(a5) 101 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_SHLD, 3, {RM_GPR|BITS64,REG_GPR|BITS64,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+66632, 30},
    /* 844 : SHLD reg16?,rm16,reg16,imm8 [vmri:     evex.nd1.nf.l0.m4.o16 24 /r ib,u  ] SM0,SM1,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(24) 112 027 : SM0,SM1,NF,NF_E,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHLD, 4, {REG_GPR|BITS16,RM_GPR|BITS16,REG_GPR|BITS16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+2169, 92},
    /* 845 : SHLD rm16,reg16,imm8 [mri:     evex.nd0.nf.l0.m4.o16 24 /r ib,u  ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(24) 101 026 : SM0,NF,NF_E,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHLD, 3, {RM_GPR|BITS16,REG_GPR|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+2180, 84},
    /* 846 : SHLD reg32?,rm32,reg32,imm8 [vmri:     evex.nd1.nf.l0.m4.o32 24 /r ib,u  ] SM0,SM1,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(24) 112 027 : SM0,SM1,NF,NF_E,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHLD, 4, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+2202, 95},
    /* 847 : SHLD rm32,reg32,imm8 [mri:     evex.nd0.nf.l0.m4.o32 24 /r ib,u  ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(24) 101 026 : SM0,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHLD, 3, {RM_GPR|BITS32,REG_GPR|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+2213, 85},
    /* 848 : SHLD reg64?,rm64,reg64,imm8 [vmri:     evex.nd1.nf.l0.m4.o64 24 /r ib,u  ] SM0,SM1,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(24) 112 027 : SM0,SM1,NF,NF_E,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHLD, 4, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+2235, 95},
    /* 849 : SHLD rm64,reg64,imm8 [mri:     evex.nd0.nf.l0.m4.o64 24 /r ib,u  ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(24) 101 026 : SM0,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHLD, 3, {RM_GPR|BITS64,REG_GPR|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+2246, 85},
    /* 850 : SHLD reg16?,rm16,reg16,reg_cl [vmr-:     evex.nd1.nf.l0.m4.o16 a5 /r       ] SM0,SM1,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(a5) 112 : SM0,SM1,NF,NF_E,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHLD, 4, {REG_GPR|BITS16,RM_GPR|BITS16,REG_GPR|BITS16,REG_CL,0}, NO_DECORATOR, nasm_bytecodes+12723, 92},
    /* 851 : SHLD rm16,reg16,reg_cl [mr-:     evex.nd0.nf.l0.m4.o16 a5 /r       ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(a5) 101 : SM0,NF,NF_E,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHLD, 3, {RM_GPR|BITS16,REG_GPR|BITS16,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+12733, 84},
    /* 852 : SHLD reg32?,rm32,reg32,reg_cl [vmr-:     evex.nd1.nf.l0.m4.o32 a5 /r       ] SM0,SM1,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(a5) 112 : SM0,SM1,NF,NF_E,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHLD, 4, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,REG_CL,0}, NO_DECORATOR, nasm_bytecodes+12753, 95},
    /* 853 : SHLD rm32,reg32,reg_cl [mr-:     evex.nd0.nf.l0.m4.o32 a5 /r       ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(a5) 101 : SM0,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHLD, 3, {RM_GPR|BITS32,REG_GPR|BITS32,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+12763, 85},
    /* 854 : SHLD reg64?,rm64,reg64,reg_cl [vmr-:     evex.nd1.nf.l0.m4.o64 a5 /r       ] SM0,SM1,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(a5) 112 : SM0,SM1,NF,NF_E,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHLD, 4, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,REG_CL,0}, NO_DECORATOR, nasm_bytecodes+12783, 95},
    /* 855 : SHLD rm64,reg64,reg_cl [mr-:     evex.nd0.nf.l0.m4.o64 a5 /r       ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(a5) 101 : SM0,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHLD, 3, {RM_GPR|BITS64,REG_GPR|BITS64,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+12793, 85},
    /* 856 : SHRD rm16,reg16,imm8 [mri:      o16 0f ac /r ib,u                 ] SM0-1,FL,386 */
        /* 320 355 001(ac) 101 026 : SM0,SM1,FL,386 */
        {I_SHRD, 3, {RM_GPR|BITS16,REG_GPR|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+55234, 29},
    /* 857 : SHRD rm32,reg32,imm8 [mri:      o32 0f ac /r ib,u                 ] SM0-1,FL,386 */
        /* 321 355 001(ac) 101 026 : SM0,SM1,FL,386 */
        {I_SHRD, 3, {RM_GPR|BITS32,REG_GPR|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+55241, 29},
    /* 858 : SHRD rm64,reg64,imm8 [mri:      o64 0f ac /r ib,u                 ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 355 001(ac) 101 026 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_SHRD, 3, {RM_GPR|BITS64,REG_GPR|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+55248, 30},
    /* 859 : SHRD rm16,reg16,reg_cl [mr-:      o16 0f ad /r                      ] SM0-1,FL,386 */
        /* 320 355 001(ad) 101 : SM0,SM1,FL,386 */
        {I_SHRD, 3, {RM_GPR|BITS16,REG_GPR|BITS16,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+66638, 29},
    /* 860 : SHRD rm32,reg32,reg_cl [mr-:      o32 0f ad /r                      ] SM0-1,FL,386 */
        /* 321 355 001(ad) 101 : SM0,SM1,FL,386 */
        {I_SHRD, 3, {RM_GPR|BITS32,REG_GPR|BITS32,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+66644, 29},
    /* 861 : SHRD rm64,reg64,reg_cl [mr-:      o64 0f ad /r                      ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 355 001(ad) 101 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_SHRD, 3, {RM_GPR|BITS64,REG_GPR|BITS64,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+66650, 30},
    /* 862 : SHRD reg16?,rm16,reg16,imm8 [vmri:     evex.nd1.nf.l0.m4.o16 2c /r ib,u  ] SM0,SM1,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(2c) 112 027 : SM0,SM1,NF,NF_E,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHRD, 4, {REG_GPR|BITS16,RM_GPR|BITS16,REG_GPR|BITS16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+2268, 92},
    /* 863 : SHRD rm16,reg16,imm8 [mri:     evex.nd0.nf.l0.m4.o16 2c /r ib,u  ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(2c) 101 026 : SM0,NF,NF_E,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHRD, 3, {RM_GPR|BITS16,REG_GPR|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+2279, 84},
    /* 864 : SHRD reg32?,rm32,reg32,imm8 [vmri:     evex.nd1.nf.l0.m4.o32 2c /r ib,u  ] SM0,SM1,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(2c) 112 027 : SM0,SM1,NF,NF_E,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHRD, 4, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+2301, 95},
    /* 865 : SHRD rm32,reg32,imm8 [mri:     evex.nd0.nf.l0.m4.o32 2c /r ib,u  ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(2c) 101 026 : SM0,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHRD, 3, {RM_GPR|BITS32,REG_GPR|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+2312, 85},
    /* 866 : SHRD reg64?,rm64,reg64,imm8 [vmri:     evex.nd1.nf.l0.m4.o64 2c /r ib,u  ] SM0,SM1,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(2c) 112 027 : SM0,SM1,NF,NF_E,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHRD, 4, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+2334, 95},
    /* 867 : SHRD rm64,reg64,imm8 [mri:     evex.nd0.nf.l0.m4.o64 2c /r ib,u  ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(2c) 101 026 : SM0,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHRD, 3, {RM_GPR|BITS64,REG_GPR|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+2345, 85},
    /* 868 : SHRD reg16?,rm16,reg16,reg_cl [vmr-:     evex.nd1.nf.l0.m4.o16 ad /r       ] SM0,SM1,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(ad) 112 : SM0,SM1,NF,NF_E,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHRD, 4, {REG_GPR|BITS16,RM_GPR|BITS16,REG_GPR|BITS16,REG_CL,0}, NO_DECORATOR, nasm_bytecodes+12813, 92},
    /* 869 : SHRD rm16,reg16,reg_cl [mr-:     evex.nd0.nf.l0.m4.o16 ad /r       ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(ad) 101 : SM0,NF,NF_E,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHRD, 3, {RM_GPR|BITS16,REG_GPR|BITS16,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+12823, 84},
    /* 870 : SHRD reg32?,rm32,reg32,reg_cl [vmr-:     evex.nd1.nf.l0.m4.o32 ad /r       ] SM0,SM1,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(ad) 112 : SM0,SM1,NF,NF_E,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHRD, 4, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,REG_CL,0}, NO_DECORATOR, nasm_bytecodes+12843, 95},
    /* 871 : SHRD rm32,reg32,reg_cl [mr-:     evex.nd0.nf.l0.m4.o32 ad /r       ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(ad) 101 : SM0,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHRD, 3, {RM_GPR|BITS32,REG_GPR|BITS32,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+12853, 85},
    /* 872 : SHRD reg64?,rm64,reg64,reg_cl [vmr-:     evex.nd1.nf.l0.m4.o64 ad /r       ] SM0,SM1,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(ad) 112 : SM0,SM1,NF,NF_E,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHRD, 4, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,REG_CL,0}, NO_DECORATOR, nasm_bytecodes+12873, 95},
    /* 873 : SHRD rm64,reg64,reg_cl [mr-:     evex.nd0.nf.l0.m4.o64 ad /r       ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(ad) 101 : SM0,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHRD, 3, {RM_GPR|BITS64,REG_GPR|BITS64,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+12883, 85},
    /* 874 : BT rm16,reg16 [mr:       o16 0f a3 /r                      ] SM0-1,FL,386 */
        /* 320 355 001(a3) 101 : SM0,SM1,FL,386 */
        {I_BT, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66656, 29},
    /* 875 : BT rm32,reg32 [mr:       o32 0f a3 /r                      ] SM0-1,FL,386 */
        /* 321 355 001(a3) 101 : SM0,SM1,FL,386 */
        {I_BT, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66662, 29},
    /* 876 : BT rm64,reg64 [mr:       o64 0f a3 /r                      ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 355 001(a3) 101 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_BT, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66668, 30},
    /* 877 : BT rm16,imm8 [mi:       o16 0f ba /4 ib,u                 ] FL,386 */
        /* 320 355 001(ba) 204 025 : FL,386 */
        {I_BT, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+55255, 55},
    /* 878 : BT rm32,imm8 [mi:       o32 0f ba /4 ib,u                 ] FL,386 */
        /* 321 355 001(ba) 204 025 : FL,386 */
        {I_BT, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+55262, 55},
    /* 879 : BT rm64,imm8 [mi:       o64 0f ba /4 ib,u                 ] LONG,FL,PROT,X86_64 */
        /* 324 355 001(ba) 204 025 : LONG,FL,PROT,X86_64 */
        {I_BT, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+55269, 56},
    /* 880 : BTC rm16,reg16 [mr:       o16 0f bb /r                      ] SM0-1,LOCK,FL,386 */
        /* 320 355 001(bb) 101 : SM0,SM1,LOCK,FL,386 */
        {I_BTC, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66674, 26},
    /* 881 : BTC rm32,reg32 [mr:       o32 0f bb /r                      ] SM0-1,LOCK,FL,386 */
        /* 321 355 001(bb) 101 : SM0,SM1,LOCK,FL,386 */
        {I_BTC, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66680, 26},
    /* 882 : BTC rm64,reg64 [mr:       o64 0f bb /r                      ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 324 355 001(bb) 101 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_BTC, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66686, 27},
    /* 883 : BTC rm16,imm8 [mi:       o16 0f ba /7 ib,u                 ] LOCK,FL,386 */
        /* 320 355 001(ba) 207 025 : LOCK,FL,386 */
        {I_BTC, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+55276, 75},
    /* 884 : BTC rm32,imm8 [mi:       o32 0f ba /7 ib,u                 ] LOCK,FL,386 */
        /* 321 355 001(ba) 207 025 : LOCK,FL,386 */
        {I_BTC, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+55283, 75},
    /* 885 : BTC rm64,imm8 [mi:       o64 0f ba /7 ib,u                 ] LOCK,LONG,FL,PROT,X86_64 */
        /* 324 355 001(ba) 207 025 : LOCK,LONG,FL,PROT,X86_64 */
        {I_BTC, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+55290, 76},
    /* 886 : BTR rm16,reg16 [mr:       o16 0f b3 /r                      ] SM0-1,LOCK,FL,386 */
        /* 320 355 001(b3) 101 : SM0,SM1,LOCK,FL,386 */
        {I_BTR, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66692, 26},
    /* 887 : BTR rm32,reg32 [mr:       o32 0f b3 /r                      ] SM0-1,LOCK,FL,386 */
        /* 321 355 001(b3) 101 : SM0,SM1,LOCK,FL,386 */
        {I_BTR, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66698, 26},
    /* 888 : BTR rm64,reg64 [mr:       o64 0f b3 /r                      ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 324 355 001(b3) 101 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_BTR, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66704, 27},
    /* 889 : BTR rm16,imm8 [mi:       o16 0f ba /6 ib,u                 ] LOCK,FL,386 */
        /* 320 355 001(ba) 206 025 : LOCK,FL,386 */
        {I_BTR, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+55297, 75},
    /* 890 : BTR rm32,imm8 [mi:       o32 0f ba /6 ib,u                 ] LOCK,FL,386 */
        /* 321 355 001(ba) 206 025 : LOCK,FL,386 */
        {I_BTR, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+55304, 75},
    /* 891 : BTR rm64,imm8 [mi:       o64 0f ba /6 ib,u                 ] LOCK,LONG,FL,PROT,X86_64 */
        /* 324 355 001(ba) 206 025 : LOCK,LONG,FL,PROT,X86_64 */
        {I_BTR, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+55311, 76},
    /* 892 : BTS rm16,reg16 [mr:       o16 0f ab /r                      ] SM0-1,LOCK,FL,386 */
        /* 320 355 001(ab) 101 : SM0,SM1,LOCK,FL,386 */
        {I_BTS, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66710, 26},
    /* 893 : BTS rm32,reg32 [mr:       o32 0f ab /r                      ] SM0-1,LOCK,FL,386 */
        /* 321 355 001(ab) 101 : SM0,SM1,LOCK,FL,386 */
        {I_BTS, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66716, 26},
    /* 894 : BTS rm64,reg64 [mr:       o64 0f ab /r                      ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 324 355 001(ab) 101 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_BTS, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66722, 27},
    /* 895 : BTS rm16,imm8 [mi:       o16 0f ba /5 ib,u                 ] LOCK,FL,386 */
        /* 320 355 001(ba) 205 025 : LOCK,FL,386 */
        {I_BTS, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+55318, 75},
    /* 896 : BTS rm32,imm8 [mi:       o32 0f ba /5 ib,u                 ] LOCK,FL,386 */
        /* 321 355 001(ba) 205 025 : LOCK,FL,386 */
        {I_BTS, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+55325, 75},
    /* 897 : BTS rm64,imm8 [mi:       o64 0f ba /5 ib,u                 ] LOCK,LONG,FL,PROT,X86_64 */
        /* 324 355 001(ba) 205 025 : LOCK,LONG,FL,PROT,X86_64 */
        {I_BTS, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+55332, 76},
    /* 898 : BSF reg16,rm16 [rm:       o16 nof3 0f bc /r                 ] SM0-1,386 */
        /* 320 326 355 001(bc) 110 : SM0,SM1,386 */
        {I_BSF, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55339, 13},
    /* 899 : BSF reg32,rm32 [rm:       o32 nof3 0f bc /r                 ] SM0-1,386 */
        /* 321 326 355 001(bc) 110 : SM0,SM1,386 */
        {I_BSF, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+55346, 13},
    /* 900 : BSF reg64,rm64 [rm:       o64 nof3 0f bc /r                 ] SM0-1,LONG,PROT,X86_64 */
        /* 324 326 355 001(bc) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_BSF, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+55353, 14},
    /* 901 : BSR reg16,rm16 [rm:       o16 nof3 0f bd /r                 ] SM0-1,386 */
        /* 320 326 355 001(bd) 110 : SM0,SM1,386 */
        {I_BSR, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55360, 13},
    /* 902 : BSR reg32,rm32 [rm:       o32 nof3 0f bd /r                 ] SM0-1,386 */
        /* 321 326 355 001(bd) 110 : SM0,SM1,386 */
        {I_BSR, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+55367, 13},
    /* 903 : BSR reg64,rm64 [rm:       o64 nof3 0f bd /r                 ] SM0-1,LONG,PROT,X86_64 */
        /* 324 326 355 001(bd) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_BSR, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+55374, 14},
    /* 904 : LZCNT reg16,rm16 [rm:       o16 f3i 0f bd /r                  ] SM0-1,FL,LZCNT,FUTURE */
        /* 320 333 355 001(bd) 110 : SM0,SM1,FL,LZCNT,FUTURE */
        {I_LZCNT, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55381, 122},
    /* 905 : LZCNT reg32,rm32 [rm:       o32 f3i 0f bd /r                  ] SM0-1,FL,LZCNT,FUTURE */
        /* 321 333 355 001(bd) 110 : SM0,SM1,FL,LZCNT,FUTURE */
        {I_LZCNT, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+55388, 122},
    /* 906 : LZCNT reg64,rm64 [rm:       o64 f3i 0f bd /r                  ] SM0-1,LONG,FL,PROT,LZCNT,X86_64 */
        /* 324 333 355 001(bd) 110 : SM0,SM1,LONG,FL,PROT,LZCNT,X86_64 */
        {I_LZCNT, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+55395, 123},
    /* 907 : LZCNT reg16,rm16 [rm:       evex.nd0.nf.l0.m4.o16 f5 /r       ] SM0-1,NF,LONG,PROT,APX,LZCNT,X86_64 */
        /* 250(f4 7d 08) 300 320 001(f5) 110 : SM0,SM1,NF,NF_E,LONG,EVEX,PROT,APX,LZCNT,X86_64 */
        {I_LZCNT, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+12903, 124},
    /* 908 : LZCNT reg32,rm32 [rm:       evex.nd0.nf.l0.m4.o32 f5 /r       ] SM0-1,NF,LONG,PROT,APX,LZCNT,X86_64 */
        /* 250(f4 7c 08) 300 321 001(f5) 110 : SM0,SM1,NF,NF_E,WW,LONG,EVEX,PROT,APX,LZCNT,X86_64 */
        {I_LZCNT, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+12913, 125},
    /* 909 : LZCNT reg64,rm64 [rm:       evex.nd0.nf.l0.m4.o64 f5 /r       ] SM0-1,NF,LONG,PROT,APX,LZCNT,X86_64 */
        /* 250(f4 fc 08) 300 324 001(f5) 110 : SM0,SM1,NF,NF_E,WW,LONG,EVEX,PROT,APX,LZCNT,X86_64 */
        {I_LZCNT, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+12923, 125},
    /* 910 : TZCNT reg16,rm16 [rm:       o16 f3i 0f bc /r                  ] SM0-1,FL,BMI1,FUTURE */
        /* 320 333 355 001(bc) 110 : SM0,SM1,FL,BMI1,FUTURE */
        {I_TZCNT, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55402, 126},
    /* 911 : TZCNT reg32,rm32 [rm:       o32 f3i 0f bc /r                  ] SM0-1,FL,BMI1,FUTURE */
        /* 321 333 355 001(bc) 110 : SM0,SM1,FL,BMI1,FUTURE */
        {I_TZCNT, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+55409, 126},
    /* 912 : TZCNT reg64,rm64 [rm:       o64 f3i 0f bc /r                  ] SM0-1,LONG,FL,PROT,BMI1,X86_64 */
        /* 324 333 355 001(bc) 110 : SM0,SM1,LONG,FL,PROT,BMI1,X86_64 */
        {I_TZCNT, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+55416, 127},
    /* 913 : TZCNT reg16,rm16 [rm:       evex.nd0.nf.l0.m4.o16 f4 /r       ] SM0-1,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(f4) 110 : SM0,SM1,NF,NF_E,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_TZCNT, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+12933, 128},
    /* 914 : TZCNT reg32,rm32 [rm:       evex.nd0.nf.l0.m4.o32 f4 /r       ] SM0-1,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(f4) 110 : SM0,SM1,NF,NF_E,WW,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_TZCNT, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+12943, 129},
    /* 915 : TZCNT reg64,rm64 [rm:       evex.nd0.nf.l0.m4.o64 f4 /r       ] SM0-1,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(f4) 110 : SM0,SM1,NF,NF_E,WW,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_TZCNT, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+12953, 129},
    /* 916 : ANDN reg32,reg32*,rm32 [rvm:      vex.nds.lz.0f38.w0 f2 /r          ] SM0,SM1,SM2,FL,BMI1,FUTURE */
        /* 261(002 000) 001(f2) 120 : SM0,SM1,SM2,NOAPX,FL,VEX,BMI1,FUTURE */
        {I_ANDN, 3, {REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55423, 130},
    /* 917 : ANDN reg64,reg64*,rm64 [rvm:      vex.nds.lz.0f38.w1 f2 /r          ] SM0,SM1,SM2,LONG,FL,PROT,BMI1,X86_64 */
        /* 261(002 200) 001(f2) 120 : SM0,SM1,SM2,NOAPX,LONG,FL,VEX,PROT,BMI1,X86_64 */
        {I_ANDN, 3, {REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55437, 132},
    /* 918 : ANDN reg32,reg32*,rm32 [rvm:      evex.np.nf.lz.0f38.w0 f2 /r       ] SM0,SM1,SM2,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 241(f2 7c 08) 300 001(f2) 120 : SM0,SM1,SM2,NF,NF_E,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_ANDN, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+24498, 134},
    /* 919 : ANDN reg64,reg64*,rm64 [rvm:      evex.np.nf.lz.0f38.w1 f2 /r       ] SM0,SM1,SM2,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 241(f2 fc 08) 300 001(f2) 120 : SM0,SM1,SM2,NF,NF_E,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_ANDN, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+24516, 134},
    /* 920 : BEXTR reg32,rm32*,reg32 [rmv:      vex.nds.lz.0f38.w0 f7 /r          ] SM0,SM1,SM2,FL,BMI1,FUTURE */
        /* 262(002 000) 001(f7) 110 : SM0,SM1,SM2,NOAPX,FL,VEX,BMI1,FUTURE */
        {I_BEXTR, 3, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55451, 130},
    /* 921 : BEXTR reg64,rm64*,reg64 [rmv:      vex.nds.lz.0f38.w1 f7 /r          ] SM0,SM1,SM2,LONG,FL,PROT,BMI1,X86_64 */
        /* 262(002 200) 001(f7) 110 : SM0,SM1,SM2,NOAPX,LONG,FL,VEX,PROT,BMI1,X86_64 */
        {I_BEXTR, 3, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55465, 132},
    /* 922 : BEXTR reg32,rm32*,reg32 [rmv:      evex.np.nf.lz.0f38.w0 f7 /r       ] SM0,SM1,SM2,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 242(f2 7c 08) 300 001(f7) 110 : SM0,SM1,SM2,NF,NF_E,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_BEXTR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+24534, 134},
    /* 923 : BEXTR reg64,rm64*,reg64 [rmv:      evex.np.nf.lz.0f38.w1 f7 /r       ] SM0,SM1,SM2,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 242(f2 fc 08) 300 001(f7) 110 : SM0,SM1,SM2,NF,NF_E,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_BEXTR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+24552, 134},
    /* 924 : BLSMSK reg32,rm32 [vm:       vex.ndd.lz.0f38.w0 f3 /2          ] SM0-1,FL,BMI1,FUTURE */
        /* 260(002 000) 001(f3) 212 : SM0,SM1,NOAPX,FL,VEX,BMI1,FUTURE */
        {I_BLSMSK, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55479, 131},
    /* 925 : BLSMSK reg64,rm64 [vm:       vex.ndd.lz.0f38.w1 f3 /2          ] SM0-1,LONG,FL,PROT,BMI1,X86_64 */
        /* 260(002 200) 001(f3) 212 : SM0,SM1,NOAPX,LONG,FL,VEX,PROT,BMI1,X86_64 */
        {I_BLSMSK, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55486, 133},
    /* 926 : BLSMSK reg32,rm32 [vm:       evex.np.nf.lz.0f38.w0 f3 /2       ] SM0-1,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 240(f2 7c 08) 300 001(f3) 212 : SM0,SM1,NF,NF_E,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_BLSMSK, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+24570, 128},
    /* 927 : BLSMSK reg64,rm64 [vm:       evex.np.nf.lz.0f38.w1 f3 /2       ] SM0-1,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 240(f2 fc 08) 300 001(f3) 212 : SM0,SM1,NF,NF_E,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_BLSMSK, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+24579, 128},
    /* 928 : BLSR reg32,rm32 [vm:       vex.ndd.lz.0f38.w0 f3 /1          ] SM0-1,FL,BMI1,FUTURE */
        /* 260(002 000) 001(f3) 211 : SM0,SM1,NOAPX,FL,VEX,BMI1,FUTURE */
        {I_BLSR, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55493, 131},
    /* 929 : BLSR reg64,rm64 [vm:       vex.ndd.lz.0f38.w1 f3 /1          ] SM0-1,LONG,FL,PROT,BMI1,X86_64 */
        /* 260(002 200) 001(f3) 211 : SM0,SM1,NOAPX,LONG,FL,VEX,PROT,BMI1,X86_64 */
        {I_BLSR, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55500, 133},
    /* 930 : BLSR reg32,rm32 [vm:       evex.np.nf.lz.0f38.w0 f3 /1       ] SM0-1,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 240(f2 7c 08) 300 001(f3) 211 : SM0,SM1,NF,NF_E,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_BLSR, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+24588, 128},
    /* 931 : BLSR reg64,rm64 [vm:       evex.np.nf.lz.0f38.w1 f3 /1       ] SM0-1,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 240(f2 fc 08) 300 001(f3) 211 : SM0,SM1,NF,NF_E,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_BLSR, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+24597, 128},
    /* 932 : BLSI reg32,rm32 [vm:       vex.ndd.lz.0f38.w0 f3 /3          ] SM0-1,FL,BMI1,FUTURE */
        /* 260(002 000) 001(f3) 213 : SM0,SM1,NOAPX,FL,VEX,BMI1,FUTURE */
        {I_BLSI, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55507, 131},
    /* 933 : BLSI reg64,rm64 [vm:       vex.ndd.lz.0f38.w1 f3 /3          ] SM0-1,LONG,FL,PROT,BMI1,X86_64 */
        /* 260(002 200) 001(f3) 213 : SM0,SM1,NOAPX,LONG,FL,VEX,PROT,BMI1,X86_64 */
        {I_BLSI, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55514, 133},
    /* 934 : BLSI reg32,rm32 [vm:       evex.np.nf.lz.0f38.w0 f3 /3       ] SM0-1,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 240(f2 7c 08) 300 001(f3) 213 : SM0,SM1,NF,NF_E,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_BLSI, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+24606, 128},
    /* 935 : BLSI reg64,rm64 [vm:       evex.np.nf.lz.0f38.w1 f3 /3       ] SM0-1,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 240(f2 fc 08) 300 001(f3) 213 : SM0,SM1,NF,NF_E,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_BLSI, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+24615, 128},
    /* 936 : BZHI reg32,rm32*,reg32 [rmv:      vex.nds.lz.0f38.w0 f5 /r          ] SM0,SM1,SM2,FL,BMI2,FUTURE */
        /* 262(002 000) 001(f5) 110 : SM0,SM1,SM2,NOAPX,FL,VEX,BMI2,FUTURE */
        {I_BZHI, 3, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55521, 135},
    /* 937 : BZHI reg64,rm64*,reg64 [rmv:      vex.nds.lz.0f38.w1 f5 /r          ] SM0,SM1,SM2,LONG,FL,PROT,BMI2,X86_64 */
        /* 262(002 200) 001(f5) 110 : SM0,SM1,SM2,NOAPX,LONG,FL,VEX,PROT,BMI2,X86_64 */
        {I_BZHI, 3, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55535, 137},
    /* 938 : BZHI reg32,rm32*,reg32 [rmv:      evex.np.nf.lz.0f38.w0 f5 /r       ] SM0,SM1,SM2,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 242(f2 7c 08) 300 001(f5) 110 : SM0,SM1,SM2,NF,NF_E,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_BZHI, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+24624, 134},
    /* 939 : BZHI reg64,rm64*,reg64 [rmv:      evex.np.nf.lz.0f38.w1 f5 /r       ] SM0,SM1,SM2,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 242(f2 fc 08) 300 001(f5) 110 : SM0,SM1,SM2,NF,NF_E,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_BZHI, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+24642, 134},
    /* 940 : PDEP reg32,reg32*,rm32 [rvm:      vex.nds.lz.f2.0f38.w0 f5 /r      ] SM0,SM1,SM2,BMI2,FUTURE */
        /* 261(002 003) 001(f5) 120 : SM0,SM1,SM2,NOAPX,VEX,BMI2,FUTURE */
        {I_PDEP, 3, {REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55549, 139},
    /* 941 : PDEP reg32,reg32*,rm32 [rvm:      evex.nds.lz.f2.0f38.w0 f5 /r      ] SM0,SM1,SM2,BMI2,FUTURE,APX */
        /* 241(f2 7f 08) 300 001(f5) 120 : SM0,SM1,SM2,LONG,EVEX,PROT,BMI2,APX,FUTURE */
        {I_PDEP, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+24660, 140},
    /* 942 : PDEP reg64,reg64*,rm64 [rvm:      vex.nds.lz.f2.0f38.w1 f5 /r      ] SM0,SM1,SM2,LONG,PROT,BMI2,X86_64 */
        /* 261(002 203) 001(f5) 120 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,BMI2,X86_64 */
        {I_PDEP, 3, {REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55563, 142},
    /* 943 : PDEP reg64,reg64*,rm64 [rvm:      evex.nds.lz.f2.0f38.w1 f5 /r      ] SM0,SM1,SM2,LONG,PROT,BMI2,X86_64,APX */
        /* 241(f2 ff 08) 300 001(f5) 120 : SM0,SM1,SM2,LONG,EVEX,PROT,BMI2,APX,X86_64 */
        {I_PDEP, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+24678, 143},
    /* 944 : PEXT reg32,reg32*,rm32 [rvm:      vex.nds.lz.f3.0f38.w0 f5 /r      ] SM0,SM1,SM2,BMI2,FUTURE */
        /* 261(002 002) 001(f5) 120 : SM0,SM1,SM2,NOAPX,VEX,BMI2,FUTURE */
        {I_PEXT, 3, {REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55577, 139},
    /* 945 : PEXT reg32,reg32*,rm32 [rvm:      evex.nds.lz.f3.0f38.w0 f5 /r      ] SM0,SM1,SM2,BMI2,FUTURE,APX */
        /* 241(f2 7e 08) 300 001(f5) 120 : SM0,SM1,SM2,LONG,EVEX,PROT,BMI2,APX,FUTURE */
        {I_PEXT, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+24696, 140},
    /* 946 : PEXT reg64,reg64*,rm64 [rvm:      vex.nds.lz.f3.0f38.w1 f5 /r      ] SM0,SM1,SM2,LONG,PROT,BMI2,X86_64 */
        /* 261(002 202) 001(f5) 120 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,BMI2,X86_64 */
        {I_PEXT, 3, {REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55591, 142},
    /* 947 : PEXT reg64,reg64*,rm64 [rvm:      evex.nds.lz.f3.0f38.w1 f5 /r      ] SM0,SM1,SM2,LONG,PROT,BMI2,X86_64,APX */
        /* 241(f2 fe 08) 300 001(f5) 120 : SM0,SM1,SM2,LONG,EVEX,PROT,BMI2,APX,X86_64 */
        {I_PEXT, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+24714, 143},
    /* 948 : BEXTR reg32,rm32*,imm32 [rmi:      xop.m10.lz.w0 10 /r id            ] SM0,SM1,FL,TBM,OBSOLETE,FUTURE,AMD */
        /* 270(112 000) 001(10) 110 042 : SM0,SM1,NOAPX,FL,VEX,TBM,OBSOLETE,FUTURE,AMD */
        {I_BEXTR, 3, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+51125, 145},
    /* 949 : BEXTR reg64,rm64*,imm32 [rmi:      xop.m10.lz.w1 10 /r id            ] SM0,SM1,LONG,FL,PROT,TBM,OBSOLETE,X86_64,AMD */
        /* 270(112 200) 001(10) 110 042 : SM0,SM1,NOAPX,LONG,FL,VEX,PROT,TBM,OBSOLETE,X86_64,AMD */
        {I_BEXTR, 3, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+51141, 147},
    /* 950 : BLCI reg32,rm32 [vm:       xop.ndd.lz.m9.w0 02 /6            ] SM0-1,FL,TBM,OBSOLETE,FUTURE,AMD */
        /* 260(111 000) 001(02) 216 : SM0,SM1,NOAPX,FL,VEX,TBM,OBSOLETE,FUTURE,AMD */
        {I_BLCI, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55605, 145},
    /* 951 : BLCI reg64,rm64 [vm:       xop.ndd.lz.m9.w1 02 /6            ] SM0-1,LONG,FL,PROT,TBM,OBSOLETE,X86_64,AMD */
        /* 260(111 200) 001(02) 216 : SM0,SM1,NOAPX,LONG,FL,VEX,PROT,TBM,OBSOLETE,X86_64,AMD */
        {I_BLCI, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55612, 147},
    /* 952 : BLCIC reg32,rm32 [vm:       xop.ndd.lz.m9.w0 01 /5            ] SM0-1,FL,TBM,OBSOLETE,FUTURE,AMD */
        /* 260(111 000) 001(01) 215 : SM0,SM1,NOAPX,FL,VEX,TBM,OBSOLETE,FUTURE,AMD */
        {I_BLCIC, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55619, 145},
    /* 953 : BLCIC reg64,rm64 [vm:       xop.ndd.lz.m9.w1 01 /5            ] SM0-1,LONG,FL,PROT,TBM,OBSOLETE,X86_64,AMD */
        /* 260(111 200) 001(01) 215 : SM0,SM1,NOAPX,LONG,FL,VEX,PROT,TBM,OBSOLETE,X86_64,AMD */
        {I_BLCIC, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55626, 147},
    /* 954 : BLSIC reg32,rm32 [vm:       xop.ndd.lz.m9.w0 01 /6            ] SM0-1,FL,TBM,OBSOLETE,FUTURE,AMD */
        /* 260(111 000) 001(01) 216 : SM0,SM1,NOAPX,FL,VEX,TBM,OBSOLETE,FUTURE,AMD */
        {I_BLSIC, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55633, 145},
    /* 955 : BLSIC reg64,rm64 [vm:       xop.ndd.lz.m9.w1 01 /6            ] SM0-1,LONG,FL,PROT,TBM,OBSOLETE,X86_64,AMD */
        /* 260(111 200) 001(01) 216 : SM0,SM1,NOAPX,LONG,FL,VEX,PROT,TBM,OBSOLETE,X86_64,AMD */
        {I_BLSIC, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55640, 147},
    /* 956 : BLCFILL reg32,rm32 [vm:       xop.ndd.lz.m9.w0 01 /1            ] SM0-1,TBM,OBSOLETE,FUTURE,AMD */
        /* 260(111 000) 001(01) 211 : SM0,SM1,NOAPX,VEX,TBM,OBSOLETE,FUTURE,AMD */
        {I_BLCFILL, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55647, 149},
    /* 957 : BLCFILL reg64,rm64 [vm:       xop.ndd.lz.m9.w1 01 /1            ] SM0-1,LONG,PROT,TBM,OBSOLETE,X86_64,AMD */
        /* 260(111 200) 001(01) 211 : SM0,SM1,NOAPX,LONG,VEX,PROT,TBM,OBSOLETE,X86_64,AMD */
        {I_BLCFILL, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55654, 150},
    /* 958 : BLSFILL reg32,rm32 [vm:       xop.ndd.lz.m9.w0 01 /2            ] TBM,OBSOLETE,FUTURE,AMD */
        /* 260(111 000) 001(01) 212 : NOAPX,VEX,TBM,OBSOLETE,FUTURE,AMD */
        {I_BLSFILL, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55661, 151},
    /* 959 : BLSFILL reg64,rm64 [vm:       xop.ndd.lz.m9.w1 01 /2            ] LONG,PROT,TBM,OBSOLETE,X86_64,AMD */
        /* 260(111 200) 001(01) 212 : NOAPX,LONG,VEX,PROT,TBM,OBSOLETE,X86_64,AMD */
        {I_BLSFILL, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55668, 152},
    /* 960 : BLCMSK reg32,rm32 [vm:       xop.ndd.lz.m9.w0 02 /1            ] FL,TBM,OBSOLETE,FUTURE,AMD */
        /* 260(111 000) 001(02) 211 : NOAPX,FL,VEX,TBM,OBSOLETE,FUTURE,AMD */
        {I_BLCMSK, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55675, 153},
    /* 961 : BLCMSK reg64,rm64 [vm:       xop.ndd.lz.m9.w1 02 /1            ] LONG,FL,PROT,TBM,OBSOLETE,X86_64,AMD */
        /* 260(111 200) 001(02) 211 : NOAPX,LONG,FL,VEX,PROT,TBM,OBSOLETE,X86_64,AMD */
        {I_BLCMSK, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55682, 154},
    /* 962 : BLCS reg32,rm32 [vm:       xop.ndd.lz.m9.w0 01 /3            ] TBM,OBSOLETE,FUTURE,AMD */
        /* 260(111 000) 001(01) 213 : NOAPX,VEX,TBM,OBSOLETE,FUTURE,AMD */
        {I_BLCS, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55689, 151},
    /* 963 : BLCS reg64,rm64 [vm:       xop.ndd.lz.m9.w1 01 /3            ] LONG,PROT,TBM,OBSOLETE,X86_64,AMD */
        /* 260(111 200) 001(01) 213 : NOAPX,LONG,VEX,PROT,TBM,OBSOLETE,X86_64,AMD */
        {I_BLCS, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55696, 152},
    /* 964 : TZMSK reg32,rm32 [vm:       xop.ndd.lz.m9.w0 01 /4            ] TBM,OBSOLETE,FUTURE,AMD */
        /* 260(111 000) 001(01) 214 : NOAPX,VEX,TBM,OBSOLETE,FUTURE,AMD */
        {I_TZMSK, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55703, 151},
    /* 965 : TZMSK reg64,rm64 [vm:       xop.ndd.lz.m9.w1 01 /4            ] LONG,PROT,TBM,OBSOLETE,X86_64,AMD */
        /* 260(111 200) 001(01) 214 : NOAPX,LONG,VEX,PROT,TBM,OBSOLETE,X86_64,AMD */
        {I_TZMSK, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55710, 152},
    /* 966 : T1MSKC reg32,rm32 [vm:       xop.ndd.lz.m9.w0 01 /7            ] TBM,OBSOLETE,FUTURE,AMD */
        /* 260(111 000) 001(01) 217 : NOAPX,VEX,TBM,OBSOLETE,FUTURE,AMD */
        {I_T1MSKC, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55717, 151},
    /* 967 : T1MSKC reg64,rm64 [vm:       xop.ndd.lz.m9.w1 01 /7            ] LONG,PROT,TBM,OBSOLETE,X86_64,AMD */
        /* 260(111 200) 001(01) 217 : NOAPX,LONG,VEX,PROT,TBM,OBSOLETE,X86_64,AMD */
        {I_T1MSKC, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55724, 152},
    /* 968 : AAA void [          37                                ] NOREX,NOAPX,NOLONG,FL,8086 */
        /* 001(37) : NOREX,NOAPX,NOLONG,FL,8086 */
        {I_AAA, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73290, 72},
    /* 969 : AAD void [          d5 0a                             ] NOREX,NOAPX,NOLONG,FL,8086 */
        /* 002(d5 0a) : NOREX,NOAPX,NOLONG,FL,8086 */
        {I_AAD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72505, 72},
    /* 970 : AAD imm8 [i:        d5 ib,u                           ] AR0,SB,NOREX,NOAPX,NOLONG,FL,8086 */
        /* 001(d5) 024 : AR0,SB,NOREX,NOAPX,NOLONG,FL,8086 */
        {I_AAD, 1, {IMM_NORMAL|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72509, 155},
    /* 971 : AAM void [          d4 0a                             ] NOREX,NOAPX,NOLONG,FL,8086 */
        /* 002(d4 0a) : NOREX,NOAPX,NOLONG,FL,8086 */
        {I_AAM, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72513, 72},
    /* 972 : AAM imm8 [i:        d4 ib,u                           ] AR0,SB,NOREX,NOAPX,NOLONG,FL,8086 */
        /* 001(d4) 024 : AR0,SB,NOREX,NOAPX,NOLONG,FL,8086 */
        {I_AAM, 1, {IMM_NORMAL|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72517, 155},
    /* 973 : AAS void [          3f                                ] NOREX,NOAPX,NOLONG,FL,8086 */
        /* 001(3f) : NOREX,NOAPX,NOLONG,FL,8086 */
        {I_AAS, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73293, 72},
    /* 974 : DAA void [          27                                ] NOREX,NOAPX,NOLONG,FL,8086 */
        /* 001(27) : NOREX,NOAPX,NOLONG,FL,8086 */
        {I_DAA, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73296, 72},
    /* 975 : DAS void [          2f                                ] NOREX,NOAPX,NOLONG,FL,8086 */
        /* 001(2f) : NOREX,NOAPX,NOLONG,FL,8086 */
        {I_DAS, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73299, 72},
    /* 976 : BSWAP reg32 [r:        o32 0f c8+r                       ] 486 */
        /* 321 355 010 310 : 486 */
        {I_BSWAP, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71483, 156},
    /* 977 : BSWAP reg64 [r:        o64 0f c8+r                       ] LONG,PROT,X86_64 */
        /* 324 355 010 310 : LONG,PROT,X86_64 */
        {I_BSWAP, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71488, 24},
    /* 978 : MOVBE reg16,mem16 [rm:       o16 norep 0f38 f0 /r              ] SM0-1,MOVBE,NEHALEM */
        /* 320 331 356 001(f0) 110 : SM0,SM1,NOAPX,MOVBE,NEHALEM */
        {I_MOVBE, 2, {REG_GPR|BITS16|RN_L16,MEMORY|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55731, 158},
    /* 979 : MOVBE reg32,mem32 [rm:       o32 norep 0f38 f0 /r              ] SM0-1,MOVBE,NEHALEM */
        /* 321 331 356 001(f0) 110 : SM0,SM1,NOAPX,MOVBE,NEHALEM */
        {I_MOVBE, 2, {REG_GPR|BITS32|RN_L16,MEMORY|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55738, 158},
    /* 980 : MOVBE reg64,mem64 [rm:       o64 norep 0f38 f0 /r              ] SM0-1,LONG,PROT,MOVBE,X86_64,NEHALEM */
        /* 324 331 356 001(f0) 110 : SM0,SM1,NOAPX,LONG,PROT,MOVBE,NEHALEM */
        {I_MOVBE, 2, {REG_GPR|BITS64|RN_L16,MEMORY|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55745, 159},
    /* 981 : MOVBE mem16,reg16 [mr:       o16 norep 0f38 f1 /r              ] SM0-1,MOVBE,NEHALEM */
        /* 320 331 356 001(f1) 101 : SM0,SM1,NOAPX,MOVBE,NEHALEM */
        {I_MOVBE, 2, {MEMORY|BITS16|RN_L16,REG_GPR|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55752, 158},
    /* 982 : MOVBE mem32,reg32 [mr:       o32 norep 0f38 f1 /r              ] SM0-1,MOVBE,NEHALEM */
        /* 321 331 356 001(f1) 101 : SM0,SM1,NOAPX,MOVBE,NEHALEM */
        {I_MOVBE, 2, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55759, 158},
    /* 983 : MOVBE mem64,reg64 [mr:       o64 norep 0f38 f1 /r              ] SM0-1,LONG,PROT,MOVBE,X86_64,NEHALEM */
        /* 324 331 356 001(f1) 101 : SM0,SM1,NOAPX,LONG,PROT,MOVBE,NEHALEM */
        {I_MOVBE, 2, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55766, 159},
    /* 984 : MOVBE reg16,mem16 [rm:       evex.nd0.nf0.l0.m4.o16 60 /r      ] SM0-1,LONG,PROT,APX,MOVBE,X86_64,NEHALEM */
        /* 250(f4 7d 08) 300 320 001(60) 110 : SM0,SM1,LONG,EVEX,PROT,APX,MOVBE,NEHALEM */
        {I_MOVBE, 2, {REG_GPR|BITS16,MEMORY|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+12963, 160},
    /* 985 : MOVBE reg32,mem32 [rm:       evex.nd0.nf0.l0.m4.o32 60 /r      ] SM0-1,LONG,PROT,APX,MOVBE,X86_64,NEHALEM */
        /* 250(f4 7c 08) 300 321 001(60) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,MOVBE,NEHALEM */
        {I_MOVBE, 2, {REG_GPR|BITS32,MEMORY|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+12973, 161},
    /* 986 : MOVBE reg64,mem64 [rm:       evex.nd0.nf0.l0.m4.o64 60 /r      ] SM0-1,LONG,PROT,APX,MOVBE,X86_64,NEHALEM */
        /* 250(f4 fc 08) 300 324 001(60) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,MOVBE,NEHALEM */
        {I_MOVBE, 2, {REG_GPR|BITS64,MEMORY|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+12983, 161},
    /* 987 : MOVBE mem16,reg16 [mr:       evex.nd0.nf0.l0.m4.o16 61 /r      ] SM0-1,LONG,PROT,APX,MOVBE,X86_64,NEHALEM */
        /* 250(f4 7d 08) 300 320 001(61) 101 : SM0,SM1,LONG,EVEX,PROT,APX,MOVBE,NEHALEM */
        {I_MOVBE, 2, {MEMORY|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+12993, 160},
    /* 988 : MOVBE mem32,reg32 [mr:       evex.nd0.nf0.l0.m4.o32 61 /r      ] SM0-1,LONG,PROT,APX,MOVBE,X86_64,NEHALEM */
        /* 250(f4 7c 08) 300 321 001(61) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,MOVBE,NEHALEM */
        {I_MOVBE, 2, {MEMORY|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+13003, 161},
    /* 989 : MOVBE mem64,reg64 [mr:       evex.nd0.nf0.l0.m4.o64 61 /r      ] SM0-1,LONG,PROT,APX,MOVBE,X86_64,NEHALEM */
        /* 250(f4 fc 08) 300 324 001(61) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,MOVBE,NEHALEM */
        {I_MOVBE, 2, {MEMORY|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+13013, 161},
    /* 990 : CBW void [          o16 98                            ] 8086 */
        /* 320 001(98) : 8086 */
        {I_CBW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72537, 23},
    /* 991 : CDQ void [          o32 99                            ] 386 */
        /* 321 001(99) : 386 */
        {I_CDQ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72541, 3},
    /* 992 : CDQE void [          o64 98                            ] LONG,PROT,X86_64 */
        /* 324 001(98) : LONG,PROT,X86_64 */
        {I_CDQE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72545, 24},
    /* 993 : CQO void [          o64 99                            ] LONG,PROT,X86_64 */
        /* 324 001(99) : LONG,PROT,X86_64 */
        {I_CQO, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72549, 24},
    /* 994 : CWD void [          o16 99                            ] 8086 */
        /* 320 001(99) : 8086 */
        {I_CWD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72553, 23},
    /* 995 : CWDE void [          o32 98                            ] 386 */
        /* 321 001(98) : 386 */
        {I_CWDE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72557, 3},
    /* 996 : MOVSX reg16,rm8 [rm:       o16 0f be /r                      ] AR0-1,SX,386 */
        /* 320 355 001(be) 110 : AR0,AR1,SX,386 */
        {I_MOVSX, 2, {REG_GPR|BITS16,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66752, 165},
    /* 997 : MOVSX reg32,rm8 [rm:       o32 0f be /r                      ] AR0-1,SX,386 */
        /* 321 355 001(be) 110 : AR0,AR1,SX,386 */
        {I_MOVSX, 2, {REG_GPR|BITS32,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66758, 165},
    /* 998 : MOVSX reg64,rm8 [rm:       o64 0f be /r                      ] AR0-1,SX,LONG,PROT,X86_64 */
        /* 324 355 001(be) 110 : AR0,AR1,SX,LONG,PROT,X86_64 */
        {I_MOVSX, 2, {REG_GPR|BITS64,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66764, 166},
    /* 999 : MOVSX reg16,rm16 [rm:       o16 0f bf /r                      ] AR0-1,SX,386 */
        /* 320 355 001(bf) 110 : AR0,AR1,SX,386 */
        {I_MOVSX, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66770, 165},
    /* 1000 : MOVSX reg32,rm16 [rm:       o32 0f bf /r                      ] AR0-1,SX,386 */
        /* 321 355 001(bf) 110 : AR0,AR1,SX,386 */
        {I_MOVSX, 2, {REG_GPR|BITS32,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66776, 165},
    /* 1001 : MOVSX reg64,rm16 [rm:       o64 0f bf /r                      ] AR0-1,SX,LONG,PROT,X86_64 */
        /* 324 355 001(bf) 110 : AR0,AR1,SX,LONG,PROT,X86_64 */
        {I_MOVSX, 2, {REG_GPR|BITS64,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66782, 166},
    /* 1002 : MOVSXD reg16,rm32 [rm:       o16 63 /r                         ] LONG,PROT,X86_64 */
        /* 320 001(63) 110 : LONG,PROT,X86_64 */
        {I_MOVSXD, 2, {REG_GPR|BITS16,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71493, 24},
    /* 1003 : MOVSXD reg32,rm32 [rm:       o32 63 /r                         ] LONG,PROT,X86_64 */
        /* 321 001(63) 110 : LONG,PROT,X86_64 */
        {I_MOVSXD, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71498, 24},
    /* 1004 : MOVSXD reg64,rm32 [rm:       o64 63 /r                         ] LONG,PROT,X86_64 */
        /* 324 001(63) 110 : LONG,PROT,X86_64 */
        {I_MOVSXD, 2, {REG_GPR|BITS64,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71503, 24},
    /* 1005 : MOVZX reg16,rm8 [rm:       o16 0f b6 /r                      ] AR0-1,SX,386 */
        /* 320 355 001(b6) 110 : AR0,AR1,SX,386 */
        {I_MOVZX, 2, {REG_GPR|BITS16,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66788, 165},
    /* 1006 : MOVZX reg32,rm8 [rm:       o32 0f b6 /r                      ] AR0-1,SX,386 */
        /* 321 355 001(b6) 110 : AR0,AR1,SX,386 */
        {I_MOVZX, 2, {REG_GPR|BITS32,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66794, 165},
    /* 1007 : MOVZX reg64,rm8 [rm:       o64 0f b6 /r                      ] AR0-1,SX,LONG,PROT,X86_64 */
        /* 324 355 001(b6) 110 : AR0,AR1,SX,LONG,PROT,X86_64 */
        {I_MOVZX, 2, {REG_GPR|BITS64,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66800, 166},
    /* 1008 : MOVZX reg16,rm16 [rm:       o16 0f b7 /r                      ] AR0-1,SX,386 */
        /* 320 355 001(b7) 110 : AR0,AR1,SX,386 */
        {I_MOVZX, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66806, 165},
    /* 1009 : MOVZX reg32,rm16 [rm:       o32 0f b7 /r                      ] AR0-1,SX,386 */
        /* 321 355 001(b7) 110 : AR0,AR1,SX,386 */
        {I_MOVZX, 2, {REG_GPR|BITS32,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66812, 165},
    /* 1010 : MOVZX reg64,rm16 [rm:       o64 0f b7 /r                      ] AR0-1,SX,LONG,PROT,X86_64 */
        /* 324 355 001(b7) 110 : AR0,AR1,SX,LONG,PROT,X86_64 */
        {I_MOVZX, 2, {REG_GPR|BITS64,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66818, 166},
    /* 1011 : CMPXCHG rm8,reg8 [mr:       hle o8 0f b0 /r                   ] SM0-1,LOCK,FL,PENT */
        /* 273 355 001(b0) 101 : SM0,SM1,LOCK,FL,PENT */
        {I_CMPXCHG, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66842, 173},
    /* 1012 : CMPXCHG rm16,reg16 [mr:       hle o16 0f b1 /r                  ] SM0-1,LOCK,FL,PENT */
        /* 273 320 355 001(b1) 101 : SM0,SM1,LOCK,FL,PENT */
        {I_CMPXCHG, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55773, 173},
    /* 1013 : CMPXCHG rm32,reg32 [mr:       hle o32 0f b1 /r                  ] SM0-1,LOCK,FL,PENT */
        /* 273 321 355 001(b1) 101 : SM0,SM1,LOCK,FL,PENT */
        {I_CMPXCHG, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+55780, 173},
    /* 1014 : CMPXCHG rm64,reg64 [mr:       hle o64 0f b1 /r                  ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 355 001(b1) 101 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_CMPXCHG, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+55787, 27},
    /* 1015 : CMPXCHG8B mem64 [m:        hle norexw 0f c7 /1               ] LOCK,FL,PENT */
        /* 273 317 355 001(c7) 201 : LOCK,FL,PENT */
        {I_CMPXCHG8B, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55794, 174},
    /* 1016 : CMPXCHG16B mem128 [m:        o64 0f c7 /1                      ] LOCK,LONG,FL,PROT,X86_64 */
        /* 324 355 001(c7) 201 : LOCK,LONG,FL,PROT,X86_64 */
        {I_CMPXCHG16B, 1, {MEMORY|BITS128,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66848, 76},
    /* 1017 : XADD rm8,reg8 [mr:       hle o8 0f c0 /r                   ] SM0-1,LOCK,FL,486 */
        /* 273 355 001(c0) 101 : SM0,SM1,LOCK,FL,486 */
        {I_XADD, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66854, 175},
    /* 1018 : XADD rm16,reg16 [mr:       hle o16 0f c1 /r                  ] SM0-1,LOCK,FL,486 */
        /* 273 320 355 001(c1) 101 : SM0,SM1,LOCK,FL,486 */
        {I_XADD, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55801, 175},
    /* 1019 : XADD rm32,reg32 [mr:       hle o32 0f c1 /r                  ] SM0-1,LOCK,FL,486 */
        /* 273 321 355 001(c1) 101 : SM0,SM1,LOCK,FL,486 */
        {I_XADD, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+55808, 175},
    /* 1020 : XADD rm64,reg64 [mr:       hle o64 0f c1 /r                  ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 355 001(c1) 101 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_XADD, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+55815, 27},
    /* 1021 : XCHG reg_ax,reg16 [-r:       o16 90+r                          ] 8086 */
        /* 320 011 220 : 8086 */
        {I_XCHG, 2, {REG_AX,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72561, 23},
    /* 1022 : XCHG reg_rax,reg64 [-r:       o64 90+r                          ] ZU,LONG,PROT,X86_64 */
        /* 324 011 220 : ZU,LONG,PROT,X86_64 */
        {I_XCHG, 2, {REG_RAX,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+72565, 176},
    /* 1023 : XCHG reg_eax,reg32na [-r:       o32 90+r                          ] ZU,386 */
        /* 321 011 220 : ZU,386 */
        {I_XCHG, 2, {REG_EAX,REG_NA|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+72577, 177},
    /* 1024 : XCHG reg8,rm8 [rm:       hlenl o8 86 /r                    ] SM0-1,LOCK1,8086 */
        /* 272 001(86) 110 : SM0,SM1,LOCK1,8086 */
        {I_XCHG, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+71508, 179},
    /* 1025 : XCHG reg16,rm16 [rm:       hlenl o16 87 /r                   ] SM0-1,LOCK1,8086 */
        /* 272 320 001(87) 110 : SM0,SM1,LOCK1,8086 */
        {I_XCHG, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66860, 179},
    /* 1026 : XCHG reg32,rm32 [rm:       hlenl o32 87 /r                   ] SM0-1,LOCK1,386 */
        /* 272 321 001(87) 110 : SM0,SM1,LOCK1,386 */
        {I_XCHG, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66866, 180},
    /* 1027 : XCHG reg64,rm64 [rm:       hlenl o64 87 /r                   ] SM0-1,LOCK1,LONG,PROT,X86_64 */
        /* 272 324 001(87) 110 : SM0,SM1,LOCK1,LONG,PROT,X86_64 */
        {I_XCHG, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66872, 181},
    /* 1028 : JMPABS imm64|near [i:        a64 np rex2 a1 iq                 ] LONG,PROT,APX,X86_64 */
        /* 313 360 350 001(a1) 054 : LONG,REX2,PROT,APX,X86_64 */
        {I_JMPABS, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55822, 186},
    /* 1029 : JMP imm16|near [i:        jmp8 nw o16 eb rel8               ] AR0,OSIZE,JMP_RELAX,NOREX,NOAPX,NOLONG,8086 */
        /* 371 327 320 001(eb) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,NOREX,NOAPX,NOLONG,8086 */
        {I_JMP, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55829, 189},
    /* 1030 : JMP imm32|near [i:        jmp8 nw o32 eb rel8               ] AR0,OSIZE,JMP_RELAX,NOREX,NOAPX,NOLONG,8086 */
        /* 371 327 321 001(eb) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,NOREX,NOAPX,NOLONG,8086 */
        {I_JMP, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55836, 189},
    /* 1031 : JMP imm64|near [i:        jmp8 nw o64 eb rel8               ] AR0,OSIZE,JMP_RELAX,NOAPX,LONG,PROT,X86_64 */
        /* 371 327 324 001(eb) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,NOAPX,LONG,PROT,X86_64 */
        {I_JMP, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55843, 190},
    /* 1032 : JMP imm16|near [i:        nw o16 e9 rel                     ] AR0,OSIZE,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 327 320 001(e9) 064 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JMP, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66896, 192},
    /* 1033 : JMP imm32|near [i:        nw o32 e9 rel                     ] AR0,OSIZE,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 327 321 001(e9) 064 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JMP, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66902, 192},
    /* 1034 : JMP imm64|near [i:        nw o64 e9 rel                     ] AR0,OSIZE,LONG,BND,PROT,X86_64 */
        /* 327 324 001(e9) 064 : AR0,NWSIZE,OSIZE,LONG,BND,PROT,X86_64 */
        {I_JMP, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66908, 193},
    /* 1035 : JMP rm16|near [m:        nw o16 ff /4                      ] AR0,OSIZE,BND,8086 */
        /* 327 320 001(ff) 204 : AR0,NWSIZE,OSIZE,BND,8086 */
        {I_JMP, 1, {RM_GPR|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66914, 195},
    /* 1036 : JMP rm32|near [m:        nw o32 ff /4                      ] AR0,OSIZE,BND,386 */
        /* 327 321 001(ff) 204 : AR0,NWSIZE,OSIZE,BND,386 */
        {I_JMP, 1, {RM_GPR|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66920, 196},
    /* 1037 : JMP rm64|near [m:        nw o64 ff /4                      ] AR0,OSIZE,LONG,BND,PROT,X86_64 */
        /* 327 324 001(ff) 204 : AR0,NWSIZE,OSIZE,LONG,BND,PROT,X86_64 */
        {I_JMP, 1, {RM_GPR|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66926, 193},
    /* 1038 : JMP imm16:imm16 [ji:       o16 ea iw iw                      ] AR1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 320 001(ea) 031 030 : AR1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_JMP, 2, {IMM_NORMAL|BITS16|COLON,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66944, 201},
    /* 1039 : JMP imm16:imm32 [ji:       o32 ea id iw                      ] AR1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 321 001(ea) 041 030 : AR1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_JMP, 2, {IMM_NORMAL|BITS16|COLON,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66950, 202},
    /* 1040 : JMP mem16|far [m:        o16 ff /5                         ] AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 320 001(ff) 205 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_JMP, 1, {MEMORY|BITS16|RN_L16|FAR|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71518, 187},
    /* 1041 : JMP mem32|far [m:        o32 ff /5                         ] AR0,NWSIZE,OSIZE,386 */
        /* 321 001(ff) 205 : AR0,NWSIZE,OSIZE,386 */
        {I_JMP, 1, {MEMORY|BITS32|FAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71523, 203},
    /* 1042 : JMP mem64|far [m:        o64 ff /5                         ] AR0,NWSIZE,OSIZE,LONG,386,X86_64 */
        /* 324 001(ff) 205 : AR0,NWSIZE,OSIZE,LONG,PROT,X86_64 */
        {I_JMP, 1, {MEMORY|BITS64|FAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71528, 204},
    /* 1043 : JA imm16|near [i:        jcc8 nw o16 77 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(77) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JA, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55850, 207},
    /* 1044 : JC imm16|near [i:        jcc8 nw o16 72 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(72) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JC, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55864, 207},
    /* 1045 : JG imm16|near [i:        jcc8 nw o16 7f rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(7f) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JG, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55885, 207},
    /* 1046 : JL imm16|near [i:        jcc8 nw o16 7c rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(7c) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JL, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55899, 207},
    /* 1047 : JNA imm16|near [i:        jcc8 nw o16 76 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(76) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNA, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55871, 207},
    /* 1048 : JNC imm16|near [i:        jcc8 nw o16 73 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(73) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNC, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55857, 207},
    /* 1049 : JNG imm16|near [i:        jcc8 nw o16 7e rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(7e) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNG, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55906, 207},
    /* 1050 : JNL imm16|near [i:        jcc8 nw o16 7d rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(7d) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNL, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55892, 207},
    /* 1051 : JNO imm16|near [i:        jcc8 nw o16 71 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(71) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNO, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55920, 207},
    /* 1052 : JNS imm16|near [i:        jcc8 nw o16 79 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(79) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNS, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55934, 207},
    /* 1053 : JNZ imm16|near [i:        jcc8 nw o16 75 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(75) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNZ, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55913, 207},
    /* 1054 : JO imm16|near [i:        jcc8 nw o16 70 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(70) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JO, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55941, 207},
    /* 1055 : JPE imm16|near [i:        jcc8 nw o16 7a rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(7a) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JPE, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55948, 207},
    /* 1056 : JPO imm16|near [i:        jcc8 nw o16 7b rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(7b) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JPO, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55927, 207},
    /* 1057 : JS imm16|near [i:        jcc8 nw o16 78 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(78) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JS, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55955, 207},
    /* 1058 : JZ imm16|near [i:        jcc8 nw o16 74 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(74) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JZ, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55878, 207},
    /* 1059 : JA imm32|near [i:        jcc8 nw o32 77 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(77) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JA, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55962, 207},
    /* 1060 : JC imm32|near [i:        jcc8 nw o32 72 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(72) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JC, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55976, 207},
    /* 1061 : JG imm32|near [i:        jcc8 nw o32 7f rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(7f) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JG, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55997, 207},
    /* 1062 : JL imm32|near [i:        jcc8 nw o32 7c rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(7c) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JL, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56011, 207},
    /* 1063 : JNA imm32|near [i:        jcc8 nw o32 76 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(76) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNA, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55983, 207},
    /* 1064 : JNC imm32|near [i:        jcc8 nw o32 73 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(73) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNC, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55969, 207},
    /* 1065 : JNG imm32|near [i:        jcc8 nw o32 7e rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(7e) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNG, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56018, 207},
    /* 1066 : JNL imm32|near [i:        jcc8 nw o32 7d rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(7d) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNL, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56004, 207},
    /* 1067 : JNO imm32|near [i:        jcc8 nw o32 71 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(71) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNO, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56032, 207},
    /* 1068 : JNS imm32|near [i:        jcc8 nw o32 79 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(79) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNS, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56046, 207},
    /* 1069 : JNZ imm32|near [i:        jcc8 nw o32 75 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(75) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNZ, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56025, 207},
    /* 1070 : JO imm32|near [i:        jcc8 nw o32 70 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(70) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JO, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56053, 207},
    /* 1071 : JPE imm32|near [i:        jcc8 nw o32 7a rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(7a) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JPE, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56060, 207},
    /* 1072 : JPO imm32|near [i:        jcc8 nw o32 7b rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(7b) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JPO, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56039, 207},
    /* 1073 : JS imm32|near [i:        jcc8 nw o32 78 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(78) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JS, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56067, 207},
    /* 1074 : JZ imm32|near [i:        jcc8 nw o32 74 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(74) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JZ, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55990, 207},
    /* 1075 : JA imm64|near [i:        jcc8 nw o64 77 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(77) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JA, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56074, 208},
    /* 1076 : JC imm64|near [i:        jcc8 nw o64 72 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(72) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JC, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56088, 208},
    /* 1077 : JG imm64|near [i:        jcc8 nw o64 7f rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(7f) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JG, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56109, 208},
    /* 1078 : JL imm64|near [i:        jcc8 nw o64 7c rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(7c) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JL, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56123, 208},
    /* 1079 : JNA imm64|near [i:        jcc8 nw o64 76 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(76) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNA, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56095, 208},
    /* 1080 : JNC imm64|near [i:        jcc8 nw o64 73 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(73) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNC, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56081, 208},
    /* 1081 : JNG imm64|near [i:        jcc8 nw o64 7e rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(7e) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNG, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56130, 208},
    /* 1082 : JNL imm64|near [i:        jcc8 nw o64 7d rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(7d) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNL, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56116, 208},
    /* 1083 : JNO imm64|near [i:        jcc8 nw o64 71 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(71) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNO, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56144, 208},
    /* 1084 : JNS imm64|near [i:        jcc8 nw o64 79 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(79) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNS, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56158, 208},
    /* 1085 : JNZ imm64|near [i:        jcc8 nw o64 75 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(75) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNZ, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56137, 208},
    /* 1086 : JO imm64|near [i:        jcc8 nw o64 70 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(70) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JO, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56165, 208},
    /* 1087 : JPE imm64|near [i:        jcc8 nw o64 7a rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(7a) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JPE, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56172, 208},
    /* 1088 : JPO imm64|near [i:        jcc8 nw o64 7b rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(7b) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JPO, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56151, 208},
    /* 1089 : JS imm64|near [i:        jcc8 nw o64 78 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(78) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JS, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56179, 208},
    /* 1090 : JZ imm64|near [i:        jcc8 nw o64 74 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(74) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JZ, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56102, 208},
    /* 1091 : JA imm16|near [i:        nw o16 0f 87 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(87) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JA, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56186, 210},
    /* 1092 : JC imm16|near [i:        nw o16 0f 82 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(82) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JC, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56200, 210},
    /* 1093 : JG imm16|near [i:        nw o16 0f 8f rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(8f) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JG, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56221, 210},
    /* 1094 : JL imm16|near [i:        nw o16 0f 8c rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(8c) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JL, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56235, 210},
    /* 1095 : JNA imm16|near [i:        nw o16 0f 86 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(86) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNA, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56207, 210},
    /* 1096 : JNC imm16|near [i:        nw o16 0f 83 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(83) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNC, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56193, 210},
    /* 1097 : JNG imm16|near [i:        nw o16 0f 8e rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(8e) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNG, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56242, 210},
    /* 1098 : JNL imm16|near [i:        nw o16 0f 8d rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(8d) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNL, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56228, 210},
    /* 1099 : JNO imm16|near [i:        nw o16 0f 81 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(81) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNO, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56256, 210},
    /* 1100 : JNS imm16|near [i:        nw o16 0f 89 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(89) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNS, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56270, 210},
    /* 1101 : JNZ imm16|near [i:        nw o16 0f 85 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(85) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNZ, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56249, 210},
    /* 1102 : JO imm16|near [i:        nw o16 0f 80 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(80) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JO, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56277, 210},
    /* 1103 : JPE imm16|near [i:        nw o16 0f 8a rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(8a) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JPE, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56284, 210},
    /* 1104 : JPO imm16|near [i:        nw o16 0f 8b rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(8b) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JPO, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56263, 210},
    /* 1105 : JS imm16|near [i:        nw o16 0f 88 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(88) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JS, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56291, 210},
    /* 1106 : JZ imm16|near [i:        nw o16 0f 84 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(84) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JZ, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56214, 210},
    /* 1107 : JA imm32|near [i:        nw o32 0f 87 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(87) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JA, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56298, 210},
    /* 1108 : JC imm32|near [i:        nw o32 0f 82 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(82) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JC, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56312, 210},
    /* 1109 : JG imm32|near [i:        nw o32 0f 8f rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(8f) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JG, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56333, 210},
    /* 1110 : JL imm32|near [i:        nw o32 0f 8c rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(8c) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JL, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56347, 210},
    /* 1111 : JNA imm32|near [i:        nw o32 0f 86 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(86) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNA, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56319, 210},
    /* 1112 : JNC imm32|near [i:        nw o32 0f 83 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(83) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNC, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56305, 210},
    /* 1113 : JNG imm32|near [i:        nw o32 0f 8e rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(8e) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNG, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56354, 210},
    /* 1114 : JNL imm32|near [i:        nw o32 0f 8d rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(8d) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNL, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56340, 210},
    /* 1115 : JNO imm32|near [i:        nw o32 0f 81 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(81) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNO, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56368, 210},
    /* 1116 : JNS imm32|near [i:        nw o32 0f 89 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(89) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNS, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56382, 210},
    /* 1117 : JNZ imm32|near [i:        nw o32 0f 85 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(85) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNZ, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56361, 210},
    /* 1118 : JO imm32|near [i:        nw o32 0f 80 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(80) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JO, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56389, 210},
    /* 1119 : JPE imm32|near [i:        nw o32 0f 8a rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(8a) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JPE, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56396, 210},
    /* 1120 : JPO imm32|near [i:        nw o32 0f 8b rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(8b) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JPO, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56375, 210},
    /* 1121 : JS imm32|near [i:        nw o32 0f 88 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(88) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JS, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56403, 210},
    /* 1122 : JZ imm32|near [i:        nw o32 0f 84 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(84) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JZ, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56326, 210},
    /* 1123 : JA imm64|near [i:        nw o64 0f 87 rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(87) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JA, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56410, 206},
    /* 1124 : JC imm64|near [i:        nw o64 0f 82 rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(82) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JC, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56424, 206},
    /* 1125 : JG imm64|near [i:        nw o64 0f 8f rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(8f) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JG, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56445, 206},
    /* 1126 : JL imm64|near [i:        nw o64 0f 8c rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(8c) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JL, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56459, 206},
    /* 1127 : JNA imm64|near [i:        nw o64 0f 86 rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(86) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNA, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56431, 206},
    /* 1128 : JNC imm64|near [i:        nw o64 0f 83 rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(83) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNC, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56417, 206},
    /* 1129 : JNG imm64|near [i:        nw o64 0f 8e rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(8e) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNG, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56466, 206},
    /* 1130 : JNL imm64|near [i:        nw o64 0f 8d rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(8d) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNL, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56452, 206},
    /* 1131 : JNO imm64|near [i:        nw o64 0f 81 rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(81) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNO, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56480, 206},
    /* 1132 : JNS imm64|near [i:        nw o64 0f 89 rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(89) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNS, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56494, 206},
    /* 1133 : JNZ imm64|near [i:        nw o64 0f 85 rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(85) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNZ, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56473, 206},
    /* 1134 : JO imm64|near [i:        nw o64 0f 80 rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(80) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JO, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56501, 206},
    /* 1135 : JPE imm64|near [i:        nw o64 0f 8a rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(8a) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JPE, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56508, 206},
    /* 1136 : JPO imm64|near [i:        nw o64 0f 8b rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(8b) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JPO, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56487, 206},
    /* 1137 : JS imm64|near [i:        nw o64 0f 88 rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(88) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JS, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56515, 206},
    /* 1138 : JZ imm64|near [i:        nw o64 0f 84 rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(84) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JZ, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56438, 206},
    /* 1139 : JCXZ imm16|near|short [i:        a16 nw o16 e3 rel8                ] AR0,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 310 327 320 001(e3) 050 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_JCXZ, 1, {IMM_NORMAL|BITS16|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56634, 187},
    /* 1140 : JECXZ imm16|near|short [i:        a32 nw o16 e3 rel8                ] AR0,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 311 327 320 001(e3) 050 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_JECXZ, 1, {IMM_NORMAL|BITS16|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56641, 213},
    /* 1141 : JCXZ imm32|near|short [i:        a16 nw o32 e3 rel8                ] AR0,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 310 327 321 001(e3) 050 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_JCXZ, 1, {IMM_NORMAL|BITS32|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56648, 187},
    /* 1142 : JECXZ imm32|near|short [i:        a32 nw o32 e3 rel8                ] AR0,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 311 327 321 001(e3) 050 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_JECXZ, 1, {IMM_NORMAL|BITS32|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56655, 213},
    /* 1143 : JECXZ imm64|near|short [i:        a32 nw o64 e3 rel8                ] AR0,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 311 327 324 001(e3) 050 : AR0,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_JECXZ, 1, {IMM_NORMAL|BITS64|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56662, 188},
    /* 1144 : JRCXZ imm64|near|short [i:        a64 nw o64 e3 rel8                ] AR0,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 313 327 324 001(e3) 050 : AR0,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_JRCXZ, 1, {IMM_NORMAL|BITS64|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56669, 188},
    /* 1145 : LOOP imm16|near|short [i:        asm nw o16 e2 rel8                ] AR0,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 312 327 320 001(e2) 050 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOP, 1, {IMM_NORMAL|BITS16|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56676, 187},
    /* 1146 : LOOP imm32|near|short [i:        asm nw o32 e2 rel8                ] AR0,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 312 327 321 001(e2) 050 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOP, 1, {IMM_NORMAL|BITS32|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56697, 187},
    /* 1147 : LOOP imm64|near|short [i:        asm nw o64 e2 rel8                ] AR0,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 312 327 324 001(e2) 050 : AR0,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_LOOP, 1, {IMM_NORMAL|BITS64|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56718, 188},
    /* 1148 : LOOPE imm16|near|short [i:        asm nw o16 e1 rel8                ] AR0,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 312 327 320 001(e1) 050 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOPE, 1, {IMM_NORMAL|BITS16|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56739, 187},
    /* 1149 : LOOPE imm32|near|short [i:        asm nw o32 e1 rel8                ] AR0,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 312 327 321 001(e1) 050 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOPE, 1, {IMM_NORMAL|BITS32|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56760, 187},
    /* 1150 : LOOPE imm64|near|short [i:        asm nw o64 e1 rel8                ] AR0,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 312 327 324 001(e1) 050 : AR0,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_LOOPE, 1, {IMM_NORMAL|BITS64|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56781, 188},
    /* 1151 : LOOPNE imm16|near|short [i:        asm nw o16 e0 rel8                ] AR0,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 312 327 320 001(e0) 050 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOPNE, 1, {IMM_NORMAL|BITS16|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56802, 187},
    /* 1152 : LOOPNE imm32|near|short [i:        asm nw o32 e0 rel8                ] AR0,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 312 327 321 001(e0) 050 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOPNE, 1, {IMM_NORMAL|BITS32|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56823, 187},
    /* 1153 : LOOPNE imm64|near|short [i:        asm nw o64 e0 rel8                ] AR0,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 312 327 324 001(e0) 050 : AR0,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_LOOPNE, 1, {IMM_NORMAL|BITS64|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56844, 188},
    /* 1154 : LOOP imm16|near|short,reg_cx [i-:       a16 nw o16 e2 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 310 327 320 001(e2) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOP, 2, {IMM_NORMAL|BITS16|NEAR|SHORT,REG_CX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56683, 214},
    /* 1155 : LOOP imm16|near|short,reg_ecx [i-:       a32 nw o16 e2 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 311 327 320 001(e2) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_LOOP, 2, {IMM_NORMAL|BITS16|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56690, 215},
    /* 1156 : LOOP imm32|near|short,reg_cx [i-:       a16 nw o32 e2 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 310 327 321 001(e2) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOP, 2, {IMM_NORMAL|BITS32|NEAR|SHORT,REG_CX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56704, 214},
    /* 1157 : LOOP imm32|near|short,reg_ecx [i-:       a32 nw o32 e2 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 311 327 321 001(e2) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_LOOP, 2, {IMM_NORMAL|BITS32|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56711, 215},
    /* 1158 : LOOP imm64|near|short,reg_ecx [i-:       a32 nw o64 e2 rel8                ] AR0-1,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 311 327 324 001(e2) 050 : AR0,AR1,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_LOOP, 2, {IMM_NORMAL|BITS64|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56725, 216},
    /* 1159 : LOOP imm64|near|short,reg_rcx [i-:       a64 nw o64 e2 rel8                ] AR0-1,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 313 327 324 001(e2) 050 : AR0,AR1,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_LOOP, 2, {IMM_NORMAL|BITS64|NEAR|SHORT,REG_RCX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56732, 216},
    /* 1160 : LOOPE imm16|near|short,reg_cx [i-:       a16 nw o16 e1 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 310 327 320 001(e1) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOPE, 2, {IMM_NORMAL|BITS16|NEAR|SHORT,REG_CX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56746, 214},
    /* 1161 : LOOPE imm16|near|short,reg_ecx [i-:       a32 nw o16 e1 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 311 327 320 001(e1) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_LOOPE, 2, {IMM_NORMAL|BITS16|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56753, 215},
    /* 1162 : LOOPE imm32|near|short,reg_cx [i-:       a16 nw o32 e1 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 310 327 321 001(e1) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOPE, 2, {IMM_NORMAL|BITS32|NEAR|SHORT,REG_CX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56767, 214},
    /* 1163 : LOOPE imm32|near|short,reg_ecx [i-:       a32 nw o32 e1 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 311 327 321 001(e1) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_LOOPE, 2, {IMM_NORMAL|BITS32|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56774, 215},
    /* 1164 : LOOPE imm64|near|short,reg_ecx [i-:       a32 nw o64 e1 rel8                ] AR0-1,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 311 327 324 001(e1) 050 : AR0,AR1,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_LOOPE, 2, {IMM_NORMAL|BITS64|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56788, 216},
    /* 1165 : LOOPE imm64|near|short,reg_rcx [i-:       a64 nw o64 e1 rel8                ] AR0-1,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 313 327 324 001(e1) 050 : AR0,AR1,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_LOOPE, 2, {IMM_NORMAL|BITS64|NEAR|SHORT,REG_RCX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56795, 216},
    /* 1166 : LOOPNE imm16|near|short,reg_cx [i-:       a16 nw o16 e0 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 310 327 320 001(e0) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOPNE, 2, {IMM_NORMAL|BITS16|NEAR|SHORT,REG_CX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56809, 214},
    /* 1167 : LOOPNE imm16|near|short,reg_ecx [i-:       a32 nw o16 e0 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 311 327 320 001(e0) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_LOOPNE, 2, {IMM_NORMAL|BITS16|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56816, 215},
    /* 1168 : LOOPNE imm32|near|short,reg_cx [i-:       a16 nw o32 e0 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 310 327 321 001(e0) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOPNE, 2, {IMM_NORMAL|BITS32|NEAR|SHORT,REG_CX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56830, 214},
    /* 1169 : LOOPNE imm32|near|short,reg_ecx [i-:       a32 nw o32 e0 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 311 327 321 001(e0) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_LOOPNE, 2, {IMM_NORMAL|BITS32|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56837, 215},
    /* 1170 : LOOPNE imm64|near|short,reg_ecx [i-:       a32 nw o64 e0 rel8                ] AR0-1,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 311 327 324 001(e0) 050 : AR0,AR1,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_LOOPNE, 2, {IMM_NORMAL|BITS64|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56851, 216},
    /* 1171 : LOOPNE imm64|near|short,reg_rcx [i-:       a64 nw o64 e0 rel8                ] AR0-1,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 313 327 324 001(e0) 050 : AR0,AR1,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_LOOPNE, 2, {IMM_NORMAL|BITS64|NEAR|SHORT,REG_RCX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56858, 216},
    /* 1172 : LOOPZ imm16|near|short,reg_cx [i-:       a16 nw o16 e1 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 310 327 320 001(e1) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOPZ, 2, {IMM_NORMAL|BITS16|NEAR|SHORT,REG_CX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56746, 214},
    /* 1173 : LOOPZ imm16|near|short,reg_ecx [i-:       a32 nw o16 e1 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 311 327 320 001(e1) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_LOOPZ, 2, {IMM_NORMAL|BITS16|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56753, 215},
    /* 1174 : LOOPZ imm32|near|short,reg_cx [i-:       a16 nw o32 e1 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 310 327 321 001(e1) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOPZ, 2, {IMM_NORMAL|BITS32|NEAR|SHORT,REG_CX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56767, 214},
    /* 1175 : LOOPZ imm32|near|short,reg_ecx [i-:       a32 nw o32 e1 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 311 327 321 001(e1) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_LOOPZ, 2, {IMM_NORMAL|BITS32|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56774, 215},
    /* 1176 : LOOPZ imm64|near|short,reg_ecx [i-:       a32 nw o64 e1 rel8                ] AR0-1,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 311 327 324 001(e1) 050 : AR0,AR1,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_LOOPZ, 2, {IMM_NORMAL|BITS64|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56788, 216},
    /* 1177 : LOOPZ imm64|near|short,reg_rcx [i-:       a64 nw o64 e1 rel8                ] AR0-1,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 313 327 324 001(e1) 050 : AR0,AR1,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_LOOPZ, 2, {IMM_NORMAL|BITS64|NEAR|SHORT,REG_RCX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56795, 216},
    /* 1178 : LOOPNZ imm16|near|short,reg_cx [i-:       a16 nw o16 e0 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 310 327 320 001(e0) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOPNZ, 2, {IMM_NORMAL|BITS16|NEAR|SHORT,REG_CX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56809, 214},
    /* 1179 : LOOPNZ imm16|near|short,reg_ecx [i-:       a32 nw o16 e0 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 311 327 320 001(e0) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_LOOPNZ, 2, {IMM_NORMAL|BITS16|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56816, 215},
    /* 1180 : LOOPNZ imm32|near|short,reg_cx [i-:       a16 nw o32 e0 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 310 327 321 001(e0) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOPNZ, 2, {IMM_NORMAL|BITS32|NEAR|SHORT,REG_CX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56830, 214},
    /* 1181 : LOOPNZ imm32|near|short,reg_ecx [i-:       a32 nw o32 e0 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 311 327 321 001(e0) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_LOOPNZ, 2, {IMM_NORMAL|BITS32|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56837, 215},
    /* 1182 : LOOPNZ imm64|near|short,reg_ecx [i-:       a32 nw o64 e0 rel8                ] AR0-1,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 311 327 324 001(e0) 050 : AR0,AR1,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_LOOPNZ, 2, {IMM_NORMAL|BITS64|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56851, 216},
    /* 1183 : LOOPNZ imm64|near|short,reg_rcx [i-:       a64 nw o64 e0 rel8                ] AR0-1,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 313 327 324 001(e0) 050 : AR0,AR1,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_LOOPNZ, 2, {IMM_NORMAL|BITS64|NEAR|SHORT,REG_RCX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56858, 216},
    /* 1184 : JMPE imm16|near [i:        nw o16 0f b8 rel                  ] AR0,OSIZE,NOREX,NOAPX,NOLONG,IA64 */
        /* 327 320 355 001(b8) 064 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,IA64 */
        {I_JMPE, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56865, 217},
    /* 1185 : JMPE imm32|near [i:        nw o32 0f b8 rel                  ] AR0,OSIZE,NOREX,NOAPX,NOLONG,IA64 */
        /* 327 321 355 001(b8) 064 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,IA64 */
        {I_JMPE, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56872, 217},
    /* 1186 : JMPE imm64|near [i:        nw o64 0f b8 rel                  ] AR0,OSIZE,LONG,PROT,X86_64 */
        /* 327 324 355 001(b8) 064 : AR0,NWSIZE,OSIZE,LONG,PROT,X86_64 */
        {I_JMPE, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56879, 204},
    /* 1187 : JMPE rm16|near [m:        nw o16 np 0f 00 /6                ] AR0,OSIZE,IA64 */
        /* 327 320 360 355 001(00) 206 : AR0,NWSIZE,OSIZE,IA64 */
        {I_JMPE, 1, {RM_GPR|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+51157, 219},
    /* 1188 : JMPE rm32|near [m:        nw o32 np 0f 00 /6                ] AR0,OSIZE,IA64 */
        /* 327 321 360 355 001(00) 206 : AR0,NWSIZE,OSIZE,IA64 */
        {I_JMPE, 1, {RM_GPR|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+51165, 219},
    /* 1189 : JMPE rm64|near [m:        nw o64 np 0f 00 /6                ] AR0,OSIZE,LONG,PROT,X86_64 */
        /* 327 324 360 355 001(00) 206 : AR0,NWSIZE,OSIZE,LONG,PROT,X86_64 */
        {I_JMPE, 1, {RM_GPR|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+51173, 204},
    /* 1190 : JMPE rm16|near [m:        o16 0f 00 /6                      ] AR0,OSIZE,NOREX,NOAPX,NOLONG,IA64 */
        /* 320 355 001(00) 206 : AR0,OSIZE,NOREX,NOAPX,NOLONG,IA64 */
        {I_JMPE, 1, {RM_GPR|BITS16|RN_L16|NEAR|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66956, 220},
    /* 1191 : JMPE rm32|near [m:        o32 0f 00 /6                      ] AR0,OSIZE,NOREX,NOAPX,NOLONG,IA64 */
        /* 321 355 001(00) 206 : AR0,OSIZE,NOREX,NOAPX,NOLONG,IA64 */
        {I_JMPE, 1, {RM_GPR|BITS32|RN_L16|NEAR|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66962, 220},
    /* 1192 : CALL imm16|near [i:        nw o16 e8 rel                     ] AR0,OSIZE,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 327 320 001(e8) 064 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_CALL, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66968, 192},
    /* 1193 : CALL imm32|near [i:        nw o32 e8 rel                     ] AR0,OSIZE,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 327 321 001(e8) 064 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_CALL, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66974, 192},
    /* 1194 : CALL imm64|near [i:        nw o64 e8 rel                     ] AR0,OSIZE,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 001(e8) 064 : AR0,NWSIZE,OSIZE,NOAPX,LONG,BND,PROT,X86_64 */
        {I_CALL, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66980, 221},
    /* 1195 : CALL rm16|near [m:        nw o16 ff /2                      ] AR0,OSIZE,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 327 320 001(ff) 202 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_CALL, 1, {RM_GPR|BITS16|RN_L16|NEAR|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66986, 192},
    /* 1196 : CALL rm32|near [m:        nw o32 ff /2                      ] AR0,OSIZE,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 001(ff) 202 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,BND,386 */
        {I_CALL, 1, {RM_GPR|BITS32|RN_L16|NEAR|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66992, 223},
    /* 1197 : CALL rm64|near [m:        nw o64 ff /2                      ] LONG,BND,PROT,X86_64 */
        /* 327 324 001(ff) 202 : NWSIZE,LONG,BND,PROT,X86_64 */
        {I_CALL, 1, {RM_GPR|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66998, 224},
    /* 1198 : CALL imm16:imm16 [ji:       o16 9a iw iw                      ] AR1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 320 001(9a) 031 030 : AR1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_CALL, 2, {IMM_NORMAL|BITS16|COLON,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67016, 201},
    /* 1199 : CALL imm16:imm32 [ji:       o32 9a id iw                      ] AR1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 321 001(9a) 041 030 : AR1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_CALL, 2, {IMM_NORMAL|BITS16|COLON,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+67022, 202},
    /* 1200 : CALL mem16|far [m:        o16 ff /3                         ] AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 320 001(ff) 203 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_CALL, 1, {MEMORY|BITS16|RN_L16|FAR|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71533, 187},
    /* 1201 : CALL mem32|far [m:        o32 ff /3                         ] AR0,NWSIZE,OSIZE,386 */
        /* 321 001(ff) 203 : AR0,NWSIZE,OSIZE,386 */
        {I_CALL, 1, {MEMORY|BITS32|FAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71538, 203},
    /* 1202 : CALL mem64|far [m:        o64 ff /3                         ] AR0,NWSIZE,OSIZE,LONG,386,X86_64 */
        /* 324 001(ff) 203 : AR0,NWSIZE,OSIZE,LONG,PROT,X86_64 */
        {I_CALL, 1, {MEMORY|BITS64|FAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71543, 204},
    /* 1203 : RET void [          nw osm c3                         ] BND,8086 */
        /* 327 342 001(c3) : NWSIZE,BND,8086 */
        {I_RET, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71548, 225},
    /* 1204 : RETW void [          nw o16 c3                         ] BND,8086 */
        /* 327 320 001(c3) : NWSIZE,BND,8086 */
        {I_RETW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71553, 225},
    /* 1205 : RETD void [          nw o32 c3                         ] NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 001(c3) : NWSIZE,NOREX,NOAPX,NOLONG,BND,386 */
        {I_RETD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71558, 226},
    /* 1206 : RETQ void [          nw o64 c3                         ] LONG,BND,386,X86_64 */
        /* 327 324 001(c3) : NWSIZE,LONG,BND,PROT,X86_64 */
        {I_RETQ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71563, 224},
    /* 1207 : RET imm16 [i:        nw osm c2 iw                      ] BND,8086 */
        /* 327 342 001(c2) 030 : NWSIZE,BND,8086 */
        {I_RET, 1, {IMM_NORMAL|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67028, 225},
    /* 1208 : RETW imm16 [i:        nw o16 c2 iw                      ] BND,8086 */
        /* 327 320 001(c2) 030 : NWSIZE,BND,8086 */
        {I_RETW, 1, {IMM_NORMAL|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67034, 225},
    /* 1209 : RETD imm16 [i:        nw o32 c2 iw                      ] NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 001(c2) 030 : NWSIZE,NOREX,NOAPX,NOLONG,BND,386 */
        {I_RETD, 1, {IMM_NORMAL|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67040, 226},
    /* 1210 : RETQ imm16 [i:        nw o64 c2 iw                      ] LONG,BND,386,X86_64 */
        /* 327 324 001(c2) 030 : NWSIZE,LONG,BND,PROT,X86_64 */
        {I_RETQ, 1, {IMM_NORMAL|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67046, 224},
    /* 1211 : RETF void [          odf cb                            ] 8086 */
        /* 322 001(cb) : 8086 */
        {I_RETF, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72589, 23},
    /* 1212 : RETFW void [          o16 cb                            ] 8086 */
        /* 320 001(cb) : 8086 */
        {I_RETFW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72593, 23},
    /* 1213 : RETFD void [          o32 cb                            ] 386 */
        /* 321 001(cb) : 386 */
        {I_RETFD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72597, 3},
    /* 1214 : RETFQ void [          o64 cb                            ] LONG,386,X86_64 */
        /* 324 001(cb) : LONG,PROT,X86_64 */
        {I_RETFQ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72601, 24},
    /* 1215 : RETF imm16 [i:        odf ca iw                         ] 8086 */
        /* 322 001(ca) 030 : 8086 */
        {I_RETF, 1, {IMM_NORMAL|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71568, 23},
    /* 1216 : RETFW imm16 [i:        o16 ca iw                         ] 8086 */
        /* 320 001(ca) 030 : 8086 */
        {I_RETFW, 1, {IMM_NORMAL|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71573, 23},
    /* 1217 : RETFD imm16 [i:        o32 ca iw                         ] 386 */
        /* 321 001(ca) 030 : 386 */
        {I_RETFD, 1, {IMM_NORMAL|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71578, 3},
    /* 1218 : RETFQ imm16 [i:        o64 ca iw                         ] LONG,386,X86_64 */
        /* 324 001(ca) 030 : LONG,PROT,X86_64 */
        {I_RETFQ, 1, {IMM_NORMAL|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71583, 24},
    /* 1219 : INT imm8 [i:        cd ib,u                           ] 8086 */
        /* 001(cd) 024 : 8086 */
        {I_INT, 1, {IMM_NORMAL|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72605, 23},
    /* 1220 : INT1 void [          f1                                ] 386 */
        /* 001(f1) : 386 */
        {I_INT1, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73302, 3},
    /* 1221 : INT3 void [          cc                                ] 8086 */
        /* 001(cc) : 8086 */
        {I_INT3, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73305, 23},
    /* 1222 : INTO void [          ce                                ] NOREX,NOAPX,NOLONG,8086 */
        /* 001(ce) : NOREX,NOAPX,NOLONG,8086 */
        {I_INTO, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73308, 227},
    /* 1223 : SYSCALL void [          0f 05                             ] P6,AMD */
        /* 355 001(05) : P6,AMD */
        {I_SYSCALL, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72609, 228},
    /* 1224 : SYSENTER void [          0f 34                             ] NOAPX,P6 */
        /* 355 001(34) : NOAPX,P6 */
        {I_SYSENTER, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72613, 229},
    /* 1225 : SYSEXIT void [          0f 35                             ] NOAPX,PRIV,P6 */
        /* 355 001(35) : NOAPX,PRIV,P6 */
        {I_SYSEXIT, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72617, 230},
    /* 1226 : SYSRET void [          0f 07                             ] PRIV,P6,AMD */
        /* 355 001(07) : PRIV,P6,AMD */
        {I_SYSRET, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72621, 231},
    /* 1227 : IRET void [          osm cf                            ] FL,8086 */
        /* 342 001(cf) : FL,8086 */
        {I_IRET, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72625, 54},
    /* 1228 : IRETW void [          o16 cf                            ] FL,8086 */
        /* 320 001(cf) : FL,8086 */
        {I_IRETW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72629, 54},
    /* 1229 : IRETD void [          o32 cf                            ] FL,386 */
        /* 321 001(cf) : FL,386 */
        {I_IRETD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72633, 55},
    /* 1230 : IRETQ void [          o64 cf                            ] LONG,FL,386,X86_64 */
        /* 324 001(cf) : LONG,FL,PROT,X86_64 */
        {I_IRETQ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72637, 56},
    /* 1231 : ERETS void [          f2 0f 01 ca                       ] LONG,PRIV,PROT,FRED,X86_64 */
        /* 332 355 002(01 ca) : LONG,PRIV,PROT,FRED,X86_64 */
        {I_ERETS, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67052, 232},
    /* 1232 : ERETU void [          f3 0f 01 ca                       ] LONG,PRIV,PROT,FRED,X86_64 */
        /* 333 355 002(01 ca) : LONG,PRIV,PROT,FRED,X86_64 */
        {I_ERETU, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67058, 232},
    /* 1233 : CLC void [          f8                                ] FL,8086 */
        /* 001(f8) : FL,8086 */
        {I_CLC, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71785, 54},
    /* 1234 : CLD void [          fc                                ] FL,8086 */
        /* 001(fc) : FL,8086 */
        {I_CLD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67139, 54},
    /* 1235 : CLI void [          fa                                ] FL,8086 */
        /* 001(fa) : FL,8086 */
        {I_CLI, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67349, 54},
    /* 1236 : CLAC void [          np 0f 01 ca                       ] FL,PRIV,SMAP,FUTURE */
        /* 360 355 002(01 ca) : FL,PRIV,SMAP,FUTURE */
        {I_CLAC, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67064, 233},
    /* 1237 : STC void [          f9                                ] FL,8086 */
        /* 001(f9) : FL,8086 */
        {I_STC, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71675, 54},
    /* 1238 : STD void [          fd                                ] FL,8086 */
        /* 001(fd) : FL,8086 */
        {I_STD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73311, 54},
    /* 1239 : STI void [          fb                                ] FL,8086 */
        /* 001(fb) : FL,8086 */
        {I_STI, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71725, 54},
    /* 1240 : STAC void [          np 0f 01 cb                       ] FL,PRIV,SMAP,FUTURE */
        /* 360 355 002(01 cb) : FL,PRIV,SMAP,FUTURE */
        {I_STAC, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67070, 233},
    /* 1241 : CMC void [          f5                                ] FL,8086 */
        /* 001(f5) : FL,8086 */
        {I_CMC, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73314, 54},
    /* 1242 : LAHF void [          9f                                ] 8086 */
        /* 001(9f) : 8086 */
        {I_LAHF, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73317, 23},
    /* 1243 : SAHF void [          9e                                ] FL,8086 */
        /* 001(9e) : FL,8086 */
        {I_SAHF, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57226, 54},
    /* 1244 : SALC void [          d6                                ] NOREX,NOAPX,NOLONG,UNDOC,8086 */
        /* 001(d6) : NOREX,NOAPX,NOLONG,UNDOC,8086 */
        {I_SALC, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72380, 234},
    /* 1245 : PUSHF void [          nw osm 9c                         ] 8086 */
        /* 327 342 001(9c) : NWSIZE,8086 */
        {I_PUSHF, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71588, 235},
    /* 1246 : PUSHFW void [          nw o16 9c                         ] 8086 */
        /* 327 320 001(9c) : NWSIZE,8086 */
        {I_PUSHFW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71593, 235},
    /* 1247 : PUSHFD void [          nw o32 9c                         ] 386 */
        /* 327 321 001(9c) : NWSIZE,386 */
        {I_PUSHFD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71598, 236},
    /* 1248 : PUSHFQ void [          nw o64 9c                         ] LONG,386,X86_64 */
        /* 327 324 001(9c) : NWSIZE,LONG,PROT,X86_64 */
        {I_PUSHFQ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71603, 237},
    /* 1249 : POPF void [          nw osm 9d                         ] FL,8086 */
        /* 327 342 001(9d) : NWSIZE,FL,8086 */
        {I_POPF, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71608, 238},
    /* 1250 : POPFW void [          nw o16 9d                         ] FL,8086 */
        /* 327 320 001(9d) : NWSIZE,FL,8086 */
        {I_POPFW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71613, 238},
    /* 1251 : POPFD void [          nw o32 9d                         ] FL,386 */
        /* 327 321 001(9d) : NWSIZE,FL,386 */
        {I_POPFD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71618, 239},
    /* 1252 : POPFQ void [          nw o64 9d                         ] LONG,FL,386,X86_64 */
        /* 327 324 001(9d) : NWSIZE,LONG,FL,PROT,X86_64 */
        {I_POPFQ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71623, 240},
    /* 1253 : CMPSB void [          repe o8 a6                        ] NOAPX,FL,8086 */
        /* 335 001(a6) : NOAPX,FL,8086 */
        {I_CMPSB, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72641, 241},
    /* 1254 : CMPSW void [          repe o16 a7                       ] NOAPX,FL,8086 */
        /* 335 320 001(a7) : NOAPX,FL,8086 */
        {I_CMPSW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71628, 241},
    /* 1255 : CMPSD void [          repe o32 a7                       ] NOAPX,FL,386 */
        /* 335 321 001(a7) : NOAPX,FL,386 */
        {I_CMPSD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71633, 242},
    /* 1256 : CMPSQ void [          repe o64 a7                       ] NOAPX,LONG,FL,386,X86_64 */
        /* 335 324 001(a7) : NOAPX,LONG,FL,PROT,X86_64 */
        {I_CMPSQ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71638, 243},
    /* 1257 : LODSB void [          o8 ac                             ] NOAPX,8086 */
        /* 001(ac) : NOAPX,8086 */
        {I_LODSB, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73320, 244},
    /* 1258 : LODSW void [          o16 ad                            ] NOAPX,8086 */
        /* 320 001(ad) : NOAPX,8086 */
        {I_LODSW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72645, 244},
    /* 1259 : LODSD void [          o32 ad                            ] NOAPX,386 */
        /* 321 001(ad) : NOAPX,386 */
        {I_LODSD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72649, 245},
    /* 1260 : LODSQ void [          o64 ad                            ] NOAPX,LONG,386,X86_64 */
        /* 324 001(ad) : NOAPX,LONG,PROT,X86_64 */
        {I_LODSQ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72653, 246},
    /* 1261 : MOVSB void [          o8 a4                             ] NOAPX,8086 */
        /* 001(a4) : NOAPX,8086 */
        {I_MOVSB, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57254, 244},
    /* 1262 : MOVSW void [          o16 a5                            ] NOAPX,8086 */
        /* 320 001(a5) : NOAPX,8086 */
        {I_MOVSW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72657, 244},
    /* 1263 : MOVSD void [          o32 a5                            ] NOAPX,386 */
        /* 321 001(a5) : NOAPX,386 */
        {I_MOVSD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72661, 245},
    /* 1264 : MOVSQ void [          o64 a5                            ] NOAPX,LONG,386,X86_64 */
        /* 324 001(a5) : NOAPX,LONG,PROT,X86_64 */
        {I_MOVSQ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72665, 246},
    /* 1265 : STOSB void [          o8 aa                             ] NOAPX,8086 */
        /* 001(aa) : NOAPX,8086 */
        {I_STOSB, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72758, 244},
    /* 1266 : STOSW void [          o16 ab                            ] NOAPX,8086 */
        /* 320 001(ab) : NOAPX,8086 */
        {I_STOSW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72669, 244},
    /* 1267 : STOSD void [          o32 ab                            ] NOAPX,386 */
        /* 321 001(ab) : NOAPX,386 */
        {I_STOSD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72673, 245},
    /* 1268 : STOSQ void [          o64 ab                            ] NOAPX,LONG,386,X86_64 */
        /* 324 001(ab) : NOAPX,LONG,PROT,X86_64 */
        {I_STOSQ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72677, 246},
    /* 1269 : SCASB void [          repe o8 ae                        ] NOAPX,FL,8086 */
        /* 335 001(ae) : NOAPX,FL,8086 */
        {I_SCASB, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72681, 241},
    /* 1270 : SCASW void [          repe o16 af                       ] NOAPX,FL,8086 */
        /* 335 320 001(af) : NOAPX,FL,8086 */
        {I_SCASW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71643, 241},
    /* 1271 : SCASD void [          repe o32 af                       ] NOAPX,FL,386 */
        /* 335 321 001(af) : NOAPX,FL,386 */
        {I_SCASD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71648, 242},
    /* 1272 : SCASQ void [          repe o64 af                       ] NOAPX,LONG,FL,386,X86_64 */
        /* 335 324 001(af) : NOAPX,LONG,FL,PROT,X86_64 */
        {I_SCASQ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71653, 243},
    /* 1273 : INSB void [          o8 6c                             ] 186 */
        /* 001(6c) : 186 */
        {I_INSB, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73323, 247},
    /* 1274 : INSW void [          o16 6d                            ] 186 */
        /* 320 001(6d) : 186 */
        {I_INSW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72685, 247},
    /* 1275 : INSD void [          o32 6d                            ] 386 */
        /* 321 001(6d) : 386 */
        {I_INSD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72689, 3},
    /* 1276 : OUTSB void [          o8 6e                             ] 186 */
        /* 001(6e) : 186 */
        {I_OUTSB, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73326, 247},
    /* 1277 : OUTSW void [          o16 6f                            ] 186 */
        /* 320 001(6f) : 186 */
        {I_OUTSW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72693, 247},
    /* 1278 : OUTSD void [          o32 6f                            ] 386 */
        /* 321 001(6f) : 386 */
        {I_OUTSD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72697, 3},
    /* 1279 : LFENCE void [          np 0f ae e8                       ] LONG,PROT,X86_64,AMD */
        /* 360 355 002(ae e8) : LONG,PROT,X86_64,AMD */
        {I_LFENCE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67076, 248},
    /* 1280 : MFENCE void [          np 0f ae f0                       ] LONG,PROT,X86_64,AMD */
        /* 360 355 002(ae f0) : LONG,PROT,X86_64,AMD */
        {I_MFENCE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67082, 248},
    /* 1281 : SFENCE void [          np 0f ae f8                       ] LONG,PROT,X86_64,AMD */
        /* 360 355 002(ae f8) : LONG,PROT,X86_64,AMD */
        {I_SFENCE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67088, 248},
    /* 1282 : SERIALIZE void [          np 0f 01 e8                       ] SERIALIZE,FUTURE */
        /* 360 355 002(01 e8) : SERIALIZE,FUTURE */
        {I_SERIALIZE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67094, 249},
    /* 1283 : CLFLUSH mem [m:        np 0f ae /7                       ] SSE2,WILLAMETTE */
        /* 360 355 001(ae) 207 : SSE2,WILLAMETTE */
        {I_CLFLUSH, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67100, 250},
    /* 1284 : CLFLUSHOPT mem [m:        66 0f ae /7                       ] CLFLUSHOPT,FUTURE */
        /* 361 355 001(ae) 207 : CLFLUSHOPT,FUTURE */
        {I_CLFLUSHOPT, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67106, 251},
    /* 1285 : CLWB mem [m:        66 0f ae /6                       ] CLWB,FUTURE */
        /* 361 355 001(ae) 206 : CLWB,FUTURE */
        {I_CLWB, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67112, 252},
    /* 1286 : PCOMMIT void [          66 0f ae f8                       ] OBSOLETE,NEVER,NOP,FUTURE */
        /* 361 355 002(ae f8) : OBSOLETE,NEVER,NOP,FUTURE */
        {I_PCOMMIT, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67118, 253},
    /* 1287 : CLZERO reg_ax [-:        a16 0f 01 fc                      ] NOLONG,CLZERO,FUTURE,AMD */
        /* 310 355 002(01 fc) : NOREX,NOAPX,NOLONG,CLZERO,FUTURE,AMD */
        {I_CLZERO, 1, {REG_AX|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67124, 254},
    /* 1288 : CLZERO reg_eax [-:        a32 0f 01 fc                      ] CLZERO,FUTURE,AMD */
        /* 311 355 002(01 fc) : CLZERO,FUTURE,AMD */
        {I_CLZERO, 1, {REG_EAX,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67130, 255},
    /* 1289 : CLZERO reg_rax [-:        a64 0f 01 fc                      ] LONG,PROT,CLZERO,X86_64,AMD */
        /* 313 355 002(01 fc) : LONG,PROT,CLZERO,X86_64,AMD */
        {I_CLZERO, 1, {REG_RAX,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67136, 256},
    /* 1290 : INVD void [          0f 08                             ] PRIV,486 */
        /* 355 001(08) : PRIV,486 */
        {I_INVD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72701, 257},
    /* 1291 : WBINVD void [          np 0f 09                          ] PRIV,486 */
        /* 360 355 001(09) : PRIV,486 */
        {I_WBINVD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71658, 257},
    /* 1292 : WBNOINVD void [          f3 0f 09                          ] PRIV,WBNOINVD,FUTURE */
        /* 333 355 001(09) : PRIV,WBNOINVD,FUTURE */
        {I_WBNOINVD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71663, 258},
    /* 1293 : INVPCID reg32,mem128 [rm:       66 0f38 82 /r                     ] NOREX,NOAPX,NOLONG,PRIV,INVPCID,FUTURE */
        /* 361 356 001(82) 110 : NOREX,NOAPX,NOLONG,PRIV,INVPCID,FUTURE */
        {I_INVPCID, 2, {REG_GPR|BITS32|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67142, 259},
    /* 1294 : INVPCID reg64,mem128 [rm:       66 0f38 82 /r                     ] LONG,PRIV,PROT,INVPCID,X86_64 */
        /* 361 356 001(82) 110 : NOAPX,LONG,PRIV,PROT,INVPCID,X86_64 */
        {I_INVPCID, 2, {REG_GPR|BITS64|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67142, 260},
    /* 1295 : INVPCID reg64,mem128 [rm:       evex.nf0.nd0.l0.f3.m4.w1 f2 /r    ] LONG,PRIV,PROT,APX,INVPCID,X86_64 */
        /* 250(f4 fe 08) 300 001(f2) 110 : LONG,EVEX,PRIV,PROT,APX,INVPCID,X86_64 */
        {I_INVPCID, 2, {REG_GPR|BITS64,MEMORY|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+24732, 261},
    /* 1296 : INVLPG mem [m:        0f 01 /7                          ] PRIV,486 */
        /* 355 001(01) 207 : PRIV,486 */
        {I_INVLPG, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71668, 257},
    /* 1297 : INVLPGA reg_ax,reg_ecx [--:       a16 0f 01 df                      ] NOLONG,X86_64,AMD */
        /* 310 355 002(01 df) : NOREX,NOAPX,NOLONG,X86_64,AMD */
        {I_INVLPGA, 2, {REG_AX|RN_L16,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67148, 262},
    /* 1298 : INVLPGA reg_eax,reg_ecx [--:       a32 0f 01 df                      ] X86_64,AMD */
        /* 311 355 002(01 df) : X86_64,AMD */
        {I_INVLPGA, 2, {REG_EAX,REG_ECX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67154, 263},
    /* 1299 : INVLPGA reg_rax,reg_ecx [--:       a64 0f 01 df                      ] LONG,PROT,X86_64,AMD */
        /* 313 355 002(01 df) : LONG,PROT,X86_64,AMD */
        {I_INVLPGA, 2, {REG_RAX,REG_ECX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67160, 248},
    /* 1300 : RDPMC void [          0f 33                             ] NOAPX,P6 */
        /* 355 001(33) : NOAPX,P6 */
        {I_RDPMC, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72705, 229},
    /* 1301 : RDTSC void [          0f 31                             ] NOAPX,PENT */
        /* 355 001(31) : NOAPX,PENT */
        {I_RDTSC, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72709, 264},
    /* 1302 : RDTSCP void [          0f 01 f9                          ] X86_64 */
        /* 355 002(01 f9) : X86_64 */
        {I_RDTSCP, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71673, 265},
    /* 1303 : RDRAND reg16 [m:        o16 0f c7 /6                      ] FL,RDRAND,FUTURE */
        /* 320 355 001(c7) 206 : FL,RDRAND,FUTURE */
        {I_RDRAND, 1, {REG_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67172, 266},
    /* 1304 : RDRAND reg32 [m:        o32 0f c7 /6                      ] FL,RDRAND,FUTURE */
        /* 321 355 001(c7) 206 : FL,RDRAND,FUTURE */
        {I_RDRAND, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67178, 266},
    /* 1305 : RDRAND reg64 [m:        o64 0f c7 /6                      ] LONG,FL,PROT,RDRAND,X86_64 */
        /* 324 355 001(c7) 206 : LONG,FL,PROT,RDRAND,X86_64 */
        {I_RDRAND, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67184, 267},
    /* 1306 : RDSEED reg16 [m:        o16 0f c7 /7                      ] FL,RDSEED,FUTURE */
        /* 320 355 001(c7) 207 : FL,RDSEED,FUTURE */
        {I_RDSEED, 1, {REG_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67190, 268},
    /* 1307 : RDSEED reg32 [m:        o32 0f c7 /7                      ] FL,RDSEED,FUTURE */
        /* 321 355 001(c7) 207 : FL,RDSEED,FUTURE */
        {I_RDSEED, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+51183, 268},
    /* 1308 : RDSEED reg64 [m:        o64 0f c7 /7                      ] LONG,FL,PROT,RDSEED,X86_64 */
        /* 324 355 001(c7) 207 : LONG,FL,PROT,RDSEED,X86_64 */
        {I_RDSEED, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+51191, 269},
    /* 1309 : RDPID reg32 [m:        f3 !osp o32 0f c7 /7              ] RDPID,FUTURE */
        /* 333 364 321 355 001(c7) 207 : RDPID,FUTURE */
        {I_RDPID, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+51181, 271},
    /* 1310 : RDPID reg64 [m:        f3 !osp o64 0f c7 /7              ] LONG,PROT,RDPID,X86_64 */
        /* 333 364 324 355 001(c7) 207 : LONG,PROT,RDPID,X86_64 */
        {I_RDPID, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+51189, 272},
    /* 1311 : CLTS void [          0f 06                             ] PRIV,286 */
        /* 355 001(06) : PRIV,286 */
        {I_CLTS, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72713, 273},
    /* 1312 : CPUID void [          0f a2                             ] PENT */
        /* 355 001(a2) : PENT */
        {I_CPUID, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72717, 274},
    /* 1313 : LMSW rm16 [m:        0f 01 /6                          ] PRIV,286 */
        /* 355 001(01) 206 : PRIV,286 */
        {I_LMSW, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71678, 273},
    /* 1314 : SMSW rm16 [m:        0f 01 /4                          ] 286 */
        /* 355 001(01) 204 : 286 */
        {I_SMSW, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67221, 275},
    /* 1315 : SMSW reg16 [m:        o16 0f 01 /4                      ] 286 */
        /* 320 355 001(01) 204 : 286 */
        {I_SMSW, 1, {REG_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67208, 275},
    /* 1316 : SMSW reg32 [m:        o32 0f 01 /4                      ] 386 */
        /* 321 355 001(01) 204 : 386 */
        {I_SMSW, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67214, 3},
    /* 1317 : SMSW reg64 [m:        o64 0f 01 /4                      ] LONG,PROT,X86_64 */
        /* 324 355 001(01) 204 : LONG,PROT,X86_64 */
        {I_SMSW, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67220, 24},
    /* 1318 : MOV reg32,reg_creg [mr:       rex.l 0f 20 /r                    ] NOREX,NOAPX,NOLONG,PRIV,386 */
        /* 334 355 001(20) 101 : NOREX,NOAPX,NOLONG,PRIV,386 */
        {I_MOV, 2, {REG_GPR|BITS32|RN_L16,REG_CREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67226, 276},
    /* 1319 : MOV reg_creg,reg32 [rm:       rex.l 0f 22 /r                    ] NOREX,NOAPX,NOLONG,PRIV,386 */
        /* 334 355 001(22) 110 : NOREX,NOAPX,NOLONG,PRIV,386 */
        {I_MOV, 2, {REG_CREG|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67232, 276},
    /* 1320 : MOV reg64,reg_creg [mr:       o64nw 0f 20 /r                    ] LONG,PRIV,PROT,X86_64 */
        /* 323 355 001(20) 101 : NWSIZE,LONG,PRIV,PROT,X86_64 */
        {I_MOV, 2, {REG_GPR|BITS64,REG_CREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+67238, 277},
    /* 1321 : MOV reg_creg,reg64 [rm:       o64nw 0f 22 /r                    ] LONG,PRIV,PROT,X86_64 */
        /* 323 355 001(22) 110 : NWSIZE,LONG,PRIV,PROT,X86_64 */
        {I_MOV, 2, {REG_CREG,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+67244, 277},
    /* 1322 : MOV reg32,reg_dreg [mr:       0f 21 /r                          ] NOREX,NOAPX,NOLONG,PRIV,386 */
        /* 355 001(21) 101 : NOREX,NOAPX,NOLONG,PRIV,386 */
        {I_MOV, 2, {REG_GPR|BITS32|RN_L16,REG_DREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67251, 276},
    /* 1323 : MOV reg_dreg,reg32 [rm:       0f 23 /r                          ] NOREX,NOAPX,NOLONG,PRIV,386 */
        /* 355 001(23) 110 : NOREX,NOAPX,NOLONG,PRIV,386 */
        {I_MOV, 2, {REG_DREG|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67257, 276},
    /* 1324 : MOV reg64,reg_dreg [mr:       o64nw 0f 21 /r                    ] LONG,PRIV,PROT,X86_64 */
        /* 323 355 001(21) 101 : NWSIZE,LONG,PRIV,PROT,X86_64 */
        {I_MOV, 2, {REG_GPR|BITS64,REG_DREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+67250, 277},
    /* 1325 : MOV reg_dreg,reg64 [rm:       o64nw 0f 23 /r                    ] LONG,PRIV,PROT,X86_64 */
        /* 323 355 001(23) 110 : NWSIZE,LONG,PRIV,PROT,X86_64 */
        {I_MOV, 2, {REG_DREG,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+67256, 277},
    /* 1326 : WRMSR void [          0f 30                             ] NOAPX,PRIV,PENT */
        /* 355 001(30) : NOAPX,PRIV,PENT */
        {I_WRMSR, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72721, 279},
    /* 1327 : RDMSR void [          0f 32                             ] NOAPX,PRIV,PENT */
        /* 355 001(32) : NOAPX,PRIV,PENT */
        {I_RDMSR, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72725, 279},
    /* 1328 : RDMSR reg64,imm32 [mi:       vex.128.f2.m7.w0 f6 /0 id        ] LONG,PRIV,PROT,APX,MSR_IMM,X86_64,FUTURE */
        /* 270(007 003) 001(f6) 200 041 : NOAPX,LONG,VEX,PRIV,PROT,APX,MSR_IMM,FUTURE */
        {I_RDMSR, 2, {REG_GPR|BITS64|RN_L16,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+51197, 280},
    /* 1329 : RDMSR reg64,imm32 [mi:       evex.128.f2.m7.w0 f6 /0 id        ] LONG,PRIV,PROT,APX,MSR_IMM,X86_64,FUTURE,APX */
        /* 250(f7 7f 08) 300 001(f6) 200 041 : LONG,EVEX,PRIV,PROT,APX,MSR_IMM,FUTURE */
        {I_RDMSR, 2, {REG_GPR|BITS64,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+13023, 281},
    /* 1330 : WRMSRNS void [          np 0f 01 c6                       ] PRIV,WRMSRNS,FUTURE */
        /* 360 355 002(01 c6) : PRIV,WRMSRNS,FUTURE */
        {I_WRMSRNS, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67262, 282},
    /* 1331 : WRMSRNS imm32,reg64 [im:       vex.128.f3.m7.w0 f6 /0 id        ] LONG,PRIV,PROT,APX,MSR_IMM,X86_64,FUTURE */
        /* 270(007 002) 001(f6) 210 040 : NOAPX,LONG,VEX,PRIV,PROT,APX,MSR_IMM,FUTURE */
        {I_WRMSRNS, 2, {IMM_NORMAL|BITS32,REG_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+51205, 280},
    /* 1332 : WRMSRNS imm32,reg64 [im:       evex.128.f3.m7.w0 f6 /0 id        ] LONG,PRIV,PROT,APX,MSR_IMM,X86_64,FUTURE,APX */
        /* 250(f7 7e 08) 300 001(f6) 210 040 : LONG,EVEX,PRIV,PROT,APX,MSR_IMM,FUTURE */
        {I_WRMSRNS, 2, {IMM_NORMAL|BITS32,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+13033, 281},
    /* 1333 : RDMSRLIST void [          f2 0f 01 c6                       ] LONG,PRIV,PROT,MSRLIST,X86_64 */
        /* 332 355 002(01 c6) : LONG,PRIV,PROT,MSRLIST,X86_64 */
        {I_RDMSRLIST, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67268, 283},
    /* 1334 : WRMSRLIST void [          f3 0f 01 c6                       ] LONG,PRIV,PROT,MSRLIST,X86_64 */
        /* 333 355 002(01 c6) : LONG,PRIV,PROT,MSRLIST,X86_64 */
        {I_WRMSRLIST, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67274, 283},
    /* 1335 : URDMSR reg64,reg64 [mr:       f2 0f 38 f8 /r                    ] NOAPX,FUTURE */
        /* 332 355 002(38 f8) 101 : NOAPX,FUTURE */
        {I_URDMSR, 2, {REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56886, 284},
    /* 1336 : URDMSR reg64,reg64 [mr:       evex.nf0.nd0.l0.f2.m4.w0 f8 /r    ] LONG,PROT,APX,X86_64,FUTURE */
        /* 250(f4 7f 08) 300 001(f8) 101 : LONG,EVEX,PROT,APX,FUTURE */
        {I_URDMSR, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+24741, 285},
    /* 1337 : URDMSR reg64,imm32 [mi:       vex.128.f2.m7.w0 f8 /0 id        ] FUTURE */
        /* 270(007 003) 001(f8) 200 041 : NOAPX,VEX,FUTURE */
        {I_URDMSR, 2, {REG_GPR|BITS64|RN_L16,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+51213, 286},
    /* 1338 : URDMSR reg64,imm32 [mi:       evex.128.f2.m7.w0 f8 /0 id        ] FUTURE,APX */
        /* 250(f7 7f 08) 300 001(f8) 200 041 : LONG,EVEX,PROT,APX,FUTURE */
        {I_URDMSR, 2, {REG_GPR|BITS64,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+13043, 285},
    /* 1339 : UWRMSR reg64,reg64 [mr:       f3 0f 38 f8 /r                    ] NOAPX,FUTURE */
        /* 333 355 002(38 f8) 101 : NOAPX,FUTURE */
        {I_UWRMSR, 2, {REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56893, 284},
    /* 1340 : UWRMSR reg64,reg64 [mr:       evex.nf0.nd0.l0.f3.m4.w0 f8 /r    ] LONG,PROT,APX,X86_64,FUTURE */
        /* 250(f4 7e 08) 300 001(f8) 101 : LONG,EVEX,PROT,APX,FUTURE */
        {I_UWRMSR, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+24750, 285},
    /* 1341 : UWRMSR reg64,imm32 [mi:       vex.128.f3.m7.w0 f8 /0 id        ] FUTURE */
        /* 270(007 002) 001(f8) 200 041 : NOAPX,VEX,FUTURE */
        {I_UWRMSR, 2, {REG_GPR|BITS64|RN_L16,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+51221, 286},
    /* 1342 : UWRMSR reg64,imm32 [mi:       evex.128.f3.m7.w0 f8 /0 id        ] FUTURE,APX */
        /* 250(f7 7e 08) 300 001(f8) 200 041 : LONG,EVEX,PROT,APX,FUTURE */
        {I_UWRMSR, 2, {REG_GPR|BITS64,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+13053, 285},
    /* 1343 : RSM void [          0f aa                             ] FL,SMM,PENT */
        /* 355 001(aa) : FL,SMM,PENT */
        {I_RSM, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72757, 291},
    /* 1344 : HLT void [          f4                                ] PRIV,8086 */
        /* 001(f4) : PRIV,8086 */
        {I_HLT, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73329, 294},
    /* 1345 : PAUSE void [          osz f3i 90                        ] 8086 */
        /* 330 333 001(90) : 8086 */
        {I_PAUSE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71713, 23},
    /* 1346 : MONITOR void [          asm 0f 01 c8                      ] MONITOR,PRESCOTT */
        /* 312 355 002(01 c8) : MONITOR,PRESCOTT */
        {I_MONITOR, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67304, 295},
    /* 1347 : MONITORW void [          a16 0f 01 c8                      ] NOLONG,MONITOR,PRESCOTT */
        /* 310 355 002(01 c8) : NOREX,NOAPX,NOLONG,MONITOR,PRESCOTT */
        {I_MONITORW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67310, 296},
    /* 1348 : MONITORD void [          a32 0f 01 c8                      ] MONITOR,PRESCOTT */
        /* 311 355 002(01 c8) : MONITOR,PRESCOTT */
        {I_MONITORD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67316, 295},
    /* 1349 : MONITORQ void [          a64 0f 01 c8                      ] LONG,MONITOR,PRESCOTT,X86_64 */
        /* 313 355 002(01 c8) : LONG,PROT,MONITOR,X86_64 */
        {I_MONITORQ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67322, 297},
    /* 1350 : MWAIT void [          0f 01 c9                          ] MONITOR,PRESCOTT */
        /* 355 002(01 c9) : MONITOR,PRESCOTT */
        {I_MWAIT, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71718, 295},
    /* 1351 : MONITORX void [          asm 0f 01 fa                      ] MONITORX,FUTURE,AMD */
        /* 312 355 002(01 fa) : MONITORX,FUTURE,AMD */
        {I_MONITORX, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67328, 298},
    /* 1352 : MONITORX void [          a16 0f 01 fa                      ] NOLONG,MONITORX,FUTURE,AMD */
        /* 310 355 002(01 fa) : NOREX,NOAPX,NOLONG,MONITORX,FUTURE,AMD */
        {I_MONITORX, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67334, 299},
    /* 1353 : MONITORX void [          a32 0f 01 fa                      ] MONITORX,FUTURE,AMD */
        /* 311 355 002(01 fa) : MONITORX,FUTURE,AMD */
        {I_MONITORX, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67340, 298},
    /* 1354 : MONITORX void [          a64 0f 01 fa                      ] LONG,MONITORX,X86_64,FUTURE,AMD */
        /* 313 355 002(01 fa) : LONG,PROT,MONITORX,FUTURE,AMD */
        {I_MONITORX, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67346, 300},
    /* 1355 : MWAITX void [          0f 01 fb                          ] MONITORX,FUTURE,AMD */
        /* 355 002(01 fb) : MONITORX,FUTURE,AMD */
        {I_MWAITX, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71723, 298},
    /* 1356 : TPAUSE reg32 [m:        66 0f ae /6                       ] FL,WAITPKG,FUTURE */
        /* 361 355 001(ae) 206 : FL,WAITPKG,FUTURE */
        {I_TPAUSE, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67112, 302},
    /* 1357 : UMONITOR reg16 [m:        a16 f3 0f ae /6                   ] NOLONG,WAITPKG,FUTURE */
        /* 310 333 355 001(ae) 206 : NOREX,NOAPX,NOLONG,WAITPKG,FUTURE */
        {I_UMONITOR, 1, {REG_GPR|BITS16|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56928, 303},
    /* 1358 : UMONITOR reg32 [m:        a32 f3 0f ae /6                   ] WAITPKG,FUTURE */
        /* 311 333 355 001(ae) 206 : WAITPKG,FUTURE */
        {I_UMONITOR, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56935, 304},
    /* 1359 : UMONITOR reg64 [m:        a64 f3 0f ae /6                   ] LONG,PROT,WAITPKG,X86_64 */
        /* 313 333 355 001(ae) 206 : LONG,PROT,WAITPKG,X86_64 */
        {I_UMONITOR, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56942, 305},
    /* 1360 : UMWAIT reg32 [m:        f2 0f ae /6                       ] WAITPKG,FUTURE */
        /* 332 355 001(ae) 206 : WAITPKG,FUTURE */
        {I_UMWAIT, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67352, 304},
    /* 1361 : IN reg_al,imm8 [-i:       o8 e4 ib,u                        ] NOAPX,8086 */
        /* 001(e4) 025 : NOAPX,8086 */
        {I_IN, 2, {REG_AL|RN_L16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+72761, 244},
    /* 1362 : IN reg_ax,imm8 [-i:       o16 e5 ib,u                       ] NOAPX,8086 */
        /* 320 001(e5) 025 : NOAPX,8086 */
        {I_IN, 2, {REG_AX|RN_L16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+71728, 244},
    /* 1363 : IN reg_eax,imm8 [-i:       o32 e5 ib,u                       ] NOAPX,ZU,386 */
        /* 321 001(e5) 025 : NOAPX,ZU,386 */
        {I_IN, 2, {REG_EAX|RN_L16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+71733, 306},
    /* 1364 : IN reg_al,reg_dx [--:       o8 ec                             ] NOAPX,8086 */
        /* 001(ec) : NOAPX,8086 */
        {I_IN, 2, {REG_AL|RN_L16,REG_DX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+70031, 244},
    /* 1365 : IN reg_ax,reg_dx [--:       o16 ed                            ] NOAPX,8086 */
        /* 320 001(ed) : NOAPX,8086 */
        {I_IN, 2, {REG_AX|RN_L16,REG_DX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72765, 244},
    /* 1366 : IN reg_eax,reg_dx [--:       o32 ed                            ] NOAPX,ZU,386 */
        /* 321 001(ed) : NOAPX,ZU,386 */
        {I_IN, 2, {REG_EAX|RN_L16,REG_DX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72769, 306},
    /* 1367 : OUT imm8,reg_al [i-:       o8 e6 ib,u                        ] NOAPX,8086 */
        /* 001(e6) 024 : NOAPX,8086 */
        {I_OUT, 2, {IMM_NORMAL|BITS8,REG_AL|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72773, 244},
    /* 1368 : OUT imm8,reg_ax [i-:       o16 e7 ib,u                       ] NOAPX,8086 */
        /* 320 001(e7) 024 : NOAPX,8086 */
        {I_OUT, 2, {IMM_NORMAL|BITS8,REG_AX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71738, 244},
    /* 1369 : OUT imm8,reg_eax [i-:       o32 e7 ib,u                       ] NOAPX,386 */
        /* 321 001(e7) 024 : NOAPX,386 */
        {I_OUT, 2, {IMM_NORMAL|BITS8,REG_EAX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71743, 245},
    /* 1370 : OUT reg_dx,reg_al [--:       o8 ee                             ] NOAPX,8086 */
        /* 001(ee) : NOAPX,8086 */
        {I_OUT, 2, {REG_DX|RN_L16,REG_AL|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+70013, 244},
    /* 1371 : OUT reg_dx,reg_ax [--:       o16 ef                            ] NOAPX,8086 */
        /* 320 001(ef) : NOAPX,8086 */
        {I_OUT, 2, {REG_DX|RN_L16,REG_AX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72777, 244},
    /* 1372 : OUT reg_dx,reg_eax [--:       o32 ef                            ] NOAPX,386 */
        /* 321 001(ef) : NOAPX,386 */
        {I_OUT, 2, {REG_DX|RN_L16,REG_EAX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72781, 245},
    /* 1373 : MOV mem16,reg_sreg [mr:       8c /r                             ] 8086 */
        /* 001(8c) 101 : 8086 */
        {I_MOV, 2, {MEMORY|BITS16,REG_SREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+71749, 23},
    /* 1374 : MOV reg16,reg_sreg [mr:       o16 8c /r                         ] 8086 */
        /* 320 001(8c) 101 : 8086 */
        {I_MOV, 2, {REG_GPR|BITS16,REG_SREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+71748, 23},
    /* 1375 : MOV reg32,reg_sreg [mr:       optd o32 8c /r                    ] 386 */
        /* 337 321 001(8c) 101 : 386 */
        {I_MOV, 2, {REG_GPR|BITS32,REG_SREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+67358, 3},
    /* 1376 : MOV reg64,reg_sreg [mr:       optd o64 8c /r                    ] LONG,386,X86_64 */
        /* 337 324 001(8c) 101 : LONG,PROT,X86_64 */
        {I_MOV, 2, {REG_GPR|BITS64,REG_SREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+67364, 24},
    /* 1377 : MOV reg_sreg,mem16 [rm:       8e /r                             ] 8086 */
        /* 001(8e) 110 : 8086 */
        {I_MOV, 2, {REG_SREG,MEMORY|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67384, 23},
    /* 1378 : MOV reg_sreg,reg16 [rm:       optw o16 8e /r                    ] 8086 */
        /* 336 320 001(8e) 110 : 8086 */
        {I_MOV, 2, {REG_SREG,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67370, 23},
    /* 1379 : MOV reg_sreg,reg32 [rm:       optw o32 8e /r                    ] 386 */
        /* 336 321 001(8e) 110 : 386 */
        {I_MOV, 2, {REG_SREG,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+67376, 3},
    /* 1380 : MOV reg_sreg,reg64 [rm:       optw o64 8e /r                    ] LONG,386,X86_64 */
        /* 336 324 001(8e) 110 : LONG,PROT,X86_64 */
        {I_MOV, 2, {REG_SREG,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+67382, 24},
    /* 1381 : LDS reg16,mem16 [rm:       o16 c5 /r                         ] NOREX,NOAPX,NOLONG,8086 */
        /* 320 001(c5) 110 : NOREX,NOAPX,NOLONG,8086 */
        {I_LDS, 2, {REG_GPR|BITS16|RN_L16,MEMORY|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71753, 227},
    /* 1382 : LDS reg32,mem32 [rm:       o32 c5 /r                         ] NOREX,NOAPX,NOLONG,386 */
        /* 321 001(c5) 110 : NOREX,NOAPX,NOLONG,386 */
        {I_LDS, 2, {REG_GPR|BITS32|RN_L16,MEMORY|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71758, 307},
    /* 1383 : LES reg16,mem16 [rm:       o16 c4 /r                         ] NOREX,NOAPX,NOLONG,8086 */
        /* 320 001(c4) 110 : NOREX,NOAPX,NOLONG,8086 */
        {I_LES, 2, {REG_GPR|BITS16|RN_L16,MEMORY|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71763, 227},
    /* 1384 : LES reg32,mem32 [rm:       o32 c4 /r                         ] NOREX,NOAPX,NOLONG,386 */
        /* 321 001(c4) 110 : NOREX,NOAPX,NOLONG,386 */
        {I_LES, 2, {REG_GPR|BITS32|RN_L16,MEMORY|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71768, 307},
    /* 1385 : LFS reg16,mem16 [rm:       o16 0f b4 /r                      ] SM0-1,386 */
        /* 320 355 001(b4) 110 : SM0,SM1,386 */
        {I_LFS, 2, {REG_GPR|BITS16,MEMORY|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67388, 13},
    /* 1386 : LFS reg32,mem32 [rm:       o32 0f b4 /r                      ] SM0-1,386 */
        /* 321 355 001(b4) 110 : SM0,SM1,386 */
        {I_LFS, 2, {REG_GPR|BITS32,MEMORY|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+67394, 13},
    /* 1387 : LFS reg64,mem64 [rm:       o64 0f b4 /r                      ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(b4) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_LFS, 2, {REG_GPR|BITS64,MEMORY|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+67400, 14},
    /* 1388 : LGS reg16,mem16 [rm:       o16 0f b5 /r                      ] SM0-1,386 */
        /* 320 355 001(b5) 110 : SM0,SM1,386 */
        {I_LGS, 2, {REG_GPR|BITS16,MEMORY|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67406, 13},
    /* 1389 : LGS reg32,mem32 [rm:       o32 0f b5 /r                      ] SM0-1,386 */
        /* 321 355 001(b5) 110 : SM0,SM1,386 */
        {I_LGS, 2, {REG_GPR|BITS32,MEMORY|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+67412, 13},
    /* 1390 : LGS reg64,mem64 [rm:       o64 0f b5 /r                      ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(b5) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_LGS, 2, {REG_GPR|BITS64,MEMORY|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+67418, 14},
    /* 1391 : LSS reg16,mem16 [rm:       o16 0f b2 /r                      ] SM0-1,386 */
        /* 320 355 001(b2) 110 : SM0,SM1,386 */
        {I_LSS, 2, {REG_GPR|BITS16,MEMORY|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67424, 13},
    /* 1392 : LSS reg32,mem32 [rm:       o32 0f b2 /r                      ] SM0-1,386 */
        /* 321 355 001(b2) 110 : SM0,SM1,386 */
        {I_LSS, 2, {REG_GPR|BITS32,MEMORY|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+67430, 13},
    /* 1393 : LSS reg64,mem64 [rm:       o64 0f b2 /r                      ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(b2) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_LSS, 2, {REG_GPR|BITS64,MEMORY|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+67436, 14},
    /* 1394 : PUSH reg_es [-:        06                                ] NOREX,NOAPX,NOLONG,8086 */
        /* 001(06) : NOREX,NOAPX,NOLONG,8086 */
        {I_PUSH, 1, {REG_ES|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72714, 227},
    /* 1395 : PUSH reg_cs [-:        0e                                ] NOREX,NOAPX,NOLONG,8086 */
        /* 001(0e) : NOREX,NOAPX,NOLONG,8086 */
        {I_PUSH, 1, {REG_CS|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72922, 227},
    /* 1396 : PUSH reg_ss [-:        16                                ] NOREX,NOAPX,NOLONG,8086 */
        /* 001(16) : NOREX,NOAPX,NOLONG,8086 */
        {I_PUSH, 1, {REG_SS|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+29301, 227},
    /* 1397 : PUSH reg_ds [-:        1e                                ] NOREX,NOAPX,NOLONG,8086 */
        /* 001(1e) : NOREX,NOAPX,NOLONG,8086 */
        {I_PUSH, 1, {REG_DS|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+29445, 227},
    /* 1398 : PUSH reg_fs [-:        0f a0                             ] 386 */
        /* 355 001(a0) : 386 */
        {I_PUSH, 1, {REG_FS,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72785, 3},
    /* 1399 : PUSH reg_gs [-:        0f a8                             ] 386 */
        /* 355 001(a8) : 386 */
        {I_PUSH, 1, {REG_GS,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72789, 3},
    /* 1400 : POP reg_es [-:        07                                ] NOREX,NOAPX,NOLONG,8086 */
        /* 001(07) : NOREX,NOAPX,NOLONG,8086 */
        {I_POP, 1, {REG_ES|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72622, 227},
    /* 1401 : POP reg_ss [-:        17                                ] NOREX,NOAPX,NOLONG,8086 */
        /* 001(17) : NOREX,NOAPX,NOLONG,8086 */
        {I_POP, 1, {REG_SS|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+29319, 227},
    /* 1402 : POP reg_ds [-:        1f                                ] NOREX,NOAPX,NOLONG,8086 */
        /* 001(1f) : NOREX,NOAPX,NOLONG,8086 */
        {I_POP, 1, {REG_DS|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+29463, 227},
    /* 1403 : POP reg_fs [-:        0f a1                             ] 386 */
        /* 355 001(a1) : 386 */
        {I_POP, 1, {REG_FS,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72793, 3},
    /* 1404 : POP reg_gs [-:        0f a9                             ] 386 */
        /* 355 001(a9) : 386 */
        {I_POP, 1, {REG_GS,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72797, 3},
    /* 1405 : RDFSBASE reg32 [m:        w0 f3 0f ae /0                    ] LONG,PROT,X86_64 */
        /* 317 333 355 001(ae) 200 : LONG,PROT,X86_64 */
        {I_RDFSBASE, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56949, 24},
    /* 1406 : RDFSBASE reg64 [m:        w1 f3 0f ae /0                    ] LONG,PROT,X86_64 */
        /* 324 333 355 001(ae) 200 : LONG,PROT,X86_64 */
        {I_RDFSBASE, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56956, 24},
    /* 1407 : RDGSBASE reg32 [m:        w0 f3 0f ae /1                    ] LONG,PROT,X86_64 */
        /* 317 333 355 001(ae) 201 : LONG,PROT,X86_64 */
        {I_RDGSBASE, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56963, 24},
    /* 1408 : RDGSBASE reg64 [m:        w1 f3 0f ae /1                    ] LONG,PROT,X86_64 */
        /* 324 333 355 001(ae) 201 : LONG,PROT,X86_64 */
        {I_RDGSBASE, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56970, 24},
    /* 1409 : WRFSBASE reg32 [m:        w0 f3 0f ae /2                    ] LONG,PROT,X86_64 */
        /* 317 333 355 001(ae) 202 : LONG,PROT,X86_64 */
        {I_WRFSBASE, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56977, 24},
    /* 1410 : WRFSBASE reg64 [m:        w1 f3 0f ae /2                    ] LONG,PROT,X86_64 */
        /* 324 333 355 001(ae) 202 : LONG,PROT,X86_64 */
        {I_WRFSBASE, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56984, 24},
    /* 1411 : WRGSBASE reg32 [m:        w0 f3 0f ae /3                    ] LONG,PROT,X86_64 */
        /* 317 333 355 001(ae) 203 : LONG,PROT,X86_64 */
        {I_WRGSBASE, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56991, 24},
    /* 1412 : WRGSBASE reg64 [m:        w1 f3 0f ae /3                    ] LONG,PROT,X86_64 */
        /* 324 333 355 001(ae) 203 : LONG,PROT,X86_64 */
        {I_WRGSBASE, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56998, 24},
    /* 1413 : ARPL rm16,reg16 [mr:       optw o16 63 /r                    ] SM0-1,NOREX,NOAPX,NOLONG,FL,PROT,286 */
        /* 336 320 001(63) 101 : SM0,SM1,NOREX,NOAPX,NOLONG,FL,PROT,286 */
        {I_ARPL, 2, {RM_GPR|BITS16|RN_L16,REG_GPR|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67442, 309},
    /* 1414 : ARPL rm16,reg32 [mr:       optw o32 63 /r                    ] SM0-1,NOREX,NOAPX,NOLONG,FL,PROT,386 */
        /* 336 321 001(63) 101 : SM0,SM1,NOREX,NOAPX,NOLONG,FL,PROT,386 */
        {I_ARPL, 2, {RM_GPR|BITS16|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67448, 310},
    /* 1415 : LAR reg16,rm_sel [rm:       o16 0f 02 /r                      ] FL,PROT,286 */
        /* 320 355 001(02) 110 : FL,PROT,286 */
        {I_LAR, 2, {REG_GPR|BITS16,RM_SEL,0,0,0}, NO_DECORATOR, nasm_bytecodes+67454, 311},
    /* 1416 : LAR reg32,rm_sel [rm:       optd o32 0f 02 /r                 ] FL,PROT,386 */
        /* 337 321 355 001(02) 110 : FL,PROT,386 */
        {I_LAR, 2, {REG_GPR|BITS32,RM_SEL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57005, 312},
    /* 1417 : LAR reg64,rm_sel [rm:       optd o64 0f 02 /r                 ] LONG,FL,PROT,X86_64 */
        /* 337 324 355 001(02) 110 : LONG,FL,PROT,X86_64 */
        {I_LAR, 2, {REG_GPR|BITS64,RM_SEL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57012, 56},
    /* 1418 : LSL reg16,rm_sel [rm:       o16 0f 03 /r                      ] PROT,286 */
        /* 320 355 001(03) 110 : PROT,286 */
        {I_LSL, 2, {REG_GPR|BITS16,RM_SEL,0,0,0}, NO_DECORATOR, nasm_bytecodes+67460, 313},
    /* 1419 : LSL reg32,rm_sel [rm:       optd o32 0f 03 /r                 ] PROT,386 */
        /* 337 321 355 001(03) 110 : PROT,386 */
        {I_LSL, 2, {REG_GPR|BITS32,RM_SEL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57019, 314},
    /* 1420 : LSL reg64,rm_sel [rm:       optd o64 0f 03 /r                 ] LONG,PROT,X86_64 */
        /* 337 324 355 001(03) 110 : LONG,PROT,X86_64 */
        {I_LSL, 2, {REG_GPR|BITS64,RM_SEL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57026, 24},
    /* 1421 : VERR rm_sel [m:        0f 00 /4                          ] FL,PROT,286 */
        /* 355 001(00) 204 : FL,PROT,286 */
        {I_VERR, 1, {RM_SEL,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71773, 311},
    /* 1422 : VERW rm_sel [m:        0f 00 /5                          ] FL,PROT,286 */
        /* 355 001(00) 205 : FL,PROT,286 */
        {I_VERW, 1, {RM_SEL,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71778, 311},
    /* 1423 : SWAPGS void [          0f 01 f8                          ] LONG,PROT,X86_64 */
        /* 355 002(01 f8) : LONG,PROT,X86_64 */
        {I_SWAPGS, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71783, 24},
    /* 1424 : LKGS rm_sel [m:        f2 0f 00 /6                       ] LONG,PRIV,PROT,LKGS,X86_64 */
        /* 332 355 001(00) 206 : LONG,PRIV,PROT,LKGS,X86_64 */
        {I_LKGS, 1, {RM_SEL,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67466, 315},
    /* 1425 : LGDT mem16 [m:        nw o16 0f 01 /2                   ] AR0,OSIZE,PRIV,286 */
        /* 327 320 355 001(01) 202 : AR0,NWSIZE,OSIZE,PRIV,286 */
        {I_LGDT, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57033, 316},
    /* 1426 : LGDT mem32 [m:        nw o32 0f 01 /2                   ] AR0,OSIZE,PRIV,386 */
        /* 327 321 355 001(01) 202 : AR0,NWSIZE,OSIZE,PRIV,386 */
        {I_LGDT, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57040, 317},
    /* 1427 : LGDT mem64 [m:        nw o64 0f 01 /2                   ] AR0,OSIZE,LONG,PRIV,386,X86_64 */
        /* 327 324 355 001(01) 202 : AR0,NWSIZE,OSIZE,LONG,PRIV,PROT,X86_64 */
        {I_LGDT, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57047, 318},
    /* 1428 : LIDT mem16 [m:        nw o16 0f 01 /3                   ] AR0,OSIZE,PRIV,286 */
        /* 327 320 355 001(01) 203 : AR0,NWSIZE,OSIZE,PRIV,286 */
        {I_LIDT, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57054, 316},
    /* 1429 : LIDT mem32 [m:        nw o32 0f 01 /3                   ] AR0,OSIZE,PRIV,386 */
        /* 327 321 355 001(01) 203 : AR0,NWSIZE,OSIZE,PRIV,386 */
        {I_LIDT, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57061, 317},
    /* 1430 : LIDT mem64 [m:        nw o64 0f 01 /3                   ] AR0,OSIZE,LONG,PRIV,386,X86_64 */
        /* 327 324 355 001(01) 203 : AR0,NWSIZE,OSIZE,LONG,PRIV,PROT,X86_64 */
        {I_LIDT, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57068, 318},
    /* 1431 : LLDT mem16 [m:        0f 00 /2                          ] PRIV,PROT,286 */
        /* 355 001(00) 202 : PRIV,PROT,286 */
        {I_LLDT, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57091, 319},
    /* 1432 : LLDT reg16 [m:        optw o16 0f 00 /2                 ] PRIV,PROT,286 */
        /* 336 320 355 001(00) 202 : PRIV,PROT,286 */
        {I_LLDT, 1, {REG_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57075, 319},
    /* 1433 : LLDT reg32 [m:        optw o32 0f 00 /2                 ] PRIV,PROT,386 */
        /* 336 321 355 001(00) 202 : PRIV,PROT,386 */
        {I_LLDT, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57082, 320},
    /* 1434 : LLDT reg64 [m:        optw o64 0f 00 /2                 ] LONG,PRIV,PROT,386,X86_64 */
        /* 336 324 355 001(00) 202 : LONG,PRIV,PROT,X86_64 */
        {I_LLDT, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57089, 321},
    /* 1435 : LTR mem16 [m:        0f 00 /3                          ] PRIV,PROT,286 */
        /* 355 001(00) 203 : PRIV,PROT,286 */
        {I_LTR, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57112, 319},
    /* 1436 : LTR reg16 [m:        optw o16 0f 00 /3                 ] PRIV,PROT,286 */
        /* 336 320 355 001(00) 203 : PRIV,PROT,286 */
        {I_LTR, 1, {REG_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57096, 319},
    /* 1437 : LTR reg32 [m:        optw o32 0f 00 /3                 ] PRIV,PROT,386 */
        /* 336 321 355 001(00) 203 : PRIV,PROT,386 */
        {I_LTR, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57103, 320},
    /* 1438 : LTR reg64 [m:        optw o64 0f 00 /3                 ] LONG,PRIV,PROT,386,X86_64 */
        /* 336 324 355 001(00) 203 : LONG,PRIV,PROT,X86_64 */
        {I_LTR, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57110, 321},
    /* 1439 : SGDT mem16 [m:        nw o16 0f 01 /0                   ] AR0,OSIZE,286 */
        /* 327 320 355 001(01) 200 : AR0,NWSIZE,OSIZE,286 */
        {I_SGDT, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57117, 322},
    /* 1440 : SGDT mem32 [m:        nw o32 0f 01 /0                   ] AR0,OSIZE,386 */
        /* 327 321 355 001(01) 200 : AR0,NWSIZE,OSIZE,386 */
        {I_SGDT, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57124, 203},
    /* 1441 : SGDT mem64 [m:        nw o64 0f 01 /0                   ] AR0,OSIZE,LONG,386,X86_64 */
        /* 327 324 355 001(01) 200 : AR0,NWSIZE,OSIZE,LONG,PROT,X86_64 */
        {I_SGDT, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57131, 204},
    /* 1442 : SIDT mem16 [m:        nw o16 0f 01 /1                   ] AR0,OSIZE,286 */
        /* 327 320 355 001(01) 201 : AR0,NWSIZE,OSIZE,286 */
        {I_SIDT, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57138, 322},
    /* 1443 : SIDT mem32 [m:        nw o32 0f 01 /1                   ] AR0,OSIZE,386 */
        /* 327 321 355 001(01) 201 : AR0,NWSIZE,OSIZE,386 */
        {I_SIDT, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57145, 203},
    /* 1444 : SIDT mem64 [m:        nw o64 0f 01 /1                   ] AR0,OSIZE,LONG,386,X86_64 */
        /* 327 324 355 001(01) 201 : AR0,NWSIZE,OSIZE,LONG,PROT,X86_64 */
        {I_SIDT, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57152, 204},
    /* 1445 : SLDT mem16 [m:        0f 00 /0                          ] PROT,286 */
        /* 355 001(00) 200 : PROT,286 */
        {I_SLDT, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67473, 313},
    /* 1446 : SLDT reg16 [m:        o16 0f 00 /0                      ] PROT,286 */
        /* 320 355 001(00) 200 : PROT,286 */
        {I_SLDT, 1, {REG_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67472, 313},
    /* 1447 : SLDT reg32 [m:        optd o32 0f 00 /0                 ] PROT,386 */
        /* 337 321 355 001(00) 200 : PROT,386 */
        {I_SLDT, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57159, 314},
    /* 1448 : SLDT reg64 [m:        optd o64 0f 00 /0                 ] LONG,PROT,386,X86_64 */
        /* 337 324 355 001(00) 200 : LONG,PROT,X86_64 */
        {I_SLDT, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57166, 24},
    /* 1449 : STR mem16 [m:        0f 00 /1                          ] PROT,286 */
        /* 355 001(00) 201 : PROT,286 */
        {I_STR, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67479, 313},
    /* 1450 : STR reg16 [m:        o16 0f 00 /1                      ] PROT,286 */
        /* 320 355 001(00) 201 : PROT,286 */
        {I_STR, 1, {REG_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67478, 313},
    /* 1451 : STR reg32 [m:        optd o32 0f 00 /1                 ] PROT,386 */
        /* 337 321 355 001(00) 201 : PROT,386 */
        {I_STR, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57173, 314},
    /* 1452 : STR reg64 [m:        optd o64 0f 00 /1                 ] LONG,PROT,386,X86_64 */
        /* 337 324 355 001(00) 201 : LONG,PROT,X86_64 */
        {I_STR, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57180, 24},
    /* 1453 : F2XM1 void [          d9 f0                             ] FPU,8086 */
        /* 002(d9 f0) : FPU,8086 */
        {I_F2XM1, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72801, 325},
    /* 1454 : FABS void [          d9 e1                             ] FPU,8086 */
        /* 002(d9 e1) : FPU,8086 */
        {I_FABS, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72805, 325},
    /* 1455 : FADD mem32 [m:        d8 /0                             ] FPU,8086 */
        /* 001(d8) 200 : FPU,8086 */
        {I_FADD, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69802, 325},
    /* 1456 : FADD mem64 [m:        dc /0                             ] FPU,8086 */
        /* 001(dc) 200 : FPU,8086 */
        {I_FADD, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72809, 325},
    /* 1457 : FADD fpureg|to [r:        dc c0+r                           ] FPU,8086 */
        /* 001(dc) 010 300 : FPU,8086 */
        {I_FADD, 1, {FPUREG|TO,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71788, 325},
    /* 1458 : FADD fpureg [r:        d8 c0+r                           ] FPU,8086 */
        /* 001(d8) 010 300 : FPU,8086 */
        {I_FADD, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71793, 325},
    /* 1459 : FADD fpureg,fpu0 [r-:       dc c0+r                           ] FPU,8086 */
        /* 001(dc) 010 300 : FPU,8086 */
        {I_FADD, 2, {FPUREG,FPUREG|RN_ZERO,0,0,0}, NO_DECORATOR, nasm_bytecodes+71788, 325},
    /* 1460 : FADD fpu0,fpureg [-r:       d8 c0+r                           ] FPU,8086 */
        /* 001(d8) 011 300 : FPU,8086 */
        {I_FADD, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+71798, 325},
    /* 1461 : FADDP fpureg [r:        de c0+r                           ] FPU,8086 */
        /* 001(de) 010 300 : FPU,8086 */
        {I_FADDP, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71803, 325},
    /* 1462 : FADDP fpureg,fpu0 [r-:       de c0+r                           ] FPU,8086 */
        /* 001(de) 010 300 : FPU,8086 */
        {I_FADDP, 2, {FPUREG,FPUREG|RN_ZERO,0,0,0}, NO_DECORATOR, nasm_bytecodes+71803, 325},
    /* 1463 : FBLD mem80 [m:        df /4                             ] FPU,8086 */
        /* 001(df) 204 : FPU,8086 */
        {I_FBLD, 1, {MEMORY|BITS80,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72817, 325},
    /* 1464 : FBLD mem [m:        df /4                             ] FPU,8086 */
        /* 001(df) 204 : FPU,8086 */
        {I_FBLD, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72817, 325},
    /* 1465 : FBSTP mem80 [m:        df /6                             ] FPU,8086 */
        /* 001(df) 206 : FPU,8086 */
        {I_FBSTP, 1, {MEMORY|BITS80,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72821, 325},
    /* 1466 : FBSTP mem [m:        df /6                             ] FPU,8086 */
        /* 001(df) 206 : FPU,8086 */
        {I_FBSTP, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72821, 325},
    /* 1467 : FCHS void [          d9 e0                             ] FPU,8086 */
        /* 002(d9 e0) : FPU,8086 */
        {I_FCHS, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72825, 325},
    /* 1468 : FCLEX void [          wait db e2                        ] FPU,8086 */
        /* 341 002(db e2) : FPU,8086 */
        {I_FCLEX, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71808, 325},
    /* 1469 : FCMOVB fpureg [r:        da c0+r                           ] FPU,P6 */
        /* 001(da) 010 300 : FPU,P6 */
        {I_FCMOVB, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71813, 326},
    /* 1470 : FCMOVB fpu0,fpureg [-r:       da c0+r                           ] FPU,P6 */
        /* 001(da) 011 300 : FPU,P6 */
        {I_FCMOVB, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+71818, 326},
    /* 1471 : FCMOVBE fpureg [r:        da d0+r                           ] FPU,P6 */
        /* 001(da) 010 320 : FPU,P6 */
        {I_FCMOVBE, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71823, 326},
    /* 1472 : FCMOVBE fpu0,fpureg [-r:       da d0+r                           ] FPU,P6 */
        /* 001(da) 011 320 : FPU,P6 */
        {I_FCMOVBE, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+71828, 326},
    /* 1473 : FCMOVE fpureg [r:        da c8+r                           ] FPU,P6 */
        /* 001(da) 010 310 : FPU,P6 */
        {I_FCMOVE, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71833, 326},
    /* 1474 : FCMOVE fpu0,fpureg [-r:       da c8+r                           ] FPU,P6 */
        /* 001(da) 011 310 : FPU,P6 */
        {I_FCMOVE, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+71838, 326},
    /* 1475 : FCMOVNB fpureg [r:        db c0+r                           ] FPU,P6 */
        /* 001(db) 010 300 : FPU,P6 */
        {I_FCMOVNB, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71843, 326},
    /* 1476 : FCMOVNB fpu0,fpureg [-r:       db c0+r                           ] FPU,P6 */
        /* 001(db) 011 300 : FPU,P6 */
        {I_FCMOVNB, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+71848, 326},
    /* 1477 : FCMOVNBE fpureg [r:        db d0+r                           ] FPU,P6 */
        /* 001(db) 010 320 : FPU,P6 */
        {I_FCMOVNBE, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71853, 326},
    /* 1478 : FCMOVNBE fpu0,fpureg [-r:       db d0+r                           ] FPU,P6 */
        /* 001(db) 011 320 : FPU,P6 */
        {I_FCMOVNBE, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+71858, 326},
    /* 1479 : FCMOVNE fpureg [r:        db c8+r                           ] FPU,P6 */
        /* 001(db) 010 310 : FPU,P6 */
        {I_FCMOVNE, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71863, 326},
    /* 1480 : FCMOVNE fpu0,fpureg [-r:       db c8+r                           ] FPU,P6 */
        /* 001(db) 011 310 : FPU,P6 */
        {I_FCMOVNE, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+71868, 326},
    /* 1481 : FCMOVNU fpureg [r:        db d8+r                           ] FPU,P6 */
        /* 001(db) 010 330 : FPU,P6 */
        {I_FCMOVNU, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71873, 326},
    /* 1482 : FCMOVNU fpu0,fpureg [-r:       db d8+r                           ] FPU,P6 */
        /* 001(db) 011 330 : FPU,P6 */
        {I_FCMOVNU, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+71878, 326},
    /* 1483 : FCMOVU fpureg [r:        da d8+r                           ] FPU,P6 */
        /* 001(da) 010 330 : FPU,P6 */
        {I_FCMOVU, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71883, 326},
    /* 1484 : FCMOVU fpu0,fpureg [-r:       da d8+r                           ] FPU,P6 */
        /* 001(da) 011 330 : FPU,P6 */
        {I_FCMOVU, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+71888, 326},
    /* 1485 : FCOM mem32 [m:        d8 /2                             ] FPU,8086 */
        /* 001(d8) 202 : FPU,8086 */
        {I_FCOM, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69808, 325},
    /* 1486 : FCOM mem64 [m:        dc /2                             ] FPU,8086 */
        /* 001(dc) 202 : FPU,8086 */
        {I_FCOM, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72861, 325},
    /* 1487 : FCOM fpureg [r:        d8 d0+r                           ] FPU,8086 */
        /* 001(d8) 010 320 : FPU,8086 */
        {I_FCOM, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71893, 325},
    /* 1488 : FCOM fpu0,fpureg [-r:       d8 d0+r                           ] FPU,8086 */
        /* 001(d8) 011 320 : FPU,8086 */
        {I_FCOM, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+71898, 325},
    /* 1489 : FCOMI fpureg [r:        db f0+r                           ] FL,FPU,P6 */
        /* 001(db) 010 360 : FL,FPU,P6 */
        {I_FCOMI, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71903, 327},
    /* 1490 : FCOMI fpu0,fpureg [-r:       db f0+r                           ] FL,FPU,P6 */
        /* 001(db) 011 360 : FL,FPU,P6 */
        {I_FCOMI, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+71908, 327},
    /* 1491 : FCOMIP fpureg [r:        df f0+r                           ] FL,FPU,P6 */
        /* 001(df) 010 360 : FL,FPU,P6 */
        {I_FCOMIP, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71913, 327},
    /* 1492 : FCOMIP fpu0,fpureg [-r:       df f0+r                           ] FL,FPU,P6 */
        /* 001(df) 011 360 : FL,FPU,P6 */
        {I_FCOMIP, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+71918, 327},
    /* 1493 : FCOMP mem32 [m:        d8 /3                             ] FPU,8086 */
        /* 001(d8) 203 : FPU,8086 */
        {I_FCOMP, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69796, 325},
    /* 1494 : FCOMP mem64 [m:        dc /3                             ] FPU,8086 */
        /* 001(dc) 203 : FPU,8086 */
        {I_FCOMP, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72877, 325},
    /* 1495 : FCOMP fpureg [r:        d8 d8+r                           ] FPU,8086 */
        /* 001(d8) 010 330 : FPU,8086 */
        {I_FCOMP, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71923, 325},
    /* 1496 : FCOMP fpu0,fpureg [-r:       d8 d8+r                           ] FPU,8086 */
        /* 001(d8) 011 330 : FPU,8086 */
        {I_FCOMP, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+71928, 325},
    /* 1497 : FCOMPP void [          de d9                             ] FPU,8086 */
        /* 002(de d9) : FPU,8086 */
        {I_FCOMPP, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72885, 325},
    /* 1498 : FCOS void [          d9 ff                             ] FPU,386 */
        /* 002(d9 ff) : FPU,386 */
        {I_FCOS, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72889, 328},
    /* 1499 : FDECSTP void [          d9 f6                             ] FPU,8086 */
        /* 002(d9 f6) : FPU,8086 */
        {I_FDECSTP, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72893, 325},
    /* 1500 : FDISI void [          wait db e1                        ] FPU,8086 */
        /* 341 002(db e1) : FPU,8086 */
        {I_FDISI, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71933, 325},
    /* 1501 : FDIV mem32 [m:        d8 /6                             ] FPU,8086 */
        /* 001(d8) 206 : FPU,8086 */
        {I_FDIV, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72897, 325},
    /* 1502 : FDIV mem64 [m:        dc /6                             ] FPU,8086 */
        /* 001(dc) 206 : FPU,8086 */
        {I_FDIV, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72901, 325},
    /* 1503 : FDIV fpureg|to [r:        dc f8+r                           ] FPU,8086 */
        /* 001(dc) 010 370 : FPU,8086 */
        {I_FDIV, 1, {FPUREG|TO,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71938, 325},
    /* 1504 : FDIV fpureg [r:        d8 f0+r                           ] FPU,8086 */
        /* 001(d8) 010 360 : FPU,8086 */
        {I_FDIV, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71943, 325},
    /* 1505 : FDIV fpureg,fpu0 [r-:       dc f8+r                           ] FPU,8086 */
        /* 001(dc) 010 370 : FPU,8086 */
        {I_FDIV, 2, {FPUREG,FPUREG|RN_ZERO,0,0,0}, NO_DECORATOR, nasm_bytecodes+71938, 325},
    /* 1506 : FDIV fpu0,fpureg [-r:       d8 f0+r                           ] FPU,8086 */
        /* 001(d8) 011 360 : FPU,8086 */
        {I_FDIV, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+71948, 325},
    /* 1507 : FDIVP fpureg [r:        de f8+r                           ] FPU,8086 */
        /* 001(de) 010 370 : FPU,8086 */
        {I_FDIVP, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71953, 325},
    /* 1508 : FDIVP fpureg,fpu0 [r-:       de f8+r                           ] FPU,8086 */
        /* 001(de) 010 370 : FPU,8086 */
        {I_FDIVP, 2, {FPUREG,FPUREG|RN_ZERO,0,0,0}, NO_DECORATOR, nasm_bytecodes+71953, 325},
    /* 1509 : FDIVR mem32 [m:        d8 /7                             ] FPU,8086 */
        /* 001(d8) 207 : FPU,8086 */
        {I_FDIVR, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72909, 325},
    /* 1510 : FDIVR mem64 [m:        dc /7                             ] FPU,8086 */
        /* 001(dc) 207 : FPU,8086 */
        {I_FDIVR, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72913, 325},
    /* 1511 : FDIVR fpureg|to [r:        dc f0+r                           ] FPU,8086 */
        /* 001(dc) 010 360 : FPU,8086 */
        {I_FDIVR, 1, {FPUREG|TO,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71958, 325},
    /* 1512 : FDIVR fpureg,fpu0 [r-:       dc f0+r                           ] FPU,8086 */
        /* 001(dc) 010 360 : FPU,8086 */
        {I_FDIVR, 2, {FPUREG,FPUREG|RN_ZERO,0,0,0}, NO_DECORATOR, nasm_bytecodes+71958, 325},
    /* 1513 : FDIVR fpureg [r:        d8 f8+r                           ] FPU,8086 */
        /* 001(d8) 010 370 : FPU,8086 */
        {I_FDIVR, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71963, 325},
    /* 1514 : FDIVR fpu0,fpureg [-r:       d8 f8+r                           ] FPU,8086 */
        /* 001(d8) 011 370 : FPU,8086 */
        {I_FDIVR, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+71968, 325},
    /* 1515 : FDIVRP fpureg [r:        de f0+r                           ] FPU,8086 */
        /* 001(de) 010 360 : FPU,8086 */
        {I_FDIVRP, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71973, 325},
    /* 1516 : FDIVRP fpureg,fpu0 [r-:       de f0+r                           ] FPU,8086 */
        /* 001(de) 010 360 : FPU,8086 */
        {I_FDIVRP, 2, {FPUREG,FPUREG|RN_ZERO,0,0,0}, NO_DECORATOR, nasm_bytecodes+71973, 325},
    /* 1517 : FEMMS void [          0f 0e                             ] 3DNOW,PENT */
        /* 355 001(0e) : 3DNOW,PENT */
        {I_FEMMS, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72921, 329},
    /* 1518 : FENI void [          wait db e0                        ] FPU,8086 */
        /* 341 002(db e0) : FPU,8086 */
        {I_FENI, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71978, 325},
    /* 1519 : FFREE fpureg [r:        dd c0+r                           ] FPU,8086 */
        /* 001(dd) 010 300 : FPU,8086 */
        {I_FFREE, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71983, 325},
    /* 1520 : FFREE void [          dd c1                             ] FPU,8086 */
        /* 002(dd c1) : FPU,8086 */
        {I_FFREE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72925, 325},
    /* 1521 : FFREEP fpureg [r:        df c0+r                           ] UNDOC,FPU,286 */
        /* 001(df) 010 300 : UNDOC,FPU,286 */
        {I_FFREEP, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71988, 330},
    /* 1522 : FFREEP void [          df c1                             ] UNDOC,FPU,286 */
        /* 002(df c1) : UNDOC,FPU,286 */
        {I_FFREEP, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72929, 330},
    /* 1523 : FIADD mem32 [m:        da /0                             ] FPU,8086 */
        /* 001(da) 200 : FPU,8086 */
        {I_FIADD, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72933, 325},
    /* 1524 : FIADD mem16 [m:        de /0                             ] FPU,8086 */
        /* 001(de) 200 : FPU,8086 */
        {I_FIADD, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72937, 325},
    /* 1525 : FICOM mem32 [m:        da /2                             ] FPU,8086 */
        /* 001(da) 202 : FPU,8086 */
        {I_FICOM, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72941, 325},
    /* 1526 : FICOM mem16 [m:        de /2                             ] FPU,8086 */
        /* 001(de) 202 : FPU,8086 */
        {I_FICOM, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72945, 325},
    /* 1527 : FICOMP mem32 [m:        da /3                             ] FPU,8086 */
        /* 001(da) 203 : FPU,8086 */
        {I_FICOMP, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72949, 325},
    /* 1528 : FICOMP mem16 [m:        de /3                             ] FPU,8086 */
        /* 001(de) 203 : FPU,8086 */
        {I_FICOMP, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72953, 325},
    /* 1529 : FIDIV mem32 [m:        da /6                             ] FPU,8086 */
        /* 001(da) 206 : FPU,8086 */
        {I_FIDIV, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72957, 325},
    /* 1530 : FIDIV mem16 [m:        de /6                             ] FPU,8086 */
        /* 001(de) 206 : FPU,8086 */
        {I_FIDIV, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72961, 325},
    /* 1531 : FIDIVR mem32 [m:        da /7                             ] FPU,8086 */
        /* 001(da) 207 : FPU,8086 */
        {I_FIDIVR, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72965, 325},
    /* 1532 : FIDIVR mem16 [m:        de /7                             ] FPU,8086 */
        /* 001(de) 207 : FPU,8086 */
        {I_FIDIVR, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72969, 325},
    /* 1533 : FILD mem32 [m:        db /0                             ] FPU,8086 */
        /* 001(db) 200 : FPU,8086 */
        {I_FILD, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72973, 325},
    /* 1534 : FILD mem16 [m:        df /0                             ] FPU,8086 */
        /* 001(df) 200 : FPU,8086 */
        {I_FILD, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72977, 325},
    /* 1535 : FILD mem64 [m:        df /5                             ] FPU,8086 */
        /* 001(df) 205 : FPU,8086 */
        {I_FILD, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72981, 325},
    /* 1536 : FIMUL mem32 [m:        da /1                             ] FPU,8086 */
        /* 001(da) 201 : FPU,8086 */
        {I_FIMUL, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72985, 325},
    /* 1537 : FIMUL mem16 [m:        de /1                             ] FPU,8086 */
        /* 001(de) 201 : FPU,8086 */
        {I_FIMUL, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72989, 325},
    /* 1538 : FINCSTP void [          d9 f7                             ] FPU,8086 */
        /* 002(d9 f7) : FPU,8086 */
        {I_FINCSTP, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72993, 325},
    /* 1539 : FINIT void [          wait db e3                        ] FPU,8086 */
        /* 341 002(db e3) : FPU,8086 */
        {I_FINIT, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71993, 325},
    /* 1540 : FIST mem32 [m:        db /2                             ] FPU,8086 */
        /* 001(db) 202 : FPU,8086 */
        {I_FIST, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72997, 325},
    /* 1541 : FIST mem16 [m:        df /2                             ] FPU,8086 */
        /* 001(df) 202 : FPU,8086 */
        {I_FIST, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73001, 325},
    /* 1542 : FISTP mem32 [m:        db /3                             ] FPU,8086 */
        /* 001(db) 203 : FPU,8086 */
        {I_FISTP, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73005, 325},
    /* 1543 : FISTP mem16 [m:        df /3                             ] FPU,8086 */
        /* 001(df) 203 : FPU,8086 */
        {I_FISTP, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73009, 325},
    /* 1544 : FISTP mem64 [m:        df /7                             ] FPU,8086 */
        /* 001(df) 207 : FPU,8086 */
        {I_FISTP, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73013, 325},
    /* 1545 : FISTTP mem16 [m:        df /1                             ] FPU,PRESCOTT */
        /* 001(df) 201 : FPU,PRESCOTT */
        {I_FISTTP, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73017, 331},
    /* 1546 : FISTTP mem32 [m:        db /1                             ] FPU,PRESCOTT */
        /* 001(db) 201 : FPU,PRESCOTT */
        {I_FISTTP, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73021, 331},
    /* 1547 : FISTTP mem64 [m:        dd /1                             ] FPU,PRESCOTT */
        /* 001(dd) 201 : FPU,PRESCOTT */
        {I_FISTTP, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73025, 331},
    /* 1548 : FISUB mem32 [m:        da /4                             ] FPU,8086 */
        /* 001(da) 204 : FPU,8086 */
        {I_FISUB, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73029, 325},
    /* 1549 : FISUB mem16 [m:        de /4                             ] FPU,8086 */
        /* 001(de) 204 : FPU,8086 */
        {I_FISUB, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73033, 325},
    /* 1550 : FISUBR mem32 [m:        da /5                             ] FPU,8086 */
        /* 001(da) 205 : FPU,8086 */
        {I_FISUBR, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73037, 325},
    /* 1551 : FISUBR mem16 [m:        de /5                             ] FPU,8086 */
        /* 001(de) 205 : FPU,8086 */
        {I_FISUBR, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73041, 325},
    /* 1552 : FLD mem32 [m:        d9 /0                             ] FPU,8086 */
        /* 001(d9) 200 : FPU,8086 */
        {I_FLD, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73045, 325},
    /* 1553 : FLD mem64 [m:        dd /0                             ] FPU,8086 */
        /* 001(dd) 200 : FPU,8086 */
        {I_FLD, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73049, 325},
    /* 1554 : FLD mem80 [m:        db /5                             ] FPU,8086 */
        /* 001(db) 205 : FPU,8086 */
        {I_FLD, 1, {MEMORY|BITS80,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73053, 325},
    /* 1555 : FLD fpureg [r:        d9 c0+r                           ] FPU,8086 */
        /* 001(d9) 010 300 : FPU,8086 */
        {I_FLD, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71998, 325},
    /* 1556 : FLD1 void [          d9 e8                             ] FPU,8086 */
        /* 002(d9 e8) : FPU,8086 */
        {I_FLD1, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73061, 325},
    /* 1557 : FLDCW mem [m:        d9 /5                             ] AR0,SW,FPU,8086 */
        /* 001(d9) 205 : AR0,SW,FPU,8086 */
        {I_FLDCW, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73065, 332},
    /* 1558 : FLDENV mem [m:        d9 /4                             ] FPU,8086 */
        /* 001(d9) 204 : FPU,8086 */
        {I_FLDENV, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73069, 325},
    /* 1559 : FLDL2E void [          d9 ea                             ] FPU,8086 */
        /* 002(d9 ea) : FPU,8086 */
        {I_FLDL2E, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73073, 325},
    /* 1560 : FLDL2T void [          d9 e9                             ] FPU,8086 */
        /* 002(d9 e9) : FPU,8086 */
        {I_FLDL2T, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73077, 325},
    /* 1561 : FLDLG2 void [          d9 ec                             ] FPU,8086 */
        /* 002(d9 ec) : FPU,8086 */
        {I_FLDLG2, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73081, 325},
    /* 1562 : FLDLN2 void [          d9 ed                             ] FPU,8086 */
        /* 002(d9 ed) : FPU,8086 */
        {I_FLDLN2, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73085, 325},
    /* 1563 : FLDPI void [          d9 eb                             ] FPU,8086 */
        /* 002(d9 eb) : FPU,8086 */
        {I_FLDPI, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73089, 325},
    /* 1564 : FLDZ void [          d9 ee                             ] FPU,8086 */
        /* 002(d9 ee) : FPU,8086 */
        {I_FLDZ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73093, 325},
    /* 1565 : FMUL mem32 [m:        d8 /1                             ] FPU,8086 */
        /* 001(d8) 201 : FPU,8086 */
        {I_FMUL, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69790, 325},
    /* 1566 : FMUL mem64 [m:        dc /1                             ] FPU,8086 */
        /* 001(dc) 201 : FPU,8086 */
        {I_FMUL, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73097, 325},
    /* 1567 : FMUL fpureg|to [r:        dc c8+r                           ] FPU,8086 */
        /* 001(dc) 010 310 : FPU,8086 */
        {I_FMUL, 1, {FPUREG|TO,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72003, 325},
    /* 1568 : FMUL fpureg,fpu0 [r-:       dc c8+r                           ] FPU,8086 */
        /* 001(dc) 010 310 : FPU,8086 */
        {I_FMUL, 2, {FPUREG,FPUREG|RN_ZERO,0,0,0}, NO_DECORATOR, nasm_bytecodes+72003, 325},
    /* 1569 : FMUL fpureg [r:        d8 c8+r                           ] FPU,8086 */
        /* 001(d8) 010 310 : FPU,8086 */
        {I_FMUL, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72008, 325},
    /* 1570 : FMUL fpu0,fpureg [-r:       d8 c8+r                           ] FPU,8086 */
        /* 001(d8) 011 310 : FPU,8086 */
        {I_FMUL, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+72013, 325},
    /* 1571 : FMULP fpureg [r:        de c8+r                           ] FPU,8086 */
        /* 001(de) 010 310 : FPU,8086 */
        {I_FMULP, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72018, 325},
    /* 1572 : FMULP fpureg,fpu0 [r-:       de c8+r                           ] FPU,8086 */
        /* 001(de) 010 310 : FPU,8086 */
        {I_FMULP, 2, {FPUREG,FPUREG|RN_ZERO,0,0,0}, NO_DECORATOR, nasm_bytecodes+72018, 325},
    /* 1573 : FNCLEX void [          db e2                             ] FPU,8086 */
        /* 002(db e2) : FPU,8086 */
        {I_FNCLEX, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71809, 325},
    /* 1574 : FNDISI void [          db e1                             ] FPU,8086 */
        /* 002(db e1) : FPU,8086 */
        {I_FNDISI, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71934, 325},
    /* 1575 : FNENI void [          db e0                             ] FPU,8086 */
        /* 002(db e0) : FPU,8086 */
        {I_FNENI, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71979, 325},
    /* 1576 : FNINIT void [          db e3                             ] FPU,8086 */
        /* 002(db e3) : FPU,8086 */
        {I_FNINIT, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71994, 325},
    /* 1577 : FNOP void [          d9 d0                             ] FPU,8086 */
        /* 002(d9 d0) : FPU,8086 */
        {I_FNOP, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73105, 325},
    /* 1578 : FNSAVE mem [m:        dd /6                             ] FPU,8086 */
        /* 001(dd) 206 : FPU,8086 */
        {I_FNSAVE, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72024, 325},
    /* 1579 : FNSTCW mem [m:        d9 /7                             ] AR0,SW,FPU,8086 */
        /* 001(d9) 207 : AR0,SW,FPU,8086 */
        {I_FNSTCW, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72034, 332},
    /* 1580 : FNSTENV mem [m:        d9 /6                             ] FPU,8086 */
        /* 001(d9) 206 : FPU,8086 */
        {I_FNSTENV, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72039, 325},
    /* 1581 : FNSTSW mem [m:        dd /7                             ] AR0,SW,FPU,8086 */
        /* 001(dd) 207 : AR0,SW,FPU,8086 */
        {I_FNSTSW, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72049, 332},
    /* 1582 : FNSTSW reg_ax [-:        df e0                             ] FPU,286 */
        /* 002(df e0) : FPU,286 */
        {I_FNSTSW, 1, {REG_AX,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72054, 333},
    /* 1583 : FPATAN void [          d9 f3                             ] FPU,8086 */
        /* 002(d9 f3) : FPU,8086 */
        {I_FPATAN, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73109, 325},
    /* 1584 : FPREM void [          d9 f8                             ] FPU,8086 */
        /* 002(d9 f8) : FPU,8086 */
        {I_FPREM, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73113, 325},
    /* 1585 : FPREM1 void [          d9 f5                             ] FPU,386 */
        /* 002(d9 f5) : FPU,386 */
        {I_FPREM1, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73117, 328},
    /* 1586 : FPTAN void [          d9 f2                             ] FPU,8086 */
        /* 002(d9 f2) : FPU,8086 */
        {I_FPTAN, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73121, 325},
    /* 1587 : FRNDINT void [          d9 fc                             ] FPU,8086 */
        /* 002(d9 fc) : FPU,8086 */
        {I_FRNDINT, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73125, 325},
    /* 1588 : FRSTOR mem [m:        dd /4                             ] FPU,8086 */
        /* 001(dd) 204 : FPU,8086 */
        {I_FRSTOR, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73129, 325},
    /* 1589 : FSAVE mem [m:        wait dd /6                        ] FPU,8086 */
        /* 341 001(dd) 206 : FPU,8086 */
        {I_FSAVE, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72023, 325},
    /* 1590 : FSCALE void [          d9 fd                             ] FPU,8086 */
        /* 002(d9 fd) : FPU,8086 */
        {I_FSCALE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73133, 325},
    /* 1591 : FSETPM void [          db e4                             ] FPU,286 */
        /* 002(db e4) : FPU,286 */
        {I_FSETPM, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73137, 333},
    /* 1592 : FSIN void [          d9 fe                             ] FPU,386 */
        /* 002(d9 fe) : FPU,386 */
        {I_FSIN, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73141, 328},
    /* 1593 : FSINCOS void [          d9 fb                             ] FPU,386 */
        /* 002(d9 fb) : FPU,386 */
        {I_FSINCOS, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73145, 328},
    /* 1594 : FSQRT void [          d9 fa                             ] FPU,8086 */
        /* 002(d9 fa) : FPU,8086 */
        {I_FSQRT, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73149, 325},
    /* 1595 : FST mem32 [m:        d9 /2                             ] FPU,8086 */
        /* 001(d9) 202 : FPU,8086 */
        {I_FST, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73153, 325},
    /* 1596 : FST mem64 [m:        dd /2                             ] FPU,8086 */
        /* 001(dd) 202 : FPU,8086 */
        {I_FST, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73157, 325},
    /* 1597 : FST fpureg [r:        dd d0+r                           ] FPU,8086 */
        /* 001(dd) 010 320 : FPU,8086 */
        {I_FST, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72028, 325},
    /* 1598 : FSTCW mem [m:        wait d9 /7                        ] AR0,SW,FPU,8086 */
        /* 341 001(d9) 207 : AR0,SW,FPU,8086 */
        {I_FSTCW, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72033, 332},
    /* 1599 : FSTENV mem [m:        wait d9 /6                        ] FPU,8086 */
        /* 341 001(d9) 206 : FPU,8086 */
        {I_FSTENV, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72038, 325},
    /* 1600 : FSTP mem32 [m:        d9 /3                             ] FPU,8086 */
        /* 001(d9) 203 : FPU,8086 */
        {I_FSTP, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73165, 325},
    /* 1601 : FSTP mem64 [m:        dd /3                             ] FPU,8086 */
        /* 001(dd) 203 : FPU,8086 */
        {I_FSTP, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73169, 325},
    /* 1602 : FSTP mem80 [m:        db /7                             ] FPU,8086 */
        /* 001(db) 207 : FPU,8086 */
        {I_FSTP, 1, {MEMORY|BITS80,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73173, 325},
    /* 1603 : FSTP fpureg [r:        dd d8+r                           ] FPU,8086 */
        /* 001(dd) 010 330 : FPU,8086 */
        {I_FSTP, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72043, 325},
    /* 1604 : FSTSW mem [m:        wait dd /7                        ] AR0,SW,FPU,8086 */
        /* 341 001(dd) 207 : AR0,SW,FPU,8086 */
        {I_FSTSW, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72048, 332},
    /* 1605 : FSTSW reg_ax [-:        wait df e0                        ] FPU,286 */
        /* 341 002(df e0) : FPU,286 */
        {I_FSTSW, 1, {REG_AX,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72053, 333},
    /* 1606 : FSUB mem32 [m:        d8 /4                             ] FPU,8086 */
        /* 001(d8) 204 : FPU,8086 */
        {I_FSUB, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73181, 325},
    /* 1607 : FSUB mem64 [m:        dc /4                             ] FPU,8086 */
        /* 001(dc) 204 : FPU,8086 */
        {I_FSUB, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73185, 325},
    /* 1608 : FSUB fpureg|to [r:        dc e8+r                           ] FPU,8086 */
        /* 001(dc) 010 350 : FPU,8086 */
        {I_FSUB, 1, {FPUREG|TO,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72058, 325},
    /* 1609 : FSUB fpureg,fpu0 [r-:       dc e8+r                           ] FPU,8086 */
        /* 001(dc) 010 350 : FPU,8086 */
        {I_FSUB, 2, {FPUREG,FPUREG|RN_ZERO,0,0,0}, NO_DECORATOR, nasm_bytecodes+72058, 325},
    /* 1610 : FSUB fpureg [r:        d8 e0+r                           ] FPU,8086 */
        /* 001(d8) 010 340 : FPU,8086 */
        {I_FSUB, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72063, 325},
    /* 1611 : FSUB fpu0,fpureg [-r:       d8 e0+r                           ] FPU,8086 */
        /* 001(d8) 011 340 : FPU,8086 */
        {I_FSUB, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+72068, 325},
    /* 1612 : FSUBP fpureg [r:        de e8+r                           ] FPU,8086 */
        /* 001(de) 010 350 : FPU,8086 */
        {I_FSUBP, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72073, 325},
    /* 1613 : FSUBP fpureg,fpu0 [r-:       de e8+r                           ] FPU,8086 */
        /* 001(de) 010 350 : FPU,8086 */
        {I_FSUBP, 2, {FPUREG,FPUREG|RN_ZERO,0,0,0}, NO_DECORATOR, nasm_bytecodes+72073, 325},
    /* 1614 : FSUBR mem32 [m:        d8 /5                             ] FPU,8086 */
        /* 001(d8) 205 : FPU,8086 */
        {I_FSUBR, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73193, 325},
    /* 1615 : FSUBR mem64 [m:        dc /5                             ] FPU,8086 */
        /* 001(dc) 205 : FPU,8086 */
        {I_FSUBR, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73197, 325},
    /* 1616 : FSUBR fpureg|to [r:        dc e0+r                           ] FPU,8086 */
        /* 001(dc) 010 340 : FPU,8086 */
        {I_FSUBR, 1, {FPUREG|TO,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72078, 325},
    /* 1617 : FSUBR fpureg,fpu0 [r-:       dc e0+r                           ] FPU,8086 */
        /* 001(dc) 010 340 : FPU,8086 */
        {I_FSUBR, 2, {FPUREG,FPUREG|RN_ZERO,0,0,0}, NO_DECORATOR, nasm_bytecodes+72078, 325},
    /* 1618 : FSUBR fpureg [r:        d8 e8+r                           ] FPU,8086 */
        /* 001(d8) 010 350 : FPU,8086 */
        {I_FSUBR, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72083, 325},
    /* 1619 : FSUBR fpu0,fpureg [-r:       d8 e8+r                           ] FPU,8086 */
        /* 001(d8) 011 350 : FPU,8086 */
        {I_FSUBR, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+72088, 325},
    /* 1620 : FSUBRP fpureg [r:        de e0+r                           ] FPU,8086 */
        /* 001(de) 010 340 : FPU,8086 */
        {I_FSUBRP, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72093, 325},
    /* 1621 : FSUBRP fpureg,fpu0 [r-:       de e0+r                           ] FPU,8086 */
        /* 001(de) 010 340 : FPU,8086 */
        {I_FSUBRP, 2, {FPUREG,FPUREG|RN_ZERO,0,0,0}, NO_DECORATOR, nasm_bytecodes+72093, 325},
    /* 1622 : FTST void [          d9 e4                             ] FPU,8086 */
        /* 002(d9 e4) : FPU,8086 */
        {I_FTST, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73205, 325},
    /* 1623 : FUCOM fpureg [r:        dd e0+r                           ] FPU,386 */
        /* 001(dd) 010 340 : FPU,386 */
        {I_FUCOM, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72098, 328},
    /* 1624 : FUCOM fpu0,fpureg [-r:       dd e0+r                           ] FPU,386 */
        /* 001(dd) 011 340 : FPU,386 */
        {I_FUCOM, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+72103, 328},
    /* 1625 : FUCOMI fpureg [r:        db e8+r                           ] FL,FPU,P6 */
        /* 001(db) 010 350 : FL,FPU,P6 */
        {I_FUCOMI, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72108, 327},
    /* 1626 : FUCOMI fpu0,fpureg [-r:       db e8+r                           ] FL,FPU,P6 */
        /* 001(db) 011 350 : FL,FPU,P6 */
        {I_FUCOMI, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+72113, 327},
    /* 1627 : FUCOMIP fpureg [r:        df e8+r                           ] FL,FPU,P6 */
        /* 001(df) 010 350 : FL,FPU,P6 */
        {I_FUCOMIP, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72118, 327},
    /* 1628 : FUCOMIP fpu0,fpureg [-r:       df e8+r                           ] FL,FPU,P6 */
        /* 001(df) 011 350 : FL,FPU,P6 */
        {I_FUCOMIP, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+72123, 327},
    /* 1629 : FUCOMP fpureg [r:        dd e8+r                           ] FPU,386 */
        /* 001(dd) 010 350 : FPU,386 */
        {I_FUCOMP, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72128, 328},
    /* 1630 : FUCOMP fpu0,fpureg [-r:       dd e8+r                           ] FPU,386 */
        /* 001(dd) 011 350 : FPU,386 */
        {I_FUCOMP, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+72133, 328},
    /* 1631 : FUCOMPP void [          da e9                             ] FPU,386 */
        /* 002(da e9) : FPU,386 */
        {I_FUCOMPP, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73225, 328},
    /* 1632 : FXAM void [          d9 e5                             ] FPU,8086 */
        /* 002(d9 e5) : FPU,8086 */
        {I_FXAM, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73229, 325},
    /* 1633 : FXCH fpureg [r:        d9 c8+r                           ] FPU,8086 */
        /* 001(d9) 010 310 : FPU,8086 */
        {I_FXCH, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72138, 325},
    /* 1634 : FXCH fpureg,fpu0 [r-:       d9 c8+r                           ] FPU,8086 */
        /* 001(d9) 010 310 : FPU,8086 */
        {I_FXCH, 2, {FPUREG,FPUREG|RN_ZERO,0,0,0}, NO_DECORATOR, nasm_bytecodes+72138, 325},
    /* 1635 : FXCH fpu0,fpureg [-r:       d9 c8+r                           ] FPU,8086 */
        /* 001(d9) 011 310 : FPU,8086 */
        {I_FXCH, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+72143, 325},
    /* 1636 : FXTRACT void [          d9 f4                             ] FPU,8086 */
        /* 002(d9 f4) : FPU,8086 */
        {I_FXTRACT, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73237, 325},
    /* 1637 : FYL2X void [          d9 f1                             ] FPU,8086 */
        /* 002(d9 f1) : FPU,8086 */
        {I_FYL2X, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73241, 325},
    /* 1638 : FYL2XP1 void [          d9 f9                             ] FPU,8086 */
        /* 002(d9 f9) : FPU,8086 */
        {I_FYL2XP1, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73245, 325},
    /* 1639 : EMMS void [          0f 77                             ] MMX,PENT */
        /* 355 001(77) : MMX,PENT */
        {I_EMMS, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73249, 334},
    /* 1640 : MOVD mmxreg,rm32 [rm:       np 0f 6e /r                       ] MMX,PENT */
        /* 360 355 001(6e) 110 : MMX,PENT */
        {I_MOVD, 2, {MMXREG,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+67484, 334},
    /* 1641 : MOVD rm32,mmxreg [mr:       np 0f 7e /r                       ] MMX,PENT */
        /* 360 355 001(7e) 101 : MMX,PENT */
        {I_MOVD, 2, {RM_GPR|BITS32,MMXREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+67490, 334},
    /* 1642 : MOVQ mmxreg,mmxrm64 [rm:       np 0f 6f /r                       ] MMX,PENT */
        /* 360 355 001(6f) 110 : MMX,PENT */
        {I_MOVQ, 2, {MMXREG,RM_MMX|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+67496, 334},
    /* 1643 : MOVQ mmxrm64,mmxreg [mr:       np 0f 7f /r                       ] MMX,PENT */
        /* 360 355 001(7f) 101 : MMX,PENT */
        {I_MOVQ, 2, {RM_MMX|BITS64,MMXREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+67502, 334},
    /* 1644 : MOVQ mmxreg,rm64 [rm:       np o64 0f 6e /r                   ] LONG,PROT,MMX,X86_64 */
        /* 360 324 355 001(6e) 110 : LONG,PROT,MMX,X86_64 */
        {I_MOVQ, 2, {MMXREG,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+57187, 335},
    /* 1645 : MOVQ rm64,mmxreg [mr:       np o64 0f 7e /r                   ] LONG,PROT,MMX,X86_64 */
        /* 360 324 355 001(7e) 101 : LONG,PROT,MMX,X86_64 */
        {I_MOVQ, 2, {RM_GPR|BITS64,MMXREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+57194, 335},
    /* 1646 : PACKSSDW mmxreg,mmxrm [rm:       np 0f 6b /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(6b) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PACKSSDW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67508, 336},
    /* 1647 : PACKSSWB mmxreg,mmxrm [rm:       np 0f 63 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(63) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PACKSSWB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67514, 336},
    /* 1648 : PACKUSWB mmxreg,mmxrm [rm:       np 0f 67 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(67) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PACKUSWB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67520, 336},
    /* 1649 : PADDB mmxreg,mmxrm [rm:       np 0f fc /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(fc) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PADDB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67526, 336},
    /* 1650 : PADDD mmxreg,mmxrm [rm:       np 0f fe /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(fe) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PADDD, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67532, 336},
    /* 1651 : PADDSB mmxreg,mmxrm [rm:       np 0f ec /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(ec) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PADDSB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67538, 336},
    /* 1652 : PADDSIW mmxreg,mmxrm [rm:       0f 51 /r                          ] AR0-1,SQ,MMX,PENT,CYRIX */
        /* 355 001(51) 110 : AR0,AR1,SQ,MMX,PENT,CYRIX */
        {I_PADDSIW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+69189, 337},
    /* 1653 : PADDSW mmxreg,mmxrm [rm:       np 0f ed /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(ed) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PADDSW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67544, 336},
    /* 1654 : PADDUSB mmxreg,mmxrm [rm:       np 0f dc /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(dc) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PADDUSB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67550, 336},
    /* 1655 : PADDUSW mmxreg,mmxrm [rm:       np 0f dd /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(dd) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PADDUSW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67556, 336},
    /* 1656 : PADDW mmxreg,mmxrm [rm:       np 0f fd /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(fd) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PADDW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67562, 336},
    /* 1657 : PAND mmxreg,mmxrm [rm:       np 0f db /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(db) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PAND, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67568, 336},
    /* 1658 : PANDN mmxreg,mmxrm [rm:       np 0f df /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(df) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PANDN, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67574, 336},
    /* 1659 : PAVEB mmxreg,mmxrm [rm:       0f 50 /r                          ] AR0-1,SQ,MMX,PENT,CYRIX */
        /* 355 001(50) 110 : AR0,AR1,SQ,MMX,PENT,CYRIX */
        {I_PAVEB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+69135, 337},
    /* 1660 : PAVGUSB mmxreg,mmxrm [rm:       0f 0f /r bf                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(bf) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PAVGUSB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57201, 338},
    /* 1661 : PCMPEQB mmxreg,mmxrm [rm:       np 0f 74 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(74) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PCMPEQB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67580, 336},
    /* 1662 : PCMPEQD mmxreg,mmxrm [rm:       np 0f 76 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(76) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PCMPEQD, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67586, 336},
    /* 1663 : PCMPEQW mmxreg,mmxrm [rm:       np 0f 75 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(75) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PCMPEQW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67592, 336},
    /* 1664 : PCMPGTB mmxreg,mmxrm [rm:       np 0f 64 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(64) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PCMPGTB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67598, 336},
    /* 1665 : PCMPGTD mmxreg,mmxrm [rm:       np 0f 66 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(66) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PCMPGTD, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67604, 336},
    /* 1666 : PCMPGTW mmxreg,mmxrm [rm:       np 0f 65 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(65) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PCMPGTW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67610, 336},
    /* 1667 : PDISTIB mmxreg,mem [rm:       0f 54 /r                          ] AR0-1,SQ,MMX,PENT,CYRIX */
        /* 355 001(54) 110 : AR0,AR1,SQ,MMX,PENT,CYRIX */
        {I_PDISTIB, 2, {MMXREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+68973, 337},
    /* 1668 : PF2ID mmxreg,mmxrm [rm:       0f 0f /r 1d                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(1d) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PF2ID, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57208, 338},
    /* 1669 : PFACC mmxreg,mmxrm [rm:       0f 0f /r ae                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(ae) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFACC, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57215, 338},
    /* 1670 : PFADD mmxreg,mmxrm [rm:       0f 0f /r 9e                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(9e) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFADD, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57222, 338},
    /* 1671 : PFCMPEQ mmxreg,mmxrm [rm:       0f 0f /r b0                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(b0) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFCMPEQ, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57229, 338},
    /* 1672 : PFCMPGE mmxreg,mmxrm [rm:       0f 0f /r 90                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(90) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFCMPGE, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57236, 338},
    /* 1673 : PFCMPGT mmxreg,mmxrm [rm:       0f 0f /r a0                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(a0) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFCMPGT, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57243, 338},
    /* 1674 : PFMAX mmxreg,mmxrm [rm:       0f 0f /r a4                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(a4) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFMAX, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57250, 338},
    /* 1675 : PFMIN mmxreg,mmxrm [rm:       0f 0f /r 94                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(94) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFMIN, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57257, 338},
    /* 1676 : PFMUL mmxreg,mmxrm [rm:       0f 0f /r b4                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(b4) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFMUL, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57264, 338},
    /* 1677 : PFRCP mmxreg,mmxrm [rm:       0f 0f /r 96                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(96) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFRCP, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57271, 338},
    /* 1678 : PFRCPIT1 mmxreg,mmxrm [rm:       0f 0f /r a6                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(a6) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFRCPIT1, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57278, 338},
    /* 1679 : PFRCPIT2 mmxreg,mmxrm [rm:       0f 0f /r b6                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(b6) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFRCPIT2, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57285, 338},
    /* 1680 : PFRSQIT1 mmxreg,mmxrm [rm:       0f 0f /r a7                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(a7) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFRSQIT1, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57292, 338},
    /* 1681 : PFRSQRT mmxreg,mmxrm [rm:       0f 0f /r 97                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(97) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFRSQRT, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57299, 338},
    /* 1682 : PFSUB mmxreg,mmxrm [rm:       0f 0f /r 9a                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(9a) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFSUB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57306, 338},
    /* 1683 : PFSUBR mmxreg,mmxrm [rm:       0f 0f /r aa                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(aa) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFSUBR, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57313, 338},
    /* 1684 : PI2FD mmxreg,mmxrm [rm:       0f 0f /r 0d                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(0d) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PI2FD, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57320, 338},
    /* 1685 : PMACHRIW mmxreg,mem [rm:       0f 5e /r                          ] AR0-1,SQ,MMX,PENT,CYRIX */
        /* 355 001(5e) 110 : AR0,AR1,SQ,MMX,PENT,CYRIX */
        {I_PMACHRIW, 2, {MMXREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+69069, 337},
    /* 1686 : PMADDWD mmxreg,mmxrm [rm:       np 0f f5 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(f5) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PMADDWD, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67616, 336},
    /* 1687 : PMAGW mmxreg,mmxrm [rm:       0f 52 /r                          ] AR0-1,SQ,MMX,PENT,CYRIX */
        /* 355 001(52) 110 : AR0,AR1,SQ,MMX,PENT,CYRIX */
        {I_PMAGW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+68397, 337},
    /* 1688 : PMULHRIW mmxreg,mmxrm [rm:       0f 5d /r                          ] AR0-1,SQ,MMX,PENT,CYRIX */
        /* 355 001(5d) 110 : AR0,AR1,SQ,MMX,PENT,CYRIX */
        {I_PMULHRIW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+69093, 337},
    /* 1689 : PMULHRWA mmxreg,mmxrm [rm:       0f 0f /r b7                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(b7) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PMULHRWA, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57327, 338},
    /* 1690 : PMULHRWC mmxreg,mmxrm [rm:       0f 59 /r                          ] AR0-1,SQ,MMX,PENT,CYRIX */
        /* 355 001(59) 110 : AR0,AR1,SQ,MMX,PENT,CYRIX */
        {I_PMULHRWC, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+69171, 337},
    /* 1691 : PMULHW mmxreg,mmxrm [rm:       np 0f e5 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(e5) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PMULHW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67622, 336},
    /* 1692 : PMULLW mmxreg,mmxrm [rm:       np 0f d5 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(d5) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PMULLW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67628, 336},
    /* 1693 : PMVGEZB mmxreg,mem [rm:       0f 5c /r                          ] AR0-1,SQ,MMX,PENT,CYRIX */
        /* 355 001(5c) 110 : AR0,AR1,SQ,MMX,PENT,CYRIX */
        {I_PMVGEZB, 2, {MMXREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+69201, 337},
    /* 1694 : PMVLZB mmxreg,mem [rm:       0f 5b /r                          ] AR0-1,SQ,MMX,PENT,CYRIX */
        /* 355 001(5b) 110 : AR0,AR1,SQ,MMX,PENT,CYRIX */
        {I_PMVLZB, 2, {MMXREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+69057, 337},
    /* 1695 : PMVNZB mmxreg,mem [rm:       0f 5a /r                          ] AR0-1,SQ,MMX,PENT,CYRIX */
        /* 355 001(5a) 110 : AR0,AR1,SQ,MMX,PENT,CYRIX */
        {I_PMVNZB, 2, {MMXREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+69039, 337},
    /* 1696 : PMVZB mmxreg,mem [rm:       0f 58 /r                          ] AR0-1,SQ,MMX,PENT,CYRIX */
        /* 355 001(58) 110 : AR0,AR1,SQ,MMX,PENT,CYRIX */
        {I_PMVZB, 2, {MMXREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+68961, 337},
    /* 1697 : POR mmxreg,mmxrm [rm:       np 0f eb /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(eb) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_POR, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67634, 336},
    /* 1698 : PREFETCH mem [m:        0f 0d /0                          ] AR0,SQ,3DNOW,PENT */
        /* 355 001(0d) 200 : AR0,SQ,3DNOW,PENT */
        {I_PREFETCH, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72148, 339},
    /* 1699 : PREFETCHW mem [m:        0f 0d /1                          ] AR0,SQ,3DNOW,PENT */
        /* 355 001(0d) 201 : AR0,SQ,3DNOW,PENT */
        {I_PREFETCHW, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72153, 339},
    /* 1700 : PSLLD mmxreg,mmxrm [rm:       np 0f f2 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(f2) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSLLD, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67640, 336},
    /* 1701 : PSLLD mmxreg,imm [mi:       np 0f 72 /6 ib,u                  ] MMX,PENT */
        /* 360 355 001(72) 206 025 : MMX,PENT */
        {I_PSLLD, 2, {MMXREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57334, 334},
    /* 1702 : PSLLQ mmxreg,mmxrm [rm:       np 0f f3 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(f3) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSLLQ, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67646, 336},
    /* 1703 : PSLLQ mmxreg,imm [mi:       np 0f 73 /6 ib,u                  ] MMX,PENT */
        /* 360 355 001(73) 206 025 : MMX,PENT */
        {I_PSLLQ, 2, {MMXREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57341, 334},
    /* 1704 : PSLLW mmxreg,mmxrm [rm:       np 0f f1 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(f1) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSLLW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67652, 336},
    /* 1705 : PSLLW mmxreg,imm [mi:       np 0f 71 /6 ib,u                  ] MMX,PENT */
        /* 360 355 001(71) 206 025 : MMX,PENT */
        {I_PSLLW, 2, {MMXREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57348, 334},
    /* 1706 : PSRAD mmxreg,mmxrm [rm:       np 0f e2 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(e2) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSRAD, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67658, 336},
    /* 1707 : PSRAD mmxreg,imm [mi:       np 0f 72 /4 ib,u                  ] MMX,PENT */
        /* 360 355 001(72) 204 025 : MMX,PENT */
        {I_PSRAD, 2, {MMXREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57355, 334},
    /* 1708 : PSRAW mmxreg,mmxrm [rm:       np 0f e1 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(e1) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSRAW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67664, 336},
    /* 1709 : PSRAW mmxreg,imm [mi:       np 0f 71 /4 ib,u                  ] MMX,PENT */
        /* 360 355 001(71) 204 025 : MMX,PENT */
        {I_PSRAW, 2, {MMXREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57362, 334},
    /* 1710 : PSRLD mmxreg,mmxrm [rm:       np 0f d2 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(d2) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSRLD, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67670, 336},
    /* 1711 : PSRLD mmxreg,imm [mi:       np 0f 72 /2 ib,u                  ] MMX,PENT */
        /* 360 355 001(72) 202 025 : MMX,PENT */
        {I_PSRLD, 2, {MMXREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57369, 334},
    /* 1712 : PSRLQ mmxreg,mmxrm [rm:       np 0f d3 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(d3) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSRLQ, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67676, 336},
    /* 1713 : PSRLQ mmxreg,imm [mi:       np 0f 73 /2 ib,u                  ] MMX,PENT */
        /* 360 355 001(73) 202 025 : MMX,PENT */
        {I_PSRLQ, 2, {MMXREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57376, 334},
    /* 1714 : PSRLW mmxreg,mmxrm [rm:       np 0f d1 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(d1) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSRLW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67682, 336},
    /* 1715 : PSRLW mmxreg,imm [mi:       np 0f 71 /2 ib,u                  ] MMX,PENT */
        /* 360 355 001(71) 202 025 : MMX,PENT */
        {I_PSRLW, 2, {MMXREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57383, 334},
    /* 1716 : PSUBB mmxreg,mmxrm [rm:       np 0f f8 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(f8) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSUBB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67688, 336},
    /* 1717 : PSUBD mmxreg,mmxrm [rm:       np 0f fa /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(fa) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSUBD, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67694, 336},
    /* 1718 : PSUBSB mmxreg,mmxrm [rm:       np 0f e8 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(e8) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSUBSB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67700, 336},
    /* 1719 : PSUBSIW mmxreg,mmxrm [rm:       0f 55 /r                          ] AR0-1,SQ,MMX,PENT,CYRIX */
        /* 355 001(55) 110 : AR0,AR1,SQ,MMX,PENT,CYRIX */
        {I_PSUBSIW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+68967, 337},
    /* 1720 : PSUBSW mmxreg,mmxrm [rm:       np 0f e9 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(e9) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSUBSW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67706, 336},
    /* 1721 : PSUBUSB mmxreg,mmxrm [rm:       np 0f d8 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(d8) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSUBUSB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67712, 336},
    /* 1722 : PSUBUSW mmxreg,mmxrm [rm:       np 0f d9 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(d9) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSUBUSW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67718, 336},
    /* 1723 : PSUBW mmxreg,mmxrm [rm:       np 0f f9 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(f9) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSUBW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67724, 336},
    /* 1724 : PUNPCKHBW mmxreg,mmxrm [rm:       np 0f 68 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(68) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PUNPCKHBW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67730, 336},
    /* 1725 : PUNPCKHDQ mmxreg,mmxrm [rm:       np 0f 6a /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(6a) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PUNPCKHDQ, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67736, 336},
    /* 1726 : PUNPCKHWD mmxreg,mmxrm [rm:       np 0f 69 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(69) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PUNPCKHWD, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67742, 336},
    /* 1727 : PUNPCKLBW mmxreg,mmxrm [rm:       np 0f 60 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(60) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PUNPCKLBW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67748, 336},
    /* 1728 : PUNPCKLDQ mmxreg,mmxrm [rm:       np 0f 62 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(62) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PUNPCKLDQ, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67754, 336},
    /* 1729 : PUNPCKLWD mmxreg,mmxrm [rm:       np 0f 61 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(61) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PUNPCKLWD, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67760, 336},
    /* 1730 : PUSH reg16 [r:        nw o16 50+r                       ] 8086 */
        /* 327 320 010 120 : NWSIZE,8086 */
        {I_PUSH, 1, {REG_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72158, 235},
    /* 1731 : PUSH reg32 [r:        nw o32 50+r                       ] 386 */
        /* 327 321 010 120 : NWSIZE,386 */
        {I_PUSH, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72163, 236},
    /* 1732 : PUSH reg64 [r:        nw o64 50+r                       ] LONG,PROT,X86_64 */
        /* 327 324 010 120 : NWSIZE,LONG,PROT,X86_64 */
        {I_PUSH, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72168, 237},
    /* 1733 : PUSH rm16 [m:        nw o16 ff /6                      ] AR0,OSIZE,8086 */
        /* 327 320 001(ff) 206 : AR0,NWSIZE,OSIZE,8086 */
        {I_PUSH, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67766, 340},
    /* 1734 : PUSH rm32 [m:        nw o32 ff /6                      ] AR0,OSIZE,386 */
        /* 327 321 001(ff) 206 : AR0,NWSIZE,OSIZE,386 */
        {I_PUSH, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67772, 203},
    /* 1735 : PUSH rm64 [m:        nw o64 ff /6                      ] AR0,OSIZE,LONG,PROT,X86_64 */
        /* 327 324 001(ff) 206 : AR0,NWSIZE,OSIZE,LONG,PROT,X86_64 */
        {I_PUSH, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67778, 204},
    /* 1736 : PUSH sbyteword16 [i:        nw o16 6a ib,s                    ] AR0,OSIZE,186 */
        /* 327 320 001(6a) 274 : AR0,NWSIZE,OSIZE,186 */
        {I_PUSH, 1, {SBYTEWORD|BITS16|IMM_NORMAL,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67790, 342},
    /* 1737 : PUSH sbytedword32 [i:        nw o32 6a ib,s                    ] AR0,OSIZE,386 */
        /* 327 321 001(6a) 274 : AR0,NWSIZE,OSIZE,386 */
        {I_PUSH, 1, {SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67796, 203},
    /* 1738 : PUSH sbytedword64 [i:        nw o64 6a ib,s                    ] AR0,OSIZE,LONG,386,X86_64 */
        /* 327 324 001(6a) 274 : AR0,NWSIZE,OSIZE,LONG,PROT,X86_64 */
        {I_PUSH, 1, {SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67802, 204},
    /* 1739 : PUSH imm16 [i:        nw o16 68 iw                      ] AR0,OSIZE,186 */
        /* 327 320 001(68) 030 : AR0,NWSIZE,OSIZE,186 */
        {I_PUSH, 1, {IMM_NORMAL|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67808, 342},
    /* 1740 : PUSH imm32 [i:        nw o32 68 id                      ] AR0,OSIZE,386 */
        /* 327 321 001(68) 040 : AR0,NWSIZE,OSIZE,386 */
        {I_PUSH, 1, {IMM_NORMAL|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67814, 203},
    /* 1741 : PUSH sdword64 [i:        nw o64 68 id,s                    ] AR0,OSIZE,LONG,386,X86_64 */
        /* 327 324 001(68) 254 : AR0,NWSIZE,OSIZE,LONG,PROT,X86_64 */
        {I_PUSH, 1, {SDWORD|BITS64|IMM_NORMAL,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67820, 204},
    /* 1742 : POP reg16 [r:        nw o16 58+r                       ] 8086 */
        /* 327 320 010 130 : NWSIZE,8086 */
        {I_POP, 1, {REG_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72173, 235},
    /* 1743 : POP reg32 [r:        nw o32 58+r                       ] 386 */
        /* 327 321 010 130 : NWSIZE,386 */
        {I_POP, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72178, 236},
    /* 1744 : POP reg64 [r:        nw o64 58+r                       ] LONG,PROT,X86_64 */
        /* 327 324 010 130 : NWSIZE,LONG,PROT,X86_64 */
        {I_POP, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72183, 237},
    /* 1745 : POP rm16 [m:        nw o16 8f /0                      ] 8086 */
        /* 327 320 001(8f) 200 : NWSIZE,8086 */
        {I_POP, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67826, 235},
    /* 1746 : POP rm32 [m:        nw o32 8f /0                      ] 386 */
        /* 327 321 001(8f) 200 : NWSIZE,386 */
        {I_POP, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67832, 236},
    /* 1747 : POP rm64 [m:        nw o64 8f /0                      ] LONG,PROT,X86_64 */
        /* 327 324 001(8f) 200 : NWSIZE,LONG,PROT,X86_64 */
        {I_POP, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67838, 237},
    /* 1748 : PUSHA void [          osm 60                            ] NOREX,NOAPX,NOLONG,186 */
        /* 342 001(60) : NOREX,NOAPX,NOLONG,186 */
        {I_PUSHA, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73253, 343},
    /* 1749 : PUSHAW void [          o16 60                            ] NOREX,NOAPX,NOLONG,186 */
        /* 320 001(60) : NOREX,NOAPX,NOLONG,186 */
        {I_PUSHAW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73257, 343},
    /* 1750 : PUSHAD void [          o32 60                            ] NOREX,NOAPX,NOLONG,386 */
        /* 321 001(60) : NOREX,NOAPX,NOLONG,386 */
        {I_PUSHAD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73261, 307},
    /* 1751 : POPA void [          osm 61                            ] NOREX,NOAPX,NOLONG,186 */
        /* 342 001(61) : NOREX,NOAPX,NOLONG,186 */
        {I_POPA, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73265, 343},
    /* 1752 : POPAW void [          o16 61                            ] NOREX,NOAPX,NOLONG,186 */
        /* 320 001(61) : NOREX,NOAPX,NOLONG,186 */
        {I_POPAW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73269, 343},
    /* 1753 : POPAD void [          o32 61                            ] NOREX,NOAPX,NOLONG,386 */
        /* 321 001(61) : NOREX,NOAPX,NOLONG,386 */
        {I_POPAD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73273, 307},
    /* 1754 : ENTER imm16,imm8 [ij:       nw osm c8 iw ib,u                 ] 186 */
        /* 327 342 001(c8) 030 025 : NWSIZE,186 */
        {I_ENTER, 2, {IMM_NORMAL|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+57390, 344},
    /* 1755 : ENTERW imm16,imm8 [ij:       nw o16 c8 iw ib,u                 ] 186 */
        /* 327 320 001(c8) 030 025 : NWSIZE,186 */
        {I_ENTERW, 2, {IMM_NORMAL|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+57397, 344},
    /* 1756 : ENTERD imm16,imm8 [ij:       nw o32 c8 iw ib,u                 ] 386 */
        /* 327 321 001(c8) 030 025 : NWSIZE,386 */
        {I_ENTERD, 2, {IMM_NORMAL|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+57404, 236},
    /* 1757 : ENTERQ imm16,imm8 [ij:       nw o64 c8 iw ib,u                 ] LONG,386,X86_64 */
        /* 327 324 001(c8) 030 025 : NWSIZE,LONG,PROT,X86_64 */
        {I_ENTERQ, 2, {IMM_NORMAL|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+57411, 237},
    /* 1758 : LEAVE void [          nw osm c9                         ] 186 */
        /* 327 342 001(c9) : NWSIZE,186 */
        {I_LEAVE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72188, 344},
    /* 1759 : LEAVEW void [          nw o16 c9                         ] 186 */
        /* 327 320 001(c9) : NWSIZE,186 */
        {I_LEAVEW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72193, 344},
    /* 1760 : LEAVED void [          nw o32 c9                         ] 386 */
        /* 327 321 001(c9) : NWSIZE,386 */
        {I_LEAVED, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72198, 236},
    /* 1761 : LEAVEQ void [          nw o64 c9                         ] LONG,386,X86_64 */
        /* 327 324 001(c9) : NWSIZE,LONG,PROT,X86_64 */
        {I_LEAVEQ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72203, 237},
    /* 1762 : BOUND reg16,mem [rm:       o16 62 /r                         ] NOREX,NOAPX,NOLONG,186 */
        /* 320 001(62) 110 : NOREX,NOAPX,NOLONG,186 */
        {I_BOUND, 2, {REG_GPR|BITS16|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72208, 343},
    /* 1763 : BOUND reg32,mem [rm:       o32 62 /r                         ] NOREX,NOAPX,NOLONG,386 */
        /* 321 001(62) 110 : NOREX,NOAPX,NOLONG,386 */
        {I_BOUND, 2, {REG_GPR|BITS32|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72213, 307},
    /* 1764 : PUSHP reg64 [r:        o64nw rex.w rex2 50+r             ] LONG,PROT,APX,X86_64 */
        /* 323 347 350 010 120 : NWSIZE,LONG,REX2,PROT,APX,X86_64 */
        {I_PUSHP, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67844, 345},
    /* 1765 : POPP reg64 [r:        o64nw rex.w rex2 58+r             ] LONG,PROT,APX,X86_64 */
        /* 323 347 350 010 130 : NWSIZE,LONG,REX2,PROT,APX,X86_64 */
        {I_POPP, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67850, 345},
    /* 1766 : PUSH2 reg64,reg64 [vm:       o64nw evex.nd1.l0.np.m4.w0 ff /6  ] LONG,PROT,APX,X86_64 */
        /* 323 240(f4 7c 18) 300 001(ff) 216 : NWSIZE,LONG,EVEX,PROT,APX,X86_64 */
        {I_PUSH2, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+13063, 346},
    /* 1767 : PUSH2P reg64,reg64 [vm:       o64nw evex.nd1.l0.np.m4.w1 ff /6  ] LONG,PROT,APX,X86_64 */
        /* 323 240(f4 fc 18) 300 001(ff) 216 : NWSIZE,LONG,EVEX,PROT,APX,X86_64 */
        {I_PUSH2P, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+13073, 346},
    /* 1768 : POP2 reg64,reg64 [vm:       o64nw evex.nd1.l0.np.m4.w0 8f /0  ] LONG,PROT,APX,X86_64 */
        /* 323 240(f4 7c 18) 300 001(8f) 210 : NWSIZE,LONG,EVEX,PROT,APX,X86_64 */
        {I_POP2, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+13083, 346},
    /* 1769 : POP2P reg64,reg64 [vm:       o64nw evex.nd1.l0.np.m4.w1 8f /0  ] LONG,PROT,APX,X86_64 */
        /* 323 240(f4 fc 18) 300 001(8f) 210 : NWSIZE,LONG,EVEX,PROT,APX,X86_64 */
        {I_POP2P, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+13103, 346},
    /* 1770 : PXOR mmxreg,mmxrm [rm:       np 0f ef /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(ef) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PXOR, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67856, 336},
    /* 1771 : SKINIT void [          0f 01 de                          ] LONG,PROT,X86_64 */
        /* 355 002(01 de) : LONG,PROT,X86_64 */
        {I_SKINIT, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72218, 24},
    /* 1772 : UD0 reg16,rm16 [rm:       o16 0f ff /r                      ] SM0-1,186 */
        /* 320 355 001(ff) 110 : SM0,SM1,186 */
        {I_UD0, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67862, 347},
    /* 1773 : UD0 reg32,rm32 [rm:       o32 0f ff /r                      ] SM0-1,386 */
        /* 321 355 001(ff) 110 : SM0,SM1,386 */
        {I_UD0, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+67868, 13},
    /* 1774 : UD0 reg64,rm64 [rm:       o64 0f ff /r                      ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(ff) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_UD0, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+67874, 14},
    /* 1775 : UD0 void [          0f ff                             ] 186 */
        /* 355 001(ff) : 186 */
        {I_UD0, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73277, 247},
    /* 1776 : UD1 reg16,rm16 [rm:       o16 0f b9 /r                      ] SM0-1,186 */
        /* 320 355 001(b9) 110 : SM0,SM1,186 */
        {I_UD1, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67880, 347},
    /* 1777 : UD1 reg32,rm32 [rm:       o32 0f b9 /r                      ] SM0-1,386 */
        /* 321 355 001(b9) 110 : SM0,SM1,386 */
        {I_UD1, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+67886, 13},
    /* 1778 : UD1 reg64,rm64 [rm:       o64 0f b9 /r                      ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(b9) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_UD1, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+67892, 14},
    /* 1779 : UD1 void [          0f b9                             ] 186 */
        /* 355 001(b9) : 186 */
        {I_UD1, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73281, 247},
    /* 1780 : UD2 void [          0f 0b                             ] 186 */
        /* 355 001(0b) : 186 */
        {I_UD2, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73285, 247},
    /* 1781 : UDB void [          d6                                ] LONG,PROT,X86_64 */
        /* 001(d6) : LONG,PROT,X86_64 */
        {I_UDB, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72380, 24},
    /* 1782 : FWAIT void [          wait                              ] 8086 */
        /* 341 : 8086 */
        {I_FWAIT, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73211, 23},
    /* 1783 : XLATB void [          d7                                ] 8086 */
        /* 001(d7) : 8086 */
        {I_XLATB, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69947, 23},
    /* 1784 : CCMPA spec4,rm8,reg8 [wmr:      evex.scc7.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 07) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24768, 348},
    /* 1785 : CCMPC spec4,rm8,reg8 [wmr:      evex.scc2.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 02) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24786, 348},
    /* 1786 : CCMPF spec4,rm8,reg8 [wmr:      evex.scc11.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0b) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24759, 348},
    /* 1787 : CCMPG spec4,rm8,reg8 [wmr:      evex.scc15.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0f) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24813, 348},
    /* 1788 : CCMPL spec4,rm8,reg8 [wmr:      evex.scc12.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0c) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24831, 348},
    /* 1789 : CCMPNA spec4,rm8,reg8 [wmr:      evex.scc6.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 06) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24795, 348},
    /* 1790 : CCMPNC spec4,rm8,reg8 [wmr:      evex.scc3.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 03) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24777, 348},
    /* 1791 : CCMPNG spec4,rm8,reg8 [wmr:      evex.scc14.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0e) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24840, 348},
    /* 1792 : CCMPNL spec4,rm8,reg8 [wmr:      evex.scc13.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0d) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24822, 348},
    /* 1793 : CCMPNO spec4,rm8,reg8 [wmr:      evex.scc1.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 01) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24858, 348},
    /* 1794 : CCMPNS spec4,rm8,reg8 [wmr:      evex.scc9.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 09) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24867, 348},
    /* 1795 : CCMPNZ spec4,rm8,reg8 [wmr:      evex.scc5.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 05) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24849, 348},
    /* 1796 : CCMPO spec4,rm8,reg8 [wmr:      evex.scc0.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 00) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24876, 348},
    /* 1797 : CCMPS spec4,rm8,reg8 [wmr:      evex.scc8.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 08) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24885, 348},
    /* 1798 : CCMPT spec4,rm8,reg8 [wmr:      evex.scc10.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0a) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24894, 348},
    /* 1799 : CCMPZ spec4,rm8,reg8 [wmr:      evex.scc4.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 04) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24804, 348},
    /* 1800 : CCMPA spec4,rm16,reg16 [wmr:      evex.scc7.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 07) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13133, 349},
    /* 1801 : CCMPC spec4,rm16,reg16 [wmr:      evex.scc2.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 02) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13153, 349},
    /* 1802 : CCMPF spec4,rm16,reg16 [wmr:      evex.scc11.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0b) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13123, 349},
    /* 1803 : CCMPG spec4,rm16,reg16 [wmr:      evex.scc15.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0f) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13183, 349},
    /* 1804 : CCMPL spec4,rm16,reg16 [wmr:      evex.scc12.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0c) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13203, 349},
    /* 1805 : CCMPNA spec4,rm16,reg16 [wmr:      evex.scc6.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 06) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13163, 349},
    /* 1806 : CCMPNC spec4,rm16,reg16 [wmr:      evex.scc3.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 03) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13143, 349},
    /* 1807 : CCMPNG spec4,rm16,reg16 [wmr:      evex.scc14.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0e) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13213, 349},
    /* 1808 : CCMPNL spec4,rm16,reg16 [wmr:      evex.scc13.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0d) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13193, 349},
    /* 1809 : CCMPNO spec4,rm16,reg16 [wmr:      evex.scc1.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 01) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13233, 349},
    /* 1810 : CCMPNS spec4,rm16,reg16 [wmr:      evex.scc9.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 09) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13243, 349},
    /* 1811 : CCMPNZ spec4,rm16,reg16 [wmr:      evex.scc5.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 05) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13223, 349},
    /* 1812 : CCMPO spec4,rm16,reg16 [wmr:      evex.scc0.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 00) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13253, 349},
    /* 1813 : CCMPS spec4,rm16,reg16 [wmr:      evex.scc8.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 08) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13263, 349},
    /* 1814 : CCMPT spec4,rm16,reg16 [wmr:      evex.scc10.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0a) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13273, 349},
    /* 1815 : CCMPZ spec4,rm16,reg16 [wmr:      evex.scc4.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 04) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13173, 349},
    /* 1816 : CCMPA spec4,rm32,reg32 [wmr:      evex.scc7.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 07) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13293, 350},
    /* 1817 : CCMPC spec4,rm32,reg32 [wmr:      evex.scc2.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 02) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13313, 350},
    /* 1818 : CCMPF spec4,rm32,reg32 [wmr:      evex.scc11.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0b) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13283, 350},
    /* 1819 : CCMPG spec4,rm32,reg32 [wmr:      evex.scc15.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0f) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13343, 350},
    /* 1820 : CCMPL spec4,rm32,reg32 [wmr:      evex.scc12.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0c) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13363, 350},
    /* 1821 : CCMPNA spec4,rm32,reg32 [wmr:      evex.scc6.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 06) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13323, 350},
    /* 1822 : CCMPNC spec4,rm32,reg32 [wmr:      evex.scc3.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 03) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13303, 350},
    /* 1823 : CCMPNG spec4,rm32,reg32 [wmr:      evex.scc14.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0e) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13373, 350},
    /* 1824 : CCMPNL spec4,rm32,reg32 [wmr:      evex.scc13.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0d) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13353, 350},
    /* 1825 : CCMPNO spec4,rm32,reg32 [wmr:      evex.scc1.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 01) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13393, 350},
    /* 1826 : CCMPNS spec4,rm32,reg32 [wmr:      evex.scc9.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 09) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13403, 350},
    /* 1827 : CCMPNZ spec4,rm32,reg32 [wmr:      evex.scc5.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 05) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13383, 350},
    /* 1828 : CCMPO spec4,rm32,reg32 [wmr:      evex.scc0.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 00) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13413, 350},
    /* 1829 : CCMPS spec4,rm32,reg32 [wmr:      evex.scc8.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 08) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13423, 350},
    /* 1830 : CCMPT spec4,rm32,reg32 [wmr:      evex.scc10.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0a) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13433, 350},
    /* 1831 : CCMPZ spec4,rm32,reg32 [wmr:      evex.scc4.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 04) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13333, 350},
    /* 1832 : CCMPA spec4,rm64,reg64 [wmr:      evex.scc7.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 07) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13453, 350},
    /* 1833 : CCMPC spec4,rm64,reg64 [wmr:      evex.scc2.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 02) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13473, 350},
    /* 1834 : CCMPF spec4,rm64,reg64 [wmr:      evex.scc11.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0b) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13443, 350},
    /* 1835 : CCMPG spec4,rm64,reg64 [wmr:      evex.scc15.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0f) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13503, 350},
    /* 1836 : CCMPL spec4,rm64,reg64 [wmr:      evex.scc12.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0c) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13523, 350},
    /* 1837 : CCMPNA spec4,rm64,reg64 [wmr:      evex.scc6.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 06) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13483, 350},
    /* 1838 : CCMPNC spec4,rm64,reg64 [wmr:      evex.scc3.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 03) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13463, 350},
    /* 1839 : CCMPNG spec4,rm64,reg64 [wmr:      evex.scc14.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0e) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13533, 350},
    /* 1840 : CCMPNL spec4,rm64,reg64 [wmr:      evex.scc13.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0d) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13513, 350},
    /* 1841 : CCMPNO spec4,rm64,reg64 [wmr:      evex.scc1.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 01) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13553, 350},
    /* 1842 : CCMPNS spec4,rm64,reg64 [wmr:      evex.scc9.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 09) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13563, 350},
    /* 1843 : CCMPNZ spec4,rm64,reg64 [wmr:      evex.scc5.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 05) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13543, 350},
    /* 1844 : CCMPO spec4,rm64,reg64 [wmr:      evex.scc0.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 00) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13573, 350},
    /* 1845 : CCMPS spec4,rm64,reg64 [wmr:      evex.scc8.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 08) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13583, 350},
    /* 1846 : CCMPT spec4,rm64,reg64 [wmr:      evex.scc10.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0a) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13593, 350},
    /* 1847 : CCMPZ spec4,rm64,reg64 [wmr:      evex.scc4.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 04) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13493, 350},
    /* 1848 : CCMPA spec4,reg8,rm8 [wrm:      evex.scc7.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 07) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24912, 348},
    /* 1849 : CCMPC spec4,reg8,rm8 [wrm:      evex.scc2.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 02) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24930, 348},
    /* 1850 : CCMPF spec4,reg8,rm8 [wrm:      evex.scc11.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0b) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24903, 348},
    /* 1851 : CCMPG spec4,reg8,rm8 [wrm:      evex.scc15.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0f) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24957, 348},
    /* 1852 : CCMPL spec4,reg8,rm8 [wrm:      evex.scc12.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0c) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24975, 348},
    /* 1853 : CCMPNA spec4,reg8,rm8 [wrm:      evex.scc6.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 06) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24939, 348},
    /* 1854 : CCMPNC spec4,reg8,rm8 [wrm:      evex.scc3.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 03) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24921, 348},
    /* 1855 : CCMPNG spec4,reg8,rm8 [wrm:      evex.scc14.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0e) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24984, 348},
    /* 1856 : CCMPNL spec4,reg8,rm8 [wrm:      evex.scc13.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0d) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24966, 348},
    /* 1857 : CCMPNO spec4,reg8,rm8 [wrm:      evex.scc1.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 01) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25002, 348},
    /* 1858 : CCMPNS spec4,reg8,rm8 [wrm:      evex.scc9.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 09) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25011, 348},
    /* 1859 : CCMPNZ spec4,reg8,rm8 [wrm:      evex.scc5.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 05) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24993, 348},
    /* 1860 : CCMPO spec4,reg8,rm8 [wrm:      evex.scc0.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 00) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25020, 348},
    /* 1861 : CCMPS spec4,reg8,rm8 [wrm:      evex.scc8.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 08) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25029, 348},
    /* 1862 : CCMPT spec4,reg8,rm8 [wrm:      evex.scc10.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0a) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25038, 348},
    /* 1863 : CCMPZ spec4,reg8,rm8 [wrm:      evex.scc4.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 04) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24948, 348},
    /* 1864 : CCMPA spec4,reg16,rm16 [wrm:      evex.scc7.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 07) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13613, 349},
    /* 1865 : CCMPC spec4,reg16,rm16 [wrm:      evex.scc2.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 02) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13633, 349},
    /* 1866 : CCMPF spec4,reg16,rm16 [wrm:      evex.scc11.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0b) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13603, 349},
    /* 1867 : CCMPG spec4,reg16,rm16 [wrm:      evex.scc15.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0f) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13663, 349},
    /* 1868 : CCMPL spec4,reg16,rm16 [wrm:      evex.scc12.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0c) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13683, 349},
    /* 1869 : CCMPNA spec4,reg16,rm16 [wrm:      evex.scc6.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 06) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13643, 349},
    /* 1870 : CCMPNC spec4,reg16,rm16 [wrm:      evex.scc3.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 03) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13623, 349},
    /* 1871 : CCMPNG spec4,reg16,rm16 [wrm:      evex.scc14.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0e) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13693, 349},
    /* 1872 : CCMPNL spec4,reg16,rm16 [wrm:      evex.scc13.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0d) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13673, 349},
    /* 1873 : CCMPNO spec4,reg16,rm16 [wrm:      evex.scc1.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 01) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13713, 349},
    /* 1874 : CCMPNS spec4,reg16,rm16 [wrm:      evex.scc9.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 09) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13723, 349},
    /* 1875 : CCMPNZ spec4,reg16,rm16 [wrm:      evex.scc5.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 05) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13703, 349},
    /* 1876 : CCMPO spec4,reg16,rm16 [wrm:      evex.scc0.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 00) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13733, 349},
    /* 1877 : CCMPS spec4,reg16,rm16 [wrm:      evex.scc8.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 08) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13743, 349},
    /* 1878 : CCMPT spec4,reg16,rm16 [wrm:      evex.scc10.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0a) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13753, 349},
    /* 1879 : CCMPZ spec4,reg16,rm16 [wrm:      evex.scc4.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 04) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13653, 349},
    /* 1880 : CCMPA spec4,reg32,rm32 [wrm:      evex.scc7.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 07) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13773, 350},
    /* 1881 : CCMPC spec4,reg32,rm32 [wrm:      evex.scc2.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 02) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13793, 350},
    /* 1882 : CCMPF spec4,reg32,rm32 [wrm:      evex.scc11.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0b) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13763, 350},
    /* 1883 : CCMPG spec4,reg32,rm32 [wrm:      evex.scc15.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0f) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13823, 350},
    /* 1884 : CCMPL spec4,reg32,rm32 [wrm:      evex.scc12.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0c) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13843, 350},
    /* 1885 : CCMPNA spec4,reg32,rm32 [wrm:      evex.scc6.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 06) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13803, 350},
    /* 1886 : CCMPNC spec4,reg32,rm32 [wrm:      evex.scc3.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 03) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13783, 350},
    /* 1887 : CCMPNG spec4,reg32,rm32 [wrm:      evex.scc14.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0e) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13853, 350},
    /* 1888 : CCMPNL spec4,reg32,rm32 [wrm:      evex.scc13.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0d) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13833, 350},
    /* 1889 : CCMPNO spec4,reg32,rm32 [wrm:      evex.scc1.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 01) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13873, 350},
    /* 1890 : CCMPNS spec4,reg32,rm32 [wrm:      evex.scc9.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 09) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13883, 350},
    /* 1891 : CCMPNZ spec4,reg32,rm32 [wrm:      evex.scc5.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 05) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13863, 350},
    /* 1892 : CCMPO spec4,reg32,rm32 [wrm:      evex.scc0.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 00) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13893, 350},
    /* 1893 : CCMPS spec4,reg32,rm32 [wrm:      evex.scc8.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 08) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13903, 350},
    /* 1894 : CCMPT spec4,reg32,rm32 [wrm:      evex.scc10.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0a) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13913, 350},
    /* 1895 : CCMPZ spec4,reg32,rm32 [wrm:      evex.scc4.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 04) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13813, 350},
    /* 1896 : CCMPA spec4,reg64,rm64 [wrm:      evex.scc7.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 07) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13933, 350},
    /* 1897 : CCMPC spec4,reg64,rm64 [wrm:      evex.scc2.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 02) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13953, 350},
    /* 1898 : CCMPF spec4,reg64,rm64 [wrm:      evex.scc11.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0b) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13923, 350},
    /* 1899 : CCMPG spec4,reg64,rm64 [wrm:      evex.scc15.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0f) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13983, 350},
    /* 1900 : CCMPL spec4,reg64,rm64 [wrm:      evex.scc12.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0c) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14003, 350},
    /* 1901 : CCMPNA spec4,reg64,rm64 [wrm:      evex.scc6.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 06) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13963, 350},
    /* 1902 : CCMPNC spec4,reg64,rm64 [wrm:      evex.scc3.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 03) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13943, 350},
    /* 1903 : CCMPNG spec4,reg64,rm64 [wrm:      evex.scc14.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0e) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14013, 350},
    /* 1904 : CCMPNL spec4,reg64,rm64 [wrm:      evex.scc13.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0d) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13993, 350},
    /* 1905 : CCMPNO spec4,reg64,rm64 [wrm:      evex.scc1.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 01) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14033, 350},
    /* 1906 : CCMPNS spec4,reg64,rm64 [wrm:      evex.scc9.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 09) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14043, 350},
    /* 1907 : CCMPNZ spec4,reg64,rm64 [wrm:      evex.scc5.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 05) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14023, 350},
    /* 1908 : CCMPO spec4,reg64,rm64 [wrm:      evex.scc0.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 00) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14053, 350},
    /* 1909 : CCMPS spec4,reg64,rm64 [wrm:      evex.scc8.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 08) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14063, 350},
    /* 1910 : CCMPT spec4,reg64,rm64 [wrm:      evex.scc10.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0a) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14073, 350},
    /* 1911 : CCMPZ spec4,reg64,rm64 [wrm:      evex.scc4.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 04) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13973, 350},
    /* 1912 : CCMPA spec4,rm16,sbyteword16 [wmi:      evex.scc7.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 07) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2378, 349},
    /* 1913 : CCMPC spec4,rm16,sbyteword16 [wmi:      evex.scc2.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 02) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2400, 349},
    /* 1914 : CCMPF spec4,rm16,sbyteword16 [wmi:      evex.scc11.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0b) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2367, 349},
    /* 1915 : CCMPG spec4,rm16,sbyteword16 [wmi:      evex.scc15.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0f) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2433, 349},
    /* 1916 : CCMPL spec4,rm16,sbyteword16 [wmi:      evex.scc12.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0c) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2455, 349},
    /* 1917 : CCMPNA spec4,rm16,sbyteword16 [wmi:      evex.scc6.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 06) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2411, 349},
    /* 1918 : CCMPNC spec4,rm16,sbyteword16 [wmi:      evex.scc3.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 03) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2389, 349},
    /* 1919 : CCMPNG spec4,rm16,sbyteword16 [wmi:      evex.scc14.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0e) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2466, 349},
    /* 1920 : CCMPNL spec4,rm16,sbyteword16 [wmi:      evex.scc13.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0d) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2444, 349},
    /* 1921 : CCMPNO spec4,rm16,sbyteword16 [wmi:      evex.scc1.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 01) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2488, 349},
    /* 1922 : CCMPNS spec4,rm16,sbyteword16 [wmi:      evex.scc9.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 09) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2499, 349},
    /* 1923 : CCMPNZ spec4,rm16,sbyteword16 [wmi:      evex.scc5.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 05) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2477, 349},
    /* 1924 : CCMPO spec4,rm16,sbyteword16 [wmi:      evex.scc0.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 00) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2510, 349},
    /* 1925 : CCMPS spec4,rm16,sbyteword16 [wmi:      evex.scc8.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 08) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2521, 349},
    /* 1926 : CCMPT spec4,rm16,sbyteword16 [wmi:      evex.scc10.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0a) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2532, 349},
    /* 1927 : CCMPZ spec4,rm16,sbyteword16 [wmi:      evex.scc4.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 04) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2422, 349},
    /* 1928 : CCMPA spec4,rm32,sbytedword32 [wmi:      evex.scc7.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 07) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2554, 350},
    /* 1929 : CCMPC spec4,rm32,sbytedword32 [wmi:      evex.scc2.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 02) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2576, 350},
    /* 1930 : CCMPF spec4,rm32,sbytedword32 [wmi:      evex.scc11.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0b) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2543, 350},
    /* 1931 : CCMPG spec4,rm32,sbytedword32 [wmi:      evex.scc15.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0f) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2609, 350},
    /* 1932 : CCMPL spec4,rm32,sbytedword32 [wmi:      evex.scc12.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0c) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2631, 350},
    /* 1933 : CCMPNA spec4,rm32,sbytedword32 [wmi:      evex.scc6.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 06) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2587, 350},
    /* 1934 : CCMPNC spec4,rm32,sbytedword32 [wmi:      evex.scc3.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 03) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2565, 350},
    /* 1935 : CCMPNG spec4,rm32,sbytedword32 [wmi:      evex.scc14.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0e) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2642, 350},
    /* 1936 : CCMPNL spec4,rm32,sbytedword32 [wmi:      evex.scc13.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0d) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2620, 350},
    /* 1937 : CCMPNO spec4,rm32,sbytedword32 [wmi:      evex.scc1.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 01) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2664, 350},
    /* 1938 : CCMPNS spec4,rm32,sbytedword32 [wmi:      evex.scc9.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 09) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2675, 350},
    /* 1939 : CCMPNZ spec4,rm32,sbytedword32 [wmi:      evex.scc5.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 05) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2653, 350},
    /* 1940 : CCMPO spec4,rm32,sbytedword32 [wmi:      evex.scc0.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 00) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2686, 350},
    /* 1941 : CCMPS spec4,rm32,sbytedword32 [wmi:      evex.scc8.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 08) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2697, 350},
    /* 1942 : CCMPT spec4,rm32,sbytedword32 [wmi:      evex.scc10.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0a) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2708, 350},
    /* 1943 : CCMPZ spec4,rm32,sbytedword32 [wmi:      evex.scc4.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 04) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2598, 350},
    /* 1944 : CCMPA spec4,rm64,sbytedword64 [wmi:      evex.scc7.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 07) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2730, 350},
    /* 1945 : CCMPC spec4,rm64,sbytedword64 [wmi:      evex.scc2.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 02) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2752, 350},
    /* 1946 : CCMPF spec4,rm64,sbytedword64 [wmi:      evex.scc11.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0b) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2719, 350},
    /* 1947 : CCMPG spec4,rm64,sbytedword64 [wmi:      evex.scc15.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0f) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2785, 350},
    /* 1948 : CCMPL spec4,rm64,sbytedword64 [wmi:      evex.scc12.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0c) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2807, 350},
    /* 1949 : CCMPNA spec4,rm64,sbytedword64 [wmi:      evex.scc6.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 06) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2763, 350},
    /* 1950 : CCMPNC spec4,rm64,sbytedword64 [wmi:      evex.scc3.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 03) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2741, 350},
    /* 1951 : CCMPNG spec4,rm64,sbytedword64 [wmi:      evex.scc14.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0e) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2818, 350},
    /* 1952 : CCMPNL spec4,rm64,sbytedword64 [wmi:      evex.scc13.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0d) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2796, 350},
    /* 1953 : CCMPNO spec4,rm64,sbytedword64 [wmi:      evex.scc1.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 01) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2840, 350},
    /* 1954 : CCMPNS spec4,rm64,sbytedword64 [wmi:      evex.scc9.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 09) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2851, 350},
    /* 1955 : CCMPNZ spec4,rm64,sbytedword64 [wmi:      evex.scc5.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 05) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2829, 350},
    /* 1956 : CCMPO spec4,rm64,sbytedword64 [wmi:      evex.scc0.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 00) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2862, 350},
    /* 1957 : CCMPS spec4,rm64,sbytedword64 [wmi:      evex.scc8.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 08) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2873, 350},
    /* 1958 : CCMPT spec4,rm64,sbytedword64 [wmi:      evex.scc10.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0a) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2884, 350},
    /* 1959 : CCMPZ spec4,rm64,sbytedword64 [wmi:      evex.scc4.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 04) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2774, 350},
    /* 1960 : CCMPA spec4,rm8,imm8 [wmi:      evex.scc7.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 07) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14093, 348},
    /* 1961 : CCMPC spec4,rm8,imm8 [wmi:      evex.scc2.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 02) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14113, 348},
    /* 1962 : CCMPF spec4,rm8,imm8 [wmi:      evex.scc11.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0b) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14083, 348},
    /* 1963 : CCMPG spec4,rm8,imm8 [wmi:      evex.scc15.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0f) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14143, 348},
    /* 1964 : CCMPL spec4,rm8,imm8 [wmi:      evex.scc12.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0c) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14163, 348},
    /* 1965 : CCMPNA spec4,rm8,imm8 [wmi:      evex.scc6.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 06) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14123, 348},
    /* 1966 : CCMPNC spec4,rm8,imm8 [wmi:      evex.scc3.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 03) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14103, 348},
    /* 1967 : CCMPNG spec4,rm8,imm8 [wmi:      evex.scc14.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0e) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14173, 348},
    /* 1968 : CCMPNL spec4,rm8,imm8 [wmi:      evex.scc13.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0d) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14153, 348},
    /* 1969 : CCMPNO spec4,rm8,imm8 [wmi:      evex.scc1.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 01) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14193, 348},
    /* 1970 : CCMPNS spec4,rm8,imm8 [wmi:      evex.scc9.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 09) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14203, 348},
    /* 1971 : CCMPNZ spec4,rm8,imm8 [wmi:      evex.scc5.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 05) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14183, 348},
    /* 1972 : CCMPO spec4,rm8,imm8 [wmi:      evex.scc0.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 00) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14213, 348},
    /* 1973 : CCMPS spec4,rm8,imm8 [wmi:      evex.scc8.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 08) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14223, 348},
    /* 1974 : CCMPT spec4,rm8,imm8 [wmi:      evex.scc10.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0a) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14233, 348},
    /* 1975 : CCMPZ spec4,rm8,imm8 [wmi:      evex.scc4.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 04) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14133, 348},
    /* 1976 : CCMPA spec4,rm16,imm16 [wmi:      evex.scc7.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 07) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+2906, 349},
    /* 1977 : CCMPC spec4,rm16,imm16 [wmi:      evex.scc2.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 02) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+2928, 349},
    /* 1978 : CCMPF spec4,rm16,imm16 [wmi:      evex.scc11.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0b) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+2895, 349},
    /* 1979 : CCMPG spec4,rm16,imm16 [wmi:      evex.scc15.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0f) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+2961, 349},
    /* 1980 : CCMPL spec4,rm16,imm16 [wmi:      evex.scc12.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0c) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+2983, 349},
    /* 1981 : CCMPNA spec4,rm16,imm16 [wmi:      evex.scc6.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 06) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+2939, 349},
    /* 1982 : CCMPNC spec4,rm16,imm16 [wmi:      evex.scc3.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 03) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+2917, 349},
    /* 1983 : CCMPNG spec4,rm16,imm16 [wmi:      evex.scc14.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0e) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+2994, 349},
    /* 1984 : CCMPNL spec4,rm16,imm16 [wmi:      evex.scc13.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0d) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+2972, 349},
    /* 1985 : CCMPNO spec4,rm16,imm16 [wmi:      evex.scc1.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 01) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3016, 349},
    /* 1986 : CCMPNS spec4,rm16,imm16 [wmi:      evex.scc9.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 09) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3027, 349},
    /* 1987 : CCMPNZ spec4,rm16,imm16 [wmi:      evex.scc5.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 05) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3005, 349},
    /* 1988 : CCMPO spec4,rm16,imm16 [wmi:      evex.scc0.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 00) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3038, 349},
    /* 1989 : CCMPS spec4,rm16,imm16 [wmi:      evex.scc8.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 08) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3049, 349},
    /* 1990 : CCMPT spec4,rm16,imm16 [wmi:      evex.scc10.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0a) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3060, 349},
    /* 1991 : CCMPZ spec4,rm16,imm16 [wmi:      evex.scc4.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 04) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+2950, 349},
    /* 1992 : CCMPA spec4,rm32,imm32 [wmi:      evex.scc7.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 07) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3082, 350},
    /* 1993 : CCMPC spec4,rm32,imm32 [wmi:      evex.scc2.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 02) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3104, 350},
    /* 1994 : CCMPF spec4,rm32,imm32 [wmi:      evex.scc11.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0b) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3071, 350},
    /* 1995 : CCMPG spec4,rm32,imm32 [wmi:      evex.scc15.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0f) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3137, 350},
    /* 1996 : CCMPL spec4,rm32,imm32 [wmi:      evex.scc12.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0c) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3159, 350},
    /* 1997 : CCMPNA spec4,rm32,imm32 [wmi:      evex.scc6.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 06) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3115, 350},
    /* 1998 : CCMPNC spec4,rm32,imm32 [wmi:      evex.scc3.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 03) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3093, 350},
    /* 1999 : CCMPNG spec4,rm32,imm32 [wmi:      evex.scc14.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0e) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3170, 350},
    /* 2000 : CCMPNL spec4,rm32,imm32 [wmi:      evex.scc13.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0d) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3148, 350},
    /* 2001 : CCMPNO spec4,rm32,imm32 [wmi:      evex.scc1.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 01) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3192, 350},
    /* 2002 : CCMPNS spec4,rm32,imm32 [wmi:      evex.scc9.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 09) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3203, 350},
    /* 2003 : CCMPNZ spec4,rm32,imm32 [wmi:      evex.scc5.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 05) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3181, 350},
    /* 2004 : CCMPO spec4,rm32,imm32 [wmi:      evex.scc0.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 00) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3214, 350},
    /* 2005 : CCMPS spec4,rm32,imm32 [wmi:      evex.scc8.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 08) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3225, 350},
    /* 2006 : CCMPT spec4,rm32,imm32 [wmi:      evex.scc10.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0a) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3236, 350},
    /* 2007 : CCMPZ spec4,rm32,imm32 [wmi:      evex.scc4.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 04) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3126, 350},
    /* 2008 : CCMPA spec4,rm64,sdword64 [wmi:      evex.scc7.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 07) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3258, 350},
    /* 2009 : CCMPC spec4,rm64,sdword64 [wmi:      evex.scc2.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 02) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3280, 350},
    /* 2010 : CCMPF spec4,rm64,sdword64 [wmi:      evex.scc11.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0b) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3247, 350},
    /* 2011 : CCMPG spec4,rm64,sdword64 [wmi:      evex.scc15.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0f) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3313, 350},
    /* 2012 : CCMPL spec4,rm64,sdword64 [wmi:      evex.scc12.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0c) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3335, 350},
    /* 2013 : CCMPNA spec4,rm64,sdword64 [wmi:      evex.scc6.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 06) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3291, 350},
    /* 2014 : CCMPNC spec4,rm64,sdword64 [wmi:      evex.scc3.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 03) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3269, 350},
    /* 2015 : CCMPNG spec4,rm64,sdword64 [wmi:      evex.scc14.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0e) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3346, 350},
    /* 2016 : CCMPNL spec4,rm64,sdword64 [wmi:      evex.scc13.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0d) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3324, 350},
    /* 2017 : CCMPNO spec4,rm64,sdword64 [wmi:      evex.scc1.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 01) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3368, 350},
    /* 2018 : CCMPNS spec4,rm64,sdword64 [wmi:      evex.scc9.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 09) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3379, 350},
    /* 2019 : CCMPNZ spec4,rm64,sdword64 [wmi:      evex.scc5.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 05) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3357, 350},
    /* 2020 : CCMPO spec4,rm64,sdword64 [wmi:      evex.scc0.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 00) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3390, 350},
    /* 2021 : CCMPS spec4,rm64,sdword64 [wmi:      evex.scc8.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 08) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3401, 350},
    /* 2022 : CCMPT spec4,rm64,sdword64 [wmi:      evex.scc10.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0a) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3412, 350},
    /* 2023 : CCMPZ spec4,rm64,sdword64 [wmi:      evex.scc4.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 04) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3302, 350},
    /* 2024 : CTESTA spec4,rm8,reg8 [wmr:      evex.scc7.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 07) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25056, 348},
    /* 2025 : CTESTC spec4,rm8,reg8 [wmr:      evex.scc2.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 02) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25074, 348},
    /* 2026 : CTESTF spec4,rm8,reg8 [wmr:      evex.scc11.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0b) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25047, 348},
    /* 2027 : CTESTG spec4,rm8,reg8 [wmr:      evex.scc15.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0f) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25101, 348},
    /* 2028 : CTESTL spec4,rm8,reg8 [wmr:      evex.scc12.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0c) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25119, 348},
    /* 2029 : CTESTNA spec4,rm8,reg8 [wmr:      evex.scc6.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 06) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25083, 348},
    /* 2030 : CTESTNC spec4,rm8,reg8 [wmr:      evex.scc3.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 03) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25065, 348},
    /* 2031 : CTESTNG spec4,rm8,reg8 [wmr:      evex.scc14.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0e) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25128, 348},
    /* 2032 : CTESTNL spec4,rm8,reg8 [wmr:      evex.scc13.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0d) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25110, 348},
    /* 2033 : CTESTNO spec4,rm8,reg8 [wmr:      evex.scc1.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 01) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25146, 348},
    /* 2034 : CTESTNS spec4,rm8,reg8 [wmr:      evex.scc9.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 09) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25155, 348},
    /* 2035 : CTESTNZ spec4,rm8,reg8 [wmr:      evex.scc5.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 05) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25137, 348},
    /* 2036 : CTESTO spec4,rm8,reg8 [wmr:      evex.scc0.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 00) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25164, 348},
    /* 2037 : CTESTS spec4,rm8,reg8 [wmr:      evex.scc8.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 08) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25173, 348},
    /* 2038 : CTESTT spec4,rm8,reg8 [wmr:      evex.scc10.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0a) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25182, 348},
    /* 2039 : CTESTZ spec4,rm8,reg8 [wmr:      evex.scc4.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 04) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25092, 348},
    /* 2040 : CTESTA spec4,rm16,reg16 [wmr:      evex.scc7.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 07) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14253, 349},
    /* 2041 : CTESTC spec4,rm16,reg16 [wmr:      evex.scc2.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 02) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14273, 349},
    /* 2042 : CTESTF spec4,rm16,reg16 [wmr:      evex.scc11.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0b) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14243, 349},
    /* 2043 : CTESTG spec4,rm16,reg16 [wmr:      evex.scc15.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0f) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14303, 349},
    /* 2044 : CTESTL spec4,rm16,reg16 [wmr:      evex.scc12.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0c) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14323, 349},
    /* 2045 : CTESTNA spec4,rm16,reg16 [wmr:      evex.scc6.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 06) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14283, 349},
    /* 2046 : CTESTNC spec4,rm16,reg16 [wmr:      evex.scc3.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 03) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14263, 349},
    /* 2047 : CTESTNG spec4,rm16,reg16 [wmr:      evex.scc14.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0e) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14333, 349},
    /* 2048 : CTESTNL spec4,rm16,reg16 [wmr:      evex.scc13.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0d) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14313, 349},
    /* 2049 : CTESTNO spec4,rm16,reg16 [wmr:      evex.scc1.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 01) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14353, 349},
    /* 2050 : CTESTNS spec4,rm16,reg16 [wmr:      evex.scc9.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 09) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14363, 349},
    /* 2051 : CTESTNZ spec4,rm16,reg16 [wmr:      evex.scc5.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 05) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14343, 349},
    /* 2052 : CTESTO spec4,rm16,reg16 [wmr:      evex.scc0.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 00) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14373, 349},
    /* 2053 : CTESTS spec4,rm16,reg16 [wmr:      evex.scc8.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 08) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14383, 349},
    /* 2054 : CTESTT spec4,rm16,reg16 [wmr:      evex.scc10.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0a) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14393, 349},
    /* 2055 : CTESTZ spec4,rm16,reg16 [wmr:      evex.scc4.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 04) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14293, 349},
    /* 2056 : CTESTA spec4,rm32,reg32 [wmr:      evex.scc7.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 07) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14413, 350},
    /* 2057 : CTESTC spec4,rm32,reg32 [wmr:      evex.scc2.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 02) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14433, 350},
    /* 2058 : CTESTF spec4,rm32,reg32 [wmr:      evex.scc11.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0b) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14403, 350},
    /* 2059 : CTESTG spec4,rm32,reg32 [wmr:      evex.scc15.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0f) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14463, 350},
    /* 2060 : CTESTL spec4,rm32,reg32 [wmr:      evex.scc12.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0c) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14483, 350},
    /* 2061 : CTESTNA spec4,rm32,reg32 [wmr:      evex.scc6.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 06) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14443, 350},
    /* 2062 : CTESTNC spec4,rm32,reg32 [wmr:      evex.scc3.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 03) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14423, 350},
    /* 2063 : CTESTNG spec4,rm32,reg32 [wmr:      evex.scc14.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0e) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14493, 350},
    /* 2064 : CTESTNL spec4,rm32,reg32 [wmr:      evex.scc13.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0d) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14473, 350},
    /* 2065 : CTESTNO spec4,rm32,reg32 [wmr:      evex.scc1.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 01) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14513, 350},
    /* 2066 : CTESTNS spec4,rm32,reg32 [wmr:      evex.scc9.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 09) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14523, 350},
    /* 2067 : CTESTNZ spec4,rm32,reg32 [wmr:      evex.scc5.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 05) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14503, 350},
    /* 2068 : CTESTO spec4,rm32,reg32 [wmr:      evex.scc0.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 00) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14533, 350},
    /* 2069 : CTESTS spec4,rm32,reg32 [wmr:      evex.scc8.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 08) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14543, 350},
    /* 2070 : CTESTT spec4,rm32,reg32 [wmr:      evex.scc10.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0a) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14553, 350},
    /* 2071 : CTESTZ spec4,rm32,reg32 [wmr:      evex.scc4.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 04) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14453, 350},
    /* 2072 : CTESTA spec4,rm64,reg64 [wmr:      evex.scc7.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 07) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14573, 350},
    /* 2073 : CTESTC spec4,rm64,reg64 [wmr:      evex.scc2.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 02) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14593, 350},
    /* 2074 : CTESTF spec4,rm64,reg64 [wmr:      evex.scc11.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0b) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14563, 350},
    /* 2075 : CTESTG spec4,rm64,reg64 [wmr:      evex.scc15.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0f) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14623, 350},
    /* 2076 : CTESTL spec4,rm64,reg64 [wmr:      evex.scc12.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0c) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14643, 350},
    /* 2077 : CTESTNA spec4,rm64,reg64 [wmr:      evex.scc6.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 06) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14603, 350},
    /* 2078 : CTESTNC spec4,rm64,reg64 [wmr:      evex.scc3.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 03) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14583, 350},
    /* 2079 : CTESTNG spec4,rm64,reg64 [wmr:      evex.scc14.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0e) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14653, 350},
    /* 2080 : CTESTNL spec4,rm64,reg64 [wmr:      evex.scc13.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0d) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14633, 350},
    /* 2081 : CTESTNO spec4,rm64,reg64 [wmr:      evex.scc1.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 01) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14673, 350},
    /* 2082 : CTESTNS spec4,rm64,reg64 [wmr:      evex.scc9.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 09) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14683, 350},
    /* 2083 : CTESTNZ spec4,rm64,reg64 [wmr:      evex.scc5.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 05) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14663, 350},
    /* 2084 : CTESTO spec4,rm64,reg64 [wmr:      evex.scc0.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 00) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14693, 350},
    /* 2085 : CTESTS spec4,rm64,reg64 [wmr:      evex.scc8.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 08) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14703, 350},
    /* 2086 : CTESTT spec4,rm64,reg64 [wmr:      evex.scc10.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0a) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14713, 350},
    /* 2087 : CTESTZ spec4,rm64,reg64 [wmr:      evex.scc4.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 04) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14613, 350},
    /* 2088 : CTESTA spec4,rm8,imm8 [wmi:      evex.scc7.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 07) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14733, 348},
    /* 2089 : CTESTC spec4,rm8,imm8 [wmi:      evex.scc2.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 02) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14753, 348},
    /* 2090 : CTESTF spec4,rm8,imm8 [wmi:      evex.scc11.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0b) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14723, 348},
    /* 2091 : CTESTG spec4,rm8,imm8 [wmi:      evex.scc15.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0f) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14783, 348},
    /* 2092 : CTESTL spec4,rm8,imm8 [wmi:      evex.scc12.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0c) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14803, 348},
    /* 2093 : CTESTNA spec4,rm8,imm8 [wmi:      evex.scc6.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 06) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14763, 348},
    /* 2094 : CTESTNC spec4,rm8,imm8 [wmi:      evex.scc3.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 03) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14743, 348},
    /* 2095 : CTESTNG spec4,rm8,imm8 [wmi:      evex.scc14.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0e) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14813, 348},
    /* 2096 : CTESTNL spec4,rm8,imm8 [wmi:      evex.scc13.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0d) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14793, 348},
    /* 2097 : CTESTNO spec4,rm8,imm8 [wmi:      evex.scc1.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 01) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14833, 348},
    /* 2098 : CTESTNS spec4,rm8,imm8 [wmi:      evex.scc9.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 09) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14843, 348},
    /* 2099 : CTESTNZ spec4,rm8,imm8 [wmi:      evex.scc5.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 05) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14823, 348},
    /* 2100 : CTESTO spec4,rm8,imm8 [wmi:      evex.scc0.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 00) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14853, 348},
    /* 2101 : CTESTS spec4,rm8,imm8 [wmi:      evex.scc8.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 08) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14863, 348},
    /* 2102 : CTESTT spec4,rm8,imm8 [wmi:      evex.scc10.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0a) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14873, 348},
    /* 2103 : CTESTZ spec4,rm8,imm8 [wmi:      evex.scc4.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 04) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14773, 348},
    /* 2104 : CTESTA spec4,rm16,imm16 [wmi:      evex.scc7.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 07) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3434, 349},
    /* 2105 : CTESTC spec4,rm16,imm16 [wmi:      evex.scc2.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 02) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3456, 349},
    /* 2106 : CTESTF spec4,rm16,imm16 [wmi:      evex.scc11.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0b) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3423, 349},
    /* 2107 : CTESTG spec4,rm16,imm16 [wmi:      evex.scc15.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0f) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3489, 349},
    /* 2108 : CTESTL spec4,rm16,imm16 [wmi:      evex.scc12.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0c) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3511, 349},
    /* 2109 : CTESTNA spec4,rm16,imm16 [wmi:      evex.scc6.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 06) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3467, 349},
    /* 2110 : CTESTNC spec4,rm16,imm16 [wmi:      evex.scc3.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 03) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3445, 349},
    /* 2111 : CTESTNG spec4,rm16,imm16 [wmi:      evex.scc14.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0e) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3522, 349},
    /* 2112 : CTESTNL spec4,rm16,imm16 [wmi:      evex.scc13.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0d) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3500, 349},
    /* 2113 : CTESTNO spec4,rm16,imm16 [wmi:      evex.scc1.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 01) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3544, 349},
    /* 2114 : CTESTNS spec4,rm16,imm16 [wmi:      evex.scc9.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 09) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3555, 349},
    /* 2115 : CTESTNZ spec4,rm16,imm16 [wmi:      evex.scc5.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 05) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3533, 349},
    /* 2116 : CTESTO spec4,rm16,imm16 [wmi:      evex.scc0.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 00) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3566, 349},
    /* 2117 : CTESTS spec4,rm16,imm16 [wmi:      evex.scc8.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 08) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3577, 349},
    /* 2118 : CTESTT spec4,rm16,imm16 [wmi:      evex.scc10.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0a) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3588, 349},
    /* 2119 : CTESTZ spec4,rm16,imm16 [wmi:      evex.scc4.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 04) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3478, 349},
    /* 2120 : CTESTA spec4,rm32,imm32 [wmi:      evex.scc7.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 07) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3610, 350},
    /* 2121 : CTESTC spec4,rm32,imm32 [wmi:      evex.scc2.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 02) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3632, 350},
    /* 2122 : CTESTF spec4,rm32,imm32 [wmi:      evex.scc11.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0b) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3599, 350},
    /* 2123 : CTESTG spec4,rm32,imm32 [wmi:      evex.scc15.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0f) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3665, 350},
    /* 2124 : CTESTL spec4,rm32,imm32 [wmi:      evex.scc12.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0c) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3687, 350},
    /* 2125 : CTESTNA spec4,rm32,imm32 [wmi:      evex.scc6.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 06) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3643, 350},
    /* 2126 : CTESTNC spec4,rm32,imm32 [wmi:      evex.scc3.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 03) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3621, 350},
    /* 2127 : CTESTNG spec4,rm32,imm32 [wmi:      evex.scc14.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0e) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3698, 350},
    /* 2128 : CTESTNL spec4,rm32,imm32 [wmi:      evex.scc13.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0d) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3676, 350},
    /* 2129 : CTESTNO spec4,rm32,imm32 [wmi:      evex.scc1.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 01) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3720, 350},
    /* 2130 : CTESTNS spec4,rm32,imm32 [wmi:      evex.scc9.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 09) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3731, 350},
    /* 2131 : CTESTNZ spec4,rm32,imm32 [wmi:      evex.scc5.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 05) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3709, 350},
    /* 2132 : CTESTO spec4,rm32,imm32 [wmi:      evex.scc0.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 00) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3742, 350},
    /* 2133 : CTESTS spec4,rm32,imm32 [wmi:      evex.scc8.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 08) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3753, 350},
    /* 2134 : CTESTT spec4,rm32,imm32 [wmi:      evex.scc10.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0a) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3764, 350},
    /* 2135 : CTESTZ spec4,rm32,imm32 [wmi:      evex.scc4.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 04) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3654, 350},
    /* 2136 : CTESTA spec4,rm64,sdword64 [wmi:      evex.scc7.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 07) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3786, 350},
    /* 2137 : CTESTC spec4,rm64,sdword64 [wmi:      evex.scc2.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 02) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3808, 350},
    /* 2138 : CTESTF spec4,rm64,sdword64 [wmi:      evex.scc11.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0b) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3775, 350},
    /* 2139 : CTESTG spec4,rm64,sdword64 [wmi:      evex.scc15.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0f) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3841, 350},
    /* 2140 : CTESTL spec4,rm64,sdword64 [wmi:      evex.scc12.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0c) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3863, 350},
    /* 2141 : CTESTNA spec4,rm64,sdword64 [wmi:      evex.scc6.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 06) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3819, 350},
    /* 2142 : CTESTNC spec4,rm64,sdword64 [wmi:      evex.scc3.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 03) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3797, 350},
    /* 2143 : CTESTNG spec4,rm64,sdword64 [wmi:      evex.scc14.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0e) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3874, 350},
    /* 2144 : CTESTNL spec4,rm64,sdword64 [wmi:      evex.scc13.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0d) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3852, 350},
    /* 2145 : CTESTNO spec4,rm64,sdword64 [wmi:      evex.scc1.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 01) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3896, 350},
    /* 2146 : CTESTNS spec4,rm64,sdword64 [wmi:      evex.scc9.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 09) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3907, 350},
    /* 2147 : CTESTNZ spec4,rm64,sdword64 [wmi:      evex.scc5.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 05) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3885, 350},
    /* 2148 : CTESTO spec4,rm64,sdword64 [wmi:      evex.scc0.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 00) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3918, 350},
    /* 2149 : CTESTS spec4,rm64,sdword64 [wmi:      evex.scc8.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 08) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3929, 350},
    /* 2150 : CTESTT spec4,rm64,sdword64 [wmi:      evex.scc10.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0a) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3940, 350},
    /* 2151 : CTESTZ spec4,rm64,sdword64 [wmi:      evex.scc4.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 04) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3830, 350},
    /* 2152 : CTESTA spec4,rm8,imm8 [wmi:      evex.scc7.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 07) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14893, 348},
    /* 2153 : CTESTC spec4,rm8,imm8 [wmi:      evex.scc2.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 02) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14913, 348},
    /* 2154 : CTESTF spec4,rm8,imm8 [wmi:      evex.scc11.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0b) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14883, 348},
    /* 2155 : CTESTG spec4,rm8,imm8 [wmi:      evex.scc15.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0f) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14943, 348},
    /* 2156 : CTESTL spec4,rm8,imm8 [wmi:      evex.scc12.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0c) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14963, 348},
    /* 2157 : CTESTNA spec4,rm8,imm8 [wmi:      evex.scc6.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 06) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14923, 348},
    /* 2158 : CTESTNC spec4,rm8,imm8 [wmi:      evex.scc3.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 03) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14903, 348},
    /* 2159 : CTESTNG spec4,rm8,imm8 [wmi:      evex.scc14.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0e) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14973, 348},
    /* 2160 : CTESTNL spec4,rm8,imm8 [wmi:      evex.scc13.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0d) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14953, 348},
    /* 2161 : CTESTNO spec4,rm8,imm8 [wmi:      evex.scc1.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 01) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14993, 348},
    /* 2162 : CTESTNS spec4,rm8,imm8 [wmi:      evex.scc9.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 09) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+15003, 348},
    /* 2163 : CTESTNZ spec4,rm8,imm8 [wmi:      evex.scc5.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 05) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14983, 348},
    /* 2164 : CTESTO spec4,rm8,imm8 [wmi:      evex.scc0.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 00) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+15013, 348},
    /* 2165 : CTESTS spec4,rm8,imm8 [wmi:      evex.scc8.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 08) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+15023, 348},
    /* 2166 : CTESTT spec4,rm8,imm8 [wmi:      evex.scc10.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0a) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+15033, 348},
    /* 2167 : CTESTZ spec4,rm8,imm8 [wmi:      evex.scc4.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 04) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14933, 348},
    /* 2168 : CTESTA spec4,rm16,imm16 [wmi:      evex.scc7.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 07) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3962, 349},
    /* 2169 : CTESTC spec4,rm16,imm16 [wmi:      evex.scc2.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 02) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3984, 349},
    /* 2170 : CTESTF spec4,rm16,imm16 [wmi:      evex.scc11.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0b) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3951, 349},
    /* 2171 : CTESTG spec4,rm16,imm16 [wmi:      evex.scc15.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0f) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+4017, 349},
    /* 2172 : CTESTL spec4,rm16,imm16 [wmi:      evex.scc12.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0c) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+4039, 349},
    /* 2173 : CTESTNA spec4,rm16,imm16 [wmi:      evex.scc6.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 06) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3995, 349},
    /* 2174 : CTESTNC spec4,rm16,imm16 [wmi:      evex.scc3.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 03) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3973, 349},
    /* 2175 : CTESTNG spec4,rm16,imm16 [wmi:      evex.scc14.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0e) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+4050, 349},
    /* 2176 : CTESTNL spec4,rm16,imm16 [wmi:      evex.scc13.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0d) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+4028, 349},
    /* 2177 : CTESTNO spec4,rm16,imm16 [wmi:      evex.scc1.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 01) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+4072, 349},
    /* 2178 : CTESTNS spec4,rm16,imm16 [wmi:      evex.scc9.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 09) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+4083, 349},
    /* 2179 : CTESTNZ spec4,rm16,imm16 [wmi:      evex.scc5.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 05) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+4061, 349},
    /* 2180 : CTESTO spec4,rm16,imm16 [wmi:      evex.scc0.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 00) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+4094, 349},
    /* 2181 : CTESTS spec4,rm16,imm16 [wmi:      evex.scc8.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 08) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+4105, 349},
    /* 2182 : CTESTT spec4,rm16,imm16 [wmi:      evex.scc10.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0a) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+4116, 349},
    /* 2183 : CTESTZ spec4,rm16,imm16 [wmi:      evex.scc4.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 04) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+4006, 349},
    /* 2184 : CTESTA spec4,rm32,imm32 [wmi:      evex.scc7.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 07) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4138, 350},
    /* 2185 : CTESTC spec4,rm32,imm32 [wmi:      evex.scc2.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 02) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4160, 350},
    /* 2186 : CTESTF spec4,rm32,imm32 [wmi:      evex.scc11.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0b) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4127, 350},
    /* 2187 : CTESTG spec4,rm32,imm32 [wmi:      evex.scc15.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0f) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4193, 350},
    /* 2188 : CTESTL spec4,rm32,imm32 [wmi:      evex.scc12.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0c) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4215, 350},
    /* 2189 : CTESTNA spec4,rm32,imm32 [wmi:      evex.scc6.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 06) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4171, 350},
    /* 2190 : CTESTNC spec4,rm32,imm32 [wmi:      evex.scc3.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 03) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4149, 350},
    /* 2191 : CTESTNG spec4,rm32,imm32 [wmi:      evex.scc14.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0e) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4226, 350},
    /* 2192 : CTESTNL spec4,rm32,imm32 [wmi:      evex.scc13.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0d) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4204, 350},
    /* 2193 : CTESTNO spec4,rm32,imm32 [wmi:      evex.scc1.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 01) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4248, 350},
    /* 2194 : CTESTNS spec4,rm32,imm32 [wmi:      evex.scc9.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 09) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4259, 350},
    /* 2195 : CTESTNZ spec4,rm32,imm32 [wmi:      evex.scc5.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 05) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4237, 350},
    /* 2196 : CTESTO spec4,rm32,imm32 [wmi:      evex.scc0.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 00) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4270, 350},
    /* 2197 : CTESTS spec4,rm32,imm32 [wmi:      evex.scc8.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 08) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4281, 350},
    /* 2198 : CTESTT spec4,rm32,imm32 [wmi:      evex.scc10.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0a) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4292, 350},
    /* 2199 : CTESTZ spec4,rm32,imm32 [wmi:      evex.scc4.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 04) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4182, 350},
    /* 2200 : CTESTA spec4,rm64,sdword64 [wmi:      evex.scc7.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 07) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4314, 350},
    /* 2201 : CTESTC spec4,rm64,sdword64 [wmi:      evex.scc2.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 02) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4336, 350},
    /* 2202 : CTESTF spec4,rm64,sdword64 [wmi:      evex.scc11.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0b) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4303, 350},
    /* 2203 : CTESTG spec4,rm64,sdword64 [wmi:      evex.scc15.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0f) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4369, 350},
    /* 2204 : CTESTL spec4,rm64,sdword64 [wmi:      evex.scc12.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0c) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4391, 350},
    /* 2205 : CTESTNA spec4,rm64,sdword64 [wmi:      evex.scc6.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 06) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4347, 350},
    /* 2206 : CTESTNC spec4,rm64,sdword64 [wmi:      evex.scc3.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 03) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4325, 350},
    /* 2207 : CTESTNG spec4,rm64,sdword64 [wmi:      evex.scc14.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0e) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4402, 350},
    /* 2208 : CTESTNL spec4,rm64,sdword64 [wmi:      evex.scc13.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0d) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4380, 350},
    /* 2209 : CTESTNO spec4,rm64,sdword64 [wmi:      evex.scc1.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 01) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4424, 350},
    /* 2210 : CTESTNS spec4,rm64,sdword64 [wmi:      evex.scc9.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 09) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4435, 350},
    /* 2211 : CTESTNZ spec4,rm64,sdword64 [wmi:      evex.scc5.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 05) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4413, 350},
    /* 2212 : CTESTO spec4,rm64,sdword64 [wmi:      evex.scc0.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 00) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4446, 350},
    /* 2213 : CTESTS spec4,rm64,sdword64 [wmi:      evex.scc8.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 08) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4457, 350},
    /* 2214 : CTESTT spec4,rm64,sdword64 [wmi:      evex.scc10.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0a) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4468, 350},
    /* 2215 : CTESTZ spec4,rm64,sdword64 [wmi:      evex.scc4.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 04) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4358, 350},
    /* 2216 : CMOVA reg16,rm16 [rm:       o16 0f 47 /r                    ] SM0-1,P6 */
        /* 320 355 001(47) 110 : SM0,SM1,P6 */
        {I_CMOVA, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67916, 351},
    /* 2217 : CMOVC reg16,rm16 [rm:       o16 0f 42 /r                    ] SM0-1,P6 */
        /* 320 355 001(42) 110 : SM0,SM1,P6 */
        {I_CMOVC, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67928, 351},
    /* 2218 : CMOVG reg16,rm16 [rm:       o16 0f 4f /r                    ] SM0-1,P6 */
        /* 320 355 001(4f) 110 : SM0,SM1,P6 */
        {I_CMOVG, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67946, 351},
    /* 2219 : CMOVL reg16,rm16 [rm:       o16 0f 4c /r                    ] SM0-1,P6 */
        /* 320 355 001(4c) 110 : SM0,SM1,P6 */
        {I_CMOVL, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67958, 351},
    /* 2220 : CMOVNA reg16,rm16 [rm:       o16 0f 46 /r                    ] SM0-1,P6 */
        /* 320 355 001(46) 110 : SM0,SM1,P6 */
        {I_CMOVNA, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67934, 351},
    /* 2221 : CMOVNC reg16,rm16 [rm:       o16 0f 43 /r                    ] SM0-1,P6 */
        /* 320 355 001(43) 110 : SM0,SM1,P6 */
        {I_CMOVNC, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67922, 351},
    /* 2222 : CMOVNG reg16,rm16 [rm:       o16 0f 4e /r                    ] SM0-1,P6 */
        /* 320 355 001(4e) 110 : SM0,SM1,P6 */
        {I_CMOVNG, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67964, 351},
    /* 2223 : CMOVNL reg16,rm16 [rm:       o16 0f 4d /r                    ] SM0-1,P6 */
        /* 320 355 001(4d) 110 : SM0,SM1,P6 */
        {I_CMOVNL, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67952, 351},
    /* 2224 : CMOVNO reg16,rm16 [rm:       o16 0f 41 /r                    ] SM0-1,P6 */
        /* 320 355 001(41) 110 : SM0,SM1,P6 */
        {I_CMOVNO, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67976, 351},
    /* 2225 : CMOVNS reg16,rm16 [rm:       o16 0f 49 /r                    ] SM0-1,P6 */
        /* 320 355 001(49) 110 : SM0,SM1,P6 */
        {I_CMOVNS, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67988, 351},
    /* 2226 : CMOVNZ reg16,rm16 [rm:       o16 0f 45 /r                    ] SM0-1,P6 */
        /* 320 355 001(45) 110 : SM0,SM1,P6 */
        {I_CMOVNZ, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67970, 351},
    /* 2227 : CMOVO reg16,rm16 [rm:       o16 0f 40 /r                    ] SM0-1,P6 */
        /* 320 355 001(40) 110 : SM0,SM1,P6 */
        {I_CMOVO, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67994, 351},
    /* 2228 : CMOVPE reg16,rm16 [rm:       o16 0f 4a /r                    ] SM0-1,P6 */
        /* 320 355 001(4a) 110 : SM0,SM1,P6 */
        {I_CMOVPE, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+68000, 351},
    /* 2229 : CMOVPO reg16,rm16 [rm:       o16 0f 4b /r                    ] SM0-1,P6 */
        /* 320 355 001(4b) 110 : SM0,SM1,P6 */
        {I_CMOVPO, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67982, 351},
    /* 2230 : CMOVS reg16,rm16 [rm:       o16 0f 48 /r                    ] SM0-1,P6 */
        /* 320 355 001(48) 110 : SM0,SM1,P6 */
        {I_CMOVS, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+68006, 351},
    /* 2231 : CMOVZ reg16,rm16 [rm:       o16 0f 44 /r                    ] SM0-1,P6 */
        /* 320 355 001(44) 110 : SM0,SM1,P6 */
        {I_CMOVZ, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67940, 351},
    /* 2232 : CMOVA reg32,rm32 [rm:       o32 0f 47 /r                    ] SM0-1,P6 */
        /* 321 355 001(47) 110 : SM0,SM1,P6 */
        {I_CMOVA, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68012, 351},
    /* 2233 : CMOVC reg32,rm32 [rm:       o32 0f 42 /r                    ] SM0-1,P6 */
        /* 321 355 001(42) 110 : SM0,SM1,P6 */
        {I_CMOVC, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68024, 351},
    /* 2234 : CMOVG reg32,rm32 [rm:       o32 0f 4f /r                    ] SM0-1,P6 */
        /* 321 355 001(4f) 110 : SM0,SM1,P6 */
        {I_CMOVG, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68042, 351},
    /* 2235 : CMOVL reg32,rm32 [rm:       o32 0f 4c /r                    ] SM0-1,P6 */
        /* 321 355 001(4c) 110 : SM0,SM1,P6 */
        {I_CMOVL, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68054, 351},
    /* 2236 : CMOVNA reg32,rm32 [rm:       o32 0f 46 /r                    ] SM0-1,P6 */
        /* 321 355 001(46) 110 : SM0,SM1,P6 */
        {I_CMOVNA, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68030, 351},
    /* 2237 : CMOVNC reg32,rm32 [rm:       o32 0f 43 /r                    ] SM0-1,P6 */
        /* 321 355 001(43) 110 : SM0,SM1,P6 */
        {I_CMOVNC, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68018, 351},
    /* 2238 : CMOVNG reg32,rm32 [rm:       o32 0f 4e /r                    ] SM0-1,P6 */
        /* 321 355 001(4e) 110 : SM0,SM1,P6 */
        {I_CMOVNG, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68060, 351},
    /* 2239 : CMOVNL reg32,rm32 [rm:       o32 0f 4d /r                    ] SM0-1,P6 */
        /* 321 355 001(4d) 110 : SM0,SM1,P6 */
        {I_CMOVNL, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68048, 351},
    /* 2240 : CMOVNO reg32,rm32 [rm:       o32 0f 41 /r                    ] SM0-1,P6 */
        /* 321 355 001(41) 110 : SM0,SM1,P6 */
        {I_CMOVNO, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68072, 351},
    /* 2241 : CMOVNS reg32,rm32 [rm:       o32 0f 49 /r                    ] SM0-1,P6 */
        /* 321 355 001(49) 110 : SM0,SM1,P6 */
        {I_CMOVNS, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68084, 351},
    /* 2242 : CMOVNZ reg32,rm32 [rm:       o32 0f 45 /r                    ] SM0-1,P6 */
        /* 321 355 001(45) 110 : SM0,SM1,P6 */
        {I_CMOVNZ, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68066, 351},
    /* 2243 : CMOVO reg32,rm32 [rm:       o32 0f 40 /r                    ] SM0-1,P6 */
        /* 321 355 001(40) 110 : SM0,SM1,P6 */
        {I_CMOVO, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68090, 351},
    /* 2244 : CMOVPE reg32,rm32 [rm:       o32 0f 4a /r                    ] SM0-1,P6 */
        /* 321 355 001(4a) 110 : SM0,SM1,P6 */
        {I_CMOVPE, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68096, 351},
    /* 2245 : CMOVPO reg32,rm32 [rm:       o32 0f 4b /r                    ] SM0-1,P6 */
        /* 321 355 001(4b) 110 : SM0,SM1,P6 */
        {I_CMOVPO, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68078, 351},
    /* 2246 : CMOVS reg32,rm32 [rm:       o32 0f 48 /r                    ] SM0-1,P6 */
        /* 321 355 001(48) 110 : SM0,SM1,P6 */
        {I_CMOVS, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68102, 351},
    /* 2247 : CMOVZ reg32,rm32 [rm:       o32 0f 44 /r                    ] SM0-1,P6 */
        /* 321 355 001(44) 110 : SM0,SM1,P6 */
        {I_CMOVZ, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68036, 351},
    /* 2248 : CMOVA reg64,rm64 [rm:       o64 0f 47 /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(47) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVA, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68108, 14},
    /* 2249 : CMOVC reg64,rm64 [rm:       o64 0f 42 /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(42) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVC, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68120, 14},
    /* 2250 : CMOVG reg64,rm64 [rm:       o64 0f 4f /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(4f) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVG, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68138, 14},
    /* 2251 : CMOVL reg64,rm64 [rm:       o64 0f 4c /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(4c) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVL, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68150, 14},
    /* 2252 : CMOVNA reg64,rm64 [rm:       o64 0f 46 /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(46) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVNA, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68126, 14},
    /* 2253 : CMOVNC reg64,rm64 [rm:       o64 0f 43 /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(43) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVNC, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68114, 14},
    /* 2254 : CMOVNG reg64,rm64 [rm:       o64 0f 4e /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(4e) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVNG, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68156, 14},
    /* 2255 : CMOVNL reg64,rm64 [rm:       o64 0f 4d /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(4d) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVNL, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68144, 14},
    /* 2256 : CMOVNO reg64,rm64 [rm:       o64 0f 41 /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(41) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVNO, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68168, 14},
    /* 2257 : CMOVNS reg64,rm64 [rm:       o64 0f 49 /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(49) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVNS, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68180, 14},
    /* 2258 : CMOVNZ reg64,rm64 [rm:       o64 0f 45 /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(45) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVNZ, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68162, 14},
    /* 2259 : CMOVO reg64,rm64 [rm:       o64 0f 40 /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(40) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVO, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68186, 14},
    /* 2260 : CMOVPE reg64,rm64 [rm:       o64 0f 4a /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(4a) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVPE, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68192, 14},
    /* 2261 : CMOVPO reg64,rm64 [rm:       o64 0f 4b /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(4b) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVPO, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68174, 14},
    /* 2262 : CMOVS reg64,rm64 [rm:       o64 0f 48 /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(48) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVS, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68198, 14},
    /* 2263 : CMOVZ reg64,rm64 [rm:       o64 0f 44 /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(44) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVZ, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68132, 14},
    /* 2264 : CMOVA reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 47 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(47) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVA, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15043, 352},
    /* 2265 : CMOVC reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 42 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(42) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVC, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15063, 352},
    /* 2266 : CMOVG reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 4f /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(4f) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVG, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15093, 352},
    /* 2267 : CMOVL reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 4c /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(4c) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVL, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15113, 352},
    /* 2268 : CMOVNA reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 46 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(46) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNA, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15073, 352},
    /* 2269 : CMOVNC reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 43 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(43) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNC, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15053, 352},
    /* 2270 : CMOVNG reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 4e /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(4e) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNG, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15123, 352},
    /* 2271 : CMOVNL reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 4d /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(4d) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNL, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15103, 352},
    /* 2272 : CMOVNO reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 41 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(41) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNO, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15143, 352},
    /* 2273 : CMOVNS reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 49 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(49) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNS, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15163, 352},
    /* 2274 : CMOVNZ reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 45 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(45) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNZ, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15133, 352},
    /* 2275 : CMOVO reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 40 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(40) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVO, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15173, 352},
    /* 2276 : CMOVPE reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 4a /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(4a) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVPE, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15183, 352},
    /* 2277 : CMOVPO reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 4b /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(4b) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVPO, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15153, 352},
    /* 2278 : CMOVS reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 48 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(48) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVS, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15193, 352},
    /* 2279 : CMOVZ reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 44 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(44) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVZ, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15083, 352},
    /* 2280 : CMOVA reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 47 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(47) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVA, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15203, 353},
    /* 2281 : CMOVC reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 42 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(42) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVC, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15223, 353},
    /* 2282 : CMOVG reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 4f /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(4f) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVG, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15253, 353},
    /* 2283 : CMOVL reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 4c /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(4c) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVL, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15273, 353},
    /* 2284 : CMOVNA reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 46 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(46) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNA, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15233, 353},
    /* 2285 : CMOVNC reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 43 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(43) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNC, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15213, 353},
    /* 2286 : CMOVNG reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 4e /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(4e) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNG, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15283, 353},
    /* 2287 : CMOVNL reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 4d /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(4d) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNL, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15263, 353},
    /* 2288 : CMOVNO reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 41 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(41) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNO, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15303, 353},
    /* 2289 : CMOVNS reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 49 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(49) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNS, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15323, 353},
    /* 2290 : CMOVNZ reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 45 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(45) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNZ, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15293, 353},
    /* 2291 : CMOVO reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 40 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(40) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVO, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15333, 353},
    /* 2292 : CMOVPE reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 4a /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(4a) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVPE, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15343, 353},
    /* 2293 : CMOVPO reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 4b /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(4b) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVPO, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15313, 353},
    /* 2294 : CMOVS reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 48 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(48) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVS, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15353, 353},
    /* 2295 : CMOVZ reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 44 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(44) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVZ, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15243, 353},
    /* 2296 : CMOVA reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 47 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(47) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVA, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15363, 353},
    /* 2297 : CMOVC reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 42 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(42) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVC, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15383, 353},
    /* 2298 : CMOVG reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 4f /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(4f) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVG, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15413, 353},
    /* 2299 : CMOVL reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 4c /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(4c) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVL, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15433, 353},
    /* 2300 : CMOVNA reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 46 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(46) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNA, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15393, 353},
    /* 2301 : CMOVNC reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 43 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(43) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNC, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15373, 353},
    /* 2302 : CMOVNG reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 4e /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(4e) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNG, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15443, 353},
    /* 2303 : CMOVNL reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 4d /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(4d) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNL, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15423, 353},
    /* 2304 : CMOVNO reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 41 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(41) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNO, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15463, 353},
    /* 2305 : CMOVNS reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 49 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(49) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNS, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15483, 353},
    /* 2306 : CMOVNZ reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 45 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(45) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNZ, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15453, 353},
    /* 2307 : CMOVO reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 40 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(40) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVO, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15493, 353},
    /* 2308 : CMOVPE reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 4a /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(4a) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVPE, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15503, 353},
    /* 2309 : CMOVPO reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 4b /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(4b) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVPO, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15473, 353},
    /* 2310 : CMOVS reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 48 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(48) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVS, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15513, 353},
    /* 2311 : CMOVZ reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 44 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(44) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVZ, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15403, 353},
    /* 2312 : CFCMOVA rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 47 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(47) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVA, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15523, 354},
    /* 2313 : CFCMOVC rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 42 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(42) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVC, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15543, 354},
    /* 2314 : CFCMOVG rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 4f /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(4f) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVG, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15573, 354},
    /* 2315 : CFCMOVL rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 4c /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(4c) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVL, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15593, 354},
    /* 2316 : CFCMOVNA rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 46 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(46) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNA, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15553, 354},
    /* 2317 : CFCMOVNC rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 43 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(43) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNC, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15533, 354},
    /* 2318 : CFCMOVNG rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 4e /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(4e) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNG, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15603, 354},
    /* 2319 : CFCMOVNL rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 4d /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(4d) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNL, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15583, 354},
    /* 2320 : CFCMOVNO rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 41 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(41) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNO, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15623, 354},
    /* 2321 : CFCMOVNS rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 49 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(49) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNS, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15643, 354},
    /* 2322 : CFCMOVNZ rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 45 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(45) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNZ, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15613, 354},
    /* 2323 : CFCMOVO rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 40 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(40) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVO, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15653, 354},
    /* 2324 : CFCMOVPE rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 4a /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(4a) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPE, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15663, 354},
    /* 2325 : CFCMOVPO rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 4b /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(4b) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPO, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15633, 354},
    /* 2326 : CFCMOVS rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 48 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(48) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVS, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15673, 354},
    /* 2327 : CFCMOVZ rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 44 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(44) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVZ, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15563, 354},
    /* 2328 : CFCMOVA rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 47 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(47) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVA, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15683, 355},
    /* 2329 : CFCMOVC rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 42 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(42) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVC, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15703, 355},
    /* 2330 : CFCMOVG rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 4f /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(4f) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVG, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15733, 355},
    /* 2331 : CFCMOVL rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 4c /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(4c) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVL, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15753, 355},
    /* 2332 : CFCMOVNA rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 46 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(46) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNA, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15713, 355},
    /* 2333 : CFCMOVNC rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 43 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(43) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNC, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15693, 355},
    /* 2334 : CFCMOVNG rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 4e /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(4e) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNG, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15763, 355},
    /* 2335 : CFCMOVNL rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 4d /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(4d) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNL, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15743, 355},
    /* 2336 : CFCMOVNO rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 41 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(41) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNO, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15783, 355},
    /* 2337 : CFCMOVNS rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 49 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(49) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNS, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15803, 355},
    /* 2338 : CFCMOVNZ rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 45 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(45) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNZ, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15773, 355},
    /* 2339 : CFCMOVO rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 40 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(40) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVO, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15813, 355},
    /* 2340 : CFCMOVPE rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 4a /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(4a) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPE, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15823, 355},
    /* 2341 : CFCMOVPO rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 4b /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(4b) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPO, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15793, 355},
    /* 2342 : CFCMOVS rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 48 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(48) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVS, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15833, 355},
    /* 2343 : CFCMOVZ rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 44 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(44) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVZ, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15723, 355},
    /* 2344 : CFCMOVA rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 47 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(47) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVA, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15843, 355},
    /* 2345 : CFCMOVC rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 42 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(42) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVC, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15863, 355},
    /* 2346 : CFCMOVG rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 4f /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(4f) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVG, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15893, 355},
    /* 2347 : CFCMOVL rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 4c /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(4c) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVL, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15913, 355},
    /* 2348 : CFCMOVNA rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 46 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(46) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNA, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15873, 355},
    /* 2349 : CFCMOVNC rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 43 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(43) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNC, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15853, 355},
    /* 2350 : CFCMOVNG rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 4e /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(4e) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNG, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15923, 355},
    /* 2351 : CFCMOVNL rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 4d /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(4d) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNL, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15903, 355},
    /* 2352 : CFCMOVNO rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 41 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(41) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNO, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15943, 355},
    /* 2353 : CFCMOVNS rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 49 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(49) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNS, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15963, 355},
    /* 2354 : CFCMOVNZ rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 45 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(45) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNZ, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15933, 355},
    /* 2355 : CFCMOVO rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 40 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(40) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVO, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15973, 355},
    /* 2356 : CFCMOVPE rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 4a /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(4a) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPE, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15983, 355},
    /* 2357 : CFCMOVPO rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 4b /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(4b) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPO, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15953, 355},
    /* 2358 : CFCMOVS rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 48 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(48) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVS, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15993, 355},
    /* 2359 : CFCMOVZ rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 44 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(44) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVZ, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15883, 355},
    /* 2360 : CFCMOVA reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 47 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(47) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVA, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16003, 354},
    /* 2361 : CFCMOVC reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 42 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(42) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVC, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16023, 354},
    /* 2362 : CFCMOVG reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 4f /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(4f) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVG, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16053, 354},
    /* 2363 : CFCMOVL reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 4c /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(4c) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVL, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16073, 354},
    /* 2364 : CFCMOVNA reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 46 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(46) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNA, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16033, 354},
    /* 2365 : CFCMOVNC reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 43 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(43) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNC, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16013, 354},
    /* 2366 : CFCMOVNG reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 4e /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(4e) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNG, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16083, 354},
    /* 2367 : CFCMOVNL reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 4d /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(4d) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNL, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16063, 354},
    /* 2368 : CFCMOVNO reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 41 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(41) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNO, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16103, 354},
    /* 2369 : CFCMOVNS reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 49 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(49) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNS, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16123, 354},
    /* 2370 : CFCMOVNZ reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 45 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(45) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNZ, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16093, 354},
    /* 2371 : CFCMOVO reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 40 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(40) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVO, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16133, 354},
    /* 2372 : CFCMOVPE reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 4a /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(4a) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPE, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16143, 354},
    /* 2373 : CFCMOVPO reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 4b /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(4b) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPO, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16113, 354},
    /* 2374 : CFCMOVS reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 48 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(48) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVS, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16153, 354},
    /* 2375 : CFCMOVZ reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 44 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(44) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVZ, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16043, 354},
    /* 2376 : CFCMOVA reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 47 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(47) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVA, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16163, 355},
    /* 2377 : CFCMOVC reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 42 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(42) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVC, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16183, 355},
    /* 2378 : CFCMOVG reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 4f /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(4f) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVG, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16213, 355},
    /* 2379 : CFCMOVL reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 4c /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(4c) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVL, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16233, 355},
    /* 2380 : CFCMOVNA reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 46 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(46) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNA, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16193, 355},
    /* 2381 : CFCMOVNC reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 43 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(43) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNC, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16173, 355},
    /* 2382 : CFCMOVNG reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 4e /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(4e) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNG, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16243, 355},
    /* 2383 : CFCMOVNL reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 4d /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(4d) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNL, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16223, 355},
    /* 2384 : CFCMOVNO reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 41 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(41) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNO, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16263, 355},
    /* 2385 : CFCMOVNS reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 49 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(49) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNS, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16283, 355},
    /* 2386 : CFCMOVNZ reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 45 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(45) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNZ, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16253, 355},
    /* 2387 : CFCMOVO reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 40 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(40) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVO, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16293, 355},
    /* 2388 : CFCMOVPE reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 4a /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(4a) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPE, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16303, 355},
    /* 2389 : CFCMOVPO reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 4b /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(4b) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPO, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16273, 355},
    /* 2390 : CFCMOVS reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 48 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(48) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVS, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16313, 355},
    /* 2391 : CFCMOVZ reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 44 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(44) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVZ, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16203, 355},
    /* 2392 : CFCMOVA reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 47 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(47) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVA, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16323, 355},
    /* 2393 : CFCMOVC reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 42 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(42) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVC, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16343, 355},
    /* 2394 : CFCMOVG reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 4f /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(4f) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVG, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16373, 355},
    /* 2395 : CFCMOVL reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 4c /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(4c) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVL, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16393, 355},
    /* 2396 : CFCMOVNA reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 46 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(46) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNA, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16353, 355},
    /* 2397 : CFCMOVNC reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 43 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(43) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNC, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16333, 355},
    /* 2398 : CFCMOVNG reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 4e /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(4e) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNG, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16403, 355},
    /* 2399 : CFCMOVNL reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 4d /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(4d) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNL, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16383, 355},
    /* 2400 : CFCMOVNO reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 41 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(41) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNO, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16423, 355},
    /* 2401 : CFCMOVNS reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 49 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(49) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNS, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16443, 355},
    /* 2402 : CFCMOVNZ reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 45 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(45) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNZ, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16413, 355},
    /* 2403 : CFCMOVO reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 40 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(40) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVO, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16453, 355},
    /* 2404 : CFCMOVPE reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 4a /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(4a) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPE, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16463, 355},
    /* 2405 : CFCMOVPO reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 4b /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(4b) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPO, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16433, 355},
    /* 2406 : CFCMOVS reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 48 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(48) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVS, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16473, 355},
    /* 2407 : CFCMOVZ reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 44 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(44) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVZ, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16363, 355},
    /* 2408 : CFCMOVA reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 47 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(47) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVA, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16483, 356},
    /* 2409 : CFCMOVC reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 42 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(42) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVC, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16503, 356},
    /* 2410 : CFCMOVG reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 4f /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(4f) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVG, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16533, 356},
    /* 2411 : CFCMOVL reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 4c /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(4c) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVL, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16553, 356},
    /* 2412 : CFCMOVNA reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 46 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(46) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNA, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16513, 356},
    /* 2413 : CFCMOVNC reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 43 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(43) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNC, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16493, 356},
    /* 2414 : CFCMOVNG reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 4e /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(4e) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNG, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16563, 356},
    /* 2415 : CFCMOVNL reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 4d /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(4d) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNL, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16543, 356},
    /* 2416 : CFCMOVNO reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 41 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(41) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNO, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16583, 356},
    /* 2417 : CFCMOVNS reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 49 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(49) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNS, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16603, 356},
    /* 2418 : CFCMOVNZ reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 45 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(45) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNZ, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16573, 356},
    /* 2419 : CFCMOVO reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 40 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(40) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVO, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16613, 356},
    /* 2420 : CFCMOVPE reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 4a /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(4a) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPE, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16623, 356},
    /* 2421 : CFCMOVPO reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 4b /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(4b) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPO, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16593, 356},
    /* 2422 : CFCMOVS reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 48 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(48) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVS, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16633, 356},
    /* 2423 : CFCMOVZ reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 44 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(44) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVZ, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16523, 356},
    /* 2424 : CFCMOVA reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 47 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(47) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVA, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16643, 354},
    /* 2425 : CFCMOVC reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 42 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(42) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVC, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16663, 354},
    /* 2426 : CFCMOVG reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 4f /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(4f) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVG, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16693, 354},
    /* 2427 : CFCMOVL reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 4c /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(4c) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVL, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16713, 354},
    /* 2428 : CFCMOVNA reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 46 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(46) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNA, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16673, 354},
    /* 2429 : CFCMOVNC reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 43 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(43) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNC, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16653, 354},
    /* 2430 : CFCMOVNG reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 4e /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(4e) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNG, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16723, 354},
    /* 2431 : CFCMOVNL reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 4d /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(4d) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNL, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16703, 354},
    /* 2432 : CFCMOVNO reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 41 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(41) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNO, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16743, 354},
    /* 2433 : CFCMOVNS reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 49 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(49) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNS, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16763, 354},
    /* 2434 : CFCMOVNZ reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 45 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(45) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNZ, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16733, 354},
    /* 2435 : CFCMOVO reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 40 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(40) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVO, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16773, 354},
    /* 2436 : CFCMOVPE reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 4a /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(4a) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPE, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16783, 354},
    /* 2437 : CFCMOVPO reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 4b /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(4b) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPO, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16753, 354},
    /* 2438 : CFCMOVS reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 48 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(48) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVS, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16793, 354},
    /* 2439 : CFCMOVZ reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 44 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(44) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVZ, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16683, 354},
    /* 2440 : CFCMOVA reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 47 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(47) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVA, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+16963, 358},
    /* 2441 : CFCMOVC reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 42 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(42) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVC, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+16983, 358},
    /* 2442 : CFCMOVG reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 4f /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(4f) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVG, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+17013, 358},
    /* 2443 : CFCMOVL reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 4c /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(4c) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVL, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+17033, 358},
    /* 2444 : CFCMOVNA reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 46 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(46) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNA, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+16993, 358},
    /* 2445 : CFCMOVNC reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 43 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(43) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNC, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+16973, 358},
    /* 2446 : CFCMOVNG reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 4e /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(4e) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNG, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+17043, 358},
    /* 2447 : CFCMOVNL reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 4d /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(4d) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNL, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+17023, 358},
    /* 2448 : CFCMOVNO reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 41 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(41) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNO, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+17063, 358},
    /* 2449 : CFCMOVNS reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 49 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(49) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNS, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+17083, 358},
    /* 2450 : CFCMOVNZ reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 45 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(45) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNZ, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+17053, 358},
    /* 2451 : CFCMOVO reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 40 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(40) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVO, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+17093, 358},
    /* 2452 : CFCMOVPE reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 4a /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(4a) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPE, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+17103, 358},
    /* 2453 : CFCMOVPO reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 4b /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(4b) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPO, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+17073, 358},
    /* 2454 : CFCMOVS reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 48 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(48) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVS, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+17113, 358},
    /* 2455 : CFCMOVZ reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 44 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(44) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVZ, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+17003, 358},
    /* 2456 : CFCMOVA reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 47 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(47) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVA, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17123, 355},
    /* 2457 : CFCMOVC reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 42 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(42) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVC, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17143, 355},
    /* 2458 : CFCMOVG reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 4f /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(4f) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVG, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17173, 355},
    /* 2459 : CFCMOVL reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 4c /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(4c) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVL, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17193, 355},
    /* 2460 : CFCMOVNA reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 46 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(46) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNA, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17153, 355},
    /* 2461 : CFCMOVNC reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 43 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(43) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNC, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17133, 355},
    /* 2462 : CFCMOVNG reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 4e /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(4e) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNG, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17203, 355},
    /* 2463 : CFCMOVNL reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 4d /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(4d) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNL, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17183, 355},
    /* 2464 : CFCMOVNO reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 41 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(41) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNO, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17223, 355},
    /* 2465 : CFCMOVNS reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 49 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(49) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNS, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17243, 355},
    /* 2466 : CFCMOVNZ reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 45 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(45) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNZ, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17213, 355},
    /* 2467 : CFCMOVO reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 40 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(40) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVO, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17253, 355},
    /* 2468 : CFCMOVPE reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 4a /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(4a) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPE, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17263, 355},
    /* 2469 : CFCMOVPO reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 4b /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(4b) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPO, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17233, 355},
    /* 2470 : CFCMOVS reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 48 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(48) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVS, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17273, 355},
    /* 2471 : CFCMOVZ reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 44 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(44) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVZ, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17163, 355},
    /* 2472 : CFCMOVA reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 47 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(47) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVA, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17443, 358},
    /* 2473 : CFCMOVC reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 42 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(42) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVC, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17463, 358},
    /* 2474 : CFCMOVG reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 4f /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(4f) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVG, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17493, 358},
    /* 2475 : CFCMOVL reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 4c /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(4c) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVL, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17513, 358},
    /* 2476 : CFCMOVNA reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 46 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(46) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNA, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17473, 358},
    /* 2477 : CFCMOVNC reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 43 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(43) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNC, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17453, 358},
    /* 2478 : CFCMOVNG reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 4e /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(4e) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNG, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17523, 358},
    /* 2479 : CFCMOVNL reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 4d /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(4d) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNL, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17503, 358},
    /* 2480 : CFCMOVNO reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 41 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(41) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNO, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17543, 358},
    /* 2481 : CFCMOVNS reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 49 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(49) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNS, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17563, 358},
    /* 2482 : CFCMOVNZ reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 45 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(45) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNZ, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17533, 358},
    /* 2483 : CFCMOVO reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 40 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(40) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVO, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17573, 358},
    /* 2484 : CFCMOVPE reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 4a /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(4a) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPE, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17583, 358},
    /* 2485 : CFCMOVPO reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 4b /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(4b) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPO, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17553, 358},
    /* 2486 : CFCMOVS reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 48 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(48) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVS, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17593, 358},
    /* 2487 : CFCMOVZ reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 44 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(44) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVZ, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17483, 358},
    /* 2488 : CFCMOVA reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 47 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(47) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVA, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17603, 355},
    /* 2489 : CFCMOVC reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 42 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(42) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVC, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17623, 355},
    /* 2490 : CFCMOVG reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 4f /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(4f) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVG, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17653, 355},
    /* 2491 : CFCMOVL reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 4c /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(4c) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVL, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17673, 355},
    /* 2492 : CFCMOVNA reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 46 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(46) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNA, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17633, 355},
    /* 2493 : CFCMOVNC reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 43 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(43) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNC, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17613, 355},
    /* 2494 : CFCMOVNG reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 4e /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(4e) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNG, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17683, 355},
    /* 2495 : CFCMOVNL reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 4d /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(4d) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNL, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17663, 355},
    /* 2496 : CFCMOVNO reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 41 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(41) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNO, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17703, 355},
    /* 2497 : CFCMOVNS reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 49 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(49) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNS, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17723, 355},
    /* 2498 : CFCMOVNZ reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 45 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(45) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNZ, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17693, 355},
    /* 2499 : CFCMOVO reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 40 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(40) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVO, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17733, 355},
    /* 2500 : CFCMOVPE reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 4a /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(4a) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPE, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17743, 355},
    /* 2501 : CFCMOVPO reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 4b /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(4b) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPO, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17713, 355},
    /* 2502 : CFCMOVS reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 48 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(48) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVS, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17753, 355},
    /* 2503 : CFCMOVZ reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 44 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(44) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVZ, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17643, 355},
    /* 2504 : SETA rm8 [m:        0f 97 /0                        ] AR0,SB,386 */
        /* 355 001(97) 200 : AR0,SB,386 */
        {I_SETA, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72223, 360},
    /* 2505 : SETC rm8 [m:        0f 92 /0                        ] AR0,SB,386 */
        /* 355 001(92) 200 : AR0,SB,386 */
        {I_SETC, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72233, 360},
    /* 2506 : SETG rm8 [m:        0f 9f /0                        ] AR0,SB,386 */
        /* 355 001(9f) 200 : AR0,SB,386 */
        {I_SETG, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72248, 360},
    /* 2507 : SETL rm8 [m:        0f 9c /0                        ] AR0,SB,386 */
        /* 355 001(9c) 200 : AR0,SB,386 */
        {I_SETL, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72258, 360},
    /* 2508 : SETNA rm8 [m:        0f 96 /0                        ] AR0,SB,386 */
        /* 355 001(96) 200 : AR0,SB,386 */
        {I_SETNA, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72238, 360},
    /* 2509 : SETNC rm8 [m:        0f 93 /0                        ] AR0,SB,386 */
        /* 355 001(93) 200 : AR0,SB,386 */
        {I_SETNC, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72228, 360},
    /* 2510 : SETNG rm8 [m:        0f 9e /0                        ] AR0,SB,386 */
        /* 355 001(9e) 200 : AR0,SB,386 */
        {I_SETNG, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72263, 360},
    /* 2511 : SETNL rm8 [m:        0f 9d /0                        ] AR0,SB,386 */
        /* 355 001(9d) 200 : AR0,SB,386 */
        {I_SETNL, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72253, 360},
    /* 2512 : SETNO rm8 [m:        0f 91 /0                        ] AR0,SB,386 */
        /* 355 001(91) 200 : AR0,SB,386 */
        {I_SETNO, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72273, 360},
    /* 2513 : SETNS rm8 [m:        0f 99 /0                        ] AR0,SB,386 */
        /* 355 001(99) 200 : AR0,SB,386 */
        {I_SETNS, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72283, 360},
    /* 2514 : SETNZ rm8 [m:        0f 95 /0                        ] AR0,SB,386 */
        /* 355 001(95) 200 : AR0,SB,386 */
        {I_SETNZ, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72268, 360},
    /* 2515 : SETO rm8 [m:        0f 90 /0                        ] AR0,SB,386 */
        /* 355 001(90) 200 : AR0,SB,386 */
        {I_SETO, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72288, 360},
    /* 2516 : SETPE rm8 [m:        0f 9a /0                        ] AR0,SB,386 */
        /* 355 001(9a) 200 : AR0,SB,386 */
        {I_SETPE, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72293, 360},
    /* 2517 : SETPO rm8 [m:        0f 9b /0                        ] AR0,SB,386 */
        /* 355 001(9b) 200 : AR0,SB,386 */
        {I_SETPO, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72278, 360},
    /* 2518 : SETS rm8 [m:        0f 98 /0                        ] AR0,SB,386 */
        /* 355 001(98) 200 : AR0,SB,386 */
        {I_SETS, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72298, 360},
    /* 2519 : SETZ rm8 [m:        0f 94 /0                        ] AR0,SB,386 */
        /* 355 001(94) 200 : AR0,SB,386 */
        {I_SETZ, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72243, 360},
    /* 2520 : SETA reg64 [m:        evex.nd1.l0.f2.m4.wig 47 /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(47) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETA, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25191, 361},
    /* 2521 : SETC reg64 [m:        evex.nd1.l0.f2.m4.wig 42 /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(42) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETC, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25209, 361},
    /* 2522 : SETG reg64 [m:        evex.nd1.l0.f2.m4.wig 4f /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(4f) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETG, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25236, 361},
    /* 2523 : SETL reg64 [m:        evex.nd1.l0.f2.m4.wig 4c /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(4c) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETL, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25254, 361},
    /* 2524 : SETNA reg64 [m:        evex.nd1.l0.f2.m4.wig 46 /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(46) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNA, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25218, 361},
    /* 2525 : SETNC reg64 [m:        evex.nd1.l0.f2.m4.wig 43 /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(43) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNC, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25200, 361},
    /* 2526 : SETNG reg64 [m:        evex.nd1.l0.f2.m4.wig 4e /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(4e) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNG, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25263, 361},
    /* 2527 : SETNL reg64 [m:        evex.nd1.l0.f2.m4.wig 4d /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(4d) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNL, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25245, 361},
    /* 2528 : SETNO reg64 [m:        evex.nd1.l0.f2.m4.wig 41 /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(41) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNO, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25281, 361},
    /* 2529 : SETNS reg64 [m:        evex.nd1.l0.f2.m4.wig 49 /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(49) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNS, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25299, 361},
    /* 2530 : SETNZ reg64 [m:        evex.nd1.l0.f2.m4.wig 45 /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(45) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNZ, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25272, 361},
    /* 2531 : SETO reg64 [m:        evex.nd1.l0.f2.m4.wig 40 /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(40) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETO, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25308, 361},
    /* 2532 : SETPE reg64 [m:        evex.nd1.l0.f2.m4.wig 4a /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(4a) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETPE, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25317, 361},
    /* 2533 : SETPO reg64 [m:        evex.nd1.l0.f2.m4.wig 4b /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(4b) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETPO, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25290, 361},
    /* 2534 : SETS reg64 [m:        evex.nd1.l0.f2.m4.wig 48 /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(48) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETS, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25326, 361},
    /* 2535 : SETZ reg64 [m:        evex.nd1.l0.f2.m4.wig 44 /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(44) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETZ, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25227, 361},
    /* 2536 : SETA rm8 [m:        evex.zu.l0.f2.m4.wig 47 /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(47) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETA, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25335, 362},
    /* 2537 : SETC rm8 [m:        evex.zu.l0.f2.m4.wig 42 /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(42) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETC, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25353, 362},
    /* 2538 : SETG rm8 [m:        evex.zu.l0.f2.m4.wig 4f /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(4f) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETG, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25380, 362},
    /* 2539 : SETL rm8 [m:        evex.zu.l0.f2.m4.wig 4c /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(4c) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETL, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25398, 362},
    /* 2540 : SETNA rm8 [m:        evex.zu.l0.f2.m4.wig 46 /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(46) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNA, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25362, 362},
    /* 2541 : SETNC rm8 [m:        evex.zu.l0.f2.m4.wig 43 /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(43) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNC, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25344, 362},
    /* 2542 : SETNG rm8 [m:        evex.zu.l0.f2.m4.wig 4e /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(4e) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNG, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25407, 362},
    /* 2543 : SETNL rm8 [m:        evex.zu.l0.f2.m4.wig 4d /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(4d) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNL, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25389, 362},
    /* 2544 : SETNO rm8 [m:        evex.zu.l0.f2.m4.wig 41 /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(41) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNO, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25425, 362},
    /* 2545 : SETNS rm8 [m:        evex.zu.l0.f2.m4.wig 49 /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(49) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNS, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25443, 362},
    /* 2546 : SETNZ rm8 [m:        evex.zu.l0.f2.m4.wig 45 /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(45) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNZ, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25416, 362},
    /* 2547 : SETO rm8 [m:        evex.zu.l0.f2.m4.wig 40 /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(40) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETO, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25452, 362},
    /* 2548 : SETPE rm8 [m:        evex.zu.l0.f2.m4.wig 4a /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(4a) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETPE, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25461, 362},
    /* 2549 : SETPO rm8 [m:        evex.zu.l0.f2.m4.wig 4b /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(4b) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETPO, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25434, 362},
    /* 2550 : SETS rm8 [m:        evex.zu.l0.f2.m4.wig 48 /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(48) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETS, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25470, 362},
    /* 2551 : SETZ rm8 [m:        evex.zu.l0.f2.m4.wig 44 /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(44) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETZ, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25371, 362},
    /* 2552 : CMPAXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 e7 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(e7) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPAXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57418, 363},
    /* 2553 : CMPCXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 e2 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(e2) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPCXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57432, 363},
    /* 2554 : CMPGXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 ef /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(ef) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPGXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57453, 363},
    /* 2555 : CMPLXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 ec /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(ec) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPLXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57467, 363},
    /* 2556 : CMPNAXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 e6 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(e6) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNAXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57439, 363},
    /* 2557 : CMPNCXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 e3 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(e3) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNCXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57425, 363},
    /* 2558 : CMPNGXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 ee /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(ee) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNGXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57474, 363},
    /* 2559 : CMPNLXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 ed /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(ed) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNLXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57460, 363},
    /* 2560 : CMPNOXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 e1 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(e1) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNOXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57488, 363},
    /* 2561 : CMPNSXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 e9 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(e9) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNSXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57502, 363},
    /* 2562 : CMPNZXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 e5 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(e5) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNZXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57481, 363},
    /* 2563 : CMPOXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 e0 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(e0) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPOXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57509, 363},
    /* 2564 : CMPPEXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 ea /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(ea) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPPEXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57516, 363},
    /* 2565 : CMPPOXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 eb /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(eb) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPPOXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57495, 363},
    /* 2566 : CMPSXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 e8 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(e8) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPSXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57523, 363},
    /* 2567 : CMPZXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 e4 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(e4) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPZXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57446, 363},
    /* 2568 : CMPAXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 e7 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(e7) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPAXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57530, 363},
    /* 2569 : CMPCXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 e2 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(e2) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPCXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57544, 363},
    /* 2570 : CMPGXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 ef /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(ef) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPGXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57565, 363},
    /* 2571 : CMPLXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 ec /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(ec) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPLXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57579, 363},
    /* 2572 : CMPNAXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 e6 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(e6) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNAXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57551, 363},
    /* 2573 : CMPNCXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 e3 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(e3) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNCXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57537, 363},
    /* 2574 : CMPNGXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 ee /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(ee) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNGXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57586, 363},
    /* 2575 : CMPNLXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 ed /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(ed) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNLXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57572, 363},
    /* 2576 : CMPNOXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 e1 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(e1) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNOXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57600, 363},
    /* 2577 : CMPNSXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 e9 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(e9) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNSXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57614, 363},
    /* 2578 : CMPNZXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 e5 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(e5) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNZXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57593, 363},
    /* 2579 : CMPOXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 e0 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(e0) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPOXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57621, 363},
    /* 2580 : CMPPEXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 ea /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(ea) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPPEXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57628, 363},
    /* 2581 : CMPPOXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 eb /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(eb) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPPOXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57607, 363},
    /* 2582 : CMPSXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 e8 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(e8) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPSXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57635, 363},
    /* 2583 : CMPZXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 e4 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(e4) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPZXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57558, 363},
    /* 2584 : CMPAXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 e7 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(e7) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPAXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25479, 364},
    /* 2585 : CMPCXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 e2 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(e2) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPCXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25497, 364},
    /* 2586 : CMPGXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 ef /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(ef) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPGXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25524, 364},
    /* 2587 : CMPLXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 ec /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(ec) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPLXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25542, 364},
    /* 2588 : CMPNAXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 e6 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(e6) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNAXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25506, 364},
    /* 2589 : CMPNCXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 e3 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(e3) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNCXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25488, 364},
    /* 2590 : CMPNGXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 ee /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(ee) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNGXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25551, 364},
    /* 2591 : CMPNLXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 ed /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(ed) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNLXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25533, 364},
    /* 2592 : CMPNOXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 e1 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(e1) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNOXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25569, 364},
    /* 2593 : CMPNSXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 e9 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(e9) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNSXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25587, 364},
    /* 2594 : CMPNZXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 e5 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(e5) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNZXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25560, 364},
    /* 2595 : CMPOXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 e0 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(e0) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPOXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25596, 364},
    /* 2596 : CMPPEXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 ea /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(ea) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPPEXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25605, 364},
    /* 2597 : CMPPOXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 eb /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(eb) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPPOXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25578, 364},
    /* 2598 : CMPSXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 e8 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(e8) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPSXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25614, 364},
    /* 2599 : CMPZXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 e4 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(e4) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPZXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25515, 364},
    /* 2600 : CMPAXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 e7 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(e7) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPAXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25623, 364},
    /* 2601 : CMPCXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 e2 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(e2) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPCXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25641, 364},
    /* 2602 : CMPGXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 ef /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(ef) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPGXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25668, 364},
    /* 2603 : CMPLXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 ec /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(ec) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPLXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25686, 364},
    /* 2604 : CMPNAXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 e6 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(e6) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNAXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25650, 364},
    /* 2605 : CMPNCXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 e3 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(e3) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNCXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25632, 364},
    /* 2606 : CMPNGXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 ee /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(ee) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNGXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25695, 364},
    /* 2607 : CMPNLXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 ed /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(ed) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNLXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25677, 364},
    /* 2608 : CMPNOXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 e1 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(e1) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNOXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25713, 364},
    /* 2609 : CMPNSXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 e9 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(e9) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNSXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25731, 364},
    /* 2610 : CMPNZXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 e5 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(e5) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNZXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25704, 364},
    /* 2611 : CMPOXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 e0 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(e0) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPOXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25740, 364},
    /* 2612 : CMPPEXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 ea /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(ea) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPPEXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25749, 364},
    /* 2613 : CMPPOXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 eb /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(eb) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPPOXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25722, 364},
    /* 2614 : CMPSXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 e8 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(e8) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPSXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25758, 364},
    /* 2615 : CMPZXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 e4 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(e4) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPZXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25659, 364},
    /* 2616 : ADDPS xmmreg,xmmrm128 [rm:       np 0f 58 /r                       ] SSE,KATMAI */
        /* 360 355 001(58) 110 : SSE,KATMAI */
        {I_ADDPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68204, 365},
    /* 2617 : ADDSS xmmreg,xmmrm32 [rm:       f3 0f 58 /r                       ] SSE,KATMAI */
        /* 333 355 001(58) 110 : SSE,KATMAI */
        {I_ADDSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68210, 365},
    /* 2618 : ANDNPS xmmreg,xmmrm128 [rm:       np 0f 55 /r                       ] SSE,KATMAI */
        /* 360 355 001(55) 110 : SSE,KATMAI */
        {I_ANDNPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68216, 365},
    /* 2619 : ANDPS xmmreg,xmmrm128 [rm:       np 0f 54 /r                       ] SSE,KATMAI */
        /* 360 355 001(54) 110 : SSE,KATMAI */
        {I_ANDPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68222, 365},
    /* 2620 : CMPEQPS xmmreg,xmmrm128 [rm:       np 0f c2 /r 00                    ] SSE,KATMAI */
        /* 360 355 001(c2) 110 001(00) : SSE,KATMAI */
        {I_CMPEQPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+51261, 365},
    /* 2621 : CMPEQSS xmmreg,xmmrm32 [rm:       f3 0f c2 /r 00                    ] SSE,KATMAI */
        /* 333 355 001(c2) 110 001(00) : SSE,KATMAI */
        {I_CMPEQSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+51269, 365},
    /* 2622 : CMPLEPS xmmreg,xmmrm128 [rm:       np 0f c2 /r 02                    ] SSE,KATMAI */
        /* 360 355 001(c2) 110 001(02) : SSE,KATMAI */
        {I_CMPLEPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+51277, 365},
    /* 2623 : CMPLESS xmmreg,xmmrm32 [rm:       f3 0f c2 /r 02                    ] SSE,KATMAI */
        /* 333 355 001(c2) 110 001(02) : SSE,KATMAI */
        {I_CMPLESS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+51285, 365},
    /* 2624 : CMPLTPS xmmreg,xmmrm128 [rm:       np 0f c2 /r 01                    ] SSE,KATMAI */
        /* 360 355 001(c2) 110 001(01) : SSE,KATMAI */
        {I_CMPLTPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+51293, 365},
    /* 2625 : CMPLTSS xmmreg,xmmrm32 [rm:       f3 0f c2 /r 01                    ] SSE,KATMAI */
        /* 333 355 001(c2) 110 001(01) : SSE,KATMAI */
        {I_CMPLTSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+51301, 365},
    /* 2626 : CMPNEQPS xmmreg,xmmrm128 [rm:       np 0f c2 /r 04                    ] SSE,KATMAI */
        /* 360 355 001(c2) 110 001(04) : SSE,KATMAI */
        {I_CMPNEQPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+51309, 365},
    /* 2627 : CMPNEQSS xmmreg,xmmrm32 [rm:       f3 0f c2 /r 04                    ] SSE,KATMAI */
        /* 333 355 001(c2) 110 001(04) : SSE,KATMAI */
        {I_CMPNEQSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+51317, 365},
    /* 2628 : CMPNLEPS xmmreg,xmmrm128 [rm:       np 0f c2 /r 06                    ] SSE,KATMAI */
        /* 360 355 001(c2) 110 001(06) : SSE,KATMAI */
        {I_CMPNLEPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+51325, 365},
    /* 2629 : CMPNLESS xmmreg,xmmrm32 [rm:       f3 0f c2 /r 06                    ] SSE,KATMAI */
        /* 333 355 001(c2) 110 001(06) : SSE,KATMAI */
        {I_CMPNLESS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+51333, 365},
    /* 2630 : CMPNLTPS xmmreg,xmmrm128 [rm:       np 0f c2 /r 05                    ] SSE,KATMAI */
        /* 360 355 001(c2) 110 001(05) : SSE,KATMAI */
        {I_CMPNLTPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+51341, 365},
    /* 2631 : CMPNLTSS xmmreg,xmmrm32 [rm:       f3 0f c2 /r 05                    ] SSE,KATMAI */
        /* 333 355 001(c2) 110 001(05) : SSE,KATMAI */
        {I_CMPNLTSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+51349, 365},
    /* 2632 : CMPORDPS xmmreg,xmmrm128 [rm:       np 0f c2 /r 07                    ] SSE,KATMAI */
        /* 360 355 001(c2) 110 001(07) : SSE,KATMAI */
        {I_CMPORDPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+51357, 365},
    /* 2633 : CMPORDSS xmmreg,xmmrm32 [rm:       f3 0f c2 /r 07                    ] SSE,KATMAI */
        /* 333 355 001(c2) 110 001(07) : SSE,KATMAI */
        {I_CMPORDSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+51365, 365},
    /* 2634 : CMPUNORDPS xmmreg,xmmrm128 [rm:       np 0f c2 /r 03                    ] SSE,KATMAI */
        /* 360 355 001(c2) 110 001(03) : SSE,KATMAI */
        {I_CMPUNORDPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+51373, 365},
    /* 2635 : CMPUNORDSS xmmreg,xmmrm32 [rm:       f3 0f c2 /r 03                    ] SSE,KATMAI */
        /* 333 355 001(c2) 110 001(03) : SSE,KATMAI */
        {I_CMPUNORDSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+51381, 365},
    /* 2636 : CMPPS xmmreg,xmmrm128,imm8 [rmi:      np 0f c2 /r ib,u                  ] SSE,KATMAI */
        /* 360 355 001(c2) 110 026 : SSE,KATMAI */
        {I_CMPPS, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+57642, 365},
    /* 2637 : CMPSS xmmreg,xmmrm32,imm8 [rmi:      f3 0f c2 /r ib,u                  ] SSE,KATMAI */
        /* 333 355 001(c2) 110 026 : SSE,KATMAI */
        {I_CMPSS, 3, {XMMREG,RM_XMM|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+57649, 365},
    /* 2638 : COMISS xmmreg,xmmrm32 [rm:       np 0f 2f /r                       ] FL,SSE,KATMAI */
        /* 360 355 001(2f) 110 : FL,SSE,KATMAI */
        {I_COMISS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68228, 366},
    /* 2639 : CVTPI2PS xmmreg,mmxrm64 [rm:       np 0f 2a /r                       ] MMX,SSE,KATMAI */
        /* 360 355 001(2a) 110 : MMX,SSE,KATMAI */
        {I_CVTPI2PS, 2, {XMMREG,RM_MMX|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68234, 367},
    /* 2640 : CVTPS2PI mmxreg,xmmrm64 [rm:       np 0f 2d /r                       ] MMX,SSE,KATMAI */
        /* 360 355 001(2d) 110 : MMX,SSE,KATMAI */
        {I_CVTPS2PI, 2, {MMXREG,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68240, 367},
    /* 2641 : CVTSI2SS xmmreg,rm32 [rm:       f3 0f 2a /r                       ] SSE,KATMAI */
        /* 333 355 001(2a) 110 : SSE,KATMAI */
        {I_CVTSI2SS, 2, {XMMREG,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+57657, 365},
    /* 2642 : CVTSI2SS xmmreg,rm64 [rm:       o64 f3 0f 2a /r                   ] AR1,SX,LONG,PROT,SSE,X86_64 */
        /* 324 333 355 001(2a) 110 : AR1,SX,LONG,PROT,SSE,X86_64 */
        {I_CVTSI2SS, 2, {XMMREG,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+57656, 368},
    /* 2643 : CVTSS2SI reg32,xmmrm32 [rm:       f3 0f 2d /r                       ] SSE,KATMAI */
        /* 333 355 001(2d) 110 : SSE,KATMAI */
        {I_CVTSS2SI, 2, {REG_GPR|BITS32,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+57664, 365},
    /* 2644 : CVTSS2SI reg64,xmmrm32 [rm:       o64 f3 0f 2d /r                   ] LONG,PROT,SSE,X86_64 */
        /* 324 333 355 001(2d) 110 : LONG,PROT,SSE,X86_64 */
        {I_CVTSS2SI, 2, {REG_GPR|BITS64,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+57663, 369},
    /* 2645 : CVTTPS2PI mmxreg,xmmrm64 [rm:       np 0f 2c /r                       ] MMX,SSE,KATMAI */
        /* 360 355 001(2c) 110 : MMX,SSE,KATMAI */
        {I_CVTTPS2PI, 2, {MMXREG,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68246, 367},
    /* 2646 : CVTTSS2SI reg32,xmmrm32 [rm:       f3 0f 2c /r                       ] SSE,KATMAI */
        /* 333 355 001(2c) 110 : SSE,KATMAI */
        {I_CVTTSS2SI, 2, {REG_GPR|BITS32,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+57671, 365},
    /* 2647 : CVTTSS2SI reg64,xmmrm32 [rm:       o64 f3 0f 2c /r                   ] LONG,PROT,SSE,X86_64 */
        /* 324 333 355 001(2c) 110 : LONG,PROT,SSE,X86_64 */
        {I_CVTTSS2SI, 2, {REG_GPR|BITS64,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+57670, 369},
    /* 2648 : DIVPS xmmreg,xmmrm128 [rm:       np 0f 5e /r                       ] SSE,KATMAI */
        /* 360 355 001(5e) 110 : SSE,KATMAI */
        {I_DIVPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68252, 365},
    /* 2649 : DIVSS xmmreg,xmmrm32 [rm:       f3 0f 5e /r                       ] SSE,KATMAI */
        /* 333 355 001(5e) 110 : SSE,KATMAI */
        {I_DIVSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68258, 365},
    /* 2650 : LDMXCSR mem32 [m:        np 0f ae /2                       ] SSE,KATMAI */
        /* 360 355 001(ae) 202 : SSE,KATMAI */
        {I_LDMXCSR, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+68264, 365},
    /* 2651 : MAXPS xmmreg,xmmrm128 [rm:       np 0f 5f /r                       ] SSE,KATMAI */
        /* 360 355 001(5f) 110 : SSE,KATMAI */
        {I_MAXPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68270, 365},
    /* 2652 : MAXSS xmmreg,xmmrm32 [rm:       f3 0f 5f /r                       ] SSE,KATMAI */
        /* 333 355 001(5f) 110 : SSE,KATMAI */
        {I_MAXSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68276, 365},
    /* 2653 : MINPS xmmreg,xmmrm128 [rm:       np 0f 5d /r                       ] SSE,KATMAI */
        /* 360 355 001(5d) 110 : SSE,KATMAI */
        {I_MINPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68282, 365},
    /* 2654 : MINSS xmmreg,xmmrm32 [rm:       f3 0f 5d /r                       ] SSE,KATMAI */
        /* 333 355 001(5d) 110 : SSE,KATMAI */
        {I_MINSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68288, 365},
    /* 2655 : MOVAPS xmmreg,xmmrm128 [rm:       np 0f 28 /r                       ] SSE,KATMAI */
        /* 360 355 001(28) 110 : SSE,KATMAI */
        {I_MOVAPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68294, 365},
    /* 2656 : MOVAPS xmmrm128,xmmreg [mr:       np 0f 29 /r                       ] SSE,KATMAI */
        /* 360 355 001(29) 101 : SSE,KATMAI */
        {I_MOVAPS, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68300, 365},
    /* 2657 : MOVHPS xmmreg,mem64 [rm:       np 0f 16 /r                       ] SSE,KATMAI */
        /* 360 355 001(16) 110 : SSE,KATMAI */
        {I_MOVHPS, 2, {XMMREG,MEMORY|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68306, 365},
    /* 2658 : MOVHPS mem64,xmmreg [mr:       np 0f 17 /r                       ] SSE,KATMAI */
        /* 360 355 001(17) 101 : SSE,KATMAI */
        {I_MOVHPS, 2, {MEMORY|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68312, 365},
    /* 2659 : MOVLHPS xmmreg,xmmreg [rm:       np 0f 16 /r                       ] SSE,KATMAI */
        /* 360 355 001(16) 110 : SSE,KATMAI */
        {I_MOVLHPS, 2, {XMMREG,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68306, 365},
    /* 2660 : MOVLPS xmmreg,mem64 [rm:       np 0f 12 /r                       ] SSE,KATMAI */
        /* 360 355 001(12) 110 : SSE,KATMAI */
        {I_MOVLPS, 2, {XMMREG,MEMORY|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+67286, 365},
    /* 2661 : MOVLPS mem64,xmmreg [mr:       np 0f 13 /r                       ] SSE,KATMAI */
        /* 360 355 001(13) 101 : SSE,KATMAI */
        {I_MOVLPS, 2, {MEMORY|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68318, 365},
    /* 2662 : MOVHLPS xmmreg,xmmreg [rm:       np 0f 12 /r                       ] SSE,KATMAI */
        /* 360 355 001(12) 110 : SSE,KATMAI */
        {I_MOVHLPS, 2, {XMMREG,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+67286, 365},
    /* 2663 : MOVMSKPS reg32,xmmreg [rm:       np 0f 50 /r                       ] SSE,KATMAI */
        /* 360 355 001(50) 110 : SSE,KATMAI */
        {I_MOVMSKPS, 2, {REG_GPR|BITS32,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68324, 365},
    /* 2664 : MOVMSKPS reg64,xmmreg [rm:       np o64 0f 50 /r                   ] LONG,PROT,SSE,X86_64 */
        /* 360 324 355 001(50) 110 : LONG,PROT,SSE,X86_64 */
        {I_MOVMSKPS, 2, {REG_GPR|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+57677, 369},
    /* 2665 : MOVNTPS mem128,xmmreg [mr:       np 0f 2b /r                       ] SSE,KATMAI */
        /* 360 355 001(2b) 101 : SSE,KATMAI */
        {I_MOVNTPS, 2, {MEMORY|BITS128,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68330, 365},
    /* 2666 : MOVSS xmmreg,xmmrm32 [rm:       f3 0f 10 /r                       ] SSE,KATMAI */
        /* 333 355 001(10) 110 : SSE,KATMAI */
        {I_MOVSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68336, 365},
    /* 2667 : MOVSS xmmrm32,xmmreg [mr:       f3 0f 11 /r                       ] SSE,KATMAI */
        /* 333 355 001(11) 101 : SSE,KATMAI */
        {I_MOVSS, 2, {RM_XMM|BITS32,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68342, 365},
    /* 2668 : MOVUPS xmmreg,xmmrm128 [rm:       np 0f 10 /r                       ] SSE,KATMAI */
        /* 360 355 001(10) 110 : SSE,KATMAI */
        {I_MOVUPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68348, 365},
    /* 2669 : MOVUPS xmmrm128,xmmreg [mr:       np 0f 11 /r                       ] SSE,KATMAI */
        /* 360 355 001(11) 101 : SSE,KATMAI */
        {I_MOVUPS, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68354, 365},
    /* 2670 : MULPS xmmreg,xmmrm128 [rm:       np 0f 59 /r                       ] SSE,KATMAI */
        /* 360 355 001(59) 110 : SSE,KATMAI */
        {I_MULPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68360, 365},
    /* 2671 : MULSS xmmreg,xmmrm32 [rm:       f3 0f 59 /r                       ] SSE,KATMAI */
        /* 333 355 001(59) 110 : SSE,KATMAI */
        {I_MULSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68366, 365},
    /* 2672 : ORPS xmmreg,xmmrm128 [rm:       np 0f 56 /r                       ] SSE,KATMAI */
        /* 360 355 001(56) 110 : SSE,KATMAI */
        {I_ORPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68372, 365},
    /* 2673 : RCPPS xmmreg,xmmrm128 [rm:       np 0f 53 /r                       ] SSE,KATMAI */
        /* 360 355 001(53) 110 : SSE,KATMAI */
        {I_RCPPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68378, 365},
    /* 2674 : RCPSS xmmreg,xmmrm32 [rm:       f3 0f 53 /r                       ] SSE,KATMAI */
        /* 333 355 001(53) 110 : SSE,KATMAI */
        {I_RCPSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68384, 365},
    /* 2675 : RSQRTPS xmmreg,xmmrm128 [rm:       np 0f 52 /r                       ] SSE,KATMAI */
        /* 360 355 001(52) 110 : SSE,KATMAI */
        {I_RSQRTPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68390, 365},
    /* 2676 : RSQRTSS xmmreg,xmmrm32 [rm:       f3 0f 52 /r                       ] SSE,KATMAI */
        /* 333 355 001(52) 110 : SSE,KATMAI */
        {I_RSQRTSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68396, 365},
    /* 2677 : SHUFPS xmmreg,xmmrm128,imm8 [rmi:      np 0f c6 /r ib,u                  ] SSE,KATMAI */
        /* 360 355 001(c6) 110 026 : SSE,KATMAI */
        {I_SHUFPS, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+57684, 365},
    /* 2678 : SQRTPS xmmreg,xmmrm128 [rm:       np 0f 51 /r                       ] SSE,KATMAI */
        /* 360 355 001(51) 110 : SSE,KATMAI */
        {I_SQRTPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68402, 365},
    /* 2679 : SQRTSS xmmreg,xmmrm32 [rm:       f3 0f 51 /r                       ] SSE,KATMAI */
        /* 333 355 001(51) 110 : SSE,KATMAI */
        {I_SQRTSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68408, 365},
    /* 2680 : STMXCSR mem32 [m:        np 0f ae /3                       ] SSE,KATMAI */
        /* 360 355 001(ae) 203 : SSE,KATMAI */
        {I_STMXCSR, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+68414, 365},
    /* 2681 : SUBPS xmmreg,xmmrm128 [rm:       np 0f 5c /r                       ] SSE,KATMAI */
        /* 360 355 001(5c) 110 : SSE,KATMAI */
        {I_SUBPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68420, 365},
    /* 2682 : SUBSS xmmreg,xmmrm32 [rm:       f3 0f 5c /r                       ] SSE,KATMAI */
        /* 333 355 001(5c) 110 : SSE,KATMAI */
        {I_SUBSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68426, 365},
    /* 2683 : UCOMISS xmmreg,xmmrm32 [rm:       np 0f 2e /r                       ] FL,SSE,KATMAI */
        /* 360 355 001(2e) 110 : FL,SSE,KATMAI */
        {I_UCOMISS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68432, 366},
    /* 2684 : UNPCKHPS xmmreg,xmmrm128 [rm:       np 0f 15 /r                       ] SSE,KATMAI */
        /* 360 355 001(15) 110 : SSE,KATMAI */
        {I_UNPCKHPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68438, 365},
    /* 2685 : UNPCKLPS xmmreg,xmmrm128 [rm:       np 0f 14 /r                       ] SSE,KATMAI */
        /* 360 355 001(14) 110 : SSE,KATMAI */
        {I_UNPCKLPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68444, 365},
    /* 2686 : XORPS xmmreg,xmmrm128 [rm:       np 0f 57 /r                       ] SSE,KATMAI */
        /* 360 355 001(57) 110 : SSE,KATMAI */
        {I_XORPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68450, 365},
    /* 2687 : FXRSTOR mem [m:        np 0f ae /1                       ] FPU,SSE,P6 */
        /* 360 355 001(ae) 201 : FPU,SSE,P6 */
        {I_FXRSTOR, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57692, 370},
    /* 2688 : FXRSTOR64 mem [m:        o64 np 0f ae /1                   ] LONG,PROT,FPU,SSE,X86_64 */
        /* 324 360 355 001(ae) 201 : LONG,PROT,FPU,SSE,X86_64 */
        {I_FXRSTOR64, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57691, 371},
    /* 2689 : FXSAVE mem [m:        np 0f ae /0                       ] FPU,SSE,P6 */
        /* 360 355 001(ae) 200 : FPU,SSE,P6 */
        {I_FXSAVE, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57699, 370},
    /* 2690 : FXSAVE64 mem [m:        o64 np 0f ae /0                   ] LONG,PROT,FPU,SSE,X86_64 */
        /* 324 360 355 001(ae) 200 : LONG,PROT,FPU,SSE,X86_64 */
        {I_FXSAVE64, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57698, 371},
    /* 2691 : XGETBV void [          0f 01 d0                          ] NEHALEM */
        /* 355 002(01 d0) : NEHALEM */
        {I_XGETBV, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72303, 372},
    /* 2692 : XSETBV void [          0f 01 d1                          ] PRIV,NEHALEM */
        /* 355 002(01 d1) : PRIV,NEHALEM */
        {I_XSETBV, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72308, 373},
    /* 2693 : XSAVE mem [m:        np 0f ae /4                       ] NOAPX,NEHALEM */
        /* 360 355 001(ae) 204 : NOAPX,NEHALEM */
        {I_XSAVE, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57706, 374},
    /* 2694 : XSAVE64 mem [m:        o64 np 0f ae /4                   ] NOAPX,LONG,PROT,X86_64,NEHALEM */
        /* 324 360 355 001(ae) 204 : NOAPX,LONG,PROT,NEHALEM */
        {I_XSAVE64, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57705, 375},
    /* 2695 : XSAVEC mem [m:        np 0f c7 /4                       ] NOAPX,FUTURE */
        /* 360 355 001(c7) 204 : NOAPX,FUTURE */
        {I_XSAVEC, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57713, 284},
    /* 2696 : XSAVEC64 mem [m:        o64 np 0f c7 /4                   ] NOAPX,LONG,PROT,X86_64 */
        /* 324 360 355 001(c7) 204 : NOAPX,LONG,PROT,X86_64 */
        {I_XSAVEC64, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57712, 246},
    /* 2697 : XSAVEOPT mem [m:        np 0f ae /6                       ] NOAPX,FUTURE */
        /* 360 355 001(ae) 206 : NOAPX,FUTURE */
        {I_XSAVEOPT, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57720, 284},
    /* 2698 : XSAVEOPT64 mem [m:        o64 np 0f ae /6                   ] NOAPX,LONG,PROT,X86_64 */
        /* 324 360 355 001(ae) 206 : NOAPX,LONG,PROT,X86_64 */
        {I_XSAVEOPT64, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57719, 246},
    /* 2699 : XSAVES mem [m:        np 0f c7 /5                       ] NOAPX,FUTURE */
        /* 360 355 001(c7) 205 : NOAPX,FUTURE */
        {I_XSAVES, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57727, 284},
    /* 2700 : XSAVES64 mem [m:        o64 np 0f c7 /5                   ] NOAPX,LONG,PROT,X86_64 */
        /* 324 360 355 001(c7) 205 : NOAPX,LONG,PROT,X86_64 */
        {I_XSAVES64, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57726, 246},
    /* 2701 : XRSTOR mem [m:        np 0f ae /5                       ] NOAPX,NEHALEM */
        /* 360 355 001(ae) 205 : NOAPX,NEHALEM */
        {I_XRSTOR, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57734, 374},
    /* 2702 : XRSTOR64 mem [m:        o64 np 0f ae /5                   ] NOAPX,LONG,PROT,X86_64,NEHALEM */
        /* 324 360 355 001(ae) 205 : NOAPX,LONG,PROT,NEHALEM */
        {I_XRSTOR64, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57733, 375},
    /* 2703 : XRSTORS mem [m:        np 0f c7 /3                       ] NOAPX,FUTURE */
        /* 360 355 001(c7) 203 : NOAPX,FUTURE */
        {I_XRSTORS, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57741, 284},
    /* 2704 : XRSTORS64 mem [m:        o64 np 0f c7 /3                   ] NOAPX,LONG,PROT,X86_64 */
        /* 324 360 355 001(c7) 203 : NOAPX,LONG,PROT,X86_64 */
        {I_XRSTORS64, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57740, 246},
    /* 2705 : PREFETCHNTA mem8 [m:        0f 18 /0                          ] AR0,SB,KATMAI */
        /* 355 001(18) 200 : AR0,SB,KATMAI */
        {I_PREFETCHNTA, 1, {MEMORY|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70047, 376},
    /* 2706 : PREFETCHT0 mem8 [m:        0f 18 /1                          ] AR0,SB,KATMAI */
        /* 355 001(18) 201 : AR0,SB,KATMAI */
        {I_PREFETCHT0, 1, {MEMORY|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70065, 376},
    /* 2707 : PREFETCHT1 mem8 [m:        0f 18 /2                          ] AR0,SB,KATMAI */
        /* 355 001(18) 202 : AR0,SB,KATMAI */
        {I_PREFETCHT1, 1, {MEMORY|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70083, 376},
    /* 2708 : PREFETCHT2 mem8 [m:        0f 18 /3                          ] AR0,SB,KATMAI */
        /* 355 001(18) 203 : AR0,SB,KATMAI */
        {I_PREFETCHT2, 1, {MEMORY|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70101, 376},
    /* 2709 : PREFETCHIT0 mem8 [m:        0f 18 /7                          ] AR0,SB,PREFETCHI,FUTURE */
        /* 355 001(18) 207 : AR0,SB,PREFETCHI,FUTURE */
        {I_PREFETCHIT0, 1, {MEMORY|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70173, 377},
    /* 2710 : PREFETCHIT1 mem8 [m:        0f 18 /6                          ] AR0,SB,PREFETCHI,FUTURE */
        /* 355 001(18) 206 : AR0,SB,PREFETCHI,FUTURE */
        {I_PREFETCHIT1, 1, {MEMORY|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70155, 377},
    /* 2711 : SFENCE void [          np 0f ae f8                       ] KATMAI */
        /* 360 355 002(ae f8) : KATMAI */
        {I_SFENCE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67088, 378},
    /* 2712 : MASKMOVQ mmxreg,mmxreg [rm:       np 0f f7 /r                       ] MMX,KATMAI */
        /* 360 355 001(f7) 110 : MMX,KATMAI */
        {I_MASKMOVQ, 2, {MMXREG,MMXREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68456, 379},
    /* 2713 : MOVNTQ mem,mmxreg [mr:       np 0f e7 /r                       ] AR0-1,SQ,MMX,KATMAI */
        /* 360 355 001(e7) 101 : AR0,AR1,SQ,MMX,KATMAI */
        {I_MOVNTQ, 2, {MEMORY,MMXREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68462, 380},
    /* 2714 : PAVGB mmxreg,mmxrm [rm:       np 0f e0 /r                       ] AR0-1,SQ,MMX,KATMAI */
        /* 360 355 001(e0) 110 : AR0,AR1,SQ,MMX,KATMAI */
        {I_PAVGB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+68468, 380},
    /* 2715 : PAVGW mmxreg,mmxrm [rm:       np 0f e3 /r                       ] AR0-1,SQ,MMX,KATMAI */
        /* 360 355 001(e3) 110 : AR0,AR1,SQ,MMX,KATMAI */
        {I_PAVGW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+68474, 380},
    /* 2716 : PEXTRW reg32,mmxreg,imm [rmi:      np 0f c5 /r ib,u                  ] AR2,SB,MMX,KATMAI */
        /* 360 355 001(c5) 110 026 : AR2,SB,MMX,KATMAI */
        {I_PEXTRW, 3, {REG_GPR|BITS32,MMXREG,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+57747, 381},
    /* 2717 : PINSRW mmxreg,mem,imm [rmi:      np 0f c4 /r ib,u                  ] AR2,SB,MMX,KATMAI */
        /* 360 355 001(c4) 110 026 : AR2,SB,MMX,KATMAI */
        {I_PINSRW, 3, {MMXREG,MEMORY,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+57754, 381},
    /* 2718 : PINSRW mmxreg,rm16,imm [rmi:      np 0f c4 /r ib,u                  ] AR2,SB,MMX,KATMAI */
        /* 360 355 001(c4) 110 026 : AR2,SB,MMX,KATMAI */
        {I_PINSRW, 3, {MMXREG,RM_GPR|BITS16,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+57754, 381},
    /* 2719 : PINSRW mmxreg,reg32,imm [rmi:      np 0f c4 /r ib,u                  ] AR2,SB,MMX,KATMAI */
        /* 360 355 001(c4) 110 026 : AR2,SB,MMX,KATMAI */
        {I_PINSRW, 3, {MMXREG,REG_GPR|BITS32,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+57754, 381},
    /* 2720 : PMAXSW mmxreg,mmxrm [rm:       np 0f ee /r                       ] AR0-1,SQ,MMX,KATMAI */
        /* 360 355 001(ee) 110 : AR0,AR1,SQ,MMX,KATMAI */
        {I_PMAXSW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+68480, 380},
    /* 2721 : PMAXUB mmxreg,mmxrm [rm:       np 0f de /r                       ] AR0-1,SQ,MMX,KATMAI */
        /* 360 355 001(de) 110 : AR0,AR1,SQ,MMX,KATMAI */
        {I_PMAXUB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+68486, 380},
    /* 2722 : PMINSW mmxreg,mmxrm [rm:       np 0f ea /r                       ] AR0-1,SQ,MMX,KATMAI */
        /* 360 355 001(ea) 110 : AR0,AR1,SQ,MMX,KATMAI */
        {I_PMINSW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+68492, 380},
    /* 2723 : PMINUB mmxreg,mmxrm [rm:       np 0f da /r                       ] AR0-1,SQ,MMX,KATMAI */
        /* 360 355 001(da) 110 : AR0,AR1,SQ,MMX,KATMAI */
        {I_PMINUB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+68498, 380},
    /* 2724 : PMOVMSKB reg32,mmxreg [rm:       np 0f d7 /r                       ] MMX,KATMAI */
        /* 360 355 001(d7) 110 : MMX,KATMAI */
        {I_PMOVMSKB, 2, {REG_GPR|BITS32,MMXREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68504, 379},
    /* 2725 : PMULHUW mmxreg,mmxrm [rm:       np 0f e4 /r                       ] AR0-1,SQ,MMX,KATMAI */
        /* 360 355 001(e4) 110 : AR0,AR1,SQ,MMX,KATMAI */
        {I_PMULHUW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+68510, 380},
    /* 2726 : PSADBW mmxreg,mmxrm [rm:       np 0f f6 /r                       ] AR0-1,SQ,MMX,KATMAI */
        /* 360 355 001(f6) 110 : AR0,AR1,SQ,MMX,KATMAI */
        {I_PSADBW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+68516, 380},
    /* 2727 : PSHUFW mmxreg,mmxrm,imm [rmi:      np 0f 70 /r ib                    ] AR2,SB,MMX,KATMAI */
        /* 360 355 001(70) 110 022 : AR2,SB,MMX,KATMAI */
        {I_PSHUFW, 3, {MMXREG,RM_MMX,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+57761, 381},
    /* 2728 : PF2IW mmxreg,mmxrm [rm:       0f 0f /r 1c                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(1c) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PF2IW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57768, 338},
    /* 2729 : PFNACC mmxreg,mmxrm [rm:       0f 0f /r 8a                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(8a) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFNACC, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57775, 338},
    /* 2730 : PFPNACC mmxreg,mmxrm [rm:       0f 0f /r 8e                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(8e) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFPNACC, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57782, 338},
    /* 2731 : PI2FW mmxreg,mmxrm [rm:       0f 0f /r 0c                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(0c) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PI2FW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57789, 338},
    /* 2732 : PSWAPD mmxreg,mmxrm [rm:       0f 0f /r bb                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(bb) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PSWAPD, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57796, 338},
    /* 2733 : MASKMOVDQU xmmreg,xmmreg [rm:       66 0f f7 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(f7) 110 : SSE2,WILLAMETTE */
        {I_MASKMOVDQU, 2, {XMMREG,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68522, 250},
    /* 2734 : MOVNTDQ mem,xmmreg [mr:       66 0f e7 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(e7) 101 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_MOVNTDQ, 2, {MEMORY,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68528, 382},
    /* 2735 : MOVNTI mem,reg32 [mr:       np 0f c3 /r                       ] AR0-1,SD,WILLAMETTE */
        /* 360 355 001(c3) 101 : AR0,AR1,SD,WILLAMETTE */
        {I_MOVNTI, 2, {MEMORY,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+57804, 383},
    /* 2736 : MOVNTI mem,reg64 [mr:       o64 np 0f c3 /r                   ] AR0-1,SQ,LONG,PROT,X86_64 */
        /* 324 360 355 001(c3) 101 : AR0,AR1,SQ,LONG,PROT,X86_64 */
        {I_MOVNTI, 2, {MEMORY,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+57803, 384},
    /* 2737 : MOVNTPD mem,xmmreg [mr:       66 0f 2b /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(2b) 101 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_MOVNTPD, 2, {MEMORY,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68534, 382},
    /* 2738 : LFENCE void [          np 0f ae e8                       ] SSE2,WILLAMETTE */
        /* 360 355 002(ae e8) : SSE2,WILLAMETTE */
        {I_LFENCE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67076, 250},
    /* 2739 : MFENCE void [          np 0f ae f0                       ] SSE2,WILLAMETTE */
        /* 360 355 002(ae f0) : SSE2,WILLAMETTE */
        {I_MFENCE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67082, 250},
    /* 2740 : MOVD mem,xmmreg [mr:       66 norexw 0f 7e /r                ] AR0-1,SD,SSE2,WILLAMETTE */
        /* 361 317 355 001(7e) 101 : AR0,AR1,SD,SSE2,WILLAMETTE */
        {I_MOVD, 2, {MEMORY,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+57810, 385},
    /* 2741 : MOVD xmmreg,mem [rm:       66 norexw 0f 6e /r                ] AR0-1,SD,SSE2,WILLAMETTE */
        /* 361 317 355 001(6e) 110 : AR0,AR1,SD,SSE2,WILLAMETTE */
        {I_MOVD, 2, {XMMREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+57817, 385},
    /* 2742 : MOVD xmmreg,rm32 [rm:       66 norexw 0f 6e /r                ] SSE2,WILLAMETTE */
        /* 361 317 355 001(6e) 110 : SSE2,WILLAMETTE */
        {I_MOVD, 2, {XMMREG,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+57817, 250},
    /* 2743 : MOVD rm32,xmmreg [mr:       66 norexw 0f 7e /r                ] SSE2,WILLAMETTE */
        /* 361 317 355 001(7e) 101 : SSE2,WILLAMETTE */
        {I_MOVD, 2, {RM_GPR|BITS32,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+57810, 250},
    /* 2744 : MOVDQA xmmreg,xmmrm128 [rm:       66 0f 6f /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(6f) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_MOVDQA, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68540, 382},
    /* 2745 : MOVDQA xmmrm128,xmmreg [mr:       66 0f 7f /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(7f) 101 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_MOVDQA, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68546, 382},
    /* 2746 : MOVDQU xmmreg,xmmrm128 [rm:       f3 0f 6f /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 333 355 001(6f) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_MOVDQU, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68552, 382},
    /* 2747 : MOVDQU xmmrm128,xmmreg [mr:       f3 0f 7f /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 333 355 001(7f) 101 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_MOVDQU, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68558, 382},
    /* 2748 : MOVDQ2Q mmxreg,xmmreg [rm:       f2 0f d6 /r                       ] SSE2,WILLAMETTE */
        /* 332 355 001(d6) 110 : SSE2,WILLAMETTE */
        {I_MOVDQ2Q, 2, {MMXREG,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68564, 250},
    /* 2749 : MOVQ xmmreg,xmmreg [rm:       f3 0f 7e /r                       ] SSE2,WILLAMETTE */
        /* 333 355 001(7e) 110 : SSE2,WILLAMETTE */
        {I_MOVQ, 2, {XMMREG,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68570, 250},
    /* 2750 : MOVQ xmmreg,xmmreg [mr:       66 0f d6 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(d6) 101 : SSE2,WILLAMETTE */
        {I_MOVQ, 2, {XMMREG,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68576, 250},
    /* 2751 : MOVQ mem,xmmreg [mr:       66 0f d6 /r                       ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 361 355 001(d6) 101 : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_MOVQ, 2, {MEMORY,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68576, 386},
    /* 2752 : MOVQ xmmreg,mem [rm:       f3 0f 7e /r                       ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 333 355 001(7e) 110 : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_MOVQ, 2, {XMMREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+68570, 386},
    /* 2753 : MOVQ xmmreg,rm64 [rm:       66 o64 0f 6e /r                   ] LONG,PROT,SSE2,X86_64 */
        /* 361 324 355 001(6e) 110 : LONG,PROT,SSE2,X86_64 */
        {I_MOVQ, 2, {XMMREG,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+57824, 387},
    /* 2754 : MOVQ rm64,xmmreg [mr:       66 o64 0f 7e /r                   ] LONG,PROT,SSE2,X86_64 */
        /* 361 324 355 001(7e) 101 : LONG,PROT,SSE2,X86_64 */
        {I_MOVQ, 2, {RM_GPR|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+57831, 387},
    /* 2755 : MOVQ2DQ xmmreg,mmxreg [rm:       f3 0f d6 /r                       ] SSE2,WILLAMETTE */
        /* 333 355 001(d6) 110 : SSE2,WILLAMETTE */
        {I_MOVQ2DQ, 2, {XMMREG,MMXREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68582, 250},
    /* 2756 : PACKSSWB xmmreg,xmmrm [rm:       66 0f 63 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(63) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PACKSSWB, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68588, 382},
    /* 2757 : PACKSSDW xmmreg,xmmrm [rm:       66 0f 6b /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(6b) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PACKSSDW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68594, 382},
    /* 2758 : PACKUSWB xmmreg,xmmrm [rm:       66 0f 67 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(67) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PACKUSWB, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68600, 382},
    /* 2759 : PADDB xmmreg,xmmrm [rm:       66 0f fc /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(fc) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PADDB, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68606, 382},
    /* 2760 : PADDW xmmreg,xmmrm [rm:       66 0f fd /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(fd) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PADDW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68612, 382},
    /* 2761 : PADDD xmmreg,xmmrm [rm:       66 0f fe /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(fe) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PADDD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68618, 382},
    /* 2762 : PADDQ mmxreg,mmxrm [rm:       np 0f d4 /r                       ] AR0-1,SQ,MMX,WILLAMETTE */
        /* 360 355 001(d4) 110 : AR0,AR1,SQ,MMX,WILLAMETTE */
        {I_PADDQ, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+68624, 388},
    /* 2763 : PADDQ xmmreg,xmmrm [rm:       66 0f d4 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(d4) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PADDQ, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68630, 382},
    /* 2764 : PADDSB xmmreg,xmmrm [rm:       66 0f ec /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(ec) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PADDSB, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68636, 382},
    /* 2765 : PADDSW xmmreg,xmmrm [rm:       66 0f ed /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(ed) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PADDSW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68642, 382},
    /* 2766 : PADDUSB xmmreg,xmmrm [rm:       66 0f dc /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(dc) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PADDUSB, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68648, 382},
    /* 2767 : PADDUSW xmmreg,xmmrm [rm:       66 0f dd /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(dd) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PADDUSW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68654, 382},
    /* 2768 : PAND xmmreg,xmmrm [rm:       66 0f db /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(db) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PAND, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68660, 382},
    /* 2769 : PANDN xmmreg,xmmrm [rm:       66 0f df /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(df) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PANDN, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68666, 382},
    /* 2770 : PAVGB xmmreg,xmmrm [rm:       66 0f e0 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(e0) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PAVGB, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68672, 382},
    /* 2771 : PAVGW xmmreg,xmmrm [rm:       66 0f e3 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(e3) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PAVGW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68678, 382},
    /* 2772 : PCMPEQB xmmreg,xmmrm [rm:       66 0f 74 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(74) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PCMPEQB, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68684, 382},
    /* 2773 : PCMPEQW xmmreg,xmmrm [rm:       66 0f 75 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(75) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PCMPEQW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68690, 382},
    /* 2774 : PCMPEQD xmmreg,xmmrm [rm:       66 0f 76 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(76) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PCMPEQD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68696, 382},
    /* 2775 : PCMPGTB xmmreg,xmmrm [rm:       66 0f 64 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(64) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PCMPGTB, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68702, 382},
    /* 2776 : PCMPGTW xmmreg,xmmrm [rm:       66 0f 65 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(65) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PCMPGTW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68708, 382},
    /* 2777 : PCMPGTD xmmreg,xmmrm [rm:       66 0f 66 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(66) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PCMPGTD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68714, 382},
    /* 2778 : PEXTRW reg32,xmmreg,imm [rmi:      66 0f c5 /r ib,u                  ] AR2,SB,SSE2,WILLAMETTE */
        /* 361 355 001(c5) 110 026 : AR2,SB,SSE2,WILLAMETTE */
        {I_PEXTRW, 3, {REG_GPR|BITS32,XMMREG,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+57838, 389},
    /* 2779 : PINSRW xmmreg,reg16,imm [rmi:      66 0f c4 /r ib,u                  ] AR2,SB,SSE2,WILLAMETTE */
        /* 361 355 001(c4) 110 026 : AR2,SB,SSE2,WILLAMETTE */
        {I_PINSRW, 3, {XMMREG,REG_GPR|BITS16,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+57845, 389},
    /* 2780 : PINSRW xmmreg,mem,imm [rmi:      66 0f c4 /r ib,u                  ] AR2,SB,SSE2,WILLAMETTE */
        /* 361 355 001(c4) 110 026 : AR2,SB,SSE2,WILLAMETTE */
        {I_PINSRW, 3, {XMMREG,MEMORY,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+57845, 389},
    /* 2781 : PINSRW xmmreg,mem16,imm [rmi:      66 0f c4 /r ib,u                  ] AR2,SB,SSE2,WILLAMETTE */
        /* 361 355 001(c4) 110 026 : AR2,SB,SSE2,WILLAMETTE */
        {I_PINSRW, 3, {XMMREG,MEMORY|BITS16,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+57845, 389},
    /* 2782 : PMADDWD xmmreg,xmmrm [rm:       66 0f f5 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(f5) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PMADDWD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68720, 382},
    /* 2783 : PMAXSW xmmreg,xmmrm [rm:       66 0f ee /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(ee) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PMAXSW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68726, 382},
    /* 2784 : PMAXUB xmmreg,xmmrm [rm:       66 0f de /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(de) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PMAXUB, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68732, 382},
    /* 2785 : PMINSW xmmreg,xmmrm [rm:       66 0f ea /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(ea) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PMINSW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68738, 382},
    /* 2786 : PMINUB xmmreg,xmmrm [rm:       66 0f da /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(da) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PMINUB, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68744, 382},
    /* 2787 : PMOVMSKB reg32,xmmreg [rm:       66 0f d7 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(d7) 110 : SSE2,WILLAMETTE */
        {I_PMOVMSKB, 2, {REG_GPR|BITS32,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68750, 250},
    /* 2788 : PMULHUW xmmreg,xmmrm [rm:       66 0f e4 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(e4) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PMULHUW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68756, 382},
    /* 2789 : PMULHW xmmreg,xmmrm [rm:       66 0f e5 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(e5) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PMULHW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68762, 382},
    /* 2790 : PMULLW xmmreg,xmmrm [rm:       66 0f d5 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(d5) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PMULLW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68768, 382},
    /* 2791 : PMULUDQ mmxreg,mmxrm [rm:       np 0f f4 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 360 355 001(f4) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PMULUDQ, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+68774, 382},
    /* 2792 : PMULUDQ xmmreg,xmmrm [rm:       66 0f f4 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(f4) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PMULUDQ, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68780, 382},
    /* 2793 : POR xmmreg,xmmrm [rm:       66 0f eb /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(eb) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_POR, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68786, 382},
    /* 2794 : PSADBW xmmreg,xmmrm [rm:       66 0f f6 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(f6) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSADBW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68792, 382},
    /* 2795 : PSHUFD xmmreg,xmmreg,imm [rmi:      66 0f 70 /r ib                    ] AR2,SB,SSE2,WILLAMETTE */
        /* 361 355 001(70) 110 022 : AR2,SB,SSE2,WILLAMETTE */
        {I_PSHUFD, 3, {XMMREG,XMMREG,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+57852, 389},
    /* 2796 : PSHUFD xmmreg,mem,imm [rmi:      66 0f 70 /r ib                    ] AR2,SB,SSE2,WILLAMETTE */
        /* 361 355 001(70) 110 022 : AR2,SB,SSE2,WILLAMETTE */
        {I_PSHUFD, 3, {XMMREG,MEMORY,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+57852, 389},
    /* 2797 : PSHUFHW xmmreg,xmmreg,imm [rmi:      f3 0f 70 /r ib                    ] AR2,SB,SSE2,WILLAMETTE */
        /* 333 355 001(70) 110 022 : AR2,SB,SSE2,WILLAMETTE */
        {I_PSHUFHW, 3, {XMMREG,XMMREG,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+57859, 389},
    /* 2798 : PSHUFHW xmmreg,mem,imm [rmi:      f3 0f 70 /r ib                    ] AR2,SB,SSE2,WILLAMETTE */
        /* 333 355 001(70) 110 022 : AR2,SB,SSE2,WILLAMETTE */
        {I_PSHUFHW, 3, {XMMREG,MEMORY,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+57859, 389},
    /* 2799 : PSHUFLW xmmreg,xmmreg,imm [rmi:      f2 0f 70 /r ib                    ] AR2,SB,SSE2,WILLAMETTE */
        /* 332 355 001(70) 110 022 : AR2,SB,SSE2,WILLAMETTE */
        {I_PSHUFLW, 3, {XMMREG,XMMREG,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+57866, 389},
    /* 2800 : PSHUFLW xmmreg,mem,imm [rmi:      f2 0f 70 /r ib                    ] AR2,SB,SSE2,WILLAMETTE */
        /* 332 355 001(70) 110 022 : AR2,SB,SSE2,WILLAMETTE */
        {I_PSHUFLW, 3, {XMMREG,MEMORY,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+57866, 389},
    /* 2801 : PSLLDQ xmmreg,imm [mi:       66 0f 73 /7 ib,u                  ] AR1,SB,SSE2,WILLAMETTE */
        /* 361 355 001(73) 207 025 : AR1,SB,SSE2,WILLAMETTE */
        {I_PSLLDQ, 2, {XMMREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57873, 391},
    /* 2802 : PSLLW xmmreg,xmmrm [rm:       66 0f f1 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(f1) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSLLW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68798, 382},
    /* 2803 : PSLLW xmmreg,imm [mi:       66 0f 71 /6 ib,u                  ] AR1,SB,SSE2,WILLAMETTE */
        /* 361 355 001(71) 206 025 : AR1,SB,SSE2,WILLAMETTE */
        {I_PSLLW, 2, {XMMREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57880, 391},
    /* 2804 : PSLLD xmmreg,xmmrm [rm:       66 0f f2 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(f2) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSLLD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68804, 382},
    /* 2805 : PSLLD xmmreg,imm [mi:       66 0f 72 /6 ib,u                  ] AR1,SB,SSE2,WILLAMETTE */
        /* 361 355 001(72) 206 025 : AR1,SB,SSE2,WILLAMETTE */
        {I_PSLLD, 2, {XMMREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57887, 391},
    /* 2806 : PSLLQ xmmreg,xmmrm [rm:       66 0f f3 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(f3) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSLLQ, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68810, 382},
    /* 2807 : PSLLQ xmmreg,imm [mi:       66 0f 73 /6 ib,u                  ] AR1,SB,SSE2,WILLAMETTE */
        /* 361 355 001(73) 206 025 : AR1,SB,SSE2,WILLAMETTE */
        {I_PSLLQ, 2, {XMMREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57894, 391},
    /* 2808 : PSRAW xmmreg,xmmrm [rm:       66 0f e1 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(e1) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSRAW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68816, 382},
    /* 2809 : PSRAW xmmreg,imm [mi:       66 0f 71 /4 ib,u                  ] AR1,SB,SSE2,WILLAMETTE */
        /* 361 355 001(71) 204 025 : AR1,SB,SSE2,WILLAMETTE */
        {I_PSRAW, 2, {XMMREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57901, 391},
    /* 2810 : PSRAD xmmreg,xmmrm [rm:       66 0f e2 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(e2) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSRAD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68822, 382},
    /* 2811 : PSRAD xmmreg,imm [mi:       66 0f 72 /4 ib,u                  ] AR1,SB,SSE2,WILLAMETTE */
        /* 361 355 001(72) 204 025 : AR1,SB,SSE2,WILLAMETTE */
        {I_PSRAD, 2, {XMMREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57908, 391},
    /* 2812 : PSRLDQ xmmreg,imm [mi:       66 0f 73 /3 ib,u                  ] AR1,SB,SSE2,WILLAMETTE */
        /* 361 355 001(73) 203 025 : AR1,SB,SSE2,WILLAMETTE */
        {I_PSRLDQ, 2, {XMMREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57915, 391},
    /* 2813 : PSRLW xmmreg,xmmrm [rm:       66 0f d1 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(d1) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSRLW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68828, 382},
    /* 2814 : PSRLW xmmreg,imm [mi:       66 0f 71 /2 ib,u                  ] AR1,SB,SSE2,WILLAMETTE */
        /* 361 355 001(71) 202 025 : AR1,SB,SSE2,WILLAMETTE */
        {I_PSRLW, 2, {XMMREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57922, 391},
    /* 2815 : PSRLD xmmreg,xmmrm [rm:       66 0f d2 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(d2) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSRLD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68834, 382},
    /* 2816 : PSRLD xmmreg,imm [mi:       66 0f 72 /2 ib,u                  ] AR1,SB,SSE2,WILLAMETTE */
        /* 361 355 001(72) 202 025 : AR1,SB,SSE2,WILLAMETTE */
        {I_PSRLD, 2, {XMMREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57929, 391},
    /* 2817 : PSRLQ xmmreg,xmmrm [rm:       66 0f d3 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(d3) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSRLQ, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68840, 382},
    /* 2818 : PSRLQ xmmreg,imm [mi:       66 0f 73 /2 ib,u                  ] AR1,SB,SSE2,WILLAMETTE */
        /* 361 355 001(73) 202 025 : AR1,SB,SSE2,WILLAMETTE */
        {I_PSRLQ, 2, {XMMREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57936, 391},
    /* 2819 : PSUBB xmmreg,xmmrm [rm:       66 0f f8 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(f8) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSUBB, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68846, 382},
    /* 2820 : PSUBW xmmreg,xmmrm [rm:       66 0f f9 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(f9) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSUBW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68852, 382},
    /* 2821 : PSUBD xmmreg,xmmrm [rm:       66 0f fa /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(fa) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSUBD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68858, 382},
    /* 2822 : PSUBQ mmxreg,mmxrm [rm:       np 0f fb /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 360 355 001(fb) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSUBQ, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+68864, 382},
    /* 2823 : PSUBQ xmmreg,xmmrm [rm:       66 0f fb /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(fb) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSUBQ, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68870, 382},
    /* 2824 : PSUBSB xmmreg,xmmrm [rm:       66 0f e8 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(e8) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSUBSB, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68876, 382},
    /* 2825 : PSUBSW xmmreg,xmmrm [rm:       66 0f e9 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(e9) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSUBSW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68882, 382},
    /* 2826 : PSUBUSB xmmreg,xmmrm [rm:       66 0f d8 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(d8) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSUBUSB, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68888, 382},
    /* 2827 : PSUBUSW xmmreg,xmmrm [rm:       66 0f d9 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(d9) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSUBUSW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68894, 382},
    /* 2828 : PUNPCKHBW xmmreg,xmmrm [rm:       66 0f 68 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(68) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PUNPCKHBW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68900, 382},
    /* 2829 : PUNPCKHWD xmmreg,xmmrm [rm:       66 0f 69 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(69) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PUNPCKHWD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68906, 382},
    /* 2830 : PUNPCKHDQ xmmreg,xmmrm [rm:       66 0f 6a /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(6a) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PUNPCKHDQ, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68912, 382},
    /* 2831 : PUNPCKHQDQ xmmreg,xmmrm [rm:       66 0f 6d /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(6d) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PUNPCKHQDQ, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68918, 382},
    /* 2832 : PUNPCKLBW xmmreg,xmmrm [rm:       66 0f 60 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(60) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PUNPCKLBW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68924, 382},
    /* 2833 : PUNPCKLWD xmmreg,xmmrm [rm:       66 0f 61 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(61) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PUNPCKLWD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68930, 382},
    /* 2834 : PUNPCKLDQ xmmreg,xmmrm [rm:       66 0f 62 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(62) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PUNPCKLDQ, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68936, 382},
    /* 2835 : PUNPCKLQDQ xmmreg,xmmrm [rm:       66 0f 6c /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(6c) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PUNPCKLQDQ, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68942, 382},
    /* 2836 : PXOR xmmreg,xmmrm [rm:       66 0f ef /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(ef) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PXOR, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68948, 382},
    /* 2837 : ADDPD xmmreg,xmmrm [rm:       66 0f 58 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(58) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_ADDPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68954, 382},
    /* 2838 : ADDSD xmmreg,xmmrm [rm:       f2 0f 58 /r                       ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(58) 110 : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_ADDSD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68960, 386},
    /* 2839 : ANDNPD xmmreg,xmmrm [rm:       66 0f 55 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(55) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_ANDNPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68966, 382},
    /* 2840 : ANDPD xmmreg,xmmrm [rm:       66 0f 54 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(54) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_ANDPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68972, 382},
    /* 2841 : CMPEQPD xmmreg,xmmrm [rm:       66 0f c2 /r 00                    ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(c2) 110 001(00) : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CMPEQPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51389, 382},
    /* 2842 : CMPEQSD xmmreg,xmmrm [rm:       f2 0f c2 /r 00                    ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(c2) 110 001(00) : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_CMPEQSD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51397, 386},
    /* 2843 : CMPLEPD xmmreg,xmmrm [rm:       66 0f c2 /r 02                    ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(c2) 110 001(02) : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CMPLEPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51405, 382},
    /* 2844 : CMPLESD xmmreg,xmmrm [rm:       f2 0f c2 /r 02                    ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(c2) 110 001(02) : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_CMPLESD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51413, 386},
    /* 2845 : CMPLTPD xmmreg,xmmrm [rm:       66 0f c2 /r 01                    ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(c2) 110 001(01) : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CMPLTPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51421, 382},
    /* 2846 : CMPLTSD xmmreg,xmmrm [rm:       f2 0f c2 /r 01                    ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(c2) 110 001(01) : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_CMPLTSD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51429, 386},
    /* 2847 : CMPNEQPD xmmreg,xmmrm [rm:       66 0f c2 /r 04                    ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(c2) 110 001(04) : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CMPNEQPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51437, 382},
    /* 2848 : CMPNEQSD xmmreg,xmmrm [rm:       f2 0f c2 /r 04                    ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(c2) 110 001(04) : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_CMPNEQSD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51445, 386},
    /* 2849 : CMPNLEPD xmmreg,xmmrm [rm:       66 0f c2 /r 06                    ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(c2) 110 001(06) : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CMPNLEPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51453, 382},
    /* 2850 : CMPNLESD xmmreg,xmmrm [rm:       f2 0f c2 /r 06                    ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(c2) 110 001(06) : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_CMPNLESD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51461, 386},
    /* 2851 : CMPNLTPD xmmreg,xmmrm [rm:       66 0f c2 /r 05                    ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(c2) 110 001(05) : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CMPNLTPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51469, 382},
    /* 2852 : CMPNLTSD xmmreg,xmmrm [rm:       f2 0f c2 /r 05                    ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(c2) 110 001(05) : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_CMPNLTSD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51477, 386},
    /* 2853 : CMPORDPD xmmreg,xmmrm [rm:       66 0f c2 /r 07                    ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(c2) 110 001(07) : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CMPORDPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51485, 382},
    /* 2854 : CMPORDSD xmmreg,xmmrm [rm:       f2 0f c2 /r 07                    ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(c2) 110 001(07) : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_CMPORDSD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51493, 386},
    /* 2855 : CMPUNORDPD xmmreg,xmmrm [rm:       66 0f c2 /r 03                    ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(c2) 110 001(03) : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CMPUNORDPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51501, 382},
    /* 2856 : CMPUNORDSD xmmreg,xmmrm [rm:       f2 0f c2 /r 03                    ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(c2) 110 001(03) : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_CMPUNORDSD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51509, 386},
    /* 2857 : CMPPD xmmreg,xmmrm128,imm8 [rmi:      66 0f c2 /r ib,u                  ] SSE2,WILLAMETTE */
        /* 361 355 001(c2) 110 026 : SSE2,WILLAMETTE */
        {I_CMPPD, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+57943, 250},
    /* 2858 : CMPSD xmmreg,xmmrm128,imm8 [rmi:      f2 0f c2 /r ib,u                  ] FL,SSE2,WILLAMETTE */
        /* 332 355 001(c2) 110 026 : FL,SSE2,WILLAMETTE */
        {I_CMPSD, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+57950, 392},
    /* 2859 : COMISD xmmreg,xmmrm64 [rm:       66 0f 2f /r                       ] FL,SSE2,WILLAMETTE */
        /* 361 355 001(2f) 110 : FL,SSE2,WILLAMETTE */
        {I_COMISD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68978, 392},
    /* 2860 : CVTDQ2PD xmmreg,xmmrm [rm:       f3 0f e6 /r                       ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 333 355 001(e6) 110 : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_CVTDQ2PD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68984, 386},
    /* 2861 : CVTDQ2PS xmmreg,xmmrm [rm:       np 0f 5b /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 360 355 001(5b) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CVTDQ2PS, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68990, 382},
    /* 2862 : CVTPD2DQ xmmreg,xmmrm [rm:       f2 0f e6 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 332 355 001(e6) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CVTPD2DQ, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68996, 382},
    /* 2863 : CVTPD2PI mmxreg,xmmrm [rm:       66 0f 2d /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(2d) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CVTPD2PI, 2, {MMXREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69002, 382},
    /* 2864 : CVTPD2PS xmmreg,xmmrm [rm:       66 0f 5a /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(5a) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CVTPD2PS, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69008, 382},
    /* 2865 : CVTPI2PD xmmreg,mmxrm [rm:       66 0f 2a /r                       ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 361 355 001(2a) 110 : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_CVTPI2PD, 2, {XMMREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+69014, 386},
    /* 2866 : CVTPS2DQ xmmreg,xmmrm [rm:       66 0f 5b /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(5b) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CVTPS2DQ, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69020, 382},
    /* 2867 : CVTPS2PD xmmreg,xmmrm [rm:       np 0f 5a /r                       ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 360 355 001(5a) 110 : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_CVTPS2PD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69026, 386},
    /* 2868 : CVTSD2SI reg32,xmmrm64 [rm:       norexw f2 0f 2d /r                ] SSE2,WILLAMETTE */
        /* 317 332 355 001(2d) 110 : SSE2,WILLAMETTE */
        {I_CVTSD2SI, 2, {REG_GPR|BITS32,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+57957, 250},
    /* 2869 : CVTSD2SI reg64,xmmrm64 [rm:       o64 f2 0f 2d /r                   ] LONG,PROT,SSE2,X86_64 */
        /* 324 332 355 001(2d) 110 : LONG,PROT,SSE2,X86_64 */
        {I_CVTSD2SI, 2, {REG_GPR|BITS64,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+57964, 387},
    /* 2870 : CVTSD2SS xmmreg,xmmrm64 [rm:       f2 0f 5a /r                       ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(5a) 110 : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_CVTSD2SS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+69032, 386},
    /* 2871 : CVTSI2SD xmmreg,rm32 [rm:       norexw f2 0f 2a /r                ] SSE2,WILLAMETTE */
        /* 317 332 355 001(2a) 110 : SSE2,WILLAMETTE */
        {I_CVTSI2SD, 2, {XMMREG,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+57971, 250},
    /* 2872 : CVTSI2SD xmmreg,rm64 [rm:       o64 f2 0f 2a /r                   ] AR1,SX,LONG,PROT,SSE2,X86_64 */
        /* 324 332 355 001(2a) 110 : AR1,SX,LONG,PROT,SSE2,X86_64 */
        {I_CVTSI2SD, 2, {XMMREG,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+57978, 393},
    /* 2873 : CVTSS2SD xmmreg,xmmrm [rm:       f3 0f 5a /r                       ] AR0-1,SD,SSE2,WILLAMETTE */
        /* 333 355 001(5a) 110 : AR0,AR1,SD,SSE2,WILLAMETTE */
        {I_CVTSS2SD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69038, 385},
    /* 2874 : CVTTPD2PI mmxreg,xmmrm [rm:       66 0f 2c /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(2c) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CVTTPD2PI, 2, {MMXREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69044, 382},
    /* 2875 : CVTTPD2DQ xmmreg,xmmrm [rm:       66 0f e6 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(e6) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CVTTPD2DQ, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69050, 382},
    /* 2876 : CVTTPS2DQ xmmreg,xmmrm [rm:       f3 0f 5b /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 333 355 001(5b) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CVTTPS2DQ, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69056, 382},
    /* 2877 : CVTTSD2SI reg32,xmmrm64 [rm:       norexw f2 0f 2c /r                ] SSE2,WILLAMETTE */
        /* 317 332 355 001(2c) 110 : SSE2,WILLAMETTE */
        {I_CVTTSD2SI, 2, {REG_GPR|BITS32,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+57985, 250},
    /* 2878 : CVTTSD2SI reg64,xmmrm64 [rm:       o64 f2 0f 2c /r                   ] LONG,PROT,SSE2,X86_64 */
        /* 324 332 355 001(2c) 110 : LONG,PROT,SSE2,X86_64 */
        {I_CVTTSD2SI, 2, {REG_GPR|BITS64,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+57992, 387},
    /* 2879 : DIVPD xmmreg,xmmrm [rm:       66 0f 5e /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(5e) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_DIVPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69062, 382},
    /* 2880 : DIVSD xmmreg,xmmrm [rm:       f2 0f 5e /r                       ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(5e) 110 : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_DIVSD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69068, 386},
    /* 2881 : MAXPD xmmreg,xmmrm [rm:       66 0f 5f /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(5f) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_MAXPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69074, 382},
    /* 2882 : MAXSD xmmreg,xmmrm [rm:       f2 0f 5f /r                       ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(5f) 110 : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_MAXSD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69080, 386},
    /* 2883 : MINPD xmmreg,xmmrm [rm:       66 0f 5d /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(5d) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_MINPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69086, 382},
    /* 2884 : MINSD xmmreg,xmmrm [rm:       f2 0f 5d /r                       ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(5d) 110 : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_MINSD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69092, 386},
    /* 2885 : MOVAPD xmmreg,xmmrm128 [rm:       66 0f 28 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(28) 110 : SSE2,WILLAMETTE */
        {I_MOVAPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69098, 250},
    /* 2886 : MOVAPD xmmrm128,xmmreg [mr:       66 0f 29 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(29) 101 : SSE2,WILLAMETTE */
        {I_MOVAPD, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+69104, 250},
    /* 2887 : MOVHPD mem64,xmmreg [mr:       66 0f 17 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(17) 101 : SSE2,WILLAMETTE */
        {I_MOVHPD, 2, {MEMORY|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+69110, 250},
    /* 2888 : MOVHPD xmmreg,mem64 [rm:       66 0f 16 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(16) 110 : SSE2,WILLAMETTE */
        {I_MOVHPD, 2, {XMMREG,MEMORY|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+69116, 250},
    /* 2889 : MOVLPD mem64,xmmreg [mr:       66 0f 13 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(13) 101 : SSE2,WILLAMETTE */
        {I_MOVLPD, 2, {MEMORY|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+69122, 250},
    /* 2890 : MOVLPD xmmreg,mem64 [rm:       66 0f 12 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(12) 110 : SSE2,WILLAMETTE */
        {I_MOVLPD, 2, {XMMREG,MEMORY|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+69128, 250},
    /* 2891 : MOVMSKPD reg32,xmmreg [rm:       66 0f 50 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(50) 110 : SSE2,WILLAMETTE */
        {I_MOVMSKPD, 2, {REG_GPR|BITS32,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+69134, 250},
    /* 2892 : MOVMSKPD reg64,xmmreg [rm:       66 o64 0f 50 /r                   ] LONG,PROT,SSE2,X86_64 */
        /* 361 324 355 001(50) 110 : LONG,PROT,SSE2,X86_64 */
        {I_MOVMSKPD, 2, {REG_GPR|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+57999, 387},
    /* 2893 : MOVSD xmmreg,xmmrm64 [rm:       f2 0f 10 /r                       ] SSE2,WILLAMETTE */
        /* 332 355 001(10) 110 : SSE2,WILLAMETTE */
        {I_MOVSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+69140, 250},
    /* 2894 : MOVSD xmmrm64,xmmreg [mr:       f2 0f 11 /r                       ] SSE2,WILLAMETTE */
        /* 332 355 001(11) 101 : SSE2,WILLAMETTE */
        {I_MOVSD, 2, {RM_XMM|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+69146, 250},
    /* 2895 : MOVUPD xmmreg,xmmrm128 [rm:       66 0f 10 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(10) 110 : SSE2,WILLAMETTE */
        {I_MOVUPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69152, 250},
    /* 2896 : MOVUPD xmmrm128,xmmreg [mr:       66 0f 11 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(11) 101 : SSE2,WILLAMETTE */
        {I_MOVUPD, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+69158, 250},
    /* 2897 : MULPD xmmreg,xmmrm128 [rm:       66 0f 59 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(59) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_MULPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69164, 382},
    /* 2898 : MULSD xmmreg,xmmrm64 [rm:       f2 0f 59 /r                       ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(59) 110 : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_MULSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+69170, 386},
    /* 2899 : ORPD xmmreg,xmmrm128 [rm:       66 0f 56 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(56) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_ORPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69176, 382},
    /* 2900 : SHUFPD xmmreg,xmmrm128,imm8 [rmi:      66 0f c6 /r ib,u                  ] SSE2,WILLAMETTE */
        /* 361 355 001(c6) 110 026 : SSE2,WILLAMETTE */
        {I_SHUFPD, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58006, 250},
    /* 2901 : SQRTPD xmmreg,xmmrm128 [rm:       66 0f 51 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(51) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_SQRTPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69182, 382},
    /* 2902 : SQRTSD xmmreg,xmmrm64 [rm:       f2 0f 51 /r                       ] SSE2,WILLAMETTE */
        /* 332 355 001(51) 110 : SSE2,WILLAMETTE */
        {I_SQRTSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+69188, 250},
    /* 2903 : SUBPD xmmreg,xmmrm128 [rm:       66 0f 5c /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(5c) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_SUBPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69194, 382},
    /* 2904 : SUBSD xmmreg,xmmrm64 [rm:       f2 0f 5c /r                       ] SSE2,WILLAMETTE */
        /* 332 355 001(5c) 110 : SSE2,WILLAMETTE */
        {I_SUBSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+69200, 250},
    /* 2905 : UCOMISD xmmreg,xmmrm64 [rm:       66 0f 2e /r                       ] FL,SSE2,WILLAMETTE */
        /* 361 355 001(2e) 110 : FL,SSE2,WILLAMETTE */
        {I_UCOMISD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+69206, 392},
    /* 2906 : UNPCKHPD xmmreg,xmmrm128 [rm:       66 0f 15 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(15) 110 : SSE2,WILLAMETTE */
        {I_UNPCKHPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69212, 250},
    /* 2907 : UNPCKLPD xmmreg,xmmrm128 [rm:       66 0f 14 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(14) 110 : SSE2,WILLAMETTE */
        {I_UNPCKLPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69218, 250},
    /* 2908 : XORPD xmmreg,xmmrm128 [rm:       66 0f 57 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(57) 110 : SSE2,WILLAMETTE */
        {I_XORPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69224, 250},
    /* 2909 : ADDSUBPD xmmreg,xmmrm128 [rm:       66 0f d0 /r                       ] AR0-1,SO,SSE3,PRESCOTT */
        /* 361 355 001(d0) 110 : AR0,AR1,SO,SSE3,PRESCOTT */
        {I_ADDSUBPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69230, 394},
    /* 2910 : ADDSUBPS xmmreg,xmmrm128 [rm:       f2 0f d0 /r                       ] AR0-1,SO,SSE3,PRESCOTT */
        /* 332 355 001(d0) 110 : AR0,AR1,SO,SSE3,PRESCOTT */
        {I_ADDSUBPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69236, 394},
    /* 2911 : HADDPD xmmreg,xmmrm128 [rm:       66 0f 7c /r                       ] AR0-1,SO,SSE3,PRESCOTT */
        /* 361 355 001(7c) 110 : AR0,AR1,SO,SSE3,PRESCOTT */
        {I_HADDPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69242, 394},
    /* 2912 : HADDPS xmmreg,xmmrm128 [rm:       f2 0f 7c /r                       ] AR0-1,SO,SSE3,PRESCOTT */
        /* 332 355 001(7c) 110 : AR0,AR1,SO,SSE3,PRESCOTT */
        {I_HADDPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69248, 394},
    /* 2913 : HSUBPD xmmreg,xmmrm128 [rm:       66 0f 7d /r                       ] AR0-1,SO,SSE3,PRESCOTT */
        /* 361 355 001(7d) 110 : AR0,AR1,SO,SSE3,PRESCOTT */
        {I_HSUBPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69254, 394},
    /* 2914 : HSUBPS xmmreg,xmmrm128 [rm:       f2 0f 7d /r                       ] AR0-1,SO,SSE3,PRESCOTT */
        /* 332 355 001(7d) 110 : AR0,AR1,SO,SSE3,PRESCOTT */
        {I_HSUBPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69260, 394},
    /* 2915 : LDDQU xmmreg,mem128 [rm:       f2 0f f0 /r                       ] AR0-1,SO,SSE3,PRESCOTT */
        /* 332 355 001(f0) 110 : AR0,AR1,SO,SSE3,PRESCOTT */
        {I_LDDQU, 2, {XMMREG,MEMORY|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69266, 394},
    /* 2916 : MOVDDUP xmmreg,xmmrm64 [rm:       f2 0f 12 /r                       ] AR0-1,SQ,SSE3,PRESCOTT */
        /* 332 355 001(12) 110 : AR0,AR1,SQ,SSE3,PRESCOTT */
        {I_MOVDDUP, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+69272, 395},
    /* 2917 : MOVSHDUP xmmreg,xmmrm128 [rm:       f3 0f 16 /r                       ] SSE3,PRESCOTT */
        /* 333 355 001(16) 110 : SSE3,PRESCOTT */
        {I_MOVSHDUP, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69278, 396},
    /* 2918 : MOVSLDUP xmmreg,xmmrm128 [rm:       f3 0f 12 /r                       ] SSE3,PRESCOTT */
        /* 333 355 001(12) 110 : SSE3,PRESCOTT */
        {I_MOVSLDUP, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69284, 396},
    /* 2919 : CLGI void [          0f 01 dd                          ] VMX,FUTURE,AMD */
        /* 355 002(01 dd) : VMX,FUTURE,AMD */
        {I_CLGI, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72313, 397},
    /* 2920 : STGI void [          0f 01 dc                          ] VMX,FUTURE,AMD */
        /* 355 002(01 dc) : VMX,FUTURE,AMD */
        {I_STGI, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72318, 397},
    /* 2921 : VMCALL void [          0f 01 c1                          ] VMX,FUTURE */
        /* 355 002(01 c1) : VMX,FUTURE */
        {I_VMCALL, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69327, 398},
    /* 2922 : VMCLEAR mem [m:        66 0f c7 /6                       ] VMX,FUTURE */
        /* 361 355 001(c7) 206 : VMX,FUTURE */
        {I_VMCLEAR, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69290, 398},
    /* 2923 : VMFUNC void [          0f 01 d4                          ] VMX,FUTURE */
        /* 355 002(01 d4) : VMX,FUTURE */
        {I_VMFUNC, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72323, 398},
    /* 2924 : VMLAUNCH void [          0f 01 c2                          ] VMX,FUTURE */
        /* 355 002(01 c2) : VMX,FUTURE */
        {I_VMLAUNCH, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72328, 398},
    /* 2925 : VMLOAD void [          0f 01 da                          ] VMX,FUTURE,AMD */
        /* 355 002(01 da) : VMX,FUTURE,AMD */
        {I_VMLOAD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72333, 397},
    /* 2926 : VMMCALL void [          0f 01 d9                          ] VMX,FUTURE,AMD */
        /* 355 002(01 d9) : VMX,FUTURE,AMD */
        {I_VMMCALL, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72338, 397},
    /* 2927 : VMPTRLD mem [m:        np 0f c7 /6                       ] VMX,FUTURE */
        /* 360 355 001(c7) 206 : VMX,FUTURE */
        {I_VMPTRLD, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69296, 398},
    /* 2928 : VMPTRST mem [m:        np 0f c7 /7                       ] VMX,FUTURE */
        /* 360 355 001(c7) 207 : VMX,FUTURE */
        {I_VMPTRST, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69302, 398},
    /* 2929 : VMREAD rm32,reg32 [mr:       np 0f 78 /r                       ] AR0-1,SD,NOREX,NOAPX,NOLONG,VMX,FUTURE */
        /* 360 355 001(78) 101 : AR0,AR1,SD,NOREX,NOAPX,NOLONG,VMX,FUTURE */
        {I_VMREAD, 2, {RM_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58014, 399},
    /* 2930 : VMREAD rm64,reg64 [mr:       o64nw np 0f 78 /r                 ] AR0-1,SQ,LONG,PROT,VMX,X86_64 */
        /* 323 360 355 001(78) 101 : AR0,AR1,SQ,NWSIZE,LONG,PROT,VMX,X86_64 */
        {I_VMREAD, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+58013, 400},
    /* 2931 : VMRESUME void [          0f 01 c3                          ] VMX,FUTURE */
        /* 355 002(01 c3) : VMX,FUTURE */
        {I_VMRESUME, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72343, 398},
    /* 2932 : VMRUN void [          0f 01 d8                          ] VMX,FUTURE,AMD */
        /* 355 002(01 d8) : VMX,FUTURE,AMD */
        {I_VMRUN, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72348, 397},
    /* 2933 : VMSAVE void [          0f 01 db                          ] VMX,FUTURE,AMD */
        /* 355 002(01 db) : VMX,FUTURE,AMD */
        {I_VMSAVE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72353, 397},
    /* 2934 : VMWRITE reg32,rm32 [rm:       np 0f 79 /r                       ] AR0-1,SD,NOREX,NOAPX,NOLONG,VMX,FUTURE */
        /* 360 355 001(79) 110 : AR0,AR1,SD,NOREX,NOAPX,NOLONG,VMX,FUTURE */
        {I_VMWRITE, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58021, 399},
    /* 2935 : VMWRITE reg64,rm64 [rm:       o64nw np 0f 79 /r                 ] AR0-1,SQ,LONG,PROT,VMX,X86_64 */
        /* 323 360 355 001(79) 110 : AR0,AR1,SQ,NWSIZE,LONG,PROT,VMX,X86_64 */
        {I_VMWRITE, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+58020, 400},
    /* 2936 : VMXOFF void [          0f 01 c4                          ] VMX,FUTURE */
        /* 355 002(01 c4) : VMX,FUTURE */
        {I_VMXOFF, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72358, 398},
    /* 2937 : VMXON mem [m:        f3 0f c7 /6                       ] VMX,FUTURE */
        /* 333 355 001(c7) 206 : VMX,FUTURE */
        {I_VMXON, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66211, 398},
    /* 2938 : INVEPT reg32,mem [rm:       66 0f38 80 /r                     ] AR0-1,SO,NOREX,NOAPX,NOLONG,VMX,FUTURE */
        /* 361 356 001(80) 110 : AR0,AR1,SO,NOREX,NOAPX,NOLONG,VMX,FUTURE */
        {I_INVEPT, 2, {REG_GPR|BITS32|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58028, 401},
    /* 2939 : INVEPT reg64,mem [rm:       o64nw 66 0f38 80 /r               ] AR0-1,SO,LONG,PROT,VMX,X86_64 */
        /* 323 361 356 001(80) 110 : AR0,AR1,SO,NWSIZE,NOAPX,LONG,PROT,VMX,X86_64 */
        {I_INVEPT, 2, {REG_GPR|BITS64|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58027, 402},
    /* 2940 : INVEPT reg64,mem128 [rm:       evex.nf0.nd0.l0.f3.m4.w1 f0 /r    ] AR0-1,SO,LONG,PROT,VMX,APX,X86_64 */
        /* 250(f4 fe 08) 300 001(f0) 110 : AR0,AR1,SO,LONG,EVEX,PROT,VMX,APX,X86_64 */
        {I_INVEPT, 2, {REG_GPR|BITS64,MEMORY|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+25767, 403},
    /* 2941 : INVVPID reg32,mem [rm:       66 0f38 81 /r                     ] AR0-1,SO,NOREX,NOAPX,NOLONG,VMX,FUTURE */
        /* 361 356 001(81) 110 : AR0,AR1,SO,NOREX,NOAPX,NOLONG,VMX,FUTURE */
        {I_INVVPID, 2, {REG_GPR|BITS32|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58035, 401},
    /* 2942 : INVVPID reg64,mem [rm:       o64nw 66 0f38 81 /r               ] AR0-1,SO,LONG,PROT,VMX,X86_64 */
        /* 323 361 356 001(81) 110 : AR0,AR1,SO,NWSIZE,NOAPX,LONG,PROT,VMX,X86_64 */
        {I_INVVPID, 2, {REG_GPR|BITS64|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58034, 402},
    /* 2943 : INVVPID reg64,mem128 [rm:       evex.nf0.nd0.l0.f3.m4.w1 f1 /r    ] AR0-1,SO,LONG,PROT,VMX,APX,X86_64 */
        /* 250(f4 fe 08) 300 001(f1) 110 : AR0,AR1,SO,LONG,EVEX,PROT,VMX,APX,X86_64 */
        {I_INVVPID, 2, {REG_GPR|BITS64,MEMORY|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+25776, 403},
    /* 2944 : PVALIDATE void [          f2 0f 01 ff                       ] VMX,FUTURE,AMD */
        /* 332 355 002(01 ff) : VMX,FUTURE,AMD */
        {I_PVALIDATE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69308, 397},
    /* 2945 : RMPADJUST void [          f3 0f 01 fe                       ] VMX,FUTURE,AMD */
        /* 333 355 002(01 fe) : VMX,FUTURE,AMD */
        {I_RMPADJUST, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69314, 397},
    /* 2946 : VMGEXIT void [          f2 0f 01 c1                       ] VMX,FUTURE,AMD */
        /* 332 355 002(01 c1) : VMX,FUTURE,AMD */
        {I_VMGEXIT, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69320, 397},
    /* 2947 : VMGEXIT void [          f3 0f 01 c1                       ] VMX,FUTURE,AMD */
        /* 333 355 002(01 c1) : VMX,FUTURE,AMD */
        {I_VMGEXIT, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69326, 397},
    /* 2948 : PABSB mmxreg,mmxrm [rm:       np 0f38 1c /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(1c) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PABSB, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69332, 404},
    /* 2949 : PABSB xmmreg,xmmrm128 [rm:       66 0f38 1c /r                     ] SSSE3,FUTURE */
        /* 361 356 001(1c) 110 : NOAPX,SSSE3,FUTURE */
        {I_PABSB, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69338, 405},
    /* 2950 : PABSW mmxreg,mmxrm [rm:       np 0f38 1d /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(1d) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PABSW, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69344, 404},
    /* 2951 : PABSW xmmreg,xmmrm128 [rm:       66 0f38 1d /r                     ] SSSE3,FUTURE */
        /* 361 356 001(1d) 110 : NOAPX,SSSE3,FUTURE */
        {I_PABSW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69350, 405},
    /* 2952 : PABSD mmxreg,mmxrm [rm:       np 0f38 1e /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(1e) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PABSD, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69356, 404},
    /* 2953 : PABSD xmmreg,xmmrm128 [rm:       66 0f38 1e /r                     ] SSSE3,FUTURE */
        /* 361 356 001(1e) 110 : NOAPX,SSSE3,FUTURE */
        {I_PABSD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69362, 405},
    /* 2954 : PALIGNR mmxreg,mmxrm,imm [rmi:      np 0f3a 0f /r ib,u                ] AR0-2,SQ,MMX,SSSE3,FUTURE */
        /* 360 357 001(0f) 110 026 : AR0,AR1,AR2,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PALIGNR, 3, {MMXREG|RN_L16,RM_MMX|RN_L16,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+58041, 406},
    /* 2955 : PALIGNR xmmreg,xmmrm,imm [rmi:      66 0f3a 0f /r ib,u                ] SSSE3,FUTURE */
        /* 361 357 001(0f) 110 026 : NOAPX,SSSE3,FUTURE */
        {I_PALIGNR, 3, {XMMREG|RN_L16,RM_XMM|RN_L16,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+58048, 405},
    /* 2956 : PHADDW mmxreg,mmxrm [rm:       np 0f38 01 /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(01) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PHADDW, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69368, 404},
    /* 2957 : PHADDW xmmreg,xmmrm128 [rm:       66 0f38 01 /r                     ] SSSE3,FUTURE */
        /* 361 356 001(01) 110 : NOAPX,SSSE3,FUTURE */
        {I_PHADDW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69374, 405},
    /* 2958 : PHADDD mmxreg,mmxrm [rm:       np 0f38 02 /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(02) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PHADDD, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69380, 404},
    /* 2959 : PHADDD xmmreg,xmmrm128 [rm:       66 0f38 02 /r                     ] SSSE3,FUTURE */
        /* 361 356 001(02) 110 : NOAPX,SSSE3,FUTURE */
        {I_PHADDD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69386, 405},
    /* 2960 : PHADDSW mmxreg,mmxrm [rm:       np 0f38 03 /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(03) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PHADDSW, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69392, 404},
    /* 2961 : PHADDSW xmmreg,xmmrm128 [rm:       66 0f38 03 /r                     ] SSSE3,FUTURE */
        /* 361 356 001(03) 110 : NOAPX,SSSE3,FUTURE */
        {I_PHADDSW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69398, 405},
    /* 2962 : PHSUBW mmxreg,mmxrm [rm:       np 0f38 05 /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(05) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PHSUBW, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69404, 404},
    /* 2963 : PHSUBW xmmreg,xmmrm128 [rm:       66 0f38 05 /r                     ] SSSE3,FUTURE */
        /* 361 356 001(05) 110 : NOAPX,SSSE3,FUTURE */
        {I_PHSUBW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69410, 405},
    /* 2964 : PHSUBD mmxreg,mmxrm [rm:       np 0f38 06 /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(06) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PHSUBD, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69416, 404},
    /* 2965 : PHSUBD xmmreg,xmmrm128 [rm:       66 0f38 06 /r                     ] SSSE3,FUTURE */
        /* 361 356 001(06) 110 : NOAPX,SSSE3,FUTURE */
        {I_PHSUBD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69422, 405},
    /* 2966 : PHSUBSW mmxreg,mmxrm [rm:       np 0f38 07 /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(07) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PHSUBSW, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69428, 404},
    /* 2967 : PHSUBSW xmmreg,xmmrm128 [rm:       66 0f38 07 /r                     ] SSSE3,FUTURE */
        /* 361 356 001(07) 110 : NOAPX,SSSE3,FUTURE */
        {I_PHSUBSW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69434, 405},
    /* 2968 : PMADDUBSW mmxreg,mmxrm [rm:       np 0f38 04 /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(04) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PMADDUBSW, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69440, 404},
    /* 2969 : PMADDUBSW xmmreg,xmmrm128 [rm:       66 0f38 04 /r                     ] SSSE3,FUTURE */
        /* 361 356 001(04) 110 : NOAPX,SSSE3,FUTURE */
        {I_PMADDUBSW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69446, 405},
    /* 2970 : PMULHRSW mmxreg,mmxrm [rm:       np 0f38 0b /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(0b) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PMULHRSW, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69452, 404},
    /* 2971 : PMULHRSW xmmreg,xmmrm128 [rm:       66 0f38 0b /r                     ] SSSE3,FUTURE */
        /* 361 356 001(0b) 110 : NOAPX,SSSE3,FUTURE */
        {I_PMULHRSW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69458, 405},
    /* 2972 : PSHUFB mmxreg,mmxrm [rm:       np 0f38 00 /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(00) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PSHUFB, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69464, 404},
    /* 2973 : PSHUFB xmmreg,xmmrm128 [rm:       66 0f38 00 /r                     ] SSSE3,FUTURE */
        /* 361 356 001(00) 110 : NOAPX,SSSE3,FUTURE */
        {I_PSHUFB, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69470, 405},
    /* 2974 : PSIGNB mmxreg,mmxrm [rm:       np 0f38 08 /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(08) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PSIGNB, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69476, 404},
    /* 2975 : PSIGNB xmmreg,xmmrm128 [rm:       66 0f38 08 /r                     ] SSSE3,FUTURE */
        /* 361 356 001(08) 110 : NOAPX,SSSE3,FUTURE */
        {I_PSIGNB, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69482, 405},
    /* 2976 : PSIGNW mmxreg,mmxrm [rm:       np 0f38 09 /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(09) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PSIGNW, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69488, 404},
    /* 2977 : PSIGNW xmmreg,xmmrm128 [rm:       66 0f38 09 /r                     ] SSSE3,FUTURE */
        /* 361 356 001(09) 110 : NOAPX,SSSE3,FUTURE */
        {I_PSIGNW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69494, 405},
    /* 2978 : PSIGND mmxreg,mmxrm [rm:       np 0f38 0a /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(0a) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PSIGND, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69500, 404},
    /* 2979 : PSIGND xmmreg,xmmrm128 [rm:       66 0f38 0a /r                     ] SSSE3,FUTURE */
        /* 361 356 001(0a) 110 : NOAPX,SSSE3,FUTURE */
        {I_PSIGND, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69506, 405},
    /* 2980 : EXTRQ xmmreg,imm,imm [mij:      66 0f 78 /0 ib,u ib,u             ] SSE4A,FUTURE,AMD */
        /* 361 355 001(78) 200 025 026 : SSE4A,FUTURE,AMD */
        {I_EXTRQ, 3, {XMMREG,IMM_NORMAL,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+51517, 407},
    /* 2981 : EXTRQ xmmreg,xmmreg [rm:       66 0f 79 /r                       ] SSE4A,FUTURE,AMD */
        /* 361 355 001(79) 110 : SSE4A,FUTURE,AMD */
        {I_EXTRQ, 2, {XMMREG,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+69512, 407},
    /* 2982 : INSERTQ xmmreg,xmmreg,imm,imm [rmij:     f2 0f 78 /r ib,u ib,u             ] SSE4A,FUTURE,AMD */
        /* 332 355 001(78) 110 026 027 : SSE4A,FUTURE,AMD */
        {I_INSERTQ, 4, {XMMREG,XMMREG,IMM_NORMAL,IMM_NORMAL,0}, NO_DECORATOR, nasm_bytecodes+51525, 407},
    /* 2983 : INSERTQ xmmreg,xmmreg [rm:       f2 0f 79 /r                       ] SSE4A,FUTURE,AMD */
        /* 332 355 001(79) 110 : SSE4A,FUTURE,AMD */
        {I_INSERTQ, 2, {XMMREG,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+69518, 407},
    /* 2984 : MOVNTSD mem64,xmmreg [mr:       f2 0f 2b /r                       ] AR0-1,SQ,SSE4A,FUTURE,AMD */
        /* 332 355 001(2b) 101 : AR0,AR1,SQ,SSE4A,FUTURE,AMD */
        {I_MOVNTSD, 2, {MEMORY|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+69524, 408},
    /* 2985 : MOVNTSS mem32,xmmreg [mr:       f3 0f 2b /r                       ] AR0-1,SD,SSE4A,FUTURE,AMD */
        /* 333 355 001(2b) 101 : AR0,AR1,SD,SSE4A,FUTURE,AMD */
        {I_MOVNTSS, 2, {MEMORY|BITS32,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+69530, 409},
    /* 2986 : BLENDPD xmmreg,xmmrm128,imm8 [rmi:      66 0f3a 0d /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(0d) 110 026 : NOAPX,SSE41,FUTURE */
        {I_BLENDPD, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58055, 410},
    /* 2987 : BLENDPS xmmreg,xmmrm128,imm8 [rmi:      66 0f3a 0c /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(0c) 110 026 : NOAPX,SSE41,FUTURE */
        {I_BLENDPS, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58062, 410},
    /* 2988 : BLENDVPD xmmreg,xmmrm128,xmm0 [rm-:      66 0f38 15 /r                     ] SSE41,FUTURE */
        /* 361 356 001(15) 110 : NOAPX,SSE41,FUTURE */
        {I_BLENDVPD, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_ZERO|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+69536, 410},
    /* 2989 : BLENDVPD xmmreg,xmmrm128 [rm:       66 0f38 15 /r                     ] SSE41,FUTURE */
        /* 361 356 001(15) 110 : NOAPX,SSE41,FUTURE */
        {I_BLENDVPD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69536, 410},
    /* 2990 : BLENDVPS xmmreg,xmmrm128,xmm0 [rm-:      66 0f38 14 /r                     ] SSE41,FUTURE */
        /* 361 356 001(14) 110 : NOAPX,SSE41,FUTURE */
        {I_BLENDVPS, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_ZERO|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+69542, 410},
    /* 2991 : BLENDVPS xmmreg,xmmrm128 [rm:       66 0f38 14 /r                     ] SSE41,FUTURE */
        /* 361 356 001(14) 110 : NOAPX,SSE41,FUTURE */
        {I_BLENDVPS, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69542, 410},
    /* 2992 : DPPD xmmreg,xmmrm128,imm8 [rmi:      66 0f3a 41 /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(41) 110 026 : NOAPX,SSE41,FUTURE */
        {I_DPPD, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58069, 410},
    /* 2993 : DPPS xmmreg,xmmrm128,imm8 [rmi:      66 0f3a 40 /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(40) 110 026 : NOAPX,SSE41,FUTURE */
        {I_DPPS, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58076, 410},
    /* 2994 : EXTRACTPS rm32,xmmreg,imm8 [mri:      66 0f3a 17 /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(17) 101 026 : NOAPX,SSE41,FUTURE */
        {I_EXTRACTPS, 3, {RM_GPR|BITS32|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51534, 410},
    /* 2995 : EXTRACTPS reg64,xmmreg,imm8 [mri:      o64 66 0f3a 17 /r ib,u            ] LONG,PROT,SSE41,X86_64 */
        /* 324 361 357 001(17) 101 026 : NOAPX,LONG,PROT,SSE41,X86_64 */
        {I_EXTRACTPS, 3, {REG_GPR|BITS64|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51533, 411},
    /* 2996 : INSERTPS xmmreg,xmmrm32,imm8 [rmi:      66 0f3a 21 /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(21) 110 026 : NOAPX,SSE41,FUTURE */
        {I_INSERTPS, 3, {XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58083, 410},
    /* 2997 : MOVNTDQA xmmreg,mem128 [rm:       66 0f38 2a /r                     ] SSE41,FUTURE */
        /* 361 356 001(2a) 110 : NOAPX,SSE41,FUTURE */
        {I_MOVNTDQA, 2, {XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69548, 410},
    /* 2998 : MPSADBW xmmreg,xmmrm128,imm8 [rmi:      66 0f3a 42 /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(42) 110 026 : NOAPX,SSE41,FUTURE */
        {I_MPSADBW, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58090, 410},
    /* 2999 : PACKUSDW xmmreg,xmmrm128 [rm:       66 0f38 2b /r                     ] SSE41,FUTURE */
        /* 361 356 001(2b) 110 : NOAPX,SSE41,FUTURE */
        {I_PACKUSDW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69554, 410},
    /* 3000 : PBLENDVB xmmreg,xmmrm,xmm0 [rm-:      66 0f38 10 /r                     ] SSE41,FUTURE */
        /* 361 356 001(10) 110 : NOAPX,SSE41,FUTURE */
        {I_PBLENDVB, 3, {XMMREG|RN_L16,RM_XMM|RN_L16,XMMREG|RN_ZERO|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+69560, 410},
    /* 3001 : PBLENDVB xmmreg,xmmrm128 [rm:       66 0f38 10 /r                     ] SSE41,FUTURE */
        /* 361 356 001(10) 110 : NOAPX,SSE41,FUTURE */
        {I_PBLENDVB, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69560, 410},
    /* 3002 : PBLENDW xmmreg,xmmrm128,imm8 [rmi:      66 0f3a 0e /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(0e) 110 026 : NOAPX,SSE41,FUTURE */
        {I_PBLENDW, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58097, 410},
    /* 3003 : PCMPEQQ xmmreg,xmmrm128 [rm:       66 0f38 29 /r                     ] SSE41,FUTURE */
        /* 361 356 001(29) 110 : NOAPX,SSE41,FUTURE */
        {I_PCMPEQQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69566, 410},
    /* 3004 : PEXTRB reg32,xmmreg,imm8 [mri:      66 0f3a 14 /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(14) 101 026 : NOAPX,SSE41,FUTURE */
        {I_PEXTRB, 3, {REG_GPR|BITS32|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51542, 410},
    /* 3005 : PEXTRB mem8,xmmreg,imm8 [mri:      66 0f3a 14 /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(14) 101 026 : NOAPX,SSE41,FUTURE */
        {I_PEXTRB, 3, {MEMORY|BITS8|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51542, 410},
    /* 3006 : PEXTRB reg64,xmmreg,imm8 [mri:      o64nw 66 0f3a 14 /r ib,u          ] LONG,PROT,SSE41,X86_64 */
        /* 323 361 357 001(14) 101 026 : NWSIZE,NOAPX,LONG,PROT,SSE41,X86_64 */
        {I_PEXTRB, 3, {REG_GPR|BITS64|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51541, 412},
    /* 3007 : PEXTRD rm32,xmmreg,imm8 [mri:      norexw 66 0f3a 16 /r ib,u         ] SSE41,FUTURE */
        /* 317 361 357 001(16) 101 026 : NOAPX,SSE41,FUTURE */
        {I_PEXTRD, 3, {RM_GPR|BITS32|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51549, 410},
    /* 3008 : PEXTRQ rm64,xmmreg,imm8 [mri:      o64 66 0f3a 16 /r ib,u            ] LONG,PROT,SSE41,X86_64 */
        /* 324 361 357 001(16) 101 026 : NOAPX,LONG,PROT,SSE41,X86_64 */
        {I_PEXTRQ, 3, {RM_GPR|BITS64|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51557, 411},
    /* 3009 : PEXTRW reg32,xmmreg,imm8 [mri:      66 0f3a 15 /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(15) 101 026 : NOAPX,SSE41,FUTURE */
        {I_PEXTRW, 3, {REG_GPR|BITS32|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51566, 410},
    /* 3010 : PEXTRW mem16,xmmreg,imm8 [mri:      66 0f3a 15 /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(15) 101 026 : NOAPX,SSE41,FUTURE */
        {I_PEXTRW, 3, {MEMORY|BITS16|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51566, 410},
    /* 3011 : PEXTRW reg64,xmmreg,imm8 [mri:      o64 66 0f3a 15 /r ib,u            ] LONG,PROT,SSE41,X86_64 */
        /* 324 361 357 001(15) 101 026 : NOAPX,LONG,PROT,SSE41,X86_64 */
        {I_PEXTRW, 3, {REG_GPR|BITS64|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51565, 411},
    /* 3012 : PHMINPOSUW xmmreg,xmmrm128 [rm:       66 0f38 41 /r                     ] SSE41,FUTURE */
        /* 361 356 001(41) 110 : NOAPX,SSE41,FUTURE */
        {I_PHMINPOSUW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69572, 410},
    /* 3013 : PINSRB xmmreg,mem,imm8 [rmi:      66 0f3a 20 /r ib,u                ] AR2,SB,SSE41,FUTURE */
        /* 361 357 001(20) 110 026 : AR2,SB,NOAPX,SSE41,FUTURE */
        {I_PINSRB, 3, {XMMREG|RN_L16,MEMORY|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51574, 413},
    /* 3014 : PINSRB xmmreg,rm8,imm8 [rmi:      nohi 66 0f3a 20 /r ib,u           ] AR2,SB,SSE41,FUTURE */
        /* 325 361 357 001(20) 110 026 : AR2,SB,NOAPX,SSE41,FUTURE */
        {I_PINSRB, 3, {XMMREG|RN_L16,RM_GPR|BITS8|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51573, 413},
    /* 3015 : PINSRB xmmreg,reg32,imm8 [rmi:      66 0f3a 20 /r ib,u                ] AR2,SB,SSE41,FUTURE */
        /* 361 357 001(20) 110 026 : AR2,SB,NOAPX,SSE41,FUTURE */
        {I_PINSRB, 3, {XMMREG|RN_L16,REG_GPR|BITS32|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51574, 413},
    /* 3016 : PINSRD xmmreg,rm32,imm8 [rmi:      norexw 66 0f3a 22 /r ib,u         ] AR2,SB,SSE41,FUTURE */
        /* 317 361 357 001(22) 110 026 : AR2,SB,NOAPX,SSE41,FUTURE */
        {I_PINSRD, 3, {XMMREG|RN_L16,RM_GPR|BITS32|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51581, 413},
    /* 3017 : PINSRQ xmmreg,rm64,imm8 [rmi:      o64 66 0f3a 22 /r ib,u            ] AR2,SB,LONG,PROT,SSE41,X86_64 */
        /* 324 361 357 001(22) 110 026 : AR2,SB,NOAPX,LONG,PROT,SSE41,X86_64 */
        {I_PINSRQ, 3, {XMMREG|RN_L16,RM_GPR|BITS64|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51589, 414},
    /* 3018 : PMAXSB xmmreg,xmmrm128 [rm:       66 0f38 3c /r                     ] SSE41,FUTURE */
        /* 361 356 001(3c) 110 : NOAPX,SSE41,FUTURE */
        {I_PMAXSB, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69578, 410},
    /* 3019 : PMAXSD xmmreg,xmmrm128 [rm:       66 0f38 3d /r                     ] SSE41,FUTURE */
        /* 361 356 001(3d) 110 : NOAPX,SSE41,FUTURE */
        {I_PMAXSD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69584, 410},
    /* 3020 : PMAXUD xmmreg,xmmrm128 [rm:       66 0f38 3f /r                     ] SSE41,FUTURE */
        /* 361 356 001(3f) 110 : NOAPX,SSE41,FUTURE */
        {I_PMAXUD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69590, 410},
    /* 3021 : PMAXUW xmmreg,xmmrm128 [rm:       66 0f38 3e /r                     ] SSE41,FUTURE */
        /* 361 356 001(3e) 110 : NOAPX,SSE41,FUTURE */
        {I_PMAXUW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69596, 410},
    /* 3022 : PMINSB xmmreg,xmmrm128 [rm:       66 0f38 38 /r                     ] SSE41,FUTURE */
        /* 361 356 001(38) 110 : NOAPX,SSE41,FUTURE */
        {I_PMINSB, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69602, 410},
    /* 3023 : PMINSD xmmreg,xmmrm128 [rm:       66 0f38 39 /r                     ] SSE41,FUTURE */
        /* 361 356 001(39) 110 : NOAPX,SSE41,FUTURE */
        {I_PMINSD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69608, 410},
    /* 3024 : PMINUD xmmreg,xmmrm128 [rm:       66 0f38 3b /r                     ] SSE41,FUTURE */
        /* 361 356 001(3b) 110 : NOAPX,SSE41,FUTURE */
        {I_PMINUD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69614, 410},
    /* 3025 : PMINUW xmmreg,xmmrm128 [rm:       66 0f38 3a /r                     ] SSE41,FUTURE */
        /* 361 356 001(3a) 110 : NOAPX,SSE41,FUTURE */
        {I_PMINUW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69620, 410},
    /* 3026 : PMOVSXBW xmmreg,xmmrm64 [rm:       66 0f38 20 /r                     ] AR0-1,SQ,SSE41,FUTURE */
        /* 361 356 001(20) 110 : AR0,AR1,SQ,NOAPX,SSE41,FUTURE */
        {I_PMOVSXBW, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69626, 415},
    /* 3027 : PMOVSXBD xmmreg,xmmrm32 [rm:       66 0f38 21 /r                     ] AR0-1,SD,SSE41,FUTURE */
        /* 361 356 001(21) 110 : AR0,AR1,SD,NOAPX,SSE41,FUTURE */
        {I_PMOVSXBD, 2, {XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69632, 416},
    /* 3028 : PMOVSXBQ xmmreg,xmmrm16 [rm:       66 0f38 22 /r                     ] AR0-1,SW,SSE41,FUTURE */
        /* 361 356 001(22) 110 : AR0,AR1,SW,NOAPX,SSE41,FUTURE */
        {I_PMOVSXBQ, 2, {XMMREG|RN_L16,RM_XMM|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69638, 417},
    /* 3029 : PMOVSXWD xmmreg,xmmrm64 [rm:       66 0f38 23 /r                     ] AR0-1,SQ,SSE41,FUTURE */
        /* 361 356 001(23) 110 : AR0,AR1,SQ,NOAPX,SSE41,FUTURE */
        {I_PMOVSXWD, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69644, 415},
    /* 3030 : PMOVSXWQ xmmreg,xmmrm32 [rm:       66 0f38 24 /r                     ] AR0-1,SD,SSE41,FUTURE */
        /* 361 356 001(24) 110 : AR0,AR1,SD,NOAPX,SSE41,FUTURE */
        {I_PMOVSXWQ, 2, {XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69650, 416},
    /* 3031 : PMOVSXDQ xmmreg,xmmrm64 [rm:       66 0f38 25 /r                     ] AR0-1,SQ,SSE41,FUTURE */
        /* 361 356 001(25) 110 : AR0,AR1,SQ,NOAPX,SSE41,FUTURE */
        {I_PMOVSXDQ, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69656, 415},
    /* 3032 : PMOVZXBW xmmreg,xmmrm64 [rm:       66 0f38 30 /r                     ] AR0-1,SQ,SSE41,FUTURE */
        /* 361 356 001(30) 110 : AR0,AR1,SQ,NOAPX,SSE41,FUTURE */
        {I_PMOVZXBW, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69662, 415},
    /* 3033 : PMOVZXBD xmmreg,xmmrm32 [rm:       66 0f38 31 /r                     ] AR0-1,SD,SSE41,FUTURE */
        /* 361 356 001(31) 110 : AR0,AR1,SD,NOAPX,SSE41,FUTURE */
        {I_PMOVZXBD, 2, {XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69668, 416},
    /* 3034 : PMOVZXBQ xmmreg,xmmrm16 [rm:       66 0f38 32 /r                     ] AR0-1,SW,SSE41,FUTURE */
        /* 361 356 001(32) 110 : AR0,AR1,SW,NOAPX,SSE41,FUTURE */
        {I_PMOVZXBQ, 2, {XMMREG|RN_L16,RM_XMM|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69674, 417},
    /* 3035 : PMOVZXWD xmmreg,xmmrm64 [rm:       66 0f38 33 /r                     ] AR0-1,SQ,SSE41,FUTURE */
        /* 361 356 001(33) 110 : AR0,AR1,SQ,NOAPX,SSE41,FUTURE */
        {I_PMOVZXWD, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69680, 415},
    /* 3036 : PMOVZXWQ xmmreg,xmmrm32 [rm:       66 0f38 34 /r                     ] AR0-1,SD,SSE41,FUTURE */
        /* 361 356 001(34) 110 : AR0,AR1,SD,NOAPX,SSE41,FUTURE */
        {I_PMOVZXWQ, 2, {XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69686, 416},
    /* 3037 : PMOVZXDQ xmmreg,xmmrm64 [rm:       66 0f38 35 /r                     ] AR0-1,SQ,SSE41,FUTURE */
        /* 361 356 001(35) 110 : AR0,AR1,SQ,NOAPX,SSE41,FUTURE */
        {I_PMOVZXDQ, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69692, 415},
    /* 3038 : PMULDQ xmmreg,xmmrm128 [rm:       66 0f38 28 /r                     ] SSE41,FUTURE */
        /* 361 356 001(28) 110 : NOAPX,SSE41,FUTURE */
        {I_PMULDQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69698, 410},
    /* 3039 : PMULLD xmmreg,xmmrm128 [rm:       66 0f38 40 /r                     ] SSE41,FUTURE */
        /* 361 356 001(40) 110 : NOAPX,SSE41,FUTURE */
        {I_PMULLD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69704, 410},
    /* 3040 : PTEST xmmreg,xmmrm128 [rm:       66 0f38 17 /r                     ] SSE41,FUTURE */
        /* 361 356 001(17) 110 : NOAPX,SSE41,FUTURE */
        {I_PTEST, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69710, 410},
    /* 3041 : ROUNDPD xmmreg,xmmrm128,imm8 [rmi:      66 0f3a 09 /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(09) 110 026 : NOAPX,SSE41,FUTURE */
        {I_ROUNDPD, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58104, 410},
    /* 3042 : ROUNDPS xmmreg,xmmrm128,imm8 [rmi:      66 0f3a 08 /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(08) 110 026 : NOAPX,SSE41,FUTURE */
        {I_ROUNDPS, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58111, 410},
    /* 3043 : ROUNDSD xmmreg,xmmrm64,imm8 [rmi:      66 0f3a 0b /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(0b) 110 026 : NOAPX,SSE41,FUTURE */
        {I_ROUNDSD, 3, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58118, 410},
    /* 3044 : ROUNDSS xmmreg,xmmrm32,imm8 [rmi:      66 0f3a 0a /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(0a) 110 026 : NOAPX,SSE41,FUTURE */
        {I_ROUNDSS, 3, {XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58125, 410},
    /* 3045 : CRC32 reg32,rm8 [rm:       o8 f2 0f38 f0 /r                  ] SSE42,FUTURE */
        /* 332 356 001(f0) 110 : NOAPX,SSE42,FUTURE */
        {I_CRC32, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS8|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58154, 418},
    /* 3046 : CRC32 reg32,rm16 [rm:       o16 f2 0f38 f1 /r                 ] SSE42,FUTURE */
        /* 320 332 356 001(f1) 110 : NOAPX,SSE42,FUTURE */
        {I_CRC32, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58132, 418},
    /* 3047 : CRC32 reg32,rm32 [rm:       o32 f2 0f38 f1 /r                 ] SSE42,FUTURE */
        /* 321 332 356 001(f1) 110 : NOAPX,SSE42,FUTURE */
        {I_CRC32, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58139, 418},
    /* 3048 : CRC32 reg32,rm8 [rm:       evex.l0.m4.o8 f0 /r               ] LONG,PROT,SSE42,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(f0) 110 : WIG,LONG,EVEX,PROT,SSE42,APX,X86_64 */
        {I_CRC32, 2, {REG_GPR|BITS32,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+25785, 420},
    /* 3049 : CRC32 reg32,rm16 [rm:       evex.l0.m4.o16 f1 /r              ] LONG,PROT,SSE42,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(f1) 110 : LONG,EVEX,PROT,SSE42,APX,X86_64 */
        {I_CRC32, 2, {REG_GPR|BITS32,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+17923, 421},
    /* 3050 : CRC32 reg32,rm32 [rm:       evex.l0.m4.o32 f1 /r              ] LONG,PROT,SSE42,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(f1) 110 : WW,LONG,EVEX,PROT,SSE42,APX,X86_64 */
        {I_CRC32, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17933, 422},
    /* 3051 : CRC32 reg64,rm8 [rm:       o8 f2 0f38 f0 /r                  ] OPT,LONG,PROT,SSE42,X86_64 */
        /* 332 356 001(f0) 110 : OPT,NOAPX,LONG,PROT,SSE42,X86_64 */
        {I_CRC32, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS8|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58154, 423},
    /* 3052 : CRC32 reg64,rm16 [rm:       o16 f2 0f38 f1 /r                 ] LONG,PROT,SSE42,X86_64 */
        /* 320 332 356 001(f1) 110 : NOAPX,LONG,PROT,SSE42,X86_64 */
        {I_CRC32, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58132, 419},
    /* 3053 : CRC32 reg64,rm32 [rm:       o32 f2 0f38 f1 /r                 ] LONG,PROT,SSE42,X86_64 */
        /* 321 332 356 001(f1) 110 : NOAPX,LONG,PROT,SSE42,X86_64 */
        {I_CRC32, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58139, 419},
    /* 3054 : CRC32 reg64,rm64 [rm:       o64 f2 0f38 f1 /r                 ] LONG,PROT,SSE42,X86_64 */
        /* 324 332 356 001(f1) 110 : NOAPX,LONG,PROT,SSE42,X86_64 */
        {I_CRC32, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58146, 419},
    /* 3055 : CRC32 reg64,rm8 [rm:       evex.l0.m4.o8 f0 /r               ] OPT,LONG,PROT,SSE42,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(f0) 110 : OPT,WIG,LONG,EVEX,PROT,SSE42,APX,X86_64 */
        {I_CRC32, 2, {REG_GPR|BITS64,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+25785, 424},
    /* 3056 : CRC32 reg64,rm16 [rm:       evex.l0.m4.o16 f1 /r              ] LONG,PROT,SSE42,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(f1) 110 : LONG,EVEX,PROT,SSE42,APX,X86_64 */
        {I_CRC32, 2, {REG_GPR|BITS64,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+17923, 421},
    /* 3057 : CRC32 reg64,rm32 [rm:       evex.l0.m4.o32 f1 /r              ] LONG,PROT,SSE42,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(f1) 110 : WW,LONG,EVEX,PROT,SSE42,APX,X86_64 */
        {I_CRC32, 2, {REG_GPR|BITS64,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17933, 422},
    /* 3058 : CRC32 reg64,rm64 [rm:       evex.l0.m4.o64 f1 /r              ] LONG,PROT,SSE42,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(f1) 110 : WW,LONG,EVEX,PROT,SSE42,APX,X86_64 */
        {I_CRC32, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17943, 422},
    /* 3059 : CRC32 reg64,rm8 [rm:       o64 f2 0f38 f0 /r                 ] LONG,SSE42,X86_64,FUTURE */
        /* 324 332 356 001(f0) 110 : NOAPX,LONG,PROT,SSE42,FUTURE */
        {I_CRC32, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS8|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58153, 425},
    /* 3060 : CRC32 reg64,rm8 [rm:       evex.l0.m4.o64 f0 /r              ] LONG,PROT,SSE42,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(f0) 110 : WW,LONG,EVEX,PROT,SSE42,APX,X86_64 */
        {I_CRC32, 2, {REG_GPR|BITS64,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+17953, 422},
    /* 3061 : PCMPESTRI xmmreg,xmmrm128,imm8 [rmi:      66 0f3a 61 /r ib,u                ] SSE42,FUTURE */
        /* 361 357 001(61) 110 026 : NOAPX,SSE42,FUTURE */
        {I_PCMPESTRI, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58160, 418},
    /* 3062 : PCMPESTRM xmmreg,xmmrm128,imm8 [rmi:      66 0f3a 60 /r ib,u                ] SSE42,FUTURE */
        /* 361 357 001(60) 110 026 : NOAPX,SSE42,FUTURE */
        {I_PCMPESTRM, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58167, 418},
    /* 3063 : PCMPISTRI xmmreg,xmmrm128,imm8 [rmi:      66 0f3a 63 /r ib,u                ] SSE42,FUTURE */
        /* 361 357 001(63) 110 026 : NOAPX,SSE42,FUTURE */
        {I_PCMPISTRI, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58174, 418},
    /* 3064 : PCMPISTRM xmmreg,xmmrm128,imm8 [rmi:      66 0f3a 62 /r ib,u                ] SSE42,FUTURE */
        /* 361 357 001(62) 110 026 : NOAPX,SSE42,FUTURE */
        {I_PCMPISTRM, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58181, 418},
    /* 3065 : PCMPGTQ xmmreg,xmmrm128 [rm:       66 0f38 37 /r                     ] SSE42,FUTURE */
        /* 361 356 001(37) 110 : NOAPX,SSE42,FUTURE */
        {I_PCMPGTQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69716, 418},
    /* 3066 : POPCNT reg16,rm16 [rm:       o16 f3i 0f b8 /r                  ] AR0-1,SW,FL,NEHALEM */
        /* 320 333 355 001(b8) 110 : AR0,AR1,SW,FL,NEHALEM */
        {I_POPCNT, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58188, 426},
    /* 3067 : POPCNT reg32,rm32 [rm:       o32 f3i 0f b8 /r                  ] AR0-1,SD,FL,NEHALEM */
        /* 321 333 355 001(b8) 110 : AR0,AR1,SD,FL,NEHALEM */
        {I_POPCNT, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+58195, 427},
    /* 3068 : POPCNT reg64,rm64 [rm:       o64 f3i 0f b8 /r                  ] AR0-1,SQ,LONG,FL,PROT,X86_64,NEHALEM */
        /* 324 333 355 001(b8) 110 : AR0,AR1,SQ,LONG,FL,PROT,NEHALEM */
        {I_POPCNT, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+58202, 428},
    /* 3069 : POPCNT reg16,rm16 [rm:       evex.nf.nd0.l0.m4.o16 88 /r       ] SM0-1,NF,LONG,PROT,APX,X86_64,NEHALEM */
        /* 250(f4 7d 08) 300 320 001(88) 110 : SM0,SM1,NF,NF_E,LONG,EVEX,PROT,APX,NEHALEM */
        {I_POPCNT, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+17963, 429},
    /* 3070 : POPCNT reg32,rm32 [rm:       evex.nf.nd0.l0.m4.o32 88 /r       ] SM0-1,NF,LONG,PROT,APX,X86_64,NEHALEM */
        /* 250(f4 7c 08) 300 321 001(88) 110 : SM0,SM1,NF,NF_E,WW,LONG,EVEX,PROT,APX,NEHALEM */
        {I_POPCNT, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17973, 430},
    /* 3071 : POPCNT reg64,rm64 [rm:       evex.nf.nd0.l0.m4.o64 88 /r       ] SM0-1,NF,LONG,PROT,APX,X86_64,NEHALEM */
        /* 250(f4 fc 08) 300 324 001(88) 110 : SM0,SM1,NF,NF_E,WW,LONG,EVEX,PROT,APX,NEHALEM */
        {I_POPCNT, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17983, 430},
    /* 3072 : GETSEC void [          0f 37                             ] NOAPX,FL,KATMAI */
        /* 355 001(37) : NOAPX,FL,KATMAI */
        {I_GETSEC, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73289, 431},
    /* 3073 : PFRCPV mmxreg,mmxrm [rm:       0f 0f /r 86                       ] AR0-1,SQ,3DNOW,PENT,CYRIX */
        /* 355 001(0f) 110 001(86) : AR0,AR1,SQ,3DNOW,PENT,CYRIX */
        {I_PFRCPV, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+58209, 432},
    /* 3074 : PFRSQRTV mmxreg,mmxrm [rm:       0f 0f /r 87                       ] AR0-1,SQ,3DNOW,PENT,CYRIX */
        /* 355 001(0f) 110 001(87) : AR0,AR1,SQ,3DNOW,PENT,CYRIX */
        {I_PFRSQRTV, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+58216, 432},
    /* 3075 : AESENC xmmreg,xmmrm128 [rm:       66 0f38 dc /r                     ] SSE,WESTMERE */
        /* 361 356 001(dc) 110 : NOAPX,SSE,WESTMERE */
        {I_AESENC, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69722, 433},
    /* 3076 : AESENCLAST xmmreg,xmmrm128 [rm:       66 0f38 dd /r                     ] SSE,WESTMERE */
        /* 361 356 001(dd) 110 : NOAPX,SSE,WESTMERE */
        {I_AESENCLAST, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69728, 433},
    /* 3077 : AESDEC xmmreg,xmmrm128 [rm:       66 0f38 de /r                     ] SSE,WESTMERE */
        /* 361 356 001(de) 110 : NOAPX,SSE,WESTMERE */
        {I_AESDEC, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69734, 433},
    /* 3078 : AESDECLAST xmmreg,xmmrm128 [rm:       66 0f38 df /r                     ] SSE,WESTMERE */
        /* 361 356 001(df) 110 : NOAPX,SSE,WESTMERE */
        {I_AESDECLAST, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69740, 433},
    /* 3079 : AESIMC xmmreg,xmmrm128 [rm:       66 0f38 db /r                     ] SSE,WESTMERE */
        /* 361 356 001(db) 110 : NOAPX,SSE,WESTMERE */
        {I_AESIMC, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69746, 433},
    /* 3080 : AESKEYGENASSIST xmmreg,xmmrm128,imm8 [rmi:      66 0f3a df /r ib                  ] SSE,WESTMERE */
        /* 361 357 001(df) 110 022 : NOAPX,SSE,WESTMERE */
        {I_AESKEYGENASSIST, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58223, 433},
    /* 3081 : VAESENC xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 dc /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(dc) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VAESENC, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58230, 434},
    /* 3082 : VAESENCLAST xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 dd /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(dd) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VAESENCLAST, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58244, 434},
    /* 3083 : VAESDEC xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 de /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(de) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VAESDEC, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58258, 434},
    /* 3084 : VAESDECLAST xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 df /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(df) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VAESDECLAST, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58272, 434},
    /* 3085 : VAESIMC xmmreg,xmmrm128 [rm:       vex.128.66.0f38 db /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(db) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VAESIMC, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58286, 434},
    /* 3086 : VAESKEYGENASSIST xmmreg,xmmrm128,imm8 [rmi:      vex.128.66.0f3a df /r ib          ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(df) 110 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VAESKEYGENASSIST, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51597, 434},
    /* 3087 : AESDEC128KL xmmreg,mem [rm:       f3 0f38 dd /r                     ] FL,AESKLE,FUTURE */
        /* 333 356 001(dd) 110 : NOAPX,FL,AESKLE,FUTURE */
        {I_AESDEC128KL, 2, {XMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69752, 435},
    /* 3088 : AESDEC256KL xmmreg,mem512 [rm:       f3 0f38 df /r                     ] FL,AESKLE,FUTURE */
        /* 333 356 001(df) 110 : NOAPX,FL,AESKLE,FUTURE */
        {I_AESDEC256KL, 2, {XMMREG|RN_L16,MEMORY|BITS512|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69758, 435},
    /* 3089 : AESENC128KL xmmreg,mem [rm:       f3 0f38 dc /r                     ] FL,AESKLE,FUTURE */
        /* 333 356 001(dc) 110 : NOAPX,FL,AESKLE,FUTURE */
        {I_AESENC128KL, 2, {XMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69764, 435},
    /* 3090 : AESENC256KL xmmreg,mem512 [rm:       f3 0f38 de /r                     ] FL,AESKLE,FUTURE */
        /* 333 356 001(de) 110 : NOAPX,FL,AESKLE,FUTURE */
        {I_AESENC256KL, 2, {XMMREG|RN_L16,MEMORY|BITS512|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69770, 435},
    /* 3091 : ENCODEKEY128 reg32,reg32 [rm:       f3 0f38 fa /r                     ] AESKLE,FUTURE */
        /* 333 356 001(fa) 110 : NOAPX,AESKLE,FUTURE */
        {I_ENCODEKEY128, 2, {REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69776, 436},
    /* 3092 : ENCODEKEY256 reg32,reg32 [rm:       f3 0f38 fb /r                     ] AESKLE,FUTURE */
        /* 333 356 001(fb) 110 : NOAPX,AESKLE,FUTURE */
        {I_ENCODEKEY256, 2, {REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69782, 436},
    /* 3093 : LOADIWKEY xmmreg,xmmreg [rm:       f3 0f38 dc /r                     ] FL,AESKLE,FUTURE */
        /* 333 356 001(dc) 110 : NOAPX,FL,AESKLE,FUTURE */
        {I_LOADIWKEY, 2, {XMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69764, 435},
    /* 3094 : AESDECWIDE128KL mem [m:        f3 0f38 d8 /1                     ] FL,AESKLEWIDE_KL,FUTURE */
        /* 333 356 001(d8) 201 : NOAPX,FL,AESKLEWIDE_KL,FUTURE */
        {I_AESDECWIDE128KL, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69788, 437},
    /* 3095 : AESDECWIDE256KL mem512 [m:        f3 0f38 d8 /3                     ] FL,AESKLEWIDE_KL,FUTURE */
        /* 333 356 001(d8) 203 : NOAPX,FL,AESKLEWIDE_KL,FUTURE */
        {I_AESDECWIDE256KL, 1, {MEMORY|BITS512|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69794, 437},
    /* 3096 : AESENCWIDE128KL mem [m:        f3 0f38 d8 /0                     ] FL,AESKLEWIDE_KL,FUTURE */
        /* 333 356 001(d8) 200 : NOAPX,FL,AESKLEWIDE_KL,FUTURE */
        {I_AESENCWIDE128KL, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69800, 437},
    /* 3097 : AESENCWIDE256KL mem512 [m:        f3 0f38 d8 /2                     ] FL,AESKLEWIDE_KL,FUTURE */
        /* 333 356 001(d8) 202 : NOAPX,FL,AESKLEWIDE_KL,FUTURE */
        {I_AESENCWIDE256KL, 1, {MEMORY|BITS512|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69806, 437},
    /* 3098 : VAESENC ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.wig dc /r     ] VAES,FUTURE */
        /* 261(002 005) 001(dc) 120 : NOAPX,WIG,VEX,VAES,FUTURE */
        {I_VAESENC, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58293, 438},
    /* 3099 : VAESENCLAST ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.wig dd /r     ] VAES,FUTURE */
        /* 261(002 005) 001(dd) 120 : NOAPX,WIG,VEX,VAES,FUTURE */
        {I_VAESENCLAST, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58307, 438},
    /* 3100 : VAESDEC ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.wig de /r     ] VAES,FUTURE */
        /* 261(002 005) 001(de) 120 : NOAPX,WIG,VEX,VAES,FUTURE */
        {I_VAESDEC, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58321, 438},
    /* 3101 : VAESDECLAST ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.wig df /r     ] VAES,FUTURE */
        /* 261(002 005) 001(df) 120 : NOAPX,WIG,VEX,VAES,FUTURE */
        {I_VAESDECLAST, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58335, 438},
    /* 3102 : VAESENC xmmreg,xmmreg*,xmmrm128 [rvm:fv:   evex.nds.128.66.0f38.wig dc /r    ] AVX512,AVX512VL,VAES,FUTURE */
        /* 241(f2 7d 08) 301 001(dc) 120 : WIG,EVEX,PROT,AVX512,AVX512VL,VAES,FUTURE */
        {I_VAESENC, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, NO_DECORATOR, nasm_bytecodes+25794, 439},
    /* 3103 : VAESENC ymmreg,ymmreg*,ymmrm256 [rvm:fv:   evex.nds.256.66.0f38.wig dc /r    ] AVX512,AVX512VL,VAES,FUTURE */
        /* 241(f2 7d 28) 301 001(dc) 120 : WIG,EVEX,PROT,AVX512,AVX512VL,VAES,FUTURE */
        {I_VAESENC, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, NO_DECORATOR, nasm_bytecodes+25812, 439},
    /* 3104 : VAESENCLAST xmmreg,xmmreg*,xmmrm128 [rvm:fv:   evex.nds.128.66.0f38.wig dd /r    ] AVX512,AVX512VL,VAES,FUTURE */
        /* 241(f2 7d 08) 301 001(dd) 120 : WIG,EVEX,PROT,AVX512,AVX512VL,VAES,FUTURE */
        {I_VAESENCLAST, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, NO_DECORATOR, nasm_bytecodes+25830, 439},
    /* 3105 : VAESENCLAST ymmreg,ymmreg*,ymmrm256 [rvm:fv:   evex.nds.256.66.0f38.wig dd /r    ] AVX512,AVX512VL,VAES,FUTURE */
        /* 241(f2 7d 28) 301 001(dd) 120 : WIG,EVEX,PROT,AVX512,AVX512VL,VAES,FUTURE */
        {I_VAESENCLAST, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, NO_DECORATOR, nasm_bytecodes+25848, 439},
    /* 3106 : VAESDEC xmmreg,xmmreg*,xmmrm128 [rvm:fv:   evex.nds.128.66.0f38.wig de /r    ] AVX512,AVX512VL,VAES,FUTURE */
        /* 241(f2 7d 08) 301 001(de) 120 : WIG,EVEX,PROT,AVX512,AVX512VL,VAES,FUTURE */
        {I_VAESDEC, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, NO_DECORATOR, nasm_bytecodes+25866, 439},
    /* 3107 : VAESDEC ymmreg,ymmreg*,ymmrm256 [rvm:fv:   evex.nds.256.66.0f38.wig de /r    ] AVX512,AVX512VL,VAES,FUTURE */
        /* 241(f2 7d 28) 301 001(de) 120 : WIG,EVEX,PROT,AVX512,AVX512VL,VAES,FUTURE */
        {I_VAESDEC, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, NO_DECORATOR, nasm_bytecodes+25884, 439},
    /* 3108 : VAESDECLAST xmmreg,xmmreg*,xmmrm128 [rvm:fv:   evex.nds.128.66.0f38.wig df /r    ] AVX512,AVX512VL,VAES,FUTURE */
        /* 241(f2 7d 08) 301 001(df) 120 : WIG,EVEX,PROT,AVX512,AVX512VL,VAES,FUTURE */
        {I_VAESDECLAST, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, NO_DECORATOR, nasm_bytecodes+25902, 439},
    /* 3109 : VAESDECLAST ymmreg,ymmreg*,ymmrm256 [rvm:fv:   evex.nds.256.66.0f38.wig df /r    ] AVX512,AVX512VL,VAES,FUTURE */
        /* 241(f2 7d 28) 301 001(df) 120 : WIG,EVEX,PROT,AVX512,AVX512VL,VAES,FUTURE */
        {I_VAESDECLAST, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, NO_DECORATOR, nasm_bytecodes+25920, 439},
    /* 3110 : VAESENC zmmreg,zmmreg*,zmmrm512 [rvm:fv:   evex.nds.512.66.0f38.wig dc /r    ] AVX512,VAES,FUTURE */
        /* 241(f2 7d 48) 301 001(dc) 120 : WIG,EVEX,PROT,AVX512,VAES,FUTURE */
        {I_VAESENC, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, NO_DECORATOR, nasm_bytecodes+25938, 440},
    /* 3111 : VAESENCLAST zmmreg,zmmreg*,zmmrm512 [rvm:fv:   evex.nds.512.66.0f38.wig dd /r    ] AVX512,VAES,FUTURE */
        /* 241(f2 7d 48) 301 001(dd) 120 : WIG,EVEX,PROT,AVX512,VAES,FUTURE */
        {I_VAESENCLAST, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, NO_DECORATOR, nasm_bytecodes+25956, 440},
    /* 3112 : VAESDEC zmmreg,zmmreg*,zmmrm512 [rvm:fv:   evex.nds.512.66.0f38.wig de /r    ] AVX512,VAES,FUTURE */
        /* 241(f2 7d 48) 301 001(de) 120 : WIG,EVEX,PROT,AVX512,VAES,FUTURE */
        {I_VAESDEC, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, NO_DECORATOR, nasm_bytecodes+25974, 440},
    /* 3113 : VAESDECLAST zmmreg,zmmreg*,zmmrm512 [rvm:fv:   evex.nds.512.66.0f38.wig df /r    ] AVX512,VAES,FUTURE */
        /* 241(f2 7d 48) 301 001(df) 120 : WIG,EVEX,PROT,AVX512,VAES,FUTURE */
        {I_VAESDECLAST, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, NO_DECORATOR, nasm_bytecodes+25992, 440},
    /* 3114 : VADDPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 58 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(58) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VADDPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58349, 434},
    /* 3115 : VADDPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 58 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(58) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VADDPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58363, 434},
    /* 3116 : VADDPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f 58 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(58) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VADDPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58377, 434},
    /* 3117 : VADDPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f 58 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(58) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VADDPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58391, 434},
    /* 3118 : VADDSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f 58 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(58) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VADDSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58405, 441},
    /* 3119 : VADDSS xmmreg,xmmreg*,xmmrm32 [rvm:      vex.nds.lig.f3.0f 58 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(58) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VADDSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58419, 441},
    /* 3120 : VADDSUBPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f d0 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(d0) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VADDSUBPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58433, 434},
    /* 3121 : VADDSUBPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f d0 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(d0) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VADDSUBPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58447, 434},
    /* 3122 : VADDSUBPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.f2.0f d0 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(d0) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VADDSUBPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58461, 434},
    /* 3123 : VADDSUBPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.f2.0f d0 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 007) 001(d0) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VADDSUBPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58475, 434},
    /* 3124 : VANDPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 54 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(54) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VANDPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58489, 434},
    /* 3125 : VANDPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 54 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(54) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VANDPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58503, 434},
    /* 3126 : VANDPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f 54 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(54) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VANDPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58517, 434},
    /* 3127 : VANDPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f 54 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(54) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VANDPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58531, 434},
    /* 3128 : VANDNPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 55 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(55) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VANDNPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58545, 434},
    /* 3129 : VANDNPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 55 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(55) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VANDNPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58559, 434},
    /* 3130 : VANDNPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f 55 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(55) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VANDNPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58573, 434},
    /* 3131 : VANDNPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f 55 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(55) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VANDNPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58587, 434},
    /* 3132 : VBLENDPD xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.66.0f3a 0d /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(0d) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VBLENDPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51605, 434},
    /* 3133 : VBLENDPD ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.66.0f3a 0d /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 005) 001(0d) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VBLENDPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51621, 434},
    /* 3134 : VBLENDPS xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.66.0f3a 0c /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(0c) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VBLENDPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51637, 434},
    /* 3135 : VBLENDPS ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.66.0f3a 0c /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 005) 001(0c) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VBLENDPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51653, 434},
    /* 3136 : VBLENDVPD xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.nds.128.66.0f3a.w0 4b /r /is4 ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(4b) 120 177 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VBLENDVPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+51669, 442},
    /* 3137 : VBLENDVPD ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.nds.256.66.0f3a.w0 4b /r /is4 ] AVX,SANDYBRIDGE */
        /* 261(003 005) 001(4b) 120 177 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VBLENDVPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+51685, 442},
    /* 3138 : VBLENDVPS xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.nds.128.66.0f3a.w0 4a /r /is4 ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(4a) 120 177 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VBLENDVPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+51701, 442},
    /* 3139 : VBLENDVPS ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.nds.256.66.0f3a.w0 4a /r /is4 ] AVX,SANDYBRIDGE */
        /* 261(003 005) 001(4a) 120 177 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VBLENDVPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+51717, 442},
    /* 3140 : VBROADCASTSS xmmreg,mem32 [rm:       vex.128.66.0f38.w0 18 /r          ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(18) 110 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VBROADCASTSS, 2, {XMMREG|RN_L16,MEMORY|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58601, 442},
    /* 3141 : VBROADCASTSS ymmreg,mem32 [rm:       vex.256.66.0f38.w0 18 /r          ] AVX,SANDYBRIDGE */
        /* 270(002 005) 001(18) 110 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VBROADCASTSS, 2, {YMMREG|RN_L16,MEMORY|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58608, 442},
    /* 3142 : VBROADCASTSD ymmreg,mem64 [rm:       vex.256.66.0f38.w0 19 /r          ] AVX,SANDYBRIDGE */
        /* 270(002 005) 001(19) 110 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VBROADCASTSD, 2, {YMMREG|RN_L16,MEMORY|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58615, 442},
    /* 3143 : VBROADCASTF128 ymmreg,mem128 [rm:       vex.256.66.0f38.w0 1a /r          ] AVX,SANDYBRIDGE */
        /* 270(002 005) 001(1a) 110 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VBROADCASTF128, 2, {YMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58622, 442},
    /* 3144 : VCMPEQ_OSPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 10        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(10) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_OSPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26010, 434},
    /* 3145 : VCMPEQ_OSPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 10        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(10) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_OSPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26028, 434},
    /* 3146 : VCMPEQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 00        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(00) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26046, 434},
    /* 3147 : VCMPEQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 00        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(00) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26064, 434},
    /* 3148 : VCMPLT_OSPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 01        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(01) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLT_OSPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26082, 434},
    /* 3149 : VCMPLT_OSPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 01        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(01) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLT_OSPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26100, 434},
    /* 3150 : VCMPLTPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 01        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(01) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLTPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26082, 434},
    /* 3151 : VCMPLTPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 01        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(01) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLTPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26100, 434},
    /* 3152 : VCMPLE_OSPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 02        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(02) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLE_OSPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26118, 434},
    /* 3153 : VCMPLE_OSPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 02        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(02) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLE_OSPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26136, 434},
    /* 3154 : VCMPLEPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 02        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(02) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLEPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26118, 434},
    /* 3155 : VCMPLEPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 02        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(02) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLEPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26136, 434},
    /* 3156 : VCMPUNORD_QPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 03        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(03) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORD_QPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26154, 434},
    /* 3157 : VCMPUNORD_QPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 03        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(03) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORD_QPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26172, 434},
    /* 3158 : VCMPUNORDPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 03        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(03) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORDPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26154, 434},
    /* 3159 : VCMPUNORDPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 03        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(03) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORDPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26172, 434},
    /* 3160 : VCMPNEQ_UQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 04        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(04) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_UQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26190, 434},
    /* 3161 : VCMPNEQ_UQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 04        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(04) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_UQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26208, 434},
    /* 3162 : VCMPNEQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 04        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(04) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26190, 434},
    /* 3163 : VCMPNEQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 04        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(04) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26208, 434},
    /* 3164 : VCMPNLT_USPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 05        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(05) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLT_USPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26226, 434},
    /* 3165 : VCMPNLT_USPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 05        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(05) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLT_USPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26244, 434},
    /* 3166 : VCMPNLTPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 05        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(05) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLTPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26226, 434},
    /* 3167 : VCMPNLTPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 05        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(05) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLTPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26244, 434},
    /* 3168 : VCMPNLE_USPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 06        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(06) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLE_USPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26262, 434},
    /* 3169 : VCMPNLE_USPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 06        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(06) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLE_USPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26280, 434},
    /* 3170 : VCMPNLEPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 06        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(06) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLEPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26262, 434},
    /* 3171 : VCMPNLEPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 06        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(06) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLEPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26280, 434},
    /* 3172 : VCMPORD_QPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 07        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(07) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORD_QPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26298, 434},
    /* 3173 : VCMPORD_QPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 07        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(07) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORD_QPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26316, 434},
    /* 3174 : VCMPORDPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 07        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(07) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORDPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26298, 434},
    /* 3175 : VCMPORDPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 07        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(07) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORDPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26316, 434},
    /* 3176 : VCMPEQ_UQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 08        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(08) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_UQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26334, 434},
    /* 3177 : VCMPEQ_UQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 08        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(08) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_UQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26352, 434},
    /* 3178 : VCMPNGE_USPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 09        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(09) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGE_USPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26370, 434},
    /* 3179 : VCMPNGE_USPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 09        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(09) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGE_USPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26388, 434},
    /* 3180 : VCMPNGEPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 09        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(09) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGEPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26370, 434},
    /* 3181 : VCMPNGEPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 09        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(09) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGEPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26388, 434},
    /* 3182 : VCMPNGT_USPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 0a        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(0a) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGT_USPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26406, 434},
    /* 3183 : VCMPNGT_USPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 0a        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(0a) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGT_USPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26424, 434},
    /* 3184 : VCMPNGTPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 0a        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(0a) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGTPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26406, 434},
    /* 3185 : VCMPNGTPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 0a        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(0a) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGTPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26424, 434},
    /* 3186 : VCMPFALSE_OQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 0b        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(0b) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSE_OQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26442, 434},
    /* 3187 : VCMPFALSE_OQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 0b        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(0b) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSE_OQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26460, 434},
    /* 3188 : VCMPFALSEPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 0b        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(0b) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSEPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26442, 434},
    /* 3189 : VCMPFALSEPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 0b        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(0b) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSEPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26460, 434},
    /* 3190 : VCMPNEQ_OQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 0c        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(0c) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_OQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26478, 434},
    /* 3191 : VCMPNEQ_OQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 0c        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(0c) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_OQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26496, 434},
    /* 3192 : VCMPGE_OSPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 0d        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(0d) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGE_OSPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26514, 434},
    /* 3193 : VCMPGE_OSPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 0d        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(0d) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGE_OSPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26532, 434},
    /* 3194 : VCMPGEPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 0d        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(0d) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGEPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26514, 434},
    /* 3195 : VCMPGEPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 0d        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(0d) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGEPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26532, 434},
    /* 3196 : VCMPGT_OSPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 0e        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(0e) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGT_OSPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26550, 434},
    /* 3197 : VCMPGT_OSPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 0e        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(0e) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGT_OSPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26568, 434},
    /* 3198 : VCMPGTPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 0e        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(0e) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGTPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26550, 434},
    /* 3199 : VCMPGTPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 0e        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(0e) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGTPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26568, 434},
    /* 3200 : VCMPTRUE_UQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 0f        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(0f) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUE_UQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26586, 434},
    /* 3201 : VCMPTRUE_UQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 0f        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(0f) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUE_UQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26604, 434},
    /* 3202 : VCMPTRUEPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 0f        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(0f) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUEPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26586, 434},
    /* 3203 : VCMPTRUEPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 0f        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(0f) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUEPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26604, 434},
    /* 3204 : VCMPEQ_OSPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 10        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(10) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_OSPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26010, 434},
    /* 3205 : VCMPEQ_OSPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 10        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(10) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_OSPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26028, 434},
    /* 3206 : VCMPLT_OQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 11        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(11) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLT_OQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26622, 434},
    /* 3207 : VCMPLT_OQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 11        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(11) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLT_OQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26640, 434},
    /* 3208 : VCMPLE_OQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 12        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(12) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLE_OQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26658, 434},
    /* 3209 : VCMPLE_OQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 12        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(12) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLE_OQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26676, 434},
    /* 3210 : VCMPUNORD_SPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 13        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(13) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORD_SPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26694, 434},
    /* 3211 : VCMPUNORD_SPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 13        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(13) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORD_SPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26712, 434},
    /* 3212 : VCMPNEQ_USPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 14        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(14) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_USPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26730, 434},
    /* 3213 : VCMPNEQ_USPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 14        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(14) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_USPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26748, 434},
    /* 3214 : VCMPNLT_UQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 15        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(15) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLT_UQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26766, 434},
    /* 3215 : VCMPNLT_UQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 15        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(15) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLT_UQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26784, 434},
    /* 3216 : VCMPNLE_UQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 16        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(16) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLE_UQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26802, 434},
    /* 3217 : VCMPNLE_UQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 16        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(16) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLE_UQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26820, 434},
    /* 3218 : VCMPORD_SPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 17        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(17) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORD_SPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26838, 434},
    /* 3219 : VCMPORD_SPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 17        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(17) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORD_SPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26856, 434},
    /* 3220 : VCMPEQ_USPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 18        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(18) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_USPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26874, 434},
    /* 3221 : VCMPEQ_USPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 18        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(18) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_USPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26892, 434},
    /* 3222 : VCMPNGE_UQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 19        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(19) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGE_UQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26910, 434},
    /* 3223 : VCMPNGE_UQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 19        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(19) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGE_UQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26928, 434},
    /* 3224 : VCMPNGT_UQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 1a        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(1a) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGT_UQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26946, 434},
    /* 3225 : VCMPNGT_UQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 1a        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(1a) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGT_UQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26964, 434},
    /* 3226 : VCMPFALSE_OSPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 1b        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(1b) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSE_OSPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26982, 434},
    /* 3227 : VCMPFALSE_OSPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 1b        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(1b) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSE_OSPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27000, 434},
    /* 3228 : VCMPNEQ_OSPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 1c        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(1c) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_OSPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27018, 434},
    /* 3229 : VCMPNEQ_OSPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 1c        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(1c) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_OSPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27036, 434},
    /* 3230 : VCMPGE_OQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 1d        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(1d) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGE_OQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27054, 434},
    /* 3231 : VCMPGE_OQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 1d        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(1d) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGE_OQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27072, 434},
    /* 3232 : VCMPGT_OQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 1e        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(1e) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGT_OQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27090, 434},
    /* 3233 : VCMPGT_OQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 1e        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(1e) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGT_OQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27108, 434},
    /* 3234 : VCMPTRUE_USPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 1f        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(1f) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUE_USPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27126, 434},
    /* 3235 : VCMPTRUE_USPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 1f        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(1f) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUE_USPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27144, 434},
    /* 3236 : VCMPPD xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.66.0f c2 /r ib        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51733, 434},
    /* 3237 : VCMPPD ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.66.0f c2 /r ib        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51749, 434},
    /* 3238 : VCMPEQ_OSPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 10           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(10) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_OSPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27162, 434},
    /* 3239 : VCMPEQ_OSPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 10           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(10) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_OSPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27180, 434},
    /* 3240 : VCMPEQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 00           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(00) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27198, 434},
    /* 3241 : VCMPEQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 00           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(00) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27216, 434},
    /* 3242 : VCMPLT_OSPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 01           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(01) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLT_OSPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27234, 434},
    /* 3243 : VCMPLT_OSPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 01           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(01) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLT_OSPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27252, 434},
    /* 3244 : VCMPLTPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 01           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(01) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLTPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27234, 434},
    /* 3245 : VCMPLTPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 01           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(01) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLTPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27252, 434},
    /* 3246 : VCMPLE_OSPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 02           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(02) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLE_OSPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27270, 434},
    /* 3247 : VCMPLE_OSPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 02           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(02) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLE_OSPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27288, 434},
    /* 3248 : VCMPLEPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 02           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(02) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLEPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27270, 434},
    /* 3249 : VCMPLEPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 02           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(02) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLEPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27288, 434},
    /* 3250 : VCMPUNORD_QPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 03           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(03) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORD_QPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27306, 434},
    /* 3251 : VCMPUNORD_QPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 03           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(03) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORD_QPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27324, 434},
    /* 3252 : VCMPUNORDPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 03           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(03) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORDPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27306, 434},
    /* 3253 : VCMPUNORDPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 03           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(03) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORDPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27324, 434},
    /* 3254 : VCMPNEQ_UQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 04           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(04) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_UQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27342, 434},
    /* 3255 : VCMPNEQ_UQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 04           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(04) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_UQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27360, 434},
    /* 3256 : VCMPNEQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 04           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(04) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27342, 434},
    /* 3257 : VCMPNEQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 04           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(04) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27360, 434},
    /* 3258 : VCMPNLT_USPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 05           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(05) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLT_USPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27378, 434},
    /* 3259 : VCMPNLT_USPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 05           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(05) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLT_USPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27396, 434},
    /* 3260 : VCMPNLTPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 05           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(05) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLTPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27378, 434},
    /* 3261 : VCMPNLTPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 05           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(05) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLTPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27396, 434},
    /* 3262 : VCMPNLE_USPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 06           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(06) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLE_USPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27414, 434},
    /* 3263 : VCMPNLE_USPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 06           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(06) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLE_USPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27432, 434},
    /* 3264 : VCMPNLEPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 06           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(06) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLEPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27414, 434},
    /* 3265 : VCMPNLEPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 06           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(06) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLEPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27432, 434},
    /* 3266 : VCMPORD_QPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 07           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(07) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORD_QPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27450, 434},
    /* 3267 : VCMPORD_QPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 07           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(07) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORD_QPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27468, 434},
    /* 3268 : VCMPORDPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 07           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(07) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORDPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27450, 434},
    /* 3269 : VCMPORDPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 07           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(07) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORDPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27468, 434},
    /* 3270 : VCMPEQ_UQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 08           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(08) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_UQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27486, 434},
    /* 3271 : VCMPEQ_UQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 08           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(08) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_UQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27504, 434},
    /* 3272 : VCMPNGE_USPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 09           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(09) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGE_USPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27522, 434},
    /* 3273 : VCMPNGE_USPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 09           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(09) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGE_USPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27540, 434},
    /* 3274 : VCMPNGEPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 09           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(09) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGEPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27522, 434},
    /* 3275 : VCMPNGEPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 09           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(09) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGEPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27540, 434},
    /* 3276 : VCMPNGT_USPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 0a           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(0a) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGT_USPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27558, 434},
    /* 3277 : VCMPNGT_USPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 0a           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(0a) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGT_USPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27576, 434},
    /* 3278 : VCMPNGTPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 0a           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(0a) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGTPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27558, 434},
    /* 3279 : VCMPNGTPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 0a           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(0a) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGTPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27576, 434},
    /* 3280 : VCMPFALSE_OQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 0b           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(0b) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSE_OQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27594, 434},
    /* 3281 : VCMPFALSE_OQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 0b           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(0b) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSE_OQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27612, 434},
    /* 3282 : VCMPFALSEPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 0b           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(0b) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSEPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27594, 434},
    /* 3283 : VCMPFALSEPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 0b           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(0b) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSEPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27612, 434},
    /* 3284 : VCMPNEQ_OQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 0c           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(0c) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_OQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27630, 434},
    /* 3285 : VCMPNEQ_OQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 0c           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(0c) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_OQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27648, 434},
    /* 3286 : VCMPGE_OSPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 0d           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(0d) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGE_OSPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27666, 434},
    /* 3287 : VCMPGE_OSPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 0d           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(0d) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGE_OSPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27684, 434},
    /* 3288 : VCMPGEPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 0d           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(0d) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGEPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27666, 434},
    /* 3289 : VCMPGEPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 0d           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(0d) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGEPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27684, 434},
    /* 3290 : VCMPGT_OSPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 0e           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(0e) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGT_OSPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27702, 434},
    /* 3291 : VCMPGT_OSPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 0e           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(0e) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGT_OSPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27720, 434},
    /* 3292 : VCMPGTPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 0e           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(0e) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGTPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27702, 434},
    /* 3293 : VCMPGTPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 0e           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(0e) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGTPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27720, 434},
    /* 3294 : VCMPTRUE_UQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 0f           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(0f) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUE_UQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27738, 434},
    /* 3295 : VCMPTRUE_UQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 0f           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(0f) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUE_UQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27756, 434},
    /* 3296 : VCMPTRUEPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 0f           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(0f) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUEPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27738, 434},
    /* 3297 : VCMPTRUEPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 0f           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(0f) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUEPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27756, 434},
    /* 3298 : VCMPEQ_OSPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 10           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(10) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_OSPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27162, 434},
    /* 3299 : VCMPEQ_OSPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 10           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(10) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_OSPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27180, 434},
    /* 3300 : VCMPLT_OQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 11           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(11) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLT_OQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27774, 434},
    /* 3301 : VCMPLT_OQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 11           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(11) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLT_OQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27792, 434},
    /* 3302 : VCMPLE_OQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 12           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(12) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLE_OQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27810, 434},
    /* 3303 : VCMPLE_OQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 12           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(12) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLE_OQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27828, 434},
    /* 3304 : VCMPUNORD_SPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 13           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(13) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORD_SPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27846, 434},
    /* 3305 : VCMPUNORD_SPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 13           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(13) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORD_SPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27864, 434},
    /* 3306 : VCMPNEQ_USPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 14           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(14) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_USPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27882, 434},
    /* 3307 : VCMPNEQ_USPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 14           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(14) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_USPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27900, 434},
    /* 3308 : VCMPNLT_UQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 15           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(15) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLT_UQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27918, 434},
    /* 3309 : VCMPNLT_UQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 15           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(15) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLT_UQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27936, 434},
    /* 3310 : VCMPNLE_UQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 16           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(16) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLE_UQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27954, 434},
    /* 3311 : VCMPNLE_UQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 16           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(16) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLE_UQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27972, 434},
    /* 3312 : VCMPORD_SPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 17           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(17) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORD_SPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27990, 434},
    /* 3313 : VCMPORD_SPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 17           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(17) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORD_SPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28008, 434},
    /* 3314 : VCMPEQ_USPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 18           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(18) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_USPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28026, 434},
    /* 3315 : VCMPEQ_USPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 18           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(18) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_USPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28044, 434},
    /* 3316 : VCMPNGE_UQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 19           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(19) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGE_UQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28062, 434},
    /* 3317 : VCMPNGE_UQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 19           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(19) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGE_UQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28080, 434},
    /* 3318 : VCMPNGT_UQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 1a           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(1a) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGT_UQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28098, 434},
    /* 3319 : VCMPNGT_UQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 1a           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(1a) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGT_UQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28116, 434},
    /* 3320 : VCMPFALSE_OSPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 1b           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(1b) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSE_OSPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28134, 434},
    /* 3321 : VCMPFALSE_OSPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 1b           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(1b) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSE_OSPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28152, 434},
    /* 3322 : VCMPNEQ_OSPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 1c           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(1c) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_OSPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28170, 434},
    /* 3323 : VCMPNEQ_OSPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 1c           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(1c) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_OSPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28188, 434},
    /* 3324 : VCMPGE_OQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 1d           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(1d) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGE_OQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28206, 434},
    /* 3325 : VCMPGE_OQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 1d           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(1d) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGE_OQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28224, 434},
    /* 3326 : VCMPGT_OQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 1e           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(1e) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGT_OQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28242, 434},
    /* 3327 : VCMPGT_OQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 1e           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(1e) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGT_OQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28260, 434},
    /* 3328 : VCMPTRUE_USPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 1f           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(1f) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUE_USPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28278, 434},
    /* 3329 : VCMPTRUE_USPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 1f           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(1f) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUE_USPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28296, 434},
    /* 3330 : VCMPPS xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.0f c2 /r ib           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51765, 434},
    /* 3331 : VCMPPS ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.0f c2 /r ib           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51781, 434},
    /* 3332 : VCMPEQ_OSSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 10        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(10) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_OSSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28314, 441},
    /* 3333 : VCMPEQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 00        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(00) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28332, 441},
    /* 3334 : VCMPLT_OSSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 01        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(01) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLT_OSSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28350, 441},
    /* 3335 : VCMPLTSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 01        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(01) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLTSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28350, 441},
    /* 3336 : VCMPLE_OSSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 02        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(02) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLE_OSSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28368, 441},
    /* 3337 : VCMPLESD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 02        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(02) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLESD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28368, 441},
    /* 3338 : VCMPUNORD_QSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 03        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(03) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORD_QSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28386, 441},
    /* 3339 : VCMPUNORDSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 03        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(03) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORDSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28386, 441},
    /* 3340 : VCMPNEQ_UQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 04        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(04) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_UQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28404, 441},
    /* 3341 : VCMPNEQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 04        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(04) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28404, 441},
    /* 3342 : VCMPNLT_USSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 05        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(05) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLT_USSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28422, 441},
    /* 3343 : VCMPNLTSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 05        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(05) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLTSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28422, 441},
    /* 3344 : VCMPNLE_USSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 06        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(06) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLE_USSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28440, 441},
    /* 3345 : VCMPNLESD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 06        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(06) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLESD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28440, 441},
    /* 3346 : VCMPORD_QSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 07        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(07) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORD_QSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28458, 441},
    /* 3347 : VCMPORDSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 07        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(07) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORDSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28458, 441},
    /* 3348 : VCMPEQ_UQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 08        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(08) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_UQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28476, 441},
    /* 3349 : VCMPNGE_USSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 09        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(09) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGE_USSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28494, 441},
    /* 3350 : VCMPNGESD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 09        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(09) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGESD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28494, 441},
    /* 3351 : VCMPNGT_USSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 0a        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(0a) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGT_USSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28512, 441},
    /* 3352 : VCMPNGTSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 0a        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(0a) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGTSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28512, 441},
    /* 3353 : VCMPFALSE_OQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 0b        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(0b) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSE_OQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28530, 441},
    /* 3354 : VCMPFALSESD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 0b        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(0b) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSESD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28530, 441},
    /* 3355 : VCMPNEQ_OQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 0c        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(0c) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_OQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28548, 441},
    /* 3356 : VCMPGE_OSSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 0d        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(0d) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGE_OSSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28566, 441},
    /* 3357 : VCMPGESD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 0d        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(0d) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGESD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28566, 441},
    /* 3358 : VCMPGT_OSSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 0e        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(0e) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGT_OSSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28584, 441},
    /* 3359 : VCMPGTSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 0e        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(0e) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGTSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28584, 441},
    /* 3360 : VCMPTRUE_UQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 0f        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(0f) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUE_UQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28602, 441},
    /* 3361 : VCMPTRUESD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 0f        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(0f) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUESD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28602, 441},
    /* 3362 : VCMPEQ_OSSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 10        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(10) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_OSSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28314, 441},
    /* 3363 : VCMPLT_OQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 11        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(11) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLT_OQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28620, 441},
    /* 3364 : VCMPLE_OQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 12        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(12) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLE_OQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28638, 441},
    /* 3365 : VCMPUNORD_SSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 13        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(13) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORD_SSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28656, 441},
    /* 3366 : VCMPNEQ_USSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 14        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(14) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_USSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28674, 441},
    /* 3367 : VCMPNLT_UQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 15        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(15) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLT_UQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28692, 441},
    /* 3368 : VCMPNLE_UQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 16        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(16) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLE_UQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28710, 441},
    /* 3369 : VCMPORD_SSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 17        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(17) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORD_SSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28728, 441},
    /* 3370 : VCMPEQ_USSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 18        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(18) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_USSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28746, 441},
    /* 3371 : VCMPNGE_UQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 19        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(19) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGE_UQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28764, 441},
    /* 3372 : VCMPNGT_UQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 1a        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(1a) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGT_UQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28782, 441},
    /* 3373 : VCMPFALSE_OSSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 1b        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(1b) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSE_OSSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28800, 441},
    /* 3374 : VCMPNEQ_OSSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 1c        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(1c) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_OSSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28818, 441},
    /* 3375 : VCMPGE_OQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 1d        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(1d) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGE_OQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28836, 441},
    /* 3376 : VCMPGT_OQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 1e        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(1e) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGT_OQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28854, 441},
    /* 3377 : VCMPTRUE_USSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 1f        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(1f) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUE_USSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28872, 441},
    /* 3378 : VCMPSD xmmreg,xmmreg*,xmmrm64,imm8 [rvmi:     vex.nds.lig.f2.0f c2 /r ib        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 023 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPSD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51797, 441},
    /* 3379 : VCMPEQ_OSSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 10        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(10) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_OSSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28890, 441},
    /* 3380 : VCMPEQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 00        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(00) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28908, 441},
    /* 3381 : VCMPLT_OSSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 01        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(01) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLT_OSSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28926, 441},
    /* 3382 : VCMPLTSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 01        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(01) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLTSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28926, 441},
    /* 3383 : VCMPLE_OSSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 02        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(02) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLE_OSSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28944, 441},
    /* 3384 : VCMPLESS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 02        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(02) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLESS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28944, 441},
    /* 3385 : VCMPUNORD_QSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 03        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(03) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORD_QSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28962, 441},
    /* 3386 : VCMPUNORDSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 03        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(03) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORDSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28962, 441},
    /* 3387 : VCMPNEQ_UQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 04        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(04) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_UQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28980, 441},
    /* 3388 : VCMPNEQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 04        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(04) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28980, 441},
    /* 3389 : VCMPNLT_USSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 05        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(05) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLT_USSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28998, 441},
    /* 3390 : VCMPNLTSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 05        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(05) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLTSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28998, 441},
    /* 3391 : VCMPNLE_USSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 06        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(06) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLE_USSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29016, 441},
    /* 3392 : VCMPNLESS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 06        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(06) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLESS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29016, 441},
    /* 3393 : VCMPORD_QSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 07        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(07) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORD_QSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29034, 441},
    /* 3394 : VCMPORDSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 07        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(07) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORDSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29034, 441},
    /* 3395 : VCMPEQ_UQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 08        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(08) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_UQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29052, 441},
    /* 3396 : VCMPNGE_USSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 09        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(09) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGE_USSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29070, 441},
    /* 3397 : VCMPNGESS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 09        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(09) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGESS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29070, 441},
    /* 3398 : VCMPNGT_USSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 0a        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(0a) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGT_USSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29088, 441},
    /* 3399 : VCMPNGTSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 0a        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(0a) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGTSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29088, 441},
    /* 3400 : VCMPFALSE_OQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 0b        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(0b) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSE_OQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29106, 441},
    /* 3401 : VCMPFALSESS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 0b        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(0b) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSESS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29106, 441},
    /* 3402 : VCMPNEQ_OQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 0c        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(0c) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_OQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29124, 441},
    /* 3403 : VCMPGE_OSSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 0d        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(0d) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGE_OSSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29142, 441},
    /* 3404 : VCMPGESS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 0d        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(0d) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGESS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29142, 441},
    /* 3405 : VCMPGT_OSSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 0e        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(0e) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGT_OSSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29160, 441},
    /* 3406 : VCMPGTSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 0e        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(0e) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGTSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29160, 441},
    /* 3407 : VCMPTRUE_UQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 0f        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(0f) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUE_UQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29178, 441},
    /* 3408 : VCMPTRUESS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 0f        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(0f) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUESS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29178, 441},
    /* 3409 : VCMPEQ_OSSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 10        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(10) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_OSSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28890, 441},
    /* 3410 : VCMPLT_OQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 11        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(11) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLT_OQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29196, 441},
    /* 3411 : VCMPLE_OQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 12        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(12) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLE_OQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29214, 441},
    /* 3412 : VCMPUNORD_SSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 13        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(13) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORD_SSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29232, 441},
    /* 3413 : VCMPNEQ_USSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 14        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(14) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_USSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29250, 441},
    /* 3414 : VCMPNLT_UQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 15        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(15) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLT_UQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29268, 441},
    /* 3415 : VCMPNLE_UQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 16        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(16) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLE_UQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29286, 441},
    /* 3416 : VCMPORD_SSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 17        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(17) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORD_SSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29304, 441},
    /* 3417 : VCMPEQ_USSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 18        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(18) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_USSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29322, 441},
    /* 3418 : VCMPNGE_UQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 19        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(19) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGE_UQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29340, 441},
    /* 3419 : VCMPNGT_UQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 1a        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(1a) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGT_UQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29358, 441},
    /* 3420 : VCMPFALSE_OSSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 1b        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(1b) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSE_OSSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29376, 441},
    /* 3421 : VCMPNEQ_OSSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 1c        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(1c) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_OSSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29394, 441},
    /* 3422 : VCMPGE_OQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 1d        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(1d) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGE_OQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29412, 441},
    /* 3423 : VCMPGT_OQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 1e        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(1e) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGT_OQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29430, 441},
    /* 3424 : VCMPTRUE_USSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 1f        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(1f) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUE_USSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29448, 441},
    /* 3425 : VCMPSS xmmreg,xmmreg*,xmmrm64,imm8 [rvmi:     vex.nds.lig.f3.0f c2 /r ib        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 023 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPSS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51813, 441},
    /* 3426 : VCOMISD xmmreg,xmmrm64 [rm:       vex.lig.66.0f 2f /r               ] FL,AVX,SANDYBRIDGE */
        /* 270(001 001) 001(2f) 110 : NOAPX,LIG,WIG,FL,VEX,AVX,SANDYBRIDGE */
        {I_VCOMISD, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58629, 443},
    /* 3427 : VCOMISS xmmreg,xmmrm32 [rm:       vex.lig.0f 2f /r                  ] FL,AVX,SANDYBRIDGE */
        /* 270(001 000) 001(2f) 110 : NOAPX,LIG,WIG,FL,VEX,AVX,SANDYBRIDGE */
        {I_VCOMISS, 2, {XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58636, 443},
    /* 3428 : VCVTDQ2PD xmmreg,xmmrm64 [rm:       vex.128.f3.0f e6 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 002) 001(e6) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTDQ2PD, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58643, 434},
    /* 3429 : VCVTDQ2PD ymmreg,xmmrm128 [rm:       vex.256.f3.0f e6 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 006) 001(e6) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTDQ2PD, 2, {YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58650, 434},
    /* 3430 : VCVTDQ2PS xmmreg,xmmrm128 [rm:       vex.128.0f 5b /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(5b) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTDQ2PS, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58657, 434},
    /* 3431 : VCVTDQ2PS ymmreg,ymmrm256 [rm:       vex.256.0f 5b /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 004) 001(5b) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTDQ2PS, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58664, 434},
    /* 3432 : VCVTPD2DQ xmmreg,xmmreg [rm:       vex.128.f2.0f e6 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 003) 001(e6) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTPD2DQ, 2, {XMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58671, 434},
    /* 3433 : VCVTPD2DQ xmmreg,mem128 [rm:       vex.128.f2.0f e6 /r               ] AR0-1,SO,AVX,SANDYBRIDGE */
        /* 270(001 003) 001(e6) 110 : AR0,AR1,SO,NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTPD2DQ, 2, {XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58671, 444},
    /* 3434 : VCVTPD2DQ xmmreg,ymmreg [rm:       vex.256.f2.0f e6 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 007) 001(e6) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTPD2DQ, 2, {XMMREG|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58678, 434},
    /* 3435 : VCVTPD2DQ xmmreg,mem256 [rm:       vex.256.f2.0f e6 /r               ] AR0-1,SY,AVX,SANDYBRIDGE */
        /* 270(001 007) 001(e6) 110 : AR0,AR1,SY,NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTPD2DQ, 2, {XMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58678, 445},
    /* 3436 : VCVTPD2PS xmmreg,xmmreg [rm:       vex.128.66.0f 5a /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(5a) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTPD2PS, 2, {XMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58685, 434},
    /* 3437 : VCVTPD2PS xmmreg,mem128 [rm:       vex.128.66.0f 5a /r               ] AR0-1,SO,AVX,SANDYBRIDGE */
        /* 270(001 001) 001(5a) 110 : AR0,AR1,SO,NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTPD2PS, 2, {XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58685, 444},
    /* 3438 : VCVTPD2PS xmmreg,ymmreg [rm:       vex.256.66.0f 5a /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(5a) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTPD2PS, 2, {XMMREG|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58692, 434},
    /* 3439 : VCVTPD2PS xmmreg,mem256 [rm:       vex.256.66.0f 5a /r               ] AR0-1,SY,AVX,SANDYBRIDGE */
        /* 270(001 005) 001(5a) 110 : AR0,AR1,SY,NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTPD2PS, 2, {XMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58692, 445},
    /* 3440 : VCVTPS2DQ xmmreg,xmmrm128 [rm:       vex.128.66.0f 5b /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(5b) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTPS2DQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58699, 434},
    /* 3441 : VCVTPS2DQ ymmreg,ymmrm256 [rm:       vex.256.66.0f 5b /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(5b) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTPS2DQ, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58706, 434},
    /* 3442 : VCVTPS2PD xmmreg,xmmrm64 [rm:       vex.128.0f 5a /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(5a) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTPS2PD, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58713, 434},
    /* 3443 : VCVTPS2PD ymmreg,xmmrm128 [rm:       vex.256.0f 5a /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 004) 001(5a) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTPS2PD, 2, {YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58720, 434},
    /* 3444 : VCVTSD2SI reg32,xmmrm64 [rm:       vex.lig.f2.0f.w0 2d /r            ] AVX,SANDYBRIDGE */
        /* 270(001 003) 001(2d) 110 : NOAPX,LIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTSD2SI, 2, {REG_GPR|BITS32|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58727, 446},
    /* 3445 : VCVTSD2SI reg64,xmmrm64 [rm:       vex.lig.f2.0f.w1 2d /r            ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(001 203) 001(2d) 110 : NOAPX,LIG,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VCVTSD2SI, 2, {REG_GPR|BITS64|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58734, 447},
    /* 3446 : VCVTSD2SS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f 5a /r           ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(5a) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTSD2SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58741, 441},
    /* 3447 : VCVTSI2SD xmmreg,xmmreg*,rm32 [rvm:      vex.nds.lig.f2.0f.w0 2a /r        ] AR0,AR1,AR2,SD,AVX,SANDYBRIDGE */
        /* 261(001 003) 001(2a) 120 : AR0,AR1,AR2,SD,NOAPX,LIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTSI2SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58755, 448},
    /* 3448 : VCVTSI2SD xmmreg,xmmreg*,rm64 [rvm:      vex.nds.lig.f2.0f.w1 2a /r        ] AR0,AR1,AR2,SQ,LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 261(001 203) 001(2a) 120 : AR0,AR1,AR2,SQ,NOAPX,LIG,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VCVTSI2SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58769, 450},
    /* 3449 : VCVTSI2SS xmmreg,xmmreg*,rm32 [rvm:      vex.nds.lig.f3.0f.w0 2a /r        ] AR0,AR1,AR2,SD,AVX,SANDYBRIDGE */
        /* 261(001 002) 001(2a) 120 : AR0,AR1,AR2,SD,NOAPX,LIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTSI2SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58783, 448},
    /* 3450 : VCVTSI2SS xmmreg,xmmreg*,rm64 [rvm:      vex.nds.lig.f3.0f.w1 2a /r        ] AR0,AR1,AR2,SQ,LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 261(001 202) 001(2a) 120 : AR0,AR1,AR2,SQ,NOAPX,LIG,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VCVTSI2SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58797, 450},
    /* 3451 : VCVTSS2SD xmmreg,xmmreg*,xmmrm32 [rvm:      vex.nds.lig.f3.0f 5a /r           ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(5a) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTSS2SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58811, 441},
    /* 3452 : VCVTSS2SI reg32,xmmrm32 [rm:       vex.lig.f3.0f.w0 2d /r            ] AVX,SANDYBRIDGE */
        /* 270(001 002) 001(2d) 110 : NOAPX,LIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTSS2SI, 2, {REG_GPR|BITS32|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58825, 446},
    /* 3453 : VCVTSS2SI reg64,xmmrm32 [rm:       vex.lig.f3.0f.w1 2d /r            ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(001 202) 001(2d) 110 : NOAPX,LIG,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VCVTSS2SI, 2, {REG_GPR|BITS64|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58832, 447},
    /* 3454 : VCVTTPD2DQ xmmreg,xmmreg [rm:       vex.128.66.0f e6 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(e6) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTTPD2DQ, 2, {XMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58839, 434},
    /* 3455 : VCVTTPD2DQ xmmreg,mem128 [rm:       vex.128.66.0f e6 /r               ] AR0-1,SO,AVX,SANDYBRIDGE */
        /* 270(001 001) 001(e6) 110 : AR0,AR1,SO,NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTTPD2DQ, 2, {XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58839, 444},
    /* 3456 : VCVTTPD2DQ xmmreg,ymmreg [rm:       vex.256.66.0f e6 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(e6) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTTPD2DQ, 2, {XMMREG|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58846, 434},
    /* 3457 : VCVTTPD2DQ xmmreg,mem256 [rm:       vex.256.66.0f e6 /r               ] AR0-1,SY,AVX,SANDYBRIDGE */
        /* 270(001 005) 001(e6) 110 : AR0,AR1,SY,NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTTPD2DQ, 2, {XMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58846, 445},
    /* 3458 : VCVTTPS2DQ xmmreg,xmmrm128 [rm:       vex.128.f3.0f 5b /r               ] AVX,SANDYBRIDGE */
        /* 270(001 002) 001(5b) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTTPS2DQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58853, 434},
    /* 3459 : VCVTTPS2DQ ymmreg,ymmrm256 [rm:       vex.256.f3.0f 5b /r               ] AVX,SANDYBRIDGE */
        /* 270(001 006) 001(5b) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTTPS2DQ, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58860, 434},
    /* 3460 : VCVTTSD2SI reg32,xmmrm64 [rm:       vex.lig.f2.0f.w0 2c /r            ] AVX,SANDYBRIDGE */
        /* 270(001 003) 001(2c) 110 : NOAPX,LIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTTSD2SI, 2, {REG_GPR|BITS32|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58867, 446},
    /* 3461 : VCVTTSD2SI reg64,xmmrm64 [rm:       vex.lig.f2.0f.w1 2c /r            ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(001 203) 001(2c) 110 : NOAPX,LIG,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VCVTTSD2SI, 2, {REG_GPR|BITS64|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58874, 447},
    /* 3462 : VCVTTSS2SI reg32,xmmrm32 [rm:       vex.lig.f3.0f.w0 2c /r            ] AVX,SANDYBRIDGE */
        /* 270(001 002) 001(2c) 110 : NOAPX,LIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTTSS2SI, 2, {REG_GPR|BITS32|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58881, 446},
    /* 3463 : VCVTTSS2SI reg64,xmmrm32 [rm:       vex.lig.f3.0f.w1 2c /r            ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(001 202) 001(2c) 110 : NOAPX,LIG,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VCVTTSS2SI, 2, {REG_GPR|BITS64|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58888, 447},
    /* 3464 : VDIVPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 5e /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(5e) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VDIVPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58895, 434},
    /* 3465 : VDIVPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 5e /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(5e) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VDIVPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58909, 434},
    /* 3466 : VDIVPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f 5e /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(5e) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VDIVPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58923, 434},
    /* 3467 : VDIVPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f 5e /r              ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(5e) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VDIVPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58937, 434},
    /* 3468 : VDIVSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f 5e /r           ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(5e) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VDIVSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58951, 441},
    /* 3469 : VDIVSS xmmreg,xmmreg*,xmmrm32 [rvm:      vex.nds.lig.f3.0f 5e /r           ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(5e) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VDIVSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58965, 441},
    /* 3470 : VDPPD xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.66.0f3a 41 /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(41) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VDPPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51829, 434},
    /* 3471 : VDPPS xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.66.0f3a 40 /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(40) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VDPPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51845, 434},
    /* 3472 : VDPPS ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.66.0f3a 40 /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 005) 001(40) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VDPPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51861, 434},
    /* 3473 : VEXTRACTF128 xmmrm128,ymmreg,imm8 [mri:      vex.256.66.0f3a.w0 19 /r ib       ] AVX,SANDYBRIDGE */
        /* 270(003 005) 001(19) 101 022 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VEXTRACTF128, 3, {RM_XMM|BITS128|RN_L16,YMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51877, 442},
    /* 3474 : VEXTRACTPS rm32,xmmreg,imm8 [mri:      vex.128.66.0f3a 17 /r ib          ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(17) 101 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VEXTRACTPS, 3, {RM_GPR|BITS32|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51885, 434},
    /* 3475 : VHADDPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 7c /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(7c) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VHADDPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58979, 434},
    /* 3476 : VHADDPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 7c /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(7c) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VHADDPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58993, 434},
    /* 3477 : VHADDPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.f2.0f 7c /r           ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(7c) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VHADDPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59007, 434},
    /* 3478 : VHADDPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.f2.0f 7c /r           ] AVX,SANDYBRIDGE */
        /* 261(001 007) 001(7c) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VHADDPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59021, 434},
    /* 3479 : VHSUBPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 7d /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(7d) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VHSUBPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59035, 434},
    /* 3480 : VHSUBPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 7d /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(7d) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VHSUBPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59049, 434},
    /* 3481 : VHSUBPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.f2.0f 7d /r           ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(7d) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VHSUBPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59063, 434},
    /* 3482 : VHSUBPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.f2.0f 7d /r           ] AVX,SANDYBRIDGE */
        /* 261(001 007) 001(7d) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VHSUBPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59077, 434},
    /* 3483 : VINSERTF128 ymmreg,ymmreg*,xmmrm128,imm8 [rvmi:     vex.nds.256.66.0f3a.w0 18 /r ib   ] AVX,SANDYBRIDGE */
        /* 261(003 005) 001(18) 120 023 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VINSERTF128, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51893, 442},
    /* 3484 : VINSERTPS xmmreg,xmmreg*,xmmrm32,imm8 [rvmi:     vex.nds.128.66.0f3a 21 /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(21) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VINSERTPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51909, 434},
    /* 3485 : VLDDQU xmmreg,mem128 [rm:       vex.128.f2.0f f0 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 003) 001(f0) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VLDDQU, 2, {XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59091, 434},
    /* 3486 : VLDQQU ymmreg,mem256 [rm:       vex.256.f2.0f f0 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 007) 001(f0) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VLDQQU, 2, {YMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59098, 434},
    /* 3487 : VLDDQU ymmreg,mem256 [rm:       vex.256.f2.0f f0 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 007) 001(f0) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VLDDQU, 2, {YMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59098, 434},
    /* 3488 : VLDMXCSR mem32 [m:        vex.lz.0f ae /2                   ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(ae) 202 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VLDMXCSR, 1, {MEMORY|BITS32|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+59105, 434},
    /* 3489 : VMASKMOVDQU xmmreg,xmmreg [rm:       vex.128.66.0f f7 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(f7) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMASKMOVDQU, 2, {XMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59112, 434},
    /* 3490 : VMASKMOVPS xmmreg,xmmreg,mem128 [rvm:      vex.nds.128.66.0f38.w0 2c /r      ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(2c) 120 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VMASKMOVPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59119, 442},
    /* 3491 : VMASKMOVPS ymmreg,ymmreg,mem256 [rvm:      vex.nds.256.66.0f38.w0 2c /r      ] AVX,SANDYBRIDGE */
        /* 261(002 005) 001(2c) 120 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VMASKMOVPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59126, 442},
    /* 3492 : VMASKMOVPS mem128,xmmreg,xmmreg [mvr:      vex.nds.128.66.0f38.w0 2e /r      ] AR0-2,SO,AVX,SANDYBRIDGE */
        /* 261(002 001) 001(2e) 102 : AR0,AR1,AR2,SO,NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VMASKMOVPS, 3, {MEMORY|BITS128|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59133, 452},
    /* 3493 : VMASKMOVPS mem256,ymmreg,ymmreg [mvr:      vex.nds.256.66.0f38.w0 2e /r      ] AR0-2,SY,AVX,SANDYBRIDGE */
        /* 261(002 005) 001(2e) 102 : AR0,AR1,AR2,SY,NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VMASKMOVPS, 3, {MEMORY|BITS256|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59140, 453},
    /* 3494 : VMASKMOVPD xmmreg,xmmreg,mem128 [rvm:      vex.nds.128.66.0f38.w0 2d /r      ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(2d) 120 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VMASKMOVPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59147, 442},
    /* 3495 : VMASKMOVPD ymmreg,ymmreg,mem256 [rvm:      vex.nds.256.66.0f38.w0 2d /r      ] AVX,SANDYBRIDGE */
        /* 261(002 005) 001(2d) 120 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VMASKMOVPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59154, 442},
    /* 3496 : VMASKMOVPD mem128,xmmreg,xmmreg [mvr:      vex.nds.128.66.0f38.w0 2f /r      ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(2f) 102 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VMASKMOVPD, 3, {MEMORY|BITS128|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59161, 442},
    /* 3497 : VMASKMOVPD mem256,ymmreg,ymmreg [mvr:      vex.nds.256.66.0f38.w0 2f /r      ] AVX,SANDYBRIDGE */
        /* 261(002 005) 001(2f) 102 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VMASKMOVPD, 3, {MEMORY|BITS256|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59168, 442},
    /* 3498 : VMAXPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 5f /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(5f) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMAXPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59175, 434},
    /* 3499 : VMAXPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 5f /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(5f) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMAXPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59189, 434},
    /* 3500 : VMAXPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f 5f /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(5f) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMAXPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59203, 434},
    /* 3501 : VMAXPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f 5f /r              ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(5f) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMAXPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59217, 434},
    /* 3502 : VMAXSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f 5f /r           ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(5f) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMAXSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59231, 441},
    /* 3503 : VMAXSS xmmreg,xmmreg*,xmmrm32 [rvm:      vex.nds.lig.f3.0f 5f /r           ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(5f) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMAXSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59245, 441},
    /* 3504 : VMINPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 5d /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(5d) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMINPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59259, 434},
    /* 3505 : VMINPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 5d /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(5d) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMINPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59273, 434},
    /* 3506 : VMINPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f 5d /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(5d) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMINPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59287, 434},
    /* 3507 : VMINPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f 5d /r              ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(5d) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMINPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59301, 434},
    /* 3508 : VMINSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f 5d /r           ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(5d) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMINSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59315, 441},
    /* 3509 : VMINSS xmmreg,xmmreg*,xmmrm32 [rvm:      vex.nds.lig.f3.0f 5d /r           ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(5d) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMINSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59329, 441},
    /* 3510 : VMOVAPD xmmreg,xmmrm128 [rm:       vex.128.66.0f 28 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(28) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVAPD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59343, 434},
    /* 3511 : VMOVAPD xmmrm128,xmmreg [mr:       vex.128.66.0f 29 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(29) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVAPD, 2, {RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59350, 434},
    /* 3512 : VMOVAPD ymmreg,ymmrm256 [rm:       vex.256.66.0f 28 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(28) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVAPD, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59357, 434},
    /* 3513 : VMOVAPD ymmrm256,ymmreg [mr:       vex.256.66.0f 29 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(29) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVAPD, 2, {RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59364, 434},
    /* 3514 : VMOVAPS xmmreg,xmmrm128 [rm:       vex.128.0f 28 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(28) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVAPS, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59371, 434},
    /* 3515 : VMOVAPS xmmrm128,xmmreg [mr:       vex.128.0f 29 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(29) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVAPS, 2, {RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59378, 434},
    /* 3516 : VMOVAPS ymmreg,ymmrm256 [rm:       vex.256.0f 28 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 004) 001(28) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVAPS, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59385, 434},
    /* 3517 : VMOVAPS ymmrm256,ymmreg [mr:       vex.256.0f 29 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 004) 001(29) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVAPS, 2, {RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59392, 434},
    /* 3518 : VMOVD xmmreg,rm32 [rm:       vex.128.66.0f.w0 6e /r            ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(6e) 110 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VMOVD, 2, {XMMREG|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59399, 442},
    /* 3519 : VMOVD rm32,xmmreg [mr:       vex.128.66.0f.w0 7e /r            ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(7e) 101 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VMOVD, 2, {RM_GPR|BITS32|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59406, 442},
    /* 3520 : VMOVQ xmmreg,xmmrm64 [rm:       vex.128.f3.0f 7e /r               ] AR0-1,SQ,AVX,SANDYBRIDGE */
        /* 270(001 002) 001(7e) 110 : AR0,AR1,SQ,NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVQ, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59413, 454},
    /* 3521 : VMOVQ xmmrm64,xmmreg [mr:       vex.128.66.0f d6 /r               ] AR0-1,SQ,AVX,SANDYBRIDGE */
        /* 270(001 001) 001(d6) 101 : AR0,AR1,SQ,NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVQ, 2, {RM_XMM|BITS64|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59420, 454},
    /* 3522 : VMOVQ xmmreg,rm64 [rm:       vex.128.66.0f.w1 6e /r            ] AR0-1,SQ,LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(001 201) 001(6e) 110 : AR0,AR1,SQ,NOAPX,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VMOVQ, 2, {XMMREG|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59427, 455},
    /* 3523 : VMOVQ rm64,xmmreg [mr:       vex.128.66.0f.w1 7e /r            ] AR0-1,SQ,LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(001 201) 001(7e) 101 : AR0,AR1,SQ,NOAPX,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VMOVQ, 2, {RM_GPR|BITS64|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59434, 455},
    /* 3524 : VMOVDDUP xmmreg,xmmrm64 [rm:       vex.128.f2.0f 12 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 003) 001(12) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVDDUP, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59441, 434},
    /* 3525 : VMOVDDUP ymmreg,ymmrm256 [rm:       vex.256.f2.0f 12 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 007) 001(12) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVDDUP, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59448, 434},
    /* 3526 : VMOVDQA xmmreg,xmmrm128 [rm:       vex.128.66.0f 6f /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(6f) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVDQA, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59455, 434},
    /* 3527 : VMOVDQA xmmrm128,xmmreg [mr:       vex.128.66.0f 7f /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(7f) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVDQA, 2, {RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59462, 434},
    /* 3528 : VMOVQQA ymmreg,ymmrm256 [rm:       vex.256.66.0f 6f /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(6f) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVQQA, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59469, 434},
    /* 3529 : VMOVQQA ymmrm256,ymmreg [mr:       vex.256.66.0f 7f /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(7f) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVQQA, 2, {RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59476, 434},
    /* 3530 : VMOVDQA ymmreg,ymmrm256 [rm:       vex.256.66.0f 6f /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(6f) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVDQA, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59469, 434},
    /* 3531 : VMOVDQA ymmrm256,ymmreg [mr:       vex.256.66.0f 7f /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(7f) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVDQA, 2, {RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59476, 434},
    /* 3532 : VMOVDQU xmmreg,xmmrm128 [rm:       vex.128.f3.0f 6f /r               ] AVX,SANDYBRIDGE */
        /* 270(001 002) 001(6f) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVDQU, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59483, 434},
    /* 3533 : VMOVDQU xmmrm128,xmmreg [mr:       vex.128.f3.0f 7f /r               ] AVX,SANDYBRIDGE */
        /* 270(001 002) 001(7f) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVDQU, 2, {RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59490, 434},
    /* 3534 : VMOVQQU ymmreg,ymmrm256 [rm:       vex.256.f3.0f 6f /r               ] AVX,SANDYBRIDGE */
        /* 270(001 006) 001(6f) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVQQU, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59497, 434},
    /* 3535 : VMOVQQU ymmrm256,ymmreg [mr:       vex.256.f3.0f 7f /r               ] AVX,SANDYBRIDGE */
        /* 270(001 006) 001(7f) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVQQU, 2, {RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59504, 434},
    /* 3536 : VMOVDQU ymmreg,ymmrm256 [rm:       vex.256.f3.0f 6f /r               ] AVX,SANDYBRIDGE */
        /* 270(001 006) 001(6f) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVDQU, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59497, 434},
    /* 3537 : VMOVDQU ymmrm256,ymmreg [mr:       vex.256.f3.0f 7f /r               ] AVX,SANDYBRIDGE */
        /* 270(001 006) 001(7f) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVDQU, 2, {RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59504, 434},
    /* 3538 : VMOVHLPS xmmreg,xmmreg*,xmmreg [rvm:      vex.nds.128.0f 12 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(12) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVHLPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59511, 434},
    /* 3539 : VMOVHPD xmmreg,xmmreg*,mem64 [rvm:      vex.nds.128.66.0f 16 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(16) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVHPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,MEMORY|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59525, 434},
    /* 3540 : VMOVHPD mem64,xmmreg [mr:       vex.128.66.0f 17 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(17) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVHPD, 2, {MEMORY|BITS64|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59539, 434},
    /* 3541 : VMOVHPS xmmreg,xmmreg*,mem64 [rvm:      vex.nds.128.0f 16 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(16) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVHPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,MEMORY|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59546, 434},
    /* 3542 : VMOVHPS mem64,xmmreg [mr:       vex.128.0f 17 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(17) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVHPS, 2, {MEMORY|BITS64|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59560, 434},
    /* 3543 : VMOVLHPS xmmreg,xmmreg*,xmmreg [rvm:      vex.nds.128.0f 16 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(16) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVLHPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59546, 434},
    /* 3544 : VMOVLPD xmmreg,xmmreg*,mem64 [rvm:      vex.nds.128.66.0f 12 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(12) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVLPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,MEMORY|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59567, 434},
    /* 3545 : VMOVLPD mem64,xmmreg [mr:       vex.128.66.0f 13 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(13) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVLPD, 2, {MEMORY|BITS64|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59581, 434},
    /* 3546 : VMOVLPS xmmreg,xmmreg*,mem64 [rvm:      vex.nds.128.0f 12 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(12) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVLPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,MEMORY|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59511, 434},
    /* 3547 : VMOVLPS mem64,xmmreg [mr:       vex.128.0f 13 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(13) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVLPS, 2, {MEMORY|BITS64|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59588, 434},
    /* 3548 : VMOVMSKPD reg64,xmmreg [rm:       vex.128.66.0f 50 /r               ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(001 001) 001(50) 110 : NOAPX,WIG,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VMOVMSKPD, 2, {REG_GPR|BITS64|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59595, 456},
    /* 3549 : VMOVMSKPD reg32,xmmreg [rm:       vex.128.66.0f 50 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(50) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVMSKPD, 2, {REG_GPR|BITS32|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59595, 434},
    /* 3550 : VMOVMSKPD reg64,ymmreg [rm:       vex.256.66.0f 50 /r               ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(001 005) 001(50) 110 : NOAPX,WIG,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VMOVMSKPD, 2, {REG_GPR|BITS64|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59602, 456},
    /* 3551 : VMOVMSKPD reg32,ymmreg [rm:       vex.256.66.0f 50 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(50) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVMSKPD, 2, {REG_GPR|BITS32|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59602, 434},
    /* 3552 : VMOVMSKPS reg64,xmmreg [rm:       vex.128.0f 50 /r                  ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(001 000) 001(50) 110 : NOAPX,WIG,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VMOVMSKPS, 2, {REG_GPR|BITS64|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59609, 456},
    /* 3553 : VMOVMSKPS reg32,xmmreg [rm:       vex.128.0f 50 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(50) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVMSKPS, 2, {REG_GPR|BITS32|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59609, 434},
    /* 3554 : VMOVMSKPS reg64,ymmreg [rm:       vex.256.0f 50 /r                  ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(001 004) 001(50) 110 : NOAPX,WIG,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VMOVMSKPS, 2, {REG_GPR|BITS64|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59616, 456},
    /* 3555 : VMOVMSKPS reg32,ymmreg [rm:       vex.256.0f 50 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 004) 001(50) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVMSKPS, 2, {REG_GPR|BITS32|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59616, 434},
    /* 3556 : VMOVNTDQ mem128,xmmreg [mr:       vex.128.66.0f e7 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(e7) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVNTDQ, 2, {MEMORY|BITS128|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59623, 434},
    /* 3557 : VMOVNTQQ mem256,ymmreg [mr:       vex.256.66.0f e7 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(e7) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVNTQQ, 2, {MEMORY|BITS256|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59630, 434},
    /* 3558 : VMOVNTDQ mem256,ymmreg [mr:       vex.256.66.0f e7 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(e7) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVNTDQ, 2, {MEMORY|BITS256|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59630, 434},
    /* 3559 : VMOVNTDQA xmmreg,mem128 [rm:       vex.128.66.0f38 2a /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(2a) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVNTDQA, 2, {XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59637, 434},
    /* 3560 : VMOVNTPD mem128,xmmreg [mr:       vex.128.66.0f 2b /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(2b) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVNTPD, 2, {MEMORY|BITS128|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59644, 434},
    /* 3561 : VMOVNTPD mem256,ymmreg [mr:       vex.256.66.0f 2b /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(2b) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVNTPD, 2, {MEMORY|BITS256|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59651, 434},
    /* 3562 : VMOVNTPS mem128,xmmreg [mr:       vex.128.0f 2b /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(2b) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVNTPS, 2, {MEMORY|BITS128|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59658, 434},
    /* 3563 : VMOVNTPS mem256,ymmreg [mr:       vex.256.0f 2b /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 004) 001(2b) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVNTPS, 2, {MEMORY|BITS256|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59665, 434},
    /* 3564 : VMOVSD xmmreg,xmmreg*,xmmreg [rvm:      vex.nds.lig.f2.0f 10 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(10) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59672, 441},
    /* 3565 : VMOVSD xmmreg,mem64 [rm:       vex.lig.f2.0f 10 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 003) 001(10) 110 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVSD, 2, {XMMREG|RN_L16,MEMORY|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59686, 441},
    /* 3566 : VMOVSD xmmreg,xmmreg*,xmmreg [mvr:      vex.nds.lig.f2.0f 11 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(11) 102 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59693, 441},
    /* 3567 : VMOVSD mem64,xmmreg [mr:       vex.lig.f2.0f 11 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 003) 001(11) 101 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVSD, 2, {MEMORY|BITS64|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59707, 441},
    /* 3568 : VMOVSHDUP xmmreg,xmmrm128 [rm:       vex.128.f3.0f 16 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 002) 001(16) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVSHDUP, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59714, 434},
    /* 3569 : VMOVSHDUP ymmreg,ymmrm256 [rm:       vex.256.f3.0f 16 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 006) 001(16) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVSHDUP, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59721, 434},
    /* 3570 : VMOVSLDUP xmmreg,xmmrm128 [rm:       vex.128.f3.0f 12 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 002) 001(12) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVSLDUP, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59728, 434},
    /* 3571 : VMOVSLDUP ymmreg,ymmrm256 [rm:       vex.256.f3.0f 12 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 006) 001(12) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVSLDUP, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59735, 434},
    /* 3572 : VMOVSS xmmreg,xmmreg*,xmmreg [rvm:      vex.nds.lig.f3.0f 10 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(10) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59742, 441},
    /* 3573 : VMOVSS xmmreg,mem32 [rm:       vex.lig.f3.0f 10 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 002) 001(10) 110 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVSS, 2, {XMMREG|RN_L16,MEMORY|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59756, 441},
    /* 3574 : VMOVSS xmmreg,xmmreg*,xmmreg [mvr:      vex.nds.lig.f3.0f 11 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(11) 102 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59763, 441},
    /* 3575 : VMOVSS mem32,xmmreg [mr:       vex.lig.f3.0f 11 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 002) 001(11) 101 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVSS, 2, {MEMORY|BITS32|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59777, 441},
    /* 3576 : VMOVUPD xmmreg,xmmrm128 [rm:       vex.128.66.0f 10 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(10) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVUPD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59784, 434},
    /* 3577 : VMOVUPD xmmrm128,xmmreg [mr:       vex.128.66.0f 11 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(11) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVUPD, 2, {RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59791, 434},
    /* 3578 : VMOVUPD ymmreg,ymmrm256 [rm:       vex.256.66.0f 10 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(10) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVUPD, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59798, 434},
    /* 3579 : VMOVUPD ymmrm256,ymmreg [mr:       vex.256.66.0f 11 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(11) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVUPD, 2, {RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59805, 434},
    /* 3580 : VMOVUPS xmmreg,xmmrm128 [rm:       vex.128.0f 10 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(10) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVUPS, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59812, 434},
    /* 3581 : VMOVUPS xmmrm128,xmmreg [mr:       vex.128.0f 11 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(11) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVUPS, 2, {RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59819, 434},
    /* 3582 : VMOVUPS ymmreg,ymmrm256 [rm:       vex.256.0f 10 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 004) 001(10) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVUPS, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59826, 434},
    /* 3583 : VMOVUPS ymmrm256,ymmreg [mr:       vex.256.0f 11 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 004) 001(11) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVUPS, 2, {RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59833, 434},
    /* 3584 : VMPSADBW xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.66.0f3a 42 /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(42) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMPSADBW, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51925, 434},
    /* 3585 : VMULPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 59 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(59) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMULPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59840, 434},
    /* 3586 : VMULPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 59 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(59) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMULPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59854, 434},
    /* 3587 : VMULPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f 59 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(59) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMULPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59868, 434},
    /* 3588 : VMULPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f 59 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(59) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMULPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59882, 434},
    /* 3589 : VMULSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f 59 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(59) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMULSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59896, 441},
    /* 3590 : VMULSS xmmreg,xmmreg*,xmmrm32 [rvm:      vex.nds.lig.f3.0f 59 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(59) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMULSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59910, 441},
    /* 3591 : VORPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 56 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(56) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VORPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59924, 434},
    /* 3592 : VORPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 56 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(56) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VORPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59938, 434},
    /* 3593 : VORPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f 56 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(56) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VORPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59952, 434},
    /* 3594 : VORPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f 56 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(56) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VORPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59966, 434},
    /* 3595 : VPABSB xmmreg,xmmrm128 [rm:       vex.128.66.0f38 1c /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(1c) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPABSB, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59980, 434},
    /* 3596 : VPABSW xmmreg,xmmrm128 [rm:       vex.128.66.0f38 1d /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(1d) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPABSW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59987, 434},
    /* 3597 : VPABSD xmmreg,xmmrm128 [rm:       vex.128.66.0f38 1e /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(1e) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPABSD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59994, 434},
    /* 3598 : VPACKSSWB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 63 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(63) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPACKSSWB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60001, 434},
    /* 3599 : VPACKSSDW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 6b /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(6b) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPACKSSDW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60015, 434},
    /* 3600 : VPACKUSWB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 67 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(67) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPACKUSWB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60029, 434},
    /* 3601 : VPACKUSDW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 2b /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(2b) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPACKUSDW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60043, 434},
    /* 3602 : VPADDB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f fc /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(fc) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPADDB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60057, 434},
    /* 3603 : VPADDW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f fd /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(fd) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPADDW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60071, 434},
    /* 3604 : VPADDD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f fe /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(fe) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPADDD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60085, 434},
    /* 3605 : VPADDQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f d4 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(d4) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPADDQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60099, 434},
    /* 3606 : VPADDSB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f ec /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(ec) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPADDSB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60113, 434},
    /* 3607 : VPADDSW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f ed /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(ed) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPADDSW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60127, 434},
    /* 3608 : VPADDUSB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f dc /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(dc) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPADDUSB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60141, 434},
    /* 3609 : VPADDUSW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f dd /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(dd) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPADDUSW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60155, 434},
    /* 3610 : VPALIGNR xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.66.0f3a 0f /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(0f) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPALIGNR, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51941, 434},
    /* 3611 : VPAND xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f db /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(db) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPAND, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60169, 434},
    /* 3612 : VPANDN xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f df /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(df) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPANDN, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60183, 434},
    /* 3613 : VPAVGB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f e0 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(e0) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPAVGB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60197, 434},
    /* 3614 : VPAVGW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f e3 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(e3) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPAVGW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60211, 434},
    /* 3615 : VPBLENDVB xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.nds.128.66.0f3a.w0 4c /r /is4 ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(4c) 120 177 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPBLENDVB, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+51957, 442},
    /* 3616 : VPBLENDW xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.66.0f3a 0e /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(0e) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPBLENDW, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51973, 434},
    /* 3617 : VPCMPESTRI xmmreg,xmmrm128,imm8 [rmi:      vex.128.66.0f3a 61 /r ib          ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(61) 110 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCMPESTRI, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51989, 434},
    /* 3618 : VPCMPESTRM xmmreg,xmmrm128,imm8 [rmi:      vex.128.66.0f3a 60 /r ib          ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(60) 110 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCMPESTRM, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51997, 434},
    /* 3619 : VPCMPISTRI xmmreg,xmmrm128,imm8 [rmi:      vex.128.66.0f3a 63 /r ib          ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(63) 110 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCMPISTRI, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52005, 434},
    /* 3620 : VPCMPISTRM xmmreg,xmmrm128,imm8 [rmi:      vex.128.66.0f3a 62 /r ib          ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(62) 110 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCMPISTRM, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52013, 434},
    /* 3621 : VPCMPEQB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 74 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(74) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCMPEQB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60225, 434},
    /* 3622 : VPCMPEQW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 75 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(75) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCMPEQW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60239, 434},
    /* 3623 : VPCMPEQD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 76 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(76) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCMPEQD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60253, 434},
    /* 3624 : VPCMPEQQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 29 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(29) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCMPEQQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60267, 434},
    /* 3625 : VPCMPGTB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 64 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(64) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCMPGTB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60281, 434},
    /* 3626 : VPCMPGTW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 65 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(65) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCMPGTW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60295, 434},
    /* 3627 : VPCMPGTD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 66 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(66) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCMPGTD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60309, 434},
    /* 3628 : VPCMPGTQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 37 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(37) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCMPGTQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60323, 434},
    /* 3629 : VPERMILPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38.w0 0d /r      ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(0d) 120 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPERMILPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60337, 442},
    /* 3630 : VPERMILPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.w0 0d /r      ] AVX,SANDYBRIDGE */
        /* 261(002 005) 001(0d) 120 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPERMILPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60351, 442},
    /* 3631 : VPERMILPD xmmreg,xmmrm128,imm8 [rmi:      vex.128.66.0f3a.w0 05 /r ib       ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(05) 110 022 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPERMILPD, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52021, 442},
    /* 3632 : VPERMILPD ymmreg,ymmrm256,imm8 [rmi:      vex.256.66.0f3a.w0 05 /r ib       ] AVX,SANDYBRIDGE */
        /* 270(003 005) 001(05) 110 022 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPERMILPD, 3, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52029, 442},
    /* 3633 : VPERMILPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38.w0 0c /r      ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(0c) 120 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPERMILPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60365, 442},
    /* 3634 : VPERMILPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.w0 0c /r      ] AVX,SANDYBRIDGE */
        /* 261(002 005) 001(0c) 120 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPERMILPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60379, 442},
    /* 3635 : VPERMILPS xmmreg,xmmrm128,imm8 [rmi:      vex.128.66.0f3a.w0 04 /r ib       ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(04) 110 022 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPERMILPS, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52037, 442},
    /* 3636 : VPERMILPS ymmreg,ymmrm256,imm8 [rmi:      vex.256.66.0f3a.w0 04 /r ib       ] AVX,SANDYBRIDGE */
        /* 270(003 005) 001(04) 110 022 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPERMILPS, 3, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52045, 442},
    /* 3637 : VPERM2F128 ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.66.0f3a.w0 06 /r ib   ] AVX,SANDYBRIDGE */
        /* 261(003 005) 001(06) 120 023 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPERM2F128, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52053, 442},
    /* 3638 : VPEXTRB reg64,xmmreg,imm8 [mri:      vex.128.66.0f3a.w0 14 /r ib       ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(003 001) 001(14) 101 022 : NOAPX,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VPEXTRB, 3, {REG_GPR|BITS64|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52069, 457},
    /* 3639 : VPEXTRB reg32,xmmreg,imm8 [mri:      vex.128.66.0f3a.w0 14 /r ib       ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(14) 101 022 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPEXTRB, 3, {REG_GPR|BITS32|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52069, 442},
    /* 3640 : VPEXTRB mem8,xmmreg,imm8 [mri:      vex.128.66.0f3a.w0 14 /r ib       ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(14) 101 022 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPEXTRB, 3, {MEMORY|BITS8|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52069, 442},
    /* 3641 : VPEXTRW reg64,xmmreg,imm8 [rmi:      vex.128.66.0f.w0 c5 /r ib         ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(001 001) 001(c5) 110 022 : NOAPX,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VPEXTRW, 3, {REG_GPR|BITS64|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52077, 457},
    /* 3642 : VPEXTRW reg32,xmmreg,imm8 [rmi:      vex.128.66.0f.w0 c5 /r ib         ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(c5) 110 022 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPEXTRW, 3, {REG_GPR|BITS32|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52077, 442},
    /* 3643 : VPEXTRW reg64,xmmreg,imm8 [mri:      vex.128.66.0f3a.w0 15 /r ib       ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(003 001) 001(15) 101 022 : NOAPX,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VPEXTRW, 3, {REG_GPR|BITS64|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52085, 457},
    /* 3644 : VPEXTRW reg32,xmmreg,imm8 [mri:      vex.128.66.0f3a.w0 15 /r ib       ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(15) 101 022 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPEXTRW, 3, {REG_GPR|BITS32|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52085, 442},
    /* 3645 : VPEXTRW mem16,xmmreg,imm8 [mri:      vex.128.66.0f3a.w0 15 /r ib       ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(15) 101 022 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPEXTRW, 3, {MEMORY|BITS16|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52085, 442},
    /* 3646 : VPEXTRD reg64,xmmreg,imm8 [mri:      vex.128.66.0f3a.w0 16 /r ib       ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(003 001) 001(16) 101 022 : NOAPX,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VPEXTRD, 3, {REG_GPR|BITS64|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52093, 457},
    /* 3647 : VPEXTRD rm32,xmmreg,imm8 [mri:      vex.128.66.0f3a.w0 16 /r ib       ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(16) 101 022 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPEXTRD, 3, {RM_GPR|BITS32|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52093, 442},
    /* 3648 : VPEXTRQ rm64,xmmreg,imm8 [mri:      vex.128.66.0f3a.w1 16 /r ib       ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(003 201) 001(16) 101 022 : NOAPX,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VPEXTRQ, 3, {RM_GPR|BITS64|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52101, 457},
    /* 3649 : VPHADDW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 01 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(01) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPHADDW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60393, 434},
    /* 3650 : VPHADDD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 02 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(02) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPHADDD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60407, 434},
    /* 3651 : VPHADDSW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 03 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(03) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPHADDSW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60421, 434},
    /* 3652 : VPHMINPOSUW xmmreg,xmmrm128 [rm:       vex.128.66.0f38 41 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(41) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPHMINPOSUW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60435, 434},
    /* 3653 : VPHSUBW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 05 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(05) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPHSUBW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60442, 434},
    /* 3654 : VPHSUBD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 06 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(06) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPHSUBD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60456, 434},
    /* 3655 : VPHSUBSW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 07 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(07) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPHSUBSW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60470, 434},
    /* 3656 : VPINSRB xmmreg,xmmreg*,mem8,imm8 [rvmi:     vex.nds.128.66.0f3a 20 /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(20) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPINSRB, 4, {XMMREG|RN_L16,XMMREG|RN_L16,MEMORY|BITS8|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52109, 434},
    /* 3657 : VPINSRB xmmreg,xmmreg*,rm8,imm8 [rvmi:     vex.nds.128.66.0f3a 20 /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(20) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPINSRB, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_GPR|BITS8|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52109, 434},
    /* 3658 : VPINSRB xmmreg,xmmreg*,reg32,imm8 [rvmi:     vex.nds.128.66.0f3a 20 /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(20) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPINSRB, 4, {XMMREG|RN_L16,XMMREG|RN_L16,REG_GPR|BITS32|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52109, 434},
    /* 3659 : VPINSRW xmmreg,xmmreg*,mem16,imm8 [rvmi:     vex.nds.128.66.0f c4 /r ib        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c4) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPINSRW, 4, {XMMREG|RN_L16,XMMREG|RN_L16,MEMORY|BITS16|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52125, 434},
    /* 3660 : VPINSRW xmmreg,xmmreg*,rm16,imm8 [rvmi:     vex.nds.128.66.0f c4 /r ib        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c4) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPINSRW, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_GPR|BITS16|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52125, 434},
    /* 3661 : VPINSRW xmmreg,xmmreg*,reg32,imm8 [rvmi:     vex.nds.128.66.0f c4 /r ib        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c4) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPINSRW, 4, {XMMREG|RN_L16,XMMREG|RN_L16,REG_GPR|BITS32|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52125, 434},
    /* 3662 : VPINSRD xmmreg,xmmreg*,mem32,imm8 [rvmi:     vex.nds.128.66.0f3a.w0 22 /r ib   ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(22) 120 023 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPINSRD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,MEMORY|BITS32|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52141, 442},
    /* 3663 : VPINSRD xmmreg,xmmreg*,rm32,imm8 [rvmi:     vex.nds.128.66.0f3a.w0 22 /r ib   ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(22) 120 023 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPINSRD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_GPR|BITS32|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52141, 442},
    /* 3664 : VPINSRQ xmmreg,xmmreg*,mem64,imm8 [rvmi:     vex.nds.128.66.0f3a.w1 22 /r ib   ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 261(003 201) 001(22) 120 023 : NOAPX,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VPINSRQ, 4, {XMMREG|RN_L16,XMMREG|RN_L16,MEMORY|BITS64|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52157, 457},
    /* 3665 : VPINSRQ xmmreg,xmmreg*,rm64,imm8 [rvmi:     vex.nds.128.66.0f3a.w1 22 /r ib   ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 261(003 201) 001(22) 120 023 : NOAPX,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VPINSRQ, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_GPR|BITS64|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52157, 457},
    /* 3666 : VPMADDWD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f f5 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(f5) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMADDWD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60484, 434},
    /* 3667 : VPMADDUBSW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 04 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(04) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMADDUBSW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60498, 434},
    /* 3668 : VPMAXSB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 3c /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(3c) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMAXSB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60512, 434},
    /* 3669 : VPMAXSW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f ee /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(ee) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMAXSW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60526, 434},
    /* 3670 : VPMAXSD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 3d /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(3d) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMAXSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60540, 434},
    /* 3671 : VPMAXUB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f de /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(de) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMAXUB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60554, 434},
    /* 3672 : VPMAXUW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 3e /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(3e) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMAXUW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60568, 434},
    /* 3673 : VPMAXUD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 3f /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(3f) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMAXUD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60582, 434},
    /* 3674 : VPMINSB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 38 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(38) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMINSB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60596, 434},
    /* 3675 : VPMINSW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f ea /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(ea) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMINSW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60610, 434},
    /* 3676 : VPMINSD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 39 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(39) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMINSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60624, 434},
    /* 3677 : VPMINUB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f da /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(da) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMINUB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60638, 434},
    /* 3678 : VPMINUW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 3a /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(3a) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMINUW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60652, 434},
    /* 3679 : VPMINUD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 3b /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(3b) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMINUD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60666, 434},
    /* 3680 : VPMOVMSKB reg64,xmmreg [rm:       vex.128.66.0f d7 /r               ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(001 001) 001(d7) 110 : NOAPX,WIG,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VPMOVMSKB, 2, {REG_GPR|BITS64|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60680, 456},
    /* 3681 : VPMOVMSKB reg32,xmmreg [rm:       vex.128.66.0f d7 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(d7) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMOVMSKB, 2, {REG_GPR|BITS32|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60680, 434},
    /* 3682 : VPMOVSXBW xmmreg,xmmrm64 [rm:       vex.128.66.0f38 20 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(20) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMOVSXBW, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60687, 434},
    /* 3683 : VPMOVSXBD xmmreg,xmmrm32 [rm:       vex.128.66.0f38 21 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(21) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMOVSXBD, 2, {XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60694, 434},
    /* 3684 : VPMOVSXBQ xmmreg,xmmrm16 [rm:       vex.128.66.0f38 22 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(22) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMOVSXBQ, 2, {XMMREG|RN_L16,RM_XMM|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60701, 434},
    /* 3685 : VPMOVSXWD xmmreg,xmmrm64 [rm:       vex.128.66.0f38 23 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(23) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMOVSXWD, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60708, 434},
    /* 3686 : VPMOVSXWQ xmmreg,xmmrm32 [rm:       vex.128.66.0f38 24 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(24) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMOVSXWQ, 2, {XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60715, 434},
    /* 3687 : VPMOVSXDQ xmmreg,xmmrm64 [rm:       vex.128.66.0f38 25 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(25) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMOVSXDQ, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60722, 434},
    /* 3688 : VPMOVZXBW xmmreg,xmmrm64 [rm:       vex.128.66.0f38 30 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(30) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMOVZXBW, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60729, 434},
    /* 3689 : VPMOVZXBD xmmreg,xmmrm32 [rm:       vex.128.66.0f38 31 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(31) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMOVZXBD, 2, {XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60736, 434},
    /* 3690 : VPMOVZXBQ xmmreg,xmmrm16 [rm:       vex.128.66.0f38 32 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(32) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMOVZXBQ, 2, {XMMREG|RN_L16,RM_XMM|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60743, 434},
    /* 3691 : VPMOVZXWD xmmreg,xmmrm64 [rm:       vex.128.66.0f38 33 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(33) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMOVZXWD, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60750, 434},
    /* 3692 : VPMOVZXWQ xmmreg,xmmrm32 [rm:       vex.128.66.0f38 34 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(34) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMOVZXWQ, 2, {XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60757, 434},
    /* 3693 : VPMOVZXDQ xmmreg,xmmrm64 [rm:       vex.128.66.0f38 35 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(35) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMOVZXDQ, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60764, 434},
    /* 3694 : VPMULHUW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f e4 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(e4) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMULHUW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60771, 434},
    /* 3695 : VPMULHRSW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 0b /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(0b) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMULHRSW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60785, 434},
    /* 3696 : VPMULHW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f e5 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(e5) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMULHW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60799, 434},
    /* 3697 : VPMULLW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f d5 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(d5) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMULLW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60813, 434},
    /* 3698 : VPMULLD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 40 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(40) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMULLD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60827, 434},
    /* 3699 : VPMULUDQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f f4 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(f4) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMULUDQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60841, 434},
    /* 3700 : VPMULDQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 28 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(28) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMULDQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60855, 434},
    /* 3701 : VPOR xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f eb /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(eb) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPOR, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60869, 434},
    /* 3702 : VPSADBW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f f6 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(f6) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSADBW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60883, 434},
    /* 3703 : VPSHUFB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 00 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(00) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSHUFB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60897, 434},
    /* 3704 : VPSHUFD xmmreg,xmmrm128,imm8 [rmi:      vex.128.66.0f 70 /r ib            ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(70) 110 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSHUFD, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52173, 434},
    /* 3705 : VPSHUFHW xmmreg,xmmrm128,imm8 [rmi:      vex.128.f3.0f 70 /r ib            ] AVX,SANDYBRIDGE */
        /* 270(001 002) 001(70) 110 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSHUFHW, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52181, 434},
    /* 3706 : VPSHUFLW xmmreg,xmmrm128,imm8 [rmi:      vex.128.f2.0f 70 /r ib            ] AVX,SANDYBRIDGE */
        /* 270(001 003) 001(70) 110 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSHUFLW, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52189, 434},
    /* 3707 : VPSIGNB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 08 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(08) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSIGNB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60911, 434},
    /* 3708 : VPSIGNW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 09 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(09) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSIGNW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60925, 434},
    /* 3709 : VPSIGND xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 0a /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(0a) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSIGND, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60939, 434},
    /* 3710 : VPSLLDQ xmmreg,xmmreg*,imm8 [vmi:      vex.ndd.128.66.0f 73 /7 ib        ] AVX,SANDYBRIDGE */
        /* 260(001 001) 001(73) 217 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSLLDQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52197, 434},
    /* 3711 : VPSRLDQ xmmreg,xmmreg*,imm8 [vmi:      vex.ndd.128.66.0f 73 /3 ib        ] AVX,SANDYBRIDGE */
        /* 260(001 001) 001(73) 213 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSRLDQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52213, 434},
    /* 3712 : VPSLLW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f f1 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(f1) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSLLW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60953, 434},
    /* 3713 : VPSLLW xmmreg,xmmreg*,imm8 [vmi:      vex.ndd.128.66.0f 71 /6 ib        ] AVX,SANDYBRIDGE */
        /* 260(001 001) 001(71) 216 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSLLW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52229, 434},
    /* 3714 : VPSLLD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f f2 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(f2) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSLLD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60967, 434},
    /* 3715 : VPSLLD xmmreg,xmmreg*,imm8 [vmi:      vex.ndd.128.66.0f 72 /6 ib        ] AVX,SANDYBRIDGE */
        /* 260(001 001) 001(72) 216 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSLLD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52245, 434},
    /* 3716 : VPSLLQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f f3 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(f3) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSLLQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60981, 434},
    /* 3717 : VPSLLQ xmmreg,xmmreg*,imm8 [vmi:      vex.ndd.128.66.0f 73 /6 ib        ] AVX,SANDYBRIDGE */
        /* 260(001 001) 001(73) 216 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSLLQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52261, 434},
    /* 3718 : VPSRAW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f e1 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(e1) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSRAW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60995, 434},
    /* 3719 : VPSRAW xmmreg,xmmreg*,imm8 [vmi:      vex.ndd.128.66.0f 71 /4 ib        ] AVX,SANDYBRIDGE */
        /* 260(001 001) 001(71) 214 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSRAW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52277, 434},
    /* 3720 : VPSRAD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f e2 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(e2) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSRAD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61009, 434},
    /* 3721 : VPSRAD xmmreg,xmmreg*,imm8 [vmi:      vex.ndd.128.66.0f 72 /4 ib        ] AVX,SANDYBRIDGE */
        /* 260(001 001) 001(72) 214 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSRAD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52293, 434},
    /* 3722 : VPSRLW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f d1 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(d1) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSRLW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61023, 434},
    /* 3723 : VPSRLW xmmreg,xmmreg*,imm8 [vmi:      vex.ndd.128.66.0f 71 /2 ib        ] AVX,SANDYBRIDGE */
        /* 260(001 001) 001(71) 212 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSRLW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52309, 434},
    /* 3724 : VPSRLD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f d2 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(d2) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSRLD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61037, 434},
    /* 3725 : VPSRLD xmmreg,xmmreg*,imm8 [vmi:      vex.ndd.128.66.0f 72 /2 ib        ] AVX,SANDYBRIDGE */
        /* 260(001 001) 001(72) 212 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSRLD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52325, 434},
    /* 3726 : VPSRLQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f d3 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(d3) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSRLQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61051, 434},
    /* 3727 : VPSRLQ xmmreg,xmmreg*,imm8 [vmi:      vex.ndd.128.66.0f 73 /2 ib        ] AVX,SANDYBRIDGE */
        /* 260(001 001) 001(73) 212 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSRLQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52341, 434},
    /* 3728 : VPTEST xmmreg,xmmrm128 [rm:       vex.128.66.0f38 17 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(17) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPTEST, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61065, 434},
    /* 3729 : VPTEST ymmreg,ymmrm256 [rm:       vex.256.66.0f38 17 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 005) 001(17) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPTEST, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61072, 434},
    /* 3730 : VPSUBB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f f8 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(f8) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSUBB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61079, 434},
    /* 3731 : VPSUBW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f f9 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(f9) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSUBW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61093, 434},
    /* 3732 : VPSUBD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f fa /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(fa) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSUBD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61107, 434},
    /* 3733 : VPSUBQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f fb /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(fb) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSUBQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61121, 434},
    /* 3734 : VPSUBSB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f e8 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(e8) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSUBSB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61135, 434},
    /* 3735 : VPSUBSW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f e9 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(e9) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSUBSW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61149, 434},
    /* 3736 : VPSUBUSB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f d8 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(d8) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSUBUSB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61163, 434},
    /* 3737 : VPSUBUSW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f d9 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(d9) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSUBUSW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61177, 434},
    /* 3738 : VPUNPCKHBW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 68 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(68) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPUNPCKHBW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61191, 434},
    /* 3739 : VPUNPCKHWD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 69 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(69) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPUNPCKHWD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61205, 434},
    /* 3740 : VPUNPCKHDQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 6a /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(6a) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPUNPCKHDQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61219, 434},
    /* 3741 : VPUNPCKHQDQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 6d /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(6d) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPUNPCKHQDQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61233, 434},
    /* 3742 : VPUNPCKLBW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 60 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(60) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPUNPCKLBW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61247, 434},
    /* 3743 : VPUNPCKLWD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 61 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(61) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPUNPCKLWD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61261, 434},
    /* 3744 : VPUNPCKLDQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 62 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(62) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPUNPCKLDQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61275, 434},
    /* 3745 : VPUNPCKLQDQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 6c /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(6c) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPUNPCKLQDQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61289, 434},
    /* 3746 : VPXOR xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f ef /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(ef) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPXOR, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61303, 434},
    /* 3747 : VRCPPS xmmreg,xmmrm128 [rm:       vex.128.0f 53 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(53) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VRCPPS, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61317, 434},
    /* 3748 : VRCPPS ymmreg,ymmrm256 [rm:       vex.256.0f 53 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 004) 001(53) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VRCPPS, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61324, 434},
    /* 3749 : VRCPSS xmmreg,xmmreg*,xmmrm32 [rvm:      vex.nds.lig.f3.0f 53 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(53) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VRCPSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61331, 441},
    /* 3750 : VRSQRTPS xmmreg,xmmrm128 [rm:       vex.128.0f 52 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(52) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VRSQRTPS, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61345, 434},
    /* 3751 : VRSQRTPS ymmreg,ymmrm256 [rm:       vex.256.0f 52 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 004) 001(52) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VRSQRTPS, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61352, 434},
    /* 3752 : VRSQRTSS xmmreg,xmmreg*,xmmrm32 [rvm:      vex.nds.lig.f3.0f 52 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(52) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VRSQRTSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61359, 441},
    /* 3753 : VROUNDPD xmmreg,xmmrm128,imm8 [rmi:      vex.128.66.0f3a 09 /r ib          ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(09) 110 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VROUNDPD, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52357, 434},
    /* 3754 : VROUNDPD ymmreg,ymmrm256,imm8 [rmi:      vex.256.66.0f3a 09 /r ib          ] AVX,SANDYBRIDGE */
        /* 270(003 005) 001(09) 110 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VROUNDPD, 3, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52365, 434},
    /* 3755 : VROUNDPS xmmreg,xmmrm128,imm8 [rmi:      vex.128.66.0f3a 08 /r ib          ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(08) 110 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VROUNDPS, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52373, 434},
    /* 3756 : VROUNDPS ymmreg,ymmrm256,imm8 [rmi:      vex.256.66.0f3a 08 /r ib          ] AVX,SANDYBRIDGE */
        /* 270(003 005) 001(08) 110 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VROUNDPS, 3, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52381, 434},
    /* 3757 : VROUNDSD xmmreg,xmmreg*,xmmrm64,imm8 [rvmi:     vex.nds.128.66.0f3a 0b /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(0b) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VROUNDSD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52389, 434},
    /* 3758 : VROUNDSS xmmreg,xmmreg*,xmmrm32,imm8 [rvmi:     vex.nds.128.66.0f3a 0a /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(0a) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VROUNDSS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52405, 434},
    /* 3759 : VSHUFPD xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.66.0f c6 /r ib        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c6) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSHUFPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52421, 434},
    /* 3760 : VSHUFPD ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.66.0f c6 /r ib        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c6) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSHUFPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52437, 434},
    /* 3761 : VSHUFPS xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.0f c6 /r ib           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c6) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSHUFPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52453, 434},
    /* 3762 : VSHUFPS ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.0f c6 /r ib           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c6) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSHUFPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52469, 434},
    /* 3763 : VSQRTPD xmmreg,xmmrm128 [rm:       vex.128.66.0f 51 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(51) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSQRTPD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61373, 434},
    /* 3764 : VSQRTPD ymmreg,ymmrm256 [rm:       vex.256.66.0f 51 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(51) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSQRTPD, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61380, 434},
    /* 3765 : VSQRTPS xmmreg,xmmrm128 [rm:       vex.128.0f 51 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(51) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSQRTPS, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61387, 434},
    /* 3766 : VSQRTPS ymmreg,ymmrm256 [rm:       vex.256.0f 51 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 004) 001(51) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSQRTPS, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61394, 434},
    /* 3767 : VSQRTSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f 51 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(51) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSQRTSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61401, 441},
    /* 3768 : VSQRTSS xmmreg,xmmreg*,xmmrm32 [rvm:      vex.nds.lig.f3.0f 51 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(51) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSQRTSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61415, 441},
    /* 3769 : VSTMXCSR mem32 [m:        vex.128.0f ae /3                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(ae) 203 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSTMXCSR, 1, {MEMORY|BITS32|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+61429, 434},
    /* 3770 : VSUBPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 5c /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(5c) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSUBPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61436, 434},
    /* 3771 : VSUBPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 5c /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(5c) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSUBPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61450, 434},
    /* 3772 : VSUBPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f 5c /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(5c) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSUBPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61464, 434},
    /* 3773 : VSUBPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f 5c /r              ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(5c) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSUBPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61478, 434},
    /* 3774 : VSUBSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f 5c /r           ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(5c) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSUBSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61492, 441},
    /* 3775 : VSUBSS xmmreg,xmmreg*,xmmrm32 [rvm:      vex.nds.lig.f3.0f 5c /r           ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(5c) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSUBSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61506, 441},
    /* 3776 : VTESTPS xmmreg,xmmrm128 [rm:       vex.128.66.0f38.w0 0e /r          ] FL,AVX,SANDYBRIDGE */
        /* 270(002 001) 001(0e) 110 : NOAPX,FL,VEX,AVX,SANDYBRIDGE */
        {I_VTESTPS, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61520, 458},
    /* 3777 : VTESTPS ymmreg,ymmrm256 [rm:       vex.256.66.0f38.w0 0e /r          ] FL,AVX,SANDYBRIDGE */
        /* 270(002 005) 001(0e) 110 : NOAPX,FL,VEX,AVX,SANDYBRIDGE */
        {I_VTESTPS, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61527, 458},
    /* 3778 : VTESTPD xmmreg,xmmrm128 [rm:       vex.128.66.0f38.w0 0f /r          ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(0f) 110 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VTESTPD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61534, 442},
    /* 3779 : VTESTPD ymmreg,ymmrm256 [rm:       vex.256.66.0f38.w0 0f /r          ] AVX,SANDYBRIDGE */
        /* 270(002 005) 001(0f) 110 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VTESTPD, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61541, 442},
    /* 3780 : VUCOMISD xmmreg,xmmrm64 [rm:       vex.lig.66.0f 2e /r               ] FL,AVX,SANDYBRIDGE */
        /* 270(001 001) 001(2e) 110 : NOAPX,LIG,WIG,FL,VEX,AVX,SANDYBRIDGE */
        {I_VUCOMISD, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61548, 443},
    /* 3781 : VUCOMISS xmmreg,xmmrm32 [rm:       vex.lig.0f 2e /r                  ] FL,AVX,SANDYBRIDGE */
        /* 270(001 000) 001(2e) 110 : NOAPX,LIG,WIG,FL,VEX,AVX,SANDYBRIDGE */
        {I_VUCOMISS, 2, {XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61555, 443},
    /* 3782 : VUNPCKHPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 15 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(15) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VUNPCKHPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61562, 434},
    /* 3783 : VUNPCKHPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 15 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(15) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VUNPCKHPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61576, 434},
    /* 3784 : VUNPCKHPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f 15 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(15) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VUNPCKHPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61590, 434},
    /* 3785 : VUNPCKHPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f 15 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(15) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VUNPCKHPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61604, 434},
    /* 3786 : VUNPCKLPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 14 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(14) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VUNPCKLPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61618, 434},
    /* 3787 : VUNPCKLPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 14 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(14) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VUNPCKLPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61632, 434},
    /* 3788 : VUNPCKLPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f 14 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(14) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VUNPCKLPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61646, 434},
    /* 3789 : VUNPCKLPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f 14 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(14) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VUNPCKLPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61660, 434},
    /* 3790 : VXORPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 57 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(57) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VXORPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61674, 434},
    /* 3791 : VXORPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 57 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(57) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VXORPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61688, 434},
    /* 3792 : VXORPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f 57 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(57) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VXORPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61702, 434},
    /* 3793 : VXORPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f 57 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(57) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VXORPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61716, 434},
    /* 3794 : VZEROALL void [          vex.256.0f.w0 77                  ] AVX,SANDYBRIDGE */
        /* 270(001 004) 001(77) : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VZEROALL, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69812, 442},
    /* 3795 : VZEROUPPER void [          vex.128.0f.w0 77                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(77) : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VZEROUPPER, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69818, 442},
    /* 3796 : PCLMULLQLQDQ xmmreg,xmmrm128 [rm:       66 0f3a 44 /r 00                  ] SSE,WESTMERE */
        /* 361 357 001(44) 110 001(00) : NOAPX,SSE,WESTMERE */
        {I_PCLMULLQLQDQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+52485, 433},
    /* 3797 : PCLMULHQLQDQ xmmreg,xmmrm128 [rm:       66 0f3a 44 /r 01                  ] SSE,WESTMERE */
        /* 361 357 001(44) 110 001(01) : NOAPX,SSE,WESTMERE */
        {I_PCLMULHQLQDQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+52493, 433},
    /* 3798 : PCLMULLQHQDQ xmmreg,xmmrm128 [rm:       66 0f3a 44 /r 10                  ] SSE,WESTMERE */
        /* 361 357 001(44) 110 001(10) : NOAPX,SSE,WESTMERE */
        {I_PCLMULLQHQDQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+52501, 433},
    /* 3799 : PCLMULHQHQDQ xmmreg,xmmrm128 [rm:       66 0f3a 44 /r 11                  ] SSE,WESTMERE */
        /* 361 357 001(44) 110 001(11) : NOAPX,SSE,WESTMERE */
        {I_PCLMULHQHQDQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+52509, 433},
    /* 3800 : PCLMULQDQ xmmreg,xmmrm128,imm8 [rmi:      66 0f3a 44 /r ib                  ] SSE,WESTMERE */
        /* 361 357 001(44) 110 022 : NOAPX,SSE,WESTMERE */
        {I_PCLMULQDQ, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+61730, 433},
    /* 3801 : VPCLMULLQLQDQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f3a 44 /r 00      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(44) 120 001(00) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCLMULLQLQDQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29466, 434},
    /* 3802 : VPCLMULHQLQDQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f3a 44 /r 01      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(44) 120 001(01) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCLMULHQLQDQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29484, 434},
    /* 3803 : VPCLMULLQHQDQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f3a 44 /r 10      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(44) 120 001(10) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCLMULLQHQDQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29502, 434},
    /* 3804 : VPCLMULHQHQDQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f3a 44 /r 11      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(44) 120 001(11) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCLMULHQHQDQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29520, 434},
    /* 3805 : VPCLMULQDQ xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.66.0f3a 44 /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(44) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCLMULQDQ, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52517, 434},
    /* 3806 : VPCLMULLQLQDQ ymmreg,ymmreg*,ymmrm256 [rvm:fv:   vex.nds.256.66.0f3a.wig 44 /r 00  ] VPCLMULQDQ,FUTURE */
        /* 261(003 005) 001(44) 120 001(00) : NOAPX,WIG,VEX,VPCLMULQDQ,FUTURE */
        {I_VPCLMULLQLQDQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29538, 459},
    /* 3807 : VPCLMULHQLQDQ ymmreg,ymmreg*,ymmrm256 [rvm:fv:   vex.nds.256.66.0f3a.wig 44 /r 01  ] VPCLMULQDQ,FUTURE */
        /* 261(003 005) 001(44) 120 001(01) : NOAPX,WIG,VEX,VPCLMULQDQ,FUTURE */
        {I_VPCLMULHQLQDQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29556, 459},
    /* 3808 : VPCLMULLQHQDQ ymmreg,ymmreg*,ymmrm256 [rvm:fv:   vex.nds.256.66.0f3a.wig 44 /r 10  ] VPCLMULQDQ,FUTURE */
        /* 261(003 005) 001(44) 120 001(10) : NOAPX,WIG,VEX,VPCLMULQDQ,FUTURE */
        {I_VPCLMULLQHQDQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29574, 459},
    /* 3809 : VPCLMULHQHQDQ ymmreg,ymmreg*,ymmrm256 [rvm:fv:   vex.nds.256.66.0f3a.wig 44 /r 11  ] VPCLMULQDQ,FUTURE */
        /* 261(003 005) 001(44) 120 001(11) : NOAPX,WIG,VEX,VPCLMULQDQ,FUTURE */
        {I_VPCLMULHQHQDQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29592, 459},
    /* 3810 : VPCLMULQDQ ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:fv:  vex.nds.256.66.0f3a.wig 44 /r ib  ] VPCLMULQDQ,FUTURE */
        /* 261(003 005) 001(44) 120 023 : NOAPX,WIG,VEX,VPCLMULQDQ,FUTURE */
        {I_VPCLMULQDQ, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52533, 459},
    /* 3811 : VPCLMULLQLQDQ xmmreg,xmmreg*,xmmrm128 [rvm:fv:   evex.nds.128.66.0f3a.wig 44 /r 00 ] AVX512VL,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 08) 301 001(44) 120 001(00) : WIG,EVEX,PROT,AVX512VL,VPCLMULQDQ,FUTURE */
        {I_VPCLMULLQLQDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, NO_DECORATOR, nasm_bytecodes+4479, 460},
    /* 3812 : VPCLMULHQLQDQ xmmreg,xmmreg*,xmmrm128 [rvm:fv:   evex.nds.128.66.0f3a.wig 44 /r 01 ] AVX512VL,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 08) 301 001(44) 120 001(01) : WIG,EVEX,PROT,AVX512VL,VPCLMULQDQ,FUTURE */
        {I_VPCLMULHQLQDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, NO_DECORATOR, nasm_bytecodes+4501, 460},
    /* 3813 : VPCLMULLQHQDQ xmmreg,xmmreg*,xmmrm128 [rvm:fv:   evex.nds.128.66.0f3a.wig 44 /r 10 ] AVX512VL,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 08) 301 001(44) 120 001(10) : WIG,EVEX,PROT,AVX512VL,VPCLMULQDQ,FUTURE */
        {I_VPCLMULLQHQDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, NO_DECORATOR, nasm_bytecodes+4523, 460},
    /* 3814 : VPCLMULHQHQDQ xmmreg,xmmreg*,xmmrm128 [rvm:fv:   evex.nds.128.66.0f3a.wig 44 /r 11 ] AVX512VL,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 08) 301 001(44) 120 001(11) : WIG,EVEX,PROT,AVX512VL,VPCLMULQDQ,FUTURE */
        {I_VPCLMULHQHQDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, NO_DECORATOR, nasm_bytecodes+4545, 460},
    /* 3815 : VPCLMULQDQ xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.wig 44 /r ib ] AVX512VL,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 08) 301 001(44) 120 023 : WIG,EVEX,PROT,AVX512VL,VPCLMULQDQ,FUTURE */
        {I_VPCLMULQDQ, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+17993, 460},
    /* 3816 : VPCLMULLQLQDQ ymmreg,ymmreg*,ymmrm256 [rvm:fv:   evex.nds.256.66.0f3a.wig 44 /r 00 ] AVX512VL,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 28) 301 001(44) 120 001(00) : WIG,EVEX,PROT,AVX512VL,VPCLMULQDQ,FUTURE */
        {I_VPCLMULLQLQDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, NO_DECORATOR, nasm_bytecodes+4567, 460},
    /* 3817 : VPCLMULHQLQDQ ymmreg,ymmreg*,ymmrm256 [rvm:fv:   evex.nds.256.66.0f3a.wig 44 /r 01 ] AVX512VL,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 28) 301 001(44) 120 001(01) : WIG,EVEX,PROT,AVX512VL,VPCLMULQDQ,FUTURE */
        {I_VPCLMULHQLQDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, NO_DECORATOR, nasm_bytecodes+4589, 460},
    /* 3818 : VPCLMULLQHQDQ ymmreg,ymmreg*,ymmrm256 [rvm:fv:   evex.nds.256.66.0f3a.wig 44 /r 10 ] AVX512VL,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 28) 301 001(44) 120 001(10) : WIG,EVEX,PROT,AVX512VL,VPCLMULQDQ,FUTURE */
        {I_VPCLMULLQHQDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, NO_DECORATOR, nasm_bytecodes+4611, 460},
    /* 3819 : VPCLMULHQHQDQ ymmreg,ymmreg*,ymmrm256 [rvm:fv:   evex.nds.256.66.0f3a.wig 44 /r 11 ] AVX512VL,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 28) 301 001(44) 120 001(11) : WIG,EVEX,PROT,AVX512VL,VPCLMULQDQ,FUTURE */
        {I_VPCLMULHQHQDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, NO_DECORATOR, nasm_bytecodes+4633, 460},
    /* 3820 : VPCLMULQDQ ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.wig 44 /r ib ] AVX512VL,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 28) 301 001(44) 120 023 : WIG,EVEX,PROT,AVX512VL,VPCLMULQDQ,FUTURE */
        {I_VPCLMULQDQ, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+18013, 460},
    /* 3821 : VPCLMULLQLQDQ zmmreg,zmmreg*,zmmrm512 [rvm:fv:   evex.nds.512.66.0f3a.wig 44 /r 00 ] AVX512,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 48) 301 001(44) 120 001(00) : WIG,EVEX,PROT,AVX512,VPCLMULQDQ,FUTURE */
        {I_VPCLMULLQLQDQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, NO_DECORATOR, nasm_bytecodes+4655, 461},
    /* 3822 : VPCLMULHQLQDQ zmmreg,zmmreg*,zmmrm512 [rvm:fv:   evex.nds.512.66.0f3a.wig 44 /r 01 ] AVX512,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 48) 301 001(44) 120 001(01) : WIG,EVEX,PROT,AVX512,VPCLMULQDQ,FUTURE */
        {I_VPCLMULHQLQDQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, NO_DECORATOR, nasm_bytecodes+4677, 461},
    /* 3823 : VPCLMULLQHQDQ zmmreg,zmmreg*,zmmrm512 [rvm:fv:   evex.nds.512.66.0f3a.wig 44 /r 10 ] AVX512,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 48) 301 001(44) 120 001(10) : WIG,EVEX,PROT,AVX512,VPCLMULQDQ,FUTURE */
        {I_VPCLMULLQHQDQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, NO_DECORATOR, nasm_bytecodes+4699, 461},
    /* 3824 : VPCLMULHQHQDQ zmmreg,zmmreg*,zmmrm512 [rvm:fv:   evex.nds.512.66.0f3a.wig 44 /r 11 ] AVX512,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 48) 301 001(44) 120 001(11) : WIG,EVEX,PROT,AVX512,VPCLMULQDQ,FUTURE */
        {I_VPCLMULHQHQDQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, NO_DECORATOR, nasm_bytecodes+4721, 461},
    /* 3825 : VPCLMULQDQ zmmreg,zmmreg*,zmmrm512,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.wig 44 /r ib ] AVX512,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 48) 301 001(44) 120 023 : WIG,EVEX,PROT,AVX512,VPCLMULQDQ,FUTURE */
        {I_VPCLMULQDQ, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+18033, 461},
    /* 3826 : VFMADD132PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 98 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(98) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD132PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61737, 462},
    /* 3827 : VFMADD132PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 98 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(98) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD132PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61744, 462},
    /* 3828 : VFMADD132PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 98 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(98) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD132PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61751, 462},
    /* 3829 : VFMADD132PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 98 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(98) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD132PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61758, 462},
    /* 3830 : VFMADD312PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 98 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(98) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD312PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61737, 462},
    /* 3831 : VFMADD312PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 98 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(98) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD312PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61744, 462},
    /* 3832 : VFMADD312PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 98 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(98) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD312PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61751, 462},
    /* 3833 : VFMADD312PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 98 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(98) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD312PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61758, 462},
    /* 3834 : VFMADD213PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 a8 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(a8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD213PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61765, 462},
    /* 3835 : VFMADD213PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 a8 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(a8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD213PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61772, 462},
    /* 3836 : VFMADD213PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 a8 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(a8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD213PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61779, 462},
    /* 3837 : VFMADD213PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 a8 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(a8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD213PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61786, 462},
    /* 3838 : VFMADD123PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 a8 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(a8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD123PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61765, 462},
    /* 3839 : VFMADD123PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 a8 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(a8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD123PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61772, 462},
    /* 3840 : VFMADD123PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 a8 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(a8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD123PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61779, 462},
    /* 3841 : VFMADD123PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 a8 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(a8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD123PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61786, 462},
    /* 3842 : VFMADD231PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 b8 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(b8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD231PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61793, 462},
    /* 3843 : VFMADD231PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 b8 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(b8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD231PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61800, 462},
    /* 3844 : VFMADD231PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 b8 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(b8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD231PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61807, 462},
    /* 3845 : VFMADD231PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 b8 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(b8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD231PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61814, 462},
    /* 3846 : VFMADD321PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 b8 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(b8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD321PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61793, 462},
    /* 3847 : VFMADD321PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 b8 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(b8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD321PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61800, 462},
    /* 3848 : VFMADD321PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 b8 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(b8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD321PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61807, 462},
    /* 3849 : VFMADD321PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 b8 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(b8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD321PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61814, 462},
    /* 3850 : VFMADDSUB132PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 96 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(96) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB132PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61821, 462},
    /* 3851 : VFMADDSUB132PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 96 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(96) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB132PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61828, 462},
    /* 3852 : VFMADDSUB132PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 96 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(96) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB132PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61835, 462},
    /* 3853 : VFMADDSUB132PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 96 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(96) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB132PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61842, 462},
    /* 3854 : VFMADDSUB312PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 96 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(96) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB312PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61821, 462},
    /* 3855 : VFMADDSUB312PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 96 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(96) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB312PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61828, 462},
    /* 3856 : VFMADDSUB312PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 96 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(96) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB312PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61835, 462},
    /* 3857 : VFMADDSUB312PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 96 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(96) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB312PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61842, 462},
    /* 3858 : VFMADDSUB213PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 a6 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(a6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB213PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61849, 462},
    /* 3859 : VFMADDSUB213PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 a6 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(a6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB213PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61856, 462},
    /* 3860 : VFMADDSUB213PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 a6 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(a6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB213PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61863, 462},
    /* 3861 : VFMADDSUB213PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 a6 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(a6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB213PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61870, 462},
    /* 3862 : VFMADDSUB123PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 a6 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(a6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB123PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61849, 462},
    /* 3863 : VFMADDSUB123PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 a6 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(a6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB123PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61856, 462},
    /* 3864 : VFMADDSUB123PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 a6 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(a6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB123PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61863, 462},
    /* 3865 : VFMADDSUB123PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 a6 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(a6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB123PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61870, 462},
    /* 3866 : VFMADDSUB231PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 b6 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(b6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB231PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61877, 462},
    /* 3867 : VFMADDSUB231PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 b6 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(b6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB231PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61884, 462},
    /* 3868 : VFMADDSUB231PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 b6 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(b6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB231PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61891, 462},
    /* 3869 : VFMADDSUB231PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 b6 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(b6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB231PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61898, 462},
    /* 3870 : VFMADDSUB321PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 b6 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(b6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB321PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61877, 462},
    /* 3871 : VFMADDSUB321PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 b6 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(b6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB321PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61884, 462},
    /* 3872 : VFMADDSUB321PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 b6 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(b6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB321PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61891, 462},
    /* 3873 : VFMADDSUB321PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 b6 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(b6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB321PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61898, 462},
    /* 3874 : VFMSUB132PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 9a /r      ] FMA,FUTURE */
        /* 261(002 001) 001(9a) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB132PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61905, 462},
    /* 3875 : VFMSUB132PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 9a /r      ] FMA,FUTURE */
        /* 261(002 005) 001(9a) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB132PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61912, 462},
    /* 3876 : VFMSUB132PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 9a /r      ] FMA,FUTURE */
        /* 261(002 201) 001(9a) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB132PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61919, 462},
    /* 3877 : VFMSUB132PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 9a /r      ] FMA,FUTURE */
        /* 261(002 205) 001(9a) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB132PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61926, 462},
    /* 3878 : VFMSUB312PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 9a /r      ] FMA,FUTURE */
        /* 261(002 001) 001(9a) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB312PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61905, 462},
    /* 3879 : VFMSUB312PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 9a /r      ] FMA,FUTURE */
        /* 261(002 005) 001(9a) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB312PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61912, 462},
    /* 3880 : VFMSUB312PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 9a /r      ] FMA,FUTURE */
        /* 261(002 201) 001(9a) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB312PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61919, 462},
    /* 3881 : VFMSUB312PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 9a /r      ] FMA,FUTURE */
        /* 261(002 205) 001(9a) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB312PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61926, 462},
    /* 3882 : VFMSUB213PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 aa /r      ] FMA,FUTURE */
        /* 261(002 001) 001(aa) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB213PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61933, 462},
    /* 3883 : VFMSUB213PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 aa /r      ] FMA,FUTURE */
        /* 261(002 005) 001(aa) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB213PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61940, 462},
    /* 3884 : VFMSUB213PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 aa /r      ] FMA,FUTURE */
        /* 261(002 201) 001(aa) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB213PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61947, 462},
    /* 3885 : VFMSUB213PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 aa /r      ] FMA,FUTURE */
        /* 261(002 205) 001(aa) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB213PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61954, 462},
    /* 3886 : VFMSUB123PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 aa /r      ] FMA,FUTURE */
        /* 261(002 001) 001(aa) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB123PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61933, 462},
    /* 3887 : VFMSUB123PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 aa /r      ] FMA,FUTURE */
        /* 261(002 005) 001(aa) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB123PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61940, 462},
    /* 3888 : VFMSUB123PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 aa /r      ] FMA,FUTURE */
        /* 261(002 201) 001(aa) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB123PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61947, 462},
    /* 3889 : VFMSUB123PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 aa /r      ] FMA,FUTURE */
        /* 261(002 205) 001(aa) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB123PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61954, 462},
    /* 3890 : VFMSUB231PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 ba /r      ] FMA,FUTURE */
        /* 261(002 001) 001(ba) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB231PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61961, 462},
    /* 3891 : VFMSUB231PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 ba /r      ] FMA,FUTURE */
        /* 261(002 005) 001(ba) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB231PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61968, 462},
    /* 3892 : VFMSUB231PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 ba /r      ] FMA,FUTURE */
        /* 261(002 201) 001(ba) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB231PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61975, 462},
    /* 3893 : VFMSUB231PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 ba /r      ] FMA,FUTURE */
        /* 261(002 205) 001(ba) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB231PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61982, 462},
    /* 3894 : VFMSUB321PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 ba /r      ] FMA,FUTURE */
        /* 261(002 001) 001(ba) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB321PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61961, 462},
    /* 3895 : VFMSUB321PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 ba /r      ] FMA,FUTURE */
        /* 261(002 005) 001(ba) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB321PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61968, 462},
    /* 3896 : VFMSUB321PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 ba /r      ] FMA,FUTURE */
        /* 261(002 201) 001(ba) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB321PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61975, 462},
    /* 3897 : VFMSUB321PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 ba /r      ] FMA,FUTURE */
        /* 261(002 205) 001(ba) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB321PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61982, 462},
    /* 3898 : VFMSUBADD132PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 97 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(97) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD132PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61989, 462},
    /* 3899 : VFMSUBADD132PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 97 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(97) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD132PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61996, 462},
    /* 3900 : VFMSUBADD132PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 97 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(97) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD132PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62003, 462},
    /* 3901 : VFMSUBADD132PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 97 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(97) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD132PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62010, 462},
    /* 3902 : VFMSUBADD312PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 97 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(97) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD312PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61989, 462},
    /* 3903 : VFMSUBADD312PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 97 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(97) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD312PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61996, 462},
    /* 3904 : VFMSUBADD312PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 97 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(97) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD312PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62003, 462},
    /* 3905 : VFMSUBADD312PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 97 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(97) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD312PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62010, 462},
    /* 3906 : VFMSUBADD213PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 a7 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(a7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD213PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62017, 462},
    /* 3907 : VFMSUBADD213PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 a7 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(a7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD213PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62024, 462},
    /* 3908 : VFMSUBADD213PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 a7 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(a7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD213PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62031, 462},
    /* 3909 : VFMSUBADD213PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 a7 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(a7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD213PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62038, 462},
    /* 3910 : VFMSUBADD123PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 a7 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(a7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD123PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62017, 462},
    /* 3911 : VFMSUBADD123PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 a7 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(a7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD123PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62024, 462},
    /* 3912 : VFMSUBADD123PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 a7 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(a7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD123PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62031, 462},
    /* 3913 : VFMSUBADD123PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 a7 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(a7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD123PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62038, 462},
    /* 3914 : VFMSUBADD231PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 b7 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(b7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD231PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62045, 462},
    /* 3915 : VFMSUBADD231PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 b7 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(b7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD231PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62052, 462},
    /* 3916 : VFMSUBADD231PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 b7 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(b7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD231PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62059, 462},
    /* 3917 : VFMSUBADD231PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 b7 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(b7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD231PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62066, 462},
    /* 3918 : VFMSUBADD321PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 b7 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(b7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD321PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62045, 462},
    /* 3919 : VFMSUBADD321PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 b7 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(b7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD321PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62052, 462},
    /* 3920 : VFMSUBADD321PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 b7 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(b7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD321PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62059, 462},
    /* 3921 : VFMSUBADD321PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 b7 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(b7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD321PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62066, 462},
    /* 3922 : VFNMADD132PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 9c /r      ] FMA,FUTURE */
        /* 261(002 001) 001(9c) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD132PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62073, 462},
    /* 3923 : VFNMADD132PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 9c /r      ] FMA,FUTURE */
        /* 261(002 005) 001(9c) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD132PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62080, 462},
    /* 3924 : VFNMADD132PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 9c /r      ] FMA,FUTURE */
        /* 261(002 201) 001(9c) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD132PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62087, 462},
    /* 3925 : VFNMADD132PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 9c /r      ] FMA,FUTURE */
        /* 261(002 205) 001(9c) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD132PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62094, 462},
    /* 3926 : VFNMADD312PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 9c /r      ] FMA,FUTURE */
        /* 261(002 001) 001(9c) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD312PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62073, 462},
    /* 3927 : VFNMADD312PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 9c /r      ] FMA,FUTURE */
        /* 261(002 005) 001(9c) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD312PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62080, 462},
    /* 3928 : VFNMADD312PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 9c /r      ] FMA,FUTURE */
        /* 261(002 201) 001(9c) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD312PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62087, 462},
    /* 3929 : VFNMADD312PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 9c /r      ] FMA,FUTURE */
        /* 261(002 205) 001(9c) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD312PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62094, 462},
    /* 3930 : VFNMADD213PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 ac /r      ] FMA,FUTURE */
        /* 261(002 001) 001(ac) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD213PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62101, 462},
    /* 3931 : VFNMADD213PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 ac /r      ] FMA,FUTURE */
        /* 261(002 005) 001(ac) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD213PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62108, 462},
    /* 3932 : VFNMADD213PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 ac /r      ] FMA,FUTURE */
        /* 261(002 201) 001(ac) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD213PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62115, 462},
    /* 3933 : VFNMADD213PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 ac /r      ] FMA,FUTURE */
        /* 261(002 205) 001(ac) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD213PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62122, 462},
    /* 3934 : VFNMADD123PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 ac /r      ] FMA,FUTURE */
        /* 261(002 001) 001(ac) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD123PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62101, 462},
    /* 3935 : VFNMADD123PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 ac /r      ] FMA,FUTURE */
        /* 261(002 005) 001(ac) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD123PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62108, 462},
    /* 3936 : VFNMADD123PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 ac /r      ] FMA,FUTURE */
        /* 261(002 201) 001(ac) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD123PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62115, 462},
    /* 3937 : VFNMADD123PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 ac /r      ] FMA,FUTURE */
        /* 261(002 205) 001(ac) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD123PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62122, 462},
    /* 3938 : VFNMADD231PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 bc /r      ] FMA,FUTURE */
        /* 261(002 001) 001(bc) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD231PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62129, 462},
    /* 3939 : VFNMADD231PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 bc /r      ] FMA,FUTURE */
        /* 261(002 005) 001(bc) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD231PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62136, 462},
    /* 3940 : VFNMADD231PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 bc /r      ] FMA,FUTURE */
        /* 261(002 201) 001(bc) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD231PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62143, 462},
    /* 3941 : VFNMADD231PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 bc /r      ] FMA,FUTURE */
        /* 261(002 205) 001(bc) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD231PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62150, 462},
    /* 3942 : VFNMADD321PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 bc /r      ] FMA,FUTURE */
        /* 261(002 001) 001(bc) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD321PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62129, 462},
    /* 3943 : VFNMADD321PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 bc /r      ] FMA,FUTURE */
        /* 261(002 005) 001(bc) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD321PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62136, 462},
    /* 3944 : VFNMADD321PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 bc /r      ] FMA,FUTURE */
        /* 261(002 201) 001(bc) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD321PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62143, 462},
    /* 3945 : VFNMADD321PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 bc /r      ] FMA,FUTURE */
        /* 261(002 205) 001(bc) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD321PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62150, 462},
    /* 3946 : VFNMSUB132PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 9e /r      ] FMA,FUTURE */
        /* 261(002 001) 001(9e) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB132PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62157, 462},
    /* 3947 : VFNMSUB132PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 9e /r      ] FMA,FUTURE */
        /* 261(002 005) 001(9e) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB132PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62164, 462},
    /* 3948 : VFNMSUB132PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 9e /r      ] FMA,FUTURE */
        /* 261(002 201) 001(9e) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB132PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62171, 462},
    /* 3949 : VFNMSUB132PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 9e /r      ] FMA,FUTURE */
        /* 261(002 205) 001(9e) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB132PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62178, 462},
    /* 3950 : VFNMSUB312PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 9e /r      ] FMA,FUTURE */
        /* 261(002 001) 001(9e) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB312PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62157, 462},
    /* 3951 : VFNMSUB312PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 9e /r      ] FMA,FUTURE */
        /* 261(002 005) 001(9e) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB312PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62164, 462},
    /* 3952 : VFNMSUB312PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 9e /r      ] FMA,FUTURE */
        /* 261(002 201) 001(9e) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB312PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62171, 462},
    /* 3953 : VFNMSUB312PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 9e /r      ] FMA,FUTURE */
        /* 261(002 205) 001(9e) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB312PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62178, 462},
    /* 3954 : VFNMSUB213PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 ae /r      ] FMA,FUTURE */
        /* 261(002 001) 001(ae) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB213PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62185, 462},
    /* 3955 : VFNMSUB213PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 ae /r      ] FMA,FUTURE */
        /* 261(002 005) 001(ae) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB213PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62192, 462},
    /* 3956 : VFNMSUB213PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 ae /r      ] FMA,FUTURE */
        /* 261(002 201) 001(ae) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB213PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62199, 462},
    /* 3957 : VFNMSUB213PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 ae /r      ] FMA,FUTURE */
        /* 261(002 205) 001(ae) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB213PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62206, 462},
    /* 3958 : VFNMSUB123PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 ae /r      ] FMA,FUTURE */
        /* 261(002 001) 001(ae) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB123PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62185, 462},
    /* 3959 : VFNMSUB123PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 ae /r      ] FMA,FUTURE */
        /* 261(002 005) 001(ae) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB123PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62192, 462},
    /* 3960 : VFNMSUB123PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 ae /r      ] FMA,FUTURE */
        /* 261(002 201) 001(ae) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB123PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62199, 462},
    /* 3961 : VFNMSUB123PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 ae /r      ] FMA,FUTURE */
        /* 261(002 205) 001(ae) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB123PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62206, 462},
    /* 3962 : VFNMSUB231PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 be /r      ] FMA,FUTURE */
        /* 261(002 001) 001(be) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB231PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62213, 462},
    /* 3963 : VFNMSUB231PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 be /r      ] FMA,FUTURE */
        /* 261(002 005) 001(be) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB231PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62220, 462},
    /* 3964 : VFNMSUB231PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 be /r      ] FMA,FUTURE */
        /* 261(002 201) 001(be) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB231PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62227, 462},
    /* 3965 : VFNMSUB231PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 be /r      ] FMA,FUTURE */
        /* 261(002 205) 001(be) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB231PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62234, 462},
    /* 3966 : VFNMSUB321PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 be /r      ] FMA,FUTURE */
        /* 261(002 001) 001(be) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB321PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62213, 462},
    /* 3967 : VFNMSUB321PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 be /r      ] FMA,FUTURE */
        /* 261(002 005) 001(be) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB321PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62220, 462},
    /* 3968 : VFNMSUB321PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 be /r      ] FMA,FUTURE */
        /* 261(002 201) 001(be) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB321PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62227, 462},
    /* 3969 : VFNMSUB321PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 be /r      ] FMA,FUTURE */
        /* 261(002 205) 001(be) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB321PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62234, 462},
    /* 3970 : VFMADD132SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 99 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(99) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD132SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62241, 462},
    /* 3971 : VFMADD132SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 99 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(99) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD132SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62248, 462},
    /* 3972 : VFMADD312SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 99 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(99) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD312SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62241, 462},
    /* 3973 : VFMADD312SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 99 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(99) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD312SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62248, 462},
    /* 3974 : VFMADD213SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 a9 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(a9) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD213SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62255, 462},
    /* 3975 : VFMADD213SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 a9 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(a9) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD213SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62262, 462},
    /* 3976 : VFMADD123SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 a9 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(a9) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD123SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62255, 462},
    /* 3977 : VFMADD123SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 a9 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(a9) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD123SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62262, 462},
    /* 3978 : VFMADD231SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 b9 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(b9) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD231SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62269, 462},
    /* 3979 : VFMADD231SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 b9 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(b9) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD231SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62276, 462},
    /* 3980 : VFMADD321SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 b9 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(b9) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD321SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62269, 462},
    /* 3981 : VFMADD321SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 b9 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(b9) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD321SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62276, 462},
    /* 3982 : VFMSUB132SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 9b /r      ] FMA,FUTURE */
        /* 261(002 001) 001(9b) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB132SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62283, 462},
    /* 3983 : VFMSUB132SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 9b /r      ] FMA,FUTURE */
        /* 261(002 201) 001(9b) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB132SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62290, 462},
    /* 3984 : VFMSUB312SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 9b /r      ] FMA,FUTURE */
        /* 261(002 001) 001(9b) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB312SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62283, 462},
    /* 3985 : VFMSUB312SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 9b /r      ] FMA,FUTURE */
        /* 261(002 201) 001(9b) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB312SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62290, 462},
    /* 3986 : VFMSUB213SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 ab /r      ] FMA,FUTURE */
        /* 261(002 001) 001(ab) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB213SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62297, 462},
    /* 3987 : VFMSUB213SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 ab /r      ] FMA,FUTURE */
        /* 261(002 201) 001(ab) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB213SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62304, 462},
    /* 3988 : VFMSUB123SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 ab /r      ] FMA,FUTURE */
        /* 261(002 001) 001(ab) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB123SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62297, 462},
    /* 3989 : VFMSUB123SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 ab /r      ] FMA,FUTURE */
        /* 261(002 201) 001(ab) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB123SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62304, 462},
    /* 3990 : VFMSUB231SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 bb /r      ] FMA,FUTURE */
        /* 261(002 001) 001(bb) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB231SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62311, 462},
    /* 3991 : VFMSUB231SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 bb /r      ] FMA,FUTURE */
        /* 261(002 201) 001(bb) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB231SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62318, 462},
    /* 3992 : VFMSUB321SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 bb /r      ] FMA,FUTURE */
        /* 261(002 001) 001(bb) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB321SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62311, 462},
    /* 3993 : VFMSUB321SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 bb /r      ] FMA,FUTURE */
        /* 261(002 201) 001(bb) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB321SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62318, 462},
    /* 3994 : VFNMADD132SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 9d /r      ] FMA,FUTURE */
        /* 261(002 001) 001(9d) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD132SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62325, 462},
    /* 3995 : VFNMADD132SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 9d /r      ] FMA,FUTURE */
        /* 261(002 201) 001(9d) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD132SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62332, 462},
    /* 3996 : VFNMADD312SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 9d /r      ] FMA,FUTURE */
        /* 261(002 001) 001(9d) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD312SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62325, 462},
    /* 3997 : VFNMADD312SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 9d /r      ] FMA,FUTURE */
        /* 261(002 201) 001(9d) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD312SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62332, 462},
    /* 3998 : VFNMADD213SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 ad /r      ] FMA,FUTURE */
        /* 261(002 001) 001(ad) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD213SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62339, 462},
    /* 3999 : VFNMADD213SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 ad /r      ] FMA,FUTURE */
        /* 261(002 201) 001(ad) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD213SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62346, 462},
    /* 4000 : VFNMADD123SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 ad /r      ] FMA,FUTURE */
        /* 261(002 001) 001(ad) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD123SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62339, 462},
    /* 4001 : VFNMADD123SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 ad /r      ] FMA,FUTURE */
        /* 261(002 201) 001(ad) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD123SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62346, 462},
    /* 4002 : VFNMADD231SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 bd /r      ] FMA,FUTURE */
        /* 261(002 001) 001(bd) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD231SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62353, 462},
    /* 4003 : VFNMADD231SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 bd /r      ] FMA,FUTURE */
        /* 261(002 201) 001(bd) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD231SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62360, 462},
    /* 4004 : VFNMADD321SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 bd /r      ] FMA,FUTURE */
        /* 261(002 001) 001(bd) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD321SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62353, 462},
    /* 4005 : VFNMADD321SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 bd /r      ] FMA,FUTURE */
        /* 261(002 201) 001(bd) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD321SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62360, 462},
    /* 4006 : VFNMSUB132SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 9f /r      ] FMA,FUTURE */
        /* 261(002 001) 001(9f) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB132SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62367, 462},
    /* 4007 : VFNMSUB132SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 9f /r      ] FMA,FUTURE */
        /* 261(002 201) 001(9f) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB132SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62374, 462},
    /* 4008 : VFNMSUB312SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 9f /r      ] FMA,FUTURE */
        /* 261(002 001) 001(9f) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB312SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62367, 462},
    /* 4009 : VFNMSUB312SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 9f /r      ] FMA,FUTURE */
        /* 261(002 201) 001(9f) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB312SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62374, 462},
    /* 4010 : VFNMSUB213SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 af /r      ] FMA,FUTURE */
        /* 261(002 001) 001(af) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB213SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62381, 462},
    /* 4011 : VFNMSUB213SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 af /r      ] FMA,FUTURE */
        /* 261(002 201) 001(af) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB213SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62388, 462},
    /* 4012 : VFNMSUB123SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 af /r      ] FMA,FUTURE */
        /* 261(002 001) 001(af) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB123SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62381, 462},
    /* 4013 : VFNMSUB123SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 af /r      ] FMA,FUTURE */
        /* 261(002 201) 001(af) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB123SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62388, 462},
    /* 4014 : VFNMSUB231SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 bf /r      ] FMA,FUTURE */
        /* 261(002 001) 001(bf) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB231SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62395, 462},
    /* 4015 : VFNMSUB231SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 bf /r      ] FMA,FUTURE */
        /* 261(002 201) 001(bf) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB231SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62402, 462},
    /* 4016 : VFNMSUB321SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 bf /r      ] FMA,FUTURE */
        /* 261(002 001) 001(bf) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB321SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62395, 462},
    /* 4017 : VFNMSUB321SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 bf /r      ] FMA,FUTURE */
        /* 261(002 201) 001(bf) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB321SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62402, 462},
    /* 4018 : VCVTPH2PS ymmreg,xmmrm128 [rm:       vex.256.66.0f38.w0 13 /r          ] F16C,FUTURE */
        /* 270(002 005) 001(13) 110 : NOAPX,VEX,F16C,FUTURE */
        {I_VCVTPH2PS, 2, {YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62409, 463},
    /* 4019 : VCVTPH2PS xmmreg,xmmrm64 [rm:       vex.128.66.0f38.w0 13 /r          ] F16C,FUTURE */
        /* 270(002 001) 001(13) 110 : NOAPX,VEX,F16C,FUTURE */
        {I_VCVTPH2PS, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62416, 463},
    /* 4020 : VCVTPS2PH xmmrm128,ymmreg,imm8 [mri:      vex.256.66.0f3a.w0 1d /r ib       ] F16C,FUTURE */
        /* 270(003 005) 001(1d) 101 022 : NOAPX,VEX,F16C,FUTURE */
        {I_VCVTPS2PH, 3, {RM_XMM|BITS128|RN_L16,YMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52549, 463},
    /* 4021 : VCVTPS2PH xmmrm64,xmmreg,imm8 [mri:      vex.128.66.0f3a.w0 1d /r ib       ] F16C,FUTURE */
        /* 270(003 001) 001(1d) 101 022 : NOAPX,VEX,F16C,FUTURE */
        {I_VCVTPS2PH, 3, {RM_XMM|BITS64|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52557, 463},
    /* 4022 : XSTORE void [          0f a7 c0                          ] PENT,CYRIX */
        /* 355 002(a7 c0) : PENT,CYRIX */
        {I_XSTORE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72363, 464},
    /* 4023 : XCRYPTECB void [          f3 0f a7 c8                       ] PENT,CYRIX */
        /* 333 355 002(a7 c8) : PENT,CYRIX */
        {I_XCRYPTECB, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69824, 464},
    /* 4024 : XCRYPTCBC void [          f3 0f a7 d0                       ] PENT,CYRIX */
        /* 333 355 002(a7 d0) : PENT,CYRIX */
        {I_XCRYPTCBC, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69830, 464},
    /* 4025 : XCRYPTCTR void [          f3 0f a7 d8                       ] PENT,CYRIX */
        /* 333 355 002(a7 d8) : PENT,CYRIX */
        {I_XCRYPTCTR, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69836, 464},
    /* 4026 : XCRYPTCFB void [          f3 0f a7 e0                       ] PENT,CYRIX */
        /* 333 355 002(a7 e0) : PENT,CYRIX */
        {I_XCRYPTCFB, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69842, 464},
    /* 4027 : XCRYPTOFB void [          f3 0f a7 e8                       ] PENT,CYRIX */
        /* 333 355 002(a7 e8) : PENT,CYRIX */
        {I_XCRYPTOFB, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69848, 464},
    /* 4028 : MONTMUL void [          f3 0f a6 c0                       ] PENT,CYRIX */
        /* 333 355 002(a6 c0) : PENT,CYRIX */
        {I_MONTMUL, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69854, 464},
    /* 4029 : XSHA1 void [          f3 0f a6 c8                       ] PENT,CYRIX */
        /* 333 355 002(a6 c8) : PENT,CYRIX */
        {I_XSHA1, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69860, 464},
    /* 4030 : XSHA256 void [          f3 0f a6 d0                       ] PENT,CYRIX */
        /* 333 355 002(a6 d0) : PENT,CYRIX */
        {I_XSHA256, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69866, 464},
    /* 4031 : LLWPCB reg32 [m:        xop.m9.w0.l0.p0 12 /0             ] 386,AMD */
        /* 270(111 000) 001(12) 200 : NOAPX,VEX,386,AMD */
        {I_LLWPCB, 1, {REG_GPR|BITS32|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+62423, 465},
    /* 4032 : LLWPCB reg64 [m:        xop.m9.w1.l0.p0 12 /0             ] LONG,PROT,X86_64,AMD */
        /* 270(111 200) 001(12) 200 : NOAPX,LONG,VEX,PROT,X86_64,AMD */
        {I_LLWPCB, 1, {REG_GPR|BITS64|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+62430, 466},
    /* 4033 : SLWPCB reg32 [m:        xop.m9.w0.l0.p0 12 /1             ] 386,AMD */
        /* 270(111 000) 001(12) 201 : NOAPX,VEX,386,AMD */
        {I_SLWPCB, 1, {REG_GPR|BITS32|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+62437, 465},
    /* 4034 : SLWPCB reg64 [m:        xop.m9.w1.l0.p0 12 /1             ] LONG,PROT,X86_64,AMD */
        /* 270(111 200) 001(12) 201 : NOAPX,LONG,VEX,PROT,X86_64,AMD */
        {I_SLWPCB, 1, {REG_GPR|BITS64|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+62444, 466},
    /* 4035 : LWPVAL reg32,rm32,imm32 [vmi:      xop.m10.w0.ndd.l0.p0 12 /1 id     ] 386,AMD */
        /* 260(112 000) 001(12) 211 042 : NOAPX,VEX,386,AMD */
        {I_LWPVAL, 3, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+52565, 465},
    /* 4036 : LWPVAL reg64,rm32,imm32 [vmi:      xop.m10.w1.ndd.l0.p0 12 /1 id     ] LONG,PROT,X86_64,AMD */
        /* 260(112 200) 001(12) 211 042 : NOAPX,LONG,VEX,PROT,X86_64,AMD */
        {I_LWPVAL, 3, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS32|RN_L16,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+52573, 466},
    /* 4037 : LWPINS reg32,rm32,imm32 [vmi:      xop.m10.w0.ndd.l0.p0 12 /0 id     ] 386,AMD */
        /* 260(112 000) 001(12) 210 042 : NOAPX,VEX,386,AMD */
        {I_LWPINS, 3, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+52581, 465},
    /* 4038 : LWPINS reg64,rm32,imm32 [vmi:      xop.m10.w1.ndd.l0.p0 12 /0 id     ] LONG,PROT,X86_64,AMD */
        /* 260(112 200) 001(12) 210 042 : NOAPX,LONG,VEX,PROT,X86_64,AMD */
        {I_LWPINS, 3, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS32|RN_L16,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+52589, 466},
    /* 4039 : VFMADDPD xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 69 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(69) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52597, 467},
    /* 4040 : VFMADDPD ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.m3.w0.nds.l1.p1 69 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 005) 001(69) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52613, 467},
    /* 4041 : VFMADDPD xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     vex.m3.w1.nds.l0.p1 69 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(69) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52629, 467},
    /* 4042 : VFMADDPD ymmreg,ymmreg*,ymmreg,ymmrm256 [rvsm:     vex.m3.w1.nds.l1.p1 69 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 205) 001(69) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52645, 467},
    /* 4043 : VFMADDPS xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 68 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(68) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52661, 467},
    /* 4044 : VFMADDPS ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.m3.w0.nds.l1.p1 68 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 005) 001(68) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52677, 467},
    /* 4045 : VFMADDPS xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     vex.m3.w1.nds.l0.p1 68 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(68) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52693, 467},
    /* 4046 : VFMADDPS ymmreg,ymmreg*,ymmreg,ymmrm256 [rvsm:     vex.m3.w1.nds.l1.p1 68 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 205) 001(68) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52709, 467},
    /* 4047 : VFMADDSD xmmreg,xmmreg*,xmmrm64,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 6b /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(6b) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDSD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52725, 467},
    /* 4048 : VFMADDSD xmmreg,xmmreg*,xmmreg,xmmrm64 [rvsm:     vex.m3.w1.nds.l0.p1 6b /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(6b) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDSD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52741, 467},
    /* 4049 : VFMADDSS xmmreg,xmmreg*,xmmrm32,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 6a /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(6a) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDSS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52757, 467},
    /* 4050 : VFMADDSS xmmreg,xmmreg*,xmmreg,xmmrm32 [rvsm:     vex.m3.w1.nds.l0.p1 6a /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(6a) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDSS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52773, 467},
    /* 4051 : VFMADDSUBPD xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 5d /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(5d) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDSUBPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52789, 467},
    /* 4052 : VFMADDSUBPD ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.m3.w0.nds.l1.p1 5d /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 005) 001(5d) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDSUBPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52805, 467},
    /* 4053 : VFMADDSUBPD xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     vex.m3.w1.nds.l0.p1 5d /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(5d) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDSUBPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52821, 467},
    /* 4054 : VFMADDSUBPD ymmreg,ymmreg*,ymmreg,ymmrm256 [rvsm:     vex.m3.w1.nds.l1.p1 5d /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 205) 001(5d) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDSUBPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52837, 467},
    /* 4055 : VFMADDSUBPS xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 5c /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(5c) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDSUBPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52853, 467},
    /* 4056 : VFMADDSUBPS ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.m3.w0.nds.l1.p1 5c /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 005) 001(5c) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDSUBPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52869, 467},
    /* 4057 : VFMADDSUBPS xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     vex.m3.w1.nds.l0.p1 5c /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(5c) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDSUBPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52885, 467},
    /* 4058 : VFMADDSUBPS ymmreg,ymmreg*,ymmreg,ymmrm256 [rvsm:     vex.m3.w1.nds.l1.p1 5c /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 205) 001(5c) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDSUBPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52901, 467},
    /* 4059 : VFMSUBADDPD xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 5f /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(5f) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBADDPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52917, 467},
    /* 4060 : VFMSUBADDPD ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.m3.w0.nds.l1.p1 5f /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 005) 001(5f) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBADDPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52933, 467},
    /* 4061 : VFMSUBADDPD xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     vex.m3.w1.nds.l0.p1 5f /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(5f) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBADDPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52949, 467},
    /* 4062 : VFMSUBADDPD ymmreg,ymmreg*,ymmreg,ymmrm256 [rvsm:     vex.m3.w1.nds.l1.p1 5f /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 205) 001(5f) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBADDPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52965, 467},
    /* 4063 : VFMSUBADDPS xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 5e /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(5e) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBADDPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52981, 467},
    /* 4064 : VFMSUBADDPS ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.m3.w0.nds.l1.p1 5e /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 005) 001(5e) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBADDPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52997, 467},
    /* 4065 : VFMSUBADDPS xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     vex.m3.w1.nds.l0.p1 5e /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(5e) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBADDPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53013, 467},
    /* 4066 : VFMSUBADDPS ymmreg,ymmreg*,ymmreg,ymmrm256 [rvsm:     vex.m3.w1.nds.l1.p1 5e /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 205) 001(5e) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBADDPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53029, 467},
    /* 4067 : VFMSUBPD xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 6d /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(6d) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53045, 467},
    /* 4068 : VFMSUBPD ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.m3.w0.nds.l1.p1 6d /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 005) 001(6d) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53061, 467},
    /* 4069 : VFMSUBPD xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     vex.m3.w1.nds.l0.p1 6d /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(6d) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53077, 467},
    /* 4070 : VFMSUBPD ymmreg,ymmreg*,ymmreg,ymmrm256 [rvsm:     vex.m3.w1.nds.l1.p1 6d /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 205) 001(6d) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53093, 467},
    /* 4071 : VFMSUBPS xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 6c /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(6c) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53109, 467},
    /* 4072 : VFMSUBPS ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.m3.w0.nds.l1.p1 6c /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 005) 001(6c) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53125, 467},
    /* 4073 : VFMSUBPS xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     vex.m3.w1.nds.l0.p1 6c /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(6c) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53141, 467},
    /* 4074 : VFMSUBPS ymmreg,ymmreg*,ymmreg,ymmrm256 [rvsm:     vex.m3.w1.nds.l1.p1 6c /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 205) 001(6c) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53157, 467},
    /* 4075 : VFMSUBSD xmmreg,xmmreg*,xmmrm64,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 6f /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(6f) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBSD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53173, 467},
    /* 4076 : VFMSUBSD xmmreg,xmmreg*,xmmreg,xmmrm64 [rvsm:     vex.m3.w1.nds.l0.p1 6f /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(6f) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBSD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53189, 467},
    /* 4077 : VFMSUBSS xmmreg,xmmreg*,xmmrm32,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 6e /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(6e) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBSS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53205, 467},
    /* 4078 : VFMSUBSS xmmreg,xmmreg*,xmmreg,xmmrm32 [rvsm:     vex.m3.w1.nds.l0.p1 6e /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(6e) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBSS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53221, 467},
    /* 4079 : VFNMADDPD xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 79 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(79) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMADDPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53237, 467},
    /* 4080 : VFNMADDPD ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.m3.w0.nds.l1.p1 79 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 005) 001(79) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMADDPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53253, 467},
    /* 4081 : VFNMADDPD xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     vex.m3.w1.nds.l0.p1 79 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(79) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMADDPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53269, 467},
    /* 4082 : VFNMADDPD ymmreg,ymmreg*,ymmreg,ymmrm256 [rvsm:     vex.m3.w1.nds.l1.p1 79 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 205) 001(79) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMADDPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53285, 467},
    /* 4083 : VFNMADDPS xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 78 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(78) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMADDPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53301, 467},
    /* 4084 : VFNMADDPS ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.m3.w0.nds.l1.p1 78 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 005) 001(78) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMADDPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53317, 467},
    /* 4085 : VFNMADDPS xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     vex.m3.w1.nds.l0.p1 78 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(78) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMADDPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53333, 467},
    /* 4086 : VFNMADDPS ymmreg,ymmreg*,ymmreg,ymmrm256 [rvsm:     vex.m3.w1.nds.l1.p1 78 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 205) 001(78) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMADDPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53349, 467},
    /* 4087 : VFNMADDSD xmmreg,xmmreg*,xmmrm64,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 7b /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(7b) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMADDSD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53365, 467},
    /* 4088 : VFNMADDSD xmmreg,xmmreg*,xmmreg,xmmrm64 [rvsm:     vex.m3.w1.nds.l0.p1 7b /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(7b) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMADDSD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53381, 467},
    /* 4089 : VFNMADDSS xmmreg,xmmreg*,xmmrm32,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 7a /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(7a) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMADDSS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53397, 467},
    /* 4090 : VFNMADDSS xmmreg,xmmreg*,xmmreg,xmmrm32 [rvsm:     vex.m3.w1.nds.l0.p1 7a /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(7a) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMADDSS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53413, 467},
    /* 4091 : VFNMSUBPD xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 7d /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(7d) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMSUBPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53429, 467},
    /* 4092 : VFNMSUBPD ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.m3.w0.nds.l1.p1 7d /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 005) 001(7d) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMSUBPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53445, 467},
    /* 4093 : VFNMSUBPD xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     vex.m3.w1.nds.l0.p1 7d /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(7d) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMSUBPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53461, 467},
    /* 4094 : VFNMSUBPD ymmreg,ymmreg*,ymmreg,ymmrm256 [rvsm:     vex.m3.w1.nds.l1.p1 7d /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 205) 001(7d) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMSUBPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53477, 467},
    /* 4095 : VFNMSUBPS xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 7c /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(7c) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMSUBPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53493, 467},
    /* 4096 : VFNMSUBPS ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.m3.w0.nds.l1.p1 7c /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 005) 001(7c) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMSUBPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53509, 467},
    /* 4097 : VFNMSUBPS xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     vex.m3.w1.nds.l0.p1 7c /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(7c) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMSUBPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53525, 467},
    /* 4098 : VFNMSUBPS ymmreg,ymmreg*,ymmreg,ymmrm256 [rvsm:     vex.m3.w1.nds.l1.p1 7c /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 205) 001(7c) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMSUBPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53541, 467},
    /* 4099 : VFNMSUBSD xmmreg,xmmreg*,xmmrm64,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 7f /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(7f) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMSUBSD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53557, 467},
    /* 4100 : VFNMSUBSD xmmreg,xmmreg*,xmmreg,xmmrm64 [rvsm:     vex.m3.w1.nds.l0.p1 7f /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(7f) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMSUBSD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53573, 467},
    /* 4101 : VFNMSUBSS xmmreg,xmmreg*,xmmrm32,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 7e /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(7e) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMSUBSS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53589, 467},
    /* 4102 : VFNMSUBSS xmmreg,xmmreg*,xmmreg,xmmrm32 [rvsm:     vex.m3.w1.nds.l0.p1 7e /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(7e) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMSUBSS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53605, 467},
    /* 4103 : VFRCZPD xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 81 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(81) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFRCZPD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62451, 467},
    /* 4104 : VFRCZPD ymmreg,ymmrm256* [rm:       xop.m9.w0.l1.p0 81 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 004) 001(81) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFRCZPD, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62465, 467},
    /* 4105 : VFRCZPS xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 80 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(80) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFRCZPS, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62479, 467},
    /* 4106 : VFRCZPS ymmreg,ymmrm256* [rm:       xop.m9.w0.l1.p0 80 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 004) 001(80) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFRCZPS, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62493, 467},
    /* 4107 : VFRCZSD xmmreg,xmmrm64* [rm:       xop.m9.w0.l0.p0 83 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(83) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFRCZSD, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62507, 467},
    /* 4108 : VFRCZSS xmmreg,xmmrm32* [rm:       xop.m9.w0.l0.p0 82 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(82) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFRCZSS, 2, {XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62521, 467},
    /* 4109 : VPCMOV xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 a2 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(a2) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPCMOV, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53621, 467},
    /* 4110 : VPCMOV ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     xop.m8.w0.nds.l1.p0 a2 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 004) 001(a2) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPCMOV, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53637, 467},
    /* 4111 : VPCMOV xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     xop.m8.w1.nds.l0.p0 a2 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 200) 001(a2) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPCMOV, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53653, 467},
    /* 4112 : VPCMOV ymmreg,ymmreg*,ymmreg,ymmrm256 [rvsm:     xop.m8.w1.nds.l1.p0 a2 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 204) 001(a2) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPCMOV, 4, {YMMREG|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53669, 467},
    /* 4113 : VPCOMB xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     xop.m8.w0.nds.l0.p0 cc /r ib      ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(cc) 120 023 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPCOMB, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+53685, 467},
    /* 4114 : VPCOMD xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     xop.m8.w0.nds.l0.p0 ce /r ib      ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(ce) 120 023 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPCOMD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+53701, 467},
    /* 4115 : VPCOMQ xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     xop.m8.w0.nds.l0.p0 cf /r ib      ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(cf) 120 023 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPCOMQ, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+53717, 467},
    /* 4116 : VPCOMUB xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     xop.m8.w0.nds.l0.p0 ec /r ib      ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(ec) 120 023 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPCOMUB, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+53733, 467},
    /* 4117 : VPCOMUD xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     xop.m8.w0.nds.l0.p0 ee /r ib      ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(ee) 120 023 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPCOMUD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+53749, 467},
    /* 4118 : VPCOMUQ xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     xop.m8.w0.nds.l0.p0 ef /r ib      ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(ef) 120 023 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPCOMUQ, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+53765, 467},
    /* 4119 : VPCOMUW xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     xop.m8.w0.nds.l0.p0 ed /r ib      ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(ed) 120 023 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPCOMUW, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+53781, 467},
    /* 4120 : VPCOMW xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     xop.m8.w0.nds.l0.p0 cd /r ib      ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(cd) 120 023 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPCOMW, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+53797, 467},
    /* 4121 : VPHADDBD xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 c2 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(c2) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHADDBD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62535, 467},
    /* 4122 : VPHADDBQ xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 c3 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(c3) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHADDBQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62549, 467},
    /* 4123 : VPHADDBW xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 c1 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(c1) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHADDBW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62563, 467},
    /* 4124 : VPHADDDQ xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 cb /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(cb) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHADDDQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62577, 467},
    /* 4125 : VPHADDUBD xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 d2 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(d2) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHADDUBD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62591, 467},
    /* 4126 : VPHADDUBQ xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 d3 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(d3) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHADDUBQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62605, 467},
    /* 4127 : VPHADDUBW xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 d1 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(d1) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHADDUBW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62619, 467},
    /* 4128 : VPHADDUDQ xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 db /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(db) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHADDUDQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62633, 467},
    /* 4129 : VPHADDUWD xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 d6 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(d6) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHADDUWD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62647, 467},
    /* 4130 : VPHADDUWQ xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 d7 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(d7) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHADDUWQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62661, 467},
    /* 4131 : VPHADDWD xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 c6 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(c6) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHADDWD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62675, 467},
    /* 4132 : VPHADDWQ xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 c7 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(c7) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHADDWQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62689, 467},
    /* 4133 : VPHSUBBW xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 e1 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(e1) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHSUBBW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62703, 467},
    /* 4134 : VPHSUBDQ xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 e3 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(e3) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHSUBDQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62717, 467},
    /* 4135 : VPHSUBWD xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 e2 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(e2) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHSUBWD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62731, 467},
    /* 4136 : VPMACSDD xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 9e /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(9e) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPMACSDD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53813, 467},
    /* 4137 : VPMACSDQH xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 9f /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(9f) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPMACSDQH, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53829, 467},
    /* 4138 : VPMACSDQL xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 97 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(97) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPMACSDQL, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53845, 467},
    /* 4139 : VPMACSSDD xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 8e /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(8e) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPMACSSDD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53861, 467},
    /* 4140 : VPMACSSDQH xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 8f /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(8f) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPMACSSDQH, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53877, 467},
    /* 4141 : VPMACSSDQL xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 87 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(87) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPMACSSDQL, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53893, 467},
    /* 4142 : VPMACSSWD xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 86 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(86) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPMACSSWD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53909, 467},
    /* 4143 : VPMACSSWW xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 85 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(85) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPMACSSWW, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53925, 467},
    /* 4144 : VPMACSWD xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 96 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(96) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPMACSWD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53941, 467},
    /* 4145 : VPMACSWW xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 95 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(95) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPMACSWW, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53957, 467},
    /* 4146 : VPMADCSSWD xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 a6 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(a6) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPMADCSSWD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53973, 467},
    /* 4147 : VPMADCSWD xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 b6 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(b6) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPMADCSWD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53989, 467},
    /* 4148 : VPPERM xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     xop.m8.w1.nds.l0.p0 a3 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 200) 001(a3) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPPERM, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+54005, 467},
    /* 4149 : VPPERM xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 a3 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(a3) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPPERM, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+54021, 467},
    /* 4150 : VPROTB xmmreg,xmmrm128*,xmmreg [rmv:      xop.m9.w0.nds.l0.p0 90 /r         ] SSE5,FUTURE,AMD */
        /* 262(111 000) 001(90) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPROTB, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62745, 467},
    /* 4151 : VPROTB xmmreg,xmmreg*,xmmrm128 [rvm:      xop.m9.w1.nds.l0.p0 90 /r         ] SSE5,FUTURE,AMD */
        /* 261(111 200) 001(90) 120 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPROTB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62759, 467},
    /* 4152 : VPROTB xmmreg,xmmrm128*,imm8 [rmi:      xop.m8.w0.l0.p0 c0 /r ib          ] SSE5,FUTURE,AMD */
        /* 270(110 000) 001(c0) 110 022 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPROTB, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54037, 467},
    /* 4153 : VPROTD xmmreg,xmmrm128*,xmmreg [rmv:      xop.m9.w0.nds.l0.p0 92 /r         ] SSE5,FUTURE,AMD */
        /* 262(111 000) 001(92) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPROTD, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62773, 467},
    /* 4154 : VPROTD xmmreg,xmmreg*,xmmrm128 [rvm:      xop.m9.w1.nds.l0.p0 92 /r         ] SSE5,FUTURE,AMD */
        /* 261(111 200) 001(92) 120 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPROTD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62787, 467},
    /* 4155 : VPROTD xmmreg,xmmrm128*,imm8 [rmi:      xop.m8.w0.l0.p0 c2 /r ib          ] SSE5,FUTURE,AMD */
        /* 270(110 000) 001(c2) 110 022 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPROTD, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54053, 467},
    /* 4156 : VPROTQ xmmreg,xmmrm128*,xmmreg [rmv:      xop.m9.w0.nds.l0.p0 93 /r         ] SSE5,FUTURE,AMD */
        /* 262(111 000) 001(93) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPROTQ, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62801, 467},
    /* 4157 : VPROTQ xmmreg,xmmreg*,xmmrm128 [rvm:      xop.m9.w1.nds.l0.p0 93 /r         ] SSE5,FUTURE,AMD */
        /* 261(111 200) 001(93) 120 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPROTQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62815, 467},
    /* 4158 : VPROTQ xmmreg,xmmrm128*,imm8 [rmi:      xop.m8.w0.l0.p0 c3 /r ib          ] SSE5,FUTURE,AMD */
        /* 270(110 000) 001(c3) 110 022 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPROTQ, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54069, 467},
    /* 4159 : VPROTW xmmreg,xmmrm128*,xmmreg [rmv:      xop.m9.w0.nds.l0.p0 91 /r         ] SSE5,FUTURE,AMD */
        /* 262(111 000) 001(91) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPROTW, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62829, 467},
    /* 4160 : VPROTW xmmreg,xmmreg*,xmmrm128 [rvm:      xop.m9.w1.nds.l0.p0 91 /r         ] SSE5,FUTURE,AMD */
        /* 261(111 200) 001(91) 120 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPROTW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62843, 467},
    /* 4161 : VPROTW xmmreg,xmmrm128*,imm8 [rmi:      xop.m8.w0.l0.p0 c1 /r ib          ] SSE5,FUTURE,AMD */
        /* 270(110 000) 001(c1) 110 022 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPROTW, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54085, 467},
    /* 4162 : VPSHAB xmmreg,xmmrm128*,xmmreg [rmv:      xop.m9.w0.nds.l0.p0 98 /r         ] SSE5,FUTURE,AMD */
        /* 262(111 000) 001(98) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHAB, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62857, 467},
    /* 4163 : VPSHAB xmmreg,xmmreg*,xmmrm128 [rvm:      xop.m9.w1.nds.l0.p0 98 /r         ] SSE5,FUTURE,AMD */
        /* 261(111 200) 001(98) 120 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHAB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62871, 467},
    /* 4164 : VPSHAD xmmreg,xmmrm128*,xmmreg [rmv:      xop.m9.w0.nds.l0.p0 9a /r         ] SSE5,FUTURE,AMD */
        /* 262(111 000) 001(9a) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHAD, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62885, 467},
    /* 4165 : VPSHAD xmmreg,xmmreg*,xmmrm128 [rvm:      xop.m9.w1.nds.l0.p0 9a /r         ] SSE5,FUTURE,AMD */
        /* 261(111 200) 001(9a) 120 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHAD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62899, 467},
    /* 4166 : VPSHAQ xmmreg,xmmrm128*,xmmreg [rmv:      xop.m9.w0.nds.l0.p0 9b /r         ] SSE5,FUTURE,AMD */
        /* 262(111 000) 001(9b) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHAQ, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62913, 467},
    /* 4167 : VPSHAQ xmmreg,xmmreg*,xmmrm128 [rvm:      xop.m9.w1.nds.l0.p0 9b /r         ] SSE5,FUTURE,AMD */
        /* 261(111 200) 001(9b) 120 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHAQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62927, 467},
    /* 4168 : VPSHAW xmmreg,xmmrm128*,xmmreg [rmv:      xop.m9.w0.nds.l0.p0 99 /r         ] SSE5,FUTURE,AMD */
        /* 262(111 000) 001(99) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHAW, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62941, 467},
    /* 4169 : VPSHAW xmmreg,xmmreg*,xmmrm128 [rvm:      xop.m9.w1.nds.l0.p0 99 /r         ] SSE5,FUTURE,AMD */
        /* 261(111 200) 001(99) 120 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHAW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62955, 467},
    /* 4170 : VPSHLB xmmreg,xmmrm128*,xmmreg [rmv:      xop.m9.w0.nds.l0.p0 94 /r         ] SSE5,FUTURE,AMD */
        /* 262(111 000) 001(94) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHLB, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62969, 467},
    /* 4171 : VPSHLB xmmreg,xmmreg*,xmmrm128 [rvm:      xop.m9.w1.nds.l0.p0 94 /r         ] SSE5,FUTURE,AMD */
        /* 261(111 200) 001(94) 120 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHLB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62983, 467},
    /* 4172 : VPSHLD xmmreg,xmmrm128*,xmmreg [rmv:      xop.m9.w0.nds.l0.p0 96 /r         ] SSE5,FUTURE,AMD */
        /* 262(111 000) 001(96) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHLD, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62997, 467},
    /* 4173 : VPSHLD xmmreg,xmmreg*,xmmrm128 [rvm:      xop.m9.w1.nds.l0.p0 96 /r         ] SSE5,FUTURE,AMD */
        /* 261(111 200) 001(96) 120 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHLD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63011, 467},
    /* 4174 : VPSHLQ xmmreg,xmmrm128*,xmmreg [rmv:      xop.m9.w0.nds.l0.p0 97 /r         ] SSE5,FUTURE,AMD */
        /* 262(111 000) 001(97) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHLQ, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63025, 467},
    /* 4175 : VPSHLQ xmmreg,xmmreg*,xmmrm128 [rvm:      xop.m9.w1.nds.l0.p0 97 /r         ] SSE5,FUTURE,AMD */
        /* 261(111 200) 001(97) 120 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHLQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63039, 467},
    /* 4176 : VPSHLW xmmreg,xmmrm128*,xmmreg [rmv:      xop.m9.w0.nds.l0.p0 95 /r         ] SSE5,FUTURE,AMD */
        /* 262(111 000) 001(95) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHLW, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63053, 467},
    /* 4177 : VPSHLW xmmreg,xmmreg*,xmmrm128 [rvm:      xop.m9.w1.nds.l0.p0 95 /r         ] SSE5,FUTURE,AMD */
        /* 261(111 200) 001(95) 120 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHLW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63067, 467},
    /* 4178 : VMPSADBW ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.66.0f3a 42 /r ib      ] AVX2,FUTURE */
        /* 261(003 005) 001(42) 120 023 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VMPSADBW, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+54101, 468},
    /* 4179 : VPABSB ymmreg,ymmrm256 [rm:       vex.256.66.0f38 1c /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(1c) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPABSB, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63081, 468},
    /* 4180 : VPABSW ymmreg,ymmrm256 [rm:       vex.256.66.0f38 1d /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(1d) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPABSW, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63088, 468},
    /* 4181 : VPABSD ymmreg,ymmrm256 [rm:       vex.256.66.0f38 1e /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(1e) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPABSD, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63095, 468},
    /* 4182 : VPACKSSWB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 63 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(63) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPACKSSWB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63102, 468},
    /* 4183 : VPACKSSDW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 6b /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(6b) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPACKSSDW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63116, 468},
    /* 4184 : VPACKUSDW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 2b /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(2b) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPACKUSDW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63130, 468},
    /* 4185 : VPACKUSWB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 67 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(67) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPACKUSWB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63144, 468},
    /* 4186 : VPADDB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f fc /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(fc) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPADDB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63158, 468},
    /* 4187 : VPADDW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f fd /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(fd) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPADDW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63172, 468},
    /* 4188 : VPADDD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f fe /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(fe) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPADDD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63186, 468},
    /* 4189 : VPADDQ ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f d4 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(d4) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPADDQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63200, 468},
    /* 4190 : VPADDSB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f ec /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(ec) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPADDSB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63214, 468},
    /* 4191 : VPADDSW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f ed /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(ed) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPADDSW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63228, 468},
    /* 4192 : VPADDUSB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f dc /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(dc) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPADDUSB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63242, 468},
    /* 4193 : VPADDUSW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f dd /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(dd) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPADDUSW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63256, 468},
    /* 4194 : VPALIGNR ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.66.0f3a 0f /r ib      ] AVX2,FUTURE */
        /* 261(003 005) 001(0f) 120 023 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPALIGNR, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+54117, 468},
    /* 4195 : VPAND ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f db /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(db) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPAND, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63270, 468},
    /* 4196 : VPANDN ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f df /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(df) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPANDN, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63284, 468},
    /* 4197 : VPAVGB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f e0 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(e0) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPAVGB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63298, 468},
    /* 4198 : VPAVGW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f e3 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(e3) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPAVGW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63312, 468},
    /* 4199 : VPBLENDVB ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.nds.256.66.0f3a 4c /r /is4    ] AVX2,FUTURE */
        /* 261(003 005) 001(4c) 120 177 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPBLENDVB, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+54133, 468},
    /* 4200 : VPBLENDW ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.66.0f3a 0e /r ib      ] AVX2,FUTURE */
        /* 261(003 005) 001(0e) 120 023 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPBLENDW, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+54149, 468},
    /* 4201 : VPCMPEQB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 74 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(74) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPCMPEQB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63326, 468},
    /* 4202 : VPCMPEQW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 75 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(75) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPCMPEQW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63340, 468},
    /* 4203 : VPCMPEQD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 76 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(76) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPCMPEQD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63354, 468},
    /* 4204 : VPCMPEQQ ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 29 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(29) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPCMPEQQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63368, 468},
    /* 4205 : VPCMPGTB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 64 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(64) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPCMPGTB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63382, 468},
    /* 4206 : VPCMPGTW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 65 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(65) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPCMPGTW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63396, 468},
    /* 4207 : VPCMPGTD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 66 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(66) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPCMPGTD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63410, 468},
    /* 4208 : VPCMPGTQ ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 37 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(37) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPCMPGTQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63424, 468},
    /* 4209 : VPHADDW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 01 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(01) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPHADDW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63438, 468},
    /* 4210 : VPHADDD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 02 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(02) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPHADDD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63452, 468},
    /* 4211 : VPHADDSW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 03 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(03) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPHADDSW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63466, 468},
    /* 4212 : VPHSUBW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 05 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(05) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPHSUBW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63480, 468},
    /* 4213 : VPHSUBD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 06 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(06) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPHSUBD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63494, 468},
    /* 4214 : VPHSUBSW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 07 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(07) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPHSUBSW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63508, 468},
    /* 4215 : VPMADDUBSW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 04 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(04) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMADDUBSW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63522, 468},
    /* 4216 : VPMADDWD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f f5 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(f5) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMADDWD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63536, 468},
    /* 4217 : VPMAXSB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 3c /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(3c) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMAXSB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63550, 468},
    /* 4218 : VPMAXSW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f ee /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(ee) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMAXSW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63564, 468},
    /* 4219 : VPMAXSD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 3d /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(3d) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMAXSD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63578, 468},
    /* 4220 : VPMAXUB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f de /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(de) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMAXUB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63592, 468},
    /* 4221 : VPMAXUW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 3e /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(3e) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMAXUW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63606, 468},
    /* 4222 : VPMAXUD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 3f /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(3f) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMAXUD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63620, 468},
    /* 4223 : VPMINSB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 38 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(38) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMINSB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63634, 468},
    /* 4224 : VPMINSW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f ea /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(ea) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMINSW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63648, 468},
    /* 4225 : VPMINSD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 39 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(39) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMINSD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63662, 468},
    /* 4226 : VPMINUB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f da /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(da) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMINUB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63676, 468},
    /* 4227 : VPMINUW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 3a /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(3a) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMINUW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63690, 468},
    /* 4228 : VPMINUD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 3b /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(3b) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMINUD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63704, 468},
    /* 4229 : VPMOVMSKB reg32,ymmreg [rm:       vex.256.66.0f d7 /r               ] AVX2,FUTURE */
        /* 270(001 005) 001(d7) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVMSKB, 2, {REG_GPR|BITS32|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63718, 468},
    /* 4230 : VPMOVMSKB reg64,ymmreg [rm:       vex.256.66.0f d7 /r               ] AVX2,FUTURE */
        /* 270(001 005) 001(d7) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVMSKB, 2, {REG_GPR|BITS64|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63718, 468},
    /* 4231 : VPMOVSXBW ymmreg,xmmrm128 [rm:       vex.256.66.0f38 20 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(20) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVSXBW, 2, {YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63725, 468},
    /* 4232 : VPMOVSXBD ymmreg,mem64 [rm:       vex.256.66.0f38 21 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(21) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVSXBD, 2, {YMMREG|RN_L16,MEMORY|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63732, 468},
    /* 4233 : VPMOVSXBD ymmreg,xmmreg [rm:       vex.256.66.0f38 21 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(21) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVSXBD, 2, {YMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63732, 468},
    /* 4234 : VPMOVSXBQ ymmreg,mem32 [rm:       vex.256.66.0f38 22 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(22) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVSXBQ, 2, {YMMREG|RN_L16,MEMORY|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63739, 468},
    /* 4235 : VPMOVSXBQ ymmreg,xmmreg [rm:       vex.256.66.0f38 22 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(22) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVSXBQ, 2, {YMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63739, 468},
    /* 4236 : VPMOVSXWD ymmreg,xmmrm128 [rm:       vex.256.66.0f38 23 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(23) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVSXWD, 2, {YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63746, 468},
    /* 4237 : VPMOVSXWQ ymmreg,mem64 [rm:       vex.256.66.0f38 24 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(24) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVSXWQ, 2, {YMMREG|RN_L16,MEMORY|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63753, 468},
    /* 4238 : VPMOVSXWQ ymmreg,xmmreg [rm:       vex.256.66.0f38 24 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(24) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVSXWQ, 2, {YMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63753, 468},
    /* 4239 : VPMOVSXDQ ymmreg,xmmrm128 [rm:       vex.256.66.0f38 25 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(25) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVSXDQ, 2, {YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63760, 468},
    /* 4240 : VPMOVZXBW ymmreg,xmmrm128 [rm:       vex.256.66.0f38 30 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(30) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVZXBW, 2, {YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63767, 468},
    /* 4241 : VPMOVZXBD ymmreg,mem64 [rm:       vex.256.66.0f38 31 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(31) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVZXBD, 2, {YMMREG|RN_L16,MEMORY|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63774, 468},
    /* 4242 : VPMOVZXBD ymmreg,xmmreg [rm:       vex.256.66.0f38 31 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(31) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVZXBD, 2, {YMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63774, 468},
    /* 4243 : VPMOVZXBQ ymmreg,mem32 [rm:       vex.256.66.0f38 32 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(32) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVZXBQ, 2, {YMMREG|RN_L16,MEMORY|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63781, 468},
    /* 4244 : VPMOVZXBQ ymmreg,xmmreg [rm:       vex.256.66.0f38 32 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(32) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVZXBQ, 2, {YMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63781, 468},
    /* 4245 : VPMOVZXWD ymmreg,xmmrm128 [rm:       vex.256.66.0f38 33 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(33) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVZXWD, 2, {YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63788, 468},
    /* 4246 : VPMOVZXWQ ymmreg,mem64 [rm:       vex.256.66.0f38 34 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(34) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVZXWQ, 2, {YMMREG|RN_L16,MEMORY|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63795, 468},
    /* 4247 : VPMOVZXWQ ymmreg,xmmreg [rm:       vex.256.66.0f38 34 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(34) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVZXWQ, 2, {YMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63795, 468},
    /* 4248 : VPMOVZXDQ ymmreg,xmmrm128 [rm:       vex.256.66.0f38 35 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(35) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVZXDQ, 2, {YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63802, 468},
    /* 4249 : VPMULDQ ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 28 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(28) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMULDQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63809, 468},
    /* 4250 : VPMULHRSW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 0b /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(0b) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMULHRSW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63823, 468},
    /* 4251 : VPMULHUW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f e4 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(e4) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMULHUW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63837, 468},
    /* 4252 : VPMULHW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f e5 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(e5) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMULHW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63851, 468},
    /* 4253 : VPMULLW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f d5 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(d5) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMULLW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63865, 468},
    /* 4254 : VPMULLD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 40 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(40) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMULLD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63879, 468},
    /* 4255 : VPMULUDQ ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f f4 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(f4) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMULUDQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63893, 468},
    /* 4256 : VPOR ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f eb /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(eb) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPOR, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63907, 468},
    /* 4257 : VPSADBW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f f6 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(f6) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSADBW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63921, 468},
    /* 4258 : VPSHUFB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 00 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(00) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSHUFB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63935, 468},
    /* 4259 : VPSHUFD ymmreg,ymmrm256,imm8 [rmi:      vex.256.66.0f 70 /r ib            ] AVX2,FUTURE */
        /* 270(001 005) 001(70) 110 022 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSHUFD, 3, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54165, 468},
    /* 4260 : VPSHUFHW ymmreg,ymmrm256,imm8 [rmi:      vex.256.f3.0f 70 /r ib            ] AVX2,FUTURE */
        /* 270(001 006) 001(70) 110 022 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSHUFHW, 3, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54173, 468},
    /* 4261 : VPSHUFLW ymmreg,ymmrm256,imm8 [rmi:      vex.256.f2.0f 70 /r ib            ] AVX2,FUTURE */
        /* 270(001 007) 001(70) 110 022 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSHUFLW, 3, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54181, 468},
    /* 4262 : VPSIGNB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 08 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(08) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSIGNB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63949, 468},
    /* 4263 : VPSIGNW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 09 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(09) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSIGNW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63963, 468},
    /* 4264 : VPSIGND ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 0a /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(0a) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSIGND, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63977, 468},
    /* 4265 : VPSLLDQ ymmreg,ymmreg*,imm8 [vmi:      vex.ndd.256.66.0f 73 /7 ib        ] AVX2,FUTURE */
        /* 260(001 005) 001(73) 217 022 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSLLDQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54189, 468},
    /* 4266 : VPSLLW ymmreg,ymmreg*,xmmrm128 [rvm:      vex.nds.256.66.0f f1 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(f1) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSLLW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63991, 468},
    /* 4267 : VPSLLW ymmreg,ymmreg*,imm8 [vmi:      vex.ndd.256.66.0f 71 /6 ib        ] AVX2,FUTURE */
        /* 260(001 005) 001(71) 216 022 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSLLW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54205, 468},
    /* 4268 : VPSLLD ymmreg,ymmreg*,xmmrm128 [rvm:      vex.nds.256.66.0f f2 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(f2) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSLLD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64005, 468},
    /* 4269 : VPSLLD ymmreg,ymmreg*,imm8 [vmi:      vex.ndd.256.66.0f 72 /6 ib        ] AVX2,FUTURE */
        /* 260(001 005) 001(72) 216 022 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSLLD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54221, 468},
    /* 4270 : VPSLLQ ymmreg,ymmreg*,xmmrm128 [rvm:      vex.nds.256.66.0f f3 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(f3) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSLLQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64019, 468},
    /* 4271 : VPSLLQ ymmreg,ymmreg*,imm8 [vmi:      vex.ndd.256.66.0f 73 /6 ib        ] AVX2,FUTURE */
        /* 260(001 005) 001(73) 216 022 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSLLQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54237, 468},
    /* 4272 : VPSRAW ymmreg,ymmreg*,xmmrm128 [rvm:      vex.nds.256.66.0f e1 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(e1) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSRAW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64033, 468},
    /* 4273 : VPSRAW ymmreg,ymmreg*,imm8 [vmi:      vex.ndd.256.66.0f 71 /4 ib        ] AVX2,FUTURE */
        /* 260(001 005) 001(71) 214 022 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSRAW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54253, 468},
    /* 4274 : VPSRAD ymmreg,ymmreg*,xmmrm128 [rvm:      vex.nds.256.66.0f e2 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(e2) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSRAD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64047, 468},
    /* 4275 : VPSRAD ymmreg,ymmreg*,imm8 [vmi:      vex.ndd.256.66.0f 72 /4 ib        ] AVX2,FUTURE */
        /* 260(001 005) 001(72) 214 022 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSRAD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54269, 468},
    /* 4276 : VPSRLDQ ymmreg,ymmreg*,imm8 [vmi:      vex.ndd.256.66.0f 73 /3 ib        ] AVX2,FUTURE */
        /* 260(001 005) 001(73) 213 022 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSRLDQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54285, 468},
    /* 4277 : VPSRLW ymmreg,ymmreg*,xmmrm128 [rvm:      vex.nds.256.66.0f d1 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(d1) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSRLW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64061, 468},
    /* 4278 : VPSRLW ymmreg,ymmreg*,imm8 [vmi:      vex.ndd.256.66.0f 71 /2 ib        ] AVX2,FUTURE */
        /* 260(001 005) 001(71) 212 022 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSRLW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54301, 468},
    /* 4279 : VPSRLD ymmreg,ymmreg*,xmmrm128 [rvm:      vex.nds.256.66.0f d2 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(d2) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSRLD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64075, 468},
    /* 4280 : VPSRLD ymmreg,ymmreg*,imm8 [vmi:      vex.ndd.256.66.0f 72 /2 ib        ] AVX2,FUTURE */
        /* 260(001 005) 001(72) 212 022 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSRLD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54317, 468},
    /* 4281 : VPSRLQ ymmreg,ymmreg*,xmmrm128 [rvm:      vex.nds.256.66.0f d3 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(d3) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSRLQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64089, 468},
    /* 4282 : VPSRLQ ymmreg,ymmreg*,imm8 [vmi:      vex.ndd.256.66.0f.wig 73 /2 ib    ] AVX2,FUTURE */
        /* 260(001 005) 001(73) 212 022 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSRLQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54333, 468},
    /* 4283 : VPSUBB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f f8 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(f8) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSUBB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64103, 468},
    /* 4284 : VPSUBW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f f9 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(f9) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSUBW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64117, 468},
    /* 4285 : VPSUBD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f fa /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(fa) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSUBD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64131, 468},
    /* 4286 : VPSUBQ ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f fb /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(fb) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSUBQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64145, 468},
    /* 4287 : VPSUBSB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f e8 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(e8) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSUBSB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64159, 468},
    /* 4288 : VPSUBSW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f e9 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(e9) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSUBSW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64173, 468},
    /* 4289 : VPSUBUSB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f d8 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(d8) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSUBUSB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64187, 468},
    /* 4290 : VPSUBUSW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f d9 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(d9) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSUBUSW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64201, 468},
    /* 4291 : VPUNPCKHBW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 68 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(68) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPUNPCKHBW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64215, 468},
    /* 4292 : VPUNPCKHWD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 69 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(69) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPUNPCKHWD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64229, 468},
    /* 4293 : VPUNPCKHDQ ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 6a /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(6a) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPUNPCKHDQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64243, 468},
    /* 4294 : VPUNPCKHQDQ ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 6d /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(6d) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPUNPCKHQDQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64257, 468},
    /* 4295 : VPUNPCKLBW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 60 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(60) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPUNPCKLBW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64271, 468},
    /* 4296 : VPUNPCKLWD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 61 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(61) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPUNPCKLWD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64285, 468},
    /* 4297 : VPUNPCKLDQ ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 62 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(62) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPUNPCKLDQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64299, 468},
    /* 4298 : VPUNPCKLQDQ ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 6c /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(6c) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPUNPCKLQDQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64313, 468},
    /* 4299 : VPXOR ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f ef /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(ef) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPXOR, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64327, 468},
    /* 4300 : VMOVNTDQA ymmreg,mem256 [rm:       vex.256.66.0f38 2a /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(2a) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VMOVNTDQA, 2, {YMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64341, 468},
    /* 4301 : VBROADCASTSS xmmreg,xmmreg [rm:       vex.128.66.0f38.w0 18 /r          ] AVX2,FUTURE */
        /* 270(002 001) 001(18) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VBROADCASTSS, 2, {XMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58601, 469},
    /* 4302 : VBROADCASTSS ymmreg,xmmreg [rm:       vex.256.66.0f38.w0 18 /r          ] AVX2,FUTURE */
        /* 270(002 005) 001(18) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VBROADCASTSS, 2, {YMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58608, 469},
    /* 4303 : VBROADCASTSD ymmreg,xmmreg [rm:       vex.256.66.0f38.w0 19 /r          ] AVX2,FUTURE */
        /* 270(002 005) 001(19) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VBROADCASTSD, 2, {YMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58615, 469},
    /* 4304 : VBROADCASTI128 ymmreg,mem128 [rm:       vex.256.66.0f38.w0 5a /r          ] AVX2,FUTURE */
        /* 270(002 005) 001(5a) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VBROADCASTI128, 2, {YMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64348, 469},
    /* 4305 : VPBLENDD xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.66.0f3a.w0 02 /r ib   ] AVX2,FUTURE */
        /* 261(003 001) 001(02) 120 023 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBLENDD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+54349, 469},
    /* 4306 : VPBLENDD ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.66.0f3a.w0 02 /r ib   ] AVX2,FUTURE */
        /* 261(003 005) 001(02) 120 023 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBLENDD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+54365, 469},
    /* 4307 : VPBROADCASTB xmmreg,mem8 [rm:       vex.128.66.0f38.w0 78 /r          ] AVX2,FUTURE */
        /* 270(002 001) 001(78) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTB, 2, {XMMREG|RN_L16,MEMORY|BITS8|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64355, 469},
    /* 4308 : VPBROADCASTB xmmreg,xmmreg [rm:       vex.128.66.0f38.w0 78 /r          ] AVX2,FUTURE */
        /* 270(002 001) 001(78) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTB, 2, {XMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64355, 469},
    /* 4309 : VPBROADCASTB ymmreg,mem8 [rm:       vex.256.66.0f38.w0 78 /r          ] AVX2,FUTURE */
        /* 270(002 005) 001(78) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTB, 2, {YMMREG|RN_L16,MEMORY|BITS8|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64362, 469},
    /* 4310 : VPBROADCASTB ymmreg,xmmreg [rm:       vex.256.66.0f38.w0 78 /r          ] AVX2,FUTURE */
        /* 270(002 005) 001(78) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTB, 2, {YMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64362, 469},
    /* 4311 : VPBROADCASTW xmmreg,mem16 [rm:       vex.128.66.0f38.w0 79 /r          ] AVX2,FUTURE */
        /* 270(002 001) 001(79) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTW, 2, {XMMREG|RN_L16,MEMORY|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64369, 469},
    /* 4312 : VPBROADCASTW xmmreg,xmmreg [rm:       vex.128.66.0f38.w0 79 /r          ] AVX2,FUTURE */
        /* 270(002 001) 001(79) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTW, 2, {XMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64369, 469},
    /* 4313 : VPBROADCASTW ymmreg,mem16 [rm:       vex.256.66.0f38.w0 79 /r          ] AVX2,FUTURE */
        /* 270(002 005) 001(79) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTW, 2, {YMMREG|RN_L16,MEMORY|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64376, 469},
    /* 4314 : VPBROADCASTW ymmreg,xmmreg [rm:       vex.256.66.0f38.w0 79 /r          ] AVX2,FUTURE */
        /* 270(002 005) 001(79) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTW, 2, {YMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64376, 469},
    /* 4315 : VPBROADCASTD xmmreg,mem32 [rm:       vex.128.66.0f38.w0 58 /r          ] AVX2,FUTURE */
        /* 270(002 001) 001(58) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTD, 2, {XMMREG|RN_L16,MEMORY|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64383, 469},
    /* 4316 : VPBROADCASTD xmmreg,xmmreg [rm:       vex.128.66.0f38.w0 58 /r          ] AVX2,FUTURE */
        /* 270(002 001) 001(58) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTD, 2, {XMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64383, 469},
    /* 4317 : VPBROADCASTD ymmreg,mem32 [rm:       vex.256.66.0f38.w0 58 /r          ] AVX2,FUTURE */
        /* 270(002 005) 001(58) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTD, 2, {YMMREG|RN_L16,MEMORY|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64390, 469},
    /* 4318 : VPBROADCASTD ymmreg,xmmreg [rm:       vex.256.66.0f38.w0 58 /r          ] AVX2,FUTURE */
        /* 270(002 005) 001(58) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTD, 2, {YMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64390, 469},
    /* 4319 : VPBROADCASTQ xmmreg,mem64 [rm:       vex.128.66.0f38.w0 59 /r          ] AVX2,FUTURE */
        /* 270(002 001) 001(59) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTQ, 2, {XMMREG|RN_L16,MEMORY|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64397, 469},
    /* 4320 : VPBROADCASTQ xmmreg,xmmreg [rm:       vex.128.66.0f38.w0 59 /r          ] AVX2,FUTURE */
        /* 270(002 001) 001(59) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTQ, 2, {XMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64397, 469},
    /* 4321 : VPBROADCASTQ ymmreg,mem64 [rm:       vex.256.66.0f38.w0 59 /r          ] AVX2,FUTURE */
        /* 270(002 005) 001(59) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTQ, 2, {YMMREG|RN_L16,MEMORY|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64404, 469},
    /* 4322 : VPBROADCASTQ ymmreg,xmmreg [rm:       vex.256.66.0f38.w0 59 /r          ] AVX2,FUTURE */
        /* 270(002 005) 001(59) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTQ, 2, {YMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64404, 469},
    /* 4323 : VPERMD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.w0 36 /r      ] AVX2,FUTURE */
        /* 261(002 005) 001(36) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPERMD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64411, 469},
    /* 4324 : VPERMPD ymmreg,ymmrm256,imm8 [rmi:      vex.256.66.0f3a.w1 01 /r ib       ] AVX2,FUTURE */
        /* 270(003 205) 001(01) 110 022 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPERMPD, 3, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54381, 469},
    /* 4325 : VPERMPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.w0 16 /r      ] AVX2,FUTURE */
        /* 261(002 005) 001(16) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPERMPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64425, 469},
    /* 4326 : VPERMQ ymmreg,ymmrm256,imm8 [rmi:      vex.256.66.0f3a.w1 00 /r ib       ] AVX2,FUTURE */
        /* 270(003 205) 001(00) 110 022 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPERMQ, 3, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54389, 469},
    /* 4327 : VPERM2I128 ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.66.0f3a.w0 46 /r ib   ] AVX2,FUTURE */
        /* 261(003 005) 001(46) 120 023 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPERM2I128, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+54397, 469},
    /* 4328 : VEXTRACTI128 xmmrm128,ymmreg,imm8 [mri:      vex.256.66.0f3a.w0 39 /r ib       ] AVX2,FUTURE */
        /* 270(003 005) 001(39) 101 022 : NOAPX,VEX,AVX2,FUTURE */
        {I_VEXTRACTI128, 3, {RM_XMM|BITS128|RN_L16,YMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54413, 469},
    /* 4329 : VINSERTI128 ymmreg,ymmreg*,xmmrm128,imm8 [rvmi:     vex.nds.256.66.0f3a.w0 38 /r ib   ] AVX2,FUTURE */
        /* 261(003 005) 001(38) 120 023 : NOAPX,VEX,AVX2,FUTURE */
        {I_VINSERTI128, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+54421, 469},
    /* 4330 : VPMASKMOVD xmmreg,xmmreg*,mem128 [rvm:      vex.nds.128.66.0f38.w0 8c /r      ] AVX2,FUTURE */
        /* 261(002 001) 001(8c) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPMASKMOVD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64439, 469},
    /* 4331 : VPMASKMOVD ymmreg,ymmreg*,mem256 [rvm:      vex.nds.256.66.0f38.w0 8c /r      ] AVX2,FUTURE */
        /* 261(002 005) 001(8c) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPMASKMOVD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64453, 469},
    /* 4332 : VPMASKMOVQ xmmreg,xmmreg*,mem128 [rvm:      vex.nds.128.66.0f38.w1 8c /r      ] AVX2,FUTURE */
        /* 261(002 201) 001(8c) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPMASKMOVQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64467, 469},
    /* 4333 : VPMASKMOVQ ymmreg,ymmreg*,mem256 [rvm:      vex.nds.256.66.0f38.w1 8c /r      ] AVX2,FUTURE */
        /* 261(002 205) 001(8c) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPMASKMOVQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64481, 469},
    /* 4334 : VPMASKMOVD mem128,xmmreg,xmmreg [mvr:      vex.nds.128.66.0f38.w0 8e /r      ] AVX2,FUTURE */
        /* 261(002 001) 001(8e) 102 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPMASKMOVD, 3, {MEMORY|BITS128|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64495, 469},
    /* 4335 : VPMASKMOVD mem256,ymmreg,ymmreg [mvr:      vex.nds.256.66.0f38.w0 8e /r      ] AVX2,FUTURE */
        /* 261(002 005) 001(8e) 102 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPMASKMOVD, 3, {MEMORY|BITS256|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64502, 469},
    /* 4336 : VPMASKMOVQ mem128,xmmreg,xmmreg [mvr:      vex.nds.128.66.0f38.w1 8e /r      ] AVX2,FUTURE */
        /* 261(002 201) 001(8e) 102 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPMASKMOVQ, 3, {MEMORY|BITS128|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64509, 469},
    /* 4337 : VPMASKMOVQ mem256,ymmreg,ymmreg [mvr:      vex.nds.256.66.0f38.w1 8e /r      ] AVX2,FUTURE */
        /* 261(002 205) 001(8e) 102 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPMASKMOVQ, 3, {MEMORY|BITS256|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64516, 469},
    /* 4338 : VPSLLVD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38.w0 47 /r      ] AVX2,FUTURE */
        /* 261(002 001) 001(47) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPSLLVD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64523, 469},
    /* 4339 : VPSLLVQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38.w1 47 /r      ] AVX2,FUTURE */
        /* 261(002 201) 001(47) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPSLLVQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64537, 469},
    /* 4340 : VPSLLVD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.w0 47 /r      ] AVX2,FUTURE */
        /* 261(002 005) 001(47) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPSLLVD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64551, 469},
    /* 4341 : VPSLLVQ ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.w1 47 /r      ] AVX2,FUTURE */
        /* 261(002 205) 001(47) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPSLLVQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64565, 469},
    /* 4342 : VPSRAVD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38.w0 46 /r      ] AVX2,FUTURE */
        /* 261(002 001) 001(46) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPSRAVD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64579, 469},
    /* 4343 : VPSRAVD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.w0 46 /r      ] AVX2,FUTURE */
        /* 261(002 005) 001(46) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPSRAVD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64593, 469},
    /* 4344 : VPSRLVD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38.w0 45 /r      ] AVX2,FUTURE */
        /* 261(002 001) 001(45) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPSRLVD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64607, 469},
    /* 4345 : VPSRLVQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38.w1 45 /r      ] AVX2,FUTURE */
        /* 261(002 201) 001(45) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPSRLVQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64621, 469},
    /* 4346 : VPSRLVD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.w0 45 /r      ] AVX2,FUTURE */
        /* 261(002 005) 001(45) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPSRLVD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64635, 469},
    /* 4347 : VPSRLVQ ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.w1 45 /r      ] AVX2,FUTURE */
        /* 261(002 205) 001(45) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPSRLVQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64649, 469},
    /* 4348 : VGATHERDPD xmmreg,xmem64,xmmreg [rmv:      vm32x vex.dds.128.66.0f38.w1 92 /r] AVX2,FUTURE */
        /* 374 262(002 201) 001(92) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VGATHERDPD, 3, {XMMREG|RN_L16,XMEM|BITS64|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54437, 469},
    /* 4349 : VGATHERQPD xmmreg,xmem64,xmmreg [rmv:      vm64x vex.dds.128.66.0f38.w1 93 /r] AVX2,FUTURE */
        /* 374 262(002 201) 001(93) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VGATHERQPD, 3, {XMMREG|RN_L16,XMEM|BITS64|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54445, 469},
    /* 4350 : VGATHERDPD ymmreg,xmem64,ymmreg [rmv:      vm32x vex.dds.256.66.0f38.w1 92 /r] AVX2,FUTURE */
        /* 374 262(002 205) 001(92) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VGATHERDPD, 3, {YMMREG|RN_L16,XMEM|BITS64|RN_L16,YMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54453, 469},
    /* 4351 : VGATHERQPD ymmreg,ymem64,ymmreg [rmv:      vm64y vex.dds.256.66.0f38.w1 93 /r] AVX2,FUTURE */
        /* 375 262(002 205) 001(93) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VGATHERQPD, 3, {YMMREG|RN_L16,YMEM|BITS64|RN_L16,YMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54461, 469},
    /* 4352 : VGATHERDPS xmmreg,xmem32,xmmreg [rmv:      vm32x vex.dds.128.66.0f38.w0 92 /r] AVX2,FUTURE */
        /* 374 262(002 001) 001(92) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VGATHERDPS, 3, {XMMREG|RN_L16,XMEM|BITS32|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54469, 469},
    /* 4353 : VGATHERQPS xmmreg,xmem32,xmmreg [rmv:      vm64x vex.dds.128.66.0f38.w0 93 /r] AVX2,FUTURE */
        /* 374 262(002 001) 001(93) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VGATHERQPS, 3, {XMMREG|RN_L16,XMEM|BITS32|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54477, 469},
    /* 4354 : VGATHERDPS ymmreg,ymem32,ymmreg [rmv:      vm32y vex.dds.256.66.0f38.w0 92 /r] AVX2,FUTURE */
        /* 375 262(002 005) 001(92) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VGATHERDPS, 3, {YMMREG|RN_L16,YMEM|BITS32|RN_L16,YMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54485, 469},
    /* 4355 : VGATHERQPS xmmreg,ymem32,xmmreg [rmv:      vm64y vex.dds.256.66.0f38.w0 93 /r] AVX2,FUTURE */
        /* 375 262(002 005) 001(93) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VGATHERQPS, 3, {XMMREG|RN_L16,YMEM|BITS32|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54493, 469},
    /* 4356 : VPGATHERDD xmmreg,xmem32,xmmreg [rmv:      vm32x vex.dds.128.66.0f38.w0 90 /r] AVX2,FUTURE */
        /* 374 262(002 001) 001(90) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPGATHERDD, 3, {XMMREG|RN_L16,XMEM|BITS32|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54501, 469},
    /* 4357 : VPGATHERQD xmmreg,xmem32,xmmreg [rmv:      vm64x vex.dds.128.66.0f38.w0 91 /r] AVX2,FUTURE */
        /* 374 262(002 001) 001(91) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPGATHERQD, 3, {XMMREG|RN_L16,XMEM|BITS32|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54509, 469},
    /* 4358 : VPGATHERDD ymmreg,ymem32,ymmreg [rmv:      vm32y vex.dds.256.66.0f38.w0 90 /r] AVX2,FUTURE */
        /* 375 262(002 005) 001(90) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPGATHERDD, 3, {YMMREG|RN_L16,YMEM|BITS32|RN_L16,YMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54517, 469},
    /* 4359 : VPGATHERQD xmmreg,ymem32,xmmreg [rmv:      vm64y vex.dds.256.66.0f38.w0 91 /r] AVX2,FUTURE */
        /* 375 262(002 005) 001(91) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPGATHERQD, 3, {XMMREG|RN_L16,YMEM|BITS32|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54525, 469},
    /* 4360 : VPGATHERDQ xmmreg,xmem64,xmmreg [rmv:      vm32x vex.dds.128.66.0f38.w1 90 /r] AVX2,FUTURE */
        /* 374 262(002 201) 001(90) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPGATHERDQ, 3, {XMMREG|RN_L16,XMEM|BITS64|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54533, 469},
    /* 4361 : VPGATHERQQ xmmreg,xmem64,xmmreg [rmv:      vm64x vex.dds.128.66.0f38.w1 91 /r] AVX2,FUTURE */
        /* 374 262(002 201) 001(91) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPGATHERQQ, 3, {XMMREG|RN_L16,XMEM|BITS64|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54541, 469},
    /* 4362 : VPGATHERDQ ymmreg,xmem64,ymmreg [rmv:      vm32x vex.dds.256.66.0f38.w1 90 /r] AVX2,FUTURE */
        /* 374 262(002 205) 001(90) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPGATHERDQ, 3, {YMMREG|RN_L16,XMEM|BITS64|RN_L16,YMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54549, 469},
    /* 4363 : VPGATHERQQ ymmreg,ymem64,ymmreg [rmv:      vm64y vex.dds.256.66.0f38.w1 91 /r] AVX2,FUTURE */
        /* 375 262(002 205) 001(91) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPGATHERQQ, 3, {YMMREG|RN_L16,YMEM|BITS64|RN_L16,YMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54557, 469},
    /* 4364 : XABORT imm8 [i:        c6 f8 ib                          ] RTM,FUTURE */
        /* 002(c6 f8) 020 : RTM,FUTURE */
        {I_XABORT, 1, {IMM_NORMAL|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72368, 470},
    /* 4365 : XBEGIN imm16|near [i:        nw o16 c7 f8 rel                  ] AR0,OSIZE,NOREX,NOAPX,NOLONG,RTM,FUTURE */
        /* 327 320 002(c7 f8) 064 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,RTM,FUTURE */
        {I_XBEGIN, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+64663, 471},
    /* 4366 : XBEGIN imm32|near [i:        nw o32 c7 f8 rel                  ] AR0,OSIZE,NOREX,NOAPX,NOLONG,RTM,FUTURE */
        /* 327 321 002(c7 f8) 064 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,RTM,FUTURE */
        {I_XBEGIN, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+64670, 471},
    /* 4367 : XBEGIN imm64|near [i:        nw o64 c7 f8 rel                  ] AR0,OSIZE,LONG,PROT,RTM,X86_64 */
        /* 327 324 002(c7 f8) 064 : AR0,NWSIZE,OSIZE,LONG,PROT,RTM,X86_64 */
        {I_XBEGIN, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+64677, 472},
    /* 4368 : XEND void [          0f 01 d5                          ] RTM,FUTURE */
        /* 355 002(01 d5) : RTM,FUTURE */
        {I_XEND, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72373, 470},
    /* 4369 : XTEST void [          0f 01 d6                          ] HLE,FL,RTM,FUTURE */
        /* 355 002(01 d6) : HLE,FL,RTM,FUTURE */
        {I_XTEST, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72378, 474},
    /* 4370 : PREFETCHWT1 mem8 [m:        0f 0d /2                          ] PREFETCHWT1,FUTURE */
        /* 355 001(0d) 202 : PREFETCHWT1,FUTURE */
        {I_PREFETCHWT1, 1, {MEMORY|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72383, 475},
    /* 4371 : BNDMK bndreg,mem [rm:       f3 0f 1b /r                       ] MIB,MPX,FUTURE */
        /* 333 355 001(1b) 110 : MIB,MPX,FUTURE */
        {I_BNDMK, 2, {BNDREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+69872, 476},
    /* 4372 : BNDCL bndreg,mem [rm:       f3 0f 1a /r                       ] MPX,FUTURE */
        /* 333 355 001(1a) 110 : MPX,FUTURE */
        {I_BNDCL, 2, {BNDREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+64685, 477},
    /* 4373 : BNDCL bndreg,reg32 [rm:       f3 0f 1a /r                       ] NOREX,NOAPX,NOLONG,MPX,FUTURE */
        /* 333 355 001(1a) 110 : NOREX,NOAPX,NOLONG,MPX,FUTURE */
        {I_BNDCL, 2, {BNDREG|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64685, 478},
    /* 4374 : BNDCL bndreg,reg64 [rm:       o64nw f3 0f 1a /r                 ] LONG,PROT,MPX,X86_64 */
        /* 323 333 355 001(1a) 110 : NWSIZE,LONG,PROT,MPX,X86_64 */
        {I_BNDCL, 2, {BNDREG,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+64684, 479},
    /* 4375 : BNDCU bndreg,mem [rm:       f2 0f 1a /r                       ] MPX,FUTURE */
        /* 332 355 001(1a) 110 : MPX,FUTURE */
        {I_BNDCU, 2, {BNDREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+64692, 477},
    /* 4376 : BNDCU bndreg,reg32 [rm:       f2 0f 1a /r                       ] NOREX,NOAPX,NOLONG,MPX,FUTURE */
        /* 332 355 001(1a) 110 : NOREX,NOAPX,NOLONG,MPX,FUTURE */
        {I_BNDCU, 2, {BNDREG|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64692, 478},
    /* 4377 : BNDCU bndreg,reg64 [rm:       o64nw f2 0f 1a /r                 ] LONG,PROT,MPX,X86_64 */
        /* 323 332 355 001(1a) 110 : NWSIZE,LONG,PROT,MPX,X86_64 */
        {I_BNDCU, 2, {BNDREG,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+64691, 479},
    /* 4378 : BNDCN bndreg,mem [rm:       f2 0f 1b /r                       ] MPX,FUTURE */
        /* 332 355 001(1b) 110 : MPX,FUTURE */
        {I_BNDCN, 2, {BNDREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+64699, 477},
    /* 4379 : BNDCN bndreg,reg32 [rm:       f2 0f 1b /r                       ] NOREX,NOAPX,NOLONG,MPX,FUTURE */
        /* 332 355 001(1b) 110 : NOREX,NOAPX,NOLONG,MPX,FUTURE */
        {I_BNDCN, 2, {BNDREG|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64699, 478},
    /* 4380 : BNDCN bndreg,reg64 [rm:       o64nw f2 0f 1b /r                 ] LONG,PROT,MPX,X86_64 */
        /* 323 332 355 001(1b) 110 : NWSIZE,LONG,PROT,MPX,X86_64 */
        {I_BNDCN, 2, {BNDREG,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+64698, 479},
    /* 4381 : BNDMOV bndreg,bndreg [rm:       66 0f 1a /r                       ] MPX,FUTURE */
        /* 361 355 001(1a) 110 : MPX,FUTURE */
        {I_BNDMOV, 2, {BNDREG,BNDREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+69878, 477},
    /* 4382 : BNDMOV bndreg,mem [rm:       66 0f 1a /r                       ] MPX,FUTURE */
        /* 361 355 001(1a) 110 : MPX,FUTURE */
        {I_BNDMOV, 2, {BNDREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+69878, 477},
    /* 4383 : BNDMOV bndreg,bndreg [mr:       66 0f 1b /r                       ] MPX,FUTURE */
        /* 361 355 001(1b) 101 : MPX,FUTURE */
        {I_BNDMOV, 2, {BNDREG,BNDREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+69884, 477},
    /* 4384 : BNDMOV mem,bndreg [mr:       66 0f 1b /r                       ] MPX,FUTURE */
        /* 361 355 001(1b) 101 : MPX,FUTURE */
        {I_BNDMOV, 2, {MEMORY,BNDREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+69884, 477},
    /* 4385 : BNDLDX bndreg,mem [rm:       0f 1a /r                          ] MIB,MPX,FUTURE */
        /* 355 001(1a) 110 : MIB,MPX,FUTURE */
        {I_BNDLDX, 2, {BNDREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+69879, 476},
    /* 4386 : BNDLDX bndreg,mem,reg32 [rmx:      0f 1a /r                          ] NOREX,NOAPX,NOLONG,MIB,MPX,FUTURE */
        /* 355 001(1a) 016 110 : NOREX,NOAPX,NOLONG,MIB,MPX,FUTURE */
        {I_BNDLDX, 3, {BNDREG|RN_L16,MEMORY|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+69890, 480},
    /* 4387 : BNDLDX bndreg,mem,reg64 [rmx:      0f 1a /r                          ] LONG,MIB,PROT,MPX,X86_64 */
        /* 355 001(1a) 016 110 : LONG,MIB,PROT,MPX,X86_64 */
        {I_BNDLDX, 3, {BNDREG,MEMORY,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+69890, 481},
    /* 4388 : BNDSTX mem,bndreg [mr:       0f 1b /r                          ] MIB,MPX,FUTURE */
        /* 355 001(1b) 101 : MIB,MPX,FUTURE */
        {I_BNDSTX, 2, {MEMORY,BNDREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+69885, 476},
    /* 4389 : BNDSTX mem,reg32,bndreg [mxr:      0f 1b /r                          ] NOREX,NOAPX,NOLONG,MIB,MPX,FUTURE */
        /* 355 001(1b) 015 102 : NOREX,NOAPX,NOLONG,MIB,MPX,FUTURE */
        {I_BNDSTX, 3, {MEMORY|RN_L16,REG_GPR|BITS32|RN_L16,BNDREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+69896, 480},
    /* 4390 : BNDSTX mem,reg64,bndreg [mxr:      0f 1b /r                          ] LONG,MIB,PROT,MPX,X86_64 */
        /* 355 001(1b) 015 102 : LONG,MIB,PROT,MPX,X86_64 */
        {I_BNDSTX, 3, {MEMORY,REG_GPR|BITS64,BNDREG,0,0}, NO_DECORATOR, nasm_bytecodes+69896, 481},
    /* 4391 : BNDSTX mem,bndreg,reg32 [mrx:      0f 1b /r                          ] NOREX,NOAPX,NOLONG,MIB,MPX,FUTURE */
        /* 355 001(1b) 016 101 : NOREX,NOAPX,NOLONG,MIB,MPX,FUTURE */
        {I_BNDSTX, 3, {MEMORY|RN_L16,BNDREG|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+69902, 480},
    /* 4392 : BNDSTX mem,bndreg,reg64 [mrx:      0f 1b /r                          ] LONG,MIB,PROT,MPX,X86_64 */
        /* 355 001(1b) 016 101 : LONG,MIB,PROT,MPX,X86_64 */
        {I_BNDSTX, 3, {MEMORY,BNDREG,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+69902, 481},
    /* 4393 : SHA1MSG1 xmmreg,xmmrm128 [rm:       0f38 c9 /r                        ] SHA,FUTURE */
        /* 356 001(c9) 110 : NOAPX,SHA,FUTURE */
        {I_SHA1MSG1, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72388, 482},
    /* 4394 : SHA1MSG2 xmmreg,xmmrm128 [rm:       0f38 ca /r                        ] SHA,FUTURE */
        /* 356 001(ca) 110 : NOAPX,SHA,FUTURE */
        {I_SHA1MSG2, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72393, 482},
    /* 4395 : SHA1NEXTE xmmreg,xmmrm128 [rm:       0f38 c8 /r                        ] SHA,FUTURE */
        /* 356 001(c8) 110 : NOAPX,SHA,FUTURE */
        {I_SHA1NEXTE, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72398, 482},
    /* 4396 : SHA1RNDS4 xmmreg,xmmrm128,imm8 [rmi:      0f3a cc /r ib                     ] SHA,FUTURE */
        /* 357 001(cc) 110 022 : NOAPX,SHA,FUTURE */
        {I_SHA1RNDS4, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+69908, 482},
    /* 4397 : SHA256MSG1 xmmreg,xmmrm128 [rm:       0f38 cc /r                        ] SHA,FUTURE */
        /* 356 001(cc) 110 : NOAPX,SHA,FUTURE */
        {I_SHA256MSG1, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72403, 482},
    /* 4398 : SHA256MSG2 xmmreg,xmmrm128 [rm:       0f38 cd /r                        ] SHA,FUTURE */
        /* 356 001(cd) 110 : NOAPX,SHA,FUTURE */
        {I_SHA256MSG2, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72408, 482},
    /* 4399 : SHA256RNDS2 xmmreg,xmmrm128,xmm0 [rm-:      0f38 cb /r                        ] SHA,FUTURE */
        /* 356 001(cb) 110 : NOAPX,SHA,FUTURE */
        {I_SHA256RNDS2, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_ZERO|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+72413, 482},
    /* 4400 : SHA256RNDS2 xmmreg,xmmrm128 [rm:       0f38 cb /r                        ] SHA,FUTURE */
        /* 356 001(cb) 110 : NOAPX,SHA,FUTURE */
        {I_SHA256RNDS2, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72413, 482},
    /* 4401 : VSHA512MSG1 ymmreg,xmmreg [rm:       vex.256.f2.0f38.w0 cc /r          ] AVX,SHA512,FUTURE */
        /* 270(002 007) 001(cc) 110 : NOAPX,VEX,AVX,SHA512,FUTURE */
        {I_VSHA512MSG1, 2, {YMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64705, 483},
    /* 4402 : VSHA512MSG2 ymmreg,ymmreg [rm:       vex.256.f2.0f38.w0 cd /r          ] AVX,SHA512,FUTURE */
        /* 270(002 007) 001(cd) 110 : NOAPX,VEX,AVX,SHA512,FUTURE */
        {I_VSHA512MSG2, 2, {YMMREG|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64712, 483},
    /* 4403 : VSHA512RNDS2 ymmreg,ymmreg,xmmreg [rvm:      vex.nds.256.f2.0f38.w0 cb /r      ] AVX,SHA512,FUTURE */
        /* 261(002 007) 001(cb) 120 : NOAPX,VEX,AVX,SHA512,FUTURE */
        {I_VSHA512RNDS2, 3, {YMMREG|RN_L16,YMMREG|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64719, 483},
    /* 4404 : VSM3MSG1 xmmreg,xmmreg,xmmreg [rvm:      vex.nds.128.p0.0f38.w0 da /r      ] AVX,HSM3,FUTURE */
        /* 261(002 000) 001(da) 120 : NOAPX,VEX,AVX,HSM3,FUTURE */
        {I_VSM3MSG1, 3, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64726, 484},
    /* 4405 : VSM3MSG2 xmmreg,xmmreg,xmmreg [rvm:      vex.nds.128.66.0f38.w0 da /r      ] AVX,HSM3,FUTURE */
        /* 261(002 001) 001(da) 120 : NOAPX,VEX,AVX,HSM3,FUTURE */
        {I_VSM3MSG2, 3, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64733, 484},
    /* 4406 : VSM3RNDS2 xmmreg,xmmreg,xmmreg,imm8 [rvmi:     vex.nds.128.66.0f3a.w0 de /r ib   ] AVX,HSM3,FUTURE */
        /* 261(003 001) 001(de) 120 023 : NOAPX,VEX,AVX,HSM3,FUTURE */
        {I_VSM3RNDS2, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+54565, 484},
    /* 4407 : VSM4KEY4 xmmreg,xmmreg,xmmrm128 [rvm:      vex.nds.128.f3.0f38.w0 da /r      ] AVX,HSM4,FUTURE */
        /* 261(002 002) 001(da) 120 : NOAPX,VEX,AVX,HSM4,FUTURE */
        {I_VSM4KEY4, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64740, 485},
    /* 4408 : VSM4KEY4 ymmreg,ymmreg,ymmrm256 [rvm:      vex.nds.256.f3.0f38.w0 da /r      ] AVX,HSM4,FUTURE */
        /* 261(002 006) 001(da) 120 : NOAPX,VEX,AVX,HSM4,FUTURE */
        {I_VSM4KEY4, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64747, 485},
    /* 4409 : VSM4KEY4 xmmreg,xmmreg,xmmrm128 [rvm:      evex.nds.128.f3.0f38.w0 da /r     ] AVX,HSM4,FUTURE */
        /* 241(f2 7e 08) 300 001(da) 120 : EVEX,PROT,AVX,HSM4,FUTURE */
        {I_VSM4KEY4, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, NO_DECORATOR, nasm_bytecodes+29610, 486},
    /* 4410 : VSM4KEY4 ymmreg,ymmreg,ymmrm256 [rvm:      evex.nds.256.f3.0f38.w0 da /r     ] AVX,HSM4,FUTURE */
        /* 241(f2 7e 28) 300 001(da) 120 : EVEX,PROT,AVX,HSM4,FUTURE */
        {I_VSM4KEY4, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, NO_DECORATOR, nasm_bytecodes+29619, 486},
    /* 4411 : VSM4KEY4 zmmreg,zmmreg,zmmrm512 [rvm:      evex.nds.512.f3.0f38.w0 da /r     ] AVX,HSM4,FUTURE */
        /* 241(f2 7e 48) 300 001(da) 120 : EVEX,PROT,AVX,HSM4,FUTURE */
        {I_VSM4KEY4, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, NO_DECORATOR, nasm_bytecodes+29628, 486},
    /* 4412 : VSM4RNDS4 xmmreg,xmmreg,xmmrm128 [rvm:      vex.nds.128.f2.0f38.w0 da /r      ] AVX,HSM4,FUTURE */
        /* 261(002 003) 001(da) 120 : NOAPX,VEX,AVX,HSM4,FUTURE */
        {I_VSM4RNDS4, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64754, 485},
    /* 4413 : VSM4RNDS4 ymmreg,ymmreg,ymmrm256 [rvm:      vex.nds.256.f2.0f38.w0 da /r      ] AVX,HSM4,FUTURE */
        /* 261(002 007) 001(da) 120 : NOAPX,VEX,AVX,HSM4,FUTURE */
        {I_VSM4RNDS4, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64761, 485},
    /* 4414 : VSM4RNDS4 xmmreg,xmmreg,xmmrm128 [rvm:      evex.nds.128.f2.0f38.w0 da /r     ] AVX,HSM4,FUTURE */
        /* 241(f2 7f 08) 300 001(da) 120 : EVEX,PROT,AVX,HSM4,FUTURE */
        {I_VSM4RNDS4, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, NO_DECORATOR, nasm_bytecodes+29637, 486},
    /* 4415 : VSM4RNDS4 ymmreg,ymmreg,ymmrm256 [rvm:      evex.nds.256.f2.0f38.w0 da /r     ] AVX,HSM4,FUTURE */
        /* 241(f2 7f 28) 300 001(da) 120 : EVEX,PROT,AVX,HSM4,FUTURE */
        {I_VSM4RNDS4, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, NO_DECORATOR, nasm_bytecodes+29646, 486},
    /* 4416 : VSM4RNDS4 zmmreg,zmmreg,zmmrm512 [rvm:      evex.nds.512.f2.0f38.w0 da /r     ] AVX,HSM4,FUTURE */
        /* 241(f2 7f 48) 300 001(da) 120 : EVEX,PROT,AVX,HSM4,FUTURE */
        {I_VSM4RNDS4, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, NO_DECORATOR, nasm_bytecodes+29655, 486},
    /* 4417 : VBCSTNEBF16PS xmmreg,mem16 [rm:       vex.128.f3.0f38.w0 b1 /r          ] AR0-1,SW,AVXNECONVERT,FUTURE */
        /* 270(002 002) 001(b1) 110 : AR0,AR1,SW,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VBCSTNEBF16PS, 2, {XMMREG|RN_L16,MEMORY|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64768, 487},
    /* 4418 : VBCSTNEBF16PS ymmreg,mem16 [rm:       vex.256.f3.0f38.w0 b1 /r          ] AR0-1,SW,AVXNECONVERT,FUTURE */
        /* 270(002 006) 001(b1) 110 : AR0,AR1,SW,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VBCSTNEBF16PS, 2, {YMMREG|RN_L16,MEMORY|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64775, 487},
    /* 4419 : VBCSTNEBF162PS xmmreg,mem16 [rm:       vex.128.f3.0f38.w0 b1 /r          ] AR0-1,SW,AVXNECONVERT,FUTURE */
        /* 270(002 002) 001(b1) 110 : AR0,AR1,SW,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VBCSTNEBF162PS, 2, {XMMREG|RN_L16,MEMORY|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64768, 487},
    /* 4420 : VBCSTNEBF162PS ymmreg,mem16 [rm:       vex.256.f3.0f38.w0 b1 /r          ] AR0-1,SW,AVXNECONVERT,FUTURE */
        /* 270(002 006) 001(b1) 110 : AR0,AR1,SW,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VBCSTNEBF162PS, 2, {YMMREG|RN_L16,MEMORY|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64775, 487},
    /* 4421 : VBCSTNESH2PS xmmreg,mem16 [rm:       vex.128.66.0f38.w0 b1 /r          ] AR0-1,SW,AVXNECONVERT,FUTURE */
        /* 270(002 001) 001(b1) 110 : AR0,AR1,SW,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VBCSTNESH2PS, 2, {XMMREG|RN_L16,MEMORY|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64782, 487},
    /* 4422 : VBCSTNESH2PS ymmreg,mem16 [rm:       vex.256.66.0f38.w0 b1 /r          ] AR0-1,SW,AVXNECONVERT,FUTURE */
        /* 270(002 005) 001(b1) 110 : AR0,AR1,SW,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VBCSTNESH2PS, 2, {YMMREG|RN_L16,MEMORY|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64789, 487},
    /* 4423 : VCVTNEEBF162PS xmmreg,mem128 [rm:       vex.128.f3.0f38.w0 b0 /r          ] AR0-1,SO,AVXNECONVERT,FUTURE */
        /* 270(002 002) 001(b0) 110 : AR0,AR1,SO,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VCVTNEEBF162PS, 2, {XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64796, 488},
    /* 4424 : VCVTNEEBF162PS ymmreg,mem256 [rm:       vex.256.f3.0f38.w0 b0 /r          ] AR0-1,SY,AVXNECONVERT,FUTURE */
        /* 270(002 006) 001(b0) 110 : AR0,AR1,SY,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VCVTNEEBF162PS, 2, {YMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64803, 489},
    /* 4425 : VCVTNEEPH2PS xmmreg,mem128 [rm:       vex.128.66.0f38.w0 b0 /r          ] AR0-1,SO,AVXNECONVERT,FUTURE */
        /* 270(002 001) 001(b0) 110 : AR0,AR1,SO,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VCVTNEEPH2PS, 2, {XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64810, 488},
    /* 4426 : VCVTNEEPH2PS ymmreg,mem256 [rm:       vex.256.66.0f38.w0 b0 /r          ] AR0-1,SY,AVXNECONVERT,FUTURE */
        /* 270(002 005) 001(b0) 110 : AR0,AR1,SY,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VCVTNEEPH2PS, 2, {YMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64817, 489},
    /* 4427 : VCVTNEOBF162PS xmmreg,mem128 [rm:       vex.128.f2.0f38.w0 b0 /r          ] AR0-1,SO,AVXNECONVERT,FUTURE */
        /* 270(002 003) 001(b0) 110 : AR0,AR1,SO,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VCVTNEOBF162PS, 2, {XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64824, 488},
    /* 4428 : VCVTNEOBF162PS ymmreg,mem256 [rm:       vex.256.f2.0f38.w0 b0 /r          ] AR0-1,SY,AVXNECONVERT,FUTURE */
        /* 270(002 007) 001(b0) 110 : AR0,AR1,SY,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VCVTNEOBF162PS, 2, {YMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64831, 489},
    /* 4429 : VCVTNEOPH2PS xmmreg,mem128 [rm:       vex.128.np.0f38.w0 b0 /r          ] AR0-1,SO,AVXNECONVERT,FUTURE */
        /* 270(002 000) 001(b0) 110 : AR0,AR1,SO,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VCVTNEOPH2PS, 2, {XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64838, 488},
    /* 4430 : VCVTNEOPH2PS ymmreg,mem256 [rm:       vex.256.np.0f38.w0 b0 /r          ] AR0-1,SY,AVXNECONVERT,FUTURE */
        /* 270(002 004) 001(b0) 110 : AR0,AR1,SY,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VCVTNEOPH2PS, 2, {YMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64845, 489},
    /* 4431 : VCVTNEPS2BF16 xmmreg,xmmrm128 [rm:       vex.128.f3.0f38.w0 72 /r          ] AR0-1,SO,LATEVEX,AVXNECONVERT,FUTURE */
        /* 270(002 002) 001(72) 110 : AR0,AR1,SO,LATEVEX,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VCVTNEPS2BF16, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64852, 490},
    /* 4432 : VCVTNEPS2BF16 xmmreg,ymmrm256 [rm:       vex.256.f3.0f38.w0 72 /r          ] AR0-1,SY,LATEVEX,AVXNECONVERT,FUTURE */
        /* 270(002 006) 001(72) 110 : AR0,AR1,SY,LATEVEX,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VCVTNEPS2BF16, 2, {XMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64859, 491},
    /* 4433 : VPDPBUSD xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.66.0f38.w0 50 /r          ] AR0-2,SO,LATEVEX,AVXVNNI,FUTURE */
        /* 261(002 001) 001(50) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVXVNNI,FUTURE */
        {I_VPDPBUSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64866, 492},
    /* 4434 : VPDPBUSD ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.66.0f38.w0 50 /r          ] AR0-2,SY,LATEVEX,AVXVNNI,FUTURE */
        /* 261(002 005) 001(50) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVXVNNI,FUTURE */
        {I_VPDPBUSD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64873, 493},
    /* 4435 : VPDPBUSDS xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.66.0f38.w0 51 /r          ] AR0-2,SO,LATEVEX,AVXVNNI,FUTURE */
        /* 261(002 001) 001(51) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVXVNNI,FUTURE */
        {I_VPDPBUSDS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64880, 492},
    /* 4436 : VPDPBUSDS ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.66.0f38.w0 51 /r          ] AR0-2,SY,LATEVEX,AVXVNNI,FUTURE */
        /* 261(002 005) 001(51) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVXVNNI,FUTURE */
        {I_VPDPBUSDS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64887, 493},
    /* 4437 : VPDPWSSD xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.66.0f38.w0 52 /r          ] AR0-2,SO,LATEVEX,AVXVNNI,FUTURE */
        /* 261(002 001) 001(52) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVXVNNI,FUTURE */
        {I_VPDPWSSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64894, 492},
    /* 4438 : VPDPWSSD ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.66.0f38.w0 52 /r          ] AR0-2,SY,LATEVEX,AVXVNNI,FUTURE */
        /* 261(002 005) 001(52) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVXVNNI,FUTURE */
        {I_VPDPWSSD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64901, 493},
    /* 4439 : VPDPWSSDS xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.66.0f38.w0 53 /r          ] AR0-2,SO,LATEVEX,AVXVNNI,FUTURE */
        /* 261(002 001) 001(53) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVXVNNI,FUTURE */
        {I_VPDPWSSDS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64908, 492},
    /* 4440 : VPDPWSSDS ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.66.0f38.w0 53 /r          ] AR0-2,SY,LATEVEX,AVXVNNI,FUTURE */
        /* 261(002 005) 001(53) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVXVNNI,FUTURE */
        {I_VPDPWSSDS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64915, 493},
    /* 4441 : VPDPBSSD xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.f2.0f38.w0 50 /r          ] AR0-2,SO,LATEVEX,AVX,AVXVNNIINT8,FUTURE */
        /* 261(002 003) 001(50) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT8,FUTURE */
        {I_VPDPBSSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64922, 494},
    /* 4442 : VPDPBSSD ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.f2.0f38.w0 50 /r          ] AR0-2,SY,LATEVEX,AVX,AVXVNNIINT8,FUTURE */
        /* 261(002 007) 001(50) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT8,FUTURE */
        {I_VPDPBSSD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64929, 495},
    /* 4443 : VPDPBSSDS xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.f2.0f38.w0 51 /r          ] AR0-2,SO,LATEVEX,AVX,AVXVNNIINT8,FUTURE */
        /* 261(002 003) 001(51) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT8,FUTURE */
        {I_VPDPBSSDS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64936, 494},
    /* 4444 : VPDPBSSDS ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.f2.0f38.w0 51 /r          ] AR0-2,SY,LATEVEX,AVX,AVXVNNIINT8,FUTURE */
        /* 261(002 007) 001(51) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT8,FUTURE */
        {I_VPDPBSSDS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64943, 495},
    /* 4445 : VPDPBSUD xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.f3.0f38.w0 50 /r          ] AR0-2,SO,LATEVEX,AVX,AVXVNNIINT8,FUTURE */
        /* 261(002 002) 001(50) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT8,FUTURE */
        {I_VPDPBSUD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64950, 494},
    /* 4446 : VPDPBSUD ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.f3.0f38.w0 50 /r          ] AR0-2,SY,LATEVEX,AVX,AVXVNNIINT8,FUTURE */
        /* 261(002 006) 001(50) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT8,FUTURE */
        {I_VPDPBSUD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64957, 495},
    /* 4447 : VPDPBSUDS xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.f3.0f38.w0 51 /r          ] AR0-2,SO,LATEVEX,AVX,AVXVNNIINT8,FUTURE */
        /* 261(002 002) 001(51) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT8,FUTURE */
        {I_VPDPBSUDS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64964, 494},
    /* 4448 : VPDPBSUDS ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.f3.0f38.w0 51 /r          ] AR0-2,SY,LATEVEX,AVX,AVXVNNIINT8,FUTURE */
        /* 261(002 006) 001(51) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT8,FUTURE */
        {I_VPDPBSUDS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64971, 495},
    /* 4449 : VPDPBUUD xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.np.0f38.w0 50 /r          ] AR0-2,SO,LATEVEX,AVX,AVXVNNIINT8,FUTURE */
        /* 261(002 000) 001(50) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT8,FUTURE */
        {I_VPDPBUUD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64978, 494},
    /* 4450 : VPDPBUUD ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.np.0f38.w0 50 /r          ] AR0-2,SY,LATEVEX,AVX,AVXVNNIINT8,FUTURE */
        /* 261(002 004) 001(50) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT8,FUTURE */
        {I_VPDPBUUD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64985, 495},
    /* 4451 : VPDPBUUDS xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.np.0f38.w0 51 /r          ] AR0-2,SO,LATEVEX,AVX,AVXVNNIINT8,FUTURE */
        /* 261(002 000) 001(51) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT8,FUTURE */
        {I_VPDPBUUDS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64992, 494},
    /* 4452 : VPDPBUUDS ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.np.0f38.w0 51 /r          ] AR0-2,SY,LATEVEX,AVX,AVXVNNIINT8,FUTURE */
        /* 261(002 004) 001(51) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT8,FUTURE */
        {I_VPDPBUUDS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64999, 495},
    /* 4453 : VPDPWSUD xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.f3.0f38.w0 d2 /r          ] AR0-2,SO,LATEVEX,AVX,AVXVNNIINT16,FUTURE */
        /* 261(002 002) 001(d2) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT16,FUTURE */
        {I_VPDPWSUD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65006, 496},
    /* 4454 : VPDPWSUD ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.f3.0f38.w0 d2 /r          ] AR0-2,SY,LATEVEX,AVX,AVXVNNIINT16,FUTURE */
        /* 261(002 006) 001(d2) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT16,FUTURE */
        {I_VPDPWSUD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65013, 497},
    /* 4455 : VPDPWSUDS xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.f3.0f38.w0 d3 /r          ] AR0-2,SO,LATEVEX,AVX,AVXVNNIINT16,FUTURE */
        /* 261(002 002) 001(d3) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT16,FUTURE */
        {I_VPDPWSUDS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65020, 496},
    /* 4456 : VPDPWSUDS ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.f3.0f38.w0 d3 /r          ] AR0-2,SY,LATEVEX,AVX,AVXVNNIINT16,FUTURE */
        /* 261(002 006) 001(d3) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT16,FUTURE */
        {I_VPDPWSUDS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65027, 497},
    /* 4457 : VPDPWUSD xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.66.0f38.w0 d2 /r          ] AR0-2,SO,LATEVEX,AVX,AVXVNNIINT16,FUTURE */
        /* 261(002 001) 001(d2) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT16,FUTURE */
        {I_VPDPWUSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65034, 496},
    /* 4458 : VPDPWUSD ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.66.0f38.w0 d2 /r          ] AR0-2,SY,LATEVEX,AVX,AVXVNNIINT16,FUTURE */
        /* 261(002 005) 001(d2) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT16,FUTURE */
        {I_VPDPWUSD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65041, 497},
    /* 4459 : VPDPWUSDS xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.66.0f38.w0 d3 /r          ] AR0-2,SO,LATEVEX,AVX,AVXVNNIINT16,FUTURE */
        /* 261(002 001) 001(d3) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT16,FUTURE */
        {I_VPDPWUSDS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65048, 496},
    /* 4460 : VPDPWUSDS ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.66.0f38.w0 d3 /r          ] AR0-2,SY,LATEVEX,AVX,AVXVNNIINT16,FUTURE */
        /* 261(002 005) 001(d3) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT16,FUTURE */
        {I_VPDPWUSDS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65055, 497},
    /* 4461 : VPDPWUUD xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.np.0f38.w0 d2 /r          ] AR0-2,SO,LATEVEX,AVX,AVXVNNIINT16,FUTURE */
        /* 261(002 000) 001(d2) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT16,FUTURE */
        {I_VPDPWUUD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65062, 496},
    /* 4462 : VPDPWUUD ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.np.0f38.w0 d2 /r          ] AR0-2,SY,LATEVEX,AVX,AVXVNNIINT16,FUTURE */
        /* 261(002 004) 001(d2) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT16,FUTURE */
        {I_VPDPWUUD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65069, 497},
    /* 4463 : VPDPWUUDS xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.np.0f38.w0 d3 /r          ] AR0-2,SO,LATEVEX,AVX,AVXVNNIINT16,FUTURE */
        /* 261(002 000) 001(d3) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT16,FUTURE */
        {I_VPDPWUUDS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65076, 496},
    /* 4464 : VPDPWUUDS ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.np.0f38.w0 d3 /r          ] AR0-2,SY,LATEVEX,AVX,AVXVNNIINT16,FUTURE */
        /* 261(002 004) 001(d3) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT16,FUTURE */
        {I_VPDPWUUDS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65083, 497},
    /* 4465 : VPMADD52HUQ xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.66.0f38.w1 b5 /r          ] AR0-2,SO,LATEVEX,AVXIFMA,FUTURE */
        /* 261(002 201) 001(b5) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVXIFMA,FUTURE */
        {I_VPMADD52HUQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65090, 498},
    /* 4466 : VPMADD52HUQ ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.66.0f38.w1 b5 /r          ] AR0-2,SY,LATEVEX,AVXIFMA,FUTURE */
        /* 261(002 205) 001(b5) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVXIFMA,FUTURE */
        {I_VPMADD52HUQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65097, 499},
    /* 4467 : VPMADD52LUQ xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.66.0f38.w1 b4 /r          ] AR0-2,SO,LATEVEX,AVXIFMA,FUTURE */
        /* 261(002 201) 001(b4) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVXIFMA,FUTURE */
        {I_VPMADD52LUQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65104, 498},
    /* 4468 : VPMADD52LUQ ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.66.0f38.w1 b4 /r          ] AR0-2,SY,LATEVEX,AVXIFMA,FUTURE */
        /* 261(002 205) 001(b4) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVXIFMA,FUTURE */
        {I_VPMADD52LUQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65111, 499},
    /* 4469 : KMOVB kreg8,krm8 [rm:       vex.l0.0f.ko8 90 /r              ] ZU,AVX512DQ,FUTURE */
        /* 270(001 001) 001(90) 110 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KMOVB, 2, {OPMASKREG|BITS8|RN_L16,RM_OPMASK|BITS8|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65118, 500},
    /* 4470 : KMOVB kreg8,krm8 [rm:       evex.l0.0f.ko8 90 /r              ] ZU,AVX512DQ,FUTURE,APX */
        /* 250(f1 7d 08) 300 001(90) 110 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KMOVB, 2, {OPMASKREG|BITS8,RM_OPMASK|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+29664, 501},
    /* 4471 : KMOVW kreg16,krm16 [rm:       vex.l0.0f.ko16 90 /r             ] ZU,AVX512F,FUTURE */
        /* 270(001 000) 001(90) 110 : NOAPX,ZU,VEX,AVX512F,FUTURE */
        {I_KMOVW, 2, {OPMASKREG|BITS16|RN_L16,RM_OPMASK|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65125, 502},
    /* 4472 : KMOVW kreg16,krm16 [rm:       evex.l0.0f.ko16 90 /r             ] ZU,AVX512F,FUTURE,APX */
        /* 250(f1 7c 08) 300 001(90) 110 : ZU,LONG,EVEX,PROT,AVX512F,APX,FUTURE */
        {I_KMOVW, 2, {OPMASKREG|BITS16,RM_OPMASK|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+29673, 503},
    /* 4473 : KMOVD kreg32,krm32 [rm:       vex.l0.0f.ko32 90 /r             ] ZU,AVX512BW,FUTURE */
        /* 270(001 201) 001(90) 110 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KMOVD, 2, {OPMASKREG|BITS32|RN_L16,RM_OPMASK|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65132, 504},
    /* 4474 : KMOVD kreg32,krm32 [rm:       evex.l0.0f.ko32 90 /r             ] ZU,AVX512BW,FUTURE,APX */
        /* 250(f1 fd 08) 300 001(90) 110 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KMOVD, 2, {OPMASKREG|BITS32,RM_OPMASK|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+29682, 505},
    /* 4475 : KMOVQ kreg64,krm64 [rm:       vex.l0.0f.ko64 90 /r             ] ZU,AVX512BW,FUTURE */
        /* 270(001 200) 001(90) 110 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KMOVQ, 2, {OPMASKREG|BITS64|RN_L16,RM_OPMASK|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65139, 504},
    /* 4476 : KMOVQ kreg64,krm64 [rm:       evex.l0.0f.ko64 90 /r             ] ZU,AVX512BW,FUTURE,APX */
        /* 250(f1 fc 08) 300 001(90) 110 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KMOVQ, 2, {OPMASKREG|BITS64,RM_OPMASK|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+29691, 505},
    /* 4477 : KMOVB mem8,kreg8 [mr:       vex.l0.0f.ko8 91 /r              ] ZU,AVX512DQ,FUTURE */
        /* 270(001 001) 001(91) 101 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KMOVB, 2, {MEMORY|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65146, 500},
    /* 4478 : KMOVB mem8,kreg8 [mr:       evex.l0.0f.ko8 91 /r              ] ZU,AVX512DQ,FUTURE,APX */
        /* 250(f1 7d 08) 300 001(91) 101 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KMOVB, 2, {MEMORY|BITS8,OPMASKREG|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+29700, 501},
    /* 4479 : KMOVW mem16,kreg16 [mr:       vex.l0.0f.ko16 91 /r             ] ZU,AVX512DQ,FUTURE */
        /* 270(001 000) 001(91) 101 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KMOVW, 2, {MEMORY|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65153, 500},
    /* 4480 : KMOVW mem16,kreg16 [mr:       evex.l0.0f.ko16 91 /r             ] ZU,AVX512DQ,FUTURE,APX */
        /* 250(f1 7c 08) 300 001(91) 101 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KMOVW, 2, {MEMORY|BITS16,OPMASKREG|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+29709, 501},
    /* 4481 : KMOVD mem32,kreg32 [mr:       vex.l0.0f.ko32 91 /r             ] ZU,AVX512BW,FUTURE */
        /* 270(001 201) 001(91) 101 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KMOVD, 2, {MEMORY|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65160, 504},
    /* 4482 : KMOVD mem32,kreg32 [mr:       evex.l0.0f.ko32 91 /r             ] ZU,AVX512BW,FUTURE,APX */
        /* 250(f1 fd 08) 300 001(91) 101 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KMOVD, 2, {MEMORY|BITS32,OPMASKREG|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+29718, 505},
    /* 4483 : KMOVQ mem64,kreg64 [mr:       vex.l0.0f.ko64 91 /r             ] ZU,AVX512BW,FUTURE */
        /* 270(001 200) 001(91) 101 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KMOVQ, 2, {MEMORY|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65167, 504},
    /* 4484 : KMOVQ mem64,kreg64 [mr:       evex.l0.0f.ko64 91 /r             ] ZU,AVX512BW,FUTURE,APX */
        /* 250(f1 fc 08) 300 001(91) 101 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KMOVQ, 2, {MEMORY|BITS64,OPMASKREG|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+29727, 505},
    /* 4485 : KMOVD kreg32,reg32 [rm:       vex.l0.0f.f2.w0 92 /r            ] ZU,AVX512BW,FUTURE */
        /* 270(001 003) 001(92) 110 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KMOVD, 2, {OPMASKREG|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65174, 504},
    /* 4486 : KMOVD kreg32,reg32 [rm:       evex.l0.0f.f2.w0 92 /r            ] ZU,AVX512BW,FUTURE,APX */
        /* 250(f1 7f 08) 300 001(92) 110 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KMOVD, 2, {OPMASKREG|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+29736, 505},
    /* 4487 : KMOVQ kreg64,reg64 [rm:       vex.l0.0f.f2.w1 92 /r            ] ZU,LONG,PROT,AVX512BW,X86_64 */
        /* 270(001 203) 001(92) 110 : NOAPX,ZU,LONG,VEX,PROT,AVX512BW,X86_64 */
        {I_KMOVQ, 2, {OPMASKREG|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65181, 506},
    /* 4488 : KMOVQ kreg64,reg64 [rm:       evex.l0.0f.f2.w1 92 /r            ] ZU,LONG,PROT,AVX512BW,X86_64,APX */
        /* 250(f1 ff 08) 300 001(92) 110 : ZU,LONG,EVEX,PROT,AVX512BW,APX,X86_64 */
        {I_KMOVQ, 2, {OPMASKREG|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+29745, 507},
    /* 4489 : KMOVD reg32,kreg32 [rm:       vex.l0.0f.f2.w0 93 /r            ] ZU,AVX512BW,FUTURE */
        /* 270(001 003) 001(93) 110 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KMOVD, 2, {REG_GPR|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65188, 504},
    /* 4490 : KMOVD reg32,kreg32 [rm:       evex.l0.0f.f2.w0 93 /r            ] ZU,AVX512BW,FUTURE,APX */
        /* 250(f1 7f 08) 300 001(93) 110 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KMOVD, 2, {REG_GPR|BITS32,OPMASKREG|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+29754, 505},
    /* 4491 : KMOVQ reg64,kreg64 [rm:       vex.l0.0f.f2.w1 93 /r            ] ZU,LONG,PROT,AVX512BW,X86_64 */
        /* 270(001 203) 001(93) 110 : NOAPX,ZU,LONG,VEX,PROT,AVX512BW,X86_64 */
        {I_KMOVQ, 2, {REG_GPR|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65195, 506},
    /* 4492 : KMOVQ reg64,kreg64 [rm:       evex.l0.0f.f2.w1 93 /r            ] ZU,LONG,PROT,AVX512BW,X86_64,APX */
        /* 250(f1 ff 08) 300 001(93) 110 : ZU,LONG,EVEX,PROT,AVX512BW,APX,X86_64 */
        {I_KMOVQ, 2, {REG_GPR|BITS64,OPMASKREG|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+29763, 507},
    /* 4493 : KMOVB kreg8,reg32 [rm:       vex.l0.0f.66.w0 92 /r            ] ZU,AVX512DQ,FUTURE */
        /* 270(001 001) 001(92) 110 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KMOVB, 2, {OPMASKREG|BITS8|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65202, 500},
    /* 4494 : KMOVB kreg8,reg32 [rm:       evex.l0.0f.66.w0 92 /r            ] ZU,AVX512DQ,FUTURE,APX */
        /* 250(f1 7d 08) 300 001(92) 110 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KMOVB, 2, {OPMASKREG|BITS8,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+29772, 501},
    /* 4495 : KMOVW kreg16,reg32 [rm:       vex.l0.0f.np.w0 92 /r            ] ZU,AVX512F,FUTURE */
        /* 270(001 000) 001(92) 110 : NOAPX,ZU,VEX,AVX512F,FUTURE */
        {I_KMOVW, 2, {OPMASKREG|BITS16|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65209, 502},
    /* 4496 : KMOVW kreg16,reg32 [rm:       evex.l0.0f.np.w0 92 /r            ] ZU,AVX512F,FUTURE,APX */
        /* 250(f1 7c 08) 300 001(92) 110 : ZU,LONG,EVEX,PROT,AVX512F,APX,FUTURE */
        {I_KMOVW, 2, {OPMASKREG|BITS16,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+29781, 503},
    /* 4497 : KMOVB reg32,kreg8 [rm:       vex.l0.0f.66.w0 93 /r            ] ZU,AVX512DQ,FUTURE */
        /* 270(001 001) 001(93) 110 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KMOVB, 2, {REG_GPR|BITS32|RN_L16,OPMASKREG|BITS8|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65216, 500},
    /* 4498 : KMOVB reg32,kreg8 [rm:       evex.l0.0f.66.w0 93 /r            ] ZU,AVX512DQ,FUTURE,APX */
        /* 250(f1 7d 08) 300 001(93) 110 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KMOVB, 2, {REG_GPR|BITS32,OPMASKREG|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+29790, 501},
    /* 4499 : KMOVW reg32,kreg16 [rm:       vex.l0.0f.np.w0 93 /r            ] ZU,AVX512F,FUTURE */
        /* 270(001 000) 001(93) 110 : NOAPX,ZU,VEX,AVX512F,FUTURE */
        {I_KMOVW, 2, {REG_GPR|BITS32|RN_L16,OPMASKREG|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65223, 502},
    /* 4500 : KMOVW reg32,kreg16 [rm:       evex.l0.0f.np.w0 93 /r            ] ZU,AVX512F,FUTURE,APX */
        /* 250(f1 7c 08) 300 001(93) 110 : ZU,LONG,EVEX,PROT,AVX512F,APX,FUTURE */
        {I_KMOVW, 2, {REG_GPR|BITS32,OPMASKREG|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+29799, 503},
    /* 4501 : KADDB kreg8,kreg8*,kreg8 [rvm:      vex.l1.0f.ko8 4a /r              ] ZU,AVX512DQ,FUTURE */
        /* 261(001 005) 001(4a) 120 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KADDB, 3, {OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65230, 500},
    /* 4502 : KADDB kreg8,kreg8*,kreg8 [rvm:      evex.l1.0f.ko8 4a /r              ] ZU,AVX512DQ,FUTURE,APX */
        /* 241(f1 7d 28) 300 001(4a) 120 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KADDB, 3, {OPMASKREG|BITS8,OPMASKREG|BITS8,OPMASKREG|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+29808, 501},
    /* 4503 : KADDW kreg16,kreg16*,kreg16 [rvm:      vex.l1.0f.ko16 4a /r             ] ZU,AVX512DQ,FUTURE */
        /* 261(001 004) 001(4a) 120 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KADDW, 3, {OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65244, 500},
    /* 4504 : KADDW kreg16,kreg16*,kreg16 [rvm:      evex.l1.0f.ko16 4a /r             ] ZU,AVX512DQ,FUTURE,APX */
        /* 241(f1 7c 28) 300 001(4a) 120 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KADDW, 3, {OPMASKREG|BITS16,OPMASKREG|BITS16,OPMASKREG|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+29826, 501},
    /* 4505 : KADDD kreg32,kreg32*,kreg32 [rvm:      vex.l1.0f.ko32 4a /r             ] ZU,AVX512BW,FUTURE */
        /* 261(001 205) 001(4a) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KADDD, 3, {OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65258, 504},
    /* 4506 : KADDD kreg32,kreg32*,kreg32 [rvm:      evex.l1.0f.ko32 4a /r             ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 fd 28) 300 001(4a) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KADDD, 3, {OPMASKREG|BITS32,OPMASKREG|BITS32,OPMASKREG|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+29844, 505},
    /* 4507 : KADDQ kreg64,kreg64*,kreg64 [rvm:      vex.l1.0f.ko64 4a /r             ] ZU,AVX512BW,FUTURE */
        /* 261(001 204) 001(4a) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KADDQ, 3, {OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65272, 504},
    /* 4508 : KADDQ kreg64,kreg64*,kreg64 [rvm:      evex.l1.0f.ko64 4a /r             ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 fc 28) 300 001(4a) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KADDQ, 3, {OPMASKREG|BITS64,OPMASKREG|BITS64,OPMASKREG|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+29862, 505},
    /* 4509 : KANDB kreg8,kreg8*,kreg8 [rvm:      vex.l1.0f.ko8 41 /r              ] ZU,AVX512DQ,FUTURE */
        /* 261(001 005) 001(41) 120 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KANDB, 3, {OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65286, 500},
    /* 4510 : KANDB kreg8,kreg8*,kreg8 [rvm:      evex.l1.0f.ko8 41 /r              ] ZU,AVX512DQ,FUTURE,APX */
        /* 241(f1 7d 28) 300 001(41) 120 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KANDB, 3, {OPMASKREG|BITS8,OPMASKREG|BITS8,OPMASKREG|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+29880, 501},
    /* 4511 : KANDW kreg16,kreg16*,kreg16 [rvm:      vex.l1.0f.ko16 41 /r             ] ZU,AVX512F,FUTURE */
        /* 261(001 004) 001(41) 120 : NOAPX,ZU,VEX,AVX512F,FUTURE */
        {I_KANDW, 3, {OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65300, 502},
    /* 4512 : KANDW kreg16,kreg16*,kreg16 [rvm:      evex.l1.0f.ko16 41 /r             ] ZU,AVX512F,FUTURE,APX */
        /* 241(f1 7c 28) 300 001(41) 120 : ZU,LONG,EVEX,PROT,AVX512F,APX,FUTURE */
        {I_KANDW, 3, {OPMASKREG|BITS16,OPMASKREG|BITS16,OPMASKREG|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+29898, 503},
    /* 4513 : KANDD kreg32,kreg32*,kreg32 [rvm:      vex.l1.0f.ko32 41 /r             ] ZU,AVX512BW,FUTURE */
        /* 261(001 205) 001(41) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KANDD, 3, {OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65314, 504},
    /* 4514 : KANDD kreg32,kreg32*,kreg32 [rvm:      evex.l1.0f.ko32 41 /r             ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 fd 28) 300 001(41) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KANDD, 3, {OPMASKREG|BITS32,OPMASKREG|BITS32,OPMASKREG|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+29916, 505},
    /* 4515 : KANDQ kreg64,kreg64*,kreg64 [rvm:      vex.l1.0f.ko64 41 /r             ] ZU,AVX512BW,FUTURE */
        /* 261(001 204) 001(41) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KANDQ, 3, {OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65328, 504},
    /* 4516 : KANDQ kreg64,kreg64*,kreg64 [rvm:      evex.l1.0f.ko64 41 /r             ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 fc 28) 300 001(41) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KANDQ, 3, {OPMASKREG|BITS64,OPMASKREG|BITS64,OPMASKREG|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+29934, 505},
    /* 4517 : KANDNB kreg8,kreg8*,kreg8 [rvm:      vex.l1.0f.ko8 42 /r              ] ZU,AVX512DQ,FUTURE */
        /* 261(001 005) 001(42) 120 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KANDNB, 3, {OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65342, 500},
    /* 4518 : KANDNB kreg8,kreg8*,kreg8 [rvm:      evex.l1.0f.ko8 42 /r              ] ZU,AVX512DQ,FUTURE,APX */
        /* 241(f1 7d 28) 300 001(42) 120 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KANDNB, 3, {OPMASKREG|BITS8,OPMASKREG|BITS8,OPMASKREG|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+29952, 501},
    /* 4519 : KANDNW kreg16,kreg16*,kreg16 [rvm:      vex.l1.0f.ko16 42 /r             ] ZU,AVX512F,FUTURE */
        /* 261(001 004) 001(42) 120 : NOAPX,ZU,VEX,AVX512F,FUTURE */
        {I_KANDNW, 3, {OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65356, 502},
    /* 4520 : KANDNW kreg16,kreg16*,kreg16 [rvm:      evex.l1.0f.ko16 42 /r             ] ZU,AVX512F,FUTURE,APX */
        /* 241(f1 7c 28) 300 001(42) 120 : ZU,LONG,EVEX,PROT,AVX512F,APX,FUTURE */
        {I_KANDNW, 3, {OPMASKREG|BITS16,OPMASKREG|BITS16,OPMASKREG|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+29970, 503},
    /* 4521 : KANDND kreg32,kreg32*,kreg32 [rvm:      vex.l1.0f.ko32 42 /r             ] ZU,AVX512BW,FUTURE */
        /* 261(001 205) 001(42) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KANDND, 3, {OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65370, 504},
    /* 4522 : KANDND kreg32,kreg32*,kreg32 [rvm:      evex.l1.0f.ko32 42 /r             ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 fd 28) 300 001(42) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KANDND, 3, {OPMASKREG|BITS32,OPMASKREG|BITS32,OPMASKREG|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+29988, 505},
    /* 4523 : KANDNQ kreg64,kreg64*,kreg64 [rvm:      vex.l1.0f.ko64 42 /r             ] ZU,AVX512BW,FUTURE */
        /* 261(001 204) 001(42) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KANDNQ, 3, {OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65384, 504},
    /* 4524 : KANDNQ kreg64,kreg64*,kreg64 [rvm:      evex.l1.0f.ko64 42 /r             ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 fc 28) 300 001(42) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KANDNQ, 3, {OPMASKREG|BITS64,OPMASKREG|BITS64,OPMASKREG|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+30006, 505},
    /* 4525 : KNOTB kreg8,kreg8* [rm:       vex.l0.0f.ko8 44 /r              ] ZU,AVX512DQ,FUTURE */
        /* 270(001 001) 001(44) 110 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KNOTB, 2, {OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65398, 500},
    /* 4526 : KNOTB kreg8,kreg8* [rm:       evex.l0.0f.ko8 44 /r              ] ZU,AVX512DQ,FUTURE,APX */
        /* 250(f1 7d 08) 300 001(44) 110 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KNOTB, 2, {OPMASKREG|BITS8,OPMASKREG|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+30024, 501},
    /* 4527 : KNOTW kreg16,kreg16* [rm:       vex.l0.0f.ko16 44 /r             ] ZU,AVX512F,FUTURE */
        /* 270(001 000) 001(44) 110 : NOAPX,ZU,VEX,AVX512F,FUTURE */
        {I_KNOTW, 2, {OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65412, 502},
    /* 4528 : KNOTW kreg16,kreg16* [rm:       evex.l0.0f.ko16 44 /r             ] ZU,AVX512F,FUTURE,APX */
        /* 250(f1 7c 08) 300 001(44) 110 : ZU,LONG,EVEX,PROT,AVX512F,APX,FUTURE */
        {I_KNOTW, 2, {OPMASKREG|BITS16,OPMASKREG|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+30042, 503},
    /* 4529 : KNOTD kreg32,kreg32* [rm:       vex.l0.0f.ko32 44 /r             ] ZU,AVX512BW,FUTURE */
        /* 270(001 201) 001(44) 110 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KNOTD, 2, {OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65426, 504},
    /* 4530 : KNOTD kreg32,kreg32* [rm:       evex.l0.0f.ko32 44 /r             ] ZU,AVX512BW,FUTURE,APX */
        /* 250(f1 fd 08) 300 001(44) 110 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KNOTD, 2, {OPMASKREG|BITS32,OPMASKREG|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+30060, 505},
    /* 4531 : KNOTQ kreg64,kreg64* [rm:       vex.l0.0f.ko64 44 /r             ] ZU,AVX512BW,FUTURE */
        /* 270(001 200) 001(44) 110 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KNOTQ, 2, {OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65440, 504},
    /* 4532 : KNOTQ kreg64,kreg64* [rm:       evex.l0.0f.ko64 44 /r             ] ZU,AVX512BW,FUTURE,APX */
        /* 250(f1 fc 08) 300 001(44) 110 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KNOTQ, 2, {OPMASKREG|BITS64,OPMASKREG|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+30078, 505},
    /* 4533 : KORB kreg8,kreg8*,kreg8 [rvm:      vex.nds.l1.0f.ko8 45 /r          ] ZU,AVX512DQ,FUTURE */
        /* 261(001 005) 001(45) 120 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KORB, 3, {OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65454, 500},
    /* 4534 : KORB kreg8,kreg8*,kreg8 [rvm:      evex.nds.l1.0f.ko8 45 /r          ] ZU,AVX512DQ,FUTURE,APX */
        /* 241(f1 7d 28) 300 001(45) 120 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KORB, 3, {OPMASKREG|BITS8,OPMASKREG|BITS8,OPMASKREG|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+30096, 501},
    /* 4535 : KORW kreg16,kreg16*,kreg16 [rvm:      vex.nds.l1.0f.ko16 45 /r         ] ZU,AVX512F,FUTURE */
        /* 261(001 004) 001(45) 120 : NOAPX,ZU,VEX,AVX512F,FUTURE */
        {I_KORW, 3, {OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65468, 502},
    /* 4536 : KORW kreg16,kreg16*,kreg16 [rvm:      evex.nds.l1.0f.ko16 45 /r         ] ZU,AVX512F,FUTURE,APX */
        /* 241(f1 7c 28) 300 001(45) 120 : ZU,LONG,EVEX,PROT,AVX512F,APX,FUTURE */
        {I_KORW, 3, {OPMASKREG|BITS16,OPMASKREG|BITS16,OPMASKREG|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+30114, 503},
    /* 4537 : KORD kreg32,kreg32*,kreg32 [rvm:      vex.nds.l1.0f.ko32 45 /r         ] ZU,AVX512BW,FUTURE */
        /* 261(001 205) 001(45) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KORD, 3, {OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65482, 504},
    /* 4538 : KORD kreg32,kreg32*,kreg32 [rvm:      evex.nds.l1.0f.ko32 45 /r         ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 fd 28) 300 001(45) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KORD, 3, {OPMASKREG|BITS32,OPMASKREG|BITS32,OPMASKREG|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+30132, 505},
    /* 4539 : KORQ kreg64,kreg64*,kreg64 [rvm:      vex.nds.l1.0f.ko64 45 /r         ] ZU,AVX512BW,FUTURE */
        /* 261(001 204) 001(45) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KORQ, 3, {OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65496, 504},
    /* 4540 : KORQ kreg64,kreg64*,kreg64 [rvm:      evex.nds.l1.0f.ko64 45 /r         ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 fc 28) 300 001(45) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KORQ, 3, {OPMASKREG|BITS64,OPMASKREG|BITS64,OPMASKREG|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+30150, 505},
    /* 4541 : KORTESTB kreg8,kreg8 [rm:       vex.l0.0f.ko8 98 /r              ] ZU,FL,AVX512DQ,FUTURE */
        /* 270(001 001) 001(98) 110 : NOAPX,ZU,FL,VEX,AVX512DQ,FUTURE */
        {I_KORTESTB, 2, {OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65510, 538},
    /* 4542 : KORTESTB kreg8,kreg8 [rm:       evex.l0.0f.ko8 98 /r              ] ZU,FL,AVX512DQ,FUTURE,APX */
        /* 250(f1 7d 08) 300 001(98) 110 : ZU,LONG,FL,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KORTESTB, 2, {OPMASKREG|BITS8,OPMASKREG|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+30168, 539},
    /* 4543 : KORTESTW kreg16,kreg16 [rm:       vex.l0.0f.ko16 98 /r             ] ZU,FL,AVX512F,FUTURE */
        /* 270(001 000) 001(98) 110 : NOAPX,ZU,FL,VEX,AVX512F,FUTURE */
        {I_KORTESTW, 2, {OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65517, 540},
    /* 4544 : KORTESTW kreg16,kreg16 [rm:       evex.l0.0f.ko16 98 /r             ] ZU,FL,AVX512F,FUTURE,APX */
        /* 250(f1 7c 08) 300 001(98) 110 : ZU,LONG,FL,EVEX,PROT,AVX512F,APX,FUTURE */
        {I_KORTESTW, 2, {OPMASKREG|BITS16,OPMASKREG|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+30177, 541},
    /* 4545 : KORTESTD kreg32,kreg32 [rm:       vex.l0.0f.ko32 98 /r             ] ZU,FL,AVX512BW,FUTURE */
        /* 270(001 201) 001(98) 110 : NOAPX,ZU,FL,VEX,AVX512BW,FUTURE */
        {I_KORTESTD, 2, {OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65524, 542},
    /* 4546 : KORTESTD kreg32,kreg32 [rm:       evex.l0.0f.ko32 98 /r             ] ZU,FL,AVX512BW,FUTURE,APX */
        /* 250(f1 fd 08) 300 001(98) 110 : ZU,LONG,FL,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KORTESTD, 2, {OPMASKREG|BITS32,OPMASKREG|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+30186, 543},
    /* 4547 : KORTESTQ kreg64,kreg64 [rm:       vex.l0.0f.ko64 98 /r             ] ZU,FL,AVX512BW,FUTURE */
        /* 270(001 200) 001(98) 110 : NOAPX,ZU,FL,VEX,AVX512BW,FUTURE */
        {I_KORTESTQ, 2, {OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65531, 542},
    /* 4548 : KORTESTQ kreg64,kreg64 [rm:       evex.l0.0f.ko64 98 /r             ] ZU,FL,AVX512BW,FUTURE,APX */
        /* 250(f1 fc 08) 300 001(98) 110 : ZU,LONG,FL,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KORTESTQ, 2, {OPMASKREG|BITS64,OPMASKREG|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+30195, 543},
    /* 4549 : KSHIFTLB kreg8,kreg8,imm8 [rmi:      vex.l0.66.0f3a.w0 32 /r ib       ] ZU,AVX512DQ,FUTURE */
        /* 270(003 001) 001(32) 110 022 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KSHIFTLB, 3, {OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54573, 500},
    /* 4550 : KSHIFTLB kreg8,kreg8,imm8 [rmi:      evex.l0.66.0f3a.w0 32 /r ib       ] ZU,AVX512DQ,FUTURE,APX */
        /* 250(f3 7d 08) 300 001(32) 110 022 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KSHIFTLB, 3, {OPMASKREG|BITS8,OPMASKREG|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+18053, 501},
    /* 4551 : KSHIFTLW kreg16,kreg16,imm8 [rmi:      vex.l0.66.0f3a.w1 32 /r ib       ] ZU,AVX512F,FUTURE */
        /* 270(003 201) 001(32) 110 022 : NOAPX,ZU,VEX,AVX512F,FUTURE */
        {I_KSHIFTLW, 3, {OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54581, 502},
    /* 4552 : KSHIFTLW kreg16,kreg16,imm8 [rmi:      evex.l0.66.0f3a.w1 32 /r ib       ] ZU,AVX512F,FUTURE,APX */
        /* 250(f3 fd 08) 300 001(32) 110 022 : ZU,LONG,EVEX,PROT,AVX512F,APX,FUTURE */
        {I_KSHIFTLW, 3, {OPMASKREG|BITS16,OPMASKREG|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+18063, 503},
    /* 4553 : KSHIFTLD kreg32,kreg32,imm8 [rmi:      vex.l0.66.0f3a.w0 33 /r ib       ] ZU,AVX512BW,FUTURE */
        /* 270(003 001) 001(33) 110 022 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KSHIFTLD, 3, {OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54589, 504},
    /* 4554 : KSHIFTLD kreg32,kreg32,imm8 [rmi:      evex.l0.66.0f3a.w0 33 /r ib       ] ZU,AVX512BW,FUTURE,APX */
        /* 250(f3 7d 08) 300 001(33) 110 022 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KSHIFTLD, 3, {OPMASKREG|BITS32,OPMASKREG|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+18073, 505},
    /* 4555 : KSHIFTLQ kreg64,kreg64,imm8 [rmi:      vex.l0.66.0f3a.w1 33 /r ib       ] ZU,AVX512BW,FUTURE */
        /* 270(003 201) 001(33) 110 022 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KSHIFTLQ, 3, {OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54597, 504},
    /* 4556 : KSHIFTLQ kreg64,kreg64,imm8 [rmi:      evex.l0.66.0f3a.w1 33 /r ib       ] ZU,AVX512BW,FUTURE,APX */
        /* 250(f3 fd 08) 300 001(33) 110 022 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KSHIFTLQ, 3, {OPMASKREG|BITS64,OPMASKREG|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+18083, 505},
    /* 4557 : KSHIFTRB kreg8,kreg8,imm8 [rmi:      vex.l0.66.0f3a.w0 30 /r ib       ] ZU,AVX512DQ,FUTURE */
        /* 270(003 001) 001(30) 110 022 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KSHIFTRB, 3, {OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54605, 500},
    /* 4558 : KSHIFTRB kreg8,kreg8,imm8 [rmi:      evex.l0.66.0f3a.w0 30 /r ib       ] ZU,AVX512DQ,FUTURE,APX */
        /* 250(f3 7d 08) 300 001(30) 110 022 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KSHIFTRB, 3, {OPMASKREG|BITS8,OPMASKREG|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+18093, 501},
    /* 4559 : KSHIFTRW kreg16,kreg16,imm8 [rmi:      vex.l0.66.0f3a.w1 30 /r ib       ] ZU,AVX512F,FUTURE */
        /* 270(003 201) 001(30) 110 022 : NOAPX,ZU,VEX,AVX512F,FUTURE */
        {I_KSHIFTRW, 3, {OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54613, 502},
    /* 4560 : KSHIFTRW kreg16,kreg16,imm8 [rmi:      evex.l0.66.0f3a.w1 30 /r ib       ] ZU,AVX512F,FUTURE,APX */
        /* 250(f3 fd 08) 300 001(30) 110 022 : ZU,LONG,EVEX,PROT,AVX512F,APX,FUTURE */
        {I_KSHIFTRW, 3, {OPMASKREG|BITS16,OPMASKREG|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+18103, 503},
    /* 4561 : KSHIFTRD kreg32,kreg32,imm8 [rmi:      vex.l0.66.0f3a.w0 31 /r ib       ] ZU,AVX512BW,FUTURE */
        /* 270(003 001) 001(31) 110 022 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KSHIFTRD, 3, {OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54621, 504},
    /* 4562 : KSHIFTRD kreg32,kreg32,imm8 [rmi:      evex.l0.66.0f3a.w0 31 /r ib       ] ZU,AVX512BW,FUTURE,APX */
        /* 250(f3 7d 08) 300 001(31) 110 022 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KSHIFTRD, 3, {OPMASKREG|BITS32,OPMASKREG|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+18113, 505},
    /* 4563 : KSHIFTRQ kreg64,kreg64,imm8 [rmi:      vex.l0.66.0f3a.w1 31 /r ib       ] ZU,AVX512BW,FUTURE */
        /* 270(003 201) 001(31) 110 022 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KSHIFTRQ, 3, {OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54629, 504},
    /* 4564 : KSHIFTRQ kreg64,kreg64,imm8 [rmi:      evex.l0.66.0f3a.w1 31 /r ib       ] ZU,AVX512BW,FUTURE,APX */
        /* 250(f3 fd 08) 300 001(31) 110 022 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KSHIFTRQ, 3, {OPMASKREG|BITS64,OPMASKREG|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+18123, 505},
    /* 4565 : KTESTB kreg8,kreg8 [rm:       vex.l0.0f.ko8 99 /r              ] ZU,FL,AVX512DQ,FUTURE */
        /* 270(001 001) 001(99) 110 : NOAPX,ZU,FL,VEX,AVX512DQ,FUTURE */
        {I_KTESTB, 2, {OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65538, 538},
    /* 4566 : KTESTB kreg8,kreg8 [rm:       evex.l0.0f.ko8 99 /r              ] ZU,FL,AVX512DQ,FUTURE,APX */
        /* 250(f1 7d 08) 300 001(99) 110 : ZU,LONG,FL,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KTESTB, 2, {OPMASKREG|BITS8,OPMASKREG|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+30204, 539},
    /* 4567 : KTESTW kreg16,kreg16 [rm:       vex.l0.0f.ko16 99 /r             ] ZU,FL,AVX512DQ,FUTURE */
        /* 270(001 000) 001(99) 110 : NOAPX,ZU,FL,VEX,AVX512DQ,FUTURE */
        {I_KTESTW, 2, {OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65545, 538},
    /* 4568 : KTESTW kreg16,kreg16 [rm:       evex.l0.0f.ko16 99 /r             ] ZU,FL,AVX512DQ,FUTURE,APX */
        /* 250(f1 7c 08) 300 001(99) 110 : ZU,LONG,FL,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KTESTW, 2, {OPMASKREG|BITS16,OPMASKREG|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+30213, 539},
    /* 4569 : KTESTD kreg32,kreg32 [rm:       vex.l0.0f.ko32 99 /r             ] ZU,FL,AVX512BW,FUTURE */
        /* 270(001 201) 001(99) 110 : NOAPX,ZU,FL,VEX,AVX512BW,FUTURE */
        {I_KTESTD, 2, {OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65552, 542},
    /* 4570 : KTESTD kreg32,kreg32 [rm:       evex.l0.0f.ko32 99 /r             ] ZU,FL,AVX512BW,FUTURE,APX */
        /* 250(f1 fd 08) 300 001(99) 110 : ZU,LONG,FL,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KTESTD, 2, {OPMASKREG|BITS32,OPMASKREG|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+30222, 543},
    /* 4571 : KTESTQ kreg64,kreg64 [rm:       vex.l0.0f.ko64 99 /r             ] ZU,FL,AVX512BW,FUTURE */
        /* 270(001 200) 001(99) 110 : NOAPX,ZU,FL,VEX,AVX512BW,FUTURE */
        {I_KTESTQ, 2, {OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65559, 542},
    /* 4572 : KTESTQ kreg64,kreg64 [rm:       evex.l0.0f.ko64 99 /r             ] ZU,FL,AVX512BW,FUTURE,APX */
        /* 250(f1 fc 08) 300 001(99) 110 : ZU,LONG,FL,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KTESTQ, 2, {OPMASKREG|BITS64,OPMASKREG|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+30231, 543},
    /* 4573 : KUNPCKBW kreg16,kreg8*,kreg8 [rvm:      vex.nds.l1.66.0f.w0 4b /r        ] ZU,AVX512F,FUTURE */
        /* 261(001 005) 001(4b) 120 : NOAPX,ZU,VEX,AVX512F,FUTURE */
        {I_KUNPCKBW, 3, {OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65566, 502},
    /* 4574 : KUNPCKBW kreg16,kreg8*,kreg8 [rvm:      evex.nds.l1.66.0f.w0 4b /r        ] ZU,AVX512F,FUTURE,APX */
        /* 241(f1 7d 28) 300 001(4b) 120 : ZU,LONG,EVEX,PROT,AVX512F,APX,FUTURE */
        {I_KUNPCKBW, 3, {OPMASKREG|BITS16,OPMASKREG|BITS8,OPMASKREG|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+30240, 503},
    /* 4575 : KUNPCKWD kreg32,kreg16*,kreg16 [rvm:      vex.nds.l1.0f.w0 4b /r           ] ZU,AVX512BW,FUTURE */
        /* 261(001 004) 001(4b) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KUNPCKWD, 3, {OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65580, 504},
    /* 4576 : KUNPCKWD kreg32,kreg16*,kreg16 [rvm:      evex.nds.l1.0f.w0 4b /r           ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 7c 28) 300 001(4b) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KUNPCKWD, 3, {OPMASKREG|BITS32,OPMASKREG|BITS16,OPMASKREG|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+30258, 505},
    /* 4577 : KUNPCKDQ kreg64,kreg32*,kreg32 [rvm:      vex.nds.l1.0f.w1 4b /r           ] ZU,AVX512BW,FUTURE */
        /* 261(001 204) 001(4b) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KUNPCKDQ, 3, {OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65594, 504},
    /* 4578 : KUNPCKDQ kreg64,kreg32*,kreg32 [rvm:      evex.nds.l1.0f.w1 4b /r           ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 fc 28) 300 001(4b) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KUNPCKDQ, 3, {OPMASKREG|BITS64,OPMASKREG|BITS32,OPMASKREG|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+30276, 505},
    /* 4579 : KXNORB kreg8,kreg8*,kreg8 [rvm:      vex.nds.l1.0f.ko8 46 /r          ] ZU,AVX512DQ,FUTURE */
        /* 261(001 005) 001(46) 120 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KXNORB, 3, {OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65608, 500},
    /* 4580 : KXNORB kreg8,kreg8*,kreg8 [rvm:      evex.nds.l1.0f.ko8 46 /r          ] ZU,AVX512DQ,FUTURE,APX */
        /* 241(f1 7d 28) 300 001(46) 120 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KXNORB, 3, {OPMASKREG|BITS8,OPMASKREG|BITS8,OPMASKREG|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+30294, 501},
    /* 4581 : KXNORW kreg16,kreg16*,kreg16 [rvm:      vex.nds.l1.0f.ko16 46 /r         ] ZU,AVX512F,FUTURE */
        /* 261(001 004) 001(46) 120 : NOAPX,ZU,VEX,AVX512F,FUTURE */
        {I_KXNORW, 3, {OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65622, 502},
    /* 4582 : KXNORW kreg16,kreg16*,kreg16 [rvm:      evex.nds.l1.0f.ko16 46 /r         ] ZU,AVX512F,FUTURE,APX */
        /* 241(f1 7c 28) 300 001(46) 120 : ZU,LONG,EVEX,PROT,AVX512F,APX,FUTURE */
        {I_KXNORW, 3, {OPMASKREG|BITS16,OPMASKREG|BITS16,OPMASKREG|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+30312, 503},
    /* 4583 : KXNORD kreg32,kreg32*,kreg32 [rvm:      vex.nds.l1.0f.ko32 46 /r         ] ZU,AVX512BW,FUTURE */
        /* 261(001 205) 001(46) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KXNORD, 3, {OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65636, 504},
    /* 4584 : KXNORD kreg32,kreg32*,kreg32 [rvm:      evex.nds.l1.0f.ko32 46 /r         ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 fd 28) 300 001(46) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KXNORD, 3, {OPMASKREG|BITS32,OPMASKREG|BITS32,OPMASKREG|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+30330, 505},
    /* 4585 : KXNORQ kreg64,kreg64*,kreg64 [rvm:      vex.nds.l1.0f.ko64 46 /r         ] ZU,AVX512BW,FUTURE */
        /* 261(001 204) 001(46) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KXNORQ, 3, {OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65650, 504},
    /* 4586 : KXNORQ kreg64,kreg64*,kreg64 [rvm:      evex.nds.l1.0f.ko64 46 /r         ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 fc 28) 300 001(46) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KXNORQ, 3, {OPMASKREG|BITS64,OPMASKREG|BITS64,OPMASKREG|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+30348, 505},
    /* 4587 : KXORB kreg8,kreg8*,kreg8 [rvm:      vex.nds.l1.0f.ko8 47 /r          ] ZU,AVX512DQ,FUTURE */
        /* 261(001 005) 001(47) 120 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KXORB, 3, {OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65664, 500},
    /* 4588 : KXORB kreg8,kreg8*,kreg8 [rvm:      evex.nds.l1.0f.ko8 47 /r          ] ZU,AVX512DQ,FUTURE,APX */
        /* 241(f1 7d 28) 300 001(47) 120 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KXORB, 3, {OPMASKREG|BITS8,OPMASKREG|BITS8,OPMASKREG|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+30366, 501},
    /* 4589 : KXORW kreg16,kreg16*,kreg16 [rvm:      vex.nds.l1.0f.ko16 47 /r         ] ZU,AVX512F,FUTURE */
        /* 261(001 004) 001(47) 120 : NOAPX,ZU,VEX,AVX512F,FUTURE */
        {I_KXORW, 3, {OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65678, 502},
    /* 4590 : KXORW kreg16,kreg16*,kreg16 [rvm:      evex.nds.l1.0f.ko16 47 /r         ] ZU,AVX512F,FUTURE,APX */
        /* 241(f1 7c 28) 300 001(47) 120 : ZU,LONG,EVEX,PROT,AVX512F,APX,FUTURE */
        {I_KXORW, 3, {OPMASKREG|BITS16,OPMASKREG|BITS16,OPMASKREG|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+30384, 503},
    /* 4591 : KXORD kreg32,kreg32*,kreg32 [rvm:      vex.nds.l1.0f.ko32 47 /r         ] ZU,AVX512BW,FUTURE */
        /* 261(001 205) 001(47) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KXORD, 3, {OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65692, 504},
    /* 4592 : KXORD kreg32,kreg32*,kreg32 [rvm:      evex.nds.l1.0f.ko32 47 /r         ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 fd 28) 300 001(47) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KXORD, 3, {OPMASKREG|BITS32,OPMASKREG|BITS32,OPMASKREG|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+30402, 505},
    /* 4593 : KXORQ kreg64,kreg64*,kreg64 [rvm:      vex.nds.l1.0f.ko64 47 /r         ] ZU,AVX512BW,FUTURE */
        /* 261(001 204) 001(47) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KXORQ, 3, {OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65706, 504},
    /* 4594 : KXORQ kreg64,kreg64*,kreg64 [rvm:      evex.nds.l1.0f.ko64 47 /r         ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 fc 28) 300 001(47) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KXORQ, 3, {OPMASKREG|BITS64,OPMASKREG|BITS64,OPMASKREG|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+30420, 505},
    /* 4595 : VADDPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 58 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(58) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VADDPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+30438, 544},
    /* 4596 : VADDPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 58 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(58) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VADDPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+30456, 544},
    /* 4597 : VADDPD zmmreg|mask|z,zmmreg*,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f.w1 58 /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(58) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VADDPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+30474, 545},
    /* 4598 : VADDPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.0f.w0 58 /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(58) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VADDPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+30492, 544},
    /* 4599 : VADDPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.0f.w0 58 /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(58) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VADDPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+30510, 544},
    /* 4600 : VADDPS zmmreg|mask|z,zmmreg*,zmmrm512|b32|er [rvm:fv:   evex.nds.512.0f.w0 58 /r          ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(58) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VADDPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+30528, 545},
    /* 4601 : VADDSD xmmreg|mask|z,xmmreg*,xmmrm64|er [rvm:t1s:  evex.nds.lig.f2.0f.w1 58 /r       ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(58) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VADDSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+30546, 546},
    /* 4602 : VADDSS xmmreg|mask|z,xmmreg*,xmmrm32|er [rvm:t1s:  evex.nds.lig.f3.0f.w0 58 /r       ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(58) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VADDSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+30564, 546},
    /* 4603 : VALIGND xmmreg|mask|z,xmmreg*,xmmrm128|b32,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w0 03 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(03) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VALIGND, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+18133, 544},
    /* 4604 : VALIGND ymmreg|mask|z,ymmreg*,ymmrm256|b32,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w0 03 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(03) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VALIGND, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+18153, 544},
    /* 4605 : VALIGND zmmreg|mask|z,zmmreg*,zmmrm512|b32,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w0 03 /r ib  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(03) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VALIGND, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+18173, 545},
    /* 4606 : VALIGNQ xmmreg|mask|z,xmmreg*,xmmrm128|b64,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w1 03 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(03) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VALIGNQ, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+18193, 544},
    /* 4607 : VALIGNQ ymmreg|mask|z,ymmreg*,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w1 03 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(03) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VALIGNQ, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+18213, 544},
    /* 4608 : VALIGNQ zmmreg|mask|z,zmmreg*,zmmrm512|b64,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w1 03 /r ib  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(03) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VALIGNQ, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+18233, 545},
    /* 4609 : VANDNPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 55 /r       ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 fd 08) 301 001(55) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VANDNPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+30582, 547},
    /* 4610 : VANDNPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 55 /r       ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 fd 28) 301 001(55) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VANDNPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+30600, 547},
    /* 4611 : VANDNPD zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 55 /r       ] AVX512DQ,FUTURE */
        /* 241(f1 fd 48) 301 001(55) 120 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VANDNPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+30618, 548},
    /* 4612 : VANDNPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.0f.w0 55 /r          ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 7c 08) 301 001(55) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VANDNPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+30636, 547},
    /* 4613 : VANDNPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.0f.w0 55 /r          ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 7c 28) 301 001(55) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VANDNPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+30654, 547},
    /* 4614 : VANDNPS zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.0f.w0 55 /r          ] AVX512DQ,FUTURE */
        /* 241(f1 7c 48) 301 001(55) 120 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VANDNPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+30672, 548},
    /* 4615 : VANDPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 54 /r       ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 fd 08) 301 001(54) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VANDPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+30690, 547},
    /* 4616 : VANDPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 54 /r       ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 fd 28) 301 001(54) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VANDPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+30708, 547},
    /* 4617 : VANDPD zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 54 /r       ] AVX512DQ,FUTURE */
        /* 241(f1 fd 48) 301 001(54) 120 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VANDPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+30726, 548},
    /* 4618 : VANDPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.0f.w0 54 /r          ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 7c 08) 301 001(54) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VANDPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+30744, 547},
    /* 4619 : VANDPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.0f.w0 54 /r          ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 7c 28) 301 001(54) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VANDPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+30762, 547},
    /* 4620 : VANDPS zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.0f.w0 54 /r          ] AVX512DQ,FUTURE */
        /* 241(f1 7c 48) 301 001(54) 120 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VANDPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+30780, 548},
    /* 4621 : VBLENDMPD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 65 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(65) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VBLENDMPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+30798, 544},
    /* 4622 : VBLENDMPD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 65 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(65) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VBLENDMPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+30807, 544},
    /* 4623 : VBLENDMPD zmmreg|mask|z,zmmreg,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 65 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(65) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VBLENDMPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+30816, 545},
    /* 4624 : VBLENDMPS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 65 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(65) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VBLENDMPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+30825, 544},
    /* 4625 : VBLENDMPS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 65 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(65) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VBLENDMPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+30834, 544},
    /* 4626 : VBLENDMPS zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 65 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(65) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VBLENDMPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+30843, 545},
    /* 4627 : VBROADCASTF32X2 ymmreg|mask|z,xmmrm64 [rm:t2:    evex.256.66.0f38.w0 19 /r         ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f2 7d 28) 311 001(19) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VBROADCASTF32X2, 2, {YMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30852, 547},
    /* 4628 : VBROADCASTF32X2 zmmreg|mask|z,xmmrm64 [rm:t2:    evex.512.66.0f38.w0 19 /r         ] AVX512DQ,FUTURE */
        /* 250(f2 7d 48) 311 001(19) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VBROADCASTF32X2, 2, {ZMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30861, 548},
    /* 4629 : VBROADCASTF32X4 ymmreg|mask|z,mem128 [rm:t4:    evex.256.66.0f38.w0 1a /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 312 001(1a) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VBROADCASTF32X4, 2, {YMMREG,MEMORY|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30870, 544},
    /* 4630 : VBROADCASTF32X4 zmmreg|mask|z,mem128 [rm:t4:    evex.512.66.0f38.w0 1a /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 312 001(1a) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VBROADCASTF32X4, 2, {ZMMREG,MEMORY|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30879, 545},
    /* 4631 : VBROADCASTF32X8 zmmreg|mask|z,mem256 [rm:t8:    evex.512.66.0f38.w0 1b /r         ] AVX512DQ,FUTURE */
        /* 250(f2 7d 48) 313 001(1b) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VBROADCASTF32X8, 2, {ZMMREG,MEMORY|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30888, 548},
    /* 4632 : VBROADCASTF64X2 ymmreg|mask|z,mem128 [rm:t2:    evex.256.66.0f38.w1 1a /r         ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f2 fd 28) 311 001(1a) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VBROADCASTF64X2, 2, {YMMREG,MEMORY|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30897, 547},
    /* 4633 : VBROADCASTF64X2 zmmreg|mask|z,mem128 [rm:t2:    evex.512.66.0f38.w1 1a /r         ] AVX512DQ,FUTURE */
        /* 250(f2 fd 48) 311 001(1a) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VBROADCASTF64X2, 2, {ZMMREG,MEMORY|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30906, 548},
    /* 4634 : VBROADCASTF64X4 zmmreg|mask|z,mem256 [rm:t4:    evex.512.66.0f38.w1 1b /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 312 001(1b) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VBROADCASTF64X4, 2, {ZMMREG,MEMORY|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30915, 545},
    /* 4635 : VBROADCASTI32X2 xmmreg|mask|z,xmmrm64 [rm:t2:    evex.128.66.0f38.w0 59 /r         ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f2 7d 08) 311 001(59) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VBROADCASTI32X2, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30924, 547},
    /* 4636 : VBROADCASTI32X2 ymmreg|mask|z,xmmrm64 [rm:t2:    evex.256.66.0f38.w0 59 /r         ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f2 7d 28) 311 001(59) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VBROADCASTI32X2, 2, {YMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30933, 547},
    /* 4637 : VBROADCASTI32X2 zmmreg|mask|z,xmmrm64 [rm:t2:    evex.512.66.0f38.w0 59 /r         ] AVX512DQ,FUTURE */
        /* 250(f2 7d 48) 311 001(59) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VBROADCASTI32X2, 2, {ZMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30942, 548},
    /* 4638 : VBROADCASTI32X4 ymmreg|mask|z,mem128 [rm:t4:    evex.256.66.0f38.w0 5a /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 312 001(5a) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VBROADCASTI32X4, 2, {YMMREG,MEMORY|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30951, 544},
    /* 4639 : VBROADCASTI32X4 zmmreg|mask|z,mem128 [rm:t4:    evex.512.66.0f38.w0 5a /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 312 001(5a) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VBROADCASTI32X4, 2, {ZMMREG,MEMORY|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30960, 545},
    /* 4640 : VBROADCASTI32X8 zmmreg|mask|z,mem256 [rm:t8:    evex.512.66.0f38.w0 5b /r         ] AVX512DQ,FUTURE */
        /* 250(f2 7d 48) 313 001(5b) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VBROADCASTI32X8, 2, {ZMMREG,MEMORY|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30969, 548},
    /* 4641 : VBROADCASTI64X2 ymmreg|mask|z,mem128 [rm:t2:    evex.256.66.0f38.w1 5a /r         ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f2 fd 28) 311 001(5a) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VBROADCASTI64X2, 2, {YMMREG,MEMORY|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30978, 547},
    /* 4642 : VBROADCASTI64X2 zmmreg|mask|z,mem128 [rm:t2:    evex.512.66.0f38.w1 5a /r         ] AVX512DQ,FUTURE */
        /* 250(f2 fd 48) 311 001(5a) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VBROADCASTI64X2, 2, {ZMMREG,MEMORY|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30987, 548},
    /* 4643 : VBROADCASTI64X4 zmmreg|mask|z,mem256 [rm:t4:    evex.512.66.0f38.w1 5b /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 312 001(5b) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VBROADCASTI64X4, 2, {ZMMREG,MEMORY|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30996, 545},
    /* 4644 : VBROADCASTSD ymmreg|mask|z,mem64 [rm:t1s:   evex.256.66.0f38.w1 19 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 306 001(19) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VBROADCASTSD, 2, {YMMREG,MEMORY|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31005, 544},
    /* 4645 : VBROADCASTSD zmmreg|mask|z,mem64 [rm:t1s:   evex.512.66.0f38.w1 19 /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 306 001(19) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VBROADCASTSD, 2, {ZMMREG,MEMORY|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31014, 545},
    /* 4646 : VBROADCASTSD ymmreg|mask|z,xmmreg [rm:       evex.256.66.0f38.w1 19 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 300 001(19) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VBROADCASTSD, 2, {YMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31023, 544},
    /* 4647 : VBROADCASTSD zmmreg|mask|z,xmmreg [rm:       evex.512.66.0f38.w1 19 /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 300 001(19) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VBROADCASTSD, 2, {ZMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31032, 545},
    /* 4648 : VBROADCASTSS xmmreg|mask|z,mem32 [rm:t1s:   evex.128.66.0f38.w0 18 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 306 001(18) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VBROADCASTSS, 2, {XMMREG,MEMORY|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31041, 544},
    /* 4649 : VBROADCASTSS ymmreg|mask|z,mem32 [rm:t1s:   evex.256.66.0f38.w0 18 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 306 001(18) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VBROADCASTSS, 2, {YMMREG,MEMORY|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31050, 544},
    /* 4650 : VBROADCASTSS zmmreg|mask|z,mem32 [rm:t1s:   evex.512.66.0f38.w0 18 /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 306 001(18) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VBROADCASTSS, 2, {ZMMREG,MEMORY|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31059, 545},
    /* 4651 : VBROADCASTSS xmmreg|mask|z,xmmreg [rm:       evex.128.66.0f38.w0 18 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 300 001(18) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VBROADCASTSS, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31068, 544},
    /* 4652 : VBROADCASTSS ymmreg|mask|z,xmmreg [rm:       evex.256.66.0f38.w0 18 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 300 001(18) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VBROADCASTSS, 2, {YMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31077, 544},
    /* 4653 : VBROADCASTSS zmmreg|mask|z,xmmreg [rm:       evex.512.66.0f38.w0 18 /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 300 001(18) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VBROADCASTSS, 2, {ZMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31086, 545},
    /* 4654 : VCMPEQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 00    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+4743, 544},
    /* 4655 : VCMPEQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 00    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+4754, 544},
    /* 4656 : VCMPEQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 00    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(00) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+4765, 545},
    /* 4657 : VCMPEQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 00       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+4776, 544},
    /* 4658 : VCMPEQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 00       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+4787, 544},
    /* 4659 : VCMPEQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 00       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(00) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+4798, 545},
    /* 4660 : VCMPEQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 00    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(00) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+4809, 546},
    /* 4661 : VCMPEQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 00    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(00) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+4820, 546},
    /* 4662 : VCMPEQ_OQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 00    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_OQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+4743, 544},
    /* 4663 : VCMPEQ_OQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 00    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_OQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+4754, 544},
    /* 4664 : VCMPEQ_OQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 00    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(00) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_OQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+4765, 545},
    /* 4665 : VCMPEQ_OQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 00       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_OQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+4776, 544},
    /* 4666 : VCMPEQ_OQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 00       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_OQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+4787, 544},
    /* 4667 : VCMPEQ_OQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 00       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(00) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_OQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+4798, 545},
    /* 4668 : VCMPEQ_OQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 00    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(00) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_OQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+4809, 546},
    /* 4669 : VCMPEQ_OQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 00    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(00) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_OQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+4820, 546},
    /* 4670 : VCMPLTPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 01    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLTPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+4831, 544},
    /* 4671 : VCMPLTPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 01    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLTPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+4842, 544},
    /* 4672 : VCMPLTPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 01    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(01) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLTPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+4853, 545},
    /* 4673 : VCMPLTPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 01       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLTPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+4864, 544},
    /* 4674 : VCMPLTPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 01       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLTPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+4875, 544},
    /* 4675 : VCMPLTPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 01       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(01) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLTPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+4886, 545},
    /* 4676 : VCMPLTSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 01    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(01) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLTSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+4897, 546},
    /* 4677 : VCMPLTSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 01    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(01) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLTSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+4908, 546},
    /* 4678 : VCMPLT_OSPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 01    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLT_OSPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+4831, 544},
    /* 4679 : VCMPLT_OSPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 01    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLT_OSPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+4842, 544},
    /* 4680 : VCMPLT_OSPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 01    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(01) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLT_OSPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+4853, 545},
    /* 4681 : VCMPLT_OSPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 01       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLT_OSPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+4864, 544},
    /* 4682 : VCMPLT_OSPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 01       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLT_OSPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+4875, 544},
    /* 4683 : VCMPLT_OSPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 01       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(01) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLT_OSPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+4886, 545},
    /* 4684 : VCMPLT_OSSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 01    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(01) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLT_OSSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+4897, 546},
    /* 4685 : VCMPLT_OSSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 01    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(01) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLT_OSSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+4908, 546},
    /* 4686 : VCMPLEPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 02    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLEPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+4919, 544},
    /* 4687 : VCMPLEPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 02    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLEPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+4930, 544},
    /* 4688 : VCMPLEPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 02    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(02) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLEPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+4941, 545},
    /* 4689 : VCMPLEPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 02       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLEPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+4952, 544},
    /* 4690 : VCMPLEPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 02       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLEPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+4963, 544},
    /* 4691 : VCMPLEPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 02       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(02) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLEPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+4974, 545},
    /* 4692 : VCMPLESD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 02    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(02) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLESD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+4985, 546},
    /* 4693 : VCMPLESS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 02    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(02) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLESS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+4996, 546},
    /* 4694 : VCMPLE_OSPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 02    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLE_OSPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+4919, 544},
    /* 4695 : VCMPLE_OSPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 02    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLE_OSPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+4930, 544},
    /* 4696 : VCMPLE_OSPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 02    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(02) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLE_OSPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+4941, 545},
    /* 4697 : VCMPLE_OSPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 02       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLE_OSPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+4952, 544},
    /* 4698 : VCMPLE_OSPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 02       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLE_OSPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+4963, 544},
    /* 4699 : VCMPLE_OSPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 02       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(02) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLE_OSPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+4974, 545},
    /* 4700 : VCMPLE_OSSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 02    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(02) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLE_OSSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+4985, 546},
    /* 4701 : VCMPLE_OSSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 02    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(02) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLE_OSSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+4996, 546},
    /* 4702 : VCMPUNORDPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 03    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(03) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPUNORDPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5007, 544},
    /* 4703 : VCMPUNORDPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 03    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(03) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPUNORDPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5018, 544},
    /* 4704 : VCMPUNORDPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 03    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(03) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPUNORDPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5029, 545},
    /* 4705 : VCMPUNORDPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 03       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(03) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPUNORDPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5040, 544},
    /* 4706 : VCMPUNORDPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 03       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(03) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPUNORDPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5051, 544},
    /* 4707 : VCMPUNORDPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 03       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(03) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPUNORDPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5062, 545},
    /* 4708 : VCMPUNORDSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 03    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(03) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPUNORDSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5073, 546},
    /* 4709 : VCMPUNORDSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 03    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(03) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPUNORDSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5084, 546},
    /* 4710 : VCMPUNORD_QPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 03    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(03) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPUNORD_QPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5007, 544},
    /* 4711 : VCMPUNORD_QPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 03    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(03) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPUNORD_QPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5018, 544},
    /* 4712 : VCMPUNORD_QPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 03    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(03) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPUNORD_QPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5029, 545},
    /* 4713 : VCMPUNORD_QPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 03       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(03) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPUNORD_QPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5040, 544},
    /* 4714 : VCMPUNORD_QPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 03       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(03) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPUNORD_QPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5051, 544},
    /* 4715 : VCMPUNORD_QPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 03       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(03) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPUNORD_QPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5062, 545},
    /* 4716 : VCMPUNORD_QSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 03    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(03) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPUNORD_QSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5073, 546},
    /* 4717 : VCMPUNORD_QSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 03    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(03) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPUNORD_QSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5084, 546},
    /* 4718 : VCMPNEQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 04    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5095, 544},
    /* 4719 : VCMPNEQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 04    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5106, 544},
    /* 4720 : VCMPNEQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 04    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(04) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5117, 545},
    /* 4721 : VCMPNEQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 04       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5128, 544},
    /* 4722 : VCMPNEQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 04       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5139, 544},
    /* 4723 : VCMPNEQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 04       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(04) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5150, 545},
    /* 4724 : VCMPNEQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 04    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(04) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5161, 546},
    /* 4725 : VCMPNEQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 04    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(04) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5172, 546},
    /* 4726 : VCMPNEQ_UQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 04    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_UQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5095, 544},
    /* 4727 : VCMPNEQ_UQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 04    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_UQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5106, 544},
    /* 4728 : VCMPNEQ_UQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 04    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(04) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_UQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5117, 545},
    /* 4729 : VCMPNEQ_UQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 04       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_UQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5128, 544},
    /* 4730 : VCMPNEQ_UQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 04       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_UQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5139, 544},
    /* 4731 : VCMPNEQ_UQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 04       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(04) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_UQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5150, 545},
    /* 4732 : VCMPNEQ_UQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 04    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(04) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_UQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5161, 546},
    /* 4733 : VCMPNEQ_UQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 04    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(04) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_UQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5172, 546},
    /* 4734 : VCMPNLTPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 05    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLTPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5183, 544},
    /* 4735 : VCMPNLTPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 05    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLTPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5194, 544},
    /* 4736 : VCMPNLTPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 05    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(05) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLTPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5205, 545},
    /* 4737 : VCMPNLTPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 05       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLTPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5216, 544},
    /* 4738 : VCMPNLTPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 05       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLTPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5227, 544},
    /* 4739 : VCMPNLTPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 05       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(05) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLTPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5238, 545},
    /* 4740 : VCMPNLTSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 05    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(05) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLTSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5249, 546},
    /* 4741 : VCMPNLTSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 05    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(05) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLTSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5260, 546},
    /* 4742 : VCMPNLT_USPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 05    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLT_USPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5183, 544},
    /* 4743 : VCMPNLT_USPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 05    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLT_USPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5194, 544},
    /* 4744 : VCMPNLT_USPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 05    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(05) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLT_USPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5205, 545},
    /* 4745 : VCMPNLT_USPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 05       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLT_USPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5216, 544},
    /* 4746 : VCMPNLT_USPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 05       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLT_USPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5227, 544},
    /* 4747 : VCMPNLT_USPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 05       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(05) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLT_USPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5238, 545},
    /* 4748 : VCMPNLT_USSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 05    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(05) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLT_USSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5249, 546},
    /* 4749 : VCMPNLT_USSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 05    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(05) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLT_USSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5260, 546},
    /* 4750 : VCMPNLEPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 06    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLEPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5271, 544},
    /* 4751 : VCMPNLEPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 06    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLEPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5282, 544},
    /* 4752 : VCMPNLEPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 06    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(06) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLEPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5293, 545},
    /* 4753 : VCMPNLEPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 06       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLEPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5304, 544},
    /* 4754 : VCMPNLEPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 06       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLEPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5315, 544},
    /* 4755 : VCMPNLEPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 06       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(06) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLEPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5326, 545},
    /* 4756 : VCMPNLESD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 06    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(06) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLESD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5337, 546},
    /* 4757 : VCMPNLESS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 06    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(06) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLESS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5348, 546},
    /* 4758 : VCMPNLE_USPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 06    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLE_USPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5271, 544},
    /* 4759 : VCMPNLE_USPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 06    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLE_USPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5282, 544},
    /* 4760 : VCMPNLE_USPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 06    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(06) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLE_USPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5293, 545},
    /* 4761 : VCMPNLE_USPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 06       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLE_USPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5304, 544},
    /* 4762 : VCMPNLE_USPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 06       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLE_USPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5315, 544},
    /* 4763 : VCMPNLE_USPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 06       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(06) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLE_USPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5326, 545},
    /* 4764 : VCMPNLE_USSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 06    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(06) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLE_USSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5337, 546},
    /* 4765 : VCMPNLE_USSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 06    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(06) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLE_USSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5348, 546},
    /* 4766 : VCMPORDPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 07    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(07) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPORDPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5359, 544},
    /* 4767 : VCMPORDPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 07    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(07) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPORDPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5370, 544},
    /* 4768 : VCMPORDPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 07    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(07) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPORDPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5381, 545},
    /* 4769 : VCMPORDPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 07       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(07) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPORDPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5392, 544},
    /* 4770 : VCMPORDPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 07       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(07) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPORDPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5403, 544},
    /* 4771 : VCMPORDPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 07       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(07) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPORDPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5414, 545},
    /* 4772 : VCMPORDSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 07    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(07) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPORDSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5425, 546},
    /* 4773 : VCMPORDSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 07    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(07) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPORDSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5436, 546},
    /* 4774 : VCMPORD_QPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 07    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(07) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPORD_QPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5359, 544},
    /* 4775 : VCMPORD_QPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 07    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(07) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPORD_QPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5370, 544},
    /* 4776 : VCMPORD_QPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 07    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(07) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPORD_QPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5381, 545},
    /* 4777 : VCMPORD_QPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 07       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(07) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPORD_QPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5392, 544},
    /* 4778 : VCMPORD_QPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 07       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(07) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPORD_QPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5403, 544},
    /* 4779 : VCMPORD_QPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 07       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(07) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPORD_QPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5414, 545},
    /* 4780 : VCMPORD_QSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 07    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(07) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPORD_QSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5425, 546},
    /* 4781 : VCMPORD_QSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 07    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(07) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPORD_QSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5436, 546},
    /* 4782 : VCMPEQ_UQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 08    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(08) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_UQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5447, 544},
    /* 4783 : VCMPEQ_UQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 08    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(08) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_UQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5458, 544},
    /* 4784 : VCMPEQ_UQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 08    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(08) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_UQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5469, 545},
    /* 4785 : VCMPEQ_UQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 08       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(08) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_UQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5480, 544},
    /* 4786 : VCMPEQ_UQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 08       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(08) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_UQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5491, 544},
    /* 4787 : VCMPEQ_UQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 08       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(08) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_UQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5502, 545},
    /* 4788 : VCMPEQ_UQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 08    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(08) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_UQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5513, 546},
    /* 4789 : VCMPEQ_UQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 08    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(08) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_UQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5524, 546},
    /* 4790 : VCMPNGEPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 09    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(09) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGEPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5535, 544},
    /* 4791 : VCMPNGEPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 09    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(09) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGEPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5546, 544},
    /* 4792 : VCMPNGEPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 09    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(09) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGEPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5557, 545},
    /* 4793 : VCMPNGEPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 09       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(09) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGEPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5568, 544},
    /* 4794 : VCMPNGEPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 09       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(09) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGEPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5579, 544},
    /* 4795 : VCMPNGEPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 09       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(09) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGEPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5590, 545},
    /* 4796 : VCMPNGESD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 09    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(09) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGESD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5601, 546},
    /* 4797 : VCMPNGESS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 09    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(09) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGESS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5612, 546},
    /* 4798 : VCMPNGE_USPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 09    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(09) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGE_USPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5535, 544},
    /* 4799 : VCMPNGE_USPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 09    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(09) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGE_USPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5546, 544},
    /* 4800 : VCMPNGE_USPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 09    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(09) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGE_USPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5557, 545},
    /* 4801 : VCMPNGE_USPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 09       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(09) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGE_USPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5568, 544},
    /* 4802 : VCMPNGE_USPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 09       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(09) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGE_USPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5579, 544},
    /* 4803 : VCMPNGE_USPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 09       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(09) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGE_USPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5590, 545},
    /* 4804 : VCMPNGE_USSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 09    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(09) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGE_USSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5601, 546},
    /* 4805 : VCMPNGE_USSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 09    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(09) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGE_USSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5612, 546},
    /* 4806 : VCMPNGTPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 0a    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(0a) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGTPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5623, 544},
    /* 4807 : VCMPNGTPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 0a    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(0a) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGTPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5634, 544},
    /* 4808 : VCMPNGTPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 0a    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(0a) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGTPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5645, 545},
    /* 4809 : VCMPNGTPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 0a       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(0a) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGTPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5656, 544},
    /* 4810 : VCMPNGTPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 0a       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(0a) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGTPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5667, 544},
    /* 4811 : VCMPNGTPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 0a       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(0a) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGTPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5678, 545},
    /* 4812 : VCMPNGTSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 0a    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(0a) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGTSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5689, 546},
    /* 4813 : VCMPNGTSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 0a    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(0a) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGTSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5700, 546},
    /* 4814 : VCMPNGT_USPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 0a    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(0a) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGT_USPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5623, 544},
    /* 4815 : VCMPNGT_USPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 0a    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(0a) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGT_USPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5634, 544},
    /* 4816 : VCMPNGT_USPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 0a    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(0a) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGT_USPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5645, 545},
    /* 4817 : VCMPNGT_USPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 0a       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(0a) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGT_USPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5656, 544},
    /* 4818 : VCMPNGT_USPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 0a       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(0a) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGT_USPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5667, 544},
    /* 4819 : VCMPNGT_USPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 0a       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(0a) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGT_USPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5678, 545},
    /* 4820 : VCMPNGT_USSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 0a    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(0a) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGT_USSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5689, 546},
    /* 4821 : VCMPNGT_USSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 0a    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(0a) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGT_USSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5700, 546},
    /* 4822 : VCMPFALSEPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 0b    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(0b) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPFALSEPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5711, 544},
    /* 4823 : VCMPFALSEPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 0b    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(0b) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPFALSEPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5722, 544},
    /* 4824 : VCMPFALSEPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 0b    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(0b) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPFALSEPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5733, 545},
    /* 4825 : VCMPFALSEPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 0b       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(0b) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPFALSEPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5744, 544},
    /* 4826 : VCMPFALSEPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 0b       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(0b) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPFALSEPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5755, 544},
    /* 4827 : VCMPFALSEPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 0b       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(0b) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPFALSEPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5766, 545},
    /* 4828 : VCMPFALSESD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 0b    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(0b) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPFALSESD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5777, 546},
    /* 4829 : VCMPFALSESS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 0b    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(0b) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPFALSESS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5788, 546},
    /* 4830 : VCMPFALSE_OQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 0b    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(0b) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPFALSE_OQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5711, 544},
    /* 4831 : VCMPFALSE_OQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 0b    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(0b) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPFALSE_OQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5722, 544},
    /* 4832 : VCMPFALSE_OQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 0b    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(0b) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPFALSE_OQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5733, 545},
    /* 4833 : VCMPFALSE_OQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 0b       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(0b) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPFALSE_OQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5744, 544},
    /* 4834 : VCMPFALSE_OQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 0b       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(0b) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPFALSE_OQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5755, 544},
    /* 4835 : VCMPFALSE_OQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 0b       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(0b) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPFALSE_OQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5766, 545},
    /* 4836 : VCMPFALSE_OQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 0b    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(0b) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPFALSE_OQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5777, 546},
    /* 4837 : VCMPFALSE_OQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 0b    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(0b) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPFALSE_OQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5788, 546},
    /* 4838 : VCMPNEQ_OQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 0c    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(0c) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_OQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5799, 544},
    /* 4839 : VCMPNEQ_OQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 0c    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(0c) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_OQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5810, 544},
    /* 4840 : VCMPNEQ_OQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 0c    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(0c) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_OQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5821, 545},
    /* 4841 : VCMPNEQ_OQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 0c       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(0c) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_OQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5832, 544},
    /* 4842 : VCMPNEQ_OQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 0c       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(0c) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_OQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5843, 544},
    /* 4843 : VCMPNEQ_OQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 0c       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(0c) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_OQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5854, 545},
    /* 4844 : VCMPNEQ_OQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 0c    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(0c) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_OQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5865, 546},
    /* 4845 : VCMPNEQ_OQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 0c    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(0c) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_OQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5876, 546},
    /* 4846 : VCMPGEPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 0d    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(0d) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGEPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5887, 544},
    /* 4847 : VCMPGEPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 0d    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(0d) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGEPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5898, 544},
    /* 4848 : VCMPGEPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 0d    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(0d) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGEPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5909, 545},
    /* 4849 : VCMPGEPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 0d       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(0d) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGEPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5920, 544},
    /* 4850 : VCMPGEPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 0d       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(0d) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGEPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5931, 544},
    /* 4851 : VCMPGEPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 0d       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(0d) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGEPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5942, 545},
    /* 4852 : VCMPGESD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 0d    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(0d) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGESD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5953, 546},
    /* 4853 : VCMPGESS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 0d    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(0d) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGESS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5964, 546},
    /* 4854 : VCMPGE_OSPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 0d    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(0d) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGE_OSPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5887, 544},
    /* 4855 : VCMPGE_OSPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 0d    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(0d) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGE_OSPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5898, 544},
    /* 4856 : VCMPGE_OSPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 0d    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(0d) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGE_OSPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5909, 545},
    /* 4857 : VCMPGE_OSPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 0d       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(0d) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGE_OSPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5920, 544},
    /* 4858 : VCMPGE_OSPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 0d       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(0d) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGE_OSPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5931, 544},
    /* 4859 : VCMPGE_OSPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 0d       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(0d) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGE_OSPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5942, 545},
    /* 4860 : VCMPGE_OSSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 0d    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(0d) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGE_OSSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5953, 546},
    /* 4861 : VCMPGE_OSSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 0d    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(0d) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGE_OSSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5964, 546},
    /* 4862 : VCMPGTPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 0e    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(0e) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGTPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5975, 544},
    /* 4863 : VCMPGTPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 0e    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(0e) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGTPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5986, 544},
    /* 4864 : VCMPGTPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 0e    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(0e) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGTPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5997, 545},
    /* 4865 : VCMPGTPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 0e       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(0e) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGTPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6008, 544},
    /* 4866 : VCMPGTPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 0e       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(0e) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGTPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6019, 544},
    /* 4867 : VCMPGTPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 0e       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(0e) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGTPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6030, 545},
    /* 4868 : VCMPGTSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 0e    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(0e) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGTSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6041, 546},
    /* 4869 : VCMPGTSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 0e    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(0e) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGTSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6052, 546},
    /* 4870 : VCMPGT_OSPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 0e    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(0e) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGT_OSPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5975, 544},
    /* 4871 : VCMPGT_OSPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 0e    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(0e) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGT_OSPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5986, 544},
    /* 4872 : VCMPGT_OSPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 0e    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(0e) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGT_OSPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5997, 545},
    /* 4873 : VCMPGT_OSPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 0e       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(0e) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGT_OSPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6008, 544},
    /* 4874 : VCMPGT_OSPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 0e       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(0e) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGT_OSPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6019, 544},
    /* 4875 : VCMPGT_OSPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 0e       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(0e) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGT_OSPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6030, 545},
    /* 4876 : VCMPGT_OSSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 0e    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(0e) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGT_OSSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6041, 546},
    /* 4877 : VCMPGT_OSSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 0e    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(0e) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGT_OSSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6052, 546},
    /* 4878 : VCMPTRUEPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 0f    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(0f) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPTRUEPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6063, 544},
    /* 4879 : VCMPTRUEPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 0f    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(0f) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPTRUEPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6074, 544},
    /* 4880 : VCMPTRUEPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 0f    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(0f) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPTRUEPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+6085, 545},
    /* 4881 : VCMPTRUEPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 0f       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(0f) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPTRUEPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6096, 544},
    /* 4882 : VCMPTRUEPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 0f       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(0f) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPTRUEPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6107, 544},
    /* 4883 : VCMPTRUEPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 0f       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(0f) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPTRUEPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6118, 545},
    /* 4884 : VCMPTRUESD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 0f    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(0f) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPTRUESD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6129, 546},
    /* 4885 : VCMPTRUESS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 0f    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(0f) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPTRUESS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6140, 546},
    /* 4886 : VCMPTRUE_UQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 0f    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(0f) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPTRUE_UQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6063, 544},
    /* 4887 : VCMPTRUE_UQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 0f    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(0f) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPTRUE_UQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6074, 544},
    /* 4888 : VCMPTRUE_UQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 0f    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(0f) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPTRUE_UQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+6085, 545},
    /* 4889 : VCMPTRUE_UQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 0f       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(0f) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPTRUE_UQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6096, 544},
    /* 4890 : VCMPTRUE_UQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 0f       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(0f) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPTRUE_UQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6107, 544},
    /* 4891 : VCMPTRUE_UQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 0f       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(0f) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPTRUE_UQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6118, 545},
    /* 4892 : VCMPTRUE_UQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 0f    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(0f) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPTRUE_UQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6129, 546},
    /* 4893 : VCMPTRUE_UQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 0f    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(0f) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPTRUE_UQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6140, 546},
    /* 4894 : VCMPEQ_OSPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 10    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(10) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_OSPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6151, 544},
    /* 4895 : VCMPEQ_OSPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 10    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(10) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_OSPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6162, 544},
    /* 4896 : VCMPEQ_OSPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 10    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(10) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_OSPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+6173, 545},
    /* 4897 : VCMPEQ_OSPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 10       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(10) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_OSPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6184, 544},
    /* 4898 : VCMPEQ_OSPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 10       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(10) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_OSPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6195, 544},
    /* 4899 : VCMPEQ_OSPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 10       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(10) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_OSPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6206, 545},
    /* 4900 : VCMPEQ_OSSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 10    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(10) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_OSSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6217, 546},
    /* 4901 : VCMPEQ_OSSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 10    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(10) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_OSSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6228, 546},
    /* 4902 : VCMPLT_OQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 11    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(11) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLT_OQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6239, 544},
    /* 4903 : VCMPLT_OQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 11    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(11) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLT_OQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6250, 544},
    /* 4904 : VCMPLT_OQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 11    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(11) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLT_OQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+6261, 545},
    /* 4905 : VCMPLT_OQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 11       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(11) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLT_OQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6272, 544},
    /* 4906 : VCMPLT_OQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 11       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(11) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLT_OQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6283, 544},
    /* 4907 : VCMPLT_OQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 11       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(11) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLT_OQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6294, 545},
    /* 4908 : VCMPLT_OQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 11    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(11) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLT_OQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6305, 546},
    /* 4909 : VCMPLT_OQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 11    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(11) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLT_OQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6316, 546},
    /* 4910 : VCMPLE_OQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 12    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(12) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLE_OQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6327, 544},
    /* 4911 : VCMPLE_OQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 12    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(12) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLE_OQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6338, 544},
    /* 4912 : VCMPLE_OQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 12    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(12) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLE_OQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+6349, 545},
    /* 4913 : VCMPLE_OQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 12       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(12) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLE_OQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6360, 544},
    /* 4914 : VCMPLE_OQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 12       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(12) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLE_OQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6371, 544},
    /* 4915 : VCMPLE_OQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 12       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(12) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLE_OQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6382, 545},
    /* 4916 : VCMPLE_OQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 12    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(12) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLE_OQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6393, 546},
    /* 4917 : VCMPLE_OQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 12    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(12) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLE_OQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6404, 546},
    /* 4918 : VCMPUNORD_SPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 13    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(13) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPUNORD_SPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6415, 544},
    /* 4919 : VCMPUNORD_SPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 13    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(13) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPUNORD_SPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6426, 544},
    /* 4920 : VCMPUNORD_SPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 13    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(13) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPUNORD_SPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+6437, 545},
    /* 4921 : VCMPUNORD_SPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 13       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(13) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPUNORD_SPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6448, 544},
    /* 4922 : VCMPUNORD_SPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 13       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(13) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPUNORD_SPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6459, 544},
    /* 4923 : VCMPUNORD_SPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 13       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(13) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPUNORD_SPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6470, 545},
    /* 4924 : VCMPUNORD_SSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 13    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(13) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPUNORD_SSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6481, 546},
    /* 4925 : VCMPUNORD_SSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 13    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(13) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPUNORD_SSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6492, 546},
    /* 4926 : VCMPNEQ_USPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 14    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(14) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_USPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6503, 544},
    /* 4927 : VCMPNEQ_USPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 14    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(14) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_USPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6514, 544},
    /* 4928 : VCMPNEQ_USPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 14    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(14) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_USPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+6525, 545},
    /* 4929 : VCMPNEQ_USPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 14       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(14) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_USPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6536, 544},
    /* 4930 : VCMPNEQ_USPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 14       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(14) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_USPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6547, 544},
    /* 4931 : VCMPNEQ_USPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 14       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(14) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_USPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6558, 545},
    /* 4932 : VCMPNEQ_USSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 14    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(14) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_USSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6569, 546},
    /* 4933 : VCMPNEQ_USSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 14    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(14) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_USSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6580, 546},
    /* 4934 : VCMPNLT_UQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 15    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(15) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLT_UQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6591, 544},
    /* 4935 : VCMPNLT_UQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 15    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(15) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLT_UQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6602, 544},
    /* 4936 : VCMPNLT_UQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 15    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(15) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLT_UQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+6613, 545},
    /* 4937 : VCMPNLT_UQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 15       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(15) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLT_UQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6624, 544},
    /* 4938 : VCMPNLT_UQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 15       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(15) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLT_UQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6635, 544},
    /* 4939 : VCMPNLT_UQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 15       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(15) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLT_UQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6646, 545},
    /* 4940 : VCMPNLT_UQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 15    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(15) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLT_UQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6657, 546},
    /* 4941 : VCMPNLT_UQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 15    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(15) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLT_UQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6668, 546},
    /* 4942 : VCMPNLE_UQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 16    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(16) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLE_UQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6679, 544},
    /* 4943 : VCMPNLE_UQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 16    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(16) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLE_UQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6690, 544},
    /* 4944 : VCMPNLE_UQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 16    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(16) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLE_UQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+6701, 545},
    /* 4945 : VCMPNLE_UQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 16       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(16) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLE_UQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6712, 544},
    /* 4946 : VCMPNLE_UQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 16       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(16) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLE_UQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6723, 544},
    /* 4947 : VCMPNLE_UQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 16       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(16) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLE_UQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6734, 545},
    /* 4948 : VCMPNLE_UQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 16    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(16) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLE_UQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6745, 546},
    /* 4949 : VCMPNLE_UQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 16    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(16) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLE_UQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6756, 546},
    /* 4950 : VCMPORD_SPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 17    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(17) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPORD_SPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6767, 544},
    /* 4951 : VCMPORD_SPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 17    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(17) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPORD_SPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6778, 544},
    /* 4952 : VCMPORD_SPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 17    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(17) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPORD_SPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+6789, 545},
    /* 4953 : VCMPORD_SPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 17       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(17) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPORD_SPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6800, 544},
    /* 4954 : VCMPORD_SPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 17       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(17) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPORD_SPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6811, 544},
    /* 4955 : VCMPORD_SPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 17       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(17) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPORD_SPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6822, 545},
    /* 4956 : VCMPORD_SSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 17    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(17) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPORD_SSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6833, 546},
    /* 4957 : VCMPORD_SSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 17    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(17) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPORD_SSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6844, 546},
    /* 4958 : VCMPEQ_USPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 18    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(18) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_USPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6855, 544},
    /* 4959 : VCMPEQ_USPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 18    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(18) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_USPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6866, 544},
    /* 4960 : VCMPEQ_USPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 18    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(18) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_USPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+6877, 545},
    /* 4961 : VCMPEQ_USPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 18       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(18) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_USPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6888, 544},
    /* 4962 : VCMPEQ_USPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 18       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(18) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_USPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6899, 544},
    /* 4963 : VCMPEQ_USPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 18       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(18) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_USPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6910, 545},
    /* 4964 : VCMPEQ_USSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 18    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(18) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_USSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6921, 546},
    /* 4965 : VCMPEQ_USSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 18    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(18) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_USSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6932, 546},
    /* 4966 : VCMPNGE_UQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 19    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(19) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGE_UQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6943, 544},
    /* 4967 : VCMPNGE_UQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 19    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(19) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGE_UQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6954, 544},
    /* 4968 : VCMPNGE_UQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 19    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(19) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGE_UQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+6965, 545},
    /* 4969 : VCMPNGE_UQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 19       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(19) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGE_UQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6976, 544},
    /* 4970 : VCMPNGE_UQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 19       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(19) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGE_UQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6987, 544},
    /* 4971 : VCMPNGE_UQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 19       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(19) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGE_UQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6998, 545},
    /* 4972 : VCMPNGE_UQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 19    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(19) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGE_UQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7009, 546},
    /* 4973 : VCMPNGE_UQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 19    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(19) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGE_UQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7020, 546},
    /* 4974 : VCMPNGT_UQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 1a    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(1a) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGT_UQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7031, 544},
    /* 4975 : VCMPNGT_UQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 1a    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(1a) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGT_UQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7042, 544},
    /* 4976 : VCMPNGT_UQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 1a    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(1a) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGT_UQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+7053, 545},
    /* 4977 : VCMPNGT_UQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 1a       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(1a) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGT_UQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7064, 544},
    /* 4978 : VCMPNGT_UQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 1a       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(1a) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGT_UQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7075, 544},
    /* 4979 : VCMPNGT_UQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 1a       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(1a) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGT_UQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+7086, 545},
    /* 4980 : VCMPNGT_UQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 1a    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(1a) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGT_UQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7097, 546},
    /* 4981 : VCMPNGT_UQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 1a    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(1a) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGT_UQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7108, 546},
    /* 4982 : VCMPFALSE_OSPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 1b    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(1b) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPFALSE_OSPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7119, 544},
    /* 4983 : VCMPFALSE_OSPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 1b    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(1b) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPFALSE_OSPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7130, 544},
    /* 4984 : VCMPFALSE_OSPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 1b    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(1b) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPFALSE_OSPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+7141, 545},
    /* 4985 : VCMPFALSE_OSPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 1b       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(1b) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPFALSE_OSPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7152, 544},
    /* 4986 : VCMPFALSE_OSPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 1b       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(1b) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPFALSE_OSPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7163, 544},
    /* 4987 : VCMPFALSE_OSPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 1b       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(1b) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPFALSE_OSPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+7174, 545},
    /* 4988 : VCMPFALSE_OSSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 1b    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(1b) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPFALSE_OSSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7185, 546},
    /* 4989 : VCMPFALSE_OSSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 1b    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(1b) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPFALSE_OSSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7196, 546},
    /* 4990 : VCMPNEQ_OSPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 1c    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(1c) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_OSPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7207, 544},
    /* 4991 : VCMPNEQ_OSPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 1c    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(1c) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_OSPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7218, 544},
    /* 4992 : VCMPNEQ_OSPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 1c    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(1c) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_OSPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+7229, 545},
    /* 4993 : VCMPNEQ_OSPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 1c       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(1c) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_OSPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7240, 544},
    /* 4994 : VCMPNEQ_OSPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 1c       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(1c) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_OSPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7251, 544},
    /* 4995 : VCMPNEQ_OSPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 1c       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(1c) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_OSPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+7262, 545},
    /* 4996 : VCMPNEQ_OSSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 1c    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(1c) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_OSSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7273, 546},
    /* 4997 : VCMPNEQ_OSSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 1c    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(1c) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_OSSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7284, 546},
    /* 4998 : VCMPGE_OQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 1d    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(1d) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGE_OQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7295, 544},
    /* 4999 : VCMPGE_OQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 1d    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(1d) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGE_OQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7306, 544},
    /* 5000 : VCMPGE_OQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 1d    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(1d) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGE_OQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+7317, 545},
    /* 5001 : VCMPGE_OQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 1d       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(1d) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGE_OQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7328, 544},
    /* 5002 : VCMPGE_OQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 1d       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(1d) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGE_OQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7339, 544},
    /* 5003 : VCMPGE_OQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 1d       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(1d) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGE_OQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+7350, 545},
    /* 5004 : VCMPGE_OQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 1d    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(1d) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGE_OQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7361, 546},
    /* 5005 : VCMPGE_OQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 1d    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(1d) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGE_OQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7372, 546},
    /* 5006 : VCMPGT_OQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 1e    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(1e) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGT_OQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7383, 544},
    /* 5007 : VCMPGT_OQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 1e    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(1e) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGT_OQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7394, 544},
    /* 5008 : VCMPGT_OQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 1e    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(1e) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGT_OQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+7405, 545},
    /* 5009 : VCMPGT_OQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 1e       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(1e) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGT_OQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7416, 544},
    /* 5010 : VCMPGT_OQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 1e       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(1e) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGT_OQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7427, 544},
    /* 5011 : VCMPGT_OQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 1e       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(1e) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGT_OQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+7438, 545},
    /* 5012 : VCMPGT_OQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 1e    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(1e) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGT_OQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7449, 546},
    /* 5013 : VCMPGT_OQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 1e    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(1e) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGT_OQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7460, 546},
    /* 5014 : VCMPTRUE_USPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 1f    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(1f) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPTRUE_USPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7471, 544},
    /* 5015 : VCMPTRUE_USPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 1f    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(1f) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPTRUE_USPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7482, 544},
    /* 5016 : VCMPTRUE_USPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 1f    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(1f) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPTRUE_USPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+7493, 545},
    /* 5017 : VCMPTRUE_USPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 1f       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(1f) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPTRUE_USPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7504, 544},
    /* 5018 : VCMPTRUE_USPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 1f       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(1f) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPTRUE_USPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7515, 544},
    /* 5019 : VCMPTRUE_USPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 1f       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(1f) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPTRUE_USPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+7526, 545},
    /* 5020 : VCMPTRUE_USSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 1f    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(1f) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPTRUE_USSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7537, 546},
    /* 5021 : VCMPTRUE_USSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 1f    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(1f) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPTRUE_USSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7548, 546},
    /* 5022 : VCMPPD kreg|mask,xmmreg,xmmrm128|b64,imm8 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r ib    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPPD, 4, {OPMASKREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK,0,B64,0,0}, nasm_bytecodes+18253, 544},
    /* 5023 : VCMPPD kreg|mask,ymmreg,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r ib    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPPD, 4, {OPMASKREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK,0,B64,0,0}, nasm_bytecodes+18263, 544},
    /* 5024 : VCMPPD kreg|mask,zmmreg,zmmrm512|b64|sae,imm8 [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r ib    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPPD, 4, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+18273, 545},
    /* 5025 : VCMPPS kreg|mask,xmmreg,xmmrm128|b32,imm8 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r ib       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPPS, 4, {OPMASKREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK,0,B32,0,0}, nasm_bytecodes+18283, 544},
    /* 5026 : VCMPPS kreg|mask,ymmreg,ymmrm256|b32,imm8 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r ib       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPPS, 4, {OPMASKREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK,0,B32,0,0}, nasm_bytecodes+18293, 544},
    /* 5027 : VCMPPS kreg|mask,zmmreg,zmmrm512|b32|sae,imm8 [rvmi:fv:  evex.nds.512.0f.w0 c2 /r ib       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPPS, 4, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+18303, 545},
    /* 5028 : VCMPSD kreg|mask,xmmreg,xmmrm64|sae,imm8 [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r ib    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 023 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPSD, 4, {OPMASKREG,XMMREG,RM_XMM|BITS64,IMM_NORMAL|BITS8,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+18313, 546},
    /* 5029 : VCMPSS kreg|mask,xmmreg,xmmrm32|sae,imm8 [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r ib    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 023 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPSS, 4, {OPMASKREG,XMMREG,RM_XMM|BITS32,IMM_NORMAL|BITS8,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+18323, 546},
    /* 5030 : VCOMISD xmmreg,xmmrm64|sae [rm:t1s:   evex.lig.66.0f.w1 2f /r           ] FL,AVX512,FUTURE */
        /* 250(f1 fd 08) 306 001(2f) 110 : LIG,FL,EVEX,PROT,AVX512,FUTURE */
        {I_VCOMISD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+31095, 549},
    /* 5031 : VCOMISS xmmreg,xmmrm32|sae [rm:t1s:   evex.lig.0f.w0 2f /r              ] FL,AVX512,FUTURE */
        /* 250(f1 7c 08) 306 001(2f) 110 : LIG,FL,EVEX,PROT,AVX512,FUTURE */
        {I_VCOMISS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+31104, 549},
    /* 5032 : VCOMPRESSPD mem128|mask,xmmreg [mr:t1s:   evex.128.66.0f38.w1 8a /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 306 001(8a) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCOMPRESSPD, 2, {MEMORY|BITS128,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+31113, 544},
    /* 5033 : VCOMPRESSPD mem256|mask,ymmreg [mr:t1s:   evex.256.66.0f38.w1 8a /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 306 001(8a) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCOMPRESSPD, 2, {MEMORY|BITS256,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+31122, 544},
    /* 5034 : VCOMPRESSPD mem512|mask,zmmreg [mr:t1s:   evex.512.66.0f38.w1 8a /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 306 001(8a) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VCOMPRESSPD, 2, {MEMORY|BITS512,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+31131, 545},
    /* 5035 : VCOMPRESSPD xmmreg|mask|z,xmmreg [mr:       evex.128.66.0f38.w1 8a /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 300 001(8a) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCOMPRESSPD, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31140, 544},
    /* 5036 : VCOMPRESSPD ymmreg|mask|z,ymmreg [mr:       evex.256.66.0f38.w1 8a /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 300 001(8a) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCOMPRESSPD, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31149, 544},
    /* 5037 : VCOMPRESSPD zmmreg|mask|z,zmmreg [mr:       evex.512.66.0f38.w1 8a /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 300 001(8a) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VCOMPRESSPD, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31158, 545},
    /* 5038 : VCOMPRESSPS mem128|mask,xmmreg [mr:t1s:   evex.128.66.0f38.w0 8a /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 306 001(8a) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCOMPRESSPS, 2, {MEMORY|BITS128,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+31167, 544},
    /* 5039 : VCOMPRESSPS mem256|mask,ymmreg [mr:t1s:   evex.256.66.0f38.w0 8a /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 306 001(8a) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCOMPRESSPS, 2, {MEMORY|BITS256,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+31176, 544},
    /* 5040 : VCOMPRESSPS mem512|mask,zmmreg [mr:t1s:   evex.512.66.0f38.w0 8a /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 306 001(8a) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VCOMPRESSPS, 2, {MEMORY|BITS512,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+31185, 545},
    /* 5041 : VCOMPRESSPS xmmreg|mask|z,xmmreg [mr:       evex.128.66.0f38.w0 8a /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 300 001(8a) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCOMPRESSPS, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31194, 544},
    /* 5042 : VCOMPRESSPS ymmreg|mask|z,ymmreg [mr:       evex.256.66.0f38.w0 8a /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 300 001(8a) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCOMPRESSPS, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31203, 544},
    /* 5043 : VCOMPRESSPS zmmreg|mask|z,zmmreg [mr:       evex.512.66.0f38.w0 8a /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 300 001(8a) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VCOMPRESSPS, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31212, 545},
    /* 5044 : VCVTDQ2PD xmmreg|mask|z,xmmrm64|b32 [rm:hv:    evex.128.f3.0f.w0 e6 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 08) 302 001(e6) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTDQ2PD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31221, 544},
    /* 5045 : VCVTDQ2PD ymmreg|mask|z,xmmrm128|b32 [rm:hv:    evex.256.f3.0f.w0 e6 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 28) 302 001(e6) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTDQ2PD, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31230, 544},
    /* 5046 : VCVTDQ2PD zmmreg|mask|z,ymmrm256|b32|er [rm:hv:    evex.512.f3.0f.w0 e6 /r           ] AVX512,FUTURE */
        /* 250(f1 7e 48) 302 001(e6) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTDQ2PD, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32|ER,0,0,0}, nasm_bytecodes+31239, 545},
    /* 5047 : VCVTDQ2PS xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.0f.w0 5b /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 08) 301 001(5b) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTDQ2PS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31248, 544},
    /* 5048 : VCVTDQ2PS ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.0f.w0 5b /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 28) 301 001(5b) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTDQ2PS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31257, 544},
    /* 5049 : VCVTDQ2PS zmmreg|mask|z,zmmrm512|b32|er [rm:fv:    evex.512.0f.w0 5b /r              ] AVX512,FUTURE */
        /* 250(f1 7c 48) 301 001(5b) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTDQ2PS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|ER,0,0,0}, nasm_bytecodes+31266, 545},
    /* 5050 : VCVTPD2DQ xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.f2.0f.w1 e6 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 ff 08) 301 001(e6) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPD2DQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31275, 544},
    /* 5051 : VCVTPD2DQ xmmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.f2.0f.w1 e6 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 ff 28) 301 001(e6) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPD2DQ, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31284, 544},
    /* 5052 : VCVTPD2DQ ymmreg|mask|z,zmmrm512|b64|er [rm:fv:    evex.512.f2.0f.w1 e6 /r           ] AVX512,FUTURE */
        /* 250(f1 ff 48) 301 001(e6) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTPD2DQ, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|ER,0,0,0}, nasm_bytecodes+31293, 545},
    /* 5053 : VCVTPD2PS xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.0f.w1 5a /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 08) 301 001(5a) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPD2PS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31302, 544},
    /* 5054 : VCVTPD2PS xmmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.0f.w1 5a /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 28) 301 001(5a) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPD2PS, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31311, 544},
    /* 5055 : VCVTPD2PS ymmreg|mask|z,zmmrm512|b64|er [rm:fv:    evex.512.66.0f.w1 5a /r           ] AVX512,FUTURE */
        /* 250(f1 fd 48) 301 001(5a) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTPD2PS, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|ER,0,0,0}, nasm_bytecodes+31320, 545},
    /* 5056 : VCVTPD2QQ xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.0f.w1 7b /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fd 08) 301 001(7b) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTPD2QQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31329, 547},
    /* 5057 : VCVTPD2QQ ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.0f.w1 7b /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fd 28) 301 001(7b) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTPD2QQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31338, 547},
    /* 5058 : VCVTPD2QQ zmmreg|mask|z,zmmrm512|b64|er [rm:fv:    evex.512.66.0f.w1 7b /r           ] AVX512DQ,FUTURE */
        /* 250(f1 fd 48) 301 001(7b) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VCVTPD2QQ, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|ER,0,0,0}, nasm_bytecodes+31347, 548},
    /* 5059 : VCVTPD2UDQ xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.0f.w1 79 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fc 08) 301 001(79) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPD2UDQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31356, 544},
    /* 5060 : VCVTPD2UDQ xmmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.0f.w1 79 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fc 28) 301 001(79) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPD2UDQ, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31365, 544},
    /* 5061 : VCVTPD2UDQ ymmreg|mask|z,zmmrm512|b64|er [rm:fv:    evex.512.0f.w1 79 /r              ] AVX512,FUTURE */
        /* 250(f1 fc 48) 301 001(79) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTPD2UDQ, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|ER,0,0,0}, nasm_bytecodes+31374, 545},
    /* 5062 : VCVTPD2UQQ xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.0f.w1 79 /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fd 08) 301 001(79) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTPD2UQQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31383, 547},
    /* 5063 : VCVTPD2UQQ ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.0f.w1 79 /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fd 28) 301 001(79) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTPD2UQQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31392, 547},
    /* 5064 : VCVTPD2UQQ zmmreg|mask|z,zmmrm512|b64|er [rm:fv:    evex.512.66.0f.w1 79 /r           ] AVX512DQ,FUTURE */
        /* 250(f1 fd 48) 301 001(79) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VCVTPD2UQQ, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|ER,0,0,0}, nasm_bytecodes+31401, 548},
    /* 5065 : VCVTPH2PS xmmreg|mask|z,xmmrm64 [rm:hvm:   evex.128.66.0f38.w0 13 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 314 001(13) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPH2PS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31410, 544},
    /* 5066 : VCVTPH2PS ymmreg|mask|z,xmmrm128 [rm:hvm:   evex.256.66.0f38.w0 13 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 314 001(13) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPH2PS, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31419, 544},
    /* 5067 : VCVTPH2PS zmmreg|mask|z,ymmrm256|sae [rm:hvm:   evex.512.66.0f38.w0 13 /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 314 001(13) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTPH2PS, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,SAE,0,0,0}, nasm_bytecodes+31428, 545},
    /* 5068 : VCVTPS2DQ xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.66.0f.w0 5b /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7d 08) 301 001(5b) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2DQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31437, 544},
    /* 5069 : VCVTPS2DQ ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.66.0f.w0 5b /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7d 28) 301 001(5b) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2DQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31446, 544},
    /* 5070 : VCVTPS2DQ zmmreg|mask|z,zmmrm512|b32|er [rm:fv:    evex.512.66.0f.w0 5b /r           ] AVX512,FUTURE */
        /* 250(f1 7d 48) 301 001(5b) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTPS2DQ, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|ER,0,0,0}, nasm_bytecodes+31455, 545},
    /* 5071 : VCVTPS2PD xmmreg|mask|z,xmmrm64|b32 [rm:hv:    evex.128.0f.w0 5a /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 08) 302 001(5a) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2PD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31464, 544},
    /* 5072 : VCVTPS2PD ymmreg|mask|z,xmmrm128|b32 [rm:hv:    evex.256.0f.w0 5a /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 28) 302 001(5a) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2PD, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31473, 544},
    /* 5073 : VCVTPS2PD zmmreg|mask|z,ymmrm256|b32|sae [rm:hv:    evex.512.0f.w0 5a /r              ] AVX512,FUTURE */
        /* 250(f1 7c 48) 302 001(5a) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTPS2PD, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+31482, 545},
    /* 5074 : VCVTPS2PH xmmreg|mask|z,xmmreg,imm8 [mri:hvm:  evex.128.66.0f3a.w0 1d /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 08) 314 001(1d) 101 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2PH, 3, {XMMREG,XMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18333, 544},
    /* 5075 : VCVTPS2PH xmmreg|mask|z,ymmreg,imm8 [mri:hvm:  evex.256.66.0f3a.w0 1d /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 28) 314 001(1d) 101 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2PH, 3, {XMMREG,YMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18343, 544},
    /* 5076 : VCVTPS2PH ymmreg|mask|z,zmmreg|sae,imm8 [mri:hvm:  evex.512.66.0f3a.w0 1d /r ib      ] AVX512,FUTURE */
        /* 250(f3 7d 48) 314 001(1d) 101 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTPS2PH, 3, {YMMREG,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,SAE,0,0,0}, nasm_bytecodes+18353, 545},
    /* 5077 : VCVTPS2PH mem64|mask,xmmreg,imm8 [mri:hvm:  evex.128.66.0f3a.w0 1d /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 08) 314 001(1d) 101 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2PH, 3, {MEMORY|BITS64,XMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18333, 544},
    /* 5078 : VCVTPS2PH mem128|mask,ymmreg,imm8 [mri:hvm:  evex.256.66.0f3a.w0 1d /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 28) 314 001(1d) 101 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2PH, 3, {MEMORY|BITS128,YMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18343, 544},
    /* 5079 : VCVTPS2PH mem256|mask,zmmreg|sae,imm8 [mri:hvm:  evex.512.66.0f3a.w0 1d /r ib      ] AVX512,FUTURE */
        /* 250(f3 7d 48) 314 001(1d) 101 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTPS2PH, 3, {MEMORY|BITS256,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,SAE,0,0,0}, nasm_bytecodes+18353, 545},
    /* 5080 : VCVTPS2QQ xmmreg|mask|z,xmmrm64|b32 [rm:hv:    evex.128.66.0f.w0 7b /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 7d 08) 302 001(7b) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTPS2QQ, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31491, 547},
    /* 5081 : VCVTPS2QQ ymmreg|mask|z,xmmrm128|b32 [rm:hv:    evex.256.66.0f.w0 7b /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 7d 28) 302 001(7b) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTPS2QQ, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31500, 547},
    /* 5082 : VCVTPS2QQ zmmreg|mask|z,ymmrm256|b32|er [rm:hv:    evex.512.66.0f.w0 7b /r           ] AVX512DQ,FUTURE */
        /* 250(f1 7d 48) 302 001(7b) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VCVTPS2QQ, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32|ER,0,0,0}, nasm_bytecodes+31509, 548},
    /* 5083 : VCVTPS2UDQ xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.0f.w0 79 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 08) 301 001(79) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2UDQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31518, 544},
    /* 5084 : VCVTPS2UDQ ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.0f.w0 79 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 28) 301 001(79) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2UDQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31527, 544},
    /* 5085 : VCVTPS2UDQ zmmreg|mask|z,zmmrm512|b32|er [rm:fv:    evex.512.0f.w0 79 /r              ] AVX512,FUTURE */
        /* 250(f1 7c 48) 301 001(79) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTPS2UDQ, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|ER,0,0,0}, nasm_bytecodes+31536, 545},
    /* 5086 : VCVTPS2UQQ xmmreg|mask|z,xmmrm64|b32 [rm:hv:    evex.128.66.0f.w0 79 /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 7d 08) 302 001(79) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTPS2UQQ, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31545, 547},
    /* 5087 : VCVTPS2UQQ ymmreg|mask|z,xmmrm128|b32 [rm:hv:    evex.256.66.0f.w0 79 /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 7d 28) 302 001(79) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTPS2UQQ, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31554, 547},
    /* 5088 : VCVTPS2UQQ zmmreg|mask|z,ymmrm256|b32|er [rm:hv:    evex.512.66.0f.w0 79 /r           ] AVX512DQ,FUTURE */
        /* 250(f1 7d 48) 302 001(79) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VCVTPS2UQQ, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32|ER,0,0,0}, nasm_bytecodes+31563, 548},
    /* 5089 : VCVTQQ2PD xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.f3.0f.w1 e6 /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fe 08) 301 001(e6) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTQQ2PD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31572, 547},
    /* 5090 : VCVTQQ2PD ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.f3.0f.w1 e6 /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fe 28) 301 001(e6) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTQQ2PD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31581, 547},
    /* 5091 : VCVTQQ2PD zmmreg|mask|z,zmmrm512|b64|er [rm:fv:    evex.512.f3.0f.w1 e6 /r           ] AVX512DQ,FUTURE */
        /* 250(f1 fe 48) 301 001(e6) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VCVTQQ2PD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|ER,0,0,0}, nasm_bytecodes+31590, 548},
    /* 5092 : VCVTQQ2PS xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.0f.w1 5b /r              ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fc 08) 301 001(5b) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTQQ2PS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31599, 547},
    /* 5093 : VCVTQQ2PS xmmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.0f.w1 5b /r              ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fc 28) 301 001(5b) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTQQ2PS, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31608, 547},
    /* 5094 : VCVTQQ2PS ymmreg|mask|z,zmmrm512|b64|er [rm:fv:    evex.512.0f.w1 5b /r              ] AVX512DQ,FUTURE */
        /* 250(f1 fc 48) 301 001(5b) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VCVTQQ2PS, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|ER,0,0,0}, nasm_bytecodes+31617, 548},
    /* 5095 : VCVTSD2SI reg32,xmmrm64|er [rm:t1f64: evex.128.f2.0f.w0 2d /r           ] AVX512,FUTURE */
        /* 250(f1 7f 08) 310 001(2d) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSD2SI, 2, {REG_GPR|BITS32,RM_XMM|BITS64,0,0,0}, {0,ER,0,0,0}, nasm_bytecodes+31626, 545},
    /* 5096 : VCVTSD2SI reg64,xmmrm64|er [rm:t1f64: evex.128.f2.0f.w1 2d /r           ] AVX512,FUTURE */
        /* 250(f1 ff 08) 310 001(2d) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSD2SI, 2, {REG_GPR|BITS64,RM_XMM|BITS64,0,0,0}, {0,ER,0,0,0}, nasm_bytecodes+31635, 545},
    /* 5097 : VCVTSD2SS xmmreg|mask|z,xmmreg*,xmmrm64|er [rvm:t1s:  evex.nds.lig.f2.0f.w1 5a /r       ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(5a) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSD2SS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+31644, 546},
    /* 5098 : VCVTSD2USI reg32,xmmrm64|er [rm:t1f64: evex.128.f2.0f.w0 79 /r           ] AVX512,FUTURE */
        /* 250(f1 7f 08) 310 001(79) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSD2USI, 2, {REG_GPR|BITS32,RM_XMM|BITS64,0,0,0}, {0,ER,0,0,0}, nasm_bytecodes+31662, 545},
    /* 5099 : VCVTSD2USI reg64,xmmrm64|er [rm:t1f64: evex.128.f2.0f.w1 79 /r           ] AVX512,FUTURE */
        /* 250(f1 ff 08) 310 001(79) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSD2USI, 2, {REG_GPR|BITS64,RM_XMM|BITS64,0,0,0}, {0,ER,0,0,0}, nasm_bytecodes+31671, 545},
    /* 5100 : VCVTSI2SD xmmreg,xmmreg*,rm32 [rvm:t1s:  evex.nds.lig.f2.0f.w0 2a /r       ] AVX512,FUTURE */
        /* 241(f1 7f 08) 306 001(2a) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSI2SD, 3, {XMMREG,XMMREG,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+31680, 546},
    /* 5101 : VCVTSI2SD xmmreg,xmmreg*,rm64|er [rvm:t1s:  evex.nds.lig.f2.0f.w1 2a /r       ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(2a) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSI2SD, 3, {XMMREG,XMMREG,RM_GPR|BITS64,0,0}, {0,0,ER,0,0}, nasm_bytecodes+31698, 546},
    /* 5102 : VCVTSI2SS xmmreg,xmmreg*,rm32|er [rvm:t1s:  evex.nds.lig.f3.0f.w0 2a /r       ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(2a) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSI2SS, 3, {XMMREG,XMMREG,RM_GPR|BITS32,0,0}, {0,0,ER,0,0}, nasm_bytecodes+31716, 546},
    /* 5103 : VCVTSI2SS xmmreg,xmmreg*,rm64|er [rvm:t1s:  evex.nds.lig.f3.0f.w1 2a /r       ] AVX512,FUTURE */
        /* 241(f1 fe 08) 306 001(2a) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSI2SS, 3, {XMMREG,XMMREG,RM_GPR|BITS64,0,0}, {0,0,ER,0,0}, nasm_bytecodes+31734, 546},
    /* 5104 : VCVTSS2SD xmmreg|mask|z,xmmreg*,xmmrm32|sae [rvm:t1s:  evex.nds.lig.f3.0f.w0 5a /r       ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(5a) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSS2SD, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+31752, 546},
    /* 5105 : VCVTSS2SI reg32,xmmrm32|er [rm:t1f32: evex.128.f3.0f.w0 2d /r           ] AVX512,FUTURE */
        /* 250(f1 7e 08) 307 001(2d) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSS2SI, 2, {REG_GPR|BITS32,RM_XMM|BITS32,0,0,0}, {0,ER,0,0,0}, nasm_bytecodes+31770, 545},
    /* 5106 : VCVTSS2SI reg64,xmmrm32|er [rm:t1f32: evex.128.f3.0f.w1 2d /r           ] AVX512,FUTURE */
        /* 250(f1 fe 08) 307 001(2d) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSS2SI, 2, {REG_GPR|BITS64,RM_XMM|BITS32,0,0,0}, {0,ER,0,0,0}, nasm_bytecodes+31779, 545},
    /* 5107 : VCVTSS2USI reg32,xmmrm32|er [rm:t1f32: evex.128.f3.0f.w0 79 /r           ] AVX512,FUTURE */
        /* 250(f1 7e 08) 307 001(79) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSS2USI, 2, {REG_GPR|BITS32,RM_XMM|BITS32,0,0,0}, {0,ER,0,0,0}, nasm_bytecodes+31788, 545},
    /* 5108 : VCVTSS2USI reg64,xmmrm32|er [rm:t1f32: evex.128.f3.0f.w1 79 /r           ] AVX512,FUTURE */
        /* 250(f1 fe 08) 307 001(79) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSS2USI, 2, {REG_GPR|BITS64,RM_XMM|BITS32,0,0,0}, {0,ER,0,0,0}, nasm_bytecodes+31797, 545},
    /* 5109 : VCVTTPD2DQ xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.0f.w1 e6 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 08) 301 001(e6) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTTPD2DQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31806, 544},
    /* 5110 : VCVTTPD2DQ xmmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.0f.w1 e6 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 28) 301 001(e6) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTTPD2DQ, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31815, 544},
    /* 5111 : VCVTTPD2DQ ymmreg|mask|z,zmmrm512|b64|sae [rm:fv:    evex.512.66.0f.w1 e6 /r           ] AVX512,FUTURE */
        /* 250(f1 fd 48) 301 001(e6) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTTPD2DQ, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+31824, 545},
    /* 5112 : VCVTTPD2QQ xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.0f.w1 7a /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fd 08) 301 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTTPD2QQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31833, 547},
    /* 5113 : VCVTTPD2QQ ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.0f.w1 7a /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fd 28) 301 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTTPD2QQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31842, 547},
    /* 5114 : VCVTTPD2QQ zmmreg|mask|z,zmmrm512|b64|sae [rm:fv:    evex.512.66.0f.w1 7a /r           ] AVX512DQ,FUTURE */
        /* 250(f1 fd 48) 301 001(7a) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VCVTTPD2QQ, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+31851, 548},
    /* 5115 : VCVTTPD2UDQ xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.0f.w1 78 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fc 08) 301 001(78) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTTPD2UDQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31860, 544},
    /* 5116 : VCVTTPD2UDQ xmmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.0f.w1 78 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fc 28) 301 001(78) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTTPD2UDQ, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31869, 544},
    /* 5117 : VCVTTPD2UDQ ymmreg|mask|z,zmmrm512|b64|sae [rm:fv:    evex.512.0f.w1 78 /r              ] AVX512,FUTURE */
        /* 250(f1 fc 48) 301 001(78) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTTPD2UDQ, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+31878, 545},
    /* 5118 : VCVTTPD2UQQ xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.0f.w1 78 /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fd 08) 301 001(78) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTTPD2UQQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31887, 547},
    /* 5119 : VCVTTPD2UQQ ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.0f.w1 78 /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fd 28) 301 001(78) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTTPD2UQQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31896, 547},
    /* 5120 : VCVTTPD2UQQ zmmreg|mask|z,zmmrm512|b64|sae [rm:fv:    evex.512.66.0f.w1 78 /r           ] AVX512DQ,FUTURE */
        /* 250(f1 fd 48) 301 001(78) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VCVTTPD2UQQ, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+31905, 548},
    /* 5121 : VCVTTPS2DQ xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.f3.0f.w0 5b /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 08) 301 001(5b) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTTPS2DQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31914, 544},
    /* 5122 : VCVTTPS2DQ ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.f3.0f.w0 5b /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 28) 301 001(5b) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTTPS2DQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31923, 544},
    /* 5123 : VCVTTPS2DQ zmmreg|mask|z,zmmrm512|b32|sae [rm:fv:    evex.512.f3.0f.w0 5b /r           ] AVX512,FUTURE */
        /* 250(f1 7e 48) 301 001(5b) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTTPS2DQ, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+31932, 545},
    /* 5124 : VCVTTPS2QQ xmmreg|mask|z,xmmrm64|b32 [rm:hv:    evex.128.66.0f.w0 7a /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 7d 08) 302 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTTPS2QQ, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31941, 547},
    /* 5125 : VCVTTPS2QQ ymmreg|mask|z,xmmrm128|b32 [rm:hv:    evex.256.66.0f.w0 7a /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 7d 28) 302 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTTPS2QQ, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31950, 547},
    /* 5126 : VCVTTPS2QQ zmmreg|mask|z,ymmrm256|b32|sae [rm:hv:    evex.512.66.0f.w0 7a /r           ] AVX512DQ,FUTURE */
        /* 250(f1 7d 48) 302 001(7a) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VCVTTPS2QQ, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+31959, 548},
    /* 5127 : VCVTTPS2UDQ xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.0f.w0 78 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 08) 301 001(78) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTTPS2UDQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31968, 544},
    /* 5128 : VCVTTPS2UDQ ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.0f.w0 78 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 28) 301 001(78) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTTPS2UDQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31977, 544},
    /* 5129 : VCVTTPS2UDQ zmmreg|mask|z,zmmrm512|b32|sae [rm:fv:    evex.512.0f.w0 78 /r              ] AVX512,FUTURE */
        /* 250(f1 7c 48) 301 001(78) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTTPS2UDQ, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+31986, 545},
    /* 5130 : VCVTTPS2UQQ xmmreg|mask|z,xmmrm64|b32 [rm:hv:    evex.128.66.0f.w0 78 /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 7d 08) 302 001(78) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTTPS2UQQ, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31995, 547},
    /* 5131 : VCVTTPS2UQQ ymmreg|mask|z,xmmrm128|b32 [rm:hv:    evex.256.66.0f.w0 78 /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 7d 28) 302 001(78) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTTPS2UQQ, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+32004, 547},
    /* 5132 : VCVTTPS2UQQ zmmreg|mask|z,ymmrm256|b32|sae [rm:hv:    evex.512.66.0f.w0 78 /r           ] AVX512DQ,FUTURE */
        /* 250(f1 7d 48) 302 001(78) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VCVTTPS2UQQ, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+32013, 548},
    /* 5133 : VCVTTSD2SI reg32,xmmrm64|sae [rm:t1f64: evex.128.f2.0f.w0 2c /r           ] AVX512,FUTURE */
        /* 250(f1 7f 08) 310 001(2c) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTTSD2SI, 2, {REG_GPR|BITS32,RM_XMM|BITS64,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+32022, 545},
    /* 5134 : VCVTTSD2SI reg64,xmmrm64|sae [rm:t1f64: evex.128.f2.0f.w1 2c /r           ] AVX512,FUTURE */
        /* 250(f1 ff 08) 310 001(2c) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTTSD2SI, 2, {REG_GPR|BITS64,RM_XMM|BITS64,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+32031, 545},
    /* 5135 : VCVTTSD2USI reg32,xmmrm64|sae [rm:t1f64: evex.128.f2.0f.w0 78 /r           ] AVX512,FUTURE */
        /* 250(f1 7f 08) 310 001(78) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTTSD2USI, 2, {REG_GPR|BITS32,RM_XMM|BITS64,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+32040, 545},
    /* 5136 : VCVTTSD2USI reg64,xmmrm64|sae [rm:t1f64: evex.128.f2.0f.w1 78 /r           ] AVX512,FUTURE */
        /* 250(f1 ff 08) 310 001(78) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTTSD2USI, 2, {REG_GPR|BITS64,RM_XMM|BITS64,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+32049, 545},
    /* 5137 : VCVTTSS2SI reg32,xmmrm32|sae [rm:t1f32: evex.128.f3.0f.w0 2c /r           ] AVX512,FUTURE */
        /* 250(f1 7e 08) 307 001(2c) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTTSS2SI, 2, {REG_GPR|BITS32,RM_XMM|BITS32,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+32058, 545},
    /* 5138 : VCVTTSS2SI reg64,xmmrm32|sae [rm:t1f32: evex.128.f3.0f.w1 2c /r           ] AVX512,FUTURE */
        /* 250(f1 fe 08) 307 001(2c) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTTSS2SI, 2, {REG_GPR|BITS64,RM_XMM|BITS32,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+32067, 545},
    /* 5139 : VCVTTSS2USI reg32,xmmrm32|sae [rm:t1f32: evex.128.f3.0f.w0 78 /r           ] AVX512,FUTURE */
        /* 250(f1 7e 08) 307 001(78) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTTSS2USI, 2, {REG_GPR|BITS32,RM_XMM|BITS32,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+32076, 545},
    /* 5140 : VCVTTSS2USI reg64,xmmrm32|sae [rm:t1f32: evex.128.f3.0f.w1 78 /r           ] AVX512,FUTURE */
        /* 250(f1 fe 08) 307 001(78) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTTSS2USI, 2, {REG_GPR|BITS64,RM_XMM|BITS32,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+32085, 545},
    /* 5141 : VCVTUDQ2PD xmmreg|mask|z,xmmrm64|b32 [rm:hv:    evex.128.f3.0f.w0 7a /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 08) 302 001(7a) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTUDQ2PD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+32094, 544},
    /* 5142 : VCVTUDQ2PD ymmreg|mask|z,xmmrm128|b32 [rm:hv:    evex.256.f3.0f.w0 7a /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 28) 302 001(7a) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTUDQ2PD, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+32103, 544},
    /* 5143 : VCVTUDQ2PD zmmreg|mask|z,ymmrm256|b32|er [rm:hv:    evex.512.f3.0f.w0 7a /r           ] AVX512,FUTURE */
        /* 250(f1 7e 48) 302 001(7a) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTUDQ2PD, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32|ER,0,0,0}, nasm_bytecodes+32112, 545},
    /* 5144 : VCVTUDQ2PS xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.f2.0f.w0 7a /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7f 08) 301 001(7a) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTUDQ2PS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+32121, 544},
    /* 5145 : VCVTUDQ2PS ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.f2.0f.w0 7a /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7f 28) 301 001(7a) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTUDQ2PS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+32130, 544},
    /* 5146 : VCVTUDQ2PS zmmreg|mask|z,zmmrm512|b32|er [rm:fv:    evex.512.f2.0f.w0 7a /r           ] AVX512,FUTURE */
        /* 250(f1 7f 48) 301 001(7a) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTUDQ2PS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|ER,0,0,0}, nasm_bytecodes+32139, 545},
    /* 5147 : VCVTUQQ2PD xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.f3.0f.w1 7a /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fe 08) 301 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTUQQ2PD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+32148, 547},
    /* 5148 : VCVTUQQ2PD ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.f3.0f.w1 7a /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fe 28) 301 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTUQQ2PD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+32157, 547},
    /* 5149 : VCVTUQQ2PD zmmreg|mask|z,zmmrm512|b64|er [rm:fv:    evex.512.f3.0f.w1 7a /r           ] AVX512DQ,FUTURE */
        /* 250(f1 fe 48) 301 001(7a) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VCVTUQQ2PD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|ER,0,0,0}, nasm_bytecodes+32166, 548},
    /* 5150 : VCVTUQQ2PS xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.f2.0f.w1 7a /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 ff 08) 301 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTUQQ2PS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+32175, 547},
    /* 5151 : VCVTUQQ2PS xmmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.f2.0f.w1 7a /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 ff 28) 301 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTUQQ2PS, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+32184, 547},
    /* 5152 : VCVTUQQ2PS ymmreg|mask|z,zmmrm512|b64|er [rm:fv:    evex.512.f2.0f.w1 7a /r           ] AVX512DQ,FUTURE */
        /* 250(f1 ff 48) 301 001(7a) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VCVTUQQ2PS, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|ER,0,0,0}, nasm_bytecodes+32193, 548},
    /* 5153 : VCVTUSI2SD xmmreg,xmmreg|er,rm32 [rvm:t1s:  evex.nds.lig.f2.0f.w0 7b /r       ] AVX512,FUTURE */
        /* 241(f1 7f 08) 306 001(7b) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTUSI2SD, 3, {XMMREG,XMMREG,RM_GPR|BITS32,0,0}, {0,ER,0,0,0}, nasm_bytecodes+32202, 546},
    /* 5154 : VCVTUSI2SD xmmreg,xmmreg|er,rm64 [rvm:t1s:  evex.nds.lig.f2.0f.w1 7b /r       ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(7b) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTUSI2SD, 3, {XMMREG,XMMREG,RM_GPR|BITS64,0,0}, {0,ER,0,0,0}, nasm_bytecodes+32211, 546},
    /* 5155 : VCVTUSI2SS xmmreg,xmmreg|er,rm32 [rvm:t1s:  evex.nds.lig.f3.0f.w0 7b /r       ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(7b) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTUSI2SS, 3, {XMMREG,XMMREG,RM_GPR|BITS32,0,0}, {0,ER,0,0,0}, nasm_bytecodes+32220, 546},
    /* 5156 : VCVTUSI2SS xmmreg,xmmreg|er,rm64 [rvm:t1s:  evex.nds.lig.f3.0f.w1 7b /r       ] AVX512,FUTURE */
        /* 241(f1 fe 08) 306 001(7b) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTUSI2SS, 3, {XMMREG,XMMREG,RM_GPR|BITS64,0,0}, {0,ER,0,0,0}, nasm_bytecodes+32229, 546},
    /* 5157 : VDBPSADBW xmmreg|mask|z,xmmreg*,xmmrm128,imm8 [rvmi:fvm: evex.nds.128.66.0f3a.w0 42 /r ib  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(42) 120 023 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VDBPSADBW, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18363, 550},
    /* 5158 : VDBPSADBW ymmreg|mask|z,ymmreg*,ymmrm256,imm8 [rvmi:fvm: evex.nds.256.66.0f3a.w0 42 /r ib  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(42) 120 023 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VDBPSADBW, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18383, 550},
    /* 5159 : VDBPSADBW zmmreg|mask|z,zmmreg*,zmmrm512,imm8 [rvmi:fvm: evex.nds.512.66.0f3a.w0 42 /r ib  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(42) 120 023 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VDBPSADBW, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18403, 551},
    /* 5160 : VDIVPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 5e /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(5e) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VDIVPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32238, 544},
    /* 5161 : VDIVPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 5e /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(5e) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VDIVPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32256, 544},
    /* 5162 : VDIVPD zmmreg|mask|z,zmmreg*,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f.w1 5e /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(5e) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VDIVPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+32274, 545},
    /* 5163 : VDIVPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.0f.w0 5e /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(5e) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VDIVPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32292, 544},
    /* 5164 : VDIVPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.0f.w0 5e /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(5e) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VDIVPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32310, 544},
    /* 5165 : VDIVPS zmmreg|mask|z,zmmreg*,zmmrm512|b32|er [rvm:fv:   evex.nds.512.0f.w0 5e /r          ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(5e) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VDIVPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+32328, 545},
    /* 5166 : VDIVSD xmmreg|mask|z,xmmreg*,xmmrm64|er [rvm:t1s:  evex.nds.lig.f2.0f.w1 5e /r       ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(5e) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VDIVSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+32346, 546},
    /* 5167 : VDIVSS xmmreg|mask|z,xmmreg*,xmmrm32|er [rvm:t1s:  evex.nds.lig.f3.0f.w0 5e /r       ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(5e) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VDIVSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+32364, 546},
    /* 5168 : VEXP2PD zmmreg|mask|z,zmmrm512|b64|sae [rm:fv:    evex.512.66.0f38.w1 c8 /r         ] AVX512ER,FUTURE */
        /* 250(f2 fd 48) 301 001(c8) 110 : EVEX,PROT,AVX512ER,FUTURE */
        {I_VEXP2PD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+32382, 552},
    /* 5169 : VEXP2PS zmmreg|mask|z,zmmrm512|b32|sae [rm:fv:    evex.512.66.0f38.w0 c8 /r         ] AVX512ER,FUTURE */
        /* 250(f2 7d 48) 301 001(c8) 110 : EVEX,PROT,AVX512ER,FUTURE */
        {I_VEXP2PS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+32391, 552},
    /* 5170 : VEXPANDPD xmmreg|mask|z,mem128 [rm:t1s:   evex.128.66.0f38.w1 88 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 306 001(88) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VEXPANDPD, 2, {XMMREG,MEMORY|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+32400, 544},
    /* 5171 : VEXPANDPD ymmreg|mask|z,mem256 [rm:t1s:   evex.256.66.0f38.w1 88 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 306 001(88) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VEXPANDPD, 2, {YMMREG,MEMORY|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+32409, 544},
    /* 5172 : VEXPANDPD zmmreg|mask|z,mem512 [rm:t1s:   evex.512.66.0f38.w1 88 /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 306 001(88) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VEXPANDPD, 2, {ZMMREG,MEMORY|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+32418, 545},
    /* 5173 : VEXPANDPD xmmreg|mask|z,xmmreg [rm:t1s:   evex.128.66.0f38.w1 88 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 306 001(88) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VEXPANDPD, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+32400, 544},
    /* 5174 : VEXPANDPD ymmreg|mask|z,ymmreg [rm:t1s:   evex.256.66.0f38.w1 88 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 306 001(88) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VEXPANDPD, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+32409, 544},
    /* 5175 : VEXPANDPD zmmreg|mask|z,zmmreg [rm:t1s:   evex.512.66.0f38.w1 88 /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 306 001(88) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VEXPANDPD, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+32418, 545},
    /* 5176 : VEXPANDPS xmmreg|mask|z,mem128 [rm:t1s:   evex.128.66.0f38.w0 88 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 306 001(88) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VEXPANDPS, 2, {XMMREG,MEMORY|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+32427, 544},
    /* 5177 : VEXPANDPS ymmreg|mask|z,mem256 [rm:t1s:   evex.256.66.0f38.w0 88 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 306 001(88) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VEXPANDPS, 2, {YMMREG,MEMORY|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+32436, 544},
    /* 5178 : VEXPANDPS zmmreg|mask|z,mem512 [rm:t1s:   evex.512.66.0f38.w0 88 /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 306 001(88) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VEXPANDPS, 2, {ZMMREG,MEMORY|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+32445, 545},
    /* 5179 : VEXPANDPS xmmreg|mask|z,xmmreg [rm:t1s:   evex.128.66.0f38.w0 88 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 306 001(88) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VEXPANDPS, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+32427, 544},
    /* 5180 : VEXPANDPS ymmreg|mask|z,ymmreg [rm:t1s:   evex.256.66.0f38.w0 88 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 306 001(88) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VEXPANDPS, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+32436, 544},
    /* 5181 : VEXPANDPS zmmreg|mask|z,zmmreg [rm:t1s:   evex.512.66.0f38.w0 88 /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 306 001(88) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VEXPANDPS, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+32445, 545},
    /* 5182 : VEXTRACTF32X4 xmmreg|mask|z,ymmreg,imm8 [mri:      evex.256.66.0f3a.w0 19 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 28) 300 001(19) 101 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VEXTRACTF32X4, 3, {XMMREG,YMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18423, 544},
    /* 5183 : VEXTRACTF32X4 xmmreg|mask|z,zmmreg,imm8 [mri:      evex.512.66.0f3a.w0 19 /r ib      ] AVX512,FUTURE */
        /* 250(f3 7d 48) 300 001(19) 101 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VEXTRACTF32X4, 3, {XMMREG,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18433, 545},
    /* 5184 : VEXTRACTF32X4 mem128|mask,ymmreg,imm8 [mri:t4:   evex.256.66.0f3a.w0 19 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 28) 312 001(19) 101 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VEXTRACTF32X4, 3, {MEMORY|BITS128,YMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18443, 544},
    /* 5185 : VEXTRACTF32X4 mem128|mask,zmmreg,imm8 [mri:t4:   evex.512.66.0f3a.w0 19 /r ib      ] AVX512,FUTURE */
        /* 250(f3 7d 48) 312 001(19) 101 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VEXTRACTF32X4, 3, {MEMORY|BITS128,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18453, 545},
    /* 5186 : VEXTRACTF32X8 ymmreg|mask|z,zmmreg,imm8 [mri:      evex.512.66.0f3a.w0 1b /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 7d 48) 300 001(1b) 101 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VEXTRACTF32X8, 3, {YMMREG,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18463, 548},
    /* 5187 : VEXTRACTF32X8 mem256|mask,zmmreg,imm8 [mri:t8:   evex.512.66.0f3a.w0 1b /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 7d 48) 313 001(1b) 101 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VEXTRACTF32X8, 3, {MEMORY|BITS256,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18473, 548},
    /* 5188 : VEXTRACTF64X2 xmmreg|mask|z,ymmreg,imm8 [mri:      evex.256.66.0f3a.w1 19 /r ib      ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f3 fd 28) 300 001(19) 101 022 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VEXTRACTF64X2, 3, {XMMREG,YMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18483, 547},
    /* 5189 : VEXTRACTF64X2 xmmreg|mask|z,zmmreg,imm8 [mri:      evex.512.66.0f3a.w1 19 /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 fd 48) 300 001(19) 101 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VEXTRACTF64X2, 3, {XMMREG,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18493, 548},
    /* 5190 : VEXTRACTF64X2 mem128|mask,ymmreg,imm8 [mri:t2:   evex.256.66.0f3a.w1 19 /r ib      ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f3 fd 28) 311 001(19) 101 022 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VEXTRACTF64X2, 3, {MEMORY|BITS128,YMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18503, 547},
    /* 5191 : VEXTRACTF64X2 mem128|mask,zmmreg,imm8 [mri:t2:   evex.512.66.0f3a.w1 19 /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 fd 48) 311 001(19) 101 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VEXTRACTF64X2, 3, {MEMORY|BITS128,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18513, 548},
    /* 5192 : VEXTRACTF64X4 ymmreg|mask|z,zmmreg,imm8 [mri:      evex.512.66.0f3a.w1 1b /r ib      ] AVX512,FUTURE */
        /* 250(f3 fd 48) 300 001(1b) 101 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VEXTRACTF64X4, 3, {YMMREG,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18523, 545},
    /* 5193 : VEXTRACTF64X4 mem256|mask,zmmreg,imm8 [mri:t4:   evex.512.66.0f3a.w1 1b /r ib      ] AVX512,FUTURE */
        /* 250(f3 fd 48) 312 001(1b) 101 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VEXTRACTF64X4, 3, {MEMORY|BITS256,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18533, 545},
    /* 5194 : VEXTRACTI32X4 xmmreg|mask|z,ymmreg,imm8 [mri:      evex.256.66.0f3a.w0 39 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 28) 300 001(39) 101 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VEXTRACTI32X4, 3, {XMMREG,YMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18543, 544},
    /* 5195 : VEXTRACTI32X4 xmmreg|mask|z,zmmreg,imm8 [mri:      evex.512.66.0f3a.w0 39 /r ib      ] AVX512,FUTURE */
        /* 250(f3 7d 48) 300 001(39) 101 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VEXTRACTI32X4, 3, {XMMREG,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18553, 545},
    /* 5196 : VEXTRACTI32X4 mem128|mask,ymmreg,imm8 [mri:t4:   evex.256.66.0f3a.w0 39 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 28) 312 001(39) 101 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VEXTRACTI32X4, 3, {MEMORY|BITS128,YMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18563, 544},
    /* 5197 : VEXTRACTI32X4 mem128|mask,zmmreg,imm8 [mri:t4:   evex.512.66.0f3a.w0 39 /r ib      ] AVX512,FUTURE */
        /* 250(f3 7d 48) 312 001(39) 101 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VEXTRACTI32X4, 3, {MEMORY|BITS128,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18573, 545},
    /* 5198 : VEXTRACTI32X8 ymmreg|mask|z,zmmreg,imm8 [mri:      evex.512.66.0f3a.w0 3b /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 7d 48) 300 001(3b) 101 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VEXTRACTI32X8, 3, {YMMREG,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18583, 548},
    /* 5199 : VEXTRACTI32X8 mem256|mask,zmmreg,imm8 [mri:t8:   evex.512.66.0f3a.w0 3b /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 7d 48) 313 001(3b) 101 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VEXTRACTI32X8, 3, {MEMORY|BITS256,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18593, 548},
    /* 5200 : VEXTRACTI64X2 xmmreg|mask|z,ymmreg,imm8 [mri:      evex.256.66.0f3a.w1 39 /r ib      ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f3 fd 28) 300 001(39) 101 022 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VEXTRACTI64X2, 3, {XMMREG,YMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18603, 547},
    /* 5201 : VEXTRACTI64X2 xmmreg|mask|z,zmmreg,imm8 [mri:      evex.512.66.0f3a.w1 39 /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 fd 48) 300 001(39) 101 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VEXTRACTI64X2, 3, {XMMREG,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18613, 548},
    /* 5202 : VEXTRACTI64X2 mem128|mask,ymmreg,imm8 [mri:t2:   evex.256.66.0f3a.w1 39 /r ib      ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f3 fd 28) 311 001(39) 101 022 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VEXTRACTI64X2, 3, {MEMORY|BITS128,YMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18623, 547},
    /* 5203 : VEXTRACTI64X2 mem128|mask,zmmreg,imm8 [mri:t2:   evex.512.66.0f3a.w1 39 /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 fd 48) 311 001(39) 101 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VEXTRACTI64X2, 3, {MEMORY|BITS128,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18633, 548},
    /* 5204 : VEXTRACTI64X4 ymmreg|mask|z,zmmreg,imm8 [mri:      evex.512.66.0f3a.w1 3b /r ib      ] AVX512,FUTURE */
        /* 250(f3 fd 48) 300 001(3b) 101 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VEXTRACTI64X4, 3, {YMMREG,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18643, 545},
    /* 5205 : VEXTRACTI64X4 mem256|mask,zmmreg,imm8 [mri:t4:   evex.512.66.0f3a.w1 3b /r ib      ] AVX512,FUTURE */
        /* 250(f3 fd 48) 312 001(3b) 101 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VEXTRACTI64X4, 3, {MEMORY|BITS256,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18653, 545},
    /* 5206 : VEXTRACTPS reg32,xmmreg,imm8 [mri:t1s:  evex.128.66.0f3a.wig 17 /r ib     ] AVX512,FUTURE */
        /* 250(f3 7d 08) 306 001(17) 101 022 : WIG,EVEX,PROT,AVX512,FUTURE */
        {I_VEXTRACTPS, 3, {REG_GPR|BITS32,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+18663, 553},
    /* 5207 : VEXTRACTPS reg64,xmmreg,imm8 [mri:t1s:  evex.128.66.0f3a.wig 17 /r ib     ] AVX512,FUTURE */
        /* 250(f3 7d 08) 306 001(17) 101 022 : WIG,EVEX,PROT,AVX512,FUTURE */
        {I_VEXTRACTPS, 3, {REG_GPR|BITS64,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+18663, 553},
    /* 5208 : VEXTRACTPS mem32,xmmreg,imm8 [mri:t1s:  evex.128.66.0f3a.wig 17 /r ib     ] AVX512,FUTURE */
        /* 250(f3 7d 08) 306 001(17) 101 022 : WIG,EVEX,PROT,AVX512,FUTURE */
        {I_VEXTRACTPS, 3, {MEMORY|BITS32,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+18663, 553},
    /* 5209 : VFIXUPIMMPD xmmreg|mask|z,xmmreg*,xmmrm128|b64,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w1 54 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(54) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFIXUPIMMPD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+18673, 544},
    /* 5210 : VFIXUPIMMPD ymmreg|mask|z,ymmreg*,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w1 54 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(54) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFIXUPIMMPD, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+18693, 544},
    /* 5211 : VFIXUPIMMPD zmmreg|mask|z,zmmreg*,zmmrm512|b64|sae,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w1 54 /r ib  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(54) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VFIXUPIMMPD, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64|SAE,0,0}, nasm_bytecodes+18713, 545},
    /* 5212 : VFIXUPIMMPS xmmreg|mask|z,xmmreg*,xmmrm128|b32,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w0 54 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(54) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFIXUPIMMPS, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+18733, 544},
    /* 5213 : VFIXUPIMMPS ymmreg|mask|z,ymmreg*,ymmrm256|b32,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w0 54 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(54) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFIXUPIMMPS, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+18753, 544},
    /* 5214 : VFIXUPIMMPS zmmreg|mask|z,zmmreg*,zmmrm512|b32|sae,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w0 54 /r ib  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(54) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VFIXUPIMMPS, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32|SAE,0,0}, nasm_bytecodes+18773, 545},
    /* 5215 : VFIXUPIMMSD xmmreg|mask|z,xmmreg*,xmmrm64|sae,imm8 [rvmi:t1s: evex.nds.lig.66.0f3a.w1 55 /r ib  ] AVX512,FUTURE */
        /* 241(f3 fd 08) 306 001(55) 120 023 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFIXUPIMMSD, 4, {XMMREG,XMMREG,RM_XMM|BITS64,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+18793, 546},
    /* 5216 : VFIXUPIMMSS xmmreg|mask|z,xmmreg*,xmmrm32|sae,imm8 [rvmi:t1s: evex.nds.lig.66.0f3a.w0 55 /r ib  ] AVX512,FUTURE */
        /* 241(f3 7d 08) 306 001(55) 120 023 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFIXUPIMMSS, 4, {XMMREG,XMMREG,RM_XMM|BITS32,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+18813, 546},
    /* 5217 : VFMADD132PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 98 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(98) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADD132PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32454, 544},
    /* 5218 : VFMADD132PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 98 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(98) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADD132PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32463, 544},
    /* 5219 : VFMADD132PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 98 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(98) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMADD132PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+32472, 545},
    /* 5220 : VFMADD132PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 98 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(98) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADD132PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32481, 544},
    /* 5221 : VFMADD132PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 98 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(98) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADD132PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32490, 544},
    /* 5222 : VFMADD132PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 98 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(98) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMADD132PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+32499, 545},
    /* 5223 : VFMADD132SD xmmreg|mask|z,xmmreg,xmmrm64|er [rvm:t1s:  evex.nds.lig.66.0f38.w1 99 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(99) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFMADD132SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+32508, 546},
    /* 5224 : VFMADD132SS xmmreg|mask|z,xmmreg,xmmrm32|er [rvm:t1s:  evex.nds.lig.66.0f38.w0 99 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(99) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFMADD132SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+32517, 546},
    /* 5225 : VFMADD213PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 a8 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(a8) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADD213PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32526, 544},
    /* 5226 : VFMADD213PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 a8 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(a8) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADD213PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32535, 544},
    /* 5227 : VFMADD213PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 a8 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(a8) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMADD213PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+32544, 545},
    /* 5228 : VFMADD213PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 a8 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(a8) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADD213PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32553, 544},
    /* 5229 : VFMADD213PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 a8 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(a8) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADD213PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32562, 544},
    /* 5230 : VFMADD213PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 a8 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(a8) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMADD213PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+32571, 545},
    /* 5231 : VFMADD213SD xmmreg|mask|z,xmmreg,xmmrm64|er [rvm:t1s:  evex.nds.lig.66.0f38.w1 a9 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(a9) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFMADD213SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+32580, 546},
    /* 5232 : VFMADD213SS xmmreg|mask|z,xmmreg,xmmrm32|er [rvm:t1s:  evex.nds.lig.66.0f38.w0 a9 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(a9) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFMADD213SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+32589, 546},
    /* 5233 : VFMADD231PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 b8 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(b8) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADD231PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32598, 544},
    /* 5234 : VFMADD231PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 b8 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(b8) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADD231PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32607, 544},
    /* 5235 : VFMADD231PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 b8 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(b8) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMADD231PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+32616, 545},
    /* 5236 : VFMADD231PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 b8 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(b8) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADD231PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32625, 544},
    /* 5237 : VFMADD231PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 b8 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(b8) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADD231PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32634, 544},
    /* 5238 : VFMADD231PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 b8 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(b8) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMADD231PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+32643, 545},
    /* 5239 : VFMADD231SD xmmreg|mask|z,xmmreg,xmmrm64|er [rvm:t1s:  evex.nds.lig.66.0f38.w1 b9 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(b9) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFMADD231SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+32652, 546},
    /* 5240 : VFMADD231SS xmmreg|mask|z,xmmreg,xmmrm32|er [rvm:t1s:  evex.nds.lig.66.0f38.w0 b9 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(b9) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFMADD231SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+32661, 546},
    /* 5241 : VFMADDSUB132PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 96 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(96) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADDSUB132PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32670, 544},
    /* 5242 : VFMADDSUB132PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 96 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(96) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADDSUB132PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32679, 544},
    /* 5243 : VFMADDSUB132PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 96 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(96) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMADDSUB132PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+32688, 545},
    /* 5244 : VFMADDSUB132PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 96 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(96) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADDSUB132PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32697, 544},
    /* 5245 : VFMADDSUB132PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 96 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(96) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADDSUB132PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32706, 544},
    /* 5246 : VFMADDSUB132PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 96 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(96) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMADDSUB132PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+32715, 545},
    /* 5247 : VFMADDSUB213PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 a6 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(a6) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADDSUB213PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32724, 544},
    /* 5248 : VFMADDSUB213PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 a6 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(a6) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADDSUB213PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32733, 544},
    /* 5249 : VFMADDSUB213PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 a6 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(a6) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMADDSUB213PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+32742, 545},
    /* 5250 : VFMADDSUB213PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 a6 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(a6) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADDSUB213PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32751, 544},
    /* 5251 : VFMADDSUB213PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 a6 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(a6) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADDSUB213PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32760, 544},
    /* 5252 : VFMADDSUB213PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 a6 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(a6) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMADDSUB213PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+32769, 545},
    /* 5253 : VFMADDSUB231PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 b6 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(b6) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADDSUB231PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32778, 544},
    /* 5254 : VFMADDSUB231PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 b6 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(b6) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADDSUB231PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32787, 544},
    /* 5255 : VFMADDSUB231PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 b6 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(b6) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMADDSUB231PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+32796, 545},
    /* 5256 : VFMADDSUB231PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 b6 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(b6) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADDSUB231PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32805, 544},
    /* 5257 : VFMADDSUB231PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 b6 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(b6) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADDSUB231PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32814, 544},
    /* 5258 : VFMADDSUB231PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 b6 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(b6) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMADDSUB231PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+32823, 545},
    /* 5259 : VFMSUB132PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 9a /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(9a) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUB132PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32832, 544},
    /* 5260 : VFMSUB132PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 9a /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(9a) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUB132PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32841, 544},
    /* 5261 : VFMSUB132PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 9a /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(9a) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUB132PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+32850, 545},
    /* 5262 : VFMSUB132PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 9a /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(9a) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUB132PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32859, 544},
    /* 5263 : VFMSUB132PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 9a /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(9a) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUB132PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32868, 544},
    /* 5264 : VFMSUB132PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 9a /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(9a) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUB132PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+32877, 545},
    /* 5265 : VFMSUB132SD xmmreg|mask|z,xmmreg,xmmrm64|er [rvm:t1s:  evex.nds.lig.66.0f38.w1 9b /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(9b) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUB132SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+32886, 546},
    /* 5266 : VFMSUB132SS xmmreg|mask|z,xmmreg,xmmrm32|er [rvm:t1s:  evex.nds.lig.66.0f38.w0 9b /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(9b) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUB132SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+32895, 546},
    /* 5267 : VFMSUB213PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 aa /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(aa) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUB213PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32904, 544},
    /* 5268 : VFMSUB213PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 aa /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(aa) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUB213PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32913, 544},
    /* 5269 : VFMSUB213PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 aa /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(aa) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUB213PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+32922, 545},
    /* 5270 : VFMSUB213PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 aa /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(aa) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUB213PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32931, 544},
    /* 5271 : VFMSUB213PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 aa /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(aa) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUB213PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32940, 544},
    /* 5272 : VFMSUB213PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 aa /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(aa) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUB213PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+32949, 545},
    /* 5273 : VFMSUB213SD xmmreg|mask|z,xmmreg,xmmrm64|er [rvm:t1s:  evex.nds.lig.66.0f38.w1 ab /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(ab) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUB213SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+32958, 546},
    /* 5274 : VFMSUB213SS xmmreg|mask|z,xmmreg,xmmrm32|er [rvm:t1s:  evex.nds.lig.66.0f38.w0 ab /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(ab) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUB213SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+32967, 546},
    /* 5275 : VFMSUB231PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 ba /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(ba) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUB231PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32976, 544},
    /* 5276 : VFMSUB231PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 ba /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(ba) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUB231PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32985, 544},
    /* 5277 : VFMSUB231PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 ba /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(ba) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUB231PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+32994, 545},
    /* 5278 : VFMSUB231PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 ba /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(ba) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUB231PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33003, 544},
    /* 5279 : VFMSUB231PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 ba /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(ba) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUB231PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33012, 544},
    /* 5280 : VFMSUB231PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 ba /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(ba) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUB231PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+33021, 545},
    /* 5281 : VFMSUB231SD xmmreg|mask|z,xmmreg,xmmrm64|er [rvm:t1s:  evex.nds.lig.66.0f38.w1 bb /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(bb) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUB231SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33030, 546},
    /* 5282 : VFMSUB231SS xmmreg|mask|z,xmmreg,xmmrm32|er [rvm:t1s:  evex.nds.lig.66.0f38.w0 bb /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(bb) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUB231SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33039, 546},
    /* 5283 : VFMSUBADD132PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 97 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(97) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUBADD132PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33048, 544},
    /* 5284 : VFMSUBADD132PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 97 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(97) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUBADD132PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33057, 544},
    /* 5285 : VFMSUBADD132PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 97 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(97) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUBADD132PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+33066, 545},
    /* 5286 : VFMSUBADD132PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 97 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(97) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUBADD132PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33075, 544},
    /* 5287 : VFMSUBADD132PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 97 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(97) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUBADD132PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33084, 544},
    /* 5288 : VFMSUBADD132PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 97 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(97) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUBADD132PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+33093, 545},
    /* 5289 : VFMSUBADD213PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 a7 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(a7) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUBADD213PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33102, 544},
    /* 5290 : VFMSUBADD213PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 a7 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(a7) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUBADD213PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33111, 544},
    /* 5291 : VFMSUBADD213PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 a7 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(a7) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUBADD213PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+33120, 545},
    /* 5292 : VFMSUBADD213PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 a7 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(a7) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUBADD213PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33129, 544},
    /* 5293 : VFMSUBADD213PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 a7 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(a7) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUBADD213PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33138, 544},
    /* 5294 : VFMSUBADD213PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 a7 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(a7) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUBADD213PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+33147, 545},
    /* 5295 : VFMSUBADD231PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 b7 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(b7) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUBADD231PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33156, 544},
    /* 5296 : VFMSUBADD231PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 b7 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(b7) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUBADD231PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33165, 544},
    /* 5297 : VFMSUBADD231PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 b7 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(b7) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUBADD231PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+33174, 545},
    /* 5298 : VFMSUBADD231PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 b7 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(b7) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUBADD231PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33183, 544},
    /* 5299 : VFMSUBADD231PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 b7 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(b7) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUBADD231PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33192, 544},
    /* 5300 : VFMSUBADD231PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 b7 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(b7) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUBADD231PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+33201, 545},
    /* 5301 : VFNMADD132PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 9c /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(9c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMADD132PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33210, 544},
    /* 5302 : VFNMADD132PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 9c /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(9c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMADD132PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33219, 544},
    /* 5303 : VFNMADD132PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 9c /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(9c) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFNMADD132PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+33228, 545},
    /* 5304 : VFNMADD132PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 9c /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(9c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMADD132PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33237, 544},
    /* 5305 : VFNMADD132PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 9c /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(9c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMADD132PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33246, 544},
    /* 5306 : VFNMADD132PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 9c /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(9c) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFNMADD132PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+33255, 545},
    /* 5307 : VFNMADD132SD xmmreg|mask|z,xmmreg,xmmrm64|er [rvm:t1s:  evex.nds.lig.66.0f38.w1 9d /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(9d) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFNMADD132SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33264, 546},
    /* 5308 : VFNMADD132SS xmmreg|mask|z,xmmreg,xmmrm32|er [rvm:t1s:  evex.nds.lig.66.0f38.w0 9d /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(9d) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFNMADD132SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33273, 546},
    /* 5309 : VFNMADD213PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 ac /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(ac) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMADD213PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33282, 544},
    /* 5310 : VFNMADD213PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 ac /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(ac) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMADD213PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33291, 544},
    /* 5311 : VFNMADD213PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 ac /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(ac) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFNMADD213PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+33300, 545},
    /* 5312 : VFNMADD213PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 ac /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(ac) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMADD213PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33309, 544},
    /* 5313 : VFNMADD213PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 ac /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(ac) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMADD213PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33318, 544},
    /* 5314 : VFNMADD213PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 ac /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(ac) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFNMADD213PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+33327, 545},
    /* 5315 : VFNMADD213SD xmmreg|mask|z,xmmreg,xmmrm64|er [rvm:t1s:  evex.nds.lig.66.0f38.w1 ad /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(ad) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFNMADD213SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33336, 546},
    /* 5316 : VFNMADD213SS xmmreg|mask|z,xmmreg,xmmrm32|er [rvm:t1s:  evex.nds.lig.66.0f38.w0 ad /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(ad) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFNMADD213SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33345, 546},
    /* 5317 : VFNMADD231PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 bc /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(bc) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMADD231PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33354, 544},
    /* 5318 : VFNMADD231PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 bc /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(bc) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMADD231PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33363, 544},
    /* 5319 : VFNMADD231PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 bc /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(bc) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFNMADD231PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+33372, 545},
    /* 5320 : VFNMADD231PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 bc /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(bc) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMADD231PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33381, 544},
    /* 5321 : VFNMADD231PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 bc /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(bc) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMADD231PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33390, 544},
    /* 5322 : VFNMADD231PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 bc /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(bc) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFNMADD231PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+33399, 545},
    /* 5323 : VFNMADD231SD xmmreg|mask|z,xmmreg,xmmrm64|er [rvm:t1s:  evex.nds.lig.66.0f38.w1 bd /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(bd) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFNMADD231SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33408, 546},
    /* 5324 : VFNMADD231SS xmmreg|mask|z,xmmreg,xmmrm32|er [rvm:t1s:  evex.nds.lig.66.0f38.w0 bd /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(bd) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFNMADD231SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33417, 546},
    /* 5325 : VFNMSUB132PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 9e /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(9e) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMSUB132PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33426, 544},
    /* 5326 : VFNMSUB132PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 9e /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(9e) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMSUB132PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33435, 544},
    /* 5327 : VFNMSUB132PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 9e /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(9e) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFNMSUB132PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+33444, 545},
    /* 5328 : VFNMSUB132PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 9e /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(9e) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMSUB132PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33453, 544},
    /* 5329 : VFNMSUB132PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 9e /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(9e) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMSUB132PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33462, 544},
    /* 5330 : VFNMSUB132PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 9e /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(9e) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFNMSUB132PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+33471, 545},
    /* 5331 : VFNMSUB132SD xmmreg|mask|z,xmmreg,xmmrm64|er [rvm:t1s:  evex.nds.lig.66.0f38.w1 9f /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(9f) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFNMSUB132SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33480, 546},
    /* 5332 : VFNMSUB132SS xmmreg|mask|z,xmmreg,xmmrm32|er [rvm:t1s:  evex.nds.lig.66.0f38.w0 9f /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(9f) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFNMSUB132SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33489, 546},
    /* 5333 : VFNMSUB213PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 ae /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(ae) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMSUB213PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33498, 544},
    /* 5334 : VFNMSUB213PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 ae /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(ae) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMSUB213PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33507, 544},
    /* 5335 : VFNMSUB213PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 ae /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(ae) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFNMSUB213PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+33516, 545},
    /* 5336 : VFNMSUB213PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 ae /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(ae) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMSUB213PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33525, 544},
    /* 5337 : VFNMSUB213PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 ae /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(ae) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMSUB213PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33534, 544},
    /* 5338 : VFNMSUB213PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 ae /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(ae) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFNMSUB213PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+33543, 545},
    /* 5339 : VFNMSUB213SD xmmreg|mask|z,xmmreg,xmmrm64|er [rvm:t1s:  evex.nds.lig.66.0f38.w1 af /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(af) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFNMSUB213SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33552, 546},
    /* 5340 : VFNMSUB213SS xmmreg|mask|z,xmmreg,xmmrm32|er [rvm:t1s:  evex.nds.lig.66.0f38.w0 af /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(af) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFNMSUB213SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33561, 546},
    /* 5341 : VFNMSUB231PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 be /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(be) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMSUB231PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33570, 544},
    /* 5342 : VFNMSUB231PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 be /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(be) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMSUB231PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33579, 544},
    /* 5343 : VFNMSUB231PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 be /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(be) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFNMSUB231PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+33588, 545},
    /* 5344 : VFNMSUB231PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 be /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(be) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMSUB231PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33597, 544},
    /* 5345 : VFNMSUB231PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 be /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(be) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMSUB231PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33606, 544},
    /* 5346 : VFNMSUB231PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 be /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(be) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFNMSUB231PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+33615, 545},
    /* 5347 : VFNMSUB231SD xmmreg|mask|z,xmmreg,xmmrm64|er [rvm:t1s:  evex.nds.lig.66.0f38.w1 bf /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(bf) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFNMSUB231SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33624, 546},
    /* 5348 : VFNMSUB231SS xmmreg|mask|z,xmmreg,xmmrm32|er [rvm:t1s:  evex.nds.lig.66.0f38.w0 bf /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(bf) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFNMSUB231SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33633, 546},
    /* 5349 : VFPCLASSPD kreg|mask,xmmrm128|b64,imm8 [rmi:fv:   evex.128.66.0f3a.w1 66 /r ib      ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f3 fd 08) 301 001(66) 110 022 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VFPCLASSPD, 3, {OPMASKREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK,B64,0,0,0}, nasm_bytecodes+18833, 547},
    /* 5350 : VFPCLASSPD kreg|mask,ymmrm256|b64,imm8 [rmi:fv:   evex.256.66.0f3a.w1 66 /r ib      ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f3 fd 28) 301 001(66) 110 022 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VFPCLASSPD, 3, {OPMASKREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK,B64,0,0,0}, nasm_bytecodes+18843, 547},
    /* 5351 : VFPCLASSPD kreg|mask,zmmrm512|b64,imm8 [rmi:fv:   evex.512.66.0f3a.w1 66 /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 fd 48) 301 001(66) 110 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VFPCLASSPD, 3, {OPMASKREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK,B64,0,0,0}, nasm_bytecodes+18853, 548},
    /* 5352 : VFPCLASSPS kreg|mask,xmmrm128|b32,imm8 [rmi:fv:   evex.128.66.0f3a.w0 66 /r ib      ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f3 7d 08) 301 001(66) 110 022 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VFPCLASSPS, 3, {OPMASKREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK,B32,0,0,0}, nasm_bytecodes+18863, 547},
    /* 5353 : VFPCLASSPS kreg|mask,ymmrm256|b32,imm8 [rmi:fv:   evex.256.66.0f3a.w0 66 /r ib      ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f3 7d 28) 301 001(66) 110 022 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VFPCLASSPS, 3, {OPMASKREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK,B32,0,0,0}, nasm_bytecodes+18873, 547},
    /* 5354 : VFPCLASSPS kreg|mask,zmmrm512|b32,imm8 [rmi:fv:   evex.512.66.0f3a.w0 66 /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 7d 48) 301 001(66) 110 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VFPCLASSPS, 3, {OPMASKREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK,B32,0,0,0}, nasm_bytecodes+18883, 548},
    /* 5355 : VFPCLASSSD kreg|mask,xmmrm64,imm8 [rmi:t1s:  evex.lig.66.0f3a.w1 67 /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 fd 08) 306 001(67) 110 022 : LIG,EVEX,PROT,AVX512DQ,FUTURE */
        {I_VFPCLASSSD, 3, {OPMASKREG,RM_XMM|BITS64,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18893, 554},
    /* 5356 : VFPCLASSSS kreg|mask,xmmrm32,imm8 [rmi:t1s:  evex.lig.66.0f3a.w0 67 /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 7d 08) 306 001(67) 110 022 : LIG,EVEX,PROT,AVX512DQ,FUTURE */
        {I_VFPCLASSSS, 3, {OPMASKREG,RM_XMM|BITS32,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18903, 554},
    /* 5357 : VGATHERDPD xmmreg|mask,xmem64 [rm:t1s:   vsibx evex.128.66.0f38.w1 92 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 fd 08) 306 001(92) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGATHERDPD, 2, {XMMREG,XMEM|BITS64,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18913, 544},
    /* 5358 : VGATHERDPD ymmreg|mask,xmem64 [rm:t1s:   vsibx evex.256.66.0f38.w1 92 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 fd 28) 306 001(92) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGATHERDPD, 2, {YMMREG,XMEM|BITS64,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18923, 544},
    /* 5359 : VGATHERDPD zmmreg|mask,ymem64 [rm:t1s:   vsiby evex.512.66.0f38.w1 92 /r   ] AVX512,FUTURE */
        /* 375 250(f2 fd 48) 306 001(92) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VGATHERDPD, 2, {ZMMREG,YMEM|BITS64,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18933, 545},
    /* 5360 : VGATHERDPS xmmreg|mask,xmem32 [rm:t1s:   vsibx evex.128.66.0f38.w0 92 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 7d 08) 306 001(92) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGATHERDPS, 2, {XMMREG,XMEM|BITS32,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18943, 544},
    /* 5361 : VGATHERDPS ymmreg|mask,ymem32 [rm:t1s:   vsiby evex.256.66.0f38.w0 92 /r   ] AVX512,AVX512VL,FUTURE */
        /* 375 250(f2 7d 28) 306 001(92) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGATHERDPS, 2, {YMMREG,YMEM|BITS32,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18953, 544},
    /* 5362 : VGATHERDPS zmmreg|mask,zmem32 [rm:t1s:   vsibz evex.512.66.0f38.w0 92 /r   ] AVX512,FUTURE */
        /* 376 250(f2 7d 48) 306 001(92) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VGATHERDPS, 2, {ZMMREG,ZMEM|BITS32,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18963, 545},
    /* 5363 : VGATHERPF0DPD ymem64|mask [m:t1s:    vsiby evex.512.66.0f38.w1 c6 /1   ] AVX512PF,FUTURE */
        /* 375 250(f2 fd 48) 306 001(c6) 201 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VGATHERPF0DPD, 1, {YMEM|BITS64,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18973, 555},
    /* 5364 : VGATHERPF0DPS zmem32|mask [m:t1s:    vsibz evex.512.66.0f38.w0 c6 /1   ] AVX512PF,FUTURE */
        /* 376 250(f2 7d 48) 306 001(c6) 201 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VGATHERPF0DPS, 1, {ZMEM|BITS32,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18983, 555},
    /* 5365 : VGATHERPF0QPD zmem64|mask [m:t1s:    vsibz evex.512.66.0f38.w1 c7 /1   ] AVX512PF,FUTURE */
        /* 376 250(f2 fd 48) 306 001(c7) 201 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VGATHERPF0QPD, 1, {ZMEM|BITS64,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18993, 555},
    /* 5366 : VGATHERPF0QPS zmem32|mask [m:t1s:    vsibz evex.512.66.0f38.w0 c7 /1   ] AVX512PF,FUTURE */
        /* 376 250(f2 7d 48) 306 001(c7) 201 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VGATHERPF0QPS, 1, {ZMEM|BITS32,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19003, 555},
    /* 5367 : VGATHERPF1DPD ymem64|mask [m:t1s:    vsiby evex.512.66.0f38.w1 c6 /2   ] AVX512PF,FUTURE */
        /* 375 250(f2 fd 48) 306 001(c6) 202 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VGATHERPF1DPD, 1, {YMEM|BITS64,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19013, 555},
    /* 5368 : VGATHERPF1DPS zmem32|mask [m:t1s:    vsibz evex.512.66.0f38.w0 c6 /2   ] AVX512PF,FUTURE */
        /* 376 250(f2 7d 48) 306 001(c6) 202 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VGATHERPF1DPS, 1, {ZMEM|BITS32,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19023, 555},
    /* 5369 : VGATHERPF1QPD zmem64|mask [m:t1s:    vsibz evex.512.66.0f38.w1 c7 /2   ] AVX512PF,FUTURE */
        /* 376 250(f2 fd 48) 306 001(c7) 202 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VGATHERPF1QPD, 1, {ZMEM|BITS64,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19033, 555},
    /* 5370 : VGATHERPF1QPS zmem32|mask [m:t1s:    vsibz evex.512.66.0f38.w0 c7 /2   ] AVX512PF,FUTURE */
        /* 376 250(f2 7d 48) 306 001(c7) 202 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VGATHERPF1QPS, 1, {ZMEM|BITS32,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19043, 555},
    /* 5371 : VGATHERQPD xmmreg|mask,xmem64 [rm:t1s:   vsibx evex.128.66.0f38.w1 93 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 fd 08) 306 001(93) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGATHERQPD, 2, {XMMREG,XMEM|BITS64,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19053, 544},
    /* 5372 : VGATHERQPD ymmreg|mask,ymem64 [rm:t1s:   vsiby evex.256.66.0f38.w1 93 /r   ] AVX512,AVX512VL,FUTURE */
        /* 375 250(f2 fd 28) 306 001(93) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGATHERQPD, 2, {YMMREG,YMEM|BITS64,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19063, 544},
    /* 5373 : VGATHERQPD zmmreg|mask,zmem64 [rm:t1s:   vsibz evex.512.66.0f38.w1 93 /r   ] AVX512,FUTURE */
        /* 376 250(f2 fd 48) 306 001(93) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VGATHERQPD, 2, {ZMMREG,ZMEM|BITS64,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19073, 545},
    /* 5374 : VGATHERQPS xmmreg|mask,xmem32 [rm:t1s:   vsibx evex.128.66.0f38.w0 93 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 7d 08) 306 001(93) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGATHERQPS, 2, {XMMREG,XMEM|BITS32,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19083, 544},
    /* 5375 : VGATHERQPS xmmreg|mask,ymem32 [rm:t1s:   vsiby evex.256.66.0f38.w0 93 /r   ] AVX512,AVX512VL,FUTURE */
        /* 375 250(f2 7d 28) 306 001(93) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGATHERQPS, 2, {XMMREG,YMEM|BITS32,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19093, 544},
    /* 5376 : VGATHERQPS ymmreg|mask,zmem32 [rm:t1s:   vsibz evex.512.66.0f38.w0 93 /r   ] AVX512,FUTURE */
        /* 376 250(f2 7d 48) 306 001(93) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VGATHERQPS, 2, {YMMREG,ZMEM|BITS32,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19103, 545},
    /* 5377 : VGETEXPPD xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.0f38.w1 42 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 301 001(42) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGETEXPPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+33642, 544},
    /* 5378 : VGETEXPPD ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.0f38.w1 42 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 301 001(42) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGETEXPPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+33651, 544},
    /* 5379 : VGETEXPPD zmmreg|mask|z,zmmrm512|b64|sae [rm:fv:    evex.512.66.0f38.w1 42 /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 301 001(42) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VGETEXPPD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+33660, 545},
    /* 5380 : VGETEXPPS xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.66.0f38.w0 42 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 301 001(42) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGETEXPPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+33669, 544},
    /* 5381 : VGETEXPPS ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.66.0f38.w0 42 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 301 001(42) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGETEXPPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+33678, 544},
    /* 5382 : VGETEXPPS zmmreg|mask|z,zmmrm512|b32|sae [rm:fv:    evex.512.66.0f38.w0 42 /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 301 001(42) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VGETEXPPS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+33687, 545},
    /* 5383 : VGETEXPSD xmmreg|mask|z,xmmreg,xmmrm64|sae [rvm:t1s:  evex.nds.lig.66.0f38.w1 43 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(43) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VGETEXPSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+33696, 546},
    /* 5384 : VGETEXPSS xmmreg|mask|z,xmmreg,xmmrm32|sae [rvm:t1s:  evex.nds.lig.66.0f38.w0 43 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(43) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VGETEXPSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+33705, 546},
    /* 5385 : VGETMANTPD xmmreg|mask|z,xmmrm128|b64,imm8 [rmi:fv:   evex.128.66.0f3a.w1 26 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 fd 08) 301 001(26) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGETMANTPD, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+19113, 544},
    /* 5386 : VGETMANTPD ymmreg|mask|z,ymmrm256|b64,imm8 [rmi:fv:   evex.256.66.0f3a.w1 26 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 fd 28) 301 001(26) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGETMANTPD, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+19123, 544},
    /* 5387 : VGETMANTPD zmmreg|mask|z,zmmrm512|b64|sae,imm8 [rmi:fv:   evex.512.66.0f3a.w1 26 /r ib      ] AVX512,FUTURE */
        /* 250(f3 fd 48) 301 001(26) 110 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VGETMANTPD, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+19133, 545},
    /* 5388 : VGETMANTPS xmmreg|mask|z,xmmrm128|b32,imm8 [rmi:fv:   evex.128.66.0f3a.w0 26 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 08) 301 001(26) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGETMANTPS, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+19143, 544},
    /* 5389 : VGETMANTPS ymmreg|mask|z,ymmrm256|b32,imm8 [rmi:fv:   evex.256.66.0f3a.w0 26 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 28) 301 001(26) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGETMANTPS, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+19153, 544},
    /* 5390 : VGETMANTPS zmmreg|mask|z,zmmrm512|b32|sae,imm8 [rmi:fv:   evex.512.66.0f3a.w0 26 /r ib      ] AVX512,FUTURE */
        /* 250(f3 7d 48) 301 001(26) 110 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VGETMANTPS, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+19163, 545},
    /* 5391 : VGETMANTSD xmmreg|mask|z,xmmreg,xmmrm64|sae,imm8 [rvmi:t1s: evex.nds.lig.66.0f3a.w1 27 /r ib  ] AVX512,FUTURE */
        /* 241(f3 fd 08) 306 001(27) 120 023 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VGETMANTSD, 4, {XMMREG,XMMREG,RM_XMM|BITS64,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+19173, 546},
    /* 5392 : VGETMANTSS xmmreg|mask|z,xmmreg,xmmrm32|sae,imm8 [rvmi:t1s: evex.nds.lig.66.0f3a.w0 27 /r ib  ] AVX512,FUTURE */
        /* 241(f3 7d 08) 306 001(27) 120 023 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VGETMANTSS, 4, {XMMREG,XMMREG,RM_XMM|BITS32,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+19183, 546},
    /* 5393 : VINSERTF32X4 ymmreg|mask|z,ymmreg*,xmmrm128,imm8 [rvmi:t4:  evex.nds.256.66.0f3a.w0 18 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 312 001(18) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VINSERTF32X4, 4, {YMMREG,YMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19193, 544},
    /* 5394 : VINSERTF32X4 zmmreg|mask|z,zmmreg*,xmmrm128,imm8 [rvmi:t4:  evex.nds.512.66.0f3a.w0 18 /r ib  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 312 001(18) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VINSERTF32X4, 4, {ZMMREG,ZMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19213, 545},
    /* 5395 : VINSERTF32X8 zmmreg|mask|z,zmmreg*,ymmrm256,imm8 [rvmi:t8:  evex.nds.512.66.0f3a.w0 1a /r ib  ] AVX512DQ,FUTURE */
        /* 241(f3 7d 48) 313 001(1a) 120 023 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VINSERTF32X8, 4, {ZMMREG,ZMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19233, 548},
    /* 5396 : VINSERTF64X2 ymmreg|mask|z,ymmreg*,xmmrm128,imm8 [rvmi:t2:  evex.nds.256.66.0f3a.w1 18 /r ib  ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f3 fd 28) 311 001(18) 120 023 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VINSERTF64X2, 4, {YMMREG,YMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19253, 547},
    /* 5397 : VINSERTF64X2 zmmreg|mask|z,zmmreg*,xmmrm128,imm8 [rvmi:t2:  evex.nds.512.66.0f3a.w1 18 /r ib  ] AVX512DQ,FUTURE */
        /* 241(f3 fd 48) 311 001(18) 120 023 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VINSERTF64X2, 4, {ZMMREG,ZMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19273, 548},
    /* 5398 : VINSERTF64X4 zmmreg|mask|z,zmmreg*,ymmrm256,imm8 [rvmi:t4:  evex.nds.512.66.0f3a.w1 1a /r ib  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 312 001(1a) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VINSERTF64X4, 4, {ZMMREG,ZMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19293, 545},
    /* 5399 : VINSERTI32X4 ymmreg|mask|z,ymmreg*,xmmrm128,imm8 [rvmi:t4:  evex.nds.256.66.0f3a.w0 38 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 312 001(38) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VINSERTI32X4, 4, {YMMREG,YMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19313, 544},
    /* 5400 : VINSERTI32X4 zmmreg|mask|z,zmmreg*,xmmrm128,imm8 [rvmi:t4:  evex.nds.512.66.0f3a.w0 38 /r ib  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 312 001(38) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VINSERTI32X4, 4, {ZMMREG,ZMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19333, 545},
    /* 5401 : VINSERTI32X8 zmmreg|mask|z,zmmreg*,ymmrm256,imm8 [rvmi:t8:  evex.nds.512.66.0f3a.w0 3a /r ib  ] AVX512DQ,FUTURE */
        /* 241(f3 7d 48) 313 001(3a) 120 023 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VINSERTI32X8, 4, {ZMMREG,ZMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19353, 548},
    /* 5402 : VINSERTI64X2 ymmreg|mask|z,ymmreg*,xmmrm128,imm8 [rvmi:t2:  evex.nds.256.66.0f3a.w1 38 /r ib  ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f3 fd 28) 311 001(38) 120 023 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VINSERTI64X2, 4, {YMMREG,YMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19373, 547},
    /* 5403 : VINSERTI64X2 zmmreg|mask|z,zmmreg*,xmmrm128,imm8 [rvmi:t2:  evex.nds.512.66.0f3a.w1 38 /r ib  ] AVX512DQ,FUTURE */
        /* 241(f3 fd 48) 311 001(38) 120 023 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VINSERTI64X2, 4, {ZMMREG,ZMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19393, 548},
    /* 5404 : VINSERTI64X4 zmmreg|mask|z,zmmreg*,ymmrm256,imm8 [rvmi:t4:  evex.nds.512.66.0f3a.w1 3a /r ib  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 312 001(3a) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VINSERTI64X4, 4, {ZMMREG,ZMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19413, 545},
    /* 5405 : VINSERTPS xmmreg,xmmreg*,xmmrm32,imm8 [rvmi:t1s: evex.nds.128.66.0f3a.w0 21 /r ib  ] AVX512,FUTURE */
        /* 241(f3 7d 08) 306 001(21) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VINSERTPS, 4, {XMMREG,XMMREG,RM_XMM|BITS32,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+19433, 545},
    /* 5406 : VMAXPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 5f /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(5f) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMAXPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33714, 544},
    /* 5407 : VMAXPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 5f /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(5f) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMAXPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33732, 544},
    /* 5408 : VMAXPD zmmreg|mask|z,zmmreg*,zmmrm512|b64|sae [rvm:fv:   evex.nds.512.66.0f.w1 5f /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(5f) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VMAXPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|SAE,0,0}, nasm_bytecodes+33750, 545},
    /* 5409 : VMAXPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.0f.w0 5f /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(5f) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMAXPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33768, 544},
    /* 5410 : VMAXPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.0f.w0 5f /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(5f) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMAXPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33786, 544},
    /* 5411 : VMAXPS zmmreg|mask|z,zmmreg*,zmmrm512|b32|sae [rvm:fv:   evex.nds.512.0f.w0 5f /r          ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(5f) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VMAXPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|SAE,0,0}, nasm_bytecodes+33804, 545},
    /* 5412 : VMAXSD xmmreg|mask|z,xmmreg*,xmmrm64|sae [rvm:t1s:  evex.nds.lig.f2.0f.w1 5f /r       ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(5f) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMAXSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+33822, 546},
    /* 5413 : VMAXSS xmmreg|mask|z,xmmreg*,xmmrm32|sae [rvm:t1s:  evex.nds.lig.f3.0f.w0 5f /r       ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(5f) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMAXSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+33840, 546},
    /* 5414 : VMINPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 5d /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(5d) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMINPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33858, 544},
    /* 5415 : VMINPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 5d /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(5d) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMINPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33876, 544},
    /* 5416 : VMINPD zmmreg|mask|z,zmmreg*,zmmrm512|b64|sae [rvm:fv:   evex.nds.512.66.0f.w1 5d /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(5d) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VMINPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|SAE,0,0}, nasm_bytecodes+33894, 545},
    /* 5417 : VMINPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.0f.w0 5d /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(5d) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMINPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33912, 544},
    /* 5418 : VMINPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.0f.w0 5d /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(5d) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMINPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33930, 544},
    /* 5419 : VMINPS zmmreg|mask|z,zmmreg*,zmmrm512|b32|sae [rvm:fv:   evex.nds.512.0f.w0 5d /r          ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(5d) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VMINPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|SAE,0,0}, nasm_bytecodes+33948, 545},
    /* 5420 : VMINSD xmmreg|mask|z,xmmreg*,xmmrm64|sae [rvm:t1s:  evex.nds.lig.f2.0f.w1 5d /r       ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(5d) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMINSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+33966, 546},
    /* 5421 : VMINSS xmmreg|mask|z,xmmreg*,xmmrm32|sae [rvm:t1s:  evex.nds.lig.f3.0f.w0 5d /r       ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(5d) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMINSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+33984, 546},
    /* 5422 : VMOVAPD xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.66.0f.w1 28 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 08) 303 001(28) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVAPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34002, 544},
    /* 5423 : VMOVAPD ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.66.0f.w1 28 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 28) 303 001(28) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVAPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34011, 544},
    /* 5424 : VMOVAPD zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.66.0f.w1 28 /r           ] AVX512,FUTURE */
        /* 250(f1 fd 48) 303 001(28) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVAPD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34020, 545},
    /* 5425 : VMOVAPD xmmreg|mask|z,xmmreg [mr:       evex.128.66.0f.w1 29 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 08) 300 001(29) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVAPD, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34029, 544},
    /* 5426 : VMOVAPD ymmreg|mask|z,ymmreg [mr:       evex.256.66.0f.w1 29 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 28) 300 001(29) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVAPD, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34038, 544},
    /* 5427 : VMOVAPD zmmreg|mask|z,zmmreg [mr:       evex.512.66.0f.w1 29 /r           ] AVX512,FUTURE */
        /* 250(f1 fd 48) 300 001(29) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVAPD, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34047, 545},
    /* 5428 : VMOVAPD mem128|mask,xmmreg [mr:fvm:   evex.128.66.0f.w1 29 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 08) 303 001(29) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVAPD, 2, {MEMORY|BITS128,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+34056, 544},
    /* 5429 : VMOVAPD mem256|mask,ymmreg [mr:fvm:   evex.256.66.0f.w1 29 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 28) 303 001(29) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVAPD, 2, {MEMORY|BITS256,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+34065, 544},
    /* 5430 : VMOVAPD mem512|mask,zmmreg [mr:fvm:   evex.512.66.0f.w1 29 /r           ] AVX512,FUTURE */
        /* 250(f1 fd 48) 303 001(29) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVAPD, 2, {MEMORY|BITS512,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+34074, 545},
    /* 5431 : VMOVAPS xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.0f.w0 28 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 08) 303 001(28) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVAPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34083, 544},
    /* 5432 : VMOVAPS ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.0f.w0 28 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 28) 303 001(28) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVAPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34092, 544},
    /* 5433 : VMOVAPS zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.0f.w0 28 /r              ] AVX512,FUTURE */
        /* 250(f1 7c 48) 303 001(28) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVAPS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34101, 545},
    /* 5434 : VMOVAPS xmmreg|mask|z,xmmreg [mr:       evex.128.0f.w0 29 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 08) 300 001(29) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVAPS, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34110, 544},
    /* 5435 : VMOVAPS ymmreg|mask|z,ymmreg [mr:       evex.256.0f.w0 29 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 28) 300 001(29) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVAPS, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34119, 544},
    /* 5436 : VMOVAPS zmmreg|mask|z,zmmreg [mr:       evex.512.0f.w0 29 /r              ] AVX512,FUTURE */
        /* 250(f1 7c 48) 300 001(29) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVAPS, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34128, 545},
    /* 5437 : VMOVAPS mem128|mask,xmmreg [mr:fvm:   evex.128.0f.w0 29 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 08) 303 001(29) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVAPS, 2, {MEMORY|BITS128,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+34137, 544},
    /* 5438 : VMOVAPS mem256|mask,ymmreg [mr:fvm:   evex.256.0f.w0 29 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 28) 303 001(29) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVAPS, 2, {MEMORY|BITS256,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+34146, 544},
    /* 5439 : VMOVAPS mem512|mask,zmmreg [mr:fvm:   evex.512.0f.w0 29 /r              ] AVX512,FUTURE */
        /* 250(f1 7c 48) 303 001(29) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVAPS, 2, {MEMORY|BITS512,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+34155, 545},
    /* 5440 : VMOVD xmmreg,rm32 [rm:t1s:   evex.128.66.0f.w0 6e /r           ] AVX512,FUTURE */
        /* 250(f1 7d 08) 306 001(6e) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVD, 2, {XMMREG,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+34164, 545},
    /* 5441 : VMOVD rm32,xmmreg [mr:t1s:   evex.128.66.0f.w0 7e /r           ] AVX512,FUTURE */
        /* 250(f1 7d 08) 306 001(7e) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVD, 2, {RM_GPR|BITS32,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34173, 545},
    /* 5442 : VMOVDDUP xmmreg|mask|z,xmmrm64 [rm:dup:   evex.128.f2.0f.w1 12 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 ff 08) 320 001(12) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDDUP, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34182, 544},
    /* 5443 : VMOVDDUP ymmreg|mask|z,ymmrm256 [rm:dup:   evex.256.f2.0f.w1 12 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 ff 28) 320 001(12) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDDUP, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34191, 544},
    /* 5444 : VMOVDDUP zmmreg|mask|z,zmmrm512 [rm:dup:   evex.512.f2.0f.w1 12 /r           ] AVX512,FUTURE */
        /* 250(f1 ff 48) 320 001(12) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVDDUP, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34200, 545},
    /* 5445 : VMOVDQA32 xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.66.0f.w0 6f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7d 08) 303 001(6f) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQA32, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34209, 544},
    /* 5446 : VMOVDQA32 ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.66.0f.w0 6f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7d 28) 303 001(6f) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQA32, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34218, 544},
    /* 5447 : VMOVDQA32 zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.66.0f.w0 6f /r           ] AVX512,FUTURE */
        /* 250(f1 7d 48) 303 001(6f) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVDQA32, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34227, 545},
    /* 5448 : VMOVDQA32 xmmrm128|mask|z,xmmreg [mr:fvm:   evex.128.66.0f.w0 7f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7d 08) 303 001(7f) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQA32, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34236, 544},
    /* 5449 : VMOVDQA32 ymmrm256|mask|z,ymmreg [mr:fvm:   evex.256.66.0f.w0 7f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7d 28) 303 001(7f) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQA32, 2, {RM_YMM|BITS256,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34245, 544},
    /* 5450 : VMOVDQA32 zmmrm512|mask|z,zmmreg [mr:fvm:   evex.512.66.0f.w0 7f /r           ] AVX512,FUTURE */
        /* 250(f1 7d 48) 303 001(7f) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVDQA32, 2, {RM_ZMM|BITS512,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34254, 545},
    /* 5451 : VMOVDQA64 xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.66.0f.w1 6f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 08) 303 001(6f) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQA64, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34263, 544},
    /* 5452 : VMOVDQA64 ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.66.0f.w1 6f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 28) 303 001(6f) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQA64, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34272, 544},
    /* 5453 : VMOVDQA64 zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.66.0f.w1 6f /r           ] AVX512,FUTURE */
        /* 250(f1 fd 48) 303 001(6f) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVDQA64, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34281, 545},
    /* 5454 : VMOVDQA64 xmmrm128|mask|z,xmmreg [mr:fvm:   evex.128.66.0f.w1 7f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 08) 303 001(7f) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQA64, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34290, 544},
    /* 5455 : VMOVDQA64 ymmrm256|mask|z,ymmreg [mr:fvm:   evex.256.66.0f.w1 7f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 28) 303 001(7f) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQA64, 2, {RM_YMM|BITS256,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34299, 544},
    /* 5456 : VMOVDQA64 zmmrm512|mask|z,zmmreg [mr:fvm:   evex.512.66.0f.w1 7f /r           ] AVX512,FUTURE */
        /* 250(f1 fd 48) 303 001(7f) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVDQA64, 2, {RM_ZMM|BITS512,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34308, 545},
    /* 5457 : VMOVDQU16 xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.f2.0f.w1 6f /r           ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f1 ff 08) 303 001(6f) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VMOVDQU16, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34317, 550},
    /* 5458 : VMOVDQU16 ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.f2.0f.w1 6f /r           ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f1 ff 28) 303 001(6f) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VMOVDQU16, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34326, 550},
    /* 5459 : VMOVDQU16 zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.f2.0f.w1 6f /r           ] AVX512BW,FUTURE */
        /* 250(f1 ff 48) 303 001(6f) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VMOVDQU16, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34335, 551},
    /* 5460 : VMOVDQU16 xmmrm128|mask|z,xmmreg [mr:fvm:   evex.128.f2.0f.w1 7f /r           ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f1 ff 08) 303 001(7f) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VMOVDQU16, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34344, 550},
    /* 5461 : VMOVDQU16 ymmrm256|mask|z,ymmreg [mr:fvm:   evex.256.f2.0f.w1 7f /r           ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f1 ff 28) 303 001(7f) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VMOVDQU16, 2, {RM_YMM|BITS256,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34353, 550},
    /* 5462 : VMOVDQU16 zmmrm512|mask|z,zmmreg [mr:fvm:   evex.512.f2.0f.w1 7f /r           ] AVX512BW,FUTURE */
        /* 250(f1 ff 48) 303 001(7f) 101 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VMOVDQU16, 2, {RM_ZMM|BITS512,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34362, 551},
    /* 5463 : VMOVDQU32 xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.f3.0f.w0 6f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 08) 303 001(6f) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQU32, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34371, 544},
    /* 5464 : VMOVDQU32 ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.f3.0f.w0 6f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 28) 303 001(6f) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQU32, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34380, 544},
    /* 5465 : VMOVDQU32 zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.f3.0f.w0 6f /r           ] AVX512,FUTURE */
        /* 250(f1 7e 48) 303 001(6f) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVDQU32, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34389, 545},
    /* 5466 : VMOVDQU32 xmmrm128|mask|z,xmmreg [mr:fvm:   evex.128.f3.0f.w0 7f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 08) 303 001(7f) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQU32, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34398, 544},
    /* 5467 : VMOVDQU32 ymmrm256|mask|z,ymmreg [mr:fvm:   evex.256.f3.0f.w0 7f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 28) 303 001(7f) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQU32, 2, {RM_YMM|BITS256,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34407, 544},
    /* 5468 : VMOVDQU32 zmmrm512|mask|z,zmmreg [mr:fvm:   evex.512.f3.0f.w0 7f /r           ] AVX512,FUTURE */
        /* 250(f1 7e 48) 303 001(7f) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVDQU32, 2, {RM_ZMM|BITS512,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34416, 545},
    /* 5469 : VMOVDQU64 xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.f3.0f.w1 6f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fe 08) 303 001(6f) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQU64, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34425, 544},
    /* 5470 : VMOVDQU64 ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.f3.0f.w1 6f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fe 28) 303 001(6f) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQU64, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34434, 544},
    /* 5471 : VMOVDQU64 zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.f3.0f.w1 6f /r           ] AVX512,FUTURE */
        /* 250(f1 fe 48) 303 001(6f) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVDQU64, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34443, 545},
    /* 5472 : VMOVDQU64 xmmrm128|mask|z,xmmreg [mr:fvm:   evex.128.f3.0f.w1 7f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fe 08) 303 001(7f) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQU64, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34452, 544},
    /* 5473 : VMOVDQU64 ymmrm256|mask|z,ymmreg [mr:fvm:   evex.256.f3.0f.w1 7f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fe 28) 303 001(7f) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQU64, 2, {RM_YMM|BITS256,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34461, 544},
    /* 5474 : VMOVDQU64 zmmrm512|mask|z,zmmreg [mr:fvm:   evex.512.f3.0f.w1 7f /r           ] AVX512,FUTURE */
        /* 250(f1 fe 48) 303 001(7f) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVDQU64, 2, {RM_ZMM|BITS512,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34470, 545},
    /* 5475 : VMOVDQU8 xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.f2.0f.w0 6f /r           ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f1 7f 08) 303 001(6f) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VMOVDQU8, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34479, 550},
    /* 5476 : VMOVDQU8 ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.f2.0f.w0 6f /r           ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f1 7f 28) 303 001(6f) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VMOVDQU8, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34488, 550},
    /* 5477 : VMOVDQU8 zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.f2.0f.w0 6f /r           ] AVX512BW,FUTURE */
        /* 250(f1 7f 48) 303 001(6f) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VMOVDQU8, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34497, 551},
    /* 5478 : VMOVDQU8 xmmrm128|mask|z,xmmreg [mr:fvm:   evex.128.f2.0f.w0 7f /r           ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f1 7f 08) 303 001(7f) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VMOVDQU8, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34506, 550},
    /* 5479 : VMOVDQU8 ymmrm256|mask|z,ymmreg [mr:fvm:   evex.256.f2.0f.w0 7f /r           ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f1 7f 28) 303 001(7f) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VMOVDQU8, 2, {RM_YMM|BITS256,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34515, 550},
    /* 5480 : VMOVDQU8 zmmrm512|mask|z,zmmreg [mr:fvm:   evex.512.f2.0f.w0 7f /r           ] AVX512BW,FUTURE */
        /* 250(f1 7f 48) 303 001(7f) 101 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VMOVDQU8, 2, {RM_ZMM|BITS512,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34524, 551},
    /* 5481 : VMOVHLPS xmmreg,xmmreg*,xmmreg [rvm:      evex.nds.128.0f.w0 12 /r          ] AVX512,FUTURE */
        /* 241(f1 7c 08) 300 001(12) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVHLPS, 3, {XMMREG,XMMREG,XMMREG,0,0}, NO_DECORATOR, nasm_bytecodes+34533, 545},
    /* 5482 : VMOVHPD xmmreg,xmmreg*,mem64 [rvm:t1s:  evex.nds.128.66.0f.w1 16 /r       ] AVX512,FUTURE */
        /* 241(f1 fd 08) 306 001(16) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVHPD, 3, {XMMREG,XMMREG,MEMORY|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+34551, 545},
    /* 5483 : VMOVHPD mem64,xmmreg [mr:t1s:   evex.128.66.0f.w1 17 /r           ] AVX512,FUTURE */
        /* 250(f1 fd 08) 306 001(17) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVHPD, 2, {MEMORY|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34569, 545},
    /* 5484 : VMOVHPS xmmreg,xmmreg*,mem64 [rvm:t2:   evex.nds.128.0f.w0 16 /r          ] AVX512,FUTURE */
        /* 241(f1 7c 08) 311 001(16) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVHPS, 3, {XMMREG,XMMREG,MEMORY|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+34578, 545},
    /* 5485 : VMOVHPS mem64,xmmreg [mr:t2:    evex.128.0f.w0 17 /r              ] AVX512,FUTURE */
        /* 250(f1 7c 08) 311 001(17) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVHPS, 2, {MEMORY|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34596, 545},
    /* 5486 : VMOVLHPS xmmreg,xmmreg*,xmmreg [rvm:      evex.nds.128.0f.w0 16 /r          ] AVX512,FUTURE */
        /* 241(f1 7c 08) 300 001(16) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVLHPS, 3, {XMMREG,XMMREG,XMMREG,0,0}, NO_DECORATOR, nasm_bytecodes+34605, 545},
    /* 5487 : VMOVLPD xmmreg,xmmreg*,mem64 [rvm:t1s:  evex.nds.128.66.0f.w1 12 /r       ] AVX512,FUTURE */
        /* 241(f1 fd 08) 306 001(12) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVLPD, 3, {XMMREG,XMMREG,MEMORY|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+34623, 545},
    /* 5488 : VMOVLPD mem64,xmmreg [mr:t1s:   evex.128.66.0f.w1 13 /r           ] AVX512,FUTURE */
        /* 250(f1 fd 08) 306 001(13) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVLPD, 2, {MEMORY|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34641, 545},
    /* 5489 : VMOVLPS xmmreg,xmmreg*,mem64 [rvm:t2:   evex.nds.128.0f.w0 12 /r          ] AVX512,FUTURE */
        /* 241(f1 7c 08) 311 001(12) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVLPS, 3, {XMMREG,XMMREG,MEMORY|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+34650, 545},
    /* 5490 : VMOVLPS mem64,xmmreg [mr:t2:    evex.128.0f.w0 13 /r              ] AVX512,FUTURE */
        /* 250(f1 7c 08) 311 001(13) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVLPS, 2, {MEMORY|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34668, 545},
    /* 5491 : VMOVNTDQ mem128,xmmreg [mr:fvm:   evex.128.66.0f.w0 e7 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7d 08) 303 001(e7) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVNTDQ, 2, {MEMORY|BITS128,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34677, 544},
    /* 5492 : VMOVNTDQ mem256,ymmreg [mr:fvm:   evex.256.66.0f.w0 e7 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7d 28) 303 001(e7) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVNTDQ, 2, {MEMORY|BITS256,YMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34686, 544},
    /* 5493 : VMOVNTDQ mem512,zmmreg [mr:fvm:   evex.512.66.0f.w0 e7 /r           ] AVX512,FUTURE */
        /* 250(f1 7d 48) 303 001(e7) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVNTDQ, 2, {MEMORY|BITS512,ZMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34695, 545},
    /* 5494 : VMOVNTDQA xmmreg,mem128 [rm:fvm:   evex.128.66.0f38.w0 2a /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 303 001(2a) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVNTDQA, 2, {XMMREG,MEMORY|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+34704, 544},
    /* 5495 : VMOVNTDQA ymmreg,mem256 [rm:fvm:   evex.256.66.0f38.w0 2a /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 303 001(2a) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVNTDQA, 2, {YMMREG,MEMORY|BITS256,0,0,0}, NO_DECORATOR, nasm_bytecodes+34713, 544},
    /* 5496 : VMOVNTDQA zmmreg,mem512 [rm:fvm:   evex.512.66.0f38.w0 2a /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 303 001(2a) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVNTDQA, 2, {ZMMREG,MEMORY|BITS512,0,0,0}, NO_DECORATOR, nasm_bytecodes+34722, 545},
    /* 5497 : VMOVNTPD mem128,xmmreg [mr:fvm:   evex.128.66.0f.w1 2b /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 08) 303 001(2b) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVNTPD, 2, {MEMORY|BITS128,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34731, 544},
    /* 5498 : VMOVNTPD mem256,ymmreg [mr:fvm:   evex.256.66.0f.w1 2b /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 28) 303 001(2b) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVNTPD, 2, {MEMORY|BITS256,YMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34740, 544},
    /* 5499 : VMOVNTPD mem512,zmmreg [mr:fvm:   evex.512.66.0f.w1 2b /r           ] AVX512,FUTURE */
        /* 250(f1 fd 48) 303 001(2b) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVNTPD, 2, {MEMORY|BITS512,ZMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34749, 545},
    /* 5500 : VMOVNTPS mem128,xmmreg [mr:fvm:   evex.128.0f.w0 2b /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 08) 303 001(2b) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVNTPS, 2, {MEMORY|BITS128,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34758, 544},
    /* 5501 : VMOVNTPS mem256,ymmreg [mr:fvm:   evex.256.0f.w0 2b /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 28) 303 001(2b) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVNTPS, 2, {MEMORY|BITS256,YMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34767, 544},
    /* 5502 : VMOVNTPS mem512,zmmreg [mr:fvm:   evex.512.0f.w0 2b /r              ] AVX512,FUTURE */
        /* 250(f1 7c 48) 303 001(2b) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVNTPS, 2, {MEMORY|BITS512,ZMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34776, 545},
    /* 5503 : VMOVQ xmmreg,rm64 [rm:t1s:   evex.128.66.0f.w1 6e /r           ] AVX512,FUTURE */
        /* 250(f1 fd 08) 306 001(6e) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVQ, 2, {XMMREG,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+34785, 545},
    /* 5504 : VMOVQ rm64,xmmreg [mr:t1s:   evex.128.66.0f.w1 7e /r           ] AVX512,FUTURE */
        /* 250(f1 fd 08) 306 001(7e) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVQ, 2, {RM_GPR|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34794, 545},
    /* 5505 : VMOVQ xmmreg,xmmrm64 [rm:t1s:   evex.128.f3.0f.w1 7e /r           ] AVX512,FUTURE */
        /* 250(f1 fe 08) 306 001(7e) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVQ, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+34803, 545},
    /* 5506 : VMOVQ xmmrm64,xmmreg [mr:t1s:   evex.128.66.0f.w1 d6 /r           ] AVX512,FUTURE */
        /* 250(f1 fd 08) 306 001(d6) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVQ, 2, {RM_XMM|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34812, 545},
    /* 5507 : VMOVSD xmmreg|mask|z,mem64 [rm:t1s:   evex.lig.f2.0f.w1 10 /r           ] AVX512,FUTURE */
        /* 250(f1 ff 08) 306 001(10) 110 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMOVSD, 2, {XMMREG,MEMORY|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34821, 546},
    /* 5508 : VMOVSD mem64|mask,xmmreg [mr:t1s:   evex.lig.f2.0f.w1 11 /r           ] AVX512,FUTURE */
        /* 250(f1 ff 08) 306 001(11) 101 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMOVSD, 2, {MEMORY|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+34830, 546},
    /* 5509 : VMOVSD xmmreg|mask|z,xmmreg*,xmmreg [rvm:      evex.nds.lig.f2.0f.w1 10 /r       ] AVX512,FUTURE */
        /* 241(f1 ff 08) 300 001(10) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMOVSD, 3, {XMMREG,XMMREG,XMMREG,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34839, 546},
    /* 5510 : VMOVSD xmmreg|mask|z,xmmreg*,xmmreg [mvr:      evex.nds.lig.f2.0f.w1 11 /r       ] AVX512,FUTURE */
        /* 241(f1 ff 08) 300 001(11) 102 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMOVSD, 3, {XMMREG,XMMREG,XMMREG,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34857, 546},
    /* 5511 : VMOVSHDUP xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.f3.0f.w0 16 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 08) 303 001(16) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVSHDUP, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34875, 544},
    /* 5512 : VMOVSHDUP ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.f3.0f.w0 16 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 28) 303 001(16) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVSHDUP, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34884, 544},
    /* 5513 : VMOVSHDUP zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.f3.0f.w0 16 /r           ] AVX512,FUTURE */
        /* 250(f1 7e 48) 303 001(16) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVSHDUP, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34893, 545},
    /* 5514 : VMOVSLDUP xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.f3.0f.w0 12 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 08) 303 001(12) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVSLDUP, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34902, 544},
    /* 5515 : VMOVSLDUP ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.f3.0f.w0 12 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 28) 303 001(12) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVSLDUP, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34911, 544},
    /* 5516 : VMOVSLDUP zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.f3.0f.w0 12 /r           ] AVX512,FUTURE */
        /* 250(f1 7e 48) 303 001(12) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVSLDUP, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34920, 545},
    /* 5517 : VMOVSS xmmreg|mask|z,mem32 [rm:t1s:   evex.lig.f3.0f.w0 10 /r           ] AVX512,FUTURE */
        /* 250(f1 7e 08) 306 001(10) 110 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMOVSS, 2, {XMMREG,MEMORY|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34929, 546},
    /* 5518 : VMOVSS mem32|mask,xmmreg [mr:t1s:   evex.lig.f3.0f.w0 11 /r           ] AVX512,FUTURE */
        /* 250(f1 7e 08) 306 001(11) 101 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMOVSS, 2, {MEMORY|BITS32,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+34938, 546},
    /* 5519 : VMOVSS xmmreg|mask|z,xmmreg*,xmmreg [rvm:      evex.nds.lig.f3.0f.w0 10 /r       ] AVX512,FUTURE */
        /* 241(f1 7e 08) 300 001(10) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMOVSS, 3, {XMMREG,XMMREG,XMMREG,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34947, 546},
    /* 5520 : VMOVSS xmmreg|mask|z,xmmreg*,xmmreg [mvr:      evex.nds.lig.f3.0f.w0 11 /r       ] AVX512,FUTURE */
        /* 241(f1 7e 08) 300 001(11) 102 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMOVSS, 3, {XMMREG,XMMREG,XMMREG,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34965, 546},
    /* 5521 : VMOVUPD xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.66.0f.w1 10 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 08) 303 001(10) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVUPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34983, 544},
    /* 5522 : VMOVUPD ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.66.0f.w1 10 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 28) 303 001(10) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVUPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34992, 544},
    /* 5523 : VMOVUPD zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.66.0f.w1 10 /r           ] AVX512,FUTURE */
        /* 250(f1 fd 48) 303 001(10) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVUPD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35001, 545},
    /* 5524 : VMOVUPD xmmreg|mask|z,xmmreg [mr:       evex.128.66.0f.w1 11 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 08) 300 001(11) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVUPD, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35010, 544},
    /* 5525 : VMOVUPD ymmreg|mask|z,ymmreg [mr:       evex.256.66.0f.w1 11 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 28) 300 001(11) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVUPD, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35019, 544},
    /* 5526 : VMOVUPD zmmreg|mask|z,zmmreg [mr:       evex.512.66.0f.w1 11 /r           ] AVX512,FUTURE */
        /* 250(f1 fd 48) 300 001(11) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVUPD, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35028, 545},
    /* 5527 : VMOVUPD mem128|mask,xmmreg [mr:fvm:   evex.128.66.0f.w1 11 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 08) 303 001(11) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVUPD, 2, {MEMORY|BITS128,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+35037, 544},
    /* 5528 : VMOVUPD mem256|mask,ymmreg [mr:fvm:   evex.256.66.0f.w1 11 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 28) 303 001(11) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVUPD, 2, {MEMORY|BITS256,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+35046, 544},
    /* 5529 : VMOVUPD mem512|mask,zmmreg [mr:fvm:   evex.512.66.0f.w1 11 /r           ] AVX512,FUTURE */
        /* 250(f1 fd 48) 303 001(11) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVUPD, 2, {MEMORY|BITS512,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+35055, 545},
    /* 5530 : VMOVUPS xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.0f.w0 10 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 08) 303 001(10) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVUPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35064, 544},
    /* 5531 : VMOVUPS ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.0f.w0 10 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 28) 303 001(10) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVUPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35073, 544},
    /* 5532 : VMOVUPS zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.0f.w0 10 /r              ] AVX512,FUTURE */
        /* 250(f1 7c 48) 303 001(10) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVUPS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35082, 545},
    /* 5533 : VMOVUPS xmmreg|mask|z,xmmreg [mr:       evex.128.0f.w0 11 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 08) 300 001(11) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVUPS, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35091, 544},
    /* 5534 : VMOVUPS ymmreg|mask|z,ymmreg [mr:       evex.256.0f.w0 11 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 28) 300 001(11) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVUPS, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35100, 544},
    /* 5535 : VMOVUPS zmmreg|mask|z,zmmreg [mr:       evex.512.0f.w0 11 /r              ] AVX512,FUTURE */
        /* 250(f1 7c 48) 300 001(11) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVUPS, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35109, 545},
    /* 5536 : VMOVUPS mem128|mask,xmmreg [mr:fvm:   evex.128.0f.w0 11 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 08) 303 001(11) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVUPS, 2, {MEMORY|BITS128,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+35118, 544},
    /* 5537 : VMOVUPS mem256|mask,ymmreg [mr:fvm:   evex.256.0f.w0 11 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 28) 303 001(11) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVUPS, 2, {MEMORY|BITS256,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+35127, 544},
    /* 5538 : VMOVUPS mem512|mask,zmmreg [mr:fvm:   evex.512.0f.w0 11 /r              ] AVX512,FUTURE */
        /* 250(f1 7c 48) 303 001(11) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVUPS, 2, {MEMORY|BITS512,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+35136, 545},
    /* 5539 : VMULPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 59 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(59) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMULPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+35145, 544},
    /* 5540 : VMULPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 59 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(59) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMULPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+35163, 544},
    /* 5541 : VMULPD zmmreg|mask|z,zmmreg*,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f.w1 59 /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(59) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VMULPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+35181, 545},
    /* 5542 : VMULPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.0f.w0 59 /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(59) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMULPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35199, 544},
    /* 5543 : VMULPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.0f.w0 59 /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(59) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMULPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35217, 544},
    /* 5544 : VMULPS zmmreg|mask|z,zmmreg*,zmmrm512|b32|er [rvm:fv:   evex.nds.512.0f.w0 59 /r          ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(59) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VMULPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+35235, 545},
    /* 5545 : VMULSD xmmreg|mask|z,xmmreg*,xmmrm64|er [rvm:t1s:  evex.nds.lig.f2.0f.w1 59 /r       ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(59) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMULSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+35253, 546},
    /* 5546 : VMULSS xmmreg|mask|z,xmmreg*,xmmrm32|er [rvm:t1s:  evex.nds.lig.f3.0f.w0 59 /r       ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(59) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMULSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+35271, 546},
    /* 5547 : VORPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 56 /r       ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 fd 08) 301 001(56) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VORPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+35289, 547},
    /* 5548 : VORPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 56 /r       ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 fd 28) 301 001(56) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VORPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+35307, 547},
    /* 5549 : VORPD zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 56 /r       ] AVX512DQ,FUTURE */
        /* 241(f1 fd 48) 301 001(56) 120 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VORPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+35325, 548},
    /* 5550 : VORPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.0f.w0 56 /r          ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 7c 08) 301 001(56) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VORPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35343, 547},
    /* 5551 : VORPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.0f.w0 56 /r          ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 7c 28) 301 001(56) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VORPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35361, 547},
    /* 5552 : VORPS zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.0f.w0 56 /r          ] AVX512DQ,FUTURE */
        /* 241(f1 7c 48) 301 001(56) 120 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VORPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35379, 548},
    /* 5553 : VPABSB xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.66.0f38.wig 1c /r        ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 08) 303 001(1c) 110 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPABSB, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35397, 556},
    /* 5554 : VPABSB ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.66.0f38.wig 1c /r        ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 28) 303 001(1c) 110 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPABSB, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35406, 556},
    /* 5555 : VPABSB zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.66.0f38.wig 1c /r        ] AVX512BW,FUTURE */
        /* 250(f2 7d 48) 303 001(1c) 110 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPABSB, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35415, 557},
    /* 5556 : VPABSD xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.66.0f38.w0 1e /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 301 001(1e) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPABSD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+35424, 544},
    /* 5557 : VPABSD ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.66.0f38.w0 1e /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 301 001(1e) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPABSD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+35433, 544},
    /* 5558 : VPABSD zmmreg|mask|z,zmmrm512|b32 [rm:fv:    evex.512.66.0f38.w0 1e /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 301 001(1e) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPABSD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+35442, 545},
    /* 5559 : VPABSQ xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.0f38.w1 1f /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 301 001(1f) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPABSQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+35451, 544},
    /* 5560 : VPABSQ ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.0f38.w1 1f /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 301 001(1f) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPABSQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+35460, 544},
    /* 5561 : VPABSQ zmmreg|mask|z,zmmrm512|b64 [rm:fv:    evex.512.66.0f38.w1 1f /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 301 001(1f) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPABSQ, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+35469, 545},
    /* 5562 : VPABSW xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.66.0f38.wig 1d /r        ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 08) 303 001(1d) 110 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPABSW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35478, 556},
    /* 5563 : VPABSW ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.66.0f38.wig 1d /r        ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 28) 303 001(1d) 110 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPABSW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35487, 556},
    /* 5564 : VPABSW zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.66.0f38.wig 1d /r        ] AVX512BW,FUTURE */
        /* 250(f2 7d 48) 303 001(1d) 110 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPABSW, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35496, 557},
    /* 5565 : VPACKSSDW xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f.w0 6b /r       ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 301 001(6b) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPACKSSDW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35505, 550},
    /* 5566 : VPACKSSDW ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f.w0 6b /r       ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 301 001(6b) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPACKSSDW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35523, 550},
    /* 5567 : VPACKSSDW zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f.w0 6b /r       ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 301 001(6b) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPACKSSDW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35541, 551},
    /* 5568 : VPACKSSWB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig 63 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(63) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPACKSSWB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35559, 556},
    /* 5569 : VPACKSSWB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig 63 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(63) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPACKSSWB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35577, 556},
    /* 5570 : VPACKSSWB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig 63 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(63) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPACKSSWB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35595, 557},
    /* 5571 : VPACKUSDW xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 2b /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 08) 301 001(2b) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPACKUSDW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35613, 550},
    /* 5572 : VPACKUSDW ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 2b /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 28) 301 001(2b) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPACKUSDW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35631, 550},
    /* 5573 : VPACKUSDW zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 2b /r     ] AVX512BW,FUTURE */
        /* 241(f2 7d 48) 301 001(2b) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPACKUSDW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35649, 551},
    /* 5574 : VPACKUSWB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig 67 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(67) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPACKUSWB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35667, 556},
    /* 5575 : VPACKUSWB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig 67 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(67) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPACKUSWB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35685, 556},
    /* 5576 : VPACKUSWB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig 67 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(67) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPACKUSWB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35703, 557},
    /* 5577 : VPADDB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig fc /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(fc) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPADDB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35721, 556},
    /* 5578 : VPADDB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig fc /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(fc) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPADDB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35739, 556},
    /* 5579 : VPADDB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig fc /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(fc) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPADDB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35757, 557},
    /* 5580 : VPADDD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f.w0 fe /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 08) 301 001(fe) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPADDD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35775, 544},
    /* 5581 : VPADDD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f.w0 fe /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 28) 301 001(fe) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPADDD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35793, 544},
    /* 5582 : VPADDD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f.w0 fe /r       ] AVX512,FUTURE */
        /* 241(f1 7d 48) 301 001(fe) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPADDD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35811, 545},
    /* 5583 : VPADDQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 d4 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(d4) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPADDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+35829, 544},
    /* 5584 : VPADDQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 d4 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(d4) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPADDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+35847, 544},
    /* 5585 : VPADDQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 d4 /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(d4) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPADDQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+35865, 545},
    /* 5586 : VPADDSB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig ec /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(ec) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPADDSB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35883, 556},
    /* 5587 : VPADDSB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig ec /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(ec) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPADDSB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35901, 556},
    /* 5588 : VPADDSB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig ec /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(ec) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPADDSB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35919, 557},
    /* 5589 : VPADDSW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig ed /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(ed) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPADDSW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35937, 556},
    /* 5590 : VPADDSW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig ed /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(ed) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPADDSW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35955, 556},
    /* 5591 : VPADDSW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig ed /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(ed) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPADDSW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35973, 557},
    /* 5592 : VPADDUSB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig dc /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(dc) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPADDUSB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35991, 556},
    /* 5593 : VPADDUSB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig dc /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(dc) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPADDUSB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36009, 556},
    /* 5594 : VPADDUSB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig dc /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(dc) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPADDUSB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36027, 557},
    /* 5595 : VPADDUSW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig dd /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(dd) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPADDUSW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36045, 556},
    /* 5596 : VPADDUSW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig dd /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(dd) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPADDUSW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36063, 556},
    /* 5597 : VPADDUSW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig dd /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(dd) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPADDUSW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36081, 557},
    /* 5598 : VPADDW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig fd /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(fd) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPADDW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36099, 556},
    /* 5599 : VPADDW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig fd /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(fd) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPADDW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36117, 556},
    /* 5600 : VPADDW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig fd /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(fd) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPADDW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36135, 557},
    /* 5601 : VPALIGNR xmmreg|mask|z,xmmreg*,xmmrm128,imm8 [rvmi:fvm: evex.nds.128.66.0f3a.wig 0f /r ib ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(0f) 120 023 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPALIGNR, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19453, 556},
    /* 5602 : VPALIGNR ymmreg|mask|z,ymmreg*,ymmrm256,imm8 [rvmi:fvm: evex.nds.256.66.0f3a.wig 0f /r ib ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(0f) 120 023 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPALIGNR, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19473, 556},
    /* 5603 : VPALIGNR zmmreg|mask|z,zmmreg*,zmmrm512,imm8 [rvmi:fvm: evex.nds.512.66.0f3a.wig 0f /r ib ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(0f) 120 023 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPALIGNR, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19493, 557},
    /* 5604 : VPANDD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f.w0 db /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 08) 301 001(db) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPANDD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+36153, 544},
    /* 5605 : VPANDD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f.w0 db /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 28) 301 001(db) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPANDD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+36171, 544},
    /* 5606 : VPANDD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f.w0 db /r       ] AVX512,FUTURE */
        /* 241(f1 7d 48) 301 001(db) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPANDD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+36189, 545},
    /* 5607 : VPANDND xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f.w0 df /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 08) 301 001(df) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPANDND, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+36207, 544},
    /* 5608 : VPANDND ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f.w0 df /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 28) 301 001(df) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPANDND, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+36225, 544},
    /* 5609 : VPANDND zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f.w0 df /r       ] AVX512,FUTURE */
        /* 241(f1 7d 48) 301 001(df) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPANDND, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+36243, 545},
    /* 5610 : VPANDNQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 df /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(df) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPANDNQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+36261, 544},
    /* 5611 : VPANDNQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 df /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(df) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPANDNQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+36279, 544},
    /* 5612 : VPANDNQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 df /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(df) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPANDNQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+36297, 545},
    /* 5613 : VPANDQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 db /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(db) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPANDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+36315, 544},
    /* 5614 : VPANDQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 db /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(db) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPANDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+36333, 544},
    /* 5615 : VPANDQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 db /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(db) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPANDQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+36351, 545},
    /* 5616 : VPAVGB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig e0 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(e0) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPAVGB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36369, 556},
    /* 5617 : VPAVGB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig e0 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(e0) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPAVGB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36387, 556},
    /* 5618 : VPAVGB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig e0 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(e0) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPAVGB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36405, 557},
    /* 5619 : VPAVGW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig e3 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(e3) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPAVGW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36423, 556},
    /* 5620 : VPAVGW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig e3 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(e3) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPAVGW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36441, 556},
    /* 5621 : VPAVGW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig e3 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(e3) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPAVGW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36459, 557},
    /* 5622 : VPBLENDMB xmmreg|mask|z,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w0 66 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 08) 303 001(66) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBLENDMB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36477, 550},
    /* 5623 : VPBLENDMB ymmreg|mask|z,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w0 66 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 28) 303 001(66) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBLENDMB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36486, 550},
    /* 5624 : VPBLENDMB zmmreg|mask|z,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w0 66 /r     ] AVX512BW,FUTURE */
        /* 241(f2 7d 48) 303 001(66) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPBLENDMB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36495, 551},
    /* 5625 : VPBLENDMD xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 64 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(64) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBLENDMD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+36504, 544},
    /* 5626 : VPBLENDMD ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 64 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(64) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBLENDMD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+36513, 544},
    /* 5627 : VPBLENDMD zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 64 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(64) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPBLENDMD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+36522, 545},
    /* 5628 : VPBLENDMQ xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 64 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(64) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBLENDMQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+36531, 544},
    /* 5629 : VPBLENDMQ ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 64 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(64) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBLENDMQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+36540, 544},
    /* 5630 : VPBLENDMQ zmmreg|mask|z,zmmreg,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 64 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(64) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPBLENDMQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+36549, 545},
    /* 5631 : VPBLENDMW xmmreg|mask|z,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w1 66 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 08) 303 001(66) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBLENDMW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36558, 550},
    /* 5632 : VPBLENDMW ymmreg|mask|z,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w1 66 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 28) 303 001(66) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBLENDMW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36567, 550},
    /* 5633 : VPBLENDMW zmmreg|mask|z,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w1 66 /r     ] AVX512BW,FUTURE */
        /* 241(f2 fd 48) 303 001(66) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPBLENDMW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36576, 551},
    /* 5634 : VPBROADCASTB xmmreg|mask|z,xmmrm8 [rm:t1s8:  evex.128.66.0f38.w0 78 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 08) 304 001(78) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {XMMREG,RM_XMM|BITS8,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36585, 550},
    /* 5635 : VPBROADCASTB ymmreg|mask|z,xmmrm8 [rm:t1s8:  evex.256.66.0f38.w0 78 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 28) 304 001(78) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {YMMREG,RM_XMM|BITS8,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36594, 550},
    /* 5636 : VPBROADCASTB zmmreg|mask|z,xmmrm8 [rm:t1s8:  evex.512.66.0f38.w0 78 /r         ] AVX512BW,FUTURE */
        /* 250(f2 7d 48) 304 001(78) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {ZMMREG,RM_XMM|BITS8,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36603, 551},
    /* 5637 : VPBROADCASTB xmmreg|mask|z,reg8 [rm:       evex.128.66.0f38.w0 7a /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 08) 300 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {XMMREG,REG_GPR|BITS8,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36612, 550},
    /* 5638 : VPBROADCASTB xmmreg|mask|z,reg16 [rm:       evex.128.66.0f38.w0 7a /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 08) 300 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {XMMREG,REG_GPR|BITS16,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36612, 550},
    /* 5639 : VPBROADCASTB xmmreg|mask|z,reg32 [rm:       evex.128.66.0f38.w0 7a /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 08) 300 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {XMMREG,REG_GPR|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36612, 550},
    /* 5640 : VPBROADCASTB xmmreg|mask|z,reg64 [rm:       evex.128.66.0f38.w0 7a /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 08) 300 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {XMMREG,REG_GPR|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36612, 550},
    /* 5641 : VPBROADCASTB ymmreg|mask|z,reg8 [rm:       evex.256.66.0f38.w0 7a /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 28) 300 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {YMMREG,REG_GPR|BITS8,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36621, 550},
    /* 5642 : VPBROADCASTB ymmreg|mask|z,reg16 [rm:       evex.256.66.0f38.w0 7a /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 28) 300 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {YMMREG,REG_GPR|BITS16,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36621, 550},
    /* 5643 : VPBROADCASTB ymmreg|mask|z,reg32 [rm:       evex.256.66.0f38.w0 7a /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 28) 300 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {YMMREG,REG_GPR|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36621, 550},
    /* 5644 : VPBROADCASTB ymmreg|mask|z,reg64 [rm:       evex.256.66.0f38.w0 7a /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 28) 300 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {YMMREG,REG_GPR|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36621, 550},
    /* 5645 : VPBROADCASTB zmmreg|mask|z,reg8 [rm:       evex.512.66.0f38.w0 7a /r         ] AVX512BW,FUTURE */
        /* 250(f2 7d 48) 300 001(7a) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {ZMMREG,REG_GPR|BITS8,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36630, 551},
    /* 5646 : VPBROADCASTB zmmreg|mask|z,reg16 [rm:       evex.512.66.0f38.w0 7a /r         ] AVX512BW,FUTURE */
        /* 250(f2 7d 48) 300 001(7a) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {ZMMREG,REG_GPR|BITS16,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36630, 551},
    /* 5647 : VPBROADCASTB zmmreg|mask|z,reg32 [rm:       evex.512.66.0f38.w0 7a /r         ] AVX512BW,FUTURE */
        /* 250(f2 7d 48) 300 001(7a) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {ZMMREG,REG_GPR|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36630, 551},
    /* 5648 : VPBROADCASTB zmmreg|mask|z,reg64 [rm:       evex.512.66.0f38.w0 7a /r         ] AVX512BW,FUTURE */
        /* 250(f2 7d 48) 300 001(7a) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {ZMMREG,REG_GPR|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36630, 551},
    /* 5649 : VPBROADCASTD xmmreg|mask|z,mem32 [rm:t1s:   evex.128.66.0f38.w0 58 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 306 001(58) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBROADCASTD, 2, {XMMREG,MEMORY|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36639, 544},
    /* 5650 : VPBROADCASTD ymmreg|mask|z,mem32 [rm:t1s:   evex.256.66.0f38.w0 58 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 306 001(58) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBROADCASTD, 2, {YMMREG,MEMORY|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36648, 544},
    /* 5651 : VPBROADCASTD zmmreg|mask|z,mem32 [rm:t1s:   evex.512.66.0f38.w0 58 /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 306 001(58) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPBROADCASTD, 2, {ZMMREG,MEMORY|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36657, 545},
    /* 5652 : VPBROADCASTD xmmreg|mask|z,xmmreg [rm:       evex.128.66.0f38.w0 58 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 300 001(58) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBROADCASTD, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36666, 544},
    /* 5653 : VPBROADCASTD ymmreg|mask|z,xmmreg [rm:       evex.256.66.0f38.w0 58 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 300 001(58) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBROADCASTD, 2, {YMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36675, 544},
    /* 5654 : VPBROADCASTD zmmreg|mask|z,xmmreg [rm:       evex.512.66.0f38.w0 58 /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 300 001(58) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPBROADCASTD, 2, {ZMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36684, 545},
    /* 5655 : VPBROADCASTD xmmreg|mask|z,reg32 [rm:       evex.128.66.0f38.w0 7c /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 300 001(7c) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBROADCASTD, 2, {XMMREG,REG_GPR|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36693, 544},
    /* 5656 : VPBROADCASTD ymmreg|mask|z,reg32 [rm:       evex.256.66.0f38.w0 7c /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 300 001(7c) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBROADCASTD, 2, {YMMREG,REG_GPR|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36702, 544},
    /* 5657 : VPBROADCASTD zmmreg|mask|z,reg32 [rm:       evex.512.66.0f38.w0 7c /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 300 001(7c) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPBROADCASTD, 2, {ZMMREG,REG_GPR|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36711, 545},
    /* 5658 : VPBROADCASTMB2Q xmmreg,kreg [rm:       evex.128.f3.0f38.w1 2a /r         ] AVX512CD,AVX512VL,FUTURE */
        /* 250(f2 fe 08) 300 001(2a) 110 : EVEX,PROT,AVX512CD,AVX512VL,FUTURE */
        {I_VPBROADCASTMB2Q, 2, {XMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+36720, 558},
    /* 5659 : VPBROADCASTMB2Q ymmreg,kreg [rm:       evex.256.f3.0f38.w1 2a /r         ] AVX512CD,AVX512VL,FUTURE */
        /* 250(f2 fe 28) 300 001(2a) 110 : EVEX,PROT,AVX512CD,AVX512VL,FUTURE */
        {I_VPBROADCASTMB2Q, 2, {YMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+36729, 558},
    /* 5660 : VPBROADCASTMB2Q zmmreg,kreg [rm:       evex.512.f3.0f38.w1 2a /r         ] AVX512CD,FUTURE */
        /* 250(f2 fe 48) 300 001(2a) 110 : EVEX,PROT,AVX512CD,FUTURE */
        {I_VPBROADCASTMB2Q, 2, {ZMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+36738, 559},
    /* 5661 : VPBROADCASTMW2D xmmreg,kreg [rm:       evex.128.f3.0f38.w0 3a /r         ] AVX512CD,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(3a) 110 : EVEX,PROT,AVX512CD,AVX512VL,FUTURE */
        {I_VPBROADCASTMW2D, 2, {XMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+36747, 558},
    /* 5662 : VPBROADCASTMW2D ymmreg,kreg [rm:       evex.256.f3.0f38.w0 3a /r         ] AVX512CD,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(3a) 110 : EVEX,PROT,AVX512CD,AVX512VL,FUTURE */
        {I_VPBROADCASTMW2D, 2, {YMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+36756, 558},
    /* 5663 : VPBROADCASTMW2D zmmreg,kreg [rm:       evex.512.f3.0f38.w0 3a /r         ] AVX512CD,FUTURE */
        /* 250(f2 7e 48) 300 001(3a) 110 : EVEX,PROT,AVX512CD,FUTURE */
        {I_VPBROADCASTMW2D, 2, {ZMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+36765, 559},
    /* 5664 : VPBROADCASTQ xmmreg|mask|z,mem64 [rm:t1s:   evex.128.66.0f38.w1 59 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 306 001(59) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBROADCASTQ, 2, {XMMREG,MEMORY|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36774, 544},
    /* 5665 : VPBROADCASTQ ymmreg|mask|z,mem64 [rm:t1s:   evex.256.66.0f38.w1 59 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 306 001(59) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBROADCASTQ, 2, {YMMREG,MEMORY|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36783, 544},
    /* 5666 : VPBROADCASTQ zmmreg|mask|z,mem64 [rm:t1s:   evex.512.66.0f38.w1 59 /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 306 001(59) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPBROADCASTQ, 2, {ZMMREG,MEMORY|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36792, 545},
    /* 5667 : VPBROADCASTQ xmmreg|mask|z,xmmreg [rm:       evex.128.66.0f38.w1 59 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 300 001(59) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBROADCASTQ, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36801, 544},
    /* 5668 : VPBROADCASTQ ymmreg|mask|z,xmmreg [rm:       evex.256.66.0f38.w1 59 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 300 001(59) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBROADCASTQ, 2, {YMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36810, 544},
    /* 5669 : VPBROADCASTQ zmmreg|mask|z,xmmreg [rm:       evex.512.66.0f38.w1 59 /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 300 001(59) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPBROADCASTQ, 2, {ZMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36819, 545},
    /* 5670 : VPBROADCASTQ xmmreg|mask|z,reg64 [rm:       evex.128.66.0f38.w1 7c /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 300 001(7c) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBROADCASTQ, 2, {XMMREG,REG_GPR|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36828, 544},
    /* 5671 : VPBROADCASTQ ymmreg|mask|z,reg64 [rm:       evex.256.66.0f38.w1 7c /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 300 001(7c) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBROADCASTQ, 2, {YMMREG,REG_GPR|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36837, 544},
    /* 5672 : VPBROADCASTQ zmmreg|mask|z,reg64 [rm:       evex.512.66.0f38.w1 7c /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 300 001(7c) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPBROADCASTQ, 2, {ZMMREG,REG_GPR|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36846, 545},
    /* 5673 : VPBROADCASTW xmmreg|mask|z,xmmrm16 [rm:t1s16: evex.128.66.0f38.w0 79 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 08) 305 001(79) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTW, 2, {XMMREG,RM_XMM|BITS16,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36855, 550},
    /* 5674 : VPBROADCASTW ymmreg|mask|z,xmmrm16 [rm:t1s16: evex.256.66.0f38.w0 79 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 28) 305 001(79) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTW, 2, {YMMREG,RM_XMM|BITS16,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36864, 550},
    /* 5675 : VPBROADCASTW zmmreg|mask|z,xmmrm16 [rm:t1s16: evex.512.66.0f38.w0 79 /r         ] AVX512BW,FUTURE */
        /* 250(f2 7d 48) 305 001(79) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPBROADCASTW, 2, {ZMMREG,RM_XMM|BITS16,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36873, 551},
    /* 5676 : VPBROADCASTW xmmreg|mask|z,reg16 [rm:       evex.128.66.0f38.w0 7b /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 08) 300 001(7b) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTW, 2, {XMMREG,REG_GPR|BITS16,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36882, 550},
    /* 5677 : VPBROADCASTW xmmreg|mask|z,reg32 [rm:       evex.128.66.0f38.w0 7b /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 08) 300 001(7b) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTW, 2, {XMMREG,REG_GPR|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36882, 550},
    /* 5678 : VPBROADCASTW xmmreg|mask|z,reg64 [rm:       evex.128.66.0f38.w0 7b /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 08) 300 001(7b) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTW, 2, {XMMREG,REG_GPR|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36882, 550},
    /* 5679 : VPBROADCASTW ymmreg|mask|z,reg16 [rm:       evex.256.66.0f38.w0 7b /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 28) 300 001(7b) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTW, 2, {YMMREG,REG_GPR|BITS16,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36891, 550},
    /* 5680 : VPBROADCASTW ymmreg|mask|z,reg32 [rm:       evex.256.66.0f38.w0 7b /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 28) 300 001(7b) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTW, 2, {YMMREG,REG_GPR|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36891, 550},
    /* 5681 : VPBROADCASTW ymmreg|mask|z,reg64 [rm:       evex.256.66.0f38.w0 7b /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 28) 300 001(7b) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTW, 2, {YMMREG,REG_GPR|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36891, 550},
    /* 5682 : VPBROADCASTW zmmreg|mask|z,reg16 [rm:       evex.512.66.0f38.w0 7b /r         ] AVX512BW,FUTURE */
        /* 250(f2 7d 48) 300 001(7b) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPBROADCASTW, 2, {ZMMREG,REG_GPR|BITS16,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36900, 551},
    /* 5683 : VPBROADCASTW zmmreg|mask|z,reg32 [rm:       evex.512.66.0f38.w0 7b /r         ] AVX512BW,FUTURE */
        /* 250(f2 7d 48) 300 001(7b) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPBROADCASTW, 2, {ZMMREG,REG_GPR|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36900, 551},
    /* 5684 : VPBROADCASTW zmmreg|mask|z,reg64 [rm:       evex.512.66.0f38.w0 7b /r         ] AVX512BW,FUTURE */
        /* 250(f2 7d 48) 300 001(7b) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPBROADCASTW, 2, {ZMMREG,REG_GPR|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36900, 551},
    /* 5685 : VPCMPEQB kreg|mask,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig 74 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(74) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPEQB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+36909, 556},
    /* 5686 : VPCMPEQB kreg|mask,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig 74 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(74) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPEQB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+36918, 556},
    /* 5687 : VPCMPEQB kreg|mask,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig 74 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(74) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPEQB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+36927, 557},
    /* 5688 : VPCMPEQD kreg|mask,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f.w0 76 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 08) 301 001(76) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPEQD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+36936, 544},
    /* 5689 : VPCMPEQD kreg|mask,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f.w0 76 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 28) 301 001(76) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPEQD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+36945, 544},
    /* 5690 : VPCMPEQD kreg|mask,zmmreg,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f.w0 76 /r       ] AVX512,FUTURE */
        /* 241(f1 7d 48) 301 001(76) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPEQD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+36954, 545},
    /* 5691 : VPCMPEQQ kreg|mask,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 29 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(29) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPEQQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+36963, 544},
    /* 5692 : VPCMPEQQ kreg|mask,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 29 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(29) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPEQQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+36972, 544},
    /* 5693 : VPCMPEQQ kreg|mask,zmmreg,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 29 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(29) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPEQQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+36981, 545},
    /* 5694 : VPCMPEQW kreg|mask,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig 75 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(75) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPEQW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+36990, 556},
    /* 5695 : VPCMPEQW kreg|mask,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig 75 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(75) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPEQW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+36999, 556},
    /* 5696 : VPCMPEQW kreg|mask,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig 75 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(75) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPEQW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+37008, 557},
    /* 5697 : VPCMPGTB kreg|mask,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig 64 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(64) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGTB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+37017, 556},
    /* 5698 : VPCMPGTB kreg|mask,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig 64 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(64) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGTB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+37026, 556},
    /* 5699 : VPCMPGTB kreg|mask,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig 64 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(64) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPGTB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+37035, 557},
    /* 5700 : VPCMPGTD kreg|mask,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f.w0 66 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 08) 301 001(66) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGTD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+37044, 544},
    /* 5701 : VPCMPGTD kreg|mask,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f.w0 66 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 28) 301 001(66) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGTD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+37053, 544},
    /* 5702 : VPCMPGTD kreg|mask,zmmreg,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f.w0 66 /r       ] AVX512,FUTURE */
        /* 241(f1 7d 48) 301 001(66) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPGTD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+37062, 545},
    /* 5703 : VPCMPGTQ kreg|mask,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 37 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(37) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGTQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+37071, 544},
    /* 5704 : VPCMPGTQ kreg|mask,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 37 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(37) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGTQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+37080, 544},
    /* 5705 : VPCMPGTQ kreg|mask,zmmreg,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 37 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(37) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPGTQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+37089, 545},
    /* 5706 : VPCMPGTW kreg|mask,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig 65 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(65) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGTW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+37098, 556},
    /* 5707 : VPCMPGTW kreg|mask,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig 65 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(65) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGTW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+37107, 556},
    /* 5708 : VPCMPGTW kreg|mask,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig 65 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(65) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPGTW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+37116, 557},
    /* 5709 : VPCMPEQB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3f /r 00  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3f) 120 001(00) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPEQB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7559, 550},
    /* 5710 : VPCMPEQB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3f /r 00  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3f) 120 001(00) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPEQB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7570, 550},
    /* 5711 : VPCMPEQB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3f /r 00  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3f) 120 001(00) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPEQB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7581, 551},
    /* 5712 : VPCMPEQD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1f /r 00  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1f) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPEQD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7592, 544},
    /* 5713 : VPCMPEQD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1f /r 00  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1f) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPEQD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7603, 544},
    /* 5714 : VPCMPEQD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1f /r 00  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1f) 120 001(00) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPEQD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7614, 545},
    /* 5715 : VPCMPEQQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1f /r 00  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1f) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPEQQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7625, 544},
    /* 5716 : VPCMPEQQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1f /r 00  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1f) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPEQQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7636, 544},
    /* 5717 : VPCMPEQQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1f /r 00  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1f) 120 001(00) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPEQQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7647, 545},
    /* 5718 : VPCMPEQUB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3e /r 00  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3e) 120 001(00) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPEQUB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7658, 550},
    /* 5719 : VPCMPEQUB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3e /r 00  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3e) 120 001(00) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPEQUB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7669, 550},
    /* 5720 : VPCMPEQUB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3e /r 00  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3e) 120 001(00) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPEQUB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7680, 551},
    /* 5721 : VPCMPEQUD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1e /r 00  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1e) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPEQUD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7691, 544},
    /* 5722 : VPCMPEQUD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1e /r 00  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1e) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPEQUD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7702, 544},
    /* 5723 : VPCMPEQUD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1e /r 00  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1e) 120 001(00) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPEQUD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7713, 545},
    /* 5724 : VPCMPEQUQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1e /r 00  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1e) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPEQUQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7724, 544},
    /* 5725 : VPCMPEQUQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1e /r 00  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1e) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPEQUQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7735, 544},
    /* 5726 : VPCMPEQUQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1e /r 00  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1e) 120 001(00) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPEQUQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7746, 545},
    /* 5727 : VPCMPEQUW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3e /r 00  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3e) 120 001(00) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPEQUW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7757, 550},
    /* 5728 : VPCMPEQUW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3e /r 00  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3e) 120 001(00) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPEQUW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7768, 550},
    /* 5729 : VPCMPEQUW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3e /r 00  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3e) 120 001(00) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPEQUW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7779, 551},
    /* 5730 : VPCMPEQW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3f /r 00  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3f) 120 001(00) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPEQW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7790, 550},
    /* 5731 : VPCMPEQW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3f /r 00  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3f) 120 001(00) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPEQW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7801, 550},
    /* 5732 : VPCMPEQW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3f /r 00  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3f) 120 001(00) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPEQW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7812, 551},
    /* 5733 : VPCMPGEB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3f /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3f) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGEB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7823, 550},
    /* 5734 : VPCMPGEB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3f /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3f) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGEB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7834, 550},
    /* 5735 : VPCMPGEB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3f /r 05  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3f) 120 001(05) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPGEB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7845, 551},
    /* 5736 : VPCMPGED kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1f /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1f) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGED, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7856, 544},
    /* 5737 : VPCMPGED kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1f /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1f) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGED, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7867, 544},
    /* 5738 : VPCMPGED kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1f /r 05  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1f) 120 001(05) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPGED, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7878, 545},
    /* 5739 : VPCMPGEQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1f /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1f) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGEQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7889, 544},
    /* 5740 : VPCMPGEQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1f /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1f) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGEQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7900, 544},
    /* 5741 : VPCMPGEQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1f /r 05  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1f) 120 001(05) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPGEQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7911, 545},
    /* 5742 : VPCMPGEUB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3e /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3e) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGEUB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7922, 550},
    /* 5743 : VPCMPGEUB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3e /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3e) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGEUB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7933, 550},
    /* 5744 : VPCMPGEUB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3e /r 05  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3e) 120 001(05) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPGEUB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7944, 551},
    /* 5745 : VPCMPGEUD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1e /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1e) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGEUD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7955, 544},
    /* 5746 : VPCMPGEUD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1e /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1e) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGEUD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7966, 544},
    /* 5747 : VPCMPGEUD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1e /r 05  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1e) 120 001(05) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPGEUD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7977, 545},
    /* 5748 : VPCMPGEUQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1e /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1e) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGEUQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7988, 544},
    /* 5749 : VPCMPGEUQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1e /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1e) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGEUQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7999, 544},
    /* 5750 : VPCMPGEUQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1e /r 05  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1e) 120 001(05) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPGEUQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8010, 545},
    /* 5751 : VPCMPGEUW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3e /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3e) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGEUW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8021, 550},
    /* 5752 : VPCMPGEUW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3e /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3e) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGEUW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8032, 550},
    /* 5753 : VPCMPGEUW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3e /r 05  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3e) 120 001(05) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPGEUW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8043, 551},
    /* 5754 : VPCMPGEW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3f /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3f) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGEW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8054, 550},
    /* 5755 : VPCMPGEW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3f /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3f) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGEW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8065, 550},
    /* 5756 : VPCMPGEW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3f /r 05  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3f) 120 001(05) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPGEW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8076, 551},
    /* 5757 : VPCMPGTB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3f /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3f) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGTB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8087, 550},
    /* 5758 : VPCMPGTB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3f /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3f) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGTB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8098, 550},
    /* 5759 : VPCMPGTB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3f /r 06  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3f) 120 001(06) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPGTB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8109, 551},
    /* 5760 : VPCMPGTD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1f /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1f) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGTD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8120, 544},
    /* 5761 : VPCMPGTD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1f /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1f) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGTD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8131, 544},
    /* 5762 : VPCMPGTD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1f /r 06  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1f) 120 001(06) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPGTD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8142, 545},
    /* 5763 : VPCMPGTQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1f /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1f) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGTQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8153, 544},
    /* 5764 : VPCMPGTQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1f /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1f) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGTQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8164, 544},
    /* 5765 : VPCMPGTQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1f /r 06  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1f) 120 001(06) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPGTQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8175, 545},
    /* 5766 : VPCMPGTUB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3e /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3e) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGTUB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8186, 550},
    /* 5767 : VPCMPGTUB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3e /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3e) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGTUB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8197, 550},
    /* 5768 : VPCMPGTUB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3e /r 06  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3e) 120 001(06) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPGTUB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8208, 551},
    /* 5769 : VPCMPGTUD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1e /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1e) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGTUD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8219, 544},
    /* 5770 : VPCMPGTUD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1e /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1e) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGTUD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8230, 544},
    /* 5771 : VPCMPGTUD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1e /r 06  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1e) 120 001(06) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPGTUD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8241, 545},
    /* 5772 : VPCMPGTUQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1e /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1e) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGTUQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8252, 544},
    /* 5773 : VPCMPGTUQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1e /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1e) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGTUQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8263, 544},
    /* 5774 : VPCMPGTUQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1e /r 06  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1e) 120 001(06) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPGTUQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8274, 545},
    /* 5775 : VPCMPGTUW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3e /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3e) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGTUW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8285, 550},
    /* 5776 : VPCMPGTUW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3e /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3e) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGTUW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8296, 550},
    /* 5777 : VPCMPGTUW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3e /r 06  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3e) 120 001(06) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPGTUW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8307, 551},
    /* 5778 : VPCMPGTW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3f /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3f) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGTW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8318, 550},
    /* 5779 : VPCMPGTW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3f /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3f) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGTW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8329, 550},
    /* 5780 : VPCMPGTW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3f /r 06  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3f) 120 001(06) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPGTW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8340, 551},
    /* 5781 : VPCMPLEB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3f /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3f) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLEB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8351, 550},
    /* 5782 : VPCMPLEB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3f /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3f) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLEB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8362, 550},
    /* 5783 : VPCMPLEB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3f /r 02  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3f) 120 001(02) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPLEB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8373, 551},
    /* 5784 : VPCMPLED kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1f /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1f) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLED, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8384, 544},
    /* 5785 : VPCMPLED kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1f /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1f) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLED, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8395, 544},
    /* 5786 : VPCMPLED kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1f /r 02  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1f) 120 001(02) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPLED, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8406, 545},
    /* 5787 : VPCMPLEQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1f /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1f) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLEQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8417, 544},
    /* 5788 : VPCMPLEQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1f /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1f) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLEQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8428, 544},
    /* 5789 : VPCMPLEQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1f /r 02  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1f) 120 001(02) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPLEQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8439, 545},
    /* 5790 : VPCMPLEUB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3e /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3e) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLEUB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8450, 550},
    /* 5791 : VPCMPLEUB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3e /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3e) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLEUB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8461, 550},
    /* 5792 : VPCMPLEUB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3e /r 02  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3e) 120 001(02) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPLEUB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8472, 551},
    /* 5793 : VPCMPLEUD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1e /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1e) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLEUD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8483, 544},
    /* 5794 : VPCMPLEUD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1e /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1e) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLEUD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8494, 544},
    /* 5795 : VPCMPLEUD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1e /r 02  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1e) 120 001(02) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPLEUD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8505, 545},
    /* 5796 : VPCMPLEUQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1e /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1e) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLEUQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8516, 544},
    /* 5797 : VPCMPLEUQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1e /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1e) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLEUQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8527, 544},
    /* 5798 : VPCMPLEUQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1e /r 02  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1e) 120 001(02) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPLEUQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8538, 545},
    /* 5799 : VPCMPLEUW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3e /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3e) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLEUW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8549, 550},
    /* 5800 : VPCMPLEUW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3e /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3e) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLEUW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8560, 550},
    /* 5801 : VPCMPLEUW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3e /r 02  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3e) 120 001(02) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPLEUW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8571, 551},
    /* 5802 : VPCMPLEW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3f /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3f) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLEW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8582, 550},
    /* 5803 : VPCMPLEW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3f /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3f) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLEW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8593, 550},
    /* 5804 : VPCMPLEW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3f /r 02  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3f) 120 001(02) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPLEW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8604, 551},
    /* 5805 : VPCMPLTB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3f /r 01  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3f) 120 001(01) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLTB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8615, 550},
    /* 5806 : VPCMPLTB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3f /r 01  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3f) 120 001(01) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLTB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8626, 550},
    /* 5807 : VPCMPLTB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3f /r 01  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3f) 120 001(01) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPLTB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8637, 551},
    /* 5808 : VPCMPLTD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1f /r 01  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1f) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLTD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8648, 544},
    /* 5809 : VPCMPLTD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1f /r 01  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1f) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLTD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8659, 544},
    /* 5810 : VPCMPLTD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1f /r 01  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1f) 120 001(01) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPLTD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8670, 545},
    /* 5811 : VPCMPLTQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1f /r 01  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1f) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLTQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8681, 544},
    /* 5812 : VPCMPLTQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1f /r 01  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1f) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLTQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8692, 544},
    /* 5813 : VPCMPLTQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1f /r 01  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1f) 120 001(01) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPLTQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8703, 545},
    /* 5814 : VPCMPLTUB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3e /r 01  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3e) 120 001(01) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLTUB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8714, 550},
    /* 5815 : VPCMPLTUB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3e /r 01  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3e) 120 001(01) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLTUB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8725, 550},
    /* 5816 : VPCMPLTUB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3e /r 01  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3e) 120 001(01) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPLTUB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8736, 551},
    /* 5817 : VPCMPLTUD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1e /r 01  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1e) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLTUD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8747, 544},
    /* 5818 : VPCMPLTUD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1e /r 01  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1e) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLTUD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8758, 544},
    /* 5819 : VPCMPLTUD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1e /r 01  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1e) 120 001(01) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPLTUD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8769, 545},
    /* 5820 : VPCMPLTUQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1e /r 01  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1e) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLTUQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8780, 544},
    /* 5821 : VPCMPLTUQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1e /r 01  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1e) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLTUQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8791, 544},
    /* 5822 : VPCMPLTUQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1e /r 01  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1e) 120 001(01) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPLTUQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8802, 545},
    /* 5823 : VPCMPLTUW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3e /r 01  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3e) 120 001(01) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLTUW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8813, 550},
    /* 5824 : VPCMPLTUW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3e /r 01  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3e) 120 001(01) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLTUW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8824, 550},
    /* 5825 : VPCMPLTUW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3e /r 01  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3e) 120 001(01) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPLTUW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8835, 551},
    /* 5826 : VPCMPLTW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3f /r 01  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3f) 120 001(01) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLTW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8846, 550},
    /* 5827 : VPCMPLTW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3f /r 01  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3f) 120 001(01) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLTW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8857, 550},
    /* 5828 : VPCMPLTW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3f /r 01  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3f) 120 001(01) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPLTW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8868, 551},
    /* 5829 : VPCMPNEQB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3f /r 04  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3f) 120 001(04) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNEQB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8879, 550},
    /* 5830 : VPCMPNEQB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3f /r 04  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3f) 120 001(04) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNEQB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8890, 550},
    /* 5831 : VPCMPNEQB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3f /r 04  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3f) 120 001(04) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNEQB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8901, 551},
    /* 5832 : VPCMPNEQD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1f /r 04  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1f) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNEQD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8912, 544},
    /* 5833 : VPCMPNEQD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1f /r 04  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1f) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNEQD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8923, 544},
    /* 5834 : VPCMPNEQD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1f /r 04  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1f) 120 001(04) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNEQD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8934, 545},
    /* 5835 : VPCMPNEQQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1f /r 04  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1f) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNEQQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8945, 544},
    /* 5836 : VPCMPNEQQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1f /r 04  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1f) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNEQQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8956, 544},
    /* 5837 : VPCMPNEQQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1f /r 04  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1f) 120 001(04) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNEQQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8967, 545},
    /* 5838 : VPCMPNEQUB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3e /r 04  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3e) 120 001(04) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNEQUB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8978, 550},
    /* 5839 : VPCMPNEQUB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3e /r 04  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3e) 120 001(04) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNEQUB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8989, 550},
    /* 5840 : VPCMPNEQUB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3e /r 04  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3e) 120 001(04) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNEQUB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+9000, 551},
    /* 5841 : VPCMPNEQUD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1e /r 04  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1e) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNEQUD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+9011, 544},
    /* 5842 : VPCMPNEQUD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1e /r 04  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1e) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNEQUD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+9022, 544},
    /* 5843 : VPCMPNEQUD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1e /r 04  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1e) 120 001(04) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNEQUD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+9033, 545},
    /* 5844 : VPCMPNEQUQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1e /r 04  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1e) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNEQUQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+9044, 544},
    /* 5845 : VPCMPNEQUQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1e /r 04  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1e) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNEQUQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+9055, 544},
    /* 5846 : VPCMPNEQUQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1e /r 04  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1e) 120 001(04) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNEQUQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+9066, 545},
    /* 5847 : VPCMPNEQUW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3e /r 04  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3e) 120 001(04) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNEQUW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+9077, 550},
    /* 5848 : VPCMPNEQUW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3e /r 04  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3e) 120 001(04) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNEQUW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+9088, 550},
    /* 5849 : VPCMPNEQUW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3e /r 04  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3e) 120 001(04) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNEQUW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+9099, 551},
    /* 5850 : VPCMPNEQW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3f /r 04  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3f) 120 001(04) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNEQW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+9110, 550},
    /* 5851 : VPCMPNEQW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3f /r 04  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3f) 120 001(04) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNEQW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+9121, 550},
    /* 5852 : VPCMPNEQW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3f /r 04  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3f) 120 001(04) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNEQW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+9132, 551},
    /* 5853 : VPCMPNGTB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3f /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3f) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNGTB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8351, 550},
    /* 5854 : VPCMPNGTB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3f /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3f) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNGTB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8362, 550},
    /* 5855 : VPCMPNGTB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3f /r 02  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3f) 120 001(02) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNGTB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8373, 551},
    /* 5856 : VPCMPNGTD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1f /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1f) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNGTD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8384, 544},
    /* 5857 : VPCMPNGTD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1f /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1f) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNGTD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8395, 544},
    /* 5858 : VPCMPNGTD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1f /r 02  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1f) 120 001(02) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNGTD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8406, 545},
    /* 5859 : VPCMPNGTQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1f /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1f) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNGTQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8417, 544},
    /* 5860 : VPCMPNGTQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1f /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1f) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNGTQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8428, 544},
    /* 5861 : VPCMPNGTQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1f /r 02  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1f) 120 001(02) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNGTQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8439, 545},
    /* 5862 : VPCMPNGTUB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3e /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3e) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNGTUB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8450, 550},
    /* 5863 : VPCMPNGTUB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3e /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3e) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNGTUB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8461, 550},
    /* 5864 : VPCMPNGTUB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3e /r 02  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3e) 120 001(02) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNGTUB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8472, 551},
    /* 5865 : VPCMPNGTUD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1e /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1e) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNGTUD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8483, 544},
    /* 5866 : VPCMPNGTUD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1e /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1e) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNGTUD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8494, 544},
    /* 5867 : VPCMPNGTUD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1e /r 02  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1e) 120 001(02) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNGTUD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8505, 545},
    /* 5868 : VPCMPNGTUQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1e /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1e) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNGTUQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8516, 544},
    /* 5869 : VPCMPNGTUQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1e /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1e) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNGTUQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8527, 544},
    /* 5870 : VPCMPNGTUQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1e /r 02  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1e) 120 001(02) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNGTUQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8538, 545},
    /* 5871 : VPCMPNGTUW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3e /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3e) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNGTUW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8549, 550},
    /* 5872 : VPCMPNGTUW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3e /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3e) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNGTUW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8560, 550},
    /* 5873 : VPCMPNGTUW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3e /r 02  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3e) 120 001(02) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNGTUW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8571, 551},
    /* 5874 : VPCMPNGTW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3f /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3f) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNGTW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8582, 550},
    /* 5875 : VPCMPNGTW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3f /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3f) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNGTW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8593, 550},
    /* 5876 : VPCMPNGTW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3f /r 02  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3f) 120 001(02) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNGTW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8604, 551},
    /* 5877 : VPCMPNLEB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3f /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3f) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLEB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8087, 550},
    /* 5878 : VPCMPNLEB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3f /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3f) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLEB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8098, 550},
    /* 5879 : VPCMPNLEB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3f /r 06  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3f) 120 001(06) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNLEB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8109, 551},
    /* 5880 : VPCMPNLED kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1f /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1f) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLED, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8120, 544},
    /* 5881 : VPCMPNLED kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1f /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1f) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLED, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8131, 544},
    /* 5882 : VPCMPNLED kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1f /r 06  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1f) 120 001(06) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNLED, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8142, 545},
    /* 5883 : VPCMPNLEQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1f /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1f) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLEQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8153, 544},
    /* 5884 : VPCMPNLEQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1f /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1f) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLEQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8164, 544},
    /* 5885 : VPCMPNLEQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1f /r 06  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1f) 120 001(06) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNLEQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8175, 545},
    /* 5886 : VPCMPNLEUB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3e /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3e) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLEUB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8186, 550},
    /* 5887 : VPCMPNLEUB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3e /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3e) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLEUB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8197, 550},
    /* 5888 : VPCMPNLEUB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3e /r 06  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3e) 120 001(06) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNLEUB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8208, 551},
    /* 5889 : VPCMPNLEUD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1e /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1e) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLEUD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8219, 544},
    /* 5890 : VPCMPNLEUD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1e /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1e) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLEUD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8230, 544},
    /* 5891 : VPCMPNLEUD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1e /r 06  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1e) 120 001(06) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNLEUD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8241, 545},
    /* 5892 : VPCMPNLEUQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1e /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1e) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLEUQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8252, 544},
    /* 5893 : VPCMPNLEUQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1e /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1e) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLEUQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8263, 544},
    /* 5894 : VPCMPNLEUQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1e /r 06  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1e) 120 001(06) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNLEUQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8274, 545},
    /* 5895 : VPCMPNLEUW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3e /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3e) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLEUW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8285, 550},
    /* 5896 : VPCMPNLEUW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3e /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3e) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLEUW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8296, 550},
    /* 5897 : VPCMPNLEUW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3e /r 06  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3e) 120 001(06) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNLEUW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8307, 551},
    /* 5898 : VPCMPNLEW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3f /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3f) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLEW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8318, 550},
    /* 5899 : VPCMPNLEW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3f /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3f) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLEW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8329, 550},
    /* 5900 : VPCMPNLEW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3f /r 06  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3f) 120 001(06) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNLEW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8340, 551},
    /* 5901 : VPCMPNLTB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3f /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3f) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLTB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7823, 550},
    /* 5902 : VPCMPNLTB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3f /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3f) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLTB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7834, 550},
    /* 5903 : VPCMPNLTB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3f /r 05  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3f) 120 001(05) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNLTB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7845, 551},
    /* 5904 : VPCMPNLTD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1f /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1f) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLTD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7856, 544},
    /* 5905 : VPCMPNLTD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1f /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1f) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLTD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7867, 544},
    /* 5906 : VPCMPNLTD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1f /r 05  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1f) 120 001(05) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNLTD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7878, 545},
    /* 5907 : VPCMPNLTQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1f /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1f) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLTQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7889, 544},
    /* 5908 : VPCMPNLTQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1f /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1f) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLTQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7900, 544},
    /* 5909 : VPCMPNLTQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1f /r 05  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1f) 120 001(05) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNLTQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7911, 545},
    /* 5910 : VPCMPNLTUB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3e /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3e) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLTUB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7922, 550},
    /* 5911 : VPCMPNLTUB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3e /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3e) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLTUB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7933, 550},
    /* 5912 : VPCMPNLTUB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3e /r 05  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3e) 120 001(05) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNLTUB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7944, 551},
    /* 5913 : VPCMPNLTUD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1e /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1e) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLTUD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7955, 544},
    /* 5914 : VPCMPNLTUD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1e /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1e) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLTUD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7966, 544},
    /* 5915 : VPCMPNLTUD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1e /r 05  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1e) 120 001(05) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNLTUD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7977, 545},
    /* 5916 : VPCMPNLTUQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1e /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1e) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLTUQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7988, 544},
    /* 5917 : VPCMPNLTUQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1e /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1e) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLTUQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7999, 544},
    /* 5918 : VPCMPNLTUQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1e /r 05  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1e) 120 001(05) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNLTUQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8010, 545},
    /* 5919 : VPCMPNLTUW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3e /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3e) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLTUW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8021, 550},
    /* 5920 : VPCMPNLTUW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3e /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3e) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLTUW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8032, 550},
    /* 5921 : VPCMPNLTUW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3e /r 05  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3e) 120 001(05) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNLTUW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8043, 551},
    /* 5922 : VPCMPNLTW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3f /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3f) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLTW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8054, 550},
    /* 5923 : VPCMPNLTW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3f /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3f) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLTW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8065, 550},
    /* 5924 : VPCMPNLTW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3f /r 05  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3f) 120 001(05) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNLTW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8076, 551},
    /* 5925 : VPCMPB kreg|mask,xmmreg,xmmrm128,imm8 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3f /r ib  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3f) 120 023 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPB, 4, {OPMASKREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK,0,0,0,0}, nasm_bytecodes+19513, 550},
    /* 5926 : VPCMPB kreg|mask,ymmreg,ymmrm256,imm8 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3f /r ib  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3f) 120 023 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPB, 4, {OPMASKREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK,0,0,0,0}, nasm_bytecodes+19523, 550},
    /* 5927 : VPCMPB kreg|mask,zmmreg,zmmrm512,imm8 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3f /r ib  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3f) 120 023 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPB, 4, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK,0,0,0,0}, nasm_bytecodes+19533, 551},
    /* 5928 : VPCMPD kreg|mask,xmmreg,xmmrm128|b32,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1f /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1f) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPD, 4, {OPMASKREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK,0,B32,0,0}, nasm_bytecodes+19543, 544},
    /* 5929 : VPCMPD kreg|mask,ymmreg,ymmrm256|b32,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1f /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1f) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPD, 4, {OPMASKREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK,0,B32,0,0}, nasm_bytecodes+19553, 544},
    /* 5930 : VPCMPD kreg|mask,zmmreg,zmmrm512|b32,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1f /r ib  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1f) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPD, 4, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK,0,B32,0,0}, nasm_bytecodes+19563, 545},
    /* 5931 : VPCMPQ kreg|mask,xmmreg,xmmrm128|b64,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1f /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1f) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPQ, 4, {OPMASKREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK,0,B64,0,0}, nasm_bytecodes+19573, 544},
    /* 5932 : VPCMPQ kreg|mask,ymmreg,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1f /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1f) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPQ, 4, {OPMASKREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK,0,B64,0,0}, nasm_bytecodes+19583, 544},
    /* 5933 : VPCMPQ kreg|mask,zmmreg,zmmrm512|b64,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1f /r ib  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1f) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPQ, 4, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK,0,B64,0,0}, nasm_bytecodes+19593, 545},
    /* 5934 : VPCMPUB kreg|mask,xmmreg,xmmrm128,imm8 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3e /r ib  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3e) 120 023 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPUB, 4, {OPMASKREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK,0,0,0,0}, nasm_bytecodes+19603, 550},
    /* 5935 : VPCMPUB kreg|mask,ymmreg,ymmrm256,imm8 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3e /r ib  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3e) 120 023 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPUB, 4, {OPMASKREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK,0,0,0,0}, nasm_bytecodes+19613, 550},
    /* 5936 : VPCMPUB kreg|mask,zmmreg,zmmrm512,imm8 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3e /r ib  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3e) 120 023 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPUB, 4, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK,0,0,0,0}, nasm_bytecodes+19623, 551},
    /* 5937 : VPCMPUD kreg|mask,xmmreg,xmmrm128|b32,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1e /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1e) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPUD, 4, {OPMASKREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK,0,B32,0,0}, nasm_bytecodes+19633, 544},
    /* 5938 : VPCMPUD kreg|mask,ymmreg,ymmrm256|b32,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1e /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1e) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPUD, 4, {OPMASKREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK,0,B32,0,0}, nasm_bytecodes+19643, 544},
    /* 5939 : VPCMPUD kreg|mask,zmmreg,zmmrm512|b32,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1e /r ib  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1e) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPUD, 4, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK,0,B32,0,0}, nasm_bytecodes+19653, 545},
    /* 5940 : VPCMPUQ kreg|mask,xmmreg,xmmrm128|b64,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1e /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1e) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPUQ, 4, {OPMASKREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK,0,B64,0,0}, nasm_bytecodes+19663, 544},
    /* 5941 : VPCMPUQ kreg|mask,ymmreg,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1e /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1e) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPUQ, 4, {OPMASKREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK,0,B64,0,0}, nasm_bytecodes+19673, 544},
    /* 5942 : VPCMPUQ kreg|mask,zmmreg,zmmrm512|b64,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1e /r ib  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1e) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPUQ, 4, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK,0,B64,0,0}, nasm_bytecodes+19683, 545},
    /* 5943 : VPCMPUW kreg|mask,xmmreg,xmmrm128,imm8 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3e /r ib  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3e) 120 023 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPUW, 4, {OPMASKREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK,0,0,0,0}, nasm_bytecodes+19693, 550},
    /* 5944 : VPCMPUW kreg|mask,ymmreg,ymmrm256,imm8 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3e /r ib  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3e) 120 023 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPUW, 4, {OPMASKREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK,0,0,0,0}, nasm_bytecodes+19703, 550},
    /* 5945 : VPCMPUW kreg|mask,zmmreg,zmmrm512,imm8 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3e /r ib  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3e) 120 023 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPUW, 4, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK,0,0,0,0}, nasm_bytecodes+19713, 551},
    /* 5946 : VPCMPW kreg|mask,xmmreg,xmmrm128,imm8 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3f /r ib  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3f) 120 023 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPW, 4, {OPMASKREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK,0,0,0,0}, nasm_bytecodes+19723, 550},
    /* 5947 : VPCMPW kreg|mask,ymmreg,ymmrm256,imm8 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3f /r ib  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3f) 120 023 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPW, 4, {OPMASKREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK,0,0,0,0}, nasm_bytecodes+19733, 550},
    /* 5948 : VPCMPW kreg|mask,zmmreg,zmmrm512,imm8 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3f /r ib  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3f) 120 023 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPW, 4, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK,0,0,0,0}, nasm_bytecodes+19743, 551},
    /* 5949 : VPCOMPRESSD mem128|mask,xmmreg [mr:t1s:   evex.128.66.0f38.w0 8b /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 306 001(8b) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCOMPRESSD, 2, {MEMORY|BITS128,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+37125, 544},
    /* 5950 : VPCOMPRESSD mem256|mask,ymmreg [mr:t1s:   evex.256.66.0f38.w0 8b /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 306 001(8b) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCOMPRESSD, 2, {MEMORY|BITS256,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+37134, 544},
    /* 5951 : VPCOMPRESSD mem512|mask,zmmreg [mr:t1s:   evex.512.66.0f38.w0 8b /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 306 001(8b) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPCOMPRESSD, 2, {MEMORY|BITS512,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+37143, 545},
    /* 5952 : VPCOMPRESSD xmmreg|mask|z,xmmreg [mr:       evex.128.66.0f38.w0 8b /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 300 001(8b) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCOMPRESSD, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37152, 544},
    /* 5953 : VPCOMPRESSD ymmreg|mask|z,ymmreg [mr:       evex.256.66.0f38.w0 8b /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 300 001(8b) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCOMPRESSD, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37161, 544},
    /* 5954 : VPCOMPRESSD zmmreg|mask|z,zmmreg [mr:       evex.512.66.0f38.w0 8b /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 300 001(8b) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPCOMPRESSD, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37170, 545},
    /* 5955 : VPCOMPRESSQ mem128|mask,xmmreg [mr:t1s:   evex.128.66.0f38.w1 8b /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 306 001(8b) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCOMPRESSQ, 2, {MEMORY|BITS128,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+37179, 544},
    /* 5956 : VPCOMPRESSQ mem256|mask,ymmreg [mr:t1s:   evex.256.66.0f38.w1 8b /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 306 001(8b) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCOMPRESSQ, 2, {MEMORY|BITS256,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+37188, 544},
    /* 5957 : VPCOMPRESSQ mem512|mask,zmmreg [mr:t1s:   evex.512.66.0f38.w1 8b /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 306 001(8b) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPCOMPRESSQ, 2, {MEMORY|BITS512,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+37197, 545},
    /* 5958 : VPCOMPRESSQ xmmreg|mask|z,xmmreg [mr:       evex.128.66.0f38.w1 8b /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 300 001(8b) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCOMPRESSQ, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37206, 544},
    /* 5959 : VPCOMPRESSQ ymmreg|mask|z,ymmreg [mr:       evex.256.66.0f38.w1 8b /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 300 001(8b) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCOMPRESSQ, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37215, 544},
    /* 5960 : VPCOMPRESSQ zmmreg|mask|z,zmmreg [mr:       evex.512.66.0f38.w1 8b /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 300 001(8b) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPCOMPRESSQ, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37224, 545},
    /* 5961 : VPCONFLICTD xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.66.0f38.w0 c4 /r         ] AVX512CD,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 301 001(c4) 110 : EVEX,PROT,AVX512CD,AVX512VL,FUTURE */
        {I_VPCONFLICTD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+37233, 558},
    /* 5962 : VPCONFLICTD ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.66.0f38.w0 c4 /r         ] AVX512CD,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 301 001(c4) 110 : EVEX,PROT,AVX512CD,AVX512VL,FUTURE */
        {I_VPCONFLICTD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+37242, 558},
    /* 5963 : VPCONFLICTD zmmreg|mask|z,zmmrm512|b32 [rm:fv:    evex.512.66.0f38.w0 c4 /r         ] AVX512CD,FUTURE */
        /* 250(f2 7d 48) 301 001(c4) 110 : EVEX,PROT,AVX512CD,FUTURE */
        {I_VPCONFLICTD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+37251, 559},
    /* 5964 : VPCONFLICTQ xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.0f38.w1 c4 /r         ] AVX512CD,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 301 001(c4) 110 : EVEX,PROT,AVX512CD,AVX512VL,FUTURE */
        {I_VPCONFLICTQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+37260, 558},
    /* 5965 : VPCONFLICTQ ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.0f38.w1 c4 /r         ] AVX512CD,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 301 001(c4) 110 : EVEX,PROT,AVX512CD,AVX512VL,FUTURE */
        {I_VPCONFLICTQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+37269, 558},
    /* 5966 : VPCONFLICTQ zmmreg|mask|z,zmmrm512|b64 [rm:fv:    evex.512.66.0f38.w1 c4 /r         ] AVX512CD,FUTURE */
        /* 250(f2 fd 48) 301 001(c4) 110 : EVEX,PROT,AVX512CD,FUTURE */
        {I_VPCONFLICTQ, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+37278, 559},
    /* 5967 : VPERMB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w0 8d /r     ] AVX512VL,AVX512VBMI,FUTURE */
        /* 241(f2 7d 08) 303 001(8d) 120 : EVEX,PROT,AVX512VL,AVX512VBMI,FUTURE */
        {I_VPERMB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37287, 560},
    /* 5968 : VPERMB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w0 8d /r     ] AVX512VL,AVX512VBMI,FUTURE */
        /* 241(f2 7d 28) 303 001(8d) 120 : EVEX,PROT,AVX512VL,AVX512VBMI,FUTURE */
        {I_VPERMB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37305, 560},
    /* 5969 : VPERMB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w0 8d /r     ] AVX512VBMI,FUTURE */
        /* 241(f2 7d 48) 303 001(8d) 120 : EVEX,PROT,AVX512VBMI,FUTURE */
        {I_VPERMB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37323, 561},
    /* 5970 : VPERMD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 36 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(36) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37341, 544},
    /* 5971 : VPERMD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 36 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(36) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37359, 545},
    /* 5972 : VPERMI2B xmmreg|mask|z,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w0 75 /r     ] AVX512VL,AVX512VBMI,FUTURE */
        /* 241(f2 7d 08) 303 001(75) 120 : EVEX,PROT,AVX512VL,AVX512VBMI,FUTURE */
        {I_VPERMI2B, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37377, 560},
    /* 5973 : VPERMI2B ymmreg|mask|z,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w0 75 /r     ] AVX512VL,AVX512VBMI,FUTURE */
        /* 241(f2 7d 28) 303 001(75) 120 : EVEX,PROT,AVX512VL,AVX512VBMI,FUTURE */
        {I_VPERMI2B, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37386, 560},
    /* 5974 : VPERMI2B zmmreg|mask|z,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w0 75 /r     ] AVX512VBMI,FUTURE */
        /* 241(f2 7d 48) 303 001(75) 120 : EVEX,PROT,AVX512VBMI,FUTURE */
        {I_VPERMI2B, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37395, 561},
    /* 5975 : VPERMI2D xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 76 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(76) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMI2D, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37404, 544},
    /* 5976 : VPERMI2D ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 76 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(76) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMI2D, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37413, 544},
    /* 5977 : VPERMI2D zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 76 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(76) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMI2D, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37422, 545},
    /* 5978 : VPERMI2PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 77 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(77) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMI2PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37431, 544},
    /* 5979 : VPERMI2PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 77 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(77) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMI2PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37440, 544},
    /* 5980 : VPERMI2PD zmmreg|mask|z,zmmreg,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 77 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(77) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMI2PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37449, 545},
    /* 5981 : VPERMI2PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 77 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(77) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMI2PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37458, 544},
    /* 5982 : VPERMI2PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 77 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(77) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMI2PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37467, 544},
    /* 5983 : VPERMI2PS zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 77 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(77) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMI2PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37476, 545},
    /* 5984 : VPERMI2Q xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 76 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(76) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMI2Q, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37485, 544},
    /* 5985 : VPERMI2Q ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 76 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(76) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMI2Q, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37494, 544},
    /* 5986 : VPERMI2Q zmmreg|mask|z,zmmreg,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 76 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(76) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMI2Q, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37503, 545},
    /* 5987 : VPERMI2W xmmreg|mask|z,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w1 75 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 08) 303 001(75) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPERMI2W, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37512, 550},
    /* 5988 : VPERMI2W ymmreg|mask|z,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w1 75 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 28) 303 001(75) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPERMI2W, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37521, 550},
    /* 5989 : VPERMI2W zmmreg|mask|z,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w1 75 /r     ] AVX512BW,FUTURE */
        /* 241(f2 fd 48) 303 001(75) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPERMI2W, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37530, 551},
    /* 5990 : VPERMILPD xmmreg|mask|z,xmmrm128|b64,imm8 [rmi:fv:   evex.128.66.0f3a.w1 05 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 fd 08) 301 001(05) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMILPD, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+19753, 544},
    /* 5991 : VPERMILPD ymmreg|mask|z,ymmrm256|b64,imm8 [rmi:fv:   evex.256.66.0f3a.w1 05 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 fd 28) 301 001(05) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMILPD, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+19763, 544},
    /* 5992 : VPERMILPD zmmreg|mask|z,zmmrm512|b64,imm8 [rmi:fv:   evex.512.66.0f3a.w1 05 /r ib      ] AVX512,FUTURE */
        /* 250(f3 fd 48) 301 001(05) 110 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMILPD, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+19773, 545},
    /* 5993 : VPERMILPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 0d /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(0d) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMILPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37539, 544},
    /* 5994 : VPERMILPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 0d /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(0d) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMILPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37557, 544},
    /* 5995 : VPERMILPD zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 0d /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(0d) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMILPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37575, 545},
    /* 5996 : VPERMILPS xmmreg|mask|z,xmmrm128|b32,imm8 [rmi:fv:   evex.128.66.0f3a.w0 04 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 08) 301 001(04) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMILPS, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+19783, 544},
    /* 5997 : VPERMILPS ymmreg|mask|z,ymmrm256|b32,imm8 [rmi:fv:   evex.256.66.0f3a.w0 04 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 28) 301 001(04) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMILPS, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+19793, 544},
    /* 5998 : VPERMILPS zmmreg|mask|z,zmmrm512|b32,imm8 [rmi:fv:   evex.512.66.0f3a.w0 04 /r ib      ] AVX512,FUTURE */
        /* 250(f3 7d 48) 301 001(04) 110 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMILPS, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+19803, 545},
    /* 5999 : VPERMILPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 0c /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(0c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMILPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37593, 544},
    /* 6000 : VPERMILPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 0c /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(0c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMILPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37611, 544},
    /* 6001 : VPERMILPS zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 0c /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(0c) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMILPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37629, 545},
    /* 6002 : VPERMPD ymmreg|mask|z,ymmrm256|b64,imm8 [rmi:fv:   evex.256.66.0f3a.w1 01 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 fd 28) 301 001(01) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMPD, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+19813, 544},
    /* 6003 : VPERMPD zmmreg|mask|z,zmmrm512|b64,imm8 [rmi:fv:   evex.512.66.0f3a.w1 01 /r ib      ] AVX512,FUTURE */
        /* 250(f3 fd 48) 301 001(01) 110 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMPD, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+19823, 545},
    /* 6004 : VPERMPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 16 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(16) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37647, 544},
    /* 6005 : VPERMPD zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 16 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(16) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37665, 545},
    /* 6006 : VPERMPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 16 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(16) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37683, 544},
    /* 6007 : VPERMPS zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 16 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(16) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37701, 545},
    /* 6008 : VPERMQ ymmreg|mask|z,ymmrm256|b64,imm8 [rmi:fv:   evex.256.66.0f3a.w1 00 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 fd 28) 301 001(00) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMQ, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+19833, 544},
    /* 6009 : VPERMQ zmmreg|mask|z,zmmrm512|b64,imm8 [rmi:fv:   evex.512.66.0f3a.w1 00 /r ib      ] AVX512,FUTURE */
        /* 250(f3 fd 48) 301 001(00) 110 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMQ, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+19843, 545},
    /* 6010 : VPERMQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 36 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(36) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37719, 544},
    /* 6011 : VPERMQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 36 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(36) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37737, 545},
    /* 6012 : VPERMT2B xmmreg|mask|z,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w0 7d /r     ] AVX512VL,AVX512VBMI,FUTURE */
        /* 241(f2 7d 08) 303 001(7d) 120 : EVEX,PROT,AVX512VL,AVX512VBMI,FUTURE */
        {I_VPERMT2B, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37755, 560},
    /* 6013 : VPERMT2B ymmreg|mask|z,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w0 7d /r     ] AVX512VL,AVX512VBMI,FUTURE */
        /* 241(f2 7d 28) 303 001(7d) 120 : EVEX,PROT,AVX512VL,AVX512VBMI,FUTURE */
        {I_VPERMT2B, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37764, 560},
    /* 6014 : VPERMT2B zmmreg|mask|z,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w0 7d /r     ] AVX512VBMI,FUTURE */
        /* 241(f2 7d 48) 303 001(7d) 120 : EVEX,PROT,AVX512VBMI,FUTURE */
        {I_VPERMT2B, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37773, 561},
    /* 6015 : VPERMT2D xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 7e /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(7e) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMT2D, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37782, 544},
    /* 6016 : VPERMT2D ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 7e /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(7e) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMT2D, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37791, 544},
    /* 6017 : VPERMT2D zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 7e /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(7e) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMT2D, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37800, 545},
    /* 6018 : VPERMT2PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 7f /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(7f) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMT2PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37809, 544},
    /* 6019 : VPERMT2PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 7f /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(7f) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMT2PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37818, 544},
    /* 6020 : VPERMT2PD zmmreg|mask|z,zmmreg,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 7f /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(7f) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMT2PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37827, 545},
    /* 6021 : VPERMT2PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 7f /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(7f) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMT2PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37836, 544},
    /* 6022 : VPERMT2PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 7f /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(7f) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMT2PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37845, 544},
    /* 6023 : VPERMT2PS zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 7f /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(7f) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMT2PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37854, 545},
    /* 6024 : VPERMT2Q xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 7e /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(7e) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMT2Q, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37863, 544},
    /* 6025 : VPERMT2Q ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 7e /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(7e) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMT2Q, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37872, 544},
    /* 6026 : VPERMT2Q zmmreg|mask|z,zmmreg,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 7e /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(7e) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMT2Q, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37881, 545},
    /* 6027 : VPERMT2W xmmreg|mask|z,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w1 7d /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 08) 303 001(7d) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPERMT2W, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37890, 550},
    /* 6028 : VPERMT2W ymmreg|mask|z,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w1 7d /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 28) 303 001(7d) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPERMT2W, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37899, 550},
    /* 6029 : VPERMT2W zmmreg|mask|z,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w1 7d /r     ] AVX512BW,FUTURE */
        /* 241(f2 fd 48) 303 001(7d) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPERMT2W, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37908, 551},
    /* 6030 : VPERMW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w1 8d /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 08) 303 001(8d) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPERMW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37917, 550},
    /* 6031 : VPERMW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w1 8d /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 28) 303 001(8d) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPERMW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37935, 550},
    /* 6032 : VPERMW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w1 8d /r     ] AVX512BW,FUTURE */
        /* 241(f2 fd 48) 303 001(8d) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPERMW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37953, 551},
    /* 6033 : VPEXPANDD xmmreg|mask|z,mem128 [rm:t1s:   evex.128.66.0f38.w0 89 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 306 001(89) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPEXPANDD, 2, {XMMREG,MEMORY|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37971, 544},
    /* 6034 : VPEXPANDD ymmreg|mask|z,mem256 [rm:t1s:   evex.256.66.0f38.w0 89 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 306 001(89) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPEXPANDD, 2, {YMMREG,MEMORY|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37980, 544},
    /* 6035 : VPEXPANDD zmmreg|mask|z,mem512 [rm:t1s:   evex.512.66.0f38.w0 89 /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 306 001(89) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPEXPANDD, 2, {ZMMREG,MEMORY|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37989, 545},
    /* 6036 : VPEXPANDD xmmreg|mask|z,xmmreg [rm:t1s:   evex.128.66.0f38.w0 89 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 306 001(89) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPEXPANDD, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37971, 544},
    /* 6037 : VPEXPANDD ymmreg|mask|z,ymmreg [rm:t1s:   evex.256.66.0f38.w0 89 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 306 001(89) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPEXPANDD, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37980, 544},
    /* 6038 : VPEXPANDD zmmreg|mask|z,zmmreg [rm:t1s:   evex.512.66.0f38.w0 89 /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 306 001(89) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPEXPANDD, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37989, 545},
    /* 6039 : VPEXPANDQ xmmreg|mask|z,mem128 [rm:t1s:   evex.128.66.0f38.w1 89 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 306 001(89) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPEXPANDQ, 2, {XMMREG,MEMORY|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37998, 544},
    /* 6040 : VPEXPANDQ ymmreg|mask|z,mem256 [rm:t1s:   evex.256.66.0f38.w1 89 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 306 001(89) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPEXPANDQ, 2, {YMMREG,MEMORY|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38007, 544},
    /* 6041 : VPEXPANDQ zmmreg|mask|z,mem512 [rm:t1s:   evex.512.66.0f38.w1 89 /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 306 001(89) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPEXPANDQ, 2, {ZMMREG,MEMORY|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38016, 545},
    /* 6042 : VPEXPANDQ xmmreg|mask|z,xmmreg [rm:t1s:   evex.128.66.0f38.w1 89 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 306 001(89) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPEXPANDQ, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37998, 544},
    /* 6043 : VPEXPANDQ ymmreg|mask|z,ymmreg [rm:t1s:   evex.256.66.0f38.w1 89 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 306 001(89) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPEXPANDQ, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38007, 544},
    /* 6044 : VPEXPANDQ zmmreg|mask|z,zmmreg [rm:t1s:   evex.512.66.0f38.w1 89 /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 306 001(89) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPEXPANDQ, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38016, 545},
    /* 6045 : VPEXTRB reg8,xmmreg,imm8 [mri:t1s8: evex.128.66.0f3a.wig 14 /r ib     ] AVX512BW,FUTURE */
        /* 250(f3 7d 08) 304 001(14) 101 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPEXTRB, 3, {REG_GPR|BITS8,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19853, 557},
    /* 6046 : VPEXTRB reg16,xmmreg,imm8 [mri:t1s8: evex.128.66.0f3a.wig 14 /r ib     ] AVX512BW,FUTURE */
        /* 250(f3 7d 08) 304 001(14) 101 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPEXTRB, 3, {REG_GPR|BITS16,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19853, 557},
    /* 6047 : VPEXTRB reg32,xmmreg,imm8 [mri:t1s8: evex.128.66.0f3a.wig 14 /r ib     ] AVX512BW,FUTURE */
        /* 250(f3 7d 08) 304 001(14) 101 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPEXTRB, 3, {REG_GPR|BITS32,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19853, 557},
    /* 6048 : VPEXTRB reg64,xmmreg,imm8 [mri:t1s8: evex.128.66.0f3a.wig 14 /r ib     ] AVX512BW,FUTURE */
        /* 250(f3 7d 08) 304 001(14) 101 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPEXTRB, 3, {REG_GPR|BITS64,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19853, 557},
    /* 6049 : VPEXTRB mem8,xmmreg,imm8 [mri:t1s8: evex.128.66.0f3a.wig 14 /r ib     ] AVX512BW,FUTURE */
        /* 250(f3 7d 08) 304 001(14) 101 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPEXTRB, 3, {MEMORY|BITS8,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19853, 557},
    /* 6050 : VPEXTRD rm32,xmmreg,imm8 [mri:t1s:  evex.128.66.0f3a.w0 16 /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 7d 08) 306 001(16) 101 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VPEXTRD, 3, {RM_GPR|BITS32,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19863, 548},
    /* 6051 : VPEXTRQ rm64,xmmreg,imm8 [mri:t1s:  evex.128.66.0f3a.w1 16 /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 fd 08) 306 001(16) 101 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VPEXTRQ, 3, {RM_GPR|BITS64,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19873, 548},
    /* 6052 : VPEXTRW reg16,xmmreg,imm8 [mri:t1s16:evex.128.66.0f3a.wig 15 /r ib     ] AVX512BW,FUTURE */
        /* 250(f3 7d 08) 305 001(15) 101 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPEXTRW, 3, {REG_GPR|BITS16,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19883, 557},
    /* 6053 : VPEXTRW reg32,xmmreg,imm8 [mri:t1s16:evex.128.66.0f3a.wig 15 /r ib     ] AVX512BW,FUTURE */
        /* 250(f3 7d 08) 305 001(15) 101 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPEXTRW, 3, {REG_GPR|BITS32,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19883, 557},
    /* 6054 : VPEXTRW reg64,xmmreg,imm8 [mri:t1s16:evex.128.66.0f3a.wig 15 /r ib     ] AVX512BW,FUTURE */
        /* 250(f3 7d 08) 305 001(15) 101 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPEXTRW, 3, {REG_GPR|BITS64,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19883, 557},
    /* 6055 : VPEXTRW mem16,xmmreg,imm8 [mri:t1s16:evex.128.66.0f3a.wig 15 /r ib     ] AVX512BW,FUTURE */
        /* 250(f3 7d 08) 305 001(15) 101 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPEXTRW, 3, {MEMORY|BITS16,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19883, 557},
    /* 6056 : VPEXTRW reg16,xmmreg,imm8 [rmi:      evex.128.66.0f.wig c5 /r ib       ] AVX512BW,FUTURE */
        /* 250(f1 7d 08) 300 001(c5) 110 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPEXTRW, 3, {REG_GPR|BITS16,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19893, 557},
    /* 6057 : VPEXTRW reg32,xmmreg,imm8 [rmi:      evex.128.66.0f.wig c5 /r ib       ] AVX512BW,FUTURE */
        /* 250(f1 7d 08) 300 001(c5) 110 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPEXTRW, 3, {REG_GPR|BITS32,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19893, 557},
    /* 6058 : VPEXTRW reg64,xmmreg,imm8 [rmi:      evex.128.66.0f.wig c5 /r ib       ] AVX512BW,FUTURE */
        /* 250(f1 7d 08) 300 001(c5) 110 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPEXTRW, 3, {REG_GPR|BITS64,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19893, 557},
    /* 6059 : VPGATHERDD xmmreg|mask,xmem32 [rm:t1s:   vsibx evex.128.66.0f38.w0 90 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 7d 08) 306 001(90) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPGATHERDD, 2, {XMMREG,XMEM|BITS32,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19903, 544},
    /* 6060 : VPGATHERDD ymmreg|mask,ymem32 [rm:t1s:   vsiby evex.256.66.0f38.w0 90 /r   ] AVX512,AVX512VL,FUTURE */
        /* 375 250(f2 7d 28) 306 001(90) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPGATHERDD, 2, {YMMREG,YMEM|BITS32,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19913, 544},
    /* 6061 : VPGATHERDD zmmreg|mask,zmem32 [rm:t1s:   vsibz evex.512.66.0f38.w0 90 /r   ] AVX512,FUTURE */
        /* 376 250(f2 7d 48) 306 001(90) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPGATHERDD, 2, {ZMMREG,ZMEM|BITS32,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19923, 545},
    /* 6062 : VPGATHERDQ xmmreg|mask,xmem64 [rm:t1s:   vsibx evex.128.66.0f38.w1 90 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 fd 08) 306 001(90) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPGATHERDQ, 2, {XMMREG,XMEM|BITS64,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19933, 544},
    /* 6063 : VPGATHERDQ ymmreg|mask,xmem64 [rm:t1s:   vsibx evex.256.66.0f38.w1 90 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 fd 28) 306 001(90) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPGATHERDQ, 2, {YMMREG,XMEM|BITS64,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19943, 544},
    /* 6064 : VPGATHERDQ zmmreg|mask,ymem64 [rm:t1s:   vsiby evex.512.66.0f38.w1 90 /r   ] AVX512,FUTURE */
        /* 375 250(f2 fd 48) 306 001(90) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPGATHERDQ, 2, {ZMMREG,YMEM|BITS64,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19953, 545},
    /* 6065 : VPGATHERQD xmmreg|mask,xmem32 [rm:t1s:   vsibx evex.128.66.0f38.w0 91 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 7d 08) 306 001(91) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPGATHERQD, 2, {XMMREG,XMEM|BITS32,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19963, 544},
    /* 6066 : VPGATHERQD xmmreg|mask,ymem32 [rm:t1s:   vsiby evex.256.66.0f38.w0 91 /r   ] AVX512,AVX512VL,FUTURE */
        /* 375 250(f2 7d 28) 306 001(91) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPGATHERQD, 2, {XMMREG,YMEM|BITS32,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19973, 544},
    /* 6067 : VPGATHERQD ymmreg|mask,zmem32 [rm:t1s:   vsibz evex.512.66.0f38.w0 91 /r   ] AVX512,FUTURE */
        /* 376 250(f2 7d 48) 306 001(91) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPGATHERQD, 2, {YMMREG,ZMEM|BITS32,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19983, 545},
    /* 6068 : VPGATHERQQ xmmreg|mask,xmem64 [rm:t1s:   vsibx evex.128.66.0f38.w1 91 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 fd 08) 306 001(91) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPGATHERQQ, 2, {XMMREG,XMEM|BITS64,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19993, 544},
    /* 6069 : VPGATHERQQ ymmreg|mask,ymem64 [rm:t1s:   vsiby evex.256.66.0f38.w1 91 /r   ] AVX512,AVX512VL,FUTURE */
        /* 375 250(f2 fd 28) 306 001(91) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPGATHERQQ, 2, {YMMREG,YMEM|BITS64,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20003, 544},
    /* 6070 : VPGATHERQQ zmmreg|mask,zmem64 [rm:t1s:   vsibz evex.512.66.0f38.w1 91 /r   ] AVX512,FUTURE */
        /* 376 250(f2 fd 48) 306 001(91) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPGATHERQQ, 2, {ZMMREG,ZMEM|BITS64,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20013, 545},
    /* 6071 : VPINSRB xmmreg,xmmreg*,reg32,imm8 [rvmi:t1s8:evex.nds.128.66.0f3a.wig 20 /r ib ] AVX512BW,FUTURE */
        /* 241(f3 7d 08) 304 001(20) 120 023 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPINSRB, 4, {XMMREG,XMMREG,REG_GPR|BITS32,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+20023, 557},
    /* 6072 : VPINSRB xmmreg,xmmreg*,mem8,imm8 [rvmi:t1s8:evex.nds.128.66.0f3a.wig 20 /r ib ] AVX512BW,FUTURE */
        /* 241(f3 7d 08) 304 001(20) 120 023 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPINSRB, 4, {XMMREG,XMMREG,MEMORY|BITS8,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+20023, 557},
    /* 6073 : VPINSRD xmmreg,xmmreg*,rm32,imm8 [rvmi:t1s: evex.nds.128.66.0f3a.w0 22 /r ib  ] AVX512DQ,FUTURE */
        /* 241(f3 7d 08) 306 001(22) 120 023 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VPINSRD, 4, {XMMREG,XMMREG,RM_GPR|BITS32,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+20043, 548},
    /* 6074 : VPINSRQ xmmreg,xmmreg*,rm64,imm8 [rvmi:t1s: evex.nds.128.66.0f3a.w1 22 /r ib  ] AVX512DQ,FUTURE */
        /* 241(f3 fd 08) 306 001(22) 120 023 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VPINSRQ, 4, {XMMREG,XMMREG,RM_GPR|BITS64,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+20063, 548},
    /* 6075 : VPINSRW xmmreg,xmmreg*,reg32,imm8 [rvmi:t1s16:evex.nds.128.66.0f.wig c4 /r ib   ] AVX512BW,FUTURE */
        /* 241(f1 7d 08) 305 001(c4) 120 023 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPINSRW, 4, {XMMREG,XMMREG,REG_GPR|BITS32,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+20083, 557},
    /* 6076 : VPINSRW xmmreg,xmmreg*,mem16,imm8 [rvmi:t1s16:evex.nds.128.66.0f.wig c4 /r ib   ] AVX512BW,FUTURE */
        /* 241(f1 7d 08) 305 001(c4) 120 023 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPINSRW, 4, {XMMREG,XMMREG,MEMORY|BITS16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+20083, 557},
    /* 6077 : VPLZCNTD xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.66.0f38.w0 44 /r         ] AVX512CD,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 301 001(44) 110 : EVEX,PROT,AVX512CD,AVX512VL,FUTURE */
        {I_VPLZCNTD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+38025, 558},
    /* 6078 : VPLZCNTD ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.66.0f38.w0 44 /r         ] AVX512CD,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 301 001(44) 110 : EVEX,PROT,AVX512CD,AVX512VL,FUTURE */
        {I_VPLZCNTD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+38034, 558},
    /* 6079 : VPLZCNTD zmmreg|mask|z,zmmrm512|b32 [rm:fv:    evex.512.66.0f38.w0 44 /r         ] AVX512CD,FUTURE */
        /* 250(f2 7d 48) 301 001(44) 110 : EVEX,PROT,AVX512CD,FUTURE */
        {I_VPLZCNTD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+38043, 559},
    /* 6080 : VPLZCNTQ xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.0f38.w1 44 /r         ] AVX512CD,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 301 001(44) 110 : EVEX,PROT,AVX512CD,AVX512VL,FUTURE */
        {I_VPLZCNTQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+38052, 558},
    /* 6081 : VPLZCNTQ ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.0f38.w1 44 /r         ] AVX512CD,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 301 001(44) 110 : EVEX,PROT,AVX512CD,AVX512VL,FUTURE */
        {I_VPLZCNTQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+38061, 558},
    /* 6082 : VPLZCNTQ zmmreg|mask|z,zmmrm512|b64 [rm:fv:    evex.512.66.0f38.w1 44 /r         ] AVX512CD,FUTURE */
        /* 250(f2 fd 48) 301 001(44) 110 : EVEX,PROT,AVX512CD,FUTURE */
        {I_VPLZCNTQ, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+38070, 559},
    /* 6083 : VPMADD52HUQ xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 b5 /r     ] AVX512VL,AVX512IFMA,FUTURE */
        /* 241(f2 fd 08) 301 001(b5) 120 : EVEX,PROT,AVX512VL,AVX512IFMA,FUTURE */
        {I_VPMADD52HUQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38079, 562},
    /* 6084 : VPMADD52HUQ ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 b5 /r     ] AVX512VL,AVX512IFMA,FUTURE */
        /* 241(f2 fd 28) 301 001(b5) 120 : EVEX,PROT,AVX512VL,AVX512IFMA,FUTURE */
        {I_VPMADD52HUQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38088, 562},
    /* 6085 : VPMADD52HUQ zmmreg|mask|z,zmmreg,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 b5 /r     ] AVX512IFMA,FUTURE */
        /* 241(f2 fd 48) 301 001(b5) 120 : EVEX,PROT,AVX512IFMA,FUTURE */
        {I_VPMADD52HUQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38097, 563},
    /* 6086 : VPMADD52LUQ xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 b4 /r     ] AVX512VL,AVX512IFMA,FUTURE */
        /* 241(f2 fd 08) 301 001(b4) 120 : EVEX,PROT,AVX512VL,AVX512IFMA,FUTURE */
        {I_VPMADD52LUQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38106, 562},
    /* 6087 : VPMADD52LUQ ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 b4 /r     ] AVX512VL,AVX512IFMA,FUTURE */
        /* 241(f2 fd 28) 301 001(b4) 120 : EVEX,PROT,AVX512VL,AVX512IFMA,FUTURE */
        {I_VPMADD52LUQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38115, 562},
    /* 6088 : VPMADD52LUQ zmmreg|mask|z,zmmreg,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 b4 /r     ] AVX512IFMA,FUTURE */
        /* 241(f2 fd 48) 301 001(b4) 120 : EVEX,PROT,AVX512IFMA,FUTURE */
        {I_VPMADD52LUQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38124, 563},
    /* 6089 : VPMADDUBSW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.wig 04 /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 08) 303 001(04) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMADDUBSW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38133, 556},
    /* 6090 : VPMADDUBSW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.wig 04 /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 28) 303 001(04) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMADDUBSW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38151, 556},
    /* 6091 : VPMADDUBSW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.wig 04 /r    ] AVX512BW,FUTURE */
        /* 241(f2 7d 48) 303 001(04) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMADDUBSW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38169, 557},
    /* 6092 : VPMADDWD xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig f5 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(f5) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMADDWD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38187, 556},
    /* 6093 : VPMADDWD ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig f5 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(f5) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMADDWD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38205, 556},
    /* 6094 : VPMADDWD zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig f5 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(f5) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMADDWD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38223, 557},
    /* 6095 : VPMAXSB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.wig 3c /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 08) 303 001(3c) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMAXSB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38241, 556},
    /* 6096 : VPMAXSB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.wig 3c /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 28) 303 001(3c) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMAXSB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38259, 556},
    /* 6097 : VPMAXSB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.wig 3c /r    ] AVX512BW,FUTURE */
        /* 241(f2 7d 48) 303 001(3c) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMAXSB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38277, 557},
    /* 6098 : VPMAXSD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 3d /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(3d) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMAXSD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+38295, 544},
    /* 6099 : VPMAXSD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 3d /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(3d) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMAXSD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+38313, 544},
    /* 6100 : VPMAXSD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 3d /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(3d) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMAXSD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+38331, 545},
    /* 6101 : VPMAXSQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 3d /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(3d) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMAXSQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38349, 544},
    /* 6102 : VPMAXSQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 3d /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(3d) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMAXSQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38367, 544},
    /* 6103 : VPMAXSQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 3d /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(3d) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMAXSQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38385, 545},
    /* 6104 : VPMAXSW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig ee /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(ee) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMAXSW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38403, 556},
    /* 6105 : VPMAXSW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig ee /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(ee) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMAXSW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38421, 556},
    /* 6106 : VPMAXSW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig ee /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(ee) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMAXSW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38439, 557},
    /* 6107 : VPMAXUB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig de /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(de) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMAXUB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38457, 556},
    /* 6108 : VPMAXUB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig de /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(de) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMAXUB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38475, 556},
    /* 6109 : VPMAXUB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig de /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(de) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMAXUB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38493, 557},
    /* 6110 : VPMAXUD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 3f /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(3f) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMAXUD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+38511, 544},
    /* 6111 : VPMAXUD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 3f /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(3f) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMAXUD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+38529, 544},
    /* 6112 : VPMAXUD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 3f /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(3f) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMAXUD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+38547, 545},
    /* 6113 : VPMAXUQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 3f /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(3f) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMAXUQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38565, 544},
    /* 6114 : VPMAXUQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 3f /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(3f) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMAXUQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38583, 544},
    /* 6115 : VPMAXUQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 3f /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(3f) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMAXUQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38601, 545},
    /* 6116 : VPMAXUW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.wig 3e /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 08) 303 001(3e) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMAXUW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38619, 556},
    /* 6117 : VPMAXUW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.wig 3e /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 28) 303 001(3e) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMAXUW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38637, 556},
    /* 6118 : VPMAXUW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.wig 3e /r    ] AVX512BW,FUTURE */
        /* 241(f2 7d 48) 303 001(3e) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMAXUW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38655, 557},
    /* 6119 : VPMINSB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.wig 38 /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 08) 303 001(38) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMINSB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38673, 556},
    /* 6120 : VPMINSB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.wig 38 /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 28) 303 001(38) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMINSB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38691, 556},
    /* 6121 : VPMINSB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.wig 38 /r    ] AVX512BW,FUTURE */
        /* 241(f2 7d 48) 303 001(38) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMINSB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38709, 557},
    /* 6122 : VPMINSD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 39 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(39) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMINSD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+38727, 544},
    /* 6123 : VPMINSD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 39 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(39) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMINSD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+38745, 544},
    /* 6124 : VPMINSD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 39 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(39) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMINSD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+38763, 545},
    /* 6125 : VPMINSQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 39 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(39) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMINSQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38781, 544},
    /* 6126 : VPMINSQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 39 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(39) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMINSQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38799, 544},
    /* 6127 : VPMINSQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 39 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(39) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMINSQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38817, 545},
    /* 6128 : VPMINSW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig ea /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(ea) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMINSW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38835, 556},
    /* 6129 : VPMINSW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig ea /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(ea) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMINSW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38853, 556},
    /* 6130 : VPMINSW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig ea /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(ea) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMINSW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38871, 557},
    /* 6131 : VPMINUB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig da /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(da) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMINUB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38889, 556},
    /* 6132 : VPMINUB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig da /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(da) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMINUB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38907, 556},
    /* 6133 : VPMINUB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig da /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(da) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMINUB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38925, 557},
    /* 6134 : VPMINUD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 3b /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(3b) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMINUD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+38943, 544},
    /* 6135 : VPMINUD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 3b /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(3b) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMINUD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+38961, 544},
    /* 6136 : VPMINUD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 3b /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(3b) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMINUD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+38979, 545},
    /* 6137 : VPMINUQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 3b /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(3b) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMINUQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38997, 544},
    /* 6138 : VPMINUQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 3b /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(3b) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMINUQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+39015, 544},
    /* 6139 : VPMINUQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 3b /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(3b) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMINUQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+39033, 545},
    /* 6140 : VPMINUW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.wig 3a /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 08) 303 001(3a) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMINUW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39051, 556},
    /* 6141 : VPMINUW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.wig 3a /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 28) 303 001(3a) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMINUW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39069, 556},
    /* 6142 : VPMINUW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.wig 3a /r    ] AVX512BW,FUTURE */
        /* 241(f2 7d 48) 303 001(3a) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMINUW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39087, 557},
    /* 6143 : VPMOVB2M kreg,xmmreg [rm:       evex.128.f3.0f38.w0 29 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 08) 300 001(29) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVB2M, 2, {OPMASKREG,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39105, 550},
    /* 6144 : VPMOVB2M kreg,ymmreg [rm:       evex.256.f3.0f38.w0 29 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 28) 300 001(29) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVB2M, 2, {OPMASKREG,YMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39114, 550},
    /* 6145 : VPMOVB2M kreg,zmmreg [rm:       evex.512.f3.0f38.w0 29 /r         ] AVX512BW,FUTURE */
        /* 250(f2 7e 48) 300 001(29) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMOVB2M, 2, {OPMASKREG,ZMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39123, 551},
    /* 6146 : VPMOVD2M kreg,xmmreg [rm:       evex.128.f3.0f38.w0 39 /r         ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f2 7e 08) 300 001(39) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VPMOVD2M, 2, {OPMASKREG,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39132, 547},
    /* 6147 : VPMOVD2M kreg,ymmreg [rm:       evex.256.f3.0f38.w0 39 /r         ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f2 7e 28) 300 001(39) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VPMOVD2M, 2, {OPMASKREG,YMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39141, 547},
    /* 6148 : VPMOVD2M kreg,zmmreg [rm:       evex.512.f3.0f38.w0 39 /r         ] AVX512DQ,FUTURE */
        /* 250(f2 7e 48) 300 001(39) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VPMOVD2M, 2, {OPMASKREG,ZMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39150, 548},
    /* 6149 : VPMOVDB xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 31 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(31) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVDB, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39159, 544},
    /* 6150 : VPMOVDB xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 31 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(31) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVDB, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39168, 544},
    /* 6151 : VPMOVDB xmmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 31 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(31) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVDB, 2, {XMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39177, 545},
    /* 6152 : VPMOVDB mem32|mask,xmmreg [mr:qvm:   evex.128.f3.0f38.w0 31 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 315 001(31) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVDB, 2, {MEMORY|BITS32,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39186, 544},
    /* 6153 : VPMOVDB mem64|mask,ymmreg [mr:qvm:   evex.256.f3.0f38.w0 31 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 315 001(31) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVDB, 2, {MEMORY|BITS64,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39195, 544},
    /* 6154 : VPMOVDB mem128|mask,zmmreg [mr:qvm:   evex.512.f3.0f38.w0 31 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 315 001(31) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVDB, 2, {MEMORY|BITS128,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39204, 545},
    /* 6155 : VPMOVDW xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 33 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(33) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVDW, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39213, 544},
    /* 6156 : VPMOVDW xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 33 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(33) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVDW, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39222, 544},
    /* 6157 : VPMOVDW ymmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 33 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(33) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVDW, 2, {YMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39231, 545},
    /* 6158 : VPMOVDW mem64|mask,xmmreg [mr:hvm:   evex.128.f3.0f38.w0 33 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 314 001(33) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVDW, 2, {MEMORY|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39240, 544},
    /* 6159 : VPMOVDW mem128|mask,ymmreg [mr:hvm:   evex.256.f3.0f38.w0 33 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 314 001(33) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVDW, 2, {MEMORY|BITS128,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39249, 544},
    /* 6160 : VPMOVDW mem256|mask,zmmreg [mr:hvm:   evex.512.f3.0f38.w0 33 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 314 001(33) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVDW, 2, {MEMORY|BITS256,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39258, 545},
    /* 6161 : VPMOVM2B xmmreg,kreg [rm:       evex.128.f3.0f38.w0 28 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 08) 300 001(28) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVM2B, 2, {XMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39267, 550},
    /* 6162 : VPMOVM2B ymmreg,kreg [rm:       evex.256.f3.0f38.w0 28 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 28) 300 001(28) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVM2B, 2, {YMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39276, 550},
    /* 6163 : VPMOVM2B zmmreg,kreg [rm:       evex.512.f3.0f38.w0 28 /r         ] AVX512BW,FUTURE */
        /* 250(f2 7e 48) 300 001(28) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMOVM2B, 2, {ZMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39285, 551},
    /* 6164 : VPMOVM2D xmmreg,kreg [rm:       evex.128.f3.0f38.w0 38 /r         ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f2 7e 08) 300 001(38) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VPMOVM2D, 2, {XMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39294, 547},
    /* 6165 : VPMOVM2D ymmreg,kreg [rm:       evex.256.f3.0f38.w0 38 /r         ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f2 7e 28) 300 001(38) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VPMOVM2D, 2, {YMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39303, 547},
    /* 6166 : VPMOVM2D zmmreg,kreg [rm:       evex.512.f3.0f38.w0 38 /r         ] AVX512DQ,FUTURE */
        /* 250(f2 7e 48) 300 001(38) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VPMOVM2D, 2, {ZMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39312, 548},
    /* 6167 : VPMOVM2Q xmmreg,kreg [rm:       evex.128.f3.0f38.w1 38 /r         ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f2 fe 08) 300 001(38) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VPMOVM2Q, 2, {XMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39321, 547},
    /* 6168 : VPMOVM2Q ymmreg,kreg [rm:       evex.256.f3.0f38.w1 38 /r         ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f2 fe 28) 300 001(38) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VPMOVM2Q, 2, {YMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39330, 547},
    /* 6169 : VPMOVM2Q zmmreg,kreg [rm:       evex.512.f3.0f38.w1 38 /r         ] AVX512DQ,FUTURE */
        /* 250(f2 fe 48) 300 001(38) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VPMOVM2Q, 2, {ZMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39339, 548},
    /* 6170 : VPMOVM2W xmmreg,kreg [rm:       evex.128.f3.0f38.w1 28 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 fe 08) 300 001(28) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVM2W, 2, {XMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39348, 550},
    /* 6171 : VPMOVM2W ymmreg,kreg [rm:       evex.256.f3.0f38.w1 28 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 fe 28) 300 001(28) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVM2W, 2, {YMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39357, 550},
    /* 6172 : VPMOVM2W zmmreg,kreg [rm:       evex.512.f3.0f38.w1 28 /r         ] AVX512BW,FUTURE */
        /* 250(f2 fe 48) 300 001(28) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMOVM2W, 2, {ZMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39366, 551},
    /* 6173 : VPMOVQ2M kreg,xmmreg [rm:       evex.128.f3.0f38.w1 39 /r         ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f2 fe 08) 300 001(39) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VPMOVQ2M, 2, {OPMASKREG,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39375, 547},
    /* 6174 : VPMOVQ2M kreg,ymmreg [rm:       evex.256.f3.0f38.w1 39 /r         ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f2 fe 28) 300 001(39) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VPMOVQ2M, 2, {OPMASKREG,YMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39384, 547},
    /* 6175 : VPMOVQ2M kreg,zmmreg [rm:       evex.512.f3.0f38.w1 39 /r         ] AVX512DQ,FUTURE */
        /* 250(f2 fe 48) 300 001(39) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VPMOVQ2M, 2, {OPMASKREG,ZMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39393, 548},
    /* 6176 : VPMOVQB xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 32 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(32) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVQB, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39402, 544},
    /* 6177 : VPMOVQB xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 32 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(32) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVQB, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39411, 544},
    /* 6178 : VPMOVQB xmmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 32 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(32) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVQB, 2, {XMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39420, 545},
    /* 6179 : VPMOVQB mem16|mask,xmmreg [mr:ovm:   evex.128.f3.0f38.w0 32 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 316 001(32) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVQB, 2, {MEMORY|BITS16,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39429, 544},
    /* 6180 : VPMOVQB mem32|mask,ymmreg [mr:ovm:   evex.256.f3.0f38.w0 32 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 316 001(32) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVQB, 2, {MEMORY|BITS32,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39438, 544},
    /* 6181 : VPMOVQB mem64|mask,zmmreg [mr:ovm:   evex.512.f3.0f38.w0 32 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 316 001(32) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVQB, 2, {MEMORY|BITS64,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39447, 545},
    /* 6182 : VPMOVQD xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 35 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(35) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVQD, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39456, 544},
    /* 6183 : VPMOVQD xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 35 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(35) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVQD, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39465, 544},
    /* 6184 : VPMOVQD ymmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 35 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(35) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVQD, 2, {YMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39474, 545},
    /* 6185 : VPMOVQD mem64|mask,xmmreg [mr:hvm:   evex.128.f3.0f38.w0 35 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 314 001(35) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVQD, 2, {MEMORY|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39483, 544},
    /* 6186 : VPMOVQD mem128|mask,ymmreg [mr:hvm:   evex.256.f3.0f38.w0 35 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 314 001(35) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVQD, 2, {MEMORY|BITS128,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39492, 544},
    /* 6187 : VPMOVQD mem256|mask,zmmreg [mr:hvm:   evex.512.f3.0f38.w0 35 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 314 001(35) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVQD, 2, {MEMORY|BITS256,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39501, 545},
    /* 6188 : VPMOVQW xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 34 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(34) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVQW, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39510, 544},
    /* 6189 : VPMOVQW xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 34 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(34) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVQW, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39519, 544},
    /* 6190 : VPMOVQW xmmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 34 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(34) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVQW, 2, {XMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39528, 545},
    /* 6191 : VPMOVQW mem32|mask,xmmreg [mr:qvm:   evex.128.f3.0f38.w0 34 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 315 001(34) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVQW, 2, {MEMORY|BITS32,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39537, 544},
    /* 6192 : VPMOVQW mem64|mask,ymmreg [mr:qvm:   evex.256.f3.0f38.w0 34 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 315 001(34) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVQW, 2, {MEMORY|BITS64,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39546, 544},
    /* 6193 : VPMOVQW mem128|mask,zmmreg [mr:qvm:   evex.512.f3.0f38.w0 34 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 315 001(34) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVQW, 2, {MEMORY|BITS128,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39555, 545},
    /* 6194 : VPMOVSDB xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 21 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(21) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSDB, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39564, 544},
    /* 6195 : VPMOVSDB xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 21 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(21) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSDB, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39573, 544},
    /* 6196 : VPMOVSDB xmmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 21 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(21) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSDB, 2, {XMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39582, 545},
    /* 6197 : VPMOVSDB mem32|mask,xmmreg [mr:qvm:   evex.128.f3.0f38.w0 21 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 315 001(21) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSDB, 2, {MEMORY|BITS32,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39591, 544},
    /* 6198 : VPMOVSDB mem64|mask,ymmreg [mr:qvm:   evex.256.f3.0f38.w0 21 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 315 001(21) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSDB, 2, {MEMORY|BITS64,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39600, 544},
    /* 6199 : VPMOVSDB mem128|mask,zmmreg [mr:qvm:   evex.512.f3.0f38.w0 21 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 315 001(21) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSDB, 2, {MEMORY|BITS128,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39609, 545},
    /* 6200 : VPMOVSDW xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 23 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(23) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSDW, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39618, 544},
    /* 6201 : VPMOVSDW xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 23 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(23) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSDW, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39627, 544},
    /* 6202 : VPMOVSDW ymmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 23 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(23) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSDW, 2, {YMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39636, 545},
    /* 6203 : VPMOVSDW mem64|mask,xmmreg [mr:hvm:   evex.128.f3.0f38.w0 23 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 314 001(23) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSDW, 2, {MEMORY|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39645, 544},
    /* 6204 : VPMOVSDW mem128|mask,ymmreg [mr:hvm:   evex.256.f3.0f38.w0 23 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 314 001(23) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSDW, 2, {MEMORY|BITS128,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39654, 544},
    /* 6205 : VPMOVSDW mem256|mask,zmmreg [mr:hvm:   evex.512.f3.0f38.w0 23 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 314 001(23) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSDW, 2, {MEMORY|BITS256,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39663, 545},
    /* 6206 : VPMOVSQB xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 22 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(22) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSQB, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39672, 544},
    /* 6207 : VPMOVSQB xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 22 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(22) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSQB, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39681, 544},
    /* 6208 : VPMOVSQB xmmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 22 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(22) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSQB, 2, {XMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39690, 545},
    /* 6209 : VPMOVSQB mem16|mask,xmmreg [mr:ovm:   evex.128.f3.0f38.w0 22 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 316 001(22) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSQB, 2, {MEMORY|BITS16,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39699, 544},
    /* 6210 : VPMOVSQB mem32|mask,ymmreg [mr:ovm:   evex.256.f3.0f38.w0 22 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 316 001(22) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSQB, 2, {MEMORY|BITS32,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39708, 544},
    /* 6211 : VPMOVSQB mem64|mask,zmmreg [mr:ovm:   evex.512.f3.0f38.w0 22 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 316 001(22) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSQB, 2, {MEMORY|BITS64,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39717, 545},
    /* 6212 : VPMOVSQD xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 25 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(25) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSQD, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39726, 544},
    /* 6213 : VPMOVSQD xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 25 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(25) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSQD, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39735, 544},
    /* 6214 : VPMOVSQD ymmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 25 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(25) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSQD, 2, {YMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39744, 545},
    /* 6215 : VPMOVSQD mem64|mask,xmmreg [mr:hvm:   evex.128.f3.0f38.w0 25 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 314 001(25) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSQD, 2, {MEMORY|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39753, 544},
    /* 6216 : VPMOVSQD mem128|mask,ymmreg [mr:hvm:   evex.256.f3.0f38.w0 25 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 314 001(25) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSQD, 2, {MEMORY|BITS128,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39762, 544},
    /* 6217 : VPMOVSQD mem256|mask,zmmreg [mr:hvm:   evex.512.f3.0f38.w0 25 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 314 001(25) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSQD, 2, {MEMORY|BITS256,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39771, 545},
    /* 6218 : VPMOVSQW xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 24 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(24) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSQW, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39780, 544},
    /* 6219 : VPMOVSQW xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 24 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(24) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSQW, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39789, 544},
    /* 6220 : VPMOVSQW xmmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 24 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(24) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSQW, 2, {XMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39798, 545},
    /* 6221 : VPMOVSQW mem32|mask,xmmreg [mr:qvm:   evex.128.f3.0f38.w0 24 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 315 001(24) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSQW, 2, {MEMORY|BITS32,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39807, 544},
    /* 6222 : VPMOVSQW mem64|mask,ymmreg [mr:qvm:   evex.256.f3.0f38.w0 24 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 315 001(24) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSQW, 2, {MEMORY|BITS64,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39816, 544},
    /* 6223 : VPMOVSQW mem128|mask,zmmreg [mr:qvm:   evex.512.f3.0f38.w0 24 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 315 001(24) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSQW, 2, {MEMORY|BITS128,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39825, 545},
    /* 6224 : VPMOVSWB xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 20 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 08) 300 001(20) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVSWB, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39834, 550},
    /* 6225 : VPMOVSWB xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 20 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 28) 300 001(20) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVSWB, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39843, 550},
    /* 6226 : VPMOVSWB ymmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 20 /r         ] AVX512BW,FUTURE */
        /* 250(f2 7e 48) 300 001(20) 101 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMOVSWB, 2, {YMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39852, 551},
    /* 6227 : VPMOVSWB mem64|mask,xmmreg [mr:hvm:   evex.128.f3.0f38.w0 20 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 08) 314 001(20) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVSWB, 2, {MEMORY|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39861, 550},
    /* 6228 : VPMOVSWB mem128|mask,ymmreg [mr:hvm:   evex.256.f3.0f38.w0 20 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 28) 314 001(20) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVSWB, 2, {MEMORY|BITS128,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39870, 550},
    /* 6229 : VPMOVSWB mem256|mask,zmmreg [mr:hvm:   evex.512.f3.0f38.w0 20 /r         ] AVX512BW,FUTURE */
        /* 250(f2 7e 48) 314 001(20) 101 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMOVSWB, 2, {MEMORY|BITS256,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39879, 551},
    /* 6230 : VPMOVSXBD xmmreg|mask|z,xmmrm32 [rm:qvm:   evex.128.66.0f38.wig 21 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 315 001(21) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSXBD, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39888, 564},
    /* 6231 : VPMOVSXBD ymmreg|mask|z,xmmrm64 [rm:qvm:   evex.256.66.0f38.wig 21 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 315 001(21) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSXBD, 2, {YMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39897, 564},
    /* 6232 : VPMOVSXBD zmmreg|mask|z,xmmrm128 [rm:qvm:   evex.512.66.0f38.wig 21 /r        ] AVX512,FUTURE */
        /* 250(f2 7d 48) 315 001(21) 110 : WIG,EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSXBD, 2, {ZMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39906, 553},
    /* 6233 : VPMOVSXBQ xmmreg|mask|z,xmmrm16 [rm:ovm:   evex.128.66.0f38.wig 22 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 316 001(22) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSXBQ, 2, {XMMREG,RM_XMM|BITS16,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39915, 564},
    /* 6234 : VPMOVSXBQ ymmreg|mask|z,xmmrm32 [rm:ovm:   evex.256.66.0f38.wig 22 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 316 001(22) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSXBQ, 2, {YMMREG,RM_XMM|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39924, 564},
    /* 6235 : VPMOVSXBQ zmmreg|mask|z,xmmrm64 [rm:ovm:   evex.512.66.0f38.wig 22 /r        ] AVX512,FUTURE */
        /* 250(f2 7d 48) 316 001(22) 110 : WIG,EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSXBQ, 2, {ZMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39933, 553},
    /* 6236 : VPMOVSXBW xmmreg|mask|z,xmmrm64 [rm:hvm:   evex.128.66.0f38.wig 20 /r        ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 08) 314 001(20) 110 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVSXBW, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39942, 556},
    /* 6237 : VPMOVSXBW ymmreg|mask|z,xmmrm128 [rm:hvm:   evex.256.66.0f38.wig 20 /r        ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 28) 314 001(20) 110 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVSXBW, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39951, 556},
    /* 6238 : VPMOVSXBW zmmreg|mask|z,ymmrm256 [rm:hvm:   evex.512.66.0f38.wig 20 /r        ] AVX512BW,FUTURE */
        /* 250(f2 7d 48) 314 001(20) 110 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMOVSXBW, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39960, 557},
    /* 6239 : VPMOVSXDQ xmmreg|mask|z,xmmrm64 [rm:hvm:   evex.128.66.0f38.w0 25 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 314 001(25) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSXDQ, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39969, 544},
    /* 6240 : VPMOVSXDQ ymmreg|mask|z,xmmrm128 [rm:hvm:   evex.256.66.0f38.w0 25 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 314 001(25) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSXDQ, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39978, 544},
    /* 6241 : VPMOVSXDQ zmmreg|mask|z,ymmrm256 [rm:hvm:   evex.512.66.0f38.w0 25 /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 314 001(25) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSXDQ, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39987, 545},
    /* 6242 : VPMOVSXWD xmmreg|mask|z,xmmrm64 [rm:hvm:   evex.128.66.0f38.wig 23 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 314 001(23) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSXWD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39996, 564},
    /* 6243 : VPMOVSXWD ymmreg|mask|z,xmmrm128 [rm:hvm:   evex.256.66.0f38.wig 23 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 314 001(23) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSXWD, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40005, 564},
    /* 6244 : VPMOVSXWD zmmreg|mask|z,ymmrm256 [rm:hvm:   evex.512.66.0f38.wig 23 /r        ] AVX512,FUTURE */
        /* 250(f2 7d 48) 314 001(23) 110 : WIG,EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSXWD, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40014, 553},
    /* 6245 : VPMOVSXWQ xmmreg|mask|z,xmmrm32 [rm:qvm:   evex.128.66.0f38.wig 24 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 315 001(24) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSXWQ, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40023, 564},
    /* 6246 : VPMOVSXWQ ymmreg|mask|z,xmmrm64 [rm:qvm:   evex.256.66.0f38.wig 24 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 315 001(24) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSXWQ, 2, {YMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40032, 564},
    /* 6247 : VPMOVSXWQ zmmreg|mask|z,xmmrm128 [rm:qvm:   evex.512.66.0f38.wig 24 /r        ] AVX512,FUTURE */
        /* 250(f2 7d 48) 315 001(24) 110 : WIG,EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSXWQ, 2, {ZMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40041, 553},
    /* 6248 : VPMOVUSDB xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 11 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(11) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSDB, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40050, 544},
    /* 6249 : VPMOVUSDB xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 11 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(11) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSDB, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40059, 544},
    /* 6250 : VPMOVUSDB xmmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 11 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(11) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVUSDB, 2, {XMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40068, 545},
    /* 6251 : VPMOVUSDB mem32|mask,xmmreg [mr:qvm:   evex.128.f3.0f38.w0 11 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 315 001(11) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSDB, 2, {MEMORY|BITS32,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40077, 544},
    /* 6252 : VPMOVUSDB mem64|mask,ymmreg [mr:qvm:   evex.256.f3.0f38.w0 11 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 315 001(11) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSDB, 2, {MEMORY|BITS64,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40086, 544},
    /* 6253 : VPMOVUSDB mem128|mask,zmmreg [mr:qvm:   evex.512.f3.0f38.w0 11 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 315 001(11) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVUSDB, 2, {MEMORY|BITS128,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40095, 545},
    /* 6254 : VPMOVUSDW xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 13 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(13) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSDW, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40104, 544},
    /* 6255 : VPMOVUSDW xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 13 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(13) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSDW, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40113, 544},
    /* 6256 : VPMOVUSDW ymmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 13 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(13) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVUSDW, 2, {YMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40122, 545},
    /* 6257 : VPMOVUSDW mem64|mask,xmmreg [mr:hvm:   evex.128.f3.0f38.w0 13 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 314 001(13) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSDW, 2, {MEMORY|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40131, 544},
    /* 6258 : VPMOVUSDW mem128|mask,ymmreg [mr:hvm:   evex.256.f3.0f38.w0 13 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 314 001(13) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSDW, 2, {MEMORY|BITS128,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40140, 544},
    /* 6259 : VPMOVUSDW mem256|mask,zmmreg [mr:hvm:   evex.512.f3.0f38.w0 13 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 314 001(13) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVUSDW, 2, {MEMORY|BITS256,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40149, 545},
    /* 6260 : VPMOVUSQB xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 12 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(12) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSQB, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40158, 544},
    /* 6261 : VPMOVUSQB xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 12 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(12) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSQB, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40167, 544},
    /* 6262 : VPMOVUSQB xmmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 12 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(12) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVUSQB, 2, {XMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40176, 545},
    /* 6263 : VPMOVUSQB mem16|mask,xmmreg [mr:ovm:   evex.128.f3.0f38.w0 12 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 316 001(12) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSQB, 2, {MEMORY|BITS16,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40185, 544},
    /* 6264 : VPMOVUSQB mem32|mask,ymmreg [mr:ovm:   evex.256.f3.0f38.w0 12 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 316 001(12) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSQB, 2, {MEMORY|BITS32,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40194, 544},
    /* 6265 : VPMOVUSQB mem64|mask,zmmreg [mr:ovm:   evex.512.f3.0f38.w0 12 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 316 001(12) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVUSQB, 2, {MEMORY|BITS64,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40203, 545},
    /* 6266 : VPMOVUSQD xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 15 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(15) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSQD, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40212, 544},
    /* 6267 : VPMOVUSQD xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 15 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(15) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSQD, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40221, 544},
    /* 6268 : VPMOVUSQD ymmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 15 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(15) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVUSQD, 2, {YMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40230, 545},
    /* 6269 : VPMOVUSQD mem64|mask,xmmreg [mr:hvm:   evex.128.f3.0f38.w0 15 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 314 001(15) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSQD, 2, {MEMORY|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40239, 544},
    /* 6270 : VPMOVUSQD mem128|mask,ymmreg [mr:hvm:   evex.256.f3.0f38.w0 15 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 314 001(15) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSQD, 2, {MEMORY|BITS128,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40248, 544},
    /* 6271 : VPMOVUSQD mem256|mask,zmmreg [mr:hvm:   evex.512.f3.0f38.w0 15 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 314 001(15) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVUSQD, 2, {MEMORY|BITS256,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40257, 545},
    /* 6272 : VPMOVUSQW xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 14 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(14) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSQW, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40266, 544},
    /* 6273 : VPMOVUSQW xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 14 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(14) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSQW, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40275, 544},
    /* 6274 : VPMOVUSQW xmmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 14 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(14) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVUSQW, 2, {XMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40284, 545},
    /* 6275 : VPMOVUSQW mem32|mask,xmmreg [mr:qvm:   evex.128.f3.0f38.w0 14 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 315 001(14) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSQW, 2, {MEMORY|BITS32,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40293, 544},
    /* 6276 : VPMOVUSQW mem64|mask,ymmreg [mr:qvm:   evex.256.f3.0f38.w0 14 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 315 001(14) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSQW, 2, {MEMORY|BITS64,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40302, 544},
    /* 6277 : VPMOVUSQW mem128|mask,zmmreg [mr:qvm:   evex.512.f3.0f38.w0 14 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 315 001(14) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVUSQW, 2, {MEMORY|BITS128,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40311, 545},
    /* 6278 : VPMOVUSWB xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 10 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 08) 300 001(10) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVUSWB, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40320, 550},
    /* 6279 : VPMOVUSWB xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 10 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 28) 300 001(10) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVUSWB, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40329, 550},
    /* 6280 : VPMOVUSWB ymmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 10 /r         ] AVX512BW,FUTURE */
        /* 250(f2 7e 48) 300 001(10) 101 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMOVUSWB, 2, {YMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40338, 551},
    /* 6281 : VPMOVUSWB mem64|mask,xmmreg [mr:hvm:   evex.128.f3.0f38.w0 10 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 08) 314 001(10) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVUSWB, 2, {MEMORY|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40347, 550},
    /* 6282 : VPMOVUSWB mem128|mask,ymmreg [mr:hvm:   evex.256.f3.0f38.w0 10 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 28) 314 001(10) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVUSWB, 2, {MEMORY|BITS128,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40356, 550},
    /* 6283 : VPMOVUSWB mem256|mask,zmmreg [mr:hvm:   evex.512.f3.0f38.w0 10 /r         ] AVX512BW,FUTURE */
        /* 250(f2 7e 48) 314 001(10) 101 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMOVUSWB, 2, {MEMORY|BITS256,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40365, 551},
    /* 6284 : VPMOVW2M kreg,xmmreg [rm:       evex.128.f3.0f38.w1 29 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 fe 08) 300 001(29) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVW2M, 2, {OPMASKREG,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+40374, 550},
    /* 6285 : VPMOVW2M kreg,ymmreg [rm:       evex.256.f3.0f38.w1 29 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 fe 28) 300 001(29) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVW2M, 2, {OPMASKREG,YMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+40383, 550},
    /* 6286 : VPMOVW2M kreg,zmmreg [rm:       evex.512.f3.0f38.w1 29 /r         ] AVX512BW,FUTURE */
        /* 250(f2 fe 48) 300 001(29) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMOVW2M, 2, {OPMASKREG,ZMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+40392, 551},
    /* 6287 : VPMOVWB xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 30 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 08) 300 001(30) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVWB, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40401, 550},
    /* 6288 : VPMOVWB xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 30 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 28) 300 001(30) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVWB, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40410, 550},
    /* 6289 : VPMOVWB ymmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 30 /r         ] AVX512BW,FUTURE */
        /* 250(f2 7e 48) 300 001(30) 101 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMOVWB, 2, {YMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40419, 551},
    /* 6290 : VPMOVWB mem64|mask,xmmreg [mr:hvm:   evex.128.f3.0f38.w0 30 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 08) 314 001(30) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVWB, 2, {MEMORY|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40428, 550},
    /* 6291 : VPMOVWB mem128|mask,ymmreg [mr:hvm:   evex.256.f3.0f38.w0 30 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 28) 314 001(30) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVWB, 2, {MEMORY|BITS128,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40437, 550},
    /* 6292 : VPMOVWB mem256|mask,zmmreg [mr:hvm:   evex.512.f3.0f38.w0 30 /r         ] AVX512BW,FUTURE */
        /* 250(f2 7e 48) 314 001(30) 101 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMOVWB, 2, {MEMORY|BITS256,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40446, 551},
    /* 6293 : VPMOVZXBD xmmreg|mask|z,xmmrm32 [rm:qvm:   evex.128.66.0f38.wig 31 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 315 001(31) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVZXBD, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40455, 564},
    /* 6294 : VPMOVZXBD ymmreg|mask|z,xmmrm64 [rm:qvm:   evex.256.66.0f38.wig 31 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 315 001(31) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVZXBD, 2, {YMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40464, 564},
    /* 6295 : VPMOVZXBD zmmreg|mask|z,xmmrm128 [rm:qvm:   evex.512.66.0f38.wig 31 /r        ] AVX512,FUTURE */
        /* 250(f2 7d 48) 315 001(31) 110 : WIG,EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVZXBD, 2, {ZMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40473, 553},
    /* 6296 : VPMOVZXBQ xmmreg|mask|z,xmmrm16 [rm:ovm:   evex.128.66.0f38.wig 32 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 316 001(32) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVZXBQ, 2, {XMMREG,RM_XMM|BITS16,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40482, 564},
    /* 6297 : VPMOVZXBQ ymmreg|mask|z,xmmrm32 [rm:ovm:   evex.256.66.0f38.wig 32 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 316 001(32) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVZXBQ, 2, {YMMREG,RM_XMM|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40491, 564},
    /* 6298 : VPMOVZXBQ zmmreg|mask|z,xmmrm64 [rm:ovm:   evex.512.66.0f38.wig 32 /r        ] AVX512,FUTURE */
        /* 250(f2 7d 48) 316 001(32) 110 : WIG,EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVZXBQ, 2, {ZMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40500, 553},
    /* 6299 : VPMOVZXBW xmmreg|mask|z,xmmrm64 [rm:hvm:   evex.128.66.0f38.wig 30 /r        ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 08) 314 001(30) 110 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVZXBW, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40509, 556},
    /* 6300 : VPMOVZXBW ymmreg|mask|z,xmmrm128 [rm:hvm:   evex.256.66.0f38.wig 30 /r        ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 28) 314 001(30) 110 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVZXBW, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40518, 556},
    /* 6301 : VPMOVZXBW zmmreg|mask|z,ymmrm256 [rm:hvm:   evex.512.66.0f38.wig 30 /r        ] AVX512BW,FUTURE */
        /* 250(f2 7d 48) 314 001(30) 110 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMOVZXBW, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40527, 557},
    /* 6302 : VPMOVZXDQ xmmreg|mask|z,xmmrm64 [rm:hvm:   evex.128.66.0f38.w0 35 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 314 001(35) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVZXDQ, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40536, 544},
    /* 6303 : VPMOVZXDQ ymmreg|mask|z,xmmrm128 [rm:hvm:   evex.256.66.0f38.w0 35 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 314 001(35) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVZXDQ, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40545, 544},
    /* 6304 : VPMOVZXDQ zmmreg|mask|z,ymmrm256 [rm:hvm:   evex.512.66.0f38.w0 35 /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 314 001(35) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVZXDQ, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40554, 545},
    /* 6305 : VPMOVZXWD xmmreg|mask|z,xmmrm64 [rm:hvm:   evex.128.66.0f38.wig 33 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 314 001(33) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVZXWD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40563, 564},
    /* 6306 : VPMOVZXWD ymmreg|mask|z,xmmrm128 [rm:hvm:   evex.256.66.0f38.wig 33 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 314 001(33) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVZXWD, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40572, 564},
    /* 6307 : VPMOVZXWD zmmreg|mask|z,ymmrm256 [rm:hvm:   evex.512.66.0f38.wig 33 /r        ] AVX512,FUTURE */
        /* 250(f2 7d 48) 314 001(33) 110 : WIG,EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVZXWD, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40581, 553},
    /* 6308 : VPMOVZXWQ xmmreg|mask|z,xmmrm32 [rm:qvm:   evex.128.66.0f38.wig 34 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 315 001(34) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVZXWQ, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40590, 564},
    /* 6309 : VPMOVZXWQ ymmreg|mask|z,xmmrm64 [rm:qvm:   evex.256.66.0f38.wig 34 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 315 001(34) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVZXWQ, 2, {YMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40599, 564},
    /* 6310 : VPMOVZXWQ zmmreg|mask|z,xmmrm128 [rm:qvm:   evex.512.66.0f38.wig 34 /r        ] AVX512,FUTURE */
        /* 250(f2 7d 48) 315 001(34) 110 : WIG,EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVZXWQ, 2, {ZMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40608, 553},
    /* 6311 : VPMULDQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 28 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(28) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMULDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+40617, 544},
    /* 6312 : VPMULDQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 28 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(28) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMULDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+40635, 544},
    /* 6313 : VPMULDQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 28 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(28) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMULDQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+40653, 545},
    /* 6314 : VPMULHRSW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.wig 0b /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 08) 303 001(0b) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMULHRSW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40671, 556},
    /* 6315 : VPMULHRSW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.wig 0b /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 28) 303 001(0b) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMULHRSW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40689, 556},
    /* 6316 : VPMULHRSW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.wig 0b /r    ] AVX512BW,FUTURE */
        /* 241(f2 7d 48) 303 001(0b) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMULHRSW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40707, 557},
    /* 6317 : VPMULHUW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig e4 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(e4) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMULHUW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40725, 556},
    /* 6318 : VPMULHUW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig e4 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(e4) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMULHUW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40743, 556},
    /* 6319 : VPMULHUW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig e4 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(e4) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMULHUW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40761, 557},
    /* 6320 : VPMULHW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig e5 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(e5) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMULHW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40779, 556},
    /* 6321 : VPMULHW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig e5 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(e5) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMULHW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40797, 556},
    /* 6322 : VPMULHW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig e5 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(e5) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMULHW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40815, 557},
    /* 6323 : VPMULLD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 40 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(40) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMULLD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+40833, 544},
    /* 6324 : VPMULLD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 40 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(40) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMULLD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+40851, 544},
    /* 6325 : VPMULLD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 40 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(40) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMULLD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+40869, 545},
    /* 6326 : VPMULLQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 40 /r     ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f2 fd 08) 301 001(40) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VPMULLQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+40887, 547},
    /* 6327 : VPMULLQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 40 /r     ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f2 fd 28) 301 001(40) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VPMULLQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+40905, 547},
    /* 6328 : VPMULLQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 40 /r     ] AVX512DQ,FUTURE */
        /* 241(f2 fd 48) 301 001(40) 120 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VPMULLQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+40923, 548},
    /* 6329 : VPMULLW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig d5 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(d5) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMULLW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40941, 556},
    /* 6330 : VPMULLW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig d5 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(d5) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMULLW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40959, 556},
    /* 6331 : VPMULLW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig d5 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(d5) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMULLW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40977, 557},
    /* 6332 : VPMULTISHIFTQB xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 83 /r     ] AVX512VL,AVX512VBMI,FUTURE */
        /* 241(f2 fd 08) 301 001(83) 120 : EVEX,PROT,AVX512VL,AVX512VBMI,FUTURE */
        {I_VPMULTISHIFTQB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+40995, 560},
    /* 6333 : VPMULTISHIFTQB ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 83 /r     ] AVX512VL,AVX512VBMI,FUTURE */
        /* 241(f2 fd 28) 301 001(83) 120 : EVEX,PROT,AVX512VL,AVX512VBMI,FUTURE */
        {I_VPMULTISHIFTQB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41013, 560},
    /* 6334 : VPMULTISHIFTQB zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 83 /r     ] AVX512VBMI,FUTURE */
        /* 241(f2 fd 48) 301 001(83) 120 : EVEX,PROT,AVX512VBMI,FUTURE */
        {I_VPMULTISHIFTQB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41031, 561},
    /* 6335 : VPMULUDQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 f4 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(f4) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMULUDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41049, 544},
    /* 6336 : VPMULUDQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 f4 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(f4) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMULUDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41067, 544},
    /* 6337 : VPMULUDQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 f4 /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(f4) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMULUDQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41085, 545},
    /* 6338 : VPORD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f.w0 eb /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 08) 301 001(eb) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPORD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41103, 544},
    /* 6339 : VPORD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f.w0 eb /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 28) 301 001(eb) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPORD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41121, 544},
    /* 6340 : VPORD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f.w0 eb /r       ] AVX512,FUTURE */
        /* 241(f1 7d 48) 301 001(eb) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPORD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41139, 545},
    /* 6341 : VPORQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 eb /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(eb) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPORQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41157, 544},
    /* 6342 : VPORQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 eb /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(eb) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPORQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41175, 544},
    /* 6343 : VPORQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 eb /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(eb) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPORQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41193, 545},
    /* 6344 : VPROLD xmmreg|mask|z,xmmrm128|b32*,imm8 [vmi:fv:   evex.nds.128.66.0f.w0 72 /1 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 7d 08) 301 001(72) 211 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPROLD, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20103, 544},
    /* 6345 : VPROLD ymmreg|mask|z,ymmrm256|b32*,imm8 [vmi:fv:   evex.nds.256.66.0f.w0 72 /1 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 7d 28) 301 001(72) 211 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPROLD, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20123, 544},
    /* 6346 : VPROLD zmmreg|mask|z,zmmrm512|b32*,imm8 [vmi:fv:   evex.nds.512.66.0f.w0 72 /1 ib    ] AVX512,FUTURE */
        /* 240(f1 7d 48) 301 001(72) 211 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPROLD, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20143, 545},
    /* 6347 : VPROLQ xmmreg|mask|z,xmmrm128|b64*,imm8 [vmi:fv:   evex.nds.128.66.0f.w1 72 /1 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 fd 08) 301 001(72) 211 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPROLQ, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+20163, 544},
    /* 6348 : VPROLQ ymmreg|mask|z,ymmrm256|b64*,imm8 [vmi:fv:   evex.nds.256.66.0f.w1 72 /1 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 fd 28) 301 001(72) 211 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPROLQ, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+20183, 544},
    /* 6349 : VPROLQ zmmreg|mask|z,zmmrm512|b64*,imm8 [vmi:fv:   evex.nds.512.66.0f.w1 72 /1 ib    ] AVX512,FUTURE */
        /* 240(f1 fd 48) 301 001(72) 211 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPROLQ, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+20203, 545},
    /* 6350 : VPROLVD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 15 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(15) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPROLVD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41211, 544},
    /* 6351 : VPROLVD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 15 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(15) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPROLVD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41229, 544},
    /* 6352 : VPROLVD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 15 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(15) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPROLVD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41247, 545},
    /* 6353 : VPROLVQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 15 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(15) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPROLVQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41265, 544},
    /* 6354 : VPROLVQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 15 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(15) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPROLVQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41283, 544},
    /* 6355 : VPROLVQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 15 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(15) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPROLVQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41301, 545},
    /* 6356 : VPRORD xmmreg|mask|z,xmmrm128|b32*,imm8 [vmi:fv:   evex.nds.128.66.0f.w0 72 /0 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 7d 08) 301 001(72) 210 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPRORD, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20223, 544},
    /* 6357 : VPRORD ymmreg|mask|z,ymmrm256|b32*,imm8 [vmi:fv:   evex.nds.256.66.0f.w0 72 /0 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 7d 28) 301 001(72) 210 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPRORD, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20243, 544},
    /* 6358 : VPRORD zmmreg|mask|z,zmmrm512|b32*,imm8 [vmi:fv:   evex.nds.512.66.0f.w0 72 /0 ib    ] AVX512,FUTURE */
        /* 240(f1 7d 48) 301 001(72) 210 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPRORD, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20263, 545},
    /* 6359 : VPRORQ xmmreg|mask|z,xmmrm128|b64*,imm8 [vmi:fv:   evex.nds.128.66.0f.w1 72 /0 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 fd 08) 301 001(72) 210 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPRORQ, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+20283, 544},
    /* 6360 : VPRORQ ymmreg|mask|z,ymmrm256|b64*,imm8 [vmi:fv:   evex.nds.256.66.0f.w1 72 /0 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 fd 28) 301 001(72) 210 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPRORQ, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+20303, 544},
    /* 6361 : VPRORQ zmmreg|mask|z,zmmrm512|b64*,imm8 [vmi:fv:   evex.nds.512.66.0f.w1 72 /0 ib    ] AVX512,FUTURE */
        /* 240(f1 fd 48) 301 001(72) 210 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPRORQ, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+20323, 545},
    /* 6362 : VPRORVD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 14 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(14) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPRORVD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41319, 544},
    /* 6363 : VPRORVD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 14 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(14) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPRORVD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41337, 544},
    /* 6364 : VPRORVD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 14 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(14) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPRORVD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41355, 545},
    /* 6365 : VPRORVQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 14 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(14) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPRORVQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41373, 544},
    /* 6366 : VPRORVQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 14 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(14) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPRORVQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41391, 544},
    /* 6367 : VPRORVQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 14 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(14) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPRORVQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41409, 545},
    /* 6368 : VPSADBW xmmreg,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig f6 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(f6) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSADBW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, NO_DECORATOR, nasm_bytecodes+41427, 556},
    /* 6369 : VPSADBW ymmreg,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig f6 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(f6) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSADBW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, NO_DECORATOR, nasm_bytecodes+41445, 556},
    /* 6370 : VPSADBW zmmreg,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig f6 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(f6) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSADBW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, NO_DECORATOR, nasm_bytecodes+41463, 557},
    /* 6371 : VPSCATTERDD xmem32|mask,xmmreg [mr:t1s:   vsibx evex.128.66.0f38.w0 a0 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 7d 08) 306 001(a0) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSCATTERDD, 2, {XMEM|BITS32,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20343, 544},
    /* 6372 : VPSCATTERDD ymem32|mask,ymmreg [mr:t1s:   vsiby evex.256.66.0f38.w0 a0 /r   ] AVX512,AVX512VL,FUTURE */
        /* 375 250(f2 7d 28) 306 001(a0) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSCATTERDD, 2, {YMEM|BITS32,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20353, 544},
    /* 6373 : VPSCATTERDD zmem32|mask,zmmreg [mr:t1s:   vsibz evex.512.66.0f38.w0 a0 /r   ] AVX512,FUTURE */
        /* 376 250(f2 7d 48) 306 001(a0) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSCATTERDD, 2, {ZMEM|BITS32,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20363, 545},
    /* 6374 : VPSCATTERDQ xmem64|mask,xmmreg [mr:t1s:   vsibx evex.128.66.0f38.w1 a0 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 fd 08) 306 001(a0) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSCATTERDQ, 2, {XMEM|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20373, 544},
    /* 6375 : VPSCATTERDQ xmem64|mask,ymmreg [mr:t1s:   vsibx evex.256.66.0f38.w1 a0 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 fd 28) 306 001(a0) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSCATTERDQ, 2, {XMEM|BITS64,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20383, 544},
    /* 6376 : VPSCATTERDQ ymem64|mask,zmmreg [mr:t1s:   vsiby evex.512.66.0f38.w1 a0 /r   ] AVX512,FUTURE */
        /* 375 250(f2 fd 48) 306 001(a0) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSCATTERDQ, 2, {YMEM|BITS64,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20393, 545},
    /* 6377 : VPSCATTERQD xmem32|mask,xmmreg [mr:t1s:   vsibx evex.128.66.0f38.w0 a1 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 7d 08) 306 001(a1) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSCATTERQD, 2, {XMEM|BITS32,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20403, 544},
    /* 6378 : VPSCATTERQD ymem32|mask,xmmreg [mr:t1s:   vsiby evex.256.66.0f38.w0 a1 /r   ] AVX512,AVX512VL,FUTURE */
        /* 375 250(f2 7d 28) 306 001(a1) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSCATTERQD, 2, {YMEM|BITS32,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20413, 544},
    /* 6379 : VPSCATTERQD zmem32|mask,ymmreg [mr:t1s:   vsibz evex.512.66.0f38.w0 a1 /r   ] AVX512,FUTURE */
        /* 376 250(f2 7d 48) 306 001(a1) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSCATTERQD, 2, {ZMEM|BITS32,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20423, 545},
    /* 6380 : VPSCATTERQQ xmem64|mask,xmmreg [mr:t1s:   vsibx evex.128.66.0f38.w1 a1 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 fd 08) 306 001(a1) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSCATTERQQ, 2, {XMEM|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20433, 544},
    /* 6381 : VPSCATTERQQ ymem64|mask,ymmreg [mr:t1s:   vsiby evex.256.66.0f38.w1 a1 /r   ] AVX512,AVX512VL,FUTURE */
        /* 375 250(f2 fd 28) 306 001(a1) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSCATTERQQ, 2, {YMEM|BITS64,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20443, 544},
    /* 6382 : VPSCATTERQQ zmem64|mask,zmmreg [mr:t1s:   vsibz evex.512.66.0f38.w1 a1 /r   ] AVX512,FUTURE */
        /* 376 250(f2 fd 48) 306 001(a1) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSCATTERQQ, 2, {ZMEM|BITS64,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20453, 545},
    /* 6383 : VPSHUFB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.wig 00 /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 08) 303 001(00) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSHUFB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41481, 556},
    /* 6384 : VPSHUFB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.wig 00 /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 28) 303 001(00) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSHUFB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41499, 556},
    /* 6385 : VPSHUFB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.wig 00 /r    ] AVX512BW,FUTURE */
        /* 241(f2 7d 48) 303 001(00) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSHUFB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41517, 557},
    /* 6386 : VPSHUFD xmmreg|mask|z,xmmrm128|b32,imm8 [rmi:fv:   evex.128.66.0f.w0 70 /r ib        ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7d 08) 301 001(70) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSHUFD, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20463, 544},
    /* 6387 : VPSHUFD ymmreg|mask|z,ymmrm256|b32,imm8 [rmi:fv:   evex.256.66.0f.w0 70 /r ib        ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7d 28) 301 001(70) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSHUFD, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20473, 544},
    /* 6388 : VPSHUFD zmmreg|mask|z,zmmrm512|b32,imm8 [rmi:fv:   evex.512.66.0f.w0 70 /r ib        ] AVX512,FUTURE */
        /* 250(f1 7d 48) 301 001(70) 110 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSHUFD, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20483, 545},
    /* 6389 : VPSHUFHW xmmreg|mask|z,xmmrm128,imm8 [rmi:fvm:  evex.128.f3.0f.wig 70 /r ib       ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f1 7e 08) 303 001(70) 110 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSHUFHW, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+20493, 556},
    /* 6390 : VPSHUFHW ymmreg|mask|z,ymmrm256,imm8 [rmi:fvm:  evex.256.f3.0f.wig 70 /r ib       ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f1 7e 28) 303 001(70) 110 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSHUFHW, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+20503, 556},
    /* 6391 : VPSHUFHW zmmreg|mask|z,zmmrm512,imm8 [rmi:fvm:  evex.512.f3.0f.wig 70 /r ib       ] AVX512BW,FUTURE */
        /* 250(f1 7e 48) 303 001(70) 110 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSHUFHW, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+20513, 557},
    /* 6392 : VPSHUFLW xmmreg|mask|z,xmmrm128,imm8 [rmi:fvm:  evex.128.f2.0f.wig 70 /r ib       ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f1 7f 08) 303 001(70) 110 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSHUFLW, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+20523, 556},
    /* 6393 : VPSHUFLW ymmreg|mask|z,ymmrm256,imm8 [rmi:fvm:  evex.256.f2.0f.wig 70 /r ib       ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f1 7f 28) 303 001(70) 110 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSHUFLW, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+20533, 556},
    /* 6394 : VPSHUFLW zmmreg|mask|z,zmmrm512,imm8 [rmi:fvm:  evex.512.f2.0f.wig 70 /r ib       ] AVX512BW,FUTURE */
        /* 250(f1 7f 48) 303 001(70) 110 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSHUFLW, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+20543, 557},
    /* 6395 : VPSLLD xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:m128: evex.nds.128.66.0f.w0 f2 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 08) 317 001(f2) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSLLD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41535, 544},
    /* 6396 : VPSLLD ymmreg|mask|z,ymmreg*,xmmrm128 [rvm:m128: evex.nds.256.66.0f.w0 f2 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 28) 317 001(f2) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSLLD, 3, {YMMREG,YMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41553, 544},
    /* 6397 : VPSLLD zmmreg|mask|z,zmmreg*,xmmrm128 [rvm:m128: evex.nds.512.66.0f.w0 f2 /r       ] AVX512,FUTURE */
        /* 241(f1 7d 48) 317 001(f2) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSLLD, 3, {ZMMREG,ZMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41571, 545},
    /* 6398 : VPSLLD xmmreg|mask|z,xmmrm128|b32*,imm8 [vmi:fv:   evex.nds.128.66.0f.w0 72 /6 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 7d 08) 301 001(72) 216 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSLLD, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20553, 544},
    /* 6399 : VPSLLD ymmreg|mask|z,ymmrm256|b32*,imm8 [vmi:fv:   evex.nds.256.66.0f.w0 72 /6 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 7d 28) 301 001(72) 216 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSLLD, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20573, 544},
    /* 6400 : VPSLLD zmmreg|mask|z,zmmrm512|b32*,imm8 [vmi:fv:   evex.nds.512.66.0f.w0 72 /6 ib    ] AVX512,FUTURE */
        /* 240(f1 7d 48) 301 001(72) 216 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSLLD, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20593, 545},
    /* 6401 : VPSLLDQ xmmreg,xmmrm128*,imm8 [vmi:fvm:  evex.nds.128.66.0f.wig 73 /7 ib   ] AVX512VL,AVX512BW,FUTURE */
        /* 240(f1 7d 08) 303 001(73) 217 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSLLDQ, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+20613, 556},
    /* 6402 : VPSLLDQ ymmreg,ymmrm256*,imm8 [vmi:fvm:  evex.nds.256.66.0f.wig 73 /7 ib   ] AVX512VL,AVX512BW,FUTURE */
        /* 240(f1 7d 28) 303 001(73) 217 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSLLDQ, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+20633, 556},
    /* 6403 : VPSLLDQ zmmreg,zmmrm512*,imm8 [vmi:fvm:  evex.nds.512.66.0f.wig 73 /7 ib   ] AVX512BW,FUTURE */
        /* 240(f1 7d 48) 303 001(73) 217 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSLLDQ, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+20653, 557},
    /* 6404 : VPSLLQ xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:m128: evex.nds.128.66.0f.w1 f3 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 317 001(f3) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSLLQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41589, 544},
    /* 6405 : VPSLLQ ymmreg|mask|z,ymmreg*,xmmrm128 [rvm:m128: evex.nds.256.66.0f.w1 f3 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 317 001(f3) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSLLQ, 3, {YMMREG,YMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41607, 544},
    /* 6406 : VPSLLQ zmmreg|mask|z,zmmreg*,xmmrm128 [rvm:m128: evex.nds.512.66.0f.w1 f3 /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 317 001(f3) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSLLQ, 3, {ZMMREG,ZMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41625, 545},
    /* 6407 : VPSLLQ xmmreg|mask|z,xmmrm128|b64*,imm8 [vmi:fv:   evex.nds.128.66.0f.w1 73 /6 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 fd 08) 301 001(73) 216 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSLLQ, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+20673, 544},
    /* 6408 : VPSLLQ ymmreg|mask|z,ymmrm256|b64*,imm8 [vmi:fv:   evex.nds.256.66.0f.w1 73 /6 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 fd 28) 301 001(73) 216 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSLLQ, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+20693, 544},
    /* 6409 : VPSLLQ zmmreg|mask|z,zmmrm512|b64*,imm8 [vmi:fv:   evex.nds.512.66.0f.w1 73 /6 ib    ] AVX512,FUTURE */
        /* 240(f1 fd 48) 301 001(73) 216 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSLLQ, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+20713, 545},
    /* 6410 : VPSLLVD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 47 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(47) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSLLVD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41643, 544},
    /* 6411 : VPSLLVD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 47 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(47) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSLLVD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41661, 544},
    /* 6412 : VPSLLVD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 47 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(47) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSLLVD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41679, 545},
    /* 6413 : VPSLLVQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 47 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(47) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSLLVQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41697, 544},
    /* 6414 : VPSLLVQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 47 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(47) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSLLVQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41715, 544},
    /* 6415 : VPSLLVQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 47 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(47) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSLLVQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41733, 545},
    /* 6416 : VPSLLVW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w1 12 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 08) 303 001(12) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSLLVW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41751, 550},
    /* 6417 : VPSLLVW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w1 12 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 28) 303 001(12) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSLLVW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41769, 550},
    /* 6418 : VPSLLVW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w1 12 /r     ] AVX512BW,FUTURE */
        /* 241(f2 fd 48) 303 001(12) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSLLVW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41787, 551},
    /* 6419 : VPSLLW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:m128: evex.nds.128.66.0f.wig f1 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 317 001(f1) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSLLW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41805, 556},
    /* 6420 : VPSLLW ymmreg|mask|z,ymmreg*,xmmrm128 [rvm:m128: evex.nds.256.66.0f.wig f1 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 317 001(f1) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSLLW, 3, {YMMREG,YMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41823, 556},
    /* 6421 : VPSLLW zmmreg|mask|z,zmmreg*,xmmrm128 [rvm:m128: evex.nds.512.66.0f.wig f1 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 317 001(f1) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSLLW, 3, {ZMMREG,ZMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41841, 557},
    /* 6422 : VPSLLW xmmreg|mask|z,xmmrm128*,imm8 [vmi:fvm:  evex.nds.128.66.0f.wig 71 /6 ib   ] AVX512VL,AVX512BW,FUTURE */
        /* 240(f1 7d 08) 303 001(71) 216 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSLLW, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+20733, 556},
    /* 6423 : VPSLLW ymmreg|mask|z,ymmrm256*,imm8 [vmi:fvm:  evex.nds.256.66.0f.wig 71 /6 ib   ] AVX512VL,AVX512BW,FUTURE */
        /* 240(f1 7d 28) 303 001(71) 216 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSLLW, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+20753, 556},
    /* 6424 : VPSLLW zmmreg|mask|z,zmmrm512*,imm8 [vmi:fvm:  evex.nds.512.66.0f.wig 71 /6 ib   ] AVX512BW,FUTURE */
        /* 240(f1 7d 48) 303 001(71) 216 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSLLW, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+20773, 557},
    /* 6425 : VPSRAD xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:m128: evex.nds.128.66.0f.w0 e2 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 08) 317 001(e2) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRAD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41859, 544},
    /* 6426 : VPSRAD ymmreg|mask|z,ymmreg*,xmmrm128 [rvm:m128: evex.nds.256.66.0f.w0 e2 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 28) 317 001(e2) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRAD, 3, {YMMREG,YMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41877, 544},
    /* 6427 : VPSRAD zmmreg|mask|z,zmmreg*,xmmrm128 [rvm:m128: evex.nds.512.66.0f.w0 e2 /r       ] AVX512,FUTURE */
        /* 241(f1 7d 48) 317 001(e2) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSRAD, 3, {ZMMREG,ZMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41895, 545},
    /* 6428 : VPSRAD xmmreg|mask|z,xmmrm128|b32*,imm8 [vmi:fv:   evex.nds.128.66.0f.w0 72 /4 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 7d 08) 301 001(72) 214 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRAD, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20793, 544},
    /* 6429 : VPSRAD ymmreg|mask|z,ymmrm256|b32*,imm8 [vmi:fv:   evex.nds.256.66.0f.w0 72 /4 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 7d 28) 301 001(72) 214 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRAD, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20813, 544},
    /* 6430 : VPSRAD zmmreg|mask|z,zmmrm512|b32*,imm8 [vmi:fv:   evex.nds.512.66.0f.w0 72 /4 ib    ] AVX512,FUTURE */
        /* 240(f1 7d 48) 301 001(72) 214 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSRAD, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20833, 545},
    /* 6431 : VPSRAQ xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:m128: evex.nds.128.66.0f.w1 e2 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 317 001(e2) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRAQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41913, 544},
    /* 6432 : VPSRAQ ymmreg|mask|z,ymmreg*,xmmrm128 [rvm:m128: evex.nds.256.66.0f.w1 e2 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 317 001(e2) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRAQ, 3, {YMMREG,YMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41931, 544},
    /* 6433 : VPSRAQ zmmreg|mask|z,zmmreg*,xmmrm128 [rvm:m128: evex.nds.512.66.0f.w1 e2 /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 317 001(e2) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSRAQ, 3, {ZMMREG,ZMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41949, 545},
    /* 6434 : VPSRAQ xmmreg|mask|z,xmmrm128|b64*,imm8 [vmi:fv:   evex.nds.128.66.0f.w1 72 /4 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 fd 08) 301 001(72) 214 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRAQ, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+20853, 544},
    /* 6435 : VPSRAQ ymmreg|mask|z,ymmrm256|b64*,imm8 [vmi:fv:   evex.nds.256.66.0f.w1 72 /4 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 fd 28) 301 001(72) 214 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRAQ, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+20873, 544},
    /* 6436 : VPSRAQ zmmreg|mask|z,zmmrm512|b64*,imm8 [vmi:fv:   evex.nds.512.66.0f.w1 72 /4 ib    ] AVX512,FUTURE */
        /* 240(f1 fd 48) 301 001(72) 214 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSRAQ, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+20893, 545},
    /* 6437 : VPSRAVD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 46 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(46) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRAVD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41967, 544},
    /* 6438 : VPSRAVD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 46 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(46) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRAVD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41985, 544},
    /* 6439 : VPSRAVD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 46 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(46) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSRAVD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+42003, 545},
    /* 6440 : VPSRAVQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 46 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(46) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRAVQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+42021, 544},
    /* 6441 : VPSRAVQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 46 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(46) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRAVQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+42039, 544},
    /* 6442 : VPSRAVQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 46 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(46) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSRAVQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+42057, 545},
    /* 6443 : VPSRAVW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w1 11 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 08) 303 001(11) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRAVW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42075, 550},
    /* 6444 : VPSRAVW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w1 11 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 28) 303 001(11) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRAVW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42093, 550},
    /* 6445 : VPSRAVW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w1 11 /r     ] AVX512BW,FUTURE */
        /* 241(f2 fd 48) 303 001(11) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSRAVW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42111, 551},
    /* 6446 : VPSRAW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:m128: evex.nds.128.66.0f.wig e1 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 317 001(e1) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRAW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42129, 556},
    /* 6447 : VPSRAW ymmreg|mask|z,ymmreg*,xmmrm128 [rvm:m128: evex.nds.256.66.0f.wig e1 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 317 001(e1) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRAW, 3, {YMMREG,YMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42147, 556},
    /* 6448 : VPSRAW zmmreg|mask|z,zmmreg*,xmmrm128 [rvm:m128: evex.nds.512.66.0f.wig e1 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 317 001(e1) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSRAW, 3, {ZMMREG,ZMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42165, 557},
    /* 6449 : VPSRAW xmmreg|mask|z,xmmrm128*,imm8 [vmi:fvm:  evex.nds.128.66.0f.wig 71 /4 ib   ] AVX512VL,AVX512BW,FUTURE */
        /* 240(f1 7d 08) 303 001(71) 214 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRAW, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+20913, 556},
    /* 6450 : VPSRAW ymmreg|mask|z,ymmrm256*,imm8 [vmi:fvm:  evex.nds.256.66.0f.wig 71 /4 ib   ] AVX512VL,AVX512BW,FUTURE */
        /* 240(f1 7d 28) 303 001(71) 214 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRAW, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+20933, 556},
    /* 6451 : VPSRAW zmmreg|mask|z,zmmrm512*,imm8 [vmi:fvm:  evex.nds.512.66.0f.wig 71 /4 ib   ] AVX512BW,FUTURE */
        /* 240(f1 7d 48) 303 001(71) 214 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSRAW, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+20953, 557},
    /* 6452 : VPSRLD xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:m128: evex.nds.128.66.0f.w0 d2 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 08) 317 001(d2) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRLD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42183, 544},
    /* 6453 : VPSRLD ymmreg|mask|z,ymmreg*,xmmrm128 [rvm:m128: evex.nds.256.66.0f.w0 d2 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 28) 317 001(d2) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRLD, 3, {YMMREG,YMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42201, 544},
    /* 6454 : VPSRLD zmmreg|mask|z,zmmreg*,xmmrm128 [rvm:m128: evex.nds.512.66.0f.w0 d2 /r       ] AVX512,FUTURE */
        /* 241(f1 7d 48) 317 001(d2) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSRLD, 3, {ZMMREG,ZMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42219, 545},
    /* 6455 : VPSRLD xmmreg|mask|z,xmmrm128|b32*,imm8 [vmi:fv:   evex.nds.128.66.0f.w0 72 /2 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 7d 08) 301 001(72) 212 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRLD, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20973, 544},
    /* 6456 : VPSRLD ymmreg|mask|z,ymmrm256|b32*,imm8 [vmi:fv:   evex.nds.256.66.0f.w0 72 /2 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 7d 28) 301 001(72) 212 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRLD, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20993, 544},
    /* 6457 : VPSRLD zmmreg|mask|z,zmmrm512|b32*,imm8 [vmi:fv:   evex.nds.512.66.0f.w0 72 /2 ib    ] AVX512,FUTURE */
        /* 240(f1 7d 48) 301 001(72) 212 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSRLD, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+21013, 545},
    /* 6458 : VPSRLDQ xmmreg,xmmrm128*,imm8 [vmi:fvm:  evex.nds.128.66.0f.wig 73 /3 ib   ] AVX512VL,AVX512BW,FUTURE */
        /* 240(f1 7d 08) 303 001(73) 213 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRLDQ, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+21033, 556},
    /* 6459 : VPSRLDQ ymmreg,ymmrm256*,imm8 [vmi:fvm:  evex.nds.256.66.0f.wig 73 /3 ib   ] AVX512VL,AVX512BW,FUTURE */
        /* 240(f1 7d 28) 303 001(73) 213 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRLDQ, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+21053, 556},
    /* 6460 : VPSRLDQ zmmreg,zmmrm512*,imm8 [vmi:fvm:  evex.nds.512.66.0f.wig 73 /3 ib   ] AVX512BW,FUTURE */
        /* 240(f1 7d 48) 303 001(73) 213 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSRLDQ, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+21073, 557},
    /* 6461 : VPSRLQ xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:m128: evex.nds.128.66.0f.w1 d3 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 317 001(d3) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRLQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42237, 544},
    /* 6462 : VPSRLQ ymmreg|mask|z,ymmreg*,xmmrm128 [rvm:m128: evex.nds.256.66.0f.w1 d3 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 317 001(d3) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRLQ, 3, {YMMREG,YMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42255, 544},
    /* 6463 : VPSRLQ zmmreg|mask|z,zmmreg*,xmmrm128 [rvm:m128: evex.nds.512.66.0f.w1 d3 /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 317 001(d3) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSRLQ, 3, {ZMMREG,ZMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42273, 545},
    /* 6464 : VPSRLQ xmmreg|mask|z,xmmrm128|b64*,imm8 [vmi:fv:   evex.nds.128.66.0f.w1 73 /2 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 fd 08) 301 001(73) 212 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRLQ, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+21093, 544},
    /* 6465 : VPSRLQ ymmreg|mask|z,ymmrm256|b64*,imm8 [vmi:fv:   evex.nds.256.66.0f.w1 73 /2 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 fd 28) 301 001(73) 212 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRLQ, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+21113, 544},
    /* 6466 : VPSRLQ zmmreg|mask|z,zmmrm512|b64*,imm8 [vmi:fv:   evex.nds.512.66.0f.w1 73 /2 ib    ] AVX512,FUTURE */
        /* 240(f1 fd 48) 301 001(73) 212 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSRLQ, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+21133, 545},
    /* 6467 : VPSRLVD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 45 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(45) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRLVD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+42291, 544},
    /* 6468 : VPSRLVD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 45 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(45) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRLVD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+42309, 544},
    /* 6469 : VPSRLVD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 45 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(45) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSRLVD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+42327, 545},
    /* 6470 : VPSRLVQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 45 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(45) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRLVQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+42345, 544},
    /* 6471 : VPSRLVQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 45 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(45) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRLVQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+42363, 544},
    /* 6472 : VPSRLVQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 45 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(45) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSRLVQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+42381, 545},
    /* 6473 : VPSRLVW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w1 10 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 08) 303 001(10) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRLVW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42399, 550},
    /* 6474 : VPSRLVW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w1 10 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 28) 303 001(10) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRLVW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42417, 550},
    /* 6475 : VPSRLVW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w1 10 /r     ] AVX512BW,FUTURE */
        /* 241(f2 fd 48) 303 001(10) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSRLVW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42435, 551},
    /* 6476 : VPSRLW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:m128: evex.nds.128.66.0f.wig d1 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 317 001(d1) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRLW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42453, 556},
    /* 6477 : VPSRLW ymmreg|mask|z,ymmreg*,xmmrm128 [rvm:m128: evex.nds.256.66.0f.wig d1 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 317 001(d1) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRLW, 3, {YMMREG,YMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42471, 556},
    /* 6478 : VPSRLW zmmreg|mask|z,zmmreg*,xmmrm128 [rvm:m128: evex.nds.512.66.0f.wig d1 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 317 001(d1) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSRLW, 3, {ZMMREG,ZMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42489, 557},
    /* 6479 : VPSRLW xmmreg|mask|z,xmmrm128*,imm8 [vmi:fvm:  evex.nds.128.66.0f.wig 71 /2 ib   ] AVX512VL,AVX512BW,FUTURE */
        /* 240(f1 7d 08) 303 001(71) 212 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRLW, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+21153, 556},
    /* 6480 : VPSRLW ymmreg|mask|z,ymmrm256*,imm8 [vmi:fvm:  evex.nds.256.66.0f.wig 71 /2 ib   ] AVX512VL,AVX512BW,FUTURE */
        /* 240(f1 7d 28) 303 001(71) 212 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRLW, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+21173, 556},
    /* 6481 : VPSRLW zmmreg|mask|z,zmmrm512*,imm8 [vmi:fvm:  evex.nds.512.66.0f.wig 71 /2 ib   ] AVX512BW,FUTURE */
        /* 240(f1 7d 48) 303 001(71) 212 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSRLW, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+21193, 557},
    /* 6482 : VPSUBB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig f8 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(f8) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSUBB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42507, 556},
    /* 6483 : VPSUBB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig f8 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(f8) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSUBB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42525, 556},
    /* 6484 : VPSUBB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig f8 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(f8) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSUBB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42543, 557},
    /* 6485 : VPSUBD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f.w0 fa /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 08) 301 001(fa) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSUBD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+42561, 544},
    /* 6486 : VPSUBD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f.w0 fa /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 28) 301 001(fa) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSUBD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+42579, 544},
    /* 6487 : VPSUBD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f.w0 fa /r       ] AVX512,FUTURE */
        /* 241(f1 7d 48) 301 001(fa) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSUBD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+42597, 545},
    /* 6488 : VPSUBQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 fb /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(fb) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSUBQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+42615, 544},
    /* 6489 : VPSUBQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 fb /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(fb) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSUBQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+42633, 544},
    /* 6490 : VPSUBQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 fb /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(fb) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSUBQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+42651, 545},
    /* 6491 : VPSUBSB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig e8 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(e8) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSUBSB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42669, 556},
    /* 6492 : VPSUBSB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig e8 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(e8) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSUBSB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42687, 556},
    /* 6493 : VPSUBSB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig e8 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(e8) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSUBSB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42705, 557},
    /* 6494 : VPSUBSW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig e9 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(e9) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSUBSW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42723, 556},
    /* 6495 : VPSUBSW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig e9 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(e9) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSUBSW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42741, 556},
    /* 6496 : VPSUBSW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig e9 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(e9) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSUBSW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42759, 557},
    /* 6497 : VPSUBUSB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig d8 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(d8) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSUBUSB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42777, 556},
    /* 6498 : VPSUBUSB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig d8 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(d8) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSUBUSB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42795, 556},
    /* 6499 : VPSUBUSB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig d8 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(d8) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSUBUSB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42813, 557},
    /* 6500 : VPSUBUSW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig d9 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(d9) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSUBUSW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42831, 556},
    /* 6501 : VPSUBUSW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig d9 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(d9) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSUBUSW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42849, 556},
    /* 6502 : VPSUBUSW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig d9 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(d9) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSUBUSW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42867, 557},
    /* 6503 : VPSUBW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig f9 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(f9) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSUBW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42885, 556},
    /* 6504 : VPSUBW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig f9 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(f9) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSUBW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42903, 556},
    /* 6505 : VPSUBW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig f9 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(f9) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSUBW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42921, 557},
    /* 6506 : VPTERNLOGD xmmreg|mask|z,xmmreg,xmmrm128|b32,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w0 25 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(25) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPTERNLOGD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+21213, 544},
    /* 6507 : VPTERNLOGD ymmreg|mask|z,ymmreg,ymmrm256|b32,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w0 25 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(25) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPTERNLOGD, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+21223, 544},
    /* 6508 : VPTERNLOGD zmmreg|mask|z,zmmreg,zmmrm512|b32,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w0 25 /r ib  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(25) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VPTERNLOGD, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+21233, 545},
    /* 6509 : VPTERNLOGQ xmmreg|mask|z,xmmreg,xmmrm128|b64,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w1 25 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(25) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPTERNLOGQ, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+21243, 544},
    /* 6510 : VPTERNLOGQ ymmreg|mask|z,ymmreg,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w1 25 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(25) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPTERNLOGQ, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+21253, 544},
    /* 6511 : VPTERNLOGQ zmmreg|mask|z,zmmreg,zmmrm512|b64,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w1 25 /r ib  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(25) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VPTERNLOGQ, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+21263, 545},
    /* 6512 : VPTESTMB kreg|mask,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w0 26 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 08) 303 001(26) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPTESTMB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+42939, 550},
    /* 6513 : VPTESTMB kreg|mask,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w0 26 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 28) 303 001(26) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPTESTMB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+42948, 550},
    /* 6514 : VPTESTMB kreg|mask,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w0 26 /r     ] AVX512BW,FUTURE */
        /* 241(f2 7d 48) 303 001(26) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPTESTMB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+42957, 551},
    /* 6515 : VPTESTMD kreg|mask,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 27 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(27) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPTESTMD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+42966, 544},
    /* 6516 : VPTESTMD kreg|mask,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 27 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(27) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPTESTMD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+42975, 544},
    /* 6517 : VPTESTMD kreg|mask,zmmreg,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 27 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(27) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPTESTMD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+42984, 545},
    /* 6518 : VPTESTMQ kreg|mask,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 27 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(27) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPTESTMQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+42993, 544},
    /* 6519 : VPTESTMQ kreg|mask,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 27 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(27) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPTESTMQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+43002, 544},
    /* 6520 : VPTESTMQ kreg|mask,zmmreg,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 27 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(27) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPTESTMQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+43011, 545},
    /* 6521 : VPTESTMW kreg|mask,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w1 26 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 08) 303 001(26) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPTESTMW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+43020, 550},
    /* 6522 : VPTESTMW kreg|mask,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w1 26 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 28) 303 001(26) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPTESTMW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+43029, 550},
    /* 6523 : VPTESTMW kreg|mask,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w1 26 /r     ] AVX512BW,FUTURE */
        /* 241(f2 fd 48) 303 001(26) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPTESTMW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+43038, 551},
    /* 6524 : VPTESTNMB kreg|mask,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.f3.0f38.w0 26 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7e 08) 303 001(26) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPTESTNMB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+43047, 550},
    /* 6525 : VPTESTNMB kreg|mask,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.f3.0f38.w0 26 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7e 28) 303 001(26) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPTESTNMB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+43056, 550},
    /* 6526 : VPTESTNMB kreg|mask,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.f3.0f38.w0 26 /r     ] AVX512BW,FUTURE */
        /* 241(f2 7e 48) 303 001(26) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPTESTNMB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+43065, 551},
    /* 6527 : VPTESTNMD kreg|mask,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.f3.0f38.w0 27 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7e 08) 301 001(27) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPTESTNMD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+43074, 544},
    /* 6528 : VPTESTNMD kreg|mask,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.f3.0f38.w0 27 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7e 28) 301 001(27) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPTESTNMD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+43083, 544},
    /* 6529 : VPTESTNMD kreg|mask,zmmreg,zmmrm512|b32 [rvm:fv:   evex.nds.512.f3.0f38.w0 27 /r     ] AVX512,FUTURE */
        /* 241(f2 7e 48) 301 001(27) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPTESTNMD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+43092, 545},
    /* 6530 : VPTESTNMQ kreg|mask,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.f3.0f38.w1 27 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fe 08) 301 001(27) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPTESTNMQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+43101, 544},
    /* 6531 : VPTESTNMQ kreg|mask,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.f3.0f38.w1 27 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fe 28) 301 001(27) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPTESTNMQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+43110, 544},
    /* 6532 : VPTESTNMQ kreg|mask,zmmreg,zmmrm512|b64 [rvm:fv:   evex.nds.512.f3.0f38.w1 27 /r     ] AVX512,FUTURE */
        /* 241(f2 fe 48) 301 001(27) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPTESTNMQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+43119, 545},
    /* 6533 : VPTESTNMW kreg|mask,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.f3.0f38.w1 26 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fe 08) 303 001(26) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPTESTNMW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+43128, 550},
    /* 6534 : VPTESTNMW kreg|mask,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.f3.0f38.w1 26 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fe 28) 303 001(26) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPTESTNMW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+43137, 550},
    /* 6535 : VPTESTNMW kreg|mask,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.f3.0f38.w1 26 /r     ] AVX512BW,FUTURE */
        /* 241(f2 fe 48) 303 001(26) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPTESTNMW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+43146, 551},
    /* 6536 : VPUNPCKHBW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig 68 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(68) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPUNPCKHBW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43155, 556},
    /* 6537 : VPUNPCKHBW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig 68 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(68) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPUNPCKHBW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43173, 556},
    /* 6538 : VPUNPCKHBW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig 68 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(68) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPUNPCKHBW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43191, 557},
    /* 6539 : VPUNPCKHDQ xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f.w0 6a /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 08) 301 001(6a) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPUNPCKHDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+43209, 544},
    /* 6540 : VPUNPCKHDQ ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f.w0 6a /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 28) 301 001(6a) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPUNPCKHDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+43227, 544},
    /* 6541 : VPUNPCKHDQ zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f.w0 6a /r       ] AVX512,FUTURE */
        /* 241(f1 7d 48) 301 001(6a) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPUNPCKHDQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+43245, 545},
    /* 6542 : VPUNPCKHQDQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 6d /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(6d) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPUNPCKHQDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+43263, 544},
    /* 6543 : VPUNPCKHQDQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 6d /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(6d) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPUNPCKHQDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+43281, 544},
    /* 6544 : VPUNPCKHQDQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 6d /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(6d) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPUNPCKHQDQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+43299, 545},
    /* 6545 : VPUNPCKHWD xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig 69 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(69) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPUNPCKHWD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43317, 556},
    /* 6546 : VPUNPCKHWD ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig 69 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(69) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPUNPCKHWD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43335, 556},
    /* 6547 : VPUNPCKHWD zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig 69 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(69) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPUNPCKHWD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43353, 557},
    /* 6548 : VPUNPCKLBW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig 60 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(60) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPUNPCKLBW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43371, 556},
    /* 6549 : VPUNPCKLBW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig 60 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(60) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPUNPCKLBW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43389, 556},
    /* 6550 : VPUNPCKLBW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig 60 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(60) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPUNPCKLBW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43407, 557},
    /* 6551 : VPUNPCKLDQ xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f.w0 62 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 08) 301 001(62) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPUNPCKLDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+43425, 544},
    /* 6552 : VPUNPCKLDQ ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f.w0 62 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 28) 301 001(62) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPUNPCKLDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+43443, 544},
    /* 6553 : VPUNPCKLDQ zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f.w0 62 /r       ] AVX512,FUTURE */
        /* 241(f1 7d 48) 301 001(62) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPUNPCKLDQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+43461, 545},
    /* 6554 : VPUNPCKLQDQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 6c /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(6c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPUNPCKLQDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+43479, 544},
    /* 6555 : VPUNPCKLQDQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 6c /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(6c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPUNPCKLQDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+43497, 544},
    /* 6556 : VPUNPCKLQDQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 6c /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(6c) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPUNPCKLQDQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+43515, 545},
    /* 6557 : VPUNPCKLWD xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig 61 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(61) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPUNPCKLWD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43533, 556},
    /* 6558 : VPUNPCKLWD ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig 61 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(61) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPUNPCKLWD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43551, 556},
    /* 6559 : VPUNPCKLWD zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig 61 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(61) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPUNPCKLWD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43569, 557},
    /* 6560 : VPXORD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f.w0 ef /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 08) 301 001(ef) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPXORD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+43587, 544},
    /* 6561 : VPXORD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f.w0 ef /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 28) 301 001(ef) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPXORD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+43605, 544},
    /* 6562 : VPXORD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f.w0 ef /r       ] AVX512,FUTURE */
        /* 241(f1 7d 48) 301 001(ef) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPXORD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+43623, 545},
    /* 6563 : VPXORQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 ef /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(ef) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPXORQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+43641, 544},
    /* 6564 : VPXORQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 ef /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(ef) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPXORQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+43659, 544},
    /* 6565 : VPXORQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 ef /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(ef) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPXORQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+43677, 545},
    /* 6566 : VRANGEPD xmmreg|mask|z,xmmreg*,xmmrm128|b64,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w1 50 /r ib  ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f3 fd 08) 301 001(50) 120 023 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VRANGEPD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+21273, 547},
    /* 6567 : VRANGEPD ymmreg|mask|z,ymmreg*,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w1 50 /r ib  ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f3 fd 28) 301 001(50) 120 023 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VRANGEPD, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+21293, 547},
    /* 6568 : VRANGEPD zmmreg|mask|z,zmmreg*,zmmrm512|b64|sae,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w1 50 /r ib  ] AVX512DQ,FUTURE */
        /* 241(f3 fd 48) 301 001(50) 120 023 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VRANGEPD, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64|SAE,0,0}, nasm_bytecodes+21313, 548},
    /* 6569 : VRANGEPS xmmreg|mask|z,xmmreg*,xmmrm128|b32,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w0 50 /r ib  ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f3 7d 08) 301 001(50) 120 023 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VRANGEPS, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+21333, 547},
    /* 6570 : VRANGEPS ymmreg|mask|z,ymmreg*,ymmrm256|b32,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w0 50 /r ib  ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f3 7d 28) 301 001(50) 120 023 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VRANGEPS, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+21353, 547},
    /* 6571 : VRANGEPS zmmreg|mask|z,zmmreg*,zmmrm512|b32|sae,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w0 50 /r ib  ] AVX512DQ,FUTURE */
        /* 241(f3 7d 48) 301 001(50) 120 023 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VRANGEPS, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32|SAE,0,0}, nasm_bytecodes+21373, 548},
    /* 6572 : VRANGESD xmmreg|mask|z,xmmreg*,xmmrm64|sae,imm8 [rvmi:t1s: evex.nds.lig.66.0f3a.w1 51 /r ib  ] AVX512DQ,FUTURE */
        /* 241(f3 fd 08) 306 001(51) 120 023 : LIG,EVEX,PROT,AVX512DQ,FUTURE */
        {I_VRANGESD, 4, {XMMREG,XMMREG,RM_XMM|BITS64,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+21393, 554},
    /* 6573 : VRANGESS xmmreg|mask|z,xmmreg*,xmmrm32|sae,imm8 [rvmi:t1s: evex.nds.lig.66.0f3a.w0 51 /r ib  ] AVX512DQ,FUTURE */
        /* 241(f3 7d 08) 306 001(51) 120 023 : LIG,EVEX,PROT,AVX512DQ,FUTURE */
        {I_VRANGESS, 4, {XMMREG,XMMREG,RM_XMM|BITS32,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+21413, 554},
    /* 6574 : VRCP14PD xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.0f38.w1 4c /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 301 001(4c) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VRCP14PD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+43695, 544},
    /* 6575 : VRCP14PD ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.0f38.w1 4c /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 301 001(4c) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VRCP14PD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+43704, 544},
    /* 6576 : VRCP14PD zmmreg|mask|z,zmmrm512|b64 [rm:fv:    evex.512.66.0f38.w1 4c /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 301 001(4c) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VRCP14PD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+43713, 545},
    /* 6577 : VRCP14PS xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.66.0f38.w0 4c /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 301 001(4c) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VRCP14PS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+43722, 544},
    /* 6578 : VRCP14PS ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.66.0f38.w0 4c /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 301 001(4c) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VRCP14PS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+43731, 544},
    /* 6579 : VRCP14PS zmmreg|mask|z,zmmrm512|b32 [rm:fv:    evex.512.66.0f38.w0 4c /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 301 001(4c) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VRCP14PS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+43740, 545},
    /* 6580 : VRCP14SD xmmreg|mask|z,xmmreg*,xmmrm64 [rvm:t1s:  evex.nds.lig.66.0f38.w1 4d /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(4d) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VRCP14SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43749, 546},
    /* 6581 : VRCP14SS xmmreg|mask|z,xmmreg*,xmmrm32 [rvm:t1s:  evex.nds.lig.66.0f38.w0 4d /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(4d) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VRCP14SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43767, 546},
    /* 6582 : VRCP28PD zmmreg|mask|z,zmmrm512|b64|sae [rm:fv:    evex.512.66.0f38.w1 ca /r         ] AVX512ER,FUTURE */
        /* 250(f2 fd 48) 301 001(ca) 110 : EVEX,PROT,AVX512ER,FUTURE */
        {I_VRCP28PD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+43785, 552},
    /* 6583 : VRCP28PS zmmreg|mask|z,zmmrm512|b32|sae [rm:fv:    evex.512.66.0f38.w0 ca /r         ] AVX512ER,FUTURE */
        /* 250(f2 7d 48) 301 001(ca) 110 : EVEX,PROT,AVX512ER,FUTURE */
        {I_VRCP28PS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+43794, 552},
    /* 6584 : VRCP28SD xmmreg|mask|z,xmmreg*,xmmrm64|sae [rvm:t1s:  evex.nds.lig.66.0f38.w1 cb /r     ] AVX512ER,FUTURE */
        /* 241(f2 fd 08) 306 001(cb) 120 : LIG,EVEX,PROT,AVX512ER,FUTURE */
        {I_VRCP28SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+43803, 565},
    /* 6585 : VRCP28SS xmmreg|mask|z,xmmreg*,xmmrm32|sae [rvm:t1s:  evex.nds.lig.66.0f38.w0 cb /r     ] AVX512ER,FUTURE */
        /* 241(f2 7d 08) 306 001(cb) 120 : LIG,EVEX,PROT,AVX512ER,FUTURE */
        {I_VRCP28SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+43821, 565},
    /* 6586 : VREDUCEPD xmmreg|mask|z,xmmrm128|b64,imm8 [rmi:fv:   evex.128.66.0f3a.w1 56 /r ib      ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f3 fd 08) 301 001(56) 110 022 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VREDUCEPD, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+21433, 547},
    /* 6587 : VREDUCEPD ymmreg|mask|z,ymmrm256|b64,imm8 [rmi:fv:   evex.256.66.0f3a.w1 56 /r ib      ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f3 fd 28) 301 001(56) 110 022 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VREDUCEPD, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+21443, 547},
    /* 6588 : VREDUCEPD zmmreg|mask|z,zmmrm512|b64|sae,imm8 [rmi:fv:   evex.512.66.0f3a.w1 56 /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 fd 48) 301 001(56) 110 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VREDUCEPD, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+21453, 548},
    /* 6589 : VREDUCEPS xmmreg|mask|z,xmmrm128|b32,imm8 [rmi:fv:   evex.128.66.0f3a.w0 56 /r ib      ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f3 7d 08) 301 001(56) 110 022 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VREDUCEPS, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+21463, 547},
    /* 6590 : VREDUCEPS ymmreg|mask|z,ymmrm256|b32,imm8 [rmi:fv:   evex.256.66.0f3a.w0 56 /r ib      ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f3 7d 28) 301 001(56) 110 022 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VREDUCEPS, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+21473, 547},
    /* 6591 : VREDUCEPS zmmreg|mask|z,zmmrm512|b32|sae,imm8 [rmi:fv:   evex.512.66.0f3a.w0 56 /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 7d 48) 301 001(56) 110 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VREDUCEPS, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+21483, 548},
    /* 6592 : VREDUCESD xmmreg|mask|z,xmmreg*,xmmrm64|sae,imm8 [rvmi:t1s: evex.nds.lig.66.0f3a.w1 57 /r ib  ] AVX512DQ,FUTURE */
        /* 241(f3 fd 08) 306 001(57) 120 023 : LIG,EVEX,PROT,AVX512DQ,FUTURE */
        {I_VREDUCESD, 4, {XMMREG,XMMREG,RM_XMM|BITS64,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+21493, 554},
    /* 6593 : VREDUCESS xmmreg|mask|z,xmmreg*,xmmrm32|sae,imm8 [rvmi:t1s: evex.nds.lig.66.0f3a.w0 57 /r ib  ] AVX512DQ,FUTURE */
        /* 241(f3 7d 08) 306 001(57) 120 023 : LIG,EVEX,PROT,AVX512DQ,FUTURE */
        {I_VREDUCESS, 4, {XMMREG,XMMREG,RM_XMM|BITS32,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+21513, 554},
    /* 6594 : VRNDSCALEPD xmmreg|mask|z,xmmrm128|b64,imm8 [rmi:fv:   evex.128.66.0f3a.w1 09 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 fd 08) 301 001(09) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VRNDSCALEPD, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+21533, 544},
    /* 6595 : VRNDSCALEPD ymmreg|mask|z,ymmrm256|b64,imm8 [rmi:fv:   evex.256.66.0f3a.w1 09 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 fd 28) 301 001(09) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VRNDSCALEPD, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+21543, 544},
    /* 6596 : VRNDSCALEPD zmmreg|mask|z,zmmrm512|b64|sae,imm8 [rmi:fv:   evex.512.66.0f3a.w1 09 /r ib      ] AVX512,FUTURE */
        /* 250(f3 fd 48) 301 001(09) 110 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VRNDSCALEPD, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+21553, 545},
    /* 6597 : VRNDSCALEPH xmmreg|mask|z,xmmrm128|b16,imm8 [rmi:fv:   evex.128.np.0f3a.w0 08 /r ib      ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f3 7c 08) 301 001(08) 110 022 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VRNDSCALEPH, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+21563, 566},
    /* 6598 : VRNDSCALEPH ymmreg|mask|z,ymmrm256|b16,imm8 [rmi:fv:   evex.256.np.0f3a.w0 08 /r ib      ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f3 7c 28) 301 001(08) 110 022 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VRNDSCALEPH, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+21573, 566},
    /* 6599 : VRNDSCALEPH zmmreg|mask|z,zmmrm512|b16|sae,imm8 [rmi:fv:   evex.512.np.0f3a.w0 08 /r ib      ] AVX512FP16,FUTURE */
        /* 250(f3 7c 48) 301 001(08) 110 022 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VRNDSCALEPH, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+21583, 567},
    /* 6600 : VRNDSCALEPS xmmreg|mask|z,xmmrm128|b32,imm8 [rmi:fv:   evex.128.66.0f3a.w0 08 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 08) 301 001(08) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VRNDSCALEPS, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+21593, 544},
    /* 6601 : VRNDSCALEPS ymmreg|mask|z,ymmrm256|b32,imm8 [rmi:fv:   evex.256.66.0f3a.w0 08 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 28) 301 001(08) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VRNDSCALEPS, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+21603, 544},
    /* 6602 : VRNDSCALEPS zmmreg|mask|z,zmmrm512|b32|sae,imm8 [rmi:fv:   evex.512.66.0f3a.w0 08 /r ib      ] AVX512,FUTURE */
        /* 250(f3 7d 48) 301 001(08) 110 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VRNDSCALEPS, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+21613, 545},
    /* 6603 : VRNDSCALESD xmmreg|mask|z,xmmreg*,xmmrm64|sae,imm8 [rvmi:t1s: evex.nds.lig.66.0f3a.w1 0b /r ib  ] AVX512,FUTURE */
        /* 241(f3 fd 08) 306 001(0b) 120 023 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VRNDSCALESD, 4, {XMMREG,XMMREG,RM_XMM|BITS64,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+21623, 546},
    /* 6604 : VRNDSCALESH xmmreg|mask|z,xmmreg*,xmmrm16|sae,imm8 [rvmi:t1s: evex.nds.lig.np.0f3a.w0 0a /r ib  ] AVX512FP16,FUTURE */
        /* 241(f3 7c 08) 306 001(0a) 120 023 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VRNDSCALESH, 4, {XMMREG,XMMREG,RM_XMM|BITS16,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+21643, 568},
    /* 6605 : VRNDSCALESS xmmreg|mask|z,xmmreg*,xmmrm32|sae,imm8 [rvmi:t1s: evex.nds.lig.66.0f3a.w0 0a /r ib  ] AVX512,FUTURE */
        /* 241(f3 7d 08) 306 001(0a) 120 023 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VRNDSCALESS, 4, {XMMREG,XMMREG,RM_XMM|BITS32,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+21663, 546},
    /* 6606 : VRSQRT14PD xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.0f38.w1 4e /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 301 001(4e) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VRSQRT14PD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+43839, 544},
    /* 6607 : VRSQRT14PD ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.0f38.w1 4e /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 301 001(4e) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VRSQRT14PD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+43848, 544},
    /* 6608 : VRSQRT14PD zmmreg|mask|z,zmmrm512|b64 [rm:fv:    evex.512.66.0f38.w1 4e /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 301 001(4e) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VRSQRT14PD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+43857, 545},
    /* 6609 : VRSQRT14PS xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.66.0f38.w0 4e /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 301 001(4e) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VRSQRT14PS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+43866, 544},
    /* 6610 : VRSQRT14PS ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.66.0f38.w0 4e /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 301 001(4e) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VRSQRT14PS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+43875, 544},
    /* 6611 : VRSQRT14PS zmmreg|mask|z,zmmrm512|b32 [rm:fv:    evex.512.66.0f38.w0 4e /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 301 001(4e) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VRSQRT14PS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+43884, 545},
    /* 6612 : VRSQRT14SD xmmreg|mask|z,xmmreg*,xmmrm64 [rvm:t1s:  evex.nds.lig.66.0f38.w1 4f /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(4f) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VRSQRT14SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43893, 546},
    /* 6613 : VRSQRT14SS xmmreg|mask|z,xmmreg*,xmmrm32 [rvm:t1s:  evex.nds.lig.66.0f38.w0 4f /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(4f) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VRSQRT14SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43911, 546},
    /* 6614 : VRSQRT28PD zmmreg|mask|z,zmmrm512|b64|sae [rm:fv:    evex.512.66.0f38.w1 cc /r         ] AVX512ER,FUTURE */
        /* 250(f2 fd 48) 301 001(cc) 110 : EVEX,PROT,AVX512ER,FUTURE */
        {I_VRSQRT28PD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+43929, 552},
    /* 6615 : VRSQRT28PS zmmreg|mask|z,zmmrm512|b32|sae [rm:fv:    evex.512.66.0f38.w0 cc /r         ] AVX512ER,FUTURE */
        /* 250(f2 7d 48) 301 001(cc) 110 : EVEX,PROT,AVX512ER,FUTURE */
        {I_VRSQRT28PS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+43938, 552},
    /* 6616 : VRSQRT28SD xmmreg|mask|z,xmmreg*,xmmrm64|sae [rvm:t1s:  evex.nds.lig.66.0f38.w1 cd /r     ] AVX512ER,FUTURE */
        /* 241(f2 fd 08) 306 001(cd) 120 : LIG,EVEX,PROT,AVX512ER,FUTURE */
        {I_VRSQRT28SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+43947, 565},
    /* 6617 : VRSQRT28SS xmmreg|mask|z,xmmreg*,xmmrm32|sae [rvm:t1s:  evex.nds.lig.66.0f38.w0 cd /r     ] AVX512ER,FUTURE */
        /* 241(f2 7d 08) 306 001(cd) 120 : LIG,EVEX,PROT,AVX512ER,FUTURE */
        {I_VRSQRT28SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+43965, 565},
    /* 6618 : VSCALEFPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 2c /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(2c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSCALEFPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+43983, 544},
    /* 6619 : VSCALEFPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 2c /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(2c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSCALEFPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+44001, 544},
    /* 6620 : VSCALEFPD zmmreg|mask|z,zmmreg*,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 2c /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(2c) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VSCALEFPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+44019, 545},
    /* 6621 : VSCALEFPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 2c /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(2c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSCALEFPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+44037, 544},
    /* 6622 : VSCALEFPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 2c /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(2c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSCALEFPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+44055, 544},
    /* 6623 : VSCALEFPS zmmreg|mask|z,zmmreg*,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 2c /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(2c) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VSCALEFPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+44073, 545},
    /* 6624 : VSCALEFSD xmmreg|mask|z,xmmreg*,xmmrm64|er [rvm:t1s:  evex.nds.lig.66.0f38.w1 2d /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(2d) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VSCALEFSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+44091, 546},
    /* 6625 : VSCALEFSS xmmreg|mask|z,xmmreg*,xmmrm32|er [rvm:t1s:  evex.nds.lig.66.0f38.w0 2d /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(2d) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VSCALEFSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+44109, 546},
    /* 6626 : VSCATTERDPD xmem64|mask,xmmreg [mr:t1s:   vsibx evex.128.66.0f38.w1 a2 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 fd 08) 306 001(a2) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSCATTERDPD, 2, {XMEM|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21683, 544},
    /* 6627 : VSCATTERDPD xmem64|mask,ymmreg [mr:t1s:   vsibx evex.256.66.0f38.w1 a2 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 fd 28) 306 001(a2) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSCATTERDPD, 2, {XMEM|BITS64,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21693, 544},
    /* 6628 : VSCATTERDPD ymem64|mask,zmmreg [mr:t1s:   vsiby evex.512.66.0f38.w1 a2 /r   ] AVX512,FUTURE */
        /* 375 250(f2 fd 48) 306 001(a2) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VSCATTERDPD, 2, {YMEM|BITS64,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21703, 545},
    /* 6629 : VSCATTERDPS xmem32|mask,xmmreg [mr:t1s:   vsibx evex.128.66.0f38.w0 a2 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 7d 08) 306 001(a2) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSCATTERDPS, 2, {XMEM|BITS32,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21713, 544},
    /* 6630 : VSCATTERDPS ymem32|mask,ymmreg [mr:t1s:   vsiby evex.256.66.0f38.w0 a2 /r   ] AVX512,AVX512VL,FUTURE */
        /* 375 250(f2 7d 28) 306 001(a2) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSCATTERDPS, 2, {YMEM|BITS32,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21723, 544},
    /* 6631 : VSCATTERDPS zmem32|mask,zmmreg [mr:t1s:   vsibz evex.512.66.0f38.w0 a2 /r   ] AVX512,FUTURE */
        /* 376 250(f2 7d 48) 306 001(a2) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VSCATTERDPS, 2, {ZMEM|BITS32,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21733, 545},
    /* 6632 : VSCATTERPF0DPD ymem64|mask [m:t1s:    vsiby evex.512.66.0f38.w1 c6 /5   ] AVX512PF,FUTURE */
        /* 375 250(f2 fd 48) 306 001(c6) 205 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VSCATTERPF0DPD, 1, {YMEM|BITS64,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21743, 555},
    /* 6633 : VSCATTERPF0DPS zmem32|mask [m:t1s:    vsibz evex.512.66.0f38.w0 c6 /5   ] AVX512PF,FUTURE */
        /* 376 250(f2 7d 48) 306 001(c6) 205 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VSCATTERPF0DPS, 1, {ZMEM|BITS32,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21753, 555},
    /* 6634 : VSCATTERPF0QPD zmem64|mask [m:t1s:    vsibz evex.512.66.0f38.w1 c7 /5   ] AVX512PF,FUTURE */
        /* 376 250(f2 fd 48) 306 001(c7) 205 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VSCATTERPF0QPD, 1, {ZMEM|BITS64,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21763, 555},
    /* 6635 : VSCATTERPF0QPS zmem32|mask [m:t1s:    vsibz evex.512.66.0f38.w0 c7 /5   ] AVX512PF,FUTURE */
        /* 376 250(f2 7d 48) 306 001(c7) 205 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VSCATTERPF0QPS, 1, {ZMEM|BITS32,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21773, 555},
    /* 6636 : VSCATTERPF1DPD ymem64|mask [m:t1s:    vsiby evex.512.66.0f38.w1 c6 /6   ] AVX512PF,FUTURE */
        /* 375 250(f2 fd 48) 306 001(c6) 206 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VSCATTERPF1DPD, 1, {YMEM|BITS64,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21783, 555},
    /* 6637 : VSCATTERPF1DPS zmem32|mask [m:t1s:    vsibz evex.512.66.0f38.w0 c6 /6   ] AVX512PF,FUTURE */
        /* 376 250(f2 7d 48) 306 001(c6) 206 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VSCATTERPF1DPS, 1, {ZMEM|BITS32,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21793, 555},
    /* 6638 : VSCATTERPF1QPD zmem64|mask [m:t1s:    vsibz evex.512.66.0f38.w1 c7 /6   ] AVX512PF,FUTURE */
        /* 376 250(f2 fd 48) 306 001(c7) 206 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VSCATTERPF1QPD, 1, {ZMEM|BITS64,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21803, 555},
    /* 6639 : VSCATTERPF1QPS zmem32|mask [m:t1s:    vsibz evex.512.66.0f38.w0 c7 /6   ] AVX512PF,FUTURE */
        /* 376 250(f2 7d 48) 306 001(c7) 206 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VSCATTERPF1QPS, 1, {ZMEM|BITS32,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21813, 555},
    /* 6640 : VSCATTERQPD xmem64|mask,xmmreg [mr:t1s:   vsibx evex.128.66.0f38.w1 a3 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 fd 08) 306 001(a3) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSCATTERQPD, 2, {XMEM|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21823, 544},
    /* 6641 : VSCATTERQPD ymem64|mask,ymmreg [mr:t1s:   vsiby evex.256.66.0f38.w1 a3 /r   ] AVX512,AVX512VL,FUTURE */
        /* 375 250(f2 fd 28) 306 001(a3) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSCATTERQPD, 2, {YMEM|BITS64,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21833, 544},
    /* 6642 : VSCATTERQPD zmem64|mask,zmmreg [mr:t1s:   vsibz evex.512.66.0f38.w1 a3 /r   ] AVX512,FUTURE */
        /* 376 250(f2 fd 48) 306 001(a3) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VSCATTERQPD, 2, {ZMEM|BITS64,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21843, 545},
    /* 6643 : VSCATTERQPS xmem32|mask,xmmreg [mr:t1s:   vsibx evex.128.66.0f38.w0 a3 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 7d 08) 306 001(a3) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSCATTERQPS, 2, {XMEM|BITS32,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21853, 544},
    /* 6644 : VSCATTERQPS ymem32|mask,xmmreg [mr:t1s:   vsiby evex.256.66.0f38.w0 a3 /r   ] AVX512,AVX512VL,FUTURE */
        /* 375 250(f2 7d 28) 306 001(a3) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSCATTERQPS, 2, {YMEM|BITS32,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21863, 544},
    /* 6645 : VSCATTERQPS zmem32|mask,ymmreg [mr:t1s:   vsibz evex.512.66.0f38.w0 a3 /r   ] AVX512,FUTURE */
        /* 376 250(f2 7d 48) 306 001(a3) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VSCATTERQPS, 2, {ZMEM|BITS32,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21873, 545},
    /* 6646 : VSHUFF32X4 ymmreg|mask|z,ymmreg*,ymmrm256|b32,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w0 23 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(23) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSHUFF32X4, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+21883, 544},
    /* 6647 : VSHUFF32X4 zmmreg|mask|z,zmmreg*,zmmrm512|b32,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w0 23 /r ib  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(23) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VSHUFF32X4, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+21903, 545},
    /* 6648 : VSHUFF64X2 ymmreg|mask|z,ymmreg*,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w1 23 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(23) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSHUFF64X2, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+21923, 544},
    /* 6649 : VSHUFF64X2 zmmreg|mask|z,zmmreg*,zmmrm512|b64,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w1 23 /r ib  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(23) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VSHUFF64X2, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+21943, 545},
    /* 6650 : VSHUFI32X4 ymmreg|mask|z,ymmreg*,ymmrm256|b32,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w0 43 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(43) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSHUFI32X4, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+21963, 544},
    /* 6651 : VSHUFI32X4 zmmreg|mask|z,zmmreg*,zmmrm512|b32,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w0 43 /r ib  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(43) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VSHUFI32X4, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+21983, 545},
    /* 6652 : VSHUFI64X2 ymmreg|mask|z,ymmreg*,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w1 43 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(43) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSHUFI64X2, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22003, 544},
    /* 6653 : VSHUFI64X2 zmmreg|mask|z,zmmreg*,zmmrm512|b64,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w1 43 /r ib  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(43) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VSHUFI64X2, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22023, 545},
    /* 6654 : VSHUFPD xmmreg|mask|z,xmmreg*,xmmrm128|b64,imm8 [rvmi:fv:  evex.nds.128.66.0f.w1 c6 /r ib    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c6) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSHUFPD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22043, 544},
    /* 6655 : VSHUFPD ymmreg|mask|z,ymmreg*,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f.w1 c6 /r ib    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c6) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSHUFPD, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22063, 544},
    /* 6656 : VSHUFPD zmmreg|mask|z,zmmreg*,zmmrm512|b64,imm8 [rvmi:fv:  evex.nds.512.66.0f.w1 c6 /r ib    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c6) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VSHUFPD, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22083, 545},
    /* 6657 : VSHUFPS xmmreg|mask|z,xmmreg*,xmmrm128|b32,imm8 [rvmi:fv:  evex.nds.128.0f.w0 c6 /r ib       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c6) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSHUFPS, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+22103, 544},
    /* 6658 : VSHUFPS ymmreg|mask|z,ymmreg*,ymmrm256|b32,imm8 [rvmi:fv:  evex.nds.256.0f.w0 c6 /r ib       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c6) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSHUFPS, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+22123, 544},
    /* 6659 : VSHUFPS zmmreg|mask|z,zmmreg*,zmmrm512|b32,imm8 [rvmi:fv:  evex.nds.512.0f.w0 c6 /r ib       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c6) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VSHUFPS, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+22143, 545},
    /* 6660 : VSQRTPD xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.0f.w1 51 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 08) 301 001(51) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSQRTPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+44127, 544},
    /* 6661 : VSQRTPD ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.0f.w1 51 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 28) 301 001(51) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSQRTPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+44136, 544},
    /* 6662 : VSQRTPD zmmreg|mask|z,zmmrm512|b64|er [rm:fv:    evex.512.66.0f.w1 51 /r           ] AVX512,FUTURE */
        /* 250(f1 fd 48) 301 001(51) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VSQRTPD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|ER,0,0,0}, nasm_bytecodes+44145, 545},
    /* 6663 : VSQRTPS xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.0f.w0 51 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 08) 301 001(51) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSQRTPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+44154, 544},
    /* 6664 : VSQRTPS ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.0f.w0 51 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 28) 301 001(51) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSQRTPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+44163, 544},
    /* 6665 : VSQRTPS zmmreg|mask|z,zmmrm512|b32|er [rm:fv:    evex.512.0f.w0 51 /r              ] AVX512,FUTURE */
        /* 250(f1 7c 48) 301 001(51) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VSQRTPS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|ER,0,0,0}, nasm_bytecodes+44172, 545},
    /* 6666 : VSQRTSD xmmreg|mask|z,xmmreg*,xmmrm64|er [rvm:t1s:  evex.nds.lig.f2.0f.w1 51 /r       ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(51) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VSQRTSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+44181, 546},
    /* 6667 : VSQRTSS xmmreg|mask|z,xmmreg*,xmmrm32|er [rvm:t1s:  evex.nds.lig.f3.0f.w0 51 /r       ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(51) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VSQRTSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+44199, 546},
    /* 6668 : VSUBPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 5c /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(5c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSUBPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+44217, 544},
    /* 6669 : VSUBPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 5c /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(5c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSUBPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+44235, 544},
    /* 6670 : VSUBPD zmmreg|mask|z,zmmreg*,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f.w1 5c /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(5c) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VSUBPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+44253, 545},
    /* 6671 : VSUBPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.0f.w0 5c /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(5c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSUBPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+44271, 544},
    /* 6672 : VSUBPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.0f.w0 5c /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(5c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSUBPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+44289, 544},
    /* 6673 : VSUBPS zmmreg|mask|z,zmmreg*,zmmrm512|b32|er [rvm:fv:   evex.nds.512.0f.w0 5c /r          ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(5c) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VSUBPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+44307, 545},
    /* 6674 : VSUBSD xmmreg|mask|z,xmmreg*,xmmrm64|er [rvm:t1s:  evex.nds.lig.f2.0f.w1 5c /r       ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(5c) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VSUBSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+44325, 546},
    /* 6675 : VSUBSS xmmreg|mask|z,xmmreg*,xmmrm32|er [rvm:t1s:  evex.nds.lig.f3.0f.w0 5c /r       ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(5c) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VSUBSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+44343, 546},
    /* 6676 : VUCOMISD xmmreg,xmmrm64|sae [rm:t1s:   evex.lig.66.0f.w1 2e /r           ] FL,AVX512,FUTURE */
        /* 250(f1 fd 08) 306 001(2e) 110 : LIG,FL,EVEX,PROT,AVX512,FUTURE */
        {I_VUCOMISD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+44361, 549},
    /* 6677 : VUCOMISS xmmreg,xmmrm32|sae [rm:t1s:   evex.lig.0f.w0 2e /r              ] FL,AVX512,FUTURE */
        /* 250(f1 7c 08) 306 001(2e) 110 : LIG,FL,EVEX,PROT,AVX512,FUTURE */
        {I_VUCOMISS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+44370, 549},
    /* 6678 : VUNPCKHPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 15 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(15) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VUNPCKHPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+44379, 544},
    /* 6679 : VUNPCKHPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 15 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(15) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VUNPCKHPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+44397, 544},
    /* 6680 : VUNPCKHPD zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 15 /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(15) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VUNPCKHPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+44415, 545},
    /* 6681 : VUNPCKHPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.0f.w0 15 /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(15) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VUNPCKHPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+44433, 544},
    /* 6682 : VUNPCKHPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.0f.w0 15 /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(15) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VUNPCKHPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+44451, 544},
    /* 6683 : VUNPCKHPS zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.0f.w0 15 /r          ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(15) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VUNPCKHPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+44469, 545},
    /* 6684 : VUNPCKLPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 14 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(14) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VUNPCKLPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+44487, 544},
    /* 6685 : VUNPCKLPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 14 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(14) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VUNPCKLPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+44505, 544},
    /* 6686 : VUNPCKLPD zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 14 /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(14) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VUNPCKLPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+44523, 545},
    /* 6687 : VUNPCKLPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.0f.w0 14 /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(14) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VUNPCKLPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+44541, 544},
    /* 6688 : VUNPCKLPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.0f.w0 14 /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(14) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VUNPCKLPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+44559, 544},
    /* 6689 : VUNPCKLPS zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.0f.w0 14 /r          ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(14) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VUNPCKLPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+44577, 545},
    /* 6690 : VXORPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 57 /r       ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 fd 08) 301 001(57) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VXORPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+44595, 547},
    /* 6691 : VXORPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 57 /r       ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 fd 28) 301 001(57) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VXORPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+44613, 547},
    /* 6692 : VXORPD zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 57 /r       ] AVX512DQ,FUTURE */
        /* 241(f1 fd 48) 301 001(57) 120 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VXORPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+44631, 548},
    /* 6693 : VXORPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.0f.w0 57 /r          ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 7c 08) 301 001(57) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VXORPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+44649, 547},
    /* 6694 : VXORPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.0f.w0 57 /r          ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 7c 28) 301 001(57) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VXORPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+44667, 547},
    /* 6695 : VXORPS zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.0f.w0 57 /r          ] AVX512DQ,FUTURE */
        /* 241(f1 7c 48) 301 001(57) 120 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VXORPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+44685, 548},
    /* 6696 : RDPKRU void [          0f 01 ee                          ] LONG,PROT,PKU,X86_64 */
        /* 355 002(01 ee) : LONG,PROT,PKU,X86_64 */
        {I_RDPKRU, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70011, 569},
    /* 6697 : WRPKRU void [          0f 01 ef                          ] LONG,PROT,PKU,X86_64 */
        /* 355 002(01 ef) : LONG,PROT,PKU,X86_64 */
        {I_WRPKRU, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70017, 569},
    /* 6698 : PTWRITE rm32 [m:        f3 o32 0f ae /4                   ] PTWRITE,FUTURE */
        /* 333 321 355 001(ae) 204 : PTWRITE,FUTURE */
        {I_PTWRITE, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+65720, 570},
    /* 6699 : PTWRITE rm64 [m:        f3 o64 0f ae /4                   ] AR0,SX,LONG,PROT,PTWRITE,X86_64 */
        /* 333 324 355 001(ae) 204 : AR0,SX,LONG,PROT,PTWRITE,X86_64 */
        {I_PTWRITE, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+65727, 571},
    /* 6700 : CLDEMOTE mem [m:        np 0f 1c /0                       ] CLDEMOTE,FUTURE */
        /* 360 355 001(1c) 200 : CLDEMOTE,FUTURE */
        {I_CLDEMOTE, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69914, 572},
    /* 6701 : MOVDIRI mem32,reg32 [mr:       np o32 0f38 f9 /r                 ] MOVDIRI,FUTURE */
        /* 360 321 356 001(f9) 101 : NOAPX,MOVDIRI,FUTURE */
        {I_MOVDIRI, 2, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65734, 573},
    /* 6702 : MOVDIRI mem64,reg64 [mr:       np o64 0f38 f9 /r                 ] LONG,PROT,MOVDIRI,X86_64 */
        /* 360 324 356 001(f9) 101 : NOAPX,LONG,PROT,MOVDIRI,X86_64 */
        {I_MOVDIRI, 2, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65741, 574},
    /* 6703 : MOVDIRI mem32,reg32 [mr:       evex.nd0.nf0.np.l0.m4.w0 f9 /r    ] LONG,PROT,APX,MOVDIRI,X86_64 */
        /* 250(f4 7c 08) 300 001(f9) 101 : LONG,EVEX,PROT,APX,MOVDIRI,X86_64 */
        {I_MOVDIRI, 2, {MEMORY|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+44703, 575},
    /* 6704 : MOVDIRI mem64,reg64 [mr:       evex.nd0.nf0.np.l0.m4.w1 f9 /r    ] LONG,PROT,APX,MOVDIRI,X86_64 */
        /* 250(f4 fc 08) 300 001(f9) 101 : LONG,EVEX,PROT,APX,MOVDIRI,X86_64 */
        {I_MOVDIRI, 2, {MEMORY|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+44712, 575},
    /* 6705 : MOVDIR64B reg16,mem512 [rm:       a16 66 0f38 f8 /r                 ] NOLONG,MOVDIR64B,FUTURE */
        /* 310 361 356 001(f8) 110 : NOREX,NOAPX,NOLONG,MOVDIR64B,FUTURE */
        {I_MOVDIR64B, 2, {REG_GPR|BITS16|RN_L16,MEMORY|BITS512|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65748, 576},
    /* 6706 : MOVDIR64B reg32,mem512 [rm:       a32 66 0f38 f8 /r                 ] MOVDIR64B,FUTURE */
        /* 311 361 356 001(f8) 110 : NOAPX,MOVDIR64B,FUTURE */
        {I_MOVDIR64B, 2, {REG_GPR|BITS32|RN_L16,MEMORY|BITS512|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65755, 577},
    /* 6707 : MOVDIR64B reg64,mem512 [rm:       a64 66 0f38 f8 /r                 ] LONG,PROT,MOVDIR64B,X86_64 */
        /* 313 361 356 001(f8) 110 : NOAPX,LONG,PROT,MOVDIR64B,X86_64 */
        {I_MOVDIR64B, 2, {REG_GPR|BITS64|RN_L16,MEMORY|BITS512|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65762, 578},
    /* 6708 : MOVDIR64B reg64,mem512 [rm:       evex.nd0.nf0.66.l0.m4.w1 f8 /r    ] LONG,PROT,APX,MOVDIR64B,X86_64 */
        /* 250(f4 fd 08) 300 001(f8) 110 : LONG,EVEX,PROT,APX,MOVDIR64B,X86_64 */
        {I_MOVDIR64B, 2, {REG_GPR|BITS64,MEMORY|BITS512,0,0,0}, NO_DECORATOR, nasm_bytecodes+44721, 579},
    /* 6709 : PCONFIG void [          np 0f 01 c5                       ] FL,PCONFIG,FUTURE */
        /* 360 355 002(01 c5) : FL,PCONFIG,FUTURE */
        {I_PCONFIG, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69920, 580},
    /* 6710 : PBNDKB void [          np 0f 01 c7                       ] PBNDKB,FUTURE */
        /* 360 355 002(01 c7) : PBNDKB,FUTURE */
        {I_PBNDKB, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69926, 581},
    /* 6711 : PREFETCHRST2 mem8 [m:        0f 18 /4                          ] MOVRS,FUTURE */
        /* 355 001(18) 204 : MOVRS,FUTURE */
        {I_PREFETCHRST2, 1, {MEMORY|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70119, 582},
    /* 6712 : GF2P8AFFINEINVQB xmmreg,xmmrm128,imm8 [rmi:      66 0f3a cf /r ib                  ] SSE,GFNI,FUTURE */
        /* 361 357 001(cf) 110 022 : NOAPX,SSE,GFNI,FUTURE */
        {I_GF2P8AFFINEINVQB, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+65769, 583},
    /* 6713 : VGF2P8AFFINEINVQB xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.66.0f3a.w1 cf /r ib   ] AVX,GFNI,FUTURE */
        /* 261(003 201) 001(cf) 120 023 : NOAPX,VEX,AVX,GFNI,FUTURE */
        {I_VGF2P8AFFINEINVQB, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+54637, 584},
    /* 6714 : VGF2P8AFFINEINVQB ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.66.0f3a.w1 cf /r ib   ] AVX,GFNI,FUTURE */
        /* 261(003 205) 001(cf) 120 023 : NOAPX,VEX,AVX,GFNI,FUTURE */
        {I_VGF2P8AFFINEINVQB, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+54653, 584},
    /* 6715 : VGF2P8AFFINEINVQB xmmreg|mask|z,xmmreg*,xmmrm128|b64,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w1 cf /r ib  ] AVX512VL,GFNI,FUTURE */
        /* 241(f3 fd 08) 301 001(cf) 120 023 : EVEX,PROT,AVX512VL,GFNI,FUTURE */
        {I_VGF2P8AFFINEINVQB, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22163, 585},
    /* 6716 : VGF2P8AFFINEINVQB ymmreg|mask|z,ymmreg*,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w1 cf /r ib  ] AVX512VL,GFNI,FUTURE */
        /* 241(f3 fd 28) 301 001(cf) 120 023 : EVEX,PROT,AVX512VL,GFNI,FUTURE */
        {I_VGF2P8AFFINEINVQB, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22183, 585},
    /* 6717 : VGF2P8AFFINEINVQB zmmreg|mask|z,zmmreg*,zmmrm512|b64,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w1 cf /r ib  ] AVX512,GFNI,FUTURE */
        /* 241(f3 fd 48) 301 001(cf) 120 023 : EVEX,PROT,AVX512,GFNI,FUTURE */
        {I_VGF2P8AFFINEINVQB, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22203, 586},
    /* 6718 : GF2P8AFFINEQB xmmreg,xmmrm128,imm8 [rmi:      66 0f3a ce /r ib                  ] SSE,GFNI,FUTURE */
        /* 361 357 001(ce) 110 022 : NOAPX,SSE,GFNI,FUTURE */
        {I_GF2P8AFFINEQB, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+65776, 583},
    /* 6719 : VGF2P8AFFINEQB xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.66.0f3a.w1 ce /r ib   ] AVX,GFNI,FUTURE */
        /* 261(003 201) 001(ce) 120 023 : NOAPX,VEX,AVX,GFNI,FUTURE */
        {I_VGF2P8AFFINEQB, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+54669, 584},
    /* 6720 : VGF2P8AFFINEQB ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.66.0f3a.w1 ce /r ib   ] AVX,GFNI,FUTURE */
        /* 261(003 205) 001(ce) 120 023 : NOAPX,VEX,AVX,GFNI,FUTURE */
        {I_VGF2P8AFFINEQB, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+54685, 584},
    /* 6721 : VGF2P8AFFINEQB xmmreg|mask|z,xmmreg*,xmmrm128|b64,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w1 ce /r ib  ] AVX512VL,GFNI,FUTURE */
        /* 241(f3 fd 08) 301 001(ce) 120 023 : EVEX,PROT,AVX512VL,GFNI,FUTURE */
        {I_VGF2P8AFFINEQB, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22223, 585},
    /* 6722 : VGF2P8AFFINEQB ymmreg|mask|z,ymmreg*,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w1 ce /r ib  ] AVX512VL,GFNI,FUTURE */
        /* 241(f3 fd 28) 301 001(ce) 120 023 : EVEX,PROT,AVX512VL,GFNI,FUTURE */
        {I_VGF2P8AFFINEQB, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22243, 585},
    /* 6723 : VGF2P8AFFINEQB zmmreg|mask|z,zmmreg*,zmmrm512|b64,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w1 ce /r ib  ] AVX512,GFNI,FUTURE */
        /* 241(f3 fd 48) 301 001(ce) 120 023 : EVEX,PROT,AVX512,GFNI,FUTURE */
        {I_VGF2P8AFFINEQB, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22263, 586},
    /* 6724 : GF2P8MULB xmmreg,xmmrm128 [rm:       66 0f38 cf /r                     ] SSE,GFNI,FUTURE */
        /* 361 356 001(cf) 110 : NOAPX,SSE,GFNI,FUTURE */
        {I_GF2P8MULB, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69932, 583},
    /* 6725 : VGF2P8MULB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38.w0 cf /r      ] AVX,GFNI,FUTURE */
        /* 261(002 001) 001(cf) 120 : NOAPX,VEX,AVX,GFNI,FUTURE */
        {I_VGF2P8MULB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65783, 584},
    /* 6726 : VGF2P8MULB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.w0 cf /r      ] AVX,GFNI,FUTURE */
        /* 261(002 005) 001(cf) 120 : NOAPX,VEX,AVX,GFNI,FUTURE */
        {I_VGF2P8MULB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65797, 584},
    /* 6727 : VGF2P8MULB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w0 cf /r     ] AVX512VL,GFNI,FUTURE */
        /* 241(f2 7d 08) 303 001(cf) 120 : EVEX,PROT,AVX512VL,GFNI,FUTURE */
        {I_VGF2P8MULB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44730, 585},
    /* 6728 : VGF2P8MULB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w0 cf /r     ] AVX512VL,GFNI,FUTURE */
        /* 241(f2 7d 28) 303 001(cf) 120 : EVEX,PROT,AVX512VL,GFNI,FUTURE */
        {I_VGF2P8MULB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44748, 585},
    /* 6729 : VGF2P8MULB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w0 cf /r     ] AVX512,GFNI,FUTURE */
        /* 241(f2 7d 48) 303 001(cf) 120 : EVEX,PROT,AVX512,GFNI,FUTURE */
        {I_VGF2P8MULB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44766, 586},
    /* 6730 : VPCOMPRESSB mem128|mask,xmmreg [mr:t1s8:  evex.128.66.0f38.w0 63 /r         ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 250(f2 7d 08) 304 001(63) 101 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPCOMPRESSB, 2, {MEMORY|BITS128,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+44784, 587},
    /* 6731 : VPCOMPRESSB mem256|mask,ymmreg [mr:t1s8:  evex.256.66.0f38.w0 63 /r         ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 250(f2 7d 28) 304 001(63) 101 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPCOMPRESSB, 2, {MEMORY|BITS256,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+44793, 587},
    /* 6732 : VPCOMPRESSB mem512|mask,zmmreg [mr:t1s8:  evex.512.66.0f38.w0 63 /r         ] AVX512VBMI2,FUTURE */
        /* 250(f2 7d 48) 304 001(63) 101 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPCOMPRESSB, 2, {MEMORY|BITS512,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+44802, 588},
    /* 6733 : VPCOMPRESSB xmmreg|mask|z,xmmreg [mr:       evex.128.66.0f38.w0 63 /r         ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 250(f2 7d 08) 300 001(63) 101 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPCOMPRESSB, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44811, 587},
    /* 6734 : VPCOMPRESSB ymmreg|mask|z,ymmreg [mr:       evex.256.66.0f38.w0 63 /r         ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 250(f2 7d 28) 300 001(63) 101 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPCOMPRESSB, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44820, 587},
    /* 6735 : VPCOMPRESSB zmmreg|mask|z,zmmreg [mr:       evex.512.66.0f38.w0 63 /r         ] AVX512VBMI2,FUTURE */
        /* 250(f2 7d 48) 300 001(63) 101 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPCOMPRESSB, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44829, 588},
    /* 6736 : VPCOMPRESSW mem128|mask,xmmreg [mr:t1s16: evex.128.66.0f38.w1 63 /r         ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 250(f2 fd 08) 305 001(63) 101 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPCOMPRESSW, 2, {MEMORY|BITS128,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+44838, 587},
    /* 6737 : VPCOMPRESSW mem256|mask,ymmreg [mr:t1s16: evex.256.66.0f38.w1 63 /r         ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 250(f2 fd 28) 305 001(63) 101 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPCOMPRESSW, 2, {MEMORY|BITS256,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+44847, 587},
    /* 6738 : VPCOMPRESSW mem512|mask,zmmreg [mr:t1s16: evex.512.66.0f38.w1 63 /r         ] AVX512VBMI2,FUTURE */
        /* 250(f2 fd 48) 305 001(63) 101 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPCOMPRESSW, 2, {MEMORY|BITS512,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+44856, 588},
    /* 6739 : VPCOMPRESSW xmmreg|mask|z,xmmreg [mr:       evex.128.66.0f38.w1 63 /r         ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 250(f2 fd 08) 300 001(63) 101 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPCOMPRESSW, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44865, 587},
    /* 6740 : VPCOMPRESSW ymmreg|mask|z,ymmreg [mr:       evex.256.66.0f38.w1 63 /r         ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 250(f2 fd 28) 300 001(63) 101 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPCOMPRESSW, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44874, 587},
    /* 6741 : VPCOMPRESSW zmmreg|mask|z,zmmreg [mr:       evex.512.66.0f38.w1 63 /r         ] AVX512VBMI2,FUTURE */
        /* 250(f2 fd 48) 300 001(63) 101 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPCOMPRESSW, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44883, 588},
    /* 6742 : VPEXPANDB xmmreg|mask|z,xmmrm128 [rm:t1s8:  evex.128.66.0f38.w0 62 /r         ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 250(f2 7d 08) 304 001(62) 110 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPEXPANDB, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44892, 587},
    /* 6743 : VPEXPANDB ymmreg|mask|z,ymmrm256 [rm:t1s8:  evex.256.66.0f38.w0 62 /r         ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 250(f2 7d 28) 304 001(62) 110 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPEXPANDB, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44901, 587},
    /* 6744 : VPEXPANDB zmmreg|mask|z,zmmrm512 [rm:t1s8:  evex.512.66.0f38.w0 62 /r         ] AVX512VBMI2,FUTURE */
        /* 250(f2 7d 48) 304 001(62) 110 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPEXPANDB, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44910, 588},
    /* 6745 : VPEXPANDW xmmreg|mask|z,xmmrm128 [rm:t1s16: evex.128.66.0f38.w1 62 /r         ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 250(f2 fd 08) 305 001(62) 110 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPEXPANDW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44919, 587},
    /* 6746 : VPEXPANDW ymmreg|mask|z,ymmrm256 [rm:t1s16: evex.256.66.0f38.w1 62 /r         ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 250(f2 fd 28) 305 001(62) 110 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPEXPANDW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44928, 587},
    /* 6747 : VPEXPANDW zmmreg|mask|z,zmmrm512 [rm:t1s16: evex.512.66.0f38.w1 62 /r         ] AVX512VBMI2,FUTURE */
        /* 250(f2 fd 48) 305 001(62) 110 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPEXPANDW, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44937, 588},
    /* 6748 : VPSHLDW xmmreg|mask|z,xmmreg*,xmmrm128,imm8 [rvmi:fvm: evex.nds.128.66.0f3a.w1 70 /r ib  ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f3 fd 08) 303 001(70) 120 023 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHLDW, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+22283, 587},
    /* 6749 : VPSHLDW ymmreg|mask|z,ymmreg*,ymmrm256,imm8 [rvmi:fvm: evex.nds.256.66.0f3a.w1 70 /r ib  ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f3 fd 28) 303 001(70) 120 023 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHLDW, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+22303, 587},
    /* 6750 : VPSHLDW zmmreg|mask|z,zmmreg*,zmmrm512,imm8 [rvmi:fvm: evex.nds.512.66.0f3a.w1 70 /r ib  ] AVX512VBMI2,FUTURE */
        /* 241(f3 fd 48) 303 001(70) 120 023 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPSHLDW, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+22323, 588},
    /* 6751 : VPSHLDD xmmreg|mask|z,xmmreg*,xmmrm128|b32,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w0 71 /r ib  ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f3 7d 08) 301 001(71) 120 023 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHLDD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+22343, 587},
    /* 6752 : VPSHLDD ymmreg|mask|z,ymmreg*,ymmrm256|b32,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w0 71 /r ib  ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f3 7d 28) 301 001(71) 120 023 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHLDD, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+22363, 587},
    /* 6753 : VPSHLDD zmmreg|mask|z,zmmreg*,zmmrm512|b32,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w0 71 /r ib  ] AVX512VBMI2,FUTURE */
        /* 241(f3 7d 48) 301 001(71) 120 023 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPSHLDD, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+22383, 588},
    /* 6754 : VPSHLDQ xmmreg|mask|z,xmmreg*,xmmrm128|b64,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w1 71 /r ib  ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f3 fd 08) 301 001(71) 120 023 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHLDQ, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22403, 587},
    /* 6755 : VPSHLDQ ymmreg|mask|z,ymmreg*,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w1 71 /r ib  ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f3 fd 28) 301 001(71) 120 023 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHLDQ, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22423, 587},
    /* 6756 : VPSHLDQ zmmreg|mask|z,zmmreg*,zmmrm512|b64,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w1 71 /r ib  ] AVX512VBMI2,FUTURE */
        /* 241(f3 fd 48) 301 001(71) 120 023 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPSHLDQ, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22443, 588},
    /* 6757 : VPSHLDVW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm: evex.dds.128.66.0f38.w1 70 /r     ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f2 fd 08) 303 001(70) 120 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHLDVW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44946, 587},
    /* 6758 : VPSHLDVW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm: evex.dds.256.66.0f38.w1 70 /r     ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f2 fd 28) 303 001(70) 120 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHLDVW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44964, 587},
    /* 6759 : VPSHLDVW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm: evex.dds.512.66.0f38.w1 70 /r     ] AVX512VBMI2,FUTURE */
        /* 241(f2 fd 48) 303 001(70) 120 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPSHLDVW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44982, 588},
    /* 6760 : VPSHLDVD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:  evex.dds.128.66.0f38.w0 71 /r     ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f2 7d 08) 301 001(71) 120 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHLDVD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45000, 587},
    /* 6761 : VPSHLDVD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:  evex.dds.256.66.0f38.w0 71 /r     ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f2 7d 28) 301 001(71) 120 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHLDVD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45018, 587},
    /* 6762 : VPSHLDVD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:  evex.dds.512.66.0f38.w0 71 /r     ] AVX512VBMI2,FUTURE */
        /* 241(f2 7d 48) 301 001(71) 120 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPSHLDVD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45036, 588},
    /* 6763 : VPSHLDVQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:  evex.dds.128.66.0f38.w1 71 /r     ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f2 fd 08) 301 001(71) 120 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHLDVQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+45054, 587},
    /* 6764 : VPSHLDVQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:  evex.dds.256.66.0f38.w1 71 /r     ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f2 fd 28) 301 001(71) 120 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHLDVQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+45072, 587},
    /* 6765 : VPSHLDVQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:  evex.dds.512.66.0f38.w1 71 /r     ] AVX512VBMI2,FUTURE */
        /* 241(f2 fd 48) 301 001(71) 120 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPSHLDVQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+45090, 588},
    /* 6766 : VPSHRDW xmmreg|mask|z,xmmreg*,xmmrm128,imm8 [rvmi:fvm: evex.nds.128.66.0f3a.w1 72 /r ib  ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f3 fd 08) 303 001(72) 120 023 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHRDW, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+22463, 587},
    /* 6767 : VPSHRDW ymmreg|mask|z,ymmreg*,ymmrm256,imm8 [rvmi:fvm: evex.nds.256.66.0f3a.w1 72 /r ib  ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f3 fd 28) 303 001(72) 120 023 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHRDW, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+22483, 587},
    /* 6768 : VPSHRDW zmmreg|mask|z,zmmreg*,zmmrm512,imm8 [rvmi:fvm: evex.nds.512.66.0f3a.w1 72 /r ib  ] AVX512VBMI2,FUTURE */
        /* 241(f3 fd 48) 303 001(72) 120 023 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPSHRDW, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+22503, 588},
    /* 6769 : VPSHRDD xmmreg|mask|z,xmmreg*,xmmrm128|b32,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w0 73 /r ib  ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f3 7d 08) 301 001(73) 120 023 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHRDD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+22523, 587},
    /* 6770 : VPSHRDD ymmreg|mask|z,ymmreg*,ymmrm256|b32,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w0 73 /r ib  ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f3 7d 28) 301 001(73) 120 023 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHRDD, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+22543, 587},
    /* 6771 : VPSHRDD zmmreg|mask|z,zmmreg*,zmmrm512|b32,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w0 73 /r ib  ] AVX512VBMI2,FUTURE */
        /* 241(f3 7d 48) 301 001(73) 120 023 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPSHRDD, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+22563, 588},
    /* 6772 : VPSHRDQ xmmreg|mask|z,xmmreg*,xmmrm128|b64,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w1 73 /r ib  ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f3 fd 08) 301 001(73) 120 023 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHRDQ, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22583, 587},
    /* 6773 : VPSHRDQ ymmreg|mask|z,ymmreg*,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w1 73 /r ib  ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f3 fd 28) 301 001(73) 120 023 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHRDQ, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22603, 587},
    /* 6774 : VPSHRDQ zmmreg|mask|z,zmmreg*,zmmrm512|b64,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w1 73 /r ib  ] AVX512VBMI2,FUTURE */
        /* 241(f3 fd 48) 301 001(73) 120 023 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPSHRDQ, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22623, 588},
    /* 6775 : VPSHRDVW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm: evex.dds.128.66.0f38.w1 72 /r     ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f2 fd 08) 303 001(72) 120 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHRDVW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45108, 587},
    /* 6776 : VPSHRDVW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm: evex.dds.256.66.0f38.w1 72 /r     ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f2 fd 28) 303 001(72) 120 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHRDVW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45126, 587},
    /* 6777 : VPSHRDVW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm: evex.dds.512.66.0f38.w1 72 /r     ] AVX512VBMI2,FUTURE */
        /* 241(f2 fd 48) 303 001(72) 120 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPSHRDVW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45144, 588},
    /* 6778 : VPSHRDVD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:  evex.dds.128.66.0f38.w0 73 /r     ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f2 7d 08) 301 001(73) 120 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHRDVD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45162, 587},
    /* 6779 : VPSHRDVD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:  evex.dds.256.66.0f38.w0 73 /r     ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f2 7d 28) 301 001(73) 120 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHRDVD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45180, 587},
    /* 6780 : VPSHRDVD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:  evex.dds.512.66.0f38.w0 73 /r     ] AVX512VBMI2,FUTURE */
        /* 241(f2 7d 48) 301 001(73) 120 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPSHRDVD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45198, 588},
    /* 6781 : VPSHRDVQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:  evex.dds.128.66.0f38.w1 73 /r     ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f2 fd 08) 301 001(73) 120 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHRDVQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+45216, 587},
    /* 6782 : VPSHRDVQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:  evex.dds.256.66.0f38.w1 73 /r     ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f2 fd 28) 301 001(73) 120 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHRDVQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+45234, 587},
    /* 6783 : VPSHRDVQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:  evex.dds.512.66.0f38.w1 73 /r     ] AVX512VBMI2,FUTURE */
        /* 241(f2 fd 48) 301 001(73) 120 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPSHRDVQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+45252, 588},
    /* 6784 : VPDPBUSD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.dds.128.66.0f38.w0 50 /r     ] AVX512VL,AVX512VNNI,FUTURE */
        /* 241(f2 7d 08) 301 001(50) 120 : EVEX,PROT,AVX512VL,AVX512VNNI,FUTURE */
        {I_VPDPBUSD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45270, 589},
    /* 6785 : VPDPBUSD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.dds.256.66.0f38.w0 50 /r     ] AVX512VL,AVX512VNNI,FUTURE */
        /* 241(f2 7d 28) 301 001(50) 120 : EVEX,PROT,AVX512VL,AVX512VNNI,FUTURE */
        {I_VPDPBUSD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45288, 589},
    /* 6786 : VPDPBUSD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.dds.512.66.0f38.w0 50 /r     ] AVX512VNNI,FUTURE */
        /* 241(f2 7d 48) 301 001(50) 120 : EVEX,PROT,AVX512VNNI,FUTURE */
        {I_VPDPBUSD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45306, 590},
    /* 6787 : VPDPBUSDS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.dds.128.66.0f38.w0 51 /r     ] AVX512VL,AVX512VNNI,FUTURE */
        /* 241(f2 7d 08) 301 001(51) 120 : EVEX,PROT,AVX512VL,AVX512VNNI,FUTURE */
        {I_VPDPBUSDS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45324, 589},
    /* 6788 : VPDPBUSDS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.dds.256.66.0f38.w0 51 /r     ] AVX512VL,AVX512VNNI,FUTURE */
        /* 241(f2 7d 28) 301 001(51) 120 : EVEX,PROT,AVX512VL,AVX512VNNI,FUTURE */
        {I_VPDPBUSDS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45342, 589},
    /* 6789 : VPDPBUSDS zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.dds.512.66.0f38.w0 51 /r     ] AVX512VNNI,FUTURE */
        /* 241(f2 7d 48) 301 001(51) 120 : EVEX,PROT,AVX512VNNI,FUTURE */
        {I_VPDPBUSDS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45360, 590},
    /* 6790 : VPDPWSSD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.dds.128.66.0f38.w0 52 /r     ] AVX512VL,AVX512VNNI,FUTURE */
        /* 241(f2 7d 08) 301 001(52) 120 : EVEX,PROT,AVX512VL,AVX512VNNI,FUTURE */
        {I_VPDPWSSD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45378, 589},
    /* 6791 : VPDPWSSD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.dds.256.66.0f38.w0 52 /r     ] AVX512VL,AVX512VNNI,FUTURE */
        /* 241(f2 7d 28) 301 001(52) 120 : EVEX,PROT,AVX512VL,AVX512VNNI,FUTURE */
        {I_VPDPWSSD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45396, 589},
    /* 6792 : VPDPWSSD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.dds.512.66.0f38.w0 52 /r     ] AVX512VNNI,FUTURE */
        /* 241(f2 7d 48) 301 001(52) 120 : EVEX,PROT,AVX512VNNI,FUTURE */
        {I_VPDPWSSD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45414, 590},
    /* 6793 : VPDPWSSDS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.dds.128.66.0f38.w0 53 /r     ] AVX512VL,AVX512VNNI,FUTURE */
        /* 241(f2 7d 08) 301 001(53) 120 : EVEX,PROT,AVX512VL,AVX512VNNI,FUTURE */
        {I_VPDPWSSDS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45432, 589},
    /* 6794 : VPDPWSSDS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.dds.256.66.0f38.w0 53 /r     ] AVX512VL,AVX512VNNI,FUTURE */
        /* 241(f2 7d 28) 301 001(53) 120 : EVEX,PROT,AVX512VL,AVX512VNNI,FUTURE */
        {I_VPDPWSSDS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45450, 589},
    /* 6795 : VPDPWSSDS zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.dds.512.66.0f38.w0 53 /r     ] AVX512VNNI,FUTURE */
        /* 241(f2 7d 48) 301 001(53) 120 : EVEX,PROT,AVX512VNNI,FUTURE */
        {I_VPDPWSSDS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45468, 590},
    /* 6796 : VPOPCNTB xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.66.0f38.w0 54 /r         ] AVX512VL,AVX512BITALG,FUTURE */
        /* 250(f2 7d 08) 303 001(54) 110 : EVEX,PROT,AVX512VL,AVX512BITALG,FUTURE */
        {I_VPOPCNTB, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45486, 591},
    /* 6797 : VPOPCNTB ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.66.0f38.w0 54 /r         ] AVX512VL,AVX512BITALG,FUTURE */
        /* 250(f2 7d 28) 303 001(54) 110 : EVEX,PROT,AVX512VL,AVX512BITALG,FUTURE */
        {I_VPOPCNTB, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45495, 591},
    /* 6798 : VPOPCNTB zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.66.0f38.w0 54 /r         ] AVX512BITALG,FUTURE */
        /* 250(f2 7d 48) 303 001(54) 110 : EVEX,PROT,AVX512BITALG,FUTURE */
        {I_VPOPCNTB, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45504, 592},
    /* 6799 : VPOPCNTW xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.66.0f38.w1 54 /r         ] AVX512VL,AVX512BITALG,FUTURE */
        /* 250(f2 fd 08) 303 001(54) 110 : EVEX,PROT,AVX512VL,AVX512BITALG,FUTURE */
        {I_VPOPCNTW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45513, 591},
    /* 6800 : VPOPCNTW ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.66.0f38.w1 54 /r         ] AVX512VL,AVX512BITALG,FUTURE */
        /* 250(f2 fd 28) 303 001(54) 110 : EVEX,PROT,AVX512VL,AVX512BITALG,FUTURE */
        {I_VPOPCNTW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45522, 591},
    /* 6801 : VPOPCNTW zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.66.0f38.w1 54 /r         ] AVX512BITALG,FUTURE */
        /* 250(f2 fd 48) 303 001(54) 110 : EVEX,PROT,AVX512BITALG,FUTURE */
        {I_VPOPCNTW, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45531, 592},
    /* 6802 : VPOPCNTD xmmreg|mask|z,xmmrm128 [rm:fv:    evex.128.66.0f38.w0 55 /r         ] AVX512VL,AVX512VPOPCNTDQ,FUTURE */
        /* 250(f2 7d 08) 301 001(55) 110 : EVEX,PROT,AVX512VL,AVX512VPOPCNTDQ,FUTURE */
        {I_VPOPCNTD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45540, 593},
    /* 6803 : VPOPCNTD ymmreg|mask|z,ymmrm256 [rm:fv:    evex.256.66.0f38.w0 55 /r         ] AVX512VL,AVX512VPOPCNTDQ,FUTURE */
        /* 250(f2 7d 28) 301 001(55) 110 : EVEX,PROT,AVX512VL,AVX512VPOPCNTDQ,FUTURE */
        {I_VPOPCNTD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45549, 593},
    /* 6804 : VPOPCNTD zmmreg|mask|z,zmmrm512 [rm:fv:    evex.512.66.0f38.w0 55 /r         ] AVX512VPOPCNTDQ,FUTURE */
        /* 250(f2 7d 48) 301 001(55) 110 : EVEX,PROT,AVX512VPOPCNTDQ,FUTURE */
        {I_VPOPCNTD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45558, 594},
    /* 6805 : VPOPCNTQ xmmreg|mask|z,xmmrm128 [rm:fv:    evex.128.66.0f38.w1 55 /r         ] AVX512VL,AVX512VPOPCNTDQ,FUTURE */
        /* 250(f2 fd 08) 301 001(55) 110 : EVEX,PROT,AVX512VL,AVX512VPOPCNTDQ,FUTURE */
        {I_VPOPCNTQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45567, 593},
    /* 6806 : VPOPCNTQ ymmreg|mask|z,ymmrm256 [rm:fv:    evex.256.66.0f38.w1 55 /r         ] AVX512VL,AVX512VPOPCNTDQ,FUTURE */
        /* 250(f2 fd 28) 301 001(55) 110 : EVEX,PROT,AVX512VL,AVX512VPOPCNTDQ,FUTURE */
        {I_VPOPCNTQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45576, 593},
    /* 6807 : VPOPCNTQ zmmreg|mask|z,zmmrm512 [rm:fv:    evex.512.66.0f38.w1 55 /r         ] AVX512VPOPCNTDQ,FUTURE */
        /* 250(f2 fd 48) 301 001(55) 110 : EVEX,PROT,AVX512VPOPCNTDQ,FUTURE */
        {I_VPOPCNTQ, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45585, 594},
    /* 6808 : VPSHUFBITQMB kreg|mask,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w0 8f /r     ] AVX512VL,AVX512BITALG,FUTURE */
        /* 241(f2 7d 08) 303 001(8f) 120 : EVEX,PROT,AVX512VL,AVX512BITALG,FUTURE */
        {I_VPSHUFBITQMB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+45594, 591},
    /* 6809 : VPSHUFBITQMB kreg|mask,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w0 8f /r     ] AVX512VL,AVX512BITALG,FUTURE */
        /* 241(f2 7d 28) 303 001(8f) 120 : EVEX,PROT,AVX512VL,AVX512BITALG,FUTURE */
        {I_VPSHUFBITQMB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+45603, 591},
    /* 6810 : VPSHUFBITQMB kreg|mask,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w0 8f /r     ] AVX512BITALG,FUTURE */
        /* 241(f2 7d 48) 303 001(8f) 120 : EVEX,PROT,AVX512BITALG,FUTURE */
        {I_VPSHUFBITQMB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+45612, 592},
    /* 6811 : V4FMADDPS zmmreg|mask|z,zmmreg|rs4,mem [rvm:m128: evex.dds.512.f2.0f38.w0 9a /r     ] AR0-2,SO,AVX5124FMAPS,FUTURE */
        /* 241(f2 7f 48) 317 001(9a) 120 : AR0,AR1,AR2,SO,EVEX,PROT,AVX5124FMAPS,FUTURE */
        {I_V4FMADDPS, 3, {ZMMREG,ZMMREG|RS4,MEMORY,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45621, 595},
    /* 6812 : V4FNMADDPS zmmreg|mask|z,zmmreg|rs4,mem [rvm:m128: evex.dds.512.f2.0f38.w0 aa /r     ] AR0-2,SO,AVX5124FMAPS,FUTURE */
        /* 241(f2 7f 48) 317 001(aa) 120 : AR0,AR1,AR2,SO,EVEX,PROT,AVX5124FMAPS,FUTURE */
        {I_V4FNMADDPS, 3, {ZMMREG,ZMMREG|RS4,MEMORY,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45630, 595},
    /* 6813 : V4FMADDSS zmmreg|mask|z,zmmreg|rs4,mem [rvm:m128: evex.dds.lig.f2.0f38.w0 9b /r     ] AR0-2,SO,AVX5124FMAPS,FUTURE */
        /* 241(f2 7f 08) 317 001(9b) 120 : AR0,AR1,AR2,SO,LIG,EVEX,PROT,AVX5124FMAPS,FUTURE */
        {I_V4FMADDSS, 3, {ZMMREG,ZMMREG|RS4,MEMORY,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45639, 596},
    /* 6814 : V4FNMADDSS zmmreg|mask|z,zmmreg|rs4,mem [rvm:m128: evex.dds.lig.f2.0f38.w0 ab /r     ] AR0-2,SO,AVX5124FMAPS,FUTURE */
        /* 241(f2 7f 08) 317 001(ab) 120 : AR0,AR1,AR2,SO,LIG,EVEX,PROT,AVX5124FMAPS,FUTURE */
        {I_V4FNMADDSS, 3, {ZMMREG,ZMMREG|RS4,MEMORY,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45648, 596},
    /* 6815 : V4DPWSSDS zmmreg|mask|z,zmmreg|rs4,mem [rvm:m128: evex.dds.512.f2.0f38.w0 53 /r     ] AR0-2,SO,AVX5124VNNIW,FUTURE */
        /* 241(f2 7f 48) 317 001(53) 120 : AR0,AR1,AR2,SO,EVEX,PROT,AVX5124VNNIW,FUTURE */
        {I_V4DPWSSDS, 3, {ZMMREG,ZMMREG|RS4,MEMORY,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45657, 597},
    /* 6816 : V4DPWSSD zmmreg|mask|z,zmmreg|rs4,mem [rvm:m128: evex.dds.512.f2.0f38.w0 52 /r     ] AR0-2,SO,AVX5124VNNIW,FUTURE */
        /* 241(f2 7f 48) 317 001(52) 120 : AR0,AR1,AR2,SO,EVEX,PROT,AVX5124VNNIW,FUTURE */
        {I_V4DPWSSD, 3, {ZMMREG,ZMMREG|RS4,MEMORY,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45666, 597},
    /* 6817 : ENCLS void [          np 0f 01 cf                       ] SGX,FUTURE */
        /* 360 355 002(01 cf) : SGX,FUTURE */
        {I_ENCLS, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69938, 598},
    /* 6818 : ENCLU void [          np 0f 01 d7                       ] SGX,FUTURE */
        /* 360 355 002(01 d7) : SGX,FUTURE */
        {I_ENCLU, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69944, 598},
    /* 6819 : ENCLV void [          np 0f 01 c0                       ] SGX,FUTURE */
        /* 360 355 002(01 c0) : SGX,FUTURE */
        {I_ENCLV, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69950, 598},
    /* 6820 : CLRSSBSY mem64 [m:        f3 0f ae /6                       ] FL,CET,FUTURE */
        /* 333 355 001(ae) 206 : FL,CET,FUTURE */
        {I_CLRSSBSY, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56943, 599},
    /* 6821 : ENDBR32 void [          f3 0f 1e fb                       ] CET,FUTURE */
        /* 333 355 002(1e fb) : CET,FUTURE */
        {I_ENDBR32, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69956, 600},
    /* 6822 : ENDBR64 void [          f3 0f 1e fa                       ] CET,FUTURE */
        /* 333 355 002(1e fa) : CET,FUTURE */
        {I_ENDBR64, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69962, 600},
    /* 6823 : INCSSPD reg32 [m:        o32 f3 0f ae /5                   ] CET,FUTURE */
        /* 321 333 355 001(ae) 205 : CET,FUTURE */
        {I_INCSSPD, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+65811, 600},
    /* 6824 : INCSSPQ reg64 [m:        o64 f3 0f ae /5                   ] LONG,PROT,CET,X86_64 */
        /* 324 333 355 001(ae) 205 : LONG,PROT,CET,X86_64 */
        {I_INCSSPQ, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+65818, 601},
    /* 6825 : RDSSPD reg32 [m:        o32 f3 0f 1e /1                   ] CET,FUTURE */
        /* 321 333 355 001(1e) 201 : CET,FUTURE */
        {I_RDSSPD, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+65825, 600},
    /* 6826 : RDSSPQ reg64 [m:        o64 f3 0f 1e /1                   ] LONG,PROT,CET,X86_64 */
        /* 324 333 355 001(1e) 201 : LONG,PROT,CET,X86_64 */
        {I_RDSSPQ, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+65832, 601},
    /* 6827 : RSTORSSP mem64 [m:        f3 0f 01 /5                       ] CET,FUTURE */
        /* 333 355 001(01) 205 : CET,FUTURE */
        {I_RSTORSSP, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69968, 600},
    /* 6828 : SAVEPREVSSP void [          f3 0f 01 ea                       ] CET,FUTURE */
        /* 333 355 002(01 ea) : CET,FUTURE */
        {I_SAVEPREVSSP, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69974, 600},
    /* 6829 : SETSSBSY void [          f3 0f 01 e8                       ] CET,FUTURE */
        /* 333 355 002(01 e8) : CET,FUTURE */
        {I_SETSSBSY, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69980, 600},
    /* 6830 : WRUSSD mem32,reg32 [mr:       o32 66 0f38 f5 /r                 ] CET,FUTURE */
        /* 321 361 356 001(f5) 101 : NOAPX,CET,FUTURE */
        {I_WRUSSD, 2, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65839, 602},
    /* 6831 : WRUSSD mem32,reg32 [mr:       evex.nf0.nd0.l0.66.m4.w0 65 /r    ] LONG,PROT,CET,APX,X86_64 */
        /* 250(f4 7d 08) 300 001(65) 101 : LONG,EVEX,PROT,CET,APX,X86_64 */
        {I_WRUSSD, 2, {MEMORY|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+45675, 603},
    /* 6832 : WRUSSQ mem64,reg64 [mr:       o64 66 0f38 f5 /r                 ] LONG,PROT,CET,X86_64 */
        /* 324 361 356 001(f5) 101 : NOAPX,LONG,PROT,CET,X86_64 */
        {I_WRUSSQ, 2, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65846, 604},
    /* 6833 : WRUSSQ mem64,reg64 [mr:       evex.nf0.nd0.l0.66.m4.w1 65 /r    ] LONG,PROT,CET,APX,X86_64 */
        /* 250(f4 fd 08) 300 001(65) 101 : LONG,EVEX,PROT,CET,APX,X86_64 */
        {I_WRUSSQ, 2, {MEMORY|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+45684, 603},
    /* 6834 : WRSSD mem32,reg32 [mr:       o32 0f38 f6 /r                    ] CET,FUTURE */
        /* 321 356 001(f6) 101 : NOAPX,CET,FUTURE */
        {I_WRSSD, 2, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69986, 602},
    /* 6835 : WRSSD mem32,reg32 [mr:       evex.nf0.nd0.l0.m4.w0 66 /r       ] LONG,PROT,CET,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(66) 101 : LONG,EVEX,PROT,CET,APX,X86_64 */
        {I_WRSSD, 2, {MEMORY|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+45693, 603},
    /* 6836 : WRSSQ mem64,reg64 [mr:       o64 0f38 f6 /r                    ] LONG,PROT,CET,X86_64 */
        /* 324 356 001(f6) 101 : NOAPX,LONG,PROT,CET,X86_64 */
        {I_WRSSQ, 2, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69992, 604},
    /* 6837 : WRSSQ mem64,reg64 [mr:       evex.nf0.nd0.l0.m4.w1 66 /r       ] LONG,PROT,CET,APX,X86_64 */
        /* 250(f4 fc 08) 300 001(66) 101 : LONG,EVEX,PROT,CET,APX,X86_64 */
        {I_WRSSQ, 2, {MEMORY|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+45702, 603},
    /* 6838 : ENQCMD reg16,mem512 [rm:       a16 f2 0f38 f8 /r                 ] AR0-1,SZ,NOREX,NOAPX,NOLONG,FL,ENQCMD,FUTURE */
        /* 310 332 356 001(f8) 110 : AR0,AR1,SZ,NOREX,NOAPX,NOLONG,FL,ENQCMD,FUTURE */
        {I_ENQCMD, 2, {REG_GPR|BITS16|RN_L16,MEMORY|BITS512|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65853, 605},
    /* 6839 : ENQCMD reg32,mem512 [rm:       a32 f2 0f38 f8 /r                 ] AR0-1,SZ,FL,ENQCMD,FUTURE */
        /* 311 332 356 001(f8) 110 : AR0,AR1,SZ,NOAPX,FL,ENQCMD,FUTURE */
        {I_ENQCMD, 2, {REG_GPR|BITS32|RN_L16,MEMORY|BITS512|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65860, 606},
    /* 6840 : ENQCMD reg64,mem512 [rm:       a64 f2 0f38 f8 /r                 ] AR0-1,SZ,LONG,FL,PROT,ENQCMD,X86_64 */
        /* 313 332 356 001(f8) 110 : AR0,AR1,SZ,NOAPX,LONG,FL,PROT,ENQCMD,X86_64 */
        {I_ENQCMD, 2, {REG_GPR|BITS64|RN_L16,MEMORY|BITS512|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65867, 607},
    /* 6841 : ENQCMD reg64,mem512 [rm:       evex.nf0.nd0.l0.f2.m4.w1 f8 /r    ] AR0-1,SZ,LONG,FL,PRIV,PROT,ENQCMD,APX,X86_64 */
        /* 250(f4 ff 08) 300 001(f8) 110 : AR0,AR1,SZ,LONG,FL,EVEX,PRIV,PROT,ENQCMD,APX,X86_64 */
        {I_ENQCMD, 2, {REG_GPR|BITS64,MEMORY|BITS512,0,0,0}, NO_DECORATOR, nasm_bytecodes+45711, 608},
    /* 6842 : ENQCMDS reg16,mem512 [rm:       a16 f3 0f38 f8 /r                 ] AR0-1,SZ,NOREX,NOAPX,NOLONG,FL,PRIV,ENQCMD,FUTURE */
        /* 310 333 356 001(f8) 110 : AR0,AR1,SZ,NOREX,NOAPX,NOLONG,FL,PRIV,ENQCMD,FUTURE */
        {I_ENQCMDS, 2, {REG_GPR|BITS16|RN_L16,MEMORY|BITS512|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65874, 609},
    /* 6843 : ENQCMDS reg32,mem512 [rm:       a32 f3 0f38 f8 /r                 ] AR0-1,SZ,FL,PRIV,ENQCMD,FUTURE */
        /* 311 333 356 001(f8) 110 : AR0,AR1,SZ,NOAPX,FL,PRIV,ENQCMD,FUTURE */
        {I_ENQCMDS, 2, {REG_GPR|BITS32|RN_L16,MEMORY|BITS512|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65881, 610},
    /* 6844 : ENQCMDS reg64,mem512 [rm:       a64 f3 0f38 f8 /r                 ] AR0-1,SZ,LONG,FL,PRIV,PROT,ENQCMD,X86_64 */
        /* 313 333 356 001(f8) 110 : AR0,AR1,SZ,NOAPX,LONG,FL,PRIV,PROT,ENQCMD,X86_64 */
        {I_ENQCMDS, 2, {REG_GPR|BITS64|RN_L16,MEMORY|BITS512|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65888, 611},
    /* 6845 : ENQCMDS reg64,mem512 [rm:       evex.nf0.nd0.l0.f3.m4.w1 f8 /r    ] AR0-1,SZ,LONG,FL,PRIV,PROT,ENQCMD,APX,X86_64 */
        /* 250(f4 fe 08) 300 001(f8) 110 : AR0,AR1,SZ,LONG,FL,EVEX,PRIV,PROT,ENQCMD,APX,X86_64 */
        {I_ENQCMDS, 2, {REG_GPR|BITS64,MEMORY|BITS512,0,0,0}, NO_DECORATOR, nasm_bytecodes+45720, 608},
    /* 6846 : PCONFIG void [          np 0f 01 c5                       ] FL,PRIV,PCONFIG,FUTURE */
        /* 360 355 002(01 c5) : FL,PRIV,PCONFIG,FUTURE */
        {I_PCONFIG, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69920, 612},
    /* 6847 : XRESLDTRK void [          f2 0f 01 e9                       ] TSXLDTRK,FUTURE */
        /* 332 355 002(01 e9) : TSXLDTRK,FUTURE */
        {I_XRESLDTRK, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69998, 613},
    /* 6848 : XSUSLDTRK void [          f2 0f 01 e8                       ] TSXLDTRK,FUTURE */
        /* 332 355 002(01 e8) : TSXLDTRK,FUTURE */
        {I_XSUSLDTRK, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70004, 613},
    /* 6849 : VCVTNE2PS2BF16 xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.128.f2.0f38.w0 72 /r         ] AVX512VL,AVX512BF16,FUTURE */
        /* 241(f2 7f 08) 301 001(72) 120 : EVEX,PROT,AVX512VL,AVX512BF16,FUTURE */
        {I_VCVTNE2PS2BF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45729, 614},
    /* 6850 : VCVTNE2PS2BF16 ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.256.f2.0f38.w0 72 /r         ] AVX512VL,AVX512BF16,FUTURE */
        /* 241(f2 7f 28) 301 001(72) 120 : EVEX,PROT,AVX512VL,AVX512BF16,FUTURE */
        {I_VCVTNE2PS2BF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45747, 614},
    /* 6851 : VCVTNE2PS2BF16 zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.512.f2.0f38.w0 72 /r         ] AVX512BF16,FUTURE */
        /* 241(f2 7f 48) 301 001(72) 120 : EVEX,PROT,AVX512BF16,FUTURE */
        {I_VCVTNE2PS2BF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45765, 615},
    /* 6852 : VCVTNEPS2BF16 xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.f3.0f38.w0 72 /r         ] AVX512VL,AVX512BF16,FUTURE */
        /* 250(f2 7e 08) 301 001(72) 110 : EVEX,PROT,AVX512VL,AVX512BF16,FUTURE */
        {I_VCVTNEPS2BF16, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+45783, 614},
    /* 6853 : VCVTNEPS2BF16 xmmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.f3.0f38.w0 72 /r         ] AVX512VL,AVX512BF16,FUTURE */
        /* 250(f2 7e 28) 301 001(72) 110 : EVEX,PROT,AVX512VL,AVX512BF16,FUTURE */
        {I_VCVTNEPS2BF16, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+45792, 614},
    /* 6854 : VCVTNEPS2BF16 ymmreg|mask|z,zmmrm512|b32 [rm:fv:    evex.512.f3.0f38.w0 72 /r         ] AVX512BF16,FUTURE */
        /* 250(f2 7e 48) 301 001(72) 110 : EVEX,PROT,AVX512BF16,FUTURE */
        {I_VCVTNEPS2BF16, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+45801, 615},
    /* 6855 : VDPBF16PS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.128.f3.0f38.w0 52 /r         ] AVX512VL,AVX512BF16,FUTURE */
        /* 241(f2 7e 08) 301 001(52) 120 : EVEX,PROT,AVX512VL,AVX512BF16,FUTURE */
        {I_VDPBF16PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45810, 614},
    /* 6856 : VDPBF16PS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.256.f3.0f38.w0 52 /r         ] AVX512VL,AVX512BF16,FUTURE */
        /* 241(f2 7e 28) 301 001(52) 120 : EVEX,PROT,AVX512VL,AVX512BF16,FUTURE */
        {I_VDPBF16PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45828, 614},
    /* 6857 : VDPBF16PS zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.512.f3.0f38.w0 52 /r         ] AVX512BF16,FUTURE */
        /* 241(f2 7e 48) 301 001(52) 120 : EVEX,PROT,AVX512BF16,FUTURE */
        {I_VDPBF16PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45846, 615},
    /* 6858 : VP2INTERSECTD kreg|rs2,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.f2.0f38.w0 68 /r     ] AVX512VL,AVX512VP2INTERSECT,FUTURE */
        /* 241(f2 7f 08) 301 001(68) 120 : EVEX,PROT,AVX512VL,AVX512VP2INTERSECT,FUTURE */
        {I_VP2INTERSECTD, 3, {OPMASKREG|RS2,XMMREG,RM_XMM|BITS128,0,0}, {0,0,B32,0,0}, nasm_bytecodes+45864, 616},
    /* 6859 : VP2INTERSECTD kreg|rs2,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.f2.0f38.w0 68 /r     ] AVX512VL,AVX512VP2INTERSECT,FUTURE */
        /* 241(f2 7f 28) 301 001(68) 120 : EVEX,PROT,AVX512VL,AVX512VP2INTERSECT,FUTURE */
        {I_VP2INTERSECTD, 3, {OPMASKREG|RS2,YMMREG,RM_YMM|BITS256,0,0}, {0,0,B32,0,0}, nasm_bytecodes+45873, 616},
    /* 6860 : VP2INTERSECTD kreg|rs2,zmmreg,zmmrm512|b32 [rvm:fv:   evex.nds.512.f2.0f38.w0 68 /r     ] AVX512F,AVX512VP2INTERSECT,FUTURE */
        /* 241(f2 7f 48) 301 001(68) 120 : EVEX,PROT,AVX512F,AVX512VP2INTERSECT,FUTURE */
        {I_VP2INTERSECTD, 3, {OPMASKREG|RS2,ZMMREG,RM_ZMM|BITS512,0,0}, {0,0,B32,0,0}, nasm_bytecodes+45882, 617},
    /* 6861 : VP2INTERSECTQ kreg|rs2,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.f2.0f38.w1 68 /r     ] AVX512VL,AVX512VP2INTERSECT,FUTURE */
        /* 241(f2 ff 08) 301 001(68) 120 : EVEX,PROT,AVX512VL,AVX512VP2INTERSECT,FUTURE */
        {I_VP2INTERSECTQ, 3, {OPMASKREG|RS2,XMMREG,RM_XMM|BITS128,0,0}, {0,0,B64,0,0}, nasm_bytecodes+45891, 616},
    /* 6862 : VP2INTERSECTQ kreg|rs2,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.f2.0f38.w1 68 /r     ] AVX512VL,AVX512VP2INTERSECT,FUTURE */
        /* 241(f2 ff 28) 301 001(68) 120 : EVEX,PROT,AVX512VL,AVX512VP2INTERSECT,FUTURE */
        {I_VP2INTERSECTQ, 3, {OPMASKREG|RS2,YMMREG,RM_YMM|BITS256,0,0}, {0,0,B64,0,0}, nasm_bytecodes+45900, 616},
    /* 6863 : VP2INTERSECTQ kreg|rs2,zmmreg,zmmrm512|b64 [rvm:fv:   evex.nds.512.f2.0f38.w1 68 /r     ] AVX512F,AVX512VP2INTERSECT,FUTURE */
        /* 241(f2 ff 48) 301 001(68) 120 : EVEX,PROT,AVX512F,AVX512VP2INTERSECT,FUTURE */
        {I_VP2INTERSECTQ, 3, {OPMASKREG|RS2,ZMMREG,RM_ZMM|BITS512,0,0}, {0,0,B64,0,0}, nasm_bytecodes+45909, 617},
    /* 6864 : LDTILECFG mem512 [m:        vex.128.np.0f38.w0 49 /0         ] AR0,SZ,LONG,PROT,AMXTILE,X86_64 */
        /* 270(002 000) 001(49) 200 : AR0,SZ,NOAPX,LONG,VEX,PROT,AMXTILE,X86_64 */
        {I_LDTILECFG, 1, {MEMORY|BITS512|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+65895, 618},
    /* 6865 : LDTILECFG mem512 [m:        evex.128.np.0f38.w0 49 /0         ] AR0,SZ,LONG,PROT,AMXTILE,X86_64,APX */
        /* 250(f2 7c 08) 300 001(49) 200 : AR0,SZ,LONG,EVEX,PROT,AMXTILE,APX,X86_64 */
        {I_LDTILECFG, 1, {MEMORY|BITS512,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+45918, 619},
    /* 6866 : STTILECFG mem512 [m:        vex.128.66.0f38.w0 49 /0         ] AR0,SZ,LONG,PROT,AMXTILE,X86_64 */
        /* 270(002 001) 001(49) 200 : AR0,SZ,NOAPX,LONG,VEX,PROT,AMXTILE,X86_64 */
        {I_STTILECFG, 1, {MEMORY|BITS512|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+65902, 618},
    /* 6867 : STTILECFG mem512 [m:        evex.128.66.0f38.w0 49 /0         ] AR0,SZ,LONG,PROT,AMXTILE,X86_64,APX */
        /* 250(f2 7d 08) 300 001(49) 200 : AR0,SZ,LONG,EVEX,PROT,AMXTILE,APX,X86_64 */
        {I_STTILECFG, 1, {MEMORY|BITS512,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+45927, 619},
    /* 6868 : TDPBF16PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.f3.0f38.w0 5c /r          ] LONG,PROT,AMXBF16,X86_64 */
        /* 262(002 002) 001(5c) 110 : NOAPX,LONG,VEX,PROT,AMXBF16,X86_64 */
        {I_TDPBF16PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65909, 620},
    /* 6869 : TDPFP16PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.f2.0f38.w0 5c /r          ] LONG,PROT,AMXFP16,X86_64 */
        /* 262(002 003) 001(5c) 110 : NOAPX,LONG,VEX,PROT,AMXFP16,X86_64 */
        {I_TDPFP16PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65916, 621},
    /* 6870 : TCMMIMFP16PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.66.0f38.w0 6c /r          ] LONG,PROT,AMXCOMPLEX,X86_64 */
        /* 262(002 001) 001(6c) 110 : NOAPX,LONG,VEX,PROT,AMXCOMPLEX,X86_64 */
        {I_TCMMIMFP16PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65923, 622},
    /* 6871 : TCMMRLFP16PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.np.0f38.w0 6c /r          ] LONG,PROT,AMXCOMPLEX,X86_64 */
        /* 262(002 000) 001(6c) 110 : NOAPX,LONG,VEX,PROT,AMXCOMPLEX,X86_64 */
        {I_TCMMRLFP16PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65930, 622},
    /* 6872 : TDPBSSD tmmreg,tmmreg,tmmreg [rmv:      vex.128.f2.0f38.w0 5e /r          ] LONG,PROT,AMXINT8,X86_64 */
        /* 262(002 003) 001(5e) 110 : NOAPX,LONG,VEX,PROT,AMXINT8,X86_64 */
        {I_TDPBSSD, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65937, 623},
    /* 6873 : TDPBSUD tmmreg,tmmreg,tmmreg [rmv:      vex.128.f3.0f38.w0 5e /r          ] LONG,PROT,AMXINT8,X86_64 */
        /* 262(002 002) 001(5e) 110 : NOAPX,LONG,VEX,PROT,AMXINT8,X86_64 */
        {I_TDPBSUD, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65944, 623},
    /* 6874 : TDPBUSD tmmreg,tmmreg,tmmreg [rmv:      vex.128.66.0f38.w0 5e /r          ] LONG,PROT,AMXINT8,X86_64 */
        /* 262(002 001) 001(5e) 110 : NOAPX,LONG,VEX,PROT,AMXINT8,X86_64 */
        {I_TDPBUSD, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65951, 623},
    /* 6875 : TDPBUUD tmmreg,tmmreg,tmmreg [rmv:      vex.128.np.0f38.w0 5e /r          ] LONG,PROT,AMXINT8,X86_64 */
        /* 262(002 000) 001(5e) 110 : NOAPX,LONG,VEX,PROT,AMXINT8,X86_64 */
        {I_TDPBUUD, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65958, 623},
    /* 6876 : TILELOADD tmmreg,mem [rm:       vex.128.f2.0f38.w0 4b /r         ] AR1,ANYSIZE,SIB,LONG,MIB,PROT,AMXTILE,APX,X86_64 */
        /* 270(002 003) 001(4b) 110 : AR1,ANYSIZE,NOAPX,SIB,LONG,MIB,VEX,PROT,AMXTILE,APX,X86_64 */
        {I_TILELOADD, 2, {TMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65965, 624},
    /* 6877 : TILELOADD tmmreg,mem [rm:       evex.128.f2.0f38.w0 4b /r         ] AR1,ANYSIZE,SIB,LONG,MIB,PROT,AMXTILE,APX,X86_64,APX */
        /* 250(f2 7f 08) 300 001(4b) 110 : AR1,ANYSIZE,SIB,LONG,MIB,EVEX,PROT,AMXTILE,APX,X86_64 */
        {I_TILELOADD, 2, {TMMREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+45936, 625},
    /* 6878 : TILELOADDT1 tmmreg,mem [rm:       vex.128.66.0f38.w0 4b /r         ] AR1,ANYSIZE,SIB,LONG,MIB,PROT,AMXTILE,APX,X86_64 */
        /* 270(002 001) 001(4b) 110 : AR1,ANYSIZE,NOAPX,SIB,LONG,MIB,VEX,PROT,AMXTILE,APX,X86_64 */
        {I_TILELOADDT1, 2, {TMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65972, 624},
    /* 6879 : TILELOADDT1 tmmreg,mem [rm:       evex.128.66.0f38.w0 4b /r         ] AR1,ANYSIZE,SIB,LONG,MIB,PROT,AMXTILE,APX,X86_64,APX */
        /* 250(f2 7d 08) 300 001(4b) 110 : AR1,ANYSIZE,SIB,LONG,MIB,EVEX,PROT,AMXTILE,APX,X86_64 */
        {I_TILELOADDT1, 2, {TMMREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+45945, 625},
    /* 6880 : TILERELEASE void [          vex.128.np.0f38.w0 49 c0          ] LONG,PROT,AMXTILE,X86_64 */
        /* 270(002 000) 002(49 c0) : NOAPX,LONG,VEX,PROT,AMXTILE,X86_64 */
        {I_TILERELEASE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+65979, 626},
    /* 6881 : TILESTORED mem,tmmreg [mr:       vex.128.f3.0f38.w0 4b /r         ] AR0,ANYSIZE,SIB,LONG,MIB,PROT,AMXTILE,APX,X86_64 */
        /* 270(002 002) 001(4b) 101 : AR0,ANYSIZE,NOAPX,SIB,LONG,MIB,VEX,PROT,AMXTILE,APX,X86_64 */
        {I_TILESTORED, 2, {MEMORY|RN_L16,TMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65986, 627},
    /* 6882 : TILESTORED mem,tmmreg [mr:       evex.128.f3.0f38.w0 4b /r         ] AR0,ANYSIZE,SIB,LONG,MIB,PROT,AMXTILE,APX,X86_64,APX */
        /* 250(f2 7e 08) 300 001(4b) 101 : AR0,ANYSIZE,SIB,LONG,MIB,EVEX,PROT,AMXTILE,APX,X86_64 */
        {I_TILESTORED, 2, {MEMORY,TMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+45954, 628},
    /* 6883 : TILEZERO tmmreg [r:        vex.128.f2.0f38.w0 49 /3r0        ] LONG,PROT,AMXTILE,X86_64 */
        /* 270(002 003) 001(49) 171(300) : NOAPX,LONG,VEX,PROT,AMXTILE,X86_64 */
        {I_TILEZERO, 1, {TMMREG|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+54701, 626},
    /* 6884 : TILELOADDRS tmmreg,mem [rm:       vex.128.f2.0f38.w0 4a /r         ] AR1,ANYSIZE,SIB,LONG,MIB,PROT,AMXTILE,AMXMOVRS,APX,X86_64,FUTURE */
        /* 270(002 003) 001(4a) 110 : AR1,ANYSIZE,NOAPX,SIB,LONG,MIB,VEX,PROT,AMXTILE,AMXMOVRS,APX,FUTURE */
        {I_TILELOADDRS, 2, {TMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65993, 629},
    /* 6885 : TILELOADDRS tmmreg,mem [rm:       evex.128.f2.0f38.w0 4a /r         ] AR1,ANYSIZE,SIB,LONG,MIB,PROT,AMXTILE,AMXMOVRS,APX,X86_64,FUTURE,APX */
        /* 250(f2 7f 08) 300 001(4a) 110 : AR1,ANYSIZE,SIB,LONG,MIB,EVEX,PROT,AMXTILE,AMXMOVRS,APX,FUTURE */
        {I_TILELOADDRS, 2, {TMMREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+45963, 630},
    /* 6886 : TILELOADDRST1 tmmreg,mem [rm:       vex.128.66.0f38.w0 4a /r         ] AR1,ANYSIZE,SIB,LONG,MIB,PROT,AMXTILE,AMXMOVRS,APX,X86_64,FUTURE */
        /* 270(002 001) 001(4a) 110 : AR1,ANYSIZE,NOAPX,SIB,LONG,MIB,VEX,PROT,AMXTILE,AMXMOVRS,APX,FUTURE */
        {I_TILELOADDRST1, 2, {TMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66000, 629},
    /* 6887 : TILELOADDRST1 tmmreg,mem [rm:       evex.128.66.0f38.w0 4a /r         ] AR1,ANYSIZE,SIB,LONG,MIB,PROT,AMXTILE,AMXMOVRS,APX,X86_64,FUTURE,APX */
        /* 250(f2 7d 08) 300 001(4a) 110 : AR1,ANYSIZE,SIB,LONG,MIB,EVEX,PROT,AMXTILE,AMXMOVRS,APX,FUTURE */
        {I_TILELOADDRST1, 2, {TMMREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+45972, 630},
    /* 6888 : T2RPNTLVWZ0 tmmreg,mem [rm:       vex.128.np.0f38.w0 6e /r          ] SIB,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 270(002 000) 001(6e) 110 : NOAPX,SIB,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_T2RPNTLVWZ0, 2, {TMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66007, 631},
    /* 6889 : T2RPNTLVWZ0T1 tmmreg,mem [rm:       vex.128.np.0f38.w0 6f /r          ] SIB,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 270(002 000) 001(6f) 110 : NOAPX,SIB,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_T2RPNTLVWZ0T1, 2, {TMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66014, 631},
    /* 6890 : T2RPNTLVWZ1 tmmreg,mem [rm:       vex.128.66.0f38.w0 6e /r          ] SIB,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 270(002 001) 001(6e) 110 : NOAPX,SIB,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_T2RPNTLVWZ1, 2, {TMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66021, 631},
    /* 6891 : T2RPNTLVWZ1T1 tmmreg,mem [rm:       vex.128.66.0f38.w0 6f /r          ] SIB,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 270(002 001) 001(6f) 110 : NOAPX,SIB,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_T2RPNTLVWZ1T1, 2, {TMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66028, 631},
    /* 6892 : T2RPNTLVWZ0RS tmmreg,mem [rm:       vex.128.np.map5.w0 f8 /r          ] SIB,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 270(005 000) 001(f8) 110 : NOAPX,SIB,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_T2RPNTLVWZ0RS, 2, {TMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66035, 631},
    /* 6893 : T2RPNTLVWZ0RST1 tmmreg,mem [rm:       vex.128.np.map5.w0 f9 /r          ] SIB,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 270(005 000) 001(f9) 110 : NOAPX,SIB,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_T2RPNTLVWZ0RST1, 2, {TMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66042, 631},
    /* 6894 : T2RPNTLVWZ1RS tmmreg,mem [rm:       vex.128.66.map5.w0 f8 /r          ] SIB,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 270(005 001) 001(f8) 110 : NOAPX,SIB,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_T2RPNTLVWZ1RS, 2, {TMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66049, 631},
    /* 6895 : T2RPNTLVWZ1RST1 tmmreg,mem [rm:       vex.128.66.map5.w0 f9 /r          ] SIB,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 270(005 001) 001(f9) 110 : NOAPX,SIB,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_T2RPNTLVWZ1RST1, 2, {TMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66056, 631},
    /* 6896 : TCONJTCMMIMFP16PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.np.0f38.w0 6b /r          ] AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 262(002 000) 001(6b) 110 : NOAPX,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_TCONJTCMMIMFP16PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66063, 632},
    /* 6897 : TCONJTFP16 tmmreg,tmmreg [rm:       vex.128.66.0f38.w0 6b /r          ] AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 270(002 001) 001(6b) 110 : NOAPX,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_TCONJTFP16, 2, {TMMREG|RN_L16,TMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66070, 632},
    /* 6898 : TCVTROWD2PS zmmreg,tmmreg,reg32 [rmv:      evex.512.f3.0f38.w0 4a /r         ] AMXAVX512,FUTURE */
        /* 242(f2 7e 48) 300 001(4a) 110 : EVEX,PROT,AMXAVX512,FUTURE */
        {I_TCVTROWD2PS, 3, {ZMMREG,TMMREG,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+45981, 633},
    /* 6899 : TCVTROWD2PS zmmreg,tmmreg,imm8 [rmi:      evex.512.f3.0f3a.w0 07 /r ib      ] AMXAVX512,FUTURE */
        /* 250(f3 7e 48) 300 001(07) 110 022 : EVEX,PROT,AMXAVX512,FUTURE */
        {I_TCVTROWD2PS, 3, {ZMMREG,TMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+22643, 633},
    /* 6900 : TCVTROWPS2BF16H zmmreg,tmmreg,reg32 [rmv:      evex.512.f2.0f38.w0 6d /r         ] AMXAVX512,FUTURE */
        /* 242(f2 7f 48) 300 001(6d) 110 : EVEX,PROT,AMXAVX512,FUTURE */
        {I_TCVTROWPS2BF16H, 3, {ZMMREG,TMMREG,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+45990, 633},
    /* 6901 : TCVTROWPS2BF16H zmmreg,tmmreg,imm8 [rmi:      evex.512.f2.0f3a.w0 07 /r ib      ] AMXAVX512,FUTURE */
        /* 250(f3 7f 48) 300 001(07) 110 022 : EVEX,PROT,AMXAVX512,FUTURE */
        {I_TCVTROWPS2BF16H, 3, {ZMMREG,TMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+22653, 633},
    /* 6902 : TCVTROWPS2BF16L zmmreg,tmmreg,reg32 [rmv:      evex.512.f3.0f38.w0 6d /r         ] AMXAVX512,FUTURE */
        /* 242(f2 7e 48) 300 001(6d) 110 : EVEX,PROT,AMXAVX512,FUTURE */
        {I_TCVTROWPS2BF16L, 3, {ZMMREG,TMMREG,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+45999, 633},
    /* 6903 : TCVTROWPS2BF16L zmmreg,tmmreg,imm8 [rmi:      evex.512.f3.0f3a.w0 77 /r ib      ] AMXAVX512,FUTURE */
        /* 250(f3 7e 48) 300 001(77) 110 022 : EVEX,PROT,AMXAVX512,FUTURE */
        {I_TCVTROWPS2BF16L, 3, {ZMMREG,TMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+22663, 633},
    /* 6904 : TCVTROWPS2PHH zmmreg,tmmreg,reg32 [rmv:      evex.512.np.0f38.w0 6d /r         ] AMXAVX512,FUTURE */
        /* 242(f2 7c 48) 300 001(6d) 110 : EVEX,PROT,AMXAVX512,FUTURE */
        {I_TCVTROWPS2PHH, 3, {ZMMREG,TMMREG,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+46008, 633},
    /* 6905 : TCVTROWPS2PHH zmmreg,tmmreg,imm8 [rmi:      evex.512.np.0f3a.w0 07 /r ib      ] AMXAVX512,FUTURE */
        /* 250(f3 7c 48) 300 001(07) 110 022 : EVEX,PROT,AMXAVX512,FUTURE */
        {I_TCVTROWPS2PHH, 3, {ZMMREG,TMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+22673, 633},
    /* 6906 : TCVTROWPS2PHL zmmreg,tmmreg,reg32 [rmv:      evex.512.66.0f38.w0 6d /r         ] AMXAVX512,FUTURE */
        /* 242(f2 7d 48) 300 001(6d) 110 : EVEX,PROT,AMXAVX512,FUTURE */
        {I_TCVTROWPS2PHL, 3, {ZMMREG,TMMREG,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+46017, 633},
    /* 6907 : TCVTROWPS2PHL zmmreg,tmmreg,imm8 [rmi:      evex.512.f2.0f3a.w0 77 /r ib      ] AMXAVX512,FUTURE */
        /* 250(f3 7f 48) 300 001(77) 110 022 : EVEX,PROT,AMXAVX512,FUTURE */
        {I_TCVTROWPS2PHL, 3, {ZMMREG,TMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+22683, 633},
    /* 6908 : TDPBF8PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.np.map5.w0 fd /r          ] AMXFP8,FUTURE */
        /* 262(005 000) 001(fd) 110 : NOAPX,VEX,AMXFP8,FUTURE */
        {I_TDPBF8PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66077, 634},
    /* 6909 : TDPBHF8PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.f2.map5.w0 fd /r          ] AMXFP8,FUTURE */
        /* 262(005 003) 001(fd) 110 : NOAPX,VEX,AMXFP8,FUTURE */
        {I_TDPBHF8PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66084, 634},
    /* 6910 : TDPHBF8PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.f3.map5.w0 fd /r          ] AMXFP8,FUTURE */
        /* 262(005 002) 001(fd) 110 : NOAPX,VEX,AMXFP8,FUTURE */
        {I_TDPHBF8PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66091, 634},
    /* 6911 : TDPHF8PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.66.map5.w0 fd /r          ] AMXFP8,FUTURE */
        /* 262(005 001) 001(fd) 110 : NOAPX,VEX,AMXFP8,FUTURE */
        {I_TDPHF8PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66098, 634},
    /* 6912 : TILEMOVROW zmmreg,tmmreg,imm8 [rmi:      evex.512.66.0f3a.w0 07 /r ib      ] AMXAVX512,FUTURE */
        /* 250(f3 7d 48) 300 001(07) 110 022 : EVEX,PROT,AMXAVX512,FUTURE */
        {I_TILEMOVROW, 3, {ZMMREG,TMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+22693, 633},
    /* 6913 : TILEMOVROW zmmreg,tmmreg,reg32 [rmv:      evex.512.66.0f38.w0 4a /r         ] AMXAVX512,FUTURE */
        /* 242(f2 7d 48) 300 001(4a) 110 : EVEX,PROT,AMXAVX512,FUTURE */
        {I_TILEMOVROW, 3, {ZMMREG,TMMREG,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+46026, 633},
    /* 6914 : TMMULTF32PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.66.0f38.w0 48 /r          ] AMXTF32,FUTURE */
        /* 262(002 001) 001(48) 110 : NOAPX,VEX,AMXTF32,FUTURE */
        {I_TMMULTF32PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66105, 635},
    /* 6915 : TTCMMIMFP16PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.f2.0f38.w0 6b /r          ] AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 262(002 003) 001(6b) 110 : NOAPX,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_TTCMMIMFP16PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66112, 632},
    /* 6916 : TTCMMRLFP16PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.f3.0f38.w0 6b /r          ] AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 262(002 002) 001(6b) 110 : NOAPX,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_TTCMMRLFP16PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66119, 632},
    /* 6917 : TTDPBF16PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.f3.0f38.w0 6c /r          ] AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 262(002 002) 001(6c) 110 : NOAPX,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_TTDPBF16PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66126, 632},
    /* 6918 : TTDPFP16PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.f2.0f38.w0 6c /r          ] AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 262(002 003) 001(6c) 110 : NOAPX,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_TTDPFP16PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66133, 632},
    /* 6919 : TTMMULTF32PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.np.0f38.w0 48 /r          ] AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 262(002 000) 001(48) 110 : NOAPX,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_TTMMULTF32PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66140, 632},
    /* 6920 : TTRANSPOSED tmmreg,tmmreg [rm:       vex.128.f3.0f38.w0 5f /r          ] AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 270(002 002) 001(5f) 110 : NOAPX,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_TTRANSPOSED, 2, {TMMREG|RN_L16,TMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66147, 632},
    /* 6921 : VADDPH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.np.map5.w0 58 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 08) 301 001(58) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VADDPH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+46035, 566},
    /* 6922 : VADDPH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.np.map5.w0 58 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 28) 301 001(58) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VADDPH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+46053, 566},
    /* 6923 : VADDPH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.np.map5.w0 58 /r     ] AVX512FP16,FUTURE */
        /* 241(f5 7c 48) 301 001(58) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VADDPH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+46071, 567},
    /* 6924 : VADDSH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.f3.map5.w0 58 /r     ] AVX512FP16,FUTURE */
        /* 241(f5 7e 08) 306 001(58) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VADDSH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+46089, 568},
    /* 6925 : VCMPPH kreg|mask,xmmreg*,xmmrm128|b16,imm8 [rvmi:fv:  evex.nds.128.np.0f3a.w0 C2 /r ib  ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f3 7c 08) 301 001(c2) 120 023 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCMPPH, 4, {OPMASKREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK,0,B16,0,0}, nasm_bytecodes+22703, 566},
    /* 6926 : VCMPPH kreg|mask,ymmreg*,ymmrm256|b16,imm8 [rvmi:fv:  evex.nds.256.np.0f3a.w0 C2 /r ib  ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f3 7c 28) 301 001(c2) 120 023 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCMPPH, 4, {OPMASKREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK,0,B16,0,0}, nasm_bytecodes+22723, 566},
    /* 6927 : VCMPPH kreg|mask,zmmreg*,zmmrm512|b16|sae,imm8 [rvmi:fv:  evex.nds.512.np.0f3a.w0 C2 /r ib  ] AVX512FP16,FUTURE */
        /* 241(f3 7c 48) 301 001(c2) 120 023 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCMPPH, 4, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK,0,B16|SAE,0,0}, nasm_bytecodes+22743, 567},
    /* 6928 : VCMPSH kreg|mask,xmmreg*,xmmrm16|sae,imm8 [rvmi:t1s: evex.nds.lig.f3.0f3a.w0 C2 /r ib  ] AVX512FP16,FUTURE */
        /* 241(f3 7e 08) 306 001(c2) 120 023 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCMPSH, 4, {OPMASKREG,XMMREG,RM_XMM|BITS16,IMM_NORMAL|BITS8,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+22763, 568},
    /* 6929 : VCOMISH xmmreg,xmmrm16|sae [rm:fv:    evex.lig.np.map5.w0 2F /r         ] FL,AVX512FP16,FUTURE */
        /* 250(f5 7c 08) 301 001(2f) 110 : LIG,FL,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCOMISH, 2, {XMMREG,RM_XMM|BITS16,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+46107, 636},
    /* 6930 : VCVTDQ2PH xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.np.map5.w0 5B /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 08) 301 001(5b) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTDQ2PH, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+46116, 566},
    /* 6931 : VCVTDQ2PH ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.np.map5.w0 5B /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 28) 301 001(5b) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTDQ2PH, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+46125, 566},
    /* 6932 : VCVTDQ2PH zmmreg|mask|z,zmmrm512|b32|er [rm:fv:    evex.512.np.map5.w0 5B /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7c 48) 301 001(5b) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTDQ2PH, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|ER,0,0,0}, nasm_bytecodes+46134, 567},
    /* 6933 : VCVTPD2PH xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.map5.w1 5A /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 fd 08) 301 001(5a) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPD2PH, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+46143, 566},
    /* 6934 : VCVTPD2PH ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.map5.w1 5A /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 fd 28) 301 001(5a) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPD2PH, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+46152, 566},
    /* 6935 : VCVTPD2PH zmmreg|mask|z,zmmrm512|b64|er [rm:fv:    evex.512.66.map5.w1 5A /r         ] AVX512FP16,FUTURE */
        /* 250(f5 fd 48) 301 001(5a) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTPD2PH, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|ER,0,0,0}, nasm_bytecodes+46161, 567},
    /* 6936 : VCVTPH2DQ xmmreg|mask|z,xmmrm64|b16 [rm:hv:    evex.128.66.map5.w0 5B /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 08) 302 001(5b) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2DQ, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46170, 566},
    /* 6937 : VCVTPH2DQ ymmreg|mask|z,xmmrm128|b16 [rm:hv:    evex.256.66.map5.w0 5B /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 28) 302 001(5b) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2DQ, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46179, 566},
    /* 6938 : VCVTPH2DQ zmmreg|mask|z,ymmrm256|b16|er [rm:hv:    evex.512.66.map5.w0 5B /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7d 48) 302 001(5b) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTPH2DQ, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16|ER,0,0,0}, nasm_bytecodes+46188, 567},
    /* 6939 : VCVTPH2PD xmmreg|mask|z,xmmrm32|b16 [rm:qvm:   evex.128.np.map5.w0 5A /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 08) 315 001(5a) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2PD, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46197, 566},
    /* 6940 : VCVTPH2PD ymmreg|mask|z,xmmrm64|b16 [rm:qvm:   evex.256.np.map5.w0 5A /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 28) 315 001(5a) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2PD, 2, {YMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46206, 566},
    /* 6941 : VCVTPH2PD zmmreg|mask|z,xmmrm128|b16|sae [rm:qvm:   evex.512.np.map5.w0 5A /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7c 48) 315 001(5a) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTPH2PD, 2, {ZMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+46215, 567},
    /* 6942 : VCVTPH2PS xmmreg|mask|z,xmmrm64 [rm:hvm:   evex.128.66.0f38.w0 13 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 314 001(13) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPH2PS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31410, 544},
    /* 6943 : VCVTPH2PS ymmreg|mask|z,xmmrm128 [rm:hvm:   evex.256.66.0f38.w0 13 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 314 001(13) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPH2PS, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31419, 544},
    /* 6944 : VCVTPH2PS zmmreg|mask|z,ymmrm256|sae [rm:hvm:   evex.512.66.0f38.w0 13 /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 314 001(13) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTPH2PS, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,SAE,0,0,0}, nasm_bytecodes+31428, 545},
    /* 6945 : VCVTPH2PSX xmmreg|mask|z,xmmrm64|b16 [rm:hv:    evex.128.66.map6.w0 13 /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f6 7d 08) 302 001(13) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2PSX, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46224, 566},
    /* 6946 : VCVTPH2PSX ymmreg|mask|z,xmmrm128|b16 [rm:hv:    evex.256.66.map6.w0 13 /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f6 7d 28) 302 001(13) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2PSX, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46233, 566},
    /* 6947 : VCVTPH2PSX zmmreg|mask|z,ymmrm256|b16|sae [rm:hv:    evex.512.66.map6.w0 13 /r         ] AVX512FP16,FUTURE */
        /* 250(f6 7d 48) 302 001(13) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTPH2PSX, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+46242, 567},
    /* 6948 : VCVTPH2QQ xmmreg|mask|z,xmmrm32|b16 [rm:qvm:   evex.128.66.map5.w0 7b /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 08) 315 001(7b) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2QQ, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46251, 566},
    /* 6949 : VCVTPH2QQ ymmreg|mask|z,xmmrm64|b16 [rm:qvm:   evex.256.66.map5.w0 7b /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 28) 315 001(7b) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2QQ, 2, {YMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46260, 566},
    /* 6950 : VCVTPH2QQ zmmreg|mask|z,xmmrm128|b16|er [rm:qvm:   evex.512.66.map5.w0 7b /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7d 48) 315 001(7b) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTPH2QQ, 2, {ZMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16|ER,0,0,0}, nasm_bytecodes+46269, 567},
    /* 6951 : VCVTPH2UDQ xmmreg|mask|z,xmmrm32|b16 [rm:hv:    evex.128.map5.w0 79 /r            ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 08) 302 001(79) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2UDQ, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46278, 566},
    /* 6952 : VCVTPH2UDQ ymmreg|mask|z,xmmrm64|b16 [rm:hv:    evex.256.map5.w0 79 /r            ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 28) 302 001(79) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2UDQ, 2, {YMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46287, 566},
    /* 6953 : VCVTPH2UDQ zmmreg|mask|z,xmmrm128|b16|er [rm:hv:    evex.512.map5.w0 79 /r            ] AVX512FP16,FUTURE */
        /* 250(f5 7c 48) 302 001(79) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTPH2UDQ, 2, {ZMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16|ER,0,0,0}, nasm_bytecodes+46296, 567},
    /* 6954 : VCVTPH2UQQ xmmreg|mask|z,xmmrm32|b16 [rm:qvm:   evex.128.66.map5.w0 79 /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 08) 315 001(79) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2UQQ, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46305, 566},
    /* 6955 : VCVTPH2UQQ ymmreg|mask|z,xmmrm64|b16 [rm:qvm:   evex.256.66.map5.w0 79 /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 28) 315 001(79) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2UQQ, 2, {YMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46314, 566},
    /* 6956 : VCVTPH2UQQ zmmreg|mask|z,xmmrm128|b16|er [rm:qvm:   evex.512.66.map5.w0 79 /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7d 48) 315 001(79) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTPH2UQQ, 2, {ZMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16|ER,0,0,0}, nasm_bytecodes+46323, 567},
    /* 6957 : VCVTPH2UW xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.np.map5.w0 7d /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 08) 301 001(7d) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2UW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46332, 566},
    /* 6958 : VCVTPH2UW ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.np.map5.w0 7d /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 28) 301 001(7d) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2UW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46341, 566},
    /* 6959 : VCVTPH2UW zmmreg|mask|z,zmmrm512|b16|er [rm:fv:    evex.512.np.map5.w0 7d /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7c 48) 301 001(7d) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTPH2UW, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16|ER,0,0,0}, nasm_bytecodes+46350, 567},
    /* 6960 : VCVTPH2W xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.66.map5.w0 7d /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 08) 301 001(7d) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2W, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46359, 566},
    /* 6961 : VCVTPH2W ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.66.map5.w0 7d /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 28) 301 001(7d) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2W, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46368, 566},
    /* 6962 : VCVTPH2W zmmreg|mask|z,zmmrm512|b16|er [rm:fv:    evex.512.66.map5.w0 7d /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7d 48) 301 001(7d) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTPH2W, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16|ER,0,0,0}, nasm_bytecodes+46377, 567},
    /* 6963 : VCVTPS2PH xmmreg|mask|z,xmmreg,imm8 [mri:hvm:  evex.128.66.0f3a.w0 1d /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 08) 314 001(1d) 101 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2PH, 3, {XMMREG,XMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18333, 544},
    /* 6964 : VCVTPS2PH mem64|mask,xmmreg,imm8 [mri:hvm:  evex.128.66.0f3a.w0 1d /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 08) 314 001(1d) 101 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2PH, 3, {MEMORY|BITS64,XMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18333, 544},
    /* 6965 : VCVTPS2PH xmmreg|mask|z,ymmreg,imm8 [mri:hvm:  evex.256.66.0f3a.w0 1d /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 28) 314 001(1d) 101 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2PH, 3, {XMMREG,YMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18343, 544},
    /* 6966 : VCVTPS2PH mem128|mask,ymmreg,imm8 [mri:hvm:  evex.256.66.0f3a.w0 1d /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 28) 314 001(1d) 101 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2PH, 3, {MEMORY|BITS128,YMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18343, 544},
    /* 6967 : VCVTPS2PH ymmreg|mask|z,zmmreg|sae,imm8 [mri:hvm:  evex.512.66.0f3a.w0 1d /r ib      ] AVX512,FUTURE */
        /* 250(f3 7d 48) 314 001(1d) 101 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTPS2PH, 3, {YMMREG,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,SAE,0,0,0}, nasm_bytecodes+18353, 545},
    /* 6968 : VCVTPS2PH mem256|mask,zmmreg|sae,imm8 [mri:hvm:  evex.512.66.0f3a.w0 1d /r ib      ] AVX512,FUTURE */
        /* 250(f3 7d 48) 314 001(1d) 101 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTPS2PH, 3, {MEMORY|BITS256,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,SAE,0,0,0}, nasm_bytecodes+18353, 545},
    /* 6969 : VCVTPS2PHX xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.66.map5.w0 1d /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 08) 301 001(1d) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPS2PHX, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+46386, 566},
    /* 6970 : VCVTPS2PHX xmmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.66.map5.w0 1d /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 28) 301 001(1d) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPS2PHX, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+46395, 566},
    /* 6971 : VCVTPS2PHX ymmreg|mask|z,zmmrm512|b32|er [rm:fv:    evex.512.66.map5.w0 1d /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7d 48) 301 001(1d) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTPS2PHX, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|ER,0,0,0}, nasm_bytecodes+46404, 567},
    /* 6972 : VCVTQQ2PH xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.np.map5.w1 5b /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 fc 08) 301 001(5b) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTQQ2PH, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+46413, 566},
    /* 6973 : VCVTQQ2PH ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.np.map5.w1 5b /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 fc 28) 301 001(5b) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTQQ2PH, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+46422, 566},
    /* 6974 : VCVTQQ2PH zmmreg|mask|z,zmmrm512|b64|er [rm:fv:    evex.512.np.map5.w1 5b /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 fc 48) 301 001(5b) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTQQ2PH, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|ER,0,0,0}, nasm_bytecodes+46431, 566},
    /* 6975 : VCVTSD2SH xmmreg|mask|z,xmmreg*,xmmrm64|er [rvm:t1s:  evex.nds.lig.f2.map5.w1 5a /r     ] AVX512FP16,FUTURE */
        /* 241(f5 ff 08) 306 001(5a) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTSD2SH, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+46440, 568},
    /* 6976 : VCVTSH2SD xmmreg,xmmreg*,xmmrm16|sae [rvm:t1s:  evex.nds.lig.f3.map5.w0 5a /r     ] AVX512FP16,FUTURE */
        /* 241(f5 7e 08) 306 001(5a) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTSH2SD, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {0,0,SAE,0,0}, nasm_bytecodes+46458, 568},
    /* 6977 : VCVTSH2SI reg32,xmmrm16|er [rm:t1s:   evex.lig.f3.map5.w0 2d /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7e 08) 306 001(2d) 110 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTSH2SI, 2, {REG_GPR|BITS32,RM_XMM|BITS16,0,0,0}, {0,ER,0,0,0}, nasm_bytecodes+46476, 568},
    /* 6978 : VCVTSH2SI reg64,xmmrm16|er [rm:t1s:   evex.lig.f3.map5.w1 2d /r         ] AVX512FP16,FUTURE */
        /* 250(f5 fe 08) 306 001(2d) 110 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTSH2SI, 2, {REG_GPR|BITS64,RM_XMM|BITS16,0,0,0}, {0,ER,0,0,0}, nasm_bytecodes+46485, 568},
    /* 6979 : VCVTSH2SS xmmreg|mask|z,xmmreg*,xmmrm16|sae [rvm:t1s:  evex.nds.lig.map6.w0 13 /r        ] AVX512FP16,FUTURE */
        /* 241(f6 7c 08) 306 001(13) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTSH2SS, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+46494, 568},
    /* 6980 : VCVTSH2USI reg32,xmmrm16|er [rm:t1s:   evex.lig.f3.map5.w0 79 /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7e 08) 306 001(79) 110 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTSH2USI, 2, {REG_GPR|BITS32,RM_XMM|BITS16,0,0,0}, {0,ER,0,0,0}, nasm_bytecodes+46512, 568},
    /* 6981 : VCVTSH2USI reg64,xmmrm16|er [rm:t1s:   evex.lig.f3.map5.w1 79 /r         ] AVX512FP16,FUTURE */
        /* 250(f5 fe 08) 306 001(79) 110 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTSH2USI, 2, {REG_GPR|BITS64,RM_XMM|BITS16,0,0,0}, {0,ER,0,0,0}, nasm_bytecodes+46521, 568},
    /* 6982 : VCVTSI2SH xmmreg,xmmreg*,rm32|er [rvm:t1s:  evex.nds.lig.f3.map5.w0 2a /r     ] AVX512FP16,FUTURE */
        /* 241(f5 7e 08) 306 001(2a) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTSI2SH, 3, {XMMREG,XMMREG,RM_GPR|BITS32,0,0}, {0,0,ER,0,0}, nasm_bytecodes+46530, 568},
    /* 6983 : VCVTSI2SH xmmreg,xmmreg*,rm64|er [rvm:t1s:  evex.nds.lig.f3.map5.w1 2a /r     ] AVX512FP16,FUTURE */
        /* 241(f5 fe 08) 306 001(2a) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTSI2SH, 3, {XMMREG,XMMREG,RM_GPR|BITS64,0,0}, {0,0,ER,0,0}, nasm_bytecodes+46548, 568},
    /* 6984 : VCVTSS2SH xmmreg,xmmreg*,xmmrm32|er [rvm:t1s:  evex.nds.lig.np.map5 1d /r        ] AVX512FP16,FUTURE */
        /* 241(f5 7c 08) 306 001(1d) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTSS2SH, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {0,0,ER,0,0}, nasm_bytecodes+46566, 568},
    /* 6985 : VCVTTPH2DQ xmmreg|mask|z,xmmrm64|b16 [rm:hv:    evex.128.f3.map5.w0 5b /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7e 08) 302 001(5b) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTTPH2DQ, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46584, 566},
    /* 6986 : VCVTTPH2DQ ymmreg|mask|z,xmmrm128|b16 [rm:hv:    evex.256.f3.map5.w0 5b /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7e 28) 302 001(5b) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTTPH2DQ, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46593, 566},
    /* 6987 : VCVTTPH2DQ zmmreg|mask|z,ymmrm256|b16|sae [rm:hv:    evex.512.f3.map5.w0 5b /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7e 48) 302 001(5b) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTTPH2DQ, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+46602, 567},
    /* 6988 : VCVTTPH2QQ xmmreg|mask|z,xmmrm32|b16 [rm:t1s:   evex.128.66.map5.w0 7a /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 08) 306 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTTPH2QQ, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46611, 566},
    /* 6989 : VCVTTPH2QQ ymmreg|mask|z,xmmrm64|b16 [rm:t1s:   evex.256.66.map5.w0 7a /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 28) 306 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTTPH2QQ, 2, {YMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46620, 566},
    /* 6990 : VCVTTPH2QQ zmmreg|mask|z,xmmrm128|b16|sae [rm:t1s:   evex.512.66.map5.w0 7a /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7d 48) 306 001(7a) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTTPH2QQ, 2, {ZMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+46629, 567},
    /* 6991 : VCVTTPH2UDQ xmmreg|mask|z,xmmrm64|b16 [rm:hv:    evex.128.np.map5.w0 78 /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 08) 302 001(78) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTTPH2UDQ, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46638, 566},
    /* 6992 : VCVTTPH2UDQ ymmreg|mask|z,xmmrm128|b16 [rm:hv:    evex.256.np.map5.w0 78 /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 28) 302 001(78) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTTPH2UDQ, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46647, 566},
    /* 6993 : VCVTTPH2UDQ zmmreg|mask|z,ymmrm256|b16|sae [rm:hv:    evex.512.np.map5.w0 78 /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7c 48) 302 001(78) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTTPH2UDQ, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+46656, 567},
    /* 6994 : VCVTTPH2UQQ xmmreg|mask|z,xmmrm32|b16 [rm:t1s:   evex.128.66.map5.w0 78 /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 08) 306 001(78) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTTPH2UQQ, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46665, 566},
    /* 6995 : VCVTTPH2UQQ ymmreg|mask|z,xmmrm64|b16 [rm:t1s:   evex.256.66.map5.w0 78 /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 28) 306 001(78) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTTPH2UQQ, 2, {YMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46674, 566},
    /* 6996 : VCVTTPH2UQQ zmmreg|mask|z,xmmrm128|b16|sae [rm:t1s:   evex.512.66.map5.w0 78 /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7d 48) 306 001(78) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTTPH2UQQ, 2, {ZMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+46683, 567},
    /* 6997 : VCVTTPH2UW xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.np.map5.w0 7c /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 08) 301 001(7c) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTTPH2UW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46692, 566},
    /* 6998 : VCVTTPH2UW ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.np.map5.w0 7c /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 28) 301 001(7c) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTTPH2UW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46701, 566},
    /* 6999 : VCVTTPH2UW zmmreg|mask|z,zmmrm512|b16|sae [rm:fv:    evex.512.np.map5.w0 7c /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7c 48) 301 001(7c) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTTPH2UW, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+46710, 567},
    /* 7000 : VCVTTPH2W xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.66.map5.w0 7c /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 08) 301 001(7c) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTTPH2W, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46719, 566},
    /* 7001 : VCVTTPH2W ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.66.map5.w0 7c /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 28) 301 001(7c) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTTPH2W, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46728, 566},
    /* 7002 : VCVTTPH2W zmmreg|mask|z,zmmrm512|b16|sae [rm:fv:    evex.512.66.map5.w0 7c /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7d 48) 301 001(7c) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTTPH2W, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+46737, 567},
    /* 7003 : VCVTTSH2SI reg32,xmmrm16|sae [rm:t1s:   evex.lig.f3.map5.w0 2c /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7e 08) 306 001(2c) 110 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTTSH2SI, 2, {REG_GPR|BITS32,RM_XMM|BITS16,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+46746, 568},
    /* 7004 : VCVTTSH2SI reg64,xmmrm16|sae [rm:t1s:   evex.lig.f3.map5.w1 2c /r         ] AVX512FP16,FUTURE */
        /* 250(f5 fe 08) 306 001(2c) 110 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTTSH2SI, 2, {REG_GPR|BITS64,RM_XMM|BITS16,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+46755, 568},
    /* 7005 : VCVTTSH2USI reg32,xmmrm16|sae [rm:t1s:   evex.lig.f3.map5.w0 78 /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7e 08) 306 001(78) 110 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTTSH2USI, 2, {REG_GPR|BITS32,RM_XMM|BITS16,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+46764, 568},
    /* 7006 : VCVTTSH2USI reg64,xmmrm16|sae [rm:t1s:   evex.lig.f3.map5.w1 78 /r         ] AVX512FP16,FUTURE */
        /* 250(f5 fe 08) 306 001(78) 110 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTTSH2USI, 2, {REG_GPR|BITS64,RM_XMM|BITS16,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+46773, 568},
    /* 7007 : VCVTUDQ2PH xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.f2.map5.w0 7a /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7f 08) 301 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTUDQ2PH, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+46782, 566},
    /* 7008 : VCVTUDQ2PH ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.f2.map5.w0 7a /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7f 28) 301 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTUDQ2PH, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+46791, 566},
    /* 7009 : VCVTUDQ2PH zmmreg|mask|z,zmmrm512|b32 [rm:fv:    evex.512.f2.map5.w0 7a /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7f 48) 301 001(7a) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTUDQ2PH, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+46800, 567},
    /* 7010 : VCVTUQQ2PH xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.f2.map5.w1 7a /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 ff 08) 301 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTUQQ2PH, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+46809, 566},
    /* 7011 : VCVTUQQ2PH ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.f2.map5.w1 7a /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 ff 28) 301 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTUQQ2PH, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+46818, 566},
    /* 7012 : VCVTUQQ2PH zmmreg|mask|z,zmmrm512|b32 [rm:fv:    evex.512.f2.map5.w1 7a /r         ] AVX512FP16,FUTURE */
        /* 250(f5 ff 48) 301 001(7a) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTUQQ2PH, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+46827, 567},
    /* 7013 : VCVTUSI2SH xmmreg,xmmreg|er,rm32|er [rvm:t1s:  evex.nds.lig.f3.map5.w0 7b /r     ] AVX512FP16,FUTURE */
        /* 241(f5 7e 08) 306 001(7b) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTUSI2SH, 3, {XMMREG,XMMREG,RM_GPR|BITS32,0,0}, {0,ER,ER,0,0}, nasm_bytecodes+46836, 568},
    /* 7014 : VCVTUSI2SS xmmreg,xmmreg|er,rm64|er [rvm:t1s:  evex.nds.lig.f3.map5.w1 7b /r     ] AVX512FP16,FUTURE */
        /* 241(f5 fe 08) 306 001(7b) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTUSI2SS, 3, {XMMREG,XMMREG,RM_GPR|BITS64,0,0}, {0,ER,ER,0,0}, nasm_bytecodes+46845, 568},
    /* 7015 : VCVTUW2PH xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.f2.map5.w0 7d /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7f 08) 301 001(7d) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTUW2PH, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46854, 566},
    /* 7016 : VCVTUW2PH ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.f2.map5.w0 7d /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7f 28) 301 001(7d) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTUW2PH, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46863, 566},
    /* 7017 : VCVTUW2PH zmmreg|mask|z,zmmrm512|b16|er [rm:fv:    evex.512.f2.map5.w0 7d /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7f 48) 301 001(7d) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTUW2PH, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16|ER,0,0,0}, nasm_bytecodes+46872, 567},
    /* 7018 : VCVTW2PH xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.f3.map5.w0 7d /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7e 08) 301 001(7d) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTW2PH, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46881, 566},
    /* 7019 : VCVTW2PH ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.f3.map5.w0 7d /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7e 28) 301 001(7d) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTW2PH, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46890, 566},
    /* 7020 : VCVTW2PH zmmreg|mask|z,zmmrm512|b16|er [rm:fv:    evex.512.f3.map5.w0 7d /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7e 48) 301 001(7d) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTW2PH, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16|ER,0,0,0}, nasm_bytecodes+46899, 567},
    /* 7021 : VDIVPH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.np.map5 5e /r        ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 08) 301 001(5e) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VDIVPH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+46908, 566},
    /* 7022 : VDIVPH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.np.map5 5e /r        ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 28) 301 001(5e) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VDIVPH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+46926, 566},
    /* 7023 : VDIVPH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.np.map5 5e /r        ] AVX512FP16,FUTURE */
        /* 241(f5 7c 48) 301 001(5e) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VDIVPH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+46944, 567},
    /* 7024 : VDIVSH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.f3.map5 5e /r        ] AVX512FP16,FUTURE */
        /* 241(f5 7e 08) 306 001(5e) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VDIVSH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+46962, 568},
    /* 7025 : VFCMADDCPH xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.f2.map6.w0 56 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7f 08) 301 001(56) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFCMADDCPH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+46980, 566},
    /* 7026 : VFCMADDCPH ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.f2.map6.w0 56 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7f 28) 301 001(56) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFCMADDCPH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+46998, 566},
    /* 7027 : VFCMADDCPH zmmreg|mask|z,zmmreg*,zmmrm512|b32|er [rvm:fv:   evex.nds.512.f2.map6.w0 56 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7f 48) 301 001(56) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFCMADDCPH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+47016, 566},
    /* 7028 : VFMADDCPH xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.f3.map6.w0 56 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7e 08) 301 001(56) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADDCPH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+47034, 566},
    /* 7029 : VFMADDCPH ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.f3.map6.w0 56 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7e 28) 301 001(56) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADDCPH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+47052, 566},
    /* 7030 : VFMADDCPH zmmreg|mask|z,zmmreg*,zmmrm512|b32|er [rvm:fv:   evex.nds.512.f3.map6.w0 56 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7e 48) 301 001(56) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADDCPH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+47070, 566},
    /* 7031 : VFCMADDCSH xmmreg|mask|z,xmmreg*,xmmrm32|er [rvm:t1s:  evex.nds.lig.f2.map6.w0 57 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7f 08) 306 001(57) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFCMADDCSH, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+47088, 568},
    /* 7032 : VFMADDCSH xmmreg|mask|z,xmmreg*,xmmrm32|er [rvm:t1s:  evex.nds.lig.f3.map6.w0 57 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7e 08) 306 001(57) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMADDCSH, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+47106, 568},
    /* 7033 : VFCMULCPH xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.f2.map6.w0 d6 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7f 08) 301 001(d6) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFCMULCPH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+47124, 566},
    /* 7034 : VFCMULCPH ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.f2.map6.w0 d6 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7f 28) 301 001(d6) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFCMULCPH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+47142, 566},
    /* 7035 : VFCMULCPH zmmreg|mask|z,zmmreg*,zmmrm512|b32|er [rvm:fv:   evex.nds.512.f2.map6.w0 d6 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7f 48) 301 001(d6) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFCMULCPH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+47160, 567},
    /* 7036 : VFMULCPH xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.f3.map6.w0 d6 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7e 08) 301 001(d6) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMULCPH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+47178, 566},
    /* 7037 : VFMULCPH ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.f3.map6.w0 d6 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7e 28) 301 001(d6) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMULCPH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+47196, 566},
    /* 7038 : VFMULCPH zmmreg|mask|z,zmmreg*,zmmrm512|b32|er [rvm:fv:   evex.nds.512.f3.map6.w0 d6 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7e 48) 301 001(d6) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMULCPH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+47214, 567},
    /* 7039 : VFCMULCSH xmmreg|mask|z,xmmreg*,xmmrm32|er [rvm:t1s:  evex.nds.lig.f2.map6.w0 d7 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7f 08) 306 001(d7) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFCMULCSH, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+47232, 568},
    /* 7040 : VFMULCSH xmmreg|mask|z,xmmreg*,xmmrm32|er [rvm:t1s:  evex.nds.lig.f3.map6.w0 d7 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7e 08) 306 001(d7) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMULCSH, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+47250, 568},
    /* 7041 : VFMADDSUB132PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 96 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(96) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADDSUB132PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47268, 566},
    /* 7042 : VFMADDSUB132PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 96 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(96) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADDSUB132PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47286, 566},
    /* 7043 : VFMADDSUB132PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 96 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(96) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMADDSUB132PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+47304, 567},
    /* 7044 : VFMADDSUB213PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 a6 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(a6) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADDSUB213PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47322, 566},
    /* 7045 : VFMADDSUB213PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 a6 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(a6) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADDSUB213PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47340, 566},
    /* 7046 : VFMADDSUB213PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 a6 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(a6) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMADDSUB213PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+47358, 567},
    /* 7047 : VFMADDSUB231PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 b6 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(b6) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADDSUB231PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47376, 566},
    /* 7048 : VFMADDSUB231PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 b6 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(b6) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADDSUB231PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47394, 566},
    /* 7049 : VFMADDSUB231PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 b6 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(b6) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMADDSUB231PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+47412, 567},
    /* 7050 : VFMSUBADD132PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 97 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(97) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMSUBADD132PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47430, 566},
    /* 7051 : VFMSUBADD132PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 97 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(97) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMSUBADD132PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47448, 566},
    /* 7052 : VFMSUBADD132PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 97 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(97) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMSUBADD132PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+47466, 567},
    /* 7053 : VFMSUBADD213PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 a7 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(a7) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMSUBADD213PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47484, 566},
    /* 7054 : VFMSUBADD213PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 a7 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(a7) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMSUBADD213PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47502, 566},
    /* 7055 : VFMSUBADD213PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 a7 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(a7) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMSUBADD213PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+47520, 567},
    /* 7056 : VFMSUBADD231PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 b7 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(b7) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMSUBADD231PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47538, 566},
    /* 7057 : VFMSUBADD231PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 b7 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(b7) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMSUBADD231PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47556, 566},
    /* 7058 : VFMSUBADD231PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 b7 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(b7) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMSUBADD231PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+47574, 567},
    /* 7059 : VFMADD132PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 98 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(98) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADD132PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47592, 566},
    /* 7060 : VFMADD132PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 98 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(98) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADD132PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47610, 566},
    /* 7061 : VFMADD132PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 98 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(98) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMADD132PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+47628, 567},
    /* 7062 : VFMADD213PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 a8 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(a8) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADD213PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47646, 566},
    /* 7063 : VFMADD213PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 a8 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(a8) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADD213PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47664, 566},
    /* 7064 : VFMADD213PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 a8 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(a8) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMADD213PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+47682, 567},
    /* 7065 : VFMADD231PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 b8 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(b8) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADD231PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47700, 566},
    /* 7066 : VFMADD231PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 b8 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(b8) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADD231PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47718, 566},
    /* 7067 : VFMADD231PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 b8 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(b8) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMADD231PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+47736, 567},
    /* 7068 : VFNMADD132PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 9c /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(9c) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFNMADD132PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47754, 566},
    /* 7069 : VFNMADD132PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 9c /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(9c) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFNMADD132PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47772, 566},
    /* 7070 : VFNMADD132PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 9c /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(9c) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFNMADD132PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+47790, 567},
    /* 7071 : VFNMADD213PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 ac /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(ac) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFNMADD213PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47808, 566},
    /* 7072 : VFNMADD213PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 ac /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(ac) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFNMADD213PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47826, 566},
    /* 7073 : VFNMADD213PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 ac /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(ac) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFNMADD213PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+47844, 567},
    /* 7074 : VFNMADD231PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 bc /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(bc) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFNMADD231PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47862, 566},
    /* 7075 : VFNMADD231PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 bc /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(bc) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFNMADD231PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47880, 566},
    /* 7076 : VFNMADD231PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 bc /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(bc) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFNMADD231PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+47898, 567},
    /* 7077 : VFMADD132SH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.66.map6.w0 99 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(99) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMADD132SH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+47916, 568},
    /* 7078 : VFMADD213SH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.66.map6.w0 a9 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(a9) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMADD213SH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+47934, 568},
    /* 7079 : VFMADD231SH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.66.map6.w0 b9 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(b9) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMADD231SH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+47952, 568},
    /* 7080 : VFNMADD132SH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.66.map6.w0 9d /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(9d) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFNMADD132SH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+47970, 568},
    /* 7081 : VFNMADD213SH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.66.map6.w0 ad /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(ad) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFNMADD213SH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+47988, 568},
    /* 7082 : VFNMADD231SH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.66.map6.w0 bd /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(bd) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFNMADD231SH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+48006, 568},
    /* 7083 : VFMSUB132PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 9a /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(9a) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMSUB132PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48024, 566},
    /* 7084 : VFMSUB132PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 9a /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(9a) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMSUB132PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48042, 566},
    /* 7085 : VFMSUB132PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 9a /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(9a) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMSUB132PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+48060, 567},
    /* 7086 : VFMSUB213PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 aa /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(aa) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMSUB213PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48078, 566},
    /* 7087 : VFMSUB213PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 aa /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(aa) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMSUB213PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48096, 566},
    /* 7088 : VFMSUB213PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 aa /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(aa) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMSUB213PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+48114, 567},
    /* 7089 : VFMSUB231PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 ba /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(ba) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMSUB231PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48132, 566},
    /* 7090 : VFMSUB231PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 ba /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(ba) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMSUB231PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48150, 566},
    /* 7091 : VFMSUB231PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 ba /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(ba) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMSUB231PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+48168, 567},
    /* 7092 : VFNMSUB132PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 9e /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(9e) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFNMSUB132PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48186, 566},
    /* 7093 : VFNMSUB132PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 9e /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(9e) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFNMSUB132PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48204, 566},
    /* 7094 : VFNMSUB132PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 9e /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(9e) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFNMSUB132PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+48222, 567},
    /* 7095 : VFNMSUB213PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 ae /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(ae) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFNMSUB213PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48240, 566},
    /* 7096 : VFNMSUB213PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 ae /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(ae) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFNMSUB213PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48258, 566},
    /* 7097 : VFNMSUB213PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 ae /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(ae) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFNMSUB213PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+48276, 567},
    /* 7098 : VFNMSUB231PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 be /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(be) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFNMSUB231PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48294, 566},
    /* 7099 : VFNMSUB231PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 be /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(be) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFNMSUB231PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48312, 566},
    /* 7100 : VFNMSUB231PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 be /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(be) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFNMSUB231PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+48330, 567},
    /* 7101 : VFMSUB132SH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.66.map6.w0 9b /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(9b) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMSUB132SH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+48348, 568},
    /* 7102 : VFMSUB213SH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.66.map6.w0 ab /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(ab) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMSUB213SH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+48366, 568},
    /* 7103 : VFMSUB231SH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.66.map6.w0 bb /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(bb) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMSUB231SH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+48384, 568},
    /* 7104 : VFNMSUB132SH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.66.map6.w0 9f /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(9f) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFNMSUB132SH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+48402, 568},
    /* 7105 : VFNMSUB213SH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.66.map6.w0 af /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(af) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFNMSUB213SH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+48420, 568},
    /* 7106 : VFNMSUB231SH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.66.map6.w0 bf /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(bf) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFNMSUB231SH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+48438, 568},
    /* 7107 : VFPCLASSPH kreg|mask,xmmrm128|b16,imm8 [rmi:fv:   evex.128.np.0f3a.w0 66 /r ib      ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f3 7c 08) 301 001(66) 110 022 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFPCLASSPH, 3, {OPMASKREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK,B16,0,0,0}, nasm_bytecodes+22783, 566},
    /* 7108 : VFPCLASSPH kreg|mask,ymmrm256|b16,imm8 [rmi:fv:   evex.256.np.0f3a.w0 66 /r ib      ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f3 7c 28) 301 001(66) 110 022 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFPCLASSPH, 3, {OPMASKREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK,B16,0,0,0}, nasm_bytecodes+22793, 566},
    /* 7109 : VFPCLASSPH kreg|mask,zmmrm512|b16,imm8 [rmi:fv:   evex.512.np.0f3a.w0 66 /r ib      ] AVX512FP16,FUTURE */
        /* 250(f3 7c 48) 301 001(66) 110 022 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFPCLASSPH, 3, {OPMASKREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK,B16,0,0,0}, nasm_bytecodes+22803, 567},
    /* 7110 : VFPCLASSSH kreg|mask,xmmrm16,imm8 [rmi:t1s:  evex.lig.np.0f3a.w0 67 /r ib      ] AVX512FP16,FUTURE */
        /* 250(f3 7c 08) 306 001(67) 110 022 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFPCLASSSH, 3, {OPMASKREG,RM_XMM|BITS16,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+22813, 568},
    /* 7111 : VGETEXPPH xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.66.map6.w0 42 /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f6 7d 08) 301 001(42) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VGETEXPPH, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+48456, 566},
    /* 7112 : VGETEXPPH ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.66.map6.w0 42 /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f6 7d 28) 301 001(42) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VGETEXPPH, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+48465, 566},
    /* 7113 : VGETEXPPH zmmreg|mask|z,zmmrm512|b16|sae [rm:fv:    evex.512.66.map6.w0 42 /r         ] AVX512FP16,FUTURE */
        /* 250(f6 7d 48) 301 001(42) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VGETEXPPH, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+48474, 567},
    /* 7114 : VGETEXPSH xmmreg|mask|z,xmmrm16|sae [rm:t1s:   evex.128.66.map6.w0 43 /r         ] AVX512FP16,FUTURE */
        /* 250(f6 7d 08) 306 001(43) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VGETEXPSH, 2, {XMMREG,RM_XMM|BITS16,0,0,0}, {MASK|Z,SAE,0,0,0}, nasm_bytecodes+48483, 567},
    /* 7115 : VGETMANTPH xmmreg|mask|z,xmmrm128|b16,imm8 [rmi:fv:   evex.128.np.0f3a.w0 26 /r ib      ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f3 7c 08) 301 001(26) 110 022 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VGETMANTPH, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+22823, 566},
    /* 7116 : VGETMANTPH ymmreg|mask|z,ymmrm256|b16,imm8 [rmi:fv:   evex.256.np.0f3a.w0 26 /r ib      ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f3 7c 28) 301 001(26) 110 022 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VGETMANTPH, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+22833, 566},
    /* 7117 : VGETMANTPH zmmreg|mask|z,zmmrm512|b16|sae,imm8 [rmi:fv:   evex.512.np.0f3a.w0 26 /r ib      ] AVX512FP16,FUTURE */
        /* 250(f3 7c 48) 301 001(26) 110 022 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VGETMANTPH, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+22843, 567},
    /* 7118 : VGETMANTSH xmmreg|mask|z,xmmrm16|sae,imm8 [rmi:t1s:  evex.128.np.0f3a.w0 27 /r ib      ] AVX512FP16,FUTURE */
        /* 250(f3 7c 08) 306 001(27) 110 022 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VGETMANTSH, 3, {XMMREG,RM_XMM|BITS16,IMM_NORMAL|BITS8,0,0}, {MASK|Z,SAE,0,0,0}, nasm_bytecodes+22853, 567},
    /* 7119 : VMAXPH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.128.np.map5.w0 5f /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 08) 301 001(5f) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VMAXPH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48492, 566},
    /* 7120 : VMAXPH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.256.np.map5.w0 5f /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 28) 301 001(5f) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VMAXPH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48510, 566},
    /* 7121 : VMAXPH zmmreg|mask|z,zmmreg*,zmmrm512|b16|sae [rvm:fv:   evex.512.np.map5.w0 5f /r         ] AVX512FP16,FUTURE */
        /* 241(f5 7c 48) 301 001(5f) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMAXPH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|SAE,0,0}, nasm_bytecodes+48528, 567},
    /* 7122 : VMAXSH xmmreg|mask|z,xmmreg,xmmrm16|sae [rvm:t1s:  evex.lig.f3.map5.w0 5f /r         ] AVX512FP16,FUTURE */
        /* 241(f5 7e 08) 306 001(5f) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMAXSH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+48546, 568},
    /* 7123 : VMINPH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.128.np.map5.w0 5d /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 08) 301 001(5d) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VMINPH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48555, 566},
    /* 7124 : VMINPH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.256.np.map5.w0 5d /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 28) 301 001(5d) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VMINPH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48573, 566},
    /* 7125 : VMINPH zmmreg|mask|z,zmmreg*,zmmrm512|b16|sae [rvm:fv:   evex.512.np.map5.w0 5d /r         ] AVX512FP16,FUTURE */
        /* 241(f5 7c 48) 301 001(5d) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMINPH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|SAE,0,0}, nasm_bytecodes+48591, 567},
    /* 7126 : VMINSH xmmreg|mask|z,xmmreg,xmmrm16|sae [rvm:t1s:  evex.lig.f3.map5.w0 5d /r         ] AVX512FP16,FUTURE */
        /* 241(f5 7e 08) 306 001(5d) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMINSH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+48609, 568},
    /* 7127 : VMOVSH xmmreg|mask|z,mem16 [rm:t1s:   evex.lig.f3.map5.w0 10 /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7e 08) 306 001(10) 110 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMOVSH, 2, {XMMREG,MEMORY|BITS16,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+48618, 568},
    /* 7128 : VMOVSH mem16|mask,xmmreg [mr:t1s:   evex.lig.f3.map5.w0 11 /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7e 08) 306 001(11) 101 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMOVSH, 2, {MEMORY|BITS16,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+48627, 568},
    /* 7129 : VMOVSH xmmreg|mask|z,xmmreg*,xmmreg [rvm:      evex.nds.lig.f3.map5.w0 10 /r     ] AVX512FP16,FUTURE */
        /* 241(f5 7e 08) 300 001(10) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMOVSH, 3, {XMMREG,XMMREG,XMMREG,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+48636, 568},
    /* 7130 : VMOVSH xmmreg|mask|z,xmmreg*,xmmreg [rvm:      evex.nds.lig.f3.map5.w0 11 /r     ] AVX512FP16,FUTURE */
        /* 241(f5 7e 08) 300 001(11) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMOVSH, 3, {XMMREG,XMMREG,XMMREG,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+48654, 568},
    /* 7131 : VMOVW xmmreg|mask|z,rm16 [rm:t1s:   evex.128.66.map5.wig 6e /r        ] AVX512FP16,FUTURE */
        /* 250(f5 7d 08) 306 001(6e) 110 : WIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMOVW, 2, {XMMREG,RM_GPR|BITS16,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+48672, 637},
    /* 7132 : VMOVW rm16,xmmreg [mr:t1s:   evex.128.66.map5.wig 7e /r        ] AVX512FP16,FUTURE */
        /* 250(f5 7d 08) 306 001(7e) 101 : WIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMOVW, 2, {RM_GPR|BITS16,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+48681, 637},
    /* 7133 : VMULPH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.np.map5.w0 59 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 08) 301 001(59) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VMULPH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48690, 566},
    /* 7134 : VMULPH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.np.map5.w0 59 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 28) 301 001(59) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VMULPH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48708, 566},
    /* 7135 : VMULPH zmmreg|mask|z,zmmreg*,zmmrm512|b16 [rvm:fv:   evex.nds.512.np.map5.w0 59 /r     ] AVX512FP16,FUTURE */
        /* 241(f5 7c 48) 301 001(59) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMULPH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48726, 567},
    /* 7136 : VMULSH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.f3.map5.w0 59 /r     ] AVX512FP16,FUTURE */
        /* 241(f5 7e 08) 306 001(59) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMULSH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+48744, 568},
    /* 7137 : VRCPPH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 4c /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(4c) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VRCPPH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48762, 566},
    /* 7138 : VRCPPH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 4c /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(4c) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VRCPPH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48780, 566},
    /* 7139 : VRCPPH zmmreg|mask|z,zmmreg*,zmmrm512|b16 [rvm:fv:   evex.nds.512.66.map6.w0 4c /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(4c) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VRCPPH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48798, 567},
    /* 7140 : VRCPSH xmmreg|mask|z,xmmreg*,xmmrm16|sae [rvm:t1s:  evex.nds.lig.66.map6.w0 4d /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(4d) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VRCPSH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+48816, 568},
    /* 7141 : VREDUCEPH xmmreg|mask|z,xmmrm128|b16,imm8 [rmi:fv:   evex.128.np.0f3a.w0 56 /r ib      ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f3 7c 08) 301 001(56) 110 022 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VREDUCEPH, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+22863, 566},
    /* 7142 : VREDUCEPH ymmreg|mask|z,ymmrm256|b16,imm8 [rmi:fv:   evex.256.np.0f3a.w0 56 /r ib      ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f3 7c 28) 301 001(56) 110 022 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VREDUCEPH, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+22873, 566},
    /* 7143 : VREDUCEPH zmmreg|mask|z,zmmrm512|b16|sae,imm8 [rmi:fv:   evex.512.np.0f3a.w0 56 /r ib      ] AVX512FP16,FUTURE */
        /* 250(f3 7c 48) 301 001(56) 110 022 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VREDUCEPH, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+22883, 567},
    /* 7144 : VREDUCESH xmmreg|mask|z,xmmreg*,xmmrm16|sae,imm8 [rvmi:t1s: evex.nds.lig.np.0f3a.w0 57 /r ib  ] AVX512FP16,FUTURE */
        /* 241(f3 7c 08) 306 001(57) 120 023 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VREDUCESH, 4, {XMMREG,XMMREG,RM_XMM|BITS16,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+22893, 568},
    /* 7145 : VENDSCALEPH xmmreg|mask|z,xmmrm128|b16,imm8 [rmi:fv:   evex.128.np.0f3a.w0 08 /r ib      ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f3 7c 08) 301 001(08) 110 022 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VENDSCALEPH, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+21563, 566},
    /* 7146 : VENDSCALEPH ymmreg|mask|z,ymmrm256|b16,imm8 [rmi:fv:   evex.256.np.0f3a.w0 08 /r ib      ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f3 7c 28) 301 001(08) 110 022 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VENDSCALEPH, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+21573, 566},
    /* 7147 : VENDSCALEPH zmmreg|mask|z,zmmrm512|b16|sae,imm8 [rmi:fv:   evex.512.np.0f3a.w0 08 /r ib      ] AVX512FP16,FUTURE */
        /* 250(f3 7c 48) 301 001(08) 110 022 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VENDSCALEPH, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+21583, 567},
    /* 7148 : VENDSCALESH xmmreg|mask|z,xmmreg*,xmmrm16|sae,imm8 [rvmi:t1s: evex.nds.lig.np.0f3a.w0 0a /r ib  ] AVX512FP16,FUTURE */
        /* 241(f3 7c 08) 306 001(0a) 120 023 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VENDSCALESH, 4, {XMMREG,XMMREG,RM_XMM|BITS16,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+21643, 568},
    /* 7149 : VRSQRTPH xmmreg|mask|z,xmmrm128|b16,imm8 [rmi:fv:   evex.128.66.map6.w0 4e /r ib      ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f6 7d 08) 301 001(4e) 110 022 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VRSQRTPH, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+22913, 566},
    /* 7150 : VRSQRTPH ymmreg|mask|z,ymmrm256|b16,imm8 [rmi:fv:   evex.256.66.map6.w0 4e /r ib      ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f6 7d 28) 301 001(4e) 110 022 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VRSQRTPH, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+22923, 566},
    /* 7151 : VRSQRTPH zmmreg|mask|z,zmmrm512|b16|sae,imm8 [rmi:fv:   evex.512.66.map6.w0 4e /r ib      ] AVX512FP16,FUTURE */
        /* 250(f6 7d 48) 301 001(4e) 110 022 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VRSQRTPH, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+22933, 567},
    /* 7152 : VRSQRTSH xmmreg|mask|z,xmmreg*,xmmrm16|sae,imm8 [rvmi:t1s: evex.nds.lig.66.map6.w0 4f /r ib  ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(4f) 120 023 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VRSQRTSH, 4, {XMMREG,XMMREG,RM_XMM|BITS16,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+22943, 568},
    /* 7153 : VSCALEFPH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 2c /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(2c) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VSCALEFPH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48834, 566},
    /* 7154 : VSCALEFPH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 2c /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(2c) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VSCALEFPH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48852, 566},
    /* 7155 : VSCALEFPH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 2c /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(2c) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VSCALEFPH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+48870, 567},
    /* 7156 : VSCALEFSH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.66.map6.w0 2d /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(2d) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VSCALEFSH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+48888, 568},
    /* 7157 : VSQRTPH xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.np.map5.w0 51 /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 08) 301 001(51) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VSQRTPH, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+48906, 566},
    /* 7158 : VSQRTPH ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.np.map5.w0 51 /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 28) 301 001(51) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VSQRTPH, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+48915, 566},
    /* 7159 : VSQRTPH zmmreg|mask|z,zmmrm512|b16|er [rm:fv:    evex.512.np.map5.w0 51 /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7c 48) 301 001(51) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VSQRTPH, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16|ER,0,0,0}, nasm_bytecodes+48924, 567},
    /* 7160 : VSQRTSH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.f3.map5.w0 51 /r     ] AVX512FP16,FUTURE */
        /* 241(f5 7e 08) 306 001(51) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VSQRTSH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+48933, 568},
    /* 7161 : VSUBPH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.np.map5.w0 5c /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 08) 301 001(5c) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VSUBPH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48951, 566},
    /* 7162 : VSUBPH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.np.map5.w0 5c /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 28) 301 001(5c) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VSUBPH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48969, 566},
    /* 7163 : VSUBPH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.np.map5.w0 5c /r     ] AVX512FP16,FUTURE */
        /* 241(f5 7c 48) 301 001(5c) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VSUBPH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+48987, 567},
    /* 7164 : VSUBSH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.f3.map5.w0 5c /r     ] AVX512FP16,FUTURE */
        /* 241(f5 7e 08) 306 001(5c) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VSUBSH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+49005, 568},
    /* 7165 : VUCOMISH xmmreg,xmmrm16|sae [rm:t1s:   evex.lig.np.map5.w0 2e /r         ] FL,AVX512FP16,FUTURE */
        /* 250(f5 7c 08) 306 001(2e) 110 : LIG,FL,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VUCOMISH, 2, {XMMREG,RM_XMM|BITS16,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+49023, 636},
    /* 7166 : AADD mem32,reg32 [mr:       o32 np 0f38 fc /r                 ] AR0-1,SQ,LONG,PROT,RAOINT,X86_64 */
        /* 321 360 356 001(fc) 101 : AR0,AR1,SQ,NOAPX,LONG,PROT,RAOINT,X86_64 */
        {I_AADD, 2, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66154, 638},
    /* 7167 : AADD mem64,reg64 [mr:       o64 np 0f38 fc /r                 ] AR0-1,SQ,LONG,PROT,RAOINT,X86_64 */
        /* 324 360 356 001(fc) 101 : AR0,AR1,SQ,NOAPX,LONG,PROT,RAOINT,X86_64 */
        {I_AADD, 2, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66161, 638},
    /* 7168 : AADD mem32,reg32 [mr:       evex.nf0.nd0.l0.np.m4.o32 fc /r   ] AR0-1,SQ,LONG,PROT,RAOINT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(fc) 101 : AR0,AR1,SQ,WW,LONG,EVEX,PROT,RAOINT,APX,X86_64 */
        {I_AADD, 2, {MEMORY|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+22963, 639},
    /* 7169 : AADD mem64,reg64 [mr:       evex.nf0.nd0.l0.np.m4.o64 fc /r   ] AR0-1,SQ,LONG,PROT,RAOINT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(fc) 101 : AR0,AR1,SQ,WW,LONG,EVEX,PROT,RAOINT,APX,X86_64 */
        {I_AADD, 2, {MEMORY|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+22973, 639},
    /* 7170 : AAND mem32,reg32 [mr:       o32 66 0f38 fc /r                 ] AR0-1,SQ,LONG,PROT,RAOINT,X86_64 */
        /* 321 361 356 001(fc) 101 : AR0,AR1,SQ,NOAPX,LONG,PROT,RAOINT,X86_64 */
        {I_AAND, 2, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66168, 638},
    /* 7171 : AAND mem64,reg64 [mr:       o64 66 0f38 fc /r                 ] AR0-1,SQ,LONG,PROT,RAOINT,X86_64 */
        /* 324 361 356 001(fc) 101 : AR0,AR1,SQ,NOAPX,LONG,PROT,RAOINT,X86_64 */
        {I_AAND, 2, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66175, 638},
    /* 7172 : AAND mem32,reg32 [mr:       evex.nf0.nd0.l0.66.m4.o32 fc /r   ] AR0-1,SQ,LONG,PROT,RAOINT,APX,X86_64 */
        /* 250(f4 7d 08) 300 321 001(fc) 101 : AR0,AR1,SQ,WW,LONG,EVEX,PROT,RAOINT,APX,X86_64 */
        {I_AAND, 2, {MEMORY|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+22983, 639},
    /* 7173 : AAND mem64,reg64 [mr:       evex.nf0.nd0.l0.66.m4.o64 fc /r   ] AR0-1,SQ,LONG,PROT,RAOINT,APX,X86_64 */
        /* 250(f4 fd 08) 300 324 001(fc) 101 : AR0,AR1,SQ,WW,LONG,EVEX,PROT,RAOINT,APX,X86_64 */
        {I_AAND, 2, {MEMORY|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+22993, 639},
    /* 7174 : AOR mem32,reg32 [mr:       o32 f2 0f38 fc /r                 ] AR0-1,SQ,LONG,PROT,RAOINT,X86_64 */
        /* 321 332 356 001(fc) 101 : AR0,AR1,SQ,NOAPX,LONG,PROT,RAOINT,X86_64 */
        {I_AOR, 2, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66182, 638},
    /* 7175 : AOR mem64,reg64 [mr:       o64 f2 0f38 fc /r                 ] AR0-1,SQ,LONG,PROT,RAOINT,X86_64 */
        /* 324 332 356 001(fc) 101 : AR0,AR1,SQ,NOAPX,LONG,PROT,RAOINT,X86_64 */
        {I_AOR, 2, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66189, 638},
    /* 7176 : AOR mem32,reg32 [mr:       evex.nf0.nd0.l0.f2.m4.o32 fc /r   ] AR0-1,SQ,LONG,PROT,RAOINT,APX,X86_64 */
        /* 250(f4 7f 08) 300 321 001(fc) 101 : AR0,AR1,SQ,WW,LONG,EVEX,PROT,RAOINT,APX,X86_64 */
        {I_AOR, 2, {MEMORY|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+23003, 639},
    /* 7177 : AOR mem64,reg64 [mr:       evex.nf0.nd0.l0.f2.m4.o64 fc /r   ] AR0-1,SQ,LONG,PROT,RAOINT,APX,X86_64 */
        /* 250(f4 ff 08) 300 324 001(fc) 101 : AR0,AR1,SQ,WW,LONG,EVEX,PROT,RAOINT,APX,X86_64 */
        {I_AOR, 2, {MEMORY|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+23013, 639},
    /* 7178 : AXOR mem32,reg32 [mr:       o32 f3 0f38 fc /r                 ] AR0-1,SQ,LONG,PROT,RAOINT,X86_64 */
        /* 321 333 356 001(fc) 101 : AR0,AR1,SQ,NOAPX,LONG,PROT,RAOINT,X86_64 */
        {I_AXOR, 2, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66196, 638},
    /* 7179 : AXOR mem64,reg64 [mr:       o64 f3 0f38 fc /r                 ] AR0-1,SQ,LONG,PROT,RAOINT,X86_64 */
        /* 324 333 356 001(fc) 101 : AR0,AR1,SQ,NOAPX,LONG,PROT,RAOINT,X86_64 */
        {I_AXOR, 2, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66203, 638},
    /* 7180 : AXOR mem32,reg32 [mr:       evex.nf0.nd0.l0.f3.m4.o32 fc /r   ] AR0-1,SQ,LONG,PROT,RAOINT,APX,X86_64 */
        /* 250(f4 7e 08) 300 321 001(fc) 101 : AR0,AR1,SQ,WW,LONG,EVEX,PROT,RAOINT,APX,X86_64 */
        {I_AXOR, 2, {MEMORY|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+23023, 639},
    /* 7181 : AXOR mem64,reg64 [mr:       evex.nf0.nd0.l0.f3.m4.o64 fc /r   ] AR0-1,SQ,LONG,PROT,RAOINT,APX,X86_64 */
        /* 250(f4 fe 08) 300 324 001(fc) 101 : AR0,AR1,SQ,WW,LONG,EVEX,PROT,RAOINT,APX,X86_64 */
        {I_AXOR, 2, {MEMORY|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+23033, 639},
    /* 7182 : CLUI void [          f3 0f 01 ee                       ] LONG,PROT,UINTR,X86_64 */
        /* 333 355 002(01 ee) : LONG,PROT,UINTR,X86_64 */
        {I_CLUI, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70010, 640},
    /* 7183 : SENDUIPI reg64 [m:        o64nw f3 0f c7 /6                 ] LONG,PROT,UINTR,X86_64 */
        /* 323 333 355 001(c7) 206 : NWSIZE,LONG,PROT,UINTR,X86_64 */
        {I_SENDUIPI, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66210, 641},
    /* 7184 : STUI void [          f3 0f 01 ef                       ] LONG,PROT,UINTR,X86_64 */
        /* 333 355 002(01 ef) : LONG,PROT,UINTR,X86_64 */
        {I_STUI, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70016, 640},
    /* 7185 : TESTUI void [          f3 0f 01 ed                       ] LONG,FL,PROT,UINTR,X86_64 */
        /* 333 355 002(01 ed) : LONG,FL,PROT,UINTR,X86_64 */
        {I_TESTUI, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70022, 642},
    /* 7186 : UIRET void [          f3 0f 01 ec                       ] LONG,FL,PROT,UINTR,X86_64 */
        /* 333 355 002(01 ec) : LONG,FL,PROT,UINTR,X86_64 */
        {I_UIRET, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70028, 642},
    /* 7187 : HRESET imm,reg_eax [i-:       f3 0f3a f0 c0 ib                  ] AR0-1,SB,PRIV,HRESET,FUTURE */
        /* 333 357 002(f0 c0) 020 : AR0,AR1,SB,NOAPX,PRIV,HRESET,FUTURE */
        {I_HRESET, 2, {IMM_NORMAL,REG_EAX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66217, 643},
    /* 7188 : VADDBF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.66.map5.w0 58 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 08) 301 001(58) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VADDBF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49032, 645},
    /* 7189 : VADDBF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.66.map5.w0 58 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 28) 301 001(58) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VADDBF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49041, 645},
    /* 7190 : VADDBF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.66.map5.w0 58 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 48) 301 001(58) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VADDBF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49050, 645},
    /* 7191 : VCMPBF16 kreg|mask,xmmreg,xmmrm128|b16,imm8 [rvmi:fv:  evex.128.f2.0f3a.w0 c2 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7f 08) 301 001(c2) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCMPBF16, 4, {OPMASKREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK,0,B16,0,0}, nasm_bytecodes+23043, 645},
    /* 7192 : VCMPBF16 kreg|mask,ymmreg,ymmrm256|b16,imm8 [rvmi:fv:  evex.256.f2.0f3a.w0 c2 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7f 28) 301 001(c2) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCMPBF16, 4, {OPMASKREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK,0,B16,0,0}, nasm_bytecodes+23053, 645},
    /* 7193 : VCMPBF16 kreg|mask,zmmreg,zmmrm512|b16,imm8 [rvmi:fv:  evex.512.f2.0f3a.w0 c2 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7f 48) 301 001(c2) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCMPBF16, 4, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK,0,B16,0,0}, nasm_bytecodes+23063, 645},
    /* 7194 : VCOMISBF16 xmmreg,xmmrm16 [rm:t1s:   evex.lig.66.map5.w0 2f /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 08) 306 001(2f) 110 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VCOMISBF16, 2, {XMMREG,RM_XMM|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+49059, 646},
    /* 7195 : VDIVBF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.66.map5.w0 5e /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 08) 301 001(5e) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VDIVBF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49068, 645},
    /* 7196 : VDIVBF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.66.map5.w0 5e /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 28) 301 001(5e) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VDIVBF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49077, 645},
    /* 7197 : VDIVBF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.66.map5.w0 5e /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 48) 301 001(5e) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VDIVBF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49086, 645},
    /* 7198 : VFMADD132BF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map6.w0 98 /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 08) 301 001(98) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMADD132BF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49095, 645},
    /* 7199 : VFMADD132BF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map6.w0 98 /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 28) 301 001(98) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMADD132BF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49104, 645},
    /* 7200 : VFMADD132BF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map6.w0 98 /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 48) 301 001(98) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMADD132BF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49113, 645},
    /* 7201 : VFMADD213BF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map6.w0 a8 /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 08) 301 001(a8) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMADD213BF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49122, 645},
    /* 7202 : VFMADD213BF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map6.w0 a8 /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 28) 301 001(a8) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMADD213BF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49131, 645},
    /* 7203 : VFMADD213BF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map6.w0 a8 /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 48) 301 001(a8) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMADD213BF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49140, 645},
    /* 7204 : VFMADD231BF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map6.w0 b8 /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 08) 301 001(b8) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMADD231BF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49149, 645},
    /* 7205 : VFMADD231BF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map6.w0 b8 /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 28) 301 001(b8) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMADD231BF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49158, 645},
    /* 7206 : VFMADD231BF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map6.w0 b8 /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 48) 301 001(b8) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMADD231BF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49167, 645},
    /* 7207 : VFMSUB132BF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map6.w0 9a /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 08) 301 001(9a) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMSUB132BF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49176, 645},
    /* 7208 : VFMSUB132BF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map6.w0 9a /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 28) 301 001(9a) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMSUB132BF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49185, 645},
    /* 7209 : VFMSUB132BF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map6.w0 9a /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 48) 301 001(9a) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMSUB132BF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49194, 645},
    /* 7210 : VFMSUB213BF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map6.w0 aa /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 08) 301 001(aa) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMSUB213BF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49203, 645},
    /* 7211 : VFMSUB213BF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map6.w0 aa /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 28) 301 001(aa) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMSUB213BF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49212, 645},
    /* 7212 : VFMSUB213BF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map6.w0 aa /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 48) 301 001(aa) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMSUB213BF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49221, 645},
    /* 7213 : VFMSUB231BF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map6.w0 ba /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 08) 301 001(ba) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMSUB231BF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49230, 645},
    /* 7214 : VFMSUB231BF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map6.w0 ba /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 28) 301 001(ba) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMSUB231BF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49239, 645},
    /* 7215 : VFMSUB231BF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map6.w0 ba /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 48) 301 001(ba) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMSUB231BF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49248, 645},
    /* 7216 : VFNMADD132BF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map6.w0 9c /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 08) 301 001(9c) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMADD132BF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49257, 645},
    /* 7217 : VFNMADD132BF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map6.w0 9c /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 28) 301 001(9c) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMADD132BF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49266, 645},
    /* 7218 : VFNMADD132BF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map6.w0 9c /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 48) 301 001(9c) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMADD132BF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49275, 645},
    /* 7219 : VFNMADD213BF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map6.w0 ac /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 08) 301 001(ac) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMADD213BF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49284, 645},
    /* 7220 : VFNMADD213BF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map6.w0 ac /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 28) 301 001(ac) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMADD213BF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49293, 645},
    /* 7221 : VFNMADD213BF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map6.w0 ac /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 48) 301 001(ac) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMADD213BF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49302, 645},
    /* 7222 : VFNMADD231BF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map6.w0 bc /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 08) 301 001(bc) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMADD231BF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49311, 645},
    /* 7223 : VFNMADD231BF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map6.w0 bc /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 28) 301 001(bc) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMADD231BF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49320, 645},
    /* 7224 : VFNMADD231BF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map6.w0 bc /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 48) 301 001(bc) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMADD231BF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49329, 645},
    /* 7225 : VFNMSUB132BF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map6.w0 9e /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 08) 301 001(9e) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMSUB132BF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49338, 645},
    /* 7226 : VFNMSUB132BF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map6.w0 9e /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 28) 301 001(9e) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMSUB132BF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49347, 645},
    /* 7227 : VFNMSUB132BF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map6.w0 9e /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 48) 301 001(9e) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMSUB132BF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49356, 645},
    /* 7228 : VFNMSUB213BF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map6.w0 ae /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 08) 301 001(ae) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMSUB213BF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49365, 645},
    /* 7229 : VFNMSUB213BF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map6.w0 ae /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 28) 301 001(ae) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMSUB213BF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49374, 645},
    /* 7230 : VFNMSUB213BF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map6.w0 ae /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 48) 301 001(ae) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMSUB213BF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49383, 645},
    /* 7231 : VFNMSUB231BF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map6.w0 be /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 08) 301 001(be) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMSUB231BF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49392, 645},
    /* 7232 : VFNMSUB231BF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map6.w0 be /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 28) 301 001(be) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMSUB231BF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49401, 645},
    /* 7233 : VFNMSUB231BF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map6.w0 be /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 48) 301 001(be) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMSUB231BF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49410, 645},
    /* 7234 : VFPCLASSBF16 kreg|mask,xmmrm128|b16,imm8 [rmi:fv:   evex.128.f2.0f3a.w0 66 /r ib      ] AVX10_2,FUTURE */
        /* 250(f3 7f 08) 301 001(66) 110 022 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFPCLASSBF16, 3, {OPMASKREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK,B16,0,0,0}, nasm_bytecodes+23073, 645},
    /* 7235 : VFPCLASSBF16 kreg|mask,ymmrm256|b16,imm8 [rmi:fv:   evex.256.f2.0f3a.w0 66 /r ib      ] AVX10_2,FUTURE */
        /* 250(f3 7f 28) 301 001(66) 110 022 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFPCLASSBF16, 3, {OPMASKREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK,B16,0,0,0}, nasm_bytecodes+23083, 645},
    /* 7236 : VFPCLASSBF16 kreg|mask,zmmrm512|b16,imm8 [rmi:fv:   evex.512.f2.0f3a.w0 66 /r ib      ] AVX10_2,FUTURE */
        /* 250(f3 7f 48) 301 001(66) 110 022 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFPCLASSBF16, 3, {OPMASKREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK,B16,0,0,0}, nasm_bytecodes+23093, 645},
    /* 7237 : VGETEXPBF16 xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.np.map6.w0 42 /r         ] AVX10_2,FUTURE */
        /* 250(f6 7c 08) 301 001(42) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VGETEXPBF16, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49419, 645},
    /* 7238 : VGETEXPBF16 ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.np.map6.w0 42 /r         ] AVX10_2,FUTURE */
        /* 250(f6 7c 28) 301 001(42) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VGETEXPBF16, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49428, 645},
    /* 7239 : VGETEXPBF16 zmmreg|mask|z,zmmrm512|b16 [rm:fv:    evex.512.np.map6.w0 42 /r         ] AVX10_2,FUTURE */
        /* 250(f6 7c 48) 301 001(42) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VGETEXPBF16, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49437, 645},
    /* 7240 : VGETMANTBF16 xmmreg|mask|z,xmmrm128|b16,imm8 [rmi:fv:   evex.128.f2.0f3a.w0 26 /r ib      ] AVX10_2,FUTURE */
        /* 250(f3 7f 08) 301 001(26) 110 022 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VGETMANTBF16, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+23103, 645},
    /* 7241 : VGETMANTBF16 ymmreg|mask|z,ymmrm256|b16,imm8 [rmi:fv:   evex.256.f2.0f3a.w0 26 /r ib      ] AVX10_2,FUTURE */
        /* 250(f3 7f 28) 301 001(26) 110 022 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VGETMANTBF16, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+23113, 645},
    /* 7242 : VGETMANTBF16 zmmreg|mask|z,zmmrm512|b16,imm8 [rmi:fv:   evex.512.f2.0f3a.w0 26 /r ib      ] AVX10_2,FUTURE */
        /* 250(f3 7f 48) 301 001(26) 110 022 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VGETMANTBF16, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+23123, 645},
    /* 7243 : VMAXBF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.66.map5.w0 5f /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 08) 301 001(5f) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMAXBF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49446, 645},
    /* 7244 : VMAXBF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.66.map5.w0 5f /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 28) 301 001(5f) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMAXBF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49455, 645},
    /* 7245 : VMAXBF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.66.map5.w0 5f /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 48) 301 001(5f) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMAXBF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49464, 645},
    /* 7246 : VMINBF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.66.map5.w0 5d /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 08) 301 001(5d) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINBF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49473, 645},
    /* 7247 : VMINBF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.66.map5.w0 5d /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 28) 301 001(5d) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINBF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49482, 645},
    /* 7248 : VMINBF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.66.map5.w0 5d /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 48) 301 001(5d) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINBF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49491, 645},
    /* 7249 : VMULBF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.66.map5.w0 59 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 08) 301 001(59) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMULBF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49500, 645},
    /* 7250 : VMULBF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.66.map5.w0 59 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 28) 301 001(59) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMULBF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49509, 645},
    /* 7251 : VMULBF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.66.map5.w0 59 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 48) 301 001(59) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMULBF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49518, 645},
    /* 7252 : VRCPBF16 xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.np.map6.w0 4c /r         ] AVX10_2,FUTURE */
        /* 250(f6 7c 08) 301 001(4c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VRCPBF16, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49527, 645},
    /* 7253 : VRCPBF16 ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.np.map6.w0 4c /r         ] AVX10_2,FUTURE */
        /* 250(f6 7c 28) 301 001(4c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VRCPBF16, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49536, 645},
    /* 7254 : VRCPBF16 zmmreg|mask|z,zmmrm512|b16 [rm:fv:    evex.512.np.map6.w0 4c /r         ] AVX10_2,FUTURE */
        /* 250(f6 7c 48) 301 001(4c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VRCPBF16, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49545, 645},
    /* 7255 : VREDUCEBF16 xmmreg|mask|z,xmmrm128|b16,imm8 [rmi:fv:   evex.128.f2.0f3a.w0 56 /r ib      ] AVX10_2,FUTURE */
        /* 250(f3 7f 08) 301 001(56) 110 022 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VREDUCEBF16, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+23133, 645},
    /* 7256 : VREDUCEBF16 ymmreg|mask|z,ymmrm256|b16,imm8 [rmi:fv:   evex.256.f2.0f3a.w0 56 /r ib      ] AVX10_2,FUTURE */
        /* 250(f3 7f 28) 301 001(56) 110 022 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VREDUCEBF16, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+23143, 645},
    /* 7257 : VREDUCEBF16 zmmreg|mask|z,zmmrm512|b16,imm8 [rmi:fv:   evex.512.f2.0f3a.w0 56 /r ib      ] AVX10_2,FUTURE */
        /* 250(f3 7f 48) 301 001(56) 110 022 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VREDUCEBF16, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+23153, 645},
    /* 7258 : VRNDSCALEBF16 xmmreg|mask|z,xmmrm128|b16,imm8 [rmi:fv:   evex.128.f2.0f3a.w0 08 /r ib      ] AVX10_2,FUTURE */
        /* 250(f3 7f 08) 301 001(08) 110 022 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VRNDSCALEBF16, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+23163, 645},
    /* 7259 : VRNDSCALEBF16 ymmreg|mask|z,ymmrm256|b16,imm8 [rmi:fv:   evex.256.f2.0f3a.w0 08 /r ib      ] AVX10_2,FUTURE */
        /* 250(f3 7f 28) 301 001(08) 110 022 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VRNDSCALEBF16, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+23173, 645},
    /* 7260 : VRNDSCALEBF16 zmmreg|mask|z,zmmrm512|b16,imm8 [rmi:fv:   evex.512.f2.0f3a.w0 08 /r ib      ] AVX10_2,FUTURE */
        /* 250(f3 7f 48) 301 001(08) 110 022 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VRNDSCALEBF16, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+23183, 645},
    /* 7261 : VRSQRTBF16 xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.np.map6.w0 4e /r         ] AVX10_2,FUTURE */
        /* 250(f6 7c 08) 301 001(4e) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VRSQRTBF16, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49554, 645},
    /* 7262 : VRSQRTBF16 ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.np.map6.w0 4e /r         ] AVX10_2,FUTURE */
        /* 250(f6 7c 28) 301 001(4e) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VRSQRTBF16, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49563, 645},
    /* 7263 : VRSQRTBF16 zmmreg|mask|z,zmmrm512|b16 [rm:fv:    evex.512.np.map6.w0 4e /r         ] AVX10_2,FUTURE */
        /* 250(f6 7c 48) 301 001(4e) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VRSQRTBF16, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49572, 645},
    /* 7264 : VSCALEFBF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map6.w0 2c /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 08) 301 001(2c) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VSCALEFBF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49581, 645},
    /* 7265 : VSCALEFBF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map6.w0 2c /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 28) 301 001(2c) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VSCALEFBF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49590, 645},
    /* 7266 : VSCALEFBF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map6.w0 2c /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 48) 301 001(2c) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VSCALEFBF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49599, 645},
    /* 7267 : VSQRTBF16 xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.66.map5.w0 51 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 08) 301 001(51) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VSQRTBF16, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49608, 645},
    /* 7268 : VSQRTBF16 ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.66.map5.w0 51 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 28) 301 001(51) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VSQRTBF16, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49617, 645},
    /* 7269 : VSQRTBF16 zmmreg|mask|z,zmmrm512|b16 [rm:fv:    evex.512.66.map5.w0 51 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 48) 301 001(51) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VSQRTBF16, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49626, 645},
    /* 7270 : VSUBBF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.66.map5.w0 5c /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 08) 301 001(5c) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VSUBBF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49635, 645},
    /* 7271 : VSUBBF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.66.map5.w0 5c /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 28) 301 001(5c) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VSUBBF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49644, 645},
    /* 7272 : VSUBBF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.66.map5.w0 5c /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 48) 301 001(5c) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VSUBBF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49653, 645},
    /* 7273 : VCOMXSD xmmreg,xmmrm64|sae [rm:t1s:   evex.lig.f2.0f.w1 2f /r           ] AVX10_2,FUTURE */
        /* 250(f1 ff 08) 306 001(2f) 110 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VCOMXSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+49662, 646},
    /* 7274 : VCOMXSH xmmreg,xmmrm16|sae [rm:t1s:   evex.lig.f3.map5.w0 2f /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 08) 306 001(2f) 110 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VCOMXSH, 2, {XMMREG,RM_XMM|BITS16,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+49671, 646},
    /* 7275 : VCOMXSS xmmreg,xmmrm32|sae [rm:t1s:   evex.lig.f3.0f.w0 2f /r           ] AVX10_2,FUTURE */
        /* 250(f1 7e 08) 306 001(2f) 110 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VCOMXSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+49680, 646},
    /* 7276 : VUCOMXSD xmmreg,xmmrm64|sae [rm:t1s:   evex.lig.f2.0f.w1 2e /r           ] AVX10_2,FUTURE */
        /* 250(f1 ff 08) 306 001(2e) 110 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VUCOMXSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+49689, 646},
    /* 7277 : VUCOMXSH xmmreg,xmmrm16|sae [rm:t1s:   evex.lig.f3.map5.w0 2e /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 08) 306 001(2e) 110 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VUCOMXSH, 2, {XMMREG,RM_XMM|BITS16,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+49698, 646},
    /* 7278 : VUCOMXSS xmmreg,xmmrm32|sae [rm:t1s:   evex.lig.f3.0f.w0 2e /r           ] AVX10_2,FUTURE */
        /* 250(f1 7e 08) 306 001(2e) 110 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VUCOMXSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+49707, 646},
    /* 7279 : VCVT2PH2BF8 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.f2.0f38.w0 74 /r         ] AVX10_2,FUTURE */
        /* 241(f2 7f 08) 301 001(74) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PH2BF8, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49716, 645},
    /* 7280 : VCVT2PH2BF8 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.f2.0f38.w0 74 /r         ] AVX10_2,FUTURE */
        /* 241(f2 7f 28) 301 001(74) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PH2BF8, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49725, 645},
    /* 7281 : VCVT2PH2BF8 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.f2.0f38.w0 74 /r         ] AVX10_2,FUTURE */
        /* 241(f2 7f 48) 301 001(74) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PH2BF8, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49734, 645},
    /* 7282 : VCVT2PH2BF8S xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.f2.map5.w0 74 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7f 08) 301 001(74) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PH2BF8S, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49743, 645},
    /* 7283 : VCVT2PH2BF8S ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.f2.map5.w0 74 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7f 28) 301 001(74) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PH2BF8S, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49752, 645},
    /* 7284 : VCVT2PH2BF8S zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.f2.map5.w0 74 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7f 48) 301 001(74) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PH2BF8S, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49761, 645},
    /* 7285 : VCVT2PH2HF8 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.f2.map5.w0 18 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7f 08) 301 001(18) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PH2HF8, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49770, 645},
    /* 7286 : VCVT2PH2HF8 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.f2.map5.w0 18 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7f 28) 301 001(18) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PH2HF8, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49779, 645},
    /* 7287 : VCVT2PH2HF8 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.f2.map5.w0 18 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7f 48) 301 001(18) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PH2HF8, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49788, 645},
    /* 7288 : VCVT2PH2HF8S xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.f2.map5.w0 1b /r         ] AVX10_2,FUTURE */
        /* 241(f5 7f 08) 301 001(1b) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PH2HF8S, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49797, 645},
    /* 7289 : VCVT2PH2HF8S ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.f2.map5.w0 1b /r         ] AVX10_2,FUTURE */
        /* 241(f5 7f 28) 301 001(1b) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PH2HF8S, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49806, 645},
    /* 7290 : VCVT2PH2HF8S zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.f2.map5.w0 1b /r         ] AVX10_2,FUTURE */
        /* 241(f5 7f 48) 301 001(1b) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PH2HF8S, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49815, 645},
    /* 7291 : VCVTPH2BF8 xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.f3.0f38.w0 74 /r         ] AVX10_2,FUTURE */
        /* 250(f2 7e 08) 301 001(74) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2BF8, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49824, 645},
    /* 7292 : VCVTPH2BF8 ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.f3.0f38.w0 74 /r         ] AVX10_2,FUTURE */
        /* 250(f2 7e 28) 301 001(74) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2BF8, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49833, 645},
    /* 7293 : VCVTPH2BF8 zmmreg|mask|z,zmmrm512|b16 [rm:fv:    evex.512.f3.0f38.w0 74 /r         ] AVX10_2,FUTURE */
        /* 250(f2 7e 48) 301 001(74) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2BF8, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49842, 645},
    /* 7294 : VCVTPH2BF8S xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.f3.map5.w0 74 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 08) 301 001(74) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2BF8S, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49851, 645},
    /* 7295 : VCVTPH2BF8S ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.f3.map5.w0 74 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 28) 301 001(74) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2BF8S, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49860, 645},
    /* 7296 : VCVTPH2BF8S zmmreg|mask|z,zmmrm512|b16 [rm:fv:    evex.512.f3.map5.w0 74 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 48) 301 001(74) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2BF8S, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49869, 645},
    /* 7297 : VCVTPH2HF8 xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.f3.map5.w0 18 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 08) 301 001(18) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2HF8, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49878, 645},
    /* 7298 : VCVTPH2HF8 ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.f3.map5.w0 18 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 28) 301 001(18) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2HF8, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49887, 645},
    /* 7299 : VCVTPH2HF8 zmmreg|mask|z,zmmrm512|b16 [rm:fv:    evex.512.f3.map5.w0 18 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 48) 301 001(18) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2HF8, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49896, 645},
    /* 7300 : VCVTPH2HF8S xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.f3.map5.w0 1b /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 08) 301 001(1b) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2HF8S, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49905, 645},
    /* 7301 : VCVTPH2HF8S ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.f3.map5.w0 1b /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 28) 301 001(1b) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2HF8S, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49914, 645},
    /* 7302 : VCVTPH2HF8S zmmreg|mask|z,zmmrm512|b16 [rm:fv:    evex.512.f3.map5.w0 1b /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 48) 301 001(1b) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2HF8S, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49923, 645},
    /* 7303 : VCVT2PS2PHX xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.66.0f38.w0 67 /r         ] AVX10_2,FUTURE */
        /* 241(f2 7d 08) 301 001(67) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PS2PHX, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+49932, 645},
    /* 7304 : VCVT2PS2PHX ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.66.0f38.w0 67 /r         ] AVX10_2,FUTURE */
        /* 241(f2 7d 28) 301 001(67) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PS2PHX, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+49941, 645},
    /* 7305 : VCVT2PS2PHX zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.66.0f38.w0 67 /r         ] AVX10_2,FUTURE */
        /* 241(f2 7d 48) 301 001(67) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PS2PHX, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+49950, 645},
    /* 7306 : VCVTBIASPH2BF8 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.0f38.w0 74 /r         ] AVX10_2,FUTURE */
        /* 241(f2 7c 08) 301 001(74) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBIASPH2BF8, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49959, 645},
    /* 7307 : VCVTBIASPH2BF8 xmmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.0f38.w0 74 /r         ] AVX10_2,FUTURE */
        /* 241(f2 7c 28) 301 001(74) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBIASPH2BF8, 3, {XMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49968, 645},
    /* 7308 : VCVTBIASPH2BF8 ymmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.0f38.w0 74 /r         ] AVX10_2,FUTURE */
        /* 241(f2 7c 48) 301 001(74) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBIASPH2BF8, 3, {YMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49977, 645},
    /* 7309 : VCVTBIASPH2BF8S xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map5.w0 74 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7c 08) 301 001(74) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBIASPH2BF8S, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49986, 645},
    /* 7310 : VCVTBIASPH2BF8S xmmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map5.w0 74 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7c 28) 301 001(74) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBIASPH2BF8S, 3, {XMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49995, 645},
    /* 7311 : VCVTBIASPH2BF8S ymmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map5.w0 74 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7c 48) 301 001(74) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBIASPH2BF8S, 3, {YMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+50004, 645},
    /* 7312 : VCVTBIASPH2HF8 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map5.w0 18 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7c 08) 301 001(18) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBIASPH2HF8, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+50013, 645},
    /* 7313 : VCVTBIASPH2HF8 xmmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map5.w0 18 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7c 28) 301 001(18) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBIASPH2HF8, 3, {XMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+50022, 645},
    /* 7314 : VCVTBIASPH2HF8 ymmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map5.w0 18 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7c 48) 301 001(18) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBIASPH2HF8, 3, {YMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+50031, 645},
    /* 7315 : VCVTBIASPH2HF8S xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map5.w0 1b /r         ] AVX10_2,FUTURE */
        /* 241(f5 7c 08) 301 001(1b) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBIASPH2HF8S, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+50040, 645},
    /* 7316 : VCVTBIASPH2HF8S xmmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map5.w0 1b /r         ] AVX10_2,FUTURE */
        /* 241(f5 7c 28) 301 001(1b) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBIASPH2HF8S, 3, {XMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+50049, 645},
    /* 7317 : VCVTBIASPH2HF8S ymmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map5.w0 1b /r         ] AVX10_2,FUTURE */
        /* 241(f5 7c 48) 301 001(1b) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBIASPH2HF8S, 3, {YMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+50058, 645},
    /* 7318 : VCVTHF82PH xmmreg|mask|z,xmmrm64 [rm:hv:    evex.128.f2.map5.w0 1e /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 08) 302 001(1e) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTHF82PH, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+50067, 645},
    /* 7319 : VCVTHF82PH ymmreg|mask|z,xmmrm128 [rm:hv:    evex.256.f2.map5.w0 1e /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 28) 302 001(1e) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTHF82PH, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+50076, 645},
    /* 7320 : VCVTHF82PH zmmreg|mask|z,ymmrm256 [rm:hv:    evex.512.f2.map5.w0 1e /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 48) 302 001(1e) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTHF82PH, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+50085, 645},
    /* 7321 : VDPPHPS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.np.0f38.w0 52 /r         ] AVX10_2,FUTURE */
        /* 241(f2 7c 08) 301 001(52) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VDPPHPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50094, 645},
    /* 7322 : VDPPHPS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.np.0f38.w0 52 /r         ] AVX10_2,FUTURE */
        /* 241(f2 7c 28) 301 001(52) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VDPPHPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50103, 645},
    /* 7323 : VDPPHPS zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.np.0f38.w0 52 /r         ] AVX10_2,FUTURE */
        /* 241(f2 7c 48) 301 001(52) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VDPPHPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50112, 645},
    /* 7324 : VMPSADBW xmmreg|mask|z,xmmreg,xmmrm128,imm8 [rvmi:fvm: evex.128.f3.0f3a.w0 42 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7e 08) 303 001(42) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMPSADBW, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+23193, 645},
    /* 7325 : VMPSADBW ymmreg|mask|z,ymmreg,ymmrm256,imm8 [rvmi:fvm: evex.256.f3.0f3a.w0 42 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7e 28) 303 001(42) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMPSADBW, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+23203, 645},
    /* 7326 : VMPSADBW zmmreg|mask|z,zmmreg,zmmrm512,imm8 [rvmi:fvm: evex.512.f3.0f3a.w0 42 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7e 48) 303 001(42) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMPSADBW, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+23213, 645},
    /* 7327 : VPDPBSSD xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.f2.0f38.w0 50 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7f 08) 301 001(50) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBSSD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50121, 647},
    /* 7328 : VPDPBSSD ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.f2.0f38.w0 50 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7f 28) 301 001(50) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBSSD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50130, 647},
    /* 7329 : VPDPBSSD zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.f2.0f38.w0 50 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7f 48) 301 001(50) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBSSD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50139, 647},
    /* 7330 : VPDPBSSDS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.f2.0f38.w0 51 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7f 08) 301 001(51) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBSSDS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50148, 647},
    /* 7331 : VPDPBSSDS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.f2.0f38.w0 51 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7f 28) 301 001(51) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBSSDS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50157, 647},
    /* 7332 : VPDPBSSDS zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.f2.0f38.w0 51 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7f 48) 301 001(51) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBSSDS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50166, 647},
    /* 7333 : VPDPBSUD xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.f3.0f38.w0 50 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7e 08) 301 001(50) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBSUD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50175, 647},
    /* 7334 : VPDPBSUD ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.f3.0f38.w0 50 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7e 28) 301 001(50) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBSUD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50184, 647},
    /* 7335 : VPDPBSUD zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.f3.0f38.w0 50 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7e 48) 301 001(50) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBSUD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50193, 647},
    /* 7336 : VPDPBSUDS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.f3.0f38.w0 51 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7e 08) 301 001(51) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBSUDS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50202, 647},
    /* 7337 : VPDPBSUDS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.f3.0f38.w0 51 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7e 28) 301 001(51) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBSUDS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50211, 647},
    /* 7338 : VPDPBSUDS zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.f3.0f38.w0 51 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7e 48) 301 001(51) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBSUDS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50220, 647},
    /* 7339 : VPDPBUUD xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.np.0f38.w0 50 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7c 08) 301 001(50) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBUUD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50229, 647},
    /* 7340 : VPDPBUUD ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.np.0f38.w0 50 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7c 28) 301 001(50) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBUUD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50238, 647},
    /* 7341 : VPDPBUUD zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.np.0f38.w0 50 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7c 48) 301 001(50) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBUUD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50247, 647},
    /* 7342 : VPDPBUUDS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.np.0f38.w0 51 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7c 08) 301 001(51) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBUUDS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50256, 647},
    /* 7343 : VPDPBUUDS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.np.0f38.w0 51 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7c 28) 301 001(51) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBUUDS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50265, 647},
    /* 7344 : VPDPBUUDS zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.np.0f38.w0 51 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7c 48) 301 001(51) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBUUDS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50274, 647},
    /* 7345 : VPDPWSUD xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.f3.0f38.w0 d2 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7e 08) 301 001(d2) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWSUD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50283, 647},
    /* 7346 : VPDPWSUD ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.f3.0f38.w0 d2 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7e 28) 301 001(d2) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWSUD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50292, 647},
    /* 7347 : VPDPWSUD zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.f3.0f38.w0 d2 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7e 48) 301 001(d2) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWSUD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50301, 647},
    /* 7348 : VPDPWSUDS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.f3.0f38.w0 d3 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7e 08) 301 001(d3) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWSUDS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50310, 647},
    /* 7349 : VPDPWSUDS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.f3.0f38.w0 d3 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7e 28) 301 001(d3) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWSUDS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50319, 647},
    /* 7350 : VPDPWSUDS zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.f3.0f38.w0 d3 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7e 48) 301 001(d3) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWSUDS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50328, 647},
    /* 7351 : VPDPWUSD xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.66.0f38.w0 d2 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7d 08) 301 001(d2) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWUSD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50337, 647},
    /* 7352 : VPDPWUSD ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.66.0f38.w0 d2 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7d 28) 301 001(d2) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWUSD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50346, 647},
    /* 7353 : VPDPWUSD zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.66.0f38.w0 d2 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7d 48) 301 001(d2) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWUSD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50355, 647},
    /* 7354 : VPDPWUSDS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.66.0f38.w0 d3 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7d 08) 301 001(d3) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWUSDS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50364, 647},
    /* 7355 : VPDPWUSDS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.66.0f38.w0 d3 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7d 28) 301 001(d3) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWUSDS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50373, 647},
    /* 7356 : VPDPWUSDS zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.66.0f38.w0 d3 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7d 48) 301 001(d3) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWUSDS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50382, 647},
    /* 7357 : VPDPWUUD xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.np.0f38.w0 d2 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7c 08) 301 001(d2) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWUUD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50391, 647},
    /* 7358 : VPDPWUUD ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.np.0f38.w0 d2 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7c 28) 301 001(d2) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWUUD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50400, 647},
    /* 7359 : VPDPWUUD zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.np.0f38.w0 d2 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7c 48) 301 001(d2) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWUUD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50409, 647},
    /* 7360 : VPDPWUUDS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.np.0f38.w0 d3 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7c 08) 301 001(d3) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWUUDS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50418, 647},
    /* 7361 : VPDPWUUDS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.np.0f38.w0 d3 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7c 28) 301 001(d3) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWUUDS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50427, 647},
    /* 7362 : VPDPWUUDS zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.np.0f38.w0 d3 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7c 48) 301 001(d3) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWUUDS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50436, 647},
    /* 7363 : VMINMAXBF16 xmmreg|mask|z,xmmreg,xmmrm128|b16,imm8 [rvmi:fv:  evex.128.f2.0f3a.w0 52 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7f 08) 301 001(52) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXBF16, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+23223, 645},
    /* 7364 : VMINMAXBF16 ymmreg|mask|z,ymmreg,ymmrm256|b16,imm8 [rvmi:fv:  evex.256.f2.0f3a.w0 52 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7f 28) 301 001(52) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXBF16, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+23233, 645},
    /* 7365 : VMINMAXBF16 zmmreg|mask|z,zmmreg,zmmrm512|b16,imm8 [rvmi:fv:  evex.512.f2.0f3a.w0 52 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7f 48) 301 001(52) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXBF16, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+23243, 645},
    /* 7366 : VMINMAXPD xmmreg|mask|z,xmmreg,xmmrm128|b64,imm8 [rvmi:fv:  evex.128.66.0f3a.w1 52 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 fd 08) 301 001(52) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXPD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+23253, 645},
    /* 7367 : VMINMAXPD ymmreg|mask|z,ymmreg,ymmrm256|b64,imm8 [rvmi:fv:  evex.256.66.0f3a.w1 52 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 fd 28) 301 001(52) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXPD, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+23263, 645},
    /* 7368 : VMINMAXPD zmmreg|mask|z,zmmreg,zmmrm512|b64|sae,imm8 [rvmi:fv:  evex.512.66.0f3a.w1 52 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 fd 48) 301 001(52) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXPD, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64|SAE,0,0}, nasm_bytecodes+23273, 645},
    /* 7369 : VMINMAXPH xmmreg|mask|z,xmmreg,xmmrm128|b16,imm8 [rvmi:fv:  evex.128.np.0f3a.w0 52 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7c 08) 301 001(52) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXPH, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+23283, 645},
    /* 7370 : VMINMAXPH ymmreg|mask|z,ymmreg,ymmrm256|b16,imm8 [rvmi:fv:  evex.256.np.0f3a.w0 52 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7c 28) 301 001(52) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXPH, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+23293, 645},
    /* 7371 : VMINMAXPH zmmreg|mask|z,zmmreg,zmmrm512|b16|sae,imm8 [rvmi:fv:  evex.512.np.0f3a.w0 52 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7c 48) 301 001(52) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXPH, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B16|SAE,0,0}, nasm_bytecodes+23303, 645},
    /* 7372 : VMINMAXPS xmmreg|mask|z,xmmreg,xmmrm128|b32,imm8 [rvmi:fv:  evex.128.66.0f3a.w0 52 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7d 08) 301 001(52) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXPS, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+23313, 645},
    /* 7373 : VMINMAXPS ymmreg|mask|z,ymmreg,ymmrm256|b32,imm8 [rvmi:fv:  evex.256.66.0f3a.w0 52 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7d 28) 301 001(52) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXPS, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+23323, 645},
    /* 7374 : VMINMAXPS zmmreg|mask|z,zmmreg,zmmrm512|b32|sae,imm8 [rvmi:fv:  evex.512.66.0f3a.w0 52 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7d 48) 301 001(52) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXPS, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32|SAE,0,0}, nasm_bytecodes+23333, 645},
    /* 7375 : VMINMAXSD xmmreg|mask|z,xmmreg,xmmrm64|sae,imm8 [rvmi:t1s: evex.lig.66.0f3a.w1 53 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 fd 08) 306 001(53) 120 023 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXSD, 4, {XMMREG,XMMREG,RM_XMM|BITS64,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+23343, 646},
    /* 7376 : VMINMAXSH xmmreg|mask|z,xmmreg,xmmrm16|sae,imm8 [rvmi:t1s: evex.lig.np.0f3a.w0 53 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7c 08) 306 001(53) 120 023 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXSH, 4, {XMMREG,XMMREG,RM_XMM|BITS16,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+23353, 646},
    /* 7377 : VMINMAXSS xmmreg|mask|z,xmmreg,xmmrm32|sae,imm8 [rvmi:t1s: evex.lig.66.0f3a.w0 53 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7d 08) 306 001(53) 120 023 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXSS, 4, {XMMREG,XMMREG,RM_XMM|BITS32,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+23363, 646},
    /* 7378 : VCVTBF162IBS xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.f2.map5.w0 69 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 08) 301 001(69) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBF162IBS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50445, 645},
    /* 7379 : VCVTBF162IBS ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.f2.map5.w0 69 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 28) 301 001(69) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBF162IBS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50454, 645},
    /* 7380 : VCVTBF162IBS zmmreg|mask|z,zmmrm512|b16 [rm:fv:    evex.512.f2.map5.w0 69 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 48) 301 001(69) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBF162IBS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50463, 645},
    /* 7381 : VCVTBF162IUBS xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.f2.map5.w0 6b /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 08) 301 001(6b) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBF162IUBS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50472, 645},
    /* 7382 : VCVTBF162IUBS ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.f2.map5.w0 6b /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 28) 301 001(6b) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBF162IUBS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50481, 645},
    /* 7383 : VCVTBF162IUBS zmmreg|mask|z,zmmrm512|b16 [rm:fv:    evex.512.f2.map5.w0 6b /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 48) 301 001(6b) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBF162IUBS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50490, 645},
    /* 7384 : VCVTTBF162IBS xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.f2.map5.w0 68 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 08) 301 001(68) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTBF162IBS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50499, 645},
    /* 7385 : VCVTTBF162IBS ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.f2.map5.w0 68 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 28) 301 001(68) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTBF162IBS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50508, 645},
    /* 7386 : VCVTTBF162IBS zmmreg|mask|z,zmmrm512|b16 [rm:fv:    evex.512.f2.map5.w0 68 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 48) 301 001(68) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTBF162IBS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50517, 645},
    /* 7387 : VCVTTBF162IUBS xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.f2.map5.w0 6a /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 08) 301 001(6a) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTBF162IUBS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50526, 645},
    /* 7388 : VCVTTBF162IUBS ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.f2.map5.w0 6a /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 28) 301 001(6a) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTBF162IUBS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50535, 645},
    /* 7389 : VCVTTBF162IUBS zmmreg|mask|z,zmmrm512|b16 [rm:fv:    evex.512.f2.map5.w0 6a /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 48) 301 001(6a) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTBF162IUBS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50544, 645},
    /* 7390 : VCVTTPD2DQS xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.np.map5.w1 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 fc 08) 301 001(6d) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPD2DQS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+50553, 645},
    /* 7391 : VCVTTPD2DQS xmmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.np.map5.w1 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 fc 28) 301 001(6d) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPD2DQS, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+50562, 645},
    /* 7392 : VCVTTPD2DQS ymmreg|mask|z,zmmrm512|b64|sae [rm:fv:    evex.512.np.map5.w1 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 fc 48) 301 001(6d) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPD2DQS, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+50571, 645},
    /* 7393 : VCVTTPD2QQS xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.map5.w1 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 fd 08) 301 001(6d) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPD2QQS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+50580, 645},
    /* 7394 : VCVTTPD2QQS ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.map5.w1 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 fd 28) 301 001(6d) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPD2QQS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+50589, 645},
    /* 7395 : VCVTTPD2QQS zmmreg|mask|z,zmmrm512|b64|sae [rm:fv:    evex.512.66.map5.w1 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 fd 48) 301 001(6d) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPD2QQS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+50598, 645},
    /* 7396 : VCVTTPD2UDQS xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.np.map5.w1 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 fc 08) 301 001(6c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPD2UDQS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+50607, 645},
    /* 7397 : VCVTTPD2UDQS xmmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.np.map5.w1 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 fc 28) 301 001(6c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPD2UDQS, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+50616, 645},
    /* 7398 : VCVTTPD2UDQS ymmreg|mask|z,zmmrm512|b64|sae [rm:fv:    evex.512.np.map5.w1 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 fc 48) 301 001(6c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPD2UDQS, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+50625, 645},
    /* 7399 : VCVTTPD2UQQS xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.map5.w1 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 fd 08) 301 001(6c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPD2UQQS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+50634, 645},
    /* 7400 : VCVTTPD2UQQS ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.map5.w1 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 fd 28) 301 001(6c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPD2UQQS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+50643, 645},
    /* 7401 : VCVTTPD2UQQS zmmreg|mask|z,zmmrm512|b64|sae [rm:fv:    evex.512.66.map5.w1 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 fd 48) 301 001(6c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPD2UQQS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+50652, 645},
    /* 7402 : VCVTPH2IBS xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.np.map5.w0 69 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 08) 301 001(69) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2IBS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50661, 645},
    /* 7403 : VCVTPH2IBS ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.np.map5.w0 69 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 28) 301 001(69) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2IBS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50670, 645},
    /* 7404 : VCVTPH2IBS zmmreg|mask|z,zmmrm512|b16|er [rm:fv:    evex.512.np.map5.w0 69 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 48) 301 001(69) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2IBS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16|ER,0,0,0}, nasm_bytecodes+50679, 645},
    /* 7405 : VCVTPH2IUBS xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.np.map5.w0 6b /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 08) 301 001(6b) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2IUBS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50688, 645},
    /* 7406 : VCVTPH2IUBS ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.np.map5.w0 6b /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 28) 301 001(6b) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2IUBS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50697, 645},
    /* 7407 : VCVTPH2IUBS zmmreg|mask|z,zmmrm512|b16|er [rm:fv:    evex.512.np.map5.w0 6b /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 48) 301 001(6b) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2IUBS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16|ER,0,0,0}, nasm_bytecodes+50706, 645},
    /* 7408 : VCVTTPH2IBS xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.np.map5.w0 68 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 08) 301 001(68) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPH2IBS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50715, 645},
    /* 7409 : VCVTTPH2IBS ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.np.map5.w0 68 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 28) 301 001(68) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPH2IBS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50724, 645},
    /* 7410 : VCVTTPH2IBS zmmreg|mask|z,zmmrm512|b16|sae [rm:fv:    evex.512.np.map5.w0 68 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 48) 301 001(68) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPH2IBS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+50733, 645},
    /* 7411 : VCVTTPH2IUBS xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.np.map5.w0 6a /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 08) 301 001(6a) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPH2IUBS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50742, 645},
    /* 7412 : VCVTTPH2IUBS ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.np.map5.w0 6a /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 28) 301 001(6a) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPH2IUBS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50751, 645},
    /* 7413 : VCVTTPH2IUBS zmmreg|mask|z,zmmrm512|b16|sae [rm:fv:    evex.512.np.map5.w0 6a /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 48) 301 001(6a) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPH2IUBS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+50760, 645},
    /* 7414 : VCVTTPS2DQS xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.np.map5.w0 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 08) 301 001(6d) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2DQS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+50769, 645},
    /* 7415 : VCVTTPS2DQS ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.np.map5.w0 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 28) 301 001(6d) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2DQS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+50778, 645},
    /* 7416 : VCVTTPS2DQS zmmreg|mask|z,zmmrm512|b32|sae [rm:fv:    evex.512.np.map5.w0 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 48) 301 001(6d) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2DQS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+50787, 645},
    /* 7417 : VCVTPS2IBS xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.66.map5.w0 69 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 08) 301 001(69) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPS2IBS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+50796, 645},
    /* 7418 : VCVTPS2IBS ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.66.map5.w0 69 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 28) 301 001(69) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPS2IBS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+50805, 645},
    /* 7419 : VCVTPS2IBS zmmreg|mask|z,zmmrm512|b32|er [rm:fv:    evex.512.66.map5.w0 69 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 48) 301 001(69) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPS2IBS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|ER,0,0,0}, nasm_bytecodes+50814, 645},
    /* 7420 : VCVTPS2IUBS xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.66.map5.w0 6b /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 08) 301 001(6b) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPS2IUBS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+50823, 645},
    /* 7421 : VCVTPS2IUBS ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.66.map5.w0 6b /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 28) 301 001(6b) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPS2IUBS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+50832, 645},
    /* 7422 : VCVTPS2IUBS zmmreg|mask|z,zmmrm512|b32|er [rm:fv:    evex.512.66.map5.w0 6b /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 48) 301 001(6b) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPS2IUBS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|ER,0,0,0}, nasm_bytecodes+50841, 645},
    /* 7423 : VCVTTPS2IBS xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.66.map5.w0 68 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 08) 301 001(68) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2IBS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+50850, 645},
    /* 7424 : VCVTTPS2IBS ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.66.map5.w0 68 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 28) 301 001(68) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2IBS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+50859, 645},
    /* 7425 : VCVTTPS2IBS zmmreg|mask|z,zmmrm512|b32|sae [rm:fv:    evex.512.66.map5.w0 68 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 48) 301 001(68) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2IBS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+50868, 645},
    /* 7426 : VCVTTPS2IUBS xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.66.map5.w0 6a /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 08) 301 001(6a) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2IUBS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+50877, 645},
    /* 7427 : VCVTTPS2IUBS ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.66.map5.w0 6a /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 28) 301 001(6a) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2IUBS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+50886, 645},
    /* 7428 : VCVTTPS2IUBS zmmreg|mask|z,zmmrm512|b32|sae [rm:fv:    evex.512.66.map5.w0 6a /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 48) 301 001(6a) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2IUBS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+50895, 645},
    /* 7429 : VCVTTPS2QQS xmmreg|mask|z,xmmrm64|b32 [rm:hv:    evex.128.66.map5.w0 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 08) 302 001(6d) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2QQS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+50904, 645},
    /* 7430 : VCVTTPS2QQS ymmreg|mask|z,xmmrm128|b32 [rm:hv:    evex.256.66.map5.w0 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 28) 302 001(6d) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2QQS, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+50913, 645},
    /* 7431 : VCVTTPS2QQS zmmreg|mask|z,ymmrm256|b32|sae [rm:hv:    evex.512.66.map5.w0 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 48) 302 001(6d) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2QQS, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+50922, 645},
    /* 7432 : VCVTTPS2UDQS xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.np.map5.w0 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 08) 301 001(6c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2UDQS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+50931, 645},
    /* 7433 : VCVTTPS2UDQS ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.np.map5.w0 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 28) 301 001(6c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2UDQS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+50940, 645},
    /* 7434 : VCVTTPS2UDQS zmmreg|mask|z,zmmrm512|b32|sae [rm:fv:    evex.512.np.map5.w0 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 48) 301 001(6c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2UDQS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+50949, 645},
    /* 7435 : VCVTTPS2UQQS xmmreg|mask|z,xmmrm64|b32 [rm:hv:    evex.128.66.map5.w0 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 08) 302 001(6c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2UQQS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+50958, 645},
    /* 7436 : VCVTTPS2UQQS ymmreg|mask|z,xmmrm128|b32 [rm:hv:    evex.256.66.map5.w0 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 28) 302 001(6c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2UQQS, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+50967, 645},
    /* 7437 : VCVTTPS2UQQS zmmreg|mask|z,ymmrm256|b32|sae [rm:hv:    evex.512.66.map5.w0 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 48) 302 001(6c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2UQQS, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+50976, 645},
    /* 7438 : VCVTTSD2SIS reg32,xmmrm64|sae [rm:t1s:   evex.lig.f2.map5.w0 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 08) 306 001(6d) 110 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTSD2SIS, 2, {REG_GPR|BITS32,RM_XMM|BITS64,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+50985, 646},
    /* 7439 : VCVTTSD2SIS reg64,xmmrm64|sae [rm:t1s:   evex.lig.f2.map5.w1 6d /r         ] LONG,PROT,AVX10_2,X86_64 */
        /* 250(f5 ff 08) 306 001(6d) 110 : LIG,LONG,EVEX,PROT,AVX10_2,X86_64 */
        {I_VCVTTSD2SIS, 2, {REG_GPR|BITS64,RM_XMM|BITS64,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+50994, 648},
    /* 7440 : VCVTTSD2USIS reg32,xmmrm64|sae [rm:t1s:   evex.lig.f2.map5.w0 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 08) 306 001(6c) 110 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTSD2USIS, 2, {REG_GPR|BITS32,RM_XMM|BITS64,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+51003, 646},
    /* 7441 : VCVTTSD2USIS reg64,xmmrm64|sae [rm:t1s:   evex.lig.f2.map5.w1 6c /r         ] LONG,PROT,AVX10_2,X86_64 */
        /* 250(f5 ff 08) 306 001(6c) 110 : LIG,LONG,EVEX,PROT,AVX10_2,X86_64 */
        {I_VCVTTSD2USIS, 2, {REG_GPR|BITS64,RM_XMM|BITS64,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+51012, 648},
    /* 7442 : VCVTTSS2SIS reg32,xmmrm32|sae [rm:t1s:   evex.lig.f3.map5.w0 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 08) 306 001(6d) 110 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTSS2SIS, 2, {REG_GPR|BITS32,RM_XMM|BITS32,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+51021, 646},
    /* 7443 : VCVTTSS2SIS reg64,xmmrm32|sae [rm:t1s:   evex.lig.f3.map5.w1 6d /r         ] LONG,PROT,AVX10_2,X86_64 */
        /* 250(f5 fe 08) 306 001(6d) 110 : LIG,LONG,EVEX,PROT,AVX10_2,X86_64 */
        {I_VCVTTSS2SIS, 2, {REG_GPR|BITS64,RM_XMM|BITS32,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+51030, 648},
    /* 7444 : VCVTTSS2USIS reg32,xmmrm32|sae [rm:t1s:   evex.lig.f3.map5.w0 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 08) 306 001(6c) 110 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTSS2USIS, 2, {REG_GPR|BITS32,RM_XMM|BITS32,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+51039, 646},
    /* 7445 : VCVTTSS2USIS reg64,xmmrm32|sae [rm:t1s:   evex.lig.f3.map5.w1 6c /r         ] LONG,PROT,AVX10_2,X86_64 */
        /* 250(f5 fe 08) 306 001(6c) 110 : LIG,LONG,EVEX,PROT,AVX10_2,X86_64 */
        {I_VCVTTSS2USIS, 2, {REG_GPR|BITS64,RM_XMM|BITS32,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+51048, 648},
    /* 7446 : VMOVD xmmreg,xmmrm32 [rm:t1s:   evex.128.f3.0f.w0 7e /r           ] AVX10_2,FUTURE */
        /* 250(f1 7e 08) 306 001(7e) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMOVD, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+51057, 645},
    /* 7447 : VMOVD xmmrm32,xmmreg [mr:t1s:   evex.128.66.0f.w0 d6 /r           ] AVX10_2,FUTURE */
        /* 250(f1 7d 08) 306 001(d6) 101 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMOVD, 2, {RM_XMM|BITS32,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+51066, 645},
    /* 7448 : VMOVW xmmreg,xmmrm16 [rm:t1s:   evex.128.f3.map5.w0 6e /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 08) 306 001(6e) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMOVW, 2, {XMMREG,RM_XMM|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+51075, 645},
    /* 7449 : VMOVW xmmrm16,xmmreg [mr:t1s:   evex.128.f3.map5.w0 7e /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 08) 306 001(7e) 101 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMOVW, 2, {RM_XMM|BITS16,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+51084, 645},
    /* 7450 : NOP imm,reg16,rm16 [irm:      o16 0f ibn /r                     ] SM1-2,UNDOC,P6 */
        /* 320 355 300 121 : SM1,SM2,UNDOC,P6 */
        {I_NOP, 3, {IMM_NORMAL,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+72418, 649},
    /* 7451 : NOP imm,reg32,rm32 [irm:      o32 0f ibn /r                     ] SM1-2,UNDOC,P6 */
        /* 321 355 300 121 : SM1,SM2,UNDOC,P6 */
        {I_NOP, 3, {IMM_NORMAL,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+72423, 649},
    /* 7452 : NOP imm,reg64,rm64 [irm:      o64 0f ibn /r                     ] SM1-2,LONG,PROT,UNDOC,X86_64 */
        /* 324 355 300 121 : SM1,SM2,LONG,PROT,UNDOC,X86_64 */
        {I_NOP, 3, {IMM_NORMAL,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+72428, 650},
};

static const struct itemplate * const itbl_novex_map0_00[] = {
    instrux + 39,
    NULL
};

static const struct itemplate * const itbl_novex_map0_01[] = {
    instrux + 40,
    instrux + 41,
    instrux + 42,
    NULL
};

static const struct itemplate * const itbl_novex_map0_02[] = {
    instrux + 43,
    NULL
};

static const struct itemplate * const itbl_novex_map0_03[] = {
    instrux + 44,
    instrux + 45,
    instrux + 46,
    NULL
};

static const struct itemplate * const itbl_novex_map0_04[] = {
    instrux + 50,
    NULL
};

static const struct itemplate * const itbl_novex_map0_05[] = {
    instrux + 51,
    instrux + 52,
    instrux + 53,
    NULL
};

static const struct itemplate * const itbl_novex_map0_06[] = {
    instrux + 1394,
    NULL
};

static const struct itemplate * const itbl_novex_map0_07[] = {
    instrux + 1400,
    NULL
};

static const struct itemplate * const itbl_novex_map0_08[] = {
    instrux + 88,
    NULL
};

static const struct itemplate * const itbl_novex_map0_09[] = {
    instrux + 89,
    instrux + 90,
    instrux + 91,
    NULL
};

static const struct itemplate * const itbl_novex_map0_0a[] = {
    instrux + 92,
    NULL
};

static const struct itemplate * const itbl_novex_map0_0b[] = {
    instrux + 93,
    instrux + 94,
    instrux + 95,
    NULL
};

static const struct itemplate * const itbl_novex_map0_0c[] = {
    instrux + 99,
    NULL
};

static const struct itemplate * const itbl_novex_map0_0d[] = {
    instrux + 100,
    instrux + 101,
    instrux + 102,
    NULL
};

static const struct itemplate * const itbl_novex_map0_0e[] = {
    instrux + 1395,
    NULL
};

static const struct itemplate * const itbl_novex_map0_10[] = {
    instrux + 137,
    NULL
};

static const struct itemplate * const itbl_novex_map0_11[] = {
    instrux + 138,
    instrux + 139,
    instrux + 140,
    NULL
};

static const struct itemplate * const itbl_novex_map0_12[] = {
    instrux + 141,
    NULL
};

static const struct itemplate * const itbl_novex_map0_13[] = {
    instrux + 142,
    instrux + 143,
    instrux + 144,
    NULL
};

static const struct itemplate * const itbl_novex_map0_14[] = {
    instrux + 148,
    NULL
};

static const struct itemplate * const itbl_novex_map0_15[] = {
    instrux + 149,
    instrux + 150,
    instrux + 151,
    NULL
};

static const struct itemplate * const itbl_novex_map0_16[] = {
    instrux + 1396,
    NULL
};

static const struct itemplate * const itbl_novex_map0_17[] = {
    instrux + 1401,
    NULL
};

static const struct itemplate * const itbl_novex_map0_18[] = {
    instrux + 186,
    NULL
};

static const struct itemplate * const itbl_novex_map0_19[] = {
    instrux + 187,
    instrux + 188,
    instrux + 189,
    NULL
};

static const struct itemplate * const itbl_novex_map0_1a[] = {
    instrux + 190,
    NULL
};

static const struct itemplate * const itbl_novex_map0_1b[] = {
    instrux + 191,
    instrux + 192,
    instrux + 193,
    NULL
};

static const struct itemplate * const itbl_novex_map0_1c[] = {
    instrux + 197,
    NULL
};

static const struct itemplate * const itbl_novex_map0_1d[] = {
    instrux + 198,
    instrux + 199,
    instrux + 200,
    NULL
};

static const struct itemplate * const itbl_novex_map0_1e[] = {
    instrux + 1397,
    NULL
};

static const struct itemplate * const itbl_novex_map0_1f[] = {
    instrux + 1402,
    NULL
};

static const struct itemplate * const itbl_novex_map0_20[] = {
    instrux + 235,
    NULL
};

static const struct itemplate * const itbl_novex_map0_21[] = {
    instrux + 236,
    instrux + 237,
    instrux + 238,
    NULL
};

static const struct itemplate * const itbl_novex_map0_22[] = {
    instrux + 239,
    NULL
};

static const struct itemplate * const itbl_novex_map0_23[] = {
    instrux + 240,
    instrux + 241,
    instrux + 242,
    NULL
};

static const struct itemplate * const itbl_novex_map0_24[] = {
    instrux + 246,
    NULL
};

static const struct itemplate * const itbl_novex_map0_25[] = {
    instrux + 247,
    instrux + 248,
    instrux + 249,
    NULL
};

static const struct itemplate * const itbl_novex_map0_27[] = {
    instrux + 974,
    NULL
};

static const struct itemplate * const itbl_novex_map0_28[] = {
    instrux + 284,
    NULL
};

static const struct itemplate * const itbl_novex_map0_29[] = {
    instrux + 285,
    instrux + 286,
    instrux + 287,
    NULL
};

static const struct itemplate * const itbl_novex_map0_2a[] = {
    instrux + 288,
    NULL
};

static const struct itemplate * const itbl_novex_map0_2b[] = {
    instrux + 289,
    instrux + 290,
    instrux + 291,
    NULL
};

static const struct itemplate * const itbl_novex_map0_2c[] = {
    instrux + 295,
    NULL
};

static const struct itemplate * const itbl_novex_map0_2d[] = {
    instrux + 296,
    instrux + 297,
    instrux + 298,
    NULL
};

static const struct itemplate * const itbl_novex_map0_2f[] = {
    instrux + 975,
    NULL
};

static const struct itemplate * const itbl_novex_map0_30[] = {
    instrux + 333,
    NULL
};

static const struct itemplate * const itbl_novex_map0_31[] = {
    instrux + 334,
    instrux + 335,
    instrux + 336,
    NULL
};

static const struct itemplate * const itbl_novex_map0_32[] = {
    instrux + 337,
    NULL
};

static const struct itemplate * const itbl_novex_map0_33[] = {
    instrux + 338,
    instrux + 339,
    instrux + 340,
    NULL
};

static const struct itemplate * const itbl_novex_map0_34[] = {
    instrux + 344,
    NULL
};

static const struct itemplate * const itbl_novex_map0_35[] = {
    instrux + 345,
    instrux + 346,
    instrux + 347,
    NULL
};

static const struct itemplate * const itbl_novex_map0_37[] = {
    instrux + 968,
    NULL
};

static const struct itemplate * const itbl_novex_map0_38[] = {
    instrux + 382,
    NULL
};

static const struct itemplate * const itbl_novex_map0_39[] = {
    instrux + 383,
    instrux + 384,
    instrux + 385,
    NULL
};

static const struct itemplate * const itbl_novex_map0_3a[] = {
    instrux + 386,
    NULL
};

static const struct itemplate * const itbl_novex_map0_3b[] = {
    instrux + 387,
    instrux + 388,
    instrux + 389,
    NULL
};

static const struct itemplate * const itbl_novex_map0_3c[] = {
    instrux + 393,
    NULL
};

static const struct itemplate * const itbl_novex_map0_3d[] = {
    instrux + 394,
    instrux + 395,
    instrux + 396,
    NULL
};

static const struct itemplate * const itbl_novex_map0_3f[] = {
    instrux + 973,
    NULL
};

static const struct itemplate * const itbl_novex_map0_40[] = {
    instrux + 717,
    instrux + 718,
    NULL
};

static const struct itemplate * const itbl_novex_map0_41[] = {
    instrux + 717,
    instrux + 718,
    NULL
};

static const struct itemplate * const itbl_novex_map0_42[] = {
    instrux + 717,
    instrux + 718,
    NULL
};

static const struct itemplate * const itbl_novex_map0_43[] = {
    instrux + 717,
    instrux + 718,
    NULL
};

static const struct itemplate * const itbl_novex_map0_44[] = {
    instrux + 717,
    instrux + 718,
    NULL
};

static const struct itemplate * const itbl_novex_map0_45[] = {
    instrux + 717,
    instrux + 718,
    NULL
};

static const struct itemplate * const itbl_novex_map0_46[] = {
    instrux + 717,
    instrux + 718,
    NULL
};

static const struct itemplate * const itbl_novex_map0_47[] = {
    instrux + 717,
    instrux + 718,
    NULL
};

static const struct itemplate * const itbl_novex_map0_48[] = {
    instrux + 731,
    instrux + 732,
    NULL
};

static const struct itemplate * const itbl_novex_map0_49[] = {
    instrux + 731,
    instrux + 732,
    NULL
};

static const struct itemplate * const itbl_novex_map0_4a[] = {
    instrux + 731,
    instrux + 732,
    NULL
};

static const struct itemplate * const itbl_novex_map0_4b[] = {
    instrux + 731,
    instrux + 732,
    NULL
};

static const struct itemplate * const itbl_novex_map0_4c[] = {
    instrux + 731,
    instrux + 732,
    NULL
};

static const struct itemplate * const itbl_novex_map0_4d[] = {
    instrux + 731,
    instrux + 732,
    NULL
};

static const struct itemplate * const itbl_novex_map0_4e[] = {
    instrux + 731,
    instrux + 732,
    NULL
};

static const struct itemplate * const itbl_novex_map0_4f[] = {
    instrux + 731,
    instrux + 732,
    NULL
};

static const struct itemplate * const itbl_novex_map0_50[] = {
    instrux + 1730,
    instrux + 1731,
    instrux + 1732,
    NULL
};

static const struct itemplate * const itbl_novex_map0_51[] = {
    instrux + 1730,
    instrux + 1731,
    instrux + 1732,
    NULL
};

static const struct itemplate * const itbl_novex_map0_52[] = {
    instrux + 1730,
    instrux + 1731,
    instrux + 1732,
    NULL
};

static const struct itemplate * const itbl_novex_map0_53[] = {
    instrux + 1730,
    instrux + 1731,
    instrux + 1732,
    NULL
};

static const struct itemplate * const itbl_novex_map0_54[] = {
    instrux + 1730,
    instrux + 1731,
    instrux + 1732,
    NULL
};

static const struct itemplate * const itbl_novex_map0_55[] = {
    instrux + 1730,
    instrux + 1731,
    instrux + 1732,
    NULL
};

static const struct itemplate * const itbl_novex_map0_56[] = {
    instrux + 1730,
    instrux + 1731,
    instrux + 1732,
    NULL
};

static const struct itemplate * const itbl_novex_map0_57[] = {
    instrux + 1730,
    instrux + 1731,
    instrux + 1732,
    NULL
};

static const struct itemplate * const itbl_novex_map0_58[] = {
    instrux + 1742,
    instrux + 1743,
    instrux + 1744,
    NULL
};

static const struct itemplate * const itbl_novex_map0_59[] = {
    instrux + 1742,
    instrux + 1743,
    instrux + 1744,
    NULL
};

static const struct itemplate * const itbl_novex_map0_5a[] = {
    instrux + 1742,
    instrux + 1743,
    instrux + 1744,
    NULL
};

static const struct itemplate * const itbl_novex_map0_5b[] = {
    instrux + 1742,
    instrux + 1743,
    instrux + 1744,
    NULL
};

static const struct itemplate * const itbl_novex_map0_5c[] = {
    instrux + 1742,
    instrux + 1743,
    instrux + 1744,
    NULL
};

static const struct itemplate * const itbl_novex_map0_5d[] = {
    instrux + 1742,
    instrux + 1743,
    instrux + 1744,
    NULL
};

static const struct itemplate * const itbl_novex_map0_5e[] = {
    instrux + 1742,
    instrux + 1743,
    instrux + 1744,
    NULL
};

static const struct itemplate * const itbl_novex_map0_5f[] = {
    instrux + 1742,
    instrux + 1743,
    instrux + 1744,
    NULL
};

static const struct itemplate * const itbl_novex_map0_60[] = {
    instrux + 1748,
    instrux + 1749,
    instrux + 1750,
    NULL
};

static const struct itemplate * const itbl_novex_map0_61[] = {
    instrux + 1751,
    instrux + 1752,
    instrux + 1753,
    NULL
};

static const struct itemplate * const itbl_novex_map0_62[] = {
    instrux + 1762,
    instrux + 1763,
    NULL
};

static const struct itemplate * const itbl_novex_map0_63[] = {
    instrux + 1002,
    instrux + 1003,
    instrux + 1004,
    instrux + 1413,
    instrux + 1414,
    NULL
};

static const struct itemplate * const itbl_novex_map0_68[] = {
    instrux + 1739,
    instrux + 1740,
    instrux + 1741,
    NULL
};

static const struct itemplate * const itbl_novex_map0_69[] = {
    instrux + 759,
    instrux + 760,
    instrux + 761,
    NULL
};

static const struct itemplate * const itbl_novex_map0_6a[] = {
    instrux + 1736,
    instrux + 1737,
    instrux + 1738,
    NULL
};

static const struct itemplate * const itbl_novex_map0_6b[] = {
    instrux + 756,
    instrux + 757,
    instrux + 758,
    NULL
};

static const struct itemplate * const itbl_novex_map0_6c[] = {
    instrux + 1273,
    NULL
};

static const struct itemplate * const itbl_novex_map0_6d[] = {
    instrux + 1274,
    instrux + 1275,
    NULL
};

static const struct itemplate * const itbl_novex_map0_6e[] = {
    instrux + 1276,
    NULL
};

static const struct itemplate * const itbl_novex_map0_6f[] = {
    instrux + 1277,
    instrux + 1278,
    NULL
};

static const struct itemplate * const itbl_novex_map0_70[] = {
    instrux + 1054,
    instrux + 1070,
    instrux + 1086,
    NULL
};

static const struct itemplate * const itbl_novex_map0_71[] = {
    instrux + 1051,
    instrux + 1067,
    instrux + 1083,
    NULL
};

static const struct itemplate * const itbl_novex_map0_72[] = {
    instrux + 1044,
    instrux + 1060,
    instrux + 1076,
    NULL
};

static const struct itemplate * const itbl_novex_map0_73[] = {
    instrux + 1048,
    instrux + 1064,
    instrux + 1080,
    NULL
};

static const struct itemplate * const itbl_novex_map0_74[] = {
    instrux + 1058,
    instrux + 1074,
    instrux + 1090,
    NULL
};

static const struct itemplate * const itbl_novex_map0_75[] = {
    instrux + 1053,
    instrux + 1069,
    instrux + 1085,
    NULL
};

static const struct itemplate * const itbl_novex_map0_76[] = {
    instrux + 1047,
    instrux + 1063,
    instrux + 1079,
    NULL
};

static const struct itemplate * const itbl_novex_map0_77[] = {
    instrux + 1043,
    instrux + 1059,
    instrux + 1075,
    NULL
};

static const struct itemplate * const itbl_novex_map0_78[] = {
    instrux + 1057,
    instrux + 1073,
    instrux + 1089,
    NULL
};

static const struct itemplate * const itbl_novex_map0_79[] = {
    instrux + 1052,
    instrux + 1068,
    instrux + 1084,
    NULL
};

static const struct itemplate * const itbl_novex_map0_7a[] = {
    instrux + 1055,
    instrux + 1071,
    instrux + 1087,
    NULL
};

static const struct itemplate * const itbl_novex_map0_7b[] = {
    instrux + 1056,
    instrux + 1072,
    instrux + 1088,
    NULL
};

static const struct itemplate * const itbl_novex_map0_7c[] = {
    instrux + 1046,
    instrux + 1062,
    instrux + 1078,
    NULL
};

static const struct itemplate * const itbl_novex_map0_7d[] = {
    instrux + 1050,
    instrux + 1066,
    instrux + 1082,
    NULL
};

static const struct itemplate * const itbl_novex_map0_7e[] = {
    instrux + 1049,
    instrux + 1065,
    instrux + 1081,
    NULL
};

static const struct itemplate * const itbl_novex_map0_7f[] = {
    instrux + 1045,
    instrux + 1061,
    instrux + 1077,
    NULL
};

static const struct itemplate * const itbl_novex_map0_80[] = {
    instrux + 54,
    instrux + 103,
    instrux + 152,
    instrux + 201,
    instrux + 250,
    instrux + 299,
    instrux + 348,
    instrux + 397,
    NULL
};

static const struct itemplate * const itbl_novex_map0_81[] = {
    instrux + 55,
    instrux + 56,
    instrux + 57,
    instrux + 104,
    instrux + 105,
    instrux + 106,
    instrux + 153,
    instrux + 154,
    instrux + 155,
    instrux + 202,
    instrux + 203,
    instrux + 204,
    instrux + 251,
    instrux + 252,
    instrux + 253,
    instrux + 300,
    instrux + 301,
    instrux + 302,
    instrux + 349,
    instrux + 350,
    instrux + 351,
    instrux + 398,
    instrux + 399,
    instrux + 400,
    NULL
};

static const struct itemplate * const itbl_novex_map0_83[] = {
    instrux + 47,
    instrux + 48,
    instrux + 49,
    instrux + 96,
    instrux + 97,
    instrux + 98,
    instrux + 145,
    instrux + 146,
    instrux + 147,
    instrux + 194,
    instrux + 195,
    instrux + 196,
    instrux + 243,
    instrux + 244,
    instrux + 245,
    instrux + 292,
    instrux + 293,
    instrux + 294,
    instrux + 341,
    instrux + 342,
    instrux + 343,
    instrux + 390,
    instrux + 391,
    instrux + 392,
    NULL
};

static const struct itemplate * const itbl_novex_map0_84[] = {
    instrux + 401,
    NULL
};

static const struct itemplate * const itbl_novex_map0_85[] = {
    instrux + 402,
    instrux + 403,
    instrux + 404,
    NULL
};

static const struct itemplate * const itbl_novex_map0_86[] = {
    instrux + 1024,
    NULL
};

static const struct itemplate * const itbl_novex_map0_87[] = {
    instrux + 1025,
    instrux + 1026,
    instrux + 1027,
    NULL
};

static const struct itemplate * const itbl_novex_map0_88[] = {
    instrux + 12,
    NULL
};

static const struct itemplate * const itbl_novex_map0_89[] = {
    instrux + 13,
    instrux + 14,
    instrux + 15,
    NULL
};

static const struct itemplate * const itbl_novex_map0_8a[] = {
    instrux + 16,
    NULL
};

static const struct itemplate * const itbl_novex_map0_8b[] = {
    instrux + 17,
    instrux + 18,
    instrux + 19,
    NULL
};

static const struct itemplate * const itbl_novex_map0_8c[] = {
    instrux + 1373,
    instrux + 1374,
    instrux + 1375,
    instrux + 1376,
    NULL
};

static const struct itemplate * const itbl_novex_map0_8d[] = {
    instrux + 36,
    instrux + 37,
    instrux + 38,
    NULL
};

static const struct itemplate * const itbl_novex_map0_8e[] = {
    instrux + 1377,
    instrux + 1378,
    instrux + 1379,
    instrux + 1380,
    NULL
};

static const struct itemplate * const itbl_novex_map0_8f[] = {
    instrux + 1745,
    instrux + 1746,
    instrux + 1747,
    NULL
};

static const struct itemplate * const itbl_novex_map0_90[] = {
    instrux + 0,
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    instrux + 1345,
    NULL
};

static const struct itemplate * const itbl_novex_map0_91[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_novex_map0_92[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_novex_map0_93[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_novex_map0_94[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_novex_map0_95[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_novex_map0_96[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_novex_map0_97[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_novex_map0_98[] = {
    instrux + 990,
    instrux + 992,
    instrux + 995,
    NULL
};

static const struct itemplate * const itbl_novex_map0_99[] = {
    instrux + 991,
    instrux + 993,
    instrux + 994,
    NULL
};

static const struct itemplate * const itbl_novex_map0_9a[] = {
    instrux + 1198,
    instrux + 1199,
    NULL
};

static const struct itemplate * const itbl_novex_map0_9c[] = {
    instrux + 1245,
    instrux + 1246,
    instrux + 1247,
    instrux + 1248,
    NULL
};

static const struct itemplate * const itbl_novex_map0_9d[] = {
    instrux + 1249,
    instrux + 1250,
    instrux + 1251,
    instrux + 1252,
    NULL
};

static const struct itemplate * const itbl_novex_map0_9e[] = {
    instrux + 1243,
    NULL
};

static const struct itemplate * const itbl_novex_map0_9f[] = {
    instrux + 1242,
    NULL
};

static const struct itemplate * const itbl_novex_map0_a0[] = {
    instrux + 4,
    NULL
};

static const struct itemplate * const itbl_novex_map0_a1[] = {
    instrux + 5,
    instrux + 6,
    instrux + 7,
    NULL
};

static const struct itemplate * const itbl_novex_map0_a2[] = {
    instrux + 8,
    NULL
};

static const struct itemplate * const itbl_novex_map0_a3[] = {
    instrux + 9,
    instrux + 10,
    instrux + 11,
    NULL
};

static const struct itemplate * const itbl_novex_map0_a4[] = {
    instrux + 1261,
    NULL
};

static const struct itemplate * const itbl_novex_map0_a5[] = {
    instrux + 1262,
    instrux + 1263,
    instrux + 1264,
    NULL
};

static const struct itemplate * const itbl_novex_map0_a6[] = {
    instrux + 1253,
    NULL
};

static const struct itemplate * const itbl_novex_map0_a7[] = {
    instrux + 1254,
    instrux + 1255,
    instrux + 1256,
    NULL
};

static const struct itemplate * const itbl_novex_map0_a8[] = {
    instrux + 405,
    NULL
};

static const struct itemplate * const itbl_novex_map0_a9[] = {
    instrux + 406,
    instrux + 407,
    instrux + 408,
    NULL
};

static const struct itemplate * const itbl_novex_map0_aa[] = {
    instrux + 1265,
    NULL
};

static const struct itemplate * const itbl_novex_map0_ab[] = {
    instrux + 1266,
    instrux + 1267,
    instrux + 1268,
    NULL
};

static const struct itemplate * const itbl_novex_map0_ac[] = {
    instrux + 1257,
    NULL
};

static const struct itemplate * const itbl_novex_map0_ad[] = {
    instrux + 1258,
    instrux + 1259,
    instrux + 1260,
    NULL
};

static const struct itemplate * const itbl_novex_map0_ae[] = {
    instrux + 1269,
    NULL
};

static const struct itemplate * const itbl_novex_map0_af[] = {
    instrux + 1270,
    instrux + 1271,
    instrux + 1272,
    NULL
};

static const struct itemplate * const itbl_novex_map0_b0[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_novex_map0_b1[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_novex_map0_b2[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_novex_map0_b3[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_novex_map0_b4[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_novex_map0_b5[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_novex_map0_b6[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_novex_map0_b7[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_novex_map0_b8[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_novex_map0_b9[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_novex_map0_ba[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_novex_map0_bb[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_novex_map0_bc[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_novex_map0_bd[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_novex_map0_be[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_novex_map0_bf[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_novex_map0_c0[] = {
    instrux + 421,
    instrux + 433,
    instrux + 445,
    instrux + 457,
    instrux + 469,
    instrux + 481,
    instrux + 493,
    instrux + 505,
    NULL
};

static const struct itemplate * const itbl_novex_map0_c1[] = {
    instrux + 422,
    instrux + 423,
    instrux + 424,
    instrux + 434,
    instrux + 435,
    instrux + 436,
    instrux + 446,
    instrux + 447,
    instrux + 448,
    instrux + 458,
    instrux + 459,
    instrux + 460,
    instrux + 470,
    instrux + 471,
    instrux + 472,
    instrux + 482,
    instrux + 483,
    instrux + 484,
    instrux + 494,
    instrux + 495,
    instrux + 496,
    instrux + 506,
    instrux + 507,
    instrux + 508,
    NULL
};

static const struct itemplate * const itbl_novex_map0_c2[] = {
    instrux + 1207,
    instrux + 1208,
    instrux + 1209,
    instrux + 1210,
    NULL
};

static const struct itemplate * const itbl_novex_map0_c3[] = {
    instrux + 1203,
    instrux + 1204,
    instrux + 1205,
    instrux + 1206,
    NULL
};

static const struct itemplate * const itbl_novex_map0_c4[] = {
    instrux + 1383,
    instrux + 1384,
    NULL
};

static const struct itemplate * const itbl_novex_map0_c5[] = {
    instrux + 1381,
    instrux + 1382,
    NULL
};

static const struct itemplate * const itbl_novex_map0_c6[] = {
    instrux + 24,
    instrux + 4364,
    NULL
};

static const struct itemplate * const itbl_novex_map0_c7[] = {
    instrux + 25,
    instrux + 26,
    instrux + 27,
    instrux + 4365,
    instrux + 4366,
    instrux + 4367,
    NULL
};

static const struct itemplate * const itbl_novex_map0_c8[] = {
    instrux + 1754,
    instrux + 1755,
    instrux + 1756,
    instrux + 1757,
    NULL
};

static const struct itemplate * const itbl_novex_map0_c9[] = {
    instrux + 1758,
    instrux + 1759,
    instrux + 1760,
    instrux + 1761,
    NULL
};

static const struct itemplate * const itbl_novex_map0_ca[] = {
    instrux + 1215,
    instrux + 1216,
    instrux + 1217,
    instrux + 1218,
    NULL
};

static const struct itemplate * const itbl_novex_map0_cb[] = {
    instrux + 1211,
    instrux + 1212,
    instrux + 1213,
    instrux + 1214,
    NULL
};

static const struct itemplate * const itbl_novex_map0_cc[] = {
    instrux + 1221,
    NULL
};

static const struct itemplate * const itbl_novex_map0_cd[] = {
    instrux + 1219,
    NULL
};

static const struct itemplate * const itbl_novex_map0_ce[] = {
    instrux + 1222,
    NULL
};

static const struct itemplate * const itbl_novex_map0_cf[] = {
    instrux + 1227,
    instrux + 1228,
    instrux + 1229,
    instrux + 1230,
    NULL
};

static const struct itemplate * const itbl_novex_map0_d0[] = {
    instrux + 413,
    instrux + 425,
    instrux + 437,
    instrux + 449,
    instrux + 461,
    instrux + 473,
    instrux + 485,
    instrux + 497,
    NULL
};

static const struct itemplate * const itbl_novex_map0_d1[] = {
    instrux + 414,
    instrux + 415,
    instrux + 416,
    instrux + 426,
    instrux + 427,
    instrux + 428,
    instrux + 438,
    instrux + 439,
    instrux + 440,
    instrux + 450,
    instrux + 451,
    instrux + 452,
    instrux + 462,
    instrux + 463,
    instrux + 464,
    instrux + 474,
    instrux + 475,
    instrux + 476,
    instrux + 486,
    instrux + 487,
    instrux + 488,
    instrux + 498,
    instrux + 499,
    instrux + 500,
    NULL
};

static const struct itemplate * const itbl_novex_map0_d2[] = {
    instrux + 417,
    instrux + 429,
    instrux + 441,
    instrux + 453,
    instrux + 465,
    instrux + 477,
    instrux + 489,
    instrux + 501,
    NULL
};

static const struct itemplate * const itbl_novex_map0_d3[] = {
    instrux + 418,
    instrux + 419,
    instrux + 420,
    instrux + 430,
    instrux + 431,
    instrux + 432,
    instrux + 442,
    instrux + 443,
    instrux + 444,
    instrux + 454,
    instrux + 455,
    instrux + 456,
    instrux + 466,
    instrux + 467,
    instrux + 468,
    instrux + 478,
    instrux + 479,
    instrux + 480,
    instrux + 490,
    instrux + 491,
    instrux + 492,
    instrux + 502,
    instrux + 503,
    instrux + 504,
    NULL
};

static const struct itemplate * const itbl_novex_map0_d4[] = {
    instrux + 971,
    instrux + 972,
    NULL
};

static const struct itemplate * const itbl_novex_map0_d5[] = {
    instrux + 969,
    instrux + 970,
    NULL
};

static const struct itemplate * const itbl_novex_map0_d6[] = {
    instrux + 1244,
    instrux + 1781,
    NULL
};

static const struct itemplate * const itbl_novex_map0_d7[] = {
    instrux + 1783,
    NULL
};

static const struct itemplate * const itbl_novex_map0_d8[] = {
    instrux + 1455,
    instrux + 1458,
    instrux + 1460,
    instrux + 1485,
    instrux + 1487,
    instrux + 1488,
    instrux + 1493,
    instrux + 1495,
    instrux + 1496,
    instrux + 1501,
    instrux + 1504,
    instrux + 1506,
    instrux + 1509,
    instrux + 1513,
    instrux + 1514,
    instrux + 1565,
    instrux + 1569,
    instrux + 1570,
    instrux + 1606,
    instrux + 1610,
    instrux + 1611,
    instrux + 1614,
    instrux + 1618,
    instrux + 1619,
    NULL
};

static const struct itemplate * const itbl_novex_map0_d9[] = {
    instrux + 1453,
    instrux + 1454,
    instrux + 1467,
    instrux + 1498,
    instrux + 1499,
    instrux + 1538,
    instrux + 1552,
    instrux + 1555,
    instrux + 1556,
    instrux + 1557,
    instrux + 1558,
    instrux + 1559,
    instrux + 1560,
    instrux + 1561,
    instrux + 1562,
    instrux + 1563,
    instrux + 1564,
    instrux + 1577,
    instrux + 1579,
    instrux + 1580,
    instrux + 1583,
    instrux + 1584,
    instrux + 1585,
    instrux + 1586,
    instrux + 1587,
    instrux + 1590,
    instrux + 1592,
    instrux + 1593,
    instrux + 1594,
    instrux + 1595,
    instrux + 1598,
    instrux + 1599,
    instrux + 1600,
    instrux + 1622,
    instrux + 1632,
    instrux + 1633,
    instrux + 1634,
    instrux + 1635,
    instrux + 1636,
    instrux + 1637,
    instrux + 1638,
    NULL
};

static const struct itemplate * const itbl_novex_map0_da[] = {
    instrux + 1469,
    instrux + 1470,
    instrux + 1471,
    instrux + 1472,
    instrux + 1473,
    instrux + 1474,
    instrux + 1483,
    instrux + 1484,
    instrux + 1523,
    instrux + 1525,
    instrux + 1527,
    instrux + 1529,
    instrux + 1531,
    instrux + 1536,
    instrux + 1548,
    instrux + 1550,
    instrux + 1631,
    NULL
};

static const struct itemplate * const itbl_novex_map0_db[] = {
    instrux + 1468,
    instrux + 1475,
    instrux + 1476,
    instrux + 1477,
    instrux + 1478,
    instrux + 1479,
    instrux + 1480,
    instrux + 1481,
    instrux + 1482,
    instrux + 1489,
    instrux + 1490,
    instrux + 1500,
    instrux + 1518,
    instrux + 1533,
    instrux + 1539,
    instrux + 1540,
    instrux + 1542,
    instrux + 1546,
    instrux + 1554,
    instrux + 1573,
    instrux + 1574,
    instrux + 1575,
    instrux + 1576,
    instrux + 1591,
    instrux + 1602,
    instrux + 1625,
    instrux + 1626,
    NULL
};

static const struct itemplate * const itbl_novex_map0_dc[] = {
    instrux + 1456,
    instrux + 1457,
    instrux + 1459,
    instrux + 1486,
    instrux + 1494,
    instrux + 1502,
    instrux + 1503,
    instrux + 1505,
    instrux + 1510,
    instrux + 1511,
    instrux + 1512,
    instrux + 1566,
    instrux + 1567,
    instrux + 1568,
    instrux + 1607,
    instrux + 1608,
    instrux + 1609,
    instrux + 1615,
    instrux + 1616,
    instrux + 1617,
    NULL
};

static const struct itemplate * const itbl_novex_map0_dd[] = {
    instrux + 1519,
    instrux + 1520,
    instrux + 1547,
    instrux + 1553,
    instrux + 1578,
    instrux + 1581,
    instrux + 1588,
    instrux + 1589,
    instrux + 1596,
    instrux + 1597,
    instrux + 1601,
    instrux + 1603,
    instrux + 1604,
    instrux + 1623,
    instrux + 1624,
    instrux + 1629,
    instrux + 1630,
    NULL
};

static const struct itemplate * const itbl_novex_map0_de[] = {
    instrux + 1461,
    instrux + 1462,
    instrux + 1497,
    instrux + 1507,
    instrux + 1508,
    instrux + 1515,
    instrux + 1516,
    instrux + 1524,
    instrux + 1526,
    instrux + 1528,
    instrux + 1530,
    instrux + 1532,
    instrux + 1537,
    instrux + 1549,
    instrux + 1551,
    instrux + 1571,
    instrux + 1572,
    instrux + 1612,
    instrux + 1613,
    instrux + 1620,
    instrux + 1621,
    NULL
};

static const struct itemplate * const itbl_novex_map0_df[] = {
    instrux + 1463,
    instrux + 1464,
    instrux + 1465,
    instrux + 1466,
    instrux + 1491,
    instrux + 1492,
    instrux + 1521,
    instrux + 1522,
    instrux + 1534,
    instrux + 1535,
    instrux + 1541,
    instrux + 1543,
    instrux + 1544,
    instrux + 1545,
    instrux + 1582,
    instrux + 1605,
    instrux + 1627,
    instrux + 1628,
    NULL
};

static const struct itemplate * const itbl_novex_map0_e0[] = {
    instrux + 1151,
    instrux + 1152,
    instrux + 1153,
    instrux + 1166,
    instrux + 1167,
    instrux + 1168,
    instrux + 1169,
    instrux + 1170,
    instrux + 1171,
    instrux + 1178,
    instrux + 1179,
    instrux + 1180,
    instrux + 1181,
    instrux + 1182,
    instrux + 1183,
    NULL
};

static const struct itemplate * const itbl_novex_map0_e1[] = {
    instrux + 1148,
    instrux + 1149,
    instrux + 1150,
    instrux + 1160,
    instrux + 1161,
    instrux + 1162,
    instrux + 1163,
    instrux + 1164,
    instrux + 1165,
    instrux + 1172,
    instrux + 1173,
    instrux + 1174,
    instrux + 1175,
    instrux + 1176,
    instrux + 1177,
    NULL
};

static const struct itemplate * const itbl_novex_map0_e2[] = {
    instrux + 1145,
    instrux + 1146,
    instrux + 1147,
    instrux + 1154,
    instrux + 1155,
    instrux + 1156,
    instrux + 1157,
    instrux + 1158,
    instrux + 1159,
    NULL
};

static const struct itemplate * const itbl_novex_map0_e3[] = {
    instrux + 1139,
    instrux + 1140,
    instrux + 1141,
    instrux + 1142,
    instrux + 1143,
    instrux + 1144,
    NULL
};

static const struct itemplate * const itbl_novex_map0_e4[] = {
    instrux + 1361,
    NULL
};

static const struct itemplate * const itbl_novex_map0_e5[] = {
    instrux + 1362,
    instrux + 1363,
    NULL
};

static const struct itemplate * const itbl_novex_map0_e6[] = {
    instrux + 1367,
    NULL
};

static const struct itemplate * const itbl_novex_map0_e7[] = {
    instrux + 1368,
    instrux + 1369,
    NULL
};

static const struct itemplate * const itbl_novex_map0_e8[] = {
    instrux + 1192,
    instrux + 1193,
    instrux + 1194,
    NULL
};

static const struct itemplate * const itbl_novex_map0_e9[] = {
    instrux + 1032,
    instrux + 1033,
    instrux + 1034,
    NULL
};

static const struct itemplate * const itbl_novex_map0_ea[] = {
    instrux + 1038,
    instrux + 1039,
    NULL
};

static const struct itemplate * const itbl_novex_map0_eb[] = {
    instrux + 1029,
    instrux + 1030,
    instrux + 1031,
    NULL
};

static const struct itemplate * const itbl_novex_map0_ec[] = {
    instrux + 1364,
    NULL
};

static const struct itemplate * const itbl_novex_map0_ed[] = {
    instrux + 1365,
    instrux + 1366,
    NULL
};

static const struct itemplate * const itbl_novex_map0_ee[] = {
    instrux + 1370,
    NULL
};

static const struct itemplate * const itbl_novex_map0_ef[] = {
    instrux + 1371,
    instrux + 1372,
    NULL
};

static const struct itemplate * const itbl_novex_map0_f1[] = {
    instrux + 1220,
    NULL
};

static const struct itemplate * const itbl_novex_map0_f4[] = {
    instrux + 1344,
    NULL
};

static const struct itemplate * const itbl_novex_map0_f5[] = {
    instrux + 1241,
    NULL
};

static const struct itemplate * const itbl_novex_map0_f6[] = {
    instrux + 409,
    instrux + 745,
    instrux + 774,
    instrux + 786,
    instrux + 794,
    instrux + 802,
    instrux + 814,
    NULL
};

static const struct itemplate * const itbl_novex_map0_f7[] = {
    instrux + 410,
    instrux + 411,
    instrux + 412,
    instrux + 746,
    instrux + 747,
    instrux + 748,
    instrux + 775,
    instrux + 776,
    instrux + 777,
    instrux + 787,
    instrux + 788,
    instrux + 789,
    instrux + 795,
    instrux + 796,
    instrux + 797,
    instrux + 803,
    instrux + 804,
    instrux + 805,
    instrux + 815,
    instrux + 816,
    instrux + 817,
    NULL
};

static const struct itemplate * const itbl_novex_map0_f8[] = {
    instrux + 1233,
    NULL
};

static const struct itemplate * const itbl_novex_map0_f9[] = {
    instrux + 1237,
    NULL
};

static const struct itemplate * const itbl_novex_map0_fa[] = {
    instrux + 1235,
    NULL
};

static const struct itemplate * const itbl_novex_map0_fb[] = {
    instrux + 1239,
    NULL
};

static const struct itemplate * const itbl_novex_map0_fc[] = {
    instrux + 1234,
    NULL
};

static const struct itemplate * const itbl_novex_map0_fd[] = {
    instrux + 1238,
    NULL
};

static const struct itemplate * const itbl_novex_map0_fe[] = {
    instrux + 719,
    instrux + 733,
    NULL
};

static const struct itemplate * const itbl_novex_map0_ff[] = {
    instrux + 720,
    instrux + 721,
    instrux + 722,
    instrux + 734,
    instrux + 735,
    instrux + 736,
    instrux + 1035,
    instrux + 1036,
    instrux + 1037,
    instrux + 1040,
    instrux + 1041,
    instrux + 1042,
    instrux + 1195,
    instrux + 1196,
    instrux + 1197,
    instrux + 1200,
    instrux + 1201,
    instrux + 1202,
    instrux + 1733,
    instrux + 1734,
    instrux + 1735,
    NULL
};

static const struct itemplate * const * const itbl_novex_map0[256] = {
    itbl_novex_map0_00,
    itbl_novex_map0_01,
    itbl_novex_map0_02,
    itbl_novex_map0_03,
    itbl_novex_map0_04,
    itbl_novex_map0_05,
    itbl_novex_map0_06,
    itbl_novex_map0_07,
    itbl_novex_map0_08,
    itbl_novex_map0_09,
    itbl_novex_map0_0a,
    itbl_novex_map0_0b,
    itbl_novex_map0_0c,
    itbl_novex_map0_0d,
    itbl_novex_map0_0e,
    NULL,
    itbl_novex_map0_10,
    itbl_novex_map0_11,
    itbl_novex_map0_12,
    itbl_novex_map0_13,
    itbl_novex_map0_14,
    itbl_novex_map0_15,
    itbl_novex_map0_16,
    itbl_novex_map0_17,
    itbl_novex_map0_18,
    itbl_novex_map0_19,
    itbl_novex_map0_1a,
    itbl_novex_map0_1b,
    itbl_novex_map0_1c,
    itbl_novex_map0_1d,
    itbl_novex_map0_1e,
    itbl_novex_map0_1f,
    itbl_novex_map0_20,
    itbl_novex_map0_21,
    itbl_novex_map0_22,
    itbl_novex_map0_23,
    itbl_novex_map0_24,
    itbl_novex_map0_25,
    NULL,
    itbl_novex_map0_27,
    itbl_novex_map0_28,
    itbl_novex_map0_29,
    itbl_novex_map0_2a,
    itbl_novex_map0_2b,
    itbl_novex_map0_2c,
    itbl_novex_map0_2d,
    NULL,
    itbl_novex_map0_2f,
    itbl_novex_map0_30,
    itbl_novex_map0_31,
    itbl_novex_map0_32,
    itbl_novex_map0_33,
    itbl_novex_map0_34,
    itbl_novex_map0_35,
    NULL,
    itbl_novex_map0_37,
    itbl_novex_map0_38,
    itbl_novex_map0_39,
    itbl_novex_map0_3a,
    itbl_novex_map0_3b,
    itbl_novex_map0_3c,
    itbl_novex_map0_3d,
    NULL,
    itbl_novex_map0_3f,
    itbl_novex_map0_40,
    itbl_novex_map0_41,
    itbl_novex_map0_42,
    itbl_novex_map0_43,
    itbl_novex_map0_44,
    itbl_novex_map0_45,
    itbl_novex_map0_46,
    itbl_novex_map0_47,
    itbl_novex_map0_48,
    itbl_novex_map0_49,
    itbl_novex_map0_4a,
    itbl_novex_map0_4b,
    itbl_novex_map0_4c,
    itbl_novex_map0_4d,
    itbl_novex_map0_4e,
    itbl_novex_map0_4f,
    itbl_novex_map0_50,
    itbl_novex_map0_51,
    itbl_novex_map0_52,
    itbl_novex_map0_53,
    itbl_novex_map0_54,
    itbl_novex_map0_55,
    itbl_novex_map0_56,
    itbl_novex_map0_57,
    itbl_novex_map0_58,
    itbl_novex_map0_59,
    itbl_novex_map0_5a,
    itbl_novex_map0_5b,
    itbl_novex_map0_5c,
    itbl_novex_map0_5d,
    itbl_novex_map0_5e,
    itbl_novex_map0_5f,
    itbl_novex_map0_60,
    itbl_novex_map0_61,
    itbl_novex_map0_62,
    itbl_novex_map0_63,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map0_68,
    itbl_novex_map0_69,
    itbl_novex_map0_6a,
    itbl_novex_map0_6b,
    itbl_novex_map0_6c,
    itbl_novex_map0_6d,
    itbl_novex_map0_6e,
    itbl_novex_map0_6f,
    itbl_novex_map0_70,
    itbl_novex_map0_71,
    itbl_novex_map0_72,
    itbl_novex_map0_73,
    itbl_novex_map0_74,
    itbl_novex_map0_75,
    itbl_novex_map0_76,
    itbl_novex_map0_77,
    itbl_novex_map0_78,
    itbl_novex_map0_79,
    itbl_novex_map0_7a,
    itbl_novex_map0_7b,
    itbl_novex_map0_7c,
    itbl_novex_map0_7d,
    itbl_novex_map0_7e,
    itbl_novex_map0_7f,
    itbl_novex_map0_80,
    itbl_novex_map0_81,
    NULL,
    itbl_novex_map0_83,
    itbl_novex_map0_84,
    itbl_novex_map0_85,
    itbl_novex_map0_86,
    itbl_novex_map0_87,
    itbl_novex_map0_88,
    itbl_novex_map0_89,
    itbl_novex_map0_8a,
    itbl_novex_map0_8b,
    itbl_novex_map0_8c,
    itbl_novex_map0_8d,
    itbl_novex_map0_8e,
    itbl_novex_map0_8f,
    itbl_novex_map0_90,
    itbl_novex_map0_91,
    itbl_novex_map0_92,
    itbl_novex_map0_93,
    itbl_novex_map0_94,
    itbl_novex_map0_95,
    itbl_novex_map0_96,
    itbl_novex_map0_97,
    itbl_novex_map0_98,
    itbl_novex_map0_99,
    itbl_novex_map0_9a,
    NULL,
    itbl_novex_map0_9c,
    itbl_novex_map0_9d,
    itbl_novex_map0_9e,
    itbl_novex_map0_9f,
    itbl_novex_map0_a0,
    itbl_novex_map0_a1,
    itbl_novex_map0_a2,
    itbl_novex_map0_a3,
    itbl_novex_map0_a4,
    itbl_novex_map0_a5,
    itbl_novex_map0_a6,
    itbl_novex_map0_a7,
    itbl_novex_map0_a8,
    itbl_novex_map0_a9,
    itbl_novex_map0_aa,
    itbl_novex_map0_ab,
    itbl_novex_map0_ac,
    itbl_novex_map0_ad,
    itbl_novex_map0_ae,
    itbl_novex_map0_af,
    itbl_novex_map0_b0,
    itbl_novex_map0_b1,
    itbl_novex_map0_b2,
    itbl_novex_map0_b3,
    itbl_novex_map0_b4,
    itbl_novex_map0_b5,
    itbl_novex_map0_b6,
    itbl_novex_map0_b7,
    itbl_novex_map0_b8,
    itbl_novex_map0_b9,
    itbl_novex_map0_ba,
    itbl_novex_map0_bb,
    itbl_novex_map0_bc,
    itbl_novex_map0_bd,
    itbl_novex_map0_be,
    itbl_novex_map0_bf,
    itbl_novex_map0_c0,
    itbl_novex_map0_c1,
    itbl_novex_map0_c2,
    itbl_novex_map0_c3,
    itbl_novex_map0_c4,
    itbl_novex_map0_c5,
    itbl_novex_map0_c6,
    itbl_novex_map0_c7,
    itbl_novex_map0_c8,
    itbl_novex_map0_c9,
    itbl_novex_map0_ca,
    itbl_novex_map0_cb,
    itbl_novex_map0_cc,
    itbl_novex_map0_cd,
    itbl_novex_map0_ce,
    itbl_novex_map0_cf,
    itbl_novex_map0_d0,
    itbl_novex_map0_d1,
    itbl_novex_map0_d2,
    itbl_novex_map0_d3,
    itbl_novex_map0_d4,
    itbl_novex_map0_d5,
    itbl_novex_map0_d6,
    itbl_novex_map0_d7,
    itbl_novex_map0_d8,
    itbl_novex_map0_d9,
    itbl_novex_map0_da,
    itbl_novex_map0_db,
    itbl_novex_map0_dc,
    itbl_novex_map0_dd,
    itbl_novex_map0_de,
    itbl_novex_map0_df,
    itbl_novex_map0_e0,
    itbl_novex_map0_e1,
    itbl_novex_map0_e2,
    itbl_novex_map0_e3,
    itbl_novex_map0_e4,
    itbl_novex_map0_e5,
    itbl_novex_map0_e6,
    itbl_novex_map0_e7,
    itbl_novex_map0_e8,
    itbl_novex_map0_e9,
    itbl_novex_map0_ea,
    itbl_novex_map0_eb,
    itbl_novex_map0_ec,
    itbl_novex_map0_ed,
    itbl_novex_map0_ee,
    itbl_novex_map0_ef,
    NULL,
    itbl_novex_map0_f1,
    NULL,
    NULL,
    itbl_novex_map0_f4,
    itbl_novex_map0_f5,
    itbl_novex_map0_f6,
    itbl_novex_map0_f7,
    itbl_novex_map0_f8,
    itbl_novex_map0_f9,
    itbl_novex_map0_fa,
    itbl_novex_map0_fb,
    itbl_novex_map0_fc,
    itbl_novex_map0_fd,
    itbl_novex_map0_fe,
    itbl_novex_map0_ff,
};

static const struct itemplate * const itbl_novex_map1_00[] = {
    instrux + 1187,
    instrux + 1188,
    instrux + 1189,
    instrux + 1190,
    instrux + 1191,
    instrux + 1421,
    instrux + 1422,
    instrux + 1424,
    instrux + 1431,
    instrux + 1432,
    instrux + 1433,
    instrux + 1434,
    instrux + 1435,
    instrux + 1436,
    instrux + 1437,
    instrux + 1438,
    instrux + 1445,
    instrux + 1446,
    instrux + 1447,
    instrux + 1448,
    instrux + 1449,
    instrux + 1450,
    instrux + 1451,
    instrux + 1452,
    NULL
};

static const struct itemplate * const itbl_novex_map1_01[] = {
    instrux + 1231,
    instrux + 1232,
    instrux + 1236,
    instrux + 1240,
    instrux + 1282,
    instrux + 1287,
    instrux + 1288,
    instrux + 1289,
    instrux + 1296,
    instrux + 1297,
    instrux + 1298,
    instrux + 1299,
    instrux + 1302,
    instrux + 1313,
    instrux + 1314,
    instrux + 1315,
    instrux + 1316,
    instrux + 1317,
    instrux + 1330,
    instrux + 1333,
    instrux + 1334,
    instrux + 1346,
    instrux + 1347,
    instrux + 1348,
    instrux + 1349,
    instrux + 1350,
    instrux + 1351,
    instrux + 1352,
    instrux + 1353,
    instrux + 1354,
    instrux + 1355,
    instrux + 1423,
    instrux + 1425,
    instrux + 1426,
    instrux + 1427,
    instrux + 1428,
    instrux + 1429,
    instrux + 1430,
    instrux + 1439,
    instrux + 1440,
    instrux + 1441,
    instrux + 1442,
    instrux + 1443,
    instrux + 1444,
    instrux + 1771,
    instrux + 2691,
    instrux + 2692,
    instrux + 2919,
    instrux + 2920,
    instrux + 2921,
    instrux + 2923,
    instrux + 2924,
    instrux + 2925,
    instrux + 2926,
    instrux + 2931,
    instrux + 2932,
    instrux + 2933,
    instrux + 2936,
    instrux + 2944,
    instrux + 2945,
    instrux + 2946,
    instrux + 2947,
    instrux + 4368,
    instrux + 4369,
    instrux + 6696,
    instrux + 6697,
    instrux + 6709,
    instrux + 6710,
    instrux + 6817,
    instrux + 6818,
    instrux + 6819,
    instrux + 6827,
    instrux + 6828,
    instrux + 6829,
    instrux + 6846,
    instrux + 6847,
    instrux + 6848,
    instrux + 7182,
    instrux + 7184,
    instrux + 7185,
    instrux + 7186,
    NULL
};

static const struct itemplate * const itbl_novex_map1_02[] = {
    instrux + 1415,
    instrux + 1416,
    instrux + 1417,
    NULL
};

static const struct itemplate * const itbl_novex_map1_03[] = {
    instrux + 1418,
    instrux + 1419,
    instrux + 1420,
    NULL
};

static const struct itemplate * const itbl_novex_map1_05[] = {
    instrux + 1223,
    NULL
};

static const struct itemplate * const itbl_novex_map1_06[] = {
    instrux + 1311,
    NULL
};

static const struct itemplate * const itbl_novex_map1_07[] = {
    instrux + 1226,
    NULL
};

static const struct itemplate * const itbl_novex_map1_08[] = {
    instrux + 1290,
    NULL
};

static const struct itemplate * const itbl_novex_map1_09[] = {
    instrux + 1291,
    instrux + 1292,
    NULL
};

static const struct itemplate * const itbl_novex_map1_0b[] = {
    instrux + 1780,
    NULL
};

static const struct itemplate * const itbl_novex_map1_0d[] = {
    instrux + 1698,
    instrux + 1699,
    instrux + 4370,
    NULL
};

static const struct itemplate * const itbl_novex_map1_0e[] = {
    instrux + 1517,
    NULL
};

static const struct itemplate * const itbl_novex_map1_0f[] = {
    instrux + 1660,
    instrux + 1668,
    instrux + 1669,
    instrux + 1670,
    instrux + 1671,
    instrux + 1672,
    instrux + 1673,
    instrux + 1674,
    instrux + 1675,
    instrux + 1676,
    instrux + 1677,
    instrux + 1678,
    instrux + 1679,
    instrux + 1680,
    instrux + 1681,
    instrux + 1682,
    instrux + 1683,
    instrux + 1684,
    instrux + 1689,
    instrux + 2728,
    instrux + 2729,
    instrux + 2730,
    instrux + 2731,
    instrux + 2732,
    instrux + 3073,
    instrux + 3074,
    NULL
};

static const struct itemplate * const itbl_novex_map1_10[] = {
    instrux + 2666,
    instrux + 2668,
    instrux + 2893,
    instrux + 2895,
    NULL
};

static const struct itemplate * const itbl_novex_map1_11[] = {
    instrux + 2667,
    instrux + 2669,
    instrux + 2894,
    instrux + 2896,
    NULL
};

static const struct itemplate * const itbl_novex_map1_12[] = {
    instrux + 2660,
    instrux + 2662,
    instrux + 2890,
    instrux + 2916,
    instrux + 2918,
    NULL
};

static const struct itemplate * const itbl_novex_map1_13[] = {
    instrux + 2661,
    instrux + 2889,
    NULL
};

static const struct itemplate * const itbl_novex_map1_14[] = {
    instrux + 2685,
    instrux + 2907,
    NULL
};

static const struct itemplate * const itbl_novex_map1_15[] = {
    instrux + 2684,
    instrux + 2906,
    NULL
};

static const struct itemplate * const itbl_novex_map1_16[] = {
    instrux + 2657,
    instrux + 2659,
    instrux + 2888,
    instrux + 2917,
    NULL
};

static const struct itemplate * const itbl_novex_map1_17[] = {
    instrux + 2658,
    instrux + 2887,
    NULL
};

static const struct itemplate * const itbl_novex_map1_18[] = {
    instrux + 2705,
    instrux + 2706,
    instrux + 2707,
    instrux + 2708,
    instrux + 2709,
    instrux + 2710,
    instrux + 6711,
    instrux + 7450,
    instrux + 7451,
    instrux + 7452,
    NULL
};

static const struct itemplate * const itbl_novex_map1_19[] = {
    instrux + 7450,
    instrux + 7451,
    instrux + 7452,
    NULL
};

static const struct itemplate * const itbl_novex_map1_1a[] = {
    instrux + 4372,
    instrux + 4373,
    instrux + 4374,
    instrux + 4375,
    instrux + 4376,
    instrux + 4377,
    instrux + 4381,
    instrux + 4382,
    instrux + 4385,
    instrux + 4386,
    instrux + 4387,
    instrux + 7450,
    instrux + 7451,
    instrux + 7452,
    NULL
};

static const struct itemplate * const itbl_novex_map1_1b[] = {
    instrux + 4371,
    instrux + 4378,
    instrux + 4379,
    instrux + 4380,
    instrux + 4383,
    instrux + 4384,
    instrux + 4388,
    instrux + 4389,
    instrux + 4390,
    instrux + 4391,
    instrux + 4392,
    instrux + 7450,
    instrux + 7451,
    instrux + 7452,
    NULL
};

static const struct itemplate * const itbl_novex_map1_1c[] = {
    instrux + 6700,
    instrux + 7450,
    instrux + 7451,
    instrux + 7452,
    NULL
};

static const struct itemplate * const itbl_novex_map1_1d[] = {
    instrux + 7450,
    instrux + 7451,
    instrux + 7452,
    NULL
};

static const struct itemplate * const itbl_novex_map1_1e[] = {
    instrux + 6821,
    instrux + 6822,
    instrux + 6825,
    instrux + 6826,
    instrux + 7450,
    instrux + 7451,
    instrux + 7452,
    NULL
};

static const struct itemplate * const itbl_novex_map1_1f[] = {
    instrux + 1,
    instrux + 2,
    instrux + 3,
    instrux + 7450,
    instrux + 7451,
    instrux + 7452,
    NULL
};

static const struct itemplate * const itbl_novex_map1_20[] = {
    instrux + 1318,
    instrux + 1320,
    NULL
};

static const struct itemplate * const itbl_novex_map1_21[] = {
    instrux + 1322,
    instrux + 1324,
    NULL
};

static const struct itemplate * const itbl_novex_map1_22[] = {
    instrux + 1319,
    instrux + 1321,
    NULL
};

static const struct itemplate * const itbl_novex_map1_23[] = {
    instrux + 1323,
    instrux + 1325,
    NULL
};

static const struct itemplate * const itbl_novex_map1_28[] = {
    instrux + 2655,
    instrux + 2885,
    NULL
};

static const struct itemplate * const itbl_novex_map1_29[] = {
    instrux + 2656,
    instrux + 2886,
    NULL
};

static const struct itemplate * const itbl_novex_map1_2a[] = {
    instrux + 2639,
    instrux + 2641,
    instrux + 2642,
    instrux + 2865,
    instrux + 2871,
    instrux + 2872,
    NULL
};

static const struct itemplate * const itbl_novex_map1_2b[] = {
    instrux + 2665,
    instrux + 2737,
    instrux + 2984,
    instrux + 2985,
    NULL
};

static const struct itemplate * const itbl_novex_map1_2c[] = {
    instrux + 2645,
    instrux + 2646,
    instrux + 2647,
    instrux + 2874,
    instrux + 2877,
    instrux + 2878,
    NULL
};

static const struct itemplate * const itbl_novex_map1_2d[] = {
    instrux + 2640,
    instrux + 2643,
    instrux + 2644,
    instrux + 2863,
    instrux + 2868,
    instrux + 2869,
    NULL
};

static const struct itemplate * const itbl_novex_map1_2e[] = {
    instrux + 2683,
    instrux + 2905,
    NULL
};

static const struct itemplate * const itbl_novex_map1_2f[] = {
    instrux + 2638,
    instrux + 2859,
    NULL
};

static const struct itemplate * const itbl_novex_map1_30[] = {
    instrux + 1326,
    NULL
};

static const struct itemplate * const itbl_novex_map1_31[] = {
    instrux + 1301,
    NULL
};

static const struct itemplate * const itbl_novex_map1_32[] = {
    instrux + 1327,
    NULL
};

static const struct itemplate * const itbl_novex_map1_33[] = {
    instrux + 1300,
    NULL
};

static const struct itemplate * const itbl_novex_map1_34[] = {
    instrux + 1224,
    NULL
};

static const struct itemplate * const itbl_novex_map1_35[] = {
    instrux + 1225,
    NULL
};

static const struct itemplate * const itbl_novex_map1_37[] = {
    instrux + 3072,
    NULL
};

static const struct itemplate * const itbl_novex_map1_38[] = {
    instrux + 1335,
    instrux + 1339,
    NULL
};

static const struct itemplate * const itbl_novex_map1_40[] = {
    instrux + 2227,
    instrux + 2243,
    instrux + 2259,
    NULL
};

static const struct itemplate * const itbl_novex_map1_41[] = {
    instrux + 2224,
    instrux + 2240,
    instrux + 2256,
    NULL
};

static const struct itemplate * const itbl_novex_map1_42[] = {
    instrux + 2217,
    instrux + 2233,
    instrux + 2249,
    NULL
};

static const struct itemplate * const itbl_novex_map1_43[] = {
    instrux + 2221,
    instrux + 2237,
    instrux + 2253,
    NULL
};

static const struct itemplate * const itbl_novex_map1_44[] = {
    instrux + 2231,
    instrux + 2247,
    instrux + 2263,
    NULL
};

static const struct itemplate * const itbl_novex_map1_45[] = {
    instrux + 2226,
    instrux + 2242,
    instrux + 2258,
    NULL
};

static const struct itemplate * const itbl_novex_map1_46[] = {
    instrux + 2220,
    instrux + 2236,
    instrux + 2252,
    NULL
};

static const struct itemplate * const itbl_novex_map1_47[] = {
    instrux + 2216,
    instrux + 2232,
    instrux + 2248,
    NULL
};

static const struct itemplate * const itbl_novex_map1_48[] = {
    instrux + 2230,
    instrux + 2246,
    instrux + 2262,
    NULL
};

static const struct itemplate * const itbl_novex_map1_49[] = {
    instrux + 2225,
    instrux + 2241,
    instrux + 2257,
    NULL
};

static const struct itemplate * const itbl_novex_map1_4a[] = {
    instrux + 2228,
    instrux + 2244,
    instrux + 2260,
    NULL
};

static const struct itemplate * const itbl_novex_map1_4b[] = {
    instrux + 2229,
    instrux + 2245,
    instrux + 2261,
    NULL
};

static const struct itemplate * const itbl_novex_map1_4c[] = {
    instrux + 2219,
    instrux + 2235,
    instrux + 2251,
    NULL
};

static const struct itemplate * const itbl_novex_map1_4d[] = {
    instrux + 2223,
    instrux + 2239,
    instrux + 2255,
    NULL
};

static const struct itemplate * const itbl_novex_map1_4e[] = {
    instrux + 2222,
    instrux + 2238,
    instrux + 2254,
    NULL
};

static const struct itemplate * const itbl_novex_map1_4f[] = {
    instrux + 2218,
    instrux + 2234,
    instrux + 2250,
    NULL
};

static const struct itemplate * const itbl_novex_map1_50[] = {
    instrux + 1659,
    instrux + 2663,
    instrux + 2664,
    instrux + 2891,
    instrux + 2892,
    NULL
};

static const struct itemplate * const itbl_novex_map1_51[] = {
    instrux + 1652,
    instrux + 2678,
    instrux + 2679,
    instrux + 2901,
    instrux + 2902,
    NULL
};

static const struct itemplate * const itbl_novex_map1_52[] = {
    instrux + 1687,
    instrux + 2675,
    instrux + 2676,
    NULL
};

static const struct itemplate * const itbl_novex_map1_53[] = {
    instrux + 2673,
    instrux + 2674,
    NULL
};

static const struct itemplate * const itbl_novex_map1_54[] = {
    instrux + 1667,
    instrux + 2619,
    instrux + 2840,
    NULL
};

static const struct itemplate * const itbl_novex_map1_55[] = {
    instrux + 1719,
    instrux + 2618,
    instrux + 2839,
    NULL
};

static const struct itemplate * const itbl_novex_map1_56[] = {
    instrux + 2672,
    instrux + 2899,
    NULL
};

static const struct itemplate * const itbl_novex_map1_57[] = {
    instrux + 2686,
    instrux + 2908,
    NULL
};

static const struct itemplate * const itbl_novex_map1_58[] = {
    instrux + 1696,
    instrux + 2616,
    instrux + 2617,
    instrux + 2837,
    instrux + 2838,
    NULL
};

static const struct itemplate * const itbl_novex_map1_59[] = {
    instrux + 1690,
    instrux + 2670,
    instrux + 2671,
    instrux + 2897,
    instrux + 2898,
    NULL
};

static const struct itemplate * const itbl_novex_map1_5a[] = {
    instrux + 1695,
    instrux + 2864,
    instrux + 2867,
    instrux + 2870,
    instrux + 2873,
    NULL
};

static const struct itemplate * const itbl_novex_map1_5b[] = {
    instrux + 1694,
    instrux + 2861,
    instrux + 2866,
    instrux + 2876,
    NULL
};

static const struct itemplate * const itbl_novex_map1_5c[] = {
    instrux + 1693,
    instrux + 2681,
    instrux + 2682,
    instrux + 2903,
    instrux + 2904,
    NULL
};

static const struct itemplate * const itbl_novex_map1_5d[] = {
    instrux + 1688,
    instrux + 2653,
    instrux + 2654,
    instrux + 2883,
    instrux + 2884,
    NULL
};

static const struct itemplate * const itbl_novex_map1_5e[] = {
    instrux + 1685,
    instrux + 2648,
    instrux + 2649,
    instrux + 2879,
    instrux + 2880,
    NULL
};

static const struct itemplate * const itbl_novex_map1_5f[] = {
    instrux + 2651,
    instrux + 2652,
    instrux + 2881,
    instrux + 2882,
    NULL
};

static const struct itemplate * const itbl_novex_map1_60[] = {
    instrux + 1727,
    instrux + 2832,
    NULL
};

static const struct itemplate * const itbl_novex_map1_61[] = {
    instrux + 1729,
    instrux + 2833,
    NULL
};

static const struct itemplate * const itbl_novex_map1_62[] = {
    instrux + 1728,
    instrux + 2834,
    NULL
};

static const struct itemplate * const itbl_novex_map1_63[] = {
    instrux + 1647,
    instrux + 2756,
    NULL
};

static const struct itemplate * const itbl_novex_map1_64[] = {
    instrux + 1664,
    instrux + 2775,
    NULL
};

static const struct itemplate * const itbl_novex_map1_65[] = {
    instrux + 1666,
    instrux + 2776,
    NULL
};

static const struct itemplate * const itbl_novex_map1_66[] = {
    instrux + 1665,
    instrux + 2777,
    NULL
};

static const struct itemplate * const itbl_novex_map1_67[] = {
    instrux + 1648,
    instrux + 2758,
    NULL
};

static const struct itemplate * const itbl_novex_map1_68[] = {
    instrux + 1724,
    instrux + 2828,
    NULL
};

static const struct itemplate * const itbl_novex_map1_69[] = {
    instrux + 1726,
    instrux + 2829,
    NULL
};

static const struct itemplate * const itbl_novex_map1_6a[] = {
    instrux + 1725,
    instrux + 2830,
    NULL
};

static const struct itemplate * const itbl_novex_map1_6b[] = {
    instrux + 1646,
    instrux + 2757,
    NULL
};

static const struct itemplate * const itbl_novex_map1_6c[] = {
    instrux + 2835,
    NULL
};

static const struct itemplate * const itbl_novex_map1_6d[] = {
    instrux + 2831,
    NULL
};

static const struct itemplate * const itbl_novex_map1_6e[] = {
    instrux + 1640,
    instrux + 1644,
    instrux + 2741,
    instrux + 2742,
    instrux + 2753,
    NULL
};

static const struct itemplate * const itbl_novex_map1_6f[] = {
    instrux + 1642,
    instrux + 2744,
    instrux + 2746,
    NULL
};

static const struct itemplate * const itbl_novex_map1_70[] = {
    instrux + 2727,
    instrux + 2795,
    instrux + 2796,
    instrux + 2797,
    instrux + 2798,
    instrux + 2799,
    instrux + 2800,
    NULL
};

static const struct itemplate * const itbl_novex_map1_71[] = {
    instrux + 1705,
    instrux + 1709,
    instrux + 1715,
    instrux + 2803,
    instrux + 2809,
    instrux + 2814,
    NULL
};

static const struct itemplate * const itbl_novex_map1_72[] = {
    instrux + 1701,
    instrux + 1707,
    instrux + 1711,
    instrux + 2805,
    instrux + 2811,
    instrux + 2816,
    NULL
};

static const struct itemplate * const itbl_novex_map1_73[] = {
    instrux + 1703,
    instrux + 1713,
    instrux + 2801,
    instrux + 2807,
    instrux + 2812,
    instrux + 2818,
    NULL
};

static const struct itemplate * const itbl_novex_map1_74[] = {
    instrux + 1661,
    instrux + 2772,
    NULL
};

static const struct itemplate * const itbl_novex_map1_75[] = {
    instrux + 1663,
    instrux + 2773,
    NULL
};

static const struct itemplate * const itbl_novex_map1_76[] = {
    instrux + 1662,
    instrux + 2774,
    NULL
};

static const struct itemplate * const itbl_novex_map1_77[] = {
    instrux + 1639,
    NULL
};

static const struct itemplate * const itbl_novex_map1_78[] = {
    instrux + 2929,
    instrux + 2930,
    instrux + 2980,
    instrux + 2982,
    NULL
};

static const struct itemplate * const itbl_novex_map1_79[] = {
    instrux + 2934,
    instrux + 2935,
    instrux + 2981,
    instrux + 2983,
    NULL
};

static const struct itemplate * const itbl_novex_map1_7c[] = {
    instrux + 2911,
    instrux + 2912,
    NULL
};

static const struct itemplate * const itbl_novex_map1_7d[] = {
    instrux + 2913,
    instrux + 2914,
    NULL
};

static const struct itemplate * const itbl_novex_map1_7e[] = {
    instrux + 1641,
    instrux + 1645,
    instrux + 2740,
    instrux + 2743,
    instrux + 2749,
    instrux + 2752,
    instrux + 2754,
    NULL
};

static const struct itemplate * const itbl_novex_map1_7f[] = {
    instrux + 1643,
    instrux + 2745,
    instrux + 2747,
    NULL
};

static const struct itemplate * const itbl_novex_map1_80[] = {
    instrux + 1102,
    instrux + 1118,
    instrux + 1134,
    NULL
};

static const struct itemplate * const itbl_novex_map1_81[] = {
    instrux + 1099,
    instrux + 1115,
    instrux + 1131,
    NULL
};

static const struct itemplate * const itbl_novex_map1_82[] = {
    instrux + 1092,
    instrux + 1108,
    instrux + 1124,
    NULL
};

static const struct itemplate * const itbl_novex_map1_83[] = {
    instrux + 1096,
    instrux + 1112,
    instrux + 1128,
    NULL
};

static const struct itemplate * const itbl_novex_map1_84[] = {
    instrux + 1106,
    instrux + 1122,
    instrux + 1138,
    NULL
};

static const struct itemplate * const itbl_novex_map1_85[] = {
    instrux + 1101,
    instrux + 1117,
    instrux + 1133,
    NULL
};

static const struct itemplate * const itbl_novex_map1_86[] = {
    instrux + 1095,
    instrux + 1111,
    instrux + 1127,
    NULL
};

static const struct itemplate * const itbl_novex_map1_87[] = {
    instrux + 1091,
    instrux + 1107,
    instrux + 1123,
    NULL
};

static const struct itemplate * const itbl_novex_map1_88[] = {
    instrux + 1105,
    instrux + 1121,
    instrux + 1137,
    NULL
};

static const struct itemplate * const itbl_novex_map1_89[] = {
    instrux + 1100,
    instrux + 1116,
    instrux + 1132,
    NULL
};

static const struct itemplate * const itbl_novex_map1_8a[] = {
    instrux + 1103,
    instrux + 1119,
    instrux + 1135,
    NULL
};

static const struct itemplate * const itbl_novex_map1_8b[] = {
    instrux + 1104,
    instrux + 1120,
    instrux + 1136,
    NULL
};

static const struct itemplate * const itbl_novex_map1_8c[] = {
    instrux + 1094,
    instrux + 1110,
    instrux + 1126,
    NULL
};

static const struct itemplate * const itbl_novex_map1_8d[] = {
    instrux + 1098,
    instrux + 1114,
    instrux + 1130,
    NULL
};

static const struct itemplate * const itbl_novex_map1_8e[] = {
    instrux + 1097,
    instrux + 1113,
    instrux + 1129,
    NULL
};

static const struct itemplate * const itbl_novex_map1_8f[] = {
    instrux + 1093,
    instrux + 1109,
    instrux + 1125,
    NULL
};

static const struct itemplate * const itbl_novex_map1_90[] = {
    instrux + 2515,
    NULL
};

static const struct itemplate * const itbl_novex_map1_91[] = {
    instrux + 2512,
    NULL
};

static const struct itemplate * const itbl_novex_map1_92[] = {
    instrux + 2505,
    NULL
};

static const struct itemplate * const itbl_novex_map1_93[] = {
    instrux + 2509,
    NULL
};

static const struct itemplate * const itbl_novex_map1_94[] = {
    instrux + 2519,
    NULL
};

static const struct itemplate * const itbl_novex_map1_95[] = {
    instrux + 2514,
    NULL
};

static const struct itemplate * const itbl_novex_map1_96[] = {
    instrux + 2508,
    NULL
};

static const struct itemplate * const itbl_novex_map1_97[] = {
    instrux + 2504,
    NULL
};

static const struct itemplate * const itbl_novex_map1_98[] = {
    instrux + 2518,
    NULL
};

static const struct itemplate * const itbl_novex_map1_99[] = {
    instrux + 2513,
    NULL
};

static const struct itemplate * const itbl_novex_map1_9a[] = {
    instrux + 2516,
    NULL
};

static const struct itemplate * const itbl_novex_map1_9b[] = {
    instrux + 2517,
    NULL
};

static const struct itemplate * const itbl_novex_map1_9c[] = {
    instrux + 2507,
    NULL
};

static const struct itemplate * const itbl_novex_map1_9d[] = {
    instrux + 2511,
    NULL
};

static const struct itemplate * const itbl_novex_map1_9e[] = {
    instrux + 2510,
    NULL
};

static const struct itemplate * const itbl_novex_map1_9f[] = {
    instrux + 2506,
    NULL
};

static const struct itemplate * const itbl_novex_map1_a0[] = {
    instrux + 1398,
    NULL
};

static const struct itemplate * const itbl_novex_map1_a1[] = {
    instrux + 1403,
    NULL
};

static const struct itemplate * const itbl_novex_map1_a2[] = {
    instrux + 1312,
    NULL
};

static const struct itemplate * const itbl_novex_map1_a3[] = {
    instrux + 874,
    instrux + 875,
    instrux + 876,
    NULL
};

static const struct itemplate * const itbl_novex_map1_a4[] = {
    instrux + 838,
    instrux + 839,
    instrux + 840,
    NULL
};

static const struct itemplate * const itbl_novex_map1_a5[] = {
    instrux + 841,
    instrux + 842,
    instrux + 843,
    NULL
};

static const struct itemplate * const itbl_novex_map1_a6[] = {
    instrux + 4028,
    instrux + 4029,
    instrux + 4030,
    NULL
};

static const struct itemplate * const itbl_novex_map1_a7[] = {
    instrux + 4022,
    instrux + 4023,
    instrux + 4024,
    instrux + 4025,
    instrux + 4026,
    instrux + 4027,
    NULL
};

static const struct itemplate * const itbl_novex_map1_a8[] = {
    instrux + 1399,
    NULL
};

static const struct itemplate * const itbl_novex_map1_a9[] = {
    instrux + 1404,
    NULL
};

static const struct itemplate * const itbl_novex_map1_aa[] = {
    instrux + 1343,
    NULL
};

static const struct itemplate * const itbl_novex_map1_ab[] = {
    instrux + 892,
    instrux + 893,
    instrux + 894,
    NULL
};

static const struct itemplate * const itbl_novex_map1_ac[] = {
    instrux + 856,
    instrux + 857,
    instrux + 858,
    NULL
};

static const struct itemplate * const itbl_novex_map1_ad[] = {
    instrux + 859,
    instrux + 860,
    instrux + 861,
    NULL
};

static const struct itemplate * const itbl_novex_map1_ae[] = {
    instrux + 1279,
    instrux + 1280,
    instrux + 1281,
    instrux + 1283,
    instrux + 1284,
    instrux + 1285,
    instrux + 1286,
    instrux + 1356,
    instrux + 1357,
    instrux + 1358,
    instrux + 1359,
    instrux + 1360,
    instrux + 1405,
    instrux + 1406,
    instrux + 1407,
    instrux + 1408,
    instrux + 1409,
    instrux + 1410,
    instrux + 1411,
    instrux + 1412,
    instrux + 2650,
    instrux + 2680,
    instrux + 2687,
    instrux + 2688,
    instrux + 2689,
    instrux + 2690,
    instrux + 2693,
    instrux + 2694,
    instrux + 2697,
    instrux + 2698,
    instrux + 2701,
    instrux + 2702,
    instrux + 2711,
    instrux + 2738,
    instrux + 2739,
    instrux + 6698,
    instrux + 6699,
    instrux + 6820,
    instrux + 6823,
    instrux + 6824,
    NULL
};

static const struct itemplate * const itbl_novex_map1_af[] = {
    instrux + 753,
    instrux + 754,
    instrux + 755,
    NULL
};

static const struct itemplate * const itbl_novex_map1_b0[] = {
    instrux + 1011,
    NULL
};

static const struct itemplate * const itbl_novex_map1_b1[] = {
    instrux + 1012,
    instrux + 1013,
    instrux + 1014,
    NULL
};

static const struct itemplate * const itbl_novex_map1_b2[] = {
    instrux + 1391,
    instrux + 1392,
    instrux + 1393,
    NULL
};

static const struct itemplate * const itbl_novex_map1_b3[] = {
    instrux + 886,
    instrux + 887,
    instrux + 888,
    NULL
};

static const struct itemplate * const itbl_novex_map1_b4[] = {
    instrux + 1385,
    instrux + 1386,
    instrux + 1387,
    NULL
};

static const struct itemplate * const itbl_novex_map1_b5[] = {
    instrux + 1388,
    instrux + 1389,
    instrux + 1390,
    NULL
};

static const struct itemplate * const itbl_novex_map1_b6[] = {
    instrux + 1005,
    instrux + 1006,
    instrux + 1007,
    NULL
};

static const struct itemplate * const itbl_novex_map1_b7[] = {
    instrux + 1008,
    instrux + 1009,
    instrux + 1010,
    NULL
};

static const struct itemplate * const itbl_novex_map1_b8[] = {
    instrux + 1184,
    instrux + 1185,
    instrux + 1186,
    instrux + 3066,
    instrux + 3067,
    instrux + 3068,
    NULL
};

static const struct itemplate * const itbl_novex_map1_b9[] = {
    instrux + 1776,
    instrux + 1777,
    instrux + 1778,
    instrux + 1779,
    NULL
};

static const struct itemplate * const itbl_novex_map1_ba[] = {
    instrux + 877,
    instrux + 878,
    instrux + 879,
    instrux + 883,
    instrux + 884,
    instrux + 885,
    instrux + 889,
    instrux + 890,
    instrux + 891,
    instrux + 895,
    instrux + 896,
    instrux + 897,
    NULL
};

static const struct itemplate * const itbl_novex_map1_bb[] = {
    instrux + 880,
    instrux + 881,
    instrux + 882,
    NULL
};

static const struct itemplate * const itbl_novex_map1_bc[] = {
    instrux + 898,
    instrux + 899,
    instrux + 900,
    instrux + 910,
    instrux + 911,
    instrux + 912,
    NULL
};

static const struct itemplate * const itbl_novex_map1_bd[] = {
    instrux + 901,
    instrux + 902,
    instrux + 903,
    instrux + 904,
    instrux + 905,
    instrux + 906,
    NULL
};

static const struct itemplate * const itbl_novex_map1_be[] = {
    instrux + 996,
    instrux + 997,
    instrux + 998,
    NULL
};

static const struct itemplate * const itbl_novex_map1_bf[] = {
    instrux + 999,
    instrux + 1000,
    instrux + 1001,
    NULL
};

static const struct itemplate * const itbl_novex_map1_c0[] = {
    instrux + 1017,
    NULL
};

static const struct itemplate * const itbl_novex_map1_c1[] = {
    instrux + 1018,
    instrux + 1019,
    instrux + 1020,
    NULL
};

static const struct itemplate * const itbl_novex_map1_c2[] = {
    instrux + 2620,
    instrux + 2621,
    instrux + 2622,
    instrux + 2623,
    instrux + 2624,
    instrux + 2625,
    instrux + 2626,
    instrux + 2627,
    instrux + 2628,
    instrux + 2629,
    instrux + 2630,
    instrux + 2631,
    instrux + 2632,
    instrux + 2633,
    instrux + 2634,
    instrux + 2635,
    instrux + 2636,
    instrux + 2637,
    instrux + 2841,
    instrux + 2842,
    instrux + 2843,
    instrux + 2844,
    instrux + 2845,
    instrux + 2846,
    instrux + 2847,
    instrux + 2848,
    instrux + 2849,
    instrux + 2850,
    instrux + 2851,
    instrux + 2852,
    instrux + 2853,
    instrux + 2854,
    instrux + 2855,
    instrux + 2856,
    instrux + 2857,
    instrux + 2858,
    NULL
};

static const struct itemplate * const itbl_novex_map1_c3[] = {
    instrux + 2735,
    instrux + 2736,
    NULL
};

static const struct itemplate * const itbl_novex_map1_c4[] = {
    instrux + 2717,
    instrux + 2718,
    instrux + 2719,
    instrux + 2779,
    instrux + 2780,
    instrux + 2781,
    NULL
};

static const struct itemplate * const itbl_novex_map1_c5[] = {
    instrux + 2716,
    instrux + 2778,
    NULL
};

static const struct itemplate * const itbl_novex_map1_c6[] = {
    instrux + 2677,
    instrux + 2900,
    NULL
};

static const struct itemplate * const itbl_novex_map1_c7[] = {
    instrux + 1015,
    instrux + 1016,
    instrux + 1303,
    instrux + 1304,
    instrux + 1305,
    instrux + 1306,
    instrux + 1307,
    instrux + 1308,
    instrux + 1309,
    instrux + 1310,
    instrux + 2695,
    instrux + 2696,
    instrux + 2699,
    instrux + 2700,
    instrux + 2703,
    instrux + 2704,
    instrux + 2922,
    instrux + 2927,
    instrux + 2928,
    instrux + 2937,
    instrux + 7183,
    NULL
};

static const struct itemplate * const itbl_novex_map1_c8[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_novex_map1_c9[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_novex_map1_ca[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_novex_map1_cb[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_novex_map1_cc[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_novex_map1_cd[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_novex_map1_ce[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_novex_map1_cf[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_novex_map1_d0[] = {
    instrux + 2909,
    instrux + 2910,
    NULL
};

static const struct itemplate * const itbl_novex_map1_d1[] = {
    instrux + 1714,
    instrux + 2813,
    NULL
};

static const struct itemplate * const itbl_novex_map1_d2[] = {
    instrux + 1710,
    instrux + 2815,
    NULL
};

static const struct itemplate * const itbl_novex_map1_d3[] = {
    instrux + 1712,
    instrux + 2817,
    NULL
};

static const struct itemplate * const itbl_novex_map1_d4[] = {
    instrux + 2762,
    instrux + 2763,
    NULL
};

static const struct itemplate * const itbl_novex_map1_d5[] = {
    instrux + 1692,
    instrux + 2790,
    NULL
};

static const struct itemplate * const itbl_novex_map1_d6[] = {
    instrux + 2748,
    instrux + 2750,
    instrux + 2751,
    instrux + 2755,
    NULL
};

static const struct itemplate * const itbl_novex_map1_d7[] = {
    instrux + 2724,
    instrux + 2787,
    NULL
};

static const struct itemplate * const itbl_novex_map1_d8[] = {
    instrux + 1721,
    instrux + 2826,
    NULL
};

static const struct itemplate * const itbl_novex_map1_d9[] = {
    instrux + 1722,
    instrux + 2827,
    NULL
};

static const struct itemplate * const itbl_novex_map1_da[] = {
    instrux + 2723,
    instrux + 2786,
    NULL
};

static const struct itemplate * const itbl_novex_map1_db[] = {
    instrux + 1657,
    instrux + 2768,
    NULL
};

static const struct itemplate * const itbl_novex_map1_dc[] = {
    instrux + 1654,
    instrux + 2766,
    NULL
};

static const struct itemplate * const itbl_novex_map1_dd[] = {
    instrux + 1655,
    instrux + 2767,
    NULL
};

static const struct itemplate * const itbl_novex_map1_de[] = {
    instrux + 2721,
    instrux + 2784,
    NULL
};

static const struct itemplate * const itbl_novex_map1_df[] = {
    instrux + 1658,
    instrux + 2769,
    NULL
};

static const struct itemplate * const itbl_novex_map1_e0[] = {
    instrux + 2714,
    instrux + 2770,
    NULL
};

static const struct itemplate * const itbl_novex_map1_e1[] = {
    instrux + 1708,
    instrux + 2808,
    NULL
};

static const struct itemplate * const itbl_novex_map1_e2[] = {
    instrux + 1706,
    instrux + 2810,
    NULL
};

static const struct itemplate * const itbl_novex_map1_e3[] = {
    instrux + 2715,
    instrux + 2771,
    NULL
};

static const struct itemplate * const itbl_novex_map1_e4[] = {
    instrux + 2725,
    instrux + 2788,
    NULL
};

static const struct itemplate * const itbl_novex_map1_e5[] = {
    instrux + 1691,
    instrux + 2789,
    NULL
};

static const struct itemplate * const itbl_novex_map1_e6[] = {
    instrux + 2860,
    instrux + 2862,
    instrux + 2875,
    NULL
};

static const struct itemplate * const itbl_novex_map1_e7[] = {
    instrux + 2713,
    instrux + 2734,
    NULL
};

static const struct itemplate * const itbl_novex_map1_e8[] = {
    instrux + 1718,
    instrux + 2824,
    NULL
};

static const struct itemplate * const itbl_novex_map1_e9[] = {
    instrux + 1720,
    instrux + 2825,
    NULL
};

static const struct itemplate * const itbl_novex_map1_ea[] = {
    instrux + 2722,
    instrux + 2785,
    NULL
};

static const struct itemplate * const itbl_novex_map1_eb[] = {
    instrux + 1697,
    instrux + 2793,
    NULL
};

static const struct itemplate * const itbl_novex_map1_ec[] = {
    instrux + 1651,
    instrux + 2764,
    NULL
};

static const struct itemplate * const itbl_novex_map1_ed[] = {
    instrux + 1653,
    instrux + 2765,
    NULL
};

static const struct itemplate * const itbl_novex_map1_ee[] = {
    instrux + 2720,
    instrux + 2783,
    NULL
};

static const struct itemplate * const itbl_novex_map1_ef[] = {
    instrux + 1770,
    instrux + 2836,
    NULL
};

static const struct itemplate * const itbl_novex_map1_f0[] = {
    instrux + 2915,
    NULL
};

static const struct itemplate * const itbl_novex_map1_f1[] = {
    instrux + 1704,
    instrux + 2802,
    NULL
};

static const struct itemplate * const itbl_novex_map1_f2[] = {
    instrux + 1700,
    instrux + 2804,
    NULL
};

static const struct itemplate * const itbl_novex_map1_f3[] = {
    instrux + 1702,
    instrux + 2806,
    NULL
};

static const struct itemplate * const itbl_novex_map1_f4[] = {
    instrux + 2791,
    instrux + 2792,
    NULL
};

static const struct itemplate * const itbl_novex_map1_f5[] = {
    instrux + 1686,
    instrux + 2782,
    NULL
};

static const struct itemplate * const itbl_novex_map1_f6[] = {
    instrux + 2726,
    instrux + 2794,
    NULL
};

static const struct itemplate * const itbl_novex_map1_f7[] = {
    instrux + 2712,
    instrux + 2733,
    NULL
};

static const struct itemplate * const itbl_novex_map1_f8[] = {
    instrux + 1716,
    instrux + 2819,
    NULL
};

static const struct itemplate * const itbl_novex_map1_f9[] = {
    instrux + 1723,
    instrux + 2820,
    NULL
};

static const struct itemplate * const itbl_novex_map1_fa[] = {
    instrux + 1717,
    instrux + 2821,
    NULL
};

static const struct itemplate * const itbl_novex_map1_fb[] = {
    instrux + 2822,
    instrux + 2823,
    NULL
};

static const struct itemplate * const itbl_novex_map1_fc[] = {
    instrux + 1649,
    instrux + 2759,
    NULL
};

static const struct itemplate * const itbl_novex_map1_fd[] = {
    instrux + 1656,
    instrux + 2760,
    NULL
};

static const struct itemplate * const itbl_novex_map1_fe[] = {
    instrux + 1650,
    instrux + 2761,
    NULL
};

static const struct itemplate * const itbl_novex_map1_ff[] = {
    instrux + 1772,
    instrux + 1773,
    instrux + 1774,
    instrux + 1775,
    NULL
};

static const struct itemplate * const * const itbl_novex_map1[256] = {
    itbl_novex_map1_00,
    itbl_novex_map1_01,
    itbl_novex_map1_02,
    itbl_novex_map1_03,
    NULL,
    itbl_novex_map1_05,
    itbl_novex_map1_06,
    itbl_novex_map1_07,
    itbl_novex_map1_08,
    itbl_novex_map1_09,
    NULL,
    itbl_novex_map1_0b,
    NULL,
    itbl_novex_map1_0d,
    itbl_novex_map1_0e,
    itbl_novex_map1_0f,
    itbl_novex_map1_10,
    itbl_novex_map1_11,
    itbl_novex_map1_12,
    itbl_novex_map1_13,
    itbl_novex_map1_14,
    itbl_novex_map1_15,
    itbl_novex_map1_16,
    itbl_novex_map1_17,
    itbl_novex_map1_18,
    itbl_novex_map1_19,
    itbl_novex_map1_1a,
    itbl_novex_map1_1b,
    itbl_novex_map1_1c,
    itbl_novex_map1_1d,
    itbl_novex_map1_1e,
    itbl_novex_map1_1f,
    itbl_novex_map1_20,
    itbl_novex_map1_21,
    itbl_novex_map1_22,
    itbl_novex_map1_23,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map1_28,
    itbl_novex_map1_29,
    itbl_novex_map1_2a,
    itbl_novex_map1_2b,
    itbl_novex_map1_2c,
    itbl_novex_map1_2d,
    itbl_novex_map1_2e,
    itbl_novex_map1_2f,
    itbl_novex_map1_30,
    itbl_novex_map1_31,
    itbl_novex_map1_32,
    itbl_novex_map1_33,
    itbl_novex_map1_34,
    itbl_novex_map1_35,
    NULL,
    itbl_novex_map1_37,
    itbl_novex_map1_38,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map1_40,
    itbl_novex_map1_41,
    itbl_novex_map1_42,
    itbl_novex_map1_43,
    itbl_novex_map1_44,
    itbl_novex_map1_45,
    itbl_novex_map1_46,
    itbl_novex_map1_47,
    itbl_novex_map1_48,
    itbl_novex_map1_49,
    itbl_novex_map1_4a,
    itbl_novex_map1_4b,
    itbl_novex_map1_4c,
    itbl_novex_map1_4d,
    itbl_novex_map1_4e,
    itbl_novex_map1_4f,
    itbl_novex_map1_50,
    itbl_novex_map1_51,
    itbl_novex_map1_52,
    itbl_novex_map1_53,
    itbl_novex_map1_54,
    itbl_novex_map1_55,
    itbl_novex_map1_56,
    itbl_novex_map1_57,
    itbl_novex_map1_58,
    itbl_novex_map1_59,
    itbl_novex_map1_5a,
    itbl_novex_map1_5b,
    itbl_novex_map1_5c,
    itbl_novex_map1_5d,
    itbl_novex_map1_5e,
    itbl_novex_map1_5f,
    itbl_novex_map1_60,
    itbl_novex_map1_61,
    itbl_novex_map1_62,
    itbl_novex_map1_63,
    itbl_novex_map1_64,
    itbl_novex_map1_65,
    itbl_novex_map1_66,
    itbl_novex_map1_67,
    itbl_novex_map1_68,
    itbl_novex_map1_69,
    itbl_novex_map1_6a,
    itbl_novex_map1_6b,
    itbl_novex_map1_6c,
    itbl_novex_map1_6d,
    itbl_novex_map1_6e,
    itbl_novex_map1_6f,
    itbl_novex_map1_70,
    itbl_novex_map1_71,
    itbl_novex_map1_72,
    itbl_novex_map1_73,
    itbl_novex_map1_74,
    itbl_novex_map1_75,
    itbl_novex_map1_76,
    itbl_novex_map1_77,
    itbl_novex_map1_78,
    itbl_novex_map1_79,
    NULL,
    NULL,
    itbl_novex_map1_7c,
    itbl_novex_map1_7d,
    itbl_novex_map1_7e,
    itbl_novex_map1_7f,
    itbl_novex_map1_80,
    itbl_novex_map1_81,
    itbl_novex_map1_82,
    itbl_novex_map1_83,
    itbl_novex_map1_84,
    itbl_novex_map1_85,
    itbl_novex_map1_86,
    itbl_novex_map1_87,
    itbl_novex_map1_88,
    itbl_novex_map1_89,
    itbl_novex_map1_8a,
    itbl_novex_map1_8b,
    itbl_novex_map1_8c,
    itbl_novex_map1_8d,
    itbl_novex_map1_8e,
    itbl_novex_map1_8f,
    itbl_novex_map1_90,
    itbl_novex_map1_91,
    itbl_novex_map1_92,
    itbl_novex_map1_93,
    itbl_novex_map1_94,
    itbl_novex_map1_95,
    itbl_novex_map1_96,
    itbl_novex_map1_97,
    itbl_novex_map1_98,
    itbl_novex_map1_99,
    itbl_novex_map1_9a,
    itbl_novex_map1_9b,
    itbl_novex_map1_9c,
    itbl_novex_map1_9d,
    itbl_novex_map1_9e,
    itbl_novex_map1_9f,
    itbl_novex_map1_a0,
    itbl_novex_map1_a1,
    itbl_novex_map1_a2,
    itbl_novex_map1_a3,
    itbl_novex_map1_a4,
    itbl_novex_map1_a5,
    itbl_novex_map1_a6,
    itbl_novex_map1_a7,
    itbl_novex_map1_a8,
    itbl_novex_map1_a9,
    itbl_novex_map1_aa,
    itbl_novex_map1_ab,
    itbl_novex_map1_ac,
    itbl_novex_map1_ad,
    itbl_novex_map1_ae,
    itbl_novex_map1_af,
    itbl_novex_map1_b0,
    itbl_novex_map1_b1,
    itbl_novex_map1_b2,
    itbl_novex_map1_b3,
    itbl_novex_map1_b4,
    itbl_novex_map1_b5,
    itbl_novex_map1_b6,
    itbl_novex_map1_b7,
    itbl_novex_map1_b8,
    itbl_novex_map1_b9,
    itbl_novex_map1_ba,
    itbl_novex_map1_bb,
    itbl_novex_map1_bc,
    itbl_novex_map1_bd,
    itbl_novex_map1_be,
    itbl_novex_map1_bf,
    itbl_novex_map1_c0,
    itbl_novex_map1_c1,
    itbl_novex_map1_c2,
    itbl_novex_map1_c3,
    itbl_novex_map1_c4,
    itbl_novex_map1_c5,
    itbl_novex_map1_c6,
    itbl_novex_map1_c7,
    itbl_novex_map1_c8,
    itbl_novex_map1_c9,
    itbl_novex_map1_ca,
    itbl_novex_map1_cb,
    itbl_novex_map1_cc,
    itbl_novex_map1_cd,
    itbl_novex_map1_ce,
    itbl_novex_map1_cf,
    itbl_novex_map1_d0,
    itbl_novex_map1_d1,
    itbl_novex_map1_d2,
    itbl_novex_map1_d3,
    itbl_novex_map1_d4,
    itbl_novex_map1_d5,
    itbl_novex_map1_d6,
    itbl_novex_map1_d7,
    itbl_novex_map1_d8,
    itbl_novex_map1_d9,
    itbl_novex_map1_da,
    itbl_novex_map1_db,
    itbl_novex_map1_dc,
    itbl_novex_map1_dd,
    itbl_novex_map1_de,
    itbl_novex_map1_df,
    itbl_novex_map1_e0,
    itbl_novex_map1_e1,
    itbl_novex_map1_e2,
    itbl_novex_map1_e3,
    itbl_novex_map1_e4,
    itbl_novex_map1_e5,
    itbl_novex_map1_e6,
    itbl_novex_map1_e7,
    itbl_novex_map1_e8,
    itbl_novex_map1_e9,
    itbl_novex_map1_ea,
    itbl_novex_map1_eb,
    itbl_novex_map1_ec,
    itbl_novex_map1_ed,
    itbl_novex_map1_ee,
    itbl_novex_map1_ef,
    itbl_novex_map1_f0,
    itbl_novex_map1_f1,
    itbl_novex_map1_f2,
    itbl_novex_map1_f3,
    itbl_novex_map1_f4,
    itbl_novex_map1_f5,
    itbl_novex_map1_f6,
    itbl_novex_map1_f7,
    itbl_novex_map1_f8,
    itbl_novex_map1_f9,
    itbl_novex_map1_fa,
    itbl_novex_map1_fb,
    itbl_novex_map1_fc,
    itbl_novex_map1_fd,
    itbl_novex_map1_fe,
    itbl_novex_map1_ff,
};

static const struct itemplate * const itbl_novex_map2_00[] = {
    instrux + 2972,
    instrux + 2973,
    NULL
};

static const struct itemplate * const itbl_novex_map2_01[] = {
    instrux + 2956,
    instrux + 2957,
    NULL
};

static const struct itemplate * const itbl_novex_map2_02[] = {
    instrux + 2958,
    instrux + 2959,
    NULL
};

static const struct itemplate * const itbl_novex_map2_03[] = {
    instrux + 2960,
    instrux + 2961,
    NULL
};

static const struct itemplate * const itbl_novex_map2_04[] = {
    instrux + 2968,
    instrux + 2969,
    NULL
};

static const struct itemplate * const itbl_novex_map2_05[] = {
    instrux + 2962,
    instrux + 2963,
    NULL
};

static const struct itemplate * const itbl_novex_map2_06[] = {
    instrux + 2964,
    instrux + 2965,
    NULL
};

static const struct itemplate * const itbl_novex_map2_07[] = {
    instrux + 2966,
    instrux + 2967,
    NULL
};

static const struct itemplate * const itbl_novex_map2_08[] = {
    instrux + 2974,
    instrux + 2975,
    NULL
};

static const struct itemplate * const itbl_novex_map2_09[] = {
    instrux + 2976,
    instrux + 2977,
    NULL
};

static const struct itemplate * const itbl_novex_map2_0a[] = {
    instrux + 2978,
    instrux + 2979,
    NULL
};

static const struct itemplate * const itbl_novex_map2_0b[] = {
    instrux + 2970,
    instrux + 2971,
    NULL
};

static const struct itemplate * const itbl_novex_map2_10[] = {
    instrux + 3000,
    instrux + 3001,
    NULL
};

static const struct itemplate * const itbl_novex_map2_14[] = {
    instrux + 2990,
    instrux + 2991,
    NULL
};

static const struct itemplate * const itbl_novex_map2_15[] = {
    instrux + 2988,
    instrux + 2989,
    NULL
};

static const struct itemplate * const itbl_novex_map2_17[] = {
    instrux + 3040,
    NULL
};

static const struct itemplate * const itbl_novex_map2_1c[] = {
    instrux + 2948,
    instrux + 2949,
    NULL
};

static const struct itemplate * const itbl_novex_map2_1d[] = {
    instrux + 2950,
    instrux + 2951,
    NULL
};

static const struct itemplate * const itbl_novex_map2_1e[] = {
    instrux + 2952,
    instrux + 2953,
    NULL
};

static const struct itemplate * const itbl_novex_map2_20[] = {
    instrux + 3026,
    NULL
};

static const struct itemplate * const itbl_novex_map2_21[] = {
    instrux + 3027,
    NULL
};

static const struct itemplate * const itbl_novex_map2_22[] = {
    instrux + 3028,
    NULL
};

static const struct itemplate * const itbl_novex_map2_23[] = {
    instrux + 3029,
    NULL
};

static const struct itemplate * const itbl_novex_map2_24[] = {
    instrux + 3030,
    NULL
};

static const struct itemplate * const itbl_novex_map2_25[] = {
    instrux + 3031,
    NULL
};

static const struct itemplate * const itbl_novex_map2_28[] = {
    instrux + 3038,
    NULL
};

static const struct itemplate * const itbl_novex_map2_29[] = {
    instrux + 3003,
    NULL
};

static const struct itemplate * const itbl_novex_map2_2a[] = {
    instrux + 2997,
    NULL
};

static const struct itemplate * const itbl_novex_map2_2b[] = {
    instrux + 2999,
    NULL
};

static const struct itemplate * const itbl_novex_map2_30[] = {
    instrux + 3032,
    NULL
};

static const struct itemplate * const itbl_novex_map2_31[] = {
    instrux + 3033,
    NULL
};

static const struct itemplate * const itbl_novex_map2_32[] = {
    instrux + 3034,
    NULL
};

static const struct itemplate * const itbl_novex_map2_33[] = {
    instrux + 3035,
    NULL
};

static const struct itemplate * const itbl_novex_map2_34[] = {
    instrux + 3036,
    NULL
};

static const struct itemplate * const itbl_novex_map2_35[] = {
    instrux + 3037,
    NULL
};

static const struct itemplate * const itbl_novex_map2_37[] = {
    instrux + 3065,
    NULL
};

static const struct itemplate * const itbl_novex_map2_38[] = {
    instrux + 3022,
    NULL
};

static const struct itemplate * const itbl_novex_map2_39[] = {
    instrux + 3023,
    NULL
};

static const struct itemplate * const itbl_novex_map2_3a[] = {
    instrux + 3025,
    NULL
};

static const struct itemplate * const itbl_novex_map2_3b[] = {
    instrux + 3024,
    NULL
};

static const struct itemplate * const itbl_novex_map2_3c[] = {
    instrux + 3018,
    NULL
};

static const struct itemplate * const itbl_novex_map2_3d[] = {
    instrux + 3019,
    NULL
};

static const struct itemplate * const itbl_novex_map2_3e[] = {
    instrux + 3021,
    NULL
};

static const struct itemplate * const itbl_novex_map2_3f[] = {
    instrux + 3020,
    NULL
};

static const struct itemplate * const itbl_novex_map2_40[] = {
    instrux + 3039,
    NULL
};

static const struct itemplate * const itbl_novex_map2_41[] = {
    instrux + 3012,
    NULL
};

static const struct itemplate * const itbl_novex_map2_80[] = {
    instrux + 2938,
    instrux + 2939,
    NULL
};

static const struct itemplate * const itbl_novex_map2_81[] = {
    instrux + 2941,
    instrux + 2942,
    NULL
};

static const struct itemplate * const itbl_novex_map2_82[] = {
    instrux + 1293,
    instrux + 1294,
    NULL
};

static const struct itemplate * const itbl_novex_map2_8a[] = {
    instrux + 28,
    NULL
};

static const struct itemplate * const itbl_novex_map2_8b[] = {
    instrux + 29,
    instrux + 30,
    instrux + 31,
    NULL
};

static const struct itemplate * const itbl_novex_map2_c8[] = {
    instrux + 4395,
    NULL
};

static const struct itemplate * const itbl_novex_map2_c9[] = {
    instrux + 4393,
    NULL
};

static const struct itemplate * const itbl_novex_map2_ca[] = {
    instrux + 4394,
    NULL
};

static const struct itemplate * const itbl_novex_map2_cb[] = {
    instrux + 4399,
    instrux + 4400,
    NULL
};

static const struct itemplate * const itbl_novex_map2_cc[] = {
    instrux + 4397,
    NULL
};

static const struct itemplate * const itbl_novex_map2_cd[] = {
    instrux + 4398,
    NULL
};

static const struct itemplate * const itbl_novex_map2_cf[] = {
    instrux + 6724,
    NULL
};

static const struct itemplate * const itbl_novex_map2_d8[] = {
    instrux + 3094,
    instrux + 3095,
    instrux + 3096,
    instrux + 3097,
    NULL
};

static const struct itemplate * const itbl_novex_map2_db[] = {
    instrux + 3079,
    NULL
};

static const struct itemplate * const itbl_novex_map2_dc[] = {
    instrux + 3075,
    instrux + 3089,
    instrux + 3093,
    NULL
};

static const struct itemplate * const itbl_novex_map2_dd[] = {
    instrux + 3076,
    instrux + 3087,
    NULL
};

static const struct itemplate * const itbl_novex_map2_de[] = {
    instrux + 3077,
    instrux + 3090,
    NULL
};

static const struct itemplate * const itbl_novex_map2_df[] = {
    instrux + 3078,
    instrux + 3088,
    NULL
};

static const struct itemplate * const itbl_novex_map2_f0[] = {
    instrux + 978,
    instrux + 979,
    instrux + 980,
    instrux + 3045,
    instrux + 3051,
    instrux + 3059,
    NULL
};

static const struct itemplate * const itbl_novex_map2_f1[] = {
    instrux + 981,
    instrux + 982,
    instrux + 983,
    instrux + 3046,
    instrux + 3047,
    instrux + 3052,
    instrux + 3053,
    instrux + 3054,
    NULL
};

static const struct itemplate * const itbl_novex_map2_f5[] = {
    instrux + 6830,
    instrux + 6832,
    NULL
};

static const struct itemplate * const itbl_novex_map2_f6[] = {
    instrux + 826,
    instrux + 827,
    instrux + 832,
    instrux + 833,
    instrux + 6834,
    instrux + 6836,
    NULL
};

static const struct itemplate * const itbl_novex_map2_f8[] = {
    instrux + 6705,
    instrux + 6706,
    instrux + 6707,
    instrux + 6838,
    instrux + 6839,
    instrux + 6840,
    instrux + 6842,
    instrux + 6843,
    instrux + 6844,
    NULL
};

static const struct itemplate * const itbl_novex_map2_f9[] = {
    instrux + 6701,
    instrux + 6702,
    NULL
};

static const struct itemplate * const itbl_novex_map2_fa[] = {
    instrux + 3091,
    NULL
};

static const struct itemplate * const itbl_novex_map2_fb[] = {
    instrux + 3092,
    NULL
};

static const struct itemplate * const itbl_novex_map2_fc[] = {
    instrux + 7166,
    instrux + 7167,
    instrux + 7170,
    instrux + 7171,
    instrux + 7174,
    instrux + 7175,
    instrux + 7178,
    instrux + 7179,
    NULL
};

static const struct itemplate * const * const itbl_novex_map2[256] = {
    itbl_novex_map2_00,
    itbl_novex_map2_01,
    itbl_novex_map2_02,
    itbl_novex_map2_03,
    itbl_novex_map2_04,
    itbl_novex_map2_05,
    itbl_novex_map2_06,
    itbl_novex_map2_07,
    itbl_novex_map2_08,
    itbl_novex_map2_09,
    itbl_novex_map2_0a,
    itbl_novex_map2_0b,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map2_10,
    NULL,
    NULL,
    NULL,
    itbl_novex_map2_14,
    itbl_novex_map2_15,
    NULL,
    itbl_novex_map2_17,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map2_1c,
    itbl_novex_map2_1d,
    itbl_novex_map2_1e,
    NULL,
    itbl_novex_map2_20,
    itbl_novex_map2_21,
    itbl_novex_map2_22,
    itbl_novex_map2_23,
    itbl_novex_map2_24,
    itbl_novex_map2_25,
    NULL,
    NULL,
    itbl_novex_map2_28,
    itbl_novex_map2_29,
    itbl_novex_map2_2a,
    itbl_novex_map2_2b,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map2_30,
    itbl_novex_map2_31,
    itbl_novex_map2_32,
    itbl_novex_map2_33,
    itbl_novex_map2_34,
    itbl_novex_map2_35,
    NULL,
    itbl_novex_map2_37,
    itbl_novex_map2_38,
    itbl_novex_map2_39,
    itbl_novex_map2_3a,
    itbl_novex_map2_3b,
    itbl_novex_map2_3c,
    itbl_novex_map2_3d,
    itbl_novex_map2_3e,
    itbl_novex_map2_3f,
    itbl_novex_map2_40,
    itbl_novex_map2_41,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map2_80,
    itbl_novex_map2_81,
    itbl_novex_map2_82,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map2_8a,
    itbl_novex_map2_8b,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map2_c8,
    itbl_novex_map2_c9,
    itbl_novex_map2_ca,
    itbl_novex_map2_cb,
    itbl_novex_map2_cc,
    itbl_novex_map2_cd,
    NULL,
    itbl_novex_map2_cf,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map2_d8,
    NULL,
    NULL,
    itbl_novex_map2_db,
    itbl_novex_map2_dc,
    itbl_novex_map2_dd,
    itbl_novex_map2_de,
    itbl_novex_map2_df,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map2_f0,
    itbl_novex_map2_f1,
    NULL,
    NULL,
    NULL,
    itbl_novex_map2_f5,
    itbl_novex_map2_f6,
    NULL,
    itbl_novex_map2_f8,
    itbl_novex_map2_f9,
    itbl_novex_map2_fa,
    itbl_novex_map2_fb,
    itbl_novex_map2_fc,
    NULL,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_novex_map3_08[] = {
    instrux + 3042,
    NULL
};

static const struct itemplate * const itbl_novex_map3_09[] = {
    instrux + 3041,
    NULL
};

static const struct itemplate * const itbl_novex_map3_0a[] = {
    instrux + 3044,
    NULL
};

static const struct itemplate * const itbl_novex_map3_0b[] = {
    instrux + 3043,
    NULL
};

static const struct itemplate * const itbl_novex_map3_0c[] = {
    instrux + 2987,
    NULL
};

static const struct itemplate * const itbl_novex_map3_0d[] = {
    instrux + 2986,
    NULL
};

static const struct itemplate * const itbl_novex_map3_0e[] = {
    instrux + 3002,
    NULL
};

static const struct itemplate * const itbl_novex_map3_0f[] = {
    instrux + 2954,
    instrux + 2955,
    NULL
};

static const struct itemplate * const itbl_novex_map3_14[] = {
    instrux + 3004,
    instrux + 3005,
    instrux + 3006,
    NULL
};

static const struct itemplate * const itbl_novex_map3_15[] = {
    instrux + 3009,
    instrux + 3010,
    instrux + 3011,
    NULL
};

static const struct itemplate * const itbl_novex_map3_16[] = {
    instrux + 3007,
    instrux + 3008,
    NULL
};

static const struct itemplate * const itbl_novex_map3_17[] = {
    instrux + 2994,
    instrux + 2995,
    NULL
};

static const struct itemplate * const itbl_novex_map3_20[] = {
    instrux + 3013,
    instrux + 3014,
    instrux + 3015,
    NULL
};

static const struct itemplate * const itbl_novex_map3_21[] = {
    instrux + 2996,
    NULL
};

static const struct itemplate * const itbl_novex_map3_22[] = {
    instrux + 3016,
    instrux + 3017,
    NULL
};

static const struct itemplate * const itbl_novex_map3_40[] = {
    instrux + 2993,
    NULL
};

static const struct itemplate * const itbl_novex_map3_41[] = {
    instrux + 2992,
    NULL
};

static const struct itemplate * const itbl_novex_map3_42[] = {
    instrux + 2998,
    NULL
};

static const struct itemplate * const itbl_novex_map3_44[] = {
    instrux + 3796,
    instrux + 3797,
    instrux + 3798,
    instrux + 3799,
    instrux + 3800,
    NULL
};

static const struct itemplate * const itbl_novex_map3_60[] = {
    instrux + 3062,
    NULL
};

static const struct itemplate * const itbl_novex_map3_61[] = {
    instrux + 3061,
    NULL
};

static const struct itemplate * const itbl_novex_map3_62[] = {
    instrux + 3064,
    NULL
};

static const struct itemplate * const itbl_novex_map3_63[] = {
    instrux + 3063,
    NULL
};

static const struct itemplate * const itbl_novex_map3_cc[] = {
    instrux + 4396,
    NULL
};

static const struct itemplate * const itbl_novex_map3_ce[] = {
    instrux + 6718,
    NULL
};

static const struct itemplate * const itbl_novex_map3_cf[] = {
    instrux + 6712,
    NULL
};

static const struct itemplate * const itbl_novex_map3_df[] = {
    instrux + 3080,
    NULL
};

static const struct itemplate * const itbl_novex_map3_f0[] = {
    instrux + 7187,
    NULL
};

static const struct itemplate * const * const itbl_novex_map3[256] = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map3_08,
    itbl_novex_map3_09,
    itbl_novex_map3_0a,
    itbl_novex_map3_0b,
    itbl_novex_map3_0c,
    itbl_novex_map3_0d,
    itbl_novex_map3_0e,
    itbl_novex_map3_0f,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map3_14,
    itbl_novex_map3_15,
    itbl_novex_map3_16,
    itbl_novex_map3_17,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map3_20,
    itbl_novex_map3_21,
    itbl_novex_map3_22,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map3_40,
    itbl_novex_map3_41,
    itbl_novex_map3_42,
    NULL,
    itbl_novex_map3_44,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map3_60,
    itbl_novex_map3_61,
    itbl_novex_map3_62,
    itbl_novex_map3_63,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map3_cc,
    NULL,
    itbl_novex_map3_ce,
    itbl_novex_map3_cf,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map3_df,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map3_f0,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_vex_map1_10[] = {
    instrux + 3564,
    instrux + 3565,
    instrux + 3572,
    instrux + 3573,
    instrux + 3576,
    instrux + 3578,
    instrux + 3580,
    instrux + 3582,
    NULL
};

static const struct itemplate * const itbl_vex_map1_11[] = {
    instrux + 3566,
    instrux + 3567,
    instrux + 3574,
    instrux + 3575,
    instrux + 3577,
    instrux + 3579,
    instrux + 3581,
    instrux + 3583,
    NULL
};

static const struct itemplate * const itbl_vex_map1_12[] = {
    instrux + 3524,
    instrux + 3525,
    instrux + 3538,
    instrux + 3544,
    instrux + 3546,
    instrux + 3570,
    instrux + 3571,
    NULL
};

static const struct itemplate * const itbl_vex_map1_13[] = {
    instrux + 3545,
    instrux + 3547,
    NULL
};

static const struct itemplate * const itbl_vex_map1_14[] = {
    instrux + 3786,
    instrux + 3787,
    instrux + 3788,
    instrux + 3789,
    NULL
};

static const struct itemplate * const itbl_vex_map1_15[] = {
    instrux + 3782,
    instrux + 3783,
    instrux + 3784,
    instrux + 3785,
    NULL
};

static const struct itemplate * const itbl_vex_map1_16[] = {
    instrux + 3539,
    instrux + 3541,
    instrux + 3543,
    instrux + 3568,
    instrux + 3569,
    NULL
};

static const struct itemplate * const itbl_vex_map1_17[] = {
    instrux + 3540,
    instrux + 3542,
    NULL
};

static const struct itemplate * const itbl_vex_map1_28[] = {
    instrux + 3510,
    instrux + 3512,
    instrux + 3514,
    instrux + 3516,
    NULL
};

static const struct itemplate * const itbl_vex_map1_29[] = {
    instrux + 3511,
    instrux + 3513,
    instrux + 3515,
    instrux + 3517,
    NULL
};

static const struct itemplate * const itbl_vex_map1_2a[] = {
    instrux + 3447,
    instrux + 3448,
    instrux + 3449,
    instrux + 3450,
    NULL
};

static const struct itemplate * const itbl_vex_map1_2b[] = {
    instrux + 3560,
    instrux + 3561,
    instrux + 3562,
    instrux + 3563,
    NULL
};

static const struct itemplate * const itbl_vex_map1_2c[] = {
    instrux + 3460,
    instrux + 3461,
    instrux + 3462,
    instrux + 3463,
    NULL
};

static const struct itemplate * const itbl_vex_map1_2d[] = {
    instrux + 3444,
    instrux + 3445,
    instrux + 3452,
    instrux + 3453,
    NULL
};

static const struct itemplate * const itbl_vex_map1_2e[] = {
    instrux + 3780,
    instrux + 3781,
    NULL
};

static const struct itemplate * const itbl_vex_map1_2f[] = {
    instrux + 3426,
    instrux + 3427,
    NULL
};

static const struct itemplate * const itbl_vex_map1_41[] = {
    instrux + 4509,
    instrux + 4511,
    instrux + 4513,
    instrux + 4515,
    NULL
};

static const struct itemplate * const itbl_vex_map1_42[] = {
    instrux + 4517,
    instrux + 4519,
    instrux + 4521,
    instrux + 4523,
    NULL
};

static const struct itemplate * const itbl_vex_map1_44[] = {
    instrux + 4525,
    instrux + 4527,
    instrux + 4529,
    instrux + 4531,
    NULL
};

static const struct itemplate * const itbl_vex_map1_45[] = {
    instrux + 4533,
    instrux + 4535,
    instrux + 4537,
    instrux + 4539,
    NULL
};

static const struct itemplate * const itbl_vex_map1_46[] = {
    instrux + 4579,
    instrux + 4581,
    instrux + 4583,
    instrux + 4585,
    NULL
};

static const struct itemplate * const itbl_vex_map1_47[] = {
    instrux + 4587,
    instrux + 4589,
    instrux + 4591,
    instrux + 4593,
    NULL
};

static const struct itemplate * const itbl_vex_map1_4a[] = {
    instrux + 4501,
    instrux + 4503,
    instrux + 4505,
    instrux + 4507,
    NULL
};

static const struct itemplate * const itbl_vex_map1_4b[] = {
    instrux + 4573,
    instrux + 4575,
    instrux + 4577,
    NULL
};

static const struct itemplate * const itbl_vex_map1_50[] = {
    instrux + 3548,
    instrux + 3549,
    instrux + 3550,
    instrux + 3551,
    instrux + 3552,
    instrux + 3553,
    instrux + 3554,
    instrux + 3555,
    NULL
};

static const struct itemplate * const itbl_vex_map1_51[] = {
    instrux + 3763,
    instrux + 3764,
    instrux + 3765,
    instrux + 3766,
    instrux + 3767,
    instrux + 3768,
    NULL
};

static const struct itemplate * const itbl_vex_map1_52[] = {
    instrux + 3750,
    instrux + 3751,
    instrux + 3752,
    NULL
};

static const struct itemplate * const itbl_vex_map1_53[] = {
    instrux + 3747,
    instrux + 3748,
    instrux + 3749,
    NULL
};

static const struct itemplate * const itbl_vex_map1_54[] = {
    instrux + 3124,
    instrux + 3125,
    instrux + 3126,
    instrux + 3127,
    NULL
};

static const struct itemplate * const itbl_vex_map1_55[] = {
    instrux + 3128,
    instrux + 3129,
    instrux + 3130,
    instrux + 3131,
    NULL
};

static const struct itemplate * const itbl_vex_map1_56[] = {
    instrux + 3591,
    instrux + 3592,
    instrux + 3593,
    instrux + 3594,
    NULL
};

static const struct itemplate * const itbl_vex_map1_57[] = {
    instrux + 3790,
    instrux + 3791,
    instrux + 3792,
    instrux + 3793,
    NULL
};

static const struct itemplate * const itbl_vex_map1_58[] = {
    instrux + 3114,
    instrux + 3115,
    instrux + 3116,
    instrux + 3117,
    instrux + 3118,
    instrux + 3119,
    NULL
};

static const struct itemplate * const itbl_vex_map1_59[] = {
    instrux + 3585,
    instrux + 3586,
    instrux + 3587,
    instrux + 3588,
    instrux + 3589,
    instrux + 3590,
    NULL
};

static const struct itemplate * const itbl_vex_map1_5a[] = {
    instrux + 3436,
    instrux + 3437,
    instrux + 3438,
    instrux + 3439,
    instrux + 3442,
    instrux + 3443,
    instrux + 3446,
    instrux + 3451,
    NULL
};

static const struct itemplate * const itbl_vex_map1_5b[] = {
    instrux + 3430,
    instrux + 3431,
    instrux + 3440,
    instrux + 3441,
    instrux + 3458,
    instrux + 3459,
    NULL
};

static const struct itemplate * const itbl_vex_map1_5c[] = {
    instrux + 3770,
    instrux + 3771,
    instrux + 3772,
    instrux + 3773,
    instrux + 3774,
    instrux + 3775,
    NULL
};

static const struct itemplate * const itbl_vex_map1_5d[] = {
    instrux + 3504,
    instrux + 3505,
    instrux + 3506,
    instrux + 3507,
    instrux + 3508,
    instrux + 3509,
    NULL
};

static const struct itemplate * const itbl_vex_map1_5e[] = {
    instrux + 3464,
    instrux + 3465,
    instrux + 3466,
    instrux + 3467,
    instrux + 3468,
    instrux + 3469,
    NULL
};

static const struct itemplate * const itbl_vex_map1_5f[] = {
    instrux + 3498,
    instrux + 3499,
    instrux + 3500,
    instrux + 3501,
    instrux + 3502,
    instrux + 3503,
    NULL
};

static const struct itemplate * const itbl_vex_map1_60[] = {
    instrux + 3742,
    instrux + 4295,
    NULL
};

static const struct itemplate * const itbl_vex_map1_61[] = {
    instrux + 3743,
    instrux + 4296,
    NULL
};

static const struct itemplate * const itbl_vex_map1_62[] = {
    instrux + 3744,
    instrux + 4297,
    NULL
};

static const struct itemplate * const itbl_vex_map1_63[] = {
    instrux + 3598,
    instrux + 4182,
    NULL
};

static const struct itemplate * const itbl_vex_map1_64[] = {
    instrux + 3625,
    instrux + 4205,
    NULL
};

static const struct itemplate * const itbl_vex_map1_65[] = {
    instrux + 3626,
    instrux + 4206,
    NULL
};

static const struct itemplate * const itbl_vex_map1_66[] = {
    instrux + 3627,
    instrux + 4207,
    NULL
};

static const struct itemplate * const itbl_vex_map1_67[] = {
    instrux + 3600,
    instrux + 4185,
    NULL
};

static const struct itemplate * const itbl_vex_map1_68[] = {
    instrux + 3738,
    instrux + 4291,
    NULL
};

static const struct itemplate * const itbl_vex_map1_69[] = {
    instrux + 3739,
    instrux + 4292,
    NULL
};

static const struct itemplate * const itbl_vex_map1_6a[] = {
    instrux + 3740,
    instrux + 4293,
    NULL
};

static const struct itemplate * const itbl_vex_map1_6b[] = {
    instrux + 3599,
    instrux + 4183,
    NULL
};

static const struct itemplate * const itbl_vex_map1_6c[] = {
    instrux + 3745,
    instrux + 4298,
    NULL
};

static const struct itemplate * const itbl_vex_map1_6d[] = {
    instrux + 3741,
    instrux + 4294,
    NULL
};

static const struct itemplate * const itbl_vex_map1_6e[] = {
    instrux + 3518,
    instrux + 3522,
    NULL
};

static const struct itemplate * const itbl_vex_map1_6f[] = {
    instrux + 3526,
    instrux + 3528,
    instrux + 3530,
    instrux + 3532,
    instrux + 3534,
    instrux + 3536,
    NULL
};

static const struct itemplate * const itbl_vex_map1_70[] = {
    instrux + 3704,
    instrux + 3705,
    instrux + 3706,
    instrux + 4259,
    instrux + 4260,
    instrux + 4261,
    NULL
};

static const struct itemplate * const itbl_vex_map1_71[] = {
    instrux + 3713,
    instrux + 3719,
    instrux + 3723,
    instrux + 4267,
    instrux + 4273,
    instrux + 4278,
    NULL
};

static const struct itemplate * const itbl_vex_map1_72[] = {
    instrux + 3715,
    instrux + 3721,
    instrux + 3725,
    instrux + 4269,
    instrux + 4275,
    instrux + 4280,
    NULL
};

static const struct itemplate * const itbl_vex_map1_73[] = {
    instrux + 3710,
    instrux + 3711,
    instrux + 3717,
    instrux + 3727,
    instrux + 4265,
    instrux + 4271,
    instrux + 4276,
    instrux + 4282,
    NULL
};

static const struct itemplate * const itbl_vex_map1_74[] = {
    instrux + 3621,
    instrux + 4201,
    NULL
};

static const struct itemplate * const itbl_vex_map1_75[] = {
    instrux + 3622,
    instrux + 4202,
    NULL
};

static const struct itemplate * const itbl_vex_map1_76[] = {
    instrux + 3623,
    instrux + 4203,
    NULL
};

static const struct itemplate * const itbl_vex_map1_77[] = {
    instrux + 3794,
    instrux + 3795,
    NULL
};

static const struct itemplate * const itbl_vex_map1_7c[] = {
    instrux + 3475,
    instrux + 3476,
    instrux + 3477,
    instrux + 3478,
    NULL
};

static const struct itemplate * const itbl_vex_map1_7d[] = {
    instrux + 3479,
    instrux + 3480,
    instrux + 3481,
    instrux + 3482,
    NULL
};

static const struct itemplate * const itbl_vex_map1_7e[] = {
    instrux + 3519,
    instrux + 3520,
    instrux + 3523,
    NULL
};

static const struct itemplate * const itbl_vex_map1_7f[] = {
    instrux + 3527,
    instrux + 3529,
    instrux + 3531,
    instrux + 3533,
    instrux + 3535,
    instrux + 3537,
    NULL
};

static const struct itemplate * const itbl_vex_map1_90[] = {
    instrux + 4469,
    instrux + 4471,
    instrux + 4473,
    instrux + 4475,
    NULL
};

static const struct itemplate * const itbl_vex_map1_91[] = {
    instrux + 4477,
    instrux + 4479,
    instrux + 4481,
    instrux + 4483,
    NULL
};

static const struct itemplate * const itbl_vex_map1_92[] = {
    instrux + 4485,
    instrux + 4487,
    instrux + 4493,
    instrux + 4495,
    NULL
};

static const struct itemplate * const itbl_vex_map1_93[] = {
    instrux + 4489,
    instrux + 4491,
    instrux + 4497,
    instrux + 4499,
    NULL
};

static const struct itemplate * const itbl_vex_map1_98[] = {
    instrux + 4541,
    instrux + 4543,
    instrux + 4545,
    instrux + 4547,
    NULL
};

static const struct itemplate * const itbl_vex_map1_99[] = {
    instrux + 4565,
    instrux + 4567,
    instrux + 4569,
    instrux + 4571,
    NULL
};

static const struct itemplate * const itbl_vex_map1_ae[] = {
    instrux + 3488,
    instrux + 3769,
    NULL
};

static const struct itemplate * const itbl_vex_map1_c2[] = {
    instrux + 3144,
    instrux + 3145,
    instrux + 3146,
    instrux + 3147,
    instrux + 3148,
    instrux + 3149,
    instrux + 3150,
    instrux + 3151,
    instrux + 3152,
    instrux + 3153,
    instrux + 3154,
    instrux + 3155,
    instrux + 3156,
    instrux + 3157,
    instrux + 3158,
    instrux + 3159,
    instrux + 3160,
    instrux + 3161,
    instrux + 3162,
    instrux + 3163,
    instrux + 3164,
    instrux + 3165,
    instrux + 3166,
    instrux + 3167,
    instrux + 3168,
    instrux + 3169,
    instrux + 3170,
    instrux + 3171,
    instrux + 3172,
    instrux + 3173,
    instrux + 3174,
    instrux + 3175,
    instrux + 3176,
    instrux + 3177,
    instrux + 3178,
    instrux + 3179,
    instrux + 3180,
    instrux + 3181,
    instrux + 3182,
    instrux + 3183,
    instrux + 3184,
    instrux + 3185,
    instrux + 3186,
    instrux + 3187,
    instrux + 3188,
    instrux + 3189,
    instrux + 3190,
    instrux + 3191,
    instrux + 3192,
    instrux + 3193,
    instrux + 3194,
    instrux + 3195,
    instrux + 3196,
    instrux + 3197,
    instrux + 3198,
    instrux + 3199,
    instrux + 3200,
    instrux + 3201,
    instrux + 3202,
    instrux + 3203,
    instrux + 3204,
    instrux + 3205,
    instrux + 3206,
    instrux + 3207,
    instrux + 3208,
    instrux + 3209,
    instrux + 3210,
    instrux + 3211,
    instrux + 3212,
    instrux + 3213,
    instrux + 3214,
    instrux + 3215,
    instrux + 3216,
    instrux + 3217,
    instrux + 3218,
    instrux + 3219,
    instrux + 3220,
    instrux + 3221,
    instrux + 3222,
    instrux + 3223,
    instrux + 3224,
    instrux + 3225,
    instrux + 3226,
    instrux + 3227,
    instrux + 3228,
    instrux + 3229,
    instrux + 3230,
    instrux + 3231,
    instrux + 3232,
    instrux + 3233,
    instrux + 3234,
    instrux + 3235,
    instrux + 3236,
    instrux + 3237,
    instrux + 3238,
    instrux + 3239,
    instrux + 3240,
    instrux + 3241,
    instrux + 3242,
    instrux + 3243,
    instrux + 3244,
    instrux + 3245,
    instrux + 3246,
    instrux + 3247,
    instrux + 3248,
    instrux + 3249,
    instrux + 3250,
    instrux + 3251,
    instrux + 3252,
    instrux + 3253,
    instrux + 3254,
    instrux + 3255,
    instrux + 3256,
    instrux + 3257,
    instrux + 3258,
    instrux + 3259,
    instrux + 3260,
    instrux + 3261,
    instrux + 3262,
    instrux + 3263,
    instrux + 3264,
    instrux + 3265,
    instrux + 3266,
    instrux + 3267,
    instrux + 3268,
    instrux + 3269,
    instrux + 3270,
    instrux + 3271,
    instrux + 3272,
    instrux + 3273,
    instrux + 3274,
    instrux + 3275,
    instrux + 3276,
    instrux + 3277,
    instrux + 3278,
    instrux + 3279,
    instrux + 3280,
    instrux + 3281,
    instrux + 3282,
    instrux + 3283,
    instrux + 3284,
    instrux + 3285,
    instrux + 3286,
    instrux + 3287,
    instrux + 3288,
    instrux + 3289,
    instrux + 3290,
    instrux + 3291,
    instrux + 3292,
    instrux + 3293,
    instrux + 3294,
    instrux + 3295,
    instrux + 3296,
    instrux + 3297,
    instrux + 3298,
    instrux + 3299,
    instrux + 3300,
    instrux + 3301,
    instrux + 3302,
    instrux + 3303,
    instrux + 3304,
    instrux + 3305,
    instrux + 3306,
    instrux + 3307,
    instrux + 3308,
    instrux + 3309,
    instrux + 3310,
    instrux + 3311,
    instrux + 3312,
    instrux + 3313,
    instrux + 3314,
    instrux + 3315,
    instrux + 3316,
    instrux + 3317,
    instrux + 3318,
    instrux + 3319,
    instrux + 3320,
    instrux + 3321,
    instrux + 3322,
    instrux + 3323,
    instrux + 3324,
    instrux + 3325,
    instrux + 3326,
    instrux + 3327,
    instrux + 3328,
    instrux + 3329,
    instrux + 3330,
    instrux + 3331,
    instrux + 3332,
    instrux + 3333,
    instrux + 3334,
    instrux + 3335,
    instrux + 3336,
    instrux + 3337,
    instrux + 3338,
    instrux + 3339,
    instrux + 3340,
    instrux + 3341,
    instrux + 3342,
    instrux + 3343,
    instrux + 3344,
    instrux + 3345,
    instrux + 3346,
    instrux + 3347,
    instrux + 3348,
    instrux + 3349,
    instrux + 3350,
    instrux + 3351,
    instrux + 3352,
    instrux + 3353,
    instrux + 3354,
    instrux + 3355,
    instrux + 3356,
    instrux + 3357,
    instrux + 3358,
    instrux + 3359,
    instrux + 3360,
    instrux + 3361,
    instrux + 3362,
    instrux + 3363,
    instrux + 3364,
    instrux + 3365,
    instrux + 3366,
    instrux + 3367,
    instrux + 3368,
    instrux + 3369,
    instrux + 3370,
    instrux + 3371,
    instrux + 3372,
    instrux + 3373,
    instrux + 3374,
    instrux + 3375,
    instrux + 3376,
    instrux + 3377,
    instrux + 3378,
    instrux + 3379,
    instrux + 3380,
    instrux + 3381,
    instrux + 3382,
    instrux + 3383,
    instrux + 3384,
    instrux + 3385,
    instrux + 3386,
    instrux + 3387,
    instrux + 3388,
    instrux + 3389,
    instrux + 3390,
    instrux + 3391,
    instrux + 3392,
    instrux + 3393,
    instrux + 3394,
    instrux + 3395,
    instrux + 3396,
    instrux + 3397,
    instrux + 3398,
    instrux + 3399,
    instrux + 3400,
    instrux + 3401,
    instrux + 3402,
    instrux + 3403,
    instrux + 3404,
    instrux + 3405,
    instrux + 3406,
    instrux + 3407,
    instrux + 3408,
    instrux + 3409,
    instrux + 3410,
    instrux + 3411,
    instrux + 3412,
    instrux + 3413,
    instrux + 3414,
    instrux + 3415,
    instrux + 3416,
    instrux + 3417,
    instrux + 3418,
    instrux + 3419,
    instrux + 3420,
    instrux + 3421,
    instrux + 3422,
    instrux + 3423,
    instrux + 3424,
    instrux + 3425,
    NULL
};

static const struct itemplate * const itbl_vex_map1_c4[] = {
    instrux + 3659,
    instrux + 3660,
    instrux + 3661,
    NULL
};

static const struct itemplate * const itbl_vex_map1_c5[] = {
    instrux + 3641,
    instrux + 3642,
    NULL
};

static const struct itemplate * const itbl_vex_map1_c6[] = {
    instrux + 3759,
    instrux + 3760,
    instrux + 3761,
    instrux + 3762,
    NULL
};

static const struct itemplate * const itbl_vex_map1_d0[] = {
    instrux + 3120,
    instrux + 3121,
    instrux + 3122,
    instrux + 3123,
    NULL
};

static const struct itemplate * const itbl_vex_map1_d1[] = {
    instrux + 3722,
    instrux + 4277,
    NULL
};

static const struct itemplate * const itbl_vex_map1_d2[] = {
    instrux + 3724,
    instrux + 4279,
    NULL
};

static const struct itemplate * const itbl_vex_map1_d3[] = {
    instrux + 3726,
    instrux + 4281,
    NULL
};

static const struct itemplate * const itbl_vex_map1_d4[] = {
    instrux + 3605,
    instrux + 4189,
    NULL
};

static const struct itemplate * const itbl_vex_map1_d5[] = {
    instrux + 3697,
    instrux + 4253,
    NULL
};

static const struct itemplate * const itbl_vex_map1_d6[] = {
    instrux + 3521,
    NULL
};

static const struct itemplate * const itbl_vex_map1_d7[] = {
    instrux + 3680,
    instrux + 3681,
    instrux + 4229,
    instrux + 4230,
    NULL
};

static const struct itemplate * const itbl_vex_map1_d8[] = {
    instrux + 3736,
    instrux + 4289,
    NULL
};

static const struct itemplate * const itbl_vex_map1_d9[] = {
    instrux + 3737,
    instrux + 4290,
    NULL
};

static const struct itemplate * const itbl_vex_map1_da[] = {
    instrux + 3677,
    instrux + 4226,
    NULL
};

static const struct itemplate * const itbl_vex_map1_db[] = {
    instrux + 3611,
    instrux + 4195,
    NULL
};

static const struct itemplate * const itbl_vex_map1_dc[] = {
    instrux + 3608,
    instrux + 4192,
    NULL
};

static const struct itemplate * const itbl_vex_map1_dd[] = {
    instrux + 3609,
    instrux + 4193,
    NULL
};

static const struct itemplate * const itbl_vex_map1_de[] = {
    instrux + 3671,
    instrux + 4220,
    NULL
};

static const struct itemplate * const itbl_vex_map1_df[] = {
    instrux + 3612,
    instrux + 4196,
    NULL
};

static const struct itemplate * const itbl_vex_map1_e0[] = {
    instrux + 3613,
    instrux + 4197,
    NULL
};

static const struct itemplate * const itbl_vex_map1_e1[] = {
    instrux + 3718,
    instrux + 4272,
    NULL
};

static const struct itemplate * const itbl_vex_map1_e2[] = {
    instrux + 3720,
    instrux + 4274,
    NULL
};

static const struct itemplate * const itbl_vex_map1_e3[] = {
    instrux + 3614,
    instrux + 4198,
    NULL
};

static const struct itemplate * const itbl_vex_map1_e4[] = {
    instrux + 3694,
    instrux + 4251,
    NULL
};

static const struct itemplate * const itbl_vex_map1_e5[] = {
    instrux + 3696,
    instrux + 4252,
    NULL
};

static const struct itemplate * const itbl_vex_map1_e6[] = {
    instrux + 3428,
    instrux + 3429,
    instrux + 3432,
    instrux + 3433,
    instrux + 3434,
    instrux + 3435,
    instrux + 3454,
    instrux + 3455,
    instrux + 3456,
    instrux + 3457,
    NULL
};

static const struct itemplate * const itbl_vex_map1_e7[] = {
    instrux + 3556,
    instrux + 3557,
    instrux + 3558,
    NULL
};

static const struct itemplate * const itbl_vex_map1_e8[] = {
    instrux + 3734,
    instrux + 4287,
    NULL
};

static const struct itemplate * const itbl_vex_map1_e9[] = {
    instrux + 3735,
    instrux + 4288,
    NULL
};

static const struct itemplate * const itbl_vex_map1_ea[] = {
    instrux + 3675,
    instrux + 4224,
    NULL
};

static const struct itemplate * const itbl_vex_map1_eb[] = {
    instrux + 3701,
    instrux + 4256,
    NULL
};

static const struct itemplate * const itbl_vex_map1_ec[] = {
    instrux + 3606,
    instrux + 4190,
    NULL
};

static const struct itemplate * const itbl_vex_map1_ed[] = {
    instrux + 3607,
    instrux + 4191,
    NULL
};

static const struct itemplate * const itbl_vex_map1_ee[] = {
    instrux + 3669,
    instrux + 4218,
    NULL
};

static const struct itemplate * const itbl_vex_map1_ef[] = {
    instrux + 3746,
    instrux + 4299,
    NULL
};

static const struct itemplate * const itbl_vex_map1_f0[] = {
    instrux + 3485,
    instrux + 3486,
    instrux + 3487,
    NULL
};

static const struct itemplate * const itbl_vex_map1_f1[] = {
    instrux + 3712,
    instrux + 4266,
    NULL
};

static const struct itemplate * const itbl_vex_map1_f2[] = {
    instrux + 3714,
    instrux + 4268,
    NULL
};

static const struct itemplate * const itbl_vex_map1_f3[] = {
    instrux + 3716,
    instrux + 4270,
    NULL
};

static const struct itemplate * const itbl_vex_map1_f4[] = {
    instrux + 3699,
    instrux + 4255,
    NULL
};

static const struct itemplate * const itbl_vex_map1_f5[] = {
    instrux + 3666,
    instrux + 4216,
    NULL
};

static const struct itemplate * const itbl_vex_map1_f6[] = {
    instrux + 3702,
    instrux + 4257,
    NULL
};

static const struct itemplate * const itbl_vex_map1_f7[] = {
    instrux + 3489,
    NULL
};

static const struct itemplate * const itbl_vex_map1_f8[] = {
    instrux + 3730,
    instrux + 4283,
    NULL
};

static const struct itemplate * const itbl_vex_map1_f9[] = {
    instrux + 3731,
    instrux + 4284,
    NULL
};

static const struct itemplate * const itbl_vex_map1_fa[] = {
    instrux + 3732,
    instrux + 4285,
    NULL
};

static const struct itemplate * const itbl_vex_map1_fb[] = {
    instrux + 3733,
    instrux + 4286,
    NULL
};

static const struct itemplate * const itbl_vex_map1_fc[] = {
    instrux + 3602,
    instrux + 4186,
    NULL
};

static const struct itemplate * const itbl_vex_map1_fd[] = {
    instrux + 3603,
    instrux + 4187,
    NULL
};

static const struct itemplate * const itbl_vex_map1_fe[] = {
    instrux + 3604,
    instrux + 4188,
    NULL
};

static const struct itemplate * const * const itbl_vex_map1[256] = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map1_10,
    itbl_vex_map1_11,
    itbl_vex_map1_12,
    itbl_vex_map1_13,
    itbl_vex_map1_14,
    itbl_vex_map1_15,
    itbl_vex_map1_16,
    itbl_vex_map1_17,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map1_28,
    itbl_vex_map1_29,
    itbl_vex_map1_2a,
    itbl_vex_map1_2b,
    itbl_vex_map1_2c,
    itbl_vex_map1_2d,
    itbl_vex_map1_2e,
    itbl_vex_map1_2f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map1_41,
    itbl_vex_map1_42,
    NULL,
    itbl_vex_map1_44,
    itbl_vex_map1_45,
    itbl_vex_map1_46,
    itbl_vex_map1_47,
    NULL,
    NULL,
    itbl_vex_map1_4a,
    itbl_vex_map1_4b,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map1_50,
    itbl_vex_map1_51,
    itbl_vex_map1_52,
    itbl_vex_map1_53,
    itbl_vex_map1_54,
    itbl_vex_map1_55,
    itbl_vex_map1_56,
    itbl_vex_map1_57,
    itbl_vex_map1_58,
    itbl_vex_map1_59,
    itbl_vex_map1_5a,
    itbl_vex_map1_5b,
    itbl_vex_map1_5c,
    itbl_vex_map1_5d,
    itbl_vex_map1_5e,
    itbl_vex_map1_5f,
    itbl_vex_map1_60,
    itbl_vex_map1_61,
    itbl_vex_map1_62,
    itbl_vex_map1_63,
    itbl_vex_map1_64,
    itbl_vex_map1_65,
    itbl_vex_map1_66,
    itbl_vex_map1_67,
    itbl_vex_map1_68,
    itbl_vex_map1_69,
    itbl_vex_map1_6a,
    itbl_vex_map1_6b,
    itbl_vex_map1_6c,
    itbl_vex_map1_6d,
    itbl_vex_map1_6e,
    itbl_vex_map1_6f,
    itbl_vex_map1_70,
    itbl_vex_map1_71,
    itbl_vex_map1_72,
    itbl_vex_map1_73,
    itbl_vex_map1_74,
    itbl_vex_map1_75,
    itbl_vex_map1_76,
    itbl_vex_map1_77,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map1_7c,
    itbl_vex_map1_7d,
    itbl_vex_map1_7e,
    itbl_vex_map1_7f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map1_90,
    itbl_vex_map1_91,
    itbl_vex_map1_92,
    itbl_vex_map1_93,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map1_98,
    itbl_vex_map1_99,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map1_ae,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map1_c2,
    NULL,
    itbl_vex_map1_c4,
    itbl_vex_map1_c5,
    itbl_vex_map1_c6,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map1_d0,
    itbl_vex_map1_d1,
    itbl_vex_map1_d2,
    itbl_vex_map1_d3,
    itbl_vex_map1_d4,
    itbl_vex_map1_d5,
    itbl_vex_map1_d6,
    itbl_vex_map1_d7,
    itbl_vex_map1_d8,
    itbl_vex_map1_d9,
    itbl_vex_map1_da,
    itbl_vex_map1_db,
    itbl_vex_map1_dc,
    itbl_vex_map1_dd,
    itbl_vex_map1_de,
    itbl_vex_map1_df,
    itbl_vex_map1_e0,
    itbl_vex_map1_e1,
    itbl_vex_map1_e2,
    itbl_vex_map1_e3,
    itbl_vex_map1_e4,
    itbl_vex_map1_e5,
    itbl_vex_map1_e6,
    itbl_vex_map1_e7,
    itbl_vex_map1_e8,
    itbl_vex_map1_e9,
    itbl_vex_map1_ea,
    itbl_vex_map1_eb,
    itbl_vex_map1_ec,
    itbl_vex_map1_ed,
    itbl_vex_map1_ee,
    itbl_vex_map1_ef,
    itbl_vex_map1_f0,
    itbl_vex_map1_f1,
    itbl_vex_map1_f2,
    itbl_vex_map1_f3,
    itbl_vex_map1_f4,
    itbl_vex_map1_f5,
    itbl_vex_map1_f6,
    itbl_vex_map1_f7,
    itbl_vex_map1_f8,
    itbl_vex_map1_f9,
    itbl_vex_map1_fa,
    itbl_vex_map1_fb,
    itbl_vex_map1_fc,
    itbl_vex_map1_fd,
    itbl_vex_map1_fe,
    NULL,
};

static const struct itemplate * const itbl_vex_map2_00[] = {
    instrux + 3703,
    instrux + 4258,
    NULL
};

static const struct itemplate * const itbl_vex_map2_01[] = {
    instrux + 3649,
    instrux + 4209,
    NULL
};

static const struct itemplate * const itbl_vex_map2_02[] = {
    instrux + 3650,
    instrux + 4210,
    NULL
};

static const struct itemplate * const itbl_vex_map2_03[] = {
    instrux + 3651,
    instrux + 4211,
    NULL
};

static const struct itemplate * const itbl_vex_map2_04[] = {
    instrux + 3667,
    instrux + 4215,
    NULL
};

static const struct itemplate * const itbl_vex_map2_05[] = {
    instrux + 3653,
    instrux + 4212,
    NULL
};

static const struct itemplate * const itbl_vex_map2_06[] = {
    instrux + 3654,
    instrux + 4213,
    NULL
};

static const struct itemplate * const itbl_vex_map2_07[] = {
    instrux + 3655,
    instrux + 4214,
    NULL
};

static const struct itemplate * const itbl_vex_map2_08[] = {
    instrux + 3707,
    instrux + 4262,
    NULL
};

static const struct itemplate * const itbl_vex_map2_09[] = {
    instrux + 3708,
    instrux + 4263,
    NULL
};

static const struct itemplate * const itbl_vex_map2_0a[] = {
    instrux + 3709,
    instrux + 4264,
    NULL
};

static const struct itemplate * const itbl_vex_map2_0b[] = {
    instrux + 3695,
    instrux + 4250,
    NULL
};

static const struct itemplate * const itbl_vex_map2_0c[] = {
    instrux + 3633,
    instrux + 3634,
    NULL
};

static const struct itemplate * const itbl_vex_map2_0d[] = {
    instrux + 3629,
    instrux + 3630,
    NULL
};

static const struct itemplate * const itbl_vex_map2_0e[] = {
    instrux + 3776,
    instrux + 3777,
    NULL
};

static const struct itemplate * const itbl_vex_map2_0f[] = {
    instrux + 3778,
    instrux + 3779,
    NULL
};

static const struct itemplate * const itbl_vex_map2_13[] = {
    instrux + 4018,
    instrux + 4019,
    NULL
};

static const struct itemplate * const itbl_vex_map2_16[] = {
    instrux + 4325,
    NULL
};

static const struct itemplate * const itbl_vex_map2_17[] = {
    instrux + 3728,
    instrux + 3729,
    NULL
};

static const struct itemplate * const itbl_vex_map2_18[] = {
    instrux + 3140,
    instrux + 3141,
    instrux + 4301,
    instrux + 4302,
    NULL
};

static const struct itemplate * const itbl_vex_map2_19[] = {
    instrux + 3142,
    instrux + 4303,
    NULL
};

static const struct itemplate * const itbl_vex_map2_1a[] = {
    instrux + 3143,
    NULL
};

static const struct itemplate * const itbl_vex_map2_1c[] = {
    instrux + 3595,
    instrux + 4179,
    NULL
};

static const struct itemplate * const itbl_vex_map2_1d[] = {
    instrux + 3596,
    instrux + 4180,
    NULL
};

static const struct itemplate * const itbl_vex_map2_1e[] = {
    instrux + 3597,
    instrux + 4181,
    NULL
};

static const struct itemplate * const itbl_vex_map2_20[] = {
    instrux + 3682,
    instrux + 4231,
    NULL
};

static const struct itemplate * const itbl_vex_map2_21[] = {
    instrux + 3683,
    instrux + 4232,
    instrux + 4233,
    NULL
};

static const struct itemplate * const itbl_vex_map2_22[] = {
    instrux + 3684,
    instrux + 4234,
    instrux + 4235,
    NULL
};

static const struct itemplate * const itbl_vex_map2_23[] = {
    instrux + 3685,
    instrux + 4236,
    NULL
};

static const struct itemplate * const itbl_vex_map2_24[] = {
    instrux + 3686,
    instrux + 4237,
    instrux + 4238,
    NULL
};

static const struct itemplate * const itbl_vex_map2_25[] = {
    instrux + 3687,
    instrux + 4239,
    NULL
};

static const struct itemplate * const itbl_vex_map2_28[] = {
    instrux + 3700,
    instrux + 4249,
    NULL
};

static const struct itemplate * const itbl_vex_map2_29[] = {
    instrux + 3624,
    instrux + 4204,
    NULL
};

static const struct itemplate * const itbl_vex_map2_2a[] = {
    instrux + 3559,
    instrux + 4300,
    NULL
};

static const struct itemplate * const itbl_vex_map2_2b[] = {
    instrux + 3601,
    instrux + 4184,
    NULL
};

static const struct itemplate * const itbl_vex_map2_2c[] = {
    instrux + 3490,
    instrux + 3491,
    NULL
};

static const struct itemplate * const itbl_vex_map2_2d[] = {
    instrux + 3494,
    instrux + 3495,
    NULL
};

static const struct itemplate * const itbl_vex_map2_2e[] = {
    instrux + 3492,
    instrux + 3493,
    NULL
};

static const struct itemplate * const itbl_vex_map2_2f[] = {
    instrux + 3496,
    instrux + 3497,
    NULL
};

static const struct itemplate * const itbl_vex_map2_30[] = {
    instrux + 3688,
    instrux + 4240,
    NULL
};

static const struct itemplate * const itbl_vex_map2_31[] = {
    instrux + 3689,
    instrux + 4241,
    instrux + 4242,
    NULL
};

static const struct itemplate * const itbl_vex_map2_32[] = {
    instrux + 3690,
    instrux + 4243,
    instrux + 4244,
    NULL
};

static const struct itemplate * const itbl_vex_map2_33[] = {
    instrux + 3691,
    instrux + 4245,
    NULL
};

static const struct itemplate * const itbl_vex_map2_34[] = {
    instrux + 3692,
    instrux + 4246,
    instrux + 4247,
    NULL
};

static const struct itemplate * const itbl_vex_map2_35[] = {
    instrux + 3693,
    instrux + 4248,
    NULL
};

static const struct itemplate * const itbl_vex_map2_36[] = {
    instrux + 4323,
    NULL
};

static const struct itemplate * const itbl_vex_map2_37[] = {
    instrux + 3628,
    instrux + 4208,
    NULL
};

static const struct itemplate * const itbl_vex_map2_38[] = {
    instrux + 3674,
    instrux + 4223,
    NULL
};

static const struct itemplate * const itbl_vex_map2_39[] = {
    instrux + 3676,
    instrux + 4225,
    NULL
};

static const struct itemplate * const itbl_vex_map2_3a[] = {
    instrux + 3678,
    instrux + 4227,
    NULL
};

static const struct itemplate * const itbl_vex_map2_3b[] = {
    instrux + 3679,
    instrux + 4228,
    NULL
};

static const struct itemplate * const itbl_vex_map2_3c[] = {
    instrux + 3668,
    instrux + 4217,
    NULL
};

static const struct itemplate * const itbl_vex_map2_3d[] = {
    instrux + 3670,
    instrux + 4219,
    NULL
};

static const struct itemplate * const itbl_vex_map2_3e[] = {
    instrux + 3672,
    instrux + 4221,
    NULL
};

static const struct itemplate * const itbl_vex_map2_3f[] = {
    instrux + 3673,
    instrux + 4222,
    NULL
};

static const struct itemplate * const itbl_vex_map2_40[] = {
    instrux + 3698,
    instrux + 4254,
    NULL
};

static const struct itemplate * const itbl_vex_map2_41[] = {
    instrux + 3652,
    NULL
};

static const struct itemplate * const itbl_vex_map2_45[] = {
    instrux + 4344,
    instrux + 4345,
    instrux + 4346,
    instrux + 4347,
    NULL
};

static const struct itemplate * const itbl_vex_map2_46[] = {
    instrux + 4342,
    instrux + 4343,
    NULL
};

static const struct itemplate * const itbl_vex_map2_47[] = {
    instrux + 4338,
    instrux + 4339,
    instrux + 4340,
    instrux + 4341,
    NULL
};

static const struct itemplate * const itbl_vex_map2_48[] = {
    instrux + 6914,
    instrux + 6919,
    NULL
};

static const struct itemplate * const itbl_vex_map2_49[] = {
    instrux + 6864,
    instrux + 6866,
    instrux + 6880,
    instrux + 6883,
    NULL
};

static const struct itemplate * const itbl_vex_map2_4a[] = {
    instrux + 6884,
    instrux + 6886,
    NULL
};

static const struct itemplate * const itbl_vex_map2_4b[] = {
    instrux + 6876,
    instrux + 6878,
    instrux + 6881,
    NULL
};

static const struct itemplate * const itbl_vex_map2_50[] = {
    instrux + 4433,
    instrux + 4434,
    instrux + 4441,
    instrux + 4442,
    instrux + 4445,
    instrux + 4446,
    instrux + 4449,
    instrux + 4450,
    NULL
};

static const struct itemplate * const itbl_vex_map2_51[] = {
    instrux + 4435,
    instrux + 4436,
    instrux + 4443,
    instrux + 4444,
    instrux + 4447,
    instrux + 4448,
    instrux + 4451,
    instrux + 4452,
    NULL
};

static const struct itemplate * const itbl_vex_map2_52[] = {
    instrux + 4437,
    instrux + 4438,
    NULL
};

static const struct itemplate * const itbl_vex_map2_53[] = {
    instrux + 4439,
    instrux + 4440,
    NULL
};

static const struct itemplate * const itbl_vex_map2_58[] = {
    instrux + 4315,
    instrux + 4316,
    instrux + 4317,
    instrux + 4318,
    NULL
};

static const struct itemplate * const itbl_vex_map2_59[] = {
    instrux + 4319,
    instrux + 4320,
    instrux + 4321,
    instrux + 4322,
    NULL
};

static const struct itemplate * const itbl_vex_map2_5a[] = {
    instrux + 4304,
    NULL
};

static const struct itemplate * const itbl_vex_map2_5c[] = {
    instrux + 6868,
    instrux + 6869,
    NULL
};

static const struct itemplate * const itbl_vex_map2_5e[] = {
    instrux + 6872,
    instrux + 6873,
    instrux + 6874,
    instrux + 6875,
    NULL
};

static const struct itemplate * const itbl_vex_map2_5f[] = {
    instrux + 6920,
    NULL
};

static const struct itemplate * const itbl_vex_map2_6b[] = {
    instrux + 6896,
    instrux + 6897,
    instrux + 6915,
    instrux + 6916,
    NULL
};

static const struct itemplate * const itbl_vex_map2_6c[] = {
    instrux + 6870,
    instrux + 6871,
    instrux + 6917,
    instrux + 6918,
    NULL
};

static const struct itemplate * const itbl_vex_map2_6e[] = {
    instrux + 6888,
    instrux + 6890,
    NULL
};

static const struct itemplate * const itbl_vex_map2_6f[] = {
    instrux + 6889,
    instrux + 6891,
    NULL
};

static const struct itemplate * const itbl_vex_map2_72[] = {
    instrux + 4431,
    instrux + 4432,
    NULL
};

static const struct itemplate * const itbl_vex_map2_78[] = {
    instrux + 4307,
    instrux + 4308,
    instrux + 4309,
    instrux + 4310,
    NULL
};

static const struct itemplate * const itbl_vex_map2_79[] = {
    instrux + 4311,
    instrux + 4312,
    instrux + 4313,
    instrux + 4314,
    NULL
};

static const struct itemplate * const itbl_vex_map2_8c[] = {
    instrux + 4330,
    instrux + 4331,
    instrux + 4332,
    instrux + 4333,
    NULL
};

static const struct itemplate * const itbl_vex_map2_8e[] = {
    instrux + 4334,
    instrux + 4335,
    instrux + 4336,
    instrux + 4337,
    NULL
};

static const struct itemplate * const itbl_vex_map2_90[] = {
    instrux + 4356,
    instrux + 4358,
    instrux + 4360,
    instrux + 4362,
    NULL
};

static const struct itemplate * const itbl_vex_map2_91[] = {
    instrux + 4357,
    instrux + 4359,
    instrux + 4361,
    instrux + 4363,
    NULL
};

static const struct itemplate * const itbl_vex_map2_92[] = {
    instrux + 4348,
    instrux + 4350,
    instrux + 4352,
    instrux + 4354,
    NULL
};

static const struct itemplate * const itbl_vex_map2_93[] = {
    instrux + 4349,
    instrux + 4351,
    instrux + 4353,
    instrux + 4355,
    NULL
};

static const struct itemplate * const itbl_vex_map2_96[] = {
    instrux + 3850,
    instrux + 3851,
    instrux + 3852,
    instrux + 3853,
    instrux + 3854,
    instrux + 3855,
    instrux + 3856,
    instrux + 3857,
    NULL
};

static const struct itemplate * const itbl_vex_map2_97[] = {
    instrux + 3898,
    instrux + 3899,
    instrux + 3900,
    instrux + 3901,
    instrux + 3902,
    instrux + 3903,
    instrux + 3904,
    instrux + 3905,
    NULL
};

static const struct itemplate * const itbl_vex_map2_98[] = {
    instrux + 3826,
    instrux + 3827,
    instrux + 3828,
    instrux + 3829,
    instrux + 3830,
    instrux + 3831,
    instrux + 3832,
    instrux + 3833,
    NULL
};

static const struct itemplate * const itbl_vex_map2_99[] = {
    instrux + 3970,
    instrux + 3971,
    instrux + 3972,
    instrux + 3973,
    NULL
};

static const struct itemplate * const itbl_vex_map2_9a[] = {
    instrux + 3874,
    instrux + 3875,
    instrux + 3876,
    instrux + 3877,
    instrux + 3878,
    instrux + 3879,
    instrux + 3880,
    instrux + 3881,
    NULL
};

static const struct itemplate * const itbl_vex_map2_9b[] = {
    instrux + 3982,
    instrux + 3983,
    instrux + 3984,
    instrux + 3985,
    NULL
};

static const struct itemplate * const itbl_vex_map2_9c[] = {
    instrux + 3922,
    instrux + 3923,
    instrux + 3924,
    instrux + 3925,
    instrux + 3926,
    instrux + 3927,
    instrux + 3928,
    instrux + 3929,
    NULL
};

static const struct itemplate * const itbl_vex_map2_9d[] = {
    instrux + 3994,
    instrux + 3995,
    instrux + 3996,
    instrux + 3997,
    NULL
};

static const struct itemplate * const itbl_vex_map2_9e[] = {
    instrux + 3946,
    instrux + 3947,
    instrux + 3948,
    instrux + 3949,
    instrux + 3950,
    instrux + 3951,
    instrux + 3952,
    instrux + 3953,
    NULL
};

static const struct itemplate * const itbl_vex_map2_9f[] = {
    instrux + 4006,
    instrux + 4007,
    instrux + 4008,
    instrux + 4009,
    NULL
};

static const struct itemplate * const itbl_vex_map2_a6[] = {
    instrux + 3858,
    instrux + 3859,
    instrux + 3860,
    instrux + 3861,
    instrux + 3862,
    instrux + 3863,
    instrux + 3864,
    instrux + 3865,
    NULL
};

static const struct itemplate * const itbl_vex_map2_a7[] = {
    instrux + 3906,
    instrux + 3907,
    instrux + 3908,
    instrux + 3909,
    instrux + 3910,
    instrux + 3911,
    instrux + 3912,
    instrux + 3913,
    NULL
};

static const struct itemplate * const itbl_vex_map2_a8[] = {
    instrux + 3834,
    instrux + 3835,
    instrux + 3836,
    instrux + 3837,
    instrux + 3838,
    instrux + 3839,
    instrux + 3840,
    instrux + 3841,
    NULL
};

static const struct itemplate * const itbl_vex_map2_a9[] = {
    instrux + 3974,
    instrux + 3975,
    instrux + 3976,
    instrux + 3977,
    NULL
};

static const struct itemplate * const itbl_vex_map2_aa[] = {
    instrux + 3882,
    instrux + 3883,
    instrux + 3884,
    instrux + 3885,
    instrux + 3886,
    instrux + 3887,
    instrux + 3888,
    instrux + 3889,
    NULL
};

static const struct itemplate * const itbl_vex_map2_ab[] = {
    instrux + 3986,
    instrux + 3987,
    instrux + 3988,
    instrux + 3989,
    NULL
};

static const struct itemplate * const itbl_vex_map2_ac[] = {
    instrux + 3930,
    instrux + 3931,
    instrux + 3932,
    instrux + 3933,
    instrux + 3934,
    instrux + 3935,
    instrux + 3936,
    instrux + 3937,
    NULL
};

static const struct itemplate * const itbl_vex_map2_ad[] = {
    instrux + 3998,
    instrux + 3999,
    instrux + 4000,
    instrux + 4001,
    NULL
};

static const struct itemplate * const itbl_vex_map2_ae[] = {
    instrux + 3954,
    instrux + 3955,
    instrux + 3956,
    instrux + 3957,
    instrux + 3958,
    instrux + 3959,
    instrux + 3960,
    instrux + 3961,
    NULL
};

static const struct itemplate * const itbl_vex_map2_af[] = {
    instrux + 4010,
    instrux + 4011,
    instrux + 4012,
    instrux + 4013,
    NULL
};

static const struct itemplate * const itbl_vex_map2_b0[] = {
    instrux + 4423,
    instrux + 4424,
    instrux + 4425,
    instrux + 4426,
    instrux + 4427,
    instrux + 4428,
    instrux + 4429,
    instrux + 4430,
    NULL
};

static const struct itemplate * const itbl_vex_map2_b1[] = {
    instrux + 4417,
    instrux + 4418,
    instrux + 4419,
    instrux + 4420,
    instrux + 4421,
    instrux + 4422,
    NULL
};

static const struct itemplate * const itbl_vex_map2_b4[] = {
    instrux + 4467,
    instrux + 4468,
    NULL
};

static const struct itemplate * const itbl_vex_map2_b5[] = {
    instrux + 4465,
    instrux + 4466,
    NULL
};

static const struct itemplate * const itbl_vex_map2_b6[] = {
    instrux + 3866,
    instrux + 3867,
    instrux + 3868,
    instrux + 3869,
    instrux + 3870,
    instrux + 3871,
    instrux + 3872,
    instrux + 3873,
    NULL
};

static const struct itemplate * const itbl_vex_map2_b7[] = {
    instrux + 3914,
    instrux + 3915,
    instrux + 3916,
    instrux + 3917,
    instrux + 3918,
    instrux + 3919,
    instrux + 3920,
    instrux + 3921,
    NULL
};

static const struct itemplate * const itbl_vex_map2_b8[] = {
    instrux + 3842,
    instrux + 3843,
    instrux + 3844,
    instrux + 3845,
    instrux + 3846,
    instrux + 3847,
    instrux + 3848,
    instrux + 3849,
    NULL
};

static const struct itemplate * const itbl_vex_map2_b9[] = {
    instrux + 3978,
    instrux + 3979,
    instrux + 3980,
    instrux + 3981,
    NULL
};

static const struct itemplate * const itbl_vex_map2_ba[] = {
    instrux + 3890,
    instrux + 3891,
    instrux + 3892,
    instrux + 3893,
    instrux + 3894,
    instrux + 3895,
    instrux + 3896,
    instrux + 3897,
    NULL
};

static const struct itemplate * const itbl_vex_map2_bb[] = {
    instrux + 3990,
    instrux + 3991,
    instrux + 3992,
    instrux + 3993,
    NULL
};

static const struct itemplate * const itbl_vex_map2_bc[] = {
    instrux + 3938,
    instrux + 3939,
    instrux + 3940,
    instrux + 3941,
    instrux + 3942,
    instrux + 3943,
    instrux + 3944,
    instrux + 3945,
    NULL
};

static const struct itemplate * const itbl_vex_map2_bd[] = {
    instrux + 4002,
    instrux + 4003,
    instrux + 4004,
    instrux + 4005,
    NULL
};

static const struct itemplate * const itbl_vex_map2_be[] = {
    instrux + 3962,
    instrux + 3963,
    instrux + 3964,
    instrux + 3965,
    instrux + 3966,
    instrux + 3967,
    instrux + 3968,
    instrux + 3969,
    NULL
};

static const struct itemplate * const itbl_vex_map2_bf[] = {
    instrux + 4014,
    instrux + 4015,
    instrux + 4016,
    instrux + 4017,
    NULL
};

static const struct itemplate * const itbl_vex_map2_cb[] = {
    instrux + 4403,
    NULL
};

static const struct itemplate * const itbl_vex_map2_cc[] = {
    instrux + 4401,
    NULL
};

static const struct itemplate * const itbl_vex_map2_cd[] = {
    instrux + 4402,
    NULL
};

static const struct itemplate * const itbl_vex_map2_cf[] = {
    instrux + 6725,
    instrux + 6726,
    NULL
};

static const struct itemplate * const itbl_vex_map2_d2[] = {
    instrux + 4453,
    instrux + 4454,
    instrux + 4457,
    instrux + 4458,
    instrux + 4461,
    instrux + 4462,
    NULL
};

static const struct itemplate * const itbl_vex_map2_d3[] = {
    instrux + 4455,
    instrux + 4456,
    instrux + 4459,
    instrux + 4460,
    instrux + 4463,
    instrux + 4464,
    NULL
};

static const struct itemplate * const itbl_vex_map2_da[] = {
    instrux + 4404,
    instrux + 4405,
    instrux + 4407,
    instrux + 4408,
    instrux + 4412,
    instrux + 4413,
    NULL
};

static const struct itemplate * const itbl_vex_map2_db[] = {
    instrux + 3085,
    NULL
};

static const struct itemplate * const itbl_vex_map2_dc[] = {
    instrux + 3081,
    instrux + 3098,
    NULL
};

static const struct itemplate * const itbl_vex_map2_dd[] = {
    instrux + 3082,
    instrux + 3099,
    NULL
};

static const struct itemplate * const itbl_vex_map2_de[] = {
    instrux + 3083,
    instrux + 3100,
    NULL
};

static const struct itemplate * const itbl_vex_map2_df[] = {
    instrux + 3084,
    instrux + 3101,
    NULL
};

static const struct itemplate * const itbl_vex_map2_e0[] = {
    instrux + 2563,
    instrux + 2579,
    NULL
};

static const struct itemplate * const itbl_vex_map2_e1[] = {
    instrux + 2560,
    instrux + 2576,
    NULL
};

static const struct itemplate * const itbl_vex_map2_e2[] = {
    instrux + 2553,
    instrux + 2569,
    NULL
};

static const struct itemplate * const itbl_vex_map2_e3[] = {
    instrux + 2557,
    instrux + 2573,
    NULL
};

static const struct itemplate * const itbl_vex_map2_e4[] = {
    instrux + 2567,
    instrux + 2583,
    NULL
};

static const struct itemplate * const itbl_vex_map2_e5[] = {
    instrux + 2562,
    instrux + 2578,
    NULL
};

static const struct itemplate * const itbl_vex_map2_e6[] = {
    instrux + 2556,
    instrux + 2572,
    NULL
};

static const struct itemplate * const itbl_vex_map2_e7[] = {
    instrux + 2552,
    instrux + 2568,
    NULL
};

static const struct itemplate * const itbl_vex_map2_e8[] = {
    instrux + 2566,
    instrux + 2582,
    NULL
};

static const struct itemplate * const itbl_vex_map2_e9[] = {
    instrux + 2561,
    instrux + 2577,
    NULL
};

static const struct itemplate * const itbl_vex_map2_ea[] = {
    instrux + 2564,
    instrux + 2580,
    NULL
};

static const struct itemplate * const itbl_vex_map2_eb[] = {
    instrux + 2565,
    instrux + 2581,
    NULL
};

static const struct itemplate * const itbl_vex_map2_ec[] = {
    instrux + 2555,
    instrux + 2571,
    NULL
};

static const struct itemplate * const itbl_vex_map2_ed[] = {
    instrux + 2559,
    instrux + 2575,
    NULL
};

static const struct itemplate * const itbl_vex_map2_ee[] = {
    instrux + 2558,
    instrux + 2574,
    NULL
};

static const struct itemplate * const itbl_vex_map2_ef[] = {
    instrux + 2554,
    instrux + 2570,
    NULL
};

static const struct itemplate * const itbl_vex_map2_f2[] = {
    instrux + 916,
    instrux + 917,
    NULL
};

static const struct itemplate * const itbl_vex_map2_f3[] = {
    instrux + 924,
    instrux + 925,
    instrux + 928,
    instrux + 929,
    instrux + 932,
    instrux + 933,
    NULL
};

static const struct itemplate * const itbl_vex_map2_f5[] = {
    instrux + 936,
    instrux + 937,
    instrux + 940,
    instrux + 942,
    instrux + 944,
    instrux + 946,
    NULL
};

static const struct itemplate * const itbl_vex_map2_f6[] = {
    instrux + 778,
    instrux + 780,
    NULL
};

static const struct itemplate * const itbl_vex_map2_f7[] = {
    instrux + 510,
    instrux + 511,
    instrux + 512,
    instrux + 514,
    instrux + 515,
    instrux + 516,
    instrux + 920,
    instrux + 921,
    NULL
};

static const struct itemplate * const * const itbl_vex_map2[256] = {
    itbl_vex_map2_00,
    itbl_vex_map2_01,
    itbl_vex_map2_02,
    itbl_vex_map2_03,
    itbl_vex_map2_04,
    itbl_vex_map2_05,
    itbl_vex_map2_06,
    itbl_vex_map2_07,
    itbl_vex_map2_08,
    itbl_vex_map2_09,
    itbl_vex_map2_0a,
    itbl_vex_map2_0b,
    itbl_vex_map2_0c,
    itbl_vex_map2_0d,
    itbl_vex_map2_0e,
    itbl_vex_map2_0f,
    NULL,
    NULL,
    NULL,
    itbl_vex_map2_13,
    NULL,
    NULL,
    itbl_vex_map2_16,
    itbl_vex_map2_17,
    itbl_vex_map2_18,
    itbl_vex_map2_19,
    itbl_vex_map2_1a,
    NULL,
    itbl_vex_map2_1c,
    itbl_vex_map2_1d,
    itbl_vex_map2_1e,
    NULL,
    itbl_vex_map2_20,
    itbl_vex_map2_21,
    itbl_vex_map2_22,
    itbl_vex_map2_23,
    itbl_vex_map2_24,
    itbl_vex_map2_25,
    NULL,
    NULL,
    itbl_vex_map2_28,
    itbl_vex_map2_29,
    itbl_vex_map2_2a,
    itbl_vex_map2_2b,
    itbl_vex_map2_2c,
    itbl_vex_map2_2d,
    itbl_vex_map2_2e,
    itbl_vex_map2_2f,
    itbl_vex_map2_30,
    itbl_vex_map2_31,
    itbl_vex_map2_32,
    itbl_vex_map2_33,
    itbl_vex_map2_34,
    itbl_vex_map2_35,
    itbl_vex_map2_36,
    itbl_vex_map2_37,
    itbl_vex_map2_38,
    itbl_vex_map2_39,
    itbl_vex_map2_3a,
    itbl_vex_map2_3b,
    itbl_vex_map2_3c,
    itbl_vex_map2_3d,
    itbl_vex_map2_3e,
    itbl_vex_map2_3f,
    itbl_vex_map2_40,
    itbl_vex_map2_41,
    NULL,
    NULL,
    NULL,
    itbl_vex_map2_45,
    itbl_vex_map2_46,
    itbl_vex_map2_47,
    itbl_vex_map2_48,
    itbl_vex_map2_49,
    itbl_vex_map2_4a,
    itbl_vex_map2_4b,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map2_50,
    itbl_vex_map2_51,
    itbl_vex_map2_52,
    itbl_vex_map2_53,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map2_58,
    itbl_vex_map2_59,
    itbl_vex_map2_5a,
    NULL,
    itbl_vex_map2_5c,
    NULL,
    itbl_vex_map2_5e,
    itbl_vex_map2_5f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map2_6b,
    itbl_vex_map2_6c,
    NULL,
    itbl_vex_map2_6e,
    itbl_vex_map2_6f,
    NULL,
    NULL,
    itbl_vex_map2_72,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map2_78,
    itbl_vex_map2_79,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map2_8c,
    NULL,
    itbl_vex_map2_8e,
    NULL,
    itbl_vex_map2_90,
    itbl_vex_map2_91,
    itbl_vex_map2_92,
    itbl_vex_map2_93,
    NULL,
    NULL,
    itbl_vex_map2_96,
    itbl_vex_map2_97,
    itbl_vex_map2_98,
    itbl_vex_map2_99,
    itbl_vex_map2_9a,
    itbl_vex_map2_9b,
    itbl_vex_map2_9c,
    itbl_vex_map2_9d,
    itbl_vex_map2_9e,
    itbl_vex_map2_9f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map2_a6,
    itbl_vex_map2_a7,
    itbl_vex_map2_a8,
    itbl_vex_map2_a9,
    itbl_vex_map2_aa,
    itbl_vex_map2_ab,
    itbl_vex_map2_ac,
    itbl_vex_map2_ad,
    itbl_vex_map2_ae,
    itbl_vex_map2_af,
    itbl_vex_map2_b0,
    itbl_vex_map2_b1,
    NULL,
    NULL,
    itbl_vex_map2_b4,
    itbl_vex_map2_b5,
    itbl_vex_map2_b6,
    itbl_vex_map2_b7,
    itbl_vex_map2_b8,
    itbl_vex_map2_b9,
    itbl_vex_map2_ba,
    itbl_vex_map2_bb,
    itbl_vex_map2_bc,
    itbl_vex_map2_bd,
    itbl_vex_map2_be,
    itbl_vex_map2_bf,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map2_cb,
    itbl_vex_map2_cc,
    itbl_vex_map2_cd,
    NULL,
    itbl_vex_map2_cf,
    NULL,
    NULL,
    itbl_vex_map2_d2,
    itbl_vex_map2_d3,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map2_da,
    itbl_vex_map2_db,
    itbl_vex_map2_dc,
    itbl_vex_map2_dd,
    itbl_vex_map2_de,
    itbl_vex_map2_df,
    itbl_vex_map2_e0,
    itbl_vex_map2_e1,
    itbl_vex_map2_e2,
    itbl_vex_map2_e3,
    itbl_vex_map2_e4,
    itbl_vex_map2_e5,
    itbl_vex_map2_e6,
    itbl_vex_map2_e7,
    itbl_vex_map2_e8,
    itbl_vex_map2_e9,
    itbl_vex_map2_ea,
    itbl_vex_map2_eb,
    itbl_vex_map2_ec,
    itbl_vex_map2_ed,
    itbl_vex_map2_ee,
    itbl_vex_map2_ef,
    NULL,
    NULL,
    itbl_vex_map2_f2,
    itbl_vex_map2_f3,
    NULL,
    itbl_vex_map2_f5,
    itbl_vex_map2_f6,
    itbl_vex_map2_f7,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_vex_map3_00[] = {
    instrux + 4326,
    NULL
};

static const struct itemplate * const itbl_vex_map3_01[] = {
    instrux + 4324,
    NULL
};

static const struct itemplate * const itbl_vex_map3_02[] = {
    instrux + 4305,
    instrux + 4306,
    NULL
};

static const struct itemplate * const itbl_vex_map3_04[] = {
    instrux + 3635,
    instrux + 3636,
    NULL
};

static const struct itemplate * const itbl_vex_map3_05[] = {
    instrux + 3631,
    instrux + 3632,
    NULL
};

static const struct itemplate * const itbl_vex_map3_06[] = {
    instrux + 3637,
    NULL
};

static const struct itemplate * const itbl_vex_map3_08[] = {
    instrux + 3755,
    instrux + 3756,
    NULL
};

static const struct itemplate * const itbl_vex_map3_09[] = {
    instrux + 3753,
    instrux + 3754,
    NULL
};

static const struct itemplate * const itbl_vex_map3_0a[] = {
    instrux + 3758,
    NULL
};

static const struct itemplate * const itbl_vex_map3_0b[] = {
    instrux + 3757,
    NULL
};

static const struct itemplate * const itbl_vex_map3_0c[] = {
    instrux + 3134,
    instrux + 3135,
    NULL
};

static const struct itemplate * const itbl_vex_map3_0d[] = {
    instrux + 3132,
    instrux + 3133,
    NULL
};

static const struct itemplate * const itbl_vex_map3_0e[] = {
    instrux + 3616,
    instrux + 4200,
    NULL
};

static const struct itemplate * const itbl_vex_map3_0f[] = {
    instrux + 3610,
    instrux + 4194,
    NULL
};

static const struct itemplate * const itbl_vex_map3_14[] = {
    instrux + 3638,
    instrux + 3639,
    instrux + 3640,
    NULL
};

static const struct itemplate * const itbl_vex_map3_15[] = {
    instrux + 3643,
    instrux + 3644,
    instrux + 3645,
    NULL
};

static const struct itemplate * const itbl_vex_map3_16[] = {
    instrux + 3646,
    instrux + 3647,
    instrux + 3648,
    NULL
};

static const struct itemplate * const itbl_vex_map3_17[] = {
    instrux + 3474,
    NULL
};

static const struct itemplate * const itbl_vex_map3_18[] = {
    instrux + 3483,
    NULL
};

static const struct itemplate * const itbl_vex_map3_19[] = {
    instrux + 3473,
    NULL
};

static const struct itemplate * const itbl_vex_map3_1d[] = {
    instrux + 4020,
    instrux + 4021,
    NULL
};

static const struct itemplate * const itbl_vex_map3_20[] = {
    instrux + 3656,
    instrux + 3657,
    instrux + 3658,
    NULL
};

static const struct itemplate * const itbl_vex_map3_21[] = {
    instrux + 3484,
    NULL
};

static const struct itemplate * const itbl_vex_map3_22[] = {
    instrux + 3662,
    instrux + 3663,
    instrux + 3664,
    instrux + 3665,
    NULL
};

static const struct itemplate * const itbl_vex_map3_30[] = {
    instrux + 4557,
    instrux + 4559,
    NULL
};

static const struct itemplate * const itbl_vex_map3_31[] = {
    instrux + 4561,
    instrux + 4563,
    NULL
};

static const struct itemplate * const itbl_vex_map3_32[] = {
    instrux + 4549,
    instrux + 4551,
    NULL
};

static const struct itemplate * const itbl_vex_map3_33[] = {
    instrux + 4553,
    instrux + 4555,
    NULL
};

static const struct itemplate * const itbl_vex_map3_38[] = {
    instrux + 4329,
    NULL
};

static const struct itemplate * const itbl_vex_map3_39[] = {
    instrux + 4328,
    NULL
};

static const struct itemplate * const itbl_vex_map3_40[] = {
    instrux + 3471,
    instrux + 3472,
    NULL
};

static const struct itemplate * const itbl_vex_map3_41[] = {
    instrux + 3470,
    NULL
};

static const struct itemplate * const itbl_vex_map3_42[] = {
    instrux + 3584,
    instrux + 4178,
    NULL
};

static const struct itemplate * const itbl_vex_map3_44[] = {
    instrux + 3801,
    instrux + 3802,
    instrux + 3803,
    instrux + 3804,
    instrux + 3805,
    instrux + 3806,
    instrux + 3807,
    instrux + 3808,
    instrux + 3809,
    instrux + 3810,
    NULL
};

static const struct itemplate * const itbl_vex_map3_46[] = {
    instrux + 4327,
    NULL
};

static const struct itemplate * const itbl_vex_map3_4a[] = {
    instrux + 3138,
    instrux + 3139,
    NULL
};

static const struct itemplate * const itbl_vex_map3_4b[] = {
    instrux + 3136,
    instrux + 3137,
    NULL
};

static const struct itemplate * const itbl_vex_map3_4c[] = {
    instrux + 3615,
    instrux + 4199,
    NULL
};

static const struct itemplate * const itbl_vex_map3_5c[] = {
    instrux + 4055,
    instrux + 4056,
    instrux + 4057,
    instrux + 4058,
    NULL
};

static const struct itemplate * const itbl_vex_map3_5d[] = {
    instrux + 4051,
    instrux + 4052,
    instrux + 4053,
    instrux + 4054,
    NULL
};

static const struct itemplate * const itbl_vex_map3_5e[] = {
    instrux + 4063,
    instrux + 4064,
    instrux + 4065,
    instrux + 4066,
    NULL
};

static const struct itemplate * const itbl_vex_map3_5f[] = {
    instrux + 4059,
    instrux + 4060,
    instrux + 4061,
    instrux + 4062,
    NULL
};

static const struct itemplate * const itbl_vex_map3_60[] = {
    instrux + 3618,
    NULL
};

static const struct itemplate * const itbl_vex_map3_61[] = {
    instrux + 3617,
    NULL
};

static const struct itemplate * const itbl_vex_map3_62[] = {
    instrux + 3620,
    NULL
};

static const struct itemplate * const itbl_vex_map3_63[] = {
    instrux + 3619,
    NULL
};

static const struct itemplate * const itbl_vex_map3_68[] = {
    instrux + 4043,
    instrux + 4044,
    instrux + 4045,
    instrux + 4046,
    NULL
};

static const struct itemplate * const itbl_vex_map3_69[] = {
    instrux + 4039,
    instrux + 4040,
    instrux + 4041,
    instrux + 4042,
    NULL
};

static const struct itemplate * const itbl_vex_map3_6a[] = {
    instrux + 4049,
    instrux + 4050,
    NULL
};

static const struct itemplate * const itbl_vex_map3_6b[] = {
    instrux + 4047,
    instrux + 4048,
    NULL
};

static const struct itemplate * const itbl_vex_map3_6c[] = {
    instrux + 4071,
    instrux + 4072,
    instrux + 4073,
    instrux + 4074,
    NULL
};

static const struct itemplate * const itbl_vex_map3_6d[] = {
    instrux + 4067,
    instrux + 4068,
    instrux + 4069,
    instrux + 4070,
    NULL
};

static const struct itemplate * const itbl_vex_map3_6e[] = {
    instrux + 4077,
    instrux + 4078,
    NULL
};

static const struct itemplate * const itbl_vex_map3_6f[] = {
    instrux + 4075,
    instrux + 4076,
    NULL
};

static const struct itemplate * const itbl_vex_map3_78[] = {
    instrux + 4083,
    instrux + 4084,
    instrux + 4085,
    instrux + 4086,
    NULL
};

static const struct itemplate * const itbl_vex_map3_79[] = {
    instrux + 4079,
    instrux + 4080,
    instrux + 4081,
    instrux + 4082,
    NULL
};

static const struct itemplate * const itbl_vex_map3_7a[] = {
    instrux + 4089,
    instrux + 4090,
    NULL
};

static const struct itemplate * const itbl_vex_map3_7b[] = {
    instrux + 4087,
    instrux + 4088,
    NULL
};

static const struct itemplate * const itbl_vex_map3_7c[] = {
    instrux + 4095,
    instrux + 4096,
    instrux + 4097,
    instrux + 4098,
    NULL
};

static const struct itemplate * const itbl_vex_map3_7d[] = {
    instrux + 4091,
    instrux + 4092,
    instrux + 4093,
    instrux + 4094,
    NULL
};

static const struct itemplate * const itbl_vex_map3_7e[] = {
    instrux + 4101,
    instrux + 4102,
    NULL
};

static const struct itemplate * const itbl_vex_map3_7f[] = {
    instrux + 4099,
    instrux + 4100,
    NULL
};

static const struct itemplate * const itbl_vex_map3_ce[] = {
    instrux + 6719,
    instrux + 6720,
    NULL
};

static const struct itemplate * const itbl_vex_map3_cf[] = {
    instrux + 6713,
    instrux + 6714,
    NULL
};

static const struct itemplate * const itbl_vex_map3_de[] = {
    instrux + 4406,
    NULL
};

static const struct itemplate * const itbl_vex_map3_df[] = {
    instrux + 3086,
    NULL
};

static const struct itemplate * const itbl_vex_map3_f0[] = {
    instrux + 509,
    instrux + 513,
    NULL
};

static const struct itemplate * const * const itbl_vex_map3[256] = {
    itbl_vex_map3_00,
    itbl_vex_map3_01,
    itbl_vex_map3_02,
    NULL,
    itbl_vex_map3_04,
    itbl_vex_map3_05,
    itbl_vex_map3_06,
    NULL,
    itbl_vex_map3_08,
    itbl_vex_map3_09,
    itbl_vex_map3_0a,
    itbl_vex_map3_0b,
    itbl_vex_map3_0c,
    itbl_vex_map3_0d,
    itbl_vex_map3_0e,
    itbl_vex_map3_0f,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map3_14,
    itbl_vex_map3_15,
    itbl_vex_map3_16,
    itbl_vex_map3_17,
    itbl_vex_map3_18,
    itbl_vex_map3_19,
    NULL,
    NULL,
    NULL,
    itbl_vex_map3_1d,
    NULL,
    NULL,
    itbl_vex_map3_20,
    itbl_vex_map3_21,
    itbl_vex_map3_22,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map3_30,
    itbl_vex_map3_31,
    itbl_vex_map3_32,
    itbl_vex_map3_33,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map3_38,
    itbl_vex_map3_39,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map3_40,
    itbl_vex_map3_41,
    itbl_vex_map3_42,
    NULL,
    itbl_vex_map3_44,
    NULL,
    itbl_vex_map3_46,
    NULL,
    NULL,
    NULL,
    itbl_vex_map3_4a,
    itbl_vex_map3_4b,
    itbl_vex_map3_4c,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map3_5c,
    itbl_vex_map3_5d,
    itbl_vex_map3_5e,
    itbl_vex_map3_5f,
    itbl_vex_map3_60,
    itbl_vex_map3_61,
    itbl_vex_map3_62,
    itbl_vex_map3_63,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map3_68,
    itbl_vex_map3_69,
    itbl_vex_map3_6a,
    itbl_vex_map3_6b,
    itbl_vex_map3_6c,
    itbl_vex_map3_6d,
    itbl_vex_map3_6e,
    itbl_vex_map3_6f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map3_78,
    itbl_vex_map3_79,
    itbl_vex_map3_7a,
    itbl_vex_map3_7b,
    itbl_vex_map3_7c,
    itbl_vex_map3_7d,
    itbl_vex_map3_7e,
    itbl_vex_map3_7f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map3_ce,
    itbl_vex_map3_cf,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map3_de,
    itbl_vex_map3_df,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map3_f0,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_vex_map5_f8[] = {
    instrux + 6892,
    instrux + 6894,
    NULL
};

static const struct itemplate * const itbl_vex_map5_f9[] = {
    instrux + 6893,
    instrux + 6895,
    NULL
};

static const struct itemplate * const itbl_vex_map5_fd[] = {
    instrux + 6908,
    instrux + 6909,
    instrux + 6910,
    instrux + 6911,
    NULL
};

static const struct itemplate * const * const itbl_vex_map5[256] = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map5_f8,
    itbl_vex_map5_f9,
    NULL,
    NULL,
    NULL,
    itbl_vex_map5_fd,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_vex_map7_f6[] = {
    instrux + 1328,
    instrux + 1331,
    NULL
};

static const struct itemplate * const itbl_vex_map7_f8[] = {
    instrux + 1337,
    instrux + 1341,
    NULL
};

static const struct itemplate * const * const itbl_vex_map7[256] = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map7_f6,
    NULL,
    itbl_vex_map7_f8,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_xop_map8_85[] = {
    instrux + 4143,
    NULL
};

static const struct itemplate * const itbl_xop_map8_86[] = {
    instrux + 4142,
    NULL
};

static const struct itemplate * const itbl_xop_map8_87[] = {
    instrux + 4141,
    NULL
};

static const struct itemplate * const itbl_xop_map8_8e[] = {
    instrux + 4139,
    NULL
};

static const struct itemplate * const itbl_xop_map8_8f[] = {
    instrux + 4140,
    NULL
};

static const struct itemplate * const itbl_xop_map8_95[] = {
    instrux + 4145,
    NULL
};

static const struct itemplate * const itbl_xop_map8_96[] = {
    instrux + 4144,
    NULL
};

static const struct itemplate * const itbl_xop_map8_97[] = {
    instrux + 4138,
    NULL
};

static const struct itemplate * const itbl_xop_map8_9e[] = {
    instrux + 4136,
    NULL
};

static const struct itemplate * const itbl_xop_map8_9f[] = {
    instrux + 4137,
    NULL
};

static const struct itemplate * const itbl_xop_map8_a2[] = {
    instrux + 4109,
    instrux + 4110,
    instrux + 4111,
    instrux + 4112,
    NULL
};

static const struct itemplate * const itbl_xop_map8_a3[] = {
    instrux + 4148,
    instrux + 4149,
    NULL
};

static const struct itemplate * const itbl_xop_map8_a6[] = {
    instrux + 4146,
    NULL
};

static const struct itemplate * const itbl_xop_map8_b6[] = {
    instrux + 4147,
    NULL
};

static const struct itemplate * const itbl_xop_map8_c0[] = {
    instrux + 4152,
    NULL
};

static const struct itemplate * const itbl_xop_map8_c1[] = {
    instrux + 4161,
    NULL
};

static const struct itemplate * const itbl_xop_map8_c2[] = {
    instrux + 4155,
    NULL
};

static const struct itemplate * const itbl_xop_map8_c3[] = {
    instrux + 4158,
    NULL
};

static const struct itemplate * const itbl_xop_map8_cc[] = {
    instrux + 4113,
    NULL
};

static const struct itemplate * const itbl_xop_map8_cd[] = {
    instrux + 4120,
    NULL
};

static const struct itemplate * const itbl_xop_map8_ce[] = {
    instrux + 4114,
    NULL
};

static const struct itemplate * const itbl_xop_map8_cf[] = {
    instrux + 4115,
    NULL
};

static const struct itemplate * const itbl_xop_map8_ec[] = {
    instrux + 4116,
    NULL
};

static const struct itemplate * const itbl_xop_map8_ed[] = {
    instrux + 4119,
    NULL
};

static const struct itemplate * const itbl_xop_map8_ee[] = {
    instrux + 4117,
    NULL
};

static const struct itemplate * const itbl_xop_map8_ef[] = {
    instrux + 4118,
    NULL
};

static const struct itemplate * const * const itbl_xop_map8[256] = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map8_85,
    itbl_xop_map8_86,
    itbl_xop_map8_87,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map8_8e,
    itbl_xop_map8_8f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map8_95,
    itbl_xop_map8_96,
    itbl_xop_map8_97,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map8_9e,
    itbl_xop_map8_9f,
    NULL,
    NULL,
    itbl_xop_map8_a2,
    itbl_xop_map8_a3,
    NULL,
    NULL,
    itbl_xop_map8_a6,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map8_b6,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map8_c0,
    itbl_xop_map8_c1,
    itbl_xop_map8_c2,
    itbl_xop_map8_c3,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map8_cc,
    itbl_xop_map8_cd,
    itbl_xop_map8_ce,
    itbl_xop_map8_cf,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map8_ec,
    itbl_xop_map8_ed,
    itbl_xop_map8_ee,
    itbl_xop_map8_ef,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_xop_map9_01[] = {
    instrux + 952,
    instrux + 953,
    instrux + 954,
    instrux + 955,
    instrux + 956,
    instrux + 957,
    instrux + 958,
    instrux + 959,
    instrux + 962,
    instrux + 963,
    instrux + 964,
    instrux + 965,
    instrux + 966,
    instrux + 967,
    NULL
};

static const struct itemplate * const itbl_xop_map9_02[] = {
    instrux + 950,
    instrux + 951,
    instrux + 960,
    instrux + 961,
    NULL
};

static const struct itemplate * const itbl_xop_map9_12[] = {
    instrux + 4031,
    instrux + 4032,
    instrux + 4033,
    instrux + 4034,
    NULL
};

static const struct itemplate * const itbl_xop_map9_80[] = {
    instrux + 4105,
    instrux + 4106,
    NULL
};

static const struct itemplate * const itbl_xop_map9_81[] = {
    instrux + 4103,
    instrux + 4104,
    NULL
};

static const struct itemplate * const itbl_xop_map9_82[] = {
    instrux + 4108,
    NULL
};

static const struct itemplate * const itbl_xop_map9_83[] = {
    instrux + 4107,
    NULL
};

static const struct itemplate * const itbl_xop_map9_90[] = {
    instrux + 4150,
    instrux + 4151,
    NULL
};

static const struct itemplate * const itbl_xop_map9_91[] = {
    instrux + 4159,
    instrux + 4160,
    NULL
};

static const struct itemplate * const itbl_xop_map9_92[] = {
    instrux + 4153,
    instrux + 4154,
    NULL
};

static const struct itemplate * const itbl_xop_map9_93[] = {
    instrux + 4156,
    instrux + 4157,
    NULL
};

static const struct itemplate * const itbl_xop_map9_94[] = {
    instrux + 4170,
    instrux + 4171,
    NULL
};

static const struct itemplate * const itbl_xop_map9_95[] = {
    instrux + 4176,
    instrux + 4177,
    NULL
};

static const struct itemplate * const itbl_xop_map9_96[] = {
    instrux + 4172,
    instrux + 4173,
    NULL
};

static const struct itemplate * const itbl_xop_map9_97[] = {
    instrux + 4174,
    instrux + 4175,
    NULL
};

static const struct itemplate * const itbl_xop_map9_98[] = {
    instrux + 4162,
    instrux + 4163,
    NULL
};

static const struct itemplate * const itbl_xop_map9_99[] = {
    instrux + 4168,
    instrux + 4169,
    NULL
};

static const struct itemplate * const itbl_xop_map9_9a[] = {
    instrux + 4164,
    instrux + 4165,
    NULL
};

static const struct itemplate * const itbl_xop_map9_9b[] = {
    instrux + 4166,
    instrux + 4167,
    NULL
};

static const struct itemplate * const itbl_xop_map9_c1[] = {
    instrux + 4123,
    NULL
};

static const struct itemplate * const itbl_xop_map9_c2[] = {
    instrux + 4121,
    NULL
};

static const struct itemplate * const itbl_xop_map9_c3[] = {
    instrux + 4122,
    NULL
};

static const struct itemplate * const itbl_xop_map9_c6[] = {
    instrux + 4131,
    NULL
};

static const struct itemplate * const itbl_xop_map9_c7[] = {
    instrux + 4132,
    NULL
};

static const struct itemplate * const itbl_xop_map9_cb[] = {
    instrux + 4124,
    NULL
};

static const struct itemplate * const itbl_xop_map9_d1[] = {
    instrux + 4127,
    NULL
};

static const struct itemplate * const itbl_xop_map9_d2[] = {
    instrux + 4125,
    NULL
};

static const struct itemplate * const itbl_xop_map9_d3[] = {
    instrux + 4126,
    NULL
};

static const struct itemplate * const itbl_xop_map9_d6[] = {
    instrux + 4129,
    NULL
};

static const struct itemplate * const itbl_xop_map9_d7[] = {
    instrux + 4130,
    NULL
};

static const struct itemplate * const itbl_xop_map9_db[] = {
    instrux + 4128,
    NULL
};

static const struct itemplate * const itbl_xop_map9_e1[] = {
    instrux + 4133,
    NULL
};

static const struct itemplate * const itbl_xop_map9_e2[] = {
    instrux + 4135,
    NULL
};

static const struct itemplate * const itbl_xop_map9_e3[] = {
    instrux + 4134,
    NULL
};

static const struct itemplate * const * const itbl_xop_map9[256] = {
    NULL,
    itbl_xop_map9_01,
    itbl_xop_map9_02,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map9_12,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map9_80,
    itbl_xop_map9_81,
    itbl_xop_map9_82,
    itbl_xop_map9_83,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map9_90,
    itbl_xop_map9_91,
    itbl_xop_map9_92,
    itbl_xop_map9_93,
    itbl_xop_map9_94,
    itbl_xop_map9_95,
    itbl_xop_map9_96,
    itbl_xop_map9_97,
    itbl_xop_map9_98,
    itbl_xop_map9_99,
    itbl_xop_map9_9a,
    itbl_xop_map9_9b,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map9_c1,
    itbl_xop_map9_c2,
    itbl_xop_map9_c3,
    NULL,
    NULL,
    itbl_xop_map9_c6,
    itbl_xop_map9_c7,
    NULL,
    NULL,
    NULL,
    itbl_xop_map9_cb,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map9_d1,
    itbl_xop_map9_d2,
    itbl_xop_map9_d3,
    NULL,
    NULL,
    itbl_xop_map9_d6,
    itbl_xop_map9_d7,
    NULL,
    NULL,
    NULL,
    itbl_xop_map9_db,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map9_e1,
    itbl_xop_map9_e2,
    itbl_xop_map9_e3,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_xop_map10_10[] = {
    instrux + 948,
    instrux + 949,
    NULL
};

static const struct itemplate * const itbl_xop_map10_12[] = {
    instrux + 4035,
    instrux + 4036,
    instrux + 4037,
    instrux + 4038,
    NULL
};

static const struct itemplate * const * const itbl_xop_map10[256] = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map10_10,
    NULL,
    itbl_xop_map10_12,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_evex_map1_10[] = {
    instrux + 5507,
    instrux + 5509,
    instrux + 5517,
    instrux + 5519,
    instrux + 5521,
    instrux + 5522,
    instrux + 5523,
    instrux + 5530,
    instrux + 5531,
    instrux + 5532,
    NULL
};

static const struct itemplate * const itbl_evex_map1_11[] = {
    instrux + 5508,
    instrux + 5510,
    instrux + 5518,
    instrux + 5520,
    instrux + 5524,
    instrux + 5525,
    instrux + 5526,
    instrux + 5527,
    instrux + 5528,
    instrux + 5529,
    instrux + 5533,
    instrux + 5534,
    instrux + 5535,
    instrux + 5536,
    instrux + 5537,
    instrux + 5538,
    NULL
};

static const struct itemplate * const itbl_evex_map1_12[] = {
    instrux + 5442,
    instrux + 5443,
    instrux + 5444,
    instrux + 5481,
    instrux + 5487,
    instrux + 5489,
    instrux + 5514,
    instrux + 5515,
    instrux + 5516,
    NULL
};

static const struct itemplate * const itbl_evex_map1_13[] = {
    instrux + 5488,
    instrux + 5490,
    NULL
};

static const struct itemplate * const itbl_evex_map1_14[] = {
    instrux + 6684,
    instrux + 6685,
    instrux + 6686,
    instrux + 6687,
    instrux + 6688,
    instrux + 6689,
    NULL
};

static const struct itemplate * const itbl_evex_map1_15[] = {
    instrux + 6678,
    instrux + 6679,
    instrux + 6680,
    instrux + 6681,
    instrux + 6682,
    instrux + 6683,
    NULL
};

static const struct itemplate * const itbl_evex_map1_16[] = {
    instrux + 5482,
    instrux + 5484,
    instrux + 5486,
    instrux + 5511,
    instrux + 5512,
    instrux + 5513,
    NULL
};

static const struct itemplate * const itbl_evex_map1_17[] = {
    instrux + 5483,
    instrux + 5485,
    NULL
};

static const struct itemplate * const itbl_evex_map1_28[] = {
    instrux + 5422,
    instrux + 5423,
    instrux + 5424,
    instrux + 5431,
    instrux + 5432,
    instrux + 5433,
    NULL
};

static const struct itemplate * const itbl_evex_map1_29[] = {
    instrux + 5425,
    instrux + 5426,
    instrux + 5427,
    instrux + 5428,
    instrux + 5429,
    instrux + 5430,
    instrux + 5434,
    instrux + 5435,
    instrux + 5436,
    instrux + 5437,
    instrux + 5438,
    instrux + 5439,
    NULL
};

static const struct itemplate * const itbl_evex_map1_2a[] = {
    instrux + 5100,
    instrux + 5101,
    instrux + 5102,
    instrux + 5103,
    NULL
};

static const struct itemplate * const itbl_evex_map1_2b[] = {
    instrux + 5497,
    instrux + 5498,
    instrux + 5499,
    instrux + 5500,
    instrux + 5501,
    instrux + 5502,
    NULL
};

static const struct itemplate * const itbl_evex_map1_2c[] = {
    instrux + 5133,
    instrux + 5134,
    instrux + 5137,
    instrux + 5138,
    NULL
};

static const struct itemplate * const itbl_evex_map1_2d[] = {
    instrux + 5095,
    instrux + 5096,
    instrux + 5105,
    instrux + 5106,
    NULL
};

static const struct itemplate * const itbl_evex_map1_2e[] = {
    instrux + 6676,
    instrux + 6677,
    instrux + 7276,
    instrux + 7278,
    NULL
};

static const struct itemplate * const itbl_evex_map1_2f[] = {
    instrux + 5030,
    instrux + 5031,
    instrux + 7273,
    instrux + 7275,
    NULL
};

static const struct itemplate * const itbl_evex_map1_41[] = {
    instrux + 4510,
    instrux + 4512,
    instrux + 4514,
    instrux + 4516,
    NULL
};

static const struct itemplate * const itbl_evex_map1_42[] = {
    instrux + 4518,
    instrux + 4520,
    instrux + 4522,
    instrux + 4524,
    NULL
};

static const struct itemplate * const itbl_evex_map1_44[] = {
    instrux + 4526,
    instrux + 4528,
    instrux + 4530,
    instrux + 4532,
    NULL
};

static const struct itemplate * const itbl_evex_map1_45[] = {
    instrux + 4534,
    instrux + 4536,
    instrux + 4538,
    instrux + 4540,
    NULL
};

static const struct itemplate * const itbl_evex_map1_46[] = {
    instrux + 4580,
    instrux + 4582,
    instrux + 4584,
    instrux + 4586,
    NULL
};

static const struct itemplate * const itbl_evex_map1_47[] = {
    instrux + 4588,
    instrux + 4590,
    instrux + 4592,
    instrux + 4594,
    NULL
};

static const struct itemplate * const itbl_evex_map1_4a[] = {
    instrux + 4502,
    instrux + 4504,
    instrux + 4506,
    instrux + 4508,
    NULL
};

static const struct itemplate * const itbl_evex_map1_4b[] = {
    instrux + 4574,
    instrux + 4576,
    instrux + 4578,
    NULL
};

static const struct itemplate * const itbl_evex_map1_51[] = {
    instrux + 6660,
    instrux + 6661,
    instrux + 6662,
    instrux + 6663,
    instrux + 6664,
    instrux + 6665,
    instrux + 6666,
    instrux + 6667,
    NULL
};

static const struct itemplate * const itbl_evex_map1_54[] = {
    instrux + 4615,
    instrux + 4616,
    instrux + 4617,
    instrux + 4618,
    instrux + 4619,
    instrux + 4620,
    NULL
};

static const struct itemplate * const itbl_evex_map1_55[] = {
    instrux + 4609,
    instrux + 4610,
    instrux + 4611,
    instrux + 4612,
    instrux + 4613,
    instrux + 4614,
    NULL
};

static const struct itemplate * const itbl_evex_map1_56[] = {
    instrux + 5547,
    instrux + 5548,
    instrux + 5549,
    instrux + 5550,
    instrux + 5551,
    instrux + 5552,
    NULL
};

static const struct itemplate * const itbl_evex_map1_57[] = {
    instrux + 6690,
    instrux + 6691,
    instrux + 6692,
    instrux + 6693,
    instrux + 6694,
    instrux + 6695,
    NULL
};

static const struct itemplate * const itbl_evex_map1_58[] = {
    instrux + 4595,
    instrux + 4596,
    instrux + 4597,
    instrux + 4598,
    instrux + 4599,
    instrux + 4600,
    instrux + 4601,
    instrux + 4602,
    NULL
};

static const struct itemplate * const itbl_evex_map1_59[] = {
    instrux + 5539,
    instrux + 5540,
    instrux + 5541,
    instrux + 5542,
    instrux + 5543,
    instrux + 5544,
    instrux + 5545,
    instrux + 5546,
    NULL
};

static const struct itemplate * const itbl_evex_map1_5a[] = {
    instrux + 5053,
    instrux + 5054,
    instrux + 5055,
    instrux + 5071,
    instrux + 5072,
    instrux + 5073,
    instrux + 5097,
    instrux + 5104,
    NULL
};

static const struct itemplate * const itbl_evex_map1_5b[] = {
    instrux + 5047,
    instrux + 5048,
    instrux + 5049,
    instrux + 5068,
    instrux + 5069,
    instrux + 5070,
    instrux + 5092,
    instrux + 5093,
    instrux + 5094,
    instrux + 5121,
    instrux + 5122,
    instrux + 5123,
    NULL
};

static const struct itemplate * const itbl_evex_map1_5c[] = {
    instrux + 6668,
    instrux + 6669,
    instrux + 6670,
    instrux + 6671,
    instrux + 6672,
    instrux + 6673,
    instrux + 6674,
    instrux + 6675,
    NULL
};

static const struct itemplate * const itbl_evex_map1_5d[] = {
    instrux + 5414,
    instrux + 5415,
    instrux + 5416,
    instrux + 5417,
    instrux + 5418,
    instrux + 5419,
    instrux + 5420,
    instrux + 5421,
    NULL
};

static const struct itemplate * const itbl_evex_map1_5e[] = {
    instrux + 5160,
    instrux + 5161,
    instrux + 5162,
    instrux + 5163,
    instrux + 5164,
    instrux + 5165,
    instrux + 5166,
    instrux + 5167,
    NULL
};

static const struct itemplate * const itbl_evex_map1_5f[] = {
    instrux + 5406,
    instrux + 5407,
    instrux + 5408,
    instrux + 5409,
    instrux + 5410,
    instrux + 5411,
    instrux + 5412,
    instrux + 5413,
    NULL
};

static const struct itemplate * const itbl_evex_map1_60[] = {
    instrux + 6548,
    instrux + 6549,
    instrux + 6550,
    NULL
};

static const struct itemplate * const itbl_evex_map1_61[] = {
    instrux + 6557,
    instrux + 6558,
    instrux + 6559,
    NULL
};

static const struct itemplate * const itbl_evex_map1_62[] = {
    instrux + 6551,
    instrux + 6552,
    instrux + 6553,
    NULL
};

static const struct itemplate * const itbl_evex_map1_63[] = {
    instrux + 5568,
    instrux + 5569,
    instrux + 5570,
    NULL
};

static const struct itemplate * const itbl_evex_map1_64[] = {
    instrux + 5697,
    instrux + 5698,
    instrux + 5699,
    NULL
};

static const struct itemplate * const itbl_evex_map1_65[] = {
    instrux + 5706,
    instrux + 5707,
    instrux + 5708,
    NULL
};

static const struct itemplate * const itbl_evex_map1_66[] = {
    instrux + 5700,
    instrux + 5701,
    instrux + 5702,
    NULL
};

static const struct itemplate * const itbl_evex_map1_67[] = {
    instrux + 5574,
    instrux + 5575,
    instrux + 5576,
    NULL
};

static const struct itemplate * const itbl_evex_map1_68[] = {
    instrux + 6536,
    instrux + 6537,
    instrux + 6538,
    NULL
};

static const struct itemplate * const itbl_evex_map1_69[] = {
    instrux + 6545,
    instrux + 6546,
    instrux + 6547,
    NULL
};

static const struct itemplate * const itbl_evex_map1_6a[] = {
    instrux + 6539,
    instrux + 6540,
    instrux + 6541,
    NULL
};

static const struct itemplate * const itbl_evex_map1_6b[] = {
    instrux + 5565,
    instrux + 5566,
    instrux + 5567,
    NULL
};

static const struct itemplate * const itbl_evex_map1_6c[] = {
    instrux + 6554,
    instrux + 6555,
    instrux + 6556,
    NULL
};

static const struct itemplate * const itbl_evex_map1_6d[] = {
    instrux + 6542,
    instrux + 6543,
    instrux + 6544,
    NULL
};

static const struct itemplate * const itbl_evex_map1_6e[] = {
    instrux + 5440,
    instrux + 5503,
    NULL
};

static const struct itemplate * const itbl_evex_map1_6f[] = {
    instrux + 5445,
    instrux + 5446,
    instrux + 5447,
    instrux + 5451,
    instrux + 5452,
    instrux + 5453,
    instrux + 5457,
    instrux + 5458,
    instrux + 5459,
    instrux + 5463,
    instrux + 5464,
    instrux + 5465,
    instrux + 5469,
    instrux + 5470,
    instrux + 5471,
    instrux + 5475,
    instrux + 5476,
    instrux + 5477,
    NULL
};

static const struct itemplate * const itbl_evex_map1_70[] = {
    instrux + 6386,
    instrux + 6387,
    instrux + 6388,
    instrux + 6389,
    instrux + 6390,
    instrux + 6391,
    instrux + 6392,
    instrux + 6393,
    instrux + 6394,
    NULL
};

static const struct itemplate * const itbl_evex_map1_71[] = {
    instrux + 6422,
    instrux + 6423,
    instrux + 6424,
    instrux + 6449,
    instrux + 6450,
    instrux + 6451,
    instrux + 6479,
    instrux + 6480,
    instrux + 6481,
    NULL
};

static const struct itemplate * const itbl_evex_map1_72[] = {
    instrux + 6344,
    instrux + 6345,
    instrux + 6346,
    instrux + 6347,
    instrux + 6348,
    instrux + 6349,
    instrux + 6356,
    instrux + 6357,
    instrux + 6358,
    instrux + 6359,
    instrux + 6360,
    instrux + 6361,
    instrux + 6398,
    instrux + 6399,
    instrux + 6400,
    instrux + 6428,
    instrux + 6429,
    instrux + 6430,
    instrux + 6434,
    instrux + 6435,
    instrux + 6436,
    instrux + 6455,
    instrux + 6456,
    instrux + 6457,
    NULL
};

static const struct itemplate * const itbl_evex_map1_73[] = {
    instrux + 6401,
    instrux + 6402,
    instrux + 6403,
    instrux + 6407,
    instrux + 6408,
    instrux + 6409,
    instrux + 6458,
    instrux + 6459,
    instrux + 6460,
    instrux + 6464,
    instrux + 6465,
    instrux + 6466,
    NULL
};

static const struct itemplate * const itbl_evex_map1_74[] = {
    instrux + 5685,
    instrux + 5686,
    instrux + 5687,
    NULL
};

static const struct itemplate * const itbl_evex_map1_75[] = {
    instrux + 5694,
    instrux + 5695,
    instrux + 5696,
    NULL
};

static const struct itemplate * const itbl_evex_map1_76[] = {
    instrux + 5688,
    instrux + 5689,
    instrux + 5690,
    NULL
};

static const struct itemplate * const itbl_evex_map1_78[] = {
    instrux + 5115,
    instrux + 5116,
    instrux + 5117,
    instrux + 5118,
    instrux + 5119,
    instrux + 5120,
    instrux + 5127,
    instrux + 5128,
    instrux + 5129,
    instrux + 5130,
    instrux + 5131,
    instrux + 5132,
    instrux + 5135,
    instrux + 5136,
    instrux + 5139,
    instrux + 5140,
    NULL
};

static const struct itemplate * const itbl_evex_map1_79[] = {
    instrux + 5059,
    instrux + 5060,
    instrux + 5061,
    instrux + 5062,
    instrux + 5063,
    instrux + 5064,
    instrux + 5083,
    instrux + 5084,
    instrux + 5085,
    instrux + 5086,
    instrux + 5087,
    instrux + 5088,
    instrux + 5098,
    instrux + 5099,
    instrux + 5107,
    instrux + 5108,
    NULL
};

static const struct itemplate * const itbl_evex_map1_7a[] = {
    instrux + 5112,
    instrux + 5113,
    instrux + 5114,
    instrux + 5124,
    instrux + 5125,
    instrux + 5126,
    instrux + 5141,
    instrux + 5142,
    instrux + 5143,
    instrux + 5144,
    instrux + 5145,
    instrux + 5146,
    instrux + 5147,
    instrux + 5148,
    instrux + 5149,
    instrux + 5150,
    instrux + 5151,
    instrux + 5152,
    NULL
};

static const struct itemplate * const itbl_evex_map1_7b[] = {
    instrux + 5056,
    instrux + 5057,
    instrux + 5058,
    instrux + 5080,
    instrux + 5081,
    instrux + 5082,
    instrux + 5153,
    instrux + 5154,
    instrux + 5155,
    instrux + 5156,
    NULL
};

static const struct itemplate * const itbl_evex_map1_7e[] = {
    instrux + 5441,
    instrux + 5504,
    instrux + 5505,
    instrux + 7446,
    NULL
};

static const struct itemplate * const itbl_evex_map1_7f[] = {
    instrux + 5448,
    instrux + 5449,
    instrux + 5450,
    instrux + 5454,
    instrux + 5455,
    instrux + 5456,
    instrux + 5460,
    instrux + 5461,
    instrux + 5462,
    instrux + 5466,
    instrux + 5467,
    instrux + 5468,
    instrux + 5472,
    instrux + 5473,
    instrux + 5474,
    instrux + 5478,
    instrux + 5479,
    instrux + 5480,
    NULL
};

static const struct itemplate * const itbl_evex_map1_90[] = {
    instrux + 4470,
    instrux + 4472,
    instrux + 4474,
    instrux + 4476,
    NULL
};

static const struct itemplate * const itbl_evex_map1_91[] = {
    instrux + 4478,
    instrux + 4480,
    instrux + 4482,
    instrux + 4484,
    NULL
};

static const struct itemplate * const itbl_evex_map1_92[] = {
    instrux + 4486,
    instrux + 4488,
    instrux + 4494,
    instrux + 4496,
    NULL
};

static const struct itemplate * const itbl_evex_map1_93[] = {
    instrux + 4490,
    instrux + 4492,
    instrux + 4498,
    instrux + 4500,
    NULL
};

static const struct itemplate * const itbl_evex_map1_98[] = {
    instrux + 4542,
    instrux + 4544,
    instrux + 4546,
    instrux + 4548,
    NULL
};

static const struct itemplate * const itbl_evex_map1_99[] = {
    instrux + 4566,
    instrux + 4568,
    instrux + 4570,
    instrux + 4572,
    NULL
};

static const struct itemplate * const itbl_evex_map1_c2[] = {
    instrux + 4654,
    instrux + 4655,
    instrux + 4656,
    instrux + 4657,
    instrux + 4658,
    instrux + 4659,
    instrux + 4660,
    instrux + 4661,
    instrux + 4662,
    instrux + 4663,
    instrux + 4664,
    instrux + 4665,
    instrux + 4666,
    instrux + 4667,
    instrux + 4668,
    instrux + 4669,
    instrux + 4670,
    instrux + 4671,
    instrux + 4672,
    instrux + 4673,
    instrux + 4674,
    instrux + 4675,
    instrux + 4676,
    instrux + 4677,
    instrux + 4678,
    instrux + 4679,
    instrux + 4680,
    instrux + 4681,
    instrux + 4682,
    instrux + 4683,
    instrux + 4684,
    instrux + 4685,
    instrux + 4686,
    instrux + 4687,
    instrux + 4688,
    instrux + 4689,
    instrux + 4690,
    instrux + 4691,
    instrux + 4692,
    instrux + 4693,
    instrux + 4694,
    instrux + 4695,
    instrux + 4696,
    instrux + 4697,
    instrux + 4698,
    instrux + 4699,
    instrux + 4700,
    instrux + 4701,
    instrux + 4702,
    instrux + 4703,
    instrux + 4704,
    instrux + 4705,
    instrux + 4706,
    instrux + 4707,
    instrux + 4708,
    instrux + 4709,
    instrux + 4710,
    instrux + 4711,
    instrux + 4712,
    instrux + 4713,
    instrux + 4714,
    instrux + 4715,
    instrux + 4716,
    instrux + 4717,
    instrux + 4718,
    instrux + 4719,
    instrux + 4720,
    instrux + 4721,
    instrux + 4722,
    instrux + 4723,
    instrux + 4724,
    instrux + 4725,
    instrux + 4726,
    instrux + 4727,
    instrux + 4728,
    instrux + 4729,
    instrux + 4730,
    instrux + 4731,
    instrux + 4732,
    instrux + 4733,
    instrux + 4734,
    instrux + 4735,
    instrux + 4736,
    instrux + 4737,
    instrux + 4738,
    instrux + 4739,
    instrux + 4740,
    instrux + 4741,
    instrux + 4742,
    instrux + 4743,
    instrux + 4744,
    instrux + 4745,
    instrux + 4746,
    instrux + 4747,
    instrux + 4748,
    instrux + 4749,
    instrux + 4750,
    instrux + 4751,
    instrux + 4752,
    instrux + 4753,
    instrux + 4754,
    instrux + 4755,
    instrux + 4756,
    instrux + 4757,
    instrux + 4758,
    instrux + 4759,
    instrux + 4760,
    instrux + 4761,
    instrux + 4762,
    instrux + 4763,
    instrux + 4764,
    instrux + 4765,
    instrux + 4766,
    instrux + 4767,
    instrux + 4768,
    instrux + 4769,
    instrux + 4770,
    instrux + 4771,
    instrux + 4772,
    instrux + 4773,
    instrux + 4774,
    instrux + 4775,
    instrux + 4776,
    instrux + 4777,
    instrux + 4778,
    instrux + 4779,
    instrux + 4780,
    instrux + 4781,
    instrux + 4782,
    instrux + 4783,
    instrux + 4784,
    instrux + 4785,
    instrux + 4786,
    instrux + 4787,
    instrux + 4788,
    instrux + 4789,
    instrux + 4790,
    instrux + 4791,
    instrux + 4792,
    instrux + 4793,
    instrux + 4794,
    instrux + 4795,
    instrux + 4796,
    instrux + 4797,
    instrux + 4798,
    instrux + 4799,
    instrux + 4800,
    instrux + 4801,
    instrux + 4802,
    instrux + 4803,
    instrux + 4804,
    instrux + 4805,
    instrux + 4806,
    instrux + 4807,
    instrux + 4808,
    instrux + 4809,
    instrux + 4810,
    instrux + 4811,
    instrux + 4812,
    instrux + 4813,
    instrux + 4814,
    instrux + 4815,
    instrux + 4816,
    instrux + 4817,
    instrux + 4818,
    instrux + 4819,
    instrux + 4820,
    instrux + 4821,
    instrux + 4822,
    instrux + 4823,
    instrux + 4824,
    instrux + 4825,
    instrux + 4826,
    instrux + 4827,
    instrux + 4828,
    instrux + 4829,
    instrux + 4830,
    instrux + 4831,
    instrux + 4832,
    instrux + 4833,
    instrux + 4834,
    instrux + 4835,
    instrux + 4836,
    instrux + 4837,
    instrux + 4838,
    instrux + 4839,
    instrux + 4840,
    instrux + 4841,
    instrux + 4842,
    instrux + 4843,
    instrux + 4844,
    instrux + 4845,
    instrux + 4846,
    instrux + 4847,
    instrux + 4848,
    instrux + 4849,
    instrux + 4850,
    instrux + 4851,
    instrux + 4852,
    instrux + 4853,
    instrux + 4854,
    instrux + 4855,
    instrux + 4856,
    instrux + 4857,
    instrux + 4858,
    instrux + 4859,
    instrux + 4860,
    instrux + 4861,
    instrux + 4862,
    instrux + 4863,
    instrux + 4864,
    instrux + 4865,
    instrux + 4866,
    instrux + 4867,
    instrux + 4868,
    instrux + 4869,
    instrux + 4870,
    instrux + 4871,
    instrux + 4872,
    instrux + 4873,
    instrux + 4874,
    instrux + 4875,
    instrux + 4876,
    instrux + 4877,
    instrux + 4878,
    instrux + 4879,
    instrux + 4880,
    instrux + 4881,
    instrux + 4882,
    instrux + 4883,
    instrux + 4884,
    instrux + 4885,
    instrux + 4886,
    instrux + 4887,
    instrux + 4888,
    instrux + 4889,
    instrux + 4890,
    instrux + 4891,
    instrux + 4892,
    instrux + 4893,
    instrux + 4894,
    instrux + 4895,
    instrux + 4896,
    instrux + 4897,
    instrux + 4898,
    instrux + 4899,
    instrux + 4900,
    instrux + 4901,
    instrux + 4902,
    instrux + 4903,
    instrux + 4904,
    instrux + 4905,
    instrux + 4906,
    instrux + 4907,
    instrux + 4908,
    instrux + 4909,
    instrux + 4910,
    instrux + 4911,
    instrux + 4912,
    instrux + 4913,
    instrux + 4914,
    instrux + 4915,
    instrux + 4916,
    instrux + 4917,
    instrux + 4918,
    instrux + 4919,
    instrux + 4920,
    instrux + 4921,
    instrux + 4922,
    instrux + 4923,
    instrux + 4924,
    instrux + 4925,
    instrux + 4926,
    instrux + 4927,
    instrux + 4928,
    instrux + 4929,
    instrux + 4930,
    instrux + 4931,
    instrux + 4932,
    instrux + 4933,
    instrux + 4934,
    instrux + 4935,
    instrux + 4936,
    instrux + 4937,
    instrux + 4938,
    instrux + 4939,
    instrux + 4940,
    instrux + 4941,
    instrux + 4942,
    instrux + 4943,
    instrux + 4944,
    instrux + 4945,
    instrux + 4946,
    instrux + 4947,
    instrux + 4948,
    instrux + 4949,
    instrux + 4950,
    instrux + 4951,
    instrux + 4952,
    instrux + 4953,
    instrux + 4954,
    instrux + 4955,
    instrux + 4956,
    instrux + 4957,
    instrux + 4958,
    instrux + 4959,
    instrux + 4960,
    instrux + 4961,
    instrux + 4962,
    instrux + 4963,
    instrux + 4964,
    instrux + 4965,
    instrux + 4966,
    instrux + 4967,
    instrux + 4968,
    instrux + 4969,
    instrux + 4970,
    instrux + 4971,
    instrux + 4972,
    instrux + 4973,
    instrux + 4974,
    instrux + 4975,
    instrux + 4976,
    instrux + 4977,
    instrux + 4978,
    instrux + 4979,
    instrux + 4980,
    instrux + 4981,
    instrux + 4982,
    instrux + 4983,
    instrux + 4984,
    instrux + 4985,
    instrux + 4986,
    instrux + 4987,
    instrux + 4988,
    instrux + 4989,
    instrux + 4990,
    instrux + 4991,
    instrux + 4992,
    instrux + 4993,
    instrux + 4994,
    instrux + 4995,
    instrux + 4996,
    instrux + 4997,
    instrux + 4998,
    instrux + 4999,
    instrux + 5000,
    instrux + 5001,
    instrux + 5002,
    instrux + 5003,
    instrux + 5004,
    instrux + 5005,
    instrux + 5006,
    instrux + 5007,
    instrux + 5008,
    instrux + 5009,
    instrux + 5010,
    instrux + 5011,
    instrux + 5012,
    instrux + 5013,
    instrux + 5014,
    instrux + 5015,
    instrux + 5016,
    instrux + 5017,
    instrux + 5018,
    instrux + 5019,
    instrux + 5020,
    instrux + 5021,
    instrux + 5022,
    instrux + 5023,
    instrux + 5024,
    instrux + 5025,
    instrux + 5026,
    instrux + 5027,
    instrux + 5028,
    instrux + 5029,
    NULL
};

static const struct itemplate * const itbl_evex_map1_c4[] = {
    instrux + 6075,
    instrux + 6076,
    NULL
};

static const struct itemplate * const itbl_evex_map1_c5[] = {
    instrux + 6056,
    instrux + 6057,
    instrux + 6058,
    NULL
};

static const struct itemplate * const itbl_evex_map1_c6[] = {
    instrux + 6654,
    instrux + 6655,
    instrux + 6656,
    instrux + 6657,
    instrux + 6658,
    instrux + 6659,
    NULL
};

static const struct itemplate * const itbl_evex_map1_d1[] = {
    instrux + 6476,
    instrux + 6477,
    instrux + 6478,
    NULL
};

static const struct itemplate * const itbl_evex_map1_d2[] = {
    instrux + 6452,
    instrux + 6453,
    instrux + 6454,
    NULL
};

static const struct itemplate * const itbl_evex_map1_d3[] = {
    instrux + 6461,
    instrux + 6462,
    instrux + 6463,
    NULL
};

static const struct itemplate * const itbl_evex_map1_d4[] = {
    instrux + 5583,
    instrux + 5584,
    instrux + 5585,
    NULL
};

static const struct itemplate * const itbl_evex_map1_d5[] = {
    instrux + 6329,
    instrux + 6330,
    instrux + 6331,
    NULL
};

static const struct itemplate * const itbl_evex_map1_d6[] = {
    instrux + 5506,
    instrux + 7447,
    NULL
};

static const struct itemplate * const itbl_evex_map1_d8[] = {
    instrux + 6497,
    instrux + 6498,
    instrux + 6499,
    NULL
};

static const struct itemplate * const itbl_evex_map1_d9[] = {
    instrux + 6500,
    instrux + 6501,
    instrux + 6502,
    NULL
};

static const struct itemplate * const itbl_evex_map1_da[] = {
    instrux + 6131,
    instrux + 6132,
    instrux + 6133,
    NULL
};

static const struct itemplate * const itbl_evex_map1_db[] = {
    instrux + 5604,
    instrux + 5605,
    instrux + 5606,
    instrux + 5613,
    instrux + 5614,
    instrux + 5615,
    NULL
};

static const struct itemplate * const itbl_evex_map1_dc[] = {
    instrux + 5592,
    instrux + 5593,
    instrux + 5594,
    NULL
};

static const struct itemplate * const itbl_evex_map1_dd[] = {
    instrux + 5595,
    instrux + 5596,
    instrux + 5597,
    NULL
};

static const struct itemplate * const itbl_evex_map1_de[] = {
    instrux + 6107,
    instrux + 6108,
    instrux + 6109,
    NULL
};

static const struct itemplate * const itbl_evex_map1_df[] = {
    instrux + 5607,
    instrux + 5608,
    instrux + 5609,
    instrux + 5610,
    instrux + 5611,
    instrux + 5612,
    NULL
};

static const struct itemplate * const itbl_evex_map1_e0[] = {
    instrux + 5616,
    instrux + 5617,
    instrux + 5618,
    NULL
};

static const struct itemplate * const itbl_evex_map1_e1[] = {
    instrux + 6446,
    instrux + 6447,
    instrux + 6448,
    NULL
};

static const struct itemplate * const itbl_evex_map1_e2[] = {
    instrux + 6425,
    instrux + 6426,
    instrux + 6427,
    instrux + 6431,
    instrux + 6432,
    instrux + 6433,
    NULL
};

static const struct itemplate * const itbl_evex_map1_e3[] = {
    instrux + 5619,
    instrux + 5620,
    instrux + 5621,
    NULL
};

static const struct itemplate * const itbl_evex_map1_e4[] = {
    instrux + 6317,
    instrux + 6318,
    instrux + 6319,
    NULL
};

static const struct itemplate * const itbl_evex_map1_e5[] = {
    instrux + 6320,
    instrux + 6321,
    instrux + 6322,
    NULL
};

static const struct itemplate * const itbl_evex_map1_e6[] = {
    instrux + 5044,
    instrux + 5045,
    instrux + 5046,
    instrux + 5050,
    instrux + 5051,
    instrux + 5052,
    instrux + 5089,
    instrux + 5090,
    instrux + 5091,
    instrux + 5109,
    instrux + 5110,
    instrux + 5111,
    NULL
};

static const struct itemplate * const itbl_evex_map1_e7[] = {
    instrux + 5491,
    instrux + 5492,
    instrux + 5493,
    NULL
};

static const struct itemplate * const itbl_evex_map1_e8[] = {
    instrux + 6491,
    instrux + 6492,
    instrux + 6493,
    NULL
};

static const struct itemplate * const itbl_evex_map1_e9[] = {
    instrux + 6494,
    instrux + 6495,
    instrux + 6496,
    NULL
};

static const struct itemplate * const itbl_evex_map1_ea[] = {
    instrux + 6128,
    instrux + 6129,
    instrux + 6130,
    NULL
};

static const struct itemplate * const itbl_evex_map1_eb[] = {
    instrux + 6338,
    instrux + 6339,
    instrux + 6340,
    instrux + 6341,
    instrux + 6342,
    instrux + 6343,
    NULL
};

static const struct itemplate * const itbl_evex_map1_ec[] = {
    instrux + 5586,
    instrux + 5587,
    instrux + 5588,
    NULL
};

static const struct itemplate * const itbl_evex_map1_ed[] = {
    instrux + 5589,
    instrux + 5590,
    instrux + 5591,
    NULL
};

static const struct itemplate * const itbl_evex_map1_ee[] = {
    instrux + 6104,
    instrux + 6105,
    instrux + 6106,
    NULL
};

static const struct itemplate * const itbl_evex_map1_ef[] = {
    instrux + 6560,
    instrux + 6561,
    instrux + 6562,
    instrux + 6563,
    instrux + 6564,
    instrux + 6565,
    NULL
};

static const struct itemplate * const itbl_evex_map1_f1[] = {
    instrux + 6419,
    instrux + 6420,
    instrux + 6421,
    NULL
};

static const struct itemplate * const itbl_evex_map1_f2[] = {
    instrux + 6395,
    instrux + 6396,
    instrux + 6397,
    NULL
};

static const struct itemplate * const itbl_evex_map1_f3[] = {
    instrux + 6404,
    instrux + 6405,
    instrux + 6406,
    NULL
};

static const struct itemplate * const itbl_evex_map1_f4[] = {
    instrux + 6335,
    instrux + 6336,
    instrux + 6337,
    NULL
};

static const struct itemplate * const itbl_evex_map1_f5[] = {
    instrux + 6092,
    instrux + 6093,
    instrux + 6094,
    NULL
};

static const struct itemplate * const itbl_evex_map1_f6[] = {
    instrux + 6368,
    instrux + 6369,
    instrux + 6370,
    NULL
};

static const struct itemplate * const itbl_evex_map1_f8[] = {
    instrux + 6482,
    instrux + 6483,
    instrux + 6484,
    NULL
};

static const struct itemplate * const itbl_evex_map1_f9[] = {
    instrux + 6503,
    instrux + 6504,
    instrux + 6505,
    NULL
};

static const struct itemplate * const itbl_evex_map1_fa[] = {
    instrux + 6485,
    instrux + 6486,
    instrux + 6487,
    NULL
};

static const struct itemplate * const itbl_evex_map1_fb[] = {
    instrux + 6488,
    instrux + 6489,
    instrux + 6490,
    NULL
};

static const struct itemplate * const itbl_evex_map1_fc[] = {
    instrux + 5577,
    instrux + 5578,
    instrux + 5579,
    NULL
};

static const struct itemplate * const itbl_evex_map1_fd[] = {
    instrux + 5598,
    instrux + 5599,
    instrux + 5600,
    NULL
};

static const struct itemplate * const itbl_evex_map1_fe[] = {
    instrux + 5580,
    instrux + 5581,
    instrux + 5582,
    NULL
};

static const struct itemplate * const * const itbl_evex_map1[256] = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map1_10,
    itbl_evex_map1_11,
    itbl_evex_map1_12,
    itbl_evex_map1_13,
    itbl_evex_map1_14,
    itbl_evex_map1_15,
    itbl_evex_map1_16,
    itbl_evex_map1_17,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map1_28,
    itbl_evex_map1_29,
    itbl_evex_map1_2a,
    itbl_evex_map1_2b,
    itbl_evex_map1_2c,
    itbl_evex_map1_2d,
    itbl_evex_map1_2e,
    itbl_evex_map1_2f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map1_41,
    itbl_evex_map1_42,
    NULL,
    itbl_evex_map1_44,
    itbl_evex_map1_45,
    itbl_evex_map1_46,
    itbl_evex_map1_47,
    NULL,
    NULL,
    itbl_evex_map1_4a,
    itbl_evex_map1_4b,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map1_51,
    NULL,
    NULL,
    itbl_evex_map1_54,
    itbl_evex_map1_55,
    itbl_evex_map1_56,
    itbl_evex_map1_57,
    itbl_evex_map1_58,
    itbl_evex_map1_59,
    itbl_evex_map1_5a,
    itbl_evex_map1_5b,
    itbl_evex_map1_5c,
    itbl_evex_map1_5d,
    itbl_evex_map1_5e,
    itbl_evex_map1_5f,
    itbl_evex_map1_60,
    itbl_evex_map1_61,
    itbl_evex_map1_62,
    itbl_evex_map1_63,
    itbl_evex_map1_64,
    itbl_evex_map1_65,
    itbl_evex_map1_66,
    itbl_evex_map1_67,
    itbl_evex_map1_68,
    itbl_evex_map1_69,
    itbl_evex_map1_6a,
    itbl_evex_map1_6b,
    itbl_evex_map1_6c,
    itbl_evex_map1_6d,
    itbl_evex_map1_6e,
    itbl_evex_map1_6f,
    itbl_evex_map1_70,
    itbl_evex_map1_71,
    itbl_evex_map1_72,
    itbl_evex_map1_73,
    itbl_evex_map1_74,
    itbl_evex_map1_75,
    itbl_evex_map1_76,
    NULL,
    itbl_evex_map1_78,
    itbl_evex_map1_79,
    itbl_evex_map1_7a,
    itbl_evex_map1_7b,
    NULL,
    NULL,
    itbl_evex_map1_7e,
    itbl_evex_map1_7f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map1_90,
    itbl_evex_map1_91,
    itbl_evex_map1_92,
    itbl_evex_map1_93,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map1_98,
    itbl_evex_map1_99,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map1_c2,
    NULL,
    itbl_evex_map1_c4,
    itbl_evex_map1_c5,
    itbl_evex_map1_c6,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map1_d1,
    itbl_evex_map1_d2,
    itbl_evex_map1_d3,
    itbl_evex_map1_d4,
    itbl_evex_map1_d5,
    itbl_evex_map1_d6,
    NULL,
    itbl_evex_map1_d8,
    itbl_evex_map1_d9,
    itbl_evex_map1_da,
    itbl_evex_map1_db,
    itbl_evex_map1_dc,
    itbl_evex_map1_dd,
    itbl_evex_map1_de,
    itbl_evex_map1_df,
    itbl_evex_map1_e0,
    itbl_evex_map1_e1,
    itbl_evex_map1_e2,
    itbl_evex_map1_e3,
    itbl_evex_map1_e4,
    itbl_evex_map1_e5,
    itbl_evex_map1_e6,
    itbl_evex_map1_e7,
    itbl_evex_map1_e8,
    itbl_evex_map1_e9,
    itbl_evex_map1_ea,
    itbl_evex_map1_eb,
    itbl_evex_map1_ec,
    itbl_evex_map1_ed,
    itbl_evex_map1_ee,
    itbl_evex_map1_ef,
    NULL,
    itbl_evex_map1_f1,
    itbl_evex_map1_f2,
    itbl_evex_map1_f3,
    itbl_evex_map1_f4,
    itbl_evex_map1_f5,
    itbl_evex_map1_f6,
    NULL,
    itbl_evex_map1_f8,
    itbl_evex_map1_f9,
    itbl_evex_map1_fa,
    itbl_evex_map1_fb,
    itbl_evex_map1_fc,
    itbl_evex_map1_fd,
    itbl_evex_map1_fe,
    NULL,
};

static const struct itemplate * const itbl_evex_map2_00[] = {
    instrux + 6383,
    instrux + 6384,
    instrux + 6385,
    NULL
};

static const struct itemplate * const itbl_evex_map2_04[] = {
    instrux + 6089,
    instrux + 6090,
    instrux + 6091,
    NULL
};

static const struct itemplate * const itbl_evex_map2_0b[] = {
    instrux + 6314,
    instrux + 6315,
    instrux + 6316,
    NULL
};

static const struct itemplate * const itbl_evex_map2_0c[] = {
    instrux + 5999,
    instrux + 6000,
    instrux + 6001,
    NULL
};

static const struct itemplate * const itbl_evex_map2_0d[] = {
    instrux + 5993,
    instrux + 5994,
    instrux + 5995,
    NULL
};

static const struct itemplate * const itbl_evex_map2_10[] = {
    instrux + 6278,
    instrux + 6279,
    instrux + 6280,
    instrux + 6281,
    instrux + 6282,
    instrux + 6283,
    instrux + 6473,
    instrux + 6474,
    instrux + 6475,
    NULL
};

static const struct itemplate * const itbl_evex_map2_11[] = {
    instrux + 6248,
    instrux + 6249,
    instrux + 6250,
    instrux + 6251,
    instrux + 6252,
    instrux + 6253,
    instrux + 6443,
    instrux + 6444,
    instrux + 6445,
    NULL
};

static const struct itemplate * const itbl_evex_map2_12[] = {
    instrux + 6260,
    instrux + 6261,
    instrux + 6262,
    instrux + 6263,
    instrux + 6264,
    instrux + 6265,
    instrux + 6416,
    instrux + 6417,
    instrux + 6418,
    NULL
};

static const struct itemplate * const itbl_evex_map2_13[] = {
    instrux + 5065,
    instrux + 5066,
    instrux + 5067,
    instrux + 6254,
    instrux + 6255,
    instrux + 6256,
    instrux + 6257,
    instrux + 6258,
    instrux + 6259,
    instrux + 6942,
    instrux + 6943,
    instrux + 6944,
    NULL
};

static const struct itemplate * const itbl_evex_map2_14[] = {
    instrux + 6272,
    instrux + 6273,
    instrux + 6274,
    instrux + 6275,
    instrux + 6276,
    instrux + 6277,
    instrux + 6362,
    instrux + 6363,
    instrux + 6364,
    instrux + 6365,
    instrux + 6366,
    instrux + 6367,
    NULL
};

static const struct itemplate * const itbl_evex_map2_15[] = {
    instrux + 6266,
    instrux + 6267,
    instrux + 6268,
    instrux + 6269,
    instrux + 6270,
    instrux + 6271,
    instrux + 6350,
    instrux + 6351,
    instrux + 6352,
    instrux + 6353,
    instrux + 6354,
    instrux + 6355,
    NULL
};

static const struct itemplate * const itbl_evex_map2_16[] = {
    instrux + 6004,
    instrux + 6005,
    instrux + 6006,
    instrux + 6007,
    NULL
};

static const struct itemplate * const itbl_evex_map2_18[] = {
    instrux + 4648,
    instrux + 4649,
    instrux + 4650,
    instrux + 4651,
    instrux + 4652,
    instrux + 4653,
    NULL
};

static const struct itemplate * const itbl_evex_map2_19[] = {
    instrux + 4627,
    instrux + 4628,
    instrux + 4644,
    instrux + 4645,
    instrux + 4646,
    instrux + 4647,
    NULL
};

static const struct itemplate * const itbl_evex_map2_1a[] = {
    instrux + 4629,
    instrux + 4630,
    instrux + 4632,
    instrux + 4633,
    NULL
};

static const struct itemplate * const itbl_evex_map2_1b[] = {
    instrux + 4631,
    instrux + 4634,
    NULL
};

static const struct itemplate * const itbl_evex_map2_1c[] = {
    instrux + 5553,
    instrux + 5554,
    instrux + 5555,
    NULL
};

static const struct itemplate * const itbl_evex_map2_1d[] = {
    instrux + 5562,
    instrux + 5563,
    instrux + 5564,
    NULL
};

static const struct itemplate * const itbl_evex_map2_1e[] = {
    instrux + 5556,
    instrux + 5557,
    instrux + 5558,
    NULL
};

static const struct itemplate * const itbl_evex_map2_1f[] = {
    instrux + 5559,
    instrux + 5560,
    instrux + 5561,
    NULL
};

static const struct itemplate * const itbl_evex_map2_20[] = {
    instrux + 6224,
    instrux + 6225,
    instrux + 6226,
    instrux + 6227,
    instrux + 6228,
    instrux + 6229,
    instrux + 6236,
    instrux + 6237,
    instrux + 6238,
    NULL
};

static const struct itemplate * const itbl_evex_map2_21[] = {
    instrux + 6194,
    instrux + 6195,
    instrux + 6196,
    instrux + 6197,
    instrux + 6198,
    instrux + 6199,
    instrux + 6230,
    instrux + 6231,
    instrux + 6232,
    NULL
};

static const struct itemplate * const itbl_evex_map2_22[] = {
    instrux + 6206,
    instrux + 6207,
    instrux + 6208,
    instrux + 6209,
    instrux + 6210,
    instrux + 6211,
    instrux + 6233,
    instrux + 6234,
    instrux + 6235,
    NULL
};

static const struct itemplate * const itbl_evex_map2_23[] = {
    instrux + 6200,
    instrux + 6201,
    instrux + 6202,
    instrux + 6203,
    instrux + 6204,
    instrux + 6205,
    instrux + 6242,
    instrux + 6243,
    instrux + 6244,
    NULL
};

static const struct itemplate * const itbl_evex_map2_24[] = {
    instrux + 6218,
    instrux + 6219,
    instrux + 6220,
    instrux + 6221,
    instrux + 6222,
    instrux + 6223,
    instrux + 6245,
    instrux + 6246,
    instrux + 6247,
    NULL
};

static const struct itemplate * const itbl_evex_map2_25[] = {
    instrux + 6212,
    instrux + 6213,
    instrux + 6214,
    instrux + 6215,
    instrux + 6216,
    instrux + 6217,
    instrux + 6239,
    instrux + 6240,
    instrux + 6241,
    NULL
};

static const struct itemplate * const itbl_evex_map2_26[] = {
    instrux + 6512,
    instrux + 6513,
    instrux + 6514,
    instrux + 6521,
    instrux + 6522,
    instrux + 6523,
    instrux + 6524,
    instrux + 6525,
    instrux + 6526,
    instrux + 6533,
    instrux + 6534,
    instrux + 6535,
    NULL
};

static const struct itemplate * const itbl_evex_map2_27[] = {
    instrux + 6515,
    instrux + 6516,
    instrux + 6517,
    instrux + 6518,
    instrux + 6519,
    instrux + 6520,
    instrux + 6527,
    instrux + 6528,
    instrux + 6529,
    instrux + 6530,
    instrux + 6531,
    instrux + 6532,
    NULL
};

static const struct itemplate * const itbl_evex_map2_28[] = {
    instrux + 6161,
    instrux + 6162,
    instrux + 6163,
    instrux + 6170,
    instrux + 6171,
    instrux + 6172,
    instrux + 6311,
    instrux + 6312,
    instrux + 6313,
    NULL
};

static const struct itemplate * const itbl_evex_map2_29[] = {
    instrux + 5691,
    instrux + 5692,
    instrux + 5693,
    instrux + 6143,
    instrux + 6144,
    instrux + 6145,
    instrux + 6284,
    instrux + 6285,
    instrux + 6286,
    NULL
};

static const struct itemplate * const itbl_evex_map2_2a[] = {
    instrux + 5494,
    instrux + 5495,
    instrux + 5496,
    instrux + 5658,
    instrux + 5659,
    instrux + 5660,
    NULL
};

static const struct itemplate * const itbl_evex_map2_2b[] = {
    instrux + 5571,
    instrux + 5572,
    instrux + 5573,
    NULL
};

static const struct itemplate * const itbl_evex_map2_2c[] = {
    instrux + 6618,
    instrux + 6619,
    instrux + 6620,
    instrux + 6621,
    instrux + 6622,
    instrux + 6623,
    NULL
};

static const struct itemplate * const itbl_evex_map2_2d[] = {
    instrux + 6624,
    instrux + 6625,
    NULL
};

static const struct itemplate * const itbl_evex_map2_30[] = {
    instrux + 6287,
    instrux + 6288,
    instrux + 6289,
    instrux + 6290,
    instrux + 6291,
    instrux + 6292,
    instrux + 6299,
    instrux + 6300,
    instrux + 6301,
    NULL
};

static const struct itemplate * const itbl_evex_map2_31[] = {
    instrux + 6149,
    instrux + 6150,
    instrux + 6151,
    instrux + 6152,
    instrux + 6153,
    instrux + 6154,
    instrux + 6293,
    instrux + 6294,
    instrux + 6295,
    NULL
};

static const struct itemplate * const itbl_evex_map2_32[] = {
    instrux + 6176,
    instrux + 6177,
    instrux + 6178,
    instrux + 6179,
    instrux + 6180,
    instrux + 6181,
    instrux + 6296,
    instrux + 6297,
    instrux + 6298,
    NULL
};

static const struct itemplate * const itbl_evex_map2_33[] = {
    instrux + 6155,
    instrux + 6156,
    instrux + 6157,
    instrux + 6158,
    instrux + 6159,
    instrux + 6160,
    instrux + 6305,
    instrux + 6306,
    instrux + 6307,
    NULL
};

static const struct itemplate * const itbl_evex_map2_34[] = {
    instrux + 6188,
    instrux + 6189,
    instrux + 6190,
    instrux + 6191,
    instrux + 6192,
    instrux + 6193,
    instrux + 6308,
    instrux + 6309,
    instrux + 6310,
    NULL
};

static const struct itemplate * const itbl_evex_map2_35[] = {
    instrux + 6182,
    instrux + 6183,
    instrux + 6184,
    instrux + 6185,
    instrux + 6186,
    instrux + 6187,
    instrux + 6302,
    instrux + 6303,
    instrux + 6304,
    NULL
};

static const struct itemplate * const itbl_evex_map2_36[] = {
    instrux + 5970,
    instrux + 5971,
    instrux + 6010,
    instrux + 6011,
    NULL
};

static const struct itemplate * const itbl_evex_map2_37[] = {
    instrux + 5703,
    instrux + 5704,
    instrux + 5705,
    NULL
};

static const struct itemplate * const itbl_evex_map2_38[] = {
    instrux + 6119,
    instrux + 6120,
    instrux + 6121,
    instrux + 6164,
    instrux + 6165,
    instrux + 6166,
    instrux + 6167,
    instrux + 6168,
    instrux + 6169,
    NULL
};

static const struct itemplate * const itbl_evex_map2_39[] = {
    instrux + 6122,
    instrux + 6123,
    instrux + 6124,
    instrux + 6125,
    instrux + 6126,
    instrux + 6127,
    instrux + 6146,
    instrux + 6147,
    instrux + 6148,
    instrux + 6173,
    instrux + 6174,
    instrux + 6175,
    NULL
};

static const struct itemplate * const itbl_evex_map2_3a[] = {
    instrux + 5661,
    instrux + 5662,
    instrux + 5663,
    instrux + 6140,
    instrux + 6141,
    instrux + 6142,
    NULL
};

static const struct itemplate * const itbl_evex_map2_3b[] = {
    instrux + 6134,
    instrux + 6135,
    instrux + 6136,
    instrux + 6137,
    instrux + 6138,
    instrux + 6139,
    NULL
};

static const struct itemplate * const itbl_evex_map2_3c[] = {
    instrux + 6095,
    instrux + 6096,
    instrux + 6097,
    NULL
};

static const struct itemplate * const itbl_evex_map2_3d[] = {
    instrux + 6098,
    instrux + 6099,
    instrux + 6100,
    instrux + 6101,
    instrux + 6102,
    instrux + 6103,
    NULL
};

static const struct itemplate * const itbl_evex_map2_3e[] = {
    instrux + 6116,
    instrux + 6117,
    instrux + 6118,
    NULL
};

static const struct itemplate * const itbl_evex_map2_3f[] = {
    instrux + 6110,
    instrux + 6111,
    instrux + 6112,
    instrux + 6113,
    instrux + 6114,
    instrux + 6115,
    NULL
};

static const struct itemplate * const itbl_evex_map2_40[] = {
    instrux + 6323,
    instrux + 6324,
    instrux + 6325,
    instrux + 6326,
    instrux + 6327,
    instrux + 6328,
    NULL
};

static const struct itemplate * const itbl_evex_map2_42[] = {
    instrux + 5377,
    instrux + 5378,
    instrux + 5379,
    instrux + 5380,
    instrux + 5381,
    instrux + 5382,
    NULL
};

static const struct itemplate * const itbl_evex_map2_43[] = {
    instrux + 5383,
    instrux + 5384,
    NULL
};

static const struct itemplate * const itbl_evex_map2_44[] = {
    instrux + 6077,
    instrux + 6078,
    instrux + 6079,
    instrux + 6080,
    instrux + 6081,
    instrux + 6082,
    NULL
};

static const struct itemplate * const itbl_evex_map2_45[] = {
    instrux + 6467,
    instrux + 6468,
    instrux + 6469,
    instrux + 6470,
    instrux + 6471,
    instrux + 6472,
    NULL
};

static const struct itemplate * const itbl_evex_map2_46[] = {
    instrux + 6437,
    instrux + 6438,
    instrux + 6439,
    instrux + 6440,
    instrux + 6441,
    instrux + 6442,
    NULL
};

static const struct itemplate * const itbl_evex_map2_47[] = {
    instrux + 6410,
    instrux + 6411,
    instrux + 6412,
    instrux + 6413,
    instrux + 6414,
    instrux + 6415,
    NULL
};

static const struct itemplate * const itbl_evex_map2_49[] = {
    instrux + 6865,
    instrux + 6867,
    NULL
};

static const struct itemplate * const itbl_evex_map2_4a[] = {
    instrux + 6885,
    instrux + 6887,
    instrux + 6898,
    instrux + 6913,
    NULL
};

static const struct itemplate * const itbl_evex_map2_4b[] = {
    instrux + 6877,
    instrux + 6879,
    instrux + 6882,
    NULL
};

static const struct itemplate * const itbl_evex_map2_4c[] = {
    instrux + 6574,
    instrux + 6575,
    instrux + 6576,
    instrux + 6577,
    instrux + 6578,
    instrux + 6579,
    NULL
};

static const struct itemplate * const itbl_evex_map2_4d[] = {
    instrux + 6580,
    instrux + 6581,
    NULL
};

static const struct itemplate * const itbl_evex_map2_4e[] = {
    instrux + 6606,
    instrux + 6607,
    instrux + 6608,
    instrux + 6609,
    instrux + 6610,
    instrux + 6611,
    NULL
};

static const struct itemplate * const itbl_evex_map2_4f[] = {
    instrux + 6612,
    instrux + 6613,
    NULL
};

static const struct itemplate * const itbl_evex_map2_50[] = {
    instrux + 6784,
    instrux + 6785,
    instrux + 6786,
    instrux + 7327,
    instrux + 7328,
    instrux + 7329,
    instrux + 7333,
    instrux + 7334,
    instrux + 7335,
    instrux + 7339,
    instrux + 7340,
    instrux + 7341,
    NULL
};

static const struct itemplate * const itbl_evex_map2_51[] = {
    instrux + 6787,
    instrux + 6788,
    instrux + 6789,
    instrux + 7330,
    instrux + 7331,
    instrux + 7332,
    instrux + 7336,
    instrux + 7337,
    instrux + 7338,
    instrux + 7342,
    instrux + 7343,
    instrux + 7344,
    NULL
};

static const struct itemplate * const itbl_evex_map2_52[] = {
    instrux + 6790,
    instrux + 6791,
    instrux + 6792,
    instrux + 6816,
    instrux + 6855,
    instrux + 6856,
    instrux + 6857,
    instrux + 7321,
    instrux + 7322,
    instrux + 7323,
    NULL
};

static const struct itemplate * const itbl_evex_map2_53[] = {
    instrux + 6793,
    instrux + 6794,
    instrux + 6795,
    instrux + 6815,
    NULL
};

static const struct itemplate * const itbl_evex_map2_54[] = {
    instrux + 6796,
    instrux + 6797,
    instrux + 6798,
    instrux + 6799,
    instrux + 6800,
    instrux + 6801,
    NULL
};

static const struct itemplate * const itbl_evex_map2_55[] = {
    instrux + 6802,
    instrux + 6803,
    instrux + 6804,
    instrux + 6805,
    instrux + 6806,
    instrux + 6807,
    NULL
};

static const struct itemplate * const itbl_evex_map2_58[] = {
    instrux + 5649,
    instrux + 5650,
    instrux + 5651,
    instrux + 5652,
    instrux + 5653,
    instrux + 5654,
    NULL
};

static const struct itemplate * const itbl_evex_map2_59[] = {
    instrux + 4635,
    instrux + 4636,
    instrux + 4637,
    instrux + 5664,
    instrux + 5665,
    instrux + 5666,
    instrux + 5667,
    instrux + 5668,
    instrux + 5669,
    NULL
};

static const struct itemplate * const itbl_evex_map2_5a[] = {
    instrux + 4638,
    instrux + 4639,
    instrux + 4641,
    instrux + 4642,
    NULL
};

static const struct itemplate * const itbl_evex_map2_5b[] = {
    instrux + 4640,
    instrux + 4643,
    NULL
};

static const struct itemplate * const itbl_evex_map2_62[] = {
    instrux + 6742,
    instrux + 6743,
    instrux + 6744,
    instrux + 6745,
    instrux + 6746,
    instrux + 6747,
    NULL
};

static const struct itemplate * const itbl_evex_map2_63[] = {
    instrux + 6730,
    instrux + 6731,
    instrux + 6732,
    instrux + 6733,
    instrux + 6734,
    instrux + 6735,
    instrux + 6736,
    instrux + 6737,
    instrux + 6738,
    instrux + 6739,
    instrux + 6740,
    instrux + 6741,
    NULL
};

static const struct itemplate * const itbl_evex_map2_64[] = {
    instrux + 5625,
    instrux + 5626,
    instrux + 5627,
    instrux + 5628,
    instrux + 5629,
    instrux + 5630,
    NULL
};

static const struct itemplate * const itbl_evex_map2_65[] = {
    instrux + 4621,
    instrux + 4622,
    instrux + 4623,
    instrux + 4624,
    instrux + 4625,
    instrux + 4626,
    NULL
};

static const struct itemplate * const itbl_evex_map2_66[] = {
    instrux + 5622,
    instrux + 5623,
    instrux + 5624,
    instrux + 5631,
    instrux + 5632,
    instrux + 5633,
    NULL
};

static const struct itemplate * const itbl_evex_map2_67[] = {
    instrux + 7303,
    instrux + 7304,
    instrux + 7305,
    NULL
};

static const struct itemplate * const itbl_evex_map2_68[] = {
    instrux + 6858,
    instrux + 6859,
    instrux + 6860,
    instrux + 6861,
    instrux + 6862,
    instrux + 6863,
    NULL
};

static const struct itemplate * const itbl_evex_map2_6d[] = {
    instrux + 6900,
    instrux + 6902,
    instrux + 6904,
    instrux + 6906,
    NULL
};

static const struct itemplate * const itbl_evex_map2_70[] = {
    instrux + 6757,
    instrux + 6758,
    instrux + 6759,
    NULL
};

static const struct itemplate * const itbl_evex_map2_71[] = {
    instrux + 6760,
    instrux + 6761,
    instrux + 6762,
    instrux + 6763,
    instrux + 6764,
    instrux + 6765,
    NULL
};

static const struct itemplate * const itbl_evex_map2_72[] = {
    instrux + 6775,
    instrux + 6776,
    instrux + 6777,
    instrux + 6849,
    instrux + 6850,
    instrux + 6851,
    instrux + 6852,
    instrux + 6853,
    instrux + 6854,
    NULL
};

static const struct itemplate * const itbl_evex_map2_73[] = {
    instrux + 6778,
    instrux + 6779,
    instrux + 6780,
    instrux + 6781,
    instrux + 6782,
    instrux + 6783,
    NULL
};

static const struct itemplate * const itbl_evex_map2_74[] = {
    instrux + 7279,
    instrux + 7280,
    instrux + 7281,
    instrux + 7291,
    instrux + 7292,
    instrux + 7293,
    instrux + 7306,
    instrux + 7307,
    instrux + 7308,
    NULL
};

static const struct itemplate * const itbl_evex_map2_75[] = {
    instrux + 5972,
    instrux + 5973,
    instrux + 5974,
    instrux + 5987,
    instrux + 5988,
    instrux + 5989,
    NULL
};

static const struct itemplate * const itbl_evex_map2_76[] = {
    instrux + 5975,
    instrux + 5976,
    instrux + 5977,
    instrux + 5984,
    instrux + 5985,
    instrux + 5986,
    NULL
};

static const struct itemplate * const itbl_evex_map2_77[] = {
    instrux + 5978,
    instrux + 5979,
    instrux + 5980,
    instrux + 5981,
    instrux + 5982,
    instrux + 5983,
    NULL
};

static const struct itemplate * const itbl_evex_map2_78[] = {
    instrux + 5634,
    instrux + 5635,
    instrux + 5636,
    NULL
};

static const struct itemplate * const itbl_evex_map2_79[] = {
    instrux + 5673,
    instrux + 5674,
    instrux + 5675,
    NULL
};

static const struct itemplate * const itbl_evex_map2_7a[] = {
    instrux + 5637,
    instrux + 5638,
    instrux + 5639,
    instrux + 5640,
    instrux + 5641,
    instrux + 5642,
    instrux + 5643,
    instrux + 5644,
    instrux + 5645,
    instrux + 5646,
    instrux + 5647,
    instrux + 5648,
    NULL
};

static const struct itemplate * const itbl_evex_map2_7b[] = {
    instrux + 5676,
    instrux + 5677,
    instrux + 5678,
    instrux + 5679,
    instrux + 5680,
    instrux + 5681,
    instrux + 5682,
    instrux + 5683,
    instrux + 5684,
    NULL
};

static const struct itemplate * const itbl_evex_map2_7c[] = {
    instrux + 5655,
    instrux + 5656,
    instrux + 5657,
    instrux + 5670,
    instrux + 5671,
    instrux + 5672,
    NULL
};

static const struct itemplate * const itbl_evex_map2_7d[] = {
    instrux + 6012,
    instrux + 6013,
    instrux + 6014,
    instrux + 6027,
    instrux + 6028,
    instrux + 6029,
    NULL
};

static const struct itemplate * const itbl_evex_map2_7e[] = {
    instrux + 6015,
    instrux + 6016,
    instrux + 6017,
    instrux + 6024,
    instrux + 6025,
    instrux + 6026,
    NULL
};

static const struct itemplate * const itbl_evex_map2_7f[] = {
    instrux + 6018,
    instrux + 6019,
    instrux + 6020,
    instrux + 6021,
    instrux + 6022,
    instrux + 6023,
    NULL
};

static const struct itemplate * const itbl_evex_map2_83[] = {
    instrux + 6332,
    instrux + 6333,
    instrux + 6334,
    NULL
};

static const struct itemplate * const itbl_evex_map2_88[] = {
    instrux + 5170,
    instrux + 5171,
    instrux + 5172,
    instrux + 5173,
    instrux + 5174,
    instrux + 5175,
    instrux + 5176,
    instrux + 5177,
    instrux + 5178,
    instrux + 5179,
    instrux + 5180,
    instrux + 5181,
    NULL
};

static const struct itemplate * const itbl_evex_map2_89[] = {
    instrux + 6033,
    instrux + 6034,
    instrux + 6035,
    instrux + 6036,
    instrux + 6037,
    instrux + 6038,
    instrux + 6039,
    instrux + 6040,
    instrux + 6041,
    instrux + 6042,
    instrux + 6043,
    instrux + 6044,
    NULL
};

static const struct itemplate * const itbl_evex_map2_8a[] = {
    instrux + 5032,
    instrux + 5033,
    instrux + 5034,
    instrux + 5035,
    instrux + 5036,
    instrux + 5037,
    instrux + 5038,
    instrux + 5039,
    instrux + 5040,
    instrux + 5041,
    instrux + 5042,
    instrux + 5043,
    NULL
};

static const struct itemplate * const itbl_evex_map2_8b[] = {
    instrux + 5949,
    instrux + 5950,
    instrux + 5951,
    instrux + 5952,
    instrux + 5953,
    instrux + 5954,
    instrux + 5955,
    instrux + 5956,
    instrux + 5957,
    instrux + 5958,
    instrux + 5959,
    instrux + 5960,
    NULL
};

static const struct itemplate * const itbl_evex_map2_8d[] = {
    instrux + 5967,
    instrux + 5968,
    instrux + 5969,
    instrux + 6030,
    instrux + 6031,
    instrux + 6032,
    NULL
};

static const struct itemplate * const itbl_evex_map2_8f[] = {
    instrux + 6808,
    instrux + 6809,
    instrux + 6810,
    NULL
};

static const struct itemplate * const itbl_evex_map2_90[] = {
    instrux + 6059,
    instrux + 6060,
    instrux + 6061,
    instrux + 6062,
    instrux + 6063,
    instrux + 6064,
    NULL
};

static const struct itemplate * const itbl_evex_map2_91[] = {
    instrux + 6065,
    instrux + 6066,
    instrux + 6067,
    instrux + 6068,
    instrux + 6069,
    instrux + 6070,
    NULL
};

static const struct itemplate * const itbl_evex_map2_92[] = {
    instrux + 5357,
    instrux + 5358,
    instrux + 5359,
    instrux + 5360,
    instrux + 5361,
    instrux + 5362,
    NULL
};

static const struct itemplate * const itbl_evex_map2_93[] = {
    instrux + 5371,
    instrux + 5372,
    instrux + 5373,
    instrux + 5374,
    instrux + 5375,
    instrux + 5376,
    NULL
};

static const struct itemplate * const itbl_evex_map2_96[] = {
    instrux + 5241,
    instrux + 5242,
    instrux + 5243,
    instrux + 5244,
    instrux + 5245,
    instrux + 5246,
    NULL
};

static const struct itemplate * const itbl_evex_map2_97[] = {
    instrux + 5283,
    instrux + 5284,
    instrux + 5285,
    instrux + 5286,
    instrux + 5287,
    instrux + 5288,
    NULL
};

static const struct itemplate * const itbl_evex_map2_98[] = {
    instrux + 5217,
    instrux + 5218,
    instrux + 5219,
    instrux + 5220,
    instrux + 5221,
    instrux + 5222,
    NULL
};

static const struct itemplate * const itbl_evex_map2_99[] = {
    instrux + 5223,
    instrux + 5224,
    NULL
};

static const struct itemplate * const itbl_evex_map2_9a[] = {
    instrux + 5259,
    instrux + 5260,
    instrux + 5261,
    instrux + 5262,
    instrux + 5263,
    instrux + 5264,
    instrux + 6811,
    NULL
};

static const struct itemplate * const itbl_evex_map2_9b[] = {
    instrux + 5265,
    instrux + 5266,
    instrux + 6813,
    NULL
};

static const struct itemplate * const itbl_evex_map2_9c[] = {
    instrux + 5301,
    instrux + 5302,
    instrux + 5303,
    instrux + 5304,
    instrux + 5305,
    instrux + 5306,
    NULL
};

static const struct itemplate * const itbl_evex_map2_9d[] = {
    instrux + 5307,
    instrux + 5308,
    NULL
};

static const struct itemplate * const itbl_evex_map2_9e[] = {
    instrux + 5325,
    instrux + 5326,
    instrux + 5327,
    instrux + 5328,
    instrux + 5329,
    instrux + 5330,
    NULL
};

static const struct itemplate * const itbl_evex_map2_9f[] = {
    instrux + 5331,
    instrux + 5332,
    NULL
};

static const struct itemplate * const itbl_evex_map2_a0[] = {
    instrux + 6371,
    instrux + 6372,
    instrux + 6373,
    instrux + 6374,
    instrux + 6375,
    instrux + 6376,
    NULL
};

static const struct itemplate * const itbl_evex_map2_a1[] = {
    instrux + 6377,
    instrux + 6378,
    instrux + 6379,
    instrux + 6380,
    instrux + 6381,
    instrux + 6382,
    NULL
};

static const struct itemplate * const itbl_evex_map2_a2[] = {
    instrux + 6626,
    instrux + 6627,
    instrux + 6628,
    instrux + 6629,
    instrux + 6630,
    instrux + 6631,
    NULL
};

static const struct itemplate * const itbl_evex_map2_a3[] = {
    instrux + 6640,
    instrux + 6641,
    instrux + 6642,
    instrux + 6643,
    instrux + 6644,
    instrux + 6645,
    NULL
};

static const struct itemplate * const itbl_evex_map2_a6[] = {
    instrux + 5247,
    instrux + 5248,
    instrux + 5249,
    instrux + 5250,
    instrux + 5251,
    instrux + 5252,
    NULL
};

static const struct itemplate * const itbl_evex_map2_a7[] = {
    instrux + 5289,
    instrux + 5290,
    instrux + 5291,
    instrux + 5292,
    instrux + 5293,
    instrux + 5294,
    NULL
};

static const struct itemplate * const itbl_evex_map2_a8[] = {
    instrux + 5225,
    instrux + 5226,
    instrux + 5227,
    instrux + 5228,
    instrux + 5229,
    instrux + 5230,
    NULL
};

static const struct itemplate * const itbl_evex_map2_a9[] = {
    instrux + 5231,
    instrux + 5232,
    NULL
};

static const struct itemplate * const itbl_evex_map2_aa[] = {
    instrux + 5267,
    instrux + 5268,
    instrux + 5269,
    instrux + 5270,
    instrux + 5271,
    instrux + 5272,
    instrux + 6812,
    NULL
};

static const struct itemplate * const itbl_evex_map2_ab[] = {
    instrux + 5273,
    instrux + 5274,
    instrux + 6814,
    NULL
};

static const struct itemplate * const itbl_evex_map2_ac[] = {
    instrux + 5309,
    instrux + 5310,
    instrux + 5311,
    instrux + 5312,
    instrux + 5313,
    instrux + 5314,
    NULL
};

static const struct itemplate * const itbl_evex_map2_ad[] = {
    instrux + 5315,
    instrux + 5316,
    NULL
};

static const struct itemplate * const itbl_evex_map2_ae[] = {
    instrux + 5333,
    instrux + 5334,
    instrux + 5335,
    instrux + 5336,
    instrux + 5337,
    instrux + 5338,
    NULL
};

static const struct itemplate * const itbl_evex_map2_af[] = {
    instrux + 5339,
    instrux + 5340,
    NULL
};

static const struct itemplate * const itbl_evex_map2_b4[] = {
    instrux + 6086,
    instrux + 6087,
    instrux + 6088,
    NULL
};

static const struct itemplate * const itbl_evex_map2_b5[] = {
    instrux + 6083,
    instrux + 6084,
    instrux + 6085,
    NULL
};

static const struct itemplate * const itbl_evex_map2_b6[] = {
    instrux + 5253,
    instrux + 5254,
    instrux + 5255,
    instrux + 5256,
    instrux + 5257,
    instrux + 5258,
    NULL
};

static const struct itemplate * const itbl_evex_map2_b7[] = {
    instrux + 5295,
    instrux + 5296,
    instrux + 5297,
    instrux + 5298,
    instrux + 5299,
    instrux + 5300,
    NULL
};

static const struct itemplate * const itbl_evex_map2_b8[] = {
    instrux + 5233,
    instrux + 5234,
    instrux + 5235,
    instrux + 5236,
    instrux + 5237,
    instrux + 5238,
    NULL
};

static const struct itemplate * const itbl_evex_map2_b9[] = {
    instrux + 5239,
    instrux + 5240,
    NULL
};

static const struct itemplate * const itbl_evex_map2_ba[] = {
    instrux + 5275,
    instrux + 5276,
    instrux + 5277,
    instrux + 5278,
    instrux + 5279,
    instrux + 5280,
    NULL
};

static const struct itemplate * const itbl_evex_map2_bb[] = {
    instrux + 5281,
    instrux + 5282,
    NULL
};

static const struct itemplate * const itbl_evex_map2_bc[] = {
    instrux + 5317,
    instrux + 5318,
    instrux + 5319,
    instrux + 5320,
    instrux + 5321,
    instrux + 5322,
    NULL
};

static const struct itemplate * const itbl_evex_map2_bd[] = {
    instrux + 5323,
    instrux + 5324,
    NULL
};

static const struct itemplate * const itbl_evex_map2_be[] = {
    instrux + 5341,
    instrux + 5342,
    instrux + 5343,
    instrux + 5344,
    instrux + 5345,
    instrux + 5346,
    NULL
};

static const struct itemplate * const itbl_evex_map2_bf[] = {
    instrux + 5347,
    instrux + 5348,
    NULL
};

static const struct itemplate * const itbl_evex_map2_c4[] = {
    instrux + 5961,
    instrux + 5962,
    instrux + 5963,
    instrux + 5964,
    instrux + 5965,
    instrux + 5966,
    NULL
};

static const struct itemplate * const itbl_evex_map2_c6[] = {
    instrux + 5363,
    instrux + 5364,
    instrux + 5367,
    instrux + 5368,
    instrux + 6632,
    instrux + 6633,
    instrux + 6636,
    instrux + 6637,
    NULL
};

static const struct itemplate * const itbl_evex_map2_c7[] = {
    instrux + 5365,
    instrux + 5366,
    instrux + 5369,
    instrux + 5370,
    instrux + 6634,
    instrux + 6635,
    instrux + 6638,
    instrux + 6639,
    NULL
};

static const struct itemplate * const itbl_evex_map2_c8[] = {
    instrux + 5168,
    instrux + 5169,
    NULL
};

static const struct itemplate * const itbl_evex_map2_ca[] = {
    instrux + 6582,
    instrux + 6583,
    NULL
};

static const struct itemplate * const itbl_evex_map2_cb[] = {
    instrux + 6584,
    instrux + 6585,
    NULL
};

static const struct itemplate * const itbl_evex_map2_cc[] = {
    instrux + 6614,
    instrux + 6615,
    NULL
};

static const struct itemplate * const itbl_evex_map2_cd[] = {
    instrux + 6616,
    instrux + 6617,
    NULL
};

static const struct itemplate * const itbl_evex_map2_cf[] = {
    instrux + 6727,
    instrux + 6728,
    instrux + 6729,
    NULL
};

static const struct itemplate * const itbl_evex_map2_d2[] = {
    instrux + 7345,
    instrux + 7346,
    instrux + 7347,
    instrux + 7351,
    instrux + 7352,
    instrux + 7353,
    instrux + 7357,
    instrux + 7358,
    instrux + 7359,
    NULL
};

static const struct itemplate * const itbl_evex_map2_d3[] = {
    instrux + 7348,
    instrux + 7349,
    instrux + 7350,
    instrux + 7354,
    instrux + 7355,
    instrux + 7356,
    instrux + 7360,
    instrux + 7361,
    instrux + 7362,
    NULL
};

static const struct itemplate * const itbl_evex_map2_da[] = {
    instrux + 4409,
    instrux + 4410,
    instrux + 4411,
    instrux + 4414,
    instrux + 4415,
    instrux + 4416,
    NULL
};

static const struct itemplate * const itbl_evex_map2_dc[] = {
    instrux + 3102,
    instrux + 3103,
    instrux + 3110,
    NULL
};

static const struct itemplate * const itbl_evex_map2_dd[] = {
    instrux + 3104,
    instrux + 3105,
    instrux + 3111,
    NULL
};

static const struct itemplate * const itbl_evex_map2_de[] = {
    instrux + 3106,
    instrux + 3107,
    instrux + 3112,
    NULL
};

static const struct itemplate * const itbl_evex_map2_df[] = {
    instrux + 3108,
    instrux + 3109,
    instrux + 3113,
    NULL
};

static const struct itemplate * const itbl_evex_map2_e0[] = {
    instrux + 2595,
    instrux + 2611,
    NULL
};

static const struct itemplate * const itbl_evex_map2_e1[] = {
    instrux + 2592,
    instrux + 2608,
    NULL
};

static const struct itemplate * const itbl_evex_map2_e2[] = {
    instrux + 2585,
    instrux + 2601,
    NULL
};

static const struct itemplate * const itbl_evex_map2_e3[] = {
    instrux + 2589,
    instrux + 2605,
    NULL
};

static const struct itemplate * const itbl_evex_map2_e4[] = {
    instrux + 2599,
    instrux + 2615,
    NULL
};

static const struct itemplate * const itbl_evex_map2_e5[] = {
    instrux + 2594,
    instrux + 2610,
    NULL
};

static const struct itemplate * const itbl_evex_map2_e6[] = {
    instrux + 2588,
    instrux + 2604,
    NULL
};

static const struct itemplate * const itbl_evex_map2_e7[] = {
    instrux + 2584,
    instrux + 2600,
    NULL
};

static const struct itemplate * const itbl_evex_map2_e8[] = {
    instrux + 2598,
    instrux + 2614,
    NULL
};

static const struct itemplate * const itbl_evex_map2_e9[] = {
    instrux + 2593,
    instrux + 2609,
    NULL
};

static const struct itemplate * const itbl_evex_map2_ea[] = {
    instrux + 2596,
    instrux + 2612,
    NULL
};

static const struct itemplate * const itbl_evex_map2_eb[] = {
    instrux + 2597,
    instrux + 2613,
    NULL
};

static const struct itemplate * const itbl_evex_map2_ec[] = {
    instrux + 2587,
    instrux + 2603,
    NULL
};

static const struct itemplate * const itbl_evex_map2_ed[] = {
    instrux + 2591,
    instrux + 2607,
    NULL
};

static const struct itemplate * const itbl_evex_map2_ee[] = {
    instrux + 2590,
    instrux + 2606,
    NULL
};

static const struct itemplate * const itbl_evex_map2_ef[] = {
    instrux + 2586,
    instrux + 2602,
    NULL
};

static const struct itemplate * const itbl_evex_map2_f2[] = {
    instrux + 918,
    instrux + 919,
    NULL
};

static const struct itemplate * const itbl_evex_map2_f3[] = {
    instrux + 926,
    instrux + 927,
    instrux + 930,
    instrux + 931,
    instrux + 934,
    instrux + 935,
    NULL
};

static const struct itemplate * const itbl_evex_map2_f5[] = {
    instrux + 938,
    instrux + 939,
    instrux + 941,
    instrux + 943,
    instrux + 945,
    instrux + 947,
    NULL
};

static const struct itemplate * const itbl_evex_map2_f6[] = {
    instrux + 779,
    instrux + 781,
    NULL
};

static const struct itemplate * const itbl_evex_map2_f7[] = {
    instrux + 710,
    instrux + 711,
    instrux + 712,
    instrux + 714,
    instrux + 715,
    instrux + 716,
    instrux + 922,
    instrux + 923,
    NULL
};

static const struct itemplate * const * const itbl_evex_map2[256] = {
    itbl_evex_map2_00,
    NULL,
    NULL,
    NULL,
    itbl_evex_map2_04,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map2_0b,
    itbl_evex_map2_0c,
    itbl_evex_map2_0d,
    NULL,
    NULL,
    itbl_evex_map2_10,
    itbl_evex_map2_11,
    itbl_evex_map2_12,
    itbl_evex_map2_13,
    itbl_evex_map2_14,
    itbl_evex_map2_15,
    itbl_evex_map2_16,
    NULL,
    itbl_evex_map2_18,
    itbl_evex_map2_19,
    itbl_evex_map2_1a,
    itbl_evex_map2_1b,
    itbl_evex_map2_1c,
    itbl_evex_map2_1d,
    itbl_evex_map2_1e,
    itbl_evex_map2_1f,
    itbl_evex_map2_20,
    itbl_evex_map2_21,
    itbl_evex_map2_22,
    itbl_evex_map2_23,
    itbl_evex_map2_24,
    itbl_evex_map2_25,
    itbl_evex_map2_26,
    itbl_evex_map2_27,
    itbl_evex_map2_28,
    itbl_evex_map2_29,
    itbl_evex_map2_2a,
    itbl_evex_map2_2b,
    itbl_evex_map2_2c,
    itbl_evex_map2_2d,
    NULL,
    NULL,
    itbl_evex_map2_30,
    itbl_evex_map2_31,
    itbl_evex_map2_32,
    itbl_evex_map2_33,
    itbl_evex_map2_34,
    itbl_evex_map2_35,
    itbl_evex_map2_36,
    itbl_evex_map2_37,
    itbl_evex_map2_38,
    itbl_evex_map2_39,
    itbl_evex_map2_3a,
    itbl_evex_map2_3b,
    itbl_evex_map2_3c,
    itbl_evex_map2_3d,
    itbl_evex_map2_3e,
    itbl_evex_map2_3f,
    itbl_evex_map2_40,
    NULL,
    itbl_evex_map2_42,
    itbl_evex_map2_43,
    itbl_evex_map2_44,
    itbl_evex_map2_45,
    itbl_evex_map2_46,
    itbl_evex_map2_47,
    NULL,
    itbl_evex_map2_49,
    itbl_evex_map2_4a,
    itbl_evex_map2_4b,
    itbl_evex_map2_4c,
    itbl_evex_map2_4d,
    itbl_evex_map2_4e,
    itbl_evex_map2_4f,
    itbl_evex_map2_50,
    itbl_evex_map2_51,
    itbl_evex_map2_52,
    itbl_evex_map2_53,
    itbl_evex_map2_54,
    itbl_evex_map2_55,
    NULL,
    NULL,
    itbl_evex_map2_58,
    itbl_evex_map2_59,
    itbl_evex_map2_5a,
    itbl_evex_map2_5b,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map2_62,
    itbl_evex_map2_63,
    itbl_evex_map2_64,
    itbl_evex_map2_65,
    itbl_evex_map2_66,
    itbl_evex_map2_67,
    itbl_evex_map2_68,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map2_6d,
    NULL,
    NULL,
    itbl_evex_map2_70,
    itbl_evex_map2_71,
    itbl_evex_map2_72,
    itbl_evex_map2_73,
    itbl_evex_map2_74,
    itbl_evex_map2_75,
    itbl_evex_map2_76,
    itbl_evex_map2_77,
    itbl_evex_map2_78,
    itbl_evex_map2_79,
    itbl_evex_map2_7a,
    itbl_evex_map2_7b,
    itbl_evex_map2_7c,
    itbl_evex_map2_7d,
    itbl_evex_map2_7e,
    itbl_evex_map2_7f,
    NULL,
    NULL,
    NULL,
    itbl_evex_map2_83,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map2_88,
    itbl_evex_map2_89,
    itbl_evex_map2_8a,
    itbl_evex_map2_8b,
    NULL,
    itbl_evex_map2_8d,
    NULL,
    itbl_evex_map2_8f,
    itbl_evex_map2_90,
    itbl_evex_map2_91,
    itbl_evex_map2_92,
    itbl_evex_map2_93,
    NULL,
    NULL,
    itbl_evex_map2_96,
    itbl_evex_map2_97,
    itbl_evex_map2_98,
    itbl_evex_map2_99,
    itbl_evex_map2_9a,
    itbl_evex_map2_9b,
    itbl_evex_map2_9c,
    itbl_evex_map2_9d,
    itbl_evex_map2_9e,
    itbl_evex_map2_9f,
    itbl_evex_map2_a0,
    itbl_evex_map2_a1,
    itbl_evex_map2_a2,
    itbl_evex_map2_a3,
    NULL,
    NULL,
    itbl_evex_map2_a6,
    itbl_evex_map2_a7,
    itbl_evex_map2_a8,
    itbl_evex_map2_a9,
    itbl_evex_map2_aa,
    itbl_evex_map2_ab,
    itbl_evex_map2_ac,
    itbl_evex_map2_ad,
    itbl_evex_map2_ae,
    itbl_evex_map2_af,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map2_b4,
    itbl_evex_map2_b5,
    itbl_evex_map2_b6,
    itbl_evex_map2_b7,
    itbl_evex_map2_b8,
    itbl_evex_map2_b9,
    itbl_evex_map2_ba,
    itbl_evex_map2_bb,
    itbl_evex_map2_bc,
    itbl_evex_map2_bd,
    itbl_evex_map2_be,
    itbl_evex_map2_bf,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map2_c4,
    NULL,
    itbl_evex_map2_c6,
    itbl_evex_map2_c7,
    itbl_evex_map2_c8,
    NULL,
    itbl_evex_map2_ca,
    itbl_evex_map2_cb,
    itbl_evex_map2_cc,
    itbl_evex_map2_cd,
    NULL,
    itbl_evex_map2_cf,
    NULL,
    NULL,
    itbl_evex_map2_d2,
    itbl_evex_map2_d3,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map2_da,
    NULL,
    itbl_evex_map2_dc,
    itbl_evex_map2_dd,
    itbl_evex_map2_de,
    itbl_evex_map2_df,
    itbl_evex_map2_e0,
    itbl_evex_map2_e1,
    itbl_evex_map2_e2,
    itbl_evex_map2_e3,
    itbl_evex_map2_e4,
    itbl_evex_map2_e5,
    itbl_evex_map2_e6,
    itbl_evex_map2_e7,
    itbl_evex_map2_e8,
    itbl_evex_map2_e9,
    itbl_evex_map2_ea,
    itbl_evex_map2_eb,
    itbl_evex_map2_ec,
    itbl_evex_map2_ed,
    itbl_evex_map2_ee,
    itbl_evex_map2_ef,
    NULL,
    NULL,
    itbl_evex_map2_f2,
    itbl_evex_map2_f3,
    NULL,
    itbl_evex_map2_f5,
    itbl_evex_map2_f6,
    itbl_evex_map2_f7,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_evex_map3_00[] = {
    instrux + 6008,
    instrux + 6009,
    NULL
};

static const struct itemplate * const itbl_evex_map3_01[] = {
    instrux + 6002,
    instrux + 6003,
    NULL
};

static const struct itemplate * const itbl_evex_map3_03[] = {
    instrux + 4603,
    instrux + 4604,
    instrux + 4605,
    instrux + 4606,
    instrux + 4607,
    instrux + 4608,
    NULL
};

static const struct itemplate * const itbl_evex_map3_04[] = {
    instrux + 5996,
    instrux + 5997,
    instrux + 5998,
    NULL
};

static const struct itemplate * const itbl_evex_map3_05[] = {
    instrux + 5990,
    instrux + 5991,
    instrux + 5992,
    NULL
};

static const struct itemplate * const itbl_evex_map3_07[] = {
    instrux + 6899,
    instrux + 6901,
    instrux + 6905,
    instrux + 6912,
    NULL
};

static const struct itemplate * const itbl_evex_map3_08[] = {
    instrux + 6597,
    instrux + 6598,
    instrux + 6599,
    instrux + 6600,
    instrux + 6601,
    instrux + 6602,
    instrux + 7145,
    instrux + 7146,
    instrux + 7147,
    instrux + 7258,
    instrux + 7259,
    instrux + 7260,
    NULL
};

static const struct itemplate * const itbl_evex_map3_09[] = {
    instrux + 6594,
    instrux + 6595,
    instrux + 6596,
    NULL
};

static const struct itemplate * const itbl_evex_map3_0a[] = {
    instrux + 6604,
    instrux + 6605,
    instrux + 7148,
    NULL
};

static const struct itemplate * const itbl_evex_map3_0b[] = {
    instrux + 6603,
    NULL
};

static const struct itemplate * const itbl_evex_map3_0f[] = {
    instrux + 5601,
    instrux + 5602,
    instrux + 5603,
    NULL
};

static const struct itemplate * const itbl_evex_map3_14[] = {
    instrux + 6045,
    instrux + 6046,
    instrux + 6047,
    instrux + 6048,
    instrux + 6049,
    NULL
};

static const struct itemplate * const itbl_evex_map3_15[] = {
    instrux + 6052,
    instrux + 6053,
    instrux + 6054,
    instrux + 6055,
    NULL
};

static const struct itemplate * const itbl_evex_map3_16[] = {
    instrux + 6050,
    instrux + 6051,
    NULL
};

static const struct itemplate * const itbl_evex_map3_17[] = {
    instrux + 5206,
    instrux + 5207,
    instrux + 5208,
    NULL
};

static const struct itemplate * const itbl_evex_map3_18[] = {
    instrux + 5393,
    instrux + 5394,
    instrux + 5396,
    instrux + 5397,
    NULL
};

static const struct itemplate * const itbl_evex_map3_19[] = {
    instrux + 5182,
    instrux + 5183,
    instrux + 5184,
    instrux + 5185,
    instrux + 5188,
    instrux + 5189,
    instrux + 5190,
    instrux + 5191,
    NULL
};

static const struct itemplate * const itbl_evex_map3_1a[] = {
    instrux + 5395,
    instrux + 5398,
    NULL
};

static const struct itemplate * const itbl_evex_map3_1b[] = {
    instrux + 5186,
    instrux + 5187,
    instrux + 5192,
    instrux + 5193,
    NULL
};

static const struct itemplate * const itbl_evex_map3_1d[] = {
    instrux + 5074,
    instrux + 5075,
    instrux + 5076,
    instrux + 5077,
    instrux + 5078,
    instrux + 5079,
    instrux + 6963,
    instrux + 6964,
    instrux + 6965,
    instrux + 6966,
    instrux + 6967,
    instrux + 6968,
    NULL
};

static const struct itemplate * const itbl_evex_map3_1e[] = {
    instrux + 5721,
    instrux + 5722,
    instrux + 5723,
    instrux + 5724,
    instrux + 5725,
    instrux + 5726,
    instrux + 5745,
    instrux + 5746,
    instrux + 5747,
    instrux + 5748,
    instrux + 5749,
    instrux + 5750,
    instrux + 5769,
    instrux + 5770,
    instrux + 5771,
    instrux + 5772,
    instrux + 5773,
    instrux + 5774,
    instrux + 5793,
    instrux + 5794,
    instrux + 5795,
    instrux + 5796,
    instrux + 5797,
    instrux + 5798,
    instrux + 5817,
    instrux + 5818,
    instrux + 5819,
    instrux + 5820,
    instrux + 5821,
    instrux + 5822,
    instrux + 5841,
    instrux + 5842,
    instrux + 5843,
    instrux + 5844,
    instrux + 5845,
    instrux + 5846,
    instrux + 5865,
    instrux + 5866,
    instrux + 5867,
    instrux + 5868,
    instrux + 5869,
    instrux + 5870,
    instrux + 5889,
    instrux + 5890,
    instrux + 5891,
    instrux + 5892,
    instrux + 5893,
    instrux + 5894,
    instrux + 5913,
    instrux + 5914,
    instrux + 5915,
    instrux + 5916,
    instrux + 5917,
    instrux + 5918,
    instrux + 5937,
    instrux + 5938,
    instrux + 5939,
    instrux + 5940,
    instrux + 5941,
    instrux + 5942,
    NULL
};

static const struct itemplate * const itbl_evex_map3_1f[] = {
    instrux + 5712,
    instrux + 5713,
    instrux + 5714,
    instrux + 5715,
    instrux + 5716,
    instrux + 5717,
    instrux + 5736,
    instrux + 5737,
    instrux + 5738,
    instrux + 5739,
    instrux + 5740,
    instrux + 5741,
    instrux + 5760,
    instrux + 5761,
    instrux + 5762,
    instrux + 5763,
    instrux + 5764,
    instrux + 5765,
    instrux + 5784,
    instrux + 5785,
    instrux + 5786,
    instrux + 5787,
    instrux + 5788,
    instrux + 5789,
    instrux + 5808,
    instrux + 5809,
    instrux + 5810,
    instrux + 5811,
    instrux + 5812,
    instrux + 5813,
    instrux + 5832,
    instrux + 5833,
    instrux + 5834,
    instrux + 5835,
    instrux + 5836,
    instrux + 5837,
    instrux + 5856,
    instrux + 5857,
    instrux + 5858,
    instrux + 5859,
    instrux + 5860,
    instrux + 5861,
    instrux + 5880,
    instrux + 5881,
    instrux + 5882,
    instrux + 5883,
    instrux + 5884,
    instrux + 5885,
    instrux + 5904,
    instrux + 5905,
    instrux + 5906,
    instrux + 5907,
    instrux + 5908,
    instrux + 5909,
    instrux + 5928,
    instrux + 5929,
    instrux + 5930,
    instrux + 5931,
    instrux + 5932,
    instrux + 5933,
    NULL
};

static const struct itemplate * const itbl_evex_map3_20[] = {
    instrux + 6071,
    instrux + 6072,
    NULL
};

static const struct itemplate * const itbl_evex_map3_21[] = {
    instrux + 5405,
    NULL
};

static const struct itemplate * const itbl_evex_map3_22[] = {
    instrux + 6073,
    instrux + 6074,
    NULL
};

static const struct itemplate * const itbl_evex_map3_23[] = {
    instrux + 6646,
    instrux + 6647,
    instrux + 6648,
    instrux + 6649,
    NULL
};

static const struct itemplate * const itbl_evex_map3_25[] = {
    instrux + 6506,
    instrux + 6507,
    instrux + 6508,
    instrux + 6509,
    instrux + 6510,
    instrux + 6511,
    NULL
};

static const struct itemplate * const itbl_evex_map3_26[] = {
    instrux + 5385,
    instrux + 5386,
    instrux + 5387,
    instrux + 5388,
    instrux + 5389,
    instrux + 5390,
    instrux + 7115,
    instrux + 7116,
    instrux + 7117,
    instrux + 7240,
    instrux + 7241,
    instrux + 7242,
    NULL
};

static const struct itemplate * const itbl_evex_map3_27[] = {
    instrux + 5391,
    instrux + 5392,
    instrux + 7118,
    NULL
};

static const struct itemplate * const itbl_evex_map3_30[] = {
    instrux + 4558,
    instrux + 4560,
    NULL
};

static const struct itemplate * const itbl_evex_map3_31[] = {
    instrux + 4562,
    instrux + 4564,
    NULL
};

static const struct itemplate * const itbl_evex_map3_32[] = {
    instrux + 4550,
    instrux + 4552,
    NULL
};

static const struct itemplate * const itbl_evex_map3_33[] = {
    instrux + 4554,
    instrux + 4556,
    NULL
};

static const struct itemplate * const itbl_evex_map3_38[] = {
    instrux + 5399,
    instrux + 5400,
    instrux + 5402,
    instrux + 5403,
    NULL
};

static const struct itemplate * const itbl_evex_map3_39[] = {
    instrux + 5194,
    instrux + 5195,
    instrux + 5196,
    instrux + 5197,
    instrux + 5200,
    instrux + 5201,
    instrux + 5202,
    instrux + 5203,
    NULL
};

static const struct itemplate * const itbl_evex_map3_3a[] = {
    instrux + 5401,
    instrux + 5404,
    NULL
};

static const struct itemplate * const itbl_evex_map3_3b[] = {
    instrux + 5198,
    instrux + 5199,
    instrux + 5204,
    instrux + 5205,
    NULL
};

static const struct itemplate * const itbl_evex_map3_3e[] = {
    instrux + 5718,
    instrux + 5719,
    instrux + 5720,
    instrux + 5727,
    instrux + 5728,
    instrux + 5729,
    instrux + 5742,
    instrux + 5743,
    instrux + 5744,
    instrux + 5751,
    instrux + 5752,
    instrux + 5753,
    instrux + 5766,
    instrux + 5767,
    instrux + 5768,
    instrux + 5775,
    instrux + 5776,
    instrux + 5777,
    instrux + 5790,
    instrux + 5791,
    instrux + 5792,
    instrux + 5799,
    instrux + 5800,
    instrux + 5801,
    instrux + 5814,
    instrux + 5815,
    instrux + 5816,
    instrux + 5823,
    instrux + 5824,
    instrux + 5825,
    instrux + 5838,
    instrux + 5839,
    instrux + 5840,
    instrux + 5847,
    instrux + 5848,
    instrux + 5849,
    instrux + 5862,
    instrux + 5863,
    instrux + 5864,
    instrux + 5871,
    instrux + 5872,
    instrux + 5873,
    instrux + 5886,
    instrux + 5887,
    instrux + 5888,
    instrux + 5895,
    instrux + 5896,
    instrux + 5897,
    instrux + 5910,
    instrux + 5911,
    instrux + 5912,
    instrux + 5919,
    instrux + 5920,
    instrux + 5921,
    instrux + 5934,
    instrux + 5935,
    instrux + 5936,
    instrux + 5943,
    instrux + 5944,
    instrux + 5945,
    NULL
};

static const struct itemplate * const itbl_evex_map3_3f[] = {
    instrux + 5709,
    instrux + 5710,
    instrux + 5711,
    instrux + 5730,
    instrux + 5731,
    instrux + 5732,
    instrux + 5733,
    instrux + 5734,
    instrux + 5735,
    instrux + 5754,
    instrux + 5755,
    instrux + 5756,
    instrux + 5757,
    instrux + 5758,
    instrux + 5759,
    instrux + 5778,
    instrux + 5779,
    instrux + 5780,
    instrux + 5781,
    instrux + 5782,
    instrux + 5783,
    instrux + 5802,
    instrux + 5803,
    instrux + 5804,
    instrux + 5805,
    instrux + 5806,
    instrux + 5807,
    instrux + 5826,
    instrux + 5827,
    instrux + 5828,
    instrux + 5829,
    instrux + 5830,
    instrux + 5831,
    instrux + 5850,
    instrux + 5851,
    instrux + 5852,
    instrux + 5853,
    instrux + 5854,
    instrux + 5855,
    instrux + 5874,
    instrux + 5875,
    instrux + 5876,
    instrux + 5877,
    instrux + 5878,
    instrux + 5879,
    instrux + 5898,
    instrux + 5899,
    instrux + 5900,
    instrux + 5901,
    instrux + 5902,
    instrux + 5903,
    instrux + 5922,
    instrux + 5923,
    instrux + 5924,
    instrux + 5925,
    instrux + 5926,
    instrux + 5927,
    instrux + 5946,
    instrux + 5947,
    instrux + 5948,
    NULL
};

static const struct itemplate * const itbl_evex_map3_42[] = {
    instrux + 5157,
    instrux + 5158,
    instrux + 5159,
    instrux + 7324,
    instrux + 7325,
    instrux + 7326,
    NULL
};

static const struct itemplate * const itbl_evex_map3_43[] = {
    instrux + 6650,
    instrux + 6651,
    instrux + 6652,
    instrux + 6653,
    NULL
};

static const struct itemplate * const itbl_evex_map3_44[] = {
    instrux + 3811,
    instrux + 3812,
    instrux + 3813,
    instrux + 3814,
    instrux + 3815,
    instrux + 3816,
    instrux + 3817,
    instrux + 3818,
    instrux + 3819,
    instrux + 3820,
    instrux + 3821,
    instrux + 3822,
    instrux + 3823,
    instrux + 3824,
    instrux + 3825,
    NULL
};

static const struct itemplate * const itbl_evex_map3_50[] = {
    instrux + 6566,
    instrux + 6567,
    instrux + 6568,
    instrux + 6569,
    instrux + 6570,
    instrux + 6571,
    NULL
};

static const struct itemplate * const itbl_evex_map3_51[] = {
    instrux + 6572,
    instrux + 6573,
    NULL
};

static const struct itemplate * const itbl_evex_map3_52[] = {
    instrux + 7363,
    instrux + 7364,
    instrux + 7365,
    instrux + 7366,
    instrux + 7367,
    instrux + 7368,
    instrux + 7369,
    instrux + 7370,
    instrux + 7371,
    instrux + 7372,
    instrux + 7373,
    instrux + 7374,
    NULL
};

static const struct itemplate * const itbl_evex_map3_53[] = {
    instrux + 7375,
    instrux + 7376,
    instrux + 7377,
    NULL
};

static const struct itemplate * const itbl_evex_map3_54[] = {
    instrux + 5209,
    instrux + 5210,
    instrux + 5211,
    instrux + 5212,
    instrux + 5213,
    instrux + 5214,
    NULL
};

static const struct itemplate * const itbl_evex_map3_55[] = {
    instrux + 5215,
    instrux + 5216,
    NULL
};

static const struct itemplate * const itbl_evex_map3_56[] = {
    instrux + 6586,
    instrux + 6587,
    instrux + 6588,
    instrux + 6589,
    instrux + 6590,
    instrux + 6591,
    instrux + 7141,
    instrux + 7142,
    instrux + 7143,
    instrux + 7255,
    instrux + 7256,
    instrux + 7257,
    NULL
};

static const struct itemplate * const itbl_evex_map3_57[] = {
    instrux + 6592,
    instrux + 6593,
    instrux + 7144,
    NULL
};

static const struct itemplate * const itbl_evex_map3_66[] = {
    instrux + 5349,
    instrux + 5350,
    instrux + 5351,
    instrux + 5352,
    instrux + 5353,
    instrux + 5354,
    instrux + 7107,
    instrux + 7108,
    instrux + 7109,
    instrux + 7234,
    instrux + 7235,
    instrux + 7236,
    NULL
};

static const struct itemplate * const itbl_evex_map3_67[] = {
    instrux + 5355,
    instrux + 5356,
    instrux + 7110,
    NULL
};

static const struct itemplate * const itbl_evex_map3_70[] = {
    instrux + 6748,
    instrux + 6749,
    instrux + 6750,
    NULL
};

static const struct itemplate * const itbl_evex_map3_71[] = {
    instrux + 6751,
    instrux + 6752,
    instrux + 6753,
    instrux + 6754,
    instrux + 6755,
    instrux + 6756,
    NULL
};

static const struct itemplate * const itbl_evex_map3_72[] = {
    instrux + 6766,
    instrux + 6767,
    instrux + 6768,
    NULL
};

static const struct itemplate * const itbl_evex_map3_73[] = {
    instrux + 6769,
    instrux + 6770,
    instrux + 6771,
    instrux + 6772,
    instrux + 6773,
    instrux + 6774,
    NULL
};

static const struct itemplate * const itbl_evex_map3_77[] = {
    instrux + 6903,
    instrux + 6907,
    NULL
};

static const struct itemplate * const itbl_evex_map3_c2[] = {
    instrux + 6925,
    instrux + 6926,
    instrux + 6927,
    instrux + 6928,
    instrux + 7191,
    instrux + 7192,
    instrux + 7193,
    NULL
};

static const struct itemplate * const itbl_evex_map3_ce[] = {
    instrux + 6721,
    instrux + 6722,
    instrux + 6723,
    NULL
};

static const struct itemplate * const itbl_evex_map3_cf[] = {
    instrux + 6715,
    instrux + 6716,
    instrux + 6717,
    NULL
};

static const struct itemplate * const itbl_evex_map3_f0[] = {
    instrux + 709,
    instrux + 713,
    NULL
};

static const struct itemplate * const * const itbl_evex_map3[256] = {
    itbl_evex_map3_00,
    itbl_evex_map3_01,
    NULL,
    itbl_evex_map3_03,
    itbl_evex_map3_04,
    itbl_evex_map3_05,
    NULL,
    itbl_evex_map3_07,
    itbl_evex_map3_08,
    itbl_evex_map3_09,
    itbl_evex_map3_0a,
    itbl_evex_map3_0b,
    NULL,
    NULL,
    NULL,
    itbl_evex_map3_0f,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map3_14,
    itbl_evex_map3_15,
    itbl_evex_map3_16,
    itbl_evex_map3_17,
    itbl_evex_map3_18,
    itbl_evex_map3_19,
    itbl_evex_map3_1a,
    itbl_evex_map3_1b,
    NULL,
    itbl_evex_map3_1d,
    itbl_evex_map3_1e,
    itbl_evex_map3_1f,
    itbl_evex_map3_20,
    itbl_evex_map3_21,
    itbl_evex_map3_22,
    itbl_evex_map3_23,
    NULL,
    itbl_evex_map3_25,
    itbl_evex_map3_26,
    itbl_evex_map3_27,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map3_30,
    itbl_evex_map3_31,
    itbl_evex_map3_32,
    itbl_evex_map3_33,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map3_38,
    itbl_evex_map3_39,
    itbl_evex_map3_3a,
    itbl_evex_map3_3b,
    NULL,
    NULL,
    itbl_evex_map3_3e,
    itbl_evex_map3_3f,
    NULL,
    NULL,
    itbl_evex_map3_42,
    itbl_evex_map3_43,
    itbl_evex_map3_44,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map3_50,
    itbl_evex_map3_51,
    itbl_evex_map3_52,
    itbl_evex_map3_53,
    itbl_evex_map3_54,
    itbl_evex_map3_55,
    itbl_evex_map3_56,
    itbl_evex_map3_57,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map3_66,
    itbl_evex_map3_67,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map3_70,
    itbl_evex_map3_71,
    itbl_evex_map3_72,
    itbl_evex_map3_73,
    NULL,
    NULL,
    NULL,
    itbl_evex_map3_77,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map3_c2,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map3_ce,
    itbl_evex_map3_cf,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map3_f0,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_evex_map4_00[] = {
    instrux + 66,
    instrux + 67,
    NULL
};

static const struct itemplate * const itbl_evex_map4_01[] = {
    instrux + 68,
    instrux + 69,
    instrux + 70,
    instrux + 71,
    instrux + 72,
    instrux + 73,
    NULL
};

static const struct itemplate * const itbl_evex_map4_02[] = {
    instrux + 58,
    instrux + 59,
    NULL
};

static const struct itemplate * const itbl_evex_map4_03[] = {
    instrux + 60,
    instrux + 61,
    instrux + 62,
    instrux + 63,
    instrux + 64,
    instrux + 65,
    NULL
};

static const struct itemplate * const itbl_evex_map4_08[] = {
    instrux + 115,
    instrux + 116,
    NULL
};

static const struct itemplate * const itbl_evex_map4_09[] = {
    instrux + 117,
    instrux + 118,
    instrux + 119,
    instrux + 120,
    instrux + 121,
    instrux + 122,
    NULL
};

static const struct itemplate * const itbl_evex_map4_0a[] = {
    instrux + 107,
    instrux + 108,
    NULL
};

static const struct itemplate * const itbl_evex_map4_0b[] = {
    instrux + 109,
    instrux + 110,
    instrux + 111,
    instrux + 112,
    instrux + 113,
    instrux + 114,
    NULL
};

static const struct itemplate * const itbl_evex_map4_10[] = {
    instrux + 164,
    instrux + 165,
    NULL
};

static const struct itemplate * const itbl_evex_map4_11[] = {
    instrux + 166,
    instrux + 167,
    instrux + 168,
    instrux + 169,
    instrux + 170,
    instrux + 171,
    NULL
};

static const struct itemplate * const itbl_evex_map4_12[] = {
    instrux + 156,
    instrux + 157,
    NULL
};

static const struct itemplate * const itbl_evex_map4_13[] = {
    instrux + 158,
    instrux + 159,
    instrux + 160,
    instrux + 161,
    instrux + 162,
    instrux + 163,
    NULL
};

static const struct itemplate * const itbl_evex_map4_18[] = {
    instrux + 213,
    instrux + 214,
    NULL
};

static const struct itemplate * const itbl_evex_map4_19[] = {
    instrux + 215,
    instrux + 216,
    instrux + 217,
    instrux + 218,
    instrux + 219,
    instrux + 220,
    NULL
};

static const struct itemplate * const itbl_evex_map4_1a[] = {
    instrux + 205,
    instrux + 206,
    NULL
};

static const struct itemplate * const itbl_evex_map4_1b[] = {
    instrux + 207,
    instrux + 208,
    instrux + 209,
    instrux + 210,
    instrux + 211,
    instrux + 212,
    NULL
};

static const struct itemplate * const itbl_evex_map4_20[] = {
    instrux + 262,
    instrux + 263,
    NULL
};

static const struct itemplate * const itbl_evex_map4_21[] = {
    instrux + 264,
    instrux + 265,
    instrux + 266,
    instrux + 267,
    instrux + 268,
    instrux + 269,
    NULL
};

static const struct itemplate * const itbl_evex_map4_22[] = {
    instrux + 254,
    instrux + 255,
    NULL
};

static const struct itemplate * const itbl_evex_map4_23[] = {
    instrux + 256,
    instrux + 257,
    instrux + 258,
    instrux + 259,
    instrux + 260,
    instrux + 261,
    NULL
};

static const struct itemplate * const itbl_evex_map4_24[] = {
    instrux + 844,
    instrux + 845,
    instrux + 846,
    instrux + 847,
    instrux + 848,
    instrux + 849,
    NULL
};

static const struct itemplate * const itbl_evex_map4_28[] = {
    instrux + 311,
    instrux + 312,
    NULL
};

static const struct itemplate * const itbl_evex_map4_29[] = {
    instrux + 313,
    instrux + 314,
    instrux + 315,
    instrux + 316,
    instrux + 317,
    instrux + 318,
    NULL
};

static const struct itemplate * const itbl_evex_map4_2a[] = {
    instrux + 303,
    instrux + 304,
    NULL
};

static const struct itemplate * const itbl_evex_map4_2b[] = {
    instrux + 305,
    instrux + 306,
    instrux + 307,
    instrux + 308,
    instrux + 309,
    instrux + 310,
    NULL
};

static const struct itemplate * const itbl_evex_map4_2c[] = {
    instrux + 862,
    instrux + 863,
    instrux + 864,
    instrux + 865,
    instrux + 866,
    instrux + 867,
    NULL
};

static const struct itemplate * const itbl_evex_map4_30[] = {
    instrux + 360,
    instrux + 361,
    NULL
};

static const struct itemplate * const itbl_evex_map4_31[] = {
    instrux + 362,
    instrux + 363,
    instrux + 364,
    instrux + 365,
    instrux + 366,
    instrux + 367,
    NULL
};

static const struct itemplate * const itbl_evex_map4_32[] = {
    instrux + 352,
    instrux + 353,
    NULL
};

static const struct itemplate * const itbl_evex_map4_33[] = {
    instrux + 354,
    instrux + 355,
    instrux + 356,
    instrux + 357,
    instrux + 358,
    instrux + 359,
    NULL
};

static const struct itemplate * const itbl_evex_map4_38[] = {
    instrux + 1784,
    instrux + 1785,
    instrux + 1786,
    instrux + 1787,
    instrux + 1788,
    instrux + 1789,
    instrux + 1790,
    instrux + 1791,
    instrux + 1792,
    instrux + 1793,
    instrux + 1794,
    instrux + 1795,
    instrux + 1796,
    instrux + 1797,
    instrux + 1798,
    instrux + 1799,
    NULL
};

static const struct itemplate * const itbl_evex_map4_39[] = {
    instrux + 1800,
    instrux + 1801,
    instrux + 1802,
    instrux + 1803,
    instrux + 1804,
    instrux + 1805,
    instrux + 1806,
    instrux + 1807,
    instrux + 1808,
    instrux + 1809,
    instrux + 1810,
    instrux + 1811,
    instrux + 1812,
    instrux + 1813,
    instrux + 1814,
    instrux + 1815,
    instrux + 1816,
    instrux + 1817,
    instrux + 1818,
    instrux + 1819,
    instrux + 1820,
    instrux + 1821,
    instrux + 1822,
    instrux + 1823,
    instrux + 1824,
    instrux + 1825,
    instrux + 1826,
    instrux + 1827,
    instrux + 1828,
    instrux + 1829,
    instrux + 1830,
    instrux + 1831,
    instrux + 1832,
    instrux + 1833,
    instrux + 1834,
    instrux + 1835,
    instrux + 1836,
    instrux + 1837,
    instrux + 1838,
    instrux + 1839,
    instrux + 1840,
    instrux + 1841,
    instrux + 1842,
    instrux + 1843,
    instrux + 1844,
    instrux + 1845,
    instrux + 1846,
    instrux + 1847,
    NULL
};

static const struct itemplate * const itbl_evex_map4_3a[] = {
    instrux + 1848,
    instrux + 1849,
    instrux + 1850,
    instrux + 1851,
    instrux + 1852,
    instrux + 1853,
    instrux + 1854,
    instrux + 1855,
    instrux + 1856,
    instrux + 1857,
    instrux + 1858,
    instrux + 1859,
    instrux + 1860,
    instrux + 1861,
    instrux + 1862,
    instrux + 1863,
    NULL
};

static const struct itemplate * const itbl_evex_map4_3b[] = {
    instrux + 1864,
    instrux + 1865,
    instrux + 1866,
    instrux + 1867,
    instrux + 1868,
    instrux + 1869,
    instrux + 1870,
    instrux + 1871,
    instrux + 1872,
    instrux + 1873,
    instrux + 1874,
    instrux + 1875,
    instrux + 1876,
    instrux + 1877,
    instrux + 1878,
    instrux + 1879,
    instrux + 1880,
    instrux + 1881,
    instrux + 1882,
    instrux + 1883,
    instrux + 1884,
    instrux + 1885,
    instrux + 1886,
    instrux + 1887,
    instrux + 1888,
    instrux + 1889,
    instrux + 1890,
    instrux + 1891,
    instrux + 1892,
    instrux + 1893,
    instrux + 1894,
    instrux + 1895,
    instrux + 1896,
    instrux + 1897,
    instrux + 1898,
    instrux + 1899,
    instrux + 1900,
    instrux + 1901,
    instrux + 1902,
    instrux + 1903,
    instrux + 1904,
    instrux + 1905,
    instrux + 1906,
    instrux + 1907,
    instrux + 1908,
    instrux + 1909,
    instrux + 1910,
    instrux + 1911,
    NULL
};

static const struct itemplate * const itbl_evex_map4_40[] = {
    instrux + 2275,
    instrux + 2291,
    instrux + 2307,
    instrux + 2323,
    instrux + 2339,
    instrux + 2355,
    instrux + 2371,
    instrux + 2387,
    instrux + 2403,
    instrux + 2419,
    instrux + 2435,
    instrux + 2451,
    instrux + 2467,
    instrux + 2483,
    instrux + 2499,
    instrux + 2531,
    instrux + 2547,
    NULL
};

static const struct itemplate * const itbl_evex_map4_41[] = {
    instrux + 2272,
    instrux + 2288,
    instrux + 2304,
    instrux + 2320,
    instrux + 2336,
    instrux + 2352,
    instrux + 2368,
    instrux + 2384,
    instrux + 2400,
    instrux + 2416,
    instrux + 2432,
    instrux + 2448,
    instrux + 2464,
    instrux + 2480,
    instrux + 2496,
    instrux + 2528,
    instrux + 2544,
    NULL
};

static const struct itemplate * const itbl_evex_map4_42[] = {
    instrux + 2265,
    instrux + 2281,
    instrux + 2297,
    instrux + 2313,
    instrux + 2329,
    instrux + 2345,
    instrux + 2361,
    instrux + 2377,
    instrux + 2393,
    instrux + 2409,
    instrux + 2425,
    instrux + 2441,
    instrux + 2457,
    instrux + 2473,
    instrux + 2489,
    instrux + 2521,
    instrux + 2537,
    NULL
};

static const struct itemplate * const itbl_evex_map4_43[] = {
    instrux + 2269,
    instrux + 2285,
    instrux + 2301,
    instrux + 2317,
    instrux + 2333,
    instrux + 2349,
    instrux + 2365,
    instrux + 2381,
    instrux + 2397,
    instrux + 2413,
    instrux + 2429,
    instrux + 2445,
    instrux + 2461,
    instrux + 2477,
    instrux + 2493,
    instrux + 2525,
    instrux + 2541,
    NULL
};

static const struct itemplate * const itbl_evex_map4_44[] = {
    instrux + 2279,
    instrux + 2295,
    instrux + 2311,
    instrux + 2327,
    instrux + 2343,
    instrux + 2359,
    instrux + 2375,
    instrux + 2391,
    instrux + 2407,
    instrux + 2423,
    instrux + 2439,
    instrux + 2455,
    instrux + 2471,
    instrux + 2487,
    instrux + 2503,
    instrux + 2535,
    instrux + 2551,
    NULL
};

static const struct itemplate * const itbl_evex_map4_45[] = {
    instrux + 2274,
    instrux + 2290,
    instrux + 2306,
    instrux + 2322,
    instrux + 2338,
    instrux + 2354,
    instrux + 2370,
    instrux + 2386,
    instrux + 2402,
    instrux + 2418,
    instrux + 2434,
    instrux + 2450,
    instrux + 2466,
    instrux + 2482,
    instrux + 2498,
    instrux + 2530,
    instrux + 2546,
    NULL
};

static const struct itemplate * const itbl_evex_map4_46[] = {
    instrux + 2268,
    instrux + 2284,
    instrux + 2300,
    instrux + 2316,
    instrux + 2332,
    instrux + 2348,
    instrux + 2364,
    instrux + 2380,
    instrux + 2396,
    instrux + 2412,
    instrux + 2428,
    instrux + 2444,
    instrux + 2460,
    instrux + 2476,
    instrux + 2492,
    instrux + 2524,
    instrux + 2540,
    NULL
};

static const struct itemplate * const itbl_evex_map4_47[] = {
    instrux + 2264,
    instrux + 2280,
    instrux + 2296,
    instrux + 2312,
    instrux + 2328,
    instrux + 2344,
    instrux + 2360,
    instrux + 2376,
    instrux + 2392,
    instrux + 2408,
    instrux + 2424,
    instrux + 2440,
    instrux + 2456,
    instrux + 2472,
    instrux + 2488,
    instrux + 2520,
    instrux + 2536,
    NULL
};

static const struct itemplate * const itbl_evex_map4_48[] = {
    instrux + 2278,
    instrux + 2294,
    instrux + 2310,
    instrux + 2326,
    instrux + 2342,
    instrux + 2358,
    instrux + 2374,
    instrux + 2390,
    instrux + 2406,
    instrux + 2422,
    instrux + 2438,
    instrux + 2454,
    instrux + 2470,
    instrux + 2486,
    instrux + 2502,
    instrux + 2534,
    instrux + 2550,
    NULL
};

static const struct itemplate * const itbl_evex_map4_49[] = {
    instrux + 2273,
    instrux + 2289,
    instrux + 2305,
    instrux + 2321,
    instrux + 2337,
    instrux + 2353,
    instrux + 2369,
    instrux + 2385,
    instrux + 2401,
    instrux + 2417,
    instrux + 2433,
    instrux + 2449,
    instrux + 2465,
    instrux + 2481,
    instrux + 2497,
    instrux + 2529,
    instrux + 2545,
    NULL
};

static const struct itemplate * const itbl_evex_map4_4a[] = {
    instrux + 2276,
    instrux + 2292,
    instrux + 2308,
    instrux + 2324,
    instrux + 2340,
    instrux + 2356,
    instrux + 2372,
    instrux + 2388,
    instrux + 2404,
    instrux + 2420,
    instrux + 2436,
    instrux + 2452,
    instrux + 2468,
    instrux + 2484,
    instrux + 2500,
    instrux + 2532,
    instrux + 2548,
    NULL
};

static const struct itemplate * const itbl_evex_map4_4b[] = {
    instrux + 2277,
    instrux + 2293,
    instrux + 2309,
    instrux + 2325,
    instrux + 2341,
    instrux + 2357,
    instrux + 2373,
    instrux + 2389,
    instrux + 2405,
    instrux + 2421,
    instrux + 2437,
    instrux + 2453,
    instrux + 2469,
    instrux + 2485,
    instrux + 2501,
    instrux + 2533,
    instrux + 2549,
    NULL
};

static const struct itemplate * const itbl_evex_map4_4c[] = {
    instrux + 2267,
    instrux + 2283,
    instrux + 2299,
    instrux + 2315,
    instrux + 2331,
    instrux + 2347,
    instrux + 2363,
    instrux + 2379,
    instrux + 2395,
    instrux + 2411,
    instrux + 2427,
    instrux + 2443,
    instrux + 2459,
    instrux + 2475,
    instrux + 2491,
    instrux + 2523,
    instrux + 2539,
    NULL
};

static const struct itemplate * const itbl_evex_map4_4d[] = {
    instrux + 2271,
    instrux + 2287,
    instrux + 2303,
    instrux + 2319,
    instrux + 2335,
    instrux + 2351,
    instrux + 2367,
    instrux + 2383,
    instrux + 2399,
    instrux + 2415,
    instrux + 2431,
    instrux + 2447,
    instrux + 2463,
    instrux + 2479,
    instrux + 2495,
    instrux + 2527,
    instrux + 2543,
    NULL
};

static const struct itemplate * const itbl_evex_map4_4e[] = {
    instrux + 2270,
    instrux + 2286,
    instrux + 2302,
    instrux + 2318,
    instrux + 2334,
    instrux + 2350,
    instrux + 2366,
    instrux + 2382,
    instrux + 2398,
    instrux + 2414,
    instrux + 2430,
    instrux + 2446,
    instrux + 2462,
    instrux + 2478,
    instrux + 2494,
    instrux + 2526,
    instrux + 2542,
    NULL
};

static const struct itemplate * const itbl_evex_map4_4f[] = {
    instrux + 2266,
    instrux + 2282,
    instrux + 2298,
    instrux + 2314,
    instrux + 2330,
    instrux + 2346,
    instrux + 2362,
    instrux + 2378,
    instrux + 2394,
    instrux + 2410,
    instrux + 2426,
    instrux + 2442,
    instrux + 2458,
    instrux + 2474,
    instrux + 2490,
    instrux + 2522,
    instrux + 2538,
    NULL
};

static const struct itemplate * const itbl_evex_map4_60[] = {
    instrux + 984,
    instrux + 985,
    instrux + 986,
    NULL
};

static const struct itemplate * const itbl_evex_map4_61[] = {
    instrux + 987,
    instrux + 988,
    instrux + 989,
    NULL
};

static const struct itemplate * const itbl_evex_map4_65[] = {
    instrux + 6831,
    instrux + 6833,
    NULL
};

static const struct itemplate * const itbl_evex_map4_66[] = {
    instrux + 828,
    instrux + 829,
    instrux + 830,
    instrux + 831,
    instrux + 834,
    instrux + 835,
    instrux + 836,
    instrux + 837,
    instrux + 6835,
    instrux + 6837,
    NULL
};

static const struct itemplate * const itbl_evex_map4_69[] = {
    instrux + 771,
    instrux + 772,
    instrux + 773,
    NULL
};

static const struct itemplate * const itbl_evex_map4_6b[] = {
    instrux + 768,
    instrux + 769,
    instrux + 770,
    NULL
};

static const struct itemplate * const itbl_evex_map4_80[] = {
    instrux + 80,
    instrux + 81,
    instrux + 129,
    instrux + 130,
    instrux + 178,
    instrux + 179,
    instrux + 227,
    instrux + 228,
    instrux + 276,
    instrux + 277,
    instrux + 325,
    instrux + 326,
    instrux + 374,
    instrux + 375,
    instrux + 1960,
    instrux + 1961,
    instrux + 1962,
    instrux + 1963,
    instrux + 1964,
    instrux + 1965,
    instrux + 1966,
    instrux + 1967,
    instrux + 1968,
    instrux + 1969,
    instrux + 1970,
    instrux + 1971,
    instrux + 1972,
    instrux + 1973,
    instrux + 1974,
    instrux + 1975,
    NULL
};

static const struct itemplate * const itbl_evex_map4_81[] = {
    instrux + 82,
    instrux + 83,
    instrux + 84,
    instrux + 85,
    instrux + 86,
    instrux + 87,
    instrux + 131,
    instrux + 132,
    instrux + 133,
    instrux + 134,
    instrux + 135,
    instrux + 136,
    instrux + 180,
    instrux + 181,
    instrux + 182,
    instrux + 183,
    instrux + 184,
    instrux + 185,
    instrux + 229,
    instrux + 230,
    instrux + 231,
    instrux + 232,
    instrux + 233,
    instrux + 234,
    instrux + 278,
    instrux + 279,
    instrux + 280,
    instrux + 281,
    instrux + 282,
    instrux + 283,
    instrux + 327,
    instrux + 328,
    instrux + 329,
    instrux + 330,
    instrux + 331,
    instrux + 332,
    instrux + 376,
    instrux + 377,
    instrux + 378,
    instrux + 379,
    instrux + 380,
    instrux + 381,
    instrux + 1976,
    instrux + 1977,
    instrux + 1978,
    instrux + 1979,
    instrux + 1980,
    instrux + 1981,
    instrux + 1982,
    instrux + 1983,
    instrux + 1984,
    instrux + 1985,
    instrux + 1986,
    instrux + 1987,
    instrux + 1988,
    instrux + 1989,
    instrux + 1990,
    instrux + 1991,
    instrux + 1992,
    instrux + 1993,
    instrux + 1994,
    instrux + 1995,
    instrux + 1996,
    instrux + 1997,
    instrux + 1998,
    instrux + 1999,
    instrux + 2000,
    instrux + 2001,
    instrux + 2002,
    instrux + 2003,
    instrux + 2004,
    instrux + 2005,
    instrux + 2006,
    instrux + 2007,
    instrux + 2008,
    instrux + 2009,
    instrux + 2010,
    instrux + 2011,
    instrux + 2012,
    instrux + 2013,
    instrux + 2014,
    instrux + 2015,
    instrux + 2016,
    instrux + 2017,
    instrux + 2018,
    instrux + 2019,
    instrux + 2020,
    instrux + 2021,
    instrux + 2022,
    instrux + 2023,
    NULL
};

static const struct itemplate * const itbl_evex_map4_83[] = {
    instrux + 74,
    instrux + 75,
    instrux + 76,
    instrux + 77,
    instrux + 78,
    instrux + 79,
    instrux + 123,
    instrux + 124,
    instrux + 125,
    instrux + 126,
    instrux + 127,
    instrux + 128,
    instrux + 172,
    instrux + 173,
    instrux + 174,
    instrux + 175,
    instrux + 176,
    instrux + 177,
    instrux + 221,
    instrux + 222,
    instrux + 223,
    instrux + 224,
    instrux + 225,
    instrux + 226,
    instrux + 270,
    instrux + 271,
    instrux + 272,
    instrux + 273,
    instrux + 274,
    instrux + 275,
    instrux + 319,
    instrux + 320,
    instrux + 321,
    instrux + 322,
    instrux + 323,
    instrux + 324,
    instrux + 368,
    instrux + 369,
    instrux + 370,
    instrux + 371,
    instrux + 372,
    instrux + 373,
    instrux + 1912,
    instrux + 1913,
    instrux + 1914,
    instrux + 1915,
    instrux + 1916,
    instrux + 1917,
    instrux + 1918,
    instrux + 1919,
    instrux + 1920,
    instrux + 1921,
    instrux + 1922,
    instrux + 1923,
    instrux + 1924,
    instrux + 1925,
    instrux + 1926,
    instrux + 1927,
    instrux + 1928,
    instrux + 1929,
    instrux + 1930,
    instrux + 1931,
    instrux + 1932,
    instrux + 1933,
    instrux + 1934,
    instrux + 1935,
    instrux + 1936,
    instrux + 1937,
    instrux + 1938,
    instrux + 1939,
    instrux + 1940,
    instrux + 1941,
    instrux + 1942,
    instrux + 1943,
    instrux + 1944,
    instrux + 1945,
    instrux + 1946,
    instrux + 1947,
    instrux + 1948,
    instrux + 1949,
    instrux + 1950,
    instrux + 1951,
    instrux + 1952,
    instrux + 1953,
    instrux + 1954,
    instrux + 1955,
    instrux + 1956,
    instrux + 1957,
    instrux + 1958,
    instrux + 1959,
    NULL
};

static const struct itemplate * const itbl_evex_map4_84[] = {
    instrux + 2024,
    instrux + 2025,
    instrux + 2026,
    instrux + 2027,
    instrux + 2028,
    instrux + 2029,
    instrux + 2030,
    instrux + 2031,
    instrux + 2032,
    instrux + 2033,
    instrux + 2034,
    instrux + 2035,
    instrux + 2036,
    instrux + 2037,
    instrux + 2038,
    instrux + 2039,
    NULL
};

static const struct itemplate * const itbl_evex_map4_85[] = {
    instrux + 2040,
    instrux + 2041,
    instrux + 2042,
    instrux + 2043,
    instrux + 2044,
    instrux + 2045,
    instrux + 2046,
    instrux + 2047,
    instrux + 2048,
    instrux + 2049,
    instrux + 2050,
    instrux + 2051,
    instrux + 2052,
    instrux + 2053,
    instrux + 2054,
    instrux + 2055,
    instrux + 2056,
    instrux + 2057,
    instrux + 2058,
    instrux + 2059,
    instrux + 2060,
    instrux + 2061,
    instrux + 2062,
    instrux + 2063,
    instrux + 2064,
    instrux + 2065,
    instrux + 2066,
    instrux + 2067,
    instrux + 2068,
    instrux + 2069,
    instrux + 2070,
    instrux + 2071,
    instrux + 2072,
    instrux + 2073,
    instrux + 2074,
    instrux + 2075,
    instrux + 2076,
    instrux + 2077,
    instrux + 2078,
    instrux + 2079,
    instrux + 2080,
    instrux + 2081,
    instrux + 2082,
    instrux + 2083,
    instrux + 2084,
    instrux + 2085,
    instrux + 2086,
    instrux + 2087,
    NULL
};

static const struct itemplate * const itbl_evex_map4_88[] = {
    instrux + 3069,
    instrux + 3070,
    instrux + 3071,
    NULL
};

static const struct itemplate * const itbl_evex_map4_8a[] = {
    instrux + 32,
    NULL
};

static const struct itemplate * const itbl_evex_map4_8b[] = {
    instrux + 33,
    instrux + 34,
    instrux + 35,
    NULL
};

static const struct itemplate * const itbl_evex_map4_8f[] = {
    instrux + 1768,
    instrux + 1769,
    NULL
};

static const struct itemplate * const itbl_evex_map4_a5[] = {
    instrux + 850,
    instrux + 851,
    instrux + 852,
    instrux + 853,
    instrux + 854,
    instrux + 855,
    NULL
};

static const struct itemplate * const itbl_evex_map4_ad[] = {
    instrux + 868,
    instrux + 869,
    instrux + 870,
    instrux + 871,
    instrux + 872,
    instrux + 873,
    NULL
};

static const struct itemplate * const itbl_evex_map4_af[] = {
    instrux + 762,
    instrux + 763,
    instrux + 764,
    instrux + 765,
    instrux + 766,
    instrux + 767,
    NULL
};

static const struct itemplate * const itbl_evex_map4_c0[] = {
    instrux + 533,
    instrux + 534,
    instrux + 557,
    instrux + 558,
    instrux + 581,
    instrux + 582,
    instrux + 605,
    instrux + 606,
    instrux + 629,
    instrux + 630,
    instrux + 653,
    instrux + 654,
    instrux + 677,
    instrux + 678,
    instrux + 701,
    instrux + 702,
    NULL
};

static const struct itemplate * const itbl_evex_map4_c1[] = {
    instrux + 535,
    instrux + 536,
    instrux + 537,
    instrux + 538,
    instrux + 539,
    instrux + 540,
    instrux + 559,
    instrux + 560,
    instrux + 561,
    instrux + 562,
    instrux + 563,
    instrux + 564,
    instrux + 583,
    instrux + 584,
    instrux + 585,
    instrux + 586,
    instrux + 587,
    instrux + 588,
    instrux + 607,
    instrux + 608,
    instrux + 609,
    instrux + 610,
    instrux + 611,
    instrux + 612,
    instrux + 631,
    instrux + 632,
    instrux + 633,
    instrux + 634,
    instrux + 635,
    instrux + 636,
    instrux + 655,
    instrux + 656,
    instrux + 657,
    instrux + 658,
    instrux + 659,
    instrux + 660,
    instrux + 679,
    instrux + 680,
    instrux + 681,
    instrux + 682,
    instrux + 683,
    instrux + 684,
    instrux + 703,
    instrux + 704,
    instrux + 705,
    instrux + 706,
    instrux + 707,
    instrux + 708,
    NULL
};

static const struct itemplate * const itbl_evex_map4_d0[] = {
    instrux + 517,
    instrux + 518,
    instrux + 541,
    instrux + 542,
    instrux + 565,
    instrux + 566,
    instrux + 589,
    instrux + 590,
    instrux + 613,
    instrux + 614,
    instrux + 637,
    instrux + 638,
    instrux + 661,
    instrux + 662,
    instrux + 685,
    instrux + 686,
    NULL
};

static const struct itemplate * const itbl_evex_map4_d1[] = {
    instrux + 519,
    instrux + 520,
    instrux + 521,
    instrux + 522,
    instrux + 523,
    instrux + 524,
    instrux + 543,
    instrux + 544,
    instrux + 545,
    instrux + 546,
    instrux + 547,
    instrux + 548,
    instrux + 567,
    instrux + 568,
    instrux + 569,
    instrux + 570,
    instrux + 571,
    instrux + 572,
    instrux + 591,
    instrux + 592,
    instrux + 593,
    instrux + 594,
    instrux + 595,
    instrux + 596,
    instrux + 615,
    instrux + 616,
    instrux + 617,
    instrux + 618,
    instrux + 619,
    instrux + 620,
    instrux + 639,
    instrux + 640,
    instrux + 641,
    instrux + 642,
    instrux + 643,
    instrux + 644,
    instrux + 663,
    instrux + 664,
    instrux + 665,
    instrux + 666,
    instrux + 667,
    instrux + 668,
    instrux + 687,
    instrux + 688,
    instrux + 689,
    instrux + 690,
    instrux + 691,
    instrux + 692,
    NULL
};

static const struct itemplate * const itbl_evex_map4_d2[] = {
    instrux + 525,
    instrux + 526,
    instrux + 549,
    instrux + 550,
    instrux + 573,
    instrux + 574,
    instrux + 597,
    instrux + 598,
    instrux + 621,
    instrux + 622,
    instrux + 645,
    instrux + 646,
    instrux + 669,
    instrux + 670,
    instrux + 693,
    instrux + 694,
    NULL
};

static const struct itemplate * const itbl_evex_map4_d3[] = {
    instrux + 527,
    instrux + 528,
    instrux + 529,
    instrux + 530,
    instrux + 531,
    instrux + 532,
    instrux + 551,
    instrux + 552,
    instrux + 553,
    instrux + 554,
    instrux + 555,
    instrux + 556,
    instrux + 575,
    instrux + 576,
    instrux + 577,
    instrux + 578,
    instrux + 579,
    instrux + 580,
    instrux + 599,
    instrux + 600,
    instrux + 601,
    instrux + 602,
    instrux + 603,
    instrux + 604,
    instrux + 623,
    instrux + 624,
    instrux + 625,
    instrux + 626,
    instrux + 627,
    instrux + 628,
    instrux + 647,
    instrux + 648,
    instrux + 649,
    instrux + 650,
    instrux + 651,
    instrux + 652,
    instrux + 671,
    instrux + 672,
    instrux + 673,
    instrux + 674,
    instrux + 675,
    instrux + 676,
    instrux + 695,
    instrux + 696,
    instrux + 697,
    instrux + 698,
    instrux + 699,
    instrux + 700,
    NULL
};

static const struct itemplate * const itbl_evex_map4_f0[] = {
    instrux + 2940,
    instrux + 3048,
    instrux + 3055,
    instrux + 3060,
    NULL
};

static const struct itemplate * const itbl_evex_map4_f1[] = {
    instrux + 2943,
    instrux + 3049,
    instrux + 3050,
    instrux + 3056,
    instrux + 3057,
    instrux + 3058,
    NULL
};

static const struct itemplate * const itbl_evex_map4_f2[] = {
    instrux + 1295,
    NULL
};

static const struct itemplate * const itbl_evex_map4_f4[] = {
    instrux + 913,
    instrux + 914,
    instrux + 915,
    NULL
};

static const struct itemplate * const itbl_evex_map4_f5[] = {
    instrux + 907,
    instrux + 908,
    instrux + 909,
    NULL
};

static const struct itemplate * const itbl_evex_map4_f6[] = {
    instrux + 749,
    instrux + 782,
    instrux + 790,
    instrux + 798,
    instrux + 806,
    instrux + 807,
    instrux + 818,
    instrux + 819,
    instrux + 2088,
    instrux + 2089,
    instrux + 2090,
    instrux + 2091,
    instrux + 2092,
    instrux + 2093,
    instrux + 2094,
    instrux + 2095,
    instrux + 2096,
    instrux + 2097,
    instrux + 2098,
    instrux + 2099,
    instrux + 2100,
    instrux + 2101,
    instrux + 2102,
    instrux + 2103,
    instrux + 2152,
    instrux + 2153,
    instrux + 2154,
    instrux + 2155,
    instrux + 2156,
    instrux + 2157,
    instrux + 2158,
    instrux + 2159,
    instrux + 2160,
    instrux + 2161,
    instrux + 2162,
    instrux + 2163,
    instrux + 2164,
    instrux + 2165,
    instrux + 2166,
    instrux + 2167,
    NULL
};

static const struct itemplate * const itbl_evex_map4_f7[] = {
    instrux + 750,
    instrux + 751,
    instrux + 752,
    instrux + 783,
    instrux + 784,
    instrux + 785,
    instrux + 791,
    instrux + 792,
    instrux + 793,
    instrux + 799,
    instrux + 800,
    instrux + 801,
    instrux + 808,
    instrux + 809,
    instrux + 810,
    instrux + 811,
    instrux + 812,
    instrux + 813,
    instrux + 820,
    instrux + 821,
    instrux + 822,
    instrux + 823,
    instrux + 824,
    instrux + 825,
    instrux + 2104,
    instrux + 2105,
    instrux + 2106,
    instrux + 2107,
    instrux + 2108,
    instrux + 2109,
    instrux + 2110,
    instrux + 2111,
    instrux + 2112,
    instrux + 2113,
    instrux + 2114,
    instrux + 2115,
    instrux + 2116,
    instrux + 2117,
    instrux + 2118,
    instrux + 2119,
    instrux + 2120,
    instrux + 2121,
    instrux + 2122,
    instrux + 2123,
    instrux + 2124,
    instrux + 2125,
    instrux + 2126,
    instrux + 2127,
    instrux + 2128,
    instrux + 2129,
    instrux + 2130,
    instrux + 2131,
    instrux + 2132,
    instrux + 2133,
    instrux + 2134,
    instrux + 2135,
    instrux + 2136,
    instrux + 2137,
    instrux + 2138,
    instrux + 2139,
    instrux + 2140,
    instrux + 2141,
    instrux + 2142,
    instrux + 2143,
    instrux + 2144,
    instrux + 2145,
    instrux + 2146,
    instrux + 2147,
    instrux + 2148,
    instrux + 2149,
    instrux + 2150,
    instrux + 2151,
    instrux + 2168,
    instrux + 2169,
    instrux + 2170,
    instrux + 2171,
    instrux + 2172,
    instrux + 2173,
    instrux + 2174,
    instrux + 2175,
    instrux + 2176,
    instrux + 2177,
    instrux + 2178,
    instrux + 2179,
    instrux + 2180,
    instrux + 2181,
    instrux + 2182,
    instrux + 2183,
    instrux + 2184,
    instrux + 2185,
    instrux + 2186,
    instrux + 2187,
    instrux + 2188,
    instrux + 2189,
    instrux + 2190,
    instrux + 2191,
    instrux + 2192,
    instrux + 2193,
    instrux + 2194,
    instrux + 2195,
    instrux + 2196,
    instrux + 2197,
    instrux + 2198,
    instrux + 2199,
    instrux + 2200,
    instrux + 2201,
    instrux + 2202,
    instrux + 2203,
    instrux + 2204,
    instrux + 2205,
    instrux + 2206,
    instrux + 2207,
    instrux + 2208,
    instrux + 2209,
    instrux + 2210,
    instrux + 2211,
    instrux + 2212,
    instrux + 2213,
    instrux + 2214,
    instrux + 2215,
    NULL
};

static const struct itemplate * const itbl_evex_map4_f8[] = {
    instrux + 1336,
    instrux + 1340,
    instrux + 6708,
    instrux + 6841,
    instrux + 6845,
    NULL
};

static const struct itemplate * const itbl_evex_map4_f9[] = {
    instrux + 6703,
    instrux + 6704,
    NULL
};

static const struct itemplate * const itbl_evex_map4_fc[] = {
    instrux + 7168,
    instrux + 7169,
    instrux + 7172,
    instrux + 7173,
    instrux + 7176,
    instrux + 7177,
    instrux + 7180,
    instrux + 7181,
    NULL
};

static const struct itemplate * const itbl_evex_map4_fe[] = {
    instrux + 723,
    instrux + 724,
    instrux + 737,
    instrux + 738,
    NULL
};

static const struct itemplate * const itbl_evex_map4_ff[] = {
    instrux + 725,
    instrux + 726,
    instrux + 727,
    instrux + 728,
    instrux + 729,
    instrux + 730,
    instrux + 739,
    instrux + 740,
    instrux + 741,
    instrux + 742,
    instrux + 743,
    instrux + 744,
    instrux + 1766,
    instrux + 1767,
    NULL
};

static const struct itemplate * const * const itbl_evex_map4[256] = {
    itbl_evex_map4_00,
    itbl_evex_map4_01,
    itbl_evex_map4_02,
    itbl_evex_map4_03,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_08,
    itbl_evex_map4_09,
    itbl_evex_map4_0a,
    itbl_evex_map4_0b,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_10,
    itbl_evex_map4_11,
    itbl_evex_map4_12,
    itbl_evex_map4_13,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_18,
    itbl_evex_map4_19,
    itbl_evex_map4_1a,
    itbl_evex_map4_1b,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_20,
    itbl_evex_map4_21,
    itbl_evex_map4_22,
    itbl_evex_map4_23,
    itbl_evex_map4_24,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_28,
    itbl_evex_map4_29,
    itbl_evex_map4_2a,
    itbl_evex_map4_2b,
    itbl_evex_map4_2c,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_30,
    itbl_evex_map4_31,
    itbl_evex_map4_32,
    itbl_evex_map4_33,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_38,
    itbl_evex_map4_39,
    itbl_evex_map4_3a,
    itbl_evex_map4_3b,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_40,
    itbl_evex_map4_41,
    itbl_evex_map4_42,
    itbl_evex_map4_43,
    itbl_evex_map4_44,
    itbl_evex_map4_45,
    itbl_evex_map4_46,
    itbl_evex_map4_47,
    itbl_evex_map4_48,
    itbl_evex_map4_49,
    itbl_evex_map4_4a,
    itbl_evex_map4_4b,
    itbl_evex_map4_4c,
    itbl_evex_map4_4d,
    itbl_evex_map4_4e,
    itbl_evex_map4_4f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_60,
    itbl_evex_map4_61,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_65,
    itbl_evex_map4_66,
    NULL,
    NULL,
    itbl_evex_map4_69,
    NULL,
    itbl_evex_map4_6b,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_80,
    itbl_evex_map4_81,
    NULL,
    itbl_evex_map4_83,
    itbl_evex_map4_84,
    itbl_evex_map4_85,
    NULL,
    NULL,
    itbl_evex_map4_88,
    NULL,
    itbl_evex_map4_8a,
    itbl_evex_map4_8b,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_8f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_a5,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_ad,
    NULL,
    itbl_evex_map4_af,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_c0,
    itbl_evex_map4_c1,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_d0,
    itbl_evex_map4_d1,
    itbl_evex_map4_d2,
    itbl_evex_map4_d3,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_f0,
    itbl_evex_map4_f1,
    itbl_evex_map4_f2,
    NULL,
    itbl_evex_map4_f4,
    itbl_evex_map4_f5,
    itbl_evex_map4_f6,
    itbl_evex_map4_f7,
    itbl_evex_map4_f8,
    itbl_evex_map4_f9,
    NULL,
    NULL,
    itbl_evex_map4_fc,
    NULL,
    itbl_evex_map4_fe,
    itbl_evex_map4_ff,
};

static const struct itemplate * const itbl_evex_map5_10[] = {
    instrux + 7127,
    instrux + 7129,
    NULL
};

static const struct itemplate * const itbl_evex_map5_11[] = {
    instrux + 7128,
    instrux + 7130,
    NULL
};

static const struct itemplate * const itbl_evex_map5_18[] = {
    instrux + 7285,
    instrux + 7286,
    instrux + 7287,
    instrux + 7297,
    instrux + 7298,
    instrux + 7299,
    instrux + 7312,
    instrux + 7313,
    instrux + 7314,
    NULL
};

static const struct itemplate * const itbl_evex_map5_1b[] = {
    instrux + 7288,
    instrux + 7289,
    instrux + 7290,
    instrux + 7300,
    instrux + 7301,
    instrux + 7302,
    instrux + 7315,
    instrux + 7316,
    instrux + 7317,
    NULL
};

static const struct itemplate * const itbl_evex_map5_1d[] = {
    instrux + 6969,
    instrux + 6970,
    instrux + 6971,
    instrux + 6984,
    NULL
};

static const struct itemplate * const itbl_evex_map5_1e[] = {
    instrux + 7318,
    instrux + 7319,
    instrux + 7320,
    NULL
};

static const struct itemplate * const itbl_evex_map5_2a[] = {
    instrux + 6982,
    instrux + 6983,
    NULL
};

static const struct itemplate * const itbl_evex_map5_2c[] = {
    instrux + 7003,
    instrux + 7004,
    NULL
};

static const struct itemplate * const itbl_evex_map5_2d[] = {
    instrux + 6977,
    instrux + 6978,
    NULL
};

static const struct itemplate * const itbl_evex_map5_2e[] = {
    instrux + 7165,
    instrux + 7277,
    NULL
};

static const struct itemplate * const itbl_evex_map5_2f[] = {
    instrux + 6929,
    instrux + 7194,
    instrux + 7274,
    NULL
};

static const struct itemplate * const itbl_evex_map5_51[] = {
    instrux + 7157,
    instrux + 7158,
    instrux + 7159,
    instrux + 7160,
    instrux + 7267,
    instrux + 7268,
    instrux + 7269,
    NULL
};

static const struct itemplate * const itbl_evex_map5_58[] = {
    instrux + 6921,
    instrux + 6922,
    instrux + 6923,
    instrux + 6924,
    instrux + 7188,
    instrux + 7189,
    instrux + 7190,
    NULL
};

static const struct itemplate * const itbl_evex_map5_59[] = {
    instrux + 7133,
    instrux + 7134,
    instrux + 7135,
    instrux + 7136,
    instrux + 7249,
    instrux + 7250,
    instrux + 7251,
    NULL
};

static const struct itemplate * const itbl_evex_map5_5a[] = {
    instrux + 6933,
    instrux + 6934,
    instrux + 6935,
    instrux + 6939,
    instrux + 6940,
    instrux + 6941,
    instrux + 6975,
    instrux + 6976,
    NULL
};

static const struct itemplate * const itbl_evex_map5_5b[] = {
    instrux + 6930,
    instrux + 6931,
    instrux + 6932,
    instrux + 6936,
    instrux + 6937,
    instrux + 6938,
    instrux + 6972,
    instrux + 6973,
    instrux + 6974,
    instrux + 6985,
    instrux + 6986,
    instrux + 6987,
    NULL
};

static const struct itemplate * const itbl_evex_map5_5c[] = {
    instrux + 7161,
    instrux + 7162,
    instrux + 7163,
    instrux + 7164,
    instrux + 7270,
    instrux + 7271,
    instrux + 7272,
    NULL
};

static const struct itemplate * const itbl_evex_map5_5d[] = {
    instrux + 7123,
    instrux + 7124,
    instrux + 7125,
    instrux + 7126,
    instrux + 7246,
    instrux + 7247,
    instrux + 7248,
    NULL
};

static const struct itemplate * const itbl_evex_map5_5e[] = {
    instrux + 7021,
    instrux + 7022,
    instrux + 7023,
    instrux + 7024,
    instrux + 7195,
    instrux + 7196,
    instrux + 7197,
    NULL
};

static const struct itemplate * const itbl_evex_map5_5f[] = {
    instrux + 7119,
    instrux + 7120,
    instrux + 7121,
    instrux + 7122,
    instrux + 7243,
    instrux + 7244,
    instrux + 7245,
    NULL
};

static const struct itemplate * const itbl_evex_map5_68[] = {
    instrux + 7384,
    instrux + 7385,
    instrux + 7386,
    instrux + 7408,
    instrux + 7409,
    instrux + 7410,
    instrux + 7423,
    instrux + 7424,
    instrux + 7425,
    NULL
};

static const struct itemplate * const itbl_evex_map5_69[] = {
    instrux + 7378,
    instrux + 7379,
    instrux + 7380,
    instrux + 7402,
    instrux + 7403,
    instrux + 7404,
    instrux + 7417,
    instrux + 7418,
    instrux + 7419,
    NULL
};

static const struct itemplate * const itbl_evex_map5_6a[] = {
    instrux + 7387,
    instrux + 7388,
    instrux + 7389,
    instrux + 7411,
    instrux + 7412,
    instrux + 7413,
    instrux + 7426,
    instrux + 7427,
    instrux + 7428,
    NULL
};

static const struct itemplate * const itbl_evex_map5_6b[] = {
    instrux + 7381,
    instrux + 7382,
    instrux + 7383,
    instrux + 7405,
    instrux + 7406,
    instrux + 7407,
    instrux + 7420,
    instrux + 7421,
    instrux + 7422,
    NULL
};

static const struct itemplate * const itbl_evex_map5_6c[] = {
    instrux + 7396,
    instrux + 7397,
    instrux + 7398,
    instrux + 7399,
    instrux + 7400,
    instrux + 7401,
    instrux + 7432,
    instrux + 7433,
    instrux + 7434,
    instrux + 7435,
    instrux + 7436,
    instrux + 7437,
    instrux + 7440,
    instrux + 7441,
    instrux + 7444,
    instrux + 7445,
    NULL
};

static const struct itemplate * const itbl_evex_map5_6d[] = {
    instrux + 7390,
    instrux + 7391,
    instrux + 7392,
    instrux + 7393,
    instrux + 7394,
    instrux + 7395,
    instrux + 7414,
    instrux + 7415,
    instrux + 7416,
    instrux + 7429,
    instrux + 7430,
    instrux + 7431,
    instrux + 7438,
    instrux + 7439,
    instrux + 7442,
    instrux + 7443,
    NULL
};

static const struct itemplate * const itbl_evex_map5_6e[] = {
    instrux + 7131,
    instrux + 7448,
    NULL
};

static const struct itemplate * const itbl_evex_map5_74[] = {
    instrux + 7282,
    instrux + 7283,
    instrux + 7284,
    instrux + 7294,
    instrux + 7295,
    instrux + 7296,
    instrux + 7309,
    instrux + 7310,
    instrux + 7311,
    NULL
};

static const struct itemplate * const itbl_evex_map5_78[] = {
    instrux + 6991,
    instrux + 6992,
    instrux + 6993,
    instrux + 6994,
    instrux + 6995,
    instrux + 6996,
    instrux + 7005,
    instrux + 7006,
    NULL
};

static const struct itemplate * const itbl_evex_map5_79[] = {
    instrux + 6951,
    instrux + 6952,
    instrux + 6953,
    instrux + 6954,
    instrux + 6955,
    instrux + 6956,
    instrux + 6980,
    instrux + 6981,
    NULL
};

static const struct itemplate * const itbl_evex_map5_7a[] = {
    instrux + 6988,
    instrux + 6989,
    instrux + 6990,
    instrux + 7007,
    instrux + 7008,
    instrux + 7009,
    instrux + 7010,
    instrux + 7011,
    instrux + 7012,
    NULL
};

static const struct itemplate * const itbl_evex_map5_7b[] = {
    instrux + 6948,
    instrux + 6949,
    instrux + 6950,
    instrux + 7013,
    instrux + 7014,
    NULL
};

static const struct itemplate * const itbl_evex_map5_7c[] = {
    instrux + 6997,
    instrux + 6998,
    instrux + 6999,
    instrux + 7000,
    instrux + 7001,
    instrux + 7002,
    NULL
};

static const struct itemplate * const itbl_evex_map5_7d[] = {
    instrux + 6957,
    instrux + 6958,
    instrux + 6959,
    instrux + 6960,
    instrux + 6961,
    instrux + 6962,
    instrux + 7015,
    instrux + 7016,
    instrux + 7017,
    instrux + 7018,
    instrux + 7019,
    instrux + 7020,
    NULL
};

static const struct itemplate * const itbl_evex_map5_7e[] = {
    instrux + 7132,
    instrux + 7449,
    NULL
};

static const struct itemplate * const * const itbl_evex_map5[256] = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map5_10,
    itbl_evex_map5_11,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map5_18,
    NULL,
    NULL,
    itbl_evex_map5_1b,
    NULL,
    itbl_evex_map5_1d,
    itbl_evex_map5_1e,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map5_2a,
    NULL,
    itbl_evex_map5_2c,
    itbl_evex_map5_2d,
    itbl_evex_map5_2e,
    itbl_evex_map5_2f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map5_51,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map5_58,
    itbl_evex_map5_59,
    itbl_evex_map5_5a,
    itbl_evex_map5_5b,
    itbl_evex_map5_5c,
    itbl_evex_map5_5d,
    itbl_evex_map5_5e,
    itbl_evex_map5_5f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map5_68,
    itbl_evex_map5_69,
    itbl_evex_map5_6a,
    itbl_evex_map5_6b,
    itbl_evex_map5_6c,
    itbl_evex_map5_6d,
    itbl_evex_map5_6e,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map5_74,
    NULL,
    NULL,
    NULL,
    itbl_evex_map5_78,
    itbl_evex_map5_79,
    itbl_evex_map5_7a,
    itbl_evex_map5_7b,
    itbl_evex_map5_7c,
    itbl_evex_map5_7d,
    itbl_evex_map5_7e,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_evex_map6_13[] = {
    instrux + 6945,
    instrux + 6946,
    instrux + 6947,
    instrux + 6979,
    NULL
};

static const struct itemplate * const itbl_evex_map6_2c[] = {
    instrux + 7153,
    instrux + 7154,
    instrux + 7155,
    instrux + 7264,
    instrux + 7265,
    instrux + 7266,
    NULL
};

static const struct itemplate * const itbl_evex_map6_2d[] = {
    instrux + 7156,
    NULL
};

static const struct itemplate * const itbl_evex_map6_42[] = {
    instrux + 7111,
    instrux + 7112,
    instrux + 7113,
    instrux + 7237,
    instrux + 7238,
    instrux + 7239,
    NULL
};

static const struct itemplate * const itbl_evex_map6_43[] = {
    instrux + 7114,
    NULL
};

static const struct itemplate * const itbl_evex_map6_4c[] = {
    instrux + 7137,
    instrux + 7138,
    instrux + 7139,
    instrux + 7252,
    instrux + 7253,
    instrux + 7254,
    NULL
};

static const struct itemplate * const itbl_evex_map6_4d[] = {
    instrux + 7140,
    NULL
};

static const struct itemplate * const itbl_evex_map6_4e[] = {
    instrux + 7149,
    instrux + 7150,
    instrux + 7151,
    instrux + 7261,
    instrux + 7262,
    instrux + 7263,
    NULL
};

static const struct itemplate * const itbl_evex_map6_4f[] = {
    instrux + 7152,
    NULL
};

static const struct itemplate * const itbl_evex_map6_56[] = {
    instrux + 7025,
    instrux + 7026,
    instrux + 7027,
    instrux + 7028,
    instrux + 7029,
    instrux + 7030,
    NULL
};

static const struct itemplate * const itbl_evex_map6_57[] = {
    instrux + 7031,
    instrux + 7032,
    NULL
};

static const struct itemplate * const itbl_evex_map6_96[] = {
    instrux + 7041,
    instrux + 7042,
    instrux + 7043,
    NULL
};

static const struct itemplate * const itbl_evex_map6_97[] = {
    instrux + 7050,
    instrux + 7051,
    instrux + 7052,
    NULL
};

static const struct itemplate * const itbl_evex_map6_98[] = {
    instrux + 7059,
    instrux + 7060,
    instrux + 7061,
    instrux + 7198,
    instrux + 7199,
    instrux + 7200,
    NULL
};

static const struct itemplate * const itbl_evex_map6_99[] = {
    instrux + 7077,
    NULL
};

static const struct itemplate * const itbl_evex_map6_9a[] = {
    instrux + 7083,
    instrux + 7084,
    instrux + 7085,
    instrux + 7207,
    instrux + 7208,
    instrux + 7209,
    NULL
};

static const struct itemplate * const itbl_evex_map6_9b[] = {
    instrux + 7101,
    NULL
};

static const struct itemplate * const itbl_evex_map6_9c[] = {
    instrux + 7068,
    instrux + 7069,
    instrux + 7070,
    instrux + 7216,
    instrux + 7217,
    instrux + 7218,
    NULL
};

static const struct itemplate * const itbl_evex_map6_9d[] = {
    instrux + 7080,
    NULL
};

static const struct itemplate * const itbl_evex_map6_9e[] = {
    instrux + 7092,
    instrux + 7093,
    instrux + 7094,
    instrux + 7225,
    instrux + 7226,
    instrux + 7227,
    NULL
};

static const struct itemplate * const itbl_evex_map6_9f[] = {
    instrux + 7104,
    NULL
};

static const struct itemplate * const itbl_evex_map6_a6[] = {
    instrux + 7044,
    instrux + 7045,
    instrux + 7046,
    NULL
};

static const struct itemplate * const itbl_evex_map6_a7[] = {
    instrux + 7053,
    instrux + 7054,
    instrux + 7055,
    NULL
};

static const struct itemplate * const itbl_evex_map6_a8[] = {
    instrux + 7062,
    instrux + 7063,
    instrux + 7064,
    instrux + 7201,
    instrux + 7202,
    instrux + 7203,
    NULL
};

static const struct itemplate * const itbl_evex_map6_a9[] = {
    instrux + 7078,
    NULL
};

static const struct itemplate * const itbl_evex_map6_aa[] = {
    instrux + 7086,
    instrux + 7087,
    instrux + 7088,
    instrux + 7210,
    instrux + 7211,
    instrux + 7212,
    NULL
};

static const struct itemplate * const itbl_evex_map6_ab[] = {
    instrux + 7102,
    NULL
};

static const struct itemplate * const itbl_evex_map6_ac[] = {
    instrux + 7071,
    instrux + 7072,
    instrux + 7073,
    instrux + 7219,
    instrux + 7220,
    instrux + 7221,
    NULL
};

static const struct itemplate * const itbl_evex_map6_ad[] = {
    instrux + 7081,
    NULL
};

static const struct itemplate * const itbl_evex_map6_ae[] = {
    instrux + 7095,
    instrux + 7096,
    instrux + 7097,
    instrux + 7228,
    instrux + 7229,
    instrux + 7230,
    NULL
};

static const struct itemplate * const itbl_evex_map6_af[] = {
    instrux + 7105,
    NULL
};

static const struct itemplate * const itbl_evex_map6_b6[] = {
    instrux + 7047,
    instrux + 7048,
    instrux + 7049,
    NULL
};

static const struct itemplate * const itbl_evex_map6_b7[] = {
    instrux + 7056,
    instrux + 7057,
    instrux + 7058,
    NULL
};

static const struct itemplate * const itbl_evex_map6_b8[] = {
    instrux + 7065,
    instrux + 7066,
    instrux + 7067,
    instrux + 7204,
    instrux + 7205,
    instrux + 7206,
    NULL
};

static const struct itemplate * const itbl_evex_map6_b9[] = {
    instrux + 7079,
    NULL
};

static const struct itemplate * const itbl_evex_map6_ba[] = {
    instrux + 7089,
    instrux + 7090,
    instrux + 7091,
    instrux + 7213,
    instrux + 7214,
    instrux + 7215,
    NULL
};

static const struct itemplate * const itbl_evex_map6_bb[] = {
    instrux + 7103,
    NULL
};

static const struct itemplate * const itbl_evex_map6_bc[] = {
    instrux + 7074,
    instrux + 7075,
    instrux + 7076,
    instrux + 7222,
    instrux + 7223,
    instrux + 7224,
    NULL
};

static const struct itemplate * const itbl_evex_map6_bd[] = {
    instrux + 7082,
    NULL
};

static const struct itemplate * const itbl_evex_map6_be[] = {
    instrux + 7098,
    instrux + 7099,
    instrux + 7100,
    instrux + 7231,
    instrux + 7232,
    instrux + 7233,
    NULL
};

static const struct itemplate * const itbl_evex_map6_bf[] = {
    instrux + 7106,
    NULL
};

static const struct itemplate * const itbl_evex_map6_d6[] = {
    instrux + 7033,
    instrux + 7034,
    instrux + 7035,
    instrux + 7036,
    instrux + 7037,
    instrux + 7038,
    NULL
};

static const struct itemplate * const itbl_evex_map6_d7[] = {
    instrux + 7039,
    instrux + 7040,
    NULL
};

static const struct itemplate * const * const itbl_evex_map6[256] = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map6_13,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map6_2c,
    itbl_evex_map6_2d,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map6_42,
    itbl_evex_map6_43,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map6_4c,
    itbl_evex_map6_4d,
    itbl_evex_map6_4e,
    itbl_evex_map6_4f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map6_56,
    itbl_evex_map6_57,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map6_96,
    itbl_evex_map6_97,
    itbl_evex_map6_98,
    itbl_evex_map6_99,
    itbl_evex_map6_9a,
    itbl_evex_map6_9b,
    itbl_evex_map6_9c,
    itbl_evex_map6_9d,
    itbl_evex_map6_9e,
    itbl_evex_map6_9f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map6_a6,
    itbl_evex_map6_a7,
    itbl_evex_map6_a8,
    itbl_evex_map6_a9,
    itbl_evex_map6_aa,
    itbl_evex_map6_ab,
    itbl_evex_map6_ac,
    itbl_evex_map6_ad,
    itbl_evex_map6_ae,
    itbl_evex_map6_af,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map6_b6,
    itbl_evex_map6_b7,
    itbl_evex_map6_b8,
    itbl_evex_map6_b9,
    itbl_evex_map6_ba,
    itbl_evex_map6_bb,
    itbl_evex_map6_bc,
    itbl_evex_map6_bd,
    itbl_evex_map6_be,
    itbl_evex_map6_bf,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map6_d6,
    itbl_evex_map6_d7,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_evex_map7_f6[] = {
    instrux + 1329,
    instrux + 1332,
    NULL
};

static const struct itemplate * const itbl_evex_map7_f8[] = {
    instrux + 1338,
    instrux + 1342,
    NULL
};

static const struct itemplate * const * const itbl_evex_map7[256] = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map7_f6,
    NULL,
    itbl_evex_map7_f8,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_rex2_map0_00[] = {
    instrux + 39,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_01[] = {
    instrux + 40,
    instrux + 41,
    instrux + 42,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_02[] = {
    instrux + 43,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_03[] = {
    instrux + 44,
    instrux + 45,
    instrux + 46,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_04[] = {
    instrux + 50,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_05[] = {
    instrux + 51,
    instrux + 52,
    instrux + 53,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_08[] = {
    instrux + 88,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_09[] = {
    instrux + 89,
    instrux + 90,
    instrux + 91,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_0a[] = {
    instrux + 92,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_0b[] = {
    instrux + 93,
    instrux + 94,
    instrux + 95,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_0c[] = {
    instrux + 99,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_0d[] = {
    instrux + 100,
    instrux + 101,
    instrux + 102,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_10[] = {
    instrux + 137,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_11[] = {
    instrux + 138,
    instrux + 139,
    instrux + 140,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_12[] = {
    instrux + 141,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_13[] = {
    instrux + 142,
    instrux + 143,
    instrux + 144,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_14[] = {
    instrux + 148,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_15[] = {
    instrux + 149,
    instrux + 150,
    instrux + 151,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_18[] = {
    instrux + 186,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_19[] = {
    instrux + 187,
    instrux + 188,
    instrux + 189,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_1a[] = {
    instrux + 190,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_1b[] = {
    instrux + 191,
    instrux + 192,
    instrux + 193,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_1c[] = {
    instrux + 197,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_1d[] = {
    instrux + 198,
    instrux + 199,
    instrux + 200,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_20[] = {
    instrux + 235,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_21[] = {
    instrux + 236,
    instrux + 237,
    instrux + 238,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_22[] = {
    instrux + 239,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_23[] = {
    instrux + 240,
    instrux + 241,
    instrux + 242,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_24[] = {
    instrux + 246,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_25[] = {
    instrux + 247,
    instrux + 248,
    instrux + 249,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_28[] = {
    instrux + 284,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_29[] = {
    instrux + 285,
    instrux + 286,
    instrux + 287,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_2a[] = {
    instrux + 288,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_2b[] = {
    instrux + 289,
    instrux + 290,
    instrux + 291,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_2c[] = {
    instrux + 295,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_2d[] = {
    instrux + 296,
    instrux + 297,
    instrux + 298,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_30[] = {
    instrux + 333,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_31[] = {
    instrux + 334,
    instrux + 335,
    instrux + 336,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_32[] = {
    instrux + 337,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_33[] = {
    instrux + 338,
    instrux + 339,
    instrux + 340,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_34[] = {
    instrux + 344,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_35[] = {
    instrux + 345,
    instrux + 346,
    instrux + 347,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_38[] = {
    instrux + 382,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_39[] = {
    instrux + 383,
    instrux + 384,
    instrux + 385,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_3a[] = {
    instrux + 386,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_3b[] = {
    instrux + 387,
    instrux + 388,
    instrux + 389,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_3c[] = {
    instrux + 393,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_3d[] = {
    instrux + 394,
    instrux + 395,
    instrux + 396,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_50[] = {
    instrux + 1730,
    instrux + 1731,
    instrux + 1732,
    instrux + 1764,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_51[] = {
    instrux + 1730,
    instrux + 1731,
    instrux + 1732,
    instrux + 1764,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_52[] = {
    instrux + 1730,
    instrux + 1731,
    instrux + 1732,
    instrux + 1764,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_53[] = {
    instrux + 1730,
    instrux + 1731,
    instrux + 1732,
    instrux + 1764,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_54[] = {
    instrux + 1730,
    instrux + 1731,
    instrux + 1732,
    instrux + 1764,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_55[] = {
    instrux + 1730,
    instrux + 1731,
    instrux + 1732,
    instrux + 1764,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_56[] = {
    instrux + 1730,
    instrux + 1731,
    instrux + 1732,
    instrux + 1764,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_57[] = {
    instrux + 1730,
    instrux + 1731,
    instrux + 1732,
    instrux + 1764,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_58[] = {
    instrux + 1742,
    instrux + 1743,
    instrux + 1744,
    instrux + 1765,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_59[] = {
    instrux + 1742,
    instrux + 1743,
    instrux + 1744,
    instrux + 1765,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_5a[] = {
    instrux + 1742,
    instrux + 1743,
    instrux + 1744,
    instrux + 1765,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_5b[] = {
    instrux + 1742,
    instrux + 1743,
    instrux + 1744,
    instrux + 1765,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_5c[] = {
    instrux + 1742,
    instrux + 1743,
    instrux + 1744,
    instrux + 1765,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_5d[] = {
    instrux + 1742,
    instrux + 1743,
    instrux + 1744,
    instrux + 1765,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_5e[] = {
    instrux + 1742,
    instrux + 1743,
    instrux + 1744,
    instrux + 1765,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_5f[] = {
    instrux + 1742,
    instrux + 1743,
    instrux + 1744,
    instrux + 1765,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_63[] = {
    instrux + 1002,
    instrux + 1003,
    instrux + 1004,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_68[] = {
    instrux + 1739,
    instrux + 1740,
    instrux + 1741,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_69[] = {
    instrux + 759,
    instrux + 760,
    instrux + 761,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_6a[] = {
    instrux + 1736,
    instrux + 1737,
    instrux + 1738,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_6b[] = {
    instrux + 756,
    instrux + 757,
    instrux + 758,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_6c[] = {
    instrux + 1273,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_6d[] = {
    instrux + 1274,
    instrux + 1275,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_6e[] = {
    instrux + 1276,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_6f[] = {
    instrux + 1277,
    instrux + 1278,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_80[] = {
    instrux + 54,
    instrux + 103,
    instrux + 152,
    instrux + 201,
    instrux + 250,
    instrux + 299,
    instrux + 348,
    instrux + 397,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_81[] = {
    instrux + 55,
    instrux + 56,
    instrux + 57,
    instrux + 104,
    instrux + 105,
    instrux + 106,
    instrux + 153,
    instrux + 154,
    instrux + 155,
    instrux + 202,
    instrux + 203,
    instrux + 204,
    instrux + 251,
    instrux + 252,
    instrux + 253,
    instrux + 300,
    instrux + 301,
    instrux + 302,
    instrux + 349,
    instrux + 350,
    instrux + 351,
    instrux + 398,
    instrux + 399,
    instrux + 400,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_83[] = {
    instrux + 47,
    instrux + 48,
    instrux + 49,
    instrux + 96,
    instrux + 97,
    instrux + 98,
    instrux + 145,
    instrux + 146,
    instrux + 147,
    instrux + 194,
    instrux + 195,
    instrux + 196,
    instrux + 243,
    instrux + 244,
    instrux + 245,
    instrux + 292,
    instrux + 293,
    instrux + 294,
    instrux + 341,
    instrux + 342,
    instrux + 343,
    instrux + 390,
    instrux + 391,
    instrux + 392,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_84[] = {
    instrux + 401,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_85[] = {
    instrux + 402,
    instrux + 403,
    instrux + 404,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_86[] = {
    instrux + 1024,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_87[] = {
    instrux + 1025,
    instrux + 1026,
    instrux + 1027,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_88[] = {
    instrux + 12,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_89[] = {
    instrux + 13,
    instrux + 14,
    instrux + 15,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_8a[] = {
    instrux + 16,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_8b[] = {
    instrux + 17,
    instrux + 18,
    instrux + 19,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_8c[] = {
    instrux + 1373,
    instrux + 1374,
    instrux + 1375,
    instrux + 1376,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_8d[] = {
    instrux + 36,
    instrux + 37,
    instrux + 38,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_8e[] = {
    instrux + 1377,
    instrux + 1378,
    instrux + 1379,
    instrux + 1380,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_8f[] = {
    instrux + 1745,
    instrux + 1746,
    instrux + 1747,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_90[] = {
    instrux + 0,
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    instrux + 1345,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_91[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_92[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_93[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_94[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_95[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_96[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_97[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_98[] = {
    instrux + 990,
    instrux + 992,
    instrux + 995,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_99[] = {
    instrux + 991,
    instrux + 993,
    instrux + 994,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_9c[] = {
    instrux + 1245,
    instrux + 1246,
    instrux + 1247,
    instrux + 1248,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_9d[] = {
    instrux + 1249,
    instrux + 1250,
    instrux + 1251,
    instrux + 1252,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_9e[] = {
    instrux + 1243,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_9f[] = {
    instrux + 1242,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_a1[] = {
    instrux + 1028,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_b0[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_b1[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_b2[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_b3[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_b4[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_b5[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_b6[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_b7[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_b8[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_b9[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_ba[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_bb[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_bc[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_bd[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_be[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_bf[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_c0[] = {
    instrux + 421,
    instrux + 433,
    instrux + 445,
    instrux + 457,
    instrux + 469,
    instrux + 481,
    instrux + 493,
    instrux + 505,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_c1[] = {
    instrux + 422,
    instrux + 423,
    instrux + 424,
    instrux + 434,
    instrux + 435,
    instrux + 436,
    instrux + 446,
    instrux + 447,
    instrux + 448,
    instrux + 458,
    instrux + 459,
    instrux + 460,
    instrux + 470,
    instrux + 471,
    instrux + 472,
    instrux + 482,
    instrux + 483,
    instrux + 484,
    instrux + 494,
    instrux + 495,
    instrux + 496,
    instrux + 506,
    instrux + 507,
    instrux + 508,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_c2[] = {
    instrux + 1207,
    instrux + 1208,
    instrux + 1210,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_c3[] = {
    instrux + 1203,
    instrux + 1204,
    instrux + 1206,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_c6[] = {
    instrux + 24,
    instrux + 4364,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_c7[] = {
    instrux + 25,
    instrux + 26,
    instrux + 27,
    instrux + 4367,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_c8[] = {
    instrux + 1754,
    instrux + 1755,
    instrux + 1756,
    instrux + 1757,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_c9[] = {
    instrux + 1758,
    instrux + 1759,
    instrux + 1760,
    instrux + 1761,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_ca[] = {
    instrux + 1215,
    instrux + 1216,
    instrux + 1217,
    instrux + 1218,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_cb[] = {
    instrux + 1211,
    instrux + 1212,
    instrux + 1213,
    instrux + 1214,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_cc[] = {
    instrux + 1221,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_cd[] = {
    instrux + 1219,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_cf[] = {
    instrux + 1227,
    instrux + 1228,
    instrux + 1229,
    instrux + 1230,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_d0[] = {
    instrux + 413,
    instrux + 425,
    instrux + 437,
    instrux + 449,
    instrux + 461,
    instrux + 473,
    instrux + 485,
    instrux + 497,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_d1[] = {
    instrux + 414,
    instrux + 415,
    instrux + 416,
    instrux + 426,
    instrux + 427,
    instrux + 428,
    instrux + 438,
    instrux + 439,
    instrux + 440,
    instrux + 450,
    instrux + 451,
    instrux + 452,
    instrux + 462,
    instrux + 463,
    instrux + 464,
    instrux + 474,
    instrux + 475,
    instrux + 476,
    instrux + 486,
    instrux + 487,
    instrux + 488,
    instrux + 498,
    instrux + 499,
    instrux + 500,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_d2[] = {
    instrux + 417,
    instrux + 429,
    instrux + 441,
    instrux + 453,
    instrux + 465,
    instrux + 477,
    instrux + 489,
    instrux + 501,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_d3[] = {
    instrux + 418,
    instrux + 419,
    instrux + 420,
    instrux + 430,
    instrux + 431,
    instrux + 432,
    instrux + 442,
    instrux + 443,
    instrux + 444,
    instrux + 454,
    instrux + 455,
    instrux + 456,
    instrux + 466,
    instrux + 467,
    instrux + 468,
    instrux + 478,
    instrux + 479,
    instrux + 480,
    instrux + 490,
    instrux + 491,
    instrux + 492,
    instrux + 502,
    instrux + 503,
    instrux + 504,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_d6[] = {
    instrux + 1781,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_d7[] = {
    instrux + 1783,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_d8[] = {
    instrux + 1455,
    instrux + 1458,
    instrux + 1460,
    instrux + 1485,
    instrux + 1487,
    instrux + 1488,
    instrux + 1493,
    instrux + 1495,
    instrux + 1496,
    instrux + 1501,
    instrux + 1504,
    instrux + 1506,
    instrux + 1509,
    instrux + 1513,
    instrux + 1514,
    instrux + 1565,
    instrux + 1569,
    instrux + 1570,
    instrux + 1606,
    instrux + 1610,
    instrux + 1611,
    instrux + 1614,
    instrux + 1618,
    instrux + 1619,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_d9[] = {
    instrux + 1453,
    instrux + 1454,
    instrux + 1467,
    instrux + 1498,
    instrux + 1499,
    instrux + 1538,
    instrux + 1552,
    instrux + 1555,
    instrux + 1556,
    instrux + 1557,
    instrux + 1558,
    instrux + 1559,
    instrux + 1560,
    instrux + 1561,
    instrux + 1562,
    instrux + 1563,
    instrux + 1564,
    instrux + 1577,
    instrux + 1579,
    instrux + 1580,
    instrux + 1583,
    instrux + 1584,
    instrux + 1585,
    instrux + 1586,
    instrux + 1587,
    instrux + 1590,
    instrux + 1592,
    instrux + 1593,
    instrux + 1594,
    instrux + 1595,
    instrux + 1598,
    instrux + 1599,
    instrux + 1600,
    instrux + 1622,
    instrux + 1632,
    instrux + 1633,
    instrux + 1634,
    instrux + 1635,
    instrux + 1636,
    instrux + 1637,
    instrux + 1638,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_da[] = {
    instrux + 1469,
    instrux + 1470,
    instrux + 1471,
    instrux + 1472,
    instrux + 1473,
    instrux + 1474,
    instrux + 1483,
    instrux + 1484,
    instrux + 1523,
    instrux + 1525,
    instrux + 1527,
    instrux + 1529,
    instrux + 1531,
    instrux + 1536,
    instrux + 1548,
    instrux + 1550,
    instrux + 1631,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_db[] = {
    instrux + 1468,
    instrux + 1475,
    instrux + 1476,
    instrux + 1477,
    instrux + 1478,
    instrux + 1479,
    instrux + 1480,
    instrux + 1481,
    instrux + 1482,
    instrux + 1489,
    instrux + 1490,
    instrux + 1500,
    instrux + 1518,
    instrux + 1533,
    instrux + 1539,
    instrux + 1540,
    instrux + 1542,
    instrux + 1546,
    instrux + 1554,
    instrux + 1573,
    instrux + 1574,
    instrux + 1575,
    instrux + 1576,
    instrux + 1591,
    instrux + 1602,
    instrux + 1625,
    instrux + 1626,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_dc[] = {
    instrux + 1456,
    instrux + 1457,
    instrux + 1459,
    instrux + 1486,
    instrux + 1494,
    instrux + 1502,
    instrux + 1503,
    instrux + 1505,
    instrux + 1510,
    instrux + 1511,
    instrux + 1512,
    instrux + 1566,
    instrux + 1567,
    instrux + 1568,
    instrux + 1607,
    instrux + 1608,
    instrux + 1609,
    instrux + 1615,
    instrux + 1616,
    instrux + 1617,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_dd[] = {
    instrux + 1519,
    instrux + 1520,
    instrux + 1547,
    instrux + 1553,
    instrux + 1578,
    instrux + 1581,
    instrux + 1588,
    instrux + 1589,
    instrux + 1596,
    instrux + 1597,
    instrux + 1601,
    instrux + 1603,
    instrux + 1604,
    instrux + 1623,
    instrux + 1624,
    instrux + 1629,
    instrux + 1630,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_de[] = {
    instrux + 1461,
    instrux + 1462,
    instrux + 1497,
    instrux + 1507,
    instrux + 1508,
    instrux + 1515,
    instrux + 1516,
    instrux + 1524,
    instrux + 1526,
    instrux + 1528,
    instrux + 1530,
    instrux + 1532,
    instrux + 1537,
    instrux + 1549,
    instrux + 1551,
    instrux + 1571,
    instrux + 1572,
    instrux + 1612,
    instrux + 1613,
    instrux + 1620,
    instrux + 1621,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_df[] = {
    instrux + 1463,
    instrux + 1464,
    instrux + 1465,
    instrux + 1466,
    instrux + 1491,
    instrux + 1492,
    instrux + 1521,
    instrux + 1522,
    instrux + 1534,
    instrux + 1535,
    instrux + 1541,
    instrux + 1543,
    instrux + 1544,
    instrux + 1545,
    instrux + 1582,
    instrux + 1605,
    instrux + 1627,
    instrux + 1628,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_e9[] = {
    instrux + 1034,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_f1[] = {
    instrux + 1220,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_f4[] = {
    instrux + 1344,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_f5[] = {
    instrux + 1241,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_f6[] = {
    instrux + 409,
    instrux + 745,
    instrux + 774,
    instrux + 786,
    instrux + 794,
    instrux + 802,
    instrux + 814,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_f7[] = {
    instrux + 410,
    instrux + 411,
    instrux + 412,
    instrux + 746,
    instrux + 747,
    instrux + 748,
    instrux + 775,
    instrux + 776,
    instrux + 777,
    instrux + 787,
    instrux + 788,
    instrux + 789,
    instrux + 795,
    instrux + 796,
    instrux + 797,
    instrux + 803,
    instrux + 804,
    instrux + 805,
    instrux + 815,
    instrux + 816,
    instrux + 817,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_f8[] = {
    instrux + 1233,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_f9[] = {
    instrux + 1237,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_fa[] = {
    instrux + 1235,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_fb[] = {
    instrux + 1239,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_fc[] = {
    instrux + 1234,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_fd[] = {
    instrux + 1238,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_fe[] = {
    instrux + 719,
    instrux + 733,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_ff[] = {
    instrux + 720,
    instrux + 721,
    instrux + 722,
    instrux + 734,
    instrux + 735,
    instrux + 736,
    instrux + 1035,
    instrux + 1036,
    instrux + 1037,
    instrux + 1041,
    instrux + 1042,
    instrux + 1197,
    instrux + 1201,
    instrux + 1202,
    instrux + 1733,
    instrux + 1734,
    instrux + 1735,
    NULL
};

static const struct itemplate * const * const itbl_rex2_map0[256] = {
    itbl_rex2_map0_00,
    itbl_rex2_map0_01,
    itbl_rex2_map0_02,
    itbl_rex2_map0_03,
    itbl_rex2_map0_04,
    itbl_rex2_map0_05,
    NULL,
    NULL,
    itbl_rex2_map0_08,
    itbl_rex2_map0_09,
    itbl_rex2_map0_0a,
    itbl_rex2_map0_0b,
    itbl_rex2_map0_0c,
    itbl_rex2_map0_0d,
    NULL,
    NULL,
    itbl_rex2_map0_10,
    itbl_rex2_map0_11,
    itbl_rex2_map0_12,
    itbl_rex2_map0_13,
    itbl_rex2_map0_14,
    itbl_rex2_map0_15,
    NULL,
    NULL,
    itbl_rex2_map0_18,
    itbl_rex2_map0_19,
    itbl_rex2_map0_1a,
    itbl_rex2_map0_1b,
    itbl_rex2_map0_1c,
    itbl_rex2_map0_1d,
    NULL,
    NULL,
    itbl_rex2_map0_20,
    itbl_rex2_map0_21,
    itbl_rex2_map0_22,
    itbl_rex2_map0_23,
    itbl_rex2_map0_24,
    itbl_rex2_map0_25,
    NULL,
    NULL,
    itbl_rex2_map0_28,
    itbl_rex2_map0_29,
    itbl_rex2_map0_2a,
    itbl_rex2_map0_2b,
    itbl_rex2_map0_2c,
    itbl_rex2_map0_2d,
    NULL,
    NULL,
    itbl_rex2_map0_30,
    itbl_rex2_map0_31,
    itbl_rex2_map0_32,
    itbl_rex2_map0_33,
    itbl_rex2_map0_34,
    itbl_rex2_map0_35,
    NULL,
    NULL,
    itbl_rex2_map0_38,
    itbl_rex2_map0_39,
    itbl_rex2_map0_3a,
    itbl_rex2_map0_3b,
    itbl_rex2_map0_3c,
    itbl_rex2_map0_3d,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_rex2_map0_50,
    itbl_rex2_map0_51,
    itbl_rex2_map0_52,
    itbl_rex2_map0_53,
    itbl_rex2_map0_54,
    itbl_rex2_map0_55,
    itbl_rex2_map0_56,
    itbl_rex2_map0_57,
    itbl_rex2_map0_58,
    itbl_rex2_map0_59,
    itbl_rex2_map0_5a,
    itbl_rex2_map0_5b,
    itbl_rex2_map0_5c,
    itbl_rex2_map0_5d,
    itbl_rex2_map0_5e,
    itbl_rex2_map0_5f,
    NULL,
    NULL,
    NULL,
    itbl_rex2_map0_63,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_rex2_map0_68,
    itbl_rex2_map0_69,
    itbl_rex2_map0_6a,
    itbl_rex2_map0_6b,
    itbl_rex2_map0_6c,
    itbl_rex2_map0_6d,
    itbl_rex2_map0_6e,
    itbl_rex2_map0_6f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_rex2_map0_80,
    itbl_rex2_map0_81,
    NULL,
    itbl_rex2_map0_83,
    itbl_rex2_map0_84,
    itbl_rex2_map0_85,
    itbl_rex2_map0_86,
    itbl_rex2_map0_87,
    itbl_rex2_map0_88,
    itbl_rex2_map0_89,
    itbl_rex2_map0_8a,
    itbl_rex2_map0_8b,
    itbl_rex2_map0_8c,
    itbl_rex2_map0_8d,
    itbl_rex2_map0_8e,
    itbl_rex2_map0_8f,
    itbl_rex2_map0_90,
    itbl_rex2_map0_91,
    itbl_rex2_map0_92,
    itbl_rex2_map0_93,
    itbl_rex2_map0_94,
    itbl_rex2_map0_95,
    itbl_rex2_map0_96,
    itbl_rex2_map0_97,
    itbl_rex2_map0_98,
    itbl_rex2_map0_99,
    NULL,
    NULL,
    itbl_rex2_map0_9c,
    itbl_rex2_map0_9d,
    itbl_rex2_map0_9e,
    itbl_rex2_map0_9f,
    NULL,
    itbl_rex2_map0_a1,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_rex2_map0_b0,
    itbl_rex2_map0_b1,
    itbl_rex2_map0_b2,
    itbl_rex2_map0_b3,
    itbl_rex2_map0_b4,
    itbl_rex2_map0_b5,
    itbl_rex2_map0_b6,
    itbl_rex2_map0_b7,
    itbl_rex2_map0_b8,
    itbl_rex2_map0_b9,
    itbl_rex2_map0_ba,
    itbl_rex2_map0_bb,
    itbl_rex2_map0_bc,
    itbl_rex2_map0_bd,
    itbl_rex2_map0_be,
    itbl_rex2_map0_bf,
    itbl_rex2_map0_c0,
    itbl_rex2_map0_c1,
    itbl_rex2_map0_c2,
    itbl_rex2_map0_c3,
    NULL,
    NULL,
    itbl_rex2_map0_c6,
    itbl_rex2_map0_c7,
    itbl_rex2_map0_c8,
    itbl_rex2_map0_c9,
    itbl_rex2_map0_ca,
    itbl_rex2_map0_cb,
    itbl_rex2_map0_cc,
    itbl_rex2_map0_cd,
    NULL,
    itbl_rex2_map0_cf,
    itbl_rex2_map0_d0,
    itbl_rex2_map0_d1,
    itbl_rex2_map0_d2,
    itbl_rex2_map0_d3,
    NULL,
    NULL,
    itbl_rex2_map0_d6,
    itbl_rex2_map0_d7,
    itbl_rex2_map0_d8,
    itbl_rex2_map0_d9,
    itbl_rex2_map0_da,
    itbl_rex2_map0_db,
    itbl_rex2_map0_dc,
    itbl_rex2_map0_dd,
    itbl_rex2_map0_de,
    itbl_rex2_map0_df,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_rex2_map0_e9,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_rex2_map0_f1,
    NULL,
    NULL,
    itbl_rex2_map0_f4,
    itbl_rex2_map0_f5,
    itbl_rex2_map0_f6,
    itbl_rex2_map0_f7,
    itbl_rex2_map0_f8,
    itbl_rex2_map0_f9,
    itbl_rex2_map0_fa,
    itbl_rex2_map0_fb,
    itbl_rex2_map0_fc,
    itbl_rex2_map0_fd,
    itbl_rex2_map0_fe,
    itbl_rex2_map0_ff,
};

static const struct itemplate * const itbl_rex2_map1_00[] = {
    instrux + 1187,
    instrux + 1188,
    instrux + 1189,
    instrux + 1421,
    instrux + 1422,
    instrux + 1424,
    instrux + 1431,
    instrux + 1432,
    instrux + 1433,
    instrux + 1434,
    instrux + 1435,
    instrux + 1436,
    instrux + 1437,
    instrux + 1438,
    instrux + 1445,
    instrux + 1446,
    instrux + 1447,
    instrux + 1448,
    instrux + 1449,
    instrux + 1450,
    instrux + 1451,
    instrux + 1452,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_01[] = {
    instrux + 1231,
    instrux + 1232,
    instrux + 1236,
    instrux + 1240,
    instrux + 1282,
    instrux + 1287,
    instrux + 1288,
    instrux + 1289,
    instrux + 1296,
    instrux + 1297,
    instrux + 1298,
    instrux + 1299,
    instrux + 1302,
    instrux + 1313,
    instrux + 1314,
    instrux + 1315,
    instrux + 1316,
    instrux + 1317,
    instrux + 1330,
    instrux + 1333,
    instrux + 1334,
    instrux + 1346,
    instrux + 1347,
    instrux + 1348,
    instrux + 1349,
    instrux + 1350,
    instrux + 1351,
    instrux + 1352,
    instrux + 1353,
    instrux + 1354,
    instrux + 1355,
    instrux + 1423,
    instrux + 1425,
    instrux + 1426,
    instrux + 1427,
    instrux + 1428,
    instrux + 1429,
    instrux + 1430,
    instrux + 1439,
    instrux + 1440,
    instrux + 1441,
    instrux + 1442,
    instrux + 1443,
    instrux + 1444,
    instrux + 1771,
    instrux + 2691,
    instrux + 2692,
    instrux + 2919,
    instrux + 2920,
    instrux + 2921,
    instrux + 2923,
    instrux + 2924,
    instrux + 2925,
    instrux + 2926,
    instrux + 2931,
    instrux + 2932,
    instrux + 2933,
    instrux + 2936,
    instrux + 2944,
    instrux + 2945,
    instrux + 2946,
    instrux + 2947,
    instrux + 4368,
    instrux + 4369,
    instrux + 6696,
    instrux + 6697,
    instrux + 6709,
    instrux + 6710,
    instrux + 6817,
    instrux + 6818,
    instrux + 6819,
    instrux + 6827,
    instrux + 6828,
    instrux + 6829,
    instrux + 6846,
    instrux + 6847,
    instrux + 6848,
    instrux + 7182,
    instrux + 7184,
    instrux + 7185,
    instrux + 7186,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_02[] = {
    instrux + 1415,
    instrux + 1416,
    instrux + 1417,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_03[] = {
    instrux + 1418,
    instrux + 1419,
    instrux + 1420,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_05[] = {
    instrux + 1223,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_06[] = {
    instrux + 1311,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_07[] = {
    instrux + 1226,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_08[] = {
    instrux + 1290,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_09[] = {
    instrux + 1291,
    instrux + 1292,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_0b[] = {
    instrux + 1780,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_0d[] = {
    instrux + 1698,
    instrux + 1699,
    instrux + 4370,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_0e[] = {
    instrux + 1517,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_0f[] = {
    instrux + 1660,
    instrux + 1668,
    instrux + 1669,
    instrux + 1670,
    instrux + 1671,
    instrux + 1672,
    instrux + 1673,
    instrux + 1674,
    instrux + 1675,
    instrux + 1676,
    instrux + 1677,
    instrux + 1678,
    instrux + 1679,
    instrux + 1680,
    instrux + 1681,
    instrux + 1682,
    instrux + 1683,
    instrux + 1684,
    instrux + 1689,
    instrux + 2728,
    instrux + 2729,
    instrux + 2730,
    instrux + 2731,
    instrux + 2732,
    instrux + 3073,
    instrux + 3074,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_10[] = {
    instrux + 2666,
    instrux + 2668,
    instrux + 2893,
    instrux + 2895,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_11[] = {
    instrux + 2667,
    instrux + 2669,
    instrux + 2894,
    instrux + 2896,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_12[] = {
    instrux + 2660,
    instrux + 2662,
    instrux + 2890,
    instrux + 2916,
    instrux + 2918,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_13[] = {
    instrux + 2661,
    instrux + 2889,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_14[] = {
    instrux + 2685,
    instrux + 2907,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_15[] = {
    instrux + 2684,
    instrux + 2906,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_16[] = {
    instrux + 2657,
    instrux + 2659,
    instrux + 2888,
    instrux + 2917,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_17[] = {
    instrux + 2658,
    instrux + 2887,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_18[] = {
    instrux + 2705,
    instrux + 2706,
    instrux + 2707,
    instrux + 2708,
    instrux + 2709,
    instrux + 2710,
    instrux + 6711,
    instrux + 7450,
    instrux + 7451,
    instrux + 7452,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_19[] = {
    instrux + 7450,
    instrux + 7451,
    instrux + 7452,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_1a[] = {
    instrux + 4372,
    instrux + 4374,
    instrux + 4375,
    instrux + 4377,
    instrux + 4381,
    instrux + 4382,
    instrux + 4385,
    instrux + 4387,
    instrux + 7450,
    instrux + 7451,
    instrux + 7452,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_1b[] = {
    instrux + 4371,
    instrux + 4378,
    instrux + 4380,
    instrux + 4383,
    instrux + 4384,
    instrux + 4388,
    instrux + 4390,
    instrux + 4392,
    instrux + 7450,
    instrux + 7451,
    instrux + 7452,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_1c[] = {
    instrux + 6700,
    instrux + 7450,
    instrux + 7451,
    instrux + 7452,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_1d[] = {
    instrux + 7450,
    instrux + 7451,
    instrux + 7452,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_1e[] = {
    instrux + 6821,
    instrux + 6822,
    instrux + 6825,
    instrux + 6826,
    instrux + 7450,
    instrux + 7451,
    instrux + 7452,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_1f[] = {
    instrux + 1,
    instrux + 2,
    instrux + 3,
    instrux + 7450,
    instrux + 7451,
    instrux + 7452,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_20[] = {
    instrux + 1320,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_21[] = {
    instrux + 1324,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_22[] = {
    instrux + 1321,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_23[] = {
    instrux + 1325,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_28[] = {
    instrux + 2655,
    instrux + 2885,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_29[] = {
    instrux + 2656,
    instrux + 2886,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_2a[] = {
    instrux + 2639,
    instrux + 2641,
    instrux + 2642,
    instrux + 2865,
    instrux + 2871,
    instrux + 2872,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_2b[] = {
    instrux + 2665,
    instrux + 2737,
    instrux + 2984,
    instrux + 2985,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_2c[] = {
    instrux + 2645,
    instrux + 2646,
    instrux + 2647,
    instrux + 2874,
    instrux + 2877,
    instrux + 2878,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_2d[] = {
    instrux + 2640,
    instrux + 2643,
    instrux + 2644,
    instrux + 2863,
    instrux + 2868,
    instrux + 2869,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_2e[] = {
    instrux + 2683,
    instrux + 2905,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_2f[] = {
    instrux + 2638,
    instrux + 2859,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_40[] = {
    instrux + 2227,
    instrux + 2243,
    instrux + 2259,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_41[] = {
    instrux + 2224,
    instrux + 2240,
    instrux + 2256,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_42[] = {
    instrux + 2217,
    instrux + 2233,
    instrux + 2249,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_43[] = {
    instrux + 2221,
    instrux + 2237,
    instrux + 2253,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_44[] = {
    instrux + 2231,
    instrux + 2247,
    instrux + 2263,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_45[] = {
    instrux + 2226,
    instrux + 2242,
    instrux + 2258,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_46[] = {
    instrux + 2220,
    instrux + 2236,
    instrux + 2252,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_47[] = {
    instrux + 2216,
    instrux + 2232,
    instrux + 2248,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_48[] = {
    instrux + 2230,
    instrux + 2246,
    instrux + 2262,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_49[] = {
    instrux + 2225,
    instrux + 2241,
    instrux + 2257,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_4a[] = {
    instrux + 2228,
    instrux + 2244,
    instrux + 2260,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_4b[] = {
    instrux + 2229,
    instrux + 2245,
    instrux + 2261,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_4c[] = {
    instrux + 2219,
    instrux + 2235,
    instrux + 2251,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_4d[] = {
    instrux + 2223,
    instrux + 2239,
    instrux + 2255,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_4e[] = {
    instrux + 2222,
    instrux + 2238,
    instrux + 2254,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_4f[] = {
    instrux + 2218,
    instrux + 2234,
    instrux + 2250,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_50[] = {
    instrux + 1659,
    instrux + 2663,
    instrux + 2664,
    instrux + 2891,
    instrux + 2892,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_51[] = {
    instrux + 1652,
    instrux + 2678,
    instrux + 2679,
    instrux + 2901,
    instrux + 2902,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_52[] = {
    instrux + 1687,
    instrux + 2675,
    instrux + 2676,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_53[] = {
    instrux + 2673,
    instrux + 2674,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_54[] = {
    instrux + 1667,
    instrux + 2619,
    instrux + 2840,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_55[] = {
    instrux + 1719,
    instrux + 2618,
    instrux + 2839,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_56[] = {
    instrux + 2672,
    instrux + 2899,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_57[] = {
    instrux + 2686,
    instrux + 2908,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_58[] = {
    instrux + 1696,
    instrux + 2616,
    instrux + 2617,
    instrux + 2837,
    instrux + 2838,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_59[] = {
    instrux + 1690,
    instrux + 2670,
    instrux + 2671,
    instrux + 2897,
    instrux + 2898,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_5a[] = {
    instrux + 1695,
    instrux + 2864,
    instrux + 2867,
    instrux + 2870,
    instrux + 2873,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_5b[] = {
    instrux + 1694,
    instrux + 2861,
    instrux + 2866,
    instrux + 2876,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_5c[] = {
    instrux + 1693,
    instrux + 2681,
    instrux + 2682,
    instrux + 2903,
    instrux + 2904,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_5d[] = {
    instrux + 1688,
    instrux + 2653,
    instrux + 2654,
    instrux + 2883,
    instrux + 2884,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_5e[] = {
    instrux + 1685,
    instrux + 2648,
    instrux + 2649,
    instrux + 2879,
    instrux + 2880,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_5f[] = {
    instrux + 2651,
    instrux + 2652,
    instrux + 2881,
    instrux + 2882,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_60[] = {
    instrux + 1727,
    instrux + 2832,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_61[] = {
    instrux + 1729,
    instrux + 2833,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_62[] = {
    instrux + 1728,
    instrux + 2834,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_63[] = {
    instrux + 1647,
    instrux + 2756,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_64[] = {
    instrux + 1664,
    instrux + 2775,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_65[] = {
    instrux + 1666,
    instrux + 2776,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_66[] = {
    instrux + 1665,
    instrux + 2777,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_67[] = {
    instrux + 1648,
    instrux + 2758,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_68[] = {
    instrux + 1724,
    instrux + 2828,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_69[] = {
    instrux + 1726,
    instrux + 2829,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_6a[] = {
    instrux + 1725,
    instrux + 2830,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_6b[] = {
    instrux + 1646,
    instrux + 2757,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_6c[] = {
    instrux + 2835,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_6d[] = {
    instrux + 2831,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_6e[] = {
    instrux + 1640,
    instrux + 1644,
    instrux + 2741,
    instrux + 2742,
    instrux + 2753,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_6f[] = {
    instrux + 1642,
    instrux + 2744,
    instrux + 2746,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_70[] = {
    instrux + 2727,
    instrux + 2795,
    instrux + 2796,
    instrux + 2797,
    instrux + 2798,
    instrux + 2799,
    instrux + 2800,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_71[] = {
    instrux + 1705,
    instrux + 1709,
    instrux + 1715,
    instrux + 2803,
    instrux + 2809,
    instrux + 2814,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_72[] = {
    instrux + 1701,
    instrux + 1707,
    instrux + 1711,
    instrux + 2805,
    instrux + 2811,
    instrux + 2816,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_73[] = {
    instrux + 1703,
    instrux + 1713,
    instrux + 2801,
    instrux + 2807,
    instrux + 2812,
    instrux + 2818,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_74[] = {
    instrux + 1661,
    instrux + 2772,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_75[] = {
    instrux + 1663,
    instrux + 2773,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_76[] = {
    instrux + 1662,
    instrux + 2774,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_77[] = {
    instrux + 1639,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_78[] = {
    instrux + 2930,
    instrux + 2980,
    instrux + 2982,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_79[] = {
    instrux + 2935,
    instrux + 2981,
    instrux + 2983,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_7c[] = {
    instrux + 2911,
    instrux + 2912,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_7d[] = {
    instrux + 2913,
    instrux + 2914,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_7e[] = {
    instrux + 1641,
    instrux + 1645,
    instrux + 2740,
    instrux + 2743,
    instrux + 2749,
    instrux + 2752,
    instrux + 2754,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_7f[] = {
    instrux + 1643,
    instrux + 2745,
    instrux + 2747,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_90[] = {
    instrux + 2515,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_91[] = {
    instrux + 2512,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_92[] = {
    instrux + 2505,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_93[] = {
    instrux + 2509,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_94[] = {
    instrux + 2519,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_95[] = {
    instrux + 2514,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_96[] = {
    instrux + 2508,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_97[] = {
    instrux + 2504,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_98[] = {
    instrux + 2518,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_99[] = {
    instrux + 2513,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_9a[] = {
    instrux + 2516,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_9b[] = {
    instrux + 2517,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_9c[] = {
    instrux + 2507,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_9d[] = {
    instrux + 2511,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_9e[] = {
    instrux + 2510,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_9f[] = {
    instrux + 2506,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_a0[] = {
    instrux + 1398,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_a1[] = {
    instrux + 1403,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_a2[] = {
    instrux + 1312,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_a3[] = {
    instrux + 874,
    instrux + 875,
    instrux + 876,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_a4[] = {
    instrux + 838,
    instrux + 839,
    instrux + 840,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_a5[] = {
    instrux + 841,
    instrux + 842,
    instrux + 843,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_a6[] = {
    instrux + 4028,
    instrux + 4029,
    instrux + 4030,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_a7[] = {
    instrux + 4022,
    instrux + 4023,
    instrux + 4024,
    instrux + 4025,
    instrux + 4026,
    instrux + 4027,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_a8[] = {
    instrux + 1399,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_a9[] = {
    instrux + 1404,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_aa[] = {
    instrux + 1343,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_ab[] = {
    instrux + 892,
    instrux + 893,
    instrux + 894,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_ac[] = {
    instrux + 856,
    instrux + 857,
    instrux + 858,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_ad[] = {
    instrux + 859,
    instrux + 860,
    instrux + 861,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_ae[] = {
    instrux + 1279,
    instrux + 1280,
    instrux + 1281,
    instrux + 1283,
    instrux + 1284,
    instrux + 1285,
    instrux + 1286,
    instrux + 1356,
    instrux + 1357,
    instrux + 1358,
    instrux + 1359,
    instrux + 1360,
    instrux + 1405,
    instrux + 1406,
    instrux + 1407,
    instrux + 1408,
    instrux + 1409,
    instrux + 1410,
    instrux + 1411,
    instrux + 1412,
    instrux + 2650,
    instrux + 2680,
    instrux + 2687,
    instrux + 2688,
    instrux + 2689,
    instrux + 2690,
    instrux + 2711,
    instrux + 2738,
    instrux + 2739,
    instrux + 6698,
    instrux + 6699,
    instrux + 6820,
    instrux + 6823,
    instrux + 6824,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_af[] = {
    instrux + 753,
    instrux + 754,
    instrux + 755,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_b0[] = {
    instrux + 1011,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_b1[] = {
    instrux + 1012,
    instrux + 1013,
    instrux + 1014,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_b2[] = {
    instrux + 1391,
    instrux + 1392,
    instrux + 1393,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_b3[] = {
    instrux + 886,
    instrux + 887,
    instrux + 888,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_b4[] = {
    instrux + 1385,
    instrux + 1386,
    instrux + 1387,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_b5[] = {
    instrux + 1388,
    instrux + 1389,
    instrux + 1390,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_b6[] = {
    instrux + 1005,
    instrux + 1006,
    instrux + 1007,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_b7[] = {
    instrux + 1008,
    instrux + 1009,
    instrux + 1010,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_b8[] = {
    instrux + 1186,
    instrux + 3066,
    instrux + 3067,
    instrux + 3068,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_b9[] = {
    instrux + 1776,
    instrux + 1777,
    instrux + 1778,
    instrux + 1779,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_ba[] = {
    instrux + 877,
    instrux + 878,
    instrux + 879,
    instrux + 883,
    instrux + 884,
    instrux + 885,
    instrux + 889,
    instrux + 890,
    instrux + 891,
    instrux + 895,
    instrux + 896,
    instrux + 897,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_bb[] = {
    instrux + 880,
    instrux + 881,
    instrux + 882,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_bc[] = {
    instrux + 898,
    instrux + 899,
    instrux + 900,
    instrux + 910,
    instrux + 911,
    instrux + 912,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_bd[] = {
    instrux + 901,
    instrux + 902,
    instrux + 903,
    instrux + 904,
    instrux + 905,
    instrux + 906,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_be[] = {
    instrux + 996,
    instrux + 997,
    instrux + 998,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_bf[] = {
    instrux + 999,
    instrux + 1000,
    instrux + 1001,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_c0[] = {
    instrux + 1017,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_c1[] = {
    instrux + 1018,
    instrux + 1019,
    instrux + 1020,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_c2[] = {
    instrux + 2620,
    instrux + 2621,
    instrux + 2622,
    instrux + 2623,
    instrux + 2624,
    instrux + 2625,
    instrux + 2626,
    instrux + 2627,
    instrux + 2628,
    instrux + 2629,
    instrux + 2630,
    instrux + 2631,
    instrux + 2632,
    instrux + 2633,
    instrux + 2634,
    instrux + 2635,
    instrux + 2636,
    instrux + 2637,
    instrux + 2841,
    instrux + 2842,
    instrux + 2843,
    instrux + 2844,
    instrux + 2845,
    instrux + 2846,
    instrux + 2847,
    instrux + 2848,
    instrux + 2849,
    instrux + 2850,
    instrux + 2851,
    instrux + 2852,
    instrux + 2853,
    instrux + 2854,
    instrux + 2855,
    instrux + 2856,
    instrux + 2857,
    instrux + 2858,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_c3[] = {
    instrux + 2735,
    instrux + 2736,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_c4[] = {
    instrux + 2717,
    instrux + 2718,
    instrux + 2719,
    instrux + 2779,
    instrux + 2780,
    instrux + 2781,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_c5[] = {
    instrux + 2716,
    instrux + 2778,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_c6[] = {
    instrux + 2677,
    instrux + 2900,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_c7[] = {
    instrux + 1015,
    instrux + 1016,
    instrux + 1303,
    instrux + 1304,
    instrux + 1305,
    instrux + 1306,
    instrux + 1307,
    instrux + 1308,
    instrux + 1309,
    instrux + 1310,
    instrux + 2922,
    instrux + 2927,
    instrux + 2928,
    instrux + 2937,
    instrux + 7183,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_c8[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_c9[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_ca[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_cb[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_cc[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_cd[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_ce[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_cf[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_d0[] = {
    instrux + 2909,
    instrux + 2910,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_d1[] = {
    instrux + 1714,
    instrux + 2813,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_d2[] = {
    instrux + 1710,
    instrux + 2815,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_d3[] = {
    instrux + 1712,
    instrux + 2817,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_d4[] = {
    instrux + 2762,
    instrux + 2763,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_d5[] = {
    instrux + 1692,
    instrux + 2790,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_d6[] = {
    instrux + 2748,
    instrux + 2750,
    instrux + 2751,
    instrux + 2755,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_d7[] = {
    instrux + 2724,
    instrux + 2787,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_d8[] = {
    instrux + 1721,
    instrux + 2826,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_d9[] = {
    instrux + 1722,
    instrux + 2827,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_da[] = {
    instrux + 2723,
    instrux + 2786,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_db[] = {
    instrux + 1657,
    instrux + 2768,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_dc[] = {
    instrux + 1654,
    instrux + 2766,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_dd[] = {
    instrux + 1655,
    instrux + 2767,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_de[] = {
    instrux + 2721,
    instrux + 2784,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_df[] = {
    instrux + 1658,
    instrux + 2769,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_e0[] = {
    instrux + 2714,
    instrux + 2770,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_e1[] = {
    instrux + 1708,
    instrux + 2808,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_e2[] = {
    instrux + 1706,
    instrux + 2810,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_e3[] = {
    instrux + 2715,
    instrux + 2771,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_e4[] = {
    instrux + 2725,
    instrux + 2788,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_e5[] = {
    instrux + 1691,
    instrux + 2789,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_e6[] = {
    instrux + 2860,
    instrux + 2862,
    instrux + 2875,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_e7[] = {
    instrux + 2713,
    instrux + 2734,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_e8[] = {
    instrux + 1718,
    instrux + 2824,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_e9[] = {
    instrux + 1720,
    instrux + 2825,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_ea[] = {
    instrux + 2722,
    instrux + 2785,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_eb[] = {
    instrux + 1697,
    instrux + 2793,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_ec[] = {
    instrux + 1651,
    instrux + 2764,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_ed[] = {
    instrux + 1653,
    instrux + 2765,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_ee[] = {
    instrux + 2720,
    instrux + 2783,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_ef[] = {
    instrux + 1770,
    instrux + 2836,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_f0[] = {
    instrux + 2915,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_f1[] = {
    instrux + 1704,
    instrux + 2802,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_f2[] = {
    instrux + 1700,
    instrux + 2804,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_f3[] = {
    instrux + 1702,
    instrux + 2806,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_f4[] = {
    instrux + 2791,
    instrux + 2792,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_f5[] = {
    instrux + 1686,
    instrux + 2782,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_f6[] = {
    instrux + 2726,
    instrux + 2794,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_f7[] = {
    instrux + 2712,
    instrux + 2733,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_f8[] = {
    instrux + 1716,
    instrux + 2819,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_f9[] = {
    instrux + 1723,
    instrux + 2820,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_fa[] = {
    instrux + 1717,
    instrux + 2821,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_fb[] = {
    instrux + 2822,
    instrux + 2823,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_fc[] = {
    instrux + 1649,
    instrux + 2759,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_fd[] = {
    instrux + 1656,
    instrux + 2760,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_fe[] = {
    instrux + 1650,
    instrux + 2761,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_ff[] = {
    instrux + 1772,
    instrux + 1773,
    instrux + 1774,
    instrux + 1775,
    NULL
};

static const struct itemplate * const * const itbl_rex2_map1[256] = {
    itbl_rex2_map1_00,
    itbl_rex2_map1_01,
    itbl_rex2_map1_02,
    itbl_rex2_map1_03,
    NULL,
    itbl_rex2_map1_05,
    itbl_rex2_map1_06,
    itbl_rex2_map1_07,
    itbl_rex2_map1_08,
    itbl_rex2_map1_09,
    NULL,
    itbl_rex2_map1_0b,
    NULL,
    itbl_rex2_map1_0d,
    itbl_rex2_map1_0e,
    itbl_rex2_map1_0f,
    itbl_rex2_map1_10,
    itbl_rex2_map1_11,
    itbl_rex2_map1_12,
    itbl_rex2_map1_13,
    itbl_rex2_map1_14,
    itbl_rex2_map1_15,
    itbl_rex2_map1_16,
    itbl_rex2_map1_17,
    itbl_rex2_map1_18,
    itbl_rex2_map1_19,
    itbl_rex2_map1_1a,
    itbl_rex2_map1_1b,
    itbl_rex2_map1_1c,
    itbl_rex2_map1_1d,
    itbl_rex2_map1_1e,
    itbl_rex2_map1_1f,
    itbl_rex2_map1_20,
    itbl_rex2_map1_21,
    itbl_rex2_map1_22,
    itbl_rex2_map1_23,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_rex2_map1_28,
    itbl_rex2_map1_29,
    itbl_rex2_map1_2a,
    itbl_rex2_map1_2b,
    itbl_rex2_map1_2c,
    itbl_rex2_map1_2d,
    itbl_rex2_map1_2e,
    itbl_rex2_map1_2f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_rex2_map1_40,
    itbl_rex2_map1_41,
    itbl_rex2_map1_42,
    itbl_rex2_map1_43,
    itbl_rex2_map1_44,
    itbl_rex2_map1_45,
    itbl_rex2_map1_46,
    itbl_rex2_map1_47,
    itbl_rex2_map1_48,
    itbl_rex2_map1_49,
    itbl_rex2_map1_4a,
    itbl_rex2_map1_4b,
    itbl_rex2_map1_4c,
    itbl_rex2_map1_4d,
    itbl_rex2_map1_4e,
    itbl_rex2_map1_4f,
    itbl_rex2_map1_50,
    itbl_rex2_map1_51,
    itbl_rex2_map1_52,
    itbl_rex2_map1_53,
    itbl_rex2_map1_54,
    itbl_rex2_map1_55,
    itbl_rex2_map1_56,
    itbl_rex2_map1_57,
    itbl_rex2_map1_58,
    itbl_rex2_map1_59,
    itbl_rex2_map1_5a,
    itbl_rex2_map1_5b,
    itbl_rex2_map1_5c,
    itbl_rex2_map1_5d,
    itbl_rex2_map1_5e,
    itbl_rex2_map1_5f,
    itbl_rex2_map1_60,
    itbl_rex2_map1_61,
    itbl_rex2_map1_62,
    itbl_rex2_map1_63,
    itbl_rex2_map1_64,
    itbl_rex2_map1_65,
    itbl_rex2_map1_66,
    itbl_rex2_map1_67,
    itbl_rex2_map1_68,
    itbl_rex2_map1_69,
    itbl_rex2_map1_6a,
    itbl_rex2_map1_6b,
    itbl_rex2_map1_6c,
    itbl_rex2_map1_6d,
    itbl_rex2_map1_6e,
    itbl_rex2_map1_6f,
    itbl_rex2_map1_70,
    itbl_rex2_map1_71,
    itbl_rex2_map1_72,
    itbl_rex2_map1_73,
    itbl_rex2_map1_74,
    itbl_rex2_map1_75,
    itbl_rex2_map1_76,
    itbl_rex2_map1_77,
    itbl_rex2_map1_78,
    itbl_rex2_map1_79,
    NULL,
    NULL,
    itbl_rex2_map1_7c,
    itbl_rex2_map1_7d,
    itbl_rex2_map1_7e,
    itbl_rex2_map1_7f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_rex2_map1_90,
    itbl_rex2_map1_91,
    itbl_rex2_map1_92,
    itbl_rex2_map1_93,
    itbl_rex2_map1_94,
    itbl_rex2_map1_95,
    itbl_rex2_map1_96,
    itbl_rex2_map1_97,
    itbl_rex2_map1_98,
    itbl_rex2_map1_99,
    itbl_rex2_map1_9a,
    itbl_rex2_map1_9b,
    itbl_rex2_map1_9c,
    itbl_rex2_map1_9d,
    itbl_rex2_map1_9e,
    itbl_rex2_map1_9f,
    itbl_rex2_map1_a0,
    itbl_rex2_map1_a1,
    itbl_rex2_map1_a2,
    itbl_rex2_map1_a3,
    itbl_rex2_map1_a4,
    itbl_rex2_map1_a5,
    itbl_rex2_map1_a6,
    itbl_rex2_map1_a7,
    itbl_rex2_map1_a8,
    itbl_rex2_map1_a9,
    itbl_rex2_map1_aa,
    itbl_rex2_map1_ab,
    itbl_rex2_map1_ac,
    itbl_rex2_map1_ad,
    itbl_rex2_map1_ae,
    itbl_rex2_map1_af,
    itbl_rex2_map1_b0,
    itbl_rex2_map1_b1,
    itbl_rex2_map1_b2,
    itbl_rex2_map1_b3,
    itbl_rex2_map1_b4,
    itbl_rex2_map1_b5,
    itbl_rex2_map1_b6,
    itbl_rex2_map1_b7,
    itbl_rex2_map1_b8,
    itbl_rex2_map1_b9,
    itbl_rex2_map1_ba,
    itbl_rex2_map1_bb,
    itbl_rex2_map1_bc,
    itbl_rex2_map1_bd,
    itbl_rex2_map1_be,
    itbl_rex2_map1_bf,
    itbl_rex2_map1_c0,
    itbl_rex2_map1_c1,
    itbl_rex2_map1_c2,
    itbl_rex2_map1_c3,
    itbl_rex2_map1_c4,
    itbl_rex2_map1_c5,
    itbl_rex2_map1_c6,
    itbl_rex2_map1_c7,
    itbl_rex2_map1_c8,
    itbl_rex2_map1_c9,
    itbl_rex2_map1_ca,
    itbl_rex2_map1_cb,
    itbl_rex2_map1_cc,
    itbl_rex2_map1_cd,
    itbl_rex2_map1_ce,
    itbl_rex2_map1_cf,
    itbl_rex2_map1_d0,
    itbl_rex2_map1_d1,
    itbl_rex2_map1_d2,
    itbl_rex2_map1_d3,
    itbl_rex2_map1_d4,
    itbl_rex2_map1_d5,
    itbl_rex2_map1_d6,
    itbl_rex2_map1_d7,
    itbl_rex2_map1_d8,
    itbl_rex2_map1_d9,
    itbl_rex2_map1_da,
    itbl_rex2_map1_db,
    itbl_rex2_map1_dc,
    itbl_rex2_map1_dd,
    itbl_rex2_map1_de,
    itbl_rex2_map1_df,
    itbl_rex2_map1_e0,
    itbl_rex2_map1_e1,
    itbl_rex2_map1_e2,
    itbl_rex2_map1_e3,
    itbl_rex2_map1_e4,
    itbl_rex2_map1_e5,
    itbl_rex2_map1_e6,
    itbl_rex2_map1_e7,
    itbl_rex2_map1_e8,
    itbl_rex2_map1_e9,
    itbl_rex2_map1_ea,
    itbl_rex2_map1_eb,
    itbl_rex2_map1_ec,
    itbl_rex2_map1_ed,
    itbl_rex2_map1_ee,
    itbl_rex2_map1_ef,
    itbl_rex2_map1_f0,
    itbl_rex2_map1_f1,
    itbl_rex2_map1_f2,
    itbl_rex2_map1_f3,
    itbl_rex2_map1_f4,
    itbl_rex2_map1_f5,
    itbl_rex2_map1_f6,
    itbl_rex2_map1_f7,
    itbl_rex2_map1_f8,
    itbl_rex2_map1_f9,
    itbl_rex2_map1_fa,
    itbl_rex2_map1_fb,
    itbl_rex2_map1_fc,
    itbl_rex2_map1_fd,
    itbl_rex2_map1_fe,
    itbl_rex2_map1_ff,
};

const struct itemplate * const * const * const
ndisasm_itable[] = {
    /* ---- novex ---- */
    /*  0 */ itbl_novex_map0,
    /*  1 */ itbl_novex_map1,
    /*  2 */ itbl_novex_map2,
    /*  3 */ itbl_novex_map3,
    /* ---- vex ---- */
    /*  0 */ NULL,
    /*  1 */ itbl_vex_map1,
    /*  2 */ itbl_vex_map2,
    /*  3 */ itbl_vex_map3,
    /*  4 */ NULL,
    /*  5 */ itbl_vex_map5,
    /*  6 */ NULL,
    /*  7 */ itbl_vex_map7,
    /*  8 */ NULL,
    /*  9 */ NULL,
    /* 10 */ NULL,
    /* 11 */ NULL,
    /* 12 */ NULL,
    /* 13 */ NULL,
    /* 14 */ NULL,
    /* 15 */ NULL,
    /* 16 */ NULL,
    /* 17 */ NULL,
    /* 18 */ NULL,
    /* 19 */ NULL,
    /* 20 */ NULL,
    /* 21 */ NULL,
    /* 22 */ NULL,
    /* 23 */ NULL,
    /* 24 */ NULL,
    /* 25 */ NULL,
    /* 26 */ NULL,
    /* 27 */ NULL,
    /* 28 */ NULL,
    /* 29 */ NULL,
    /* 30 */ NULL,
    /* 31 */ NULL,
    /* ---- xop ---- */
    /*  8 */ itbl_xop_map8,
    /*  9 */ itbl_xop_map9,
    /* 10 */ itbl_xop_map10,
    /* 11 */ NULL,
    /* 12 */ NULL,
    /* 13 */ NULL,
    /* 14 */ NULL,
    /* 15 */ NULL,
    /* 16 */ NULL,
    /* 17 */ NULL,
    /* 18 */ NULL,
    /* 19 */ NULL,
    /* 20 */ NULL,
    /* 21 */ NULL,
    /* 22 */ NULL,
    /* 23 */ NULL,
    /* 24 */ NULL,
    /* 25 */ NULL,
    /* 26 */ NULL,
    /* 27 */ NULL,
    /* 28 */ NULL,
    /* 29 */ NULL,
    /* 30 */ NULL,
    /* 31 */ NULL,
    /* ---- evex ---- */
    /*  0 */ NULL,
    /*  1 */ itbl_evex_map1,
    /*  2 */ itbl_evex_map2,
    /*  3 */ itbl_evex_map3,
    /*  4 */ itbl_evex_map4,
    /*  5 */ itbl_evex_map5,
    /*  6 */ itbl_evex_map6,
    /*  7 */ itbl_evex_map7,
    /* ---- rex2 ---- */
    /*  0 */ itbl_rex2_map0,
    /*  1 */ itbl_rex2_map1,
};
